PRO FRAGMENT_AuAg, cluster, nc, cshell, ncsh, frag, nf, marco_out, shell_out, Dmin, Dmax
;+
;** *************
;** User callable
;** Routine to read a structure file for a cluster of atoms ('cluster' variable)
;** Add to surface atoms defined by clustershell fragments described by 'frag' file
;** fragment is aligned along X
;** Good file format for cluster and fragment models
;** CAR format has header, block 9 columns by n rows, where n is number of atoms
;**
;** METHOD
;** generate array of cluster atom coords: ClusterArr
;** e.g. read from file
;** check that cluster is centred on (0,0,0)
;** identify outermost atoms
;** initially on radius criteria only
;** *** loop thro atoms in list
;** use R2X.pro to find the matrix that rotates surface atom, normalised vector onto X axis
;** rotate fragment onto the radial direction using inverse matrix
;** translate fragment to target atom
;** write fragment coords in array
;** *** loop
;** output a structure file for MS
;**
;** further developments:
;** use a file of  atoms to define shell
;** structure made from vertex sites with Au and FC sites with Ag
;** attach to Au sites
;** look for Ag sites more than d (e.g. 4.5A) from Au for additional sites
;**
;** call requires structure files and corresponding atom numbers, an output file
;** and the extra Ag sites where attachments are to be made, defined by Dmin, Dmax
;** now add output file for modified shell
;**
;** M.R.J. Apr 2011 (ILL)
;-

forward_function r2x

; define shift between fragment and cluster surface
BondOffset=[2.0,0.0,0.0]

; read Cluster file
; nc= number of cluster atoms
ClusterArr=fltarr(nc,3) & ClusterArrStr=strarr(nc)
ClusterAtoms=strarr(nc)
junk1=' ' & junk2=' ' & junk3=' ' & junk4=' '
on_ioerror,misfile    & cl=-1     & f_out=-1
openr, cl, cluster, /get_lun
readf, cl, junk1 & readf, cl, junk2 & readf, cl, junk3 & readf, cl, junk4
readf, cl, ClusterArrStr
close, cl & free_lun, cl
; convert string to float coords
for i=0,nc-1 do begin
  AtomDetails=strsplit(ClusterArrStr[i],/extract)
  ClusterArr[i,*]    =AtomDetails[1:3]
  ClusterAtoms[i]    =AtomDetails[0]
endfor
ClusterPar4=AtomDetails[4] & ClusterPar5=AtomDetails[5] & ClusterPar6=AtomDetails[6]
ClusterPar7=AtomDetails[7] & ClusterPar8=AtomDetails[8]
print, 'cluster file read'

; read Cluster Shell file
; ncsh= number of cluster atoms
ClusterShArr=fltarr(ncsh,3)    & ClusterShArrStr=strarr(ncsh)
ClusterShAtoms=strarr(ncsh)    & ClusterShAtomsName=strarr(nc)
ClusterShAtomsLabel=intarr(nc) & ClusterShAtomsLabel[*]=-1    ; variable for Au (=1) or Ag (=-1)
junk1=' ' & junk2=' ' & junk3=' ' & junk4=' '
openr, cl, cshell, /get_lun
readf, cl, junk1 & readf, cl, junk2 & readf, cl, junk3 & readf, cl, junk4
readf, cl, ClusterShArrStr
close, cl & free_lun, cl
; convert string to float coords
nAu=0 ; counter for Gold surface atoms
for i=0,ncsh-1 do begin
  AtomDetails=strsplit(ClusterShArrStr[i],/extract)
  ClusterShArr[i,*]    =AtomDetails[1:3]
  ClusterShAtoms[i]    =AtomDetails[0]
  ClusterShAtomsName[i]=AtomDetails[7]
  if ClusterShAtomsName[i] eq 'Au' then begin
    nAu=nAu+1
    ClusterShAtomsLabel[i]=1
  endif
endfor
print, 'shell file read'
print, nAu, ' gold atoms'

print, 'inter atomic distance calculation starting'
; calculate all inter-atomic distances in shell and look for closest Ag atoms at a certain distance from Au
DistIntAtom=fltarr(ncsh)
for i=0,ncsh-1 do begin
  ;if ClusterShAtomsName(i) eq 'Au' then print, 'Au atom  ', i
  if ClusterShAtomsName[i] eq 'Ag' then begin
  DistIntAtom[*]=100.
    for j=0,ncsh-1 do begin
      if ClusterShAtomsName[j] eq 'Au' then begin
        DistIntAtom[j]=sqrt((ClusterShArr[i,0]-ClusterShArr[j,0])^2+ $
                            (ClusterShArr[i,1]-ClusterShArr[j,1])^2+ $
                            (ClusterShArr[i,2]-ClusterShArr[j,2])^2)
      endif
    endfor
    DistIdx=sort(DistIntAtom)
    DistOrder=DistIntAtom[DistIdx]
    if DistOrder[0] gt Dmin and DistOrder[1] lt Dmax then begin   ; closest Au atom in correct range
      j=DistIdx[0]
      print, 'Ag ',i,'  Au ',j,' Distance ',DistOrder[0],'  change & break'
      ClusterShAtomsName[i]='Au'                                  ; change atom type
      nAu=nAu+1                                                   ; increment no of Au atoms
    endif
  endif
endfor
print, 'inter atomic distance calculation finished'
print, nAu, ' gold atoms inc Ag sites'

; read Fragment file
; nf= number of cluster atoms
FragArr=fltarr(nf,3) & FragArrStr=strarr(nf)
FragAtoms=strarr(nf)
openr, cl, frag, /get_lun
readf, cl, junk1 & readf, cl, junk2 & readf, cl, junk3 & readf, cl, junk4
readf, cl, FragArrStr
close, cl & free_lun, cl
; convert string to float coords
FragPar4=strarr(nf) & FragPar5=strarr(nf) & FragPar6=strarr(nf) & FragPar7=strarr(nf) & FragPar8=strarr(nf)
for i=0,nf-1 do begin
  AtomDetails=strsplit(FragArrStr[i],/extract)
  FragArr[i,*]=AtomDetails[1:3]
  Fragatoms[i]=AtomDetails[0]
  FragPar4[i] =AtomDetails[4] & FragPar5[i]=AtomDetails[5] & FragPar6[i]=AtomDetails[6]
  FragPar7[i] =AtomDetails[7] & FragPar8[i]=AtomDetails[8]
endfor

; centre cluster on origin, this works!
print, 'centring cluster'
mom=moment(ClusterArr[*,0]) & ClusterArr[*,0]=ClusterArr[*,0]-mom[0]
mom=moment(ClusterArr[*,1]) & ClusterArr[*,1]=ClusterArr[*,1]-mom[0]
mom=moment(ClusterArr[*,2]) & ClusterArr[*,2]=ClusterArr[*,2]-mom[0]

; centre cluster shell on origin, this works!
print, 'centring shell'
mom=moment(ClusterShArr[*,0]) & ClusterShArr[*,0]=ClusterShArr[*,0]-mom[0]
mom=moment(ClusterShArr[*,1]) & ClusterShArr[*,1]=ClusterShArr[*,1]-mom[0]
mom=moment(ClusterShArr[*,2]) & ClusterShArr[*,2]=ClusterShArr[*,2]-mom[0]


; orient fragment along X and put first atom  at the origin?
S=FragArr[0,*] & Cend=FragArr[10,*]
FragVec=Cend-S ;& print,'Frag direction ',FragVec
;translate molecule to origin
for i=0,nf-1 do FragArr[i,*]=FragArr[i,*]-S
;print,'S position ', FragArr(0,*)
;rotate molecule
FragLength=sqrt(transpose(FragVec)##FragVec)
FragNorm=FragVec
for i=0,2 do FragNorm[i]=FragVec[i]/FragLength
R=R2X(FragNorm) ;& Rinv=invert(R)
FragArr=R##FragArr

; loop thro surface atoms, surface selection from input file!
; R2X works for X> so puts 2 fragments on each atom
; only calculate these fragments
; put them in first half of array
; and do rest by symmetry
; special cases: y,z=0 i.e. no rotation and x<0
AllFrags=fltarr(nAu,nf,3) ; for surface atoms
iAu=0
for iSurf=0,ncsh-1 do begin
  if ClusterShAtomsName[iSurf] eq 'Au' then begin
    xyz=ClusterShArr[iSurf,*] ;& print,'Atom ',iAu,'Coords  ', xyz
    xyzNorm=xyz & Rad=sqrt(xyz[0]^2+xyz[1]^2+xyz[2]^2)
    for i=0,2 do xyzNorm[i]=xyz[i]/Rad
    NewFrag=FragArr
    for i=0,nf-1 do NewFrag[i,0]=FragArr[i,0]+Rad        ; translate along X by radius
    for i=0,2 do NewFrag[*,i]=NewFrag[*,i]+BondOffset[i] ; translate fragment
    R=R2X(xyzNorm) & Rinv=invert(R)
    NewFrag=Rinv##NewFrag                                ; rotate fragment
    ; surf atom already on x-axis i.e. no rotation
    if (xyzNorm[0]-1. lt 1e-3) and (abs(xyzNorm[1]) lt 1e-3) and (abs(xyzNorm[2]) lt 1e-3) then begin
      NewFrag=FragArr
      for i=0,nf-1 do NewFrag[i,0]=FragArr[i,0]+Rad        ; translate along X by radius for (1,0,0) case
      for i=0,2 do NewFrag[*,i]=NewFrag[*,i]+BondOffset[i] ; translate fragment
    endif
    ; surf atom already on x-axis, -ve direction i.e. no rotation
    if (abs(xyzNorm[0]+ 1.) lt 1e-3) and (abs(xyzNorm[1]) lt 1e-3) and (abs(xyzNorm[2]) lt 1e-3) then begin
      NewFrag=FragArr
      for i=0,nf-1 do NewFrag[i,0]= -1.0*FragArr[i,0]-Rad  ; invert frag and translate along X by radius
      for i=0,2 do NewFrag[*,i]=NewFrag[*,i]-BondOffset[i] ; translate fragment
    endif
    AllFrags[iAu,*,*]=NewFrag & iAu=iAu+1
  endif
endfor

print, 'Total number of atoms ', nc+nf*nAu,' ',nc,' ',nf,' ',nAu
; write cluster atoms and all fragments into structure file
; user header and column info from input files, need to keep track of atom types
openw,  f_out, marco_out, /get_lun
; car format header from input file
printf, f_out, junk1 & printf, f_out, junk2 & printf, f_out, junk3 & printf, f_out, junk4
;write cluster surface
;for i=0,ncsh-1 do begin
;  printf, f_out, format='(a-5,3f15.9,a5," ",a-7,a-9," ",a3,a6)', $
;    ClusterShAtoms(i,0), ClusterShArr(i,0), ClusterShArr(i,1), ClusterShArr(i,2), $
;    ClusterPar4, ClusterPar5, ClusterPar6, ClusterPar7, ClusterPar8
;endfor
;write cluster first
for i=0,nc-1 do begin
  printf, f_out, format='(a-5,3f15.9,a5," ",a-7,a-9," ",a3,a6)', $
    ClusterAtoms[i], ClusterArr[i,0], ClusterArr[i,1], ClusterArr[i,2], $
    ClusterPar4, ClusterPar5, ClusterPar6, ClusterPar7, ClusterPar8
endfor
; write fragments
for iSurf=0,nAu-1 do begin
  for i=0,nf-1 do begin
    printf, f_out,  format='(a-5,3f15.9,a5," ",a-7,a-9," ",a3,a6)', $
      FragAtoms[i], AllFrags[iSurf,i,0], AllFrags[iSurf,i,1], AllFrags[iSurf,i,2], $
      FragPar4[i], FragPar5[i], FragPar6[i], FragPar7[i], FragPar8[i]
  endfor
endfor
printf, f_out, 'end' & printf, f_out, 'end'
free_lun, f_out
print, 'file written'

; write modified shell into structure file
; user header and column info from input files, need to keep track of atom types
openw, f_out, shell_out, /get_lun
; car format header from input file
printf, f_out, junk1 & printf, f_out, junk2 & printf, f_out, junk3 & printf, f_out, junk4
;write cluster surface
for i=0,ncsh-1 do begin
  printf, f_out, format='(a-5,3f15.9,a5," ",a-7,a-9," ",a3,a6)', $
    ClusterShAtoms[i,0], ClusterShArr[i,0], ClusterShArr[i,1], ClusterShArr[i,2], $
    ClusterPar4, ClusterPar5, ClusterPar6, ClusterShAtomsName[i], ClusterPar8
endfor
printf, f_out, 'end' & printf, f_out, 'end'
free_lun, f_out
print, 'modified shell file written'

return

misfile: if cl    gt 0 then free_lun,cl
         if f_out gt 0 then free_lun,cl
         print,!error_state.msg
end
