function build_disp_d14, file_seed, ndirpts
;+
;******* *****************
;** User callable
;** build dispersion curves from d14 files from PHONON
;** to be used together with an appropriate .prox file
;***************
;**  function to read dispersion curves (d14) and
;**  data file is made up of blocks of n_pts_k spectra at n_k k-values
;***************
;**  call is w1=build_disp_d14('phonon_project_name', 31)
;**  31 is number of points between high symmetry points
;***************
;-

;open d14 file, get nmodes, ndirs, wavevector axis
;*************************************************
ON_IOERROR,misfile
line=' ' & u=-1
file_in=file_seed+'.d14' & print, file_in
openr,u,file_in,/get_lun
readf,u,line & line_contents=strsplit(line, /extract)
nmodes= line_contents[0] & print, 'nmodes=',nmodes
readf,u,line & line_contents=strsplit(line, /extract)
ndirs = line_contents[0] & print, 'ndirs=',ndirs

;initialise variables
;npts: test file has 81857 lines, 2 header lines, 5 ndirs, 401 pts in between -> ((5-1)*401+1)*51modes)+2
npts   =long((ndirs-1)*ndirpts+1) & print, 'npts=',npts
tmp_d14=fltarr(nmodes+1,npts);nmodes+1) ; nmodes columns +1 for wavevectors
q_vec  =fltarr(npts)

high_symm_pts=strarr(ndirs) ; read high symmetry names => saved into the pv variable
high_sym_pts=fltarr(ndirs)  ; high symmetry value in q => saved in Z dimension

readf,u,high_symm_pts
;high_symm_pts=strcompress(strjoin(high_symm_pts,' '))

for i=0,ndirs-1 do begin
  line=strsplit(high_symm_pts[i],/extract)
  high_sym_pts[i]=line[0]
  high_symm_pts[i]=line[1]
endfor

readf,u,tmp_d14
q_vect=reform(tmp_d14[0,*])
w=transpose(tmp_d14[1:nmodes,*])*4.136 ;in meV
free_lun,u & print,'d14 file read' ;read data
print, 'All disp curves read...'

;generate structure for output
datp={x:q_vect, y:findgen(nmodes)+1. , z:high_sym_pts, $
      w_tit:    'High Symm Pts: '+high_symm_pts, $
      other_tit:'(From PHONON) ' +file_seed    , $
      x_tit:    'Phonon Wave Vector', $
      y_tit:    'Frequency [meV]', $
      pv:high_symm_pts}

give_datp, datp
return, w

misfile: if u gt 0 then free_lun,u & lamp_error
end
