function build_disp_d14d51, file_seed, ndirpts, e_res, disp=dsp, D56=d56
;+
;******* *****************
;** User callable
;** build dispersion from d14 and d51 files from PHONON
;**
;***************
;**  function to read dispersion curves (d14) and
;**  coherent structure factor (d51) from Phonon
;**  data file is made up of blocks of n_pts_k spectra at n_k k-values
;***************
;**  call is w1=build_disp_d14d51('phonon_project_name', 31, 0.1, /disp)
;**  'file.d51' is coherent structure factor from PHONON
;**  31 is number of points between high symmetry points
;**  0.1 is e_res
;** /disp for no intensity (only plot dspersion curves = smae color at each point)
;** /d56  for plotting the d56 filter (ex: |e_i(j,k)|^2) as a color @ each point in disp. curve
;** instead of coherent structure factor d51
;***************
;-

;open d14 file, get nmodes, ndirs, wavevector axis
;*************************************************
ON_IOERROR,misfile
line=' ' & u=-1
file_in=file_seed+'.d14' & print, file_in
openr,u,file_in,/get_lun
readf,u,line & line_contents=strsplit(line, /extract)
nmodes= line_contents[0] & print, 'nmodes=',nmodes
readf,u,line & line_contents=strsplit(line, /extract)
ndirs = line_contents[0] & print, 'ndirs=',ndirs

;initialise variables
;npts: test file has 81857 lines, 2 header lines, 5 ndirs, 401 pts in between -> ((5-1)*401+1)*51modes)+2
npts   =long((ndirs-1)*ndirpts+1) & print, 'npts=',npts
tmp_d14=fltarr(npts,nmodes+1) ; nmodes columns +1 for wavevectors
q_vec  =fltarr(npts)

high_symm_pts=strarr(ndirs) ; read high symmetry names => saved into the pv variable
high_sym_pts=fltarr(ndirs)  ; high symmetry value in q => saved in Z dimension

readf,u,high_symm_pts
;high_symm_pts=strcompress(strjoin(high_symm_pts,' '))

for i=0,ndirs-1 do begin
  line=strsplit(high_symm_pts[i],/extract)
  high_sym_pts[i]=line[0]
  high_symm_pts[i]=line[1]
endfor
line=''
for i=0,npts-1 do begin    ; read high symmetry points
  readf,u,line & line_contents=strsplit(line, /extract)
  q_vec[i]=float(line_contents[0])
endfor
free_lun,u & print,'d14 file read' ;read data

;open d51 file
;*************

if n_elements(d56) gt 0 then file_in=file_seed+'.d56' else file_in=file_seed+'.d51'
print, 'Intensity file is '+file_in
junk=' '

;open file
openr,u,file_in,/get_lun & sk='k'
readf,u,junk & readf,u,junk & s=strmid(junk,8,1)
; deal with the different version of PHONON...
case (s) of
  sk: begin ; This is Phonon v4
    nx=6
  end
  else: begin ; This is Phonon v5
    nx=7 & readf,u,junk
  end
endcase

tmp=fltarr(nx,nmodes,npts)
readf,u,tmp & free_lun,u & print,file_in+' intensity file read' ;read data

;pick out q, frequencies and intensities
freq=reform(tmp[4,*,*])*4.136 ;in meV
ints=reform(tmp[5,*,*])

;if simple dispersion plot, set all intensities to 1
if keyword_set(dsp) then ints[*,*]=1.0

;set up freq/intensity axis
nmax  =ceil((max(freq,min=mini)-mini)/e_res)+1.0
print,'number of energy channels=', nmax ; nmax is for max energy transfer
w_axis=fltarr(nmax,npts)
;for each k-point
;go through by blocks for each high symmetry direction
ipt=0 ; set counter to 0 so first increment means first point is skipped
      ; see calculation of npts

Ifreq=round((freq-mini)/e_res)

for i=0L,ndirs-2 do begin ; -2 because the real variable is ndirs-1
 for j=0L,ndirpts-1 do begin
     ipt++
     ;get frequencies onto f_axis
     idx=where(Ifreq[*,ipt] ge 0.0)
     if idx[0] ge 0 then w_axis[Ifreq[idx,ipt],ipt] = w_axis[Ifreq[idx,ipt],ipt] + ints[idx,ipt]
 endfor
endfor
print, 'spectra made'

;if simple dispersion plot, set all intentsities to 1
;since e binning may give higher intensity (2 or 3) in some bins due to degeneracies
if keyword_set(dsp) then begin
  idx=where(w_axis gt 0.)
  if idx[0] ge 0 then w_axis[idx]=1.0
endif

;define energy axis
w=(findgen(nmax)+1.0)*e_res+mini
print, 'energy axis defined'

;generate structure for output
datp={x:q_vec, y:w, z:high_sym_pts, $
      w_tit:    'High Symm Pts: '+high_symm_pts, $
      other_tit:'(From PHONON) ' +file_seed    , $
      x_tit:    'Phonon Wave Vector', $
      y_tit:    'Frequency [meV]', $
      pv:high_symm_pts}

give_datp, datp
return, transpose(w_axis)

misfile: if u gt 0 then free_lun,u & lamp_error
end
