FUNCTION conv_ins, w_in , pc, cutoff, intrange,W_FWHM=W_FWHM,CONSTANT=CONSTANT
;+
;******* ********
;** User callable
;** function to convolute calculated SQW's with
;** energy transfer dependent resolution function
;** ------------------------------------------------------
;** convolute with normalised Gaussian resolution function
;** of variable width, FWHM=w.sqrt(2ln2)
;** y=(1/w.sqrt(pi/2))exp(-2(x-x0)^2/w^2)
;** output spectrum in wavenumbers
;** ------------------------------------------------------
;** call is w2=conv_ins(w1,0.05, 1.5, 3.0, /constant)
;** w1 is workspace to be convoluted
;** 0.05 is width of Gaussian resolution, a % of energy transfer
;** 1.5 is a cutoff to prevent divergence at small E
;** intrange is to integrate over the whole Gaussian
;** /constant indicates that pc is not a percentage but a value of FWHM constant over the total range
;** W_FWHM is the workspace number where the evolution of the FWHM as a function of energy is stored (useful for TOF)
;-

; modified 20/8/2008 to loop through a 2d array, energy along x
; comment 7/10/2008 needs further modification to work on 1D arrays
; done 13/3/2009
; Modified by SR 01/2016 to include /constant keyword and W_FWHM option


take_datp, datp & w_out = w_in & w_temp1=w_in[*,0] & w_temp2=w_in[*,0]
e_i = datp.x & N = n_elements(e_i) ; energy axis
q_info=size(w_in) & n_q=q_info[2]  ; Q axis dimension
if q_info[0] eq 1 then n_q=1       ; special case for 1D array
i_fromwksp=0
;if a dataset of FWHM is already loaded into a Workspace:
if keyword_set(W_FWHM) then begin
  TAKE_W,w_fromwksp,W=W_FWHM & i_fromwksp=1
  n1=n_elements(w_fromwksp) & if n1 ne N then begin & print,'PROBLEM... dimension of x_axis in W_in and w_fromwksp do not match... do nothing' & goto, fin & endif
endif

;find energy step in order to limit resolution at low energy transfer
e_step=abs(e_i[1]-e_i[2])

;set cutoff energy below which resolution is constant
;cutoff is determined by 1.5 x e_step, Gaussian intensity is not unity
;sampling is not sufficient
;cutoff=1.5 read as parameter
w_cutoff=cutoff*e_step

; loop thro Q
for i_q=0,n_q-1 do begin
w_temp1=w_in[*,i_q]

;now convolute, i_orig is centre of Gaussian
for i_orig=0,N-1 do begin
 ;define Gaussian, wd is width, pc is pcage of energy transfer, 1.17741 is sqrt(2ln2)
 ;0.79788 is pre-exp to give unit intensity, pc=0.02 read as parameter
if (not i_fromwksp) then begin
if (keyword_set(CONSTANT)) then wd=pc/1.177410 else wd=e_i[i_orig]*pc/1.177410 
endif else wd=w_fromwksp[i_orig]/1.177410

  if (wd LT w_cutoff) then begin
  wd=w_cutoff
  endif

 ;select range for intrangeation, intrange=3.0 read as parameter
 grange=intrange*wd/e_step &  igrange=floor(grange)
;print, igrange
 imax=i_orig+igrange &  imin=i_orig-igrange
;print, grange,igrange,i_orig,imax,imin
  if (imin LT 1) then imin=1
  if (imax GT N-1) then imax=N-1

 sum=0
  for i_g=imin,imax do begin
  g=(0.79788/wd)*exp(-(2.0/(wd^2))*(e_i[i_g]-e_i[i_orig])^2)
  sum=sum+g*w_temp1[i_g]
  ;print, g
  endfor

w_temp2[i_orig]=sum
endfor

w_out[*,i_q]=w_temp2
endfor

fin:return, w_out

end



