function gr_angle_n,w_in,angle_list, nn
;+
;******* **********
;** User callable
;**
;** The call is e.g. w1=gr_angle_n(w2,[angle_list],nn)
;**
;** w2 is a workspace of fractional coords (i.e. between 0 and 1) read by xdatcar.pro
;** side is side of box
;** n_atomtypes is number of different atoms,
;** atoms_list is a vector with n_atomtypes entries
;** all variables read from CONTCAR
;** angle_list is a vector length 3 of the 3 types of atoms defining the angle
;** if the atoms are ordered O, Na, P, then the angle OPO would be [0,2,0]
;** nn defines the number of neighbours of type 0 around central atom (2)
;** nn should be at least 2
;** this is only relevant for a symmetric bond e.g. [0,2,0]
;**
;** method
;*********
;** loop thro frames and atoms
;** choose a target atom and calculate histogram for each type of pair
;** number of partials to calculate is n_atomtypes(n_atomtypes+1)/2
;** takes each frame and centre it
;** then sort the atoms when ...
;-

; initialise arrays and variables
; get cell & system info from CONTCAR
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
sf=1.0                          ; initialise scale factor
junk1=' ' & junk4=strarr(4)     ; character variables for skipping lines
junk_int=0

openr,f_cont,'CONTCAR',/get_lun
readf,f_cont,junk1                               ; skip header
readf,f_cont,sf & print, sf                      ; read scale factor
readf,f_cont,cell_matrix & print, cell_matrix    ; read cell matrix
side=sf*cell_matrix[0,0] & print, side

readf,f_cont,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

free_lun, f_cont
print, 'CONTCAR read'

; frames and atoms
n_pars   = size(w_in)
n_frames = n_pars[1] & print, 'frames ', n_frames
n_atoms  = n_pars[3] & print, 'atoms ', n_atoms
; vector for distance to all atoms from a target
rd=fltarr(n_atoms) & rd_par=fltarr(n_atoms,n_atoms,n_atomtypes,n_atomtypes)
; matrices for atom lists
atoms_in_frame=fltarr(3,n_atoms) & shift_xyz=fltarr(3)
; array for bond angles
n_angles=180 & dangle=1.0 & bangle=fltarr(n_angles)

; set up limits for atom types e.g type1 has index 0 to n, type 2 has index n+1 to m
; if atoms_list=[90,30,30] then atoms_list_acc will be [90,120,150]
atoms_list_acc=intarr(n_atomtypes) & atoms_list_acc[0]=atoms_list[0]
for i_types=1,n_atomtypes-1 do atoms_list_acc[i_types]=atoms_list_acc[i_types-1]+atoms_list[i_types]
print, 'accumulated atom numbers ',atoms_list_acc
; array for lower limit index for atom types e.g. if atoms_list=[90,30,30]
; atoms_list_start will be [0,90,120]
atoms_list_start=intarr(n_atomtypes) & atoms_list_start[0]=0
for i_types=1,n_atomtypes-1 do atoms_list_start[i_types]=atoms_list_start[i_types-1]+atoms_list[i_types-1]
print, 'start atom numbers ',atoms_list_start
;table to determine atom types from index of atoms
p_index=intarr(n_atoms)
for i=n_atomtypes-1,0,-1 do begin
  for j = 0,atoms_list_acc[i]-1 do p_index[j]=i
endfor


; loop over frames and target atoms
target_xyz = fltarr(3) & target_xyz[*]=0.5 ; target atom centred in box
for i = 0,n_frames-1 do begin
  ;print, 'frame=',i
  rd_par[*,*,*,*]=0.0               ; initialise distance matrix for frame
  atoms_in_frame[*,*] = w_in[i,*,*] ; extract frame for each atomtype

  ; loop thro all target atoms, only has to be the central atom in the angle
  for j=atoms_list_start[angle_list[1]],atoms_list_acc[angle_list[1]]-1 do begin
    ;prepare frame for g(r) calc
    shift_xyz[*] = target_xyz[*] - atoms_in_frame[*,j] ; shift target atom to centre of box
    for ir=0,n_atoms-1 do begin
      atoms_in_frame[*,ir] = atoms_in_frame[*,ir] + shift_xyz[*]  ; apply shift to all atoms
    endfor
    for ixyz = 0,2 do begin                        ; loop for x,y,z
      tochange=where(atoms_in_frame[ixyz,*] lt 0.) ; coords less than 0 for -ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] + 1.0
      tochange=where(atoms_in_frame[ixyz,*] gt 1.) ; coords greater than 1 for +ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] - 1.0
    endfor
    ;calc r for all atoms in frame and ...
    ;build histogram for each partial, depends on j(target) and ir(surrounding)
    ;loop thro all atoms OR just atoms in angle list
    for ir=0,n_atoms-1 do begin
      d=atoms_in_frame[*,ir]-target_xyz[*] & rd[ir]=sqrt(total(d*d))
      ; assign rd to a partial from atom indices j and ir
      rd_par[j,ir,p_index[j],p_index[ir]]=rd[ir]
    endfor

  ; need to extract distances for atoms involved in angle of interest
    rd_temp=rd_par[j,*,angle_list[1],angle_list[0]]
    ; find how many zeros
    index_0=where(rd_temp lt 1e-5, count_0)
    ; sort distance array and start taking neighbours from count_0 index
    index_nn=sort(rd_temp)

    for i0=0,nn-1 do begin                 ; loop thro nn neighbours of central atom
      for i2=i0+1,nn-1 do begin            ; skip more than half of matrix since same bond types make angle
        ; define interatomic vectors
        v0=atoms_in_frame[*,j]-atoms_in_frame[*,index_nn[count_0+i0]] ; zeros from 0 to count_0-1
        v2=atoms_in_frame[*,j]-atoms_in_frame[*,index_nn[count_0+i2]] ; start counting at count_0 in ordered array
        amp_v0=sqrt(transpose(v0)#v0) & amp_v2=sqrt(transpose(v2)#v2)
        ;calculate angle and augment histogram
        dotp=transpose(v0)#v2
        theta=acos(dotp/amp_v0/amp_v2)
        i_theta=round(n_angles*abs(theta)/!pi) ; index for histogram
        bangle[i_theta]=bangle[i_theta]+1
      endfor
    endfor
  endfor ; end angle analysis

endfor   ; end of frames loop
print,'calc finished'


angles=findgen(n_angles)*dangle
tit='histograms of bond angles'
datp={x:angles,w_tit:tit,x_tit: 'angle'}

give_datp, datp
return, bangle

end
