function gr_coord_loop,w_in, r_in, dr, nr
;+
;******* *************
;** User callable
;**
;** The call is e.g. w1=gr(w2,r_cut,dr,nr)
;** w2 is a workspace of fractional coords (i.e. between 0 and 1) read by xdatcar.pro
;** side is side of box
;** n_atomtypes is number of different atoms,
;** atoms_list is a vector with n_atomtypes entries
;** all variables from CONTCAR
;** r_in defines initial r range over which coordination number is calculated
;** r_cut increases by dr, nr times.
;** w2 will contain histogram of coordination number, 2D to give partials
;**
;** this routine is a generalisation of gr_coord
;**
;** method
;*********
;** loop thro frames and atoms
;** choose a target atom and calculate histogram for each type of pair
;** number of partials to calculate is n_atomtypes(n_atomtypes+1)/2
;** takes each frame and centre it
;** then sort the atoms when ...
;-

; initialise arrays and variables
; get cell & system info from CONTCAR
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
sf=1.0                          ; initialise scale factor
junk1=' ' & junk4=strarr(4)     ; character variables for skipping lines
junk_int=0

openr,f_cont,'CONTCAR',/get_lun
readf,f_cont,junk1                               ; skip header
readf,f_cont,sf & print, sf                      ; read scale factor
readf,f_cont,cell_matrix & print, cell_matrix    ; read cell matrix
side=sf*cell_matrix[0,0] & print, side

readf,f_cont,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

free_lun, f_cont
print, 'CONTCAR read'

; frames and atoms
n_pars   = size(w_in)
n_frames = n_pars[1]                       & print, 'frames ', n_frames
n_atoms  = n_pars[3]                       & print, 'atoms ', n_atoms
; vector for distance to all atoms from a target
rd=fltarr(n_atoms) & rd_par=fltarr(n_atoms,n_atoms,n_atomtypes,n_atomtypes)
; matrices for atom lists
atoms_in_frame=fltarr(3,n_atoms) & shift_xyz=fltarr(3)
; matrix for coordination number, 2nd dimension is for partials, 1 for the time being
n_crd=71 ; max no of atoms in sphere will be 20
coord    =fltarr(nr,n_crd,n_atomtypes,n_atomtypes) ; coordination number for partials
coord_tot=fltarr(nr,n_crd)                         ; coordination number - total
coord_out=fltarr(nr,n_crd,n_atomtypes^2+1)
; set up limits for atom types e.g type1 has index 0 to n, type 2 has index n+1 to m
atoms_list_acc=intarr(n_atomtypes) & atoms_list_acc[0]=atoms_list[0]
for i_types=1,n_atomtypes-1 do atoms_list_acc[i_types]=atoms_list_acc[i_types-1]+atoms_list[i_types]
print, 'accumulated atom number ',atoms_list_acc

;table to determine atom types from index of atoms
p_index=intarr(n_atoms)
for i=n_atomtypes-1,0,-1 do begin
  for j = 0,atoms_list_acc[i]-1 do p_index[j]=i
endfor

; loop over frames and target atoms
target_xyz = fltarr(3) & target_xyz[*]=0.5 ; target atom centred in box
for i = 0,n_frames-1 do begin
  rd_par[*,*]=0.0 ; initialise distance matrix for frame
  atoms_in_frame[*,*] = w_in[i,*,*] ; extract frame for each atomtype
  for j = 0,n_atoms-1 do begin ; loop thro all target atoms
    ;prepare frame for g(r) calc
    shift_xyz[*] = target_xyz[*] - atoms_in_frame[*,j] ; shift target atom to centre of box
    for ir=0,n_atoms-1 do begin
      atoms_in_frame[*,ir] = atoms_in_frame[*,ir] + shift_xyz[*]  ; apply shift to all atoms
    endfor
    for ixyz = 0,2 do begin                   ; loop for x,y,z
      tochange=where(atoms_in_frame[ixyz,*] lt 0.) ; coords less than 0 for -ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] + 1.0
      tochange=where(atoms_in_frame[ixyz,*] gt 1.) ; coords greater than 1 for +ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] - 1.0
    endfor
    ;calc r for all atoms in frame and ...
    ;build histogram for each partial, depends on j(target) and ir(surrounding)
    for ir=0,n_atoms-1 do begin ; loop thro all atoms
      d=atoms_in_frame[*,ir]-target_xyz[*] & rd[ir]=sqrt(total(d*d))
      ; assign rd to a partial from atom indices j and ir
      rd_par[j,ir,p_index[j],p_index[ir]]=rd[ir]
    endfor
    rd=rd*side ; convert to real distances and analyse for total coord number
    for irc=0,nr-1 do begin
      r_cut=r_in+irc*dr
      i_atoms_in_sphere=n_elements(where(rd lt r_cut))
      i_atoms_in_sphere=i_atoms_in_sphere-1 ; remove dist 0 to target atom itself
      coord_tot[irc,i_atoms_in_sphere]=coord_tot[irc,i_atoms_in_sphere]+1
    endfor
  endfor ; end of target loop
  ; analyse partial coords
  rd_par=rd_par*side
  for igr=0,n_atomtypes-1 do begin
    for jgr=0,n_atomtypes-1 do begin
     for j=0,n_atoms-1 do begin
      ;transform partial matrix into vector n_atoms^2 long and search for non-zero distances
      rd_temp=rd_par[j,*,igr,jgr]
      index=where(rd_temp gt 0.0,count) ; remove zeros
      if (count gt 0) then begin ; rd_par only has non-zero entries for rectangular sub-matrices
        for irc=0,nr-1 do begin
          temp_list=rd_temp[index]
          r_cut=r_in+irc*dr
          index_r=where(temp_list lt r_cut,count_r) ; find atoms within cut-off
          coord[irc,count_r,igr,jgr]=coord[irc,count_r,igr,jgr]+1
        endfor
      endif
     endfor
    endfor
  endfor ; end partial analysis
endfor   ; end of frames loop
print,'calc finished'

; group together results in one workspace
coord_out[*,*,0]=coord_tot
kgr=1
for igr=0,n_atomtypes-1 do begin
  for jgr=0,n_atomtypes-1 do begin
    coord_out[*,*,kgr]=coord[*,*,igr,jgr]
    kgr=kgr+1
  endfor
endfor

ncd=findgen(n_crd)
tit='histograms of coord numbers for variable r_cut'
datp={x:ncd,w_tit:tit,x_tit: 'coordination number'}

give_datp, datp
return, coord_out

end
