function gr_partial,w_in
;+
;******* **********
;** User callable
;**
;** The call is e.g. w1=gr(w2)
;**
;** w2 is a workspace of fractional coords (i.e. between 0 and 1) read by xdatcar.pro
;** side is side of box
;** n_atomtypes is number of different atoms,
;** atoms_list is a vector with n_atomtypes entries
;** all variables read from CONTCAR
;** w2 will contain gr, 2D to give partials
;**
;** method
;*********
;** loop thro frames and atoms
;** choose a target atom and calculate histogram for each type of pair
;** number of partials to calculate is n_atomtypes(n_atomtypes+1)/2
;** takes each frame and centre it
;** then sort the atoms when ...
;-

; initialise arrays and variables
; get cell & system info from CONTCAR
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
sf=1.0                          ; initialise scale factor
junk1=' ' & junk4=strarr(4)     ; character variables for skipping lines
junk_int=0

openr,f_cont,'CONTCAR',/get_lun
readf,f_cont,junk1                               ; skip header
readf,f_cont,sf & print, sf                      ; read scale factor
readf,f_cont,cell_matrix & print, cell_matrix    ; read cell matrix
side=sf*cell_matrix[0,0] & print, side

readf,f_cont,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

free_lun, f_cont
print, 'CONTCAR read'

; frames and atoms
n_pars   = size(w_in)
n_frames = n_pars[1]                       & print, n_frames
n_atoms  = n_pars[3]                       & print, n_atoms
n_partials = n_atomtypes*(n_atomtypes+1)/2 & print, n_partials
; matrices for atom lists
atoms_in_frame=fltarr(3,n_atoms) & shift_xyz=fltarr(3)
; matrix for g(r), 2nd dimension is for partials, 1 for the time being
n_rpts=1000 & r=fltarr(n_rpts) & grfun=fltarr(n_rpts,n_atomtypes,n_atomtypes) & grfun[*,*,*]=0.0
; set up limits for atom types e.g type1 has index 0 to n, type 2 has index n+1 to m
atoms_list_acc=intarr(n_atomtypes) & atoms_list_acc[0]=atoms_list[0]
for i_types=1,n_atomtypes-1 do atoms_list_acc[i_types]=atoms_list_acc[i_types-1]+atoms_list[i_types]
;print, atoms_list_acc
; set up arrays to decide which partial a pair of atoms will belong to
j_partial=intarr(n_atomtypes) & ir_partial=intarr(n_atomtypes)

;table to determine atom types from index of atoms
p_index=intarr(n_atoms)
for i=n_atomtypes-1,0,-1 do begin
  for j = 0,atoms_list_acc[i]-1 do p_index[j]=i
endfor

; loop over frames and target atoms
target_xyz = fltarr(3) & target_xyz[*]=0.5 ; target atom centred in box
for i = 0,n_frames-1 do begin
  ; print, 'frame=',i
  atoms_in_frame[*,*] = w_in[i,*,*] ; extract frame for each atomtype
  for j = 0,n_atoms-2 do begin ; loop thro all target atoms except last one
    ;prepare frame for g(r) calc
    shift_xyz[*] = target_xyz[*] - atoms_in_frame[*,j] ; shift target atom to centre of box
    for ir=0,n_atoms-1 do begin
      atoms_in_frame[*,ir] = atoms_in_frame[*,ir] + shift_xyz[*]  ; apply shift to all atoms
    endfor
    for ixyz = 0,2 do begin                   ; loop for x,y,z
      tochange=where(atoms_in_frame[ixyz,*] lt 0.) ; coords less than 0 for -ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] + 1.0
      tochange=where(atoms_in_frame[ixyz,*] gt 1.) ; coords greater than 1 for +ve shift
      if (tochange[0] ne -1) then atoms_in_frame[ixyz,tochange] = atoms_in_frame[ixyz,tochange] - 1.0
    endfor
    ;calc r for all atoms in frame and ...
    ;build histogram for each partial, depends on j(target) and ir(surrounding)
    for ir=j+1,n_atoms-1 do begin ; loop from target+1 to last atom
      d=atoms_in_frame[*,ir]-target_xyz[*] & rd=sqrt(total(d*d))
      igr=round(rd*n_rpts) ; index for rd with n_rpts bins/channels
      ; calculate partial g(r) to augment from atom indices j and ir
      grfun[igr,p_index[j],p_index[ir]]=grfun[igr,p_index[j],p_index[ir]]+1.0
    endfor
  endfor
endfor

;normalise by real volume of shells for g(r) i.e. a prob of finding an atom at dist r
dr=side/n_rpts ; shell thickness for spherical normalisation
v_const=4.0*!pi/3.0 ; constant for volume normalisation
for irpt=1,n_rpts-1 do begin
  r[irpt]=(irpt+1.0)*side/n_rpts
  dv=v_const*(r[irpt]^3-(r[irpt]-dr)^3)
  grfun[irpt,*,*]=grfun[irpt,*,*]/dv
endfor
; normalise also by total volume of cell and number of frames
; volume normalisation relates to density (atoms/A^3) normalisation
; atom numbers dealt with below
grfun=grfun*side^3/n_frames

;reorganise g(r) into 2D array, row by row from the 3D g(r) array
;normalise for atom numbers, formula from Cerius manual
;factor of 2 for diagonal term since only half matrix of atoms is calculated
gr2D=fltarr(n_rpts,n_partials) & kgr=0
for igr=0,n_atomtypes-1 do begin
  for jgr=igr,n_atomtypes-1 do begin
    gr2D[*,kgr]=grfun[*,igr,jgr]/(atoms_list[igr]*atoms_list[jgr])
    if (jgr eq igr) then gr2D[*,kgr]=gr2D[*,kgr]*2.0 ; iso-atomic partials are under-counted for some reason
    kgr=kgr+1
  endfor
endfor

tit='g(r)' & datp={x:r,w_tit:tit,x_tit: 'r[Angstroem]'}

give_datp, datp
return, gr2D

end
