PRO HYDRATE, solute, nSolute, solvent, nSolvent, SolventHole, tolerance
;+
;** *******
;** User callable
;** call is
;** hydrate, 'peg.car', 40, 'h2o.car', 99, 'h2o_hole.car', 1.0
;**
;** Routine to hydrate a solute molecule
;** read cubic box of many water molecules: .car format from Mat Studio
;** 'solute' file, 'nsolute' atoms
;** read similar cubic box with  one solute molecule: .car format from Mat Studio
;** 'solvent' file, 'nsolvent' atoms
;** eliminate water molecules that conicide with the solute molecule
;** 'tolerance' typically 1A
;*** output box with solvent and hole for solute
;**
;** CAR format has header, block 9 columns by n rows, where n is number of atoms
;** ****************************************************************************
;-

; read solute file
; nc= number of cluster atoms
SoluteArr=fltarr(nSolute,3) & SoluteArrStr=strarr(nSolute)
junk1=' ' & junk2=' ' & junk3=' ' & junk4=' ' & junk5=' '
on_ioerror,misfile    & cl=-1     & f_out=-1
openr, cl, solute, /get_lun
readf, cl, junk1 & readf, cl, junk2 & readf, cl, junk3 & readf, cl, junk4
readf, cl, junk5 ; line with periodic boundary conditions
readf, cl, SoluteArrStr
close, cl & free_lun, cl
; convert string to float coords
for i=0,nSolute-1 do begin
  AtomDetails=strsplit(SoluteArrStr[i],/extract)
  SoluteArr[i,*]    =AtomDetails[1:3]
endfor
print, 'cluster file read'

; read solvent file
; store coords in 3D array (molecules, atoms, coords)
; nSolvent= number of solvent atoms
nSolventMol=nSolvent/3
SolventArr=fltarr(nSolventMol,3, 3) & SolventArrStr=strarr(nSolventMol, 3)
openr, cl, solvent, /get_lun
readf, cl, junk1 & readf, cl, junk2 & readf, cl, junk3 & readf, cl, junk4
junk5=' ' & junk6=' '
readf, cl, junk5 & print, junk5
for i=0, nSolventMol-1 do begin
  readf, cl, junk6 & SolventArrStr[i,0]=junk6
  readf, cl, junk6 & SolventArrStr[i,1]=junk6
  readf, cl, junk6 & SolventArrStr[i,2]=junk6
  readf, cl, junk6
endfor
close, cl & free_lun, cl
; convert string to float coords
for i=0,nSolventMol-1 do begin
  for j=0,2 do begin
    AtomDetails=strsplit(SolventArrStr[i,j],/extract)
    SolventArr[i,j,*]    =AtomDetails[1:3]
  endfor
endfor
print, 'Solvent file read'


; loop thro solvent molecules and calculate whether they clash with the solute
; use while to stop calc as soon as solvent clashes with one solute atom
RmArr=intarr(nSolventMol) ; array for solvent molecules that clash
for i=0,nSolventMol-1 do begin
  j=0
  while (RmArr[i] ne 1) and (j lt nSolute) do begin
    ; treat all 3 solvent atoms
    diff=SolventArr[i,0,*]-SoluteArr[j,*] & d0=sqrt(diff[0]^2+diff[1]^2+diff[2]^2)
    diff=SolventArr[i,1,*]-SoluteArr[j,*] & d1=sqrt(diff[0]^2+diff[1]^2+diff[2]^2)
    diff=SolventArr[i,2,*]-SoluteArr[j,*] & d2=sqrt(diff[0]^2+diff[1]^2+diff[2]^2)
    if (d0 lt tolerance) or (d1 lt tolerance) or (d2 lt tolerance) then begin
      RmArr[i]=1
      print, d0, d1, d2 & print, 'molecule removed is ',i
    endif
    j=j+1
  endwhile
endfor

; write non-clashing solute molecules in output file
openw,  f_out, SolventHole, /get_lun
; car format header from input file
printf, f_out, junk1 & printf, f_out, junk2 & printf, f_out, junk3
printf, f_out, junk4 & printf, f_out, junk5
idx=where(RmArr lt 1,nRm) & print, nRm, 'molecules NOT removed'
for i=0,nRm-1 do begin
  printf, f_out, SolventArrStr[idx[i],*]
  printf, f_out, 'end'
endfor
printf, f_out, 'end' & printf, f_out, 'end'
free_lun, f_out
print, 'file written'


return

misfile: if cl    gt 0 then free_lun,cl
         if f_out gt 0 then free_lun,cl
         print,!error_state.msg
end
