pro multip,x,dos,multp,NH=NH,U2_OUT=U2_OUT,Debye_W=DW
;----------------------------------------------------------------
; Multiphonon and Debye Waller in the LAGRANGE configuration
; Calculation using the mono-atomic approximation [H partial DOS]
;----------------------------------------------------------------
;
hbarmev=6.582119;e-13 ; value of reduced Planck constant (h bar) in meV.s
hbarjoule=1.0545718;e-34 ;in Joule.s
amu=1.660538;e-27 ; Atomic Mass Unit
m=1.00794*amu ; Hydrogen mass in function of amu

h2sur2m=0.5*10*6.582119^2/m            ;e(-26).e27

NH=1 & if n_elements(NH) gt 0 then N=NH ; number of Hydrogen atoms in the model
;

x_mev=x
dos_mev=dos ; Loading HPDOS

s=1./mean(x_mev[1:n_elements(x_mev)-1]-x_mev[0:n_elements(x_mev)-2])


i=where(x_mev gt 0.) & if n_elements(i) le 10 then begin & print,'ERROR: Number of positive frequencies <= 10... considered as too low... Exiting' & goto, fin & endif
x_mev=x_mev[i] & x_mev=x_mev[i]

r=int_tabulated(x_mev,dos_mev) & dos_mev=dos_mev/r ; renormalizing partial H-DOS to 1.
x_thz=0.24185*x_mev
r=int_tabulated(x_thz,dos_mev) & dos_thz=dos_mev/r ; renormalizing partial H-DOS to 1.

; Calculation of the mean-squared displacement
; ----------------------------------------------
; We make the approximation that T -> 0 so that coth(hbar omega/2KT)-> = +1
; It writes U2 = (hbar*3*N)/(2M)* Int(G(E)*coth(E/2kT)/E dE) 
; Omega should be in Thz !!!!
w=(dos_thz/x_thz)
a=int_tabulated(x_thz,w) & b=int_tabulated(x_mev,dos_mev/x_mev);
;
u2= 0.24185*a*hbarjoule*3*N*10./2./m/(2*!pi) ; u2 in Ang.^2
u2_mev=(b*h2sur2m*3*N)              ; [b]=
alpha=u2_mev/3.                          ; alpha in Ang^2
alpha_dw=u2/3.                      ; pb in mormalization....
; Q^2 Calculation... 
;-----------------------------------------------
; Hypothesis is scattering angle = 90 degree
; Ei >> Ef so that hbarOmega ~ Ei
; Q2 [\AA^2] = 0.6947^2 * E [meV]
;
q2=x_mev*(0.6947^2) ; q2 in Ang^(-2)
;
aq2=alpha_dw*q2; (1/3)*Q2.U2 term

; DW Calculation...
;-----------------------------------------------
dw=EXP(-aq2);*0.+1.
;

mup=fltarr(n_elements(dos_mev),13) & u_n=fltarr(n_elements(dos_mev),11)
mup[*,11]=dw[*]
u_n[*,0]=(h2sur2m/alpha)*dos_mev/x_mev ;
mup[*,0]=u_n[*,0]*aq2*dw[*]
mup[*,12]=mup[*,0]

;
; Calculate the multiphonons
;-----------------------------------
for i=1,10 do begin
if i lt 2 then mp=u_n[*,0] else mp=mp1
n=i+1
;mp1=convol(dos_mev,m,center=0,edge_truncate=1,/normalize) & xx=x_mev+min(x_mev)
mp1=convol(mp,u_n[*,0],s,center=0,/edge_zero) & xx=x_mev+min(x_mev)
mp1=spline(xx,mp1,x_mev)
u_n[*,i]=mp1
mup[*,i]=(mp1[*]*aq2^n/factorial(n))*dw[*]
mup[*,12]=mup[*,12]+mup[*,i]
endfor
multp=mup
;-------------------------------------------------------------------------
;Sum of contributions
;-------------------------------------------------------------------------
if n_elements(U2_OUT) gt -1 then U2_OUT=u2
fin:Print,'End of calculation of the multiphon / Debye Waller'
end