PRO pdb2contcar,d23,pdb,nframes,tol,poscar_seed
;+
;** ***********
;** User callable
;**
;** The call is e.g. pdb2contcar,d23,pdb,nframes,tol,poscar_seed
;**
;** files read are d23 and poscar files from PHONON
;** output files will be in POSCAR format for VASP
;-


; initialise arrays and variables
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
sf=1.0                          ; initialise scale factor
header=' ' & coord_type = ' ' & line2split = ' ' & junk1 = ' '
junk5=strarr(5)                 ; character variables for skipping lines
natoms_string= ' '              ; string to read

; *** get cell, system info & coords
openr,f_d23,d23,/get_lun
readf,f_d23,header           ; skip header
readf,f_d23,sf & print, sf  ; read scale factor
readf,f_d23,cell_matrix     ; read cell matrix
readf,f_d23,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes
natoms=total(atoms_list) & print, natoms
coords_frac = fltarr(3, natoms) & coords_real = fltarr(3, natoms) & & coords_pdb = fltarr(3, natoms)
readf,f_d23,coord_type & print, coord_type
for i=0,natoms-1 do begin
  readf,f_d23,line2split
  line = strsplit(line2split,/extract)
  coords_frac[*,i]=float(line[0:2])
endfor
free_lun, f_d23
print, 'd23 file read'

; *** convert to real coords
coords_real = sf * matrix_multiply(cell_matrix,coords_frac)

; *** read pdb and store frames
; format is header followed by block with labels HETATM and CONECT
big_frame = fltarr(nframes,3,natoms*3)
openr,f_pdb,pdb,/get_lun
readf,f_pdb,junk1  & print, junk1         ; skip header
for iframe=0, nframes-1 do begin
  test_end=' '                ; string to detect 'end' of block/frame
  i_atom=0
  while (test_end ne 'END') do begin
    readf,f_pdb,line2split           ; num lines third entry
    line = strsplit(line2split,/extract)
    if (line[0] eq 'HETATM') then begin
      big_frame[iframe,*,i_atom]=line[3:5]
      i_atom=i_atom+1
    endif
    n_pdb_atoms = i_atom
    test_end=line[0]
  endwhile
endfor
free_lun, f_pdb
print, 'pdb file read'

; make small array of pdb atoms
print, n_pdb_atoms
small_frame=fltarr(nframes,3,n_pdb_atoms) & small_frame=big_frame[*,*,0:n_pdb_atoms-1]
;print,small_frame

; compare d23 coords with pdb coords and write POSCAR files
; tol=0.1
for iframe=0,nframes-1 do begin
  i_found=0
  for i=0,natoms-1 do begin
    for j=0,n_pdb_atoms-1 do begin
      da  =coords_real[0,i] & db  =coords_real[1,i] & dc  =coords_real[2,i]
      dpa =small_frame[iframe,0,j] & dpb =small_frame[iframe,1,j] & dpc =small_frame[iframe,2,j]
      dda=abs(da-dpa) & ddb=abs(db-dpb) & ddc=abs(dc-dpc)
      if (dda lt tol) and (ddb lt tol) and (ddc lt tol) then begin
        print, i,' atom found, deviations = ', dda,' ',ddb,' ',ddc
        coords_pdb[*,i]=small_frame[iframe,*,j]
        i_found=i_found+1
      endif
    endfor
  endfor
  print, i_found, ' atoms found for frame ',iframe
; write POSCAR file
  file_out=poscar_seed+strtrim(string(iframe),2)
  openw,f_pos, file_out,/get_lun
  printf,f_pos, header
  printf,f_pos, sf
  printf,f_pos, cell_matrix
  printf,f_pos, format='(10i4)',atoms_list
  printf,f_pos, 'Cartesian'
  printf,f_pos, coords_pdb
  free_lun,f_pos
endfor

end
