function rd_dmol_Hdos, file_in, n_atoms, atom_choice, atom_type, molecule=ml, vamp=vmp, xpol=xdir, ypol=ydir, zpol=zdir, xyz=alldir
;+
; read a Dmol gamma point vibrational calculation - thesis Sam Ford
; modify for molecule calculations, 3n-6 modes - Marek Szymanski, David Djurado
; modified 12/12/11 to write Hdos for adamanantanone example from Monica Jimenez
; modified 25/7/12 for molecule calcs, not done by Marek above?
; add keywords /molecule & /vamp hence if statements for reading two file types
; atom_choice is -1 as default for all H, otherwise a list starting at 0
; modified 24/05/13 to write out directionally resolved DOS with keywords /x, /y, /z or /all for Adam Brewer
; modified 01/10/13 add atom_type parameter (string e.g. 'C') to select all of this atom_type when atom_choice is [-1]
;-

; reduce number of modes to read i.e. first 6 modes in molecule case
if keyword_set(ml) then n_modes=3*n_atoms-6 else n_modes=3*n_atoms

; default parameters to read dmol file
n_ModesPerBlock=9 
text_test='Frequencies (cm-1) and normal modes'
vec_pos1=2 & vec_pos2=1                           ; position to start reading eigen/displacement vectors
atom_pos=0                                        ; position of atom label on line
if keyword_set(vmp) then begin
  n_ModesPerBlock=8 
  text_test='<> Mass-Weighted Coordinate Analysis'
  vec_pos1=3 & vec_pos2=3
  atom_pos=1
endif

; routine borrowed from read_mag.pro (Didier)
openr,u,file_in,/get_lun & line=' '
readf,u,line & wt=strtrim(line,2)
ok=1
while ok do begin & readf,u,line  & if strpos(line,text_test) ge 0 then ok=0 & endwhile
print,'Found frequencies and eigenvectors'

; read to eigenfreqs line
if keyword_set(vmp) then begin
  for iline=0,5 do readf,u,line
endif 

;Dmol --> check which is the 1st mode written to file and modify n_modes accordingly
if ~keyword_set(vmp) then begin
  readf,u,line & wt=strtrim(line,2)
  firstMode = fix((strsplit(wt, ':', /extract))[0])
  n_modes = n_modes - firstMode + 1
endif

; (square, if 1st mode = 1) matrix for vibrational modes, second index will be atom/xyz
vmat=fltarr(n_modes,n_atoms*3)
; temp matrix for blocks of modes, 9 per block, length 3*n_atoms
vtemp=fltarr(n_ModesPerBlock,n_atoms*3)
; vectors for frequencies
fvec=fltarr(n_modes) & ftemp=fltarr(n_ModesPerBlock)
; vector for atom types
atom_types=strarr(n_atoms)

; loop over n_atoms/3 blocks since 9 modes per block
for i=0,n_modes/n_ModesPerBlock-1 do begin
  print, 'reading block ',i
  ; read line of frequencies - differently depending on Dmol or VAMP
  if keyword_set(vmp) then begin
    readf,u,line & wt=strtrim(line,2)
    ftemp=strsplit(wt,' ',/extract)
  endif else begin
    if i gt 0 then readf,u,line & wt=strtrim(line,2)
    tempf=strsplit(wt,' ',/extract)
    for j=0,n_ModesPerBlock-1 do ftemp[j]=tempf[j*2+1]
  endelse
  ; copy ftemp into fvec
  ifirst=i*n_ModesPerBlock & ilast=ifirst+n_ModesPerBlock-1
  fvec[ifirst:ilast]=ftemp[*]
  print, ftemp
  ; skip blank
  readf,u,line
  ; read block of 9 modes
  ; format is 1 line with atom type followed by 2 l,ines without
  for k=0, 3*n_atoms-1,3 do begin
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[*,k+0]=tempv[vec_pos1:vec_pos1+n_ModesPerBlock-1]
    if (i eq 0) then atom_types[k/3]=tempv[atom_pos]
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[*,k+1]=tempv[vec_pos2:vec_pos2+n_ModesPerBlock-1]
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[*,k+2]=tempv[vec_pos2:vec_pos2+n_ModesPerBlock-1]
    if keyword_set(vmp) then readf,u,line
  endfor
  ; read from end of eigenvectors to next line of eigenfreqs, more lines in VAMP file
  vmat[ifirst:ilast,*]=vtemp[*,*]
  if keyword_set(vmp) then begin
    readf,u,line  &  readf,u,line &  readf,u,line
  endif 
  readf,u,line &  readf,u,line
  ; copy vtemp into vec
endfor

; last block is not as big!
print,'reading last block'
n_FullBlocks=floor(n_modes/n_ModesPerBlock)
n_ModesLeft=n_modes-n_FullBlocks*n_ModesPerBlock & print, n_ModesLeft, ' remaining modes'

;n_ModesLeft=0

if n_ModesLeft gt 0 then begin
  ; read line of frequencies
  if keyword_set(vmp) then begin
    readf,u,line & wt=strtrim(line,2)
    ftemp=strsplit(wt,' ',/extract) & print, ftemp
  endif else begin
    readf,u,line & wt=strtrim(line,2)
    tempf=strsplit(wt,' ',/extract)
    for j=0,n_ModesLeft-1 do ftemp[j]=tempf[j*2+1]
  endelse
  ; copy ftemp into fvec
  ifirst=n_FullBlocks*n_ModesPerBlock & ilast=n_modes-1
  fvec[ifirst:ilast]=ftemp[0:n_ModesLeft-1]
  print, ftemp[0:n_ModesLeft-1]
  ; skip blank
  readf,u,line
  ; read block of 9 modes
  ; format is 1 line with atom type followed by 2 lines without
  for k=0, 3*n_atoms-1,3 do begin
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[0:n_ModesLeft-1,k+0]=tempv[vec_pos1:vec_pos1+n_ModesLeft-1]
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[0:n_ModesLeft-1,k+1]=tempv[vec_pos2:vec_pos2+n_ModesLeft-1]
    readf,u,line & wt=strtrim(line,2)
    tempv=strsplit(wt,' ',/extract) & vtemp[0:n_ModesLeft-1,k+2]=tempv[vec_pos2:vec_pos2+n_ModesLeft-1]
    if keyword_set(vmp) then readf,u,line
  endfor
  ; copy vtemp into vec
  vmat[ifirst:ilast,*]=vtemp[0:n_ModesLeft-1,*]
endif

; close Dmol file
free_lun,u

print, atom_types
;print, where(atom_types eq 'N')

; prepare output
vmat=vmat^2 ; make positive

if atom_choice[0] gt -1 then begin
  h_atom_elements=atom_choice                        ; overwritten by atom_choice
  nh=n_elements(atom_choice)

endif else h_atom_elements=where(atom_types eq atom_type,nh) ;atom_type selection

; know where H's are but need to  consider XYZ coords
h_xyz_elements=intarr(nh*3)
for i=0,nh-1 do begin
  h_ref=h_atom_elements[i]*3
  h_xyz_elements[i*3]=h_ref & h_xyz_elements[i*3+1]=h_ref+1 & h_xyz_elements[i*3+2]=h_ref+2
endfor
; print, h_xyz_elements
; take H only

h_vec=vmat[*,h_xyz_elements]
h_int=fltarr(n_modes,nh)
for i=0,nh-1 do begin
  if keyword_set(xdir)   then h_int[*,i]=h_vec[*,i*3+0]
  if keyword_set(ydir)   then h_int[*,i]=h_vec[*,i*3+1]
  if keyword_set(zdir)   then h_int[*,i]=h_vec[*,i*3+2]
  if keyword_set(alldir) then h_int[*,i]=total(h_vec[*,i*3+0:i*3+2],2)
endfor

; rebin on to an energy axis
nfq=500 & resol=4000.0/nfq & print, 'delta_f is', resol; nfq pts, 4000/nfq wavenumber resolution
x_axis=findgen(nfq)*resol
fq_axis=fltarr(nfq,nh)
for j=0,n_modes-1 do begin
  if fvec[j] gt 0. then begin
    ifq=round(fvec[j]/resol) & fq_axis[ifq,*]=fq_axis[ifq,*]+h_int[j,*]
  endif
endfor

datp={x: x_axis, x_tit: 'wavenumber'}
give_datp, datp
return, fq_axis

end
