function read_doscar, file_in, iatom,ALL=ALL,fermi=fermi
;+
;******* ***********
;** User callable
;**
;** The call is e.g. w1=read_doscar('filename', atom)
;**
;**   DOSCAR file from VASP will be read
;**   total DOS block returned if iatom<0
;**   otherwise iatom determines atom block to be returned
;**   fe is the fermi energy
;-

; initialise arrays and variables
atom_info = ' ' & separator = ' ' & line_info = ' '
junk1 = ' ' & junk4 = strarr(4)     ; character variables for skipping lines

; start reading file to get variables
openr,f_in,file_in,/get_lun
readf,f_in,atom_info           ; num atoms on first line, first entry
atoms = strsplit(atom_info,/extract) & n_atoms=atoms[0]
print, 'number of atoms is  ', n_atoms
readf,f_in,junk4               ; skipping
readf,f_in,separator           ; num lines third entry
lines = strsplit(separator,/extract) & n_lines=lines[2]
print, 'number of energy points is  ', n_lines
fe=lines[3] ; Fermi energy
print, 'Fermi level is  ', fe, ' eV'
; start to read first block, total DOS
readf,f_in,line_info
; find num of columns, depends on spin pol.
lines  = strsplit(line_info,/extract)
n_cols_t = n_elements(lines) & print, 'number of columns in first block is  ', n_cols_t
ispin=0
if (n_cols_t gt 3) then begin
  print, 'spin_polarised file'
  ispin=1
endif

; create array for total dos
dos_tot=fltarr(n_cols_t, n_lines-1)
; read remaining n-1 lines
readf,f_in,dos_tot

; separate energy from dos
dos_tot_x=dos_tot[0,*] & dos_tot_x=reform(dos_tot_x)
if keyword_set(fermi) then dos_tot_x=dos_tot_x-float(fe)
; dump cumulants
dos_tot_w=dos_tot[1:2,*]
; invert spin down
if (ispin eq 1) then dos_tot_w[1,*]=-1.0*dos_tot_w[1,*]



; read first atom block
readf,f_in,separator
readf,f_in,line_info
lines  = strsplit(line_info,/extract)
n_cols = n_elements(lines) & print, 'number of columns in first ATOM block is  ', n_cols
iproj=0
if (n_cols eq 19) then begin
  print, 'spd projections'
  iproj=1
endif


; create array for 1 atom dos
dos_atom=fltarr(n_cols, n_lines-1)
; and all atoms
dos_all_atoms=fltarr(n_atoms,n_cols-1, n_lines-1)

; read remaining n-1 lines and copy to all_atoms
readf,f_in,dos_atom & dos_all_atoms[0,*,*]=dos_atom[1:n_cols-1,*]

; loop thro remaining n_atoms-1
for i=1,n_atoms-1 do begin
  readf,f_in,separator
  readf,f_in,line_info
  readf,f_in,dos_atom & dos_all_atoms[i,*,*]=dos_atom[1:n_cols-1,*]
end

if (iproj eq 1) then for ip=2,n_cols-1,2 do dos_all_atoms[*,ip,*]= -1.0*dos_all_atoms[*,ip,*]

; close file
free_lun, f_in

; prepare output
datp={x:dos_tot_x,x_tit: 'energy[eV]'}
give_datp, datp

if keyword_set(ALL) then begin
  dos_w=dos_all_atoms
  return,transpose(dos_w)
  goto, all
endif

if (iatom lt 0) then begin
  return,transpose(dos_tot_w)
endif else begin
  dos_w=dos_all_atoms[iatom,1:n_cols-1,*]
  return,transpose(dos_w)
endelse

if (atom gt n_atoms) then begin
  print, 'atom number too big'
endif
all:print,'END...'
end
