pro rotate_q_w, Ap, nbofvectors, nbofk, k_step, k_min, seed_val, fileout
;+
;** **********
;** User callable
;** routine to generate list of k-vectors for PHONON to calculate S(Q,w) coherent
;** evolution of random_rotate.tcl but uses (primitive) cell matrix (A) from PHONON
;** to calculate k-vectors in PHONON reference frame (q)
;**         k=q.B & q=k.A
;**
;** routine goes with phon_q_w which reads the d51 output file from PHONON
;**
;** PARAMETERS
;** nbofvectors 49 ;# vectors per k
;** nbofk 20       ;# number of k-values
;** k_step 0.0125  ;# interval in k-values
;** k_min 1.0      ;# first k-value
;** fileout should be the seed name of the PHONON project with d11 suffix
;**
;** METHOD
;** loop thro vectors per k and k-values
;** multiply unit vector by two rotation matrices with random angles
;** scale output vector
;-

;print input parameters
print,'nbofvectors  ',nbofvectors
print,'nbofk  ',nbofk
print,'k_step  ',k_step
print,'k_min  ',k_min

; Primitive cell matrix from PHONON d07
;Ap=[[5.85842000,-2.92921000,0.00000000], $
;   [0.00000000,5.07354055,0.00000000], $
;   [0.00000000,0.00000000,22.87200000]]

Qvec_all=fltarr(3,nbofk*nbofvectors)
for i=0,nbofk-1 do begin
  amp=k_min + i*k_step & print, i, amp
  for j=0,nbofvectors-1 do begin

    random_angle=randomu(seed_val) & a=random_angle*2.0 *!pi ;& print, a
    random_angle=randomu(seed_val) & b=random_angle*2.0 *!pi ;& print, b

    RA=[[1.0,0.0   ,0.0        ], $
       [ 0.0,cos(a),-1.0*sin(a)], $
       [ 0.0,sin(a),cos(a)     ]]

    RB=[[cos(b), -1.0*sin(b), 0.0], $
       [sin(b) , cos(b)     , 0.0], $
       [0.0    , 0.0        , 1.0]]
    R=RA##RB

    vec=[1.0,0.0,0.0]
    Rvec=vec#R            ; rotate
    Kvec=amp*Rvec         ; rescale
    Qvec=Kvec#Ap/2.0/!pi  ; into Q-space PHONON
    Qvec_all[*,i*nbofvectors+j]=Qvec

  endfor
endfor

openw,u,fileout,/get_lun
vec_str=string(nbofk*nbofvectors,format='(i8)')
printf,u,vec_str+'      1      3      1  Number of wavevect & points between'
for ij=0, nbofk*nbofvectors-1 do begin
  k_str=string(Qvec_all[*,ij],format='(3f10.4)')
  printf,u,k_str + ' K List of vectors'
endfor
close,u

end

