 pro xdat2cont, nstructures, nfirst
;+
;*** *********
;** User callable
;**
;** The call is e.g. xdat2cont, nstructures, nfirst
;**
;** files read automatically are CONTCAR and XDATCAR
;** XDATCAR contains nframes
;** CONTCAR gives scale factor, cell matrix, numbers of particles
;** output will nstructures POSCAR files i.e. POSCAR.1, POSCAR.2 etc
;** nfirst allows initial structures to be skipped
;-

; initialise arrays and variables
; get cell & system info
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
sf=1.0                          ; initialise scale factor
title=' ' & junk4=strarr(4)     ; character variables for skipping lines

junk1=' '
coord_type=' '
junk_int=0

openr,f_cont,'CONTCAR',/get_lun
readf,f_cont,title           ; skip header
readf,f_cont,sf & print, sf  ; read scale factor
readf,f_cont,cell_matrix & print, cell_matrix    ; read cell matrix

readf,f_cont,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

readf,f_cont,coord_type
free_lun, f_cont
print, 'CONTCAR read'

; get coords
openr,f_xdat,'XDATCAR',/get_lun
; extract nframes from 1st line of XDATCAR
readf,f_xdat,format='(3i4)',junk_int, junk_int,nframes  & print, nframes
coords     = fltarr(nframes, 3, natoms)
coords_tmp = fltarr(3, natoms)
readf,f_xdat,junk4 & print, junk4
for i=0,nframes-1 do begin
  readf, f_xdat, junk1 & readf, f_xdat, coords_tmp
  coords[i,*,*] = coords_tmp[*,*]
endfor
free_lun, f_xdat
print, 'XDATCAR read'

jstep=floor((nframes-nfirst)/nstructures)
for j=0,nstructures-1 do begin
  jj=j*jstep+nfirst & print, jj
  filename='POSCAR.'+ string(j) & filename=strcompress(filename,/remove_all)
  openw, f_out, filename, /get_lun

  printf, f_out, title
  printf, f_out, sf
  printf, f_out, cell_matrix
  printf, f_out, natoms_string
  printf, f_out, coord_type

  coords_tmp=coords[jj,*,*] & coords_tmp=reform(coords_tmp)
  printf, f_out, coords_tmp
  free_lun, f_out
endfor

end
