function xdatcar, real=r, file_out
;+
;******* *******
;** User callable
;**
;** The call is e.g. w1=xdatcar(/real, 'filename')
;**
;** files read automatically are CONTCAR and XDATCAR
;** XDATCAR contains nframes
;** CONTCAR gives scale factor, cell matrix, numbers of particles
;** 'filename' will contain list of coordinates for python/xMoldyn
;** keyword /real used to write continuous real (Cartesian) coords
;** to workspace, default is fractional
;-

; initialise arrays and variables
; get cell & system info
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
sf=1.0                          ; initialise scale factor
junk1=' ' & junk4=strarr(4)     ; character variables for skipping lines
junk_int=0

openr,f_cont,'CONTCAR',/get_lun
readf,f_cont,junk1           ; skip header
readf,f_cont,sf & print, sf  ; read scale factor
readf,f_cont,cell_matrix & print, cell_matrix    ; read cell matrix

readf,f_cont,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

free_lun, f_cont
print, 'CONTCAR read'

; get coords
openr,f_xdat,'XDATCAR',/get_lun
; extract nframes from 1st line of XDATCAR
readf,f_xdat,format='(2i4,i5)',junk_int, junk_int,nframes  & print, nframes
coords     = fltarr(nframes, 3, natoms)
coords_tmp = fltarr(3, natoms)
readf,f_xdat,junk4
for i=0,nframes-1 do begin
  readf, f_xdat, junk1
  readf, f_xdat, coords_tmp
  coords[i,*,*] = coords_tmp[*,*]
endfor
free_lun, f_xdat
print, 'XDATCAR read'

coords_frac = coords ; save fractional coords for output

; test for continuity of coords
old_coords = fltarr(3, natoms)
old_coords[*,*]=coords[0,*,*]
for i=1, nframes-1 do begin
  for k=0,natoms-1 do begin
    for j=0,2 do begin
      if old_coords[j,k]-coords[i,j,k] gt 0.5 then begin
        coords[i,j,k]=coords[i,j,k]+1.0 ; coord going from ~1 to ~0
        ;print,'coord changed', i, j, k
      endif
      if coords[i,j,k]-old_coords[j,k] gt 0.5 then begin
        coords[i,j,k]=coords[i,j,k]-1.0 ; coord going from ~0 to ~1
        ;print,'coord changed', i, j, k
      endif
    endfor
  endfor
  old_coords[*,*]=coords[i,*,*] ; update coordinate memory for next frame
endfor
print, 'CONTINUITY checked'

; multiply cell matrix and frac coords
for i=0, nframes-1 do begin
  coords_tmp[*,*] = coords[i,*,*]
  coords[i,*,*] = sf * matrix_multiply(cell_matrix,coords_tmp)
endfor

; write output of continuous coords for infinite universe treatment in python/xMoldyn
openw, f_out, file_out, /get_lun
;velocity = '  0.0  0.0  0.0' ; dummy velocities for xMoldyn
for i=0, nframes-1 do begin
  for j=0, natoms-1 do begin
    printf, f_out, i+1,' ', coords[i,0, j],' ', coords[i,1, j],' ', coords[i,2, j] ; velocity
  endfor
endfor
free_lun, f_out

; prepare info to be return with data
title='' & title='TRAJECTORY LENGTH  ='+string(nframes)
datp={W_TIT:  title}
give_datp, datp

if keyword_set(r) then begin
  return, coords ; cartesian coords can be used to calculate m.s.d's, g(r) etc in other routines
endif else begin
  return, coords_frac ; put fractional coords {0,1} in workspace
endelse


end
