function xdatcar5, nframes, real=rl, cont=ct, xfile=xf
;+
;******* ********
;** User callable
;**
;** The call is e.g. w1=xdatcar(/real, 'filename')
;**
;** files read automatically are CONTCAR and XDATCAR
;** XDATCAR contains nframes
;** CONTCAR gives scale factor, cell matrix, numbers of particles
;** 'filename' will contain list of coordinates for python/xMoldyn
;** keyword /real used to write continuous real (Cartesian) coords
;**  to workspace, default is fractional
;** keyword /cont used to write continuous fractional coords
;**  to workspace
;** modified Jan 2010 to read long trajectories using long integer frame loops
;** check to see whether cfile and xfile are set to determine
;** contcar and xdatcar files
;** only to read VASP 5.0 files
;** nframes as parameter, all else from input files 21/7/2010 JDF
;** modified 29/6/2012 by MJ to read XDATCAR file from VASP5.2, no need for CONTCAR
;-


; initialise arrays and variables
; get cell & system info
cell_matrix = fltarr(3,3)       ; matrix notation - cols then rows
natoms_string= ' '              ; string to read
atoms_label= ' '		; string to read
sf=1.0                          ; initialise scale factor
junk1=' ' & header1=' ' & header2=' ' & header3=' ' & header4=' '	; character variables for skipping lines
junk_int=0

if n_elements(xf) eq 1 then contcar_file2read=cf else contcar_file2read='XDATCAR'
openr,f,contcar_file2read,/get_lun
readf,f,junk1           ; skip header
readf,f,sf & print, sf  ; read scale factor
readf,f,cell_matrix & print, cell_matrix    ; read cell matrix

readf,f,atoms_label
readf,f,natoms_string  & print, natoms_string     ; read and analyse numbers of atoms
atoms_list  = intarr(10)
reads,natoms_string+' 0 0 0 0 0 0 0 0 0',atoms_list
atoms_list  = atoms_list[where(atoms_list gt 0)]
n_atomtypes = n_elements(atoms_list) &  print, n_atomtypes

natoms=total(atoms_list) & print, natoms

print, 'HEADER read'

;coordinates array
coords     = fltarr(nframes, 3, natoms)
coords_tmp = fltarr(3, natoms)

;calculate cell parameters
a1=sf*cell_matrix[0,0] & a2=sf*cell_matrix[1,0] & a3=sf*cell_matrix[2,0]
b1=sf*cell_matrix[0,1] & b2=sf*cell_matrix[1,1] & b3=sf*cell_matrix[2,1]
c1=sf*cell_matrix[0,2] & c2=sf*cell_matrix[1,2] & c3=sf*cell_matrix[2,2]
a=sqrt(a1^2+a2^2+a3^2) & b=sqrt(b1^2+b2^2+b3^2) & c=sqrt(c1^2+c2^2+c3^2)
print, a,' ',b,' ',c
aa=(b1*c1+b2*c2+b3*c3)/(b*c) & alpha=acos(aa)*180.0/!pi
bb=(a1*c1+a2*c2+a3*c3)/(a*c) & beta =acos(bb)*180.0/!pi
cc=(a1*b1+a2*b2+a3*b3)/(a*b) & gamma=acos(cc)*180.0/!pi
print, alpha,' ',beta,' ',gamma

; get coords
for i=0L,nframes-1 do begin
  readf, f, junk1
  readf, f, coords_tmp
  coords[i,*,*] = coords_tmp[*,*]
endfor
free_lun, f
print, 'XDATCAR read'

coords_frac = coords ; save fractional coords for output

; test for continuity of coords
old_coords = fltarr(3, natoms)
old_coords[*,*]=coords[0,*,*]
for i=1L, nframes-1 do begin
  for k=0,natoms-1 do begin
    for j=0,2 do begin
      if old_coords[j,k]-coords[i,j,k] gt 0.5 then begin
        coords[i:nframes-1,j,k]=coords[i:nframes-1,j,k]+1.0 ; coord going from ~1 to ~0
        ;print,'coord changed', i, j, k
      endif
      if coords[i,j,k]-old_coords[j,k] gt 0.5 then begin
        coords[i:nframes-1,j,k]=coords[i:nframes-1,j,k]-1.0 ; coord going from ~0 to ~1
        ;print,'coord changed', i, j, k
      endif
    endfor
  endfor
  old_coords[*,*]=coords[i,*,*] ; update coordinate memory for next frame
endfor
print, 'CONTINUITY checked'

; multiply cell matrix and frac coords
coords_real = coords ; save fractional coords for output
; manipulate cell_matrix (last, for correct transformation to real coords)
cell_matrix=transpose(cell_matrix) & print,size(cell_matrix), size(coords_tmp), size(coords_real)
for i=0L, nframes-1 do begin
  coords_tmp[*,*]    = coords_real[i,*,*]
  coords_real[i,*,*] = sf * matrix_multiply(cell_matrix,coords_tmp)
endfor
;manipulate cell_matrix to return
cell_matrix=transpose(cell_matrix)
print, 'frac coord calc finished'

;set-up workspace parameters in datp
;construct datp structure to go with atomic coords
;first make titles of atom types and cumulative atom list
atomslist='Atoms: '+atoms_label
atomscnum='Cnumber: '
atoms_list_start=-1
for i=0,n_atomtypes-1 do begin
  atomc=atoms_list_start+atoms_list[i] & atoms_list_start=atomc
  atomscnum=atomscnum+string(atomc)+' ' & print,atomscnum
endfor
pars=fltarr(14)      & ptxt=strarr(14)
pars[0] =natoms      & ptxt[0] ='number of atoms'
pars[1] =n_atomtypes & ptxt[1] ='number of atom types'
pars[2] =999         & ptxt[2] = atomslist
pars[3] =999         & ptxt[3] = atomscnum
pars[4] =a           & ptxt[4] ='cell pars a'
pars[5] =b           & ptxt[5] ='cell pars b'
pars[6] =c           & ptxt[6] ='cell pars c'
pars[7] =alpha       & ptxt[7] ='angle alpha'
pars[8] =beta        & ptxt[8] ='angle beta'
pars[9] =gamma       & ptxt[9] ='angle gamma'
pars[10]=sf          & ptxt[10]='cell matrix (in PV) scale factor'
pars[11]=0 	     & ptxt[11]=header2
pars[12]=0           & ptxt[12]=header3
pars[13]=0	     & ptxt[13]=header4

title='' & title='TRAJECTORY LENGTH  ='+string(nframes)
DATP={W_TIT:title,    $
      X_TIT:'frame number',    $
      Y_TIT:'XYZ',             $
      Z_TIT:'atom number',     $
      P:      pars,   $
      PAR_TXT:ptxt,  $
      PV:     cell_matrix   }

give_datp, DATP

; fractional, fractional continuous or real continuous coordinates to be returned
if keyword_set(rl) then return, coords_real  else $
if keyword_set(ct) then return, coords       else   return, coords_frac

print, 'finished!'

end
