function brisp_bc2k, W_in, CH_RANGES=chrg, MASK=msk ,KERNEL=kern
;******* **********
;+
;** Calculate the 2D background from a bad statistic sample averaging
;** in a pixel window (kernel) the counts falling in a given channel range
;**
;** input: W_in= TOF sample (3D)
;**        chrg= TOF ranges containing only backgrounds , default all channels
;**              [t1a,t1b  , t2a,t2b , .....]
;**        msk = the mask , default is no mask
;**        kern= mean size [dx,dy] , [dxy] accepted , default=3
;**              shoulb be odd
;** output:The background  (2D)
;-

;Parameters checking
;-------------------
 s= size(W_in)
 if n_elements(msk)  eq 0 then msk =fltarr( s[1],s[2] )
 if n_elements(kern) eq 0 then kern=[ 3  , 3]
 if n_elements(kern) eq 1 then kern=[kern,kern]
 if n_elements(chrg) lt 2 then chrg=[ 0        , s[3]-1   ]

 if s[0] ne 3                                 then return,0
 if s[1] * s[2]           ne n_elements(msk)  then return,0
 if n_elements(kern)/2*2  ne n_elements(kern) then return,0
 if n_elements(chrg)/2*2  ne n_elements(chrg) then return,0

 chrg=chrg>0<(s[3]-1)

;Total channel ranges
;--------------------
 W_tmp=total(W_in[*,*,chrg[0]:chrg[1]],3)
 nch  =chrg[1]-chrg[0]+1.
 for i=2,n_elements(chrg)-1,2 do begin
     W_tmp=W_tmp+total(W_in[*,*,chrg[i]:chrg[i+1]],3)
     nch  =nch  +chrg[i+1]-chrg[i]+1.
 endfor
 W_tmp=W_tmp/nch
;--------------------

 s1   = s[1]-1  & s2   = s[2]-1
 cnt  = msk *0.
 W_out= cnt     & mski = 1.-msk
 ka   = (kern[0]-1)/2  & kb = (kern[1]-1)/2

;Do the work
;-----------
 for j=0,s2 do for i=0,s1 do begin
     tmpd=W_tmp[(i-ka)>0 : (i+ka)<s1 , (j-kb)>0 : (j+kb)<s2]
     mskd=mski[ (i-ka)>0 : (i+ka)<s1 , (j-kb)>0 : (j+kb)<s2]
     W_out[i,j]= total(tmpd*mskd) / (total(mskd)>1)
 endfor

 return,W_out*mski
 end
