;******* ************
function brisp_center, M , XYbeam_stop=XYbs , Rb_stop=Rb_stop, mask_in=filein , mask_out=Masq , $
                       noCenter=noctr , Center=c_in, ang_sect=ang_sect
;+
;******* ************
;** User callable
;** INPUT:  M is a 2D or 3D count from an attenuated direct beam.
;**         XYbs should be the beam-stop size [X,Y] in pixel unit seen during experiment.
;**         Rbs  should be the beam-stop rays [X,Y] in pixel unit seen during experiment: CHANGED   **************************
;**         Regions is a (4,n) matrix uses to exclude regions [[x1,y1,nx1,ny1],[x2,y2,nx2,ny2]...]
;**         /stubes if set then mask short tubes also (BRISP ONLY): not anymore: default mask is with corners masked! ********
;**         /nocenter if set then do not fit center, assume ([s1,s2])/2.
;**         center=[cx,cy] if set then do not fit center
;**
;** OUTPUT: cxy=[Xcenter,Ycenter] in pixel position (from 0.)
;**         mask_out is the 2D mask for M (returned only if XYbs or Rbs and/or Regions are present.
;**
;**         ********************   Modification made on 11/03/09 ************************************
;**         bad tubes (stored in a .pro file=filein) are now directly imported in brisp_center
;**         c=brisp_center(W1,Rb_stop=6,mask_in='msk256',mask_out=W40)
;**
;**         Rbs is the beam-stop ray in cm: works for 256 and 128 pixels: care in other cases!!!
;**         default = 6cm
;**
;**         If no mask, default mask is with 4 corners masked
;**
;**			The subdivision of the tube in a number of pixels lower than
;**			the default (256) case is taken into account with the parameter kp
;**			Mask ready only for the case 128 pixels!!!!
;**			No regrouping of tubes allowed
;**			
;**         ********************   Modification made on 11/07/10 ************************************
;**         rough masking in angular sector is possible, though not nicely written
;**         typial coiche ang_sect=[-45, 45] or [45, 90] (the latter implies also [-90,-45])
;**         c=brisp_center(W1,Rb_stop=6,mask_in='msk256',mask_out=W40,ang_sect=[-45,45])
;-

forward_function str_fit, brisp_dist3
cxy=[0.,0.]
;*****************************CHECK FOR PARAMETERS******************************
s=size(M)   & s12= s[1]* s[2]
IF (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN, [0.,0.]
IF (s[0] eq 3) then W=total(M,3) else W=M

pix2cm = [.428*256/s[1] ,1.27] ; conversion factor pixel --> cm
;
;	****  when pixels are not 256 (tipically 128)
kp=s[1]/256.; scaling between pixel actual size and reference (default=256) size
s1=s[1]-1 & s2=s[2]-1 & b=15<(s2/2) & a=kp*58<(s1/2) ; modify corners
a=round(a);

;***** The Fits *****
if n_elements(c_in) ne 2 then begin
 if not keyword_set(noctr) then begin
  if s[2] eq 160 then Xproj= total(W[*,b+1:s2-b-1],2) else Xproj= total(W,2)
  if s[2] eq 160 then Yproj= total(w[*,b+1:s2-b-1],1) else Yproj= total(W,1)
  Xpar = str_fit(Xproj,Bkgd=1,Xin=findgen(n_elements(Xproj)),/gauss) ;Gaussian with a linear background
  Ypar = str_fit(Yproj,Bkgd=1,Xin=findgen(n_elements(Yproj)),/gauss)
  if s[2] eq 160 then cxy  =[Xpar[2],Ypar[2]+b+1] else cxy=[Xpar[2],Ypar[2]]
 endif  else cxy =([s1,s2])/2. ; for tests...
endif else   cxy =  c_in

;***** The Mask *****
if (n_elements(XYbs) eq 2) or (n_elements(mask_in) eq 1) or (n_elements(Rb_stop) eq 1) or $
   (keyword_set(Stub))   then  begin
   MASQ=W*0
   MASQ[0:a,0:b]=1 &MASQ[s1-a:s1,0:b]=1 &MASQ[0:a,s2-b:s2]=1 &MASQ[s1-a:s1,s2-b:s2]=1
    ;square beam_stop
    ;****************
    if n_elements(XYbs) eq 2    then begin
       aa=round(cxy[0]-XYbs[0]/2.)  &  bb=round(aa+XYbs[0]) & aa=aa>0<s1 & bb=bb>0<s1
       cc=round(cxy[1]-XYbs[1]/2.)  &  dd=round(cc+XYbs[1]) & cc=cc>0<s2 & dd=dd>0<s2
      MASQ[aa:bb,cc:dd]=1
    endif
    ;ellipsoid beam_stop
    ;*******************
;
; for overall compatibility, use of radius [cm] while keeping the old routine: R=[pixel(256), tube]
; and scale (kp) with final pixel size
if n_elements(Rb_stop) eq 1 then begin
Rbs=fltarr(2,1)
 Rbs[0]=Rb_stop/0.428*kp
  Rbs[1]=Rb_stop/1.2712
  endif  else Rbs=[14.02,4.72]; default = 6cm

        if n_elements(Rbs)  eq 2    then begin & unit=min(Rbs)
       pixdist  =brisp_dist(MASQ,Center=cxy,pixSize=reverse(Rbs),/pixel)
       idx      =where(round(pixdist) le unit)
      MASQ[idx]=1
    endif
    ;excluded tubes (regions)
    ;************************

if n_elements(filein) eq 1 then begin

I=execute(filein+',Regions')

    if n_elements(Regions) ge 4 then for i=0,n_elements(Regions)/4-1 do begin
       regx=[Regions[0,i],Regions[0,i]+Regions[2,i]-1]>0<s1
       regy=[Regions[1,i],Regions[1,i]+Regions[3,i]-1]>0<s2
      MASQ[regx[0]:regx[1],regy[0]:regy[1]]=1
    endfor
   endif

endif

;***** Angular Sector of the Mask *****
; not very nicely written: I study here two cases:
; mask in [-45,45] ang range or in [-90,-45] + [45,90] ang range

if (n_elements(ang_sect) eq 2) then  begin

Mtheta=M*0;
ypix=-((findgen(s[1])+0.5)-cxy[0])*pix2cm[0]; pix  coordinate in cm : 
					    ; take the negative since the pixel are ordered from top to bottom
xtube=((findgen(s[2])+0.5)-cxy[1])*pix2cm[1]; tube coordinate in cm

Mtheta=ATAN(MATRIX_MULTIPLY(ypix,1./xtube))*!RADEG; matrix with angles [deg]
idth0=where(Mtheta GE ang_sect[0] AND Mtheta LE ang_sect[1])
	IF min(ang_sect) GT 0 THEN BEGIN
	idth1=where(Mtheta GE -ang_sect[1] AND Mtheta LE -ang_sect[0])
        MASQ[idth1]=1
        ENDIF
MASQ[idth0]=1
endif

return, cxy

end
