function brisp_cor, Win ,pathcal ,errors=Eout
;+
;******* *********
;** User callable
;** Correct BRISP frames: Border corr. due to electronic offsets.
;**
;**
;** Win    =raw input data 2D or 3D
;** pathcal=path for Calibration file "brisp_tubes.cor"
;** errors =if argument is present then output errors in Eout (not used)
;-

common cor2_brisp, ofs1 ,ofs2

;Check for correct input
;***********************
siz=SIZE(Win) & sy=160  & sx =256 &  short=138 & nsa=15 & nsb=144
osh=(sx-short)/2
if (siz[0] eq 3  )   then sz =siz[3] else sz =1
if arg_present(Eout) then Eok=1      else Eok=0
if (siz[1] ne sx ) or (siz[2] ne sy) then return,Win
if n_elements(pathcal) ne 1 then pathcal=''

;Read the file (first time)
;**************************
if n_elements(ofs1) le 1 then begin & on_ioerror,miscal & ok=0 & u=-1
   openr,u,pathcal+'brisp_tubes.cor',/get_lun
   print, ' Reading brisp_tubes.cor correction file ...' & wait ,.01
   tmp=fltarr(3,sy)
   readf,u,tmp & ok=1
   ofs1=ceil(reform(tmp[1,*]))-1   & ofs2=floor(reform(tmp[2,*]))-1
   miscal:if u gt 0 then free_lun,u
   if (not ok) then begin & print,!error_state.msg & return,Win & endif
endif

;Corrections
;***********
print,' Border correction started ...' & wait ,.01
Wout=Win*0. & a=systime(1)
dids=1 ; 0 or 1 -> choice of the algorithm

if dids then begin
for it=0,sy-1 do begin     ;for each tube
    v  = reform(Win[ofs1[it]:ofs2[it],it,*])
    m  =(size(v))[1]
    if  (it le nsa) or (it ge nsb) then begin & x=short & ofs=osh
                             endif else begin & x=sx    & ofs=0 & endelse
    r  = findgen(x)*((m-1.0)/((x-1.0) > 1.0))        ;Grid points in V
    rl = long(r)
    dif= v[1:*,*]-v
    for iz=0,sz-1 do begin ;for each frame
        Wout[ofs,it,iz]=V[rl,iz] + (r-rl)*dif[rl,iz] ;interpolate (linear)
    endfor
endfor

endif else begin
for it=0,sy-1 do begin     ;for each tube
    v  = reform(Win[ofs1[it]:ofs2[it],it,*])
    if  (it le nsa) or (it ge nsb) then begin & x=short & ofs=osh
                             endif else begin & x=sx    & ofs=0 & endelse
    for iz=0,sz-1 do begin ;for each frame
        Wout[ofs,it,iz]=congrid(v[*,iz],x,/interp)  ;interpolate (linear)
    endfor
endfor
endelse

print,' End of border correction ',systime(1)-a,' seconds'
if Eok then begin
  ;Eout=Eout>1.
endif
return,Wout>0.
end
