FUNCTION brisp_zone_tof, win, cal_fil_in, longueur
;+
;******* **************
;**
;** The call is w2=brisp_zone(w1, 'calibration_file_in')
;**
;** w2 will contain data corrected to remove the dead zones
;** w1 is raw data [tof,det_x, det_y]
;** w1 is extracted from raw data with reform(w1,tof,256,96)
;** detector dimensions are typically 256*96
;** of 96, 8 at each end are dead, leaving 80 active channels
;-

;define number of tube and height pixels
take_datp,datp
n_tof   =(size(datp.x))[1]
n_detx  =(size(datp.y))[1] ; x dim of detector
n_dety  =(size(datp.z))[1] ; y dim
print, n_tof, n_detx, n_dety
;define output data array, 80 active channels
n_ch=80
wout=fltarr(n_tof,n_detx,n_ch)
n_first=(n_dety-n_ch)/2 & n_last=(n_dety-n_ch)/2-1+n_ch & print, n_first, n_last
wout=win[*,*,n_first:n_last]

;read calibration file
zone_ends=fltarr(2,n_ch)
openr,f_in,cal_fil_in,/get_lun
readf,f_in,zone_ends
free_lun, f_in
print, 'calibration file read'

;loop thro active channels and expand active zones onto n_detx (256) points
; worried about errors? correct them with congrid
tempgrid=fltarr(n_detx)
for j=0,n_tof-1 do begin
  for i=0, n_ch-1 do begin
    tempgrid=wout[j,*,i]
    wout[j,*,i]=congrid(tempgrid[ceil(zone_ends[0,i]):floor(zone_ends[1,i])],n_detx,/interp)
  endfor
endfor

; other variables to give back with wout
MOD_DATP,datp,'Y',findgen(n_detx)*longueur/n_detx ;Replace X values for detector
MOD_DATP,datp,'Y_TIT','length[mm]'                ;Replace X title
;MOD_DATP,datp,'Y',findgen(n_detx)*longueur/n_detx ;Replace X values for detector
MOD_DATP,datp,'Z_TIT','height[mm]'                ;Replace X title
GIVE_DATP,datp ;Give variables back to Lamp

return, wout

end
