;******* ************
function flat_det, M , CENTER=xy0,PixSize=xypsiz, L2=dst
;+
;******* ********
;** User callable
;** to correct flat detector data for solid angle (FF 2007)
;**
;**     Call:   w2=flat_det(w1,Center=xy0,L2=dst.)
;**     ****
;** Input:  M      =2D or 3D data matrix.
;** *****   xy0    =[x0,y0] center determined by "brisp_center.pro"
;**         xypsiz =size of the pixel in Cm. Default is [.428,1.27]
;**         dst    =Distance sample-detector in Cm.
;**
;** Output: Wout
;** ******   IF M=2D then Z is the 2D matrix of angles
;-

Wout=0
;*****************************CHECK FOR PARAMETERS******************************
M=FLOAT(M)
s=size(M)   & s12= s[1]* s[2]

IF (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN, Wout
TAKE_DATP, datp                                    ;workspace variables
if n_elements(dst)    ne 1 then dst=400.           ;Distance sample-detector in Cm.
if n_elements(xypsiz) ne 2 then xypsiz=[.428,1.27] ;[vertical,horizontal] default.

IF s[0] eq 2 then M2D=M ELSE M2D=M[*,*,0]

r_pix=brisp_dist(M2D,CENTER=xy0); pixel-det_centre distance in cm
teta=ATAN(r_pix/dst);      angles[rad] seen by each pixels
L2_pix=dst/cos(teta);        pixel-sample distance in cm

dOmega=cos(teta)^3;*xypsiz[0]*xypsiz[1]/dst^2

MM=M
IF s[0] eq 2 then BEGIN ; case 2D
MM=M/dOmega
ENDIF ELSE BEGIN ;case 3D
       FOR ich=0,s[3]-1 DO MM[*,*,ich]=M[*,*,ich]/dOmega
endelse

Wout=MM

IF s[0] eq 2 then Z=TETA*!RADEG ;Make unit angles
IF s[0] eq 2 then  print,'[teta min=',MIN(Z),'],      [teta max=',MAX(Z),']'

;
;;***** Return All *****
;
IF s[0] eq 2 THEN datp.z_tit="Angles" & MOD_DATP ,datp,"Z",Z

GIVE_DATP,datp
return,Wout
end
