;******* ************
	PRO out_sqw_brisp, W_in, fileout=fileout
;******* ************
;**      Export ASCII files containing Sqw output of sqw_pix (FF, April 2010)
;        (based on output_allqs.pro)
;
;*** takes a 2-D workspace from sqw_pix.pro and writes it to a series of files, 
;*** one for each Q. y-axis must be Q and x-axis energy transfer.
;
;**** output files have names <file>.q005   Q=0.05 A-1
;****			      <file>.q010   Q=0.10 A-1
;****			       etc...
;
;ARGUMENTS
; fileout : name of output file
;
;DIMENSIONS
; w_in(dE,dQ) output from sqw_pix.pro
;
;COMMAND SYNTAX
; out_sqw_brisp, w17, file='namefile';
;
;************** INPUT WORKSPACE - CHECK FOR PARAMETERS  *********************

TAKE_DATP, datp                 ; workspace variables

Sqo=w_in & eSqo=datp.e;
Eps=datp.x & Qv=datp.y;
sw=size(Sqo) & nEps=sw[1] & nQ=sw[2]

    ON_IOERROR, getout

    FOR iQ=0,nQ-1 DO BEGIN
       QQ=Qv[iq]
       Qstring=STRTRIM(STRING(FIX(100.*QQ)),2)
       IF (QQ LT 0.1) THEN Qstring='q00'+Qstring $
       ELSE IF (QQ LT 1.0) THEN Qstring='q0'+Qstring $
       ELSE Qstring='q'+Qstring
       PRINT,'Saving to ',STRTRIM(fileout,2),'.',Qstring
       S=FLTARR(nEps) & S[*]=Sqo[*,iQ]
       dS=FLTARR(nEps)    & dS[*]=eSqo[*,iQ]
       ii=WHERE(dS GT 0, n)
       IF (n GT 0) THEN BEGIN
         E=Eps[ii]    & S=S[ii]   & dS=dS[ii]
         OPENW, u, fileout+'.'+Qstring, /GET_LUN
         ON_IOERROR, readout
         ch=STRTRIM(STRING(QQ),2)  & jj=STRPOS(ch,'.') & Q=STRMID(ch,0,jj[0]+4)
         PRINTF, u, datp.w_tit+' | S[Q,E] at Q='+Q+' Ang-1'
         PRINTF, u, 'numor: ', datp.other_tit
         PRINTF, u, datp.x_tit, ', S[Q,E], err_S(Q,E)'
         PRINTF, u, 'N points = '
         PRINTF, u, n
         FOR ii=0,n-1 DO PRINTF, u, E[ii], ' ', S[ii],' ', dS[ii]
         FREE_LUN, u 
         readout: FREE_LUN, u
      ENDIF

    ENDFOR
getout:

give_datp, datp
return
end
