function raw_bck_brisp, W_in, bck_ch=chrg, MASK=msk ,KERNEL=kern
;******* *********
;
;**  Call: W4 = raw_bck_brisp(w2,bck_ch=[1500,1600],mask=w40,kernel=[9,6]); 
;
;** 1) Calculate the average 2D bck bckg (ambient) counts in a given channel range 
;**    The bckg channel range is estimated with brisp_rad.pro
;** 2) Rescale it to the total channel frame: obtain, for each pixel, 
;**    the total nr of channels (ntot_bck) with I(ch) NE 0 (practically always Ibck(ch)=1)
;** 3) For each pixel, generate bck counts I=1 in ntot_bck channels randomly chosen 
;** 4) Obtain, for each pixel, a random I(ch) with the experimental (kernel averaged) bckg counting rate   
;**    
;** input: W_in= TOF sample (3D)
;**        chrg= TOF ranges containing only backgrounds , default all channels
;**              possible to select multiple channel ranges [t1a,t1b  , t2a,t2b , .....]
;**        msk = the mask , default is no mask
;**        kern= mean size [dx,dy] , [dxy] accepted , default=3. Should be odd (symmetric kernel)
;**              To obtain a "square" kernel: ratio dx:dy= 2:1 (256 pixels) or 3:2 (128 pixels)
;** output: background raw data (3D) input of Sqw_pix.pro 
;
;**************   (DR, FF, April 2010)
;
;Parameters checking
;-------------------
 s= size(W_in)
 if n_elements(msk)  eq 0 then msk =fltarr( s[1],s[2] )
 if n_elements(kern) eq 0 then kern=[ 3  , 3]
 if n_elements(kern) eq 1 then kern=[kern,kern]
 if n_elements(chrg) lt 2 then chrg=[ 0        , s[3]-1   ]
 if n_elements(tol)  eq 0 then tol =0.

 if s[0] ne 3                                 then return,0
 if s[1] * s[2]           ne n_elements(msk)  then return,0
 if n_elements(kern)/2*2  ne n_elements(kern) then return,0
 if n_elements(chrg)/2*2  ne n_elements(chrg) then return,0

 chrg=chrg>0<(s[3]-1); channel range where the bck is evaluated

;************************************
; integrate in time in the given channel range and normalise
;--------------------
 W_tmp=total(W_in[*,*,chrg[0]:chrg[1]],3)
 nch  =chrg[1]-chrg[0]+1.
; for i=2,n_elements(chrg)-1,2 do begin
;     W_tmp=W_tmp+total(W_in(*,*,chrg(i):chrg(i+1)),3)
;     nch  =nch  +chrg(i+1)-chrg(i)+1.
; endfor
 W_tmp=W_tmp/nch; 2D pixel bckg per channel 
 
;
;***********************************
; calculate kernel average 2D expected nr ok bck counts in the whole time frame
;
 s1 = s[1]-1  & s2 = s[2]-1 & s3 = s[3]-1
 W_bck= msk*0.  &  W_out=W_in*0  & mski = 1.-msk
 ka   = (kern[0]-1)/2  & kb = (kern[1]-1)/2
;
 for j=0,s2 do for i=0,s1 do begin
tmpd=W_tmp[(i-ka)>0 : (i+ka)<s1 , (j-kb)>0 : (j+kb)<s2]; kernel data 
mskd= mski[(i-ka)>0 : (i+ka)<s1 , (j-kb)>0 : (j+kb)<s2]; kernel mask
W_bck[i,j]=total(tmpd*mskd)/(total(mskd)>1)*mski[i,j]; for each pixel: kernel average of bckg counting rate 
                                           ; per channel in the given channels range 
if W_bck[i,j] GT 0 then begin

ntot_bck=round(s[3]*W_bck[i,j])	; expected nr of bckg counts in the whole time frame
I_bck0=floor(W_bck[i,j])      ; take into account the rare high background case where the average bckg per pixel >1 
                              ; for instance whne the total bckg count in a 1800 chnnels frame is > 1800
                              ; in this case, it is first distributed a costant offset value and later
                              ; the exceeding part (in a random way)
W_out[i,j,*]=I_bck0           ; distribute the offset value of bkg over all channels (n most cases it is = 0

;************************************
; generate a bck in ntot_bck channels randomly chosen

	ran_all=randomu(seed,s[3])	; random vector with n_ch_ok elements
	ind_ran=sort(ran_all)		; sort the indices 
	chmax=ntot_bck-1-I_bck0*s[3]; distribut the exceeding bckg counts
	ran_ok=ind_ran[0:chmax]	; extract+ a nr of indices = to the expected nr of bckg counts exceeding the offset
	W_out[i,j,ran_ok]= W_out[i,j,ran_ok]+1.
endif
 endfor
  return,W_out
  end
