;******* ************
function sqw_pix_brisp, W_in , XY0=xy0, MASK=msk, file_el=file_el, dE=dEps, $
                    dQ=dQ,Qmax=Qmax,Ib=Wbck,kikf=kikf_v,w_el=w_el_v
;+
;******* ******
;** User callable
;**       Modifications (April 2011, FF): 
;**     1) time channels are not anymore discrete (1,2,...) but randomised
;**         If Ib used, ambient background is subtracted.         
;**     2) possibility of not making the ki/kf correction (default: ki/kf factor is included)
;**     3) Elastic peak position is determined through an analisys of the channel intensity pixel by pixel,
;**        and not anymore after a radial integratrion procedure. Input data can be the same W_in, 
;**        if statistic is sufficient, or another worksheet (eg vanadium or sample) 
;**     4) Data are better centred in energy
;**
;** OUTPUT: Sqw from raw data, converting I(theta, channels) of each pixels, without rebinning.
;**         If Ib used, ambient background is subtracted. With r without ki/kf factor.         
;**            
;** *****
;** INPUT:  W_in    = 3D raw data matrix
;** *****   xy0     = [x0,y0] center determined by "brisp_center.pro"
;**         MASK    = mask determined manually and by "brisp_center.pro"
;**                   contains 1's for masked values otherwise 0's(default)
;**         
;**
;**** OPTIONAL INPUTS
;**      Ib     = 3D worksheet containing background data randomnly generated by raw_bck.pro   
;**      w_el   = 3D worksheet containing "good" statistic data. Needed to evaluate elastic channel position as a function of theta    
;**      dE     = energies step (default 0.5 meV)   
;**      kikf   = 'no': without ki/kf. Default is with ki/kf   
;**	     dQ     = Q step (default 0.1 Ang^-1)   
;**	     Qmax   = maximum Q at which Sqw is determined (default 2.0 Ang^-1)   
;**          
;**** DIMENSIONS: Wout(dE,dQ)
;**
;** Call: w20=sqw_pix(w2,XY0=c,MASK=w40,kikf='no',w_el=w2,Ib=w4,dE=0.5,dQ=0.1,Qmax=2.0)
;**
;**	(FF, April 2010)
;**
;**	  NUOVO RANDOM SUI CHANNELS
;-

; ******************** USEFUL CONVERSION FACTORS ******************************

    c1 = 5.22697          ; E(meV) =c1*V(m/ms)^2 for neutron
    c2 = 2.07193571       ; E(meV) =c2*k(A^-1)^2 for neutron
    c3 = 3.956076         ; V(m/ms)=c3/lambda(A) for neutron
    c4 = 81.8066          ; E(meV) =c4/lambda(A)^2 for neutron

;************** INPUT WORKSPACE - CHECK FOR PARAMETERS  *********************

Wout=0 & M=W_in
TAKE_DATP, datp                 ; workspace variables
par = datp.p
s=size(M)   & s12= s[1]* s[2]

IF (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN, Wout
	IF (s[0] eq 3) then begin 
	nch=s[3] & chan=datp.z 
	endif else nch=1
 if n_elements(dEps)  eq 0 then dEps = 0.5
 if n_elements(dQ)    eq 0 then dQ   = 0.1
 if n_elements(Qmax)  eq 0 then Qmax = 2.0
 if n_elements(Wbck)  ne 0 then ib = 1 else ib=0
 if n_elements(w_el_v)  gt 0 then MM = w_el_v else MM=M

;******************** INSTRUMENT SETUP *****************************

chw=	par[18]		     ; channel width (musec)
L2= 	par[27]*1000.  ; sample-detector distance (mm)
lambda=	par[21]		   ; incident wavelength (Ang)
;
; ****** DEFINE MATRIX WITH DISTANCES AND ANGLES *********************************
;
r_pix=brisp_dist(M,CENTER=xy0)	; pixel-det_centre distance in cm
L2_th=sqrt((10*r_pix)^2+L2^2)	  ; pixel-sample distance in mm
ang_r=ATAN(10*r_pix/L2)      	  ; angle seen from the pixel [rad]
ang_d=ang_r*!RADEG		          ; angle seen from the pixel [deg]
dOm=cos(ang_r)^3		            ; Solid Angle seen form the pixels
;
; ****** SET ENERGY-TOF PARAMETERS  *********************************

Ei = c4/lambda^2	; incident energy [meV]
Vi = SQRT(Ei/c1)	; incident velocity [m/ms]
T2   = L2/Vi 	    ; Tel [musec] at theta=0
T2_th= L2_th/Vi   ; Tel(theta) [musec] (ie at each pixel)
;
;********** THE MASK: imsk=maked pixels, ok=no masked pixels *******************
;

if n_elements(msk) eq s12 then begin
imsk=where(msk, COMPLEMENT=ok) 
     if imsk[0] ge 0 then th_r=ang_r[ok]; angles with mask/nomask id [rad]
endif
n_ok=n_elements(ok)			; number of nomasked pixels
jM = ARRAY_INDICES(msk, ok)		; to recover the index of nomasked pixels

;;;***** Calculate, for each pixel (ie theta), the position of the elastic channel  *****

M_2d=reform(MM,s12,nch) & M_2d=transpose(M_2d) ; create data(channel, theta): transpose is necessary for str_fit
 totM=total(M_2d,2) & ch_el1=STR_FIT(totM)    ; gaussian fit of the total I(channel) to find a good initial value
 ch_el0=round(ch_el1[2])                      ; for the following pixel fit of the elastic channel position
 ch_min0=ch_el0-75>0 & ch_max0=ch_el0 + 75 < nch ; fit range: in order to prevent nonsense results in pixels 
 M_2d=M_2d[*,ok]
par_fit = str_fit(M_2d[ch_min0:ch_max0,*],Bkgd=0) ; gaussian fit of the elastic ch position at each pixel

 ang_r_nomsk=ang_r[ok]
 sort_ang_r=sort(ang_r_nomsk)
 el_pos=par_fit[sort_ang_r,2]+ch_min0+1
 ang_r_ord=ang_r_nomsk[sort_ang_r]
 ;stop
 chmn0=ch_el0-10>0 & chmx0=ch_el0 + 50 < nch
 ;ind_e=where((el_pos GT ch_min0) and (el_pos LT ch_max0)) ; choose only values between ch_min0 and ch_max0
 ind_e=where((el_pos GT chmn0) and (el_pos LT chmx0)) ; choose only values between ch_min0 and ch_max0
 el_pos_good=el_pos[ind_e];
 ang_r_ord_good=ang_r_ord[ind_e]
 
 ; quadratic fit of elastic position versus theta
 u_mod={NAME:'Formula', NPAR:2, PARAMS:[0.,0.], FORMULA:'Y=P1+P2*X^2', BKGDFIX:1, BKGDVALUE:0.}
fit_el_theta=STR_FIT(el_pos_good,Bkgd="none",User_model=u_mod, fit=pol2_ch_el,Xin=ang_r_ord_good)
M_chel=fit_el_theta[1]+fit_el_theta[2]*ang_r^2  ;
 ;stop
 ; ** PLOT ELASTIC FIT RESULTS
 ;par_text=strtrim(fit_el_theta,2)
 ;pl1=plot(ang_r_ord_good*!RADEG,el_pos_good,XTITLE='!9 q!x [degree]',YTITLE='Elastic channel position')
 ;pl1=plot(ang_r_ord_good*!RADEG,pol2_ch_el,'r',/overplot,TITLE='fit=A + B!9q !x[rad]!E 2 !C A='+par_text(1) + '   -   B='+par_text(2))
;stop
;************* build the matrix w,Q,Int with index (teta,channel)*****************
M_hw=fltarr(n_ok,nch)		; hw(theta,ch)
M_q=fltarr(n_ok,nch)		; Q(theta,ch)
I_hw=fltarr(n_ok,nch)		; I(theta,ch)
Ib_hw=fltarr(n_ok,nch)		; I(theta,ch) for background
fact=fltarr(n_ok,nch)		; (dt/dE)(ki/kf) (theta,ch)
  FOR i=0L,n_ok-1 DO BEGIN  ; use only the nomasked pixels     
k=ok[i]       
chan_ran=chan-randomu(seed,nch)+1 
t_th_ran=T2_th[k]+chw*(chan_ran-M_chel[k])  ; travel time for each theta
Ef=c1*(L2_th[k]/t_th_ran)^2     ; Ef [meV]
M_hw[i,*]=Ei-Ef       ; hw(theta,ch)
ca=2*cos(ang_r[k])
M_q[i,*]=SQRT((Ei+Ef-ca*SQRT(Ei*Ef))/c2); Q(theta,ch)
if n_elements(kikf_v) ne 0 then fact[i,*]= SQRT(c1/4/Ef^3) else fact[i,*]= SQRT(c1*Ei)/(2.*Ef^2)  ; times dt/dE or dt/dE*ki/kf = SQRT(c1/(4.*Ef^3))*SQRT(Ei/Ef)
I_hw[i,*]=M[jM[0,i],jM[1,i],*]/dOm[k] ; Int(theta,ch)/dOmega
if ib then Ib_hw[i,*]=Wbck[jM[0,i],jM[1,i],*];/dOm(k)

  ENDFOR
 
;************* hw-Q grids ***********************
Esup=max(M_hw,min=Einf)
Emin=dEps*round(Einf/dEps) & Emax=dEps*round(Esup/dEps);
nEps = FIX((Emax - Emin) / dEps) + 1;
 Eps = Emin + dEps*FINDGEN(nEps);
dEmin= Eps  - dEps/2.;

; *****************
IF (lambda GT 1.5) then Qmax=1.0 ; to avoid numerical bug at 2 Ang
;Qmax=1.3
Qsup=max(M_q,min=Qinf)
Qmin=dQ*round(Qinf/dQ)
nQ = FIX((Qmax - Qmin) / dQ) +1
Qx =  Qmin + dQ*FINDGEN(nQ)
dQmin=Qx-dQ/2.;
;
;************* fill Q-hw grids of Sqw ***********************
sqO=fltarr(nQ,nEps) & eSqw=sqO  ; prepare output matrices

 FOR iQ=0,nQ-1 DO BEGIN

indQ = where((M_q ge dQmin[iQ]) AND (M_q lt dQmin[iQ]+dQ)); select Q: Q-dQ < Q < Q+dQ
MQ_hw=M_hw[indQ]  	; select hw, Int, Ibckg fact inside Q+/-dQ Q
IQ_hw=I_hw[indQ] & factQ=fact[indQ]  
if ib then IbQ_hw=Ib_hw[indQ] 

   FOR iE = 0,nEps-1  DO BEGIN
indE = where((MQ_hw ge dEmin[iE]) AND (MQ_hw lt dEmin[iE]+dEps)); select E: E-dE/2 < E < E+dE/2
	if indE[0] ge 0 then begin
yes=n_elements(indE)		; to normalise to effective nr of (theta,channels)
av_fact=mean(factQ[indE]) 	; variation of factor in E+/-dE is negligible: use of average values
			 	; is preferable to speed evaluation of errors

sumS= total(IQ_hw[indE]*factQ[indE]); 
sqO[iQ,iE]=sumS/yes
eSqw[iQ,iE]=SQRT(av_fact)*SQRT(sumS)/yes;
	   if ib then begin 
	   sumSb=total(IbQ_hw[indE]*factQ[indE])
	   Sbqw=sumSb/yes		; Sqw for bck
	    sqO[iQ,iE]=sqO[iQ,iE]-Sbqw > 0; 
	   endif
	endif
    ENDFOR
 ENDFOR
;
;;; ********** OUTPUT: Sqw *************
Wout=transpose(sqO) & error=transpose(eSqw) & X=Eps & Y=Qx & Z=0
datp.y_tit="Q [Ang-1]" & datp.x_tit="E [meV]" & datp.z_tit=""

MOD_DATP ,datp,"X",X
MOD_DATP ,datp,"Y",Y
MOD_DATP ,datp,"Z",Z
MOD_DATP ,datp,"E",error
GIVE_DATP,datp

return,Wout
end
