function d17_2dnorm,win,mainbeam,lrange=lrange
;+
;	LAMP function to normalize a workspace by a main beam measurement
;	Designed to run with a 1D array of mainbeam (created by, e.g., xspec)
;	Hacked from d17_waternorm
;	ARW 5.10.07
;
;	NECESSARY VARIABLES:
;	win=the input workspace
;	mainbeam=the workspace with the main beam data
;
;	OPTIONAL VARIABLES:
;	lrange = a two-element array with the maximum and minimum values of lambda
;
;	e.g. w3=d17_mbnorm(w2,w1,lrange=[2,20])
;
;	CALLS TO:
;	Last modified:   8.11.13  PG
;-

; ***	Collect the workspace information
TAKE_DATP,windatp
;PG: error calc:
if (total(windatp.e) eq 0) then MOD_DATP,windatp, 'E' , sqrt(win)
ezero=where(windatp.e eq 0)
if (ezero[0] ne -1) then windatp.e[ezero]=1

TAKE_DATP,mbdatp,/third
;PG: error calc:
if (total(mbdatp.e) eq 0) then MOD_DATP,mbdatp, 'E' , sqrt(mainbeam)
ezero=where(mbdatp.e eq 0)
if (ezero[0] ne -1) then mbdatp.e[ezero]=1

woutdatp=windatp
winsize=size(win,/DIMENSIONS)
if n_elements(winsize) eq 1 then begin
	cflag=0
	lwin=windatp.x 
endif else begin
	cflag=1
	lwin=windatp.y
endelse
lmb=mbdatp.x
xout=windatp.x
yout=windatp.y

; *** Interpolate the wavelengths of the main beam on to the wavelengths for the data
mb=interpol(mainbeam,lmb,lwin)
emb=interpol(mbdatp.e,lmb,lwin)

;mb=mainbeam
;emb=mbdatp.e


; ***	Create a workspace with main beam, the same dimensions as the data
if cflag eq 1 then begin
	dmb=reform(mb,1,n_elements(mb),/OVERWRITE)
	emb=reform(emb,1,n_elements(emb),/OVERWRITE)
endif
winsize=[winsize,0,0]
winsize=winsize[0:2]
tdmb=dmb
temb=emb
for i=1,winsize[0]-1 do begin
	dmb=[dmb,tdmb]
	emb=[emb,temb]
endfor
mbdiv=dmb
ediv=emb
for i=1,winsize[2]-1 do begin
	mbdiv=[[[mbdiv]],[[dmb]]]
	ediv=[[[ediv]],[[emb]]]
endfor

; ***	Carry out the normalization
wout=win/mbdiv
eout=sqrt((windatp.e/mbdiv)^2+(ediv*wout/mbdiv)^2)

; ***	Check for NaN and infinity
wnan=where(finite(wout) ne 1)
if wnan[0] ne -1 then begin
	wout[wnan] = 0.0
	eout[wnan] = 1.0
endif



; *** Determine whether a lambda range has been set
if n_elements(lrange) eq 2 then begin
	lout=where(lwin ge lrange[0] and lwin le lrange[1])
	if cflag eq 1 then begin
		yout=yout[[lout]]
		wout=wout[*,[lout],*]
		eout=eout[*,[lout],*] 
	endif else begin
		xout=xout[[lout]]
		wout=wout[[lout]]
		eout=eout[[lout]]
	endelse
endif else print,'lrange not set.  Output is not truncated'


; ***	Modifify the 'datp'
MOD_DATP,woutdatp,'x',xout
MOD_DATP,woutdatp,'y',yout
MOD_DATP,woutdatp,'e',eout
GIVE_DATP,woutdatp

return,wout
end
