function d17_combine,wa,wb,tol=tol
;+
;	LAMP function to append one workspace to another.
;	The sizes of the x- and z-axes must be the same 
;		for each of the two workspaces
;	The function searches for doubles in the y-axis, and rebins the 
;		y-axis by the tolerance.
;	ARW 06.08.04
;
;	NECESSARY VARIABLES
;	wa=workspace 1
;	wb=workspace 2
;
;	OPTIONAL VARIABLES:
;	tol=determines whether data at equipalent y will be averaged
;		If tol is put in, the data with the same y within a tolerance 
;		given by the value will be averaged.
;		This value should be smaller than 
;			the difference between y values in one workspace
;
;	e.g. w3=d17_combine(w1,w2,tol=0.0)
;
;	CALLS TO: d17_errmsg
;	Last updated:	ARW 17.12.10
;-

if n_elements(tol) ne 1 then d17_errmsg,1,'Data will be appended, no combining of the data will take place'

;***	Get the workspace structures
take_datp,datone
take_datp,dattwo,/third
wdatp=datone

if n_elements(datone.x) ne n_elements(dattwo.x) then begin
	d17_errmsg,1,'ERROR:  The sizes of the x-axis must be the same between workspaces'
	return,0
endif

if n_elements(datone.z) ne n_elements(dattwo.z) then begin
	d17_errmsg,1,'ERROR:  The sizes of the x-axis must be the same between workspaces'
	return,0
endif

;***	Append workspaces in to one array
yin=[datone.y,dattwo.y]
win=[[wa],[wb]]
ein=[[datone.e],[dattwo.e]]
pvin=[[datone.pv],[dattwo.pv]]

if n_elements(ein) ne n_elements(win) then ein=sqrt(win)

;***	Sort the data in ascending order relative to 'y'
yindex=sort(yin)
yin=yin[yindex]
win=win[*,yindex,*]
ein=ein[*,yindex,*]
pvin=pvin[*,yindex,*]

;***	If 'tol' is specified, average the data
ws=size(win,/DIMENSIONS)
if n_elements(ws) eq 1 then ws=[ws,1,1]
if n_elements(ws) eq 2 then ws=[ws,1]
wout=0.0*win[*,0,*]
eout=wout
pvout=0.0*pvin[*,0]
yout=0.0
if n_elements(tol) eq 1 then begin
	print,'Averaging!'
	ny=n_elements(yin)
	for i=0,ny-1 do begin
		if finite(yin[i]) then begin
			dbls=where(abs(yin-yin[i]) le tol,dbcnt)
			if dbcnt gt 1 then begin
				wout=[[wout],[reform(total(win[*,dbls,*],2),ws[0],1,ws[2],/OVERWRITE)/dbcnt]]
				eout=[[eout],[sqrt(reform(total(ein[*,dbls,*]^2,2),ws[0],1,ws[2],/OVERWRITE))/dbcnt]]
				yout=[yout,mean(yin[dbls])]
				pvout=[[pvout],[total(pvin[*,dbls],2)/dbcnt]]
				yin[dbls]=!VALUES.D_NAN
			endif else begin
				wout=[[wout],[win[*,i,*]]]
				eout=[[eout],[ein[*,i,*]]]
				yout=[yout,yin[i]]
				pvout=[[pvout],[pvin[*,i]]]			
			endelse 
		endif
	endfor
; ***	Tidy up the output variables
	wout=wout[*,1:*,*]
	yout=yout[1:*]
	pvout=pvout[*,1:*]
	eout=eout[*,1:*,*]
endif else begin
	wout=win
	yout=yin
	eout=ein
	pvout=pvin
endelse


mod_datp,wdatp,'y',yout
mod_datp,wdatp,'e',eout
mod_datp,wdatp,'pv',pvout
give_datp,wdatp
return,wout
end
