function d17_waterdef,nums,dims=dims,mask=maskstring
;+
;	LAMP function to load in a water run, mask it and normalize it
;	ARW 30.08.04
;
;	NECESSARY INPUT:
;	nums=a string with the filenumbers containing the data. 
;		Consecutive files are separated by a greater than (>)
;		Adjacent files are separated by a comma (+)
;		The string must be enclosed in quotation marks (')
;		e.g. '90230>90240+90250>90260'
;
;	OPTIONAL INPUT:
;	dims=the dimension of the output workspace.  Can be 1 or 2.  
;		If nothing is specified, the output workspace will have the same dimensions
;		as the input data
;	maskstring=your choice of masking the detector, written as a string
;			The default is no mask
;
;	e.g. type at LAMP interface:
;		w1=d17_waterdef('90230,90250',dims=1,mask='box,[190,200,63,222]')
;
;	CALLS TO:  rdopr, maskme, d17_errmsg
;	Last modified:   ARW 17.12.10
;-

forward_function rdopr

;***	Read in data
dp=1
water=rdopr(nums,datp=dp)
err=sqrt(water)
y=dp.y
mod_datp,dp,"e",err


;***	Mask if desired
if n_elements(maskstring) ne 1 then begin
	d17_errmsg,1,'No data mask is applied'
endif else water=maskme(maskstring,water,datp=dp)

;***	Check dimensions, integrate if necessary
if n_elements(dims) ne 1 then begin
	d17_errmsg,1,'No change of water dimensions'
	d17_errmsg,1,['Water dimensions are ',string(size(water,/DIMENSIONS))]
endif else begin
	watersize=size(water)
	if watersize[0] lt dims then begin
		d17_errmsg,1,'WARNING: Requested dimension > dimension of input water.  No change in dimensions implemented'
		d17_errmsg,1,['Water dimensions are ',string(size(water,/DIMENSIONS))]
		return,0
	endif
	if dims gt 2 then begin
		d17_errmsg,1,'WARNING: Requested output dimension can only be 1 or 2.  No change in dimensions implemented'
		d17_errmsg,1,['Water dimensions are ',string(size(water,/DIMENSIONS))]
		return,0
	endif
	if dims eq 1 and watersize[0] eq 2 then begin
		d17_errmsg,1,'Integrating water runs vertically'
		water=total(water,2)
		err=sqrt(total(dp.e^2,2))
		y=0
		d17_errmsg,1,['Final water array dimensions are ',string(size(water,/DIMENSIONS))]
	endif
endelse

mwater=max(water)
merr=err[where(water eq max(water))]

water=water/mwater
err=sqrt((err/mwater[0])^2+(merr[0]*water/mwater[0])^2)

mod_datp,dp,"y",y
mod_datp,dp,"e",err
give_datp,dp
return,water

end
