function d17_waternorm,win,water
;+
;	LAMP function to normalize a workspace by the water file
;	Designed to run with a 1D array of water (loaded by, e.g., aw_waterdef)
;		and with data that has been integrated vertically
;	ARW 21.10.04
;
;	NECESSARY VARIABLES:
;	win=the input workspace
;	water=the workspace with the water data
;
;	e.g. w3=d17_waternorm(w2,w1)
;
;	CALLS TO: d17_errmsg
;	Last modified:   ARW 17.10.10
;-

; ***	Collect the workspace information
TAKE_DATP,windatp
TAKE_DATP,watdatp,/third
woutdatp=windatp
ewat=watdatp.e

; ***	Determine that the workspaces have the right dimensions
winsize=size(win,/DIMENSIONS)
watsize=size(water,/DIMENSIONS)
if n_elements(watsize) ne 1 then begin
	d17_errmsg,1,'ERROR:  Water data must be a 1D array'
	d17_errmsg,1,['       The water data has the array dimensions ',string(watsize)]
	return,0
endif
if winsize[0] ne watsize[0] then begin
	d17_errmsg,1,'ERROR:	The input data and the water file have a different number of x-pixels'
	d17_errmsg,1,['     The input data has ',string(winsize[0]),' pixels']
	d17_errmsg,1,['     The water data has ',string(watsize[0]),' pixels']
	return,0
endif	

; ***	Create a workspace with water, the same dimensions as the data
if n_elements(winsize) ne 3 then winsize=[winsize,2,2]
winsize=winsize[0:2]
dwat=water
for i=1,winsize[1]-1 do begin
	dwat=[[dwat],[water]]
	ewat=[[ewat],[watdatp.e]]
endfor
wdiv=dwat
ediv=ewat
for i=1,winsize[2]-1 do begin
	wdiv=[[[wdiv]],[[dwat]]]
	ediv=[[[ediv]],[[ewat]]]
endfor

; ***	Carry out the normalization
wout=win/wdiv
eout=sqrt((windatp.e/wdiv)^2+(ediv*wout/wdiv)^2)

; ***	Check for NaN and infinity	
wnan=where(finite(wout) ne 1)
wout[wnan] = 0.0
eout[wnan] = 1.0

; ***	Modifify the 'datp' of water before dividing
MOD_DATP,woutdatp,'e',eout
GIVE_DATP,woutdatp

return,wout
end
