PRO rdid_d17, INST,numor,nvers,text,exper,scan,cnt,WOUT,vparm,param,par1,par2,par3,par4,par5,$
              WT ,XT ,YT ,ZT ,OT ,DATE ,PP ,PTXT ,XX ,YY ,ZZ ,NN ,PV ,EE
;+
;** ********
;** LAMP data-read interface for D17 data (ILL format) called from rdid.pro OR read_hdf.pro
;** ***************************************************************************************
;** 
;**               JRS,RC 9/5/00
;**       Modified for new detector:  FC & ARW 28.04.10
;**       Modified (count rate):  PG 26.07.12
;**       Modified (Chopper parameters for MAD): PG 19.04.13
;**       Modified (TOF scans to include polarisation information in Data-params buffer): TS 29.08.2014
;**       Modified (Case Monochromatic OR TOF): TS 02.09.2014
;**       Modified (Output of B1-B5 and correct TTL signals of Flipper 1+2): TS 31.10.2014  
;**       Modified (Added Temperatures): PG 12.12.2014  
;**       Modified (Removed Temperatures due to parameter clash):TS 15.10.2014
;**       Modified (Sorted Chopper openings): PG:25.04.2015
;**	  
;**       Accept NeXus parameters: DR Apr 2014 + Apr 2015
;**		  Last modified (Added back Temperatures): PG 20.11.2016
;** 
;** Parameters to return
;** ********************
;** WOUT             (modified data)
;** WT,XT,YT,ZT,OT   (titles)
;** PP,PTXT          (parameter floatting table and textarea)
;** XX,YY,ZZ         (coordinates)
;** NN               (monitors)
;** PV               (supplement parameter table of any dimension)
;** EE               (errors from WOUT)
;** DATE      (Start and finish times for the measurement)
;-
    
    
    ;Useful input parameters from rdid
    ;*********************************
    iprint=0
    
    PRINT,'Reading run',numor
    IF (iprint GT 0) THEN BEGIN
      PRINT,'rdid_d17: starting'
      HELP, INST   & PRINT, INST
      HELP, numor  & PRINT, numor
      HELP, nvers  & PRINT, nvers
      HELP, text   & PRINT, text
      HELP, exper  & PRINT, exper
      HELP, scan   & PRINT, scan
      HELP, cnt    & PRINT, cnt
      HELP, WOUT
      HELP, vparm  & PRINT, vparm
      HELP, param  & PRINT, param
      HELP, par1   & PRINT, par1
      HELP, par2   & IF (N_ELEMENTS(par2) GT 0) THEN PRINT, par2
    ENDIF

   ;PARAMETERS COMING FROM A NEXUS FILE? DR
    rawButton = long([2])
    nexus = 0
    IF (n_elements(par3) gt 10) and (n_elements(par3) eq n_elements(par4)) THEN BEGIN
       nexus = 1
      ;par3 is an input text  parameters array (from read_hdf.pro)
      ;par4 is an input value parameters array
      ;par5 is an input structure (GH_struct) containing other parameter arrays

      ;Make TEXT and EXPER compatible with ascii format
       end_time = ' ' & start_time = ' '
       userN    = ' ' & local      = ' '
       propo    = ' ' & formu      = ' '
       idxT =strpos(par3 ,' end_time: ')   &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then end_time   =  strmid(par3[idxP],idxT[idxP]+11,18)
       idxT =strpos(par3 ,' start_time: ') &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then start_time =strmid(par3[idxP],idxT[idxP]+13,18)
       idxT =strpos(par3 ,'User) name:')   &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then begin
           idxE = strpos(par3[idxP],'=',idxT[idxP])
           userN= strmid(par3[idxP ],idxT[idxP]+12, idxE-2-(idxT[idxP]+12))
       endif
       idxT =strpos(par3 ,'User) nameloc') &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then begin
           idxE = strpos(par3[idxP],'=',idxT[idxP])
           local= strmid(par3[idxP ],idxT[idxP]+24, idxE-2-(idxT[idxP]+24))
       endif
       idxT =strpos(par3 ,'User) proposal:') &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then begin
           idxE = strpos(par3[idxP],'=',idxT[idxP])
           propo= strmid(par3[idxP ],idxT[idxP]+16, idxE-2-(idxT[idxP]+16))
       endif
       idxT =strpos(par3 ,'experiment_iden') &  idx =where(idxT ge 0) & idxP = idx[0]
       if idxP ge 0 then begin
           idxE = strpos(par3[idxP],'=',idxT[idxP])
           formu= strmid(par3[idxP ],idxT[idxP]+23, idxE-2-(idxT[idxP]+23))
       endif
       
       inst      = INST[0]
       user      = userN + ' ' + local
       main_title= propo + ' ' + formu
       prop      = propo
       sub_title = propo + ' ' + formu
       sub_title = string('Numor ',strtrim(numor,1),'(nxs)   ',sub_title)
       date      = start_time+'  '+end_time
       
      ;PUT NeXus PARAMETERS in PARAM
       param = fltarr(256)
       
       idx =strpos(par3,' value degree phi ')               & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[0  ] = par4[idx]             ; Sample tilt
       idx =strpos(par3,' value mm trs ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[1  ] = par4[idx]             ; Sample trans
       idx =strpos(par3,' value degree san ')               & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[2  ] = par4[idx]             ; Sample angle
       idx =strpos(par3,' value mm s1t ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[4  ] = par4[idx]             ; Slit 1 t
       idx =strpos(par3,' value mm s1b ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[5  ] = par4[idx]             ; Slit 1 b
       idx =strpos(par3,' value mm s1l ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[6  ] = par4[idx]             ; Slit 1 l
       idx =strpos(par3,' value mm s1r ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[7  ] = par4[idx]             ; Slit 1 r
       idx =strpos(par3,' value mm s2l ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[8  ] = par4[idx]             ; Slit 2 l
       idx =strpos(par3,' value mm s2r ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[9  ] = par4[idx]             ; Slit 2 r
       idx =strpos(par3,' value mm s3t ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[10 ] = par4[idx]             ; Slit 3 t
       idx =strpos(par3,' value mm s3b ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[11 ] = par4[idx]             ; Slit 3 b
       idx =strpos(par3,' value mm s3l ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[12 ] = par4[idx]             ; Slit 3 l
       idx =strpos(par3,' value mm s3r ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[13 ] = par4[idx]             ; Slit 3 r
       idx =strpos(par3,' choppergap ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[14 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm det ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[15 ] = par4[idx]             ; Sample-Detector distance
       idx =strpos(par3,' value degree dan ')               & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[16 ] = par4[idx]             ; Detector angle
       idx =strpos(par3,' value mm trf ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[17 ] = par4[idx]             ; 
       idx =strpos(par3,' value step rof ')                 & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[18 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm trm ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[19 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm trb ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[20 ] = par4[idx]             ; 
       idx =strpos(par3,' value degree ros ')               & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[23 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm tra ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[24 ] = par4[idx]             ; 
       idx =strpos(par3,' value  fl2 ')                     & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[25 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm ran ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[27 ] = par4[idx]             ; 
       idx =strpos(par3,' value mm bst ')                   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[30 ] = par4[idx]             ; 
       idx =strpos(par3,'setpoint_rotation_speed rpm Chopper1') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[40 ] = par4[idx]
       idx =strpos(par3,'setpoint_phase degree Chopper1')       & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[41 ] = par4[idx]
       idx =strpos(par3,'setpoint_rotation_speed rpm Chopper2') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[42 ] = par4[idx]
       idx =strpos(par3,'setpoint_phase degree Chopper2')       & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[43 ] = par4[idx]
       idx =strpos(par3,'rotation_speed rpm Chopper1')          & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[44 ] = par4[idx]
       idx =strpos(par3,'phase degree Chopper1')                & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[45 ] = par4[idx]
       idx =strpos(par3,'rotation_speed rpm Chopper2')          & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[46 ] = par4[idx]
       idx =strpos(par3,'phase degree Chopper2')                & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[47 ] = par4[idx]
       idx =strpos(par3,'flip_current B1')                      & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[60 ] = par4[idx]
       idx =strpos(par3,'positioner) value  fl1 ')              & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[99 ] = par4[idx]
       idx =strpos(par3,'positioner) value  fl2 ')              & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[100] = par4[idx]

;Added by MG on 12 Jan 2017 --> Using the values given in entry 'VirtualChopper'. Someone should check if this is OK!
       idx =strpos(par3,'chopper1_speed_average  VirtualChopper ') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[109] = par4[idx]
       idx =strpos(par3,'chopper1_phase_average  VirtualChopper ') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[110] = par4[idx]
       idx =strpos(par3,'chopper1_speed_sigma  VirtualChopper ')   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[111] = par4[idx]
       idx =strpos(par3,'chopper1_phase_sigma  VirtualChopper ')   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[112] = par4[idx]
       idx =strpos(par3,'chopper2_speed_average  VirtualChopper ') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[113] = par4[idx]
       idx =strpos(par3,'chopper2_phase_average  VirtualChopper ') & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[114] = par4[idx]
       idx =strpos(par3,'chopper2_speed_sigma  VirtualChopper ')   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[115] = par4[idx]
       idx =strpos(par3,'chopper2_phase_sigma  VirtualChopper ')   & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PARAM[116] = par4[idx]
            
       ;Shuffle parameters to be consistent with order in ASCII file, but the inconsistency between ASCII and NeXuS  should be solved in the files       
       PARAM[109:116] = [PARAM[109], PARAM[113], PARAM[111], PARAM[115], PARAM[110], PARAM[114], PARAM[112], PARAM[116]]    
            
;CORRECTIONS
      ;PARAM[109] = PARAM[44 ]                                  ; Mean Chop1 speed !! 
      ;PARAM[110] = PARAM[45 ]                                  ; Mean Chop1 phase !!
      ;PARAM[114] = PARAM[47 ]                                  ; Mean Chop2 phase !!

      ;PUT NeXus PARAMETERS in PAR1
       par1 = fltarr(128)
       
       idx =strpos(par3,' duration Sec ')                       & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[2  ]  = par4[idx]
       idx =strpos(par3,' detsum  PSD')                         & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[3  ]  = par4[idx]
       idx =strpos(par3,' monsum ')                             & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[4  ]  = par4[idx]
       idx =strpos(par3,' poff ')                               & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[54 ]  = par4[idx]
       idx =strpos(par3,' dist_chop_samp ')                     & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[55 ]  = par4[idx]
       idx =strpos(par3,' open_offset ')                        & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[56 ]  = par4[idx]
       idx =strpos(par3,'wavelength ')                          & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[57 ]  = par4[idx]
       idx =strpos(par3,'entry) acquisition_mode ')             & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[87 ]  = par4[idx]             ; TOF or noTOF
       idx =strpos(par3,'time_of_flight_0')                     & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[95 ]  = par4[idx]             ; Channel Width
       idx =strpos(par3,'time_of_flight_1')                     & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[94 ]  = par4[idx]             ; Time Channels
       idx =strpos(par3,'time_of_flight_2')                     & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[96 ]  = par4[idx]             ; TOF Delay
       idx =strpos(par3,' p0x  PSD')                            & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[97 ]  = par4[idx]             ; X1 for detector binning !!
       idx =strpos(par3,' p1x  PSD')                            & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[98 ]  = par4[idx]             ; X2 for detector binning !!
       idx =strpos(par3,' p0y  PSD')                            & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[99 ]  = par4[idx]             ; Y1 for detector binning !!
       idx =strpos(par3,' p1y  PSD')                            & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[100]  = par4[idx]             ; Y2 for detector binning !!
       idx =strpos(par3,' nx  PSD')                             & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[101]  = par4[idx]             ; NX binning  denominator !!
       idx =strpos(par3,' ny  PSD')                             & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[102]  = par4[idx]             ; NY binning  denominator !!
       idx =strpos(par3,' mppx  PSD')                           & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[103]  = par4[idx]             ; mm/Xpixel
       idx =strpos(par3,' mppy  PSD')                           & idx=where(idx ge 0) & idx =idx[0]
            if idx ge 0 then PAR1[104]  = par4[idx]             ; mm/Ypixel
;MG 12 Jan 2017: Should add here the 6 temperatures stored in par1[119:124] in the ASCII file,
;but need to know how they are called in the NeXuS. 

;CORRECTIONS
       PAR1[26 ] = 1                                            ; no. of time slices !!
       PAR1[90 ] = 0                                            ; NoMad
       PAR1[94 ] = par1[94] > 1.
       if PAR1[95] eq 0. then PAR1[95] = 1.                     ; noTOF
       if PAR1[94] gt 1. then PAR1[87] = 1. else PAR1[87] = 0.  ; TOF (may be commented now)
       PAR1[[98 ,100]] = reverse(PAR1[[98 ,100]])
       PAR1[104] = 6.7
       
      ;Monitor and time_slices should be in the par5 structure.
      ;help, par5
      ;RETURN ;for the moment!

    ENDIF ELSE BEGIN
   ;PARAMETERS COMING FROM AN ASCII FILE
      inst      = STRMID(text,0,4)
      user      = STRMID(text,4,10)
      user      = STRMID(exper[0],0,10)
      main_title= STRMID(exper[0],10,39)
      prop      = STRMID(exper[0],49,9)
      sub_title = STRMID(exper[0],49,24)
      sub_title = string('Numor ',strtrim(numor,1),'   ',sub_title)
      date      = exper[1]
    ENDELSE
    
    ;;(TS 2015-04-02) Introduce a poldate variable to specify the year in which the measurement was taken
    ;this is necessary to be backwards compatible with old polarized D17 data pre 2014: (TS 2015-04-02)
    ;NOTE:: THIS MIGHT BE NECESSARY TO BE CHANGED WHEN WE SWITCH TO NEXUS FILES!!!
    poldate = STRMID(date, 7, 2)
    ;END TS (2015-04-02)
    ;
    
    IF (iprint GT 0) THEN BEGIN
      PRINT,'inst =',inst
      PRINT,'user =',user
      PRINT,'main_title =',main_title
      PRINT,'prop =',prop
      PRINT,'sub_title =',sub_title
    ENDIF
    
    ; -----------------------------------
    ; Calculate some important quantities
    s1h=param[4]+param[5]        ;slit 1 height
    s1w=param[6]+param[7]        ;slit 1 width
    s2w=param[8]+param[9]        ;slit 2 width
    s3h=param[10]+param[11]      ;slit 3 height
    s3w=param[12]+param[13]      ;slit 3 width
    
    
    ;Figure out if the old flipper assignments are to be used TS(2015-04-02)
    ;TS: 2014-10-31 Commented the following section since it is not needed anymore -
    ;Both flippers are now correctly assigned over the TTL signals
    if poldate le 13 then begin
      if param[60] lt 0.1 then F1=0 else F1=1    ;Flipper 1 state
      ;if param(25) lt 0.1 then F2=0 else F2=1 ;Flipper 2 state

      ;  NOTE:  Introduce a FUDGE here - at the moment the Flipper 2 state is kept in a SUBTITLE
      ;    i.e. 'par sub *F2=0*' should be entered before running a scan with FL2 off
      ;    I am introducing a search for this string in order to put the flipper 2 state in to the parameter block.
      ;    ARW 06.05.05
      strF2='F2=1'
      if strpos(sub_title,strF2) ne -1 then F2=1 else F2=0
     ;PRINT,'ANDY =',poldate
    endif
    ;END TS (2015-04-02)
    ;TS: 2014-10-31 Commented the following section since it is not needed anymore - 
    ;Both flippers are now correctly assigned over the TTL signals
    
    ;  Detector variables
    
    ; PG: modified for new detector
    if nvers gt 1 then begin   ;new Detector
      cen_x=132.5/par1[101]    ;NOTE: FIX THIS
      ;  cen_x=???/par1(101)   ;Geometric centre of detector in X
      cen_y=31.5/par1[102]     ;NOTE: FIX THIS
    ;  cen_y=???/par1(102)     ;Geometric centre of detector in Y
    endif else begin           ;old detector
      cen_x=135.79/par1[101]
      cen_y=135.0/par1[102]
    endelse
    

    mppx=par1[103]      ;mm per pixel in X
    mppy=par1[104]      ;mm per pixel in y

    ;  TOF quantities
    poff=par1[54]
    ;  if (poff eq 0) or (poff le 277) or (poff gt 280) then poff=279            ;Established 25.02.04
    openoff=par1[56]
    ;  if (openoff eq 0) or (openoff gt 2) or (openoff le 1) then openoff=1.8419 ;Established 14.07.06
    ;  openoff=1.8419
    ;  poff=277.8
    d0=par1[55]          ;1st chopper-sample distance
    if (d0 eq 0) or (d0 gt 4.2) or (d0 le 4) then d0=4.1685   
    if param[41] gt 360.0 then param[41]=param[41]/100.0   ; *** Due to a bug in MAD
    if param[43] gt 360.0 then param[43]=param[43]/100.0   ; *** Due to a bug in MAD
    if param[110] gt 360. then param[110]=param[110]/100.  ;PG: added mean value
    if param[114] gt 360. then param[114]=param[114]/100.
    open_req=45.0-(param[43]-param[41])-openoff	;PG: added open offset
    ;(02.09.2014) TS: The "if" block below has the issue that in a monochromatic scan, none of the chopper values are defined =>
    ;division by zero...
    ;Figure out what is TOF and what not
    
    if par1[87] eq 0. then begin
      ;print, 'monochromatic scan TOF-flag='+string(param[87])
      open_act=0
      period=0
    endif else begin ;END 02.09.2014
      if param[114] eq 0. then begin  ;PG: using mean chopper values if possible, if not required ones
        open_act= 45.0-(param[47]-param[45])-openoff	;PG: Changed to last opening
        real_open=open_act
	      period  = 60. / param[40]; TS: This causes a division by 0 if the count was chosen instead of TOFcount
      endif else begin
	      open_act= 45.0-(param[47]-param[45])-openoff		;PG: Changed to last opening
        real_open = 45. - (param[114] - param[110])-openoff	;PG: added open offset
        period   = 60. /  param[109]
      endelse
      if (abs(real_open-open_act) gt 1.0) then real_open=open_act	;PG: New NoMad bug found on 24/04/15
      ;period=60.0/param(44)  ;PG: using mean value if possible
      delay_ang = (poff-real_open)/2.0
      delay_tim = (delay_ang/360.0)*period
      cht       = param[14]/100.0      
      if (cht gt 0.5) or (cht le 0.01) then cht=0.08111 ;interchopper distance
      chopsam   = d0-cht/2.0                      ;mid-chopper-sample distance
      tofd      = chopsam+(param[15]/1000.0)      ;total TOF distance
      chanpa    = (tofd/3956.0)/(par1[95]*1e-6)
      delchan   = delay_tim/(par1[95]*1e-6)
      delechan  = par1[96]/par1[95]
    endelse
    ; -----------------------------------
    ;  Assign Parameter block
    ;
    ;  Common variables to all scans
    ; Please do NOT increase the number of npars!! Reduction codes rely on 24 
    npars      =24
    p_buf      =FLTARR(npars)
    par_txt_buf=STRARR(npars)
    
    p_buf[0] =numor      & par_txt_buf[0] =' 0 Numor             ='
    p_buf[1] =par1[2]    & par_txt_buf[1] =' 1 run time          ='
    if (par1[90] eq 1.) then p_buf[1]=par1[2]/10.
    p_buf[2] =par1[4]    & par_txt_buf[2] =' 2 monitor           ='
    p_buf[3] =param[2]   & par_txt_buf[3] =' 3 sample angle      ='
    p_buf[4] =param[16]  & par_txt_buf[4] =' 4 detector angle    ='
    p_buf[5] =s1h        & par_txt_buf[5] =' 5 s1 height         ='
    p_buf[6] =s1w        & par_txt_buf[6] =' 6 s1 width          ='
    p_buf[7] =s2w        & par_txt_buf[7] =' 7 s2 width          ='
    p_buf[8] =s3h        & par_txt_buf[8] =' 8 s3 height         ='
    p_buf[9] =s3w        & par_txt_buf[9] =' 9 s3 width          ='
    p_buf[10]=param[1]   & par_txt_buf[10]=' 10 sample trans.    ='
    p_buf[11]=param[0]   & par_txt_buf[11]=' 11 sample tilt      ='
    p_buf[12]=par1[97]   & par_txt_buf[12]=' 12 X1 (det bin)     ='
    p_buf[13]=par1[98]   & par_txt_buf[13]=' 13 X2 (det bin)     ='
    p_buf[14]=par1[99]   & par_txt_buf[14]=' 14 Y1 (det bin)     ='
    p_buf[15]=par1[100]  & par_txt_buf[15]=' 15 Y2 (det bin)     ='
    p_buf[16]=par1[101]  & par_txt_buf[16]=' 16 NX (det bin)     ='
    p_buf[17]=par1[102]  & par_txt_buf[17]=' 17 NY (det bin)     ='
    p_buf[18]=cen_x      & par_txt_buf[18]=' 18 Detector cen (x) ='
    p_buf[19]=mppx       & par_txt_buf[19]=' 19 mm per pixel (x) ='
    p_buf[20]=par1[55]   & par_txt_buf[20]=' 20 chop1 - sam dist ='
    p_buf[21]=param[15]  & par_txt_buf[21]=' 21 sam-det distance ='
    
    ;Figure out when the data was recorded in order to assign Flippers! (TS 2015-04-02)
    
    if poldate le 13 then begin
      p_buf[22]=F1          & par_txt_buf[22]=' 22 Flipper 1        ='
      p_buf[23]=F2          & par_txt_buf[23]=' 23 Flipper 2        ='
     ;PRINT,'ANDY =',poldate
    endif else begin
      ;;31.10.2014 TS
      ;;Changed Flipper states to correct parameter block number 
      p_buf[22]=param[99]   & par_txt_buf[22]=' 22 Flipper 1        ='
      p_buf[23]=param[100]  & par_txt_buf[23]=' 23 Flipper 2        ='
     ;PRINT,'MINE =',poldate
    endelse    
    ;TS: NOTE:: PLEASE DO NOT ADD ANYTHING HERE - it upsets the codes as the 
    ;numbers get shifted
    ;Please add in the parameter blocks below!
    ;PG: Added Temperatures
    ;p_buf[24]=par1[107]   & par_txt_buf[24]=' Sample Temp         ='
    ;p_buf[25]=par1[108]   & par_txt_buf[25]=' Regulation Temp     ='
    
    ;  For a time-of-flight scan
    if par1[87] ne 0 then begin
    
      npars_tof   =35
      p_buf       =[p_buf,FLTARR(npars_tof)]
      par_txt_buf =[par_txt_buf,STRARR(npars_tof)]
      
      p_buf[npars]   =par1[94]         & par_txt_buf[npars]   =[' '+strtrim(string(npars),1)   +' Time Channels    =']
      p_buf[npars+1] =par1[95]         & par_txt_buf[npars+1] =[' '+strtrim(string(npars+1),1) +' Channel width    =']
      p_buf[npars+2] =par1[96]         & par_txt_buf[npars+2] =[' '+strtrim(string(npars+2),1) +' Electr TOF delay =']
      p_buf[npars+3] =poff             & par_txt_buf[npars+3] =[' '+strtrim(string(npars+3),1) +' Poff             =']
      p_buf[npars+4] =openoff          & par_txt_buf[npars+4] =[' '+strtrim(string(npars+4),1) +' Open offset      =']
      p_buf[npars+5] =param[40]        & par_txt_buf[npars+5] =[' '+strtrim(string(npars+5),1) +' chop 1 speed req =']
      p_buf[npars+6] =param[41]/100.0  & par_txt_buf[npars+6] =[' '+strtrim(string(npars+6),1) +' chop 1 phase req =']
      p_buf[npars+7] =param[42]        & par_txt_buf[npars+7] =[' '+strtrim(string(npars+7),1) +' chop 2 speed req =']
      p_buf[npars+8] =param[43]/100.0  & par_txt_buf[npars+8] =[' '+strtrim(string(npars+8),1) +' chop 2 phase req =']
      p_buf[npars+9] =param[44]        & par_txt_buf[npars+9] =[' '+strtrim(string(npars+9),1) +' chop 1 speed act =']
      p_buf[npars+10]=param[45]        & par_txt_buf[npars+10]=[' '+strtrim(string(npars+10),1)+' chop 1 phase act =']
      p_buf[npars+11]=param[46]        & par_txt_buf[npars+11]=[' '+strtrim(string(npars+11),1)+' chop 2 speed act =']
      p_buf[npars+12]=param[47]        & par_txt_buf[npars+12]=[' '+strtrim(string(npars+12),1)+' chop 2 phase act =']
      p_buf[npars+13]=open_req         & par_txt_buf[npars+13]=[' '+strtrim(string(npars+13),1)+' chop open req    =']
      p_buf[npars+14]=open_act         & par_txt_buf[npars+14]=[' '+strtrim(string(npars+14),1)+' chop open act    =']
      p_buf[npars+15]=real_open        & par_txt_buf[npars+15]=[' '+strtrim(string(npars+15),1)+' real chop open   =']
      p_buf[npars+16]=period           & par_txt_buf[npars+16]=[' '+strtrim(string(npars+16),1)+' chop period      =']
      p_buf[npars+17]=delay_ang        & par_txt_buf[npars+17]=[' '+strtrim(string(npars+17),1)+' chop delay angle =']
      p_buf[npars+18]=delay_tim        & par_txt_buf[npars+18]=[' '+strtrim(string(npars+18),1)+' chop delay time  =']
      p_buf[npars+19]=cht              & par_txt_buf[npars+19]=[' '+strtrim(string(npars+19),1)+' interchop dist(m)=']
      p_buf[npars+20]=param[30]        & par_txt_buf[npars+20]=[' '+strtrim(string(npars+20),1)+' Beamstop         =']
      p_buf[npars+21]=par1[3]/p_buf[1] & par_txt_buf[npars+21]=[' '+strtrim(string(npars+21),1)+' Count rate       =']
      ;;29.08.2014 TS
      ;;ADDED 
      p_buf[npars+22]=param[17]        & par_txt_buf[npars+22]=  [' '+strtrim(string(npars+22),1) +' Filter trans.    =']
      p_buf[npars+23]=param[18]        & par_txt_buf[npars+23]=  [' '+strtrim(string(npars+23),1) +' Filter rot.      =']
      p_buf[npars+24]=param[24]        & par_txt_buf[npars+24]=  [' '+strtrim(string(npars+24),1) +' Analyser trans.  =']
      p_buf[npars+25]=param[27]        & par_txt_buf[npars+25]=  [' '+strtrim(string(npars+25),1) +' Analyser rot.    =']
      ;;31.10.2014 TS
      ;;ADDED B5 for magnetic field readout for users
      p_buf[npars+26]=param[64]        & par_txt_buf[npars+26]=  [' '+strtrim(string(npars+26),1) +' B5 (A)    =']
      ;PG on 20/11/2016: Added Temperatures
      p_buf[npars+27]=par1[119]   & par_txt_buf[npars+27]=[' '+strtrim(string(npars+27),1) +' Temp 1 [C]   =']
      p_buf[npars+28]=par1[120]   & par_txt_buf[npars+28]=[' '+strtrim(string(npars+28),1) +' Temp 2 [C]   =']
      p_buf[npars+29]=par1[121]   & par_txt_buf[npars+29]=[' '+strtrim(string(npars+29),1) +' Temp 3 [C]   =']
      p_buf[npars+30]=par1[122]   & par_txt_buf[npars+30]=[' '+strtrim(string(npars+30),1) +' Temp 4 [C]   =']
      p_buf[npars+31]=par1[123]   & par_txt_buf[npars+31]=[' '+strtrim(string(npars+31),1) +' Temp 5 [C]   =']
      p_buf[npars+32]=par1[124]   & par_txt_buf[npars+32]=[' '+strtrim(string(npars+32),1) +' Temp 6 [C]   =']
      p_buf[npars+33]=par1[27]    & par_txt_buf[npars+33]=[' '+strtrim(string(npars+33),1) +' Sample Temp Setpoint [K] =']
      p_buf[npars+34]=par1[28]    & par_txt_buf[npars+34]=[' '+strtrim(string(npars+34),1) +' Regulation Temp [K]      =']
      ;;END 20.11.2016
      
    endif else begin
      ;  Monochromatic scan
      npars_mono=11
      p_buf=[p_buf,FLTARR(npars_mono)]
      par_txt_buf=[par_txt_buf,STRARR(npars_mono)]
      
      p_buf[npars]=par1[57]           & par_txt_buf[npars]  =  [' '+strtrim(string(npars),1)   +' Wavelength (A^-1)=']
      p_buf[npars+1]=param[30]        & par_txt_buf[npars+1]=  [' '+strtrim(string(npars+1),1) +' Beamstop         =']
      p_buf[npars+2]=param[23]        & par_txt_buf[npars+2]=  [' '+strtrim(string(npars+2),1) +' Soller rot.      =']
      p_buf[npars+3]=param[19]        & par_txt_buf[npars+3]=  [' '+strtrim(string(npars+3),1) +' Mono trans.      =']
      p_buf[npars+4]=param[20]        & par_txt_buf[npars+4]=  [' '+strtrim(string(npars+4),1) +' Mono rot.        =']
      p_buf[npars+5]=param[17]        & par_txt_buf[npars+5]=  [' '+strtrim(string(npars+5),1) +' Filter trans.    =']
      p_buf[npars+6]=param[18]        & par_txt_buf[npars+6]=  [' '+strtrim(string(npars+6),1) +' Filter rot.      =']
      p_buf[npars+7]=param[24]        & par_txt_buf[npars+7]=  [' '+strtrim(string(npars+7),1) +' Analyser trans.  =']
      p_buf[npars+8]=param[27]        & par_txt_buf[npars+8]=  [' '+strtrim(string(npars+8),1) +' Analyser rot.    =']
      p_buf[npars+9]=par1[3]/p_buf[1] & par_txt_buf[npars+9]=  [' '+strtrim(string(npars+9),1) +' Count rate       =']
      ;;31.10.2014 TS
      ;;ADDED B5 for magnetic field readout for users
      p_buf[npars+10]=param[64]       & par_txt_buf[npars+10]= [' '+strtrim(string(npars+10),1)+'        B5 (A)    =']
    endelse

    ; -----------------------------------
    
    tot  =cnt
    tsize=LONG(par1[94])
    xsize=LONG(par1[98]-par1[97]+1)
    ysize=LONG(par1[100]-par1[99]+1)

    ;IF (tot ne ((xsize*ysize*tsize))) THEN PRINT,'read_d17: Error in data array dimensions'
    IF (iprint GT 0) THEN BEGIN
      PRINT,'tsize= ',tsize,' detector size= ',xsize*ysize
      PRINT,'xsize= ',xsize,' ysize= ',ysize,' tot1= ',xsize*ysize*tsize,' tot2= ',tot
    ENDIF
    
    
    mon=par1[4]
    ;if mon le 0. then begin
    ;mon=total(mo)
    ;mo=WOUT(xsize*ysize*tsize:cnt-1)
    ;print,'Using old monitor format'
    ;endif
    ;
    ;Check for detector/acquisition version: ;PG: implemented new acquisition card
    if nvers gt 2 then begin  ;new acquisition card (2013)
      tslices=par1[26]
      if tslices le 1 then begin ;no kinetic

        if tsize le 1 then begin ;noTOF count
          ;print,'MONOcase'
          w_buf=fltarr(xsize,ysize,tsize)
          if not nexus then det = REFORM(WOUT[2:*]      ,tsize,xsize,ysize) $
                       else det = REFORM((WOUT)         ,tsize,xsize,ysize)
          w_buf[*,*,0]=det[0,*,*]
        endif

        if tsize gt 1 then begin  ;TOF count
          ;print,'TOFcase'
          if not nexus then det = reform(WOUT[2*tsize:*],tsize,xsize) $
                       else det = reform(WOUT           ,tsize,xsize)
          ;w_buf=fltarr(xsize,ysize,tsize)
          ;for i=0,tsize-1 do w_buf[*,*,i]=det[i,*,*]
          w_buf = transpose(temporary(det), [1,0])
          w_buf = reform(w_buf, xsize,ysize,tsize)
        endif

        ; Top left pixel = 0 on detector, but lamp reads from bottom left
        if n_elements(where([ysize,tsize] eq 1)) ne 2 then w_buf=reverse(w_buf,2)
        x_buf=findgen(xsize+par1[97])
        y_buf=findgen(ysize+par1[99])
        z_buf=findgen(tsize)
        
      endif else begin  ;kinetic
        if tsize le 1 then begin ;noTOF count
          print,'MONOcase kinetic (not tested!)'
          if not nexus then det = reform(WOUT[2*tslices:*],tslices,xsize,ysize) $
                       else det = reform(WOUT             ,tslices,xsize,ysize)
          w_buf=fltarr(xsize,ysize,tslices)
          for i=0,tslices-1 do w_buf[*,*,i]=det[i,*,*]
          ; Top left pixel = 0 on detector, but lamp reads from bottom left
          if n_elements(where([ysize,tsize] eq 1)) ne 2 then w_buf=reverse(w_buf,2)
          x_buf=findgen(xsize+par1[97])
          y_buf=findgen(ysize+par1[99])
          z_buf=findgen(tsize)
        endif

        if tsize gt 1 then begin  ;TOF count
          print,'TOFcase kinetic'
          if not nexus then det = reform(WOUT[2*tsize*tslices:*],tsize,tslices,xsize) $
                       else det = reform(WOUT                   ,tsize,tslices,xsize)
          w_buf=fltarr(xsize,tsize,tslices)
          for i=0,xsize-1 do w_buf[i,*,*]=det[*,*,i]
          ; Top left pixel = 0 on detector, but lamp reads from bottom left
          ; if n_elements(where([ysize,tsize] eq 1)) ne 2 then w_buf=reverse(w_buf,2)
          x_buf=findgen(xsize+par1[97])
          y_buf=findgen(ysize+par1[99])
          z_buf=findgen(tsize)
        endif
        
      endelse
    endif
    
    ;if tsize gt 1 then mo=WOUT(xsize*ysize*tsize:cnt-1)
    ; Check for new detector vs. old detector  FC & ARW 21.04.10
    if nvers eq 2 then begin    ; New detector
      w_buf=WOUT[0:xsize*ysize*tsize-1]
      ; New detector counts TOF, then X, then Y
      det  =REFORM(w_buf,tsize,xsize,ysize)
      w_buf=fltarr(xsize,ysize,tsize)
      for i=0,tsize-1 do w_buf[*,*,i]=det[i,*,*]
      ; Top left pixel = 0 on detector, but lamp reads from bottom left
      if n_elements(where([ysize,tsize] eq 1)) ne 2 then w_buf=reverse(w_buf,2)
      x_buf=findgen(xsize+par1[97])
      y_buf=findgen(ysize+par1[99])
      z_buf=findgen(tsize)
      
    endif
    if nvers le 1 then begin      ; Old detector
      ; Old detector counts Y, then X, then TOF
      w_buf=WOUT[0:xsize*ysize*tsize-1]
      det  =REFORM(w_buf,ysize,xsize,tsize)
      w_buf=FLTARR(xsize,ysize,tsize)
      FOR i=0,tsize-1 DO w_buf[*,*,i]=ROTATE(det[*,*,i],3)
      x_buf=FINDGEN(xsize)+par1[97]
      y_buf=FINDGEN(ysize)+par1[99]
      x_buf=FINDGEN(xsize)
      z_buf=FINDGEN(tsize)
    endif
    n_buf=mon
    
    ; -----------------------------------
    WOUT = reform(temporary(w_buf)) ; (this removes W_buf)
    if nexus then PP   = [p_buf,PAR4]       else PP   = p_buf
    if nexus then PTXT = [par_txt_buf,PAR3] else PTXT = par_txt_buf
    NN   = n_buf
    ;  XX= 180/!pi*atan((cen_x-float(x_buf))*mppx/param(15))  ;Choose one!
    XX   = x_buf            ;Choose one!
    ;  YY=-180/!pi*atan((cen_y-float(y_buf))*mppy/param(15))  ;Choose one!
    YY   = y_buf            ;Choose one!
    ;;02.09.2014 TS - Need to istinguish mono and TOF scans
    ;;Figure out if monochromatic scan!!!
    if par1[87] eq 0 then begin
      ZZ= z_buf
    endif else begin
      ZZ   = (z_buf+delechan+0.5-delchan)/chanpa
    endelse
    
    ;; END 02.09.2014
    WT   = sub_title
    OT   = inst+' '+user+' '+main_title
    ;  XT='DELTA(angle) horizontal (degrees)'      ;Choose one!
    XT   ='X-pixel on detector'        ;Choose one!
    YT   ='DELTA(angle) vertical (degrees)'
    ZT   ='Wavelength ' + string("305B)
    
    ;correction for binning
    if (ysize eq 1) or (xsize eq 1) then begin
      if (tsize eq 1) then ZT='Counts'
      YY=ZZ
      YT=ZT
    endif
    
    
    IF (iprint GT 0) THEN PRINT,'rdid_d17: finished'
    
    RETURN
  END
