FUNCTION calib_zone, wkin1, wkin2,ncut, cal_fil_in, cal_fil_out, title
;+
;******* **********
;**
;** User callable
;** The call is w3=calib_zone(w1,w2,ncut,raw_calib_file,zone_calib_file,title)
;**
;** w1 & w2 are single point scans,
;** w1 at small angle, w2 at large angle
;** initially, 1st ncut tubes of w1 are used and the rest from w2
;** the beam stop is in different positions in each workspace
;-

;define number of tube and height pixels
take_datp,datp
npts  =(size(datp.x))[1] ;this dimension is nsteps*nt for a scan
nt    =128
nsteps=npts/nt
nh    =(size(datp.y))[1]
ncut  =5 ; number to separate data from small and high angle scans

;make a single-step-scan profile from the real scan
win1=fltarr(nt,nh) & win2=fltarr(nt,nh)
for i=0,nt-1 do begin
 stepmin=i*nsteps & stepmax=stepmin+nsteps-1
 win1[i,*]=total(wkin1[stepmin:stepmax,*],1)
 win2[i,*]=total(wkin2[stepmin:stepmax,*],1)
endfor
;plot = contour(win1, n_levels=20, rgb_table=13, /fill)
;plot = contour(win2, n_levels=20, rgb_table=13, /fill)

;define what fraction of background height determines cut-off
frac=2.0

;                      *** calibrating detector for position and length of detector tubes ***
;                      *** from single detector position, find detecting zones in 256 channels ***
;                      *** of a pair of tubes, interpolate these zones onto 128 channels ***
wdble_out1=fltarr(nt/2,nh*2)
rise11=intarr(nt/2) & fall11=intarr(nt/2) & rise12=intarr(nt/2) & fall12=intarr(nt/2)

FOR i=0,nt/2-1 DO BEGIN                                    ; find rise, fall, rise, fall
   wdouble=fltarr(nh*2)
   FOR j=0,nh-1 DO BEGIN
      wdouble[j]   =win1[i*2,j]                              ; make 1 double tube from a pair,
      wdouble[nh+j]=win1[i*2+1,nh-j-1]                       ; inverting 2nd tube
   ENDFOR
	 nhd=nh*2
   ave=total(wdouble)/nhd                                        ; average counts for 1 double detector tube
   idx1=where(wdouble[0:round(nhd/4)]               gt ave/frac) ; last value is index of 1st rise
   idx2=where(wdouble[round(nhd/4):round(3*nhd/4)]  lt ave/frac) ; 1st value is index of 1st fall , last value is index 2nd rise
   idx3=where(wdouble[round(3*nhd/4):nhd-1]         gt ave/frac) ; 1st value is index of 1st rise
   rise11[i]=idx1[0]                                 & fall11[i]=round(nhd/4)+idx2[0]-1
   rise12[i]=round(nhd/4)+idx2[n_elements(idx2)-1]+1 & fall12[i]=round(3*nhd/4)+idx3[n_elements(idx3)-1]-1
   wdble_out1[i,*]=wdouble
ENDFOR

;repeat for 2nd detector position
wdble_out2=fltarr(nt/2,nh*2)
rise21=intarr(nt/2) & fall21=intarr(nt/2) & rise22=intarr(nt/2) & fall22=intarr(nt/2)
FOR i=0,nt/2-1 DO BEGIN                                    ; find rise, fall, rise, fall
   wdouble=fltarr(nh*2)
   FOR j=0,nh-1 DO BEGIN
      wdouble[j]   =win2[i*2,j]                              ; make 1 double tube from a pair,
      wdouble[nh+j]=win2[i*2+1,nh-j-1]                       ; inverting 2nd tube
   ENDFOR
   nhd=nh*2
   ave=total(wdouble)/nhd                                       ; average counts for 1 double detector tube
   idx1=where(wdouble[0:round(nhd/4)]               gt ave/frac) ; last value is index of 1st rise
   idx2=where(wdouble[round(nhd/4):round(3*nhd/4)]  lt ave/frac) ; 1st value is index of 1st fall , last value is index 2nd rise
   idx3=where(wdouble[round(3*nhd/4):nhd-1]         gt ave/frac) ; 1st value is index of 1st rise
   rise21[i]=idx1[0]                                 & fall21[i]=round(nhd/4)+idx2[0]-1
   rise22[i]=round(nhd/4)+idx2[n_elements(idx2)-1]+1 & fall22[i]=round(3*nhd/4)+idx3[n_elements(idx3)-1]-1
   wdble_out2[i,*]=wdouble
ENDFOR

;combine rises and falls from both scans to eliminate beamstop
rise_fall=intarr(4,nt/2)
;take first ncut tubes from w1, the rest from w2
for i=0,ncut-1 do begin
rise_fall[0,i]=rise11[i] & rise_fall[1,i]=fall11[i] & rise_fall[2,i]=rise12[i] & rise_fall[3,i]=fall12[i]
endfor
for i=ncut,nt/2-1 do begin
rise_fall[0,i]=rise21[i] & rise_fall[1,i]=fall21[i] & rise_fall[2,i]=rise22[i] & rise_fall[3,i]=fall22[i]
endfor

;print, rise_fall

;update calibration file
openr,f_in,cal_fil_in,/get_lun
openw,f_out,cal_fil_out,/get_lun
;write new zones info
printf,f_out,title
printf,f_out, 'zones'
for i=0,nt/2-1 do begin
;insert * if active zone is badly defined
;good limits {5:20}, {115:130}, {125:140}, {235:250}
if (rise_fall[0,i] lt 5) or (rise_fall[0,i] gt 20) $
   or (rise_fall[1,i] lt 115) or (rise_fall[1,i] gt 130) $
   or (rise_fall[2,i] lt 125) or (rise_fall[2,i] gt 140) $
   or (rise_fall[3,i] lt 235) or (rise_fall[3,i] gt 255) then begin
      printf,f_out, rise_fall[0,i], rise_fall[1,i], rise_fall[2,i], rise_fall[3,i], '*'
endif else begin
   printf,f_out, rise_fall[0,i], rise_fall[1,i], rise_fall[2,i], rise_fall[3,i]
endelse
endfor
printf,f_out, 'efficiencies'
;copy rest of old calibration file
junk=''
readf,f_in,junk
while junk ne 'efficiencies' do readf,f_in,junk
while junk ne 'calib_end' do begin
   readf,f_in,junk
   printf,f_out,junk
endwhile
;printf,f_out,'calib_end'
free_lun, f_in & free_lun, f_out

;return 2 scan images in a concatenated workspace
return, [[wdble_out1],[wdble_out2]]

end
