PRO write_dat,file_out,w
;+
;** *********
;** User callable
;** procedure to write the 1D workspace (w) in the standard dat format
;** for d2b and d1a, this is similar to the gsas format
;** see PRO write_gsas
;** output file is file_out
;**
;** file header is
;** title
;** "100"/"0"/2theta_step/"1 0 0"
;** angle
;** monitor/"0."/3 temps
;-

; get structure (title, errors etc) associated with workspace w and store them in the local structure datp
take_datp, datp

;calculate parameters for header
npoints=n_elements(w) & print,npoints
two_theta_0=datp.x[0]
step=abs(datp.x[0]-datp.x[1])
monitor=datp.n[0] & print,monitor
t_req=datp.p[5] & t_reg=datp.p[6] & t_sample=datp.p[7] & print,t_sample

;transform errors into "number of times measured"
;number of counts/error^2
nerr=intarr(npoints)
nerr=round(w/datp.e^2)

;open file and write header
openw,u,file_out,/get_lun
printf,u,datp.w_tit
printf,u,format='("     100       0",f8.4,"       1       0       0")',step ;other pars
printf,u,format='(f8.4)',two_theta_0
printf,u,format='(f8.0,"      0.",3f8.2)',monitor,t_req,t_reg,t_sample

;write number of times measured/counts in 10 columns, watch out for last line
last=fix(npoints/10)*10
for i=0,last-1,10 do begin
printf,u,format='(10(i2,i6))', nerr[i],round(w[i]),nerr[i+1],round(w[i+1]),nerr[i+2],round(w[i+2]), $
                               nerr[i+3],round(w[i+3]),nerr[i+4],round(w[i+4]),nerr[i+5],round(w[i+5]), $
                               nerr[i+6],round(w[i+6]),nerr[i+7],round(w[i+7]),nerr[i+8],round(w[i+8]), $
                               nerr[i+9],round(w[i+9])
endfor
rest=npoints-last & print,rest
;if rest <>0 then last line to write will be incomplete
if (rest ne 0) then begin
   vec=intarr(2*rest)
   for i=0,rest-1 do begin & vec[2*i]=nerr[last+i] &  vec[2*i+1]=w[last+i] &  endfor
   printf,u,format='('+strtrim(string(rest),2)+'(i2,i6))',vec
endif

;close file
free_lun, u

print,stime(dum)
end
