; -----------------------------------------------------------------------------------------
;******************************************************************************************

PRO chemin, r, rp, fi, theta, xl

  precis = 1.e-7
  xm     = r * COS(fi)
  ym     = r * SIN(fi)
  
  xe     = -SQRT(1. - ym^2)
  t2t    = TAN(2.*theta)
  
  IF (ABS(theta*180./!pi - 45.) LT precis) OR (ABS(t2t) GT (1./precis)) THEN BEGIN
    xs = xm
    ys = SQRT(1. - xs*xs)
  ENDIF ELSE BEGIN
    IF t2t NE 0 THEN dsig = t2t/ABS(t2t) ELSE dsig = 1.
    par  = SIN(fi) - COS(fi)*t2t
    disc = 1. + t2t^2 - (r^2)*(par^2)
    xs   = (-r*t2t*par + dsig * SQRT(disc)) / (1. + t2t^2)
    ys   = xs * t2t + r*par
  ENDELSE
  xem = xm - xe
  xms = (xs - xm)^2 + (ys - ym)^2
  xms = SQRT(xms)
  xl  = xem + xms
  IF ABS(rp) LT precis THEN GOTO , finished
  
  ;  For a hollow cyclinder test to see if the neutron has a path in vacuum
  
  ;  Test of the outgoing beam
  ps = ABS(-xm * SIN(2.*theta) + ym * COS(2.*theta))
  IF (ps LT rp) THEN BEGIN
    ps2 = xm*xs + ym*ys - r^2
    IF (ps2 LT -precis) THEN BEGIN
      xv = 2.*SQRT(rp^2 - ps^2)	; Portion to discard on the outgoing beam path
      xl = xl - xv
    ENDIF
  ENDIF
  
  ; Test of the incoming beam
  IF (ABS(ym) LT rp) THEN BEGIN
    IF (xm GT 0.) THEN BEGIN
      xv2 = 2.*SQRT(rp^2 - ym^2)	; Portion to discard on the incoming beam path
      xl = xl - xv2
    ENDIF
  ENDIF
  IF ABS(xl) LE precis THEN xl = 0.
  
  finished:
END
; -----------------------------------------------------------------------------------------
;******************************************************************************************

FUNCTION abs_cyl, w_in, xmu = xmu, rp = rp, output_array = output_array
;+
;******* *******
;** Routine written by  B. Ouladdiaff and J. Rodriquez to correct for sample
;** attenuation in hollow cylinder geometry.
;** Only deals with 1d workspaces currently
;**  
;** xmu is mu*R(ex)
;** rp os ratio R(int)/R(ex)
;-
  
  IF (SIZE(w_in))[0] NE 1 THEN BEGIN
    s  = 'abs_cyl: Workspace must be 1d'
    ii = WIDGET_MESSAGE(s, /ERROR)
    RETURN, w_in
  ENDIF
  
  TAKE_DATP, datp
  x_in   = datp.x
  ntheta = N_ELEMENTS(x_in)
  xxl_arr = FLTARR(ntheta)
  
  nr = 20		;radial steps for integration
  nf = 20
  
  dr = (1. - rp)/FLOAT(nr)
  df = 2.*!pi/FLOAT(nf)
  
  FOR itheta = 0, ntheta - 1 DO BEGIN
    theta  = 0.5*x_in[itheta] * !pi/180.0
    stheta = SIN(theta)^2
    xxl    = 0.
    r      = rp
    xl2    = 0.
    fi     = 0.
    chemin, r, rp, fi, theta, y2
    FOR ifi = 1, nf DO BEGIN
      y0 = y2
      fi = ifi * df
      chemin, r, rp, fi, theta, y2
      fi = fi - 0.5 * df
      chemin, r, rp, fi, theta, y1
      xl2 = xl2 + r * (EXP(-xmu*y0) + 4.*EXP(-xmu*y1) + EXP(-xmu*y2)) / 6.
    ENDFOR
    
    FOR ir = 1, nr DO BEGIN
      xl0 = xl2
      r = rp + ir * dr
      xl2 = 0.
      fi = 0.
      chemin, r, rp, fi, theta, y2
      
      FOR ifi = 1, nf DO BEGIN
        y0 = y2
        fi = ifi * df
        chemin, r, rp, fi, theta, y2
        fi = fi - 0.5 * df
        chemin, r, rp, fi, theta, y1
        xl2 = xl2 + r * (EXP(-xmu*y0) + 4.*EXP(-xmu*y1) + EXP(-xmu*y2)) / 6.
      ENDFOR
      
      r = r - 0.5 * dr
      xl1 = 0.
      fi=0.
      chemin, r, rp, fi, theta, y2
      FOR ifi = 1, nf DO BEGIN
        y0 = y2
        fi = ifi * df
        chemin, r, rp, fi, theta, y2
        fi = fi - 0.5 * df
        chemin, r, rp, fi, theta, y1
        xl1 = xl1 + r * (EXP(-xmu*y0) + 4.*EXP(-xmu*y1) + EXP(-xmu*y2)) / 6.
      ENDFOR
      
      xxl = xxl + (xl0 + 4.*xl1 + xl2) / 6.
      
    ENDFOR
    xxl = xxl * dr * df / (!pi * (1. - rp*rp))
    xxl_arr[itheta] = xxl
  ENDFOR
  
  IF KEYWORD_SET(output_array) THEN BEGIN
    w_out = xxl_arr
    e_out = 0.*w_out
  ENDIF ELSE BEGIN
    w_out = w_in / xxl_arr
    e_out = datp.e / xxl_arr
  ENDELSE
  
  datp.e = e_out
  GIVE_DATP, datp
  
  RETURN, w_out
  
END
  
