; -----------------------------------------------------------------------
;************************************************************************

FUNCTION fold2, w_in
;+
;******* *****
;** User callable
;**  Takes a Qrebin or QxQy output grid, and averages over the Friedel positions
;**  (i.e. (qx,qy) and (-qx,-qy))
;** 
;** 
;**  (really) Last ever D7 macro written by.....
;**               JRS 29/11/11
;** ------------------------------------------------------------------------
;** ************************************************************************
;-
  
  TAKE_DATP, datp
  
  e_in = datp.e
  x_in = datp.x
  y_in = datp.y
  
  nQ = (SIZE(w_in))[2]
  
  minQx  = MIN(ABS(x_in))
  minQy  = MIN(ABS(y_in))
  ixzero = (WHERE(x_in EQ minQx))[0]
  iyzero = (WHERE(y_in EQ minQy))[0]
  
  
  IF (ixzero NE iyzero) THEN BEGIN
    PRINT, "fold2: Q-grid not square!"
    RETURN, w_in
  ENDIF
  
  nnQ = nQ-ixzero
  
  w_out = FLTARR(nnQ,nnQ)
  e_out = FLTARR(nnQ,nnQ)-1
  x_out = x_in[ixzero:nQ-1]
  y_out = y_in[iyzero:nQ-1]
  
  FOR qy = 0, nnQ DO BEGIN
    FOR qx = 0, nnQ DO BEGIN
      xpoint = x_in[qx]
      ypoint = y_in[qy]
      IF (xpoint GT 0 OR ypoint GT 0) THEN GOTO, exit
      ix = WHERE(-ABS(x_in) EQ xpoint)
      iy = WHERE(-ABS(y_in) EQ ypoint)
      IF(N_ELEMENTS(ix) LT 2 OR N_ELEMENTS(iy) LT 2) THEN GOTO, exit
      qxf = [ix[0],ix[0],ix[1],ix[1]]
      qyf = [iy[0],iy[1],iy[1],iy[0]]
      
      igood = WHERE(e_in[qxf,qyf] NE -1,ngood)
      
      IF ngood EQ 0 THEN BEGIN
        w_average = 0.0
        e_average = -1.0
      ENDIF ELSE BEGIN
        w_average = TOTAL(w_in[qxf[igood],qyf[igood]])/ngood
        e_average = SQRT(TOTAL(e_in[qxf[igood],qyf[igood]]^2))/ngood
      ENDELSE
      ixout = WHERE(x_out EQ ABS(xpoint))
      iyout = WHERE(y_out EQ ABS(ypoint))
      w_out[ixout,iyout] = w_average
      e_out[ixout,iyout] = e_average
      exit:
    ENDFOR
  ENDFOR
  
  ; insert x=0 and y=0 values
  i = INDGEN(nnQ)
  w_out[0,i] = w_in[ixzero,iyzero+i] +  w_in[ixzero,iyzero-i] / 2.
  e_out[0,i] = SQRT(e_in[ixzero,iyzero+i]^2 +  e_in[ixzero,iyzero-i]^2) / 2.
  w_out[i,0] = w_in[ixzero+i,iyzero] +  w_in[ixzero-i,iyzero] / 2.
  e_out[i,0] = SQRT(e_in[ixzero+i,iyzero]^2 +  e_in[ixzero-i,iyzero]^2) / 2.
  
  i = WHERE(w_out EQ 0)
  IF N_ELEMENTS(i) GE 1 THEN BEGIN
    w_out[i] = 0.0
    e_out[i] = -1
  ENDIF
  
  ; ---------------------------------------------------------------------
  ;Return data and exit
  
  PRINT, 'Fold2: Data folded into one qaudrant'
  datp.other_tit = datp.other_tit + ' -ff2'
  MOD_DATP, datp, "e", e_out
  MOD_DATP, datp, "x", x_out
  MOD_DATP, datp, "y", y_out
  
  GIVE_DATP, datp
  RETURN, w_out
  
END

