; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION input_quartz, numor = numor, nm
;+
;******* ************
;** User callable
;** Read a quartz flipping ratio file 'quartz_<numor>_<temp>.dat' and loads it
;** into desired workspace
;** 
;** ARGUMENTS:
;**  numor  : numor corresponding to quatzfile of interest
;**  temp  : temperature parameter of corrected quartzfile (see quartz.pro)
;**  (nm is obsolete, kept for backwards compatability)
;** 
;** COMMAND SYNTAX:
;**  w1=input_quartz(numor=<numor>)
;**  (optional arguments shown in square brackets)
;** 
;**               KHA,JRS 16/5/03
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
  
  iprint = 0  ; if iprint > 0, show debugging messages
  
  IF iprint THEN PRINT,'Start input_quartz:'
  
  TAKE_DATP, datp
  
  IF N_ELEMENTS(nm) GT 0 THEN numor = nm
  ; ------------------------------------------------------------------------------
  ;Open and read from input file
  
  in_file = 'quartz_' + STRTRIM(numor,2) + '.dat'
  line   = ' '
  
  IF iprint THEN PRINT,'Opening input file'
  OPENR, lun, in_file, ERROR = err, /GET_LUN
  IF (err NE 0) THEN BEGIN
    ii = dialog_message(!ERROR_STATE.msg, /ERROR)
    RETURN, 0
  ENDIF
  READF, lun, line
  READF, lun, line
  READF, lun, line
  line = line + ' -1'
  READS, line, ispec, iangle, flip, dflip, test
  IF FIX(test) EQ -1 THEN xyz = 0 ELSE xyz = 1
  
  datp.x_tit = 'Spectrum Number'
  IF NOT xyz THEN BEGIN
    q_buf      = FLTARR(4)
    datp.y_tit = 'Flipping Ratio'
  ENDIF ELSE BEGIN
    q_buf      = FLTARR(8)
    y_in       = INDGEN(3)
    datp.y_tit = 'Phase'
    datp.z_tit = 'Flipping Ratio'
  ENDELSE
  
  buf = q_buf # TRANSPOSE(INDGEN(1000) + 1)
  
  POINT_LUN, lun, 0
  READF, lun, line
  datp.w_tit = line
  READF, lun, line
  
  ON_IOERROR, end_of_file
  FOR iline = 0, 1000 DO BEGIN
    READF, lun, q_buf
    buf[*,iline] = q_buf
  ENDFOR
  
end_of_file:
nspectra = iline
ii = INDGEN(nspectra)
buf = buf[*,ii]
FREE_LUN, lun

x_in = buf[0,*]

IF xyz THEN BEGIN
  w_in = TRANSPOSE(buf[[2,4,6],*])
  e_in = TRANSPOSE(buf[[3,5,7],*])
ENDIF ELSE BEGIN
  w_in = buf[2,*]
  e_in = buf[3,*]
ENDELSE

IF iprint THEN PRINT,'End of "Read from input file" section'

; ------------------------------------------------------------------------------
;Return and exit

MOD_DATP, datp, "x", x_in
IF xyz THEN MOD_DATP, datp, "y", y_in
MOD_DATP, datp, "e", e_in

datp.other_tit = 'Input file: ' + in_file

GIVE_DATP, datp

finished:
IF iprint THEN PRINT,'End input_quartz:'

RETURN, w_in
END
