; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION input_vanadium, numor, number=number, angle=angle
;+
;******* **************
;** User callable
;** Reads the vanadium file 'vanadium_<numor>.dat', and puts it into a
;** 1-D workspace.  Also reads in multiple numors (e.g. numor=[47324,47336]) and
;** puts them into a 2-d workspace.
;** 
;** ARGUMENTS:
;**  numor    :numor(s) corresponding to vanadium file(s) to be read in
;** 
;** KEYWORDS:
;**  /number  :x-axis is spectrum number (default)
;**  /angle  :x-axis is scattering angle
;** 
;** COMMAND SYNTAX:
;**  w1=input_vanadium(<numor>[,nspectra=<nspectra>][,/number][,/angle])
;**  (optional keywords/arguments are shown in square brackets)
;** 
;**               KHA,JRS  28/5/07
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF iprint THEN PRINT,'Start input_vanadium:'
  
  TAKE_DATP, datp
  
  
  IF KEYWORD_SET(number)  THEN ix = 0
  IF KEYWORD_SET(angle)   THEN ix = 1 ELSE ix = 0
  
  ; ------------------------------------------------------------------------------
  ;Open and read from input file
  
  nruns = 1
  IF N_ELEMENTS(numor) GT 0 THEN BEGIN
    IF N_ELEMENTS(numor) EQ 2 THEN BEGIN
      nruns = numor[1] - numor[0] + 1
      nm = LONARR(nruns)
      FOR i = 0, nruns-1 DO nm[i] = numor[0]+i
      numor = nm
    ENDIF ELSE BEGIN
      nruns = N_ELEMENTS(numor)
    ENDELSE
  ENDIF
  
  x_out = FLTARR(1000)
  w_out = FLTARR(1000,nruns)  & e_out=w_out & y_out=w_out
  
  FOR irun = 0, nruns - 1 DO BEGIN
    in_file = 'vanadium_' + STRTRIM(numor[irun],2) + '.dat'
    line = ' '
    v_buf = FLTARR(4)
    buf   = FLTARR(4,1000)
    IF iprint THEN PRINT,'Opening input file ',in_file
    OPENR, lun, in_file, ERROR = err, /GET_LUN
    IF (err NE 0) THEN BEGIN
      PRINT, !ERROR_STATE.msg
      GOTO, finished
    ENDIF
    READF, lun, line
    READF, lun, line
    ON_IOERROR, end_of_file
    FOR i = 0, 900 DO BEGIN
      READF, lun, v_buf
      buf[*,i] = v_buf
    ENDFOR
    FREE_LUN, lun
  end_of_file:
  isp = INDGEN(i)
  IF irun EQ 0 THEN x_out[isp]=buf[ix,isp]
  y_out[isp,irun] = buf[ix,isp]
  w_out[isp,irun] = buf[2, isp]
  e_out[isp,irun] = buf[3, isp]
ENDFOR
w_out = w_out[isp,*]
e_out = e_out[isp,*]
y_out = y_out[isp,*]
x_out = x_out[isp]

IF iprint THEN PRINT,'End of "Read from input file" section'

; ------------------------------------------------------------------------------
;Return and exit

datp.x_tit = 'Spectrum Number'
IF ix EQ 1 THEN datp.x_tit='2-theta (degrees)'

IF nruns EQ 1 THEN datp.y_tit = 'Vanadium Integral' ELSE $
  datp.z_tit = 'Vanadium Integral'
  
IF nruns EQ 1 THEN nums = '#' + STRTRIM(STRING(numor[0]),2) ELSE $
  nums = '#' + STRTRIM(STRING(numor[0]),2) + ':' + $
  STRTRIM(STRING(numor[nruns-1]),2)
  
datp.w_tit = 'Vanadium Intensities from ' + nums

MOD_DATP, datp, "x", x_out
MOD_DATP, datp, "e", e_out
MOD_DATP, datp, "y", y_out

GIVE_DATP, datp

finished:
IF iprint THEN PRINT,'End input_vanadium:'

RETURN, w_out
END
