; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION average, w_in1, w_in2, weighted=weighted
;+
;******* *******
;** User callable
;** Takes the (weighted) average of two workspaces. Default = straight average
;** 
;**             KHA, 28/5/07
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  IF KEYWORD_SET(weighted) THEN weight=1 ELSE weight = 0
  
  TAKE_DATP, dat1
  TAKE_DATP, dat2, /THIRD
  
  e_in1 = dat1.e
  e_in2 = dat2.e
  
  IF weight THEN BEGIN
    iz1=WHERE(e_in1 EQ 0.,nz1)
    IF (nz1 NE 0) THEN e_in1[iz1]=1.
    iz2=WHERE(e_in2 EQ 0.,nz2)
    IF (nz2 NE 0) THEN e_in2[iz2]=1.
    w_out = (w_in1/e_in1^2 + w_in2/e_in2^2) / (1./e_in1^2 + 1./e_in2^2)
    e_out = 1. / SQRT(1./e_in1^2 + 1./e_in2^2)
  ENDIF ELSE BEGIN
    w_out=(w_in1+w_in2)/2.
    e_out=SQRT(e_in1^2+e_in2^2)/2.
  ENDELSE
  
  iz1=WHERE(e_in1 LT 0.,nz1)
  IF (nz1 NE 0) THEN BEGIN
    w_out[iz1]=w_in2[iz1]
    e_out[iz1]=e_in2[iz1]
  ENDIF
  iz2=WHERE(e_in2 LT 0.,nz2)
  IF (nz2 NE 0) THEN BEGIN
    w_out[iz2]=w_in1[iz2]
    e_out[iz2]=e_in1[iz2]
  ENDIF
  
  datp = dat1
  
  MOD_DATP,  datp, "e", e_out
  GIVE_DATP, datp
  
  RETURN, w_out
END
