;+
; The order of the detectors was changed during the winter shutdown 2011-2012
; This routine has been modified to use the new arrangement
; ARW 9.6.12
; Modified to fix file index leak, ARW 1.8.13
;-

FUNCTION D7_calibration, w_in0, calib=calib,raw=raw, monitor=monitor, $
         detector=detector, alldetectors=alldetectors, ei=ei, $
         madangles=madangles, zeroshift=zeroshift, nopo = nopo, $
         time=time,  inorm, ikeep
;+
;******* **************
;** User callable
;-

  TAKE_DATP, datp
  ; ----------------------------------------------------------------
  COMMON c_lamp_access, inst
  COMMON printing, iprint, outstring
  
  iprint = 0  ; if iprint>0, show debugging messages
  
  IF iprint THEN PRINT,'Start normalise:'
  
  IF N_ELEMENTS(inorm) EQ 0 THEN inorm = 1    ;default
  IF KEYWORD_SET(raw)       THEN inorm = 0
  IF KEYWORD_SET(monitor)   THEN inorm = 1
  IF KEYWORD_SET(time)      THEN inorm = 2
  IF KEYWORD_SET(detector)  THEN BEGIN
    IF(detector NE 0) THEN inorm = -detector
  ENDIF
  
  IF N_ELEMENTS(ikeep) EQ 0 THEN ikeep = 0      ;default
  IF KEYWORD_SET(madangles) AND     KEYWORD_SET(alldetectors) THEN ikeep = 1
  IF KEYWORD_SET(madangles) AND NOT KEYWORD_SET(alldetectors) THEN ikeep = -1
  IF NOT KEYWORD_SET(madangles) AND KEYWORD_SET(alldetectors) THEN ikeep = 2
  IF KEYWORD_SET(madangles) AND     KEYWORD_SET(nopo)         THEN ikeep = 3
  IF NOT KEYWORD_SET(madangles) AND KEYWORD_SET(nopo)         THEN ikeep = 4
  
  IF N_ELEMENTS(zeroshift) EQ 0 THEN zeroshift = 0.0
  IF KEYWORD_SET(keepmonitors) THEN keepmons = 1 ELSE keepmons = 0
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  ParamFile =  'D7_' + STRTRIM(calib,2) + '.dat'
  print,'Running D7_CALIBRATION'
  print, ParamFile
  OPENR, lun, ParamFile, /GET_LUN
  
  header = STRARR(2)
  READF, lun, header
  print, 'Using Parameter file:'
  print, header
  mParam = FLTARR(3, 4)
  READF, lun, mParam
  print, mParam
  lambda = FLTARR(1,1)
  READF, lun, lambda
  print, 'lambda = ', lambda
  ;  print, inorm
  
  ; ***  Next 2 lines added 1.8.13, ARW
  close,lun
  free_lun,lun
  
  m_in0     = datp.n
  x_in0     = datp.x
  y_in0     = datp.y
  z_in0     = datp.z
  e_in0     = datp.e
  parp      = datp.p
  parv      = datp.pv
  e_in0     = SQRT(w_in0)
  TOF       = LONG(parp[8])
  nchannels = LONG(parp[6])
  nspectra  = LONG(parp[1])
  nphases   = LONG(parp[2])
  se        = SIZE(e_in0)
  sw        = SIZE(w_in0)
  par_dim = (SIZE(datp.p))[1]
  
  
  ll = parp[4]
  parp[4] = lambda
  ;        IF (ll GT 3.5) AND (ll LE 5.0) THEN parp(4) = 4.89
  ;        IF (ll LT 3.5) THEN parp(4) = lambda
  ;        IF (ll GT 5.0) THEN parp(4) = 5.9
  rearrange = 0
  IF NOT TOF AND (nphases EQ 1) THEN BEGIN
    IF N_ELEMENTS(sw) EQ 4 THEN nruns = 1 ELSE nruns = sw[sw[0]]
    PRINT,'Normalise_d7: Assuming workspace contains a series of no-PA runs'
    rearrange = 1
    w_in = FLTARR(nspectra,nphases,nruns) & w_in[*,0,*]=w_in0[*,*]
    e_in = FLTARR(nspectra,nphases,nruns) & e_in[*,0,*]=e_in0[*,*]
    y_in = y_in0
    sw   = size(w_in)
    datp.p = parp
    ;    datp.pv  = parp(*,nruns-1)
    datp.pv  = parv[*,nruns-1]
  ENDIF ELSE IF sw[0] EQ 2 THEN nruns = 1 ELSE nruns = sw[3]
  norm = inorm
  keep = ikeep
  nspec_in = nspectra
  ;  print, nruns
  
  IF nruns EQ 1 THEN BEGIN
    datp.p = parp
  ;  datp.pv = datp.p
  ;  a = size(parp)
  ;  print, a(1)
  ;  datp.pv = INTARR(a(1))
  ;  b = parp
  ;  print, b
  ;  datp.pv = parp
  ENDIF ELSE BEGIN
    datp.p = parp
    a = size(parp)
    ;    print, a(1)
    ;  datp.pv = FLTARR(a(1), nruns-1)
    b = parp
    FOR i = 1,nruns-1 do b  = [[b],[parp]]
  ;  datp.pv = b
  ENDELSE
  ; Choose appropriate detectors------------------------------------------------
  
  
  IF TOF AND (norm EQ 1) THEN norm = -133
  
  IF keep LE 0 THEN BEGIN
    nspectra = 132      ;data cycle 073
    goodspecs =  [INDGEN(132)]
  ENDIF ELSE BEGIN
    IF keep GT 2 THEN BEGIN    ;nopo
      nspectra  = 132    ;data cycle 073
      goodspecs = INDGEN(nspectra)
    ENDIF ELSE BEGIN
      nspectra  = 132    ;data cycle 073
      goodspecs = INDGEN(nspectra)
    ENDELSE
  ENDELSE
  
  parp[1] = nspectra
  parp[3] = nruns
  
  IF iprint THEN BEGIN
    PRINT,'size(w_in)= ',sw
    ;  PRINT,'nvers     = ',nvers
    PRINT,'TOF       = ',TOF
    PRINT,'nchannels = ',nchannels
    PRINT,'nspectra  = ',nspectra
    PRINT,'nphases   = ',nphases
    PRINT,'nruns     = ',nruns
    PRINT,'norm      = ',norm
    PRINT,'keep      = ',keep
  ENDIF
  
  m_in = m_in0
  x_in = x_in0
  sz   = SIZE(z_in0)
  IF sz[0] EQ 3 THEN BEGIN
    ns   = sz[1]
    nr   = sz[3]
    z_in = REFORM(z_in0,ns,nr)
  ENDIF ELSE z_in = z_in0
  IF NOT rearrange THEN BEGIN
    w_in = w_in0
    e_in = e_in0
    y_in = y_in0
  ENDIF
  
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;For TOF data, find elastic peak
  
  IF TOF THEN BEGIN
  
    specs = goodspecs
    FOR iphase = 1,nphases-1 DO specs = [specs,goodspecs + nspec_in*iphase]
    ytot   = TOTAL(w_in[*,specs,*],2)
    IF nruns GT 1 THEN ytot = TOTAL(ytot,2)
    etot   = SQRT(ytot)
    xtot   = x_in
    y0     = MAX(ytot,i0)
    xmin   = xtot[(i0 - 20) > 0]
    xmax   = xtot[(i0 + 20) < (nchannels - 1)]
    gauss  = FLTARR(4)
    dgauss = FLTARR(4)
    
    FITGAUSS, xtot, ytot, etot, xmin, xmax, gauss, dgauss
    
    chel   = gauss[2]
    parp[9]= chel
    
    IF iprint THEN PRINT, 'Normalise: Elastic channel = ' + $
      STRTRIM(STRING(chel),2)
    IF iprint THEN PRINT,'End of "elastic peak" section'
  ENDIF
  
  ; ------------------------------------------------------------------------------
  ;Perform normalisation
  
  IF NOT TOF THEN BEGIN
  
    x_out = x_in[goodspecs]
    IF nruns GT 1 THEN z_out = z_in[goodspecs,*] ELSE z_out = x_out
    
    iphase = INDGEN(nphases)
    irun   = INDGEN(nruns)
    
    w_in = w_in[goodspecs,*,*]
    e_in = e_in[goodspecs,*,*]
    
    IF norm EQ 0 THEN normf = 1.
    IF norm EQ 1 THEN normf = m_in[1,iphase+1,irun]/1000.
    IF norm EQ 2 THEN normf = m_in[0,iphase+1,irun]/100.
    IF norm LT 0 THEN normf = w_in[-norm-1,iphase,irun]/1000.
    
    nspecs = (SIZE(w_in))[1]
    normf = normf[INTARR(nspecs),*,*]
    IF iprint THEN HELP, normf
    
    w_out = w_in/normf
    e_out = e_in/normf
    
    IF (nruns EQ 1) OR (nphases NE 1) THEN y_out = y_in $
    ELSE BEGIN
    y_out    = INTARR(nruns)
    y_out    = LONG(parv[0,*])
  ENDELSE
ENDIF ELSE BEGIN
  x_out    =  x_in
  y_out    = FLTARR(nspectra*nphases)
  iphase   = INDGEN(nphases)
  irun     = INDGEN(nruns)
  tofspecs = INTARR(nspectra*nphases)
  
  FOR iiphase = 0, nphases - 1 DO BEGIN
    ii = iiphase*nspectra
    tofspecs[ii:ii + nspectra - 1] = goodspecs + nspec_in*iiphase
  ENDFOR
  
  y_out = y_in[tofspecs]
  z_out = (y_out)[*,INTARR(nruns)]
  
  ; Build normalisation array
  
  CASE 1 OF
    norm EQ 0: normf = 1.
    norm EQ 1: normf = m_in[1,iphase + 1,irun]/1000.
    norm EQ 2: normf = m_in[0,iphase + 1,irun]/100.  ; time
    norm LT 0: BEGIN     ; detector
      ispec = -norm - 1 + iphase*nspec_in
      normf = TOTAL(w_in[*,ispec,irun],1)/1000.
      w_in[*,ispec,irun] = 0.
      e_in[*,ispec,irun] = -1.
    END
  ENDCASE
  
  IF norm NE 0 THEN BEGIN
    a = (FLTARR(nspec_in) + 1.) # TRANSPOSE(iphase)
    b = REFORM(a,nspec_in*nphases)
    c = REFORM(normf[b,*],1,nspec_in*nphases,nruns)
    normf = c[INTARR(nchannels),*,*]
  ENDIF
  
  w_out  = w_in/normf
  e_out  = e_in/normf
  parp[6]= nchannels
  w_out  = w_out[*,tofspecs,*]
  e_out  = e_out[*,tofspecs,*]
ENDELSE

IF iprint THEN BEGIN
  PRINT,'size(w_out)=',size(w_out)
  PRINT,'size(e_out)=',size(e_out)
  PRINT,'size(x_out)=',size(x_out)
  PRINT,'size(y_out)=',size(y_out)
  PRINT,'size(z_out)=',size(z_out)
ENDIF

IF iprint THEN PRINT,'End of "Perform normalisation" section'

; ------------------------------------------------------------------------------
;Recalculate and sort detector angles from YIG calibration
;MOST RECENT CORRECTION; YIG scan March 2009
;print, 'ooh goody'
IF (keep EQ 0) OR (keep EQ 2) OR (keep EQ 4) THEN BEGIN
  IF iprint THEN PRINT,'Recalculating detector angles'
  
  IF parp[16] EQ 0. THEN BEGIN
    i = dialog_message('Normalise: Cannot recalibrate detector angles',/ERROR)
    GOTO, endangles
  ENDIF
  
  
  ;      print, 'ooh goody'
  bank = FLTARR(4,nruns)
  IF nruns EQ 1 THEN bank =   parp[[16,17,18,19]] $
  ELSE bank =   parv[[16,17,18,19],*]
  offset4 = FLTARR(44)
  FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
    offset4[ispec] = mParam[2,0]*(ispec + 1) + mParam[2,2] ELSE $
    offset4[ispec] = mParam[2,1]*(ispec + 1) + mParam[2,3]
  offset3 = FLTARR(44)
  FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
    offset3[ispec] = mParam[1,0]*(ispec + 1) + mParam[1,2] ELSE $
    offset3[ispec] = mParam[1,1]*(ispec + 1) +  mParam[1,3]
  offset2 = FLTARR(44)
  FOR ispec = 0,43 DO IF (ispec + 1)/2 EQ FLOAT(ispec + 1)/2. THEN $
    offset2[ispec] = mParam[0,0]*(ispec + 1) + mParam[0,2] ELSE $
    offset2[ispec] = mParam[0,1]*(ispec + 1) + mParam[0,3]
  bank4 = FLTARR(44,nruns)
  bank3 = FLTARR(44,nruns)
  bank2 = FLTARR(44,nruns)
  
  FOR irun = 0, nruns - 1 DO BEGIN
    bank4[*,irun] = bank[3,irun] - offset4
    bank3[*,irun] = bank[2,irun] - offset3
    bank2[*,irun] = bank[1,irun] - offset2
  ENDFOR
  ; *** Have modified here for new detector arrangement: ARW 9.6.12
  if parp[48] lt 8 then phi = [bank4,bank3,bank2] else phi=[bank2,bank3,bank4]
  z_out = phi[goodspecs,*]
  phi = phi[goodspecs]
  IF NOT TOF THEN BEGIN
    x_out = phi
  ENDIF ELSE BEGIN
    IF nphases EQ 1 THEN y_out = phi $
    ELSE IF nphases EQ 2 THEN y_out = [phi,phi] $
  ELSE y_out = [phi,phi,phi,phi,phi,phi]
  IF iprint THEN PRINT,'y_out=',y_out
ENDELSE

IF iprint EQ 2 THEN PRINT,'z_out=',z_out
ENDIF

endangles:

IF iprint THEN PRINT,'End of "Calibrate angles" section'


; ------------------------------------------------------------------------------
;Return parameters and exit

datp.p = parp
MOD_DATP, datp, "x", x_out
MOD_DATP, datp, "z", z_out

IF NOT TOF AND nphases EQ 1 THEN BEGIN
  w_out0 = FLTARR(nspectra,nruns) & e_out0 = w_out0
  w_out0[*,*] = w_out[*,0,*]
  e_out0[*,*] = e_out[*,0,*]
  IF nruns GT 1 THEN MOD_DATP, datp, "y", y_out
ENDIF ELSE BEGIN
  w_out0 = w_out
  e_out0 = e_out
  MOD_DATP, datp, "y", y_out
ENDELSE

MOD_DATP, datp, "e", e_out0

label=' '

CASE 1 OF
  norm EQ 0: BEGIN
    nors = '/r'
    nstring = 'no normalisation'
  END
  norm EQ 1: BEGIN
    nors = '/m'
    nstring = 'normalised to M1'
  END
  norm EQ 2: BEGIN
    nors = '/t'
    nstring = 'normalised to counting time'
  END
  (norm EQ -65) OR (norm EQ -75) OR (norm EQ -115) OR (norm EQ -149) OR (norm EQ -93): BEGIN
    nors = '/m'
    nstring = 'normalised to TOF-M1'
  END
  ELSE: BEGIN
    nors = 'det='+STRTRIM(STRING(-norm),2)
    nstring='normalised to s'+STRTRIM(STRING(-norm),2)
  END
ENDCASE

CASE keep OF
  1: BEGIN
    kees = ',/all,/mad'
    kstring = 'all detectors kept, raw det angles used'
  END
  -1: BEGIN
    kees = ',/mad'
    kstring = 'hidden detectors discarded, raw det angles used'
  END
  2: BEGIN
    kees = ',/all'
    kstring = 'all detectors kept, det angles recalculated'
  END
  3: BEGIN
    kees = ',/nopo,/mad'
    kstring = '"nopo" detectors kept, raw det angles used'
  END
  4: BEGIN
    kees = ',/nopo'
    kstring = '"nopo" detectors kept, det angles recalculated'
  END
  
  ELSE: BEGIN
    kees = ''
    kstring = 'hidden detectors discarded, det angles recalculated'
  END
ENDCASE

normalisation = ' no(' + nors + kees
IF zeroshift GT 0.0 THEN BEGIN
  s=STRTRIM(STRING(zeroshift),2) & i=STRPOS(s,'.') & zers=STRMID(s,0,i+3)
  normalisation = normalisation+',zs='+zers
ENDIF
IF NOT TOF THEN normalisation = normalisation+')' ELSE BEGIN
  s=STRTRIM(STRING(chel),2) & i=STRPOS(s,'.') & chel=STRMID(s,0,i+3)
  normalisation = normalisation+',ce='+chel+')'
  cstring = '.  Elastic Channel = '+chel
ENDELSE

IF NOT TOF THEN BEGIN
  cstring = ''
  IF (nruns EQ 1) THEN BEGIN
    IF (nphases EQ 1) THEN $
      datp.y_tit = STRTRIM(label,2) $
    ELSE BEGIN
    datp.y_tit = 'Phase'
    datp.z_tit = STRTRIM(label,2)
  ENDELSE
ENDIF ELSE BEGIN
  IF (nphases EQ 1) THEN BEGIN
    datp.y_tit = 'Run Number'
    datp.z_tit = STRTRIM(label,2)
  ENDIF ELSE BEGIN
    datp.y_tit = 'Phase'
    datp.z_tit = 'Run Number'
  ENDELSE
ENDELSE
ENDIF ELSE BEGIN
  datp.y_tit = 'Scattering Angle'
  IF (nruns EQ 1) THEN datp.z_tit = STRTRIM(label,2) $
  ELSE datp.z_tit = 'Numor'
ENDELSE

s=datp.other_tit & i= strpos(/reverse_search,s,' ') & n=STRLEN(s) & numor=STRMID(s,i+1,n-1)
IF (nruns EQ 1) AND (STRPOS(numor,'>') EQ -1) THEN numor = STRTRIM(STRING(LONG(parp[0])),2)
IF (nruns EQ 1) AND (STRPOS(numor,'>') NE -1) THEN numor = numor
IF nruns NE 1 THEN $
  numor=STRTRIM(STRING(LONG(parv[0,0])),2)+':'+STRTRIM(STRING(LONG(parv[0,nruns-1])),2)
  
IF (STRPOS(s,' -ax') EQ -1 AND STRPOS(s,' -cc') EQ -1) THEN $  ; data not previously concatenated, or add_xyz'd
  datp.other_tit='D7 #'+numor+normalisation $
ELSE datp.other_tit=datp.other_tit+normalisation

outstring = nstring + ', ' + kstring + cstring

finished:
IF iprint THEN PRINT,'End normalise:'

GIVE_DATP, datp

RETURN, w_out0
END
