; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION input, file = in_file, fin
;+
;******* *****
;** User callable
;** Reads a file previously saved by output.pro (with titles), or any XYE
;** formatted file.
;** 
;** ARGUMENTS
;**  file  :name of file to input
;**  (the argument fin is obsolete, kept for backwards compatability)
;** 
;** COMMAND SYNTAX:
;**  w1=input(file='<in_file>')
;**  NB - don't forget to include single quotes
;** 
;**               KHA,JRS 15/4/03
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  iprint = 0  ; if iprint>0, show debugging messages
  IF iprint THEN PRINT,'Start input:'
  TAKE_DATP, datp
  IF N_ELEMENTS(fin) GT 0 THEN in_file = fin
  
  ; ------------------------------------------------------------------------------
  ;Open and read from input file
  
  line = ' '
  IF iprint THEN PRINT,'Input: Opening input file'
  OPENR, lun, in_file, /GET_LUN, ERROR = err
  IF err NE 0 THEN BEGIN
    s = dialog_message(!ERROR_STATE.msg, /ERROR)
    GOTO, finished
  ENDIF
  
  READF, lun, line
  oned      = 1
  oldformat = 0
  IF STRPOS(line,' [WORKSPACE]') NE -1 THEN oned = 0
  IF STRPOS(line,'SIZE') NE -1 THEN BEGIN
    oned = 0
    IF iprint THEN PRINT, 'Input: Old data format'
    oldformat = 1
  ENDIF
  
  IF oned THEN BEGIN
    IF iprint THEN PRINT,'1d data format'
    POINT_LUN, lun, 0
    ON_IOERROR, nomoredata
    
    m = 10000  & n = 0
    x_in = FLTARR(m)  & w_in = x_in  & e_in = x_in
    FOR i = 0, m - 1 DO BEGIN
      READF, lun, x, w, e
      x_in[i] = x  & w_in[i] = w  & e_in[i] = e
    ENDFOR
    nomoredata:
    x_in = x_in[0:i-1]
    w_in = w_in[0:i-1]
    e_in = e_in[0:i-1]
    n = i
    
    ON_IOERROR, endoffile
    POINT_LUN, lun, 0
    FOR i = 0, n - 1 DO READF, lun, x
    READF, lun, line
    IF STRPOS(line,'[XTITLE]') EQ -1 THEN BEGIN
      oldformat = 1
      IF iprint THEN PRINT, 'Input: Old data format'
      datp.x_tit = line
    ENDIF ELSE BEGIN
      READF, lun, line  & datp.x_tit=line
    ENDELSE
    IF NOT oldformat THEN READF, lun, line
    READF, lun, line  & datp.y_tit=line
    IF NOT oldformat THEN READF, lun, line
    READF, lun, line  & datp.w_tit=line
    IF NOT oldformat THEN READF, lun, line
    READF, lun, line  & datp.other_tit=line
    READF, lun, line
    READF, lun, line
    sp  = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
    dim = FIX(STRMID(sp,12,12))
    par = FLTARR(dim) & par_txt = STRARR(dim)
    READF, lun, par
    READF, lun, line
    READF, lun, par_txt
    
  ENDIF ELSE BEGIN
  
    ON_IOERROR, endoffile
    IF iprint THEN PRINT,'More-than-1d data format'
    IF oldformat EQ 0 THEN READF, lun, line
    sw = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
    ndims = FIX(STRMID(sw,0,12))
    dim = INTARR(ndims)
    FOR i = 0,ndims - 1 DO dim[i] = FIX(STRMID(sw,12*(i+1),12))
    IF (ndims EQ 2) THEN w_in = FLTARR(dim[0],dim[1]) $
    ELSE IF (ndims EQ 3) THEN w_in = FLTARR(dim[0],dim[1],dim[2]) $
  ELSE w_in=FLTARR(dim[0],dim[1],dim[2],dim[3])
  READF, lun, w_in
  IF iprint THEN PRINT,'w_in read OK'
  
  IF oldformat EQ 0 THEN READF, lun, line
  e_in = w_in*0.
  READF, lun, e_in
  IF iprint THEN PRINT,'e_in read OK'
  
  IF NOT oldformat THEN READF, lun, line
  READF, lun, line
  sx    = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
  ndims = FIX(STRMID(sx,0,12))
  IF ndims GE 1 THEN BEGIN
    dim = INTARR(ndims)
    FOR i = 0, ndims - 1 DO dim[i] = FIX(STRMID(sx,12*(i+1),12))
    IF ndims EQ 1 THEN x_in = FLTARR(dim[0]) $
    ELSE IF ndims EQ 2 THEN x_in = FLTARR(dim[0],dim[1]) $
  ELSE x_in = FLTARR(dim[0],dim[1],dim[2])
ENDIF ELSE x_in = 0
READF, lun, x_in
IF iprint THEN PRINT,'x_in read OK'

IF NOT oldformat THEN READF, lun, line
READF, lun, line
sy    = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
ndims = FIX(STRMID(sy,0,12))
IF ndims GE 1 THEN BEGIN
  dim = INTARR(ndims)
  FOR i = 0,ndims - 1 DO dim[i] = FIX(STRMID(sy,12*(i+1),12))
  IF ndims EQ 1 THEN y_in = FLTARR(dim[0]) $
  ELSE IF ndims EQ 2 THEN y_in = FLTARR(dim[0],dim[1]) $
ELSE y_in = FLTARR(dim[0],dim[1],dim[2])
ENDIF ELSE y_in = 0
READF, lun, y_in
IF iprint THEN PRINT,'y_in read OK'

IF NOT oldformat THEN READF, lun, line
READF, lun, line
sz    = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
ndims = FIX(STRMID(sz,0,12))
IF ndims GE 1 THEN BEGIN
  dim = INTARR(ndims)
  FOR i = 0,ndims - 1 DO dim[i] = FIX(STRMID(sz,12*(i+1),12))
  IF ndims EQ 1 THEN z_in = FLTARR(dim[0]) $
  ELSE IF ndims EQ 2 THEN z_in = FLTARR(dim[0],dim[1]) $
ELSE IF ndims EQ 3 THEN z_in = FLTARR(dim[0],dim[1],dim[2]) $
ELSE z_in = FLTARR(dim[0],dim[1],dim[2],dim[3])
ENDIF ELSE z_in = 0
READF, lun, z_in
IF iprint THEN PRINT,'z_in read OK'

IF NOT oldformat THEN READF, lun, line
READF, lun, line  & datp.x_tit=line
IF NOT oldformat THEN READF, lun, line
READF, lun, line  & datp.y_tit=line
IF NOT oldformat THEN READF, lun, line
READF, lun, line  & datp.z_tit=line
IF NOT oldformat THEN READF, lun, line
READF, lun, line  & datp.w_tit=line
IF NOT oldformat THEN READF, lun, line
READF, lun, line  & datp.other_tit=line
IF iprint THEN PRINT,'titles read OK'

READF, lun, line
READF, lun, line
sp  = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
dim = FIX(STRMID(sp,12,12))
par = FLTARR(dim) & par_txt = STRARR(dim)
READF, lun, par
READF, lun, line
READF, lun, par_txt
IF iprint THEN PRINT,'Parameters read OK'

READF, lun, line
READF, lun, line
spv    = STRMID(line, STRPOS(line,'=') + 1, STRLEN(line) - STRPOS(line,'=') - 1)
ndims = FIX(STRMID(spv,0,12))
IF ndims GE 1 THEN BEGIN
  dim = INTARR(ndims)
  FOR i = 0, ndims - 1 DO dim[i] = FIX(STRMID(spv,12*(i+1),12))
  IF ndims EQ 1 THEN pv_in = FLTARR(dim[0]) $
  ELSE IF ndims EQ 2 THEN pv_in = FLTARR(dim[0],dim[1]) $
ELSE IF ndims EQ 3 THEN pv_in = FLTARR(dim[0],dim[1],dim[2]) $
ELSE pv_in = FLTARR(dim[0],dim[1],dim[2],dim[3])
ENDIF ELSE pv_in = 0
READF, lun, pv_in
IF iprint THEN PRINT,'pv read OK'
IF iprint THEN PRINT,'PV Parameters read OK'

ENDELSE

endoffile:
FREE_LUN, lun

IF iprint THEN PRINT,'End of "Read from input file" section'

; ------------------------------------------------------------------------------
;Return and exit

MOD_DATP, datp, "x", x_in
MOD_DATP, datp, "e", e_in
MOD_DATP, datp, "p", par
MOD_DATP, datp, "par_txt", par_txt
MOD_DATP, datp, "pv", pv_in
IF NOT oned THEN BEGIN
  MOD_DATP, datp, "y", y_in
  MOD_DATP, datp, "z", z_in
ENDIF

GIVE_DATP, datp
PRINT,'Input: workspace data read from '+in_file
RETURN, w_in

finished:
END
