; ------------------------------------------------------------------------------
;*******************************************************************************
;
PRO output, w_in, file = out_file, params = params, opf
;+
;** ******
;** User callable
;** takes a workspace and writes it to a file. 1-D workspaces are saved
;** in a standard (x,y,e) 3-column format. Larger workspaces saved differently.
;** 
;** KEYWORDS
;** --------
;**  params  : If set, write the full data parameter block to the file
;**     : ensuring compatability with LAMP routine when read in with
;**     : input.pro
;** 
;** ARGUMENTS
;** ---------
;**  file    : Name of output file. If omitted, default name is
;** COMMAND SYNTAX
;**  output, w10, file = <filename>
;**               KHA,JRS 15/4/03
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  iprint = 0  ; if iprint>0, show debugging messages
  IF iprint THEN PRINT,'Start output:'
  TAKE_DATP, datp
  IF N_ELEMENTS(opf) GT 0 THEN out_file=opf
  
  ; ------------------------------------------------------------------------------
  ;Check dimensions of input workspaces
  
  x_in = datp.x
  y_in = datp.y
  z_in = datp.z
  e_in = datp.e
  
  sw   = SIZE(w_in)
  npts = sw[1]
  IF sw[0] EQ 1 THEN oned = 1 ELSE oned = 0
  
  IF N_ELEMENTS(out_file) EQ 0 THEN BEGIN
    s = dialog_message('Output: output file name must be specified',/ERROR)
    GOTO, finished
  ENDIF
  
  se = SIZE(e_in)
  FOR i=0, se[0] DO IF se[i] NE sw[i] THEN e_in = w_in*0.
  
  IF iprint THEN PRINT,'End of "w_in dimensions check etc." section'
  
  ; ------------------------------------------------------------------------------
  ;Open and write to output file
  
  IF iprint THEN PRINT,'Opening output file'
  ON_IOERROR, close_file
  
  OPENW, lun, out_file, /GET_LUN, ERROR = err
  IF (err NE 0) THEN BEGIN
    s = dialog_message(!ERROR_STATE.msg,/ERROR)
    GOTO, finished
  ENDIF
  
  IF oned THEN BEGIN
    IF iprint THEN PRINT,'Ouput: 1d data format'
    FOR i = 0, npts - 1 DO BEGIN
      IF (w_in[i] NE 0.) OR (e_in[i] GT 0.) THEN $
        PRINTF, lun, FORMAT = '(3G14.6)', x_in[i], w_in[i], e_in[i]
    ENDFOR
    IF datp.y_tit EQ '' THEN datp.y_tit = 'Intensity (arb. units)'
    PRINTF, lun, ' [XTITLE]'
    PRINTF, lun, datp.x_tit
    PRINTF, lun, ' [YTITLE]'
    PRINTF, lun, datp.y_tit
    PRINTF, lun, ' [ZTITLE]'
    PRINTF, lun, datp.w_tit
    PRINTF, lun, ' [OTITLE]'
    PRINTF, lun, datp.other_tit
    IF KEYWORD_SET(params) THEN BEGIN
      PRINTF, lun, ' [PARAMS]'
      PRINTF, lun, 'SIZE(params) =',SIZE(datp.p)
      PRINTF, lun, FORMAT = '(6G14.6)', datp.p
      PRINTF, lun, ' [PAR_TXT]'
      PRINTF, lun, FORMAT = '(A)', datp.par_txt[0:(SIZE(datp.p))[1] - 1]
      
    ENDIF
  ENDIF ELSE BEGIN
    IF iprint THEN PRINT,'More than 1d data format'
    PRINTF, lun, ' [WORKSPACE]'
    PRINTF, lun, 'SIZE(workspace) =',SIZE(w_in)
    PRINTF, lun, FORMAT = '(6G14.6)', w_in
    PRINTF, lun, ' [ERRORS]'
    PRINTF, lun, FORMAT = '(6G14.6)', e_in
    PRINTF, lun, ' [XAXIS]'
    PRINTF, lun, 'SIZE(xarray) =',SIZE(x_in)
    PRINTF, lun, FORMAT = '(6G14.6)', x_in
    PRINTF, lun, ' [YAXIS]'
    PRINTF, lun, 'SIZE(yarray) =',SIZE(y_in)
    PRINTF, lun, FORMAT = '(6G14.6)', y_in
    PRINTF, lun, ' [ZAXIS]'
    PRINTF, lun, 'SIZE(zarray) =',SIZE(z_in)
    PRINTF, lun, FORMAT = '(6G14.6)', z_in
    PRINTF, lun, ' [XTITLE]'
    PRINTF, lun, datp.x_tit
    PRINTF, lun, ' [YTITLE]'
    PRINTF, lun, datp.y_tit
    PRINTF, lun, ' [ZTITLE]'
    PRINTF, lun, datp.z_tit
    PRINTF, lun, ' [WTITLE]'
    PRINTF, lun, datp.w_tit
    PRINTF, lun, ' [OTITLE]'
    PRINTF, lun, datp.other_tit
    IF KEYWORD_SET(params) THEN BEGIN
      PRINTF, lun, ' [PARAMS]'
      PRINTF, lun, 'SIZE(params) =',SIZE(datp.p)
      PRINTF, lun, FORMAT = '(6G14.6)', datp.p
      PRINTF, lun, ' [PAR_TXT]'
      PRINTF, lun, FORMAT = '(A)', datp.par_txt[0:(SIZE(datp.p))[1] - 1]
      IF N_ELEMENTS(datp.pv) GT 1 THEN BEGIN
        PRINTF, lun, ' [PV_PARAMS]'
        PRINTF, lun, 'SIZE(pv) =',SIZE(datp.pv)
        PRINTF, FORMAT = '(6G14.6)', lun, datp.pv
      ENDIF
    ENDIF
  ENDELSE
  
  close_file:
  FREE_LUN, lun
  
  IF iprint THEN PRINT,'Output: End of "Write to output file" section'
  
  ; ------------------------------------------------------------------------------
  ;Return and exit
  
  GIVE_DATP, datp
  PRINT,'Output: workspace data saved in '+out_file
  RETURN
  
  finished:
END
