; ----------------------------------------------------------------------------

PRO FR_calculate, n_buf , j, ratio, dRatio
;+
;** ************
;** stuff for rdid_d7
;-

  IF n_buf[2,j + 1] EQ 0. THEN BEGIN
    ratio = 999. & dratio = 0.
    RETURN
  ENDIF
  
  nonflip  = n_buf[2,j]     / n_buf[1,j]
  flip     = n_buf[2,j + 1] / n_buf[1,j + 1]
  ratio    = nonflip/flip
  dnonflip = SQRT(n_buf[2,j]/n_buf[1,j]^2 + n_buf[2,j]^2/n_buf[1,j]^3)
  dflip    = SQRT(n_buf[2,j + 1]/n_buf[1,j + 1]^2 + n_buf[2,j + 1]^2/n_buf[1,j + 1]^3)
  dratio   = SQRT((dnonflip/flip)^2 + (nonflip*dflip/flip^2)^2)
  
  RETURN
END

; ------------------------------------------------------------------------------
;*******************************************************************************
;
PRO rdid_d7, INST,numor,nvers,text,exper,scan,cnt,WOUT,vparm,param, $
             par1,par2,par3,par4,par5,WT ,XT ,YT ,ZT ,OT,DATE,PP,PTXT,$
             XX ,YY ,ZZ ,NN ,PV ,EE
;+
;** *******  
;** LAMP data-read interface for D7 data (ILL format) called from rdid.pro
;** **********************************************************************
;** 
;**  Reads 4 versions of D7 data:
;**   Version 0: Up to and including cycle 961
;**   Version 1: 2nd half of cycle 965 up to and including cycle 024
;**   Version 2: From cycle 031 up to cycle 053
;**        Version 3: From cycle 061 onwards
;**   Version 4: From cycle 071 onwards
;**    Version 6: From cycle 091 onwards
;**   Version 7:
;**   Version 8: From cycle 121 onwards (swapped banks 2 and 4)
;**  Doesn't read D7 data in cycles 962-964 and the beginning of cycle
;**  965 corresponding to run numbers #7002 - #9583 which were
;**  badly formatted.  This data may be read with the macro "read_d7_96.pro"
;** 
;** 
;** Parameters to return
;** ********************
;** WOUT             (modified data)
;** WT,XT,YT,ZT,OT   (titles)
;** PP,PTXT          (parameter floatting table and textarea)
;** XX,YY,ZZ         (coordinates)
;** NN               (monitors)
;** PV               (supplement parameter table of any dimension)
;** EE               (errors from WOUT)
;** 
;**               JRS 26/6/06
;**  Last edited: ARW 19.4.12
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-

  iprint = 0
  
  ;Useful input parameters from rdid
  ;*********************************
  
  IF iprint THEN BEGIN
    HELP, INST   & PRINT, INST
    HELP, numor   & PRINT, numor
    HELP, nvers   & PRINT, nvers
    HELP, text   & PRINT, text
    HELP, exper   & PRINT, exper
    HELP, scan   & PRINT, scan
    HELP, cnt   & PRINT, cnt
    HELP, WOUT
    HELP, vparm   & PRINT, vparm
    HELP, param   & PRINT, param
    HELP, par1   & PRINT, par1
    HELP, par2  & IF (N_ELEMENTS(par2) GT 0) THEN PRINT, par2
  ENDIF
  
  IF iprint THEN PRINT,'rdid_d7: starting'
  PRINT,'Reading run',numor
  
  ; ------------------------------------------------------------------------
  ;*************************************************************************
  
  CASE 1 OF
  
    ; ------------------------------------------------------------------------
  
    nvers EQ 0: BEGIN
    
      IF iprint THEN PRINT,'D7:  Old data format -> 1996'
      
      inst   = STRMID(text,0 ,4)
      numexp = STRMID(text,4 ,10)
      date   = STRMID(text,14,18)
      main_title = numexp
      sub_title  = main_title
      start_time = date
      
      ;      -------------------------------
      ;      MONITOR VALUES in n
      ;      -------------------------------
      
      n_buf = FLTARR(4,7)
      
      n_buf[*,0] = 0.      ; first line is zeros
      n_buf[0,1] = par1[14]    ; Time      Z up-up
      n_buf[0,2] = par1[114]     ; Time      Z up-down
      n_buf[0,3] = param[0]       ; Time      X up-up
      n_buf[0,4] = param[12]       ; Time      X up-down
      n_buf[0,5] = param[24]    ; Time      Y up-up
      n_buf[0,6] = param[36]    ; Time      Y up-down
      n_buf[1,1] = par1[3]      ; Monitor 1 Z up-up
      n_buf[1,2] = par1[103]     ; Monitor 1 Z up-down
      n_buf[1,3] = par1[117]     ; Monitor 1 X up-up
      n_buf[1,4] = param[1]    ; Monitor 1 X up-down
      n_buf[1,5] = param[13]    ; Monitor 1 Y up-up
      n_buf[1,6] = param[25]    ; Monitor 1 Y up-down
      n_buf[2,1] = par1[12]      ; Monitor 2 Z up-up
      n_buf[2,2] = par1[112]     ; Monitor 2 Z up-down
      n_buf[2,3] = par1[126]     ; Monitor 2 X up-up
      n_buf[2,4] = param[10]    ; Monitor 2 X up-down
      n_buf[2,5] = param[22]    ; Monitor 2 Y up-up
      n_buf[2,6] = param[34]    ; Monitor 2 Y up-down
      n_buf[3,1] = par1[13]    ; Handshake Z up-up
      n_buf[3,2] = par1[113]    ; Handshake Z up-down
      n_buf[3,3] = par1[127]    ; Handshake X up-up
      n_buf[3,4] = param[11]    ; Handshake X up-down
      n_buf[3,5] = param[23]    ; Handshake Y up-up
      n_buf[3,6] = param[35]    ; Handshake Y up-down
      
      IF n_buf[0,2] EQ 0. THEN BEGIN
        ratioZ = 0.
        ratioX = 0.
        ratioY = 0.
      ENDIF ELSE IF n_buf[0,3] EQ 0. THEN BEGIN
        IF n_buf[2,2] EQ 0. THEN ratioZ = 999. ELSE $
          ratioZ = (n_buf[2,1]/n_buf[1,1]) / (n_buf[2,2]/n_buf[1,2])
        ratioX = 0.
        ratioY = 0.
      ENDIF ELSE BEGIN
        IF n_buf[2,2] EQ 0. THEN ratioZ = 999. ELSE $
          ratioZ = (n_buf[2,1]/n_buf[1,1]) / (n_buf[2,2]/n_buf[1,2])
        IF n_buf[2,4] EQ 0. THEN ratioX = 999. ELSE $
          ratioX = (n_buf[2,3]/n_buf[1,3]) / (n_buf[2,4]/n_buf[1,4])
        IF n_buf[2,6] EQ 0. THEN ratioY = 999. ELSE $
          ratioY = (n_buf[2,5]/n_buf[1,5]) / (n_buf[2,6]/n_buf[1,6])
      ENDELSE
      
      nspectra = 64
      IF scan EQ 1 THEN BEGIN          ; No TOF
        TOF       = 0
        nchannels = 1
        IF n_buf[0,2] EQ 0. THEN BEGIN      ; no PA
          nphases = 1
        ENDIF ELSE IF (n_buf[0,3] EQ 0.) THEN BEGIN   ; only Z-PA
          nphases = 2
        ENDIF ELSE BEGIN        ; full XYZ-PA
          nphases = 6
        ENDELSE
        
        x_buf = par1[23:86]  ; x-axis is scattering angle
        y_buf = INDGEN(nphases)
        z_buf = x_buf
        w_buf = FLTARR(nspectra,nphases)
        FOR iphase=0, nphases - 1 DO $
          w_buf[*,iphase] = WOUT[80*iphase : 80*iphase + nspectra - 1]
        chel   = 0.
        nb_chn = 1
        nb_spc = nspectra
      ENDIF ELSE BEGIN
        TOF       = 1
        nchannels = cnt
        nphases   = scan/nspectra
        x_buf     = INDGEN(nchannels)+1     ; x-axis is channel number
        y_buf     = FLTARR(scan)        ; y-axis is scattering angle
        FOR iphase=0, nphases - 1 DO $
          y_buf[iphase*nspectra : iphase*nspectra + nspectra - 1] = par1[23:86]
        z_buf     = y_buf
        w_buf     = FLTARR(nchannels,scan)
        ispec     = INDGEN(scan)
        w_buf[*,ispec] = WOUT[*,ispec]
        w_tmp     = FLTARR(nchannels)
        FOR i = 1, scan, 2 DO w_tmp = w_tmp + w_buf[*,i]
        Imax = MAX(w_tmp,ichel)
        chel = FLOAT(ichel)
      ENDELSE
      
      lambda = par1[15]
      IF (lambda GT 4.5) AND (lambda LE 5.0) THEN lambda=4.8 $
      ELSE IF (lambda LT 3.5) THEN lambda=3.1
      IF (lambda GT 5.0) THEN lambda=5.7
      
      ;      -------------------------------
      ;      PARAMETER ASSIGNMENT IN Pn
      ;      -------------------------------
      npar        = 34
      p_buf       = FLTARR(npar)
      par_txt_buf = STRARR(npar)
      
      p_buf[0]  = numor  & par_txt_buf[0]  = ' 0 Numor      ='
      p_buf[1]  = nspectra  & par_txt_buf[1]  = ' 1 Number of spectra  ='
      p_buf[2]  = nphases  & par_txt_buf[2]  = ' 2 Number of phases  ='
      p_buf[3]  = 1.    & par_txt_buf[3]  = ' 3 Number of runs    ='
      p_buf[4]  = lambda  & par_txt_buf[4]  = ' 4 Wavelength (A)    ='
      p_buf[5]  = 0.    & par_txt_buf[5]  = ' 5 Chopper Speed (rpm)  ='
      p_buf[6]  = nchannels  & par_txt_buf[6]  = ' 6 Number of Time Channels ='
      p_buf[7]  = par1[17]  & par_txt_buf[7]  = ' 7 Channel Width (mcs)  ='
      p_buf[8]  = TOF    & par_txt_buf[8]  = ' 8 TOF mode (0/1)    ='
      p_buf[9]  = chel  & par_txt_buf[9]  = ' 9 Elastic Peak Channel  ='
      p_buf[10] = 0.    & par_txt_buf[10] = '10 Set-pt Temp Start  ='
      p_buf[11] = 0.    & par_txt_buf[11] = '11 Set-pt Temp End   ='
      p_buf[12] = 0.    & par_txt_buf[12] = '12 Regul Temp Start  ='
      p_buf[13] = 0.    & par_txt_buf[13] = '13 Regul Temp End    ='
      p_buf[14] = 0.    & par_txt_buf[14] = '14 Sample Temp Start  ='
      p_buf[15] = par1[16]  & par_txt_buf[15] = '15 Sample Temp End   ='
      p_buf[16] = 0.    & par_txt_buf[16] = '16 Bank1 Position    ='
      p_buf[17] = 0.    & par_txt_buf[17] = '17 Bank2 Position    ='
      p_buf[18] = 0.    & par_txt_buf[18] = '18 Bank3 Position    ='
      p_buf[19] = 0.    & par_txt_buf[19] = '19 Bank4 Position    ='
      p_buf[20] = 0.    & par_txt_buf[20] = '20 Lower Sample Rotation  ='
      p_buf[21] = par1[22]  & par_txt_buf[21] = '21 Upper Sample Rotation  ='
      p_buf[22] = 0.    & par_txt_buf[22] = '22 Z Flipper Current  ='
      p_buf[23] = 0.    & par_txt_buf[23] = '23 Z Correction Current  ='
      p_buf[24] = 0.    & par_txt_buf[24] = '24 X Flipper Current  ='
      p_buf[25] = 0.    & par_txt_buf[25] = '25 X Correction Current  ='
      p_buf[26] = 0.    & par_txt_buf[26] = '26 Y Flipper Current  ='
      p_buf[27] = 0.    & par_txt_buf[27] = '27 Y Correction Current  ='
      p_buf[28] = ratioZ  & par_txt_buf[28] = '28 Z Flipping Ratio  ='
      p_buf[29] = ratioX  & par_txt_buf[29] = '29 X Flipping Ratio  ='
      p_buf[30] = ratioY  & par_txt_buf[30] = '30 Y Flipping Ratio  ='
      p_buf[31] = 0.    & par_txt_buf[31] = '31 Run Time (seconds)  ='
      p_buf[32] = 0.    & par_txt_buf[32] = '32 Start time (hours)  ='
      p_buf[33] = 0.    & par_txt_buf[33] = '33 End Time (hours)  ='
      p_buf[34] = nvers   & par_txt_buf[34] = '34 Version                 ='
    END
    
    ; -----------------------------------------------------------------------
    
    nvers EQ 1: BEGIN
    
      datasource=0  ; 0 => take data from appropriate counters
      ; 1 => take data from hytec counters always
      ; 2 => take data from tof counters always
      
      IF iprint THEN PRINT,'D7: Data format 1 -> 1996 - 2002'
      
      inst         = STRMID(text,0,4)
      save_time    = STRMID(text,14,18)
      main_title   = STRTRIM(exper[0],2)
      sub_title    = STRTRIM(exper[1],2)
      IF (numor GE 9609 AND numor LE 10318) OR $
        (numor GE 10327 AND numor LE 11062) THEN BEGIN
        user         = STRMID(exper[3],0,8)
        local_contact= STRMID(exper[3],8,8)
        start_time   = ''
        stop_time    = ''
        scan_type    = ''
        pol_mode     = ''
        Time_mode    = ''
      ENDIF ELSE BEGIN
        user         = STRMID(exper[2],0,8)
        local_contact= STRMID(exper[2],8,8)
        start_time   = STRMID(exper[3],0,18)
        stop_time    = STRMID(exper[3],20,18)
        scan_type    = STRMID(exper[3],38,9)
        pol_mode     = STRMID(exper[3],47,5)
        Time_mode    = STRMID(exper[3],52,6)
      ENDELSE
      
      IF iprint THEN BEGIN
        PRINT,'inst          = ',inst
        PRINT,'main_title    = ',main_title
        PRINT,'sub_title     = ',sub_title
        PRINT,'user          = ',user
        PRINT,'local_contact = ',local_contact
        PRINT,'start_time    = ',start_time
        PRINT,'stop_time     = ',stop_time
        PRINT,'save_time     = ',save_time
        PRINT,'scan_type     = ',scan_type
        PRINT,'pol_mode      = ',pol_mode
        PRINT,'Time_mode     = ',Time_mode
      ENDIF
      
      ; -----------------------------------
      ;  Monitor Values in n
      ; -----------------------------------
      
      n_buf      = FLTARR(4,7)
      n_buf[*,0] = 0.      ; first line is zeros
      n_buf[0,1] = par1[13]    ; Time      Z up-up
      n_buf[0,2] = par1[22]     ; Time      Z up-down
      n_buf[0,3] = par1[31]     ; Time      X up-up
      n_buf[0,4] = par1[40]       ; Time      X up-down
      n_buf[0,5] = par1[49]    ; Time      Y up-up
      n_buf[0,6] = par1[58]    ; Time      Y up-down
      n_buf[1,1] = par1[15]      ; Monitor 1 Z up-up
      n_buf[1,2] = par1[24]     ; Monitor 1 Z up-down
      n_buf[1,3] = par1[33]     ; Monitor 1 X up-up
      n_buf[1,4] = par1[42]    ; Monitor 1 X up-down
      n_buf[1,5] = par1[51]    ; Monitor 1 Y up-up
      n_buf[1,6] = par1[60]    ; Monitor 1 Y up-down
      n_buf[2,1] = par1[16]      ; Monitor 2 Z up-up
      n_buf[2,2] = par1[25]     ; Monitor 2 Z up-down
      n_buf[2,3] = par1[34]     ; Monitor 2 X up-up
      n_buf[2,4] = par1[43]    ; Monitor 2 X up-down
      n_buf[2,5] = par1[52]    ; Monitor 2 Y up-up
      n_buf[2,6] = par1[61]    ; Monitor 2 Y up-down
      n_buf[3,1] = par1[14]    ; Handshake Z up-up
      n_buf[3,2] = par1[23]    ; Handshake Z up-down
      n_buf[3,3] = par1[32]    ; Handshake X up-up
      n_buf[3,4] = par1[41]    ; Handshake X up-down
      n_buf[3,5] = par1[50]    ; Handshake Y up-up
      n_buf[3,6] = par1[59]    ; Handshake Y up-down
      
      IF iprint THEN PRINT,'monitors assigned OK'
      
      ; -------------------------------
      ;  Calculate Flipping Ratios
      ; -------------------------------
      
      IF n_buf[0,2] EQ 0. THEN BEGIN        ;NOP
        ratioZ = 0.  & dratioZ = 0.
        ratioX = 0.  & dratioX = 0.
        ratioY = 0.  & dratioY = 0.
      ENDIF ELSE IF n_buf[0,3] EQ 0. THEN BEGIN    ;ZPO
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        ratioX = 0. & dratioX = 0.
        ratioY = 0. & dratioY = 0.
      ENDIF ELSE BEGIN          ;XYZ
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        FR_calculate, n_buf, 3, ratioX, dratioX
        FR_calculate, n_buf, 5, ratioY, dratioY
      ENDELSE
      IF iprint THEN PRINT,'Flipping Ratios OK'
      
      ; -----------------------------
      ;  Calculate Running Times
      ; -----------------------------
      
      h1=LONG(0)  & m1=LONG(0)  & s1=LONG(0)
      h2=LONG(0)  & m2=LONG(0)  & s2=LONG(0)
      IF (start_time NE '') AND (stop_time NE '') THEN BEGIN
        READS, start_time, FORMAT = '(I2,8X,I2,1X,I2,1X,I2)', da1, h1, m1, s1
        READS, stop_time,  FORMAT = '(I2,8X,I2,1X,I2,1X,I2)', da2, h2, m2, s2
        time1 = FLOAT(3600*h1+60*m1+s1)/3600.+24.*da1
        time2 = FLOAT(3600*h2+60*m2+s2)/3600.+24.*da2
        IF (h2 GE h1) THEN run_time = FLOAT(3600*(h2-h1)+60*(m2-m1)+(s2-s1)) $
        ELSE run_time = FLOAT(3600*(24+h2-h1)+60*(m2-m1)+(s2-s1))
      ENDIF ELSE BEGIN
        time1     = 0.
        time2     = 0.
        run_time  = 0.
      ENDELSE
      
      ; -------------------------------
      ;  Counts
      ; -------------------------------
      
      nspectra  = 64
      shytec    = SIZE(param)
      c_hytec   = param
      nsp_hytec = shytec[1]
      nph_hytec = nsp_hytec/nspectra
      nphases   = nph_hytec
      
      IF (scan EQ 1) THEN TOFformat=0 ELSE TOFformat=1
      
      IF (NOT TOFformat) AND (datasource NE 1) THEN BEGIN  ; no TOF
        IF iprint THEN PRINT,'non-TOF data'
        TOF       = 0
        nsp_tof   = cnt
        nph_tof   = cnt/nspectra
        nchannels = 1
        c_tof     = WOUT
      ENDIF ELSE IF (datasource EQ 1) THEN TOF=0 ELSE BEGIN  ; with TOF
        IF iprint THEN PRINT,'TOF data'
        
        ; mon1 and mon2 tof channels added as dets 65 and 66 from #42356 (23/11/99)
        
        IF (numor GE 42356) THEN nspectra=66
        TOF       = 1
        nsp_tof   = scan
        nph_tof   = scan/nspectra
        nchannels = FIX(par2[2])
        c_tof     = WOUT
      ENDELSE
      
      IF iprint THEN PRINT,'nsp=',nsp_tof,' nph=',nph_tof,' nchannels=',nchannels
      IF iprint THEN PRINT,'nphases=',nphases, 'nspectra=',nspectra
      
      TOFfactor = FLTARR(6)  & TOFfactor[*] = 1.
      IF (TOF EQ 0) THEN BEGIN
        IF iprint THEN PRINT,'TOF = 0'
        IF (datasource EQ 2) THEN w_buf=reform(c_tof,nspectra,nphases) $
        ELSE w_buf=reform(c_hytec,nspectra,nphases)
        x_buf = par2[6:69]
        IF (nphases EQ 1) THEN y_buf = LONG(numor) $
        ELSE y_buf = INDGEN(nphases)
        z_buf = x_buf
        chel  = 0.
        norm  = 1.
      ENDIF ELSE BEGIN
        IF iprint THEN PRINT,'TOF=1'
        w_buf = c_tof
        x_buf = INDGEN(nchannels)+1
        y_buf = FLTARR(nsp_tof)
        FOR iph = 0, nph_tof - 1 DO y_buf[iph*nspectra:iph*nspectra + 63] = par2[6:69]
        z_buf = y_buf
        w_tmp = LONARR(nchannels)  & w_tmp[*] = 0
        FOR i = 1, nsp_tof, 2 DO w_tmp[*] = w_tmp[*] + w_buf[*,i]
        Imax = MAX(w_tmp,ichel)
        chel = FLOAT(ichel)
        FOR iphase = 0, nphases - 1 DO BEGIN
          lowlim  =  iphase*64
          highlim = (iphase*63) + 63 + iphase
          sumh    = TOTAL(c_hytec[lowlim:highlim])
          tott    = TOTAL(FLOAT(c_tof),1)
          lowlim  = iphase*66
          highlim = (iphase*65) + 63 + iphase
          sumt    = TOTAL(tott[lowlim:highlim])
          IF (sumh EQ 0) THEN sumh = sumt
          TOFfactor[iphase] = sumt/sumh
        ENDFOR
        IF iprint THEN PRINT,'sumh=',sumh,' sumt=',sumt
      ENDELSE
      
      IF iprint THEN PRINT,'scattering angles and chel OK'
      
      ; ------------------------
      ;  Parameters
      ; ------------------------
      
      lambda = par2[0]
      IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda=4.838 $
      ELSE IF (lambda LT 3.5) THEN lambda=3.12
      IF (lambda GT 5.0) THEN lambda=5.9
      IF (numor GE 8434 AND numor LT 73568) THEN lambda=3.02
      IF iprint THEN PRINT,'lambda=',lambda
      
      p_buf=FLTARR(50)
      par_txt_buf=STRARR(50)
      p_buf[0]  = numor    & par_txt_buf[0]  = ' 0 Numor       ='
      p_buf[1]  = nspectra    & par_txt_buf[1]  = ' 1 Number of spectra    ='
      p_buf[2]  = nph_hytec    & par_txt_buf[2]  = ' 2 Number of phases    ='
      p_buf[3]  = 1.      & par_txt_buf[3]  = ' 3 Number of runs    ='
      p_buf[4]  = lambda    & par_txt_buf[4]  = ' 4 Wavelength (A)    ='
      p_buf[5]  = par2[1]    & par_txt_buf[5]  = ' 5 Chopper Speed (rpm)    ='
      p_buf[6]  = par2[2]    & par_txt_buf[6]  = ' 6 Number of Time Channels ='
      p_buf[7]  = par2[3]    & par_txt_buf[7]  = ' 7 Channel Width (mcs)    ='
      p_buf[8]  = TOF     & par_txt_buf[8]  = ' 8 TOF mode (0/1)    ='
      p_buf[9]  = chel    & par_txt_buf[9]  = ' 9 Elastic Peak Channel    ='
      p_buf[10] = par1[0]    & par_txt_buf[10] = '10 Set-pt Temp Start    ='
      p_buf[11] = par1[3]    & par_txt_buf[11] = '11 Set-pt Temp End    ='
      p_buf[12] = par1[1]    & par_txt_buf[12] = '12 Regul Temp Start    ='
      p_buf[13] = par1[4]    & par_txt_buf[13] = '13 Regul Temp End    ='
      p_buf[14] = par1[2]    & par_txt_buf[14] = '14 Sample Temp Start    ='
      p_buf[15] = par1[5]    & par_txt_buf[15] = '15 Sample Temp End    ='
      p_buf[16] = par2[76]    & par_txt_buf[16] = '16 Bank1 Position    ='
      p_buf[17] = par2[77]    & par_txt_buf[17] = '17 Bank2 Position    ='
      p_buf[18] = par2[78]    & par_txt_buf[18] = '18 Bank3 Position    ='
      p_buf[19] = par2[79]    & par_txt_buf[19] = '19 Bank4 Position    ='
      p_buf[20] = par2[80]    & par_txt_buf[20] = '20 Upper Sample Rotation   ='
      p_buf[21] = par2[81]    & par_txt_buf[21] = '21 Lower Sample Rotation   ='
      p_buf[22] = par1[17]    & par_txt_buf[22] = '22 Z Flipper Current    ='
      p_buf[23] = par1[18]    & par_txt_buf[23] = '23 Z Correction Current    ='
      p_buf[24] = par1[35]    & par_txt_buf[24] = '24 X Flipper Current    ='
      p_buf[25] = par1[36]    & par_txt_buf[25] = '25 X Correction Current    ='
      p_buf[26] = par1[53]    & par_txt_buf[26] = '26 Y Flipper Current    ='
      p_buf[27] = par1[54]    & par_txt_buf[27] = '27 Y Correction Current    ='
      p_buf[28] = ratioZ    & par_txt_buf[28] = '28 Z Flipping ratio    ='
      p_buf[29] = ratioX    & par_txt_buf[29] = '29 X Flipping ratio    ='
      p_buf[30] = ratioY    & par_txt_buf[30] = '30 Y Flipping ratio    ='
      p_buf[31] = run_time    & par_txt_buf[31] = '31 Run Time (seconds)    ='
      p_buf[32] = time1    & par_txt_buf[32] = '32 Start time (hours)    ='
      p_buf[33] = time2    & par_txt_buf[33] = '33 End Time (hours)    ='
      p_buf[34] = par2[70]    & par_txt_buf[34] = '34 Mono1 (psiA)      ='
      p_buf[35] = par2[71]    & par_txt_buf[35] = '35 Mono2 (psiB)      ='
      p_buf[36] = par2[72]    & par_txt_buf[36] = '36 Mono3 (psiC)      ='
      p_buf[37] = par2[73]    & par_txt_buf[37] = '37 Mono4 (thetaA)    ='
      p_buf[38] = par2[74]    & par_txt_buf[38] = '38 Mono5 (thetaB)    ='
      p_buf[39] = par2[75]    & par_txt_buf[39] = '39 Mono6 (thetaC)    ='
      p_buf[40] = TOFfactor[0]  & par_txt_buf[40] = '40 Z  TOFfactor      ='
      p_buf[41] = TOFfactor[1]  & par_txt_buf[41] = '41 Z1 TOFfactor      ='
      p_buf[42] = TOFfactor[2]  & par_txt_buf[42] = '42 X  TOFfactor      ='
      p_buf[43] = TOFfactor[3]  & par_txt_buf[43] = '43 X1 TOFfactor      ='
      p_buf[44] = TOFfactor[4]  & par_txt_buf[44] = '44 Y  TOFfactor      ='
      p_buf[45] = TOFfactor[5]  & par_txt_buf[45] = '45 Y1 TOFfactor      ='
      p_buf[46] = dratioZ    & par_txt_buf[46] = '46 Z dFlipping ratio    ='
      p_buf[47] = dratioX    & par_txt_buf[47] = '47 X dFlipping ratio    ='
      p_buf[48] = dratioY    & par_txt_buf[48] = '48 Y dFlipping ratio    ='
      p_buf[49] = par2[82]    & par_txt_buf[49] = '49 Polariser Angle    ='
      
      IF (iprint GT 0) THEN PRINT,'parameters assigned OK'
      
      ; 22/9/98 Problem with bank4 - recabled using coder 5 cables. Result is 10.21
      ; degree shift in coder reading
      ; 15/10/98 Looking at magnetic Bragg peak in UCu5, another 1.2 degree shift is
      ; required to bring bank4 peak into agreement with bank1
      
      IF (numor GE 30721 AND numor LE 31064) THEN p_buf[19] = p_buf[19] + 10.21 - 1.2
      
    END
    
    ; ----------------------------------------------------------------------------
    
    nvers EQ 2: BEGIN
    
      IF (iprint GT 0) THEN PRINT,'D7: Data format 2 -> 2003 ->'
      
      ; Data from this period is during the upgrade project for D7.
      ; The major changes are in the reading of the parameter block.
      
      inst          = STRMID(text,0,4)
      save_time     = STRMID(text,14,18)
      main_title    = STRTRIM(exper[0],2)
      sub_title     = STRTRIM(exper[1],2)
      user          = STRMID(exper[2],0,8)
      local_contact = STRMID(exper[2],8,8)
      start_time    = STRMID(exper[3],0,18)
      stop_time     = STRMID(exper[3],20,18)
      scan_type     = STRMID(exper[3],38,9)
      pol_mode      = STRMID(exper[3],47,5)
      Time_mode     = STRMID(exper[3],52,6)
      
      
      IF iprint THEN BEGIN
        PRINT, 'inst         = ', inst
        PRINT, 'main_title   = ', main_title
        PRINT, 'sub_title    = ', sub_title
        PRINT, 'user         = ', user
        PRINT, 'local_contact= ', local_contact
        PRINT, 'start_time   = ', start_time
        PRINT, 'stop_time    = ', stop_time
        PRINT, 'save_time    = ', save_time
        PRINT, 'scan_type    = ', scan_type
        PRINT, 'pol_mode     = ', pol_mode
        PRINT, 'Time_mode    = ', Time_mode
      ENDIF
      
      CASE pol_mode OF
        ' NOP ': nphases = 1
        '  ZP ': nphases = 2
        'XYZP ': nphases = 6
      ENDCASE
      
      nspectra = cnt/nphases
      
      IF (scan EQ 1) THEN TOFformat=0 ELSE TOFformat=1
      
      IF NOT TOFformat THEN BEGIN      ; no TOF
        IF iprint THEN PRINT,'non-TOF data'
        TOF       = 0
        nspectra  = cnt/nphases
        nchannels = 1
        c_tof     = WOUT
      ENDIF ELSE BEGIN  ; with TOF
        TOF       = 1
        nsp_tof   = scan
        nph_tof   = nphases
        nspectra  = nsp_tof/nphases
        nchannels = FIX(par1[2])
        c_tof     = WOUT
      ENDELSE
      
      IF iprint THEN PRINT,'nspectra=',nspectra,' nphases=',nphases,' nchannels=',nchannels
      IF iprint THEN PRINT,'Data read OK from ', nspectra, 'detectors'
      
      ; -----------------------------------
      ;  Monitor Values in n
      ; -----------------------------------
      param = [param,INDGEN(10)]  ;catches array error for early runs in cycle 031
      
      n_buf      = FLTARR(4,7)
      n_buf[*,0] = 0.      ; first line is zero
      n_buf[0,1] = param[14]    ; Time      Z up-up
      n_buf[0,2] = param[24]     ; Time      Z up-down
      n_buf[0,3] = param[34]       ; Time      X up-up
      n_buf[0,4] = param[44]       ; Time      X up-down
      n_buf[0,5] = param[54]    ; Time      Y up-up
      n_buf[0,6] = param[64]    ; Time      Y up-down
      n_buf[1,1] = param[16]      ; Monitor 1 Z up-up
      n_buf[1,2] = param[26]     ; Monitor 1 Z up-down
      n_buf[1,3] = param[36]     ; Monitor 1 X up-up
      n_buf[1,4] = param[46]    ; Monitor 1 X up-down
      n_buf[1,5] = param[56]    ; Monitor 1 Y up-up
      n_buf[1,6] = param[66]    ; Monitor 1 Y up-down
      n_buf[2,1] = param[17]      ; Monitor 2 Z up-up
      n_buf[2,2] = param[27]     ; Monitor 2 Z up-down
      n_buf[2,3] = param[37]     ; Monitor 2 X up-up
      n_buf[2,4] = param[47]    ; Monitor 2 X up-down
      n_buf[2,5] = param[57]    ; Monitor 2 Y up-up
      n_buf[2,6] = param[67]    ; Monitor 2 Y up-down
      n_buf[3,1] = param[15]    ; Handshake Z up-up
      n_buf[3,2] = param[25]    ; Handshake Z up-down
      n_buf[3,3] = param[35]    ; Handshake X up-up
      n_buf[3,4] = param[45]    ; Handshake X up-down
      n_buf[3,5] = param[55]    ; Handshake Y up-up
      n_buf[3,6] = param[65]    ; Handshake Y up-down
      
      IF iprint THEN PRINT,'monitor assigned OK'
      
      ; -------------------------------
      ;  Calculate Flipping Ratios
      ; -------------------------------
      
      IF n_buf[0,2] EQ 0. THEN BEGIN        ;NOP
        ratioZ = 0.  & dratioZ = 0.
        ratioX = 0.  & dratioX = 0.
        ratioY = 0.  & dratioY = 0.
      ENDIF ELSE IF n_buf[0,3] EQ 0. THEN BEGIN    ;ZPO
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        ratioX = 0. & dratioX = 0.
        ratioY = 0. & dratioY = 0.
      ENDIF ELSE BEGIN          ;XYZ
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        FR_calculate, n_buf, 3, ratioX, dratioX
        FR_calculate, n_buf, 5, ratioY, dratioY
      ENDELSE
      IF iprint THEN PRINT,'Flipping Ratios OK'
      
      ; -----------------------------
      ;  Calculate Running Times
      ; -----------------------------
      
      h1=LONG(0)  & m1=LONG(0)  & s1=LONG(0)
      h2=LONG(0)  & m2=LONG(0)  & s2=LONG(0)
      IF (start_time NE '' AND stop_time NE '') THEN BEGIN
        READS, start_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da1, h1, m1, s1
        READS, stop_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da2, h2, m2, s2
        time1=FLOAT(3600*h1+60*m1+s1)/3600.+24.*(da1 - 1)
        time2=FLOAT(3600*h2+60*m2+s2)/3600.+24.*(da2 - 1)
        IF (h2 GE h1) THEN run_time=FLOAT(3600*(h2-h1)+60*(m2-m1)+(s2-s1)) $
        ELSE run_time=FLOAT(3600*(24+h2-h1)+60*(m2-m1)+(s2-s1))
      ENDIF ELSE BEGIN
        time1=0.
        time2=0.
        run_time=0.
      ENDELSE
      
      ; -----------------------------
      
      IF NOT TOF THEN BEGIN
        IF iprint THEN PRINT,'TOF = 0'
        w_buf = reform(c_tof,nspectra,nphases)
        x_buf = par1[6:nspectra + 5]
        IF (nphases EQ 1) THEN y_buf = LONG(numor) $
        ELSE y_buf = INDGEN(nphases)
        z_buf = x_buf
        chel  = 0.
        norm  = 1.
      ENDIF ELSE BEGIN
        IF iprint THEN PRINT,'TOF = 1'
        w_buf = c_tof
        x_buf = INDGEN(nchannels) + 1
        y_buf = FLTARR(nsp_tof)
        FOR iph = 0, nphases - 1 DO $
          y_buf[iph*nspectra:iph*nspectra + nspectra - 3] = par1[6:nspectra + 3]
        z_buf = y_buf
        w_tmp = LONARR(nchannels)
        FOR i = 0, nspectra - 1 DO w_tmp[*] = w_tmp[*] + w_buf[*,i]
        Imax = MAX(w_tmp,ichel)
        chel = FLOAT(ichel)
      ENDELSE
      
      IF iprint THEN PRINT,'scattering angles and chel OK'
      
      ; ------------------------
      ;  Parameters
      ; ------------------------
      ; For 2003 data there are 74 detectors and 114 parameters
      ; For a brief period in 2003, there are 115 parameters (mono7 added)
      ; then, with the addition of the 18 extra detector angles (92 detectors in 2004)
      ; the number of parameters becomes 133
      ; In cycle 2004 the old bank2 is replaced with the bank of 10: -> 127 parameters
      
      
      new_detectors = 0
      mono7_in      = 0
      IF (SIZE(par1))[1] GE 115 THEN mono7_in = 1
      IF (SIZE(par1))[1] EQ 133 THEN new_detectors = 18
      IF (SIZE(par1))[1] EQ 127 THEN new_detectors = 12
      
      add = mono7_in + new_detectors
      
      IF par1[1] EQ -256.0 THEN chop = 0 ELSE chop = par1[1]
      
      lambda=par1[0]
      IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.838
      IF (lambda LT 3.5) THEN lambda = 3.054
      IF (lambda GT 5.0) THEN lambda = 5.98
      IF add THEN BEGIN
        IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.815
      ENDIF
      
      IF iprint THEN PRINT,'lambda = ',lambda
      
      IF mono7_in THEN n_par = 48 ELSE n_par = 47
      p_buf       = FLTARR(n_par)
      par_txt_buf = STRARR(n_par)
      
      
      p_buf[0]  = numor    & par_txt_buf[0]  = ' 0 Numor        ='
      p_buf[1]  = nspectra    & par_txt_buf[1]  = ' 1 Number of spectra     ='
      p_buf[2]  = nphases    & par_txt_buf[2]  = ' 2 Number of phases     ='
      p_buf[3]  = 1.      & par_txt_buf[3]  = ' 3 Number of runs     ='
      p_buf[4]  = lambda    & par_txt_buf[4]  = ' 4 Wavelength (A)     ='
      p_buf[5]  = chop    & par_txt_buf[5]  = ' 5 Chopper Speed (rpm)     ='
      p_buf[6]  = par1[2]    & par_txt_buf[6]  = ' 6 Number of Time Channels   ='
      p_buf[7]  = par1[3]    & par_txt_buf[7]  = ' 7 Channel Width (mcs)     ='
      p_buf[8]  = TOF     & par_txt_buf[8]  = ' 8 TOF mode (0/1)     ='
      p_buf[9]  = chel    & par_txt_buf[9]  = ' 9 Elastic Peak Channel     ='
      p_buf[10] = param[0]    & par_txt_buf[10] = '10 Set-pt Temp Start     ='
      p_buf[11] = param[3]    & par_txt_buf[11] = '11 Set-pt Temp End     ='
      p_buf[12] = param[1]    & par_txt_buf[12] = '12 Regul Temp Start     ='
      p_buf[13] = param[4]    & par_txt_buf[13] = '13 Regul Temp End     ='
      p_buf[14] = param[2]    & par_txt_buf[14] = '14 Sample Temp Start     ='
      p_buf[15] = param[5]    & par_txt_buf[15] = '15 Sample Temp End     ='
      p_buf[16] = par1[86+add]  & par_txt_buf[16] = '16 Bank1 Position     ='
      p_buf[17] = par1[87+add]  & par_txt_buf[17] = '17 Bank2 Position     ='
      p_buf[18] = par1[88+add]  & par_txt_buf[18] = '18 Bank3 Position     ='
      p_buf[19] = par1[89+add]  & par_txt_buf[19] = '19 Bank4 Position     ='
      p_buf[20] = par1[90+add]  & par_txt_buf[20] = '20 Upper Sample Rotation    ='
      p_buf[21] = par1[91+add]  & par_txt_buf[21] = '21 Lower Sample Rotation    ='
      p_buf[22] = param[18]    & par_txt_buf[22] = '22 Z Flip Current     ='
      p_buf[23] = param[19]    & par_txt_buf[23] = '23 Z Cor_Z Current     ='
      p_buf[24] = param[23]    & par_txt_buf[24] = '24 Z Cor_X Current     ='
      p_buf[25] = param[38]    & par_txt_buf[25] = '25 X Flip Current     ='
      p_buf[26] = param[39]    & par_txt_buf[26] = '26 X Cor_Z Current     ='
      p_buf[27] = param[43]    & par_txt_buf[27] = '27 X Cor_X Current     ='
      p_buf[28] = param[58]    & par_txt_buf[28] = '28 Y Flip Current     ='
      p_buf[29] = param[59]    & par_txt_buf[29] = '29 Y Cor_Z Current     ='
      p_buf[30] = param[63]    & par_txt_buf[30] = '30 Y Cor_X Current     ='
      p_buf[31] = ratioZ    & par_txt_buf[31] = '31 Z Flipping ratio     ='
      p_buf[32] = ratioX    & par_txt_buf[32] = '32 X Flipping ratio     ='
      p_buf[33] = ratioY    & par_txt_buf[33] = '33 Y Flipping ratio     ='
      p_buf[34] = run_time    & par_txt_buf[34] = '34 Run Time (seconds)     ='
      p_buf[35] = time1    & par_txt_buf[35] = '35 Start (hrs since 1st of month) ='
      p_buf[36] = time2    & par_txt_buf[36] = '36 End  (hrs since 1st of month) ='
      p_buf[37] = par1[80+new_detectors]    & par_txt_buf[37] = '37 Mono1 (psiA)       ='
      p_buf[38] = par1[81+new_detectors]    & par_txt_buf[38] = '38 Mono2 (psiB)       ='
      p_buf[39] = par1[82+new_detectors]    & par_txt_buf[39] = '39 Mono3 (psiC)       ='
      p_buf[40] = par1[83+new_detectors]    & par_txt_buf[40] = '40 Mono4 (thetaA)     ='
      p_buf[41] = par1[84+new_detectors]    & par_txt_buf[41] = '41 Mono5 (thetaB)     ='
      p_buf[42] = par1[85+new_detectors]    & par_txt_buf[42] = '42 Mono6 (thetaC)     ='
      IF mono7_in THEN $
        p_buf[43] = par1[86+new_detectors]    & par_txt_buf[43] = '43 Mono7 (tilt angle)     ='
      p_buf[43+mono7_in] = dratioZ      & par_txt_buf[43+mono7_in] = STRTRIM(STRING(43+mono7_in),2) + ' Z dFlipping ratio     ='
      p_buf[44+mono7_in] = dratioX        & par_txt_buf[44+mono7_in] = STRTRIM(STRING(44+mono7_in),2) + ' X dFlipping ratio     ='
      p_buf[45+mono7_in] = dratioY        & par_txt_buf[45+mono7_in] = STRTRIM(STRING(45+mono7_in),2) + ' Y dFlipping ratio     ='
      p_buf[46+mono7_in] = par1[92+add]     & par_txt_buf[46+mono7_in] = STRTRIM(STRING(46+mono7_in),2) + ' Polariser Angle     ='
      IF iprint THEN PRINT,'parameters assigned OK'
      
    END
    
    ; ----------------------------------------------------------------------------
    
    nvers EQ 3: BEGIN
    
      IF (iprint GT 0) THEN PRINT,'D7: Data format 3 -> 2006 only'
      ; Data from this period is during the upgrade project for D7.
      ; The major changes are in the reading of the parameter block.
      
      inst          = STRMID(text,0,4)
      save_time     = STRMID(text,14,18)
      main_title    = STRTRIM(exper[0],2)
      sub_title     = STRTRIM(exper[1],2)
      user          = STRMID(exper[2],0,8)
      local_contact = STRMID(exper[2],8,8)
      start_time    = STRMID(exper[3],0,18)
      stop_time     = STRMID(exper[3],20,18)
      scan_type     = STRMID(exper[3],38,9)
      pol_mode      = STRMID(exper[3],47,5)
      Time_mode     = STRMID(exper[3],52,6)
      
      
      IF iprint THEN BEGIN
        PRINT, 'inst         = ', inst
        PRINT, 'main_title   = ', main_title
        PRINT, 'sub_title    = ', sub_title
        PRINT, 'user         = ', user
        PRINT, 'local_contact= ', local_contact
        PRINT, 'start_time   = ', start_time
        PRINT, 'stop_time    = ', stop_time
        PRINT, 'save_time    = ', save_time
        PRINT, 'scan_type    = ', scan_type
        PRINT, 'pol_mode     = ', pol_mode
        PRINT, 'Time_mode    = ', Time_mode
      ENDIF
      
      CASE pol_mode OF
        ' NOP ': nphases = 1
        '  ZP ': nphases = 2
        'XYZP ': nphases = 6
      ENDCASE
      
      nspectra = cnt/nphases
      
      IF (scan EQ 1) THEN TOFformat=0 ELSE TOFformat=1
      
      IF NOT TOFformat THEN BEGIN      ; no TOF
        IF iprint THEN PRINT,'non-TOF data'
        TOF       = 0
        nspectra  = cnt/nphases
        nchannels = 1
        c_tof     = WOUT
      ENDIF ELSE BEGIN  ; with TOF
        TOF       = 1
        nsp_tof   = scan
        nph_tof   = nphases
        nspectra  = nsp_tof/nphases
        nchannels = FIX(par1[2])
        c_tof     = WOUT
      ENDELSE
      
      IF iprint THEN PRINT,'nspectra=',nspectra,' nphases=',nphases,' nchannels=',nchannels
      IF iprint THEN PRINT,'Data read OK from ', nspectra, 'detectors'
      
      ; -----------------------------------
      ;  Monitor Values in n
      ; -----------------------------------
      param = [param,INDGEN(10)]  ;catches array error for early runs in cycle 031
      
      n_buf      = FLTARR(4,7)
      n_buf[*,0] = 0.      ; first line is zero
      n_buf[0,1] = param[14]    ; Time      Z up-up
      n_buf[0,2] = param[24]     ; Time      Z up-down
      n_buf[0,3] = param[34]     ; Time      X up-up
      n_buf[0,4] = param[44]       ; Time      X up-down
      n_buf[0,5] = param[54]    ; Time      Y up-up
      n_buf[0,6] = param[64]    ; Time      Y up-down
      n_buf[1,1] = param[16]      ; Monitor 1 Z up-up
      n_buf[1,2] = param[26]     ; Monitor 1 Z up-down
      n_buf[1,3] = param[36]     ; Monitor 1 X up-up
      n_buf[1,4] = param[46]    ; Monitor 1 X up-down
      n_buf[1,5] = param[56]    ; Monitor 1 Y up-up
      n_buf[1,6] = param[66]    ; Monitor 1 Y up-down
      n_buf[2,1] = param[17]      ; Monitor 2 Z up-up
      n_buf[2,2] = param[27]     ; Monitor 2 Z up-down
      n_buf[2,3] = param[37]     ; Monitor 2 X up-up
      n_buf[2,4] = param[47]    ; Monitor 2 X up-down
      n_buf[2,5] = param[57]    ; Monitor 2 Y up-up
      n_buf[2,6] = param[67]    ; Monitor 2 Y up-down
      n_buf[3,1] = param[15]    ; Handshake Z up-up
      n_buf[3,2] = param[25]    ; Handshake Z up-down
      n_buf[3,3] = param[35]    ; Handshake X up-up
      n_buf[3,4] = param[45]    ; Handshake X up-down
      n_buf[3,5] = param[55]    ; Handshake Y up-up
      n_buf[3,6] = param[65]    ; Handshake Y up-down
      
      IF iprint THEN PRINT,'monitor assigned OK'
      
      ; -------------------------------
      ;  Calculate Flipping Ratios
      ; -------------------------------
      
      IF n_buf[0,2] EQ 0. THEN BEGIN        ;NOP
        ratioZ = 0.  & dratioZ = 0.
        ratioX = 0.  & dratioX = 0.
        ratioY = 0.  & dratioY = 0.
      ENDIF ELSE IF n_buf[0,3] EQ 0. THEN BEGIN    ;ZPO
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        ratioX = 0. & dratioX = 0.
        ratioY = 0. & dratioY = 0.
      ENDIF ELSE BEGIN          ;XYZ
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        FR_calculate, n_buf, 3, ratioX, dratioX
        FR_calculate, n_buf, 5, ratioY, dratioY
      ENDELSE
      IF iprint THEN PRINT,'Flipping Ratios OK'
      
      ; -----------------------------
      ;  Calculate Running Times
      ; -----------------------------
      
      h1=LONG(0)  & m1=LONG(0)  & s1=LONG(0)
      h2=LONG(0)  & m2=LONG(0)  & s2=LONG(0)
      IF (start_time NE '' AND stop_time NE '') THEN BEGIN
        READS, start_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da1, h1, m1, s1
        READS, stop_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da2, h2, m2, s2
        time1=FLOAT(3600*h1+60*m1+s1)/3600.+24.*(da1 - 1)
        time2=FLOAT(3600*h2+60*m2+s2)/3600.+24.*(da2 - 1)
        IF (h2 GE h1) THEN run_time=FLOAT(3600*(h2-h1)+60*(m2-m1)+(s2-s1)) $
        ELSE run_time=FLOAT(3600*(24+h2-h1)+60*(m2-m1)+(s2-s1))
      ENDIF ELSE BEGIN
        time1=0.
        time2=0.
        run_time=0.
      ENDELSE
      
      ; -----------------------------
      
      IF NOT TOF THEN BEGIN
        IF iprint THEN PRINT,'TOF = 0'
        w_buf = reform(c_tof,nspectra,nphases)
        x_buf = par1[6:nspectra + 5]
        IF (nphases EQ 1) THEN y_buf = LONG(numor) $
        ELSE y_buf = INDGEN(nphases)
        z_buf = x_buf
        chel  = 0.
        norm  = 1.
      ENDIF ELSE BEGIN
        IF iprint THEN PRINT,'TOF = 1'
        w_buf = c_tof
        x_buf = INDGEN(nchannels) + 1
        y_buf = FLTARR(nsp_tof)
        FOR iph = 0, nphases - 1 DO $
          y_buf[iph*nspectra:iph*nspectra + nspectra - 3] = par1[6:nspectra + 3]
        z_buf = y_buf
        w_tmp = LONARR(nchannels)
        FOR i = 0, nspectra - 1 DO w_tmp[*] = w_tmp[*] + w_buf[*,i]
        Imax = MAX(w_tmp,ichel)
        chel = FLOAT(ichel)
      ENDELSE
      
      IF iprint THEN PRINT,'scattering angles and chel OK'
      
      ; ------------------------
      ;  Parameters
      ; ------------------------
      
      IF par1[1] EQ -256.0 THEN chop = 0 ELSE chop = par1[1]
      
      lambda=par1[0]
      IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.815
      IF (lambda LT 3.5) THEN lambda = 3.054
      IF (lambda GT 5.0) THEN lambda = 5.98
      
      
      IF iprint THEN PRINT,'lambda = ',lambda
      
      n_par = 48
      p_buf       = FLTARR(n_par)
      par_txt_buf = STRARR(n_par)
      
      
      p_buf[0]  = numor    & par_txt_buf[0]  = ' 0 Numor        ='
      p_buf[1]  = nspectra    & par_txt_buf[1]  = ' 1 Number of spectra     ='
      p_buf[2]  = nphases    & par_txt_buf[2]  = ' 2 Number of phases     ='
      p_buf[3]  = 1.      & par_txt_buf[3]  = ' 3 Number of runs     ='
      p_buf[4]  = lambda    & par_txt_buf[4]  = ' 4 Wavelength (A)     ='
      p_buf[5]  = chop    & par_txt_buf[5]  = ' 5 Chopper Speed (rpm)     ='
      p_buf[6]  = par1[2]    & par_txt_buf[6]  = ' 6 Number of Time Channels   ='
      p_buf[7]  = par1[3]    & par_txt_buf[7]  = ' 7 Channel Width (mcs)     ='
      p_buf[8]  = TOF     & par_txt_buf[8]  = ' 8 TOF mode (0/1)     ='
      p_buf[9]  = chel    & par_txt_buf[9]  = ' 9 Elastic Peak Channel     ='
      p_buf[10] = param[0]    & par_txt_buf[10] = '10 Set-pt Temp Start     ='
      p_buf[11] = param[3]    & par_txt_buf[11] = '11 Set-pt Temp End     ='
      p_buf[12] = param[1]    & par_txt_buf[12] = '12 Regul Temp Start     ='
      p_buf[13] = param[4]    & par_txt_buf[13] = '13 Regul Temp End     ='
      p_buf[14] = param[2]    & par_txt_buf[14] = '14 Sample Temp Start     ='
      p_buf[15] = param[5]    & par_txt_buf[15] = '15 Sample Temp End     ='
      p_buf[16] = par1[129]     & par_txt_buf[16] = '16 Bank1 Position     ='
      p_buf[17] = par1[130]     & par_txt_buf[17] = '17 Bank2 Position     ='
      p_buf[18] = par1[131]     & par_txt_buf[18] = '18 Bank3 Position     ='
      p_buf[19] = par1[132]     & par_txt_buf[19] = '19 Bank4 Position     ='
      p_buf[20] = par1[133]     & par_txt_buf[20] = '20 Upper Sample Rotation    ='
      p_buf[21] = par1[134]     & par_txt_buf[21] = '21 Lower Sample Rotation    ='
      p_buf[22] = param[18]    & par_txt_buf[22] = '22 Z Flip Current     ='
      p_buf[23] = param[19]    & par_txt_buf[23] = '23 Z Cor_Z Current     ='
      p_buf[24] = param[23]    & par_txt_buf[24] = '24 Z Cor_X Current     ='
      p_buf[25] = param[38]    & par_txt_buf[25] = '25 X Flip Current     ='
      p_buf[26] = param[39]    & par_txt_buf[26] = '26 X Cor_Z Current     ='
      p_buf[27] = param[43]    & par_txt_buf[27] = '27 X Cor_X Current     ='
      p_buf[28] = param[58]    & par_txt_buf[28] = '28 Y Flip Current     ='
      p_buf[29] = param[59]    & par_txt_buf[29] = '29 Y Cor_Z Current     ='
      p_buf[30] = param[63]    & par_txt_buf[30] = '30 Y Cor_X Current     ='
      p_buf[31] = ratioZ    & par_txt_buf[31] = '31 Z Flipping ratio     ='
      p_buf[32] = ratioX    & par_txt_buf[32] = '32 X Flipping ratio     ='
      p_buf[33] = ratioY    & par_txt_buf[33] = '33 Y Flipping ratio     ='
      p_buf[34] = run_time    & par_txt_buf[34] = '34 Run Time (seconds)     ='
      p_buf[35] = time1    & par_txt_buf[35] = '35 Start (hrs since 1st of month) ='
      p_buf[36] = time2    & par_txt_buf[36] = '36 End  (hrs since 1st of month) ='
      p_buf[37] = par1[122]    & par_txt_buf[37] = '37 Mono1 (psiA)       ='
      p_buf[38] = par1[123]    & par_txt_buf[38] = '38 Mono2 (psiB)       ='
      p_buf[39] = par1[124]    & par_txt_buf[39] = '39 Mono3 (psiC)       ='
      p_buf[40] = par1[125]    & par_txt_buf[40] = '40 Mono4 (thetaA)     ='
      p_buf[41] = par1[126]    & par_txt_buf[41] = '41 Mono5 (thetaB)     ='
      p_buf[42] = par1[127]    & par_txt_buf[42] = '42 Mono6 (thetaC)     ='
      p_buf[43] = par1[128]    & par_txt_buf[43] = '43 Mono7 (tilt angle)     ='
      p_buf[44] = dratioZ    & par_txt_buf[44] = '44 Z dFlipping ratio     ='
      p_buf[45] = dratioX    & par_txt_buf[45] = '45 X dFlipping ratio     ='
      p_buf[46] = dratioY    & par_txt_buf[46] = '46 Y dFlipping ratio     ='
      p_buf[47] = par1[135]     & par_txt_buf[47] = '47 Polariser Angle     ='
      IF iprint THEN PRINT,'parameters assigned OK'
      
    END
    
    ; ----------------------------------------------------------------------------
    ; ----------------------------------------------------------------------------
    
    nvers GE 4: BEGIN
    
      IF (iprint GT 0) THEN PRINT,'D7: Data format 4 -> 2007 ->'
      ; Final change as the third detector bank is installed.
      ; The major changes are in the reading of the parameter block (Detector and
      ; Monochromator positions) P.D. 13/02/07
      
      ; Lines added to account for detector banks 2 and 4 being swapped
      ; Lines are commented.  The correction goes with data version 8
      ; ARW 19.4.12
      
      inst          = STRMID(text,0,4)
      save_time     = STRMID(text,14,18)
      main_title    = STRTRIM(exper[0],2)
      sub_title     = STRTRIM(exper[1],2)
      user          = STRMID(exper[2],0,8)
      local_contact = STRMID(exper[2],8,8)
      start_time    = STRMID(exper[3],0,18)
      stop_time     = STRMID(exper[3],20,18)
      scan_type     = STRMID(exper[3],38,9)
      pol_mode      = STRMID(exper[3],47,5)
      Time_mode     = STRMID(exper[3],52,6)
      
      
      IF iprint THEN BEGIN
        PRINT, 'inst         = ', inst
        PRINT, 'main_title   = ', main_title
        PRINT, 'sub_title    = ', sub_title
        PRINT, 'user         = ', user
        PRINT, 'local_contact= ', local_contact
        PRINT, 'start_time   = ', start_time
        PRINT, 'stop_time    = ', stop_time
        PRINT, 'save_time    = ', save_time
        PRINT, 'scan_type    = ', scan_type
        PRINT, 'pol_mode     = ', pol_mode
        PRINT, 'Time_mode    = ', Time_mode
      ENDIF
      
      CASE pol_mode OF
        ' NOP ': nphases = 1
        '  ZP ': nphases = 2
        'XYZP ': nphases = 6
      ENDCASE
      
      nspectra = cnt/nphases
      
      IF (scan EQ 1) THEN TOFformat=0 ELSE TOFformat=1
      
      IF NOT TOFformat THEN BEGIN      ; no TOF
        IF iprint THEN PRINT,'non-TOF data'
        TOF       = 0
        nspectra  = cnt/nphases
        nchannels = 1
        c_tof     = WOUT
      ENDIF ELSE BEGIN  ; with TOF
        TOF       = 1
        nsp_tof   = scan
        nph_tof   = nphases
        nspectra  = nsp_tof/nphases
        nchannels = FIX(par1[2])
        c_tof     = WOUT
      ENDELSE
      
      IF iprint THEN PRINT,'nspectra=',nspectra,' nphases=',nphases,' nchannels=',nchannels
      IF iprint THEN PRINT,'Data read OK from ', nspectra, 'detectors'
      
      ; -----------------------------------
      ;  Monitor Values in n
      ; -----------------------------------
      param = [param,INDGEN(10)]  ;catches array error for early runs in cycle 031
      
      n_buf      = FLTARR(4,7)
      n_buf[*,0] = 0.      ; first line is zero
      n_buf[0,1] = param[14]    ; Time      Z up-up
      n_buf[0,2] = param[24]     ; Time      Z up-down
      n_buf[0,3] = param[34]     ; Time      X up-up
      n_buf[0,4] = param[44]       ; Time      X up-down
      n_buf[0,5] = param[54]    ; Time      Y up-up
      n_buf[0,6] = param[64]    ; Time      Y up-down
      n_buf[1,1] = param[16]      ; Monitor 1 Z up-up
      n_buf[1,2] = param[26]     ; Monitor 1 Z up-down
      n_buf[1,3] = param[36]     ; Monitor 1 X up-up
      n_buf[1,4] = param[46]    ; Monitor 1 X up-down
      n_buf[1,5] = param[56]    ; Monitor 1 Y up-up
      n_buf[1,6] = param[66]    ; Monitor 1 Y up-down
      n_buf[2,1] = param[17]      ; Monitor 2 Z up-up
      n_buf[2,2] = param[27]     ; Monitor 2 Z up-down
      n_buf[2,3] = param[37]     ; Monitor 2 X up-up
      n_buf[2,4] = param[47]    ; Monitor 2 X up-down
      n_buf[2,5] = param[57]    ; Monitor 2 Y up-up
      n_buf[2,6] = param[67]    ; Monitor 2 Y up-down
      n_buf[3,1] = param[15]    ; Handshake Z up-up
      n_buf[3,2] = param[25]    ; Handshake Z up-down
      n_buf[3,3] = param[35]    ; Handshake X up-up
      n_buf[3,4] = param[45]    ; Handshake X up-down
      n_buf[3,5] = param[55]    ; Handshake Y up-up
      n_buf[3,6] = param[65]    ; Handshake Y up-down
      
      IF iprint THEN PRINT,'monitor assigned OK'
      
      ; -------------------------------
      ;  Calculate Flipping Ratios
      ; -------------------------------
      
      IF n_buf[0,2] EQ 0. THEN BEGIN        ;NOP
        ratioZ = 0.  & dratioZ = 0.
        ratioX = 0.  & dratioX = 0.
        ratioY = 0.  & dratioY = 0.
      ENDIF ELSE IF n_buf[0,3] EQ 0. THEN BEGIN    ;ZPO
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        ratioX = 0. & dratioX = 0.
        ratioY = 0. & dratioY = 0.
      ENDIF ELSE BEGIN          ;XYZ
        FR_calculate, n_buf, 1, ratioZ, dratioZ
        FR_calculate, n_buf, 3, ratioX, dratioX
        FR_calculate, n_buf, 5, ratioY, dratioY
      ENDELSE
      IF iprint THEN PRINT,'Flipping Ratios OK'
      
      ; -----------------------------
      ;  Calculate Running Times
      ; -----------------------------
      
      h1=LONG(0)  & m1=LONG(0)  & s1=LONG(0)
      h2=LONG(0)  & m2=LONG(0)  & s2=LONG(0)
      IF (start_time NE '' AND stop_time NE '') THEN BEGIN
        READS, start_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da1, h1, m1, s1
        READS, stop_time, FORMAT='(I2,8X,I2,1X,I2,1X,I2)', da2, h2, m2, s2
        time1=FLOAT(3600*h1+60*m1+s1)/3600.+24.*(da1 - 1)
        time2=FLOAT(3600*h2+60*m2+s2)/3600.+24.*(da2 - 1)
        IF (h2 GE h1) THEN run_time=FLOAT(3600*(h2-h1)+60*(m2-m1)+(s2-s1)) $
        ELSE run_time=FLOAT(3600*(24+h2-h1)+60*(m2-m1)+(s2-s1))
      ENDIF ELSE BEGIN
        time1=0.
        time2=0.
        run_time=0.
      ENDELSE
      
      ; -----------------------------
      
      IF NOT TOF THEN BEGIN
        IF iprint THEN PRINT,'TOF = 0'
        w_buf = reform(c_tof,nspectra,nphases)
        x_buf = par1[6:nspectra + 5]
        ; Add line to swap numbering of detector banks 2 and 4, ARW 19.4.12
        if nvers gt 7 then x_buf=x_buf[[88+indgen(44),44+indgen(44),indgen(44)]]
        IF (nphases EQ 1) THEN y_buf = LONG(numor) $
        ELSE y_buf = INDGEN(nphases)
        z_buf = x_buf
        chel  = 0.
        norm  = 1.
      ENDIF ELSE BEGIN
        IF iprint THEN PRINT,'TOF = 1'
        w_buf = c_tof
        x_buf = INDGEN(nchannels) + 1
        y_buf = FLTARR(nsp_tof)
        FOR iph = 0, nphases - 1 DO $
          y_buf[iph*nspectra:iph*nspectra + nspectra - 3] = par1[6:nspectra + 3]
        z_buf = y_buf
        w_tmp = LONARR(nchannels)
        FOR i = 0, nspectra - 1 DO w_tmp[*] = w_tmp[*] + w_buf[*,i]
        Imax = MAX(w_tmp,ichel)
        chel = FLOAT(ichel)
      ENDELSE
      
      IF iprint THEN PRINT,'scattering angles and chel OK'
      
      ; ------------------------
      ;  Parameters
      ; ------------------------
      
      IF par1[1] EQ -256.0 THEN chop = 0 ELSE chop = par1[1]
      
      lambda=par1[0]
      IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.855
      IF (lambda LT 3.5) THEN lambda = 3.08
      IF (lambda GT 5.0) THEN lambda = 5.98
      IF nvers EQ 6 THEN BEGIN
        IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.78
        IF (lambda LT 3.5) THEN lambda = 3.09
        IF (lambda GT 5.0) THEN lambda = 5.98
      ENDIF
      
      IF iprint THEN PRINT,'lambda = ',lambda
      
      IF nvers EQ 4 THEN n_par = 48 ELSE n_par = 52
      p_buf       = FLTARR(n_par)
      par_txt_buf = STRARR(n_par)
      
      
      p_buf[0]  = numor    & par_txt_buf[0]  = ' 0 Numor        ='
      p_buf[1]  = nspectra    & par_txt_buf[1]  = ' 1 Number of spectra     ='
      p_buf[2]  = nphases    & par_txt_buf[2]  = ' 2 Number of phases     ='
      p_buf[3]  = 1.      & par_txt_buf[3]  = ' 3 Number of runs     ='
      p_buf[4]  = lambda    & par_txt_buf[4]  = ' 4 Wavelength (A)     ='
      p_buf[5]  = chop    & par_txt_buf[5]  = ' 5 Chopper Speed (rpm)     ='
      p_buf[6]  = par1[2]    & par_txt_buf[6]  = ' 6 Number of Time Channels   ='
      p_buf[7]  = par1[3]    & par_txt_buf[7]  = ' 7 Channel Width (mcs)     ='
      p_buf[8]  = TOF     & par_txt_buf[8]  = ' 8 TOF mode (0/1)     ='
      p_buf[9]  = chel    & par_txt_buf[9]  = ' 9 Elastic Peak Channel     ='
      p_buf[10] = param[0]    & par_txt_buf[10] = '10 Set-pt Temp Start     ='
      p_buf[11] = param[3]    & par_txt_buf[11] = '11 Set-pt Temp End     ='
      p_buf[12] = param[1]    & par_txt_buf[12] = '12 Regul Temp Start     ='
      p_buf[13] = param[4]    & par_txt_buf[13] = '13 Regul Temp End     ='
      p_buf[14] = param[2]    & par_txt_buf[14] = '14 Sample Temp Start     ='
      p_buf[15] = param[5]    & par_txt_buf[15] = '15 Sample Temp End     ='
      p_buf[16] = par1[163]     & par_txt_buf[16] = '16 Bank1 Position     ='
      p_buf[17] = par1[164]     & par_txt_buf[17] = '17 Bank2 Position     ='
      p_buf[18] = par1[165]     & par_txt_buf[18] = '18 Bank3 Position     ='
      p_buf[19] = par1[166]     & par_txt_buf[19] = '19 Bank4 Position     ='
      p_buf[20] = par1[167]     & par_txt_buf[20] = '20 Upper Sample Rotation    ='
      p_buf[21] = par1[168]     & par_txt_buf[21] = '21 Lower Sample Rotation    ='
      p_buf[22] = param[18]    & par_txt_buf[22] = '22 Z Flip Current     ='
      p_buf[23] = param[19]    & par_txt_buf[23] = '23 Z Cor_Z Current     ='
      p_buf[24] = param[23]    & par_txt_buf[24] = '24 Z Cor_X Current     ='
      p_buf[25] = param[38]    & par_txt_buf[25] = '25 X Flip Current     ='
      p_buf[26] = param[39]    & par_txt_buf[26] = '26 X Cor_Z Current     ='
      p_buf[27] = param[43]    & par_txt_buf[27] = '27 X Cor_X Current     ='
      p_buf[28] = param[58]    & par_txt_buf[28] = '28 Y Flip Current     ='
      p_buf[29] = param[59]    & par_txt_buf[29] = '29 Y Cor_Z Current     ='
      p_buf[30] = param[63]    & par_txt_buf[30] = '30 Y Cor_X Current     ='
      p_buf[31] = ratioZ    & par_txt_buf[31] = '31 Z Flipping ratio     ='
      p_buf[32] = ratioX    & par_txt_buf[32] = '32 X Flipping ratio     ='
      p_buf[33] = ratioY    & par_txt_buf[33] = '33 Y Flipping ratio     ='
      p_buf[34] = run_time    & par_txt_buf[34] = '34 Run Time (seconds)     ='
      p_buf[35] = time1    & par_txt_buf[35] = '35 Start (hrs since 1st of month) ='
      p_buf[36] = time2    & par_txt_buf[36] = '36 End  (hrs since 1st of month) ='
      p_buf[37] = par1[156]    & par_txt_buf[37] = '37 Mono1 (psiA)       ='
      p_buf[38] = par1[157]    & par_txt_buf[38] = '38 Mono2 (psiB)       ='
      p_buf[39] = par1[158]    & par_txt_buf[39] = '39 Mono3 (psiC)       ='
      p_buf[40] = par1[159]    & par_txt_buf[40] = '40 Mono4 (thetaA)     ='
      p_buf[41] = par1[160]    & par_txt_buf[41] = '41 Mono5 (thetaB)     ='
      p_buf[42] = par1[161]    & par_txt_buf[42] = '42 Mono6 (thetaC)     ='
      p_buf[43] = par1[162]    & par_txt_buf[43] = '43 Mono7 (tilt angle)     ='
      p_buf[44] = dratioZ    & par_txt_buf[44] = '44 Z dFlipping ratio     ='
      p_buf[45] = dratioX    & par_txt_buf[45] = '45 X dFlipping ratio     ='
      p_buf[46] = dratioY    & par_txt_buf[46] = '46 Y dFlipping ratio     ='
      p_buf[47] = par1[169]     & par_txt_buf[47] = '47 Polariser Angle     ='
      p_buf[48] = nvers    & par_txt_buf[48] = '48 Data Version              ='
      IF nvers GE 5 THEN BEGIN
        p_buf[49] = par1[191]     & par_txt_buf[49] = '49 Gonio Angle 1     ='
        p_buf[50] = par1[192]     & par_txt_buf[50] = '50 Gonio Angle 2      ='
        ;IF numor GE 99181 THEN BEGIN
        ;ParamFile =  '/home/tofhr/lambda/D7_TEAM/CALIBRATION/D7.dat'
        ;p_buf(51) = 1    & par_txt_buf(51) = '51 Using calibration file  ='
        ;OPENR, lun, ParamFile, /GET_LUN
        ;header = STRARR(2)
        ;  READF, lun, header
        ;mParam = FLTARR(3, 4)
        ;READF, lun, mParam
        ;lambda = FLTARR(1,1)
        ;READF, lun, lambda
        ;p_buf(4)  = lambda    & par_txt_buf(4)  = ' 4 Wavelength (A)     ='
        ;ENDIF
        ;IF numor LT 99181 THEN BEGIN
        p_buf[51] = 2    & par_txt_buf[51] = '51 Not Using calibration file  ='
        lambda=par1[0]
        IF (lambda GT 3.5) AND (lambda LE 5.0) THEN lambda = 4.855
        IF (lambda LT 3.5) THEN lambda = 3.08
        IF (lambda GT 5.0) THEN lambda = 5.98
        p_buf[4]  = lambda    & par_txt_buf[4]  =   ' 4 Wavelength (A)     ='
      ;ENDIF
      ENDIF
      IF iprint THEN PRINT,'parameters assigned OK'
    END
  ENDCASE
  
  ; ----------------------------------------------------------------------------
  
  ;  ENDCASE
  
  ; ----------------------------------------------------------------------------
  ;*****************************************************************************
  
  WOUT = w_buf
  PP   = p_buf
  PTXT = par_txt_buf
  NN   = n_buf
  XX   = x_buf
  YY   = y_buf
  ZZ   = z_buf
  WT   = sub_title
  OT   = inst+' '+main_title
  
  IF (TOF EQ 0) THEN BEGIN
    XT = 'Scattering Angle'
    IF (nphases EQ 1) THEN YT = 'Counts' $
    ELSE BEGIN
    YT = 'Phase'
    ZT = 'Counts'
  ENDELSE
ENDIF ELSE BEGIN
  XT = 'Time Channels'
  YT = 'Scattering Angle'
  ZT = 'Counts'
ENDELSE
IF iprint THEN PRINT,'rdid_d7: finished'

RETURN
END

