;+
; pro rdid_d7_NX, DATA, P, P_txt, X ,Y ,Z ,N ,PV ,E, WT, XT, YT, ZT, OT, entry=[imm,nentry]
; 
; LAMP procedure to convert NEXUS data into the input required 
;	for the D7 LAMP software reduction suite
; Called from read_hdf.pro
; ARW 6.6.12
;
; REQUIRED INPUTS:
;	DATA = input data	read_hdr will return a 132 x m x n array where m is the number of TOF channels and n is the number of spectra
;	P = input parameters
;	P_txt = parameter names
;	X = x-values
;	Y = y-values
;	Z = z-values
;	N = monitor values
;	PV = user defined parameters
;	E = error values
;	WT = measurement title
;	XT = x-title
;	YT = y-title
;	ZT = z-title
;	OT = subtitle
;
; CALLS TO:  d7_psearch (included)
; LAST MODIFIED:   ARW 17.9.14
;
; ******************
; *** d7_psearch ***
; ******************
;
; pro d7_psearch,ploc,p,n_d7,p_d7
; LAMP procedure to define the output P 
; Input parameters
;     ploc = the locations of the different parameters
;     p    = the input p values
;     n_d7 = the monitor values
; Output parameters
;     p_buf = the output p values
;
; NOTE: the datasize and TOFmode parameters are NOT set here!
;
; ARW 22.2.13
; LAST MODIFIED: ARW 22.2.13
;-

pro d7_psearch,ploc,p,n_d7,p_buf
;+
;** **********
;** LAMP procedure to define the output P 
;-

    n_par=52
    
    dP = [0,P]		; Put this in to set non-existant values (Ploc = -1) to zero
                
    Zflip=N_D7[2,1]*N_D7[1,2]/(N_D7[2,2]*N_D7[1,1])
    Xflip=N_D7[2,3]*N_D7[1,4]/(N_D7[2,4]*N_D7[1,3])
    Yflip=N_D7[2,5]*N_D7[1,6]/(N_D7[2,6]*N_D7[1,5])
    dZflip = 0
    dXflip = 0
    dYflip = 0        ; p(46) Error in Y-flipping ratio calculated by monitor 2 (NEEDS TO BE FIXED)

    p_buf       = FLTARR(n_par)

    p_buf[0]  = dP[Ploc[6]+1]      ; & par_txt_buf(0)  = ' 0 Numor 			 ='
;    p_buf(1)  = datasize(0)    ; & par_txt_buf(1)  = ' 1 Number of spectra		 ='
;    p_buf(2)  = datasize(1)    ; & par_txt_buf(2)  = ' 2 Number of phases		 ='
;    p_buf(3)  = datasize(2)    ; & par_txt_buf(3)  = ' 3 Number of runs		 ='
    p_buf[4]  = dP[Ploc[66]+1]     ; & par_txt_buf(4)  = ' 4 Wavelength (A)		 ='
    p_buf[5]  = dP[Ploc[87]+1]     ; & par_txt_buf(5)  = ' 5 Chopper Speed (rpm)		 ='
    p_buf[6]  = dP[Ploc[25]+1]     ; & par_txt_buf(6)  = ' 6 Number of Time Channels	 ='
    p_buf[7]  = dP[Ploc[24]+1]     ; & par_txt_buf(7)  = ' 7 Channel Width (mcs)		 ='
;    p_buf(8)  = TOFmode        ; & par_txt_buf(8)  = ' 8 TOF mode (0/1)		 ='
    p_buf[9]  = 0               ; & par_txt_buf(9)  = ' 9 Elastic Peak Channel  	 ='
    p_buf[10] = dP[Ploc[90]+1]     ; & par_txt_buf(10) = '10 Set-pt Temp Start		 ='
    p_buf[11] = dP[Ploc[90]+1]     ; & par_txt_buf(11) = '11 Set-pt Temp End		 ='
    p_buf[12] = dP[Ploc[88]+1]     ; & par_txt_buf(12) = '12 Regul Temp Start		 ='
    p_buf[13] = dP[Ploc[88]+1]     ; & par_txt_buf(13) = '13 Regul Temp End		 ='
    p_buf[14] = dP[Ploc[89]+1]     ; & par_txt_buf(14) = '14 Sample Temp Start		 ='
    p_buf[15] = dP[Ploc[89]+1]     ; & par_txt_buf(15) = '15 Sample Temp End		 ='
    p_buf[16] = dP[Ploc[56]+1]     ; & par_txt_buf(16) = '16 Bank1 Position		 ='
    p_buf[17] = dP[Ploc[57]+1]     ; & par_txt_buf(17) = '17 Bank2 Position		 ='
    p_buf[18] = dP[Ploc[58]+1]     ; & par_txt_buf(18) = '18 Bank3 Position		 ='
    p_buf[19] = dP[Ploc[59]+1]     ; & par_txt_buf(19) = '19 Bank4 Position		 ='
    p_buf[20] = dP[Ploc[60]+1]     ; & par_txt_buf(20) = '20 Upper Sample Rotation 	 ='
    p_buf[21] = 0               ; & par_txt_buf(21) = '21 Lower Sample Rotation 	 ='
    p_buf[22] = dP[Ploc[41]+1]     ; & par_txt_buf(22) = '22 Z Flip Current		 ='
    p_buf[23] = dP[Ploc[42]+1]     ; & par_txt_buf(23) = '23 Z Cor_Z Current		 ='
    p_buf[24] = 0               ; & par_txt_buf(24) = '24 Z Cor_X Current		 ='
    p_buf[25] = dP[Ploc[46]+1]     ; & par_txt_buf(25) = '25 X Flip Current		 ='
    p_buf[26] = dP[Ploc[47]+1]     ; & par_txt_buf(26) = '26 X Cor_Z Current		 ='
    p_buf[27] = 0               ; & par_txt_buf(27) = '27 X Cor_X Current		 ='
    p_buf[28] = dP[Ploc[51]+1]     ; & par_txt_buf(28) = '28 Y Flip Current		 ='
    p_buf[29] = dP[Ploc[52]+1]     ; & par_txt_buf(29) = '29 Y Cor_Z Current		 ='
    p_buf[30] = 0               ; & par_txt_buf(30) = '30 Y Cor_X Current		 ='
    p_buf[31] = Zflip           ; & par_txt_buf(31) = '31 Z Flipping ratio		 ='
    p_buf[32] = Xflip           ; & par_txt_buf(32) = '32 X Flipping ratio		 ='
    p_buf[33] = Yflip           ; & par_txt_buf(33) = '33 Y Flipping ratio		 ='
    p_buf[34] = dP[Ploc[0]+1]      ; & par_txt_buf(34) = '34 Run Time (seconds)		 ='
    p_buf[35] = 0               ; & par_txt_buf(35) = '35 Start (hrs since 1st of month) ='
    p_buf[36] = 0               ; & par_txt_buf(36) = '36 End	(hrs since 1st of month) ='
    p_buf[37] = dP[Ploc[67]+1]     ; & par_txt_buf(37) = '37 Mono1 (psiA)  		 ='
    p_buf[38] = dP[Ploc[68]+1]     ; & par_txt_buf(38) = '38 Mono2 (psiB)  		 ='
    p_buf[39] = dP[Ploc[69]+1]     ; & par_txt_buf(39) = '39 Mono3 (psiC)  		 ='
    p_buf[40] = dP[Ploc[70]+1]     ; & par_txt_buf(40) = '40 Mono4 (thetaA)		 ='
    p_buf[41] = dP[Ploc[71]+1]     ; & par_txt_buf(41) = '41 Mono5 (thetaB)		 ='
    p_buf[42] = dP[Ploc[72]+1]     ; & par_txt_buf(42) = '42 Mono6 (thetaC)		 ='
    p_buf[43] = 0               ; & par_txt_buf(43) = '43 Mono7 (tilt angle)		 ='
    p_buf[44] = dZflip          ; & par_txt_buf(44) = '44 Z dFlipping ratio		 ='
    p_buf[45] = dXflip          ; & par_txt_buf(45) = '45 X dFlipping ratio		 ='
    p_buf[46] = dYflip          ; & par_txt_buf(46) = '46 Y dFlipping ratio		 ='
    p_buf[47] = dP[Ploc[85]+1]     ; & par_txt_buf(47) = '47 Polariser Angle		 ='
    p_buf[48] = 8               ; & par_txt_buf(48) = '48 Data Version		       	 ='
    p_buf[49] = 0               ; & par_txt_buf(49) = '49 Gonio Angle 1		 ='
    p_buf[50] = 0               ; & par_txt_buf(50) = '50 Gonio Angle 2	 	 ='
    p_buf[51] = 2               ; & par_txt_buf(51) = '51 Not Using calibration file	='

end




; ***********************
; *** Now the routine ***
; ***********************

pro rdid_d7_NX, DATA, P, P_txt, X ,Y ,Z ,N ,PV ,E, WT, XT, YT, ZT, OT
;+
;** **********
;** LAMP procedure to convert NEXUS data into the input required 
;** for the D7 LAMP software reduction suite
;** Called from read_hdf.pro
;** ARW 6.6.12
;-

  delme=1	; Dummy variable so that the LAMP runtime will update

; Find where the parameters are stored
  Ploc=intarr(91)

; Entry section
  Ploc[0]  = where(strcmp(P_txt,'(Entry) duration',16) eq 1)	     ; run time
  Ploc[1]  = where(strcmp(P_txt,'(Entry) end_time',16) eq 1)
  Ploc[2]  = where(strcmp(P_txt,'(Entry) experime',16) eq 1)
  Ploc[3]  = where(strcmp(P_txt,'(Entry) mode:',13) eq 1)
  Ploc[4]  = where(strcmp(P_txt,'(Entry) notes:',14) eq 1)
  Ploc[5]  = where(strcmp(P_txt,'(Entry) preset',14) eq 1)
  Ploc[6]  = where(strcmp(P_txt,'(Entry) run_nu',14) eq 1)
  Ploc[7]  = where(strcmp(P_txt,'(Entry) start_',14) eq 1)

; Detectors and monitors    
  Ploc[8]  = where(strcmp(P_txt,'(I detector) bank2_offset',25) eq 1)
  Ploc[9]  = where(strcmp(P_txt,'(I detector) bank2_slop',23) eq 1)
  Ploc[10] = where(strcmp(P_txt,'(I detector) bank3_offset',25) eq 1)
  Ploc[11] = where(strcmp(P_txt,'(I detector) bank3_slop',23) eq 1)
  Ploc[12] = where(strcmp(P_txt,'(I detector) bank4_offset',25) eq 1)
  Ploc[13] = where(strcmp(P_txt,'(I detector) bank4_slop',23) eq 1)

  Ploc[14] = where(strcmp(P_txt,'(I detector) detrate',20) eq 1)
  Ploc[15] = where(strcmp(P_txt,'(I detector) detsum',19) eq 1)         ; Total detector counts

; Monitor
  Ploc[16] = where(strcmp(P_txt,'(Monitor) data',14) eq 1)        ; Monitor 1 counts
  Ploc[17] = where(strcmp(P_txt,'(Monitor) inte',14) eq 1)        ; Monitor 1 integral
  Ploc[18] = where(strcmp(P_txt,'(Monitor) monr',14) eq 1)        ; Monitor 1 rate
  Ploc[19] = where(strcmp(P_txt,'(Monitor) mons',14) eq 1)        ; Monitor 1 sum

  Ploc[20] = where(strcmp(P_txt,'(Monitor2) data',15) eq 1)       ; Monitor 2 counts
  Ploc[21] = where(strcmp(P_txt,'(Monitor2) inte',15) eq 1)       ; Monitor 2 integral
  Ploc[22] = where(strcmp(P_txt,'(Monitor2) monr',15) eq 1)       ; Monitor 2 rate
  Ploc[23] = where(strcmp(P_txt,'(Monitor2) mons',15) eq 1)       ; Monitor 2 sum

; Determine whether in TOF or No TOF
  mesmode=where(strcmp(P_txt,'(Monitor) time_',15) eq 1) ; =1 element (NoTOF), =3 elements (TOF)
  if n_elements(mesmode) eq 1 then begin
    Ploc[24] = [mesmode,-1,-1] 
    TOFmode = 0
  endif else begin
    Ploc[24:26]=mesmode
    TOFmode = 1
  endelse
  
; Flippers
  Ploc[27] = where(strcmp(P_txt,'(I flipper) b1_flip_c',21)eq 1)
  Ploc[28] = where(strcmp(P_txt,'(I flipper) b2_flip_c',21)eq 1)
  Ploc[29] = where(strcmp(P_txt,'(I flipper) b3_flip_c',21)eq 1)
  Ploc[30] = where(strcmp(P_txt,'(I flipper) b4_flip_c',21)eq 1)
  Ploc[31] = where(strcmp(P_txt,'(I flipper) b5_flip_c',21)eq 1)

  Ploc[32] = where(strcmp(P_txt,'(I flipper) b1_wanted',21)eq 1)
  Ploc[33] = where(strcmp(P_txt,'(I flipper) b2_wanted',21)eq 1)
  Ploc[34] = where(strcmp(P_txt,'(I flipper) b3_wanted',21)eq 1)
  Ploc[35] = where(strcmp(P_txt,'(I flipper) b4_wanted',21)eq 1)
  Ploc[36] = where(strcmp(P_txt,'(I flipper) b5_wanted',21)eq 1)
      
  Ploc[37] = where(strcmp(P_txt,'(I flipper) fb1b',16)eq 1)     
  Ploc[38] = where(strcmp(P_txt,'(I flipper) fxpo',16)eq 1)     
  Ploc[39] = where(strcmp(P_txt,'(I flipper) fypo',16)eq 1)     
  Ploc[40] = where(strcmp(P_txt,'(I flipper) fzpo',16)eq 1)     
     
  Ploc[41] = where(strcmp(P_txt,'(I flipper) wanted_pcz_b1',25)eq 1)
  Ploc[42] = where(strcmp(P_txt,'(I flipper) wanted_pcz_b2',25)eq 1)
  Ploc[43] = where(strcmp(P_txt,'(I flipper) wanted_pcz_b3',25)eq 1)
  Ploc[44] = where(strcmp(P_txt,'(I flipper) wanted_pcz_b4',25)eq 1)
  Ploc[45] = where(strcmp(P_txt,'(I flipper) wanted_pcz_b5',25)eq 1)
      
  Ploc[46] = where(strcmp(P_txt,'(I flipper) wanted_pcx_b1',25)eq 1)
  Ploc[47] = where(strcmp(P_txt,'(I flipper) wanted_pcx_b2',25)eq 1)
  Ploc[48] = where(strcmp(P_txt,'(I flipper) wanted_pcx_b3',25)eq 1)
  Ploc[49] = where(strcmp(P_txt,'(I flipper) wanted_pcx_b4',25)eq 1)
  Ploc[50] = where(strcmp(P_txt,'(I flipper) wanted_pcx_b5',25)eq 1)

  Ploc[51] = where(strcmp(P_txt,'(I flipper) wanted_pcy_b1',25)eq 1)
  Ploc[52] = where(strcmp(P_txt,'(I flipper) wanted_pcy_b2',25)eq 1)
  Ploc[53] = where(strcmp(P_txt,'(I flipper) wanted_pcy_b3',25)eq 1)
  Ploc[54] = where(strcmp(P_txt,'(I flipper) wanted_pcy_b4',25)eq 1)
  Ploc[55] = where(strcmp(P_txt,'(I flipper) wanted_pcy_b5',25)eq 1)

; Bank and omega positions

;  Ploc(56) = where(strcmp(P_txt,'(I positionner) actual_bank1',28)eq 1)
;  Ploc(57) = where(strcmp(P_txt,'(I positionner) actual_bank2',28)eq 1)
;  Ploc(58) = where(strcmp(P_txt,'(I positionner) actual_bank3',28)eq 1)
;  Ploc(59) = where(strcmp(P_txt,'(I positionner) actual_bank4',28)eq 1)
;  Ploc(60) = 73	; omega hard wired for the moment

;  Ploc(61) = where(strcmp(P_txt,'(I positionner) requested_bank1',31)eq 1)
;  Ploc(62) = where(strcmp(P_txt,'(I positionner) requested_bank2',31)eq 1)
;  Ploc(63) = where(strcmp(P_txt,'(I positionner) requested_bank3',31)eq 1)
;  Ploc(64) = where(strcmp(P_txt,'(I positionner) requested_bank4',31)eq 1)
;  Ploc(65) = 98	; omega hard wired for the moment

  Ploc[56] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* bank1*') eq 1)
  Ploc[57] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* bank2*') eq 1)
  Ploc[58] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* bank3*') eq 1)
  Ploc[59] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* bank4*') eq 1)
  Ploc[60] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* omega*') eq 1)

  Ploc[61] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* bank1*') eq 1)
  Ploc[62] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* bank2*') eq 1)
  Ploc[63] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* bank3*') eq 1)
  Ploc[64] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* bank4*') eq 1)
  Ploc[65] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* omega*') eq 1)

; Monochromators

  Ploc[66] = where(strcmp(P_txt,'(I monochromator) wave',22) eq 1)
     
  Ploc[67] = where(strcmp(P_txt,'(I monochromator) value_mono1',29) eq 1)
  Ploc[68] = where(strcmp(P_txt,'(I monochromator) value_mono2',29) eq 1)
  Ploc[69] = where(strcmp(P_txt,'(I monochromator) value_mono3',29) eq 1)
  Ploc[70] = where(strcmp(P_txt,'(I monochromator) value_mono4',29) eq 1)
  Ploc[71] = where(strcmp(P_txt,'(I monochromator) value_mono5',29) eq 1)
  Ploc[72] = where(strcmp(P_txt,'(I monochromator) value_mono6',29) eq 1)
      
  Ploc[73] = where(strcmp(P_txt,'(I monochromator) offset_value_mono1',36) eq 1)
  Ploc[74] = where(strcmp(P_txt,'(I monochromator) offset_value_mono2',36) eq 1)
  Ploc[75] = where(strcmp(P_txt,'(I monochromator) offset_value_mono3',36) eq 1)
  Ploc[76] = where(strcmp(P_txt,'(I monochromator) offset_value_mono4',36) eq 1)
  Ploc[77] = where(strcmp(P_txt,'(I monochromator) offset_value_mono5',36) eq 1)
  Ploc[78] = where(strcmp(P_txt,'(I monochromator) offset_value_mono6',36) eq 1)
      
  Ploc[79] = where(strcmp(P_txt,'(I monochromator) target_value_mono1',36) eq 1)
  Ploc[80] = where(strcmp(P_txt,'(I monochromator) target_value_mono2',36) eq 1)
  Ploc[81] = where(strcmp(P_txt,'(I monochromator) target_value_mono3',36) eq 1)
  Ploc[82] = where(strcmp(P_txt,'(I monochromator) target_value_mono4',36) eq 1)
  Ploc[83] = where(strcmp(P_txt,'(I monochromator) target_value_mono5',36) eq 1)
  Ploc[84] = where(strcmp(P_txt,'(I monochromator) target_value_mono6',36) eq 1)
     
; POLAR
;  Ploc(85) = 71 ; POLAR actual value hard wired for the moment
;  Ploc(86) = 101; POLAR requested value hard wired for the moment
  Ploc[85] = where(strmatch(P_txt,'*actual*')eq 1 AND strmatch(P_txt,'* polar*') eq 1)
  Ploc[86] = where(strmatch(P_txt,'*requested*')eq 1 AND strmatch(P_txt,'* polar*') eq 1)

; Chopper
  Ploc[87] = where(strmatch(P_txt,'*rotation_speed*') eq 1)  ; Chopper frequency, value hardwired for the moment
  
; Sample environment
  Ploc[88] = where(strcmp(P_txt,'(Sample) regulation_temp',24) eq 1)
  Ploc[89] = where(strcmp(P_txt,'(Sample) temperature',20) eq 1)
  Ploc[90] = where(strcmp(P_txt,'(Sample) setpoint_te',20) eq 1)

; Determine the size of the data
  datasize=size(DATA,/DIMENSIONS)
  for i=0,2-n_elements(datasize) do datasize=[datasize,1] 
  monsize=size(N,/DIMENSIONS)
  for i=0,2-n_elements(monsize) do monsize=[monsize,1]

; Determine polarization states
; First find the correct currents for the different polarization states
  polmat=fltarr(5,3)	; rows=(Z,X,Y), cols=(B1..B5)
  
  polmat[0,0]=P[Ploc[41],0,0]
  polmat[1,0]=P[Ploc[42],0,0]
  polmat[2,0]=P[Ploc[43],0,0]
  polmat[3,0]=P[Ploc[44],0,0]
  polmat[4,0]=P[Ploc[45],0,0]

  polmat[0,1]=P[Ploc[46],0,0]
  polmat[1,1]=P[Ploc[47],0,0]
  polmat[2,1]=P[Ploc[48],0,0]
  polmat[3,1]=P[Ploc[49],0,0]
  polmat[4,1]=P[Ploc[50],0,0]

  polmat[0,2]=P[Ploc[51],0,0]
  polmat[1,2]=P[Ploc[52],0,0]
  polmat[2,2]=P[Ploc[53],0,0]
  polmat[3,2]=P[Ploc[54],0,0]
  polmat[4,2]=P[Ploc[55],0,0]

; Search POL table to make sure that there are no repeats
; Mod added 17.9.14, ARW
  polmat[2,1]=polmat[2,1]+10*array_equal(polmat[2:4,0],polmat[2:4,1])
  polmat[2,2]=polmat[2,2]+10*array_equal(polmat[2:4,0],polmat[2:4,2])
  polmat[2,2]=polmat[2,2]+10*array_equal(polmat[2:4,1],polmat[2:4,2])

; Now find the desired values for the polarization states
  Bvals=fltarr(5,1,datasize[2])		; B1..5 for each measurement
  Bvals[0,0,*]=P[Ploc[32],0,*]
  Bvals[1,0,*]=P[Ploc[33],0,*]
  Bvals[2,0,*]=P[Ploc[34],0,*]
  Bvals[3,0,*]=P[Ploc[35],0,*]
  Bvals[4,0,*]=P[Ploc[36],0,*]

; Create X values
  X_D7=indgen(datasize[0])

; Create title information
  WT_D7 = strjoin([WT,', run ',strtrim(string(long(P[Ploc[6]])),1)])
  dummy_OT=strlen('(Entry) experiment_identifier:')
  OT_D7 = strmid(P_txt[Ploc[2]],dummy_OT,strlen(P_txt[Ploc[2]])-dummy_OT-2 )

; Split between NoTOF and TOF data
  if n_elements(mesmode) eq 1 then begin 	; No TOF data
; No TOF data first, conceivable measurements are:
; i)	Single count	(NOPO, ZPO, XYZPO)
; ii)	twotheta scan	(NOPO, ZPO, XYZPO)
; iii)	omega scan	(NOPO, ZPO, XYZPO) 

; Determine the bank and omega positions
; Actual
     banksa=fltarr(5,1,datasize[2])		; bank 1..4 + omega for each measurement
     banksa[0,0,*]=P[Ploc[56],0,*]
     banksa[1,0,*]=P[Ploc[57],0,*]
     banksa[2,0,*]=P[Ploc[58],0,*]
     banksa[3,0,*]=P[Ploc[59],0,*]
     banksa[4,0,*]=P[Ploc[60],0,*]	; omega hard wired for the moment

; Find how many twotheta/omega values
     NB2 =banksa[1,0,uniq(banksa[1,0,sort(banksa[1,0,*])])]
     NB3 =banksa[2,0,uniq(banksa[2,0,sort(banksa[2,0,*])])]
     NB4 =banksa[3,0,uniq(banksa[3,0,sort(banksa[3,0,*])])]
     NBom=banksa[4,0,uniq(banksa[4,0,sort(banksa[4,0,*])])]

; The last digit may fluctuate within some tolerance
; Tolerance will be taken to be +/- 0.03 for safety
     tol=0.03
     if n_elements(NB2) gt 1 then begin
          for i=1,n_elements(NB2)-1 do begin
               NBdiff=NB2[i]-NB2[i-1]
               if abs(NBdiff) le tol then NB2[i]=NB2[i-1] 
          endfor
          NB2=NB2[uniq(NB2)]
     endif
     if n_elements(NB3) gt 1 then begin
          for i=1,n_elements(NB3)-1 do begin
               NBdiff=NB3[i]-NB3[i-1]
               if abs(NBdiff) le tol then NB3[i]=NB3[i-1] 
          endfor
          NB3=NB3[uniq(NB3)]
     endif
     if n_elements(NB4) gt 1 then begin
          for i=1,n_elements(NB4)-1 do begin
               NBdiff=NB4[i]-NB4[i-1]
               if abs(NBdiff) le tol then NB4[i]=NB4[i-1] 
          endfor
          NB4=NB4[uniq(NB4)]
     endif
     if n_elements(NBom) gt 1 then begin
          for i=1,n_elements(NBom)-1 do begin
               NBdiff=NBom[i]-NBom[i-1]
               if abs(NBdiff) le tol then NBom[i]=NBom[i-1] 
          endfor
          NBom=NBom[uniq(NBom)]
     endif

     Ntth=[n_elements(NB2),n_elements(NB3),n_elements(NB4),n_elements(NBom)]
     whichbankmoves=where(Ntth gt 1)
     case whichbankmoves[0] of
	-1:	begin
		   Npts = NB2
		   Brow = 1
		end
	0:	begin
		   Npts = NB2
		   Brow = 1
		end
	1:	begin
		   Npts = NB3
		   Brow = 2
		end
	2:	begin
		   Npts = NB4
		   Brow = 3
		end
	3:	begin
		   Npts = NBom
		   Brow = 4
		end
     endcase	 

; Create arrays
     DATA_D7=fltarr(datasize[0],6,n_elements(Npts))
     N_D7=fltarr(4,7,n_elements(Npts))
; Now loop through
     for i=0,datasize[2]-1 do begin
     ; Determine which flipper state
        if array_equal(Bvals[0:1,0,i],[0,0]) eq 1 then XYZtag=0 else XYZtag=1
     ; Determine which polarization
        if array_equal(Bvals[2:4,0,i],[0,0,0]) eq 1 or array_equal(Bvals[2:4,0,i],polmat[2:4,0]) eq 1 then XYZtag=[XYZtag,0]	; Z
        if array_equal(Bvals[2:4,0,i],polmat[2:4,1]) eq 1 then XYZtag=[XYZtag,2]	; X
        if array_equal(Bvals[2:4,0,i],polmat[2:4,2]) eq 1 then XYZtag=[XYZtag,4]	; Y
     ; XYZtag = 0 (Zoff)
     ;        = 1 (Zon)
     ;        = 2 (Xoff)
     ;        = 3 (Xon)
     ;        = 4 (Y off)
     ;        = 5 (Y on)
        XYZtag=fix(total(XYZtag))

     ; Determine which omega/twotheta
        TTHtag = where(Npts eq banksa[Brow,0,i])

     ; Now fill in the array
	DATA_D7[*,XYZtag,TTHtag] = DATA[*,0,i]
        N_D7[*,XYZtag+1,TTHtag] = [P[Ploc[0],0,i]*100.0,P[Ploc[19],0,i],P[Ploc[23],0,i],P[Ploc[15],0,i]] 
     endfor

; Remove flipper states with no data in them
    DATA_D7 = DATA_D7[*,where(total(DATA_D7[*,*,0],1) ne 0),*]
;
; Create X, Y, Z values
     datasize=size(DATA_D7,/DIMENSIONS)
     for i=0,2-n_elements(datasize) do datasize=[datasize,1]
     X_D7=indgen(datasize[0])
     Y_D7 = indgen(datasize[1])
     Z_D7 = X_D7

; Create PV values if there are more than one omega/twotheta values
; NOTE: This is equivalent to 'rdand'
     PV_D7 = PV
     if whichbankmoves ne -1 then begin
        TTHtag = where(Npts[i] eq banksa[Brow,0,*])
        TTHtag = TTHtag[0]
        d7_psearch,Ploc,P[*,0,TTHtag],N_D7[*,*,i],p_buf
        PV_D7 = p_buf
        for i=1,n_elements(Npts)-1 do begin
            TTHtag = where(Npts[i] eq banksa[Brow,0,*])
            TTHtag = TTHtag[0]
            d7_psearch,Ploc,P[*,0,TTHtag],N_D7[*,*,i],p_buf
            PV_D7=[[PV_D7],[p_buf]]
        endfor
        PV_D7[1,*]=datasize[0]
        PV_D7[2,*]=datasize[1]
        PV_D7[3,*]=datasize[2]
     endif


; Create axis labels
     XT_D7 = 'Detector Number'
     ZT_D7=''
     if n_elements(Y_D7) eq 1 then YT_D7 = 'COUNTS' $
     else begin
         YT_D7 = 'Phase'
         ZT_D7 = 'Counts'
     endelse
    


  endif else begin				; TOF data
; TOF data now, conceivable measurements are:
; i)	Single count (NOPO, ZPO, XYZPO)

; Incorporate the monitor array in to the data array
    n=[[n],[fltarr(monsize[0],1,monsize[2])]]
    DATA=[[DATA],[n]]

    datasize=size(data,/DIMENSIONS)
    tof_datasize=size(data,/DIMENSIONS)	; keep for later
    for i=0,2-n_elements(datasize) do begin
	datasize=[datasize,1]
	tof_datasize=[tof_datasize,1]         ; Included this line 12.9.14 for TOF with no flipper
    endfor
        
; Determine the flipper states
     ; XYZtag = 0 (Zoff)
     ;        = 1 (Zon)
     ;        = 2 (Xoff)
     ;        = 3 (Xon)
     ;        = 4 (Y off)
     ;        = 5 (Y on)
    XYZtag=intarr(datasize[2])
    for i=0,datasize[2]-1 do begin
        if array_equal(Bvals[0:1,0,i],[0,0]) ne 1 then XYZtag[i]=XYZtag[i]+1
        if array_equal(Bvals[2:4,0,i],polmat[2:4,1]) eq 1 then XYZtag[i]=XYZtag[i]+2
        if array_equal(Bvals[2:4,0,i],polmat[2:4,2]) eq 1 then XYZtag[i]=XYZtag[i]+4
    endfor
        
; Now arrange data to have the correct output format
    DATA_D7=fltarr(datasize[0],datasize[1]*datasize[2])        
    N_D7=fltarr(4,7)
    TTH =findgen(datasize[1])           ; NOTE: Leave this in until better estimate for twotheta comes along
    Y_D7=[TTH,0,0,0]   ; NOTE: Leave this in until better estimate for twotheta comes along

    sortXYZtag=sort(XYZtag)
    for i=0,datasize[2]-1 do begin
        DATA_D7[*,datasize[1]*sortXYZtag[i]:datasize[1]*(sortXYZtag[i]+1)-1]=DATA[*,*,i]
        N_D7[*,XYZtag[i]+1]=[P[Ploc[0],0,i]*100.0,P[Ploc[19],0,i],P[Ploc[23],0,i],P[Ploc[15],0,i]]
        Y_D7=[Y_D7,TTH,0,0,0]
    endfor                

; Create PV values
    PV_D7 = PV

; Create X, Z values
    datasize=size(DATA_D7,/DIMENSIONS)
    X_D7=indgen(datasize[0])
    Z_D7=Y_D7
    
; Create axis labels
    XT_D7 = 'Time Channels'
    YT_D7 = 'Detector Number'
    ZT_D7 = 'Counts'

 endelse




; Write the parameter block
; *** Initialize the parameter values if this is the first spectrum
  n_par=52
        
    datasize=size(DATA_D7,/DIMENSIONS)
    for i=0,2-n_elements(datasize) do datasize=[datasize,1]   ; Accounts for 'no polarization' measurements
            
;    Zflip=N_D7(2,1)*N_D7(1,2)/(N_D7(2,2)*N_D7(1,1))
;    Xflip=N_D7(2,3)*N_D7(1,4)/(N_D7(2,4)*N_D7(1,3))
;    Yflip=N_D7(2,5)*N_D7(1,6)/(N_D7(2,6)*N_D7(1,5))
;    dZflip = 0
;    dXflip = 0
;    dYflip = 0                                                                ; p(46) Error in Y-flipping ratio calculated by monitor 2 (NEEDS TO BE FIXED)

;    p_buf       = FLTARR(n_par)
    par_txt_buf = STRARR(n_par)


;    p_buf(0)  = P(Ploc(6))  & par_txt_buf(0)  = ' 0 Numor 			 ='
;    p_buf(1)  = datasize(0) & par_txt_buf(1)  = ' 1 Number of spectra		 ='
;    p_buf(2)  = datasize(1) & par_txt_buf(2)  = ' 2 Number of phases		 ='
;    p_buf(3)  = datasize(2) & par_txt_buf(3)  = ' 3 Number of runs		 ='
;    p_buf(4)  = P(Ploc(66)) & par_txt_buf(4)  = ' 4 Wavelength (A)		 ='
;    p_buf(5)  = P(Ploc(87))	  & par_txt_buf(5)  = ' 5 Chopper Speed (rpm)		 ='
;    p_buf(6)  = P(Ploc(24))	  & par_txt_buf(6)  = ' 6 Number of Time Channels	 ='
;    p_buf(7)  = P(Ploc(25))	  & par_txt_buf(7)  = ' 7 Channel Width (mcs)		 ='
;    p_buf(8)  = TOFmode 	  & par_txt_buf(8)  = ' 8 TOF mode (0/1)		 ='
;    p_buf(9)  = 0   	  & par_txt_buf(9)  = ' 9 Elastic Peak Channel  	 ='
;    p_buf(10) = P(Ploc(90))	  & par_txt_buf(10) = '10 Set-pt Temp Start		 ='
;    p_buf(11) = P(Ploc(90))	  & par_txt_buf(11) = '11 Set-pt Temp End		 ='
;    p_buf(12) = P(Ploc(88))	  & par_txt_buf(12) = '12 Regul Temp Start		 ='
;    p_buf(13) = P(Ploc(88))	  & par_txt_buf(13) = '13 Regul Temp End		 ='
;    p_buf(14) = P(Ploc(89))	  & par_txt_buf(14) = '14 Sample Temp Start		 ='
;    p_buf(15) = P(Ploc(89))	  & par_txt_buf(15) = '15 Sample Temp End		 ='
;    p_buf(16) = P(Ploc(56))   & par_txt_buf(16) = '16 Bank1 Position		 ='
;    p_buf(17) = P(Ploc(57))   & par_txt_buf(17) = '17 Bank2 Position		 ='
;    p_buf(18) = P(Ploc(58))   & par_txt_buf(18) = '18 Bank3 Position		 ='
;    p_buf(19) = P(Ploc(59))   & par_txt_buf(19) = '19 Bank4 Position		 ='
;    p_buf(20) = P(Ploc(60))   & par_txt_buf(20) = '20 Upper Sample Rotation 	 ='
;    p_buf(21) = 0             & par_txt_buf(21) = '21 Lower Sample Rotation 	 ='
;    p_buf(22) = P(Ploc(41))	  & par_txt_buf(22) = '22 Z Flip Current		 ='
;    p_buf(23) = P(Ploc(42))	  & par_txt_buf(23) = '23 Z Cor_Z Current		 ='
;    p_buf(24) = 0   	  & par_txt_buf(24) = '24 Z Cor_X Current		 ='
;    p_buf(25) = P(Ploc(46))	  & par_txt_buf(25) = '25 X Flip Current		 ='
;    p_buf(26) = P(Ploc(47))	  & par_txt_buf(26) = '26 X Cor_Z Current		 ='
;    p_buf(27) = 0   	  & par_txt_buf(27) = '27 X Cor_X Current		 ='
;    p_buf(28) = P(Ploc(51))	  & par_txt_buf(28) = '28 Y Flip Current		 ='
;    p_buf(29) = P(Ploc(52))	  & par_txt_buf(29) = '29 Y Cor_Z Current		 ='
;    p_buf(30) = 0   	  & par_txt_buf(30) = '30 Y Cor_X Current		 ='
;    p_buf(31) = Zflip	  & par_txt_buf(31) = '31 Z Flipping ratio		 ='
;    p_buf(32) = Xflip	  & par_txt_buf(32) = '32 X Flipping ratio		 ='
;    p_buf(33) = Yflip	  & par_txt_buf(33) = '33 Y Flipping ratio		 ='
;    p_buf(34) = P(Ploc(0))	  & par_txt_buf(34) = '34 Run Time (seconds)		 ='
;    p_buf(35) = 0   	  & par_txt_buf(35) = '35 Start (hrs since 1st of month) ='
;    p_buf(36) = 0   	  & par_txt_buf(36) = '36 End	(hrs since 1st of month) ='
;    p_buf(37) = P(Ploc(67))	  & par_txt_buf(37) = '37 Mono1 (psiA)  		 ='
;    p_buf(38) = P(Ploc(68))	  & par_txt_buf(38) = '38 Mono2 (psiB)  		 ='
;    p_buf(39) = P(Ploc(69))	  & par_txt_buf(39) = '39 Mono3 (psiC)  		 ='
;    p_buf(40) = P(Ploc(70))	  & par_txt_buf(40) = '40 Mono4 (thetaA)		 ='
;    p_buf(41) = P(Ploc(71))	  & par_txt_buf(41) = '41 Mono5 (thetaB)		 ='
;    p_buf(42) = P(Ploc(72))	  & par_txt_buf(42) = '42 Mono6 (thetaC)		 ='
;    p_buf(43) = 0	          & par_txt_buf(43) = '43 Mono7 (tilt angle)		 ='
;    p_buf(44) = dZflip	  & par_txt_buf(44) = '44 Z dFlipping ratio		 ='
;    p_buf(45) = dXflip	  & par_txt_buf(45) = '45 X dFlipping ratio		 ='
;    p_buf(46) = dYflip	  & par_txt_buf(46) = '46 Y dFlipping ratio		 ='
;    p_buf(47) = P(Ploc(85))   & par_txt_buf(47) = '47 Polariser Angle		 ='

    par_txt_buf[0]  = ' 0 Numor 			 ='
    par_txt_buf[1]  = ' 1 Number of spectra		 ='
    par_txt_buf[2]  = ' 2 Number of phases		 ='
    par_txt_buf[3]  = ' 3 Number of runs		 ='
    par_txt_buf[4]  = ' 4 Wavelength (A)		 ='
    par_txt_buf[5]  = ' 5 Chopper Speed (rpm)		 ='
    par_txt_buf[6]  = ' 6 Number of Time Channels	 ='
    par_txt_buf[7]  = ' 7 Channel Width (mcs)		 ='
    par_txt_buf[8]  = ' 8 TOF mode (0/1)		 ='
    par_txt_buf[9]  = ' 9 Elastic Peak Channel  	 ='
    par_txt_buf[10] = '10 Set-pt Temp Start		 ='
    par_txt_buf[11] = '11 Set-pt Temp End		 ='
    par_txt_buf[12] = '12 Regul Temp Start		 ='
    par_txt_buf[13] = '13 Regul Temp End		 ='
    par_txt_buf[14] = '14 Sample Temp Start		 ='
    par_txt_buf[15] = '15 Sample Temp End		 ='
    par_txt_buf[16] = '16 Bank1 Position		 ='
    par_txt_buf[17] = '17 Bank2 Position		 ='
    par_txt_buf[18] = '18 Bank3 Position		 ='
    par_txt_buf[19] = '19 Bank4 Position		 ='
    par_txt_buf[20] = '20 Upper Sample Rotation 	 ='
    par_txt_buf[21] = '21 Lower Sample Rotation 	 ='
    par_txt_buf[22] = '22 Z Flip Current		 ='
    par_txt_buf[23] = '23 Z Cor_Z Current		 ='
    par_txt_buf[24] = '24 Z Cor_X Current		 ='
    par_txt_buf[25] = '25 X Flip Current		 ='
    par_txt_buf[26] = '26 X Cor_Z Current		 ='
    par_txt_buf[27] = '27 X Cor_X Current		 ='
    par_txt_buf[28] = '28 Y Flip Current		 ='
    par_txt_buf[29] = '29 Y Cor_Z Current		 ='
    par_txt_buf[30] = '30 Y Cor_X Current		 ='
    par_txt_buf[31] = '31 Z Flipping ratio		 ='
    par_txt_buf[32] = '32 X Flipping ratio		 ='
    par_txt_buf[33] = '33 Y Flipping ratio		 ='
    par_txt_buf[34] = '34 Run Time (seconds)		 ='
    par_txt_buf[35] = '35 Start (hrs since 1st of month) ='
    par_txt_buf[36] = '36 End	(hrs since 1st of month) ='
    par_txt_buf[37] = '37 Mono1 (psiA)  		 ='
    par_txt_buf[38] = '38 Mono2 (psiB)  		 ='
    par_txt_buf[39] = '39 Mono3 (psiC)  		 ='
    par_txt_buf[40] = '40 Mono4 (thetaA)		 ='
    par_txt_buf[41] = '41 Mono5 (thetaB)		 ='
    par_txt_buf[42] = '42 Mono6 (thetaC)		 ='
    par_txt_buf[43] = '43 Mono7 (tilt angle)		 ='
    par_txt_buf[44] = '44 Z dFlipping ratio		 ='
    par_txt_buf[45] = '45 X dFlipping ratio		 ='
    par_txt_buf[46] = '46 Y dFlipping ratio		 ='
    par_txt_buf[47] = '47 Polariser Angle		 ='
    par_txt_buf[48] = '48 Data Version		       	 ='
    par_txt_buf[49] = '49 Gonio Angle 1		 ='
    par_txt_buf[50] = '50 Gonio Angle 2	 	 ='
    par_txt_buf[51] = '51 Not Using calibration file	='
    par_txt_buf[48] = '48 Data Version		       	 ='
    par_txt_buf[49] = '49 Gonio Angle 1		 ='
    par_txt_buf[50] = '50 Gonio Angle 2	 	 ='
    par_txt_buf[51] = '51 Not Using calibration file	='

    d7_psearch,Ploc,P[*,0,0],N_D7[*,*,0],p_buf
    p_buf[8]  = TOFmode

    if TOFmode eq 0 then begin
        p_buf[1]  = datasize[0]
        p_buf[2]  = datasize[1]
        p_buf[3]  = datasize[2]
    endif else begin
	p_buf[1]  = tof_datasize[1]
        p_buf[2]  = tof_datasize[2]
        p_buf[3]  = 1		; Assuming for the moment that we won't make twotheta scans in TOF mode
    endelse

; *** Return the right values

    DATA    = DATA_D7
    N       = N_D7
    P       = p_buf
    P_txt   = par_txt_buf
    PV      = PV_D7
    X       = X_D7
    Y       = Y_D7
    Z       = Z_D7
    WT      = WT_D7
    OT      = OT_D7
    XT      = XT_D7
    YT      = YT_D7
    ZT      = ZT_D7

end
