; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION tof_QEcut, w_in0, Qx=Qx, Qy=Qy, En=En, $
                    omegashift=omegashift, fold=fold
;+
;******* *********
;** User callable 
;** 
;**             JRS 26/10/09
;** 
;** -------------------------------------------------------------------------------
;** *******************************************************************************
;-
    
  COMMON c_lamp_access, inst
  
  time = SYSTIME(1)
  TAKE_DATP, datp
  
  iprint = 1  ; if iprint > 0, show debugging messages
  debug  = 0  ; if debug > 0,  show debugging plots
  
  IF N_ELEMENTS(omegashift) LE 0 THEN omegashift = 0.
  
  ; ------------------------------------------------------------------------------
  ;Set constants
  nchannels = (SIZE(w_in0))[1]
  ntheta    = (SIZE(w_in0))[2]
  nomega    = (SIZE(w_in0))[3]
  
  IF inst EQ 'IN4' OR inst EQ 'IN6' OR inst EQ 'IN5' THEN $
    lambda   = datp.p[21] ELSE lambda   = datp.p[4]
    
  const2 = 2.07194  ; hbar^2/2m
  ki = 2.*!pi/lambda
  
  IF iprint THEN BEGIN
    PRINT, 'nchannels = ', nchannels
    PRINT, 'ntheta    = ', ntheta
    PRINT, 'nomega    = ', nomega
    PRINT, 'lambda    = ', lambda
    PRINT, 'ki        = ', ki
  ENDIF
  
  IF iprint THEN PRINT,'tof_QEcut: End of "set constants" section'
  
  ; ------------------------------------------------------------------------------
  ;Set up arrays
  
  w_in      = w_in0
  e_in      = datp.e
  en_in     = datp.x
  theta_in  = datp.y
  omega_in  = datp.z
  twotheta  = theta_in*!pi/180.    ; detector angle (rad)
  omega     = omega_in - omegashift  ; sample rotation angle (degrees)
  
  ; ------------------------------------------------------------------------------
  ;find gaps in 2theta coverage
  
  totalgaps = 0
  find_gaps:
  i        = INDGEN(ntheta-1)
  diff     = twotheta[i+1] - twotheta[i]
  meandiff = MEAN(diff)
  devdiff  = MEANABSDEV(diff)
  gaps     = WHERE(diff GT meandiff + devdiff, numgaps)
  IF numgaps EQ 0 THEN GOTO, nomoregaps
  totalgaps= totalgaps + numgaps
  gap      = FLTARR(nchannels,1,nomega)
  gaperr   = gap - 1
  
  w_buf = w_in
  e_buf = e_in
  t_buf = twotheta
  
  FOR igap = 0, numgaps - 1 DO BEGIN
    gapang = (twotheta[gaps[igap]]+twotheta[gaps[igap]+1])/2.
    w_buf   = [[[w_buf[*,0:gaps[igap]+igap,*]],[gap],   [w_buf[*,gaps[igap]+igap+1:ntheta-1+igap,*]]]]
    e_buf   = [[[e_buf[*,0:gaps[igap]+igap,*]],[gaperr],[e_buf[*,gaps[igap]+igap+1:ntheta-1+igap,*]]]]
    t_buf   = [t_buf[0:gaps[igap]+igap] ,gapang, t_buf[gaps[igap]+igap+1:ntheta-1+igap]]
  ENDFOR
  
  twotheta = t_buf
  ntheta   = N_ELEMENTS(t_buf)
  w_in     = w_buf
  e_in     = e_buf
  
  GOTO, find_gaps
  nomoregaps:
  IF iprint THEN PRINT, 'tof_QEcut: Finished "gap finder" section:' + $
    ' total of ' + STRTRIM(STRING(totalgaps),2) + ' gaps inserted'
    
  ; ------------------------------------------------------------------------------
  ; truncate input arrays to appropriate energy grid
    
  CASE N_ELEMENTS(En) OF
    0: BEGIN
      Emax = MAX(en_in)
      Emin = MIN(en_in)
      dEn  = en_in[nchannels-1] - en_in[nchannels-2]
      nEn  = nchannels
    END
    1: BEGIN
      s  = 'tof_QEcut: specify En=[E,dE], or En=[Emin,dE,Emax]'
      jj = WIDGET_MESSAGE(s,/ERROR)
    END
    2: BEGIN
      Emax = En[0] + 0.5*En[1]
      Emin = En[0] - 0.5*En[1]
      dEn   = Emax - Emin
      nEn   = 1
      cuttype = 'constant E'
    END
    3: BEGIN
      Emax = En[2]
      Emin = En[0]
      dEn   = En[1]
      nEn   = FIX((Emax - Emin) / dEn)
    END
  ENDCASE
  ii = WHERE(en_in GE Emin AND en_in LE Emax, n)
  IF n LT 1 THEN BEGIN
    s  = 'tof_QEcut: no data in cut'
    jj = WIDGET_MESSAGE(s,/ERROR)
    RETURN, 0
  ENDIF ELSE BEGIN
    w_in = w_in[ii,*,*]
    e_in = e_in[ii,*,*]
    en_in = en_in[ii]
    nchannels = n
  ENDELSE
  
  IF iprint THEN PRINT,'tof_QEcut: End of "Energy grid" section'
  
  ; ------------------------------------------------------------------------------
  ;  Sort out Q-grid parameters
  CASE 1 OF
    (N_ELEMENTS(Qx) LT 2): BEGIN
      s  = 'tof_QEcut: must specify Qx=[Qx,dQx] or Qx=[Qxmin,dQx,Qxmax]'
      jj = WIDGET_MESSAGE(s,/ERROR)
      RETURN, 0
    END
    (N_ELEMENTS(Qx) EQ 2): BEGIN
      Qxmin = Qx[0] - 0.5*Qx[1]
      Qxmax = Qx[0] + 0.5*Qx[1]
      dQx   = Qxmax - Qxmin
      nQx = 1
      cuttype = 'constant Qx'
    END
    (N_ELEMENTS(Qx) EQ 3): BEGIN
      Qxmin = Qx[0]
      dQx   = Qx[1]
      Qxmax = Qx[2]
      nQx = ROUND((Qxmax - Qxmin) / dQx) +1
    END
  ENDCASE
  CASE 1 OF
    (N_ELEMENTS(Qy) LT 2): BEGIN
      s  = 'tof_QEcut: must specify Qy=[Qy,dQy] or Qy=[Qymin,dQy,Qymax]'
      jj = WIDGET_MESSAGE(s,/ERROR)
      RETURN, 0
    END
    (N_ELEMENTS(Qy) EQ 2): BEGIN
      Qymin = Qy[0] - 0.5*Qy[1]
      Qymax = Qy[0] + 0.5*Qy[1]
      dQy   = Qymax - Qymin
      nQy   = 1
      cuttype = 'constant Qy'
    END
    (N_ELEMENTS(Qy) EQ 3): BEGIN
      Qymin = Qy[0]
      dQy   = Qy[1]
      Qymax = Qy[2]
      nQy = ROUND((Qymax - Qymin) / dQy) + 1
    END
  ENDCASE
  
  IF N_ELEMENTS(cuttype) LE 0 THEN cuttype = '3-dimensional'
  
  IF iprint THEN BEGIN
    PRINT, 'Qxmin =',Qxmin
    PRINT, 'Qxmax =',Qxmax
    PRINT, 'Qymin =',Qymin
    PRINT, 'Qymax =',Qymax
    PRINT, 'Emin  =',Emin
    PRINT, 'Emax  =',Emax
    PRINT, 'dQx   =',dQx
    PRINT, 'dQy   =',dQy
    PRINT, 'dEn   =',dEn
    PRINT, 'nQx   =',nQx
    PRINT, 'nQy   =',nQy
    PRINT, 'nEn   =',nEn
    PRINT
    PRINT, 'Cut type = ', cuttype
    PRINT
  ENDIF
  
  IF iprint THEN PRINT,'tof_QEcut: End of "Q grid" section'
  
  ; ------------------------------------------------------------------------------
  ; calculate input E-Qx-Qy grid for relevent E-values
  
  Qx = FLTARR(nchannels, ntheta, nomega)
  Qy = FLTARR(nchannels, ntheta, nomega)
  
  FOR ien = 0, nchannels-1 DO BEGIN
    kf       = SQRT(ki^2 - en_in[ien]/const2)
    Qmag     = SQRT(ki^2 + kf^2 - 2*ki*kf*COS(twotheta))
    beta     = (twotheta/(ABS(twotheta)))*(180./!pi)*ACOS((ki^2 - kf^2 + Qmag^2)/(2*ki*Qmag))
    alpha = 180. + ((FLTARR(ntheta) + 1.) # TRANSPOSE(omega)) + (beta # TRANSPOSE(FLTARR(nomega) + 1.))
    Qx[ien,*,*] = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* COS(alpha * !pi / 180.)
    Qy[ien,*,*] = (Qmag # TRANSPOSE(FLTARR(nomega) + 1.))* SIN(alpha * !pi / 180.)
  ENDFOR
  
  
  w_out = FLTARR(nchannels, nQx, nQy)
  e_out = w_out
  n_out = w_out
  x_out = en_in
  IF nQx GT 1 THEN y_out = INDGEN(nQx)*dQx + Qxmin ELSE y_out=Qxmin+0.5*dQx
  IF nQy GT 1 THEN z_out = INDGEN(nQy)*dQy + Qymin ELSE z_out=Qymin+0.5*dQy
  
  IF iprint THEN PRINT,'tof_QEcut: End of "Calculate output arrays" section'
  
  ; ------------------------------------------------------------------------------
  ;Rebin data
  FOR ien = 0, nchannels-1 DO BEGIN
    FOR itheta=0,ntheta-1 DO BEGIN
      FOR iomega=0,nomega-1 DO BEGIN
        IF debug THEN PLOT, Qx[ien,*,*], Qy[ien,*,*], PSYM=4
        IF (Qx[ien,itheta,iomega] GT Qxmax+0.5*dQx) OR $
          (Qy[ien,itheta,iomega] GT Qymax+0.5*dQy) OR $
          (Qx[ien,itheta,iomega] LT Qxmin-0.5*dQx) OR $
          (Qy[ien,itheta,iomega] LT Qymin-0.5*dQy) THEN GOTO, next
        IF e_in[ien,itheta,iomega] LT 0 THEN GOTO, next
        ; -----------------------------------------------------------------------------
        ;compute Voronoi cell for Qx,Qy point
        
        IF itheta NE 0 THEN BEGIN
          dQx_theta_minus = 0.5 * (Qx[ien,itheta-1,iomega] - Qx[ien,itheta,iomega])
          dQy_theta_minus = 0.5 * (Qy[ien,itheta-1,iomega] - Qy[ien,itheta,iomega])
        ENDIF
        IF itheta NE ntheta - 1 THEN BEGIN
          dQx_theta_plus  = 0.5 * (Qx[ien,itheta,iomega]   - Qx[ien,itheta+1,iomega])
          dQy_theta_plus  = 0.5 * (Qy[ien,itheta,iomega]   - Qy[ien,itheta+1,iomega])
        ENDIF
        IF iomega NE 0 THEN BEGIN
          dQx_omega_minus = 0.5 * (Qx[ien,itheta,iomega-1] - Qx[ien,itheta,iomega])
          dQy_omega_minus = 0.5 * (Qy[ien,itheta,iomega-1] - Qy[ien,itheta,iomega])
        ENDIF
        IF iomega NE nomega - 1 THEN BEGIN
          dQx_omega_plus  = 0.5 * (Qx[ien,itheta,iomega]   - Qx[ien,itheta,iomega+1])
          dQy_omega_plus  = 0.5 * (Qy[ien,itheta,iomega]   - Qy[ien,itheta,iomega+1])
        ENDIF
        IF itheta EQ 0 THEN BEGIN
          dQx_theta_minus = dQx_theta_plus
          dQy_theta_minus = dQy_theta_plus
        ENDIF
        IF itheta EQ ntheta-1 THEN BEGIN
          dQx_theta_plus = dQx_theta_minus
          dQy_theta_plus = dQy_theta_minus
        ENDIF
        IF iomega EQ 0 THEN BEGIN
          dQx_omega_minus = dQx_omega_plus
          dQy_omega_minus = dQy_omega_plus
        ENDIF
        IF iomega EQ nomega-1 THEN BEGIN
          dQx_omega_plus = dQx_omega_minus
          dQy_omega_plus = dQy_omega_minus
        ENDIF
        centre = [Qx[ien,itheta,iomega],Qy[ien,itheta,iomega]]
        
        ;a-b-c-d is the input quadrilateral
        a = centre - [ dQx_theta_plus  + dQx_omega_plus,   dQy_theta_plus  + dQy_omega_plus]
        b = centre - [ dQx_theta_plus  - dQx_omega_minus,  dQy_theta_plus  - dQy_omega_minus]
        c = centre - [-dQx_theta_minus - dQx_omega_minus, -dQy_theta_minus - dQy_omega_minus]
        d = centre - [-dQx_theta_minus + dQx_omega_plus,  -dQy_theta_minus + dQy_omega_plus]
        point_area = shoelace([[a],[b],[c],[d]], 0)
        
        ; ---------------------------------------------------------------------------
        ; test output bins for intersection, and assign weighted intensities
        num = 0
        frac_attributed = 0
        offx = [0,0,1,1, 1, 0,-1,-1,-1,0,1,2,2,2, 2, 2, 1, 0,-1,-2,-2,-2,-2,-2,-1]
        offy = [0,1,1,0,-1,-1,-1, 0, 1,2,2,2,1,0,-1,-2,-2,-2,-2,-2,-1, 0, 1, 2, 2]
        
        iix = WHERE(y_out GT centre[0],nx)
        iiy = WHERE(z_out GT centre[1],ny)
        IF nx EQ 0 THEN iix = nQx-1
        IF ny EQ 0 THEN iiy = nQy-1
        
        find_area:
        xbin =   iix[0]-offx[num]
        ybin =  iiy[0]-offy[num]
        IF (xbin GT nQx-1) OR (xbin LT 0) OR $
          (ybin GT nQy-1) OR (ybin LT 0) THEN BEGIN
          num = num + 1
          IF (num EQ 25) THEN GOTO, next
          GOTO, find_area
        ENDIF
        b_centre = [y_out[xbin],z_out[ybin]]
        
        ;e-f-g-h is the bin box
        e = b_centre - [ 0.5*dQx, 0.5*dQy]
        f = b_centre - [ 0.5*dQx,-0.5*dQy]
        g = b_centre - [-0.5*dQx,-0.5*dQy]
        h = b_centre - [-0.5*dQx, 0.5*dQy]
        
        intersect_poly = intersection_points(a,b,c,d,e,f,g,h)
        intersect_area = shoelace(intersect_poly, debug)
        
        frac = intersect_area / point_area
        frac_attributed = frac_attributed + frac
        
        IF intersect_area GT 0 THEN BEGIN
          w_out[ien,xbin,ybin] = w_out[ien,xbin,ybin] +  frac*w_in[ien,itheta,iomega]
          e_out[ien,xbin,ybin] = e_out[ien,xbin,ybin] + (frac*e_in[ien,itheta,iomega])^2
          n_out[ien,xbin,ybin] = n_out[ien,xbin,ybin] + frac
        ENDIF
        
        IF debug THEN BEGIN
          OPLOT, [a[0],b[0],c[0],d[0],a[0]], [a[1],b[1],c[1],d[1],a[1]]
          OPLOT, [e[0],f[0],g[0],h[0],e[0]], [e[1],f[1],g[1],h[1],e[1]]
          PRINT, 'Box centre          = ', b_centre
          int_dims = (SIZE(intersect_poly))[0]
          IF int_dims GT 1 THEN BEGIN
            int_num = (SIZE(intersect_poly))[2]
            PRINT, 'Intersection points = ', intersect_poly[*,0]
            FOR ipoint = 1, int_num-1 DO BEGIN
              PRINT, '                      ', intersect_poly[*,ipoint]
            ENDFOR
          ENDIF
          PRINT, 'Intersection area   = ', intersect_area
          PRINT, 'Fractional area     = ', frac
          PRINT, 'Fraction of bin so far...', frac_attributed
          READ,s
        ENDIF
        
        ; test for total bin allocation
        IF frac_attributed LT 0.95 THEN BEGIN
          num = num + 1
          IF (num EQ 25) THEN GOTO, next
          GOTO, find_area
        ENDIF
        next:
      ENDFOR
    ENDFOR
    
    FOR ix=0, nQx-1 DO BEGIN
      FOR iy=0, nQy-1 DO BEGIN
        IF (n_out[ien,ix,iy] GT 0.) THEN BEGIN
          w_out[ien,ix,iy] = w_out[ien,ix,iy]/n_out[ien,ix,iy]
          e_out[ien,ix,iy] = SQRT(e_out[ien,ix,iy])/n_out[ien,ix,iy]
        ENDIF ELSE BEGIN
          w_out[ien,ix,iy] =  0.
          e_out[ien,ix,iy] = -1.
        ENDELSE
      ENDFOR
    ENDFOR
  ENDFOR
  
  IF (iprint GT 0) THEN PRINT,'tof_QEcut: End of "rebin data" section'
  
  ; ------------------------------------------------------------------------------
  ;Return parameters and exit
  
  CASE cuttype OF
    'constant Qx':  BEGIN
      x_axis = x_out
      y_axis = z_out
      x_tit  = 'Energy Transfer (meV)'
      y_tit  = 'Qy (inv. Angstrom)'
    END
    'constant Qy':  BEGIN
      x_axis = x_out
      y_axis = y_out
      x_tit  = 'Energy Transfer (meV)'
      y_tit  = 'Qx (inv. Angstrom)'
    END
    'constant E':  BEGIN
      n_out = 0.*w_out
      ii = WHERE(e_out GT 0,COMPLEMENT=jj)
      n_out[ii] = n_out[ii] + 1.
      IF N_ELEMENTS(jj) GT 0 THEN e_out[jj] = 0
      w_buf = TOTAL(w_out,1)
      e_buf = TOTAL(e_out^2,1)
      n_buf = TOTAL(n_out,1)
      w_out = w_buf
      e_out = e_buf
      ii = WHERE(n_buf GT 0, n, COMPLEMENT=jj)
      IF n GT 0 THEN BEGIN
        w_out[ii] = w_buf[ii] / n_buf[ii]
        e_out[ii] = SQRT(e_buf[ii]) / n_buf[ii]
      ENDIF
      IF N_ELEMENTS(jj) GT 0 THEN BEGIN
        w_out[jj] = 0
        e_out[jj] = -1
      ENDIF
      x_axis = y_out
      y_axis = z_out
      x_tit  = 'Qx (inv. Angstrom)'
      y_tit  = 'Qy (inv. Angstrom)'
    END
  ENDCASE
  
  
  
  MOD_DATP, datp, "x", x_axis
  MOD_DATP, datp, "y", y_axis
  MOD_DATP, datp, "e", e_out
  
  datp.x_tit=x_tit
  datp.y_tit=y_tit
  
  ;  d=STRING_ROUND(STRING(dQ))
  ;  s=STRING_ROUND(STRING(omegashift))
  ;  r=STRING_ROUND(STRING(dE))
  
  ;  PRINT,'Qrebin_v2: rebinned to Qx-Qy grid with dQ='+d+'A-1 and omega-shift = '+s+' degrees'
  ;  datp.other_tit=datp.other_tit+' -qr2(dQ='+d+',o='+s+',dE='+r+')'
  IF iprint THEN PRINT,'tof_QEcut: rebinning time..', SYSTIME(1) - time,'secs'
  
  finished:
  GIVE_DATP, datp
  
  RETURN, w_out
END

; ------------------------------------------------------------------------------
