;+
; Export of Fullprof data format INSTR=10
;
;                       X,Y,Sigma format with header lines.
;                       In all cases the first 6 lines are considered
;                       as comments.
;                       If in the first line (left ajusted) appears the
;                       keyword XYDATA, then the following 5 lines are
;                       considered as the heading of the file. Among
;                       these 5 lines the following keywords and values
;                       have a meaning to the program:
;                        -> INTER  fac_x  fac_y  Interpol  Stepin
;                        -> TEMP  tsamp
;                        fac_x  internal multiplier of X-values
;                        fac_y  internal multiplier of Y and Sigma-values
;                        Interpol=0 data are given at constant step
;                                   or variable step is used in the program
;                                =1 The variable step data are interpolated
;                                   internally to the constant step Stepin.
;
;                       If no sigma values are provided the program assumes
;                       that sigma(Y)=sqrt(Y).
;                       You can add comments to the data file if they
;                       start with the character ! in the first position
;                       of the line. These lines are ignored by the program.
;-

pro export_dat, FileNameInit , Data , XC=x, YC=y , ZC=z , E=e  , N=n   $
                               , PR=p, PV=pv, PAR_TXT=p_txt       $
                               , W_tit=wt  , X_tit=xt , Y_TIT=yt  $
                               , Z_tit=zt  , OTHER_TIT=ot $
                               , datp=datp $
                               , parameter=par,comment=comment $
                               , bad=bad, unix=unix, silence=noprint

;+
;** **********
;**
;**	Standard call for a data-write procedure called by LAMP.
;**
;**	Keywords:		  
;**		  XC       =  vector of x coordinates.
;**		  YC       = [vector of y coordinates.]
;**		  ZC       = [vector of z coordinates.]
;**		  W_TIT    =   main title
;**		  X_TIT    = x axis title
;**		  Y_TIT    = y axis title
;**		  Z_TIT    = z axis title
;**		  OTHER_TIT=    sub title
;**		  N        = monitors
;**		  PR       = vector of parameter values
;**		  PAR_TXT  = string array of text associated to PR (same size)
;**		  PV       = an array of any dimensions containing other parameter values
;**		  E        = the errors associated to DATA (same size) or 0
;-

if !version.release ge '5.0' then ii=execute('FORWARD_FUNCTION sys_dep')

CATCH,stat & IF stat ne 0 then begin & print,!error_state.msg & return & endif
ON_IOERROR,mis

;IF NOT KEYWORD_SET(nolamp) THEN BEGIN
;    XiCute,'@lamp.cbk'
;ENDIF

Filename=FilenameInit

IF KEYWORD_SET(x) AND NOT KEYWORD_SET(e) THEN e=SQRT(data)
IF NOT (KEYWORD_SET(x) AND KEYWORD_SET(e) AND KEYWORD_SET(pv)) THEN BEGIN
  IF NOT KEYWORD_SET(datp) THEN BEGIN
    IF N_ELEMENTS(FilenameInit) GT 1 THEN BEGIN
      IF N_PARAMS() EQ 1 THEN data='data'
      tmp=FilenameInit
      Filename=data
      data=tmp
      flag=1
    ENDIF 
    IF NOT KEYWORD_SET(p) THEN TAKE_DATP, datp 
  ENDIF ELSE IF N_ELEMENTS(data) EQ 1 THEN BEGIN
    tmp=FilenameInit
    Filename=data
    data=tmp
  ENDIF 
  IF KEYWORD_SET(DATP) THEN BEGIN
     x=datp.x
     n=datp.n
     y=datp.y
     z=datp.z
     e=datp.e
     p=datp.p
     par_txt=datp.par_txt
     pv=datp.pv
     p_txt=datp.par_txt
     wt=datp.w_tit
     xt=datp.x_tit
     yt=datp.y_tit
     zt=datp.z_tit
     ot=datp.other_tit
  ENDIF  
ENDIF 
if not keyword_set(noprint) then help,filename,data

if n_elements(e) ne n_elements(data) then e = sqrt(data)
subs=N_ELEMENTS(data[0,*])
newpos=0
flag=0
REPEAT BEGIN
  pos=newpos
  ii=execute("newpos=STRPOS(FILENAME,SYS_DEP('DIVIDER'),pos+1)")
  IF newpos GE 0 THEN flag=1
ENDREP UNTIL newpos LT 0
IF NOT flag THEN pos=-1

IF subs GT 1 THEN BEGIN
;  FILENAME=FILENAME+'_'
;  FILENAME=STRMID(FILENAME,pos+1,3)
  if not keyword_set(noprint) then PRINT,subs,' subsets'
  if n_elements(pv[*,0]) GE 29 then begin
	 IF (pv[29,subs-1]-pv[29,0]) EQ (subs-1) THEN numflag=1 ELSE numflag=0
  endif else numflag=0
ENDIF

FOR k=0,subs-1 DO BEGIN

  sub=''
  IF subs GT 1 THEN BEGIN
    digits=STRLEN(STRCOMPRESS(subs,/RE))
    sub=STRMID(STRCOMPRESS(10^digits+k,/RE),0,digits+1)
    IF numflag THEN sub=STRCOMPRESS(ROUND(pv[29,k]),/RE)
  ENDIF
  OPENW ,unit, STRCOMPRESS(FileName)+sub+'.dat', /get_lun
  if not keyword_set(noprint) then PRINT,STRCOMPRESS(FileName,/RE)+sub+'.dat'
  IF (!version.os eq 'darwin' and not keyword_set(unix)) then begin
    PRINTF,unit,format='(%"%s\r")','XYDATA '+strmid(STRCOMPRESS(wt) ,0,73)  
    PRINTF,unit,format='(%"%s %d %d %d %d %d\r")', 'INTER ',1,1,0,0.1
	IF n_elements(pv[*,0]) gt 30 then begin ; D20
	 	PRINTF,unit,format='(%"%s %f %f %f\r")',  'TEMP ',pv[[12,11,10],k]
	endif else begin ; D1B or D2B
		if n_elements(pv[*,0]) NE N_ELEMENTS(p) THEN BEGIN ; D2B
			PRINTF,unit,format='(%"%s %f %f %f\r")',  'TEMP ',p[[7,6,5]]
		endif else begin ; D1B
			PRINTF,unit,format='(%"%s %f %f %f\r")',  'TEMP ',pv[[7,6,5],k]
		endelse
	endelse
    PRINTF,unit,format='(%"%s\r")','! '+strmid(ot ,0,76)   
    PRINTF,unit,format='(%"%s\r")','! '+strmid(ot  ,0,76)   
    PRINTF,unit,format='(%"%s %f %f\r")','! Monitor/counts, Counting time/sec: ',n[0,0,0],n[0,1<(N_ELEMENTS(n[0,*,0])-1),0]
    sortidx=sort(x[*,k<(N_ELEMENTS(x[0,*])-1)])
    data[*,k]=data[sortidx,k]
    e[*,k]=e[sortidx,k]
    sortx=x[sortidx,k<(N_ELEMENTS(x[0,*])-1)]
    FOR i=0,N_ELEMENTS(data[*,k])-1 DO BEGIN
      IF x[i,k<(N_ELEMENTS(x[0,*])-1)] GE 0 THEN PRINTF,unit,FORMAT='(F9.4,F18.3,F12.3,%"\r")',(sortx[i])<(180.),(data[i,k])<(1.0E12),(e[i,k])<(1.0E6)
   ENDFOR
  endif else begin
    PRINTF,unit,'XYDATA '+strmid(STRCOMPRESS(wt) ,0,73)  
    PRINTF,unit,'INTER ',1,1,0,0.1

	IF n_elements(pv[*,0]) gt 30 then begin ; D20
    	PRINTF,unit,'TEMP ',pv[[12,11,10],k]
    	PRINTF,unit,'! '+strmid(ot ,0,76)
    	PRINTF,unit,'! '+strmid(ot  ,0,76)
    	PRINTF,unit,'! Monitor/counts, Counting time/sec: ',n[0,0,0],n[0,1<(N_ELEMENTS(n[0,*,0])-1),0]
	endif else begin ; D1B or D2B
		if n_elements(pv[*,0]) NE N_ELEMENTS(p) THEN BEGIN ; D2B
			PRINTF,unit,'TEMP ',p[[7,6,5]]
			PRINTF,unit,'! '+strmid(ot ,0,76)
			PRINTF,unit,'! '+strmid(ot  ,0,76)
			; Trying to handle the different types N can have here
			if (size(N))[0] EQ 0 then begin
			  PRINTF,unit,'! Monitor/counts, Counting time/sec: ',n
			endif else if (size(N))[0] EQ 1 then begin
			  PRINTF,unit,'! Monitor/counts, Counting time/sec: ',n[2*k], n[2*k+1]
			endif else begin
			  PRINTF,unit,'! Monitor/counts, Counting time/sec: ',n[0,0,0],n[0,1<(N_ELEMENTS(n[0,*,0])-1),0]
			endelse
		endif else begin ; D1B
			PRINTF,unit,'TEMP ',pv[[7,6,5],k]
			tmp = strmid(ot, 0, 37) + string(pv[29,k], format='(i6)') + strmid(ot, 43, 24)
			PRINTF,unit,'! '+tmp
			PRINTF,unit,'! '+tmp
			PRINTF,unit,'! Monitor/counts, Counting time/sec: ',n[2*k], n[2*k+1]
		endelse
	endelse
    sortidx=sort(x[*,k<(N_ELEMENTS(x[0,*])-1)])
    data[*,k]=data[sortidx,k]
    e[*,k]=e[sortidx,k]
    sortx=x[sortidx,k<(N_ELEMENTS(x[0,*])-1)]
    FOR i=0,N_ELEMENTS(data[*,k])-1 DO BEGIN
      IF x[i,k<(N_ELEMENTS(x[0,*])-1)] GE 0 THEN PRINTF,unit,FORMAT='(F9.4,F18.3,F12.3)',(sortx[i])<(180.),(data[i,k])<(1.0E12),(e[i,k])<(1.0E6)
   ENDFOR
  endelse

  FREE_LUN,unit

ENDFOR

return

mis:print,!error_state.msg
end
