pro cosmos_io_init

  forward_function cosmos_str_formtheta
  forward_function cosmos_str_make
  forward_function cosmos_get_default_settings
  forward_function cosmos_read_mft_parameter
  forward_function cosmos_get_mft_parameter
  
  variables_utilities
  
end


function cosmos_extract_galaxy_parameters, files, path

  common cosmos_params, c_params
   
  on_ioerror, EXTRACT_ERROR
  
  tempStr = ""
    
  params = replicate({numor : 0L, date : 0D, pos : 0, ang : 0D, angpos : -1}, n_elements(files))
          
  flag_defined = 0
      
  for f = 0, n_elements(files) - 1 do begin
  
    valid = 0
      
    file_name = string(files[f], FORMAT = "(I06)")
          
    file_name = path + strtrim(file_name,2)
      
    raw_lun=-1
    openr, raw_lun, file_name, ERROR = tempval, /GET_LUN, /compress

    ; The RRRRRR... line.
    for i = 1, 4 do readf, raw_lun, tempStr
    
    readf, raw_lun, tempStr
        
    temp = strsplit(strmid(tempStr,14), /EXTRACT)
         
    startDate = strtrim(strsplit(temp[0],"-",/EXTRACT),2)
    day = fix(startDate[0])
    month = 1 + where(c_params.months eq startDate[1])
    if (startDate[2] ge 70) and (startDate[2] le 99) then year = 1900 +  fix(startDate[2]) else year = 2000 + fix(startDate[2])
            
    startTime = strtrim(strsplit(temp[1],":",/EXTRACT),2)    
    hour   = fix(startTime[0])
    minute = fix(startTime[1])
    second = fix(startTime[2])
    
    for i = 1, 29 do readf, raw_lun, tempStr
            
    par1 = fltarr(128)
    
    readf, raw_lun, par1
    
    numor = files[f]
    date = julday(month, day, year, hour, minute, second)
    ang = double(par1[51])
    pos = fix(par1[52])
    
    valid = 1
    
    EXTRACT_ERROR:
    
      if valid eq 1 then begin
              
        params[f].numor = numor
        params[f].date  = date
        params[f].pos   = pos
        params[f].ang   = ang
        
      endif else begin
      
        cosmos_logmessage, "Could not extract GALAXY parameters from numor: " + files[f] + ". Skipped.", /WARNING 
      
      endelse
                                                                 
      if raw_lun ne -1 then free_lun, raw_lun
          
  endfor
    
  sortedAng = params[uniq(params.ang,sort(params.ang))].ang
    
  for i = 0, n_elements(params) - 1 do params[i].angpos = where(sortedAng eq params[i].ang)
          
  return, params
  
end


function cosmos_read_mft_parameter, numor, block, position

  common cosmos_settings, c_settings
   
  on_ioerror, bad_numor
      
  valid = 0
  
  file_name = string(numor, format = "(I06)")
  file_name = c_settings.pathdir + path_sep() + strtrim(file_name,2)

  raw_lun=-1
  
  openr, raw_lun, file_name, /get_lun, /compress

  tempstr = ''
          
  readf, raw_lun, tempstr ;RRRR line
  readf, raw_lun, tempstr ;Read detector format
  temp = strsplit(tempstr,' ',/extract)
  detector_format = temp[2]
  readf, raw_lun, tempstr ;AAAA line
  readf, raw_lun, tempstr ;number of ASCII characters in following line
  readf, raw_lun, tempstr ;Read instrument
  temp = strmid(tempstr,0,4)
  inst = strupcase(strtrim(temp,2))

  ; read in next lines of text until first block of parameters
  for i = 6, 34 do readf, raw_lun, tempstr
        
  ; read 1st set of parameters
  par1 = fltarr(128)
  readf, raw_lun, par1
  
  ; read 2 line spacer
  for i = 1, 2 do readf, raw_lun, tempstr
        
  ; read 2nd set of parameters
  par2 = fltarr(256)
 readf, raw_lun, par2


  if block eq 1 then begin
     value = par1[position]
     valid = 1
  endif else if block eq 2 then begin
     value = par2[position]
     valid = 1
  endif else begin
     print, 'Bad block number'
  endelse
              
  bad_numor:
      if raw_lun gt 0 then free_lun, raw_lun
      if valid then return, value else return, -1

end


function cosmos_get_mft_parameter, numor, name

   common cosmos_settings, c_settings

   ;Figaro output for Motofit
   if c_settings.mftinstr eq 0 then begin

     case name of 
   
      'Water bath set point temperature':            value = cosmos_read_mft_parameter(numor, 1, 27)
      'Water bath regulation temperature':           value = cosmos_read_mft_parameter(numor, 1, 28)
      'Water bath sample temperature':               value = cosmos_read_mft_parameter(numor, 1, 29)
      'HPLC pump valve position':                    value = cosmos_read_mft_parameter(numor, 1, 30)
      'Liquid height number of adjustments':         value = cosmos_read_mft_parameter(numor, 1, 32)
      'Liquid height at run start':                  value = cosmos_read_mft_parameter(numor, 1, 33)
      'Liquid height at run end':                    value = cosmos_read_mft_parameter(numor, 1, 34)
      'Langmuir requested speed':                    value = cosmos_read_mft_parameter(numor, 1, 40)
      'Langmuir requested pressure':                 value = cosmos_read_mft_parameter(numor, 1, 42)
      'Langmuir actual start pressure':              value = cosmos_read_mft_parameter(numor, 1, 44)
      'Langmuir actual end pressure':                value = cosmos_read_mft_parameter(numor, 1, 45)
      'Langmuir actual mean pressure':               value = cosmos_read_mft_parameter(numor, 1, 46)
      'Langmuir actual standard deviation pressure': value = cosmos_read_mft_parameter(numor, 1, 47)
      'Sample changer position number':              value = cosmos_read_mft_parameter(numor, 1, 49)
      'Absorption trough position number':           value = cosmos_read_mft_parameter(numor, 1, 52)
      'Absorption trough optical offset':            value = cosmos_read_mft_parameter(numor, 1, 53)
      'Electronic switch':                           value = cosmos_read_mft_parameter(numor, 1, 50)
      'Measuring time (s)':                          value = cosmos_read_mft_parameter(numor, 1, 2)
      'Total counts on detector':                    value = cosmos_read_mft_parameter(numor, 1, 3)
      'Total counts on monitor 1':                   value = cosmos_read_mft_parameter(numor, 1, 4)
      'Total counts on monitor 2':                   value = cosmos_read_mft_parameter(numor, 1, 5)
      'Collimation angle wrt horizontal':            value = cosmos_read_mft_parameter(numor, 1, 51)
      'Theta':                                       value = cosmos_read_mft_parameter(numor, 1, 51)
      'dan':                                         value = cosmos_read_mft_parameter(numor, 1, 14)
      'slit s1h':                                    value = cosmos_read_mft_parameter(numor, 2, 3)
      'slit s2h': begin
                                                     value1 = cosmos_read_mft_parameter(numor, 2, 7)
                                                     value2 = cosmos_read_mft_parameter(numor, 2, 8)
                                                     value = value1 + value2
                  end                                   
      'slit s2w': begin
                                                     value1 = cosmos_read_mft_parameter(numor, 2, 9)
                                                     value2 = cosmos_read_mft_parameter(numor, 2, 10)
                                                     value = value1 + value2
                  end
      'slit s3h': begin
                                                     value1 = cosmos_read_mft_parameter(numor, 2, 14)
                                                     value2 = cosmos_read_mft_parameter(numor, 2, 15)
                                                     value = value1 + value2
                  end
      'slit s3w': begin
                                                     value1 = cosmos_read_mft_parameter(numor, 2, 16)
                                                     value2 = cosmos_read_mft_parameter(numor, 2, 17)
                                                     value = value1 + value2
                  end
      'beam block s4t':                              value = cosmos_read_mft_parameter(numor, 2, 23)
      'beam block s4b':                              value = cosmos_read_mft_parameter(numor, 2, 24)
      'san':                                         value = cosmos_read_mft_parameter(numor, 2, 21)
      'phi':                                         value = cosmos_read_mft_parameter(numor, 2, 22)
      'sht1':                                        value = cosmos_read_mft_parameter(numor, 2, 18)
      'sht2':                                        value = cosmos_read_mft_parameter(numor, 2, 19)
      'str':                                         value = cosmos_read_mft_parameter(numor, 2, 20)
      'First controlling chopper':                   value = cosmos_read_mft_parameter(numor, 2, 200)
      'Second controlling chopper':                  value = cosmos_read_mft_parameter(numor, 2, 201)
      'First order dlambda/lambda':                  value = cosmos_read_mft_parameter(numor, 2, 202)
      'Frame overlap mirror':                        value = cosmos_read_mft_parameter(numor, 2, 203)
      'Poff':                                        value = cosmos_read_mft_parameter(numor, 2, 54)
      'Open offset':                                 value = cosmos_read_mft_parameter(numor, 2, 56)
      'Normalization':                               value = c_settings.normvar
      'Factor 1':                                    value = c_settings.factor1
      'Factor 2':                                    value = c_settings.factor2
      'Angle 1 theta overwrite':                     value = c_settings.takenum1
      'Angle 2 theta overwrite':                     value = c_settings.takenum2
      'Angle 3 theta overwrite':                     value = c_settings.takenum3
      'Angle 1 width':                               value = total(c_settings.foregdwidth1) + 1
      'Angle 2 width':                               value = total(c_settings.foregdwidth2) + 1
      'Angle 3 width':                               value = total( c_settings.foregdwidth3) + 1
      'Angle 1 left background width':               value = c_settings.backgdlowwidth1
      'Angle 2 left background width':               value = c_settings.backgdlowwidth2
      'Angle 3 left background width':               value = c_settings.backgdlowwidth3
      'Angle 1 left background offset':              value = c_settings.backgdlowoffset1
      'Angle 2 left background offset':              value = c_settings.backgdlowoffset2
      'Angle 3 left background offset':              value = c_settings.backgdlowoffset3
      'Angle 1 right background width':              value = c_settings.backgdhighwidth1
      'Angle 2 right background width':              value = c_settings.backgdhighwidth2
      'Angle 3 right background width':              value = c_settings.backgdhighwidth3
      'Angle 1 right background offset':             value = c_settings.backgdhighoffset1
      'Angle 2 right background offset':             value = c_settings.backgdhighoffset2
      'Angle 3 right background offset':             value = c_settings.backgdhighoffset3
      'Angle 1 grouping (1=yes;0=no)':               value = c_settings.group
      'Angle 2 grouping (1=yes;0=no)':               value = c_settings.group
      'Angle 3 grouping (1=yes;0=no)':               value = c_settings.group
      'Angle 1 grouping value':                      value = c_settings.groupvar1
      'Angle 2 grouping value':                      value = c_settings.groupvar2
      'Angle 3 grouping value':                      value = c_settings.groupvar3
      'Angle 1 fitted position of peak':             value = c_settings.peakref[0]
      'Angle 2 fitted position of peak':             value = c_settings.peakref[1]
      'Angle 3 fitted position of peak':             value = c_settings.peakref[2]

      else: begin
               print, 'Parameter name not found in the list'
               print, 'Name searched = ', name
            end   
   
     endcase
   
   endif else begin ;D17 output for Motofit --> c_settings.mftinstr = 1
   
     case name of

       'Temp 1':                         value = cosmos_read_mft_parameter(numor, 1, 119)
       'Temp 2':                         value = cosmos_read_mft_parameter(numor, 1, 120)
       'Temp 3':                         value = cosmos_read_mft_parameter(numor, 1, 121)
       'Temp 4':                         value = cosmos_read_mft_parameter(numor, 1, 122)
       'Temp 5':                         value = cosmos_read_mft_parameter(numor, 1, 123)
       'Temp 6':                         value = cosmos_read_mft_parameter(numor, 1, 124)
       'Sample T set point':             value = cosmos_read_mft_parameter(numor, 1, 27)
       'Regulation temperature':         value = cosmos_read_mft_parameter(numor, 1, 28)
       'str motor value (mm)':           value = cosmos_read_mft_parameter(numor, 2, 26)
       'Measuring time (s)':             value = cosmos_read_mft_parameter(numor, 1, 2)
       'Total counts on detector':       value = cosmos_read_mft_parameter(numor, 1, 3)
       'Total counts on monitor':        value = cosmos_read_mft_parameter(numor, 1, 4)
       'Sample angle':                   value = cosmos_read_mft_parameter(numor, 2, 2)
       'S2 width':                       value = cosmos_read_mft_parameter(numor, 2, 8) + cosmos_read_mft_parameter(numor, 2, 9)
       'S3 width':                       value = cosmos_read_mft_parameter(numor, 2, 12) + cosmos_read_mft_parameter(numor, 2, 13)
       'Chopper1 - Sample distance (m)': value = cosmos_read_mft_parameter(numor, 1, 55)
       'S2 - Sample distance (m)':       value = cosmos_read_mft_parameter(numor, 1, 58)/1000.0
       'S3 - Sample distance (m)':       value = cosmos_read_mft_parameter(numor, 1, 59)/1000.0
       'Sample - Detector distance (m)': value = cosmos_read_mft_parameter(numor, 1, 14)/1000.0
       'Chopper 1 actual speed':         value = cosmos_read_mft_parameter(numor, 2, 44)
       'Chopper 1 opening requested':    value = 45.0 - (cosmos_read_mft_parameter(numor, 2, 43) - cosmos_read_mft_parameter(numor, 2, 41)) - cosmos_read_mft_parameter(numor, 1, 56)
       'Interchopper distance': begin
                                         value = cosmos_read_mft_parameter(numor, 2, 14)/100.0
                                         if (value gt 0.5) or (value le 0.01) then value = 0.08111 ;interchopper distance
                                end
       'Poff':                            value = cosmos_read_mft_parameter(numor, 1, 54)
       'Open offset':                     value = cosmos_read_mft_parameter(numor, 1, 56)
       'Normalization':                   value = c_settings.normvar
       'Factor 1':                        value = c_settings.factor1
       'Factor 2':                        value = c_settings.factor2
       'Angle 1 theta overwrite':         value = c_settings.takenum1
       'Angle 2 theta overwrite':         value = c_settings.takenum2
       'Angle 3 theta overwrite':         value = c_settings.takenum3
       'Angle 1 width':                   value = total(c_settings.foregdwidth1) + 1
       'Angle 2 width':                   value = total(c_settings.foregdwidth2) + 1
       'Angle 3 width':                   value = total( c_settings.foregdwidth3) + 1
       'Angle 1 left background width':   value = c_settings.backgdlowwidth1
       'Angle 2 left background width':   value = c_settings.backgdlowwidth2
       'Angle 3 left background width':   value = c_settings.backgdlowwidth3
       'Angle 1 left background offset':  value = c_settings.backgdlowoffset1
       'Angle 2 left background offset':  value = c_settings.backgdlowoffset2
       'Angle 3 left background offset':  value = c_settings.backgdlowoffset3
       'Angle 1 right background width':  value = c_settings.backgdhighwidth1
       'Angle 2 right background width':  value = c_settings.backgdhighwidth2
       'Angle 3 right background width':  value = c_settings.backgdhighwidth3
       'Angle 1 right background offset': value = c_settings.backgdhighoffset1
       'Angle 2 right background offset': value = c_settings.backgdhighoffset2
       'Angle 3 right background offset': value = c_settings.backgdhighoffset3
       'Angle 1 grouping (1=yes;0=no)':   value = c_settings.group
       'Angle 2 grouping (1=yes;0=no)':   value = c_settings.group
       'Angle 3 grouping (1=yes;0=no)':   value = c_settings.group
       'Angle 1 grouping value':          value = c_settings.groupvar1
       'Angle 2 grouping value':          value = c_settings.groupvar2
       'Angle 3 grouping value':          value = c_settings.groupvar3
       'Angle 1 fitted position of peak': value = c_settings.peakref[0]
       'Angle 2 fitted position of peak': value = c_settings.peakref[1]
       'Angle 3 fitted position of peak': value = c_settings.peakref[2]
      
       else: begin
         print, 'Parameter name not found in the list'
         print, 'Name searched = ', name
       end
      
     endcase 
    
   endelse   

   return, value

end

; write to file
pro cosmos_file_write, outfile, result, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, aft = keyaft, dat = keydat, mft = keymft, out = keyout, raw = keyraw, lam=keylam
  
  common cosmos_params, c_params
  
  common cosmos_settings, c_settings
        
  valid = 0
  
  on_ioerror, EXIT
  
  dirname = file_dirname(outfile)
  
  info = file_test(dirname, /directory)
  if not info then file_mkdir, dirname
      
  outname = strtrim(file_basename(outfile),2)
    
  if outname eq '' then begin
  
    s = cosmos_str_make(reflect1[0])
  
    if strlen(s) gt 1 then outname = cosmos_str_make(s) else outname = 'cosmosrow' + cosmos_str_make(c_params.row+1)
    
  endif
    
  if keyword_set(keyaft) then filename = outfile + '.aft'
  if keyword_set(keydat) then filename = outfile + '.dat'
  if keyword_set(keymft) then filename = outfile + '.mft'
  if keyword_set(keyraw) then filename = outfile + '.raw'
  if keyword_set(keyout) then filename = outfile + '.out'
  if keyword_set(keylam) then filename = outfile + '.lam'
    
  write_lun=-1
    
  openw, write_lun, filename, error = temp1, /get_lun
  if temp1 eq 0 then begin
      
    cosmos_logmessage, ' Writing to file ' + filename
        
    if keyword_set(keyout) then begin
    
      printf, write_lun, ' Direct beam 1: ', direct1
      printf, write_lun, ' Reflect beam 1: ', reflect1 + ' (Calc. method = ' + c_settings.method1 + ')'
      printf, write_lun, ' Instrument background 1: ', instr1
      printf, write_lun, ' Theta 1: ', cosmos_str_formtheta(theta1[0])
      printf, write_lun, ' Direct beam 2: ', direct2
      printf, write_lun, ' Reflect beam 2: ', reflect2 + ' (Calc. method = ' + c_settings.method2 + ')'
      printf, write_lun, ' Instrument background 2: ', instr2
      printf, write_lun, ' Theta 2: ', cosmos_str_formtheta(theta2[0])
      printf, write_lun, ' Direct beam 3: ', direct3
      printf, write_lun, ' Reflect beam 3: ', reflect3 + ' (Calc. method = ' + c_settings.method3 + ')'
      printf, write_lun, ' Instrument background 3: ', instr3
      printf, write_lun, ' Theta 3: ', cosmos_str_formtheta(theta3[0])
      printf, write_lun, ' Water: ', water_file
      printf, write_lun, ' Foreground 1: ', c_settings.foregdwidth1[0],' + peak_pixel + ',c_settings.foregdwidth1[1]
      printf, write_lun, ' Foreground 2: ', c_settings.foregdwidth2[0],' + peak_pixel + ',c_settings.foregdwidth2[1]
      printf, write_lun, ' Foreground 3: ', c_settings.foregdwidth3[0],' + peak_pixel + ',c_settings.foregdwidth3[1]
      printf, write_lun, ' Background Low 1 - Offset : ', c_settings.backgdlowoffset1
      printf, write_lun, ' Background Low 1 - Width  : ', c_settings.backgdlowwidth1
      printf, write_lun, ' Background Low 2 - Offset : ', c_settings.backgdlowoffset2
      printf, write_lun, ' Background Low 2 - Width  : ', c_settings.backgdlowwidth2
      printf, write_lun, ' Background Low 3 - Offset : ', c_settings.backgdlowoffset3
      printf, write_lun, ' Background Low 3 - Width  : ', c_settings.backgdlowwidth3
      printf, write_lun, ' Background High 1 - Offset : ', c_settings.backgdhighoffset1
      printf, write_lun, ' Background High 1 - Width  : ', c_settings.backgdhighwidth1
      printf, write_lun, ' Background High 2 - Offset : ', c_settings.backgdhighoffset2
      printf, write_lun, ' Background High 2 - Width  : ', c_settings.backgdhighwidth2
      printf, write_lun, ' Background High 3 - Offset : ', c_settings.backgdhighoffset3
      printf, write_lun, ' Background High 3 - Width  : ', c_settings.backgdhighwidth3
      printf, write_lun, ' Lambda Range 1: ', c_settings.lambdamin1, '-', c_settings.lambdamax1
      printf, write_lun, ' Lambda Range 2: ', c_settings.lambdamin2, '-', c_settings.lambdamax2
      printf, write_lun, ' Lambda Range 3: ', c_settings.lambdamin3, '-', c_settings.lambdamax3
      printf, write_lun, ' Useful Pixel Range 1: ', c_settings.usefulpixelmin1, '-', c_settings.usefulpixelmax1 
      printf, write_lun, ' Useful Pixel Range 2: ', c_settings.usefulpixelmin2, '-', c_settings.usefulpixelmax2 
      printf, write_lun, ' Useful Pixel Range 3: ', c_settings.usefulpixelmin3, '-', c_settings.usefulpixelmax3 
      printf, write_lun, ' Search Pixel Range 1: ', c_settings.searchpixelmin1, '-', c_settings.searchpixelmax1 
      printf, write_lun, ' Search Pixel Range 2: ', c_settings.searchpixelmin2, '-', c_settings.searchpixelmax2 
      printf, write_lun, ' Search Pixel Range 3: ', c_settings.searchpixelmin3, '-', c_settings.searchpixelmax3 
      printf, write_lun, ' Search Time Range 1: ', c_settings.searchtimemin1, '-', c_settings.searchtimemax1 
      printf, write_lun, ' Search Time Range 2: ', c_settings.searchtimemin2, '-', c_settings.searchtimemax2 
      printf, write_lun, ' Search Time Range 3: ', c_settings.searchtimemin3, '-', c_settings.searchtimemax3 
      printf, write_lun, '' ; spacer
      printf, write_lun, string(9b) + 'q' + string(9b) + '   refl' + string([9b, 9b]) + 'refl err' + string(9b) + 'q res (FWHM)'
      
      for i = 0, n_elements(result[0, *]) - 1 do begin
      
        printf, write_lun, result[0, i], string(32b), result[2, i], string(32b), result[3, i], string(32b), result[1, i]
          
      endfor
        
    endif else if keyword_set(keyaft) then begin
    
      printf, write_lun, 'AFIT'
      tmp = cosmos_str_make(reflect1[0])
      
      if strlen(tmp) le 0 or tmp eq '0' then tmp = cosmos_str_make(reflect2[0])
      
      if strlen(tmp) gt 0 then printf, write_lun, 'Results for reflect beam ' + tmp else printf, write_lun, 'Cosmos Results'
      
      printf, write_lun, format = '(f4.2)', 2.07
      printf, write_lun, format = '(i0)', n_elements(result[0, *])
      
      for i = 0, n_elements(result[0, *]) - 1 do begin
      
        printf, write_lun, format = '(g11.4,10x,g11.4,10x,g11.4,a2)', result[0, i], result[2, i], result[3, i]
        
      endfor
        
    endif else if keyword_set(keydat) then begin
    
      parrattdata = where(result[2,*] gt 0.0, parrattcount)
      
      if parrattcount gt 0 then begin
      
        for i = 0, n_elements(result[0, parrattdata]) - 1 do begin
      
          printf, write_lun, result[0, parrattdata[i]], string(9b), $
          result[2, parrattdata[i]], string(9b), result[3, parrattdata[i]]
          
        endfor
        
      endif else begin
      
        print, "No positive reflectivities found!"
        
      endelse

    endif else if keyword_set(keymft) then begin

      printf, write_lun, 'MFT'
      printf, write_lun, 'Instrument: '+strtrim(c_params.inst,2)
      printf, write_lun, 'User-local contact: '+strjoin(strtrim([c_params.user,c_params.local_contact],2),'-')
      printf, write_lun, 'Title: '+strtrim(c_params.title,2)
      printf, write_lun, 'Subtitle: '+strtrim(c_params.subtitle,2)
      printf, write_lun, 'Start date + time: '+strtrim(c_params.start_time,2)
      printf, write_lun, 'End date + time: '+strtrim(c_params.end_time,2)
      
      printf, write_lun, 'Theta 1 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_1,format='(F5.2)'),strjoin(direct1,' '),strjoin(reflect1,' ')], '; ') + $
        ' (Calc. method = ' + c_settings.method1 + ')'
      printf, write_lun, 'Theta 2 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_2,format='(F5.2)'),strjoin(direct2,' '),strjoin(reflect2,' ')], '; ') + $
        ' (Calc. method = ' + c_settings.method2 + ')'
      printf, write_lun, 'Theta 3 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_3,format='(F5.2)'),strjoin(direct3,' '),strjoin(reflect3,' ')], '; ') + $
        ' (Calc. method = ' + c_settings.method3 + ')'

      if reflect1[0] eq 0 then reflect1 = ''
      if reflect2[0] eq 0 then reflect2 = ''
      if reflect3[0] eq 0 then reflect3 = ''
      arr = [reflect1[0],reflect2[0],reflect3[0]]
      numor = where(strlen(arr) ne 0)
      numor = arr[numor[0]]

      if c_settings.mftinstr eq 0 then mftparams = c_settings.mftparamsfig else mftparams = c_settings.mftparamsd17
      
      for i = 0, n_elements(mftparams)-1 do begin
               
          pname = 'Param '+strtrim(string(i+1),2)
 
          if strlen(mftparams[i]) gt 0 then begin
             
             name = mftparams[i]
             value = cosmos_get_mft_parameter(numor, name)
             
          endif else begin
          
             name = 'Not defined'
             value = 0.
          
          endelse
          
          printf, write_lun, pname+' : ' + string(value, format='(G12.6)') + ' ' + name
          ;printf, write_lun, 'Param 1 + value + name: 0(000): '+string(0.0, format='(G12.8)')+ ' To_be_defined'
      
      endfor
      
      printf, write_lun, 'Number of file format: '+strtrim(c_params.data_format,2)
      printf, write_lun, 'Number of data points: '+string(n_elements(result[0,*]))
      printf, write_lun, ''
      printf, write_lun, string(['q','refl','refl_err','q_res (FWHM)'],format='(4A20)')
      
      for i = 0, n_elements(result[0,*])-1 do printf, write_lun, string(result[[0,2,3,1],i],format='(4(5X,G15.6))')
    
    endif else if keyword_set(keylam) then begin
      
      printf, write_lun, 'LAM'
      printf, write_lun, 'Instrument: '+strtrim(c_params.inst,2)
      printf, write_lun, 'User-local contact: '+strjoin(strtrim([c_params.user,c_params.local_contact],2),'-')
      printf, write_lun, 'Title: '+strtrim(c_params.title,2)
      printf, write_lun, 'Subtitle: '+strtrim(c_params.subtitle,2)
      printf, write_lun, 'Start date + time: '+strtrim(c_params.start_time,2)
      printf, write_lun, 'End date + time: '+strtrim(c_params.end_time,2)

      printf, write_lun, 'Theta 1 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_1,format='(F5.2)'),strjoin(direct1,' '),strjoin(reflect1,' ')], '; ') + $
      ' (Calc. method = ' + c_settings.method1 + ')'
      printf, write_lun, 'Theta 2 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_2,format='(F5.2)'),strjoin(direct2,' '),strjoin(reflect2,' ')], '; ') + $
      ' (Calc. method = ' + c_settings.method2 + ')'
      printf, write_lun, 'Theta 3 + dir + ref numbers: ' + $
        strjoin([string(c_params.angle_bragg_3,format='(F5.2)'),strjoin(direct3,' '),strjoin(reflect3,' ')], '; ') + $
      ' (Calc. method = ' + c_settings.method3 + ')'
      printf, write_lun, 'Water: ', water_file
      printf, write_lun, '' 
      printf, write_lun, format = '(11x,"Q",13x,"Ref(Q)",10x,"Error",9x,"dQ (FWHM)",6x,"Wavelength")'
      for i = 0, n_elements(result[0,*])-1 do printf, write_lun, string(result[[0,2,3,1,4],i],format='(5(1X,G15.6))')
     
    endif else begin
    
      for i = 0, n_elements(result[0, *]) - 1 do begin
      
        printf, write_lun, result[0, i], string(9b), result[2, i], string(9b), result[3, i], string(9b), result[1, i]
        
      endfor
      
    endelse
    
  endif
  
  valid = 1
  
  EXIT:
  
    if not valid then cosmos_logmessage, "Could not write the output file.", /WARNING
   
    if write_lun gt 0 then free_lun, write_lun
  
  return
  
end

; write to lamp
pro cosmos_lamp_write, workspace, array, workspacetitle, saveall = saveall
  
  valid = 0
  
  on_ioerror, EXIT
  @lamp.cbk

  if n_elements(saveall) eq 0 then saveall = 0

  ;check if variables can be used (otherwise we get an error if cosmos was launched without lamp)
  if n_elements(x_tit) gt 0 and n_elements(w_tit) gt 0 then begin

    write_x = reform(array[0, *])
    write_w = reform(array[2, *])
    write_e = reform(array[3, *])
    write_wmin = min(write_w)
    write_wmax = max(write_w)
  
    ; Save all data together into Lamp workspace w61
    if saveall then begin
      nmax = 3000 ;number of points always < 3 angles x 1000 TOF channels
      if total(w61) eq 0.0 then begin
        x61[0] = write_x & w61[0] = write_w & e61[0] = write_e
      endif else begin
        ;Add new vector in 2nd dimension
        x61 = [[x61], [fltarr(nmax,1)]] & w61 = [[w61], [fltarr(nmax,1)]] & e61 = [[e61], [fltarr(nmax,1)]]
        s = size(w61) & i = s[2]-1
        x61[0,i] = write_x & w61[0,i] = write_w & e61[0,i] = write_e
      endelse
    endif
  
    if (workspace le 0) or (workspace gt 60) then return
    
    wrksp = cosmos_str_make(workspace)
    temp = execute('x' + wrksp + ' = write_x')
    temp = execute('y' + wrksp + ' = 0')
    temp = execute('z' + wrksp + ' = 0')
    temp = execute('w' + wrksp + ' = write_w')
    temp = execute('w_max[' + wrksp + '] = write_wmax')
    temp = execute('w_min[' + wrksp + '] = write_wmin')
    temp = execute('e' + wrksp + ' = write_e')
    temp = execute('n' + wrksp + ' = 0')
    temp = execute('p' + wrksp + ' = 0')
    temp = execute('par' + wrksp + ' = 0')
    temp = execute('pv' + wrksp + ' = 0')
    temp = execute('w_tit[' + wrksp + '] = "Cosmos' + workspacetitle[0] + '"')
    temp = execute('x_tit[' + wrksp + '] = "q"')
    temp = execute('y_tit[' + wrksp + '] = ""')
    temp = execute('z_tit[' + wrksp + '] = ""')
    temp = execute('other_tit[' + wrksp + '] = ""')
    
    valid = 1
    
    cosmos_logmessage, 'Results written to LAMP workspace w' + wrksp + '.'

  endif 
        
  EXIT:
  
    if not valid then cosmos_logmessage, "Could not export the results in workspace", /WARNING
        
end

;+
; :Description:
;    Initialize Lamp workspace w61 to store the results of the full table
;
; :Params:
;
; :Returns:
;
; :Author: M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, 25 March 2015
;
; :Categories: cosmos
;
;-
pro cosmos_lamp_init

  @lamp.cbk
  
  ;check if variables can be used (otherwise we get an error if cosmos was launched without lamp)
  if n_elements(x_tit) gt 0 and n_elements(w_tit) gt 0 then begin
    ;Max number of Q points will be always less than 3 angles x 1000 TOF channels
    nmax = 3000
    x61 = fltarr(nmax) & w61 = fltarr(nmax) & e61 = fltarr(nmax)
    x61 = reform(x61,nmax,1) & w61 = reform(w61,nmax,1) & e61 = reform(e61,nmax,1)
    x_tit[61] = "q" & w_tit[61] = "Cosmos full table results"
  endif

end

;+
; :Description:
;    Remove zeros from Lamp workspace w61 at end of analysis
;
; :Params:
;
; :Returns:
;
; :Author: M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, 25 March 2015
;
; :Categories: cosmos
;
;-
pro cosmos_lamp_end

   @lamp.cbk

   ;check if variables can be used (otherwise we get an error if cosmos was launched without lamp)
   if n_elements(x_tit) gt 0 and n_elements(w_tit) gt 0 then begin
   
     s = size(x61) 
     first_zero = 0
     for i = 0, s[2]-1 do begin
         tmp = where(x61[*,i] eq 0.0, count)
         if count ne 0 then if tmp[0] gt first_zero then first_zero = tmp[0]
     endfor
     x61 = x61[0:first_zero-1,*]
     w61 = w61[0:first_zero-1,*]
     e61 = e61[0:first_zero-1,*]
     
     ;Now replace remaining zeroes by repeating last point
     for i = 0, s[2]-1 do begin
       tmp = where(x61[*,i] eq 0.0, count)
       if count ne 0 then begin
          x61[tmp,i] = x61[tmp[0]-1,i]
          w61[tmp,i] = w61[tmp[0]-1,i]
          e61[tmp,i] = e61[tmp[0]-1,i]
       endif
     endfor
  
     ;Y axis
     y61 = indgen(s[2]) + 1
     y_tit[61] = "Row number in selection" 

   endif

end

pro cosmos_logmessage, mess, warning = warning

  ;forward_function cosmos_str_make
  
  common cosmos_params, c_params
  
  common cosmos_message, cosmos_info, cosmos_warn
             
  mess = strtrim(mess, 2)
    
  if keyword_set(warning) then begin   
  
    mess = ' Row ' + cosmos_str_make(c_params.row + 1) + ': ' + mess
        
    cosmos_warn = [cosmos_warn, mess] 
        
  endif else begin
      
    cosmos_info = [cosmos_info, mess]
                
  endelse
            
end

pro cosmos_save_settings, main, filename, settings, galaxyrun = galaxyrun

  if strlen(filename) eq 0 then return
  
  on_ioerror, write_error
  
  valid = 0

  openw, saveLun, filename, /get_lun

  printf, saveLun, "OPTIONS"
  printf, saveLun, ""
    
  tags = tag_names(settings)

  for i = 0, n_elements(tags) - 1 do begin
  
    if tags[i] eq 'TABLE' or tags[i] eq 'GTABLE' then continue
        
    printf, saveLun, strtrim(tags[i],2)
    
    ; special treatment for foreground to allow saving a single value if the input was a single value
    ; note however that the code works internally with a pair of values corresponding to the number of pixels to both sides of the peak
    if (tags[i] eq 'FOREGDWIDTH1') or (tags[i] eq 'FOREGDWIDTH2') or (tags[i] eq 'FOREGDWIDTH3') then begin
       wid = widget_info(main, find_by_uname = tags[i])
       widget_control, wid, get_value = val & val = val[0]
       val1 = 0 & val2 = 0
       reads, val+',0', val1, val2
       if val2 eq 0 then printf, saveLun, strtrim(string(val1),2) $
        else printf, saveLun, strtrim(string(val1),2), ' ; ', strtrim(string(val2),2)
       printf, saveLun, '' 
       continue
    endif

    if size(settings.(i), /n_dimensions) eq 0 then begin     
      printf, saveLun, strtrim(settings.(i),2)
    endif else begin
      arr = strjoin(strtrim(settings.(i),2), ' ; ')
      for j = 0, n_elements(arr) - 1 do printf, saveLun, arr[j]
    endelse
    
    printf, saveLun, ''
    
  endfor

  if keyword_set(galaxyrun) then begin

    printf, saveLun, 'GTABLE'
    arr = strjoin(strtrim(*settings.gtable,2), ' ; ')
    for j = 0, n_elements(arr) - 1 do printf, saveLun, arr[j]
  
  endif else begin

    printf, saveLun, 'TABLE'
    arr = strjoin(strtrim(*settings.table,2), ' ; ')
    for j = 0, n_elements(arr) - 1 do printf, saveLun, arr[j]
    
   endelse
       
  valid = 1
  
  write_error:
    if not valid then cosmos_logmessage, "Error when saving input file.", /warning
    if saveLun gt 0 then free_lun, saveLun

end

function cosmos_load_settings, filename, galaxyrun = galaxyrun

  forward_function cosmos_get_default_settings

  if strlen(filename) eq 0 then return, -1
  
  on_ioerror, read_error
  
  valid = 0
  
  settings = cosmos_get_default_settings()

  loadLun = -1
  openr, loadLun, filename, /get_lun, /compress
  
  tag = ""
  val = ""
  
  loadedStruct = {}
  
  while not eof(loadLun) do begin
  
    readf, loadLun, tag
    tag = strupcase(tag)
        
    if tag eq 'OPTIONS' then begin
      readf, loadLun, val
            
    endif else if tag eq 'TABLE' then begin
      repeat begin
        readf, loadLun, val
        temp = strtrim(strsplit(val, ';', /extract),2)
        if n_elements(table) eq 0 then table = [temp] else table = [table, temp]
      endrep until eof(loadLun)
      if n_elements(table) ne 0 then loadedStruct = create_struct(loadedStruct, strtrim(tag,2), table)

    endif else if tag eq 'GTABLE' then begin
      repeat begin
        readf, loadLun, val
        temp = strtrim(strsplit(val, ';', /extract),2)
        if n_elements(gtable) eq 0 then gtable = [temp] else gtable = [gtable, temp]
      endrep until eof(loadLun)      
      if not keyword_set(galaxyrun) then continue
      if n_elements(gtable) ne 0 then loadedStruct = create_struct(loadedStruct, strtrim(tag,2), gtable)
      
    endif else if tag eq 'FOREGDWIDTH1' or tag eq 'FOREGDWIDTH2' or tag eq 'FOREGDWIDTH3' then begin   
      
      readf, loadLun, val
      temp = strtrim(strsplit(val, ';', /extract),2)
      if n_elements(temp) eq 1 then val = val + ';' + '0' 
      loadedStruct = create_struct(loadedStruct, strtrim(tag,2), strtrim(strsplit(val,';',/extract),2))    
      readf, loadLun, val
    
    endif else begin
      readf, loadLun, val
      loadedStruct = create_struct(loadedStruct, strtrim(tag,2), strtrim(strsplit(val,';',/extract),2))    
      readf, loadLun, val
            
    endelse      
          
  endwhile
    
  loadedTags = tag_names(loadedStruct)
  
  if (where(loadedTags eq "FOREGD"))[0] ne -1 then begin

    if fix(loadedstruct.colthirdang) eq 0 then nAngles = 1 else nAngles = 2 
    loadedStruct = create_struct(loadedStruct, 'nangles', nAngles)

    loadedStruct = create_struct(loadedStruct, 'instbg', loadedstruct.colinstrbackgd)

    if loadedstruct.foregd ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'foregdwidth1', loadedstruct.foregd)
      loadedStruct = create_struct(loadedStruct, 'foregdwidth2', loadedstruct.foregd)
      loadedStruct = create_struct(loadedStruct, 'foregdwidth3', loadedstruct.foregd)
    endif

    vals = strtrim(strsplit(' '+loadedstruct.backgdleft+', ', ',', /extract),2)
    if vals[0] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'backgdlowwidth1', vals[0])
      loadedStruct = create_struct(loadedStruct, 'backgdlowwidth2', vals[0])
      loadedStruct = create_struct(loadedStruct, 'backgdlowwidth3', vals[0])
    endif
    if vals[1] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'backgdlowoffset1', vals[1])
      loadedStruct = create_struct(loadedStruct, 'backgdlowoffset2', vals[1])
      loadedStruct = create_struct(loadedStruct, 'backgdlowoffset3', vals[1])
    endif

    vals = strtrim(strsplit(' '+loadedstruct.backgdright+', ', ',', /extract),2)
    if vals[0] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'backgdhighwidth1', vals[0])
      loadedStruct = create_struct(loadedStruct, 'backgdhighwidth2', vals[0])
      loadedStruct = create_struct(loadedStruct, 'backgdhighwidth3', vals[0])
    endif
    if vals[1] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'backgdhighoffset1', vals[1])
      loadedStruct = create_struct(loadedStruct, 'backgdhighoffset2', vals[1])
      loadedStruct = create_struct(loadedStruct, 'backgdhighoffset3', vals[1])
    endif

    vals = strtrim(strsplit(' '+loadedstruct.lambda+', ', ',', /extract),2)
    if vals[0] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'lambdamin1', vals[0])
      loadedStruct = create_struct(loadedStruct, 'lambdamin2', vals[0])
      loadedStruct = create_struct(loadedStruct, 'lambdamin3', vals[0])
    endif
    if vals[1] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'lambdamax1', vals[1])
      loadedStruct = create_struct(loadedStruct, 'lambdamax2', vals[1])
      loadedStruct = create_struct(loadedStruct, 'lambdamax3', vals[1])
    endif

    vals = strtrim(strsplit(' '+loadedstruct.usefulx+', ', ',', /extract),2)
    if vals[0] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmin1', vals[0])
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmin2', vals[0])
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmin3', vals[0])
    endif
    if vals[1] ne "" then begin
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmax1', vals[1])
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmax2', vals[1])
      loadedStruct = create_struct(loadedStruct, 'usefulpixelmax3', vals[1])
    endif

    vals = strtrim(strsplit(' '+loadedstruct.searchx+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel1', 1)    
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel2', 1)    
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel3', 1)        
    endif else begin
      if vals[0] ne "" then begin   
        loadedStruct = create_struct(loadedStruct, 'searchpixelmin1', vals[0])
        loadedStruct = create_struct(loadedStruct, 'searchpixelmin2', vals[0])
        loadedStruct = create_struct(loadedStruct, 'searchpixelmin3', vals[0])
      endif
      if vals[1] ne "" then begin
        loadedStruct = create_struct(loadedStruct, 'searchpixelmax1', vals[1])
        loadedStruct = create_struct(loadedStruct, 'searchpixelmax2', vals[1])
        loadedStruct = create_struct(loadedStruct, 'searchpixelmax3', vals[1])
      endif     
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchy+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchtime1', 1)    
      loadedStruct = create_struct(loadedStruct, 'autosearchtime2', 1)    
      loadedStruct = create_struct(loadedStruct, 'autosearchtime3', 1)        
    endif else begin
      if vals[0] ne "" then begin    
        loadedStruct = create_struct(loadedStruct, 'searchtimemin1', vals[0])
        loadedStruct = create_struct(loadedStruct, 'searchtimemin2', vals[0])
        loadedStruct = create_struct(loadedStruct, 'searchtimemin3', vals[0])
      endif
      if vals[1] ne "" then begin
        loadedStruct = create_struct(loadedStruct, 'searchtimemax1', vals[1])
        loadedStruct = create_struct(loadedStruct, 'searchtimemax2', vals[1])
        loadedStruct = create_struct(loadedStruct, 'searchtimemax3', vals[1])
      endif     
    endelse
    
    loadedStruct = create_struct(loadedStruct, 'poff1', loadedstruct.poff)
    loadedStruct = create_struct(loadedStruct, 'poff2', loadedstruct.poff)
    loadedStruct = create_struct(loadedStruct, 'poff3', loadedstruct.poff)
    
    loadedStruct = create_struct(loadedStruct, 'd01', loadedstruct.d0)
    loadedStruct = create_struct(loadedStruct, 'd02', loadedstruct.d0)
    loadedStruct = create_struct(loadedStruct, 'd03', loadedstruct.d0)

    loadedStruct = create_struct(loadedStruct, 'openoff1', loadedstruct.openoff)
    loadedStruct = create_struct(loadedStruct, 'openoff2', loadedstruct.openoff)
    loadedStruct = create_struct(loadedStruct, 'openoff3', loadedstruct.openoff)

    loadedStruct = create_struct(loadedStruct, 'pixelwidth1', loadedstruct.pixelwidth)
    loadedStruct = create_struct(loadedStruct, 'pixelwidth2', loadedstruct.pixelwidth)
    loadedStruct = create_struct(loadedStruct, 'pixelwidth3', loadedstruct.pixelwidth)
  
  endif else if (where(loadedTags eq "FOREGD1"))[0] ne -1 then begin
  
    if fix(loadedstruct.colthirdang) eq 0 then nAngles = 1 else nAngles = 2 
    loadedStruct = create_struct(loadedStruct, 'nangles', nAngles)

    loadedStruct = create_struct(loadedStruct, 'instbg', loadedstruct.colinstrbackgd)
            
    if loadedstruct.foregd1 ne "" then loadedStruct = create_struct(loadedStruct, 'foregdwidth1', loadedstruct.foregd1)
    if loadedstruct.foregd2 ne "" then loadedStruct = create_struct(loadedStruct, 'foregdwidth2', loadedstruct.foregd2)
    if loadedstruct.foregd3 ne "" then loadedStruct = create_struct(loadedStruct, 'foregdwidth3', loadedstruct.foregd3)
      
    vals = strtrim(strsplit(' '+loadedstruct.backgdleft1+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowwidth1', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowoffset1', vals[1])
    
    vals = strtrim(strsplit(' '+loadedstruct.backgdleft2+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowwidth2', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowoffset2', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.backgdleft3+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowwidth3', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdlowoffset3', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.backgdright1+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighwidth1', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighoffset1', vals[1])
    
    vals = strtrim(strsplit(' '+loadedstruct.backgdright2+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighwidth2', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighoffset2', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.backgdright3+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighwidth3', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'backgdhighoffset3', vals[1])
    
    vals = strtrim(strsplit(' '+loadedstruct.lambda1+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamin1', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamax1', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.lambda2+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamin2', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamax2', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.lambda3+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamin3', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'lambdamax3', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.usefulx1+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmin1', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmax1', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.usefulx2+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmin2', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmax2', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.usefulx3+', ', ',', /extract),2)
    if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmin3', vals[0])
    if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'usefulpixelmax3', vals[1])

    vals = strtrim(strsplit(' '+loadedstruct.searchx1+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel1', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmin1', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmax1', vals[1])
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchx2+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel2', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmin2', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmax2', vals[1])
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchx3+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchpixel3', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmin3', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchpixelmax3', vals[1])
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchy1+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchtime1', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemin1', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemax1', vals[1])
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchy2+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchtime2', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemin2', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemax2', vals[1])
    endelse

    vals = strtrim(strsplit(' '+loadedstruct.searchy3+', ', ',', /extract),2)
    if strupcase(vals[0]) eq 'AUTO' then begin
      loadedStruct = create_struct(loadedStruct, 'autosearchtime3', 1)    
    endif else begin    
      if vals[0] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemin3', vals[0])    
      if vals[1] ne "" then loadedStruct = create_struct(loadedStruct, 'searchtimemax3', vals[1])
    endelse
  
  endif

  loadedTags = tag_names(loadedStruct)
      
  tagNames = tag_names(settings)
    
  for i = 0, n_elements(loadedTags) - 1 do begin
    
    if loadedTags[i] eq 'TABLE' or loadedTags[i] eq 'GTABLE' then continue
      
    idx = (where(tagNames eq loadedTags[i]))[0]
        
    if idx eq -1 then continue
        
    vals = strtrim(loadedStruct.(i),2)
        
    case size(settings.(idx), /type) of
      2: settings.(idx) = fix(vals) 
      3: settings.(idx) = long(vals)
      4: settings.(idx) = float(vals)
      5: settings.(idx) = float(vals)
      else: settings.(idx) = vals
    endcase
      
  endfor
    
  if n_elements(table) ne 0 then begin

    case settings.nangles of  
      0: if settings.instbg then nCols = 7 else  nCols = 6    
      1: if settings.instbg then nCols = 12 else nCols = 10    
      2: if settings.instbg then nCols = 17 else nCols = 14    
    endcase
    
    nRows = n_elements(table)/nCols > 1
    
    table = table[0:nCols*nRows-1]
            
    *settings.table = reform(table, nCols, nRows)        
  endif      
                               
  if keyword_set(galaxyrun) then begin
    if n_elements(gtable) ne 0 then begin
    
      nRows = n_elements(gtable)/4 > 1                
      gtable = gtable[0:4*nRows-1]
      *settings.gtable = reform(gtable, 4, nRows)
    
    endif
  
  endif  
                               
  valid = 1

  read_error:
    if not valid then begin
      cosmos_logmessage, "Error when loading input file.", /warning
      return, -1
    endif
    
    if loadLun gt 0 then free_lun, loadLun
            
    return, settings

end

function cosmos_set_output_format, nCols

  case nCols of

    4:  fmt = '(A2,";",2(A10,";"),A)'

    6: fmt = '(5(A-10,";"),A-)'

    7: fmt = '(8(A-10,";"),A-)'

    10: fmt = '(9(A-10,";"),A-)'
    
    12: fmt = '(11(A-10,";"),A-)'
    
    14: fmt = '(13(A-10,";"),A-)'
    
    17: fmt = '(16(A-10,";"),A-)'
  
    else: fmt = "(A-)"
  
  endcase
  
  return, fmt

end

pro cosmos_io

  return

end
