pro cosmos_str_init

  forward_function cosmos_str_make
  forward_function cosmos_str_split

end

function cosmos_parse_numor_string, str

  valid = 0
  
  sel = [-1L]

  on_ioerror, EOR
  
  substr = strcompress(strsplit(str, ";", /EXTRACT),/REMOVE_ALL)
  
  match = long(stregex(substr,"([0-9]+):([0-9]+):([0-9]+)", /EXTRACT, /SUBEXPR))
        
  s = size(match)
  
  if s[0] eq 1 then idxmax = 1 else idxmax = s[2]
          
  for i = 0, idxmax - 1 do begin
  
    if array_equal(match[1:*,i], 0) then continue
    
    first = match[1,i]
    last = match[2,i]
    step = match[3,i]
    
    if step le 0 then continue
            
    sel = [sel, first + step*lindgen(1+(last-first)/step)]
                  
  endfor
  
  valid = 1 

  EOR:
  
    if valid eq 0 then return, -1
    
    if n_elements(sel) gt 1 then begin
    
      temp = sel[1:*]
      temp = temp[sort(temp)]
      return, temp[uniq(temp)]
      
    endif else begin
    
      return, -1
      
    endelse
              
end


; outputs string corresponding to theta value
function cosmos_str_formtheta, angle

  ;forward_function cosmos_str_make

  case angle of
  
    -1 : str = 'san'
    
    -2 : str = 'dan'
    
    -3 : str = 'det'
    
  else : str = cosmos_str_make(angle)
  
endcase

return, str

end

; convert number to string as printed
function cosmos_str_make, value

  return, strcompress(string(value, /PRINT), /REMOVE_ALL)
  
end

; parse input string, returning array
function cosmos_str_parse, original, float = f, integer = i,  flt_range = fr, int_range = ir, string_set = s

  ;forward_function cosmos_str_split
    
  substrs = cosmos_str_split(original)
  
  if keyword_set(f) then begin
  
    cosmos_str_value, substrs[n_elements(substrs) - 1], remainder, value, /FLOATING
    array = [ value ]
    return, array
    
  endif
  
  if keyword_set(i) then begin
  
    cosmos_str_value, substrs[n_elements(substrs) - 1], remainder, value, /INTEGER
    array = [ value ]
    return, array
    
  endif
  
  if keyword_set(fr) or keyword_set(ir) then begin
  
    if n_elements(substrs) eq 1 then begin
    
      if keyword_set(ir) then begin
      
        cosmos_str_value, substrs[0], remainder, value, /integer
        array = [ value, -1L ]
        return, array
        
      endif else begin
      
        cosmos_str_value, substrs[0], remainder, value, /floating
        array = [ value, -1.]
        return, array
        
      endelse
      
    endif else begin
    
      if keyword_set(ir) then begin
      
        cosmos_str_value, substrs[n_elements(substrs) - 2], remainder, int1, /integer
        cosmos_str_value, substrs[n_elements(substrs) - 1], remainder, int2, /integer
        array = [ int1, int2 ]
        return, array
        
      endif else begin
      
        cosmos_str_value, substrs[n_elements(substrs) - 2], remainder, flt1, /floating
        cosmos_str_value, substrs[n_elements(substrs) - 1], remainder, flt2, /floating
        array = [ flt1, flt2 ]
        return, array
        
      endelse
      
    endelse
    
  endif
  
  ; default or string_set
  array = [ substrs[0] ]
  curstr = 1
  cosmos_str_value, substrs[0], rempre, intpre, /INTEGER
  
  while curstr lt n_elements(substrs) do begin
  
    cosmos_str_value, substrs[curstr], remcur, intcur, /INTEGER
    remcur = strmid(remcur, 1, strlen(remcur) - 1)
    
    if (strmid(substrs[curstr], 0, 1) eq ':') and (rempre eq remcur) then begin
    
      if intcur gt intpre then begin
      
        for i = intpre + 1, intcur do array = [ array, strcompress(remcur + string(i), /remove_all) ]
        
      endif
      
    endif else array = [ array, strmid(substrs[curstr], 1, strlen(substrs[curstr]) - 1) ]
    
    curstr = curstr + 1
    rempre = remcur
    intpre = intcur
    
  endwhile
  
  return, array
  
end

; parse 'theta' string, returning float between 0 and 360 or -1 for san (default), -2 for dan
function cosmos_str_theta, string

  case strlowcase(string) of
  
    '' : value = -1.
    'san' : value = -1.
    'dan' : value = -2.
    'det' : value = -3.
    else : value = ((cosmos_str_parse(string, /float) mod 360) + 360.) mod 360 ; ensure 0 < value < 360
    
  endcase

  return, value

end

; separate string into a 1D array of component strings
; (separator is ',' or '+', keeping the separator at the front of each string)
function cosmos_str_split, original

  w = where((byte(original) eq (byte(','))[0]) or (byte(original) eq (byte('+'))[0]), count)
  arr = strarr(count + 1)
  spos = strlen(original)
  
  for i = 0, count - 1 do begin
  
    arr[count - i] = strmid(original, w[count - i - 1], spos - w[count - i - 1])
    spos = w[count - i - 1]
    
  endfor
  
  arr[0] = strmid(original, 0, spos)
  
  return, arr
  
end

; convert string to string plus number (floating or integer)
pro cosmos_str_value, original, remainder, value, floating = f, integer = i

  pos = strlen(original) - 1
  
  on_ioerror, cosmos_str_value_exit
  
  while pos ge 0 do begin
  
    if keyword_set(i) then begin
    
      value = long(strmid(original, pos, strlen(original) - pos))
      
    endif else begin
    
      value = float(strmid(original, pos, strlen(original) - pos))      
      
    endelse
    
    pos = pos - 1
    
  endwhile
  
  cosmos_str_value_exit:
  
    if keyword_set(i) then begin
    
      value = long(strmid(original, pos + 1, strlen(original) - pos - 1))
    
    endif else begin
  
      value = float(strmid(original, pos + 1, strlen(original) - pos - 1))
    
    endelse
  
    remainder = strmid(original, 0, pos + 1)
  
  return
  
end

pro cosmos_str

  return

end