pro cosmos_widgets_init

  forward_function build_path
  forward_function label_frame
  forward_function cosmos_anal_factor
  forward_function cosmos_anal_log
  forward_function cosmos_get_settings  
  forward_function cosmos_get_table_labels
  forward_function cosmos_str_split
  forward_function cosmos_getnumors
  forward_function cosmos_get_machine_params
  forward_function str_centerize
  
  paths_utilities
  strings_utilities

end

pro cosmos_display_doc, event

  common cosmos_doc, cosmos_doc
  
  uname = widget_info(event.id, /uname)
  
  if (strlen(uname) eq 0) then return
  
  r = execute(strjoin(['val=','cosmos_doc.',uname]))
  
  if r ne 1 then return
 
  widget_control, widget_info(event.top, find_by_uname = 'STATUSLINE'), set_value=val

end

pro cosmos_init_statusbar, event

  draw =  widget_info(event.top, find_by_uname = 'STATUSDRAW')
  loadct, 26
  widget_control, draw, get_value = wid        
  wset, wid
  device, decomposed = 0
  g = widget_info(draw, /geometry)
  xBoxCoords = [0, 0, g.scr_xsize, g.scr_xsize, 0]
  yBoxCoords = [0, g.scr_ysize, g.scr_ysize, 0, 0]          
  polyfill, xBoxCoords, yBoxCoords, COLOR = 0, /device
  loadct, 27
  
end

pro cosmos_update_statusbar, event

  common cosmos_params, c_params

  ; Load the color table 26.        
  loadct, 26

  ; Get the statusbar widget id.
  draw =  widget_info(event.top, find_by_uname = 'STATUSDRAW')

  ; Get the IDL window corresponding to the status bar.
  widget_control, draw, get_value = wid        

  wset, wid

  device, decomposed = 0

  ; Get the current dimension of the status bar.
  g = widget_info(draw, /geometry)

  ; Case where the run has not started. No need to update anything.
  if (c_params.rowfinish eq 0) then begin

      polyfill, [0, 0, g.scr_xsize, g.scr_xsize, 0], [0, g.scr_ysize, g.scr_ysize, 0, 0], COLOR = 0, /device

  endif else begin
  
      ; Compute the ratio of calculation done up to now.
      percentDone = float(c_params.row-c_params.rowinit)/float(c_params.rowfinish-c_params.rowinit)
            
      ; Compute the extent of the status bar that will be colored.
      xExtent = fix(g.scr_xsize*percentDone)      
      xBoxCoords = [0, 0, xExtent, xExtent, 0]   
      yBoxCoords = [0, g.scr_ysize, g.scr_ysize, 0, 0]
  
      polyfill, xBoxCoords, yBoxCoords, COLOR = 165, /device

      if (c_params.row eq c_params.rowfinish) then begin
          wait, 0.5
          polyfill, xBoxCoords, yBoxCoords, COLOR = 60, /device  
      endif

  endelse

  loadct, 27

end

pro cosmos_resize, event

  common cosmos_params, c_params

  widget_control, event.top, get_uvalue = galaxyrun
    
  ; The dialog can not be smaller than 900 along x. 
  xSize = event.x > 900
  ; The dialog can not be smaller than 800 along y. 
  ySize = event.y > 800

  status_geom = widget_info(widget_info(event.top, find_by_uname = 'STATUSBASE'), /GEOMETRY)

  statusline_geom = widget_info(widget_info(event.top, find_by_uname = 'STATUSLINE'), /GEOMETRY)
  
  widget_control, widget_info(event.top, find_by_uname = 'MAIN'), update=0

  ; The main dialog is resized according to the new dimensions.  
  widget_control, widget_info(event.top, find_by_uname = 'MAIN'), scr_xsize = xsize, scr_ysize = ysize
    
  tabs = widget_info(event.top, find_by_uname = 'TABS')
  
  if !version.os_family eq 'Windows' then begin
      widget_control, tabs, scr_xsize = xsize - 25, scr_ysize = ysize - 105 - status_geom.scr_ysize  - statusline_geom.scr_ysize  
  endif else begin
      widget_control, tabs, scr_xsize = xsize - 5, scr_ysize = ysize - 55 - status_geom.scr_ysize - statusline_geom.scr_ysize  
  endelse

  ; The geometry of the tab dialog.
  tabs_geom = widget_info(tabs, /geometry)

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB')  , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  tab_geom = widget_info(widget_info(event.top, find_by_uname = 'DATATAB'), /geometry)

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_B1'), scr_xsize=tab_geom.scr_xsize-5
        
  temp = (widget_info(widget_info(event.top, find_by_uname='DATATAB_B11'), /geometry)).scr_xsize + $
         (widget_info(widget_info(event.top, find_by_uname='DATATAB_B12'), /geometry)).scr_xsize + $
         (widget_info(widget_info(event.top, find_by_uname='DATATAB_B13'), /geometry)).scr_xsize

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_L11'), scr_xsize=(tab_geom.scr_xsize-20-temp)/2 >0 
  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_L12'), scr_xsize=(tab_geom.scr_xsize-20-temp)/2 >0

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_B2'), scr_xsize=tab_geom.scr_xsize-5

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_B3'), scr_xsize=tab_geom.scr_xsize-5
        
  temp = (widget_info(widget_info(event.top, find_by_uname='DATATAB_B31'), /geometry)).scr_xsize + $
         (widget_info(widget_info(event.top, find_by_uname='DATATAB_B32'), /geometry)).scr_xsize
        
  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_L31'), scr_xsize=(tab_geom.scr_xsize-10-temp) >0

  widget_control, widget_info(event.top, find_by_uname = 'DATATAB_B4'), scr_xsize=tab_geom.scr_xsize-5
        
  table = widget_info(event.top, find_by_uname = 'TABLE')
          
  numcols = n_elements(widget_info(table, /column_widths))
  temp = tab_geom.scr_xsize/(numcols+2)
  colwidth = make_array(numcols, /int, value = temp)
;  widget_control, table, scr_xsize = tab_geom.scr_xsize-5, scr_ysize = tab_geom.scr_ysize-160, column_widths = colwidth
;  widget_control, table, scr_xsize = tab_geom.scr_xsize-5, scr_ysize = tab_geom.scr_ysize-160, column_widths = colwidth
  widget_control, table, scr_xsize = tab_geom.scr_xsize-20, scr_ysize = tab_geom.scr_ysize-180, column_widths = colwidth
  
 
  if galaxyrun then begin  
    widget_control, table, scr_xsize = tab_geom.scr_xsize - 30, scr_ysize = tab_geom.scr_ysize - 40, column_widths = colwidth

    table = widget_info(event.top, find_by_uname = 'GTABLE')
    numcols = n_elements(widget_info(table, /column_widths))
    colwidth = make_array(numcols, /int, value = tab_geom.scr_xsize/(numcols+1))
    widget_control, table, scr_xsize = tab_geom.scr_xsize - 30, scr_ysize = tab_geom.scr_ysize - 305, column_widths = colwidth  

  endif
  
  widget_control, widget_info(event.top, find_by_uname = 'CALCTAB')  , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  widget_control, widget_info(event.top, find_by_uname = 'NORMTAB')  , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  widget_control, widget_info(event.top, find_by_uname = 'DETECTTAB'), scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  widget_control, widget_info(event.top, find_by_uname = 'MACHTAB')  , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  ; Resizing of the IO tab.
  widget_control, widget_info(event.top, find_by_uname = 'IOTAB')    , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35
  
  path_dir = widget_info(event.top, find_by_uname = 'PATHDIR')
  widget_control, widget_info(path_dir, /child), get_uvalue = temp
  widget_control, temp.TextId, scr_xsize = tab_geom.scr_xsize - 205

  output_dir = widget_info(event.top, find_by_uname = 'OUTPUTDIR')
  widget_control, widget_info(output_dir, /child), get_uvalue = temp
  widget_control, temp.TextId, scr_xsize = tab_geom.scr_xsize - 205  

  widget_control, widget_info(event.top, find_by_uname = 'LOGTAB')   , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35

  ; Resizing of the Log tab.
  widget_control, widget_info(event.top, find_by_uname = 'LOGINFO'), scr_xsize = tab_geom.scr_xsize - 10, scr_ysize = tab_geom.scr_ysize - 240
  widget_control, widget_info(event.top, find_by_uname = 'LOGWARN'), scr_xsize = tab_geom.scr_xsize - 10, scr_ysize = 195

  widget_control, widget_info(event.top, find_by_uname = 'PLOTTAB')  , scr_xsize = tabs_geom.scr_xsize - 15, scr_ysize = tabs_geom.scr_ysize - 35
  
  ; Resizing of the Plot tab.
  widget_control, widget_info(event.top, find_by_uname = 'PLOTWIN')    , scr_xsize = tab_geom.scr_xsize - 10, scr_ysize = tab_geom.scr_ysize - 190
  widget_control, widget_info(event.top, find_by_uname = 'PLOTLIST')   , scr_xsize = tab_geom.scr_xsize - 10, scr_ysize = 145
  widget_control, widget_info(event.top, find_by_uname = 'PLOTACTIONS'), scr_xsize = tab_geom.scr_xsize - 10

  widget_control, widget_info(event.top, find_by_uname = 'STATUSBASE') , scr_xsize = tabs_geom.scr_xsize, scr_ysize = 40
  widget_control, widget_info(event.top, find_by_uname = 'STATUSDRAW') , scr_xsize = tabs_geom.scr_xsize - 140

  widget_control, widget_info(event.top, find_by_uname = 'STATUSLINE') , scr_xsize = tabs_geom.scr_xsize, scr_ysize = 20
    
  cosmos_update_statusbar, event

  widget_control, widget_info(event.top, find_by_uname = 'MAIN'), update=1
    
end

pro cosmos_change_number_of_angles, main

  nAngles = widget_info(main, find_by_uname = 'NANGLES')
  widget_control, nAngles, get_value = val
    
  calcAng2 = widget_info(main, find_by_uname = 'CALCANG2')
  calcAng3 = widget_info(main, find_by_uname = 'CALCANG3')

;  detectAng2 = widget_info(main, find_by_uname = 'DETECTANG2')
;  detectAng3 = widget_info(main, find_by_uname = 'DETECTANG3')

  machAng2 = widget_info(main, find_by_uname = 'MACHANG2')
  machAng3 = widget_info(main, find_by_uname = 'MACHANG3')
  
;  widget_control, calcAng2,   sensitive = val ge 1
;  widget_control, detectAng2, sensitive = val ge 1
;  widget_control, machAng2,   sensitive = val ge 1
  
;  widget_control, calcAng3,   sensitive = val ge 2
;  widget_control, detectAng3, sensitive = val ge 2
;  widget_control, machAng3,   sensitive = val ge 2
             
end

pro cosmos_data_insert_rows, event
  
  widget_control, widget_info(event.top, find_by_uname = 'INSROW'), get_value = val
      
  if val eq 0 then return
  
  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'
      
  wTable = widget_info(event.top, find_by_uname = uname)
  widget_control, wTable, get_value = table    
  sel = widget_info(wTable, /table_select)
  
  ins = sel[3]
      
  if ins + val lt 0 then val = -ins
      
  nCols = n_elements(table[*,0])
  
  nRowsOld = n_elements(table[0,*])  
  nRows = nRowsOld + val  
  
  nTable = strarr(nCols, nRows)
  
  if val gt 0 then begin

    nTable[*,0:ins] = table[*,0:ins]
    if ins lt nRowsOld-1 then nTable[*,ins+val+1:*] = table[*,ins+1:*]
      
  endif else begin
      
    if ins+val gt 0 then nTable[*,0:ins+val-1] = table[*,0:ins+val-1]
    nTable[*,ins+val:*]   = table[*,ins:*]       
      
  endelse
  
  rowLabels = strtrim((indgen(nRows) + 1),2)
  
  widget_control, wTable, row_labels = rowLabels, ysize = nRows, alignment = 2, /editable, set_value = nTable
  
end

pro cosmos_data_delete_rows, event

  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)
  widget_control, wTable, get_value = table
    
  sel = widget_info(wTable, /table_select)
  
  nRowsOld = n_elements(table[0,*])
  
  if sel[3] - sel[1] + 1 eq nRowsOld then return
    
  nRows = nRowsOld - (sel[3] - sel[1] + 1)  
  
  nCols = n_elements(table[*,0])
      
  nTable = strarr(nCols, nRows)
    
  if sel[1] gt 0 then nTable[*,0:sel[1]-1] = table[*,0:sel[1]-1]
  if sel[3] lt nRowsOld - 1 then nTable[*,sel[1]:*] = table[*,sel[3]+1:*]
    
  rowLabels = strtrim((indgen(nRows) + 1),2)
  
  widget_control, wTable, set_table_select = [-1,-1,-1,-1], row_labels = rowLabels, ysize = nRows, $
                  alignment = 2, /editable, set_value = nTable
      
end

pro cosmos_data_increment, event

  ;forward_function cosmos_get_table_labels  
  ;forward_function cosmos_str_make  
  ;forward_function cosmos_str_split
  
  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)      
  widget_control, wTable, get_value = table
  sel = widget_info(wTable, /table_select)
  
  if (sel[3] - sel[1]) le 0 then return
    
  nCols = n_elements(table[*,0])
  
  tLabels = cosmos_get_table_labels(nCols)
  
  widget_control, widget_info(event.top, find_by_uname = 'INCRVAL'), get_value = val
    
  for c = sel[0], sel[2] do begin
  
    cName = strupcase(tLabels[c])
    
    if (strpos(cName, 'THETA') ne -1) or (strpos(cName, 'FACTOR') ne -1) then begin
    
      incr = float(val)
      
    endif else begin
    
      incr = long(val)
      
    endelse
    
    str = strcompress(table[c, sel[1]], /remove_all)
    split = cosmos_str_split(str)
    
    ;Brute way to solve the problem with the filling when , or +
    if n_elements(split) ge 2 then concatenate_operand = strmid(split[1],0,1)
    
    for r = sel[1] + 1, sel[3] do begin
        
      table[c, r] = ''
      
      if float(incr) eq 0. then begin
      
        table[c, r] = str
        
      endif else begin
      
        for i = 0, n_elements(split) - 1 do begin
        
          if (strpos(cName, 'THETA') ne -1) or (strpos(cName, 'FACTOR') ne -1 or (strpos(cName, 'NORMALISE')) ne -1) then begin
          
            cosmos_str_value, split[i], remainder, value, /floating
            
          endif else begin
            
            cosmos_str_value, split[i], remainder, value, /integer
            
          endelse
          
          str = cosmos_str_make(value + (incr * (r - sel[1])))
          
          ; keep zeros in name of output file
          if cName eq 'OUT FILE' then begin
            n_length = strlen(split[i]) - strlen(remainder)
            n_zeros = n_length - strlen(str) 
            for i_zeros = 0, n_zeros-1 do str = '0' + str
          endif
          
          if (i gt 0) then begin
             if concatenate_operand eq ',' then str = string(str, format='(I)') else str = string(str, format='(I+)')
          endif
                              
          table[c, r] = table[c, r] + remainder + str
          
        endfor
        
      endelse

      ;print, table[c,r]
      
      table[c, r] = strcompress(table[c, r], /remove_all)
      
    endfor
    
  endfor
  
  widget_control, wTable, set_value = table
  
end

pro cosmos_data_update_columns, event

  ;forward_function cosmos_get_table_labels
  
  widget_control, widget_info(event.top, find_by_uname = 'NANGLES'), get_value = nAngles
  widget_control, widget_info(event.top, find_by_uname = 'INSTBG'), get_value = instbg
      
  cosmos_change_number_of_angles, event.top
    
  wTable = widget_info(event.top, find_by_uname = 'TABLE')  
  widget_control, wTable, get_value = table      
  nCols = n_elements(table[*,0])
      
  ;determine actual number of angles in table
  case 1 of 
     (nCols eq 6) or (nCols eq 7)  : nUsedAngles = 0
     (nCols eq 10) or (nCols eq 12): nUsedAngles = 1
     (nCols eq 14) or (nCols eq 17): nUsedAngles = 2
  endcase    
  
  ;Raise warning if table size is reduced
  if nAngles lt nUsedAngles then begin
    answer = dialog_message('The information contained in the columns corresponding to angles > ' + $
      strtrim(string(nAngles+1),2) + ' will be lost! Continue?', $
      /cancel, /default_cancel, /center, dialog_parent = event.top, title = 'Warning')
    if strupcase(answer) eq 'CANCEL' then begin
       widget_control, widget_info(event.top, find_by_uname = 'NANGLES'), set_value = nUsedAngles
       return
    endif
  endif

  ;determine new number of columns corresponding to the selected number of angles
  case nAngles of
    0: if instbg then nColsNew = 7 else nColsNew = 6
    1: if instbg then nColsNew = 12 else nColsNew = 10
    2: if instbg then nColsNew = 17 else nColsNew = 14
  endcase
  
  tLabels = cosmos_get_table_labels(nColsNew)
    
  nRows = n_elements(table[0,*])
  
  newTable = strarr(nColsNew, nRows)
  
  case nColsNew of
  
    6: begin
    
      case nCols of
      
        7 : newTable   = table[[0,1,3,4,5,6],*]
        10: newTable   = table[[0,2,4,6,8,9],*]
        12: newTable   = table[[0,2,6,8,10,11],*]
        14: newTable   = table[[0,3,6,9,12,13],*]
        17: newTable   = table[[0,3,9,12,15,16],*]
        else: newTable = table
        
      endcase
      
    end
    
    7: begin
    
      case nCols of
      
        6 : newTable[[0,1,3,4,5,6],*] = table
        10: newTable[[0,1,3,4,5,6],*] = table[[0,2,4,6,8,9],*]
        12: newTable                  = table[[0,2,4,6,8,10,11],*]
        14: newTable[[0,1,3,4,5,6],*] = table[[0,3,6,9,12,13],*]
        17: newTable                  = table[[0,3,6,9,12,15,16],*]
        else: newTable                = table
        
      endcase
      
    end
    
    10: begin
    
      case nCols of
      
        6 : newTable[[0,2,4,6,8,9],*] = table
        7 : newTable[[0,2,4,6,8,9],*] = table[[0,1,3,4,5,6],*]
        12: newTable                  = table[[0,1,2,3,6,7,8,9,10,11],*]
        14: newTable                  = table[[0,1,3,4,6,7,9,10,12,13],*]
        17: newTable                  = table[[0,1,3,4,9,10,12,13,15,16],*]
        else: newTable                = table
        
      endcase
      
    end
    
    12: begin
    
      case nCols of
      
        6 : newTable[[0,2,6,8,10,11],*]             = table
        7 : newTable[[0,2,4,6,8,10,11],*]           = table
        10: newTable[[0,1,2,3,6,7,8,9,10,11],*]     = table
        14: newTable[[0,1,2,3,6,7,8,9,10,11],*]     = table[[0,1,3,4,6,7,9,10,12,13],*]
        17: newTable[[0,1,2,3,4,5,6,7,8,9,10,11],*] = table[[0,1,3,4,6,7,9,10,12,13,15,16],*]
        else: newTable                              = table
        
      endcase
      
    end
    
    14: begin
    
      case nCols of
      
        6 : newTable[[0,3,6,9,12,13],*]                   = table
        7 : newTable[[0,3,6,9,12,13],*]                   = table[[0,1,3,4,5,6],*]
        10: newTable[[0,1,3,4,6,7,9,10,12,13],*]          = table
        12: newTable[[0,1,3,4,6,7,9,10,12,13],*]          = table[[0,1,2,3,6,7,8,9,10,11],*]
        17: newTable[[0,1,2,3,4,5,6,7,8,9,10,11,12,13],*] = table[[0,1,2,3,4,5,9,10,11,12,13,14,15,16],*]
        else: newTable                                    = table
        
      endcase
      
    end
    
    17: begin
    
      case nCols of
      
        6 : newTable[[0,3,9,12,15,16],*]                     = table
        7 : newTable[[0,3,6,9,12,15,16],*]                   = table
        10: newTable[[0,1,3,4,9,10,12,13,15,16],*]           = table
        12: newTable[[0,1,3,4,6,7,9,10,12,13,15,16],*]       = table[[0,1,2,3,4,5,6,7,8,9,10,11],*]
        14: newTable[[0,1,2,3,4,5,9,10,11,12,13,14,15,16],*] = table[[0,1,2,3,4,5,6,7,8,9,10,11,12,13],*]
        else: newTable                                       = table
        
      endcase
            
    end

    else: newTable = table
    
  end
  
  widget_control, wTable, column_labels = tLabels, table_xsize = nColsNew, set_value = newTable
  
end

pro cosmos_data_clear, event

  if strmid(widget_info(event.id, /uname), 0, 8) eq 'CLEARALL' then all = 1 else all = 0

  if (all) then begin
    answer = dialog_message('This will delete all the data stored in your table.' + string(10b) + 'Continue?', $
                            /cancel, /default_cancel, /center, dialog_parent = event.top, title = 'Warning')
    if strupcase(answer) eq 'CANCEL' then return
  endif

  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)
  widget_control, wTable, get_value = table
  
  if all then begin
  
    table[*,*] = ""
    
  endif else begin
  
    sel = widget_info(wTable, /table_select)
    
    table[sel[0]:sel[2],sel[1]:sel[3]] = ""
    
  endelse
  
  widget_control, wTable, set_value = table
  
end

pro cosmos_data_set, event

  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)
    
  sel = widget_info(wTable, /table_select)
  
  widget_control, wTable, get_value = table

  widget_control, widget_info(event.top, find_by_uname = 'SETVAL'), get_value = val
  
  nCols = n_elements(table[*,0])
  tLabels = cosmos_get_table_labels(nCols)
    
  for c = sel[0], sel[2] do begin
  
    cName = strupcase(tLabels[c])
    
    if stregex(cName,"^(THETA|FACTOR)", /fold_case) ne -1 then begin
    
      val = float(val)
      
    endif else begin
    
      val = val
      
    endelse
    
    table[c, sel[1]:sel[3]] = strtrim(val,2)
    
  endfor
  
  widget_control, wTable, set_value = table
  
end

pro cosmos_data_selectall, event

  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)      
  widget_control, wTable, get_value = table  
  s = size(table)  
  widget_control, wTable, set_table_select = [0,0,s[1]-1,s[2]-1]
  
end

pro cosmos_data_copy, event

  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'

  wTable = widget_info(event.top, find_by_uname = uname)
  
  paste = widget_info(event.top, find_by_uname = 'PASTE')      
    
  sel = widget_info(wTable, /table_select)
  
  if sel[0] eq -1 then begin

    widget_control, paste, set_uvalue = -1
  
  endif else begin

    widget_control, wTable, get_value = table    
    widget_control, paste, set_uvalue = table[sel[0]:sel[2],sel[1]:sel[3]]
    
  endelse

end

pro cosmos_data_paste, event
  
  widget_control, widget_info(event.top, find_by_uname = 'PASTE'), get_uvalue = val
      
  s = size(val)
  
  if s[0] eq 0 then return
  
  nPastedCols = s[1]
  if s[0] eq 1 then nPastedRows = 1 else nPastedRows = s[2]
  
  widget_control, event.top, get_uvalue = galaxyrun
  if galaxyrun then uname = 'GTABLE' else uname = 'TABLE'
  
  wTable = widget_info(event.top, find_by_uname = uname)
  
  widget_control, wTable, get_value = table
  
  s = size(table)
  nCols = s[1]
  nRows = s[2]
  
  sel = widget_info(wTable, /table_select)
  
  nPastedRows = nPastedRows < (nRows - 1 - sel[1])
  nPastedCols = nPastedCols < (nCols - 1 - sel[0])
  
  widget_control, wTable, set_value = val, use_table_select = [sel[0],sel[1],sel[0]+nPastedCols,sel[1]+nPastedRows]
  
end


pro cosmos_data_export, event

  settings = cosmos_get_settings(event.top)

  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTDIR'), get_value = path

  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTBASE'), get_value = basename

  path = strtrim(path,2)

  filename = dialog_pickfile(title = "Select file", dialog_parent = event.top, file = strtrim(basename,2)+".csv", $
                             path = path, /overwrite_prompt, /write)

  if strlen(filename) eq 0 then return
  
  table_contents = *settings.table
  
  sz = size(table_contents)
  
  nCols = sz[1]
  
  case nCols of
    6    : tLabels = ['Direct 1', 'Reflect 1', 'Normalise', 'Theta 1'  , 'Out file' , 'Out LAMP']
    7    : tLabels = ['Direct 1', 'Reflect 1', 'Instr1'   , 'Normalise', 'Theta 1'  , 'Out file' , 'Out LAMP']
    10   : tLabels = ['Direct 1', 'Direct 2' , 'Reflect 1', 'Reflect 2', 'Normalise', 'Factor'   , 'Theta 1'  , 'Theta 2'   , 'Out file'  , 'Out LAMP']
    12   : tLabels = ['Direct 1', 'Direct 2' , 'Reflect 1', 'Reflect 2', 'Instr 1'  , 'Instr 2'  , 'Normalise', 'Factor'    , 'Theta 1'   , 'Theta 2'  , 'Out file'  , 'Out LAMP' ]
    14   : tLabels = ['Direct 1', 'Direct 2' , 'Direct 3' , 'Reflect 1', 'Reflect 2', 'Reflect 3', 'Normalise', 'Factor 1-2', 'Factor 2-3', 'Theta 1'  , 'Theta 2'   , 'Theta 3'   , 'Out file', 'Out LAMP' ]
    17   : tLabels = ['Direct 1', 'Direct 2' , 'Direct 3' , 'Reflect 1', 'Reflect 2', 'Reflect 3', 'Instr 1'  , 'Instr 2'   , 'Instr 3'   , 'Normalise', 'Factor 1-2', 'Factor 2-3', 'Theta 1' , 'Theta 2', 'Theta 3', 'Out file', 'Out LAMP' ]
    else : begin
      print, "Invalid column number."
      tLabels = string(indgen(nCols))
    end
  endcase  
  
  write_csv, filename, table_contents, header = tLabels

end


pro cosmos_data_import, event

  settings = cosmos_get_settings(event.top)

  widget_control, widget_info(event.top, find_by_uname = 'OUTPUTDIR'), get_value = path
  
  path = strtrim(path,2)
              
  filename = dialog_pickfile(title = "Select input file", dialog_parent = event.top, path = path)
  
  check = query_csv(filename, info_csvfile)
  
  if check then begin
    
      table_values = read_csv(filename) 
      
      wTable = widget_info(event.top, find_by_uname = 'TABLE')

      nCols = info_csvfile.nfields
      nRows = info_csvfile.lines - 1
      
      ;Change number of angles accordingly to CSV table
      if nCols eq 6 or nCols eq 7 then val = 0
      if nCols eq 10 or nCols eq 12 then val = 1
      if nCols eq 14 or nCols eq 17 then val = 2
      
      nAngles = widget_info(event.top, find_by_uname = 'NANGLES')
      widget_control, nAngles, set_value = val

      if nCols eq 7 or nCols eq 12 or nCols eq 17 then val = [1] else val = [0]
      instBgr = widget_info(event.top, find_by_uname = 'INSTBG')
      widget_control, instBgr, set_value = val

      nTable = strarr(nCols, nRows)
      
      for i = 0, nCols-1 do begin
          tmp = string(table_values.(i))
          zeros = where(strtrim(tmp,2) eq '0' or strmid(strtrim(tmp,2), 0, 8) eq '0.000000', /NULL)
          tmp[zeros] = ''
          nTable[i,0:n_elements(tmp)-1] = tmp
      endfor

      rowLabels = strtrim((indgen(nRows) + 1),2)
      tLabels = cosmos_get_table_labels(nCols)

      widget_control, wTable, xsize = nCols, ysize = nRows, column_labels = tLabels, row_labels = rowLabels, alignment = 2, /editable, set_value = nTable

  endif


end


pro cosmos_data_fill, event
  
  base = WIDGET_BASE(TITLE='Input values to fill cell', /COLUMN)
  numor_ini = CW_FIELD(base, TITLE='First numor: ', /frame, /long, uname="FILLINI")
  numor_end = CW_FIELD(base, TITLE='Last numor:  ', /frame, /long, uname="FILLEND")
  numor_inc = CW_FIELD(base, TITLE='Increment:   ', /frame, /long, uname="FILLINC")
  subbase = widget_base(base, /ROW, /base_align_center, /align_center)
  b = WIDGET_BUTTON(subbase, value='Apply', uname="FILLAPPLY", uvalue=event)
  b = WIDGET_BUTTON(subbase, value='Cancel', uname="FILLCANCEL")
  
  ; Realize the widgets.
  WIDGET_CONTROL, base, /REALIZE

  xmanager, 'fill_cell', base, event_handler = 'cosmos_fill_cell_events', /just_reg

end

pro cosmos_fill_cell_events, event

  catch, stat & if stat ne 0 then begin print,!error_state.msg & return & endif

  uname = widget_info(event.id, /uname)
  
  case uname of
    'FILLCANCEL': begin
                     widget_control, event.top, /destroy
                  end   
    'FILLAPPLY': begin
                    cosmos_data_fill_apply, event, cell, ok
                    if ok then begin
                        widget_control, event.id, get_uvalue=TableID
                        wTable = widget_info(TableID.top, find_by_uname = 'TABLE')
                        widget_control, wTable, get_value = table
                        sel = widget_info(wTable, /table_select)
                        widget_control, wTable, set_value = cell, use_table_select = sel
                        widget_control, event.top, /destroy
                    endif
                 end   
  endcase
end 


pro cosmos_data_fill_apply, event, cell, ok

  widget_control, widget_info(event.top, find_by_uname = 'FILLINI'), get_value = n1
  widget_control, widget_info(event.top, find_by_uname = 'FILLEND'), get_value = n2
  widget_control, widget_info(event.top, find_by_uname = 'FILLINC'), get_value = n3
  
  n1=long(n1) & n2=long(n2) & n3=long(n3) & ok = 1
  if n1 gt 999999 then ok = 0
  if n2 gt 999999 then ok = 0
  if n2 lt n1 then ok = 0
  if n1 le 0 then ok = 0
  if n2 le 0 then ok = 0
  if n3 le 0 then ok = 0
  
  cell = ''
  
  if ok then begin
      for i=n1,n2,n3 do begin
          if i eq n1 then cell = strtrim(string(i, format='(I)'),2) else $
                          cell = cell+'+'+strtrim(string(i, format='(I)'),2)
      endfor
  endif else begin
      answer = dialog_message('Wrong input!' , /error)    
  endelse

end


;pro cosmos_data_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return    
;  endif
;  
;  eventType = tag_names(event, /structure_name)
;  
;  if eventType eq 'WIDGET_CONTEXT' then begin
;    table = widget_info(event.top, find_by_uname = 'TABLE')
;    tablemenu = widget_info(event.top, find_by_uname = 'TABLEMENU')
;
;    widget_displaycontextmenu, table, event.x, event.y, tablemenu
;    
;  endif else begin
;  
;    uname = widget_info(event.id, /uname)
;  
;    case uname of    
;    
;      'CLEARALL': cosmos_data_clear, event      
;      'CLEAR':    cosmos_data_clear, event
;      'INSROW':   cosmos_data_insert_rows, event
;      'DELROW':   cosmos_data_delete_rows, event 
;      'SETVAL':   cosmos_data_set, event      
;      'INCRVAL':  cosmos_data_increment, event
;      'SELALL':   cosmos_data_selectall, event     
;      'COPY':     cosmos_data_copy, event      
;      'PASTE':    cosmos_data_paste, event
;      'NANGLES':  cosmos_data_update_columns, event
;      'INSTBG':   cosmos_data_update_columns, event
;    
;      else:
;    
;    endcase
;  
;  endelse
;              
;end

pro cosmos_data_tab, parent = parent, galaxy = galaxy
  
  ;forward_function label_frame
  
  ; The normal font.
  c_font_normal = "arial*light*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /column, title='DATA     ', tab_mode = 1, /base_align_left, $
                    uname = 'DATATAB', event_pro = 'cosmos_events', /frame)
                              
  if keyword_set(galaxy) then begin

    bb = widget_base(tab, /row, /base_align_top)  
    l  = widget_label(bb, value = 'Numors', xsize = 110)
    w  = cw_field(bb, title = 'First ', /string, uname = 'FIRSTNUMOR')
    w  = cw_field(bb, title = 'Last  ', /string, uname = 'LASTNUMOR')
    w  = cw_field(bb, title = 'Step  ', /string, uname = 'NUMORSTEP')
    
    nCols = 4
    uname = 'GTABLE'
  
  endif else begin
  
    nCols = 10
    uname = 'TABLE'
  
  endelse

  bb = widget_base(tab, /ROW, /base_align_center, UNAME='DATATAB_B1')  
              
  bbb = widget_base(bb, /row, /base_align_center, UNAME='DATATAB_B11')  
  l  = widget_label(bbb, value = 'Insert rows ', UNAME='INSERT_ROWS', /TRACKING_EVENTS)
  w  = cw_field(bbb, xsize = 10, value = 1, /integer, /return_events, title = '', uname = 'INSROW')
  w  = widget_button(bbb, value = str_centerize('Do',6), uname = 'INSROW')  

  l  = widget_label(bb, ysize=6, value = '', UNAME='DATATAB_L11')
                       
  bbb = widget_base(bb, /row, /base_align_center, UNAME='DATATAB_B12')  
  l  = widget_label(bbb, value = 'Set column(s) ', UNAME='SET_COLUMNS', /TRACKING_EVENTS)
  w  = cw_field(bbb, xsize = 10, value = 1, /floating, /return_events, title = '', uname = 'SETVAL')
  w  = widget_button(bbb, value = str_centerize('Do',6), uname = 'SETVAL')  

  l  = widget_label(bb, ysize=6, value = '', UNAME='DATATAB_L12')
        
  bbb = widget_base(bb, /row, /base_align_center, UNAME='DATATAB_B13')  
  l  = widget_label(bbb, value = 'Increment column(s) ', UNAME='INCR_COLUMNS', /TRACKING_EVENTS)
  w  = cw_field(bbb, xsize = 10, value = 1, /floating, /return_events, title = '', uname = 'INCRVAL')
  w  = widget_button(bbb, value = str_centerize('Do',6), uname = 'INCRVAL')  

  l  = widget_label(bb, ysize=6, value = '')

  bbb = widget_base(tab, /ROW, /base_align_center, UNAME='DATATAB_B2')  

  w  = widget_button(bbb, value = str_centerize('Copy',6), uname = 'COPY', /TRACKING_EVENTS)  
  w  = widget_button(bbb, value = str_centerize('Paste',6), uname = 'PASTE', /TRACKING_EVENTS)  
  w  = widget_button(bbb, value = str_centerize('Delete Row(s)',6), uname = 'DELROW', /TRACKING_EVENTS)  
  w  = widget_button(bbb, value = str_centerize('Clear',6), uname = 'CLEAR', /TRACKING_EVENTS)  
  w  = widget_button(bbb, value = str_centerize('Clear All',6), uname = 'CLEARALL', /TRACKING_EVENTS)  
  w  = widget_button(bbb, value = str_centerize('Export (CSV)',6), uname = 'EXPORTCSV', /TRACKING_EVENTS)
  w  = widget_button(bbb, value = str_centerize('Import (CSV)',6), uname = 'IMPORTCSV', /TRACKING_EVENTS)

  bb = widget_base(tab, /ROW, /base_align_center, UNAME='DATATAB_B3')  
  
  bbb = widget_base(bb, /row, /base_align_center, UNAME='DATATAB_B31')  
  l  = widget_label(bbb, value = 'Number of angles ', UNAME='N_ANGLES', /TRACKING_EVENTS)
  w  = cw_bgroup(bbb, ['1','2','3'], label_left='', column = 3, /exclusive, tab_mode = 0, set_value = 1, uname = 'NANGLES')

  l  = widget_label(bb, value = '', UNAME='DATATAB_L31')
                        
  bbb = widget_base(bb, /row, /base_align_center, UNAME='DATATAB_B32')  
  l  = widget_label(bbb, value = 'Instrument background ', UNAME='INST_BG', /TRACKING_EVENTS)
  w  = cw_bgroup(bbb, ["yes"], label_left='', column = 1, set_value = [0], /nonexclusive, tab_mode = 0, uname = 'INSTBG')

  bb = widget_base(tab, /row, /base_align_center, UNAME='DATATAB_B4')  

  l  = widget_label(bb, ysize=6, value = '')
      
  table = widget_table(bb, /editable, ignore_accelerators = 1, /resizeable_columns, /scroll, /all_events, $
                       x_scroll_size = nCols, y_scroll_size = 10, tab_mode = 0, /context_events, alignment = 0  , $
                       uname = uname)
    
  b = widget_base(table, /context_menu, uname = 'TABLEMENU')
  w = widget_button(b, value = "Clear all",           uname = 'CLEARALL')  
  w = widget_button(b, value = "Clear selection",     uname = 'CLEAR')
  w = widget_button(b, value = "Insert row(s)",       uname = 'INSROW')
  w = widget_button(b, value = "Delete row(s)",       uname = 'DELROW')
  w = widget_button(b, value = "Set column(s) value", uname = 'SETVAL')
  w = widget_button(b, value = "Increment column(s)", uname = 'INCRVAL')  
  w = widget_button(b, value = "Select all",          uname = 'SELALL')  
  w = widget_button(b, value = "Copy",                uname = 'COPY')
  w = widget_button(b, value = "Paste",               uname = 'PASTE')
  w = widget_button(b, value = "Fill with interval",  uname = 'FILLINTERVAL')

  if xregistered('LAMP',/noshow) then PUT_Progress, tab ; Memory progress-bar
  if xregistered('LAMP',/noshow) then UM_Progress ; Update Memory _ Progress Bar
  
end

pro cosmos_normalization_run, event

  common cosmos_settings, c_settings
  common cosmos_params, c_params
   
  widget_control, widget_info(event.top, find_by_uname = 'DBEAM1'), get_value = dir1
  widget_control, widget_info(event.top, find_by_uname = 'DBEAM2'), get_value = dir2
  widget_control, widget_info(event.top, find_by_uname = 'DBEAM3'), get_value = dir3

  widget_control, widget_info(event.top, find_by_uname = 'RBEAM1'), get_value = ref1
  widget_control, widget_info(event.top, find_by_uname = 'RBEAM2'), get_value = ref2
  widget_control, widget_info(event.top, find_by_uname = 'RBEAM3'), get_value = ref3
  
  widget_control, widget_info(event.top, find_by_uname = 'QMIN'), get_value = qmin
  widget_control, widget_info(event.top, find_by_uname = 'QMAX'), get_value = qmax

  widget_control, widget_info(event.top, find_by_uname = 'ANGMETHOD'), get_value = angmethod
       
  dirs = [dir1,dir2,dir3]    
  refs = [ref1,ref2,ref3]    
      
  norm_fact = -1.0
  for i=0, n_elements(dirs)-1 do begin

    dir = cosmos_getnumors(dirs[i])
    ref = cosmos_getnumors(refs[i])

    on_ioerror, misdir & okdir=0 & dir = long(dir) & okdir=1 & misdir: if not okdir then continue
    on_ioerror, misref & okref=0 & ref = long(ref) & okref=1 & misref: if not okref then continue
    if (dir eq 0 or ref eq 0) then continue 

    c_settings = cosmos_get_settings(event.top)
    
    if angmethod eq 0 then cosmos_anal_run, res, gpres, dir, ref, "", -2, i+1 else cosmos_anal_run, res, gpres, dir, ref, "", -1, i+1
    idx = where(res[0,*] ge qmin and res[0,*] le qmax, count)
    if (count gt 0) then norm_fact = 1.0/mean(res[2,idx])
    
    widget_control, widget_info(event.top, find_by_uname = "NORMVAR"), set_value = norm_fact
    widget_control, widget_info(event.top, find_by_uname = "SANGLE1"), set_value = c_params.angle_bragg_1
    
    break
  
  endfor    
  
  if norm_fact lt 0.0 then return
  
  if i eq 2 then return ;Only angle 3 given
  
  if i eq 1 then begin
  
        if dirs[i+1] lt "" or refs[i+1] lt "" then return ;Only angle 2 given
  
        dir = cosmos_getnumors(dirs[i+1])
        ref = cosmos_getnumors(refs[i+1])
        on_ioerror, misdir1 & okdir=0 & dir = long(dir) & okdir=1 & misdir1: if not okdir then return
        on_ioerror, misref1 & okref=0 & ref = long(ref) & okref=1 & misref1: if not okref then return

        if angmethod eq 0 then cosmos_anal_run, res3, gpres3, dir, ref, "", -2, 3 else cosmos_anal_run, res3, gpres3, dir, ref, "", -1, 3
        scale_factor = 1.0/cosmos_anal_factor(res, res3)
        scale_factor = scale_factor[0]

        widget_control, widget_info(event.top, find_by_uname = 'FACTOR2'), set_value = scale_factor
        
        return
  
  endif 
  
  if dirs[i+1] gt "" and refs[i+1] gt "" then begin ;Angles 1 + 2
   
        dir = cosmos_getnumors(dirs[i+1])
        ref = cosmos_getnumors(refs[i+1])
        on_ioerror, misdir2 & okdir=0 & dir = long(dir) & okdir=1 & misdir2: if not okdir then return
        on_ioerror, misref2 & okref=0 & ref = long(ref) & okref=1 & misref2: if not okref then return

        if angmethod eq 0 then cosmos_anal_run, res2, gpres2, dir, ref, "", -2, 2 else cosmos_anal_run, res2, gpres2, dir, ref, "", -1, 2

        scale_factor = 1.0/cosmos_anal_factor(res, res2)
        scale_factor = scale_factor[0]

        widget_control, widget_info(event.top, find_by_uname = 'FACTOR1'), set_value = scale_factor
        widget_control, widget_info(event.top, find_by_uname = "SANGLE2"), set_value = c_params.angle_bragg_2
        
        if dirs[i+2] gt "" and refs[i+2] gt "" then begin

          dir = cosmos_getnumors(dirs[i+2])
          ref = cosmos_getnumors(refs[i+2])
          on_ioerror, misdir3 & okdir=0 & dir = long(dir) & okdir=1 & misdir3: if not okdir then return
          on_ioerror, misref3 & okref=0 & ref = long(ref) & okref=1 & misref3: if not okref then return

          if angmethod eq 0 then cosmos_anal_run, res3, gpres3, dir, ref, "", -2, 3 else cosmos_anal_run, res3, gpres3, dir, ref, "", -1, 3
          scale_factor = 1.0/cosmos_anal_factor(res2, res3)
          scale_factor = scale_factor[0]

          widget_control, widget_info(event.top, find_by_uname = 'FACTOR2'), set_value = scale_factor
          widget_control, widget_info(event.top, find_by_uname = "SANGLE3"), set_value = c_params.angle_bragg_3
        
        endif
  
  endif else if dirs[i+2] gt "" and refs[i+2] gt "" then begin

        dir = cosmos_getnumors(dirs[i+2])
        ref = cosmos_getnumors(refs[i+2])
        on_ioerror, misdir4 & okdir=0 & dir = long(dir) & okdir=1 & misdir4: if not okdir then return
        on_ioerror, misref4 & okref=0 & ref = long(ref) & okref=1 & misref4: if not okref then return

        if angmethod eq 0 then cosmos_anal_run, res3, gpres3, dir, ref, "", -2, 3 else cosmos_anal_run, res3, gpres3, dir, ref, "", -1, 3
        scale_factor = 1.0/cosmos_anal_factor(res, res3)
        scale_factor = scale_factor[0]

        widget_control, widget_info(event.top, find_by_uname = 'FACTOR1'), set_value = scale_factor
  
  endif      
end

pro cosmos_transfer_db, event

    common cosmos_settings, c_settings

    c_settings = cosmos_get_settings(event.top)

    wTable = widget_info(event.top, find_by_uname = 'TABLE')

    widget_control, wTable, get_value = table

    nRows = n_elements(table[0,*])
    nCols = n_elements(table[*,0])
    
    case nCols of
        6  : colsDB = [0]     
        7  : colsDB = [0]     
        10 : colsDB = [0,1]   
        12 : colsDB = [0,1]   
        14 : colsDB = [0,1,2]   
        17 : colsDB = [0,1,2] 
        else: return
    endcase

    ; Direct beam
    widget_control, widget_info(event.top, find_by_uname = "DBEAM1"), get_value = db1
    if db1 gt '' then begin
      vals =  make_array(1, nRows, /long, value = db1)
      widget_control, wTable, set_value = vals, use_table_select = [colsDB[0],0,colsDB[0],nRows-1]
    endif

    if (n_elements(colsDB) ge 2) then begin  ; Direct beam 2
        widget_control, widget_info(event.top, find_by_uname = "DBEAM2"), get_value = db2
        if db2 gt '' then begin
          vals =  make_array(1, nRows, /long, value = db2)
          widget_control, wTable, set_value = vals, use_table_select = [colsDB[1],0,colsDB[1],nRows-1]
        endif
    endif

    if (n_elements(colsDB) eq 3) then begin  ; Direct beam 3
        widget_control, widget_info(event.top, find_by_uname = "DBEAM3"), get_value = db3
        if db3 gt '' then begin
          vals =  make_array(1, nRows, /long, value = db3)
          widget_control, wTable, set_value = vals, use_table_select = [colsDB[2],0,colsDB[2],nRows-1]
        endif
    endif

end

pro cosmos_transfer_theta, event

  common cosmos_settings, c_settings

  c_settings = cosmos_get_settings(event.top)

  wTable = widget_info(event.top, find_by_uname = 'TABLE')

  widget_control, wTable, get_value = table

  nRows = n_elements(table[0,*])
  nCols = n_elements(table[*,0])

  case nCols of
    6  : colsAngle = [3]       
    7  : colsAngle = [4]       
    10 : colsAngle = [6,7]     
    12 : colsAngle = [8,9]      
    14 : colsAngle = [9,10,11]  
    17 : colsAngle = [12,13,14] 
    else: return
  endcase

  ; Sample angle
  widget_control, widget_info(event.top, find_by_uname = "SANGLE1"), get_value = sample_angle
  if sample_angle gt '' then begin
    vals =  make_array(1, nRows, /float, value = sample_angle)
    widget_control, wTable, set_value = vals, use_table_select = [colsAngle[0],0,colsAngle[0],nRows-1]
  endif
  
  if (n_elements(colsAngle) ge 2) then begin  ;Angle 2
    widget_control, widget_info(event.top, find_by_uname = "SANGLE2"), get_value = sample_angle
    if sample_angle gt '' then begin
      vals =  make_array(1, nRows, /float, value = sample_angle)
      widget_control, wTable, set_value = vals, use_table_select = [colsAngle[1],0,colsAngle[1],nRows-1]
    endif
  endif

  if (n_elements(colsAngle) eq 3) then begin  ;Angle 3
    widget_control, widget_info(event.top, find_by_uname = "SANGLE3"), get_value = sample_angle
    if sample_angle gt '' then begin
      vals =  make_array(1, nRows, /float, value = sample_angle)
      widget_control, wTable, set_value = vals, use_table_select = [colsAngle[2],0,colsAngle[2],nRows-1]
    endif
  endif

end


pro cosmos_transfer_normalization_factors, event

  common cosmos_settings, c_settings

  c_settings = cosmos_get_settings(event.top)

  wTable = widget_info(event.top, find_by_uname = 'TABLE')

  widget_control, wTable, get_value = table

  nRows = n_elements(table[0,*])
  nCols = n_elements(table[*,0])

  case nCols of
    6  : colsFactor = [2]       
    7  : colsFactor = [3]      
    10 : colsFactor = [4,5]     
    12 : colsFactor = [6,7]     
    14 : colsFactor = [6,7,8]   
    17 : colsFactor = [9,10,11] 
    else: return
  endcase

;  ; Check if any of the columns to fill is not completely empty
;  tmp = table[[colsDB, colsFactor, colsAngle],*]
;  tmp = max(tmp)
;  if strtrim(tmp,2) gt '' then begin
;    answer = dialog_message('All the cells in the columns Direct #, Normalise, Factor # and Theta #' + $
;      ' will be overwritten with the values shown here! Continue?', $
;      /cancel, /default_cancel, /center, dialog_parent = event.top, title = 'Warning')
;    if strupcase(answer) eq 'CANCEL' then return
;  endif

  ; Normalization factor
  widget_control, widget_info(event.top, find_by_uname = "NORMVAR"), get_value = norm_factor
  vals =  make_array(1, nRows, /float, value = norm_factor)
  widget_control, wTable, set_value = vals, use_table_select = [colsFactor[0],0,colsFactor[0],nRows-1]

  if (n_elements(colsFactor) ge 2) then begin

    ; Factor1
    widget_control, widget_info(event.top, find_by_uname = 'FACTOR1'), get_value = scale_factor1
    vals =  make_array(1, nRows, /float, value = scale_factor1)
    widget_control, wTable, set_value = vals, use_table_select = [colsFactor[1],0,colsFactor[1],nRows-1]

  endif

  if (n_elements(colsFactor) eq 3) then begin

    ; Factor2
    widget_control, widget_info(event.top, find_by_uname = 'FACTOR2'), get_value = scale_factor2
    vals =  make_array(1, nRows, /float, value = scale_factor2)
    widget_control, wTable, set_value = vals, use_table_select = [colsFactor[2],0,colsFactor[2],nRows-1]

  endif

end


pro cosmos_select_qrange, event

  case event.index of
  
    ; Air/D2O
    0: begin
      widget_control, widget_info(event.top, find_by_uname = "QMIN"), set_value = 0.010
      widget_control, widget_info(event.top, find_by_uname = "QMAX"), set_value = 0.015
    end
    ; Si/D2O
    1: begin    
      widget_control, widget_info(event.top, find_by_uname = "QMIN"), set_value = 0.007
      widget_control, widget_info(event.top, find_by_uname = "QMAX"), set_value = 0.013
    end
    ; Si/Air
    2: begin
      widget_control, widget_info(event.top, find_by_uname = "QMIN"), set_value = 0.004
      widget_control, widget_info(event.top, find_by_uname = "QMAX"), set_value = 0.009
    end
    ; Quartz/D2O
    3: begin
      widget_control, widget_info(event.top, find_by_uname = "QMIN"), set_value = 0.004
      widget_control, widget_info(event.top, find_by_uname = "QMAX"), set_value = 0.009
    end
    ; Quartz/Air
    4: begin
      widget_control, widget_info(event.top, find_by_uname = "QMIN"), set_value = 0.007
      widget_control, widget_info(event.top, find_by_uname = "QMAX"), set_value = 0.013
    end
  
    else:
  
  endcase

end

;pro cosmos_calculation_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;  
;  uname = widget_info(event.id, /uname)
;  case uname of
;    'GROUP' : begin
;                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR1'), sensitive = event.select
;                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR2'), sensitive = event.select
;                  widget_control, widget_info(event.top, find_by_uname = 'GROUPVAR3'), sensitive = event.select
;              end
;    else:  
;  endcase
;
;end

pro cosmos_calculation_tab, parent = parent
    
  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /row, title='CALCULATION  ', tab_mode = 1, /base_align_left, uname = 'CALCTAB', event_pro = 'cosmos_events')
  
  b = widget_base(tab, /column, /base_align_left, uname = 'CALCBASE')

  bb = widget_base(b, /row, ysize = 30, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=170, value = 'Calculation Method ', uname='CALC_METHOD', /ALIGN_LEFT, /TRACKING_EVENTS)
  droplist_values = ['incoherent', 'coherent']
  w = widget_droplist(bb, title = "Angle 1 ", value=droplist_values, Uvalue=droplist_values, uname = 'CALCMETHOD1')
  ;l = widget_label(bb, xsize=15, value = "")
  w = widget_droplist(bb, title = "Angle 2 ", value=droplist_values, Uvalue=droplist_values, uname = 'CALCMETHOD2')
  ;l = widget_label(bb, xsize=18, value = "")
  w = widget_droplist(bb, title = "Angle 3 ", value=droplist_values, Uvalue=droplist_values, uname = 'CALCMETHOD3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Foreground (pixel)", uname='FG', /ALIGN_LEFT, /TRACKING_EVENTS)

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Width", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /string, uname = 'FOREGDWIDTH1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /string, uname = 'FOREGDWIDTH2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /string, uname = 'FOREGDWIDTH3')
  
  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Left Background (pixel)", uname='LEFT_BG', /ALIGN_LEFT, /TRACKING_EVENTS)
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Width", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'BACKGDLOWWIDTH1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'BACKGDLOWWIDTH2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'BACKGDLOWWIDTH3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Offset", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'BACKGDLOWOFFSET1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'BACKGDLOWOFFSET2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'BACKGDLOWOFFSET3')

  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Right Background (pixel)", uname='RIGHT_BG', /ALIGN_LEFT, /TRACKING_EVENTS)

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Width", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'BACKGDHIGHWIDTH1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'BACKGDHIGHWIDTH2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'BACKGDHIGHWIDTH3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Offset", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'BACKGDHIGHOFFSET1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'BACKGDHIGHOFFSET2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'BACKGDHIGHOFFSET3')
  
  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Lambda", uname='LAMBDA', /ALIGN_LEFT, /TRACKING_EVENTS)
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Min", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'LAMBDAMIN1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'LAMBDAMIN2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'LAMBDAMIN3')
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=125, value = " Max", /ALIGN_LEFT)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'LAMBDAMAX1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'LAMBDAMAX2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'LAMBDAMAX3')

  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=140, value = 'Grouping ', uname='GROUPING', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=28, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'GROUPVAR1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'GROUPVAR2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'GROUPVAR3')
  w = cw_bgroup(bb, [" yes"], font = c_font_big, /nonexclusive, xsize=55, label_left='', uname = 'GROUP')

  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, ysize = 30, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=170, value = 'Background Method ', uname='BG_METHOD', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_bgroup(bb, column = 2, label_left='', /exclusive, ["average","fit"], uname = "BACKMETH")

  l = widget_label(b, ysize=6, value = "")

  bb = widget_base(b, /row, ysize = 30, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=170, value = 'Discard Low Statistics Data ', uname='GROUP_DISCARD', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_bgroup(bb, ["yes"], label_left='', /nonexclusive, uname = "GROUPDISCARD")
      
end

;pro cosmos_normalization_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;  
;  uname = widget_info(event.id, /uname)
;
;  case uname of
;    'QRANGESEL' : cosmos_select_qrange, event
;    'SETNORMVAR': cosmos_normalization_run, event
;    else:  
;  endcase
;  
;end

pro cosmos_normalization_tab, parent = parent
    
  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /row, title='NORMALIZATION  ', tab_mode = 1, /base_align_left, /scroll, uname='NORMTAB', event_pro = 'cosmos_events')

  b = widget_base(tab, /column, /base_align_left, uname = 'DETECTBASE')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, value = " Normalization", /ALIGN_LEFT)

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=140, value = " Direct Beam",   uname='DIRECT_BEAM', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /string, uname = 'DBEAM1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /string, uname = 'DBEAM2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /string, uname = 'DBEAM3')
  l = widget_label(bb, xsize=20, value = "")
  w  = widget_button(bb, value = "Transfer DBs"           , uname = "TRANSFERDB")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=140, value = " D2O", uname='D2O', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /string, uname = 'RBEAM1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /string, uname = 'RBEAM2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /string, uname = 'RBEAM3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=140, value = " Sample theta", uname='SANGLE', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /string, uname = 'SANGLE1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /string, uname = 'SANGLE2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /string, uname = 'SANGLE3')
  l = widget_label(bb, xsize=20, value = "")
  w  = widget_button(bb, value = "Transfer theta"         , uname = "TRANSFERTHETA")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=140, value = " Factors",       uname='NORM_FACTOR', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Normalization ", xsize = 10, /floating, uname = 'NORMVAR')
  l = widget_label(bb, xsize=10, value = "")
  w = cw_field(bb, title = "Scale 1/2 ", xsize = 10, /floating,     uname = 'FACTOR1')
  l = widget_label(bb, xsize=10, value = "")  
  w = cw_field(bb, title = "Scale 2/3 ", xsize = 10, /floating,     uname = 'FACTOR2')
  l = widget_label(bb, xsize=10, value = "")  
  w  = widget_button(bb, value = "Calculate",                       uname = "SETNORMVAR")
  l = widget_label(bb, xsize=10, value = "")  
  w  = widget_button(bb, value = "Transfer Factors"         ,       uname = "TRANSFERNORMVAR")

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=140, value = " Q range", uname='Q_RANGE', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=42, value = "")
  w = cw_field(bb, /floating, xsize = 10, title = "Q min  ", uname = "QMIN")
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, /floating, xsize = 10, title = "Q max  ", uname = "QMAX")
  w = widget_droplist(bb, value = ["air/D2O","Si/D2O","Si/air","quartz/D2O","quartz/air"], uname = 'QRANGESEL')

  l = widget_label(b, ysize=10, value = "")
  
  ;bb = widget_base(b, /row, /base_align_center)
  ;l = widget_label(bb, font = c_font_big, xsize=175,value = 'Beam normalization ',    uname='BEAM_NORMALIZE', /ALIGN_LEFT, /TRACKING_EVENTS)
  ;w = cw_bgroup(bb, label_left='', column = 2, /exclusive, ["run time","monitor"],    uname = "NORMALISE")
  ;  
  ;l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=175,value = 'Angle method ',    uname='ANGLE_METHOD', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_bgroup(bb, label_left='', column = 2, /exclusive, ["dan", "san"], uname = "ANGMETHOD")

  l = widget_label(b, ysize=10, value = "")
  
  bb = widget_base(b, /row, ysize = 30, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=185, value = 'Detector calibration ', uname='DET_CALIBRATION', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_field(bb, /string, xsize = 10, title='', uname = "WATERRUN")

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, ysize = 30, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=200, value = 'Polarization correction factors file', uname='POL_CORR', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_field(bb, /string, xsize = 50, title='', uname = "POLCORRFILE")
  w = widget_button(bb, value = 'Browse', tab_mode = 0, uname = 'POLFILEBROWSE')
  
end

;pro cosmos_detector_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;  
;  uname = widget_info(event.id, /uname)
;  
;  case uname of  
;    'AUTOSEARCHPIXEL1': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN1'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX1'), sensitive = 1 - event.select
;    end
;    'AUTOSEARCHPIXEL2': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN2'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX2'), sensitive = 1 - event.select
;    end
;    'AUTOSEARCHPIXEL3': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMIN3'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHPIXELMAX3'), sensitive = 1 - event.select
;    end
;    'AUTOSEARCHTIME1': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN1'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX1'), sensitive = 1 - event.select
;    end
;    'AUTOSEARCHTIME2': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN2'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX2'), sensitive = 1 - event.select
;    end
;    'AUTOSEARCHTIME3': begin
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMIN3'), sensitive = 1 - event.select
;        widget_control, widget_info(event.top, find_by_uname = 'SEARCHTIMEMAX3'), sensitive = 1 - event.select
;    end
;    else:
;  endcase
;    
;end

pro cosmos_detector_tab, parent = parent
    
  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /row, title='DETECTOR  ', tab_mode = 1, /base_align_left, uname = 'DETECTTAB', event_pro = 'cosmos_events')

  b = widget_base(tab, /column, /base_align_left, uname = 'DETECTBASE')
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Pixel range", uname='USEFUL_PIXEL', /ALIGN_LEFT, /TRACKING_EVENTS)
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Min", uname='USEFUL_PIXEL_MIN', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'USEFULPIXELMIN1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'USEFULPIXELMIN2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'USEFULPIXELMIN3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Max", uname='USEFUL_PIXEL_MAX', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'USEFULPIXELMAX1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'USEFULPIXELMAX2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'USEFULPIXELMAX3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=180, value = " Search range for peak", uname='SEARCH_PIXEL', /TRACKING_EVENTS)
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Min", uname='SEARCH_PIXEL_MIN', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'SEARCHPIXELMIN1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'SEARCHPIXELMIN2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'SEARCHPIXELMIN3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Max", uname='SEARCH_PIXEL_MAX', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'SEARCHPIXELMAX1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'SEARCHPIXELMAX2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'SEARCHPIXELMAX3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Auto Search", uname='AUTO_SEARCH_PIXEL', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=35, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHPIXEL1')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=15, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHPIXEL2')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=15, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHPIXEL3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=220, value = " Search range for peak (TOF channels)", uname='SEARCH_TIME', /ALIGN_LEFT, /TRACKING_EVENTS)
  
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Min", uname='SEARCH_TIME_MIN', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'SEARCHTIMEMIN1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'SEARCHTIMEMIN2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'SEARCHTIMEMIN3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Max", uname='SEARCH_TIME_MAX', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'SEARCHTIMEMAX1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'SEARCHTIMEMAX2')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'SEARCHTIMEMAX3')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Auto Search", uname='AUTO_SEARCH_TIME', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=35, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHTIME1')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=15, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHTIME2')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=15, value = "")
  w = cw_bgroup(bb, [" yes"], xsize=100, /nonexclusive, uname = 'AUTOSEARCHTIME3')

  l = widget_label(b, ysize=10, value = "")
  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=400, value = " Reference peak position in detector (only used for Figaro + DAN)", uname='REF_PEAK_POS', /ALIGN_LEFT, /TRACKING_EVENTS)

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=100, value = " Pixel (0 = not used)", uname='DET_PIXEL', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /float, uname = 'DETPIXEL1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /float, uname = 'DETPIXEL2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /float, uname = 'DETPIXEL3')
        
end

pro cosmos_clean_log, event
  
  w = widget_info(event.top, find_by_uname = "LOGINFO")  
  widget_control, w, get_value = mess
  mess = mess[0]
  widget_control, w, set_value = mess

  w = widget_info(event.top, find_by_uname = "LOGWARN")  
  widget_control, w, get_value = mess
  mess = mess[0]
  widget_control, w, set_value = mess
    
end

pro cosmos_save_log, event
    
  mess = []
  
  logInfo = widget_info(event.top, find_by_uname = "LOGINFO")  
  widget_control, logInfo, get_value = infoMess  
  if n_elements(infoMess) gt 0 then mess = [mess, infoMess]
    
  logWarn = widget_info(event.top, find_by_uname = "LOGWARN")
  widget_control, logWarn, get_value = warnMess 
  if n_elements(warnMess) gt 0 then mess = [mess, warnMess]
       
  if n_elements(mess) eq 0 then return
    
  valid = 0
  
  save_lun = -1
        
  filename = dialog_pickfile(title = "Select input file", dialog_parent = event.handler, file = "logfile.dat")
  
  if strlen(strtrim(filename,2)) eq 0 then return
  
  res = file_info(filename)
  
  if res.exists then begin
  
    cancel = dialog_message('File exists already.' + string(10b) + 'Do you want to overwrite it?', /cancel, /default_cancel, title = 'Warning')    
    cancel = strlowcase(cancel)
    
    if cancel eq "cancel" then return
    
  endif
        
  on_ioerror, write_error

  openw, save_lun, filename, /get_lun

  for i = 0, n_elements(mess) - 1 do printf, save_lun, mess[i]    
    
  valid = 1
  
  write_error:
    if not valid then void = dialog_message("Error when saving log file.", /center, /error)
    if save_lun gt 0 then free_lun, save_lun
  
end

;pro cosmos_log_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;  
;  uname = widget_info(event.id, /uname)
;  
;  case uname of  
;    'LOGCLEAN': cosmos_clean_log, event
;    'LOGSAVE':  cosmos_save_log, event
;    else:  
;  endcase
;
;end

pro cosmos_log_tab, parent = parent

  common cosmos_message, cosmos_info, cosmos_warn

  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /column, TITLE='LOG     ', tab_mode = 1, /base_align_left, uname='LOGTAB', event_pro = 'cosmos_events')
  
  b = widget_base(tab, /column, /base_align_right, uname = "LOGB1")  
  w = widget_text(b, /editable, /sensitive, /scroll, value = cosmos_info, uname = "LOGINFO")  
  w = widget_text(b, /editable, /sensitive, /scroll, value = cosmos_warn, uname = "LOGWARN")
  
  b = widget_base(tab, /ROW, /BASE_ALIGN_TOP)  
  w = widget_button(b, value = "Clean", tooltip = "Clean log and warning windows", xsize = 60, uname = "LOGCLEAN")
  w = widget_button(b, value = "Save", tooltip = "Write contents of log and warning windows into a file", xsize = 60, uname = "LOGSAVE")
  w = widget_button(b, value = "Refresh", tooltip = "Update log during execution", xsize = 60, uname = "LOGREFRESH")
      
end

pro cosmos_take_machine_settings, event
  
  common cosmos_anal, anal
  
  ; Take the numor path.    
  widget_control, widget_info(event.top, find_by_uname = 'PATHDIR'), get_value = path
  
  path = strtrim(path, 2) + path_sep()
      
  wids = [['TAKENUM1','POFF1','D01','OPENOFF1','PIXELWIDTH','SDETD','EDELAY1','SPOFF1','XDH1','XDH2','YDET'], $
          ['TAKENUM2','POFF2','D02','OPENOFF2','PIXELWIDTH','SDETD','EDELAY2','SPOFF2','XDH1','XDH2','YDET'], $
          ['TAKENUM3','POFF3','D03','OPENOFF3','PIXELWIDTH','SDETD','EDELAY3','SPOFF3','XDH1','XDH2','YDET']]
                    
  s = size(wids)
      
  for i = 0, s[2] - 1 do begin
  
    widget_control, widget_info(event.top, find_by_uname = wids[0,i]), get_value = numor
    val = cosmos_getnumors(numor)
    if val ne 0 then begin
      params = cosmos_get_machine_params(path, val)
      if params[0] eq -1 then continue
      widget_control, widget_info(event.top, find_by_uname = wids[1,i]), set_value = params[0]
      widget_control, widget_info(event.top, find_by_uname = wids[2,i]), set_value = params[1]
      widget_control, widget_info(event.top, find_by_uname = wids[3,i]), set_value = params[2]
      widget_control, widget_info(event.top, find_by_uname = wids[4,i]), set_value = params[3]                  
      widget_control, widget_info(event.top, find_by_uname = wids[5,i]), set_value = params[4]                  
      widget_control, widget_info(event.top, find_by_uname = wids[6,i]), set_value = params[5]                  
      widget_control, widget_info(event.top, find_by_uname = wids[7,i]), set_value = params[6]                  
    endif
  endfor
                    
end

;pro cosmos_machine_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;      
;  uname = widget_info(event.id, /uname)
;  
;  case uname of  
;    'TAKE': begin
;         cosmos_take_machine_settings, event
;    end
;    'OVERRIDE': begin    
;      base = widget_info(event.top, find_by_uname = 'MACHBASE')
;      widget_control, base, sensitive = event.select
;    end  
;    else:  
;  endcase
;
;end

pro cosmos_machine_tab, parent = parent
  
  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
    
  tab = widget_base(parent, /column, title='MACHINE  ', tab_mode = 1, /base_align_left, uname = 'MACHTAB', event_pro = 'cosmos_events')
  
  b = widget_base(tab, /column,  /base_align_left, uname = 'MACHBASE')

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " d0(D17),d1(FIGARO) (m)", uname='TOF_DISTANCE', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'D01')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'D02')
  l = widget_label(bb, xsize=20, value = "")  
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'D03')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " Opening Offset", uname='OPENING_OFFSET', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'OPENOFF1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'OPENOFF2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'OPENOFF3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " Poff Offset", uname='POFF_OFFSET', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'POFF1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'POFF2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'POFF3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " TOF delay (seconds)", uname='E_DELAY', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'EDELAY1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'EDELAY2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'EDELAY3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " Sample position offset (m)", uname='SPOFF_OFFSET', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /floating, uname = 'SPOFF1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /floating, uname = 'SPOFF2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /floating, uname = 'SPOFF3')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=205, value = " Pixel Width (m)", uname='PIXEL_WIDTH', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "", xsize = 10, /floating, uname = 'PIXELWIDTH')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=205, value = " Sample-detector distance (m)", uname='DET_DIST', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "", xsize = 10, /floating, uname = 'SDETD')
  l = widget_label(bb, font = c_font_big, xsize=205, value = " (at zero collimation)")

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " Distances (FIGARO)", /ALIGN_LEFT)
  l = widget_label(bb, xsize=40, value = "")
  l = widget_label(bb, xsize=50, value = 'XDH1', uname='X_DH1', /TRACKING_EVENTS)
  w = cw_field(bb, title = '', xsize = 10, /floating, uname = 'XDH1')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=50, value = 'XDH2', uname='X_DH2', /TRACKING_EVENTS)
  w = cw_field(bb, title = '', xsize = 10, /floating, uname = 'XDH2')
  l = widget_label(bb, xsize=20, value = "")
  l = widget_label(bb, xsize=50, value = 'YDET', uname='Y_DET', /TRACKING_EVENTS)
  w = cw_field(bb, title = '', xsize = 10, /floating, uname = 'YDET')

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=150, value = " Take values from numor", uname='TAKE_VALUES', /ALIGN_LEFT, /TRACKING_EVENTS)
  l = widget_label(bb, xsize=40, value = "")
  w = cw_field(bb, title = "Angle 1 ", xsize = 10, /long, uname = 'TAKENUM1')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 2 ", xsize = 10, /long, uname = 'TAKENUM2')
  l = widget_label(bb, xsize=20, value = "")
  w = cw_field(bb, title = "Angle 3 ", xsize = 10, /long, uname = 'TAKENUM3')
  l = widget_label(bb, xsize=20, value = "")
  take = widget_button(bb, value = "Take", uname = "TAKE")

  l = widget_label(b, ysize=10, value = "")

  b = widget_base(tab, /column, /base_align_left)

  bb   = widget_base(b, /row, /base_align_center)
  l = widget_label(bb, font = c_font_big, xsize=185, value = 'Override numors values ', uname='OVERRIDE_VALUES', /ALIGN_LEFT, /TRACKING_EVENTS)
  w = cw_bgroup(bb, [" yes"], font = c_font_big, label_left='', /nonexclusive, uname = "OVERRIDE")
              
end

pro cosmos_io_tab, parent = parent, galaxy = galaxy

  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /column, title='IO     ', tab_mode = 1, /base_align_left, /scroll, uname = 'IOTAB', event_pro = 'cosmos_events')

  b = widget_base(tab, /column, /base_align_left, uname = 'IOBASE')

  bb = widget_base(b, /row, /base_align_center)  
  l = widget_label(bb, font=c_font_big, value=string(' Data path', FORMAT='(A-20)'), xsize=110)
  w = cw_field(bb, title = '', /string, xsize = 50, uname = 'PATHDIR')
  w = widget_button(bb, value = 'Browse', tab_mode = 0, uname = 'PATHBROWSE')

  l = widget_label(b, ysize=10, value = "")
                     
  bb = widget_base(b, /row, /base_align_center)  
  l  = widget_label(bb, font=c_font_big, value=string(" Output Directory", FORMAT='(A-20)'), xsize=110)
  w = cw_field(bb, title = '', xsize = 80, /string, uname = "OUTPUTDIR")
  w = widget_button(bb, value = "Browse", tab_mode = 0, uname = "OUTPUTDIRBROWSE")

  l = widget_label(b, ysize=10, value = "")

  bb = widget_base(b, /row, /base_align_center)  
  l  = widget_label(bb, font=c_font_big, value=string(" Output Basename", FORMAT='(A-20)'), xsize=110)
  w = cw_field(bb, title = '', xsize = 20, /string, uname = "OUTPUTBASE")

  l = widget_label(b, ysize=10, value = "")
    
  bb = widget_base(b, /row, /base_align_center)  
  l  = widget_label(bb, font=c_font_big, value = string(" Output File Format", FORMAT='(A-20)'), xsize=110)
  w = cw_bgroup(bb, ["aft", "dat", "mft", "out", "lam"], /nonexclusive, column = 5, uname = 'OUTPUTFMT')

  if keyword_set(galaxy) then begin

     bb = widget_base(b, /row, /base_align_center)  
     l  = widget_label(bb, font=c_font_big, value = " Parameter for sample position (FIGARO):", xsize = 240)
     w = cw_bgroup(bb, [" #52 (Air/Liquid)", " #49 (Solid/Liquid)"], /exclusive, column=2, set_value=0, uname = "INTERFACE")

     bb = widget_base(b, /row, /base_align_center)
     l  = widget_label(bb, font=c_font_big, value = " Create Sample Subdirectory", xsize = 180)
     w = cw_bgroup(bb, [" yes"], /nonexclusive, uname = "MAKESUBDIR")
  
     bb = widget_base(b, /row, /base_align_center)                        
     l  = widget_label(b, font=c_font_big, value = " Logfile contents", xsize = 180)
     bb = widget_base(b, /row, /base_align_center)                        
     l  = widget_label(bb, value = "", xsize = 40)
     w = cw_bgroup(bb, $
                   ["Sample Age", "Exchanger Position", "Angles", "Times", "Total Count", "Monitor Count", "Data Summary"], $
                   /nonexclusive, uname = "INCLUDE")
                
  endif
        
  bb = widget_base(b, /row, /base_align_center)

  l  = widget_label(bb, font=c_font_big, value = " MFT file contents", xsize = 180)
  w = cw_bgroup(bb, ["Figaro","D17"], /exclusive, column=2, set_value=0, uname="MFTINSTR", /no_release)
      
  sections = ['3\Figaro Parameter ...                                                     ',$
                  '1\Sample environment', $
                  '0\Water bath set point temperature', $
                  '0\Water bath regulation temperature', $
                  '0\Water bath sample temperature', $
                  '0\HPLC pump valve position', $
                  '0\Liquid height number of adjustments', $
                  '0\Liquid height at run start', $
                  '0\Liquid height at run end', $
                  '0\Langmuir requested speed', $
                  '0\Langmuir requested pressure', $
                  '0\Langmuir actual start pressure', $
                  '0\Langmuir actual end pressure', $
                  '0\Langmuir actual mean pressure', $
                  '0\Langmuir actual standard deviation pressure', $
                  '0\Sample changer position number', $
                  '0\Absorption trough position number', $
                  '0\Absorption trough optical offset', $
                  '2\Electronic switch', $
                  '1\Instrument', $
                  '0\Measuring time (s)', $
                  '0\Total counts on detector', $
                  '0\Total counts on monitor 1', $
                  '0\Total counts on monitor 2', $
                  '0\Collimation angle wrt horizontal', $
                  '0\Theta', $
                  '0\dan', $
                  '0\slit s1h', $
                  '0\slit s2h', $
                  '0\slit s2w', $
                  '0\slit s3h', $
                  '0\slit s3w', $
                  '0\beam block s4t', $
                  '0\beam block s4b', $
                  '0\san', $
                  '0\phi', $
                  '0\sht1', $
                  '0\sht2', $
                  '0\str', $
                  '0\First controlling chopper', $
                  '0\Second controlling chopper', $
                  '0\First order dlambda/lambda', $
                  '0\Frame overlap mirror', $
                  '0\Poff', $
                  '2\Open offset', $
                  '3\Data reduction' , $
                  '0\Normalization', $
                  '0\Factor 1', $
                  '0\Factor 2', $
                  '0\Angle 1 theta overwrite', $
                  '0\Angle 2 theta overwrite', $
                  '0\Angle 3 theta overwrite', $
                  '0\Angle 1 width', $
                  '0\Angle 2 width', $
                  '0\Angle 3 width', $
                  '0\Angle 1 left background width', $
                  '0\Angle 2 left background width', $
                  '0\Angle 3 left background width', $
                  '0\Angle 1 left background offset', $
                  '0\Angle 2 left background offset', $
                  '0\Angle 3 left background offset', $
                  '0\Angle 1 right background width', $
                  '0\Angle 2 right background width', $
                  '0\Angle 3 right background width', $
                  '0\Angle 1 right background offset', $
                  '0\Angle 2 right background offset', $
                  '0\Angle 3 right background offset', $
                  '0\Angle 1 grouping (1=yes;0=no)', $
                  '0\Angle 2 grouping (1=yes;0=no)', $
                  '0\Angle 3 grouping (1=yes;0=no)', $
                  '0\Angle 1 grouping value', $
                  '0\Angle 2 grouping value', $
                  '0\Angle 3 grouping value', $
                  '0\Angle 1 fitted position of peak', $
                  '0\Angle 2 fitted position of peak', $
                  '2\Angle 3 fitted position of peak']
                  
  mft_sections = create_struct('figaro', sections)
  
  sections = ['3\D17 Parameter ...                                                     ',$
                  '1\Sample environment', $
                  '0\Temp 1', $
                  '0\Temp 2', $
                  '0\Temp 3', $
                  '0\Temp 4', $
                  '0\Temp 5', $
                  '0\Temp 6', $
                  '0\Sample T set point', $
                  '0\Regulation temperature', $
                  '2\str motor value (mm)', $
                  '1\Instrument', $
                  '0\Measuring time (s)', $
                  '0\Total counts on detector', $
                  '0\Total counts on monitor', $
                  '0\Sample angle', $
                  '0\S2 width', $
                  '0\S3 width', $
                  '0\Chopper1 - Sample distance (m)', $
                  '0\S2 - Sample distance (m)', $
                  '0\S3 - Sample distance (m)', $
                  '0\Sample - Detector distance (m)', $
                  '0\Chopper 1 actual speed', $
                  '0\Chopper 1 opening requested', $
                  '0\Interchopper distance', $
                  '0\Poff', $
                  '2\Open offset', $
                  '3\Data reduction' , $
                  '0\Normalization', $
                  '0\Factor 1', $
                  '0\Factor 2', $
                  '0\Angle 1 theta overwrite', $
                  '0\Angle 2 theta overwrite', $
                  '0\Angle 3 theta overwrite', $
                  '0\Angle 1 width', $
                  '0\Angle 2 width', $
                  '0\Angle 3 width', $
                  '0\Angle 1 left background width', $
                  '0\Angle 2 left background width', $
                  '0\Angle 3 left background width', $
                  '0\Angle 1 left background offset', $
                  '0\Angle 2 left background offset', $
                  '0\Angle 3 left background offset', $
                  '0\Angle 1 right background width', $
                  '0\Angle 2 right background width', $
                  '0\Angle 3 right background width', $
                  '0\Angle 1 right background offset', $
                  '0\Angle 2 right background offset', $
                  '0\Angle 3 right background offset', $
                  '0\Angle 1 grouping (1=yes;0=no)', $
                  '0\Angle 2 grouping (1=yes;0=no)', $
                  '0\Angle 3 grouping (1=yes;0=no)', $
                  '0\Angle 1 grouping value', $
                  '0\Angle 2 grouping value', $
                  '0\Angle 3 grouping value', $
                  '0\Angle 1 fitted position of peak', $
                  '0\Angle 2 fitted position of peak', $
                  '2\Angle 3 fitted position of peak']

  mft_sections = create_struct(mft_sections, 'd17', sections)

  bb = widget_base(b, /row, /base_align_center)

  bbb = widget_base(bb, /column, /base_align_center, uvalue=0, uname="MFTPARAMSFIG")

  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=0, uname="MFTPARAM1")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=1, uname="MFTPARAM2")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=2, uname="MFTPARAM3")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=3, uname="MFTPARAM4")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=4, uname="MFTPARAM5")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=5, uname="MFTPARAM6")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=6, uname="MFTPARAM7")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=7, uname="MFTPARAM8")
  w = cw_pdmenu(bbb, mft_sections.(0), /return_name, uvalue=8, uname="MFTPARAM9")

  bbb = widget_base(bb, /column, /base_align_center, uvalue=1, uname="MFTPARAMSD17")

  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=0, uname="MFTPARAM1")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=1, uname="MFTPARAM2")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=2, uname="MFTPARAM3")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=3, uname="MFTPARAM4")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=4, uname="MFTPARAM5")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=5, uname="MFTPARAM6")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=6, uname="MFTPARAM7")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=7, uname="MFTPARAM8")
  w = cw_pdmenu(bbb, mft_sections.(1), /return_name, uvalue=8, uname="MFTPARAM9")
     
end

pro cosmos_show_plot, event, pidx
  
  common cosmos_results, c_results

  plotlist = widget_info(event.top, find_by_uname = 'PLOTLIST')
    
  if n_elements(pidx) eq 0 then begin  
      pidx = widget_info(plotlist, /list_select)
      if pidx[0] eq -1 then begin
        pidx = 0
      endif
  endif
  
  widget_control, plotlist, set_list_select=pidx
      
  plotNames = tag_names(c_results)
  
  pName = plotNames[pidx]
  pVals = c_results.(pidx)
    
  plotwin = widget_info(event.top, find_by_uname = 'PLOTWIN')  
  widget_control, plotwin, get_value = win_id  
  wset, win_id
  
  pts = where(pVals[2, *] gt 0., nNonZeros)
  
  if nNonZeros eq 0 then begin
    erase
    xyouts, 200, 300, 'Empty plot for ' + pName, /device
    return
  endif
    
  results = fltarr(4, nNonZeros)
  results[0,*] = pVals[0, pts]
  results[1,*] = pVals[1, pts]
  results[2,*] = pVals[2, pts]
  results[3,*] = pVals[3, pts]
                      
  qPlotStyle = widget_info(event.top, find_by_uname = "QPLOTSTYLE")  
  widget_control, qPlotStyle, get_value = qStyle
  idx = widget_info(qPlotStyle, /droplist_select)
  qStyle = qStyle[idx]
    
  rPlotStyle = widget_info(event.top, find_by_uname = "RPLOTSTYLE")  
  widget_control, rPlotStyle, get_value = rStyle
  idx = widget_info(rPlotStyle, /droplist_select)
  rStyle = rStyle[idx]

  x  = results[0, *]
  y  = results[2, *]
  dy = results[3, *]

  case rStyle of
    "R"       : begin
      ytitle = "Reflectivity"      
      ylog = 0
    end    
    "RQ4"     : begin
      ytitle = "Reflectivity * Q^4"
      y = y*x^4
      dy = dy*x^4
      ylog = 0
    end
    "log(R)"  : begin
      ytitle = "log10( Reflectivity )"
      ylog = 1
    end
    "log(RQ4)": begin
      ytitle = "log10( Reflectivity * Q^4)"
      dy = dy*x^4
      y = y*x^4
      ylog = 1
    end
    else:    
  endcase
    
  case qStyle of
    "Q" : begin
      xtitle = "Q, AA-1"      
      xlog=0      
    end    
    "log(Q)" : begin
      xtitle = "Q, AA-1"
      xlog=1      
    end    
    "Q2": begin
      xtitle = "Q^2, AA-2"      
      x = x^2
      xlog=0      
    end
    "Q4": begin
      xtitle = "Q^4, AA-4"      
      x = x^4
      xlog=0      
    end
    else:    
  endcase
                    
  erase
  plot, x, y, title = pName, xtitle = xtitle, /ynozero, ytitle = ytitle, xlog=xlog, ylog = ylog    
  errplot, x, y - dy, y + dy
            
end

pro cosmos_show_first_plot, event

  plotList = widget_info(event.top, find_by_uname = 'PLOTLIST')
  
  nPlots = widget_info(plotList, /list_number)
  
  if nPlots le 0 then return
  
  selPlot = 0
  
  widget_control, plotList, set_list_select = selPlot
  
  cosmos_show_plot, event, selPlot
  
end

pro cosmos_show_previous_plot, event

  plotList = widget_info(event.top, find_by_uname = 'PLOTLIST')
  
  nPlots = widget_info(plotList, /list_number)
  
  if nPlots le 0 then return
  
  selPlot = widget_info(plotList, /list_select)
  
  if selPlot eq -1 then selPlot = 0
      
  if selPlot eq 0 then begin
  
    nPlots = widget_info(plotList, /list_number)  
    selPlot = nPlots - 1
    
  endif else begin
  
    selPlot = selPlot - 1
    
  endelse
    
  widget_control, plotList, set_list_select = selPlot    

  cosmos_show_plot, event, selPlot
      
end

pro cosmos_show_next_plot, event

  plotList = widget_info(event.top, find_by_uname = 'PLOTLIST')

  nPlots = widget_info(plotList, /list_number)
  
  if nPlots le 0 then return
    
  selPlot = widget_info(plotList, /list_select)
    
  if selPlot eq -1 then selPlot = 0
      
  if selPlot eq nPlots - 1 then selPlot = 0 else selPlot = selPlot + 1

  widget_control, plotList, set_list_select = selPlot    
      
  cosmos_show_plot, event, selPlot
  
end

pro cosmos_show_last_plot, event

  plotList = widget_info(event.top, find_by_uname = 'PLOTLIST')

  nPlots = widget_info(plotList, /list_number)
  
  if nPlots le 0 then return
  
  selPlot = nPlots - 1  

  widget_control, plotList, set_list_select = selPlot    
  
  cosmos_show_plot, event, selPlot
  
end

pro cosmos_stop_animation, event

  s = 1
        
  ; Set the uvalue of the "Stop animation" button to 1.
  widget_control, widget_info(event.top, find_by_uname = 'STOP'), set_uvalue = 1
  
  widget_control, widget_info(event.top, find_by_uname = 'FIRST'),      sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PREV'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'NEXT'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'LAST'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PLOTLIST'),   sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PLAY'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'QPLOTSTYLE'), sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'RPLOTSTYLE'), sensitive = s
  
  return

end

pro cosmos_play_animation, event

  s = 0
        
  ; Set the UVALUE of the "Stop animation" button to 0.
  widget_control, widget_info(event.top, find_by_uname = 'STOP'), set_uvalue = 0
        
  ; Show the next plot.
  cosmos_show_next_plot, event
          
  ; Store the animation speed in |val|.
  widget_control, widget_info(event.top, find_by_uname = 'SPEED'), get_value = val
                  
  ; Generate a timer event.
  widget_control, widget_info(event.top, find_by_uname = 'PLAY'), timer = abs(val)

  widget_control, widget_info(event.top, find_by_uname = 'FIRST'),      sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PREV'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'NEXT'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'LAST'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PLOTLIST'),   sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'PLAY'),       sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'QPLOTSTYLE'), sensitive = s
  widget_control, widget_info(event.top, find_by_uname = 'RPLOTSTYLE'), sensitive = s
  
  return
        
end

pro cosmos_run_animation, event

  ; Store the uvalue of the "Stop animation" button in |stopAnim|.
  widget_control, widget_info(event.top, find_by_uname = 'STOP'), get_uvalue = stopAnimation
    
  ; If 'stopAnimation' is 0, show the next plot and generates a new timer event.
  if stopAnimation eq 0 then begin
            
    ; Get the animation speed.
    widget_control, widget_info(event.top, find_by_uname = 'SPEED'), get_value = val

    ; Show the next plot if the animation speed is >= 0.
    if val ge 0 then begin
      cosmos_show_next_plot, event
      
    ; Otherwise, show the previous plot if the animation speed is >= 0.
    endif else begin
      cosmos_show_previous_plot, event
    
    endelse
            
    ; Generate a new timer event.
    widget_control, widget_info(event.top, find_by_uname = 'PLAY'), timer = abs(val)
      
  endif

end

;pro cosmos_plot_tab_events, event
;
;  catch, theError
;  if theError ne 0 then begin
;    catch, /cancel
;    void = dialog_message(!error_state.msg, /center, /error)
;    return
;  endif
;
;  eventType = tag_names(event, /structure_name)
;  
;  uname = widget_info(event.id, /uname)
;
;  if eventType eq 'WIDGET_TIMER' then begin
;  
;    case uname of
;      'PLAY' : cosmos_run_animation, event
;      else:    
;    endcase
;  
;  endif else begin
;    
;    case uname of
;  
;      'PLOTLIST'   : cosmos_show_plot, event
;      'FIRST'      : cosmos_show_first_plot, event
;      'PREV'       : cosmos_show_previous_plot, event
;      'NEXT'       : cosmos_show_next_plot, event
;      'LAST'       : cosmos_show_last_plot, event
;      'STOP'       : cosmos_stop_animation, event
;      'PLAY'       : cosmos_play_animation, event
;      'QPLOTSTYLE' : cosmos_show_plot, event
;      'RPLOTSTYLE' : cosmos_show_plot, event
;      else:
;  
;    endcase
;    
;  endelse
;          
;end

pro cosmos_plot_tab, parent = parent
    
  tab = widget_base(parent, /column, title='PLOT     ', tab_mode = 1, /base_align_left, uname = 'PLOTTAB', event_pro = 'cosmos_events')
  
;  plotb1 = widget_base(tab, /column, /base_align_center, uname = 'PLOTB1')
  
  w = widget_draw(tab, uname = 'PLOTWIN',Xsize=600,Ysize=300)  

  w = widget_list(tab, scr_xsize=600, scr_ysize = 120, uname = 'PLOTLIST')
            
  bb = widget_base(tab, /row, /base_align_center, uname='PLOTACTIONS')

  w = widget_droplist(bb, title = "f(R)", value = ["log(R)","R","RQ4","log(RQ4)"], uname = "RPLOTSTYLE")
  w = widget_droplist(bb, title = "f(Q)", value = ["Q","log(Q)","Q2","Q4"], uname = "QPLOTSTYLE")

  l  = widget_label(bb, xsize=30, value='')

  w = widget_button(bb, value=str_centerize('First',8), tooltip = "Display first plot", uname = 'FIRST')
  w = widget_button(bb, value=str_centerize('Previous',8), tooltip = "Display previous plot", uname = 'PREV')
  w = widget_button(bb, value=str_centerize('Next',8), tooltip = "Display next plot", uname = 'NEXT')
  w = widget_button(bb, value=str_centerize('Last',8), tooltip = "Display last plot", uname = 'LAST')

  l  = widget_label(bb, xsize=30, value='')

  w = widget_button(bb, value=str_centerize('Stop',8), uvalue = 0, tooltip = "Stop plot animation", uname = "STOP")
  w = cw_field(bb, title="Rate", value = 1.0, xsize = 8, /floating, uname = "SPEED")
  w = widget_button(bb, value=str_centerize('Play',8), tooltip = "Play plot animation", uname = "PLAY")
      
end

pro cosmos_widgets

end
