;+
;************************************************************************ 
;*** TRAJECTORY OF A NEUTRON 
;************************************************************************
;-

function parabolicTrajectory,startPoint,stopPoint

g =9810
mNeutron=1.674928e-27 

spx=startPoint.x
spy=startPoint.y
epx=stopPoint.x
epy=stopPoint.y
dx=startPoint.vx
dy=startPoint.vy


;* compute first 100 points for the trajectory
x=findgen(100)/100 *(epx-spx)+spx;
y=((2*dy*(x - spx))/dx - (g*(x - spx)^2)/dx^2 + 2*spy)/2.

return,[[x],[y]]
end

;+
;************************************************************************ 
;*** PARABOLA VALUE
;************************************************************************
;-
function parabola,point,xValue
g =9810
spx=point.x
spy=point.y
dx=point.vx
dy=point.vy

yvalue=(2*dx^2*spy - g*(spx - xValue)^2 + 2*dx*dy*(-spx + xValue))/(2.*dx^2)
return,{x:xValue,y:yvalue,vx:point.vx,vy:point.vy,exists:1,distance:sqrt((spx-xValue)^2+(spy-yValue)^2)}

end

;+
;************************************************************************ 
;*** STRAIGHT LINE VALUE
;************************************************************************
;-
function straightLine,point,xValue

spx=point.x
spy=point.y
dx=point.vx
dy=point.vy

if(dx NE 0) then begin
	yvalue=(dx*spy + dy*(-spx + xValue))/dx
	return,{x:xValue,y:yvalue,vx:point.vx,vy:point.vy,exists:1,distance:sqrt((spx-xValue)^2+(spy-yValue)^2)}
end else begin
	return,{x:point.x,y:point.y,vx:point.vx,vy:point.vy,exists:0,distance:0.0}
	
end

end

;+
;************************************************************************ 
;*** DRAWING AND ELEMENT
;************************************************************************
;-
pro drawElement,element,col

	siz=size(element.segments)
	nbSegments=siz[2]
	
	if(nbSegments GE 3) then begin
		polyfill,element.segments[0,*],element.segments[1,*],/data,color=col
	end else begin	
		plots,element.segments[0,*],element.segments[1,*],/data,color=col,thick=2
	end
	
end

;+
;************************************************************************ 
;*** PLOT NEUTRON TRAJECTORY
;************************************************************************
;-
pro plotNeutronTrajectory,path,col

siz=size(path)
n=siz[1]
lastPoint=path[0]
i=1;
presentPoint=path[i]
while(presentPoint.exists AND i LT n) do begin
	
	trajectory=parabolicTrajectory(lastPoint,presentPoint)
	
	plots,[lastPoint.x,presentPoint.x],[lastPoint.y,presentPoint.y],/data,color=col,psym=6
	plots,trajectory[*,0],trajectory[*,1],/data,color=col

	lastPoint=presentPoint
	i++
	presentPoint=path[i]
end

end

;+
;************************************************************************ 
;*** PLOT STRAIGHTLINE TRAJECTORY
;************************************************************************
;-
pro plotStraightLineTrajectory,path,col

siz=size(path)
n=siz[1]
lastPoint=path[0]
i=1;
presentPoint=path[i]
while(presentPoint.exists AND i LT n) do begin
	
	trajectory=parabolicTrajectory(lastPoint,presentPoint)
	
	;plots,[lastPoint.x,presentPoint.x],[lastPoint.y,presentPoint.y],/data,color=col,psym=6
	plots,[lastPoint.x,presentPoint.x],[lastPoint.y,presentPoint.y],/data,color=col

	lastPoint=presentPoint
	i++
	presentPoint=path[i]
end

end

;+
;************************************************************************ 
;*** PLOTTING INTERSECTION
;************************************************************************
;-
pro plotIntersection,intersection,col
	plots,[intersection[0],intersection[0]],[intersection[1],intersection[1]],color=col,thick=1,/data,psym=6
end

;+
;************************************************************************ 
;*** POINT ON SEGMENT
;************************************************************************
;-
function pointOnSegment,point,segment

p1x=segment[0]
p1y=segment[1]
p2x=segment[2]
p2y=segment[3]
xi=point[0]
yi=point[1]

return,abs( -Sqrt((-p1x + p2x)^2 + (-p1y + p2y)^2) + Sqrt((-p1x + xi)^2 + (-p1y + yi)^2) + Sqrt((-p2x + xi)^2 + (-p2y + yi)^2)) LE 1e-3

end

;+
;************************************************************************ 
;*** GENERAL INTERSECTION FUNCTIONS
;*** input: segment, a point
;*** output: point
;************************************************************************
;-
function intersectionOfParabola,point,segment

g =9810

p1x=segment[0]
p1y=segment[1]
p2x=segment[2]
p2y=segment[3]
spx=point.x
spy=point.y
dx=point.vx
dy=point.vy
reflecting=segment[4]
noInteractionPoint=point
noInteractionPoint.exists=0

;print,'segment=',p1x,p1y,p2x,p2y

if(dx NE 0) then begin
	; case when the start speed is not vertical
	
	if(p1x NE p2x) then begin ;**** Not a vertical segment
	
	xi1 =  (dx*dy*(p1x - p2x) + dx^2*(-p1y + p2y) + g*(p1x - p2x)*spx - $
    Sqrt(dx^2*(dy^2*(p1x - p2x)^2 + dx^2*(p1y - p2y)^2 + 2*dx*dy*(p1x - p2x)*(-p1y + p2y) + $
    2*g*(p1x - p2x)*(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx + p1x*spy - p2x*spy))))/(g*(p1x - p2x))

    yi1= (dx*dy*(p1x - p2x)*(p1y - p2y) - dx^2*(p1y - p2y)^2 + g*(p1x - p2x)*(p2y*(p1x - spx) + p1y*(-p2x + spx)) + $
    (-p1y + p2y)*Sqrt(dx^2*(dy^2*(p1x - p2x)^2 + dx^2*(p1y - p2y)^2 + 2*dx*dy*(p1x - p2x)*(-p1y + p2y) + $
    2*g*(p1x - p2x)*(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx + p1x*spy - p2x*spy))))/(g*(p1x - p2x)^2)

    xi2 = (dx*dy*(p1x - p2x) + dx^2*(-p1y + p2y) + g*(p1x - p2x)*spx + $
    Sqrt(dx^2*(dy^2*(p1x - p2x)^2 + dx^2*(p1y - p2y)^2 + 2*dx*dy*(p1x - p2x)*(-p1y + p2y) + $
    2*g*(p1x - p2x)*(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx + p1x*spy - p2x*spy))))/(g*(p1x - p2x))

	yi2 = (dx*dy*(p1x - p2x)*(p1y - p2y) - dx^2*(p1y - p2y)^2 + g*(p1x - p2x)*(p2y*(p1x - spx) + p1y*(-p2x + spx)) + $
    (p1y - p2y)*Sqrt(dx^2*(dy^2*(p1x - p2x)^2 + dx^2*(p1y - p2y)^2 + 2*dx*dy*(p1x - p2x)*(-p1y + p2y) + $
    2*g*(p1x - p2x)*(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx + p1x*spy - p2x*spy))))/(g*(p1x - p2x)^2)

	;print,"intersectionOfParabola",xi1,yi1,xi2,yi2

	; which solution is correct
	
	;// compute distances
	d1=sqrt((xi1-spx)^2+(yi1-spy)^2)
	d2=sqrt((xi2-spx)^2+(yi2-spy)^2)
	
	;// on segments
	onSegment1=pointOnSegment([xi1,yi1],segment)
	onSegment2=pointOnSegment([xi2,yi2],segment)
	;print,"onSegment1",onSegment1
	;print,"onSegment2",onSegment2
	
	;// compute reflected speed
	vInx=dx
	vIny=dy
	vOutx=((p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vInx + 2*(p1x - p2x)*(p1y - p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOuty=(2*(p1x - p2x)*(p1y - p2y)*vInx - (p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOutx*=reflecting
	vOuty*=reflecting
	
	if(onSegment1 AND NOT onSegment2) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1};
	if(NOT onSegment1 AND onSegment2) then return,{x:xi2,y:yi2,vx:vOutx,vy:vOuty,exists:1,distance:d2};
	if(NOT onSegment1 AND NOT onSegment2) then begin
				return,noInteractionPoint;
				end
	; *** from here both are correct, take the minimum distance one
	if(d1 GT d2) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1} else return,{x:xi2,y:yi2,vx:vOutx,vy:vOuty,exists:1,distance:d2}
	end else begin
			; ******** a vertical segment
			xi1=p1x
			yi1=(2*dx^2*spy - g*(spx - xi1)^2 + 2*dx*dy*(-spx + xi1))/(2.*dx^2)						
		
			;print,"intersectionOfParabola (vertical segment)",xi1,yi1
		
			vInx=dx
			vIny=dy
			vOutx=((p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vInx + 2*(p1x - p2x)*(p1y - p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
			vOuty=(2*(p1x - p2x)*(p1y - p2y)*vInx - (p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
			vOutx*=reflecting
			vOuty*=reflecting	
			
						
			
			
	
			;// compute distances
			d1=sqrt((xi1-spx)^2+(yi1-spy)^2)
			onSegment1=pointOnSegment([xi1,yi1],segment)
			;print,"intersectionOfParabola onSegment1",onSegment1
			if(onSegment1) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1} else return,noInteractionPoint;
	
	end
	
end else begin
	; case when dx = 0, the trajectory is a vertical straight line

	xi1=spx
	yi1=(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx)/(-p1x + p2x)
	ytop=dy^2/(2.*g) + spy
	
	vInx=dx
	vIny=dy
	vOutx=((p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vInx + 2*(p1x - p2x)*(p1y - p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOuty=(2*(p1x - p2x)*(p1y - p2y)*vInx - (p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOutx*=reflecting
	vOuty*=reflecting	
	
	;// compute distances
	d1=sqrt((xi1-spx)^2+(yi1-spy)^2)
	onSegment1=(onSegment1=pointOnSegment([xi1,yi1],segment)  AND yi1 GT ytop)
	if(onSegment1) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1}
	return,noInteractionPoint;
end


end

;+
;***** NOW FOR A STRAIGHT LINE
;-
function intersectionOfStraightLine,point,segment

g =9810

p1x=segment[0]
p1y=segment[1]
p2x=segment[2]
p2y=segment[3]
spx=point.x
spy=point.y
dx=point.vx
dy=point.vy
reflecting=segment[4]

noInteractionPoint=point
noInteractionPoint.exists=0

if(dx NE 0) then begin
	; case when the start speed is not vertical
	
	xi1 =  (dy*(-p1x + p2x)*spx + dx*(p1y*p2x - p1x*p2y + p1x*spy - p2x*spy))/(dy*(-p1x + p2x) + dx*(p1y - p2y))
    yi1= (dy*(p1y*(p2x - spx) + p2y*(-p1x + spx)) + dx*(p1y - p2y)*spy)/(dy*(-p1x + p2x) + dx*(p1y - p2y))
    alpha1= ((-p1x + p2x)*(-p1x + xi1) + (-p1y + p2y)*(-p1y + yi1))/ Sqrt((-p1x + p2x)^2 + (-p1y + p2y)^2 + (-p1x + xi1)^2 + (-p1y + yi1)^2)
	
	;// compute distances
	d1=sqrt((xi1-spx)^2+(yi1-spy)^2)

	;// on segments
	onSegment1=pointOnSegment([xi1,yi1],segment)
	
	;// compute reflected speed
	vInx=dx
	vIny=dy
	vOutx=((p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vInx + 2*(p1x - p2x)*(p1y - p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOuty=(2*(p1x - p2x)*(p1y - p2y)*vInx - (p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOutx*=reflecting
	vOuty*=reflecting	
	
	if(onSegment1) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1};
	return,noInteractionPoint;
	
end else begin
	; case when dx = 0, the trajectory is a vertical straight line

	xi1=spx
	yi1=(p1y*p2x - p1x*p2y - p1y*spx + p2y*spx)/(-p1x + p2x)
	
	vInx=dx
	vIny=dy
	vOutx=((p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vInx + 2*(p1x - p2x)*(p1y - p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOuty=(2*(p1x - p2x)*(p1y - p2y)*vInx - (p1x + p1y - p2x - p2y)*(p1x - p1y - p2x + p2y)*vIny)/((p1x - p2x)^2 + (p1y - p2y)^2)
	vOutx*=reflecting
	vOuty*=reflecting

	
	;// compute distances
	d1=sqrt((xi1-spx)^2+(yi1-spy)^2)
	onSegment1=pointOnSegment([xi1,yi1],segment)
	if(onSegment1) then return,{x:xi1,y:yi1,vx:vOutx,vy:vOuty,exists:1,distance:d1}
	return,noInteractionPoint;
end


end

;+
;************************************************************************ 
;*** INTERSECTION BETWEEN A RAY AND AN ELEMENT
;*** input: a point, an element
;*** output: intersection point
;************************************************************************
;-
function intersectionWithElement,point,element,useGravity
	
	siz=size(element.segments)
	nbSegments=siz[2]
	
	;*** strategy: retain the existing intersection with the smallest distance.
	;*** if no intersection exist, then return the incoming ray point
	
	resultIntersection=point;
	resultIntersection.exists=0
	resultIntersection.distance=9999.99 ; make the distance very very long ... dirty for testing distances bu working
	
	for i=0,nbSegments-1 do begin
	
		segment=element.segments[*,i]
		if(useGravity) then intersection=intersectionOfParabola(point,segment) $
		else intersection=intersectionOfStraightLine(point,segment)
		
		;print,'intersectionWithElement',intersection
		
		if(intersection.exists) then begin ; *** the intersection exists
				;print,'intersectionWithElement intersection exists'
			if(intersection.distance LT resultIntersection.distance) then resultIntersection = intersection;			
		end
		
		
		
	end
	return,resultIntersection
end

;+
;************************************************************************ 
;*** INTERSECTION BETWEEN A RAY AND AN SEVERAL ELEMENTS
;*** input: a point, the elements
;*** output: intersection points array
;************************************************************************
;-
function neutronPath,point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,sample,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom,slit4Top,slit4Bottom,detector,detectorIntersection

;*** init
previousIntersection=point;
emptyPoint={x:0.0,y:0.0,vx:0.0,vy:0.0,exists:0,distance:0.0}


intersections=replicate(emptyPoint,16)	   
intersections[0]=point
indexIntersection=1

; *** slit 1
;print,">>>> doing slit1Top intersection "
intersection=intersectionWithElement(previousIntersection,slit1Top,1);
if(intersection.exists) then begin
			;print,">>>>slit1Top intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing slit1Bottom intersection "
intersection=intersectionWithElement(previousIntersection,slit1Bottom,1);
if(intersection.exists) then begin
			;print,">>>>slit1Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end  

end

;**** mirror 1
;print,">>>> doing mirror1Top intersection "
intersection=intersectionWithElement(previousIntersection,mirror1Top,1);
if(intersection.exists) then begin
			;print,">>>>mirror1Top intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 
;print,">>>> doing mirror1Bottom intersection "
intersection=intersectionWithElement(previousIntersection,mirror1Bottom,1);
if(intersection.exists) then begin
			;print,">>>>mirror1Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 



;**** mirror 2
;print,">>>> doing mirror2Top intersection "
intersection=intersectionWithElement(previousIntersection,mirror2Top,1);
if(intersection.exists) then begin
			;print,">>>>mirror2Top intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 

;print,">>>> doing mirror2Bottom intersection "
intersection=intersectionWithElement(previousIntersection,mirror2Bottom,1);
if(intersection.exists) then begin
			;print,">>>>mirror2Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 


; *** slit 2
;print,">>>> doing slit2Top intersection "
intersection=intersectionWithElement(previousIntersection,slit2Top,1);
if(intersection.exists) then begin
			;print,"slit2Top intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing slit2Bottom intersection "
intersection=intersectionWithElement(previousIntersection,slit2Bottom,1);
if(intersection.exists) then begin
			;print,"slit2Bottom intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end 
end


; *** diaphragm 3
;print,">>>> doing bsTop intersection "
intersection=intersectionWithElement(previousIntersection,bsTop,1);
;print,">>>>  bsTop intersection ",intersection
if(intersection.exists) then begin
			;print,"bsTop intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing diaphragm3Bottom intersection "
intersection=intersectionWithElement(previousIntersection,diaphragm3Bottom,1);
;print,">>>>  diaphragm3Bottom intersection ",intersection
if(intersection.exists) then begin
			;print,"diaphragm3Bottom intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end 
end

; *** slit 3
;print,">>>> doing slit3Top intersection "
intersection=intersectionWithElement(previousIntersection,slit3Top,1);
;print,">>>>  slit3Top intersection ",intersection
if(intersection.exists) then begin
			;print,"slit3Top intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing slit3Bottom intersection "
intersection=intersectionWithElement(previousIntersection,slit3Bottom,1);
;print,">>>>  slit3Bottom intersection ",intersection
if(intersection.exists) then begin
			;print,"slit3Bottom intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end 
end


;**** sample
;print,">>>> doing sample intersection "
intersection=intersectionWithElement(previousIntersection,sample,1);
if(intersection.exists) then begin
			;print,"sample intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 

; *** slit 2
;print,">>>> doing slit4Top intersection "
intersection=intersectionWithElement(previousIntersection,slit4Top,1);
if(intersection.exists) then begin
			;print,"slit4Top intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing slit4Bottom intersection "
intersection=intersectionWithElement(previousIntersection,slit4Bottom,1);
if(intersection.exists) then begin
			;print,"slit4Bottom intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			return,intersections
end 
end 

;**** detector
;print,">>>> doing detector intersection "
intersection=intersectionWithElement(previousIntersection,detector,1);
if(intersection.exists) then begin
			;print,"sample intersection ",previousIntersection,intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
			detectorIntersection=[1,intersection.y]
end else detectorIntersection=[0,0] ; *** no intersection



;**** finishes with the end point 
;print,"Make end point"
endPoint={x:float(scale[1]),y:parabola(previousIntersection,scale[1]),vx:0.0,vy:0.0,exists:1,distance:0.0}
intersections[indexIntersection]=parabola(previousIntersection,scale[1]);

return,intersections

end

;+
;************************************************************************ 
;*** COMPUTING THE PATH FOR AN UMBRA/PENUMBRA ray
;*** input: a point, an element
;*** output: intersection points array
;************************************************************************
;-
function straightPath,point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom


previousIntersection=point;
emptyPoint={x:0.0,y:0.0,vx:0.0,vy:0.0,exists:0,distance:0.0}


intersections=replicate(emptyPoint,16)	   
intersections[0]=point
indexIntersection=1

;**** diaphragm 3
;print,">>>> doing bsTop intersection "
intersection=intersectionWithElement(previousIntersection,bsTop,0);
if(intersection.exists) then begin
			;print,">>>>bsTop intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing diaphragm3Bottom intersection "
intersection=intersectionWithElement(previousIntersection,diaphragm3Bottom,0);
if(intersection.exists) then begin
			;print,">>>>diaphragm3Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end 
end



;**** mirror 2
;print,">>>> doing mirror2Top intersection "
intersection=intersectionWithElement(previousIntersection,mirror2Top,0);
if(intersection.exists) then begin
			;print,">>>>mirror2Top intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 

;print,">>>> doing mirror2Bottom intersection "
intersection=intersectionWithElement(previousIntersection,mirror2Bottom,0);
if(intersection.exists) then begin
			;print,">>>>mirror2Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 

;**** mirror 1
;print,">>>> doing mirror1Top intersection "
intersection=intersectionWithElement(previousIntersection,mirror1Top,0);
if(intersection.exists) then begin
			;print,">>>>mirror1Top intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end 
;print,">>>> doing mirror1Bottom intersection "
intersection=intersectionWithElement(previousIntersection,mirror1Bottom,0);
if(intersection.exists) then begin
			;print,">>>>mirror1Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			indexIntersection++
			previousIntersection=intersection
end

; *** slit 1
;print,">>>> doing slit1Top intersection "
intersection=intersectionWithElement(previousIntersection,slit1Top,0);
if(intersection.exists) then begin
			;print,">>>>slit1Top intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end else begin
;print,">>>> doing slit1Bottom intersection "
intersection=intersectionWithElement(previousIntersection,slit1Bottom,0);
if(intersection.exists) then begin
			;print,">>>>slit1Bottom intersection ",intersection
			intersections[indexIntersection]=intersection
			return,intersections
end  

end



;**** finishes with the end point 
;print,"Make end point"
intersections[indexIntersection]=straightLine(previousIntersection,scale[0]);

return,intersections


end

;+
;************************************************************************ 
;*** DRAWING UMBRA AND PENUMBRA 
;************************************************************************
;-
pro drawUmbra,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom,slit4Top,slit4Bottom

colorUmbra='B3B3B3'x

; UPPER 
segmentSlit2=slit2Top.segments
segmentSlit3=slit3Top.segments
spx=segmentSlit3[0]
spy=segmentSlit3[1]
dx=segmentSlit2[0]-segmentSlit3[0]
dy=segmentSlit2[1]-segmentSlit3[1]

point={x:spx,y:spy,vx:dx,vy:dy,exists:0,distance:0.0}
path=straightPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom)
		 
plotStraightLineTrajectory,path,colorUmbra
pointTowardSample=straightLine({x:spx,y:spy,vx:-dx,vy:-dy,exists:0,distance:0.0},scale[1])
plots,[point.x,pointTowardSample.x],[point.y,pointTowardSample.y],color=colorUmbra


; DOWN
segmentSlit2=slit2Bottom.segments
segmentSlit3=slit3Bottom.segments
spx=segmentSlit3[0]
spy=segmentSlit3[1]
dx=segmentSlit2[0]-segmentSlit3[0]
dy=segmentSlit2[1]-segmentSlit3[1]

point={x:spx,y:spy,vx:dx,vy:dy,exists:0,distance:0.0}
path=straightPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom)		 

plotStraightLineTrajectory,path,colorUmbra
pointTowardSample=straightLine({x:spx,y:spy,vx:-dx,vy:-dy,exists:0,distance:0.0},scale[1])
plots,[point.x,pointTowardSample.x],[point.y,pointTowardSample.y],color=colorUmbra



; CROSS
segmentSlit2=slit2Bottom.segments
segmentSlit3=slit3Top.segments
spx=segmentSlit3[0]
spy=segmentSlit3[1]
dx=segmentSlit2[0]-segmentSlit3[0]
dy=segmentSlit2[1]-segmentSlit3[1]

point={x:spx,y:spy,vx:dx,vy:dy,exists:0,distance:0.0}
path=straightPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom)
		 
plotStraightLineTrajectory,path,colorUmbra
pointTowardSample=straightLine({x:spx,y:spy,vx:-dx,vy:-dy,exists:0,distance:0.0},scale[1])
plots,[point.x,pointTowardSample.x],[point.y,pointTowardSample.y],color=colorUmbra


; CROSS
segmentSlit2=slit2Top.segments
segmentSlit3=slit3Bottom.segments
spx=segmentSlit3[0]
spy=segmentSlit3[1]
dx=segmentSlit2[0]-segmentSlit3[0]
dy=segmentSlit2[1]-segmentSlit3[1]

point={x:spx,y:spy,vx:dx,vy:dy,exists:0,distance:0.0}
path=straightPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,slit1Top,slit1Bottom,$	
		 slit2Top,slit2Bottom,bsTop,diaphragm3Bottom,slit3Top,slit3Bottom)
		 
plotStraightLineTrajectory,path,colorUmbra
pointTowardSample=straightLine({x:spx,y:spy,vx:-dx,vy:-dy,exists:0,distance:0.0},scale[1])
plots,[point.x,pointTowardSample.x],[point.y,pointTowardSample.y],color=colorUmbra


end

;+
;************************************************************************ 
;*** MAIN 
;************************************************************************
;-
pro figaro,thetaParam,s2h

;scale

xMin = 0
xMax = 10000.0 ; max is detector position
yMin=-400
yMax=400


scale=[xMin,xMax,yMin,yMax];
lambda1C=0
lambda2C=0


;*********************************
;***** GEOMETRICAL CONSTANTS *****
;*********************************
ban=-0.67 *!pi / 180
guideTilt=ban


s1l=2105.36
m1l=2281.63 - 92.5
m1h1l=2281.63
m1h2l=3531.63
m2l=3736.63 - 55.5
m2h1l=3736.63 
m2h2l=3736.63  + 590
s2l=4550
col=4562.63 + 24
coh1l=4562.63 + 225
coh2l=4562.63 + 225 + 1450
bsh1l=6568.63 
;bsh1l = coh2l +393
bsh=21.4
sl=7099.63
s4l=sl+200
dh1l=sl+1000
dh2l=sl+2000
dl=sl+2500

s1Length=80
m1Length=1448
m2Length=803
coLength=1900
bsLength=80
coThickness=80
coAperture=50
mAperture=50
mThickness=80
diaph3b=200
s3DeltaZ=100
samplelength=200
detHeight=300
beamAperture=30

PVM1Y = -150 ;// this is the pivot distance for mirror 1
PVM2Y = -150 ;// this is the pivot distance for mirror 2
PVCOY = -120  ;// this is the pivot distance for collimation guide

;*********************************
;***** SCATTERING CONDITIONS *****
;*********************************
theta=thetaParam* !pi / 180
dan=2*theta +0 *!pi/180
san=-ban ; for liquids


; *** CHOICE OF WICH MIRRORS AND SIDES WILL BE USED, DEPENDING ON THETA
if(theta GE 2* !pi / 180 AND theta LE 4* !pi / 180) then begin	
	useTopSide1=1
	useTopSide2=1
	useBottomSide1=0
	useBottomSide2=0
end else $
if(theta GT -ban AND theta LT 2* !pi / 180) then begin
	useTopSide1=1
	useTopSide2=0
	useBottomSide1=0
	useBottomSide2=0
end else $
if(theta EQ -ban) then begin	
	useTopSide1=0
	useTopSide2=0
	useBottomSide1=0
	useBottomSide2=0
end else $
if(theta LT -ban AND theta GT -2* !pi / 180) then begin
	useTopSide1=0
	useTopSide2=0
	useBottomSide1=1
	useBottomSide2=0
end else $
if(theta LE -2* !pi / 180 AND theta GE -4* !pi / 180) then begin	
	useTopSide1=0
	useTopSide2=0
	useBottomSide1=1
	useBottomSide2=1
end else begin  
	useTopSide1=0
	useTopSide2=0
	useBottomSide1=0
	useBottomSide2=0
	print,"THETA OUT OF RANGE !!!"
end

useMirror1=useTopSide1 || useBottomSide1
useMirror2=useTopSide2 || useBottomSide2

; **** COMPUTE SOLUTIONS FOR THETA AND THE MIRROR CHOICE 

if(useMirror1 AND useMirror2) then begin
		print,"Case useMirror1 AND useMirror2"
		; set the collimation
		coa=-ban-theta
		; set mirror angles
		m1a=coa/4.0
		m2a=coa*3/4.0

		; set m1h in the beam, and mh2 such that the beam hits is center
		m1h=0
		m2h=(m1h1l + m1h2l - m2h1l - m2h2l)/2.*tan(-m1a*2)
		
		; define the height of the sample such that the beam hits the mirror 2 in its center
		sht=m2h + ((-m2h1l - m2h2l)/2. + sl)*tan(coa)
		
		; defining  the detector height
		dh=sht+(dl - sl)*tan(coa+dan)
		
		; define then the height of the collimation, from the postion
		; of the mirror 2, and its angle
		coh=m2h + ((coh1l + coh2l)/2. - (m2h1l +m2h2l)/2.)*tan(coa)
		
		; compute the position of the slits S2 and S3
		;s2y=m2h + ((-m2h1l - m2h2l)/2. + s2l)*tan(coa)
		;s3y=m2h + ((-m2h1l - m2h2l)/2. + bsh1l + s3DeltaZ)*tan(coa)
		; compute the position of slit 4
		;s4y=sht+(s4l-sl)*sin(-2*coa+2*ban)
			
		; mean angles of incidence
		theta1=abs(m1a)
		theta2=m2a-2*m1a
		lambda1C=4*!pi*sin(theta1)/(0.022*4)
		lambda2C=4*!pi*sin(theta2)/(0.022*4)
		print,"Critical lambda mirror 1 =",lambda1C
		print,"Critical lambda mirror 2 =",lambda2C

end else $
if(useMirror1 AND NOT useMirror2) then begin
		print,"Case useMirror1 AND NOT useMirror2"

		; set the collimation
		coa=-theta-ban
		
		; set m1h in the beam and its angle equal to half theta
		m1h=0
		m1a=coa/2
		; set mirror2 to be untouched, meaning in the middle of the beam
		useTopSide2=1
		m2h=mAperture/2+((m2h2l+m2h1l)/2 -(m1h2l+m1h1l)/2) *tan(coa)
		m2a=coa
		
		; define the height of the sample such that the beam hits the mirror 1 in its center
		sht=m1h + ((-m1h1l - m1h2l)/2. + sl)*Tan(coa)
		
		; defining  the detector height
		dh=sht+(dl - sl)*tan(coa+dan)
		
		
		; define then the height of the collimation
		coh=m1h + ((coh1l + coh2l)/2. + (-m1h1l - m1h2l)/2.)*tan(coa)
		
		
		; compute the position of the slits S2 and S3
		;s2y=m1h + ((-m1h1l - m1h2l)/2. + s2l)*tan(coa)
		;s3y=m1h + ((-m1h1l - m1h2l)/2. + bsh1l + s3DeltaZ)*tan(coa)
		; compute the position of slit 4
		;s4y=sht+(s4l-sl)*sin(coa)		

		; mean angles of incidence
		theta1=abs(m1a)
		lambda1C=4*!pi*sin(theta1)/(0.022*4)
		print,"Critical lambda mirror 1 =",lambda1C
		

end else $
if(NOT useMirror1 AND useMirror2) then begin
		print,"NOT useMirror1 AND useMirror2"

		; set the collimation
		coa=-theta-ban
		
		; set m2h in the beam and its angle equal to half theta
		m2h=0
		m2a=coa/2
		; set mirror2 to be untouched, meaning in the middle of the beam
		m1h=mAperture/2
		m1a=0
		
		; define the height of the sample such that the beam hits the mirror 2 in its center
		sht=m2h + ((-m2h1l - m2h2l)/2. + sl)*Tan(coa)
		
		; defining  the detector height
		dh=sht+(dl - sl)*tan(coa+dan)
		
		; define then the height of the collimation
		coh=m2h + ((coh1l + coh2l)/2. + (-m2h1l - m2h2l)/2.)*tan(coa)

		; compute the position of the slits S2 and S3
		;s2y=m2h - ((-m2h1l - m2h2l)/2. + s2l)*tan(coa)
		;s3y=m2h - ((-m2h1l - m2h2l)/2. + bsh1l + s3DeltaZ)*tan(coa)
		; compute the position of slit 4
		;s4y=sht+(s4l-sl)*sin(coa)
		


		; mean angles of incidence
		theta2=abs(m2a)
		lambda2C=4*!pi*sin(theta2)/(0.022*4)
		print,"Critical lambda mirror 2 =",lambda2C

end else $
if(NOT useMirror1 AND NOT useMirror2) then begin

		print,"NOT useMirror1 AND NOT useMirror2"
		; set mirrors such that beams get straight through
		m2h=mAperture/2
		m2a=0.0
		m1h=mAperture/2
		m1a=0.0
		
		; set the collimation angle
		coa=-theta-ban
		
		; define the height of the sample such that the beam hits the guide in its center 
		sht=0
		
		; define then the height of the collimation
		coh=sht+(sl-((bsh1l+s3DeltaZ)+s2l)/2)*tan(coa)
		
		; defining  the detector height
		dh=(dl - sl)*tan(coa+dan)
		
		; compute the position of the slits S2 and S3
		;s2y= - ( s2l)*tan(coa)
		;s3y= - (s3DeltaZ)*tan(coa)
		; compute the position of slit 4
		;s4y=sht+(s4l-sl)*sin(coa)

end

;// Print virtual axis here
print, "coa=",coa/!pi*180.
print, "m1a=",m1a/!pi*180.
print, "m2a=",m2a/!pi*180.
print, "m1h=",m1h
print, "m2h=",m2h
print, "sht=",sht
print, "dh=",dh
print, "coh=",coh
;print, "theta1=",theta1/!pi*180.
;print, "theta2=",theta2/!pi*180.

; // TEST FORCING ABSTRACT VALUES


;***************************
;***** ABSTRACT VALUES *****
;***************************
s1h=60
;s2h=8
s3h=s2h/2.
s4h=30
s1y=0.0
s1w=0.1
s1x=0
s2w=0.1
s2x=0
s3w=0.1
s3x=0
s4w=0.1
s4x=0

s2y=coh-((coh1l+coh2l)/2 -s2l)*tan(coa)
s3y=coh+((bsh1l+s3Deltaz)-(coh1l+coh2l)/2)*tan(coa)
print, "s2y=",s2y
print, "s3y=",s3y


print,"Real axis from here"

;***********************
;***** AXIS VALUES *****
;***********************
;****** Slit S1
s1b=(-s1h + 2*s1y)/2.
s1t=(s1h + 2*s1y)/2.
;s1r=(-s1w + 2*s1x)/2.
;s1l=(s1w + 2*s1x)/2
;print, "s1b ",s1b
;print, "s1t ",s1t

;****** Slit S2
s2b=coh - ((coh1l + coh2l)/2. - s2l)*tan(coa)+ s2h/2. - s2y 
s2t=coh - ((coh1l + coh2l)/2. - s2l)*tan(coa) + s2h/2  - s2y
;s2r=(-s2w + 2*s2x)/2.
;s2l=(s2w + 2*s2x)/2
print, "s2b position",s2b
print, "s2t position ",s2t


; ****** Slit background slit
bsh1=s3y

;****** Slit S3
s3b=coh + ((bsh1l+s3DeltaZ)-(coh1l + coh2l)/2.)*tan(coa) + s3h/2.  -bsh1
s3t=coh + ((bsh1l+s3DeltaZ)-(coh1l + coh2l)/2.)*tan(coa) + s3h/2   -bsh1
;s3r=(-s3w + 2*s3x)/2.
;s3l=(s3w + 2*s3x)/2
print, "s3b position",s3b
print, "s3t position",s3t

;****** Slit S4
s4b=-s4h/2.
s4t=s4h/2.
;s4r=(-s4w + 2*s4x)/2.
;s4l=(s4w + 2*s4x)/2
;print, "s4b position",s4b
;print, "s4t position",s4t

;****** Mirror 1
if(useBottomSide1) then begin
	m1h2=(2*m1h - 2*pvm1y*Cos(ban) + 2*pvm1y*Cos(ban + m1a) + m1h1l*Sin(ban) - m1h2l*Sin(ban) - m1h1l*Sin(ban + m1a) + m1h2l*Sin(ban + m1a))/2.+mAperture
	m1h1=m1h2 + m1h1l*Tan(m1a) - m1h2l*Tan(m1a)
end else begin
	m1h2=(2*m1h - 2*pvm1y*Cos(ban) + 2*pvm1y*Cos(ban + m1a) + m1h1l*Sin(ban) - m1h2l*Sin(ban) - m1h1l*Sin(ban + m1a) + m1h2l*Sin(ban + m1a))/2.
	m1h1=m1h2 + m1h1l*Tan(m1a) - m1h2l*Tan(m1a)
end
print, "m1h1 position",m1h1
print, "m1h2 position",m1h2

;****** Mirror 2
if(useBottomSide2) then begin
	m2h1=(2*m2h - 2*pvm2y*Cos(ban) + 2*pvm2y*Cos(ban + m2a) - m2h1l*Sin(ban) + m2h2l*Sin(ban) + m2h1l*Sin(ban + m2a) - m2h2l*Sin(ban + m2a))/2.+mAperture
	m2h2=m2h1 - m2h1l*Tan(m2a) + m2h2l*Tan(m2a)
end else begin
	m2h1=(2*m2h - 2*pvm2y*Cos(ban) + 2*pvm2y*Cos(ban + m2a) - m2h1l*Sin(ban) + m2h2l*Sin(ban) + m2h1l*Sin(ban + m2a) - m2h2l*Sin(ban + m2a))/2.
	m2h2=m2h1 - m2h1l*Tan(m2a) + m2h2l*Tan(m2a)
end
print, "m2h1 position",m2h1
print, "m2h2 position",m2h2

;****** Collimation
; coh1 and coh2 defined when the collimation guide touches the beam
coaGuide=coa
cohGuide=coh
coh1 = (2*cohGuide - 2*pvcoy*Cos(ban) + 2*pvcoy*Cos(ban + coa) - coh1l*Sin(ban) + coh2l*Sin(ban) + coh1l*Sin(ban + coa) - coh2l*Sin(ban + coa))/2.;
coh2 = coh1 - coh1l*Tan(coaGuide) + coh2l*Tan(coaGuide)
print, "coh1 position",coh1
print, "coh2 position",coh2


;****** Detector
dh1=dh + (dh2l - dl)*tan(coa+dan) + (dh1l - dh2l)*tan(coa+dan)
dh2=dh + (dh2l - dl)*tan(coa+dan)
;print, "dh1=",dh1
;print, "dh2=",dh2

print,"bsh1 position",bsh1
print,"sht1 position",sht


; // TEST FORCING MOTOR VALUES


;***********************
;***********************
;***********************

C = 1.0/(3.956032e-7)

; *** some colors
colorMin='ff0000'x
colorPeak='00ff00'x
colorMax='0000ff'x

lambdaMin=1.5e-10
lambdaPeak=7e-10
lambdaMax=30e-10

;***************  creating the skew matrix ***************
skew=[[1, 0],[Tan(ban), 1]]

;***************  creating elements for graphics and ray tracing
;*** beam guide aperture
p0 = [0, 0]
p1 = p0 + beamAperture/2
p2 = p0 - beamAperture/2

sp0=skew##p0
sp1=skew##p1
sp2=skew##p2

guideApertureElm={segments:[ $
		[sp2[0],sp2[1],sp1[0],sp1[1],0], $
		[sp1[0],sp1[1],sp2[0],sp2[1],0] $
		] }
		
;***** Slit 1
p3 = [s1l, s1t]
p4 = [s1l, s1b]
p73 = p3 - [s1Length/2, 0]
p72 = [s1l - s1Length/2, ymax]
p74 = p3 + [s1Length/2, 0]
p69 = [s1l + s1Length/2, ymax]
p77 = p4 - [s1length/2, 0]
p75 = p4 + [s1length/2, 0]
p76 = [s1l - s1Length/2, ymin]
p5 = [s1l + s1Length/2, ymin]
p68 = (p3 + p4)/2

sp3 = skew##p3
sp4 = skew##p4
sp73 = skew##p73
sp72 = skew##p72
sp74 = skew##p74
sp69 = skew##p69
sp77 = skew##p77
sp75 = skew##p75
sp76 = skew##p76
sp5 = skew##p5
sp68 = skew##p68


slit1TopElm={segments:[ $
		[sp74[0],sp74[1],sp73[0],sp73[1],0], $
		[sp73[0],sp73[1],sp72[0],sp72[1],0], $
		[sp72[0],sp72[1],sp69[0],sp69[1],0], $
		[sp69[0],sp69[1],sp74[0],sp74[1],1] $
		] }
		
slit1BottomElm={segments:[ $
		[sp75[0],sp75[1],sp77[0],sp77[1],0], $
		[sp77[0],sp77[1],sp76[0],sp76[1],0], $
		[sp76[0],sp76[1],sp5[0],sp5[1],0], $
		[sp5[0],sp5[1],sp75[0],sp75[1],1] $
		] }
		
;***** Mirror 1


p67 = [0, 0, 1]
p7 = p67 - [m1Length/2, 0, 0]
p13 = p67 + [m1Length/2, 0, 0]
p6 = p7 + [0, mThickness, 0]
p12 = p13 + [0, mThickness, 0]
p8 = p7 - [0, mAperture, 0]
p14 = p13 - [0, mAperture, 0]
p9 = p8 - [0, mThickness, 0]
p15 = p14 - [0, mThickness, 0]
p83 = [(m1h2l - m1h1l)/2, pvm1y, 1]
p10 = p67 + [-(m1h2l - m1h1l)/2, 0, 0] ;
p11 = p67 + [(m1h2l - m1h1l)/2, 0, 0] ;


angle=atan(m1h2-m1h1,m1h2l-m1h1l)
Tm1=[[Cos(angle + ban),-Sin(angle + ban),m1h2l + ((m1h1l - m1h2l)*Cos(angle + ban))/2. + pvm1y*Sin(angle + ban)],$
   	 [Sin(angle + ban),Cos(angle + ban),m1h2 + pvm1y*Cos(ban) - pvm1y*Cos(angle + ban) + m1h2l*Sin(ban) + ((m1h1l - m1h2l)*Sin(angle + ban))/2.],$
   	 [0,0,1]]
 
   	
   	
sp10 = Tm1##p10
sp11 = Tm1##p11
sp7 = Tm1##p7
sp13 = Tm1##p13
sp6 = Tm1##p6
sp12 = Tm1##p12
sp8 = Tm1##p8
sp9 = Tm1##p9
sp14 = Tm1##p14
sp15 = Tm1##p15
sp67 = Tm1##p67
sp83 = Tm1##p83


mirror1TopElm={segments:[ $
		[sp7[0],sp7[1],sp13[0],sp13[1],1], $
		[sp13[0],sp13[1],sp12[0],sp12[1],0], $
		[sp12[0],sp12[1],sp6[0],sp6[1],0], $
		[sp6[0],sp6[1],sp7[0],sp7[1],0] $
		]}

		
mirror1BottomElm={segments:[ $
		[sp14[0],sp14[1],sp8[0],sp8[1],1], $
		[sp8[0],sp8[1],sp9[0],sp9[1],0], $
		[sp9[0],sp9[1],sp15[0],sp15[1],0], $
		[sp15[0],sp15[1],sp14[0],sp14[1],0] $
		]}


;***** Mirror 2
p65 = [0, 0, 1];
p17 = p65 - [m2Length/2, 0, 0];
p23 = p65 + [m2Length/2, 0, 0];
p16 = p17 + [0, mThickness, 0];
p22 = p23 + [0, mThickness, 0];
p18 = p17 - [0, mAperture, 0];
p24 = p23 - [0, mAperture, 0];
p19 = p18 - [0, mThickness, 0];
p25 = p24 - [0, mThickness, 0];
p84 = [-(m2h2l - m2h1l)/2, pvm2y, 1];
p20 = p65 + [-(m2h2l - m2h1l)/2, 0, 0] ;
p21 = p65 + [(m2h2l - m2h1l)/2, 0, 0] ;

angle=atan(m2h2-m2h1,m2h2l-m2h1l)
Tm2=[[Cos(angle + ban),-Sin(angle + ban),m2h1l + ((-m2h1l + m2h2l)*Cos(angle + ban))/2. + pvm2y*Sin(angle + ban)],$
   	[Sin(angle + ban),Cos(angle + ban),m2h1 + pvm2y*Cos(ban) - pvm2y*Cos(angle + ban) + m2h1l*Sin(ban) + ((-m2h1l + m2h2l)*Sin(angle + ban))/2.],$
   	[0,0,1]]

sp65 = Tm2##p65;
sp17 = Tm2##p17;
sp23 = Tm2##p23;
sp16 = Tm2##p16;
sp22 = Tm2##p22;
sp18 = Tm2##p18;
sp24 = Tm2##p24;
sp19 = Tm2##p19;
sp25 = Tm2##p25;
sp84 = Tm2##p84;
sp20 = Tm2##p20;
sp21 = Tm2##p21;

mirror2TopElm={segments:[ $
		[sp23[0],sp23[1],sp17[0],sp17[1],1], $
		[sp17[0],sp17[1],sp16[0],sp16[1],0], $
		[sp16[0],sp16[1],sp22[0],sp22[1],0], $
		[sp22[0],sp22[1],sp23[0],sp23[1],0] $
		] }
mirror2BottomElm={segments:[ $
		[sp24[0],sp24[1],sp18[0],sp18[1],1], $
		[sp18[0],sp18[1],sp19[0],sp19[1],0], $
		[sp19[0],sp19[1],sp25[0],sp25[1],0], $
		[sp25[0],sp25[1],sp24[0],sp24[1],0] $
		] }


;***** Collimation guide
p64 = [0, 0, 1];
p31 = [-coLength/2, coAperture/2, 1];
p37 = [coLength/2, coAperture/2, 1];
p30 = p31 + [0, coThickness, 0];
p36 = p37 + [0, coThickness, 0];
p32 = [-coLength/2, -coAperture/2, 1];
p38 = [coLength/2, -coAperture/2, 1];
p33 = p32 + [0, -coThickness, 0];
p39 = p38 + [0, -coThickness, 0];
p34 = [-(coh2l - coh1l)/2, coAperture/2, 1];
p35 = [(coh2l - coh1l)/2, coAperture/2, 1];
p85 = [-(coh2l - coh1l)/2, pvcoy, 1];

angle=atan(coh2-coh1,coh2l-coh1l)
Tco=[[Cos(ban + angle),-Sin(ban + angle),coh1l + ((-coh1l + coh2l)*Cos(ban + angle))/2. + pvcoy*Sin(ban + angle)],$
   	  [Sin(ban + angle),Cos(ban + angle),coh1 + pvcoy*Cos(ban) - pvcoy*Cos(ban + angle) + coh1l*Sin(ban) + ((-coh1l + coh2l)*Sin(ban + angle))/2.],$
   	  [0,0,1]]


sp64 = Tco##p64;
sp31 = Tco##p31;
sp37 = Tco##p37;
sp30 = Tco##p30;
sp36 = Tco##p36;
sp32 = Tco##p32;
sp38 = Tco##p38;
sp33 = Tco##p33;
sp39 = Tco##p39;
sp34 = Tco##p34;
sp35 = Tco##p35;
sp85 = Tco##p85;

collimationTopElm={segments:[ $
		[sp37[0],sp37[1],sp31[0],sp31[1],0], $
		[sp31[0],sp31[1],sp30[0],sp30[1],0], $
		[sp30[0],sp30[1],sp36[0],sp36[1],0], $
		[sp36[0],sp36[1],sp37[0],sp37[1],1] $
		] }
collimationBottomElm={segments:[ $
		[sp38[0],sp38[1],sp39[0],sp39[1],0], $
		[sp39[0],sp39[1],sp33[0],sp33[1],0], $
		[sp33[0],sp33[1],sp32[0],sp32[1],0], $
		[sp32[0],sp32[1],sp38[0],sp38[1],1] $
		] }

;******* Slit 2 - attached to the collimation
p66=[s2l,coh1 - (coh1l - s2l)*Sin(atan(-coh1 + coh2,-coh1l + coh2l))]
p27 = [s2l, s2t+p66[1] ]
p28 = [s2l, s2b+p66[1] ]
p26 = [s2l, ymax]
p29 = [s2l, ymin]

sp27 = skew##p27
sp28 = skew##p28
sp66 = skew##p66
sp26 = skew##p26
sp29 = skew##p29

slit2TopElm={segments:[ $
		[sp27[0],sp27[1],sp26[0],sp26[1],0], $
		[sp26[0],sp26[1],sp27[0],sp27[1],0] $
		] }
slit2BottomElm={segments:[ $
		[sp28[0],sp28[1],sp29[0],sp29[1],0], $
		[sp29[0],sp29[1],sp28[0],sp28[1],0] $
		] }


;******* Diaphragm 3
p82 = [bsh1l, bsh1];
p70 = p82 + [0, bsh/2];
p41 = p70 - [bsLength/2, 0]
p45 = p70 + [bsLength/2, 0]
p40 = [p41[0], ymax]
p44 = [p45[0], ymax]
p71  = p82 - [0, bsh/2];
p42 = p71 + [-bsLength/2, 0]
p46 = p71 + [bsLength/2, 0]
p43 = [p42[0], ymin]
p47 = [p46[0], ymin]

sp82 = skew##p82
sp70 = skew##p70
sp41 = skew##p41
sp45 = skew##p45
sp40 = skew##p40
sp44 = skew##p44
sp71 = skew##p71
sp42 = skew##p42
sp46 = skew##p46
sp43 = skew##p43
sp47 = skew##p47

bsTopElm={segments:[ $
		[sp45[0],sp45[1],sp41[0],sp41[1],0], $
		[sp41[0],sp41[1],sp40[0],sp40[1],0], $
		[sp40[0],sp40[1],sp44[0],sp44[1],0], $
		[sp44[0],sp44[1],sp45[0],sp45[1],1] $
		] }
bsBottomElm={segments:[ $
		[sp46[0],sp46[1],sp47[0],sp47[1],0], $
		[sp47[0],sp47[1],sp43[0],sp43[1],0], $
		[sp43[0],sp43[1],sp42[0],sp42[1],0], $
		[sp42[0],sp42[1],sp46[0],sp46[1],1] $
		] }

;******* Slit 3 - attached to the diaphragm 3
p49 = [bsh1l + s3DeltaZ, bsh1 + s3t]
p50 = [bsh1l + s3DeltaZ, bsh1 + s3b]
p48 = [p49[0], ymax]
p51 = [p50[0], ymin]
p63 = (p49 + p50)/2

sp49 = skew##p49
sp50 = skew##p50
sp48 = skew##p48
sp51 = skew##p51
sp63 = skew##p63

slit3TopElm={segments:[ $
		[sp49[0],sp49[1],sp48[0],sp48[1],0], $
		[sp48[0],sp48[1],sp49[0],sp49[1],0] $
		] }
slit3BottomElm={segments:[ $
		[sp50[0],sp50[1],sp51[0],sp51[1],0], $
		[sp51[0],sp51[1],sp50[0],sp50[1],0] $
		] }

;******* Sample
p52 = [sl - sampleLength/2, sht+sampleLength/2*sin(ban)] ;/// SAMPLE IS ALWAYS HORIZONTAL (at least in the calculations )
p53 = [sl + sampleLength/2, sht-sampleLength/2*sin(ban)]

sp52 = skew##p52
sp53 = skew##p53

sampleElm={segments:[ $
		[sp53[0],sp53[1],sp52[0],sp52[1],1], $
		[sp52[0],sp52[1],sp53[0],sp53[1],1] $
		] }


;******* Reflection center
p79 = [sl,p63[1]  + (sl - (bsh1l + s3DeltaZ))*Sin(atan(p63[1]-p66[1],p63[0]-p66[0]))]

sp79 = skew##p79

;******* Detector
p58 = [dh1l, dh1]
p59 = [dh2l, dh2]
p61 = p59 + [(dl - dh2l), (dl - dh2l)*Sin(  atan(-dh1 + dh2, -dh1l + dh2l) )]
p60 = p61 + [0, detHeight/2]
p62 = p61 + [0, -detHeight/2]
p78 = p58 + [-(dh1l - sl), (dh1l - sl)*Sin( -atan(-dh1 + dh2, -dh1l + dh2l))]

sp58 = skew##p58
sp59 = skew##p59
sp61 = skew##p61
sp60 = skew##p60
sp62 = skew##p62
sp78 = skew##p78

detectorElm={segments:[ $
		[sp62[0],sp62[1],sp60[0],sp60[1],0], $
		[sp60[0],sp60[1],sp62[0],sp62[1],0] $
		] }

;******* Slit 4 - attached to the detector axis
p81 = [s4l,dh1+(dh1l-s4l)*Sin(-atan(-dh1 + dh2, -dh1l + dh2l))]
p55 = p81+[0,s4t]
p56 = p81+[0,s4b]
p54 = [s4l, ymax]
p57 = [s4l, ymin]

sp55 = skew##p55
sp56 = skew##p56
sp81 = skew##p81
sp54 = skew##p54
sp57 = skew##p57

slit4TopElm={segments:[ $
		[sp55[0],sp55[1],sp54[0],sp54[1],0], $
		[sp54[0],sp54[1],sp55[0],sp55[1],0]  $
		] }
slit4BottomElm={segments:[ $
		[sp56[0],sp56[1],sp57[0],sp57[1],0], $
		[sp57[0],sp57[1],sp56[0],sp56[1],0]  $
		]}

;******** Collimation center
p80 = [(coh1l + coh2l)/2, p66[1] + ((coh2l + coh1l)/2 - s2l)*Sin( atan(p63[1] - p66[1],p63[0] - p66[0]) )]
sp80=skew##p80

;******* DRAWING THE FULL SCENE

; Make the plot frame
plot,[xMin,xmax],[yMin,yMax],/nodata,xstyle=3,xrange=[xMax,xMin]

; draws the guide inclination
plots,[xMin,xMax],[0,xMax*sin(ban)],color='4C4C4C'x,linestyle=1

; plots a theta line from the sample
;x=findgen(100)/100*(xMax-xMin)+xMin
;y=(x-sl)*tan(-theta)
;plots,x,y,/data,color='ff'x


drawElement,mirror1TopElm,'00FFFF'x
drawElement,mirror1BottomElm,'00FFFF'x
drawElement,mirror2TopElm,'66FFFF'x
drawElement,mirror2BottomElm,'66FFFF'x


;**** draw the guide aperture
drawElement,guideApertureElm,'66FFFF'x

drawElement,sampleElm,'FF8888'x
drawElement,collimationTopElm,'4C4C4C'x
drawElement,collimationBottomElm,'4C4C4C'x
drawElement,slit1TopElm,'0080FF'x
drawElement,slit1BottomElm,'0080FF'x
drawElement,slit2TopElm,'0080FF'x
drawElement,slit2BottomElm,'0080FF'x
drawElement,slit3TopElm,'0080FF'x
drawElement,slit3BottomElm,'0080FF'x
drawElement,slit4TopElm,'0080FF'x
drawElement,slit4BottomElm,'0080FF'x
drawElement,bsTopElm,'0080FF'x
drawElement,bsBottomElm,'0080FF'x
drawElement,detectorElm,'8000FF'x


;**** draw the axis
plots,[sp10[0],sp10[0]],[sp10[1],m1h1l*sin(ban)],color='0000FF'x,/data  	;/// m1h1
plots,[sp11[0],sp11[0]],[sp11[1],m1h2l*sin(ban)],color='0000FF'x,/data	;/// m1h2
plots,[sp20[0],sp20[0]],[sp20[1],m2h1l*sin(ban)],color='0000FF'x,/data  	;/// m2h1
plots,[sp21[0],sp21[0]],[sp21[1],m2h2l*sin(ban)],color='0000FF'x,/data	;/// m2h2
plots,[sp35[0],sp35[0]],[sp35[1],coh2l*sin(ban)],color='0000FF'x,/data  	;/// coh1
plots,[sp34[0],sp34[0]],[sp34[1],coh1l*sin(ban)],color='0000FF'x,/data	;/// coh2
plots,[sp58[0],sp58[0]],[sp58[1],dh1l*sin(ban)],color='0000FF'x,/data  	;/// dh1
plots,[sp59[0],sp59[0]],[sp59[1],dh2l*sin(ban)],color='0000FF'x,/data	;/// dh2

;// plot the detector axis
plots,[sl,dl],[sp78[1],sp61[1]],color='4C4C4C'x,linestyle=1

tmp=skew##[dl,dh]
tmp1=skew##[sl,sht]
plots,[tmp[0],tmp1[0]],[tmp[1],tmp1[1]]
;plots,[xmin,xmax],[sp78[1],sp78[1]]

;// plot the reflection center
plots,[sp79[0],sp79[0]],[sp79[1],sp79[1]],psym=4

;// plot the collimation center center
plots,[sp80[0],sp80[0]],[sp80[1],sp80[1]],psym=4

;// plot the mirror1 center
plots,[sp67[0],sp67[0]],[sp67[1],sp67[1]],psym=4

;// plot the mirror1 rotation pivot
plots,[sp83[0],sp83[0]],[sp83[1],sp83[1]],psym=7,color=0

;// plot the mirror2 center
plots,[sp65[0],sp65[0]],[sp65[1],sp65[1]],psym=4

;// plot the mirror2 rotation pivot
plots,[sp84[0],sp84[0]],[sp84[1],sp84[1]],psym=7,color=0

;// plot the collimation rotation pivot
plots,[sp85[0],sp85[0]],[sp85[1],sp85[1]],psym=7,color='FFFFFF'x


;//// TESTING DETECTOR ORIENTATION
testPoint1=[sl,sht]
sTestPoint1=skew##testPoint1

sTestPoint=sTestPoint1+[-(sl-(coh1l+coh2l)/2),(sl-(coh1l+coh2l)/2)*tan(ban+coa+dan)]
sTestPoint=sTestPoint1+[-(sl-(coh1l+coh2l)/2),(sl-(coh1l+coh2l)/2)*tan(theta)]

plots,[sTestPoint1[0],sTestPoint[0]],[sTestPoint1[1],sTestPoint[1]],color='00FF00'x

;***** POINT STRUCTURE X,Y,VX,VY,EXIST,DISTANCE FROM LAST POINT

detectorIntersection=[0,0,0]

;vNeutron=1.0/(C*lambdaMax)
;point=[x:0.0,y:0.0,vx:vNeutron*cos(guideTilt) ,vy:vNeutron*sin(guideTilt),exists:0,distance:0.0]
;plotNeutronTrajectory,neutronPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,sample,slit1Top,slit1Bottom,$
;slit2Top,slit2Bottom,slit3Top,slit3Bottom,slit4Top,slit4Bottom,detector,detectorIntersection),colorMax


;vNeutron=1.0/(C*lambdaPeak)
;point=[x:0.0,y:0.0,vx:vNeutron*cos(guideTilt) ,vy:vNeutron*sin(guideTilt),exists:0,distance:0.0]
;plotNeutronTrajectory,neutronPath(point,scale,mirror1Top,mirror1Bottom,mirror2Top,mirror2Bottom,sample,slit1Top,slit1Bottom,$
;slit2Top,slit2Bottom,slit3Top,slit3Bottom,slit4Top,slit4Bottom,detector,detectorIntersection),colorPeak

vNeutron=1000.0/(C*lambdaMin)
point={x:0.0,y:0.0,vx:vNeutron*cos(guideTilt) ,vy:vNeutron*sin(guideTilt),exists:0,distance:0.0}
plotNeutronTrajectory,neutronPath(point,scale,mirror1TopElm,mirror1BottomElm,mirror2TopElm,mirror2BottomElm,sampleElm,slit1TopElm,slit1BottomElm,$	
		 slit2TopElm,slit2BottomElm,bsTopElm,bsBottomElm,slit3TopElm,slit3BottomElm,slit4TopElm,slit4BottomElm,detectorElm,detectorIntersection),colorMin


drawUmbra,scale,mirror1TopElm,mirror1BottomElm,mirror2TopElm,mirror2BottomElm,slit1TopElm,slit1BottomElm,$	
		 slit2TopElm,slit2BottomElm,bsTopElm,bsBottomElm,slit3TopElm,slit3BottomElm,slit4TopElm,slit4BottomElm



;*** TEST INTERSECTION OF REFLECTED BEAM WITH DETECTOR
;** convert the lab intersection altitude into a detector relative altitude
reflectedBeamOnDetector=0
if(detectorIntersection[0]) then begin
	yid=detectorIntersection[1] + dl*Tan(ban) - dh
	print,"Reflected beam on detector at height = ",yid
	reflectedBeamOnDetector=1
end else print,"WARNING: REFLECTED BEAM NOT ON DETECTOR"

;*** TEST INTERSECTION OF DEFLECTED BEAM WITH DETECTOR
;** convert the lab intersection altitude into a detector relative altitude
ydbd=coh - Tan(coa)*(dl - (s2l+(bsh1l+s3DeltaZ))/2)
deflectedBeamOnDetector=0
if(ydbd LE dh + detheight/2 AND ydbd GE dh - detheight/2) then begin
	print,"DEFLECTED BEAM ON DETECTOR !"
	deflectedBeamOnDetector=1
end	



; AND DIsplay the logo of course .... :-)
;logo=read_png("Figaro_logo_V2.png")
;tvscl,logo,true=1,xMin+1500,ymax-100,/data


;*** draw legends, etc....
dx=-50
dy=10

; Slits names
xyouts,s1l+dx,yMin+dy,"!5S1",size=1.2,color='0080FF'x
xyouts,s2l+dx,yMin+dy,"!5S2",size=1.2,color='0080FF'x
xyouts,(bsh1l+s3DeltaZ)+dx,yMin+dy,"!5S3",size=1.2,color='0080FF'x
xyouts,s4l+dx,yMin+dy,"!5S4",size=1.2,color='0080FF'x

;mirror values
xyouts,1500,-150,"m1h="+string(m1h,format="(F0.2)")+" mm"+$
				 "!Cm1a="+string(m1a*180/!pi,format="(F0.2)")+" deg"+$
				 "!Cm1h1="+string(m1h1,format="(F0.2)")+" mm"+$
				 "!Cm1h2="+string(m1h2,format="(F0.2)")+" mm",$
				 color='00FFFF'x,size=1.2
xyouts,1500,-232,"m2h="+string(m2h,format="(F0.2)")+" mm"+$
				 "!Cm2a="+string(m2a*180/!pi,format="(F0.2)")+" deg"+$
				 "!Cm2h1="+string(m2h1,format="(F0.2)")+" mm"+$
				 "!Cm2h2="+string(m2h2,format="(F0.2)")+" mm",$
				 color='00FFFF'x,size=1.2


;slits values
xyouts,1500,-300,"s1h="+string(s1h,format="(F0.2)")+" mm"+$
				 "!Cs2h="+string(s2h,format="(F0.2)")+" mm"+$
				 "!Cs3h="+string(s3h,format="(F0.2)")+" mm"+$
				 "!Cs4h="+string(s4h,format="(F0.2)")+" mm",$
				 color='0080FF'x,size=1.2


;critical lambdas
;xyouts,6,-0.35,string("san=",san*180/!pi,"!Csht=",sht),color='FF8888'x,size=1.2

xyouts,4000,-300,"!4k!3c1="+string(lambda1C,format="(F0.2)")+" Ang"+$
			   "!C!4k!3c2="+string(lambda2C,format="(F0.2)")+" Ang",$
				 color='00FFFF'x,size=1.2

; coaMeca cohMeca,coh1,coh2
;xyouts,1.5,-0.35,string("coh=",coh,"!Ccoa=",coa*180/!pi),color='0080FF'x,size=1.2
xyouts,4000,-330,"coh1="+string(coh1,format="(F0.2)")+" mm"+$
			   "!Ccoh2="+string(coh2,format="(F0.2)")+" mm"+$
			   "!CcohMeca="+string((coh2+coh1)/2,format="(F0.2)")+" mm"+$
			   "!CcoaMeca="+string(atan(coh2-coh1,coh2l-coh1l)*180/!pi,format="(F0.2)")+" deg",$
				 color='4C4C4C'x,size=1.2



;SAN and SHT
;xyouts,6,-0.35,string("san=",san*180/!pi,"!Csht=",sht),color='FF8888'x,size=1.2


xyouts,6000,-300,"san="+string(san*180/!pi,format="(F0.2)")+" deg"+$
				 "!Csht="+string(sht,format="(F0.2)")+" mm",$
				 color='FF8888'x,size=1.2

; coa coh,theta
;xyouts,1.5,-0.35,string("coh=",coh,"!Ccoa=",coa*180/!pi),color='0080FF'x,size=1.2
xyouts,6000,-350,"coh="+string(coh,format="(F0.2)")+" mm"+$
				 "!Ccoa="+string(coa*180/!pi,format="(F0.2)")+" deg"+$
				 "!Ctheta="+string(theta*180/!pi,format="(F0.2)")+" deg",$
				 color='0080FF'x,size=1.2






;DAN, dh, d1h ,etc ... intersection
stringDet="dan="+string(dan*180/!pi,format="(F0.2)")+" deg"
stringDet+="!Cd1h="+string(dh1,format="(F0.2)")+" mm"
stringDet+="!Cd2h="+string(dh2,format="(F0.2)")+" mm"
;if(reflectedBeamOnDetector) then stringDet+=("!CReflected beam at "+string(yid,format="(F0.2)")+"mm") else  stringDet+=("!CReflected beam not on detector")
;if(deflectedBeamOnDetector) then stringDet+=string("!CDIRECT BEAM ON DETECTOR!!!")
xyouts,9000,-300,stringDet,color='8000FF'x,size=1.2


stop


end
