pro galaxy_init

  forward_function build_path
  forward_function label_frame
  forward_function cosmos_extract_galaxy_parameters  
  forward_function cosmos_get_default_settings
  forward_function cosmos_get_table_labels
  forward_function cosmos_table_check
  forward_function str_centerize

  gui_utilities
  paths_utilities
  strings_utilities
  web_utilities

  cosmos_widgets
  cosmos, /compile

end

pro galaxy_cosmos_tab, parent = parent

  ; The normal font.
  c_font_normal = "arial*normal*14"
  
  ; The big font.
  c_font_big = "arial*bold*15"
  
  tab = widget_base(parent, /column, title = "COSMOS TABLE", tab_mode = 1, /base_align_left, $
                    uname = 'COSMOSTAB')

  table = widget_table(tab, editable=0, ignore_accelerators = 1, /resizeable_columns, /scroll, /all_events, $                       
                       x_scroll_size = nCols, y_scroll_size = 10, tab_mode = 0, /context_events, alignment = 0, $
                       uname = 'TABLE', sensitive = 1, background_color = [220,200,220])

end

function cosmos_extract_galaxy_parameters, numors, path, interface

  common cosmos_params, c_params
   
  on_ioerror, EXTRACT_ERROR
     
  tempStr = ""
    
  params = replicate({numor : 0L, date : 0D, pos : 0, ang : 0D, angpos : -1}, n_elements(numors))
          
  flag_defined = 0
        
  for f = 0, n_elements(numors) - 1 do begin
  
    ok = 0
      
    numor = string(numors[f], format = '(I06)')

    numor = path + path_sep() + strtrim(numor,2)
      
    raw_lun=-1
    openr, raw_lun, numor, /get_lun

    ; The RRRRRR... line.
    for i = 1, 4 do readf, raw_lun, tempStr
    
    readf, raw_lun, tempStr
        
    temp = strsplit(strmid(tempStr,14), /extract)
         
    startDate = strtrim(strsplit(temp[0],"-",/extract),2)
   
    day = fix(startDate[0])
    month = 1 + where(c_params.months eq startDate[1])
    if (startDate[2] ge 70) and (startDate[2] le 99) then year = 1900 +  fix(startDate[2]) else year = 2000 + fix(startDate[2])
            
    startTime = strtrim(strsplit(temp[1],":",/extract),2)    
    hour   = fix(startTime[0])
    minute = fix(startTime[1])
    second = fix(startTime[2])
    
    for i = 1, 29 do readf, raw_lun, tempStr
            
    par1 = fltarr(128)
    
    readf, raw_lun, par1
    
    numor = numors[f]
    date = julday(month, day, year, hour, minute, second)
    ang = double(par1[51])
    if interface eq 0 then pos = fix(par1[52]) else pos = fix(par1[49])
    
    ok = 1
    
    extract_error:
    
      if ok then begin
              
        params[f].numor = numor
        params[f].date  = date
        params[f].pos   = pos
        params[f].ang   = ang
        
      endif else begin
      
        s = dialog_message('Could not extract GALAXY parameters from numor ' + numors[f], /center, /default_no, dialog_parent = event.top, title = 'Warning')
        return, !NULL
              
      endelse
                                                                 
      if raw_lun ne -1 then free_lun, raw_lun
          
  endfor
    
  sortedAng = params[uniq(params.ang,sort(params.ang))].ang
    
  for i = 0, n_elements(params) - 1 do params[i].angpos = where(sortedAng eq params[i].ang)
          
  return, params
  
end

pro galaxy_run, event
  
  widget_control, event.top, get_uvalue = galaxyrun
                                                                  
  widget_control, event.top, get_uvalue = galaxyrun
  if not galaxyrun then return
  
  wTable = widget_info(event.top, find_by_uname = 'GTABLE')
    
  selCells = widget_info(wTable, /table_select)  
  
  widget_control, wTable, get_value = table
    
  checkedRows = cosmos_table_check(event.top, table)
  
  idx = where(checkedRows eq 2, nEmptyRows)

  nRows = n_elements(table[0,*])
  
  if nEmptyRows eq nRows then begin  
    s = dialog_message('Please enter some data.', /center, /default_no, dialog_parent = event.top, title = 'Warning')
    return    
  endif
  
  idx = where(checkedRows eq 1, nInvalidRows)
           
  if nInvalidRows gt 0 then begin  
    mess = 'Error when checking rows.' + string(10b) + $
           'The following rows seem to be misformatted: ' + string(10b) + $
           strcompress(strjoin(idx)) + string(10b) + 'Continue skipping those raws ?'            
    cancel = dialog_message(mess, /cancel, /default_cancel, title = 'Warning')
    cancel = strupcase(cancel)
    if cancel eq "CANCEL" then return
  endif

  idx = where(checkedRows eq 0)
  table = table[*,idx]
  nRows = n_elements(table[0,*])
  
  if nEmptyRows + nInvalidRows eq nRows then return
  
  widget_control, widget_info(event.top, find_by_uname = 'FIRSTNUMOR'), get_value = first  
  on_ioerror, BADFIRST
  ok = 0 
  first = long(first[0])
  ok = 1
  if first le 0 then ok = 0
  BADFIRST: 
    if not ok then begin
      s = dialog_message('Bad value for first numor entry', /center, /default_no, dialog_parent = event.top, title = "Error")
      return     
    endif

  widget_control, widget_info(event.top, find_by_uname = 'LASTNUMOR'), get_value = last
  on_ioerror, BADLAST
  ok = 0 
  last = long(last[0])
  ok = 1
  if last lt first then ok = 0
  BADLAST: 
    if not ok then begin
      s = dialog_message('Bad value for last numor entry', /center, /default_no, dialog_parent = event.top, title = "Error")
      return     
    endif
    
  widget_control, widget_info(event.top, find_by_uname = 'NUMORSTEP'), get_value = step
  on_ioerror, BADSTEP
  ok = 0 
  step = long(step[0])
  ok = 1
  if step le 0 then ok = 0
  BADSTEP: 
    if not ok then begin
      s = dialog_message('Bad value for numor step entry', /center, /default_no, dialog_parent = event.top, title = "Error")
      return     
    endif
              
  selectedNumors = strtrim(first + step*lindgen(1L+(last-first)/step),2)
  
  widget_control, widget_info(event.top, find_by_uname = 'PATHDIR'), get_value = path  

  widget_control, widget_info(event.top, find_by_uname = 'INTERFACE'), get_value = interface
      
  numorParams = cosmos_extract_galaxy_parameters(selectedNumors,path,interface)
    
  widget_control, widget_info(event.top, find_by_uname = 'MAKESUBDIR'), get_value = makeSubdir
                            
  sTable = replicate({row : 0, pos : 0, date : 0D, info : '', out : ''},nRows)  
        
  for i = 0, nRows - 1 do begin
    
    ok = 0
    
    on_ioerror, ROW_ERROR

    pos = fix(table[0,i])

    startDate = strtrim(strsplit(table[1,i], "/", /extract),2)
    day = fix(startDate[0])
    month = fix(startDate[1])
    if (startDate[2] ge 70) and (startDate[2] le 99) then year = 1900 +  fix(startDate[2]) else year = 2000 + fix(startDate[2])
            
    startTime = strtrim(strsplit(table[2,i], ":", /extract),2)    
    hour   = fix(startTime[0])
    minute = fix(startTime[1])
    second = fix(startTime[2])

    date = julday(month, day, year, hour, minute, second)
                  
    if strtrim(table[3,i],2) eq '' then info = strjoin(strcompress(table[0:2,i], /remove_all),"_") else info = table[3,i]
            
    if makesubdir eq 1 then begin
    
      subdir = strcompress(info + '_' + string(date, format = '(C(CDI02, CMOI02, CYI02, "_", CHI02, CMI02, CSI02))'), /remove_all) 
      
    endif else subdir = ''
        
    out = subdir
    
    ok = 1
      
    ROW_ERROR:
      
      if not ok then begin
        s = dialog_message('Error when parsing GALAXY table', /center, /default_no, dialog_parent = event.top, title = "Error")
        return           
      endif
                
      sTable[i] = {row : i, pos : pos, date : date, info : info, out : out}

  endfor
          
  widget_control, widget_info(event.top, find_by_uname = 'TABLE'), get_value = temp
  
  nCols = n_elements(temp[*,0])
  nRows = n_elements(numorParams)
  cosmos_table = strarr(nCols, nRows)
    
  widget_control, widget_info(event.top, find_by_uname = 'NANGLES'), get_value = nAngles
      
  dBeamCols = numorParams.angpos
  case nAngles of
    0: rBeamCols = ([1])[numorParams.angpos]
    1: rBeamCols = ([2,3])[numorParams.angpos]
    2: rBeamCols = ([3,4,5])[numorParams.angpos]
  endcase
  outfileCols = ([4,8,12])[nAngles]  

  widget_control, widget_info(event.top, find_by_uname = 'DBEAM1'), get_value = dir1
  on_ioerror, misdir1 & okdir1=0 & dir1 = long(dir1) & okdir1=1 & misdir1: if not okdir1 then return  
  widget_control, widget_info(event.top, find_by_uname = 'DBEAM2'), get_value = dir2
  on_ioerror, misdir2 & okdir2=0 & dir2 = long(dir2) & okdir2=1 & misdir2: if not okdir2 then return  
  widget_control, widget_info(event.top, find_by_uname = 'DBEAM3'), get_value = dir3
  on_ioerror, misdir3 & okdir3=0 & dir3 = long(dir3) & okdir3=1 & misdir3: if not okdir3 then return
  
  indexes = lindgen(nRows)

  dBeams = [dir1,dir2,dir3]
  cosmos_table[dBeamCols,indexes] = dBeams[numorParams.angpos]
  
  cosmos_table[rBeamCols,indexes] = numorParams.numor
  cosmos_table[outfileCols,indexes] = strtrim(numorParams.numor,2)
  
  sTable = sTable[sort(sTable.date)]    
  sortedRows = sort(sTable.row)      
  sortedIdx = [-1]

  ok = 1
             
  for i = 0, n_elements(sortedRows) - 1 do begin
      
    row  = sortedRows[i]
    pos  = sTable[row].pos
    tMin = sTable[row].date
      
    idx = where(sTable.pos eq pos and sTable.date gt tMin)
      
    if idx[0] eq -1 then tMax = 999999999.9D else tMax = sTable[idx[0]].date
      
    tMin = tMin - 0.0001D
    tMax = tMax + 0.0001D
      
    idx1 = where((numorParams.pos eq pos) and (numorParams.date ge tMin) and (numorParams.date le tMax))
                  
    if idx1[0] eq -1 then begin
      ok = 0
      break
    endif
    
    cosmos_table[nCols - 2,idx1] = strtrim(sTable[row].out,2) + path_sep() + cosmos_table[nCols - 2,idx1] 
          
    sortedIdx = [sortedIdx,idx1]
      
  endfor
      
  if not ok then begin
    s = dialog_message('Mismatch between GALAXY and COSMOS table.', /center, /default_no, dialog_parent = event.top, title = 'Error')
    return
      
  endif
  
  sortedIdx = sortedIdx[1:*]
      
  cosmos_table = cosmos_table[*,sortedIdx]
      
  nRows = n_elements(cosmos_table[0,*])
      
  tLabels = cosmos_get_table_labels(nCols)
 
  widget_control, widget_info(event.top, find_by_uname = 'TABLE'), set_value = cosmos_table, $
    column_labels = tLabels, table_ysize = nRows
  
  ; Transfer normalization values to Cosmos table
  cosmos_transfer_normalization_factors, event

  cosmos_run, event
    
end

pro galaxy

  if xregistered('cosmos') then return
  if xregistered('galaxy') then return
  
  galaxy_init

  title = 'GALAXY version 0.91 (1/12/14, only for FIGARO)'
  
  p = file_dirname(file_which('galaxy.pro'))
          
  rcDir = build_path([p, "rc"])
  
  im = build_path([rcDir, 'galaxy_logo.jpg'])
      
  ; Sets the default group_leader if none is given.
  if n_elements(group_leader) eq 0 then group_leader = 0L
  
  cosmos_startup_dialog, 'Building the ' + title + ' user interface ... please wait', title, im

  ; create main window
  main = widget_base(group_leader = group_leader, /base_align_left, /column, title = title, mbar = bar, $
                     /tlb_kill_request_events, /tlb_size_events, uvalue = 1, uname = 'MAIN')
    
  b = widget_base(main, /row, /toolbar)
  ;w = widget_button(b, value = build_path([rcDir, "loadfile.bmp"]), /bitmap, tooltip = "Load input file", uname = 'LOAD')
  ;w = widget_button(b, value = build_path([rcDir, "savefile.bmp"]), /bitmap, tooltip = "Save input file", uname = 'SAVE')
  ;w = widget_button(b, value = build_path([rcDir, "default.bmp"]), /bitmap, tooltip = "Back to default values", uname = 'DEFAULT')
  ;w = widget_button(b, value = build_path([rcDir, "run.bmp"]), /bitmap, tooltip = "Run", uname = 'RUN')
  ;w = widget_button(b, value = build_path([rcDir, "quit.bmp"]), /bitmap, tooltip = "Quit", uname = 'QUIT')
  ;w = widget_button(b, value = build_path([rcDir, "help.bmp"]), /bitmap, tooltip = "Help", uname = 'HELP')
  ;w = widget_button(b, value = build_path([rcDir, "info.bmp"]), /bitmap, tooltip = "About", uname = 'ABOUT')
  w = widget_button(b, value = "Load", tooltip = "Load input file", uname = 'LOAD')
  w = widget_button(b, value = "Save", tooltip = "Save input file", uname = 'SAVE')
  w = widget_button(b, value = "Default", tooltip = "Back to default values", uname = 'DEFAULT')
  w = widget_button(b, value = "Run", tooltip = "Run", uname = 'RUN')
  w = widget_button(b, value = "Quit", tooltip = "Quit", uname = 'QUIT')
  w = widget_button(b, value = "Help", tooltip = "Help", uname = 'HELP')
  w = widget_button(b, value = "Info", tooltip = "About", uname = 'ABOUT')
  
  ; The 'File' menu button.
  b = widget_button(bar, value = 'File', /menu)
  w = widget_button(b, value = 'Load',    accelerator = "Ctrl+O", /checked_menu, uname = 'LOAD')
  w = widget_button(b, value = 'Save',    accelerator = "Ctrl+S", /checked_menu, uname = 'SAVE')
  w = widget_button(b, value = 'Default', accelerator = "Ctrl+D", /checked_menu, uname = 'DEFAULT')
  w = widget_button(b, value = 'Run',     accelerator = "Ctrl+R", /checked_menu, uname = 'RUN')
  w = widget_button(b, value = 'Quit',    accelerator = "Ctrl+Q", /checked_menu, uname = 'QUIT')
  
  ; The 'Help' menu button.
  b = widget_button(bar, value = 'Help', /menu, /help)
  w = widget_button(b, value = 'Help', accelerator = "Ctrl+H", /checked_menu, uname = 'HELPFM')
  w = widget_button(b, value = 'About', /checked_menu, uname = 'ABOUTFM')

  tabs = widget_tab(main, uname = 'TABS')    
  cosmos_data_tab,          parent = tabs, /galaxy 
  cosmos_calculation_tab,   parent = tabs
  cosmos_detector_tab,      parent = tabs  
  cosmos_machine_tab,       parent = tabs    
  cosmos_normalization_tab, parent = tabs    
  cosmos_io_tab,            parent = tabs, /galaxy
  galaxy_cosmos_tab,        parent = tabs  
  cosmos_log_tab,           parent = tabs  
  cosmos_plot_tab,          parent = tabs

  b = label_frame(main, /row, /base_align_center, value = ' Job status ', font = c_font_big, /frame, uname = 'STATUSBASE')
  w = widget_draw(b, ysize = 20, uname = 'STATUSDRAW')
  w = widget_button(b, value = 'Cancel', xsize = 50, ysize = 20, uvalue = 0, uname = 'CANCEL')
 
  l = widget_label(main, value='', ysize=20, /sunken_frame, /align_left, uname='STATUSLINE')

  settings = cosmos_get_default_settings()
  cosmos_set_settings, main, settings

  widget_control, main, /realize
  
  plotwin = widget_info(main, find_by_uname = 'PLOTWIN')        
  widget_control, plotwin, get_value = wid
  wset, wid
            
  ;centertlb, main  ;MG 30.10.2013: Commented because it caused to draw the table window out of the screen
      
  cosmos_resize, {WIDGET_BASE, ID : main, TOP : main, HANDLER : main, X : 900, Y : 700}  
        
  xmanager, 'galaxy', main, event_handler = 'cosmos_events', /no_block

end