

pro grav,th,lam,thetat,delta,dz,poffoff,openofflam,dh,x1,x2,x3,dy1,dy2

;Edited on 16/07/2012 by PG changed dtheta_over_theta for gaussian equivalent dtheta/theta
;macro when given straight line theta (th (deg)), wavelength (lam) and sam-midchop distance x3(m) calculates
; true theta (thetat (deg)), the displacement for the beam along the sample (delta (m))
; the drop of the beam at the sample centre (dz (m)) the offset in poff (poffoff (deg))
; and the offset in the opening (openofflam (deg))

;choppers turn clockwise from the reactor by default so poff=poff-poffoff
; opening offset only changes the apparent opening and does NOT make a time delay
; the normal delay associatyed with opening the chopper comes from changing the phase of
;the second controling chopper

print,'ingrav ',x1,x2,x3,lam,dy1,dy2
dpr=180./!pi
g=9.81  ;gravity
th=th/dpr
cr=305.e-3   ;chopper radius to the middle of guide
;x1=0.37   ;distance from sample to slit s3 (just before sample)
y1=tan(th)*x1+dy2

;x2=2.165  ;distance from sample to s2 (first collimation slit)
y2=tan(th)*x2+dy1

;x3 distance from sample to mid chopper distance  (this should be given to this routine rather then hardwired as it depends
; on chopper resolution)
x4=3.054   ;sam-det distance

;origin is the centre of the sample

v=3956./lam  ;neutron velocity
k=g/(2.*v^2) ;a characteristic inverse length
;x0=0.5*(tan(th)/k-((x2^2-x1^2)/(x1-x2)))  ; this is the distance in x to the top of the parabola
; change this as we need it in terms of y

x0=((y1-y2)/k-(x2^2-x1^2))/(2.*(x1-x2))


h=y1+k*(x1-x0)^2   ; h is the height for the top of the parabola

;equation for parabola is y=h-(x-x0)^2


dh=(h-k*(x4+x0)^2)+x4*tan(th)


delta=x0-sqrt(h/k)  ; shift in x along sample due to gravity
print,'x0 h delta y1 y2 x1 x2',x0,h,delta,y1,y2,x1,x2


grad=(2.*k)*(x0-delta)  ;gradient of parabola ( to find true theta at y=0)

thetat=dpr*atan(grad)
dz=h-k*(x0^2) ; if beam carrys on to x=0 what height would it be below sample

chopz=x3*tan(th)-(h-k*(x3-x0)^2)

poffoff=dpr*(chopz/cr)

gradc=(2.*k)*(x0-x3)  ;gradient at the midchopper distance
openofflam=dpr*(atan(gradc)-th)

th=th*dpr
end


; Programme to calculate and display the intensity profile of neutrons incident
; on a sample trough two slits.

;===============================================================================
;shows error message window
;===============================================================================
PRO errormessage,messy

; common block holding the base's id
COMMON base_id,base

; desensitise base
    WIDGET_CONTROL,base,SENSITIVE=0
;create floating message widget
    msg_base=WIDGET_BASE(group_leader=base,/FLOATING,TITLE='Error Message',/COLUMN)

; label and give widget a button with 'ok' on
    mess=WIDGET_LABEL(msg_base,VALUE=messy)
    errbut=WIDGET_BUTTON(msg_base,VALUE='OK',UVALUE='ok')

; realise and pass control to event handler
    WIDGET_CONTROL,msg_base,/REALIZE
    XMANAGER,'message',msg_base
END

;===============================================================================
;handles events from errormessage
;===============================================================================
PRO message_event,event

; common block for base widget ID
COMMON base_id,base

; get user value from button that caused event
    WIDGET_CONTROL,event.id,GET_UVALUE=ev,GET_VALUE=val

; If Okay button was pressed then resensitise the base widget and destroy
; the message widget
IF ev EQ 'ok' THEN BEGIN
    WIDGET_CONTROL,event.top,/DESTROY
    WIDGET_CONTROL,base,SENSITIVE=1
ENDIF

END

;===============================================================================
; Function to extract a floating point from a text widget specified by
; widget_id. Returns a floating point or an error if incorrectly formatted.
;===============================================================================
FUNCTION getfield, widget_id

; Common block used to pass errors
COMMON eflags,eflag

; initialise: no error
    eflag=0

; get string from text widget and remove white space
    WIDGET_CONTROL,widget_id,GET_VALUE=text
    text=STRCOMPRESS(text[0],/REMOVE_ALL)
; convert to a float
    number = FLOAT(text)
; convert back to string to compare to original string. If they're not the same
; then the string isn't a correctly formatted floating point number so signal error
    checkstr = STRCOMPRESS(STRING(number),/REMOVE_ALL)
; check to see if one is substring of the other (either will do). Can't just check
; that, e.g. checkstr is a substring of text 'coz if text is '1.' then checkstr
; ='1.000000' and vice vera
    IF (STRPOS(checkstr,text) EQ -1) AND  (STRPOS(text,checkstr) EQ -1)THEN BEGIN
       eflag = 1
       RETURN,0.0
; otherwise return the number
    ENDIF ELSE RETURN,number
END

;===============================================================================
; Procedure that does the calculations.
; No comments because I'm not really sure how it works.
;===============================================================================
PRO calculate,d1,d2,d3,s1,s2,sam,theta,lam

COMMON base_id,base
COMMON plot_base,plotbase,illum,delth

; convert theta to rad.
    theta=!pi*theta/180.0
; calc. footprint of sample (projection onto slit)
    footprint=sam*sin(theta)

    IF s1 GT s2 THEN fwhm=s1*180.0/(d1*!pi)    ELSE fwhm=s2*180.0/(d1*!pi)

    arange=2.0*ATAN((s1+s2)/(2.0*d1))


    del=fwhm*!pi/(theta*180.0)
    arange=arange*180.0/!pi

    IF s1 NE s2 THEN BEGIN
       x=d2-((d1*s2)/(s1-s2))
       w2=ABS((s2*x)/(d2-x))
    ENDIF ELSE BEGIN
       w2=s1
       x=9999.0
    END

    w1 = s2+((d2*(s1+s2))/d1)

;calculate for lambda max illumination


   x3=d3/1000.  ; midchop to sample dist
   x1=d2/1000.  ; sam-s3 dist
   x2=(d1+d2)/1000.  ;sam-s2 dist

   dy1=-1.*s1/2000.
   dy2=s2/2000.
   grav,theta*180./!pi,lam,thetat,delta,dz,poffoff,openofflam,dh,x1,x2,x3,dy1,dy2
   print,lam,delta
   l1=(delta)*1000.+sam/2.


   dy1=s1/2000.
   dy2=s2/2000.
   grav,theta*180./!pi,lam,thetat,delta,dz,poffoff,openofflam,dh,x1,x2,x3,dy1,dy2
   print,lam,delta
   l2=(delta)*1000.+sam/2.

   dy1=-1.*s1/2000.
   dy2=-1.*s2/2000.
   grav,theta*180./!pi,lam,thetat,delta,dz,poffoff,openofflam,dh,x1,x2,x3,dy1,dy2
   print,lam,delta
   r1=(delta)*1000.+sam/2.

   dy1=1.*s1/2000.
   dy2=-1.*s2/2000.
   grav,theta*180./!pi,lam,thetat,delta,dz,poffoff,openofflam,dh,x1,x2,x3,dy1,dy2
   print,lam,delta
   r2=(delta)*1000.+sam/2.
   print,l1,l2,r1,r2
    nt=100

    xg=FLTARR(nt+1)
    yg=FLTARR(nt+1)

    FOR i=0,nt DO BEGIN
       xg[i]=(FLOAT(i)/nt)*sam
       y=(FLOAT(i)/nt)*sam
       IF y LT l1 THEN z=0
       IF (y GE l1) AND (y LT l2) THEN z=(y-l1)/(l2-l1)
       IF (y GE l2) AND (y LT r1) THEN z=1
       IF (y GE r1) AND (y LT r2) THEN z=(r2-y)/(r2-r1)
       IF y GE r2 THEN z=0
       yg[i]=z
    END


  glim=r2




; "Convert to and intensity distribution on sample" (apparently)
    th1=ATAN((s1+s2)/(2.0*d1))
    IF s1 EQ s2 THEN th2 = 0 ELSE th2 = ATAN(s2/(2.0*(d2-x)))

    r1=(sam/2.0)+(w2/(2.0*COS(theta)*(TAN(theta)+TAN(th2))))
    r2=(sam/2.0)+(w1/(2.0*COS(theta)*(TAN(theta)+TAN(th1))))
    l1=(sam/2.0)-(w1/(2.0*COS(theta)*(TAN(theta)-TAN(th1))))
    l2=(sam/2.0)-(w2/(2.0*COS(theta)*(TAN(theta)-TAN(th2))))
 print,l1,l2,r1,r2
    nt=100

    xx=FLTARR(nt+1)
    yy=FLTARR(nt+1)

    FOR i=0,nt DO BEGIN
       xx[i]=(FLOAT(i)/nt)*sam
       y=(FLOAT(i)/nt)*sam
       IF y LT l1 THEN z=0
       IF (y GE l1) AND (y LT l2) THEN z=(y-l1)/(l2-l1)
       IF (y GE l2) AND (y LT r1) THEN z=1
       IF (y GE r1) AND (y LT r2) THEN z=(r2-y)/(r2-r1)
       IF y GE r2 THEN z=0
       yy[i]=z
    END

     dth_by_th = 0.98*(atan( (s1 + s2) / (2 * d1))^3 - atan(abs(s1-s2)/(2 *d1))^3)/(atan( (s1 + s2) / (2 * d1)) - atan(abs(s1-s2)/(2 *d1)))/(atan( (s1 + s2) / (2 * d1)) + atan(abs(s1-s2)/(2 *d1)))/ theta	;PG: This is the Gaussian equivalent FWHM calculated via the mean absolute deviation
    ;dth_by_th = 2*ATAN((s1+s2)/(2.0*d1))/theta ;only valid for triangular 2*FWHM
    illumination = 100*(glim-l1)/sam

    IF illumination GT 100 THEN $
    illumination = 'More than 100% ! (sample footprint smaller than beam)' $
    ELSE illumination = STRCOMPRESS(STRING(illumination),/REMOVE_ALL)+'%'

; check that plot base exists if it doesn't then create the widget
    WIDGET_CONTROL,plotbase,BAD_ID=check
    IF check NE 0 THEN BEGIN
; Create a drawing widget with the plot in
       plotbase=WIDGET_BASE(GROUP_LEADER=base,TITLE='Intensity across sample (mm)',/COLUMN)
       illum=WIDGET_LABEL(plotbase,/ALIGN_CENTER,/DYNAMIC_RESIZE,VALUE='Illumination = '+ illumination)
       delth=WIDGET_LABEL(plotbase,/ALIGN_CENTER,SCR_YSIZE=35,VALUE='d(theta)/theta = '+STRCOMPRESS(STRING(dth_by_th),/REMOVE_ALL))
       graph=WIDGET_DRAW(plotbase,XSIZE=700,YSIZE=500,RETAIN=2)

; give it an ok button
       butt=WIDGET_BUTTON(plotbase,VALUE='OK',UVALUE='ok')
       WIDGET_CONTROL,plotbase,/REALIZE
       PLOT,xx,yy,YRANGE=[0,1.1]
       OPLOT,xg,yg,linestyle=2

; handle click of OK button using general message event handler
       XMANAGER,'message',plotbase
; If plotbase does exist then just replot
    ENDIF ELSE BEGIN
       PLOT,xx,yy,YRANGE=[0,1.1]
       WIDGET_CONTROL,illum,SET_VALUE=   'Illumination = '+illumination
       WIDGET_CONTROL,delth,SET_VALUE='d(theta)/theta = '+STRCOMPRESS(STRING(dth_by_th),/REMOVE_ALL)
    END

END

;===============================================================================
; Event handler for slitwit
;===============================================================================
PRO slitwit_event, ev

; common block for error flags
COMMON eflags,eflag

; common block containing ID's for text widgets
COMMON editfields, fields


; get user value (text string) to determine whihc button was pressed
    WIDGET_CONTROL, ev.id, GET_UVALUE=button_id

; If quit button was pressed then end
    IF (ev.select) AND (button_id eq 'QUIT') THEN WIDGET_CONTROL, ev.top, /DESTROY

; If do button was pressed perform calcs.
    IF (ev.select) AND (button_id eq 'DO') THEN BEGIN

       d1=getfield(fields[0])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Inter Slit distance must be floating point!'
         RETURN
       END
       d2=getfield(fields[1])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Sample-Slit distance must be floating point!'
         RETURN
       END
       sam=getfield(fields[2])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Sample length must be floating point!'
         RETURN
       END
       theta=getfield(fields[3])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Theta must be floating point!'
         RETURN
       END
       s2=getfield(fields[4])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Sample slit width must be floating point!'
         RETURN
       END
       s1=getfield(fields[5])
       IF eflag NE 0 THEN BEGIN
         errormessage,'1st slit width must be floating point!'
         RETURN
       END
        d3=getfield(fields[6])
       IF eflag NE 0 THEN BEGIN
         errormessage,'Mid chop to sample dist must be floating point!'
         RETURN
       END

        lam=getfield(fields[7])
       IF eflag NE 0 THEN BEGIN
         errormessage,'wavelength must be floating point!'
         RETURN
       END

; If everything checks ok then do calculations
       calculate,d1,d2,d3,s1,s2,sam,theta,lam
    END
END

;===============================================================================
PRO gitwit

COMMON base_id,base

; common block to pass ID's of field widgets
COMMON editfields, fields

COMMON plot_base,plotbase,illum,delth

; Initialise plotbase: widget ID
    plotbase=-1

; base widget
    base = WIDGET_BASE(TITLE='GitWit',/COLUMN)

; arrays for the id numbers of the label and field widgets
    labels = LONARR(8)
    fields = LONARR(8)

; base containing all of the fields to be enterd
    edit_base = WIDGET_BASE(base,COLUMN=2)

; column of labels
    labels[0] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Inter Slit distance (mm)')
    labels[1] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Sample-slit distance (mm)')
    labels[2] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Sample length (mm)')
    labels[3] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Theta (degrees)')
    labels[4] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Sample slit    width,s3w (mm)')
    labels[5] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='1st slit width,s2w (mm)')
    labels[6] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Mid chopper to sample distance (mm)')
    labels[7] = WIDGET_LABEL(edit_base,/ALIGN_RIGHT,SCR_YSIZE=35,VALUE='Maximum wavelength (A)')
; column of field entry text widgets
    fields[0] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='2165.0')
    fields[1] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='390.0')
    fields[2] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='80.0')
    fields[3] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='0.624')
    fields[4] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='0')
    fields[5] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='0')
    fields[6] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='5331.0')
    fields[7] = WIDGET_TEXT(edit_base,SCR_YSIZE=35,/EDITABLE,VALUE='30.0')

; Do and Quit buttons. UVALUE='...' allows the button to be indetified in the
; event handler without knowing which button the id corresponds to
    quit_button = WIDGET_BUTTON(base, VALUE='QUIT', UVALUE='QUIT')
    do_button = WIDGET_BUTTON(base, VALUE='DO IT!', UVALUE='DO')

    WIDGET_CONTROL, base, /REALIZE
    XMANAGER, 'SlitWit', base
END
