function elascan_constant, w_data, background_w=n_container, f=f
;+
;******* ****************
;** User callable
;*******************************************************************************
;**
;** Substract the background multiplied by the given factor f from the sample signal
;** on an elastic scan.
;**
;** Call w6=elascan_constant (w1, background_w=2, f=0.9)
;**
;**
;** MAG (23/4/2009)
;**
;**
;*******************************************************************************
;-


; ------------------------------------------------------------------------------
;Read data and background workspaces (for background use only first spectrum at the moment)

;take sample data
take_datp, datp & x_data=datp.x & y_data=datp.y & e_data=datp.e & n_data=datp.n & par = datp.p

print
; check dimensions
sw=n_elements(w_data) & se=n_elements(e_data)
sx=n_elements(x_data) & sy=n_elements(y_data) & sn=n_elements(n_data)
if (sw ne se) then begin & print,'Elascan_constant: Problem! w & e do not have the same number of elements' & return,1 & endif
if (sx*sy ne sw) then begin & print,'Elascan_constant: Problem! Incompatible sizes of x, y and w' & return,1 & endif
if (sx ne sn) then begin & print,'Elascan_constant: Problem! x & n do not have the same dimensions' & return,1 & endif
print,'Elascan_constant:'
print,' Number of temperatures in sample measurement = ',sx

if n_elements(n_container) gt 0 then begin
   ;get background data
   take_datp, datb, W=n_container & take_w,w_background,W=n_container
   par = datb.p
   sb = size(w_background)
   if (sb[2] ne sy) then begin
      print,'Elascan_constant:'
      print,' ERROR! Different number of detectors in sample and background measurements'
      return,0
   endif
   x_background = datb.x  ;temperatures
   y_background = datb.y  ;scattering angles
   e_background = datb.e
   n_background = datb.n
   svx=n_elements(x_background)
   print,' Number of temperatures in background measurement = ',svx
endif else begin
   print,'Elascan_constant:'
   print,' No background measurement given: w_out = w_in!'
   w_out = w_data
   e_out = e_data
   x_out = x_data
   y_out = y_data
   mod_datp, datp, 'X', x_out
   mod_datp, datp, 'Y', y_out
   mod_datp, datp, 'E', e_out
   mod_datp, datp, 'N', n_out
   give_datp, datp
   give_datp, datp
   return, w_out
endelse

w_out = w_data
e_out = e_data
x_out = x_data
y_out = y_data

; ------------------------------------------------------------------------------
; Check input

if (n_elements(f) eq 0) then f=1.0


; ------------------------------------------------------------------------------
;  Subtract background for each temperature. Background substracted is the one measured
;  at the closest T at which the sample has been measured.

openw,u,'temperatures.dat',/get_lun
printf,u,'      T_sample   T_background'
for i=0,sx-1 do begin

   ; Find index corresponding to smaller T difference between sample and background
   diff_temp = abs(x_background - x_data[i])
   min_temp = min(diff_temp)
   itemp = where(diff_temp eq min_temp)
   ibackground = itemp[0]
   printf,u,x_data[i],x_background[ibackground]

   ; Subtract corresponding background using the appropriate correction factors
   ; Scale background by monitor ratio if sample data and background have not been normalized
   ; Note that scale is an array of 1 element --> use scale(0) to address its value

   scale = float(n_data[i]) / float(n_background[ibackground])

   w_out[i,*] = w_data[i,*] - f * w_background[ibackground,*] * scale[0]
   e_out[i,*] = sqrt( e_data[i,*]^2 + (f * e_background[ibackground,*] * scale[0])^2 )

endfor
free_lun,u

;Return parameters and exit
mod_datp, datp, 'X', x_out
mod_datp, datp, 'Y', y_out
mod_datp, datp, 'E', e_out
mod_datp, datp, 'N', n_out
give_datp, datp

return,w_out
end
