
FUNCTION elascan_format, w_in, low_in16psd = low, up_in16psd = up
;+
;******* **************
;** User callable
;*******************************************************************************
;**
;** Change the format of the data read in a single workspace in order to have a
;** common format for the elastic scan data, independent of the instrument
;** (IN10, IN13 or IN16) were they were taken.
;**
;** Call: w2 = elascan_format(w1)
;**
;** And the data are stored in the following way:
;**   w2 = 2D-array[n_temperatures, n_detectors] (duplicated in X if only 1 temperature)
;**   x2 = 1D-array[n_temperatures]
;**   y2 = 1D-array[n_detectors]
;**   e2 = 2D-array[n_temperatures, n_detectors]
;**   n2 = 1D-array[n_temperatures]
;**
;** MAG (11/5/2009)
;**
;*******************************************************************************
;-

COMMON c_lamp_access, inst

;take sample data
take_datp, datp ;& help,datp,/struc
par = datp.p

print
print,'Elascan_format:'
print,' Instrument = ', inst

if (inst eq 'IN10') then begin

   if datp.p[0] eq 1 then begin  ;Old MAD numors read
   
      sw  = size(w_in)
      if (sw[0] gt 1) then ntemperatures  = sw[1] else ntemperatures =1
      if (sw[0] gt 1) then ndetectors = sw[2] else ndetectors = sw[1]
      print,' Number of temperatures =', ntemperatures
      print,' Number of detectors =', ndetectors
      datp.x_tit = 'T/K'
      datp.y_tit = 'Angle (degrees)'
      if (ntemperatures gt 1) then begin
         w_out = fltarr(ntemperatures,ndetectors)
         e_out = fltarr(ntemperatures,ndetectors)
         x_out = fltarr(ntemperatures) & x_out = datp.x  ;temperature values
         y_out = fltarr(ndetectors) & y_out = datp.y     ;scattering angles
         w_out = w_in
         e_out = sqrt(w_out)
         n_out = fltarr(ntemperatures) & n_out = datp.n  ;one monitor value per temperature
      endif else begin
         w_out = fltarr(2,ndetectors)
         e_out = fltarr(2,ndetectors)
         x_out = fltarr(2) & x_out[0]=300. & x_out[1]=300.;no temperature information, setting RT
         y_out = fltarr(ndetectors) & y_out = datp.p[14:14+ndetectors-1]  ;scattering angles
         w_out[0,*] = w_in & w_out[1,*] = w_in
         e_out = sqrt(w_out)
         n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
         print,' WARNING! Only one temperature value. Array duplicated!'
         print,'          Temperature not read, so T set to 300 K'
         print,'          If needed, change the value with the command xN(0)=T & xN(1)=T'
      endelse
      
   endif else begin ;New NOMAD numors read --> angle/temperatures reversed

      sw  = size(w_in)
      if (sw[0] gt 1) then ntemperatures  = sw[2] else ntemperatures =1
      ndetectors = sw[1]
      print,' Number of temperatures =', ntemperatures
      print,' Number of detectors =', ndetectors
      datp.x_tit = 'T/K'
      datp.y_tit = 'Angle (degrees)'

      if (ntemperatures gt 1) then begin
         w_out = fltarr(ntemperatures,ndetectors)
         e_out = fltarr(ntemperatures,ndetectors)
         x_out = fltarr(ntemperatures) & x_out = datp.y   ;temperature values
         y_out = fltarr(ndetectors) & y_out = datp.x[*,0] ;scattering angles
         w_out = transpose(w_in)
         e_out = sqrt(w_out)
         n_out = fltarr(ntemperatures) & n_out = datp.n  ;one monitor value per temperature
      endif else begin
         w_out = fltarr(2,ndetectors)
         e_out = fltarr(2,ndetectors)
         x_out = fltarr(2) & x_out[0]=datp.y & x_out[1]=datp.y ;temperature 
         y_out = fltarr(ndetectors) & y_out = datp.x  ;scattering angles
         w_out[0,*] = w_in & w_out[1,*] = w_in
         e_out = sqrt(w_out)
         n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
         print,' WARNING! Only one temperature value. Array duplicated!'
         print,'          If needed, change the value with the command xN(0)=T & xN(1)=T'
      endelse

   endelse   

endif

if (inst EQ 'IN13') then begin
   sw  = size(w_in)
   if (sw[0] gt 1) then ntemperatures  = sw[2] else ntemperatures =1
   ndetectors = sw[1]
   print,' Number of temperatures =', ntemperatures
   print,' Number of detectors =', ndetectors
   if datp.x_tit ne 'T/K' then begin
      datp.x_tit = 'T/K'
      datp.y_tit = 'Angle (degrees)'
      if (ntemperatures gt 1) then begin
         w_out = fltarr(ntemperatures,ndetectors)
         e_out = fltarr(ntemperatures,ndetectors)
         x_out = fltarr(ntemperatures) & x_out = datp.y   ;temperatures
         y_out = fltarr(ndetectors) & y_out = datp.x[*,0] ;scattering angles
         w_out = transpose(w_in)
         e_out = transpose(datp.e)
         n_out = fltarr(ntemperatures) & n_out = datp.n[*] ;one monitor value per temperature
      endif else begin
         w_out = fltarr(2,ndetectors)
         e_out = fltarr(2,ndetectors)
         x_out = fltarr(2) & x_out[0]=datp.y & x_out[1]=datp.y ;T
         y_out = fltarr(ndetectors) & y_out = datp.x[*,0] ;scattering angles
         w_out[0,*] = w_in[*] & w_out[1,*] = w_in[*]
         e_out[0,*] = datp.e[*] & e_out[1,*] = datp.e[*]
         n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
         print,' WARNING! Only one temperature value. Array duplicated!'
      endelse
   endif else begin
      print,' Data already formatted for elascan: X = T/K, Y = Detector angles'
      w_out = fltarr(ntemperatures,ndetectors)
      e_out = fltarr(ntemperatures,ndetectors)
      x_out = fltarr(ntemperatures) & x_out = datp.x  ;temperature values
      y_out = fltarr(ndetectors) & y_out = datp.y     ;scattering angles
      w_out = w_in
      e_out = sqrt(w_out)
      n_out = fltarr(ntemperatures) & n_out = datp.n  ;one monitor value per temperature
   endelse
endif


if (inst EQ 'IN16') then begin

   ;date = strmid(datp.time,1,/reverse_offset)
   ;date = fix(date) ;convert to integer
   ;if date le 70 then year = 2000+date else year = 1900+date
   ;print,' Data from year = ',year

   ;check if elastic scan:
   ;  type of scan =  1 --> MAD format (all temperatures in a single numor)
   ;                 21 --> NOMAD format (one T per numor) with multidetector
   ;                 31 --> NOMAD format (one T per numor) with PSD
   
   case long(datp.p[0]) of 
   
      1: BEGIN

         print, ' MAD elastic scan '
         sw  = size(w_in)
         if (sw[0] gt 1) then ntemperatures  = sw[1] else ntemperatures = 1
         if (sw[0] gt 1) then ndetectors = sw[2] else ndetectors = sw[1]
         print, ' Number of temperatures = ', ntemperatures
         print, ' Number of detectors = ', ndetectors
         datp.x_tit = 'T/K'
         datp.y_tit = 'Angle (degrees)'
         if (ntemperatures gt 1) then begin
            w_out = fltarr(ntemperatures,ndetectors)
            e_out = fltarr(ntemperatures,ndetectors)
            x_out = fltarr(ntemperatures) & x_out = datp.x  ;temperature values
            y_out = fltarr(ndetectors) & y_out = datp.y     ;scattering angles
            w_out = w_in
            e_out = sqrt(w_out)
            n_out = fltarr(ntemperatures) & n_out = datp.n  ;one monitor value per temperature
         endif else begin
            w_out = fltarr(2,ndetectors)
            e_out = fltarr(2,ndetectors)
            x_out = fltarr(2) & x_out[0]=300. & x_out[1]=300.;no temperature information, setting RT
            ;getting scattering angles
            if datp.p[19] gt 0 then begin ;multidetector present
               nmd = datp.p[30] ;number of tubes in MD
               nsingle = ndetectors - nmd ;number of single detectors present
               y_out = fltarr(ndetectors)
               if nsingle gt 0 then begin
                  y_out[0:nsingle-1] = datp.p[20:20+nsingle-1]
                  for id=0,nmd-1 do y_out[nsingle+id] = datp.p[28] + id*datp.p[29]
               endif else for id=0,nmd-1 do y_out[id] = datp.p[28] + id*datp.p[29]
            endif else begin ;only single detectors
               if ndetectors gt 8 then print, $
                  ' Warning: More than 8 single detectors? Only eight angles written in params! '
               y_out = fltarr(ndetectors) & y_out = datp.p[20:20+ndetectors-1]
            endelse
            w_out[0,*] = w_in[*] & w_out[1,*] = w_in[*]
            e_out = sqrt(w_out)
            n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
            print, ' WARNING! Only one temperature value. Array duplicated! '
            print, '          Temperature not read, so T set to 300 K '
            print, '          If needed, change the value with the command xN(0)=T & xN(1)=T '
         endelse

         END
      
     21: BEGIN

         print, ' NOMAD elastic scan with multidetector'
         sw  = size(w_in)
         ndetectors = sw[1]
         w_tmp = w_in
         if sw[0] eq 1 then ntemperatures = 1 else ntemperatures = sw[2]
         print, ' Number of temperatures = ', ntemperatures
         print, ' Number of detectors = ', ndetectors
         y_tmp = datp.x[*,0]  ;scattering angles will be written in y
         ;eliminate dummy detectors
         id = where( y_tmp > 0.0)
         sw = size(id)
         ndetectors = sw[1]
         print,' Number of useful detectors =', ndetectors
         y_out = fltarr(ndetectors) & y_out = y_tmp[id]
         datp.x_tit = 'T/K'
         datp.y_tit = 'Angle (degrees)'
      
         if (ntemperatures gt 1) then begin
 
            x_out = fltarr(ntemperatures) & x_out[*] = datp.pv[9,*]   ;temperatures
            w_out = fltarr(ntemperatures,ndetectors) & w_out = transpose(w_tmp[id,*])
            e_out = fltarr(ntemperatures,ndetectors) & e_out = sqrt(w_out)
            n_out = datp.n

         endif else begin
 
            w_out = fltarr(2,ndetectors)
            e_out = fltarr(2,ndetectors)
            x_out = fltarr(2) & x_out[0]=datp.p[9] & x_out[1]=datp.p[9] ;T
            w_out[0,*] = w_tmp[id] & w_out[1,*] = w_tmp[id]
            e_out = sqrt(w_out)
            n_out = fltarr(2) & n_out[0] = total(datp.n[*,0],1) & n_out[1] = total(datp.n[*,0],1)
            print, ' WARNING! Only one temperature value. Array duplicated! '

         endelse
   
         END

     31: BEGIN
     
         print, ' NOMAD elastic scan with PSD (not integrated) '
         sw  = size(w_in)
         ndetectors = sw[1]
         w_tmp = w_in
         if n_elements(low) eq 0 then low = 0      ;define integration limits in PSD
         if n_elements(up) eq 0 then up = sw[2]-1
         if sw[0] eq 2 then begin
            ntemperatures = 1 
            w_tmp = total(w_in[*,low:up],2)
         endif else begin
            ntemperatures = sw[3]
            w_tmp = total(w_in[*,low:up,*],2)
         endelse   
         if (low gt 0) and (up lt sw[2]-1) then begin
            ; Correct single detectors (in case counting lost because of integral over reduced limits)
            nsd = ndetectors - datp.p[30] ;datp.p[30] = number of PSD tubes
            if ntemperatures eq 1 then w_tmp[0:nsd-1] = w_in[0:nsd-1,0] else w_tmp[0:nsd-1,*] = w_in[0:nsd-1,0,*]
         endif    
         print, ' Number of temperatures = ', ntemperatures
         print, ' Number of detectors = ', ndetectors
         y_tmp = datp.x[*,0]  ;scattering angles will be written in y
         ;eliminate dummy detectors
         id = where( y_tmp > 0.0)
         sw = size(id)
         ndetectors = sw[1]
         print, ' Number of useful detectors = ', ndetectors
         y_out = fltarr(ndetectors) & y_out = y_tmp[id]
         datp.x_tit = 'T/K'
         datp.y_tit = 'Angle (degrees)'
      
         if (ntemperatures gt 1) then begin
 
            x_out = fltarr(ntemperatures) & x_out[*] = datp.pv[9,*]   ;temperatures
            w_out = fltarr(ntemperatures,ndetectors) & w_out = transpose(w_tmp[id,*])
            e_out = fltarr(ntemperatures,ndetectors) & e_out = sqrt(w_out)
            n_out = datp.n

            ;get one monitor value per temperature
            ;s = size(datp.n)
            ;if s[0] eq 1 then monitor = reform(datp.n,n_elements(datp.x),ntemperatures) else $
            ; monitor = reform(datp.n[*,0,*],n_elements(datp.x),ntemperatures) ;using first monitor if more than one available
            ;n_out = fltarr(ntemperatures) & n_out(*) = total(monitor,1)  
 
         endif else begin
 
            w_out = fltarr(2,ndetectors)
            e_out = fltarr(2,ndetectors)
            x_out = fltarr(2) & x_out[0]=datp.p[9] & x_out[1]=datp.p[9] ;T
            w_out[0,*] = w_tmp[id] & w_out[1,*] = w_tmp[id]
            e_out = sqrt(w_out)
            n_out = fltarr(2) & n_out[0] = total(datp.n[*,0],1) & n_out[1] = total(datp.n[*,0],1)
            print, ' WARNING! Only one temperature value. Array duplicated! '

         endelse
  
         END
 
     41: BEGIN
     
         print, ' NOMAD elastic scan with PSD (already integrated over tubes and normalized to monitor) '
         sw  = size(w_in)
         ndetectors = sw[1]
         w_tmp = w_in
         if sw[0] eq 1 then ntemperatures = 1 else ntemperatures = sw[2]
         print, ' Number of temperatures = ', ntemperatures
         print, ' Number of detectors = ', ndetectors
         y_tmp = datp.x[*,0]  ;scattering angles will be written in y
         ;eliminate dummy detectors
         id = where( y_tmp > 0.0)
         sw = size(id)
         ndetectors = sw[1]
         print, ' Number of useful detectors = ', ndetectors
         y_out = fltarr(ndetectors) & y_out = y_tmp[id]
         datp.x_tit = 'T/K'
         datp.y_tit = 'Angle (degrees)'
      
         if (ntemperatures gt 1) then begin
 
            x_out = fltarr(ntemperatures) & x_out[*] = datp.pv[9,*]   ;temperatures
            w_out = fltarr(ntemperatures,ndetectors) & w_out = transpose(w_tmp[id,*])
            e_out = fltarr(ntemperatures,ndetectors) 
            if (total(datp.e) > 0) then e_out = transpose(datp.e[id,*]) else e_out = sqrt(w_out)
            ;n_out = datp.n[0,0,*]
            ;n_out = reform(n_out, ntemperatures)
            n_out = datp.n
 
         endif else begin
 
            w_out = fltarr(2,ndetectors)
            e_out = fltarr(2,ndetectors)
            x_out = fltarr(2) & x_out[0]=datp.p[9] & x_out[1]=datp.p[9] ;T
            w_out[0,*] = w_tmp[id] & w_out[1,*] = w_tmp[id]
            if (total(datp.e) > 0) then e_out[0,*] = datp.e[id] else e_out[0,*] = sqrt(w_out[0,*])
            e_out[1,*] = e_out[0,*]
            n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
            print, ' WARNING! Only one temperature value. Array duplicated! '

         endelse
 
         END


   else: BEGIN
         print, ' Type of scan not recognized! '
         print, ' Valid scans for elascan_format are only '
         print, '   1: MAD temperature scan '
         print, '  21: NOMAD elastic scan with multidetector '
         print, '  31: NOMAD elastic scan with PSD '
         print, '  41: NOMAD elastic scan with PSD integrated over tubes'
         print, ' Current type = ', long(datp.p[0])
         print, ' Returning input workspace'
         return, w_in
         END

   endcase      

endif


if (inst EQ 'IN16B') then begin

      sw  = size(w_in)
      if (sw[0] gt 1) then ntemperatures  = sw[1] else ntemperatures = 1
      if (sw[0] gt 1) then ndetectors = sw[2] else ndetectors = sw[1]
      print, ' Number of temperatures = ', ntemperatures
      print, ' Number of detectors = ', ndetectors
      datp.x_tit = 'T/K'
      datp.y_tit = 'Angle (degrees)'
      if (ntemperatures gt 1) then begin
        w_out = fltarr(ntemperatures,ndetectors)
        e_out = fltarr(ntemperatures,ndetectors)
        x_out = fltarr(ntemperatures) & x_out = transpose(datp.pv[9,*])  ;temperature values
        y_out = fltarr(ndetectors) & y_out = datp.x[*,0]                 ;scattering angles
        w_out = transpose(w_in)
        e_out = sqrt(w_out)
        n_out = fltarr(ntemperatures) & n_out = datp.n  ;one monitor value per temperature
      endif else begin
        w_out = fltarr(2,ndetectors)
        e_out = fltarr(2,ndetectors)
        x_out = fltarr(2) & x_out[0]=datp.p[9] & x_out[1]=datp.p[9]
        y_out = datp.x
        w_out[0,*] = w_in[*] & w_out[1,*] = w_in[*]
        e_out = sqrt(w_out)
        n_out = fltarr(2) & n_out[0] = datp.n & n_out[1] = datp.n
        print, ' WARNING! Only one temperature value. Array duplicated! '
      endelse

endif

; Return parameters
mod_datp, datp, 'X', x_out
mod_datp, datp, 'Y', y_out
mod_datp, datp, 'E', e_out
mod_datp, datp, 'N', n_out
give_datp, datp

return, w_out
end

