function elascan_groupt, w_in, dT=dT, average=average, sort=srt
;+
;******* **************
;** User callable
;*******************************************************************************
;**
;** Group points of an elastic scan having the same temperature (y+-dT)
;**
;** Call: w2=elascan_groupt(w1)
;**
;** Spectra are added unless average is set.
;** If input spectra are not sorted you can use the /sort keyword
;-

if n_elements(dT) eq 1 then d=dT else d=0.0

;take sample data
take_datp, datp & x=datp.x & y=datp.y & e=datp.e & n=datp.n & par = datp.p
w_tmp = w_in

print
; check x axis contains temperatures & all dimensions are OK
if datp.x_tit ne 'T/K' then print,'Elascan_groupt: X title is not T/K. Continuing, but please check it!'
sx=n_elements(x) & sy=n_elements(y) & sw=n_elements(w_tmp) & se=n_elements(e) & sn=n_elements(n)
if (sw ne se) then begin & print,'Elascan_groupt: Problem! w & e do not have the same number of elements' & return,1 & endif
if (sx*sy ne sw) then begin & print,'Elascan_groupt: Problem! Incompatible sizes of x, y and w' & return,1 & endif
norm_data = 0
if sn eq 1 or max(n) eq 1 then begin
  print, 'Elascan_groupt: Grouping normalized data'
  norm_data = 1
endif else begin
  if (sx ne sn) then begin & print,'Elascan_groupt: Problem! x & n do not have the same dimensions' & return,1 & endif
endelse
print,'Elascan_groupt:'
print,' Number of temperatures before grouping = ',sx

; Sort temperatures?
if keyword_set(srt) then begin & idx=sort(x) & x=x[idx] & w_tmp=w_tmp[idx,*] & e=e[idx,*] & n=n[idx] & endif

; Prepare output
flg = lonarr(sx)+1
w_out = w_tmp*0 & w_out[0,*] = w_tmp[0,*]
e_out = e*0 & e_out[0,*] = e[0,*]^2 ;add square errors into e_out
x_out = x*0 & x_out[0] = x[0]
n_out = x*0 
if norm_data then n_out[0] = 1 else n_out[0] = n[0]
j = 0 & xj=x[0]

; Grouping
for i=1,sx-1 do begin
   if abs(x[i]-xj) le d then begin
     w_out[j,*] = w_out[j,*] + w_tmp[i,*]
     x_out[j] = x_out[j] + x[i]
     e_out[j,*] = e_out[j,*] + e[i,*]^2 ;add square errors into e_out
     if norm_data then n_out[j] = n_out[j] + 1 else n_out[j] = n_out[j] + n[i]
     flg[j] = flg[j]+1
   endif else begin
     j=j+1 & xj = x[i]
     w_out[j,*] = w_tmp[i,*]
     x_out[j] = x[i]
     e_out[j,*] = e[i,*]^2 ;add square errors into e_out
     if norm_data then n_out[j] = 1 else n_out[j] = n[i]
   endelse
endfor

; If only one point remains after grouping, duplicate it to keep same structure of arrays
if j+1 eq 1 then begin
   w_out[j+1,*] = w_out[j,*]
   x_out[j+1] = x_out[j]
   e_out[j+1,*] = e_out[j,*]
   n_out[j+1] = n_out[j]
   flg[j+1] = flg[j]
   j = j+1
   print,' One single temperature after grouping. Duplicated in output workspace'
endif else print,' Number of temperatures after grouping = ',j+1

;Set error bars:
;  e_out contains the sum of the squared standard deviations passed in datp.e, so
;  sigma = sqrt(sigma_1^2 + sigma_2^2 + ...)
e_out = sqrt(e_out[0:j,*])

; Perform average?
if keyword_set(average) then for i=0,j do begin
   w_out[i,*] = w_out[i,*]/flg[i]
   e_out[i,*] = e_out[i,*]/flg[i]
   n_out[i] = n_out[i]/flg[i]
endfor

x_out = x_out[0:j]/flg[0:j]
w_out = w_out[0:j,*]
e_out = e_out[0:j,*]
n_out = n_out[0:j]

; Return parameters
mod_datp, datp, 'X', x_out
mod_datp, datp, 'Y', y_out
mod_datp, datp, 'E', e_out
mod_datp, datp, 'N', n_out
give_datp, datp

return,w_out
end

