function elascan_logplot, w_in,norm=norm
;+
;******* ***************
;** User callable
;*******************************************************************************
;**
;** Changes x axis and intensities in order to plot ln(I) vs Q^2
;** Call: w2=elascan_logplot(w1, norm='q' or 't' or 'qt')
;**
;** MAG (May 2009)
;**
;*******************************************************************************
;-

COMMON c_lamp_access, inst

w_out=w_in
s_in=size(w_in)
if (s_in[0] ne 2) then begin
   print
   print,'Elascan_logplot:'
   print,' Data workspace is not a 2D array. Not valid input!'
   print,' NOTHING DONE!!!'
   return,0
endif
ntemperatures  = s_in[1] & ndetectors = s_in[2]

take_datp, datp

w_out = w_in
e_out = datp.e

;modify Y axis
datp.y=datp.y * datp.y
datp.y_tit='Q^2 (A^-2)'

;normalize intensities

if (n_elements(norm) gt 0) then begin

   select = 0
   if ((norm eq 'q') or (norm eq 'Q')) then begin
      for i = 0,ntemperatures-1 do begin
        w_out[i,*] = w_in[i,*]/w_in[i,0]
        e_out[i,*] = e_out[i,*]/w_in[i,0]
      endfor
      print
      print,'Elascan_logplot: Normalization by I[Q_min]'
      select = 1
      endif
   if ((norm eq 't') or (norm eq 'T'))then begin
      for i = 0,ndetectors-1 do begin
         w_out[*,i] = w_in[*,i]/w_in[0,i]
         e_out[*,i] = e_out[*,i]/w_in[0,i]
      endfor
      print
      print,'Elascan_logplot: Normalization by I[T_min]'
      select = 1
   endif
   if ((norm eq 'qt') or (norm eq 'QT'))then begin
      w_out = w_in / w_in[0,0]
      e_out = e_out/ w_in[0,0]
      print
      print,'Elascan_logplot: Normalization by I[Q_min,T_min]'
      select = 1
   endif
   if (NOT select) then print,'Elascan_logplot: Only T or Q or QT are valid choices for norm. Not normalized!'

endif

;compute ln(i) and estimate error

e_out = 0.5*abs(alog((w_out+e_out)>1e-5)-alog((w_out-e_out)>1e-5))
w_out = alog(w_out >1e-5)

datp.e = e_out
give_datp, datp
return, w_out

end

