PRO elascan_output, w_in, file = file
;+
;** **************
;** User callable
;*******************************************************************************
;**
;** Writes two files containing the output of an elastic scan
;** Give the filename in file (without extension) and the macro will
;** write two files containing I(T) vs Q (name: file_t.dat) and
;** I(Q) vs T (name: file_q.dat)
;**
;** MAG 2009
;**
;*******************************************************************************
;
; Corrections:
; 04.11.09 Error when writing a file containing a single replicated temperature
;          corrected.
;
;-

; ------------------------------------------------------------------------------
; Check for consistencies

s_in=size(w_in)
if (s_in[0] ne 2) then begin
   print
   print,'Elascan_output:'
   print,' Data workspace is not a 2D array. Something wrong!'
   print,' CHECK OUTPUT FILE!!!'
endif
ntemperatures  = s_in[1] & ndetectors = s_in[2]

take_datp, sample ;take parameters corresponding to w_in

; Check if one single (duplicated) temperature
if ntemperatures eq 2 then if sample.x[0] eq sample.x[1] then ntemperatures = 1

OPENW, lun, file + '_t.dat', /GET_LUN
PRINTF, lun, FORMAT = '("# Title = ",A)', sample.w_tit
PRINTF, lun, FORMAT = '("# Number of temperatures = ",I5)', ntemperatures
PRINTF, lun, FORMAT = '("# Number of detectors = ",I5)', ndetectors
PRINTF, lun, FORMAT = '("#",8X,"T",12X,'+strtrim(ndetectors,2)+'("Q=",F5.2,8X,"error",10X))', sample.y
for itemp = 0,ntemperatures-1 do begin
   idx1 = indgen(ndetectors)*2
   idx2 = indgen(ndetectors)*2 + 1
   w_buf = fltarr(2*ndetectors)
   w_buf[idx1] = w_in[itemp,*]
   w_buf[idx2] = sample.e[itemp,*]
   printf, lun, format = '(G,'+strtrim(2*ndetectors,2)+'G)', sample.x[itemp],w_buf
endfor
FREE_LUN, lun

OPENW, lun, file + '_q.dat', /GET_LUN
PRINTF, lun, FORMAT = '("# Title = ",A)', sample.w_tit
PRINTF, lun, FORMAT = '("# Number of temperatures = ",I5)', ntemperatures
PRINTF, lun, FORMAT = '("# Number of detectors = ",I5)', ndetectors
if ntemperatures gt 1 then $
   PRINTF, lun, FORMAT = '("#",8X,"Q",12X,'+strtrim(ntemperatures,2)+'("T=",F6.2,7X,"error",10X))', sample.x $
   else PRINTF, lun, FORMAT = '("#",8X,"Q",12X,"T=",F6.2,7X,"error",10X)', sample.x[0]
for idet = 0,ndetectors-1 do begin
   idx1 = indgen(ntemperatures)*2
   idx2 = indgen(ntemperatures)*2 + 1
   w_buf = fltarr(2*ntemperatures)
   if ntemperatures gt 1 then w_buf[idx1] = w_in[*,idet] else w_buf[idx1] = w_in[0,idet]
   if ntemperatures gt 1 then w_buf[idx2] = sample.e[*,idet] else w_buf[idx2] = sample.e[0,idet]
   printf, lun, format = '(G,'+strtrim(2*ntemperatures,2)+'G)', sample.y[idet],w_buf
endfor
FREE_LUN, lun

end
