function elascan_phi2q, w_in, dq=dq
;+
;******* *************
;** User callable
;*******************************************************************************
;**
;** Tranform from angle to Q and sort and join detectors
;** Call:    w2=elascan_phi2q(w1,dQ=0.1)
;** The keyword dQ is optional. If it is given with a value dq, then
;** the macro also joins detectors having Q values differing by less than dQ!
;**
;** MAG (Mar 2008)
;**
;*******************************************************************************
;-

COMMON c_lamp_access, inst

; ------------------------------------------------------------------------------
; Check for consistencies

w_out=w_in
s_in=size(w_in)
if (s_in[0] ne 2) then begin
   print
   print,'Elascan_phi2q:'
   print,' Data workspace is not a 2D array. Not valid input!'
   print,' NOTHING DONE!!!'
   return,0
endif
ntemperatures  = s_in[1] & ndetectors = s_in[2]

take_datp, sample         ;take parameters corresponding to w_in

; ------------------------------------------------------------------------------
;calculate Q axis
phi=sample.y * !dtor ;angle (in radians)
if (inst eq 'IN13') then wavelength = sample.p[6]
if (inst eq 'IN10') then wavelength = 6.27
if (inst eq 'IN16') then wavelength = 6.27
if (inst eq 'IN16B') then wavelength = 6.27
q = (4.0*!pi/wavelength) * sin(phi/2.0)
sample.y=q
sample.y_tit='Q  (A^-1)'

w_out = w_in
e_out = sample.e ;errors
y_out = sample.y ;Q values
x_out = sample.x ;temperatures
n_out = sample.n ;monitor

;sort by increasing Q value
idx = sort(sample.y)
for i=0,ntemperatures-1 do begin
   y_out = sample.y[idx]
   w_out[i,*] = w_in[i,idx]
   e_out[i,*] = sample.e[i,idx]
endfor

; ------------------------------------------------------------------------------
;join detectors having Q values differing less than dQ (same way as in sumbank)

if n_elements(dq) eq 0 then dq=0.

print
print,'Elascan_phi2q:'

Qmin = -1.
iout = -1
for idet=0,ndetectors-1 do begin
   Q = y_out[idet]
   if (abs(Q-Qmin) gt dQ) then begin
     if idet ne 0 then begin ;new point, so compute average of the preceding point iout before continuing
      y_out[iout]   = y_out[iout]/nout
      w_out[*,iout] = w_out[*,iout]/nout
      e_out[*,iout] = sqrt(e_out[*,iout])/nout
      print,'Group ',iout,': ',nout,' detectors joined into it.'
     endif
     Qmin = Q
     iout = iout + 1
     nout = 1
     y_out[iout]   = y_out[idet]
     w_out[*,iout] = w_out[*,idet]
     e_out[*,iout] = e_out[*,idet]^2 ;add squared errors into e_out
   endif else begin
     nout = nout + 1
     y_out[iout]   = y_out[iout]   + y_out[idet]
     w_out[*,iout] = w_out[*,iout] + w_out[*,idet]
     e_out[*,iout] = e_out[*,iout] + e_out[*,idet]^2 ;add squared errors into e_out
   endelse
endfor

y_out[iout]   = y_out[iout]/nout
w_out[*,iout] = w_out[*,iout]/nout
e_out[*,iout] = sqrt(e_out[*,iout])/nout
print,'Group ',iout,': ',nout,' detectors joined into it.'

y_out = y_out[0:iout]
w_out = w_out[*,0:iout]
e_out = e_out[*,0:iout]
ngroups = iout + 1

print
print,' Data grouped into ',ngroups,' Q values.'

; Return parameters
mod_datp, sample, 'X', x_out
mod_datp, sample, 'Y', y_out
mod_datp, sample, 'E', e_out
mod_datp, sample, 'N', n_out
give_datp, sample
return,w_out

end

