;+
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
;&&&&&&&&&&&&&    PSDs' radial integration   &&&&&&&&&&&&&&&&&&&&&&
;&&&&&&  Elena Cornicchi, Universit� di Perugia, Italia (2004)&&&&&
;-

pro psd_integration,wout_psd0,qm1,qm2,qm3,i1,i2,i3,e1,e2,e3


  wavel=2.23  ; wavelenght(expressed in Angstrom) of IN13
  ss=size(wout_psd0) ; wout_psd0 is a 16 columns X 128 lines array
  
  ;*****************************************************************************
  ;wout_psd0 has to be "uploaded" taking into account for the empty space between
  ;the 8th and the 9th tube. This empty space correspond to 3 "virtual" tubes
  ;*****************************************************************************
  
  ntub=ss[1]+3.
  wout_psd1=wout_psd0[0:7,0:126]    ; tubes 1-8
  wout_psd2= bytarr(3,ss[2]-1)      ; 3 "virtual" tubes
  wout_psd3=wout_psd0[8:15,0:126]   ; tubes 9-16
  
  wout_psd0=[wout_psd1,wout_psd2,wout_psd3] ;19 columns X 128 lines array
  
  
  ; vertical resolution: 2.1 cm. Binning on steps of 6 cells
  
  wout_psd=fltarr(19,21)
  s=size(wout_psd)
  
  for k=0,18 do begin
    j=0
    for i=1,126,6 do begin
      wout_psd[k,j]=total(wout_psd0[k,i:i+5])
      j=j+1
    endfor
  endfor
  
  ;************* one cell's dimensions ****************
  ;areatot = total area of one cell
  ;rap = ratio   vertical lenght/horizontal lenght of one cell
  ;UNICELL = lenght (n cm) of one cell
  
  Rcell=sqrt(0.5)
  rap=1.
  areatot= rap
  UNICELL= 2.
  
  ;*** MASK: radii's array with the same dimension of the PSDs'one ****
  
  tx=findgen(ntub)+1
  tx=tx*rap
  tx=[[tx],[tx]]
  xx=congrid(tx,ntub,s[2])
  
  
  ty=transpose(findgen(s[2])+1)
  yy=congrid(ty,ntub,s[2])
  
  x0y0=([s[1],s[2]]+1.)/2.   ;PSDs' centre
  x0=x0y0[0]*rap& y0=x0y0[1]
  
  r=sqrt((xx-x0)^2+(yy-y0)^2)
  rmax=MAX(r,MIN=rmin)
  
  
  ;************* radii of crowns of integration **********************
  ; the 2 radii of each integration's crown are: (0,r1),(r1,r2),(r2,r3)
  
  r1=3.618    ; r1*rap gives the value in cm
  r2=6.163
  r3=8.616
  step=findgen(3)
  
  step[0]=r1
  step[1]=r2-r1
  step[2]=r3-r2
  
  ;**** q values for the crowns of integration ******
  
  q1=(4*!pi/wavel)*sin(atan(r1*unicell/52.)/2.)
  q2=(4*!pi/wavel)*sin(atan(r2*unicell/52.)/2.)
  q3=(4*!pi/wavel)*sin(atan(r3*unicell/52.)/2.)
  qmax=(4*!pi/wavel)*sin(atan(rmax*unicell/52.)/2.)
  
  
  ;******* mean q of the crowns of integration *************
  
  qm1=(q1)/2.
  qm2=(q1+q2)/2.
  qm3=(q2+q3)/2.
  
  ;**** getting started with the radial integraton..... *****
  
  i=0.
  FOR h=0,2 do begin
  
    temp=where((r ge (i-Rcell)) and (r le (i+step[h]+Rcell)))   ; r values:  r(i) < r < r(i)+step+Rcell
    st=size(temp)
    
    IF  (st[0] gt 0) THEN BEGIN
    
    
      xxtemp=xx[temp]
      yytemp=yy[temp]
      c=i+step[h]
      d = (c-r[temp])   ;d = difference between r(i)+step and r(temp)
      dd= (r[temp]-i)
      weight=fltarr(st[1])  ; weight of each cell
      r_eff=r[temp]
      
      FOR j=0,st[1]-1 do begin
        if (c eq r_eff[j]) then begin
          weight[j]=0.5
        endif else begin
        
          if (yy[j] eq y0) then begin
          
            if (d[j] ge 0.5) then begin
              weight[j]=1.
            endif else begin
            
              if (d[j] ge 0. and d[j] lt 0.5) then begin
                area=0.5+d[j]
                weight[j]=area
                
              endif else begin
              
                if (d[j] lt 0. and abs(d[j]) ge 0.5) then begin
                  weight[j]=0.
                endif else begin
                
                  if (d[j] lt 0. and abs(d[j]) lt 0.5) then begin
                    weight[j]=0.5-abs(d[j])
                  endif
                endelse
              endelse
            endelse
            
          endif else begin
          
            if(yy[j] ne y0 and d[j] ge 0.) then begin
              if (d[j] ge sqrt(0.5)) then begin
                weight[j]=1.
              endif else begin
              
                at=d[j]^2+Rcell^2-2.*Rcell*abs(d[j])
                area=(1.-at)
                weight[j]=area
              endelse
            endif else begin
            
              if(yy[j] ne y0 and d[j] lt 0.) then begin
                if(abs(d[j]) ge sqrt(0.5)) then begin
                  weight[j]=0.
                endif else begin
                
                  at=d[j]^2+Rcell^2-2.*Rcell*abs(d[j])
                  area=at
                  weight[j]=area
                  
                endelse
              endif
            endelse
            
          endelse
          
          
          if (weight[j] eq 1.) then begin
          
            if( dd[j] ge 0.) then begin
            
              if (dd[j] ge sqrt(0.5)) then begin
                weight[j]=1.
              endif else begin
                at=dd[j]^2+Rcell^2-2.*Rcell*abs(dd[j])
                area=at
                weight[j]=1.-area
              endelse
              
              
            endif else begin
              if( dd[j] lt 0.) then begin
                if (abs(dd[j]) lt sqrt(0.5)) then begin
                
                  at=dd[j]^2+Rcell^2-2.*Rcell*abs(dd[j])
                  area=at
                  weight[j]=area
                endif else begin
                  weight[j]=0.
                  
                endelse
              endif
            endelse
            
          endif
          
        endelse
        
      ENDFOR
      
      
      totm=total(weight*wout_psd[temp]) ; final result of each radial integration
      
      
      if (i eq 0.) then begin
      
        i1=totm
        
      endif  else begin
        if (i eq r1) then begin
        
          i2=totm
          
        endif else begin
          if (i eq r2) then begin
          
            i3=totm
            
          endif
        endelse
      endelse
      
      
      i=i+step[h]
      
    ENDIF ELSE BEGIN
      IF (I EQ R1) THEN  BEGIN
        i2=0. & i3=0.
      ENDIF
      
      IF (I EQ R2) THEN BEGIN
        i3=0.
      ENDIF
    ENDELSE
    
  ENDFOR
  e1=sqrt(i1)  ; error of each resulting intensities
  e2=sqrt(i2)
  e3=sqrt(i3)
  return
end

;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
