;+
; **
; ** Program in4_multi
; **
; ** This module permits to have a 2D picture of the small angle multidetector on IN4
; ** S. Rols 11/09 rols@ill.eu
; ** USE: in4_multi,'W2',/log
; ** w2 is the workspace of data
; ** /log is an optional keyword to have the log of the intensity printed
; ** Last modification: 09/11/09
; **
;-

pro in4_multi,W_str,LOG=LOG,MAX=MAX,MIN=MIN
; ** ********
; ** Main body of this complete routine ... the first to be called
;
@lamp.cbk

if (!D.flags and 65536) eq 0  then print,'set_plot,"X" before using rdfilter' else  $
if (xregistered('in4_multi') le 0) then begin
  P_MUS,'mus_harp'          ; PLAY a tune

if n_elements(Group) eq 0 then GROUP=lamp_b1

junk       = { CW_PDMENU_S, flags:0, name:'' }

; the whole lamp size depending modifications is here
sl_size =16
if lamp_siz gt 900 then begin
    xsiz=800 & ysiz=220
endif else begin
    xsiz=700 & ysiz=280
endelse
if lamp_siz lt 800 then begin   ; adapt to lamp size
    xsiz=550 & ysiz=250     ; adapt plotting window size
    sl_size=15
endif

; ** Start definition of the widget object
; ** *************************************

; ** First Base
;sw_ba_leader=widget_base(group_leader=Group, /column, $
;          Title='IN4_Multidetector',$
;          resource_name='lamp' $;)
;          ,MBAR=m_bar)
sw_ba_leader=widget_base(group_leader=Group, /column, Title='IN4_Multidetector',resource_name='lamp')

; ** Menu Bar
;sw_bu_filem=widget_button(m_bar,/menu,value='File')
;sw_bu_help=widget_button(m_bar,/menu,value='Help',sensitive=1)
; ** START THE HELP!
;sw_la_help=widget_button(sw_bu_help,value='See complete doc at http://www.ill.fr/Computing/TOFHR-help/QENS_fit.pdf',uvalue=[999])

; ** First up : bases
;sw_ba_up=widget_base(sw_ba_leader,Title='Base row up',frame=1,/row)
; ** First up : button, labels and texts

; ** *********************************************************************************************
; ** Second mid : graph
sw_pl_area=widget_draw(sw_ba_leader,/Button_Events,Frame=5,Retain=2,Colors=-8, $
                 XSize=3*1.1*xsiz/4,YSize=18*1.1*ysiz/8 ,uvalue=[7])
;if sys_dep('VERSION') ge 4.0 then ii=execute('str=widget_info(peak_menu2,/geometry) & szsl=str.xsize')
bid=sys_dep('DYNLAB', sw_ba_leader, 1)

;widget_control, bad_id=i, sw_ba_leader, /Realize & put_logo

; ** Realization of the Base
widget_control, sw_ba_leader, /Realize; & put_logo
xmanager,'in4_multi',sw_ba_leader,/just_reg
; ** Get the window Id for the plotting area and store it in s_pl_wid
widget_control, bad_id=i, sw_pl_area, get_value=s_pl_wid    ; Store the number assigned to this window in s_pl_wid
wset,s_pl_wid                          ; Set the s_pl_wid active

; ** Get the window Id for the plotting area #2 and store it in s_pl_widbot
;widget_control, bad_id=i, sw_pl_areabot, get_value=s_pl_widbot  ; Store the number assigned to this window in s_pl_wid
;wset,s_pl_widbot                          ; Set the s_pl_wid active


endif

;This example uses POLAR_CONTOUR with regularly-gridded data:
;Handle TrueColor displays:
DEVICE, DECOMPOSED=0

;Load color table
;TEK_COLOR

nr = 12 ; number of radii
nt = 360 ; number of Thetas
m=2
; Create a vector of radii:
r = FINDGEN(nr+m)

; Create a vector of Thetas:
theta = 2*!PI * FINDGEN(nt)/(nt-1)

; Create some data values to be contoured:
z = 0.*theta # r +1.e10 ; z=Array[nt,nr]

XICUTE,'w41=in4strip('+w_str+',/small)'
XICUTE,'w41=total(w41,1)' & XICUTE,'w41=reform(w41,8,12)'
w_in2=w41

POLAR_CONTOUR, z, theta, r, /FILL;, c_color=[2, 3, 4, 5]

; Sector S3 is [-22.5, 22.5]
l=where((theta le !pi/8.)    or  (theta gt 2.*!pi-!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[2,i] ; S3
l=where((theta le 3*!pi/8.)  and (theta gt !pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[1,i]     ;S2
l=where((theta le 5*!pi/8.)  and (theta gt 3*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[0,i]     ;S1
l=where((theta le 7*!pi/8.)  and (theta gt 5*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[7,i]     ;S8
l=where((theta le 9*!pi/8.)  and (theta gt 7*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[6,i]     ;S7
l=where((theta le 11*!pi/8.) and (theta gt 9*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[5,i]     ;S6
l=where((theta le 13*!pi/8.) and (theta gt 11*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[4,i]     ;S5
l=where((theta le 15*!pi/8.) and (theta gt 13*!pi/8.)) & for i=0,nr-1 do z[l,i+m]=w_in2[3,i]     ;S4

;z= theta # (r-0.5)^2
; Create the polar contour plot:
if n_elements(min) gt 0 then imin=min else imin=0
if n_elements(max) gt 0 then imax=max else imax=max(w_in2)
w_in2=w_in2<imax
if not keyword_set(log) then POLAR_CONTOUR, z, theta, r, /cell_fill, NLEVELS=1000,min_value=imin,max_value=imax,/isotropic,TITLE='Integrated intensity',Xstyle=4,Ystyle=4
if keyword_set(log) then POLAR_CONTOUR, alog(z), theta, r, /cell_fill,NLEVELS=1000,min_value=alog(min(w_in2)>1e-10),max_value=alog(imax),/isotropic,TITLE='Log[Integrated intensity]',Xstyle=4,Ystyle=4
print,w_in2
end
