FUNCTION in4strip, w_in, bottom=bottom, top=top, middle=middle,$
                   small=small, wide=wide, sector=sector, remove=remove,$
                   bad=bad,add_bad=add_bad,list_bad=list_bad,no_abs=no_abs,channels=channels,$
                   hvolt=hvolt
;+
;******* ********       
;** User callable
;** extracts sectors of the IN4 detector bank
;** KEYWORDS:
;** ---------
;**     /bottom:     extracts bottom row of detectors
;**     /middle:     extracts middle row of detectors
;**     /top:        extracts top row of detectors
;**     /small:      extracts small angle bank
;**     /wide:       extracts wide angle bank
;**     /bad:        remove the bad detectors (action performed first)
;**     add_bad=list allows one to add its own detectors in addition to the default list
;**     list_bad=list allows one to define its own detectors list
;** 
;**    /remove       removes the detectors from the workspace
;**    /no_abs       prevent to return the absolute values of the angle (default). Useful for single crystal
;**    /channels     allows to return the results in channels number rather than spectrum angle (default)
;** 
;** Variables:
;** ----------
;**     sector = 1,2,..        if /small selected, chooses sector of small angle bank.
;**                             JRS 13/11/06 (last modification : 28/08/12 by SR)
;**  Last modification 14/12/10 by SR, to incoprorate the new list of Bad detectors, and the way to handle them, and add_bad list
;**  Last modification 19/04/11 by SR, to incorporate the modifications induced by the change in the data format
;**  Last modification 04/07/2011 by AO : incorporate bad detectors for the small angle bank
;**  Last modification 28/08/2012 by SR : incorporate /hvolt keyword (organization by high voltage value)
;** 
;-
        
    iprint = 0        ; iprint > 0, show debug messages
    
    TAKE_DATP, datp
    
    nchannels = (SIZE(w_in))[1]
    nspectra = (SIZE(w_in))[2]
    IF (SIZE(w_in))[0] EQ 2 THEN nruns = 1 ELSE nruns = (SIZE(w_in))[3]
    IF iprint THEN BEGIN
        PRINT, 'nchannels: ', nchannels
        PRINT, ' nspectra: ', nspectra
        PRINT, '    nruns: ', nruns
    ENDIF
    
    y_in = datp.y
    e_in = datp.e
    z_in = datp.z
    par  = datp.p
    if n_elements(par) le 32 then nvers = 1 else nvers = par[32] > 1
    if nvers ge 10. then nvers = 2 ; Keep compatibilities for old hdf files
    st = ''
    
    CASE nvers of
        1: BEGIN
            stb = ''
            offsmall=0
            y_buf=y_in
            e_buf = e_in
            w_buf = w_in
            IF nruns GT 1 THEN parv = datp.pv
            
            IF MIN(y_in) LT 10. THEN offset = 96 ELSE offset = 0  ;is sans bank in?
            
            bot_dets = [2,3,7,8,12,13,17,18,24,27,30,33,37,39,42,45,48,51,54,57,62,$
                65,68,71,73,76,79,82,87,90,93,96,99,102,105,107,109,112,115,$
                118,123,126,129,132,135,138,141,143,146,149,152,155,160,163,$
                166,169,172,175,177,179,181,184,187,190,195,198,201,204,207,$
                210,213,215,217,220,223,226,231,234,237,240,243,246,249,251,$
                254,257,260,263,267,270,273,276,279,282,285,287,290,293,296,$
                299] + offset
            mid_dets = [4,9,14,19,20,21,22,25,28,31,34,35,40,43,46,49,52,55,58,59,$
                60,63,66,69,74,77,80,83,84,85,88,91,94,97,100,103,110,113,$
                116,119,120,121,124,127,130,133,136,139,144,147,150,153,156,$
                157,158,161,164,167,170,173,182,185,188,191,192,193,196,199,$
                202,205,208,211,218,221,224,227,228,229,232,235,238,241,244,$
                247,253,256,259,262,264,265,268,271,274,277,280,283,288,291,$
                294,297] + offset
            top_dets = [0,1,5,6,10,11,15,16,23,26,29,32,36,38,41,44,47,50,53,56,61,$
                64,67,70,72,75,78,81,86,89,92,95,98,101,104,106,108,111,114,$
                117,122,125,128,131,134,137,140,142,145,148,151,154,159,162,$
                165,168,171,174,176,178,180,183,186,189,194,197,200,203,206,$
                209,212,214,216,219,222,225,230,233,236,239,242,245,248,250,$
                252,255,258,261,266,269,272,275,278,281,284,286,289,292,295,$
                298] + offset
            small_dets= indgen(96)
            dets = [-1]
            bad_dets = [65,73,81,89,92,97,101,102,107,112,134,148,158,161,164,167,245,251,385,395,396]-96-1+offset ;BadDets list 14/12/2010
        END
        2: BEGIN
            stb = ''
            ;  j = SORT(z_in) & nsp = n_elements(y_in)
            ;  i=where(j lt nsp) & j=j(i)
            offset=300 & offsmall=300
            y_buf=y_in;(j)
            e_buf = e_in;(*,j,*)
            w_buf = w_in;(*,j,*)
            small_dets= indgen(96)+300
            bot_dets = indgen(100)+200
            mid_dets = indgen(100)+100
            top_dets = indgen(100)
            dets = [-1]
            ;  bad_dets = [100,107,133,134,176,305,395,396]-1 & bad_dets=z_in(bad_dets)-1
            bad_dets = [13,100,101,213,300]-1 ;& bad_dets=z_in(bad_dets)-1 ; From B. Fak's list 11/2014
        END
        else:
    ENDCASE
    
    ; -------------- SEE DIFFERENT SECTORS FROM ROSACE SMALL ANGLE --------------
    
    IF N_ELEMENTS(sector) NE 0 THEN BEGIN
        FOR isector = 0, N_ELEMENTS(sector) - 1 DO BEGIN
            IF nvers eq 1 THEN dets = [dets,offsmall+INDGEN(12)*8 + sector[isector] - 1]$
            ELSE dets = [dets,offsmall+12*(sector[isector]-1) + INDGEN(12)]
            st = st + ',sect='+STRTRIM(STRING(sector[isector]),2)
        ENDFOR
        GOTO, fin
    ENDIF
    
    ; ----------------------------- BAD DETECTORS -----------------------------
    IF KEYWORD_SET(bad) THEN BEGIN
      stb = stb + '/bad'

; Determine list of detectors to remove:
    IF n_elements(list_bad) ne 0 then begin
       bad_dets=list_bad-1
    ENDIF
; Add more det?
    IF n_elements(add_bad) ne 0 then begin
        bad_to_add=add_bad-1
        bad_dets=[bad_dets,bad_to_add]
    ENDIF
; reorganize the list    
    bad_dets=bad_dets[UNIQ(bad_dets,sort(bad_dets))]
    
        for i=0,n_elements(bad_dets)-1 do begin
            ind_top=where(top_dets eq bad_dets[i])
            ind_bot=where(bot_dets eq bad_dets[i])
            ind_mid=where(mid_dets eq bad_dets[i])
            ind_small=where(small_dets eq bad_dets[i])
            if ind_top ne -1 then begin
                print,'Removing bad det from Top Det bank::',top_dets[ind_top[0]]+1
                if ind_top[0] EQ 0 THEN top_dets=top_dets[ind_top[0]+1:n_elements(top_dets)-1] ELSE $
                    IF ind_top[0] EQ n_elements(top_dets)-1 THEN top_dets=top_dets[0:n_elements(top_dets)-2] ELSE $
                    top_dets=[top_dets[0:ind_top[0]-1],top_dets[ind_top[0]+1:n_elements(top_dets)-1]]
            endif
            if ind_mid ne -1 then begin
                print,'Removing bad det from Middle Det bank:',mid_dets[ind_mid[0]]+1
                if ind_mid[0] EQ 0 THEN mid_dets=mid_dets[ind_mid[0]+1:n_elements(mid_dets)-1] ELSE $
                    IF ind_mid[0] EQ n_elements(mid_dets)-1 THEN mid_dets=mid_dets[0:n_elements(mid_dets)-2] ELSE $
                    mid_dets=[mid_dets[0:ind_mid[0]-1],mid_dets[ind_mid[0]+1:n_elements(mid_dets)-1]]
            endif
            if ind_bot ne -1 then begin
                print,'Removing bad det from Bottom Det bank::',bot_dets[ind_bot[0]]+1
                if ind_bot[0] EQ 0 THEN bot_dets=bot_dets[ind_bot[0]+1:n_elements(bot_dets)-1] ELSE $
                    IF ind_bot[0] EQ n_elements(bot_dets)-1 THEN bot_dets=bot_dets[0:n_elements(bot_dets)-2] ELSE $
                    bot_dets=[bot_dets[0:ind_bot[0]-1],bot_dets[ind_bot[0]+1:n_elements(bot_dets)-1]]
            endif
            if ind_small ne -1 then begin
                print,'Removing bad det from Small Angle Det bank::',small_dets[ind_small[0]]+1
                if ind_small[0] EQ 0 THEN small_dets=small_dets[ind_small[0]+1:n_elements(small_dets)-1] ELSE $
                    IF ind_small[0] EQ n_elements(small_dets)-1 THEN small_dets=small_dets[0:n_elements(small_dets)-2] ELSE $
                    small_dets=[small_dets[0:ind_small[0]-1],small_dets[ind_small[0]+1:n_elements(small_dets)-1]]
            endif
        endfor
        
    ENDIF
    ; -------------------------------------------------------------------------------------
    
    IF KEYWORD_SET(bottom) THEN BEGIN
        st = st + '/bottom'
        dets = [dets,bot_dets]
    ENDIF ELSE BEGIN
        IF KEYWORD_SET(middle) THEN BEGIN
            st = st + '/middle'
            dets = [dets,mid_dets]
        ENDIF ELSE BEGIN
            IF KEYWORD_SET(top)    THEN BEGIN
                st = st + '/top'
                dets = [dets,top_dets]
            ENDIF ELSE BEGIN
                IF KEYWORD_SET(wide)   THEN BEGIN
                    IF KEYWORD_SET(top) OR KEYWORD_SET(middle) OR KEYWORD_SET(bottom) THEN BEGIN
                        s = 'in4strip: if /wide is selected, then /top, /middle and /bottom are ignored'
                        ii = dialog_message(s,/ERROR)
                        dets = [0]
                        st = ''
                    ENDIF
                    st = '/wide'
                    dets = [dets ,top_dets, mid_dets, bot_dets]
                ENDIF ELSE BEGIN
                    IF KEYWORD_SET(small) THEN BEGIN
                        IF offset EQ 0 THEN BEGIN
                            s = 'in4strip: small angles already extracted'
                            i = dialog_message(s,/ERROR)
                            RETURN, w_in
                        ENDIF
                        st = st + '/small'
                        dets=[dets,small_dets]
                    ENDIF ELSE BEGIN
                        st = '/all'
                        dets = [dets ,small_dets,top_dets, mid_dets, bot_dets]
                    ENDELSE
                ENDELSE
            ENDELSE
        ENDELSE
    ENDELSE
    
    ;    i = SORT(dets)
    ;    dets = dets(i)
    fin:    dets = dets[1:*]            ;remove -1 point
    
    IF KEYWORD_SET(remove) THEN BEGIN
        IF (nspectra - offset) LT 300 THEN BEGIN
            s = 'in4strip: detectors already removed'
            ii = dialog_message(s, /ERROR)
        ENDIF
        indets   = FLTARR(300 + offset)
        indets[dets] = 1
        dets = WHERE(indets EQ 0)
    ENDIF
    
    
    IF iprint THEN BEGIN
        PRINT, 'in4strip: The following detectors were extracted'
        PRINT, dets + 1
    ENDIF
    
    ; -----------------------------------------------------------------------------
    ;return workspace, params and exit
    
    w_out = w_buf[*,dets,*] & e_out = e_buf[*,dets,*] & z_out=z_in[dets]
    if n_elements(channels) eq 0 then begin
        y_out = y_buf[dets]
        if N_ELEMENTS(no_abs) EQ 0 THEN y_out=abs(y_out)
        j=SORT(y_out) & y_out=y_out[j]
        w_out=w_out[*,j,*] & e_out=e_out[*,j,*] & z_out=z_out[j] ;dets(j)
    endif else begin
        y_out = dets + 1
    endelse
    
    if n_elements(hvolt) ne 0 then begin
        st = st + ', hvolt='+strcompress(string(hvolt),/remove_all)
        Print,'Selection via high voltage: HT'+strcompress(string(hvolt),/remove_all)+' selected'
        j=where(z_out eq hvolt)
        w_out=w_out[*,j,*] & e_out=e_out[*,j,*] & z_out=z_out[j] & y_out=y_out[j]
    endif
    
    datp.p[15] = N_ELEMENTS(dets)
    IF nruns GT 1 THEN datp.pv[15,*] = N_ELEMENTS(dets)
    
    datp.other_tit = datp.other_tit + ' -in4('+st+')'
    
    IF KEYWORD_SET(remove) THEN aa = 'Removing ' ELSE aa = 'Using '
    IF KEYWORD_SET(top) OR KEYWORD_SET(middle) OR  KEYWORD_SET(bottom) $
        THEN bb = ' row of' ELSE bb = ' angle'
        
        
    PRINT, 'in4strip: '+aa+'the '+STRMID(st,1)+bb+' detectors'
    If stb ne '' then PRINT,'in4strip: the following (bad) detectors were removed:',bad_dets+1
    
    MOD_DATP, datp, "y", y_out
    MOD_DATP, datp, "e", e_out
    MOD_DATP,datp,"z", z_out
    GIVE_DATP, datp
    
    RETURN, w_out
END
