FUNCTION in5_mask, w_in, w_van0, $
		 chmin=chmin, chmax=chmax, $
         Thresh=thresh, $
		 verbose=verbose
;-------------------------------------------------------------------------------
;*******************************************************************************
;
;   For IN5 PSD data
;
;+
;
;<h2>NAME:</h2>
;
;   mask
;   
;<h2>PURPOSES:</h2>
; 
;   Create a mask for hidding pixels when creating the
;   .spe files or any output file
;
;
;<h2>DIMENSIONS:</h2>
; w_in=w_out(nspectra, nY, nchannels)
;
;<h2>COMMAND SYNTAX:</h2>
;
; w3 = in5_mask(w1,chmin=&lt;min&gt;,chmax=&lt;max&gt;[Thresh=thresh,/verbose])
;      - normalises data in w1 to vanadium data in w2 and store the result in w3
;        (optional arguments shown in square brackets).
;
;   chmin = min channel for ELP
;   chmax = max channel for ELP
;   Thresh = threshold for discarding below (abs. value)
;
;<h2>EXAMPLES:</h2>
;
; w3 = vnorm(w1,/verbose) 
; w3 = vnorm(w1,Thresh=0.01,/verbose) 
;
;-
;
; Written by: JOR, Fri Apr 20 17:55:41 CEST 2012
;
;
;*******************************************************************************
;-------------------------------------------------------------------------------
	COMMON c_lamp_access, inst

; for functions that has to appear later but are unknown from here when saving
    forward_function str_fit

	take_datp, datp

;-------------------------------------------------------------------------------
;  Check workspace sizes
;
;  JOR, Mon Nov  7 17:30:04 CET 2011
;  If data is 3D => single crystal data presumably. In that case, date format is
;  [Phi,Z,ToF] (if data in ToF) and [hw,Phi,Z] if data in energy.
;  1st treat the simple case where data in ToF.
;
;-------------------------------------------------------------------------------
   isPSD = -1
   size_win = SIZE(w_in)
;  IF keyword_set(verbose) THEN PRINT,'SIZE(w_in)=',size_win
   nchannels = size_win[1]
    
    IF size_win[0] EQ 3  THEN BEGIN              ; 3D [Phi,z,ToF] Debye-Scherrer usual dataset 
        isPSD = 1
       nchannels = size_win[3]
        nPhi     = size_win[1]
        nZ       = size_win[2]
        nspectra = nPhi*nZ
    ENDIF ELSE BEGIN
        print, 'MASK:ERROR: Data are not in 3D PSD format!'
        return, -1
    ENDELSE
        
    ; The average spectra center and width in safe mode
    x = datp.z
    y = transpose( total(w_in,1) )
    gauss   = str_fit(y, Bkgd = 0, Xin = x)
     
    ; If keywords reduce the ELP range to them               
    if ~keyword_set(chmin) then chmin = fix(mean(gauss[*,2])-3*mean(gauss[*,3]))
    if ~keyword_set(chmax) then chmax = fix(mean(gauss[*,2])+3*mean(gauss[*,3]))

    if keyword_set(verbose) then $
        PRINT,'MASK: PSD: Define elastic integral, channels ', $
        STRTRIM(STRING(chmin),2),' to ',STRTRIM(STRING(chmax),2)

    w_mask  = TOTAL(w_in[*,*,chmin-1:chmax-1],3)
        
    if ~keyword_set(Thresh) then Thresh = max(w_mask)/3.0 $
    else Thresh = max(w_mask)*Thresh
    if keyword_set(verbose) then $
        PRINT,'MASK: PSD: Define threshold to:  ', STRTRIM(STRING(Thresh),2), $
        ' (',STRTRIM(STRING(100.*Thresh/max(w_mask)),2),'% of the max).'

    w_mask[where(w_mask LT Thresh, /NULL, nNull)] = 0
    w_mask[where(w_mask GT Thresh, /NULL)] = 1
    
    if keyword_set(verbose) then $
        PRINT,'MASK: PSD: ',STRTRIM(STRING(nNull),2),' pixels discarded (',STRTRIM(STRING(100.0*nNull/nspectra),2),'% of total)'

    return, w_mask
end
