; ----------------------------------------------------------------------------------------------
;
;+
; NAME:
;    regularise
;
; PURPOSES:
;    set the angles as a regular grid and set data accordingly
;
; USAGE:
;
;  w2 = in5_regularise(w1, /verbose)
; 
;  Is generaly used during the preprocessing of the IN5 data, eg. inside @in5_reduce
;  
;
;
;  HISTORY:
;
;   Written:    JO, Wed Nov 11 18:32:46 CET 2008
;   Changes:    JO, Wed Dec 10 18:32:46 CET 2008: Correct error on eout after rdsum
;               JO, Tue Feb  9 19:18:18 CET 2010: change algo to D. Richard one. Fast and exact.
;
;-
; ----------------------------------------------------------------------------------------------
function in5_regularise, Win, verbose=verbose

    st1=systime(1)
    
; ----------------------------------------------------------------------------------------------
;  hard-coded Constant ....
; ----------------------------------------------------------------------------------------------
    nPanels = 12    ; Nb panels on IN5
    nTubes  = 32    ; Nb tubes / panel
    
    take_datp,datp
    Xin   = datp.X 
    Yin   = datp.y  
    Zin   = datp.z  
    Ein   = datp.e
      
	sizeWin = size(Win)    
	sizeX   = sizeWin[1]
    errorOK = (n_elements(datp.e) eq  n_elements(Win))
	 
    if sizeWin[0] gt 1 then sizeY = sizeWin[2]  else   sizeY =1 
    ; ---------------------------------------------------------------------------------
    ;  hard-coded Constant ....
    ;  Compute dPhi for estimating the total number on X axis ( =  size Wout along X)
    ; ---------------------------------------------------------------------------------
     maxPhi = max(Xin,min=minPhi) 
     dPhi   = double( 2.0*asin(26.0/2.0/4000.0) * 180.0/!PI ) 

    ; --------------------------------------------------------------
    ; define outputs
    ; --------------------------------------------------------------
    if sizeWin[0] gt 2 then sizeZ = sizeWin[3] else sizeZ =1 
    sizeXout  = round( ( maxPhi - minPhi ) / dPhi ) + 1   

    if keyword_set(verbose) then $
        print, 'REGULARISE: regularise ',strtrim(sizeX,2),' angles in ',strtrim(sizeXout,2),' regular grid angles.'
    
    Xout = dindgen(sizeXout)*dPhi + minPhi
    Xout = reverse( Xout )
    Wout = reform( fltarr(sizeXout,sizeY,sizeZ) )
    if errorOK then Eout = Wout else Eout = datp.e
    errorSquared   = Ein^2

; --------------------------------------------------------------
;  Fast fast fast ... But it duzn't wuk!
;
; JOR, Tue Feb  9 18:25:38 CET 2010
;   This one below is working well in 4s instead of 15s...
;   adapted from read_hdf D. richard
; --------------------------------------------------------------
    for k=1,nPanels do begin
        a  = (k-1)*nTubes
        b  = k*nTubes-1  
        ar = a+k-1 
        br = b+k-1
        Wout[ar:br,*,*] =  Win[a:b,*,*] 
        if errorOK then $
             Eout[ar:br,*,*] = Ein[a:b,*,*]
         Xout[ar:br]        =  Xin[a:b]
        if k lt nPanels then begin
           Wout[br+1,*,*] = (Win[b,*,*] + Win[b+1,*,*])/2.
           if errorOK then $
               Eout[br+1,*,*] = sqrt(errorSquared[b,*,*] + errorSquared[b+1,*,*])
           Xout[br+1 ]        = (Xin[b]+Xin[b+1])/2.  
        endif
    endfor

    if keyword_set(verbose) then $
        print, 'REGULARISE: Done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'
    ; --------------------------------------------------------------
    ;  Output values
    ; --------------------------------------------------------------
    mod_datp,datp,'X',Xout 
    mod_datp,datp,'Y',Yin
    mod_datp,datp,'Z',Zin 
    mod_datp,datp,'E',Eout
    give_datp, datp
    
    return, Wout	
	
end
