	FUNCTION in5_remove_spectra, w_in, badspecs, zero=zero, nolog=nolog, $
                 verbose=verbose, BadSpectra = BadSpectra
;------------------------------------------------------------------------------
;
;+
;
;<h2>NAME:</h2>
;
;     remove_spectra
;
;
; For IN4, IN5, IN6, DCS and HET, ...
; Removes spectra with numbers given in array badspecs. Spectrum numbers
; run from 1 to nspectra.
;
;
;<H2>KEYWORDS:</H2>
;
; /nolog	: if set, supresses output of badspecs in other_tit
;		  This is necessary in cases where other_tit exceeds 160 characters
;
;
;<H2>ARGUMENTS:</H2>
;
; badspecs	: array of spectrum numbers to be deleted
;
;
;<H2>DIMENSIONS:</H2>
;
; w_in(nchannels,nspectra) -> w_out(nchannels,nspectra-n) - n=number of bad specs
;
;
;<H2>COMMAND SYNTAX:</H2>
;
;   w2 = remove_spectra(w1,[3,90,202]) (remember square brackets)
;   w2 = remove_spectra(w1,BadSpectra = '1-87,112-121,234,235,236,256-273')	
;
;
;
;<h2>HISTORY:</h2>
;
;  KHA,JRS 7/05/05
;  JO, Mon Jun 16 14:24:35 CEST 2008  Modif to take a group of detectors instead 
;                                     of individual numbers.
;  e.g. w2 = remove_spectra(w1,BadSpectra = '1-87,112-121,234,235,236,256-273')	
;       in that case, the BadSpectra string replace the badspecs input.
;
;
;-
;
; -------------------------------------------------------------------------------

	take_datp, datp

; -------------------------------------------------------------------------------
;   Set up input workspace
; -------------------------------------------------------------------------------

	IF (SIZE(w_in))[0] EQ 2 THEN nspectra = (SIZE(w_in))[2] ELSE $
	     RETURN, w_in
	IF ( N_ELEMENTS(badspecs) EQ 0 ) AND ( NOT keyword_set(BadSpectra) ) THEN $
             RETURN, w_in


; -------------------------------------------------------------------------------
;   Remove bad spectra
;   In case of a string defining intervals, generate the numbers and 
;    superseed 'badspecs' entry
; -------------------------------------------------------------------------------
        if ( N_ELEMENTS(badspecs) GT 0 ) then begin
 	    no_specs = WHERE( badspecs GT nspectra, n)
	    IF n GT 0 THEN BEGIN
		s = 'Spectrum no.: '+ STRTRIM(STRING(badspecs[no_specs]),2) + $
		    ' does not exist '
		i = DIALOG_MESSAGE(s)
		yes_specs = WHERE( badspecs LE nspectra, n)
		IF n EQ 0 THEN RETURN, w_in
		badspecs = badspecs[yes_specs]
	    ENDIF
        endif else begin
            Bad = -1
            A = strsplit(BadSpectra,',', /extract)
            if n_elements(A) GE 1 then begin
                for i = 0,n_elements(A) - 1 do begin
                     C = strsplit(A[i],'-', /extract)
                     if n_elements(C) GT 1 then $
                         dummy = ulong( C[0] ) + indgen( ulong( C[1] ) - ulong( C[0] ) + 1 )  $
                     else $
                         dummy = ulong( C[0] )
                     Bad =  [ Bad,  dummy ] 
                endfor
            endif
            badspecs = Bad[ 1:n_elements(Bad)-1 ]
        endelse

; -------------------------------------------------------------------------------
; Now do the same job than previously
; -------------------------------------------------------------------------------
	w_buf = w_in
	w_buf[*, badspecs - 1] = 0.0
	specs = FLTARR(nspectra) + 1.
	specs[badspecs - 1] = 0.
	IF KEYWORD_SET(zero) THEN ispec = WHERE( TOTAL(w_buf,1) NE 0.0) ELSE ispec = WHERE(specs NE 0.0)
	w_out = w_in[*,ispec]
	e_out = datp.e[*,ispec]
	y_out = datp.y[ispec]
	z_out = datp.z[ispec]
	


if keyword_set(verbose) then begin
    print, 'REMOVE_SPECTRA: Remove spectra numbers: ',strtrim( badspecs,2 )
    print, 'REMOVE_SPECTRA: ',strtrim( n_elements(badspecs),2 ), ' removed out of ',strtrim( nspectra,2 ),' (',strtrim( 1000 * n_elements(badspecs)/nspectra/10,2 ),'%)'
endif
; -------------------------------------------------------------------------------
;   Return parameters and exit
; -------------------------------------------------------------------------------

	mod_datp, datp, "y", y_out
	mod_datp, datp, "e", e_out
	mod_datp, datp, "z", z_out

	bad = STRTRIM(badspecs,2) & badstring = bad[0]
	FOR i=1, N_ELEMENTS(badspecs)-1 DO badstring=badstring+','+bad[i]
	badstring='('+badstring+')'

;;;	PRINT,'REMOVE_SPECTRA: Removed spectra number: ' + badstring
	IF NOT KEYWORD_SET(nolog) THEN $
		datp.other_tit=datp.other_tit+' -rs'+badstring

	give_datp, datp

	RETURN, w_out

	END
