	FUNCTION in5_sumbank, w_in, dQ=dQ, qb, noaverage = noaverage, verbose=verbose
;-------------------------------------------------------------------------------
;*******************************************************************************
;
;
;+
;
;<h2>NAME:</h2>
;
;     sumbank
;
; For IN4, IN5 and IN6 data.
;
;<h2>ARGUMENTS:</h2>\;
; dQ:	Q-spacing with which to sum detectors. If dQ is not specified, nothing
;	is done for IN5 data. For IN6 or IN4, adds data in centre, upper and lower
;	detector banks by default.
; /noaverage keyword when sum needed instead of average on the summed detectors.
;
;
;
;
;<h2>DIMENSIONS: </h2>
;
; w_in=w_out(nchannels,nspectra)
;
;
;<h2>COMMAND SYNTAX:</h2>
;
; w2=sumbank(w1,dQ=<dQ>, /noaverage)
;
;
;
;<h2>HISTORY:</h2>
;
;
; Authors: KHA,JRS 16/03/05
; Changes: JO, Wed Jan 31 17:39:38 CET 2007 add verbosity plus ???
;
;-
;
;-------------------------------------------------------------------------------
;*******************************************************************************

	COMMON c_lamp_access, inst
	forward_function string_round

        ; if iprint>0, show debugging messages
	if keyword_set(verbose) then iprint = 1 else  iprint = 0

	IF iprint THEN PRINT,'Start sumbank:'

	take_datp, datp
	IF(N_ELEMENTS(qb) GT 0) THEN dQ=qb

;-------------------------------------------------------------------------------
;Set up input workspace

	IF (SIZE(w_in))[0] EQ 2 THEN nspectra=(SIZE(w_in))[2] ELSE RETURN, w_in
	IF (iprint) THEN PRINT,' nspectra =',nspectra
	IF TOTAL(SIZE(datp.e)) NE TOTAL(SIZE(w_in)) THEN e_in=SQRT(w_in) ELSE e_in=datp.e
	IF (N_ELEMENTS(dQ) EQ 0) THEN dQ = 0
	lambda = datp.p[21]
	IF (iprint) THEN PRINT,' dQ =',dQ,' lambda =', lambda

	IF (iprint) THEN PRINT,'End of input setup section'

	y_out = datp.y
	w_out = w_in
	e_out = e_in

;-------------------------------------------------------------------------------
;add detectors together with dQ spacing

	k    = 2.*!pi/lambda
	Qarr = 2.*k*SIN(y_out*!pi/360.)	; convert scattering angle to Q

	Qmin = -1.
	iout = -1
                
	FOR ispec=0,nspectra-1 DO BEGIN
		Q = Qarr[ispec]
;;		IF (iprint) THEN PRINT,' ispec =',ispec,' Q =',Q
		IF (ABS(Q-Qmin) GT dQ) THEN BEGIN
			IF ispec NE 0 THEN BEGIN
				y_out[iout]   = y_out[iout]/nout
				w_out[*,iout] = w_out[*,iout]/nout
				e_out[*,iout] = e_out[*,iout]/nout
				IF (iprint) THEN PRINT,' Adding ',strtrim(nout,2),' detectors at 2theta = ',strtrim(y_out[iout],2),' deg'
			ENDIF
			Qmin = Q
			iout = iout + 1
			nout = 1
			y_out[iout]   = y_out[ispec]
			w_out[*,iout] = w_out[*,ispec]
			e_out[*,iout] = e_out[*,ispec]^2
		ENDIF ELSE BEGIN

                        if not keyword_set(noaverage) then nout = nout + 1
                        
			y_out[iout]   = y_out[iout]   + y_out[ispec]
			w_out[*,iout] = w_out[*,iout] + w_out[*,ispec]
			e_out[*,iout] = e_out[*,iout] + e_out[*,ispec]^2
		ENDELSE
	ENDFOR
	IF (iprint) THEN PRINT,' iout =',iout

	y_out[iout]   = y_out[iout]/nout
	w_out[*,iout] = w_out[*,iout]/nout
	e_out[*,iout] = e_out[*,iout]/nout

	y_out = y_out[0:iout]
	w_out = w_out[*,0:iout]
	e_out = SQRT(e_out[*,0:iout])
	nspectra = iout + 1


;-------------------------------------------------------------------------------
;Return parameters and exit

	mod_datp, datp, "y", y_out
	mod_datp, datp, "e", e_out

	s = STRTRIM(STRING(dQ),2) & s = string_round(s)
	CASE inst OF
		'IN5': Qstring = inst + ' sum over dQ = ' + s + ' A-1'
		ELSE: BEGIN
			IF dQ EQ 0.0 THEN Qstring = inst + ' - standard sum over angles'
			IF dQ NE 0.0 THEN Qstring = inst + ' sum over dQ = ' + s + ' A-1'
		END
	ENDCASE
        if keyword_set(verbose)	then print,'SUMBANK: '  +Qstring
	IF dQ EQ 0.0 THEN qs = '' ELSE qs = ':dQ=' + s
	datp.other_tit = datp.other_tit+' -sb('+inst+qs+')'

	give_datp, datp

finished:
	RETURN, w_out

	END
