;+
; NAME:
;       LOADMCSTAS
;
; PURPOSE:
;       Load McStas data after pre-processing.
; CATEGORY:
;       
; EXPLANATION:
;       Load the McStas data for comparison with the experimental data
;
; CALLING SEQUENCE:
;       
;
; INPUT:
;       image = 2-D array (matrix)
;
; OPTIONAL INPUT KEYWORDS:
;      
; RESULT:
;      
;
; EXAMPLES:
;       
;
; EXTERNAL CALLS:
;      
;
; PROCEDURE:
;       
; HISTORY:
;       
;-
function loadmcstas, filename=filename, verbose=verbose

    COMMON c_lamp_access, inst
 
    ; --------------------------------------------------------
    ; Read the data from the formatted file
    ; --------------------------------------------------------
    openr, lun, filename,/get_lun

    ; Read the Q values
    header = STRARR(1)
    READF, lun, header
    print, header
    junk = "" ; A string variable
    nbQ = 0  ; An integer variable
    READS, header[0], junk, nbQ, FORMAT='(A13, x, I0)'

    Q = fltarr(nbQ)
    READF, lun, Q

    ; Read the energy values
    READF, lun, header
    print, header
    junk = "" ; A string variable
    nbE = 0  ; An integer variable
    READS, header[0], junk, nbE, FORMAT='(A21, x, I0)'
    En = fltarr(nbE)
    READF, lun, En

    READF, lun, header
    print, header
    Data = fltarr(nbQ, nbE)
    READF,lun, Data

    free_lun, lun


    ; --------------------------------------------------------
    ; Set the Lamp data worspace 
    ; --------------------------------------------------------
    ; return the associated structure:
    DATP={X: Q , Y:En , Z:0 , E: Data*0.0 , N:0   ,$
          W_TIT:  'McStas:'+filename  , X_TIT:'2Theta [deg]'  , Y_TIT:'Energy [meV]',$
          Z_TIT:  ''  , OTHER_TIT:''            ,$
          P:     [nbQ, nbE, NbQ*nbE]   ,    $
          PAR_TXT:'',   $
          PV: 0       }

    GIVE_DATP,datp

    ; return the workspace:
    return, Data
end


