; ----------------------------------------------------------------------------------------------
;
;+
; NAME:
;    regularise
;
; PURPOSES:
;    set the angles as a regular grid and set data accoedingly
;
;
;  
;
;
;  
;
;
;  HISTORY:
;   Written:    JO, Wed Nov 11 18:32:46 CET 2008
;   Changes:    JO, Wed Dec 10 18:32:46 CET 2008: Correct error on eout after rdsum
;
;
;
;-
; ----------------------------------------------------------------------------------------------
function regularise, Win, verbose=verbose

    st1=systime(1)
    
    take_datp,datp
	X   = datp.X 
	Y   = [datp.y]  
	Z   = [datp.z]  

	sizeWin = size(Win)    
	sizeX   = sizeWin[1]
    errorOK = (n_elements(datp.e) eq  n_elements(Win))
	 
    if sizeWin[0] gt 1 then $
        sizeY = sizeWin[2] $
	else $
	    sizeY =1 
	osizeX = sizeX                               ; osx, sx
	sizeXmoinsUn = sizeX - 1                     ; sx1
                                                 ; bx = dPhi
     maxPhi = max(X,min=minPhi) 
     dPhi   = double( 2.0*asin(26.0/2.0/4000.0) * 180.0/!PI ) 
     endx   = (X[1] -X[0])*sizeXmoinsUn + minPhi

;    if (endx le maxPhi+dPhi/10.) and (endx ge maxPhi-dPhi/10.)  then $
;        regX=1 $
;    else $
;        regX=0

    ; --------------------------------------------------------------
    ; define outputs
    ; --------------------------------------------------------------
    if sizeWin[0] gt 2 then $
        sizeZ = sizeWin[3] $
    else $
        sizeZ =1 
    osizeY = sizeY 
    sizeYMoinsUn = sizeY-1 
    ny1 = n_elements(Y)-1
    
    if X[n_elements(X)-1] lt X[0] then $
        directionPhi = -1 $
    else $
        directionPhi = 1          
    sizeZMoinsUn = sizeZ - 1

    osizeX  = round( ( maxPhi - minPhi ) / dPhi ) + 1    ; osx, sx

    if keyword_set(verbose) then $
        print, 'REGULARISE: regularise ',strtrim(sizeX,2),' angles in ',strtrim(osizeX,2),' regular grid angles.'
    
    Xout = dindgen(osizeX)*dPhi + minPhi
    Xout = reverse( Xout )
    Wout = reform( fltarr(osizeX,osizeY,sizeZ) )
    if errorOK then Eout = Wout else Eout = datp.e
    errorSquared   = datp.e^2
    Wcnt = reform( fltarr(osizeX,osizeY) )

; --------------------------------------------------------------
;  Fast fast fast ... But it duzn't wuk :
; a new algo. available , see: remplissage_regulier_des_angles_pour_regularise.pro
; --------------------------------------------------------------
;    ind = value_locate(X,Xout)
;    ind = ind + 1
;    for k = 0,n_elements(ind)-1 do $
;        Wout( k,*,* )     = Wout( k,*,* ) + Win( ind(k),*,*)

      	
   ; --------------------------------------------------------------
   ; Main loop
   ; --------------------------------------------------------------
    tt = ( X[0:sizeXMoinsUn - 1] + X[1:*] )/2.0 
    xa = [ X[0] - dPhi*directionPhi, tt ] 
    xb = [ tt, X[sizeXMoinsUn] + dPhi*directionPhi ]
    for i = 0,sizeXMoinsUn do begin  ;** Loop on X
        ; -------------------------------------------------
        ;   Find Xout indices
        ; -------------------------------------------------
        if xa[i] le xb[i] then $
            xw = where((Xout ge xa[i]) and (Xout le xb[i])) $
        else $
            xw = where((Xout ge xb[i]) and (Xout le xa[i]))
        ; -------------------------------------------------
        ;   Fill output
        ; -------------------------------------------------
        if (xw[0] ge 0) then begin
            for n=0,n_elements(xw)-1 do begin
                if errorOK then $
                    Eout[xw[n],*,*] = Eout[xw[n],*,*] + errorSquared[i,*,*]
                Wout[xw[n],*,*]     = Wout[xw[n],*,*] + Win[         i,*,*]
                Wcnt[xw[n],*  ]  = Wcnt[xw[n],*  ] + 1.0
            endfor
        endif
    endfor          ;**X

    ; --------------------------------------------------------------
    ;  Rectifie les doubles comptages
    ; --------------------------------------------------------------
    Wcnt=Wcnt>1.
    for k=0,sizeZMoinsUn do $
        Wout[*,*,k]=Wout[*,*,k]/Wcnt
    if errorOK then $
        for k=0,sizeZMoinsUn do $
            Eout[*,*,k]=sqrt(Eout[*,*,k]/Wcnt)
   
    ; --------------------------------------------------------------
    ;  
    ; --------------------------------------------------------------
    if keyword_set(verbose) then $
        print, 'REGULARISE: Done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'
    ; --------------------------------------------------------------
    ;  
    ; --------------------------------------------------------------
    mod_datp,datp,'X',Xout 
    mod_datp,datp,'Y',Y
    mod_datp,datp,'Z',Z 
    mod_datp,datp,'E',Eout
    give_datp, datp
    
    return, Wout	
	
end
