; --------------------------------------------------------------------------------------------
;
;  saveDaveAscii
;
;  Create the .dave files to be used by the Dave (and PAN) NIST suite.
;
;  Usage:
;   saveDaveAscii, W1, file = 'filename.spe', [/verbose]
;
;  If constant-Q, W1 should be in output format from sqw_rebin: x = Q, y = energies
;
;  Written by: J. Combet 19/10/2011
;  Changes   : JOR, Tue Feb  5 14:27:44 CET 2013: Converted to a nornmal .pro procedure
;
; --------------------------------------------------------------------------------------------
PRO writeDaveAscii, w_in, file = file, verbose=verbose
    
;;;    print, 'WRITEDAVEASCII: Warning: Data should be (Q,hw) format !' 
    
    COMMON c_lamp_access, inst

    DEG2RAD = !pi/180.0
    RAD2DEG = 180.0/!pi

    st1 = systime(1)

    TAKE_DATP, datp
    x_in = datp.x
    y_in = datp.y
    e_in = datp.e
    x_tit = datp.x_tit
    y_tit = datp.y_tit
;;;    e_in = e_in<1.0e3  ; limit the size of ein in prder to avoid ****** in output file

    sizeWin   = size(w_in)
    NbEnergy  = sizeWin[1]
    NbQval    = sizeWin[2]
       
    u=0 & OPENW,u,file,/get_lun
    printf,u, '# '+x_tit ; , format='(a31)'        
    printf,u, NbEnergy, format='(I0-)'                                 
    printf,u, '# '+y_tit  ; , format='(a31)'        
    printf,u, NbQval, format='(I0-)'                                   
    printf,u, '#  '+x_tit ; , format='(a22)'     
    printf,u, x_in, format='(g0-)'                          
    printf,u, '# '+y_tit  ; ', format='(a22)'     
    printf,u, y_in, format='(g0-)'                         

    for i=0,NbQval-1 do begin 
        printf,u, '# Group No ', i,' (',y_in[i],')',format='(A-,I2-,A-,g0-,A-)'
        for j=0, NbEnergy-1 do $
            printf,u, w_in[j,i], '    ',e_in[j,i], format='(g0-,A2,g0-)'
    endfor
    FREE_LUN,u


    ; ----------------------------------------------------------------------------------       
    ;  Elapsed time
    ; ----------------------------------------------------------------------------------       
    if keyword_set(verbose) then $
        print, 'WRITEDAVEASCII: Everything done in ',strtrim(round( 1000*(systime(1)-st1) ) ,2),' ms'

END    
