pro rdid_pel_NX, DATA, P, P_txt, X ,Y ,Z ,N ,PV ,E, WT, XT, YT, ZT, OT,RAWB,GH_STRUCT
;+
;** gjk Oct2013
;** LAMP procedure to convert NEXUS data into the input required 
;** for the Pelican LAMP software reduction suite.
;** Called from read_hdf.pro
; This is only called for single runs or when it is the last in a series read.
; If the raw button is on then it only sorts out parameters and monitors leaving the original 3D data array
; In addition, the raw button is off it makes data into a 2D array (TOF,Angle) and corrects for 
; time parallax and D-S cones. Also divides by monitor (then factored up to counts-ish) and gets the error array   
;-
  delme=1    ; Dummy variable so that the LAMP runtime will update
;
  s=size( data)           ;size vector: number of dimension, values of dimensions  
  print,'gjk 18/3/14  '
; Set up fixed values

  X_pel=findgen(s[3])     ; put integers in for channel numbers
  n_dets=s[1]             ; number of detectors,including monitors
  n_pix=s[2]              ; number of pixels per tube
  n_dead=3                ; number of dead pixels at the end of each tube
  nch=s[3]                ; number of time channels
  n_mons=8                ; number of monitor spectra (including junk)
  which_mon=7             ; the number of the incident monitor spectrum
  nint=15                 ; number of channels either side of max for epp calc
  tot_tubes=n_dets-n_mons ; number of detector tubes
  tube_length=100.0       ; detector tube length in cm
  tube_width=2.63097      ; width occupied by each detector tube 
  rads=180.0/!pi          ; rads to degrees
  npix2=n_pix/2.0         ; half number of pixels
  pix_l=1.6667            ; pixel length in cm ****This should be 1.0!
  tank_angle_offset=60.4688      ; 2theta of tank_centre
  tim_am=252.78           ; time for 1A per M in microsec
  monitor_factor=1.0e6    ; factor for dividing monitor counts
  foul_up=0               ; error flag
;
; Get the parameters first so that we can use them out-of-the-box
; Search nexus text string then vlaue          
  idx0  = strpos( p_txt, 'bm1_time' )         & idx0  = where( idx0  ge 0 ) & idx0  = idx0[ 0 ] & if idx0  ge 0 then idx0 =P[idx0 ]/60. else idx0 =0.
  idx2  = strpos( p_txt, 'bm1_counts' )       & idx2  = where( idx2  ge 0 ) & idx2  = idx2[ 0 ] & if idx2  ge 0 then idx2 =P[idx2 ] else idx2 =0.
  idx3  = strpos( p_txt, 'bm2_counts' )       & idx3  = where( idx3  ge 0 ) & idx3  = idx3[ 0 ] & if idx3  ge 0 then idx3 =P[idx3 ] else idx3 =0.
  idx4  = strpos( p_txt, 'bm3_counts' )       & idx4  = where( idx4  ge 0 ) & idx4  = idx4[ 0 ] & if idx4  ge 0 then idx4 =P[idx4 ] else idx4 =0.
  idx5  = strpos( p_txt, 'total_counts count')& idx5  = where( idx5  ge 0 ) & idx5  = idx5[ 0 ] & if idx5  ge 0 then idx5 =P[idx5 ] else idx5 =0.
  idx8  = strpos( p_txt, ' run_number ' )     & idx8  = where( idx8  ge 0 ) & idx8  = idx8[ 0 ] & if idx8  ge 0 then idx8 =P[idx8 ] else idx8 =0.
  idx9  = strpos( p_txt, ' temperature ' )    & idx9  = where( idx9  ge 0 ) & idx9  = idx9[ 0 ] & if idx9  ge 0 then idx9 =P[idx9 ] else idx9 =0.
  idx10 = strpos( p_txt, ' regulation_temp' ) & idx10 = where( idx10 ge 0 ) & idx10 = idx10[0 ] & if idx10 ge 0 then idx10=P[idx10] else idx10=0.
  idx13 = strpos( p_txt, 'mchs rpm' )         & idx13 = where( idx13 ge 0 ) & idx13 = idx13[0 ] & if idx13 ge 0 then idx13=P[idx13] else idx13=0.
  idx14 = strpos( p_txt, 'schs rpm' )         & idx14 = where( idx14 ge 0 ) & idx14 = idx14[0 ] & if idx14 ge 0 then idx14=P[idx14] else idx14=0.
  idx15 = strpos( p_txt, 'or) phase' )        & idx15 = where( idx15 ge 0 ) & idx15 = idx15[0 ] & if idx15 ge 0 then idx15=P[idx15] else idx15=0.
  idx16 = strpos( p_txt, 'tofw micro' )       & idx16 = where( idx16 ge 0 ) & idx16 = idx16[0 ] & if idx16 ge 0 then idx16=P[idx16] else idx16=0.
  idx18 = strpos( p_txt, 'wavelength' )       & idx18 = where( idx18 ge 0 ) & idx18 = idx18[0 ] & if idx18 ge 0 then idx18=P[idx18] else idx18=0.
  idx23 = strpos( p_txt, 'radius mm' )        & idx23 = where( idx23 ge 0 ) & idx23 = idx23[0 ] & if idx23 ge 0 then idx23=P[idx23] else idx23=0.
  idx24 = strpos( p_txt, 'stth' )             & idx24 = where( idx24 ge 0 ) & idx24 = idx24[0 ] & if idx24 ge 0 then idx24=P[idx24] else idx24=0.
  tank_angle=abs(idx24)
  side='left'
  if idx24 lt 0.0 then side='right'
  n_par_pel=39
  roar='raw  on'
  if rawb eq 0 then roar='raw off'

;
; Text for pelican paramters
  p_txt_pel=strarr(n_par_pel)  
  p_txt_pel[0] ='0)  Run time [min] (' + roar + ')        ='  
  p_txt_pel[1] ='1)  Monitor normalisation factor    ='
  p_txt_pel[2] ='2)  Number of time channels         ='  
  p_txt_pel[3] ='3)  Total Counts in   M1            ='
  p_txt_pel[4] ='4)  Total Counts in M2              =' 
  p_txt_pel[5] ='5)  Total Counts in   M3            ='
  p_txt_pel[6] ='6)  Total counts on detector        =' 
  p_txt_pel[7] ='7)  Infinite-energy channel         =' 
  p_txt_pel[8] ='8)  Sample angle [deg.]             ='  
  p_txt_pel[9] ='9)  Detector elastic-peak channel   ='
  p_txt_pel[10]='10) Monitor1 elastic-peak channel   ='
  p_txt_pel[11]='11) Sample   temperature [K]        ='
  p_txt_pel[12]='12) Regulation temperature [K]      =' 
  p_txt_pel[13]='13) Repetition   period [microsec]  ='
  p_txt_pel[14]='14) Chopper Ratio                   =' 
  p_txt_pel[15]='15) F_Chopper speed   [rpm]         ='
  p_txt_pel[16]='16) Overlap chopper speed [rpm]     =' 
  p_txt_pel[17]='17) Overlap chopper phase           ='
  p_txt_pel[18]='18) Channel width [microsec]        ='
  p_txt_pel[19]='19) Number of time channels         ='  
  p_txt_pel[20]='20) TOF delay [microsec]            =' 
  p_txt_pel[21]='21) Wavelength   [angstroms]        ='
  p_txt_pel[22]='22) Distance F_chopper - M1 [m]     ='  
  p_txt_pel[23]='23) Distance F_chopper - Sample [m] ='
  p_txt_pel[24]='24) Detector-tank angle (' + side + ')      ='
  p_txt_pel[25]='25) Distance M1  -   M2 [m]         ='
  p_txt_pel[26]='26) Distance M1  - M3 [m]           =' 
  p_txt_pel[27]='27) Distance Sample - detectors [m] ='
  p_txt_pel[28]='28) Number of angles                ='
  p_txt_pel[29]='29) Number of heights               ='
;
; Now put in values (where we can) 
;
; get detector assemblies as angles (Y_pel) and figure out central angle
  tube_ang=GH_STRUCT.x_pixel_angular_offset[n_mons:n_dets-1]
  GH_STRUCT = 0 ; free memory
;
; Make tube angles positive regardless of detector-bank side.
  tube_ang=abs(tube_ang)
  Y_pel=abs((tank_angle-tank_angle_offset)+tube_ang) 
  y_size=size(Y_pel)
  P_pel=fltarr(n_par_pel )          
  P_pel[0]  = idx0 
  p_pel[2]  = nch
  P_pel[4]  = idx3
  P_pel[5]  = idx4
  P_pel[8]  = 0.
  P_pel[9]  = 0. 
  P_pel[11] = idx9
  P_pel[12] = idx10
  P_pel[13] = 1.0e6/(idx13*2.0/60.0)
  P_pel[14] = 0.
  if idx14 gt 0.0 then P_pel[14] = idx15/idx14
  P_pel[15] = idx13
  P_pel[16] = idx14
  P_pel[17] = idx15
  P_pel[18] = idx16
  p_pel[19] = nch
  P_pel[20] = 0.
  P_pel[21] = idx18
  P_pel[22] = 0.
  P_pel[23] = 0.
  P_pel[24] = tank_angle
  P_pel[25] = 0.
  P_pel[26] = 0.
  P_pel[27] = idx23/1000.
  P_pel[28] = y_size[1]   
  P_pel[29] = 60
;
; get the monitor spectrum in N and integrate peak
  N_pel  =total(data[7,*,*],2)
  Det_tot=total(data[8:n_dets-1,*,*])
  P_pel[6]=Det_tot
  mmax=max(N_pel,mepp)   ; get integer of monitor elastic peak channel
  ip_1=mepp-nint         ; set integration limits and check in spectrum
  ip_2=mepp+nint
  P_pel[10]=mepp
  if (ip_1 lt 0) or (ip_2 ge s[3])then begin
     print,'Warning: Elastic peak in monitor too close to end of spectrum'
     print,'Monitor set to simple integral of whole spectrum'
     print,string(7B)
     p_txt_pel[10]='10) Warning: monitor integrated - peak unsuitable     '
     mon_int=total(N_pel)
     foul_up=1
  endif else begin
;
;   Check that count-rate in peak is > 10*average
     mon_int=total(N_pel[ip_1:ip_2])
     pk_mean=mon_int/(nint*2)
     mon_mean=total(N_pel)/nch
     if (pk_mean lt 10.0*mon_mean) then begin
        print,'Warning: No decent peak in monitor',' pk_mean:',pk_mean,' mon_mean:',mon_mean
        print,'Monitor set to simple integral of whole spectrum'
        p_txt_pel[10]='10) Warning: monitor integrated - peak unsuitable     '
        print,string(7B)
        mon_int=total(N_pel)
        if (mon_int lt 1) then print,'Warning: monitor integral less than 1, integral set ot 1'
        foul_up=1
     endif 
  endelse    
  P_pel[1] = mon_int/monitor_factor 
  P_pel[3] = mon_int
  f_pth_cm = P_pel[27]*100
  work=data[n_mons:n_dets-1,n_dead-1:n_pix-n_dead,*] ;working copy of data without monitors and dead pixels
  data=0 ;free memory
  n_pix=n_pix-(2.0*n_dead)
  npix2=n_pix/2
  tot_spec=total(work[*,npix2-1:npix2,*],2)          ;sum spectra of middle 2 pixels 
  tot_spec=total(tot_spec,1)
  w_shift =fltarr(tot_tubes,n_pix,nch)               ;set up shifted array-space
;
; get overall elastic peak position, find max and step back nint channels for cog determination
  smax=max(tot_spec,nepp)   ; get integer elastic peak channel
  ip_1=nepp-nint            ; got cog of peak underneath
  ip_2=nepp+nint
  epp =(total(X_pel[ip_1:ip_2]*tot_spec[ip_1:ip_2])/total(tot_spec[ip_1:ip_2]))
  P_pel[9]= epp
  ch_einf = epp-tim_am*P_pel[27]*P_pel[21]/P_pel[18]  ; infinite energy channel should be the same in all!
  P_pel[7]= ch_einf
;
; if not raw then make 2D 
;
;****************** Split raw and not-raw here ***********************
;
  if rawb eq 0 then begin
; Reverse the data along angle-axis if tank on left.    
    if side eq 'left' then work = reverse(work,1,/overwrite)
; make monitor spectrum unity
    N_pel=1.0
;
; *************get time/distance wrapped up****************
    tim_m=P_pel[21]*tim_am
;
; Then work out shifted channels per pixel
    pix_ar=findgen(n_pix/2)*pix_l+pix_l  ; vector of vertical pixel positions
    pix_r =sqrt(f_pth_cm^2+pix_ar^2)     ; radius sample to each pixel, only need 50
;
; Find the shift, then make the shift
    last_chan=nch-1
    for j=0,last_chan do begin
      for i=0,npix2-1 do begin
        new_chan=j-round((ch_einf+((X_pel[j]-ch_einf)/(pix_r[i])*pix_r[0]))-X_pel[j])
        if new_chan gt last_chan then new_chan=new_chan-last_chan   ;wrap back ends
        if new_chan lt 0 then new_chan=last_chan+new_chan
        w_shift[*,(npix2-1)-i,j]=work[*,npix2-i,new_chan]
        w_shift[*,(npix2+i),j]  =work[*,npix2+i,new_chan]   ;copy the other half
      endfor
    endfor
    work=0 ;free memory
;
; ********************get the angles for the pixels****************************
; Do this as the dot product of vectors to get angle of pixel
    v1  =[0,0,1.0] ; direct beam along Z
    v2x =f_pth_cm*sin(Y_pel/rads)
    v2xS=v2x^2
    v2y =findgen(tube_length/2) ; take points up the tube
    v2yS=v2y^2
    v2z =f_pth_cm*cos(Y_pel/rads)
    v2zS=v2z^2
    ang =fltarr(tot_tubes,npix2)
    for j=0,tot_tubes-1 do begin
      for i=0,npix2-1 do begin
        vecnorm=sqrt(v2xS[j]+v2yS[i]+v2zS[j]) ; normalise vector
        v2 =[ v2x[j] , v2y[i] , v2z[j] ] /vecnorm
        dot=total(v2*v1)
        ang[j,i]=rads*acos(dot) ;horrid bodge 0.1
      endfor
    endfor
    v2x=0 & v2y=0 & v2z=0 & v2xS=0 & v2yS=0 & v2zS=0 ;free memory
    
; sum into nearest (truncated) detector angle
    w_debye=fltarr(nch,tot_tubes)
    for j=0,tot_tubes-1 do begin
      for i=0,npix2-1 do begin
;        new_tube=value_locate(Y_pel,ang[j,i])    ;locate new tube
         kt=value_locate(Y_pel,ang[j,i])    ;locate lower match
         new_tube=kt
         if kt ge 0 then begin
           if kt lt tot_tubes-1 then begin
             half=(ang[j,i]-Y_pel[kt])/(Y_pel[kt+1]-Y_pel[kt])
             if half gt 0.5 then new_tube=kt+1
           endif
           w_debye[*,new_tube] += w_shift[j,(npix2-1)-i,*]
           w_debye[*,new_tube] += w_shift[j,npix2+i,*]
         endif
      endfor
    endfor
;   data=transpose(w_shift,[2,0,1]) ;useful for length corrected whole spec
    w_shift=0 ;free memory
; tidy for return
    E=sqrt(w_debye)
    xt='TOF channels (r-corrected)'
    yt='Angle (deg. cone corrected)'
    data=temporary(w_debye)/P_pel[1] ;and free w_debye
    E=E/P_pel[1]
;
;    
  endif else begin ; Was not raw data
; For the raw 
;
; Transpose data to sane orientation T,Ang,Ht, and reverse if bank on left.
    data=transpose(temporary(work),[2,0,1]) ;and free work
    if side eq 'left' then data = reverse(data,2,/overwrite)
    E   =0
   ;E   =sqrt(data) ; Can be done in Do buttons if necessary
    xt  ='TOF (channels)'
    yt  ='Detector Angle (deg.)'
    zt  ='Detector height (pix.)'

  endelse
;
; Return the results
  N=n_pel
  P=P_pel
  P_txt=P_txt_pel
  X=X_pel   ;angles
  Y=Y_pel   ;channels
end
