function sans_inst_init, flagg, index, detres=detres
;+
;******* **************
;**
;** Sets any instrument dependent variables.
;-

if n_elements(detres) ne 2 then detres=[0,0] ; used when detector may have many resolutions

case flagg of
                ; 0     1     2       3        4        5        6        7        8     9        10
'List1': return,['D11','D22','BRISP','MILAND','D16<09','D11<07','D11<98','D22<07','D33','D11<02','PAXY','D11_256'] ;List name of supported instruments
'List2': return,['D11','D22','BRISP','D16'   ,'D16'   ,'D11'   ,'D11'   ,'D22'   ,'D33','D11'   ,'PAXY','D11'    ] ;Real name of instruments
'LstOn': return,[ 1   , 1   , 1     , 1      , 1      , 1      , 0      , 1      , 1   , 0      ,1     ,0]      ;Active instruments
'LstSR': return,[ 1   , 1   , 1     , 2      , 2      , 1      , 1      , 1      , 1   , 1      ,1     ,1]      ;0=reflectometry 1=sans 2=scan

'Inits': begin        ;Default parameters index set in rdid or read_hdf
                      ;0         1         2       3           4          5           6      7           8
                      ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
               paridx=[15      , 9       , 19    , 18        , 17       , 2         , 26   , 27        , 28,$
                       24      , 22      , 10    , 16        , 6        , 5         , -1   , -1        , -1  , -1  , -1]
                      ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight, Xdet , Ydet      , Mode, att2, Tau
                      ;9         10        11      12          13         14          15     16        , 17  , 18  , 19
                      
               Qdep    =''      ;Qdep  formula should contain 'Q' and/or 'DIST'(m) and/or 'THETHA' and/or 'RADIUS'(cm) and/or 'WAVE'
               XWdep   =''      ;XWdep formula for the predefined water cross section should contain 'WAVE'
               txtWater='Water' ;or 'Vanad' or 'Plexi'
               Mon_Tim =0       ;0 use monitors , 1 use time (default) for transm corrections
               disQdep =0       ;1 disable Qdep
               SAc     =1       ;1 Solide angle   correction (default)
               THc     =1       ;1 Thickness F(Q) correction (default)
               Raxe    =0.      ;rotation axe of detector (cm)
               NegMask =0       ;>=1 means nb of panels and negative counts must be masked (inter panels)
               xyUnit  =0       ;1 XYaxis=2theta at read

               bfit    =1       ;default gaussfit for center , 0 centroid
               ofa     =1       ;0:disable offset , 1:enable offset detector , 2:enable offset distance
               beamf   =1       ;1 rectangular , 2 elipsoid
               tau     =0.      ;dead time factor
               ftau    =1       ;fortau function index [None , I/(1-tau*tot/time) , I/exp(-tau*tot/time) , by vert tubes , by hori tubes]
               FX      =0       ;0 first index is (1) , -1 first index is (0)
               attenuF =0       ;attenuation factor versus collimation
               FCT     =[1.,1.] ;128./[nX,nY] image detector
  
               case index of
               0:  begin ;** D11 ** from 2007
                   collima= '1.5,2.5,4.0,5.5,8.0,10.5,13.5,16.5,20.5,28.,34.,40.5'
                   att2017 = '1., 324., 1052., 2813.' ;attenuators used until 2017
                   att2018 = ', 8.53, 30.65, 69.29, 256.28, 603.85, 1990.0, 14889.27' ;new attenuators installed on 2018
                   attenua = att2017 + att2018
                   ; attenuF = correction to attenuation due to collimation dependence, tested in 2017, not needed with 2018 atts            
                   ; attenuF=[[1.   , 1.  , 1.   , 1.   , 1., 1.  , 1.  , 1.  , 1.  , 1.  , 1.  , 1.   ],$ ;No attenuator
                   ;          [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134],$ ;Attenuator1 factor/collimation
                   ;          [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134],$ ;Attenuator2 factor/collimation (temp)
                   ;          [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134]]  ;Attenuator3 factor/collimation (temp)
                   Qdep = '(cos(THETA^3/dist*2)^2)*(1-(radius/(max(radius)>(1e-9))*.095))'
                   XWdep= '.7078+ .03652*Lambda+ .0013467*Lambda^2+ 3.6842e-5*Lambda^3'
                   fct    =[1.,1.]       & centex='63.50 '  & centey='63.50 '  & FX  =-1
                   beamf  =1 & Xdet=.75  & Ydet=.75         & tau=.33e-6       & ftau= 1 ; tau=.42e-6 -> Jun 2014
                   ofa    =0 & bfit=0    & disQdep=1
                   paridx[15:16] = [37,38] ;Pixel size (cm)
                   paridx[19]    = [41]    ;Tau (ns)
                   rdset, /RAW
                   if (detres[0] eq .375) and (detres[1] eq .375) then begin ;Detector is 256*256 (default was 128*128)

                     fct = [0.5 , 0.5]   & centex='127.50 ' & centey='127.50 '
                               Xdet=.375 & Ydet=.375        & tau=.42e-6
                   endif
                   end
               1:  begin ;** D22 **
                   collima='1.4,2.0,2.8,4.0,5.6,8.0,11.2,14.4,17.6'
                   attenua='1,147,902,2874'                 & Qdep ='.14*exp(-4.*alog(2.)*(((THETAX-.588)<0.)/.414)^2)'
                                                              XWdep=''
                   fct    =[1.,1.]       & centex='64.50 '  & centey='64.50 '  & FX  = 0
                   beamf  =1 & Xdet=.8   & Ydet=.8          & tau=2.0e-6       & ftau= 3
                   ofa    =1 & bfit=1    & Raxe=20.
                   paridx[18] = 34
                   paridx[15:16] = [37,38]
                   rdset, /RAW
                   if (detres[0] eq .8) and (detres[1] eq .4) then begin ;Detector is 128*256 (default was 128*128)

                     fct = [1.0 , 0.5]   & centex='64.50 '  & centey='128.50 '
                               Xdet=.8   & Ydet=.4          & tau=2.0e-6 
                   endif
                   end
               2:  begin ;** BRISP **
                   collima='1.,2.,4.,6.'
                   attenua='1'                              & Qdep =''
                   fct    =[.5,.8]       & centex='128.50'  & centey='80.50 '  & FX  = 0
                   beamf  =2 & Xdet=.428 & Ydet=1.27        & tau=0.           & ftau= 0
                   ofa    =1 & bfit=1    & txtWater='Vanad'
                   rdset, /RAW
                   end
               3:  begin ;** MILAND **
                   collima='2.5'
                   attenua='1,10,100,1000'                  & Qdep =''
                   fct    =[.4,.4]       & centex='160.50'  & centey='160.50 ' & FX  =-1
                   beamf  =1 & Xdet=.1   & Ydet=.1          & tau=0.           & ftau= 0
                   ofa    =0 & bfit=1    & xyUnit=1         & SAc=0            & THc = 0
                   paridx=[ 8      , 34      , 36    ,  0        , 35       , 37        , 26   , 27        , 28 , $
                           29      ,  1      , 11    , 31        , 32       , 33        , -1   , -1        , -1 , -1 , -1]
                   rdset, /NORAW ;Correct bad detectors
                   end
               4:  begin ;** D16 **
                   collima='2.5'
                   attenua='1,10,100,1000'                  & Qdep =''
                   fct    =[1.,1.]       & centex='63.0 '   & centey='63.0 '   & FX  =-1
                   fct    =[128./127.,128./127.]
                   beamf  =1 & Xdet=.2   & Ydet=.2          & tau=0.           & ftau= 0
                   ofa    =0 & bfit=1    & xyUnit=1         & SAc=0            & THc = 0
                   paridx=[ 8      , 34      , 36    ,  0        , 35       , 37        , 26   , 27        , 28 , $
                           29      ,  1      , 11    , 31        , 32       , 33        , -1   , -1        , -1 , -1 , -1]
                   rdset, /NORAW ;Correct bad detectors
                   end
               5:  begin ;** D11<07 ** before 2007
                   collima='1.5,2.5,4.0,5.5,8.0,10.5,13.5,16.5,20.5,28.,34.,40.5'
                   attenua='1,289.017,917.431,2840.91'      & Qdep =''
                   fct    =[2.,2.]       & centex='31.50 '  & centey='31.50 '  & FX  =-1
                   beamf  =1 & Xdet=1.   & Ydet=1.          & tau=.85e-6       & ftau= 1
                   ofa    =0 & bfit=0
                   rdset, /RAW
                   end
               6:  begin ;** D11<98 ** only value of Tau has changed
                   collima='1.5,2.5,4.0,5.5,8.0,10.5,13.5,16.5,20.5,28.,34.,40.5'
                   attenua='1,289.017,917.431,2840.91'      & Qdep =''
                   fct    =[2.,2.]       & centex='31.50 '  & centey='31.50 '  & FX  =-1
                   beamf  =1 & Xdet=1.   & Ydet=1.          & tau=2.17e-6      & ftau= 1
                   ofa    =0 & bfit=0
                   rdset, /RAW
                   end
               7:  begin ;** D22<07 **
                   collima='1.4,2.0,2.8,4.0,5.6,8.0,11.2,14.4,17.6'
                   attenua='1,147,902,2874'                 & Qdep =''
                   fct    =[1.,1.]       & centex='64.50 '  & centey='64.50 '  & FX  = 0
                   beamf  =1 & Xdet=.75  & Ydet=.75         & tau=.42e-6       & ftau= 3
                   ofa    =1 & bfit=1    & Raxe=20.
                   rdset, /RAW
                   end
               8:  begin ;** D33 **
                   collima='2.8,5.3,7.8,10.3,12.8'
                   attenua='1,118,449,2061'                ;& Qdep ='' ; Defined in the reading procedure
                  ;         2.8    5.3   7.8    10.3   12.8
                   attenuF=[[1.   , 1.  , 1.   , 1.   , 1.],$ ;No attenuator
                            [.932, 1.008, 1., 1.008, 1.085],$ ;Attenuator1 factor/collimation
                            [.931, 1.011, 1.,  .991, 1.073],$ ;Attenuator2 factor/collimation (temp)
                            [.906, 1.002, 1.,  .986, 1.048]]  ;Attenuator3 factor/collimation (temp)
                   fct    =[.5,1.]       & centex='128.50 ' & centey='64.50 '  & FX  = 0
                   beamf  =1 & Xdet=.25  & Ydet=.5          & tau=1.5e-6       & ftau= 4 ;when Rear panel only
                   ofa    =0 & bfit=1    & Mon_Tim=1        & NegMask=5
                   paridx[15:17] = [37,38,30]
                   rdset, /NORAW ;TOF & Panels merging
                   end
               9:  begin ;** D11<02 ** only value of Tau has changed
                   collima='1.5,2.5,4.0,5.5,8.0,10.5,13.5,16.5,20.5,28.,34.,40.5'
                   attenua='1,289.017,917.431,2840.91'      & Qdep =''
                   fct    =[2.,2.]       & centex='31.50 '  & centey='31.50 '  & FX  =-1
                   beamf  =1 & Xdet=1.   & Ydet=1.          & tau=1.81e-6      & ftau= 1
                   ofa    =0 & bfit=0
                   rdset, /RAW
                   end
               10: begin ;** PAXY **
                   collima='2.25,5.0,7.0'
                   attenua='1.'
                   fct    =[1.,1.]       & centex='64.50 ' & centey='64.50 '   & FX  = 0
                   beamf  =1 & Xdet=.5   & Ydet=.5         & tau=3.6e-6        & ftau= 1
                   ofa    =0 & bfit=1
                   rdset, /RAW
                   end
               11: begin ;** D11/ 256*256 ** new electronic pixel resolution
                   collima='1.5,2.5,4.0,5.5,8.0,10.5,13.5,16.5,20.5,28.,34.,40.5'
                   attenua='1.,324.,1052.,2813.'            & Qdep ='(cos(THETA^3/dist*2)^2)*(1-(radius/(max(radius)>(1e-9))*.095))'
                                                              XWdep='.7078+ .03652*Lambda+ .0013467*Lambda^2+ 3.6842e-5*Lambda^3'
                  ;         1.5    2.5   4.     5.5    8.  10.5  13.5  16.5  20.5  28.   34.   40.5
                   attenuF=[[1.   , 1.  , 1.   , 1.   , 1., 1.  , 1.  , 1.  , 1.  , 1.  , 1.  , 1.   ],$ ;No attenuator
                            [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134],$ ;Attenuator1 factor/collimation
                            [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134],$ ;Attenuator2 factor/collimation (temp)
                            [.9505, .969, 1.052, 1.017, 1., .967, .805, .856, .862, .735, .686, 1.134]]  ;Attenuator3 factor/collimation (temp)
                   fct    =[.5,.5]       & centex='127.50 '   & centey='127.50 ' & FX  =-1
                   beamf  =1 & Xdet=.375  & Ydet=.375         & tau=.33e-6       & ftau= 1 ; tau=.42e-6 -> Jun 2014
                   ofa    =0 & bfit=0    & disQdep=1
                   rdset, /RAW
                   end
               else: return, 'NeverHappen'
               endcase
               
               return, {COLLIMA:collima, ATTENUA:attenua, QDEP:qdep, FCT:fct, CENTEX:centex, CENTEY:centey, FX:fx, $
                        BEAMF:beamf, XDET:xdet, YDET:ydet, TAU:tau,FTAU:ftau, OFA:ofa, BFIT:bfit, PARIDX:paridx, $
                        TXTWATER:txtwater, RAXE:raxe, XYUNIT:xyunit, SAC:sac, THC:thc, MON_TIM:mon_tim, XWdep:xwdep,$
                        disQDEP:disQdep, attenuF:attenuF, NegMask:NegMask}
         end

'Mask':  begin & case index of ;Initial mask for selected instruments, [first entry is for size] else [x1,y1,nx,ny]
               0:  begin     ;** D11 **
                   sans_masq =[[1L,1,128,128],    [1,1 ,128,1],[1,2 ,128,1],[1,128 ,128,1]]
                   sans_masq =[[sans_masq],[1,1,1,128],[128,1,1,128],[2,3,1,1],[127,3,1,1],[2,126,2,2],[126,126,2,2]]  
                                
                   if (detres[0] eq .375) and (detres[1] eq .375) then begin ;Detector is 256*256 (default was 128*128)
                     sans_masq =[[1L,1,256,256],    [1,1 ,256,2],[1,3 ,256,2],[1,255 ,256,2]]
                     sans_masq =[[sans_masq],[1,1,2,256],[255,1,2,256],[3,5,2,2],[253,5,2,2],[3,251,4,4],[251,251,4,4]]
                   endif
                   end
               1:  begin     ;** D22 **
                   sans_masq =[[1L,1,128,128],    [1,1 ,128,1],[1,2 ,128,1],[1,128 ,128,1],[1,127 ,128,1],[1,126 ,128,1]]
                   
                   if (detres[0] eq .8) and (detres[1] eq .4) then begin ;Detector is 128*256 (default was 128*128)
                     sans_masq =[[1L,1,128,256],    [1,1 ,128,4],[1,251 ,128,6]]
                   endif
                   end
               2:  begin     ;** BRISP **
                   sans_masq =[[1L,1,256,160],    [1,1 ,59,16],[198,1 ,59,16],[1,145 ,59,16],[198,145 ,59,16]]
                   end
               3:  begin     ;** MILAND **
                   sans_masq =[[1L,1,320,320]]
                   end
               4:  begin     ;** D16<09 **
                   sans_masq =[[1L,1,127,127]]
                   end
               5:  begin     ;** D11<07 **
                   sans_masq =[[1L,1,64,64],      [1,1 ,1,64] ,[64,1 ,1,64]]
                   sans_masq =[[sans_masq],[1,1 ,13,1],[1,2 ,12,1],[1,3 ,10,1],[1,4 ,9,1],[1,5 ,8,1],[1,6 ,7,1],[1,7 ,5,1],[1,8 ,5,1],[1,9 ,4,1],[1,10 ,3,1],[1,11 ,2,1]]
                   sans_masq =[[sans_masq],[52,1 ,13,1],[53,2 ,12,1],[54,3 ,11,1],[56,4 ,9,1],[57,5 ,8,1],[58,6 ,7,1],[59,7 ,5,1],[60,8 ,5,1],[61,9 ,4,1],[62,10 ,3,1],[63,11 ,2,1]]
                   sans_masq =[[sans_masq],[51,64 ,14,1],[54,63 ,11,1],[55,62 ,10,1],[56,61 ,9,1],[57,60 ,8,1],[58,59 ,7,1],[59,58 ,5,1],[60,57 ,5,1],[61,56 ,4,1],[62,55 ,3,1],[63,54 ,2,1]]
                   sans_masq =[[sans_masq],[1,64 ,14,1],[1,63 ,11,1],[1,62 ,10,1],[1,61 ,9,1],[1,60 ,8,1],[1,59 ,6,1],[1,58 ,5,1],[1,57 ,4,1],[1,56 ,4,1],[1,55 ,3,1],[1,54 ,2,1]]
                   end
               6:  begin     ;** D11<98 **
                   sans_masq =[[1L,1,64,64],      [1,1 ,1,64] ,[64,1 ,1,64]]
                   sans_masq =[[sans_masq],[1,1 ,13,1],[1,2 ,12,1],[1,3 ,10,1],[1,4 ,9,1],[1,5 ,8,1],[1,6 ,7,1],[1,7 ,5,1],[1,8 ,5,1],[1,9 ,4,1],[1,10 ,3,1],[1,11 ,2,1]]
                   sans_masq =[[sans_masq],[52,1 ,13,1],[53,2 ,12,1],[54,3 ,11,1],[56,4 ,9,1],[57,5 ,8,1],[58,6 ,7,1],[59,7 ,5,1],[60,8 ,5,1],[61,9 ,4,1],[62,10 ,3,1],[63,11 ,2,1]]
                   sans_masq =[[sans_masq],[51,64 ,14,1],[54,63 ,11,1],[55,62 ,10,1],[56,61 ,9,1],[57,60 ,8,1],[58,59 ,7,1],[59,58 ,5,1],[60,57 ,5,1],[61,56 ,4,1],[62,55 ,3,1],[63,54 ,2,1]]
                   sans_masq =[[sans_masq],[1,64 ,14,1],[1,63 ,11,1],[1,62 ,10,1],[1,61 ,9,1],[1,60 ,8,1],[1,59 ,6,1],[1,58 ,5,1],[1,57 ,4,1],[1,56 ,4,1],[1,55 ,3,1],[1,54 ,2,1]]
                   end
               7:  begin     ;** D22<07 **
                   sans_masq =[[1L,1,128,128],    [1,1 ,128,1],[1,2 ,128,1],[1,128 ,128,1]]
                   sans_masq =[[sans_masq],[1,1,1,128],[128,1,128,128],[2,3,1,1],[127,3,1,1],[2,126,2,2],[126,126,2,2]]
                   end
               8:  begin     ;** D33 **
                   sans_masq =[[1L,1,256,128]]
                  ;sans_masq =[[sans_masq],[1,1 ,256,1],[1,128 ,128,1]] ; Not for TOF neither with panels
                   end
               9:  begin     ;** D11<02 **
                   sans_masq =[[1L,1,64,64],      [1,1 ,1,64] ,[64,1 ,1,64]]
                   sans_masq =[[sans_masq],[1,1 ,13,1],[1,2 ,12,1],[1,3 ,10,1],[1,4 ,9,1],[1,5 ,8,1],[1,6 ,7,1],[1,7 ,5,1],[1,8 ,5,1],[1,9 ,4,1],[1,10 ,3,1],[1,11 ,2,1]]
                   sans_masq =[[sans_masq],[52,1 ,13,1],[53,2 ,12,1],[54,3 ,11,1],[56,4 ,9,1],[57,5 ,8,1],[58,6 ,7,1],[59,7 ,5,1],[60,8 ,5,1],[61,9 ,4,1],[62,10 ,3,1],[63,11 ,2,1]]
                   sans_masq =[[sans_masq],[51,64 ,14,1],[54,63 ,11,1],[55,62 ,10,1],[56,61 ,9,1],[57,60 ,8,1],[58,59 ,7,1],[59,58 ,5,1],[60,57 ,5,1],[61,56 ,4,1],[62,55 ,3,1],[63,54 ,2,1]]
                   sans_masq =[[sans_masq],[1,64 ,14,1],[1,63 ,11,1],[1,62 ,10,1],[1,61 ,9,1],[1,60 ,8,1],[1,59 ,6,1],[1,58 ,5,1],[1,57 ,4,1],[1,56 ,4,1],[1,55 ,3,1],[1,54 ,2,1]]
                   end
               10: begin     ;** PAXY **
                   sans_masq =[[1L,1,128,128]]
                   end
               11: begin     ;** D11/ 256*256**
                   sans_masq =[[1L,1,256,256],    [1,1 ,256,2],[1,3 ,256,2],[1,255 ,256,2]]
                   sans_masq =[[sans_masq],[1,1,2,256],[255,1,2,256],[3,5,2,2],[253,5,2,2],[3,251,4,4],[251,251,4,4]]               
                   end
               else:   sans_masq =[[1L,1,128,128]] ;** UNKNOWN **
               endcase
               return, sans_masq
          end
else:
endcase

return, 'NeverHappen'
end
