;+
; ** ***************************************************************************************
; **
; ** Program qens_fit
; **
; ** This module permits to fit Quasi-elastic data to a set of
; ** different models (translations, rotations, jumps ...)
; ** S. Rols 09/01 srols@anl.gov, now rols@lcvn.univ-montp2.fr, now rols@ill.eu
; ** Last modification: 02/04/02
; ** Last modification: 15/05/02 - correction of several buggs when fitting all spectra
; **                             - modification of the output files format
; ** Last modification: 21/05/02 - modification of the convolution routine ... use of convol
; ** Last modification: 28/05/02 - modification of the convolution routine ... use of convol
; ** Last modification: 07/07/02 - symetrization of the slopping backg or constant backg
; ** Last modification: 27/11/02 - change of the resolution function color
; **                             - put the Reduced Chi square instead of the Chi square
; ** Last modification: 14/03/03 - fix of a problem on the convolution part ... still unresolved
; ** Last modification: 01/07/03 - Do the convolution again without the /CENTER keywords.
; **                             - Tests are under process ...
; ** Last modification: 02/07/03 - Fix a problem when saving curves.
; **                             - Put the plot dimensions larger
; ** Last modification: 07/07/03 - Restructuration of the file backbone.
; **                             - file fit_basegl.pro contains the fit_basegl,fit_basegl2
; **                               and funlg_to_fit subroutines and functions
; ** Last modification: 08/07/03 - Put a lot of message .... trapping of the errors.
; ** Last modification: 15/07/03 - Correction of several buggs.
; ** Last modification: 16/07/03 - Dealing with res func energy range. Now fixed ...
; **                             - Addition of the saving param. in Q P1 P2 P3 ... format
; ** Last modification: 17/07/03 - Corrrection of a bug on the center of the RF
; **                             - No more intensity but amplitudes for Lorentzian and Gaussian
; **                             - Try to put constraints on the parameters
; ** Last modification: 22/07/03 - Constraints on the parameters are set
; ** Last modification: 25/07/03 - Put a short help
; ** Last modification: 04/08/03 - Correct from several buggs that happened when pushing on "enter"
; ** Last modification: 13/01/04 - Correct from several buggs that happened when saving parameters
; ** Last modification: 07/07/04 - Put the possibility of 2D fitting (Q,w)
; ** Last modification: 10/09/04 - Put the possibility of fitting with a Gumbel function
; ** Last modification: 22/06/05 - Put the possibility of building user defined models
; **                             - Correct from some buggs ...
; ** Last modification: 27/06/05 - Add a choice for the COOLFIT engine
; **                             - Add a STOP button which produces an interrupt event
; **                             - Split common variables in SC_NEW for Lamp-runtime compatibility
; **                             - Access to the CLIP-BOARD and E_LogBook (right click on drawable)
; ** Last modification: 07/07/05 - reverse the resolution function before convolution
; **                             - bad error bars correctly managed and plotted
; ** Last modification: 12/07/05 - correct from buggs in the streched exponentials function
; ** Last modification: 03/10/06 - The write button loads the parameter in the workspace chosen by user
; ** Last modification: 27/10/06 - Put Title to the graph with current spectrum value
; ** Last modification: 07/11/06 - Put the abs(data-fit) and abs((data-fit)/data) graph + print the Re-Chisqr
; **                             - Also modified the error bars to PERR*SQRT(Re-Chisqr) see mpfit.pro documentation
; ** Last modification: 04/05/07 - Modify some of the verifications of the input data (ex: binning...)
; ** Last modification: 20/05/07 - Correct from an error in the Gaussian function definition
; ** Last modification: 30/06/07 - Change of some windows design and functionalites
; ** Last modification: 24/08/07 - Correction of an error in the spherical bessel function definition
; ** Last modification: 08/11/07 - Solved the +/- infinity problem from spline when sx_res and sx_in not the
; **                               same dimension (from IN13 LVE)
; ** Last modification: 16/11/07 - Solve the problem of the central 0.meV peak
; ** Last modification: 27/08/08 - Plot the subfunctions multiplied by the detailed balance factor if checked
; ** Last modification: 28/05/10 - Replace 'P()' by 'P[]'
; ** Last modification: 11/01/11 - Change the load and save buttons in the interface to allow using the 60 lamp workspaces (by SR)
; ** Last modification: 18/01/11 - Gaussian Res Function Input (SR)
; ** Last modification: 07/11/11 - Adds the possibility to choose which parameters to propagate to every spectra
; **                             - Adds the possibility to import saved parameters (CB). Checks for dimension errors
; **
; ** ****************************************************************************************
;-
;** **************************************************************
pro create_commons_qens
;+
;** **************************************************************
;** Just to create correct commons while compiling.
;** Explains also variables
;-

@qens.cbk

;common sc_wid, $
;         sw_ba_leader,                    $;Widget Id of leader base
;         sw_bu_filem, sw_bu_exitm,              $
;         sw_bu_savec,sw_bu_savep,     $
;         sw_bu_fitm,sw_bu_lgm,sw_bu_isorotm,       $
;         sw_bu_axrotm,sw_bu_rwm,             $
;         sw_ba_up,sw_ba_up1,sw_ba_up2,sw_ba_up3,   $
;         sw_ba_up11,sw_ba_up12,               $
;         sw_ba_up21,sw_ba_up22,               $
;         sw_ba_up31,sw_ba_up32, sw_ba_up31i,              $
;         sw_bu_gwin,sw_bu_gres,sw_bu_wwou,         $
;         sw_bu_wpara_in,sw_bu_para_in,              $
;         sw_bu_winf,sw_bu_resf,sw_bu_wouf,         $
;         sw_bu_winb,sw_bu_resb,sw_bu_woub,         $
;         sw_la_win,sw_la_res,sw_la_wou,          $
;         sw_ba_mid,sw_ba_mid2,sw_ba_mid3,        $
;         sw_ba_mid11,sw_ba_mid12,sw_ba_mid13,     $
;         sw_ba_mid14,sw_ba_mid15,           $
;         sw_la_xmin,sw_la_xmax,sw_la_ymin,         $
;         sw_la_ymax,sw_la_wsub,sw_tx_xmin,         $
;         sw_tx_xmax,sw_tx_ymin,sw_tx_ymax,         $
;         sw_tx_wsub,sw_bu_plot,sw_bu_plre,         $
;         sw_pl_area,sw_ba_plre,sw_bu_norm,         $
;         sw_ba_bot,sw_bu_start,sw_bu_cancel,       $
;         sw_la_nlor,sw_la_ngau,sw_tx_resmin,       $
;         sw_tx_nlor,sw_tx_ngau,sw_tx_resmax,       $
;         sw_tx_param,sw_tx_cstr,sw_tx_fwsub,       $
;         sw_tx_fxmin,sw_tx_fxmax,sw_tx_cycl,       $
;         sw_pl_areabot,sw_bu_gdos,sw_bu_fit,         $
;         sw_bu_done,sw_bu_try,sw_bu_fitall,          $
;         sw_bu_acquire,sw_tx_acquire,sw_bu_reverse,  $
;         sw_bu_fromlast,sw_bu_symsqw,sw_tx_temp
;
;
;common sc_loc, $
;         sw_in,sw_res,sw_ou,sw_result,          $ ; input Workspaces (input,res fun,output)
;         str_win,str_wres,str_wou,      $ ; input Worksp strings (input,res fun,output)
;         sx_in,sx_res,sx_ou,sx_result,          $ ; input X
;         sy_in,sy_res,sy_ou,sy_result,          $ ; input Y
;         se_in,se_res,se_ou,se_result,          $ ; input errors
;         s_pl_wid,num_spectrum,nb_spe_tot,         $
;         nb_channels,x_to_plot,w_wk,w_wkres,e_wk, $ ; variables de plot (x,S(q,w),Resf(Q,w) normee, error)
;         w_resc,e_resc,ce,                   $ ; res. func treated and its error for the convolution
;         xmin,xmax,ymin,ymax,s_pl_widbot
;
;
;common sc_flag, $
;         flag_pl_res,flag_norm,               $ ;
;         flag_lfit,flag_gfit,flag_lcfit,flag_gcfit,   $ ;
;         flag_lwfit,flag_gwfit,flag_lifit,flag_gifit,$ ;
;         flag_dfit,flag_dcfit,flag_difit,        $ ;
;         flag_bfit,flag_bcfit,flag_bifit,        $ ;
;         flag_lgifit,flag_pl_fit,flag_lect_rf, $
;      flag_pl_fit2,flag_all,flag_reverse,flag_fromlast, $
;      flag_sym
;
;common sc_fit,  $
;         num_lor,num_gau,rfmin,rfmax,          $;               $ ;
;         sf_index,lowerb,upperb,lowerbl,          $ ;
;         upperbl,sf_tie,sf_cstr_write,          $ ;
;         sf_values,ibmin,ibmintx,idmin,             $ ;
;         idmintx,igmin,igmintx,igminind,idminind, $ ;
;         ibminind,sf_param_to_fit,sf_new_values,   $ ;
;         sf_last_values,sf_xmin,sf_xmax,          $ ;
;         sf_sp_num_min,sf_sp_num_max,sf_iter_num, $ ;
;         sf_w_fitted,sf_e_values,sf_w_exp_fit,sf_w_exp_fit_rel,     $ ;
;         sf_sub_function,name_func,sf_new,sf_e,   $ ;
;         sf_functions_labels,sf_errors_labels,     $ ;
;         sf_functions_labels2,sf_errors_labels2,     $ ;
;         chisqr,betatemp
;
;common sc_convs,$
;         aalpha,abeta,lmin,lmax,cel,fwhm_rf,center_rf
;
;common sc_lorgau,$
;         n_l,n_g,ispecc,falpha,fbeta,w_resc2,w_resc2_interp
;
;common sc_userfunction,$
;            User_Modl,nb_subfunctions
;
;common sc_new, sw_bu_supsp,sw_bu_infsp, $
;            flag_fit2D,                        $ ;new common in SC_FLAG
;            n_param_1D,n_param_2D,                      $ ;new common in SC_FIT
;            sw_bu_fit2D,sw_bu_giveall,sw_dl_engine,str_engine, $;        ;new common in SC_WID
;            degoffreed, $
;            sw_bu_win,sw_bu_res,sw_bu_wou,sw_bu_cur,sw_bu_wcur,sw_la_wcur,sw_bu_wsf,sw_la_wsf,sw_dl_wsf, $
;            sf_sub_function_name,sw_la_wall,sw_bu_wall,sw_tx_give


end

;** **************************************************************
function sph_beselj,xinput,n
;+
;** **************************************************************
;** Calculate the spherical bessel functions jl
;-

jl=0.
;if n+0.5 gt 20 then begin
;Print,'** WARNING !! order of Bessel function too large. It should be <=20'
;return,jl
;endif
jl=sqrt(0.5*!PI/xinput)*BESELJ(xinput,n+0.5)
return, jl
end

;** **************************************************************
function bose_qens,xinput
;+
;** **************************************************************
;** Calculate the bose factor
;-

bosex=xinput/(1-exp(-1.*xinput*11.6045/300))
bosex[where(abs(xinput) le 1e-5)]=1.
return, bosex
end

;** **************************************************************
function n_lorentz,xinput,p1,p2
;+
;** **************************************************************
;** p1=position p2=FWHM p3=amplitude (intensity is !PI*P2*P3/2)
;** lorentz(x)=P3*P2^2/(4*(x-P1)^2+P2^2)
;-

; ** Compute the function first
qlor=(p2/!PI)/((xinput-p1)^2+p2^2)
if abs(p2) le 1e-20 then print,'Probleme p2'
return,[qlor]
end

;** **************************************************************
function dho,xinput,p1,p2,p3
;+
;** **************************************************************
;** p1=position p2=FWHM p3=amplitude
;** dho(x)=P3*P2*x/((x^2-P1^2)^2+P2^2)
;-

; ** Compute the function first
;qdho=p3*abs(xinput)*p2/((xinput^2-p1^2)^2+p2^2)
qdho=p3*p1^2*p2/((xinput^2-p1^2)^2+xinput^2*p2^2)
if abs(p2) le 1e-20 then print,'Probleme p2'
return,[qdho]
end

;** **************************************************************
function sexp,xinput,p1,p2,p3,p4
;+
;** **************************************************************
;** p1=position p2=beta p3=tau p4=amplitude
;** dho(x)=P4*FT(exp(-(x/P3)^p2)) centered at p1
;-

; ** Compute the function first
if (p2 le 0.5 or p2 ge 1) then begin
print,'** WARNING Stretched expo : Beta should be >0.5 and <1 !! Put it to 0.75'
p2=0.75
endif
if (p3 le 0.001) then begin
print,' ** WARNING Tau too small ... Put it to 1.'
p3=1.
endif
tmax=(5*ALOG(10))^(1/p2) & qsexp=xinput*0.
wmax=MAX(ABS(xinput)) & if n_elements(wmax) gt 1 then wmax=wmax[0] & dt=2*!PI/wmax/10
nbrt=fix(tmax/dt) & nbrt= nbrt > 200 & nbrt=nbrt < 1000
t=p3*findgen(nbrt)*tmax/nbrt
print,'P1=',P1,' P2=',P2,' P3=',P3,' P4=',P4
print,'**************************************************************************************'
print,'** calculating the stretched exponential using ',nbrt,' values of t for integration **'
print,'**************************************************************************************'
for iw=0,n_elements(xinput)-1 do begin
qsexp[iw]=INT_TABULATED(t,cos(xinput[iw]*t)*exp(-(t/p3)^p2),/DOUBLE)
end
; treating the bizarre things about sexp
index=where(abs(qsexp) le (max(abs(qsexp))/250))
if index[0] ne -1 then begin
xx=xinput[index] & xmin=min(ABS(xx))
ind=where(abs(xinput) le xmin,complement=nind)
if ind[0] ne -1 then qsexp[nind]=0.
endif
fin:return,[qsexp]
end

;** **************************************************************
function qlorentz,xinput,p1,p2,p3
;+
;** **************************************************************
;** p1=position p2=FWHM p3=amplitude (intensity is !PI*P2*P3/2)
;** lorentz(x)=P3*P2^2/(4*(x-P1)^2+P2^2)
;-

; ** Compute the function first
qlor=(p3*p2^2)/(4*(xinput-p1)^2+p2^2)
if abs(p2) le 1e-20 then print,'Probleme p2'
return,[qlor]
end

;** **************************************************************
function qgauss,xinput,p1,p2,p3
;+
;** **************************************************************
;** p1=position p2=FWHM p3=amplitude (intensity is P3*P2P*SQRT(!PI))
;** gauss(x)=(P3/(p2'*sqrt(PI))*EXP(-(x-P1)^2/(P2')^2)
;** in that case, P2'=P2/(2sqrt(Ln(2))) is not the FWHM
;-

; ** Compute the function first
p2p=p2/(2.*sqrt(alog(2.)))
if abs(p2p) le 1e-20 then print,'Probleme p2p'
qgau=p3*EXP(-((xinput-p1)^2)/(p2p^2))
return,[qgau]
end


;** **************************************************************
function myfunc,P,XVAL=X,YVAL=Y,ERRVAL=E
;+
;** **************************************************************
;** P=sf_new_values : values of the fit parameters
;** X=x values constrainted by the fit range ... Y=corresponding Y values
;** and E=corresponding errors
;** model=returned array containing the model values
;-

common sc_fit
common sc_wid
common sc_loc
common sc_flag
common sc_userfunction
common sc_new
FORWARD_FUNCTION funlg_to_fit,fun_isorot,fun_rwtrans
FORWARD_FUNCTION fun_rotctran,fun_unirot,fun_gumbel,fun_userdefined;,fun_daniel

evv=widget_event(sw_bu_fit,/nowait,bad_id=ii); Check for Stop button
if evv.id eq sw_bu_fit then begin
maystop=1
PRINT,'STOP ENCOUNTERED ... stopping'
widget_control,bad_id=i,sw_bu_fit,set_value='   FIT !   '
retall
endif
widget_control,/hourglass
ispe_min=num_spectrum & ispe_max=num_spectrum
SP=P
if (flag_fit2D eq -1) then begin
ispe_min=1 & ispe_max=nb_spe_tot
SP=reform(P,n_param_1D,nb_spe_tot)
endif
model=0.
for ispe=ispe_min,ispe_max do begin
SPP=P & if (flag_fit2D eq -1) then SPP=SP[*,ispe-1]
case name_func of
    'LOR_GAU':model_lg=funlg_to_fit(X,SPP,ispe)
    'ISO_ROT':model_lg=fun_isorot(X,SPP,ispe)
    'UNI_ROT':model_lg=fun_unirot(X,SPP,ispe)
    'ISO_TRANS':model_lg=fun_rwtrans(X,SPP,ispe)
    'ROT_TRAN':model_lg=fun_rotctran(X,SPP,ispe)
    'GUMBEL':model_lg=fun_gumbel(X,SPP,ispe)
    'USER_FUNC':model_lg=fun_userdefined(X,SPP,ispe)
    'DANIEL':model_lg=CALL_FUNCTION('fun_daniel',X,SPP,ispe)
else:stop
endcase
model=[model,model_lg]
endfor
model=model[1:n_elements(model)-1]
return,(Y-model)/E

end
;** **************************************************************
pro prepare_2_fit,X,Y,E,parinfo,sstat
;+
;** **************************************************************
;** Called when the FIT button is pressed
;** Call back the new values of the fit parameters as well as
;** other necessary features
;-

common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_lorgau
common sc_userfunction
common sc_new

sstat=1
ON_IOERROR,iok4
; ** First we check the input for the fit
; ** Now deal with the string to float or integer transformation (see "plot it")
widget_control,sw_tx_fxmin,get_value=s & sf_xmin=float(strcompress(s,/remove_all)) & sf_xmin=sf_xmin[0] ;x min fit range
widget_control,sw_tx_fxmax,get_value=s & sf_xmax=float(strcompress(s,/remove_all)) & sf_xmax=sf_xmax[0] ;x max fit range
;widget_control,sw_tx_cycl,get_value=s & sf_iter_num=fix(strcompress(s,/remove_all)) & sf_iter_num=sf_iter_num(0) ;number of iteration maximum to perform
sf_iter_num=100 ;number of iteration maximum to perform

n_par2fit=0. & degoffreed=1.
; ** parinfo is the structure containing the indication on how the fit has to be performed
case flag_fit2D of
-1:begin
parinfo=replicate({value:0.,fixed:0,parname:'P0',tied:'',limited:[0,0],limits:[0.,0.]},n_param_2D)
sf_last_values=sf_new_values    ; new values become last values
isppmin=1 & isppmax=nb_spe_tot
end
else:begin
parinfo=replicate({value:0.,fixed:0,parname:'P0',tied:'',limited:[0,0],limits:[0.,0.]},n_param_1D)
sf_last_values[num_spectrum-1,*]=sf_new_values[num_spectrum-1,*]    ; new values become last values
transl=0.
isppmin=num_spectrum & isppmax=num_spectrum
end
endcase

iprint=0

; ** in this part we record the new values and load them into the sf_new_values table
; ** this part depends on the function chosen by the user
; ** ... thus should be placed into the coresponding file ... well maybe not ...


for ispp=isppmin,isppmax do begin
if not flag_all then begin
for i=0,n_param_1D-1 do begin   ; sf_new_values must read the new value of the parameters
if not flag_fit2D then begin
widget_control,sw_tx_param[3*i],get_value=str_balg2 & flt_balg2=float(strcompress(str_balg2,/remove_all)) & flt_balg2=flt_balg2[0]
sf_new_values[ispp-1,i]=flt_balg2
endif
; ** ************************************************************************
; ** now deal with the constraints ...
if flag_fit2D then begin
str_balg2=sf_cstr_write[ispp-1,i] & str_l=strlen(str_balg2)
transl=(ispp-1)*n_param_1D
endif else begin
widget_control,sw_tx_cstr[i],get_value=str_balg2 ; on lit ce qu'il y a dans le dernier champ du parametre i
str_balg2=strcompress(str_balg2[0],/remove_all) & str_l=strlen(str_balg2)
sf_cstr_write[ispp-1,i]=str_balg2
transl=0.
endelse
CASE str_l of
1:begin ;case FIXED
flt_balg2=fix(strcompress(str_balg2,/remove_all)) & flt_balg2=flt_balg2[0]
sf_index[ispp-1,i]=flt_balg2
lowerb[ispp-1,i]=0 & upperb[ispp-1,i]=0
lowerbl[ispp-1,i]=0. & upperbl[ispp-1,i]=0.
sf_tie[ispp-1,i]=''
end
ELSE:begin ;case TIED OR LIMITS
str_first=strmid(str_balg2,0,1) ;extract the first character of the expression str_balg2
IF ((str_first eq '2') and (flag_fit2D)) then begin
sf_new_values[ispp-1,i]=sf_new_values[0,i]
;sf_index(ispp-1,i)=flt_balg2
;str_fit2D(num_spectrum-1,i)='P'+strcompress(string(i),/remove_all)
sf_tie[1:nb_spe_tot-1,i]='P['+strcompress(string(i),/remove_all)+']'
goto,suite
ENDIF ELSE IF ((str_first eq '2') and (not flag_fit2D)) then goto, suite
CASE (str_first eq '>') or (str_first eq '<') of
0:begin ;0=false => CASE TIED
sf_index[ispp-1,i]=1
lowerb[ispp-1,i]=0 & upperb[ispp-1,i]=0
lowerbl[ispp-1,i]=0. & upperbl[ispp-1,i]=0.
;now deal with parameters' labels ... might be dangerous ...
iloopi=1
new_str=strupcase(str_balg2)
  WHILE iloopi ne 0 DO BEGIN
  iloopi=0
  for itie=0,n_param_1D-1 do begin
  nstr_l=strlen(new_str)
  ;print,new_str,sf_functions_labels2(itie)
  pos=stregex(new_str,sf_functions_labels2[itie],length=len)
  case pos of
  0:begin
  new_str='P['+strcompress(string(transl+itie),/remove_all)+']'+strmid(new_str,len,nstr_l-len)
  iloopi=1
  end
  -1:
  else:begin
  new_str=strmid(new_str,0,pos)+'P['+strcompress(string(transl+itie),/remove_all)+']'+strmid(new_str,pos+len,nstr_l-pos-len)
  iloopi=1
  end
  endcase
  endfor
  ENDWHILE
  sf_tie[ispp-1,i]=new_str
end
1:begin;1=true => CASE LIMITS
pos=strpos(str_balg2,';')
sf_index[ispp-1,i]=1
sf_tie[ispp-1,i]=''
if pos eq -1 then begin
  if (str_first eq '>') then begin
  low=1 & up=0
  A=float(strcompress(strmid(str_balg2,1,str_l-1),/remove_all)) & A=A[0] & B=0.
  endif else begin
  low=0 & up=1
  B=float(strcompress(strmid(str_balg2,1,str_l-1),/remove_all)) & B=B[0] & A=0.
  endelse
endif else begin
  low=1 & up=1
  if (str_first eq '>') then begin
    A=float(strcompress(strmid(str_balg2,1,pos-1),/remove_all)) & A=A[0]
    B=float(strcompress(strmid(str_balg2,pos+2,str_l-1),/remove_all)) & B=B[0]
  endif else begin
    B=float(strcompress(strmid(str_balg2,1,pos-1),/remove_all)) & B=B[0]
    A=float(strcompress(strmid(str_balg2,pos+2,str_l-1),/remove_all)) & A=A[0]
  endelse
endelse
lowerbl[ispp-1,i]=A & upperbl[ispp-1,i]=B
lowerb[ispp-1,i]=low & upperb[ispp-1,i]=up
end
ENDCASE
end
ENDCASE
suite:widget_control,sw_tx_param[3*i],sensitive=abs(sf_index[ispp-1,i])
endfor
endif else begin
if flag_reverse then iadprev=1 else iadprev=-1
num_prev=num_spectrum+1*iadprev
if not((num_prev lt 1 or num_prev gt nb_spe_tot) or (not flag_fromlast)) then begin
sf_new_values[num_spectrum-1,*]=sf_new_values[num_prev-1,*]
sf_index[num_spectrum-1,*]=sf_index[num_prev-1,*]
lowerb[num_spectrum-1,*]=lowerb[num_prev-1,*] & lowerbl[num_spectrum-1,*]=lowerbl[num_prev-1,*]
upperb[num_spectrum-1,*]=upperb[num_prev-1,*] & upperbl[num_spectrum-1,*]=upperbl[num_prev-1,*]
sf_tie[num_spectrum-1,*]=sf_tie[num_prev-1,*]
sf_cstr_write[num_spectrum-1,*]=sf_cstr_write[num_prev-1,*]
endif
endelse
endfor ; boucle sur ispp
sstat=0
iok4:if sstat ne 0 then return
;
IF NOT flag_fit2D THEN BEGIN ; we change nothing for the moment ...

sf_param_to_fit=intarr(n_param_1D)
sf_param_to_fit[*]=abs(sf_index[num_spectrum-1,*]-1)    ; contains the indices of the parameters to be fitted
;
rin=where((sx_in le sf_xmax) and (sx_in ge sf_xmin),npt,complement=out)
falpha=rin[0] & fbeta=rin[npt-1]
if sf_xmax le sf_xmin then begin  
print, "PROBLEM: Fitting range, sf_xmin not lt sf_xmax !!"
stop
endif
if iprint ne 0 then begin
print,"falpha=",falpha,"fbeta=",fbeta
print,"rin=",rin
print,"out=",out
endif
w_resc2=w_resc[falpha:fbeta,*]
X=sx_in[rin] & Y=sw_in[rin,num_spectrum-1]
E=se_in[rin,num_spectrum-1] & rrr=where(E le 0.) & if rrr[0] ne -1 then E[rrr]=100000
sf_w_fitted[*,num_spectrum-1]=0.
;ispecc=sf_sp_num_min;-1
sf_new=fltarr(n_param_1D) & sf_new[*]=sf_new_values[num_spectrum-1,*]
sf_e=fltarr(n_param_1D) & sf_e[*]=sf_e_values[num_spectrum-1,*]
parinfo.value=sf_new
parinfo.fixed=sf_param_to_fit
parinfo.parname=sf_functions_labels2
parinfo[*].limited[0]=transpose(lowerb[num_spectrum-1,*])
parinfo[*].limited[1]=transpose(upperb[num_spectrum-1,*])
parinfo[*].limits[0]=transpose(lowerbl[num_spectrum-1,*])
parinfo[*].limits[1]=transpose(upperbl[num_spectrum-1,*])
parinfo[*].tied=transpose(sf_tie[num_spectrum-1,*])
xxx=where(sf_param_to_fit eq 0,n_par2fit)
degoffreed=npt-n_par2fit ; to calculate the reduced chi square
print,'npt=',npt,'n_par2fit=',n_par2fit
print,'degree of freedom=',degoffreed

ENDIF ELSE BEGIN ; FIT 2D
print, " ***** 2D surface (Q,w) fitting *****"
sf_param_to_fit=reform(transpose(abs(sf_index-1)),n_param_2D)
;
rin=where((sx_in le sf_xmax) and (sx_in ge sf_xmin),npt,complement=out)
falpha=rin[0] & fbeta=rin[npt-1]
if sf_xmax le sf_xmin then begin  
print, "PROBLEM: Fitting range, sf_xmin not lt sf_xmax !!"
stop
endif
if iprint ne 0 then begin
print,"falpha=",falpha,"fbeta=",fbeta
print,"rin=",rin
print,"out=",out
endif
w_resc2=w_resc[falpha:fbeta,*]
X=sx_in[rin] & Y=reform(sw_in[rin,*],nb_spe_tot*npt)
E=reform(se_in[rin,*],nb_spe_tot*npt) & rrr=where(E le 0.) & if rrr[0] ne -1 then E[rrr]=100000
sf_w_fitted[*,*]=0.
;ispecc=sf_sp_num_min;-1
sf_new=reform(transpose(sf_new_values),n_param_2D)
sf_e=reform(transpose(sf_e_values),n_param_2D)
parinfo.value=sf_new
parinfo.fixed=sf_param_to_fit
sff=sf_functions_labels2
for i=0,nb_spe_tot-2 do sff=[sff,sf_functions_labels2]
parinfo.parname=sff
parinfo[*].limited[0]=reform(transpose(lowerb),n_param_2D)
parinfo[*].limited[1]=reform(transpose(upperb),n_param_2D)
parinfo[*].limits[0]=reform(transpose(lowerbl),n_param_2D)
parinfo[*].limits[1]=reform(transpose(upperbl),n_param_2D)
parinfo[*].tied=reform(transpose(sf_tie),n_param_2D)       ;ATTENTION PROBLEME DE TIE QUAND PASSAGe A 2D
xxx=where(sf_param_to_fit eq 0,n_par2fit)
degoffreed=nb_spe_tot*npt-n_par2fit ; to calculate the reduced chi square
print,'nb_spe_tot*npt=',nb_spe_tot*npt,'n_par2fit=',n_par2fit
print,'degree of freedom=',degoffreed
ENDELSE
end

;** **************************************************************
pro change_spectrum
;+
;** **************************************************************
;** qens_fit stuff
;-

common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_lorgau
common sc_userfunction
common sc_new
                valid=0
                ON_IOERROR,nok2
                widget_control,sw_tx_wsub,get_value=str_wsub & str_wsub=strlowcase(strcompress(str_wsub,/remove_all))
                num_spectrum=1 & str_wsub=str_wsub[0]
                num_spectrum=fix(str_wsub) & num_spectrum=num_spectrum[0]
                valid=1
                if ((num_spectrum le 0) or (num_spectrum gt nb_spe_tot)) then begin
                print,'ERROR num_spectrum out of range ... put it to 1'
                num_spectrum=1
                widget_control,sw_tx_wsub,set_value=' 1'
                endif
nok2:           if not valid then begin
                Rrrr=DIALOG_MESSAGE('PROBLEM:Bad input type for value of spectrum to treat ... put it to 1',/ERROR)
                widget_control,sw_tx_wsub,set_value=' 1'
                num_spectrum=1
                endif
             plot_it
               if flag_pl_fit2 then begin
                ; Change the parameters text box values
                for i=0,n_param_1D-1 do begin ; sf_new_values must print the new value of the parameters
                str_balg2=string(sf_new_values[num_spectrum-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                widget_control,sw_tx_param[3*i],set_value=str_balg2,sensitive=abs(sf_index[num_spectrum-1,i])
                str_balg2=string(sf_last_values[num_spectrum-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                widget_control,sw_tx_param[3*i+1],set_value=str_balg2
                str_balg2=string(sf_e_values[num_spectrum-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                widget_control,sw_tx_param[3*i+2],set_value=str_balg2
                widget_control,sw_tx_cstr[i],set_value=sf_cstr_write[num_spectrum-1,i]
                endfor
              endif
end

;** **************************************************************
pro qens_fit_event, Event
;+
;** **************************************************************
;** Called when a qens_fit event is generated
;-

@lamp.cbk
common c_trap, trap_x1,trap_x2,trap_y1,trap_y2,trap_ws, trap_current
common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_lorgau
common sc_convs
common sc_userfunction
common sc_new
forward_function funlg_to_fit
forward_function fun_isorot
forward_function fun_rwtrans
forward_function fun_rotctran
forward_function fun_unirot
forward_function fun_gumbel
;forward_function fun_daniel
forward_function fun_userdefined
forward_function mp2cool
stat=0
catch,stat

if stat  ne 0  then begin
       catch,/cancel
       print,!error_state.msg
       return & endif

if  TAG_NAMES(Event,/structure_name) ne 'WIDGET_DROPLIST' then begin
    if (TAG_NAMES(Event,/structure_name) ne 'WIDGET_DRAW') then widget_control,Event.Id,/Hourglass
    widget_control, bad_id=i, Event.Id, geT_uvalue=Ev, Get_Value=value
no_plot=0
endif
if  TAG_NAMES(Event,/structure_name) eq 'WIDGET_DROPLIST' then begin
if Event.Id eq sw_dl_engine then begin
widget_control,bad_id=i,sw_dl_engine,get_uvalue=Ev
case Event.Index of
0:str_engine='MPFIT'
1:str_engine='COOLFIT'
2:
3:
endcase
endif
if Event.Id eq sw_la_win then begin
widget_control,bad_id=i,sw_la_win,get_uvalue=Ev
idp=Event.Index+1
str_win=string(idp) & str_win=strcompress(str_win[0],/remove_all)
endif
if Event.Id eq sw_la_res then begin
widget_control,bad_id=i,sw_la_res,get_uvalue=Ev
idp=Event.Index+1
str_wres=string(idp) & str_wres=strcompress(str_wres[0],/remove_all)
endif
if Event.Id eq sw_la_wou then begin
widget_control,bad_id=i,sw_la_wou,get_uvalue=Ev
idp=Event.Index+1
str_wou=string(idp) & str_wou=strcompress(str_wou[0],/remove_all)
endif
if Event.Id eq sw_la_wcur then begin
widget_control,bad_id=i,sw_la_wcur,get_uvalue=Ev
idp=Event.Index+1
str_wcur=string(idp) & str_wcur=strcompress(str_wcur[0],/remove_all)
endif
endif

if n_elements(Ev) eq 0 then begin
Ev=fltarr(1) & Ev[0]=999
endif

case Ev[0] of
    0:case Ev[1] of                     ;Menu Event
          0:case Ev[2] of
                0:widget_control,event.top,/DESTROY ;Exit Event
                1:begin                                                     ;Save Curves Event
                valid=0
                ON_IOERROR,nok5
                ts=transpose(sf_sub_function)
                sz_ts=size(ts) & ;sztsx
                f2print=reform(sf_w_fitted,1,nb_channels,nb_spe_tot)
                dat2print=reform(sw_in,1,nb_channels,nb_spe_tot)
                err2print=reform(se_in,1,nb_channels,nb_spe_tot)
                x2print=sx_in & for i=0,nb_spe_tot-2 do x2print=[x2print,sx_in] & x2print=reform(x2print,1,nb_channels,nb_spe_tot)
                w2print=[x2print,dat2print,err2print,f2print,ts]
                nsub=size(w2print) & nsub2=nsub[1] & iformat=nsub2
                sformat=strcompress(string(iformat),/remove_all)
                sformat='('+sformat+'(G15.8,1X))'
                strdate=systime(0)
                  case name_func of
                    'LOR_GAU':str2add='Data were fitted using a set of independent Lorentzian and Gaussian functions'
                    'ISO_ROT':str2add='Data were fitted using an isotropic rotation model'
                    'UNI_ROT':str2add='Data were fitted using an uniaxial rotation model'
                    'ISO_TRANS':str2add='Data were fitted using an random walk translation model'
                    'ROT_TRAN':str2add='Data were fitted using an isotropic rotation convoluted to a rw-translation model'
                    'GUMBEL':str2add='Data were fitted using a GUMBEL function'
                    'USER_FUNC':str2add='Data were fitted using a User Defined function'
                  else: str2add='Model non existent'
                  endcase
                save_file=dialog_pickfile(file='fit_curves.sav',filter='*.sav',title='Save in File:')
                openw,unit,save_file,/get_lun
                printf,unit,'File printed on '+strdate
                printf,unit,str2add
                printf,unit,'--------------------------------------------------',format='(a50)'
                for isp=0,nb_spe_tot-1 do begin
                printf,unit,'# Spectrum# ',isp+1,'Angle/Q=',sy_in[isp]
                printf,unit,w2print[*,*,isp],format=sformat
                printf,unit,'##################################################',format='(a50)'
                endfor
                free_lun,unit
                valid=1
nok5:           if not valid then begin
                print,'Problem in Output.... check dimension:'
                help,ts
                help,f2print
                help,dat2print
                help,err2print
                help,x2print
                help,w2print
                return
                endif
                end
                2:begin
                valid=0
                on_ioerror,nok6
                save_file=dialog_pickfile(file='fit_param.sav',filter='*.sav',title='Save in File:')
                openw,unit,save_file,/get_lun
                openw,unit2,save_file+'.qpp',/get_lun
                strdate=systime(0)
                  case name_func of
                    'LOR_GAU':str2add='Data were fitted using a set of independent Lorentzian and Gaussian functions'
                    'ISO_ROT':str2add='Data were fitted using an isotropic rotation model'
                    'UNI_ROT':str2add='Data were fitted using an uniaxial rotation model'
                    'ISO_TRANS':str2add='Data were fitted using an random walk translation model'
                    'ROT_TRAN':str2add='Data were fitted using an isotropic rotation convoluted to a rw-translation model'
                    'GUMBEL':str2add='Data were fitted using a GUMBEL function'
                    'USER_FUNC':str2add='Data were fitted using a User Defined function'
                    else: str2add='Model non existent'
                  endcase
                printf,unit,'File printed on '+strdate &  ;printf,unit2,'# '+'File printed on '+strdate
                printf,unit,str2add       ;& printf,unit2,'# '+name_func
                printf,unit,'--------------------------------------------------',format='(a50)'
;                printf,unit2,'# '+'--------------------------------------------------'
                for isp=0,nb_spe_tot-1 do begin
                printf,unit,'# Spectrum#',isp+1,'Angle/Q=',sy_in[isp],'Reduced CHISQR=',chisqr[isp],format='(a,1x,i4,1x,a,1x,f7.3,3x,a,1x,f9.3)'
                for isub=0,n_elements(sf_functions_labels)-1 do begin
                printf,unit,sf_functions_labels[isub],sf_new_values[isp,isub],sf_errors_labels[isub],sf_e_values[isp,isub]$
                ,format='(a26,3x,G15.8,3x,a,3x,G15.8)'
                endfor
                printf,unit,'##################################################',format='(a50)'
                endfor
                sformat=strcompress(string(2*n_elements(sf_functions_labels)+1),/remove_all) & sformat='('+sformat+'(G15.8,1X))'
                sformat2=strcompress(string(2*n_elements(sf_functions_labels2)+1),/remove_all) & sformat2='('+sformat2+'(A,1X))'
                printf,unit2,' #2theta/Q',sf_functions_labels2,sf_errors_labels2,format=sformat2
                if n_elements(sy_in) ge 2 then begin
                printf,unit2,[transpose(sy_in),transpose(sf_new_values),transpose(sf_e_values)],format=sformat
                endif else begin
                printf,unit2,[sy_in,transpose(sf_new_values),transpose(sf_e_values)],format=sformat
                endelse
                free_lun,unit
                free_lun,unit2
                valid=1
nok6:           if not valid then begin
                print,'Problem in output... check the dimension of the vectors...'
                return
                endif
                end
            endcase
          1:case Ev[2] of
                0:begin                               ; Lor-Gauss fit
                RESOLVE_ROUTINE, "fit_basegl",/COMPILE_FULL_FILE
                fit_basegl
                end
                1:begin                               ; Isotropic rotation
                RESOLVE_ROUTINE, "fit_ba_isorot",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
                fit_ba_isorot
                end
                2:begin                               ; Uniaxial rotation
                RESOLVE_ROUTINE, "fit_ba_unirot",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
                fit_ba_unirot
                end                                   ; Uniaxial rotation
                3:begin                               ; RW translation
                RESOLVE_ROUTINE, "fit_ba_rwtrans",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
                fit_ba_rwtrans
                end
                4:begin                               ; RW translation
                RESOLVE_ROUTINE, "fit_ba_rotctran",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
                fit_ba_rotctran
                end
          5:begin                               ; Gumbel function
                RESOLVE_ROUTINE, "fit_ba_gumbel",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
                fit_ba_gumbel
                end
          6:begin
          resolve_routine,'qens_user_defined',/COMPILE_FULL_FILE  ; User defined function
                treat_resf,sstat
             if sstat ne 0 then return
                qens_user_defined, sw_ba_leader, lamp_b1
                end
                                         ; Gumbel function
          7:begin
 ;         RESOLVE_ROUTINE, "fit_ba_daniel",/COMPILE_FULL_FILE
                treat_resf,sstat
             if sstat ne 0 then return
           call_procedure,"fit_ba_daniel"
                end
          8:print,'Do nothing at the moment'
            endcase
      endcase
    1:begin                    ; First Line Event
      case Ev[1] of
          1:begin ; Get data event
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_win,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_win,set_value='W'+str_test
         endelse   
         sw_in=0 & sx_in=0 & sy_in=0 & se_in=0
         iii=EXECUTE("w_in=w"+str_test)
         iii=EXECUTE("x_in=x"+str_test)
         iii=EXECUTE("y_in=y"+str_test)
         iii=EXECUTE("e_in=e"+str_test)
         sw_in=w_in & sx_in=x_in & sy_in=y_in & se_in=e_in
         sz_win=size(sw_in)
         case n_elements(sz_win) of
         4:begin
         nb_spe_tot=1 & nb_channels=sz_win[1]
         end
         5:begin
         nb_spe_tot=sz_win[2] & nb_channels=sz_win[1]
         end
         else:begin
         Print,"Problem in array dimension ... quitting"
         return
         end
         endcase
         ispec=0

         ;**** Check the constant bining of the energy grid
         ;**** Reb if necessary
         nb_channels=n_elements(sx_in)
         rdist=sx_in[1:nb_channels-1]-sx_in[0:nb_channels-2]
         rdistav=mean(rdist) & rdist_test=abs(rdist-rdistav)
         indnot=where(rdist_test ge 1e-5,ncount)
         if ncount ne 0 then begin
         Rrrr=DIALOG_MESSAGE("Problem: the energy bining is not constant...rebining to "+strcompress(string(rdistav),/remove_all)+'... this will not change input wkspace',/error)
         XICUTE,'w61=reb(w'+str_test+',de='+strcompress(string(rdistav),/remove_all)+',/force)'
         print,'w61=reb(w'+str_test+',de='+strcompress(string(rdistav),/remove_all)+',/force)'
         ii=EXECUTE("w_in=w61")
         ii=EXECUTE("x_in=x61")
         ii=EXECUTE("y_in=y61")
         ii=EXECUTE("e_in=e61")
         sw_in=w_in & sx_in=x_in & sy_in=y_in & se_in=e_in
         endif

         ;** 1/ Test if the 0 meV point is included into the energy range. IT SHOULD BE !!
         indzero=where(abs(sx_in) le 1.e-6,nzero)
         if nzero ne 1 then begin
         Rrrr=DIALOG_MESSAGE("PROBLEM: either the 0 meV point is not included into the energy range, or there are many points le 1.e-6 meV! Number of 0 meV points found:"+string(nzero),/error)
         Print,"PROBLEM: either the 0 meV point is not included into the energy range, or there are many points le 1.e-6 meV! Number of 0 meV points found:"+string(nzero)
         Print,"Adding the 0.meV/mmeV point to the energy grid. A rebinning of the data will be done. Check the effect of this operation."
         sx_in=[sx_in,0.] & ind_sorted=SORT(sx_in) & sx_in=sx_in[ind_sorted] & ind0=where(sx_in eq 0.,nzero)
         if nzero gt 1 then begin
         Rrrr=DIALOG_MESSAGE("PROBLEM: Too many 0. s...:"+string(nzero)+"... quitting"+string(nzero),/error)
         return
         endif
         sw_in2=sw_in & se_in2=se_in
         sw_in=fltarr(nb_channels,nb_spe_tot) & se_in=sw_in
         sw_in[0:nb_channels-1,*]=sw_in2[0:nb_channels-1,*] & se_in[0:nb_channels-1,*]=se_in2[0:nb_channels-1,*]
         sw_in=sw_in[ind_sorted,*] & se_in=se_in[ind_sorted,*]
         for is=0,nb_spe_tot-1 do begin
         moy=0.5*(sw_in[ind0-1]+sw_in[ind0+1]) & sw_in[ind0,is]=moy
         moy=0.5*(se_in[ind0-1]+se_in[ind0+1]) & se_in[ind0,is]=moy
         endfor
         endif

         sw_result=sw_in & sx_result=sx_in & sy_result=sy_in & se_result=se_in
         sz_win=size(sw_in)
         case n_elements(sz_win) of
         4:begin
         nb_spe_tot=1 & nb_channels=sz_win[1]
         end
         5:begin
         nb_spe_tot=sz_win[2] & nb_channels=sz_win[1]
         end
         else:begin
         Print,"Problem in array dimension ... quitting"
         return
         end
         endcase
         if flag_lect_rf then begin
         sz_wres=size(sw_res) & sz_win=size(sw_in)
         if n_elements(sz_wres) ne n_elements(sz_win) then begin
         errmess=DIALOG_MESSAGE('Dimensions of W_in and Res. Funct. do not match ... continue')
         print,n_elements(sz_wres),n_elements(sz_win)
         goto,outbcle
         endif
         if (n_elements(sz_wres) gt 4) and (sz_wres[2] ne nb_spe_tot) then begin
         errmess=DIALOG_MESSAGE('Dimensions of W_in and Res. Funct. do not match ... continue')
         goto,outbcle
         endif
         if sz_wres[1] ne nb_channels then begin
         errmess=DIALOG_MESSAGE('Dimensions of W_in and Res. Funct. do not match ... continue')
         goto,outbcle
         endif
         endif
outbcle: ; CHECK THAT THE 0 meV POINT IS INCLUDED INTO SX_IN AND IS UNIQUE !!!
         widget_control,sw_tx_fxmin,set_value=strcompress(string(min(sx_in)),/remove_all)
         widget_control,sw_tx_fxmax,set_value=strcompress(string(max(sx_in)),/remove_all)
         sf_w_fitted=fltarr(nb_channels,nb_spe_tot)   ; the final fitted array has the same dimensions as the input array
         sf_w_exp_fit=sf_w_fitted           ; the difference in between the experimental and the fitted curves
         sf_w_exp_fit_rel=sf_w_exp_fit
         widget_control,sw_bu_gres,sensitive=1
           end
         2:begin    ; Get RESF Event
         If (Ev[2] eq 999) then begin ; make a gaussian RF
         print,'******************'
         print,'Make a gaussian RF'
         print,'******************'
         widget_control,sw_la_res,set_value='FWHM='
         inputfwhm
         endif else begin
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_res,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_res,set_value='W'+str_test
         endelse   
         sw_res=0 & sx_res=0 & sy_res=0 & se_res=0
         iii=execute("w_in=w"+str_test)
         iii=execute("x_in=x"+str_test) ;& sx_res=sx_res(*,0)
         iii=execute("y_in=y"+str_test)
         iii=execute("e_in=e"+str_test)
         sw_res=w_in & sx_res=x_in & sy_res=y_in & se_res=e_in
         sz_wres=size(w_in) & sz_eres=size(e_in)
         sx_resold=sx_res & sw_resold=sw_res & se_resold=se_res
         if sz_eres[0] eq 0 then se_res=SQRT(sw_res)
         ; ** Check that data and res.funct has the same energy and theta grid...
         ; ** *******************************************************************
         case n_elements(sz_wres) of
         4:begin
         nb_spe_res=1 & nb_channels_res=sz_wres[1]
         end
         5:begin
         nb_spe_res=sz_wres[2] & nb_channels_res=sz_wres[1]
         end
         else:begin
         errmess=DIALOG_MESSAGE("Problem in array dimension ... quitting")
         Print,"Problem in array dimension ... quitting"
         Return
         end
         endcase
         if (nb_spe_res ne nb_spe_tot) then begin
         errmess=DIALOG_MESSAGE("Res. Funct. dimensions do not match W_in dimensions ... see main IDL Windows for details. Quiting",/ERROR)
         Print,"Problem: Res. Funct. dimensions do not match W_in dimensions ... quiting"
         Print,"W_in. dimensions:x=",nb_channels,"; y=",nb_spe_tot
         Print,"Res. Funct. dimensions:x=",nb_channels_res,"; y=",nb_spe_res
         Return
         endif
         if (nb_channels_res ne nb_channels) then begin
         errmess=DIALOG_MESSAGE("Res. Funct. energy dimensions do not match W_in dimensions ... see main IDL Windows for details. Rebining to W_in energy grid")
         Print,"Problem: Res. Funct. dimensions do not match W_in dimensions ... Rebining to W_in energy grid ... that's fairly correct ..."
         Print,"W_in. dimensions:x=",nb_channels,"; y=",nb_spe_tot
         Print,"Res. Funct. dimensions:x=",nb_channels_res,"; y=",nb_spe_res
         sw_res=fltarr(nb_channels,nb_spe_tot) & sx_res=sx_in & se_res=sw_res
         for iis=0,nb_spe_tot-1 do begin
         sw_res[*,iis]=spline(sx_resold,sw_resold[*,iis],sx_res)
         se_res[*,iis]=spline(sx_resold,se_resold[*,iis],sx_res)
         endfor
         if (nb_spe_tot eq 1) then begin
         sw_res=REFORM(sw_res, /OVERWRITE)
         se_res=REFORM(se_res, /OVERWRITE)
         endif
         endif
         testx=abs(sx_in-sx_res) & testy=abs(sy_in-sy_res)
         indtestx=where(testx gt 1.E-4,ncountx) & indtesty=where(testy gt 1.E-4,ncounty)
         if ncounty ne 0 then begin
         Rrrr=DIALOG_MESSAGE('Data Q grid and Res. Func. Q grid are not the same ... quitting',/ERROR)
         print,'PROBLEM:Data Q grid and Res. Func. Q grid are not the same:'
         for i=0,ncounty-1 do print,'index#',indtesty[i],' yrf=',sy_res[indtesty[i]],' ydat=',sy_in[indtesty[i]]
         return
         endif
         if ncountx ne 0 then begin
         Rrrr=DIALOG_MESSAGE('Data energy grid and Res. Func. energy grid are not the same ... Rebining to W_in energy grid')
         print,'PROBLEM:Data energy grid and Res. Func. energy grid are not the same ... Rebining to W_in energy grid'
         ;for i=0,ncountx-1 do print,'index#',indtestx(i),' xrf=',sx_res(indtestx(i)),' xdat=',sx_in(indtestx(i))
         sw_res=fltarr(nb_channels,nb_spe_tot) & sx_res=sx_in & se_res=sw_res
         for iis=0,nb_spe_tot-1 do begin
         sw_res[*,iis]=spline(sx_resold,sw_resold[*,iis],sx_res)
         se_res[*,iis]=spline(sx_resold,se_resold[*,iis],sx_res)
         endfor
         if (nb_spe_tot eq 1) then begin
         sw_res=REFORM(sw_res, /OVERWRITE)
         se_res=REFORM(se_res, /OVERWRITE)
         endif
;         return
         endif
;     ************   SPLINE PROBLEM   ***************************
         ; Spline tends to go to � infinity where the
         ; spline x values exceed the x values of the original
         ; to-be-splined res function. Make the spline 0 outside
         ; of the resol x-range.
         xmax_resold=max(sx_resold)
         xmin_resold=min(sx_resold)
         maxpts=where(sx_res gt xmax_resold,n1) & minpts=where(sx_res lt xmin_resold,n2)
         if (n2 ne 0) then begin
         for iis=0,nb_spe_tot-1 do begin
         sw_res[minpts,iis] = 0.0 & se_res[minpts,iis] = 1000.0
         endfor
         endif
         if (n1 ne 0) then begin
         for iis=0,nb_spe_tot-1 do begin
         sw_res[maxpts,iis] = 0.0 & se_res[maxpts,iis] = 1000.0
         endfor
         endif
         widget_control,sw_bu_gres,sensitive=0         
         endelse
         
         ; ** Everything is OK concerning RF input ... we go forward
         flag_lect_rf=1
         widget_control,sw_bu_fitm,sensitive=1
;         widget_control,sw_bu_gres,sensitive=0
         widget_control,sw_bu_plot,sensitive=1
         widget_control,sw_bu_supsp,sensitive=1
         widget_control,sw_bu_infsp,sensitive=1
         end
    3:begin                 ; Write Parameters event
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_wou,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_wou,set_value='W'+str_test
         endelse   
         w_out=sf_new_values & x_out=sy_in & y_out=sy_in*0. & e_out=sf_e_values ;se_result
         iii=execute("w"+str_test+"=w_out")
         iii=execute("x"+str_test+"=x_out") ;& sx_res=sx_res(*,0)
         iii=execute("y"+str_test+"=y_out")
         iii=execute("e"+str_test+"=e_out")
      end
    33:begin               ; Import Parameters event
         w_test=fix(Ev[2]) 
         ; Writes the workspace name on the interface and retrieves its number in str_test :
         if (w_test eq 0) then begin 
         widget_control,sw_la_wou,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_wou,set_value='W'+str_test
         endelse   
         ; Retrieves the data in w_param_in and e_param_in
         w_param_in ='W'+str_test & e_param_in = 'E'+str_test
         iii=execute("w_param_in = " + w_param_in)
         iii=execute("e_param_in = " + e_param_in)
         ; Checks the size of w_in and of the sf_new_values array containing the parameters
         if ARRAY_EQUAL(size(w_param_in,/DIMENSIONS),size(sf_new_values,/DIMENSIONS)) then begin
            sf_new_values = w_param_in
            sf_e_values = e_param_in
            print, 'Imported parameters from W'+ str_test +': '+ w_tit[str_test]
         endif else begin
            temp = size(w_param_in,/DIMENSIONS)
            temp = (temp[1]-4)/3  
            if temp eq 1 then Bowdel=DIALOG_MESSAGE('The number of parameters do not match. There is only 1 curve in the input workspace. Try again !',/error)
            if temp gt 1 then Bowdel=DIALOG_MESSAGE('The number of parameters do not match. There are ' + strtrim(temp,2) + ' curves in the input workspace. Try again !',/error)
         endelse
      end
    4:begin               ; Write curves fit results
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_wcur,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_wcur,set_value='W'+str_test
         endelse  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      print,'Saving fitting function in W'+str_test
      sz=size(sf_sub_function)
      if sz[0] lt 3 then return
      num_subf=sz[3]  & num_subf=num_subf[0]
      print,'Number of subfunctions=',num_subf
      w_out=fltarr(nb_channels,nb_spe_tot,num_subf+2) & e_out=w_out
      x_out=sx_in & y_out=sy_in
      for i=0,num_subf-1 do begin
         for j=0,nb_spe_tot-1 do begin
         w_out[*,j,i+2]=sf_sub_function[j,*,i]
         endfor
      endfor
      for j=0,nb_spe_tot-1 do begin
      w_out[*,j,0]=sw_in[*,j] & e_out[*,j,0]=se_in[*,j]
      w_out[*,j,1]=sf_w_fitted[*,j]
      endfor
      ii=EXECUTE("w61=w_out")
      ii=EXECUTE("x61=x_out")
      ii=EXECUTE("y61=y_out")
      ii=EXECUTE("e61=e_out")
      w_out2=fltarr(nb_channels,num_subf+2) & e_out2=w_out2
      w_out2[*,*]=w_out[*,num_spectrum-1,*] & e_out2[*,*]=e_out[*,num_spectrum-1,*]
      ii=EXECUTE("w"+str_test+"=w_out2")
      ii=EXECUTE("x"+str_test+"=x_out")
      ii=EXECUTE("y"+str_test+"=y_out")
      ii=EXECUTE("e"+str_test+"=e_out2")
      end
     5:begin               ; Write Sub_func
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_wsf,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_wsf,set_value='W'+str_test
         endelse
         widget_control,sw_dl_wsf,get_value=str_subf
         ind=widget_info(sw_dl_wsf,/droplist_select) 
         print,'Saving Subfunction '+sf_sub_function_name[ind]+' in Workspace W'+str_test
         w_out=fltarr(nb_channels,nb_spe_tot) & e_out=w_out
         x_out=sx_in & y_out=sy_in
         for j=0,nb_spe_tot-1 do begin
         w_out[*,j]=sf_sub_function[j,*,ind]
         endfor
         ii=EXECUTE("w"+str_test+"=w_out")
         ii=EXECUTE("x"+str_test+"=x_out")
         ii=EXECUTE("y"+str_test+"=y_out")
         ii=EXECUTE("e"+str_test+"=e_out")
      end
     6:begin               ; Write All
         w_test=fix(Ev[2]) 
         if (w_test eq 0) then begin
         widget_control,sw_la_wall,get_value=str_test
         str_test=strmid(str_test,1)
         endif else begin 
         str_test=strcompress(string(w_test),/remove_all)
         widget_control,sw_la_wall,set_value='W'+str_test
         endelse
;         widget_control,sw_dl_wall,get_value=str_subf
;         ind=widget_info(sw_dl_wall,/droplist_select) 
         print,'Saving all Qs and all function (fit, data, sub_func) in W'+str_test+' which will be a 4D matrix (energy,Qs/thetas,subfunctions+fit+data)'
         sz=size(sf_sub_function)
         if sz[0] lt 3 then return
         num_subf=sz[3]  & num_subf=num_subf[0]
         print,'Number of subfunctions=',num_subf
         w_out=fltarr(nb_channels,nb_spe_tot,num_subf+2) & e_out=w_out
         x_out=sx_in & y_out=sy_in
         for i=0,num_subf-1 do begin
         for j=0,nb_spe_tot-1 do begin
         w_out[*,j,i+2]=sf_sub_function[j,*,i]
         endfor
         endfor
         for j=0,nb_spe_tot-1 do begin
         w_out[*,j,0]=sw_in[*,j] & e_out[*,j,0]=se_in[*,j]
         w_out[*,j,1]=sf_w_fitted[*,j]
         endfor
         ii=EXECUTE("w"+str_test+"=w_out")
         ii=EXECUTE("x"+str_test+"=x_out")
         ii=EXECUTE("y"+str_test+"=y_out")
         ii=EXECUTE("e"+str_test+"=e_out")
         end      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       endcase
       end
    2: case Ev[2] of               ; RES FUNCTION FWHM Event
          1:begin
          fwhm_res=1.
          widget_control,sw_tx_nlor,get_value=str_fwhm_res
          str_fwhm_res=strcompress(str_fwhm_res,/remove_all) & fwhm_res=float(str_fwhm_res) & fwhm_res=fwhm_res[0]
          print,'Gaussian FWHM=',fwhm_res
          str_fwhm_res='FWHM='+str_fwhm_res[0]
          widget_control,sw_la_res,set_value=str_fwhm_res        
          widget_control,bad_id=i,sw_ba_bot,/DESTROY
         sw_res=fltarr(nb_channels,nb_spe_tot) & sx_res=sx_in & se_res=sw_res & sy_res=sy_in
         for iis=0,nb_spe_tot-1 do begin
         sw_res[*,iis]=qgauss(sx_in,0.,fwhm_res,1)
         se_res[*,iis]=0.
         endfor
         if (nb_spe_tot eq 1) then begin
         sw_res=REFORM(sw_res, /OVERWRITE)
         se_res=REFORM(se_res, /OVERWRITE)
         endif
          end                ; Start Event
          2:widget_control,bad_id=i,sw_ba_bot,/DESTROY       ; Done Event
      endcase
    3:begin                     ; Mid Line Event
      case Ev[1] of
          1:                    ; Mid column1 Event
          2:begin
            case Ev[2] of
                1:begin
                change_spectrum           ; "Accept !" Event
                if flag_pl_fit then begin
                prepare_2_fit, X,Y,E,parinfo,sstat
                yyy=myfunc(sf_new,XVAL=X,YVAL=Y,ERRVAL=E)
                plot_it
                endif
             end
             2:begin
             flag_pl_res= not flag_pl_res
             wset,s_pl_wid
             plot_it
             end
             3:begin
             flag_norm= not flag_norm
             wset,s_pl_wid
             plot_it
             end
             4:begin
             widget_control,sw_tx_wsub,get_value=str_wtest
          w_test=fix(strcompress(str_wtest,/remove_all))
                w_test=w_test-1 & w_test=w_test[0]
                if w_test lt 1 then w_test=nb_spe_tot
                widget_control,sw_tx_wsub,set_value=strcompress(string(w_test),/remove_all)
                change_spectrum
                if flag_pl_fit then begin
                prepare_2_fit, X,Y,E,parinfo,sstat
                yyy=myfunc(sf_new,XVAL=X,YVAL=Y,ERRVAL=E)
                plot_it
                endif
                end
             5:begin
             widget_control,sw_tx_wsub,get_value=str_wtest
          w_test=fix(strcompress(str_wtest,/remove_all))
                w_test=w_test+1 & w_test=w_test[0]
                if w_test gt nb_spe_tot then w_test=1
                widget_control,sw_tx_wsub,set_value=strcompress(string(w_test),/remove_all)
                change_spectrum
                if flag_pl_fit then begin
                prepare_2_fit, X,Y,E,parinfo,sstat
                yyy=myfunc(sf_new,XVAL=X,YVAL=Y,ERRVAL=E)
                plot_it
                endif
                end
             6:
            endcase
            end
          3:    no_plot=1                 ; Mid column3 Event ... plot should not occur
      endcase
      end
; ------------------------------------------------------------------------------------------------------
; Partie qui gere la boite de dialogue 'demande nombre de lor et gau'
    4:case Ev[2] of               ; Bottom Event
          1:begin
          num_lor=0 & num_gau=0
          widget_control,sw_tx_nlor,get_value=str_numlor
          widget_control,sw_tx_ngau,get_value=str_numgau
          str_numlor=strcompress(str_numlor,/remove_all) & num_lor=fix(str_numlor) & num_lor=num_lor[0]
          str_numgau=strcompress(str_numgau,/remove_all) & num_gau=fix(str_numgau) & num_gau=num_gau[0]
          print,'num_lor=',num_lor
          print,'num_gau=',num_gau
          n_l=num_lor & n_g=num_gau
          if ((num_lor lt 0) or (num_lor gt 5)) then begin
          str_numlor='1'
          widget_control,sw_tx_nlor,set_value=str_numlor
          endif $
          else if ((num_gau lt 0) or (num_gau gt 5)) then begin
          str_numgau='1'
          widget_control,sw_tx_ngau,set_value=str_numgau
          endif else begin
          widget_control,bad_id=i,sw_ba_bot,/DESTROY
          treat_resf,sstat
          if sstat ne 0 then return
          fit_basegl2
          endelse
          end                ; Start Event
          2:widget_control,bad_id=i,sw_ba_bot,/DESTROY       ; Done Event
      endcase
; ---------------------------------------------------------------------------------------------------------------------------------
    6:case Ev[1] of
          0:begin
                widget_control,sw_tx_wsub,get_value=str_wsub & str_wsub=strlowcase(strcompress(str_wsub,/remove_all))
                num_spectrum=1 & str_wsub=str_wsub[0]
                num_spectrum=fix(str_wsub) & num_spectrum=num_spectrum[0]
                if ((num_spectrum le 0) or (num_spectrum gt nb_spe_tot)) then begin
                num_spectrum=1
                widget_control,sw_tx_wsub,set_value=' 1'
                endif
;                ispecc=num_spectrum
                betatemp=0
                if flag_sym then begin
                widget_control,sw_tx_temp,get_value=str_temp & str_temp=strlowcase(strcompress(str_temp,/remove_all))
                str_temp=str_temp[0] & temp=float(str_temp) & temp=temp[0]
                if temp le 1.5 then begin
                widget_control,sw_tx_temp,set_value='0.',sensitive=0
                flag_sym=not flag_sym
                widget_control,sw_bu_symsqw,set_button=abs(flag_sym)
                endif else $
                betatemp=11.6045/temp
                endif
;                print,'betatemp=',betatemp,'temp-meV=',1./betatemp
          case Ev[2] of
                0:begin
                on_error,0
                ; ** THE TRY BUTTON HAS BEEN PRESSED !!!
                prepare_2_fit, X,Y,E,parinfo,sstat
                if sstat ne 0 then begin
                Rrrr=DIALOG_MESSAGE('Something got wrong in the input of the fiting parameters ... quitting',/ERROR)
                Print,"PROBLEM: problem in the prepare_to_fit routine ... stat=",stat
                return
                endif
                yyy=myfunc(sf_new,XVAL=X,YVAL=Y,ERRVAL=E)
                change_spectrum
                end
                1:begin
                on_error,0
                ispecc_min=num_spectrum & ispecc_max=num_spectrum
                widget_control,bad_id=i,sw_bu_fit,set_value='  STOP!   '
                if flag_all then begin & ispecc_min=1 & ispecc_max=nb_spe_tot & endif
         ;Fitting Loop ----------------------------------
                maystop=0
                for ispecc=ispecc_min,ispecc_max do begin
         ;Fitting Loop ----------------------------------
                if not flag_reverse then num_spectrum=ispecc else num_spectrum=nb_spe_tot-ispecc+1
                print,'FITTING USING THE ',str_engine,' ENGINE'
                print,"Treating spectrum# ", num_spectrum
                ; ** THE FIT BUTTON HAS BEEN PRESSED !!!
;                stop
                prepare_2_fit, X,Y,E,parinfo,sstat
                if sstat ne 0 then begin
                Rrrr=DIALOG_MESSAGE('Something got wrong in the input of the fiting parameters ... quitting',/ERROR)
                Print,"PROBLEM: problem in the prepare_to_fit routine ... stat=",stat
                return
                endif
                case str_engine of
                'MPFIT':begin
                ; ** Now I can launch the fitting procedure
                ; ************** FIT FIT FIT ***************************************************
                sf_new=MPFIT('myfunc',sf_new,FUNCTARGS={XVAL:X,YVAL:Y,ERRVAL:E},QUIET=1,$
                PERROR=sf_e,BESTNorM=chisq,PARINFO=parinfo,ERRMSG='WARNING: PROBLEM IN MPFIT')
                ; ******************************************************************************
                ;chisq=chisq/degoffreed  ; value of the reduced chi-square=chisq/degof freedom
                end
                'COOLFIT':begin
                ; ** Now I can launch the fitting procedure
                ; ************** FIT FIT FIT ***************************************************
                sf_new=mp2cool('myfunc',sf_new,FUNCTARGS={XVAL:X,YVAL:Y,ERRVAL:E},$
                PERROR=sf_e,BESTNorM=chisq,PARINFO=parinfo,ERRMSG='WARNING: PROBLEM IN COOLFIT')
                end
                endcase

          if flag_fit2D then begin
          sf_new_values=transpose(reform(sf_new,n_param_1D,nb_spe_tot))
;          sf_e_values=transpose(reform(2.*sf_e*sqrt(2.*ALOG(2.)),n_param_1D,nb_spe_tot)) ; A verifier la multiplication des erreurs!!
          sf_e_values=transpose(reform(sf_e*sqrt(chisq/degoffreed),n_param_1D,nb_spe_tot)) ; A verifier la multiplication des erreurs!!
          endif else begin
                sf_new_values[num_spectrum-1,*]=sf_new[*]
;                sf_e_values(num_spectrum-1,*)=2.*sf_e(*)*sqrt(2.*ALOG(2.))  ; A verifier la multiplication des erreurs!
;                sf_e_values(num_spectrum-1,*)=sf_e(*)  ; A verifier la multiplication des erreurs!
                sf_e_values[num_spectrum-1,*]=sf_e[*]*sqrt(chisq/degoffreed)  ; A verifier la multiplication des erreurs!
                endelse
                de=abs(sx_in[1]-sx_in[0])
                print,"de=",de," degree of freedom=",degoffreed
                print,"Re-CHISQ*DE=",chisq*de/degoffreed
                chisqr[num_spectrum-1]=chisq/degoffreed
                print,'For spectrum# ',num_spectrum,' Re-CHISQ=',chisq/degoffreed
         ;End of Loop ----------------------------------
;                 evv=widget_event(sw_bu_fit,/nowait,bad_id=ii); Check for Stop button
;                 if evv.id eq sw_bu_fit then begin maystop=1 & break & endif & widget_control,/hourglass
                endfor
         ;End of Loop ----------------------------------
                widget_control,bad_id=i,sw_bu_fit,set_value='   FIT !   '
                if maystop then begin & wait,2. & print,'Interrupted at spectrum#',num_spectrum
                endif else ispecc=ispecc-1
                widget_control,bad_id=i,sw_bu_fit,/clear_events
                end
           endcase
; -------------------------------------------------------------------------------------------------------------------------------------------------------
           ispecc_min=num_spectrum & ispecc_max=num_spectrum
           if (flag_all or flag_fit2D) then begin & ispecc_min=1 & ispecc_max=nb_spe_tot & endif
           for ispect=ispecc_min-1,ispecc_max-1 do begin
                num_spectrum=ispect+1
                print,"calculating spectrum# ", num_spectrum
                str_wsub=string(num_spectrum) & str_wsub=strcompress(str_wsub[0],/remove_all) & widget_control,sw_tx_wsub,set_value=str_wsub
                 ; ** And I can write the results
               if not flag_pl_fit then flag_pl_fit=not flag_pl_fit
               change_spectrum
               sw_result=sf_w_fitted & widget_control,sw_bu_wwou,sensitive=1& widget_control,sw_bu_wpara_in,sensitive=1 & widget_control,sw_bu_wcur,sensitive=1
               widget_control,sw_bu_wsf,sensitive=1 & widget_control,sw_dl_wsf,sensitive=1
               sz=size(SF_SUB_FUNCTION) & num_subf=sz[3]
               widget_control,sw_dl_wsf,set_value=sf_sub_function_name
               widget_control,sw_bu_wall,sensitive=1
;            plot_it
            widget_control,sw_bu_savec,sensitive=1
            widget_control,sw_bu_savep,sensitive=1
         endfor
; ---------------------------------------------------------------------------------------------------------------------------------------------------------
                if flag_fit2D then begin
                flag_fit2D=not flag_fit2D
                widget_control,sw_bu_fit2D,set_button=0
                widget_control,sw_bu_try,sensitive=1
                widget_control,sw_bu_fitall,set_button=0,sensitive=1
                widget_control,sw_bu_acquire,sensitive=1
                widget_control,sw_tx_acquire,sensitive=1
                widget_control,sw_bu_giveall,sensitive=1
                widget_control,sw_bu_reverse,set_button=0
                widget_control,sw_bu_fromlast,set_button=0
                ; Changer les valeurs de sf_tied qui ont '2D' en '1'
                endif
                if flag_all then begin
                flag_all=not flag_all
                widget_control,sw_bu_try,sensitive=abs(not flag_all)
                widget_control,sw_bu_fitall,set_button=abs(flag_all)
                widget_control,sw_bu_acquire,sensitive=abs(not flag_all)
                widget_control,sw_tx_acquire,sensitive=abs(not flag_all)
                widget_control,sw_bu_giveall,sensitive=abs(not flag_all)
                widget_control,sw_bu_fit2D,sensitive=abs(not flag_all)
                if not flag_all then begin
                flag_fromlast=0 & flag_reverse=0
                widget_control,sw_bu_reverse,set_button=0
                widget_control,sw_bu_fromlast,set_button=0
                endif
                widget_control,sw_bu_reverse,sensitive=abs(flag_all)
                widget_control,sw_bu_fromlast,sensitive=abs(flag_all)
                endif
                end
          1:begin
                widget_control,sw_tx_wsub,get_value=str_wsub & str_wsub=strlowcase(strcompress(str_wsub,/remove_all))
                num_spectrum=1 & str_wsub=str_wsub[0]
                num_spectrum=fix(str_wsub) & num_spectrum=num_spectrum[0]
                if ((num_spectrum le 0) or (num_spectrum gt nb_spe_tot)) then begin
                num_spectrum=1
                widget_control,sw_tx_wsub,set_value=' 1'
                endif
            i2fix=Ev[2]
            sf_index[num_spectrum-1,i2fix]=not sf_index[num_spectrum-1,i2fix]
              widget_control,sw_tx_param[i2fix*3],sensitive=sf_index[num_spectrum-1,i2fix]
            widget_control,sw_bu_balg2[i2fix],set_button=abs(not sf_index[num_spectrum-1,i2fix])
            end
          2:case Ev[2] of
                0:begin
                widget_control,bad_id=i,sw_ba_bot,/DESTROY       ; Done Event
                widget_control,bad_id=i,sw_bu_fit,sensitive=0   ;Set the FIT button non-sensitive
                widget_control,bad_id=i,sw_bu_done,sensitive=0          ;Set the DONE button non-sensitive
;                widget_control,sw_tx_resmin,sensitive=1         ;set the x res ranges actives
                widget_control,sw_tx_resmax,sensitive=1
                if flag_pl_fit then flag_pl_fit=not flag_pl_fit
                if flag_pl_fit2 then flag_pl_fit2=not flag_pl_fit2
                if flag_all then flag_all=not flag_all
                end
                1:begin                                         ;"Fit all Sp" check box
                flag_all=not flag_all
                flag_fit2D=0 & widget_control,sw_bu_fit2D,set_button=0,sensitive=abs(not flag_all)
                widget_control,sw_bu_try,sensitive=abs(not flag_all)
                widget_control,sw_bu_fitall,set_button=abs(flag_all)
                widget_control,sw_bu_acquire,sensitive=abs(not flag_all)
                widget_control,sw_tx_acquire,sensitive=abs(not flag_all)
                widget_control,sw_bu_giveall,sensitive=abs(not flag_all)
                if not flag_all then begin
                flag_fromlast=0 & flag_reverse=0
                widget_control,sw_bu_reverse,set_button=0
                widget_control,sw_bu_fromlast,set_button=0
                endif
                widget_control,sw_bu_reverse,sensitive=abs(flag_all)
                widget_control,sw_bu_fromlast,sensitive=abs(flag_all)
                end
                2:begin                                              ;"Acquire from Sp#" button
                widget_control,sw_tx_acquire,get_value=str_acq & str_acq=strlowcase(strcompress(str_acq,/remove_all))
                i_acq=1 & str_acq=str_acq[0]
                i_acq=fix(str_acq) & i_acq=i_acq[0]
                if ((i_acq le 0) or (i_acq gt nb_spe_tot)) then begin
                print,'ERROR num_spectrum out of range ... put it to 1'
                i_acq=1
                widget_control,sw_tx_acquire,set_value=' 1'
                endif
                fact=1
                ; Change the parameters text box values
                for i=0,n_param_1D-1 do begin ; sf_new_values must print the new value of the parameters
                str_balg2=string(sf_new_values[i_acq-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                ;widget_control,sw_tx_param(3*i),set_value=str_balg2,sensitive=sf_index(i_acq-1,i)
          widget_control,sw_tx_param[3*i],set_value=str_balg2,sensitive=abs(sf_index[i_acq-1,i])
                str_balg2=string(sf_last_values[i_acq-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                widget_control,sw_tx_param[3*i+1],set_value=str_balg2
                str_balg2=string(sf_e_values[i_acq-1,i]) & str_balg2=strcompress(str_balg2[0],/remove_all)
                widget_control,sw_tx_param[3*i+2],set_value=str_balg2
                sf_index[num_spectrum-1,i]=sf_index[i_acq-1,i]
                sf_new_values[num_spectrum-1,i]=sf_new_values[i_acq-1,i]
                lowerb[num_spectrum-1,i]=lowerb[i_acq-1,i] & lowerbl[num_spectrum-1,i]=lowerbl[i_acq-1,i]
                upperb[num_spectrum-1,i]=upperb[i_acq-1,i] & upperbl[num_spectrum-1,i]=upperbl[i_acq-1,i]
                sf_tie[num_spectrum-1,i]=sf_tie[i_acq-1,i]
                sf_cstr_write[num_spectrum-1,i]=sf_cstr_write[i_acq-1,i]
                widget_control,sw_tx_cstr[i],set_value=sf_cstr_write[num_spectrum-1,i]
                endfor
                end
                3:begin                                      ; REVERSE check box
                flag_reverse=not flag_reverse
                widget_control,sw_bu_reverse,set_button=abs(flag_reverse)
                end
                4:begin                                      ; FROM LAST check box
                flag_fromlast=not flag_fromlast
                widget_control,sw_bu_fromlast,set_button=abs(flag_fromlast)
                end
                5:begin                                      ; SYMETRIZED  check box
                flag_sym=not flag_sym
                widget_control,sw_bu_symsqw,set_button=abs(flag_sym)
                widget_control,sw_tx_temp,sensitive=abs(flag_sym)
                ;sw_bu_symsqw,sw_tx_temp
                end
                6:begin                                      ; FIT 2D check box
          flag_fit2D=not flag_fit2D
                widget_control,sw_bu_fit2D,set_button=abs(flag_fit2D)
                flag_reverse=0 & widget_control,sw_bu_reverse,set_button=0,sensitive=0
                flag_fromlast=0 & widget_control,sw_bu_fromlast,set_button=0,sensitive=0
                flag_all=0 & widget_control,sw_bu_fitall,set_button=0,sensitive=abs(not flag_fit2D)
                widget_control,sw_bu_try,sensitive=abs(not flag_fit2D)
                widget_control,sw_bu_acquire,sensitive=abs(not flag_fit2D)
                widget_control,sw_tx_acquire,sensitive=abs(not flag_fit2D)
                widget_control,sw_bu_giveall,sensitive=abs(not flag_fit2D)
                end
                7:begin
                ; recuperer les champs et analyse
                widget_control,sw_tx_give,get_value=str_balg2
                str_balg2=strupcase(strcompress(str_balg2,/remove_all)) & str_l=strlen(str_balg2)
                fact=1
                pv = STRPOS(str_balg2, ';')
                if pv ne -1 then begin   
                    parametre = STRMID(str_balg2, 0, pv) & parametre=parametre[0] ; type cl1,wl2, etc...
                    parametre2 = STRMID(str_balg2, pv+1, str_l-1) & parametre2=parametre2[0] ; type c,v,all pour constraint, value, all
                    numorparam = where(sf_functions_labels2 eq parametre)
                    if parametre2 eq 'C' then begin
                      sf_index[*,numorparam]=sf_index[num_spectrum-1,numorparam] ;pour fixer les valeurs
                      sf_cstr_write[*,numorparam]=sf_cstr_write[num_spectrum-1,numorparam]
                    endif
                    if parametre2 eq 'V' then begin
                      sf_new_values[*,numorparam]=sf_new_values[num_spectrum-1,numorparam]
                    endif
                    if parametre2 eq 'ALL' then begin
                      sf_index[*,numorparam]=sf_index[num_spectrum-1,numorparam] ;pour fixer les valeurs
                      sf_cstr_write[*,numorparam]=sf_cstr_write[num_spectrum-1,numorparam]
                      sf_new_values[*,numorparam]=sf_new_values[num_spectrum-1,numorparam]
                    endif
                    lowerb[*,numorparam]=lowerb[num_spectrum-1,numorparam]
                    lowerbl[num_spectrum-1,numorparam]=lowerbl[num_spectrum-1,numorparam]
                    upperb[*,numorparam]=upperb[num_spectrum-1,numorparam]
                    upperbl[num_spectrum-1,numorparam]=upperbl[num_spectrum-1,numorparam]
                    sf_tie[*,numorparam]=sf_tie[num_spectrum-1,numorparam]
                endif
                if pv eq -1 then begin 
                  if str_balg2 eq 'ALL' then begin 
                    for i=0,n_param_1D-1 do begin ; sf_new_values must print the new value of the parameters
                      sf_index[*,i]=sf_index[num_spectrum-1,i] ;pour fixer les valeurs
                      sf_new_values[*,i]=sf_new_values[num_spectrum-1,i]
                      lowerb[*,i]=lowerb[num_spectrum-1,i] & lowerbl[num_spectrum-1,i]=lowerbl[num_spectrum-1,i]
                      upperb[*,i]=upperb[num_spectrum-1,i] & upperbl[num_spectrum-1,i]=upperbl[num_spectrum-1,i]
                      sf_tie[*,i]=sf_tie[num_spectrum-1,i]
                      sf_cstr_write[*,i]=sf_cstr_write[num_spectrum-1,i]
                    endfor
                  endif
                  if str_balg2 eq 'C' then begin 
                    for i=0,n_param_1D-1 do begin 
                      sf_index[*,i]=sf_index[num_spectrum-1,i] 
                      lowerb[*,i]=lowerb[num_spectrum-1,i] & lowerbl[num_spectrum-1,i]=lowerbl[num_spectrum-1,i]
                      upperb[*,i]=upperb[num_spectrum-1,i] & upperbl[num_spectrum-1,i]=upperbl[num_spectrum-1,i]
                      sf_tie[*,i]=sf_tie[num_spectrum-1,i]
                      sf_cstr_write[*,i]=sf_cstr_write[num_spectrum-1,i]
                    endfor
                  endif
                  if str_balg2 eq 'V' then begin 
                    for i=0,n_param_1D-1 do begin ; sf_new_values must print the new value of the parameters
                      sf_new_values[*,i]=sf_new_values[num_spectrum-1,i]
                      lowerb[*,i]=lowerb[num_spectrum-1,i] & lowerbl[num_spectrum-1,i]=lowerbl[num_spectrum-1,i]
                      upperb[*,i]=upperb[num_spectrum-1,i] & upperbl[num_spectrum-1,i]=upperbl[num_spectrum-1,i]
                      sf_tie[*,i]=sf_tie[num_spectrum-1,i]
                      sf_cstr_write[*,i]=sf_cstr_write[num_spectrum-1,i]
                    endfor
                  endif
                endif                
                end
                8:begin ; to choose the engine value ...
                print,'FITTING WITH THE ',str_engine,' ENGINE'
                end
                9:
         endcase
         3:
      endcase
    7:if Event.type eq 1 then if (Event.release eq 4) then call_procedure,'lamp_event_parser',Event,[-88,0] ;Clipboard

    999:;do nothing ... this is a cul de sac
    else:
endcase
end

;** **************************************************************
pro plot_it
;+
;** **************************************************************
;** This routine defines the plot-limits required
;-

@lamp.cbk
common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_new

; ** Getting the information about the desired plot (xrange,yrange and subplot)
widget_control,sw_tx_xmin,get_value=str_xmin & str_xmin=strlowcase(strcompress(str_xmin,/remove_all))
widget_control,sw_tx_xmax,get_value=str_xmax & str_xmax=strlowcase(strcompress(str_xmax,/remove_all))
widget_control,sw_tx_ymin,get_value=str_ymin & str_ymin=strlowcase(strcompress(str_ymin,/remove_all))
widget_control,sw_tx_ymax,get_value=str_ymax & str_ymax=strlowcase(strcompress(str_ymax,/remove_all))



; ** Some necessary transformation from string to float or integer
str_xmin=str_xmin[0] & str_ymin=str_ymin[0]
str_xmax=str_xmax[0] & str_ymax=str_ymax[0]

; ** Putting the arrays into one and two dimensional array
w_wk=sw_in[*,num_spectrum-1]
w_wkres=sw_res[*,num_spectrum-1]
w_wkfit=sf_w_fitted[*,num_spectrum-1]
d_wk=sf_w_exp_fit[*,num_spectrum-1]
d_wk_rel=sf_w_exp_fit_rel[*,num_spectrum-1]

; ** Normalize to elastic peak intensity when normalize button is pushed
if flag_norm then w_wkres=w_wkres/(max(w_wkres)/max(w_wk))
e_wk=se_in[*,num_spectrum-1]
x_to_plot=sx_in & sss=size(x_to_plot)
if sss[0] gt 1 then x_to_plot=x_to_plot[0]
; ** Checking the input value for errors
valid=0
while valid eq 0 do begin
on_ioerror,ok3
if str_xmin eq 'min' then xmin=min(sx_in) $
else begin
xmin=float(str_xmin) & xmin=xmin[0]
endelse
if str_xmax eq 'max' then xmax=max(sx_in) $
else begin
xmax=float(str_xmax) & xmax=xmax[0]
endelse
if str_ymin eq 'min' then ymin=min(w_wk)  $
else begin
ymin=float(str_ymin) & ymin=ymin[0]
endelse
if str_ymax eq 'max' then ymax=max(w_wk)  $
else begin
ymax=float(str_ymax) & ymax=ymax[0]
endelse
valid=1
ok3:if not valid then begin
Rrrr=DIALOG_MESSAGE('Type Conversion Error: Check input in plot range')
return
endif
endwhile
if ((xmin ge xmax) or (ymin ge ymax)) then begin
xmin=min(sx_in) & xmax=max(sx_in) & ymin=min(w_wk) & ymax=max(w_wk)
endif
;
; Adapt vector to current plot zone
;ind=where((sx_in ge xmin) and (sx_in le xmax) and (w_wk le ymax) and (w_wk ge ymin), nb_pt1) & ind=ind(0) ;last one to have a 1d vector
; do the plot
!x.title=''
!y.title='Intensity (a.u)'
str_title='Spectra # '+strcompress(num_spectrum,/remove_all)+' Value='+strcompress(sy_in[num_spectrum-1],/remove_all)
widget_control, bad_id=i, sw_pl_area, get_value=s_pl_wid    ; Store the number assigned to this window in s_pl_wid
wset,s_pl_wid                                                  ; Set the s_pl_wid active
plot,x_to_plot,w_wk,psym=7,xrange=[xmin,xmax],yrange=[ymin,ymax],TITLE=str_title,pos=[0.095,0.39,0.99,0.95];,/NOERASE
;indzero=where(errlow lt 0.) & if (indzero(0) ne -1) then errlow(indzero)=abs(10.*w_wk(indzero))
indzero=where(e_wk lt 0.) & if (indzero[0] ne -1) then e_wk[indzero]=10.*max(abs(w_wk));abs(10.*w_wk(indzero))
errlow=w_wk-abs(e_wk) & errhigh=w_wk+abs(e_wk)
;errplot,x_to_plot,w_wk-e_wk,w_wk+e_wk
errplot,x_to_plot,errlow,errhigh;,color=1
if flag_pl_res then oplot,x_to_plot,w_wkres,color=234557685
if flag_pl_fit then begin
badin=where(abs(w_wkfit) gt 10000) & if badin ne -1 then w_wkfit[badin]=0.
oplot,x_to_plot,w_wkfit,color=686543,linestyle=0,thick=2
sz=size(sf_sub_function)
if sz[0] lt 3 then goto,nosubf
num_subf=sz[3]  & num_subf=num_subf[0]
;for i=0,num_lor+num_gau+1 do begin
if betatemp le 2. then begin
t_fact=exp(0.5*betatemp*x_to_plot)
endif else begin
t_fact=(abs(x_to_plot)/x_to_plot)/(1-exp(-1.*betatemp*x_to_plot))
endelse
for i=0,num_subf-3 do begin
sf_sub_function[num_spectrum-1,*,i]=sf_sub_function[num_spectrum-1,*,i]*t_fact
oplot,x_to_plot,sf_sub_function[num_spectrum-1,*,i],color=11+i*10,linestyle=0,thick=1.5
endfor
for i=num_subf-2,num_subf-1 do begin
sf_sub_function[num_spectrum-1,*,i]=sf_sub_function[num_spectrum-1,*,i]*t_fact
oplot,x_to_plot,sf_sub_function[num_spectrum-1,*,i],color=1000+i*100,linestyle=2,thick=1
endfor
str_chisqr=strcompress(string(chisqr[num_spectrum-1]),/remove_all)
XYOUTS,xmin+0.1*(xmax-xmin),ymin+0.9*(ymax-ymin),'Re-Chisqr = '+str_chisqr;,/DEVICE ;print the Re-Chisq value for this angle/Q
nosubf:
;widget_control, bad_id=i, sw_pl_area, get_value=s_pl_wid
;widget_control, bad_id=i, sw_pl_areabot, get_value=s_pl_widbot   ; Store the number assigned to this window in s_pl_wid
;wset,s_pl_widbot
;wset,s_pl_wid
!x.title=''
!y.title='DIFF';!y.title='exp-th '
plot,x_to_plot,d_wk,xrange=[xmin,xmax],pos=[0.095,0.22,0.99,0.34],/NOERASE,XSTYLE=9,YSTYLE=9
!x.title='Energy transfert (meV)'
!y.title='%';!y.title='exp-th '
plot,x_to_plot,d_wk_rel,xrange=[xmin,xmax],yrange=[0,max(d_wk_rel)<100],pos=[0.095,0.05,0.99,0.17],/NOERASE,XSTYLE=9,YSTYLE=9
wset,s_pl_wid
endif
;plot_flag=1
;trap_current=!D.window
return
end

;** **************************************************************************
;** **************************************************************
pro inputfwhm
;+
;** **************************************************************
;** create the base of the fitting routine
;-

@lamp.cbk
common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_new

if (xregistered('fit_base') le 0) then begin

; ** Start definition of the widget object
; ** *************************************

; ** Base for the fit
sw_ba_bot=widget_base(group_leader=sw_ba_leader, /row, $
          Title='Enter FWHM for Gaussian Resolution',$
          resource_name='lamp')
;sw_ba_bot=widget_base(sw_ba_leader,Title='Base row bottom',/row)

; ** First up : button, labels and text boxes
sw_la_nlor=widget_label(sw_ba_bot,Font=ft_b_smaller,value='Enter FWHM ?')
sw_tx_nlor=widget_text(sw_ba_bot,Font=ft_b_smaller,/editable,value='0.1',uvalue=[2,0,1],xsize=10)
sw_bu_start=widget_button(sw_ba_bot,Font=ft_b_smaller,Frame=2,value='Enter',uvalue=[2,0,1])
sw_bu_cancel=widget_button(sw_ba_bot,Font=ft_b_smaller,Frame=2,value='Cancel',uvalue=[2,0,2])

; ** Realization of the Base
widget_control, sw_ba_bot, /Realize; & put_logo
xmanager,'qens_fit',sw_ba_bot,/just_reg

endif

end



;** **************************************************************************
pro treat_resf,sstat
;+
;** **************************************************************************
;** This program gives several treatment to the resolution function spectrum
;** Basically, il substracts a flat background, performs the normalization to 1
;** and deal with the range under consideration (e.g w where res(w) ne 0)
;** sw_res=vector containing the exp. res. function
;** w_resc=vector containing the treated res function
;** rfmin and rfmax define the range of res. fun. to consider [rfmin,rfmax]
;** treatment for spectrum ispmin to ispmax but for the moment, full treatment e.g
;** ispmin and ispmax=(size(w_res))(2) (nspectra)
;-

common sc_loc
common sc_fit
common sc_convs
common sc_wid
common sc_new

; ** On recupere les donnees de la boite de dialogue concernant le  Res. fonction range
; ** **********************************************************************************
          Print,'Traitement Res function'
          valid=0
          ON_IOERROR,nok
;          widget_control,sw_tx_resmin,get_value=str_resmin
;          str_test=strcompress(str_resmin,/remove_all) & rfmin=float(str_resmin) & rfmin=rfmin(0)
          widget_control,sw_tx_resmax,get_value=str_resmax
          str_test=strcompress(str_resmax,/remove_all) & rfmax=float(str_resmax) & rfmax=rfmax[0] & rfmin=-1.*rfmax
          valid=1
nok:       if not valid then begin
         Rrrr=DIALOG_MESSAGE('Bad Type conversion in min and max for the res. function_range. Check correct type ... quitting',/ERROR)
         return
         endif
          if rfmax le rfmin then begin  
         Rrrr=DIALOG_MESSAGE("PROBLEM: range of Res. function, min > max !! ... quitting",/ERROR)
         return
         endif

; ** Check error
; ** *****************************
sstat=0

; ** recovering the data parameter
; ** *****************************
x_rf=sx_res ; energy array
y_rf=sy_res ; Q or theta array
e_rf=se_res ; error array
w_rf=sw_res   ; raw RF array
iprint=0 & ispmax=1
aalpha=y_rf*0. & abeta=aalpha & cel=aalpha
s_w_rf=size(sw_res) & nrfx=s_w_rf[1]
s_x_rf=size(sx_in) & nrx=s_x_rf[1]
ispmin=1 & ispmax=nb_spe_tot
fwhm_rf=fltarr(nb_spe_tot) & center_rf=fwhm_rf
;ispmin=1 & if s_w_rf(0) eq 2 then ispmax=s_w_rf(2)
if nrfx ne nrx then begin
Rrrr=DIALOG_MESSAGE("WARNING: Dimension of data and resol. function,don't match !!",/Warning)
;sstat=1
;return
endif

; ** Start the loop on the spectra
; ** ******************************
;for ispec=ispmin-1,ispmax-1 do begin
;if iprint ne 0 then print,"Spectrum #",ispec
;
; ** obtaining alpha, beta and ce
; ** cad x_rf(alpha)=rfmin & x_rf(beta)=rfmax (limits)
; ** and x_rf(ce)=0. meV (elastic channel)
; ** *************************************************
ispec=0
;rfin=where((x_rf lt rfmax) and (x_rf gt rfmin),nrfpts, complement=rfout)
;aalpha(0:ispmax-1)=rfin(0) & abeta(0:ispmax-1)=rfin(nrfpts-1)
;** Test if the 0 meV point is included into the energy range. IT SHOULD BE !!
indzero=where(abs(x_rf) le 1.e-6,nrfptszero)
if nrfptszero ne 1 then begin
Rrrr=DIALOG_MESSAGE("PROBLEM: either the 0 meV point is not included into the energy range, or there are many points le 1.e-6 meV! Number of 0 meV points found:"+string(nrfptszero),/error)
Rrrr= DIALOG_MESSAGE("Should I Quit (=Yes) or Continue (=No) with 0meV as elastic peak position ??",/DEFAULT_NO,/QUESTION) ;] [, RESOURCE_NAME=string] [, TITLE=string] )
If Rrrr eq 'Yes' then begin
sstat=1
goto,bof
endif
endif

;** Test the symmetricity of the energy range !!
;** and make it symmetric if not chosen so ...
testok=0
xrfcut=abs(rfmin)
if abs(rfmin) ne rfmax then xrfcut=min([abs(rfmin),rfmax])
; now check that limits are correct ...
while not testok do begin
;if (xrfcut lt x_rf(nb_channels-2)) and (-1.*xrfcut gt x_rf(1)) then testok=1
xrdiff=abs(x_rf-xrfcut) & indmax=where(xrdiff eq min(xrdiff))
iwidth=abs(indmax-indzero) & indmin=indzero-iwidth & indmax=indzero+iwidth & indmin=indmin[0] & indmax=indmax[0]
if (indmax lt nb_channels-2) and (indmin gt 1) then testok=1 $
else xrfcut=x_rf[indmax-1]
endwhile
xrfinf0=x_rf[indmin] & xrfsup0=x_rf[indmax]
aalpha[0:ispmax-1]=indmin & abeta[0:ispmax-1]=indmax
str_xrfmin=strcompress(string(xrfinf0),/remove_all) & str_xrfmax=strcompress(string(xrfsup0),/remove_all)
;widget_control,sw_tx_resmin,set_value=str_xrfmin
widget_control,sw_tx_resmax,set_value=str_xrfmax
;** Test if number of points included into the convolution is odd and symetric:
nrfpts=fix(indmax-indmin+1) & nrfpts=nrfpts[0]
rfin=indgen(nrfpts)+indmin[0]
if (floor(nrfpts/2.)*2) eq nrfpts then begin
sstat=1
print,'nrfpts=',nrfpts
goto, bof
endif
bof: if sstat eq 1 then begin
Rrrr=DIALOG_MESSAGE('PROBLEM: number of points in the Res. Function_range should be odd and symetric % 0meV ... quitting. See main IDL window for details',/ERROR)
print,'**************************** PROBLEM PROBLEM PROBLEM PROBLEM *************************'
print,'PROBLEM: number of points in the Res. Function range should be odd and symetric % 0meV.'
print,'Try the following:'
print,'1/change sligthly the resolution function range to include one more point'
print,'2/rebin both the data and the resolution function to a different energy grid'
print,'Current energy grid for the Res. Function is:',sx_in[rfin]
sstat=1
return
endif

for ispec=ispmin-1,ispmax-1 do begin
if iprint ne 0 then print,"Spectrum #",ispec
ce=where(w_rf[*,ispec] eq max(w_rf[*,ispec])) & e_el=x_rf[ce]
cel[ispec]=ce[0] & ce=ce[0] & test=where(ce eq rfin) & test=test[0]

if test eq -1 then begin  
print, "PROBLEM: range of Res. function,doesn't have the elastic peak !!"
print, "Check the Resolution function, group#", ispec+1
errtxt="!!!: range of Res. function,doesn't have the elastic peak !! CONTINUE ??"
Result = DIALOG_MESSAGE(errtxt, /DEFAULT_NO, /QUESTION)
if Result eq 'No' then begin
sstat=1
return
endif
ce=where(w_rf[rfin,ispec] eq max(w_rf[rfin,ispec])) & e_el=x_rf[ce]
cel[ispec]=ce[0] & ce=ce[0]
endif
if nrfpts le 3 then begin  
print, "PROBLEM: not enough points in the resolution function_range !!"
sstat=1
return
endif
if iprint ne 0 then begin
print,"aalpha=",aalpha[ispec],"abeta=",abeta[ispec],"ce=",ce,"e_el=",e_el
print,"rfin=",rfin
print,"rfout=",rfout
print,"test=",test
endif

; ** Fitting the resolution function with a Gaussian shape to have
; ** the width at half maximum
; ** **************************************************************

x_in_fit=x_rf[rfin]
y_in_fit=w_rf[rfin,ispec]
reso_fitted= gaussfit(x_in_fit,y_in_fit,A_param,NTERMS=4)
;print,A_param
;plot,x_in_fit,y_in_fit
;oplot,x_in_fit,reso_fitted,color=80000
fwhm_rf[ispec]=2*A_param[2]*sqrt(2*alog(2.)) & print,'FWHM for the RF at group#',ispec+1,fwhm_rf[ispec]
center_rf[ispec]=A_param[1] & print,'Center for the RF at group#',ispec+1,A_param[1]
if ispec eq ispmax-1 then print,'Resolution function_energy range:',x_rf[rfin]
;wait,0.2
; ** Substraction of a flat background
; ** *************************************
if iprint ne 0 then print, w_rf[*,ispec]
rfalpha=total(w_rf[aalpha[ispec]:aalpha[ispec]+9,ispec],1)/10. & rfbeta=total(w_rf[abeta[ispec]-9:abeta[ispec],ispec],1)/10.
backg=rfalpha < rfbeta
if iprint ne 0 then print,backg,rfalpha,rfbeta
if backg le 0 then backg=0.
w_rf[*,ispec]=w_rf[*,ispec]-backg
w_rf[0:aalpha[ispec]-1,ispec]=0. & w_rf[abeta[ispec]+1:nrfx-1,ispec]=0.
e_rf[0:aalpha[ispec]-1,ispec]=0. & e_rf[abeta[ispec]+1:nrfx-1,ispec]=0.
infzero=where(w_rf[*,ispec] lt 0.)
if infzero[0] ne -1 then begin
w_rf[infzero,ispec]=0. & e_rf[infzero,ispec]=0.
endif
if iprint ne 0 then print, w_rf[*,ispec]

; ** Normalization to unity
; ** *************************************
deltaw1=x_rf[1]-x_rf[0] & deltaw2=x_rf[nrx-11]-x_rf[nrx-12]
deltaw=x_rf[ce+1]-x_rf[ce]
diff1=abs(deltaw1-deltaw) & diff2=abs(deltaw2-deltaw)
if (diff1 gt 0.00001) or (diff2 gt 0.00001) then begin
Rrrr=DIALOG_MESSAGE("PROBLEM: non constant energy step in W_in energy range ... quitting",/ERROR)
print,"PROBLEM: non constant energy step"
print,"Deltaw1=",deltaw1," Deltaw2=",deltaw2," Deltaw=",deltaw
print,"Diff1=",diff1," Diff2=",diff2
sstat=1
return
endif
factor=total(w_rf[*,ispec])*deltaw & sfac=size(factor)
if iprint ne 0 then print,"factor=",factor," deltaw=",deltaw
w_rf[*,ispec]=w_rf[*,ispec]/factor
e_rf[*,ispec]=e_rf[*,ispec]/factor
if iprint ne 0 then print,"NEWSUM=",total(w_rf[*,ispec])*deltaw


; ** End loop on ispec
; *********************
endfor

; ** Return modified values in w_out
; ** *******************************
w_resc=w_rf
sw_result=w_rf
lmin=cel-aalpha & lmax=abeta-cel

; ** Set the text boxes relative to the x range to consider inactives
; ** ****************************************************************
;widget_control,sw_tx_resmin,sensitive=0
widget_control,sw_tx_resmax,sensitive=0

; ** End of Subroutine convs
; ** ***********************
end

;** **************************************************************
pro qens_fit, Just=just, Group=group, Tripx=tripx, tx_param
;+
;** **************************************************************
;** Main body of this complete routine ... the first to be called
;-

@lamp.cbk
common sc_wid
common sc_loc
common sc_flag
common sc_fit
common sc_new

forward_function funlg_to_fit
forward_function fun_isorot
forward_function fun_rwtrans
forward_function fun_rotctran
forward_function fun_unirot
forward_function fun_gumbel
forward_function fun_userdefined
;resolve_routine,'str_fit',/compile_full_file,/is_function

;Factors for using measured widths for peaks
;
;resolve_routine,'treat_resf'
;resolve_routine,'convs',/IS_function
gw_fac=2.0*sqrt(2.)
lw_fac=0.5
nb_channels=0
num_spectrum=1
;print,"ON Y EST ! C'est parti"
flag_pl_res=0
flag_norm=0
flag_pl_fit=0 & flag_pl_fit2=0 & flag_all=0
flag_lect_rf=0

if keyword_set(just) then g_ctrl_panel=1 else g_ctrl_panel=0
if (!D.flags and 65536) eq 0  then print,'set_plot,"X" before using rdfilter' else  $
if (xregistered('qens_fit') le 0) then begin
    cp_nb   =0
    cp_bck  =0
    cp_pos  =0
    cp_hgt  =0
    cp_wid  =0
    cp_pf   =0
    cp_hf   =0
    cp_wf   =0
    cp_fb   =0
    cp_sb   =0
    cp_fbf  =0
    cp_sbf  =0

  g_old_plot_wid=!D.window
  P_MUS,'mus_harp'          ; PLAY a tune
  g_fct_name    ='Gauss' ; Default is Gauss
  g_old_pk_nb   =1     ; old peak number
  g_afitisdone  =0    ; No fit was done
  g_pk_nb   =1     ; number of peak processed
  g_char    =0.8    ;
  g_print_nb    =0  ;
  g_show_subfct =1   ; if this flag is set, subfunctions are plotted
  bb1     =0    ;l'une des bornes d'un intervalle a exclure (sert a la saisie)
  bb2     =0    ;idem bb1
  g_nb_pk_max   =6     ;<----change peaks maximum number
  max_nb_int    =3  ;<----change excluded zone maximum number
  sauv   =4   ;deplacement (dans viewarr) donnant la zone a afficher initiale
  nbzone    =3  ;stored zones number
  no_zone   =1     ;initializing zone number
  g_bg_but  =0    ;fixed as background button is unset by default
  viewarr   =fltarr(8+nbzone*4)    ;dim zone to view array
  excl_param    =fltarr(2*max_nb_int)   ;bornes des intervalles exclus
  excl_widge    =lonarr(2*max_nb_int)   ;widget IDs des fenetres d'affichage des bornes
  g_parameters  =fltarr(3*g_nb_pk_max+3) ;parametres position,height,width + flat,slope
  g_step    =strarr(3*g_nb_pk_max+2) ;extensions des champs pos,height,width,flat,slope
  g_error   =fltarr(3*g_nb_pk_max+2) ;erreurs sur les parametres correspondants de g_parameters
  g_fct_type    =strarr(g_nb_pk_max)
  gw_pos_pop    =lonarr(10)        ;widget ID of pos parameter menu
  gw_int_pop    =lonarr(10)        ;widget ID of int parameter menu
  gw_wdt_pop    =lonarr(10)        ;widget ID of wdt parameter menu
  gw_fbg_pop    =lonarr(10)        ;widget ID of fbg parameter menu
  gw_sbg_pop    =lonarr(10)        ;widget ID of sbg parameter menu
  sw_ni     =[0]
  g_npeaks  =1
  g_ncycles =2
  peaky     =0.0
  g_resid   =1.0e12

  g_fct_type[*] =g_fct_name
  nw_ni=0 & nw_out=1
  g_tripx=1 &wdtr='0'
  str_win='W1 ' & str_wres='W2 ' & str_wou='W3 ' & str_wcur='W4 ' & str_wsf='W5 ' & str_wall='W6'

  ;if g_tripx then nw_ni=23 else begin
      if lamp_b1 ne 0 then p_did_getw_cur, nw_ni, wstr  ; get current w number
      if nw_ni eq 0 then nw_ni=1    ; if not defined then w=1
      if nw_ni gt 20 then nw_ni=1
      nw_out    =nw_ni+1
  ;endelse
  if nw_out gt 20 then nw_out=20

if n_elements(Group) eq 0 then GROUP=lamp_b1
junk       = { CW_PDMENU_S, flags:0, name:'' }

; the whole lamp size depending modifications is here
sl_size =16
if lamp_siz gt 900 then begin
    xsiz=800 & ysiz=440
endif else begin
    xsiz=700 & ysiz=280
endelse
if lamp_siz lt 800 then begin   ; adapt to lamp size
    xsiz=550 & ysiz=250     ; adapt plotting window size
    sl_size=15
endif
txt_hlp=  '(LEFT press=position  release=width)....(RIGHT= Define Fitting Area)'

listw_1to20=['w1','w2','w3','w4','w5','w6','w7','w8','w9','w10','w11','w12','w13','w14','w15','w16','w17','w18','w19','w20']
listw_21to40=['w21','w22','w23','w24','w25','w26','w27','w28','w29','w30','w31','w32','w33','w34','w35','w36','w37','w38','w39','w40']
listw_41to60=['w41','w42','w43','w44','w45','w46','w47','w48','w49','w50','w51','w52','w53','w54','w55','w56','w57','w58','w59','w60']
listw=[' ',' ']


; ** Start definition of the widget object
; ** *************************************

; ** First Base
sw_ba_leader=widget_base(group_leader=Group, /column, $
          Title='Lamp QENS_FIT v4 09/2007',$
          resource_name='lamp' $;)
          ,MBAR=m_bar)
; ** Menu Bar
sw_bu_filem=widget_button(m_bar,/menu,value='File')
sw_bu_fitm=widget_button(m_bar,/menu,value='Fitting Model',sensitive=0)
sw_bu_savec=widget_button(sw_bu_filem,value='Save curves',uvalue=[0,0,1],sensitive=0)
sw_bu_savep=widget_button(sw_bu_filem,value='Save parameters',uvalue=[0,0,2],sensitive=0)
sw_bu_exitm=widget_button(sw_bu_filem,value='Exit',uvalue=[0,0,0])
sw_bu_lgm=widget_button(sw_bu_fitm,value='Lorentzian and Gaussian',uvalue=[0,1,0])
sw_bu_isorotm=widget_button(sw_bu_fitm,value='Isotropic rotation',uvalue=[0,1,1])
sw_bu_axrotm=widget_button(sw_bu_fitm,value='Axial rotation',uvalue=[0,1,2])
sw_bu_rwm=widget_button(sw_bu_fitm,value='Random Walk translations',uvalue=[0,1,3])
sw_bu_gdos=widget_button(sw_bu_fitm,value='iso_rot*iso_trans',uvalue=[0,1,4])
sw_bu_gumb=widget_button(sw_bu_fitm,value='Gumbel function',uvalue=[0,1,5])
sw_bu_daniel=widget_button(sw_bu_fitm,value='Daniel Page function',uvalue=[0,1,7])
;
sw_la_help=widget_button(sw_bu_fitm,value='',uvalue=[999])
sw_bu_userdefined=widget_button(sw_bu_fitm,value='User Defined function',uvalue=[0,1,6])

;sw_bu_plm=widget_button(m_bar,/menu,value='Analysis',sensitive=0)
;sw_bu_plwq=widget_button(sw_bu_plm,value='Plot width',uvalue=[0,2,0])
;sw_bu_plwi=widget_button(sw_bu_plm,value='Plot intensity',uvalue=[0,2,1])
;sw_pl_eisf=widget_button(sw_bu_plm,value='Plot EISF',uvalue=[0,2,2])
sw_bu_help=widget_button(m_bar,/menu,value='Help',sensitive=1)
; ** START THE HELP!
sw_la_help=widget_button(sw_bu_help,value='See complete doc at http://www.ill.fr/Computing/TOFHR-help/QENS_fit.pdf',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='I   -------- LOADING / SAVING THE WORKSPACES --------',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='1/ Select the Data/Resolution workspaces with the arrows',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='2/ Load them by clicking on the "Get W_in" / "Get Res_fun" button',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='3/ Export the fit parameters (values and errors) into the selected workspace by clicking on the "Write W_out" button',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='II  -------- VIEWING THE DATA  --------',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='1/ Select the energy and intensity range (min and max are for extrema)',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='2/ Select the spectrum (Sp#) to be viewed',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='3/ Click on the "accept" button to view',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='4/ Click on the toggle buttons to view the Res. function',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='III -------- FITTING THE DATA  --------',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='1/ Select the energy range in which to consider the Resolution function',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='   This should be symetric about the elastic line',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='2/ Choose the model to use in the Fitting Model menu',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='   --> answer the model dependent requirements ...',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='3/ Choose the Fitting range and the maximum number of iterations (cycles) to perform',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='4/ Choose initial values of the parameters for the fit (see below) ',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='5/ Click on the Fit! button to perform the fit',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='IV  -------- CONTROLING THE FIT PARAMETERS  --------',uvalue=[999])
;sw_la_help=widget_button(sw_bu_help,value='Each model is controled via a unique window which contains several functions.',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='1/ Each parameter is controled and displayed in the following way:"F1" | "P1""F1" | value | old_value | error | constraint',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='"F1"= the functionl_abel for the function_number 1 (ex:"L1" for lorentzian #1 or "D" for Delta)',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='"P1"= the first parameter label (ex:"C" for centre or "S" for slope)',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='| value | old_value | error | constraint, stand for the actual value of this parameter, its previous value, the error on it and its constraint during the fit',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='Example of constraints: "1" = free parameter; "0" = fixed parameter',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='Example of constraints: ">0"; "<10"; or ">0;<10"= parameter should be >0,<10 or in between 0 and 10 during the fit respectively',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='Example of constraints: "CL1"; "3*WL2"; or "IL1+IL2"= parameters are linked to other parameters and vary according to the latter',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='2/ Click on "Try and enter" to assign the current values to the selected spectrum and to have a look at your guess after modifying values',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='3/ Select the "fit all" option if you want to fit all the spectra',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value=' a- then select the "from last" option if you want the last parameters fitted to be given to the next spectrum to be fitted',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value=' b- then select the "reverse" option if you want the fit to be performed from the last spectrum toward the first spectrum',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='4/ Click on the "Acquire" button to use parameters from a previously fitted spectrum',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='5/ Select the Sym. S(Q,w) button to apply the balanced detail factor (and give the temperature in K)',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='6/ To propagate all the current parameters to all spectra, click "Give to All" with "All" in the textbox',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='    Type CL1;V to propagate only the Value of CL1, WG2;C to propagate the Constraint on WG2, CD;ALL to propagate value and constraint of CD',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='    Type C to propagate all constraints, V to propagate all values. It is not case sensitive.',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value='IV  -------- SAVING THE CURVES / PARAMETERS  --------',uvalue=[999])
;sw_la_help=widget_button(sw_bu_help,value='Each model is controled via a unique window and contains several functions.',uvalue=[999])
sw_la_help=widget_button(sw_bu_help,value=' ... is possible by several options in the file "menu"',uvalue=[999])
; ** END OF THE HELP
sw_bu_about=widget_button(m_bar,/menu,value='About ...',sensitive=1)
sw_la_help=widget_button(sw_bu_about,value=' Program written and developed by Stephane Rols',uvalue=[999])
sw_la_help=widget_button(sw_bu_about,value=' I.L.L ',uvalue=[999])
sw_la_help=widget_button(sw_bu_about,value=' rols@ill.eu',uvalue=[999])
sw_la_help=widget_button(sw_bu_about,value=' qens_fit v4 01/2010',uvalue=[999])

; ** First up : bases
sw_ba_up=widget_base(sw_ba_leader,Title='Base row up',frame=1,/row)
sw_ba_up1=widget_base(sw_ba_up,Title='Base row up 1',/column,/align_center,Frame=1)
sw_ba_up2=widget_base(sw_ba_up,Title='Base row up 2',/column,/align_center,Frame=1)
sw_ba_up3=widget_base(sw_ba_up,Title='Base row up 3',/column,/align_center,Frame=1)
sw_ba_up3p=widget_base(sw_ba_up,Title='Base row up 3p',/column,/align_center,frame=1)
sw_ba_up3pp=widget_base(sw_ba_up,Title='Base row up 3pp',/row,/align_center,Frame=1)
;sw_ba_up3ppp=widget_base(sw_ba_up,Title='Base row up 3ppp',/column,/align_center)
;sw_ba_up40=widget_base(sw_ba_up,Title='Base row up 40',/row,/align_center)
sw_ba_up4=widget_base(sw_ba_up,Title='Base row up 4',/column,/align_center,Frame=1)
sw_ba_up11=widget_base(sw_ba_up1,Title='Base row up 11',/row,/align_center)
sw_ba_up12=widget_base(sw_ba_up1,Title='Base row up 12',/row,/align_center)
sw_ba_up21=widget_base(sw_ba_up2,Title='Base row up 21',/row,/align_center)
sw_ba_up22=widget_base(sw_ba_up2,Title='Base row up 22',/row,/align_center)
sw_ba_up31=widget_base(sw_ba_up3,Title='Base row up 31',/row,/align_center)
sw_ba_up31i=widget_base(sw_ba_up3,Title='Base row up 31i',/row,/align_center)
sw_ba_up32=widget_base(sw_ba_up3,Title='Base row up 32',/row,/align_center)
sw_ba_up31p=widget_base(sw_ba_up3p,Title='Base row up 31p',/row,/align_center)
sw_ba_up32p=widget_base(sw_ba_up3p,Title='Base row up 32p',/row,/align_center)
sw_ba_up31pp=widget_base(sw_ba_up3pp,Title='Base row up 31pp',/column,/align_center)
sw_ba_up32pp=widget_base(sw_ba_up3pp,Title='Base row up 32pp',/column,/align_center)
;sw_ba_up31ppp=widget_base(sw_ba_up3ppp,Title='Base row up 31ppp',/row,/align_center)
;sw_ba_up32ppp=widget_base(sw_ba_up3ppp,Title='Base row up 32ppp',/row,/align_center)
sw_ba_up41=widget_base(sw_ba_up4,Title='Base row up 41',/row,/align_center)
sw_ba_up42=widget_base(sw_ba_up4,Title='Base row up 42',/row,/align_center)
; ** First up : button, labels and texts
;sw_bu_gwin=widget_button(sw_ba_up11,Font=ft_b_smaller,Frame=2,/menu,value='Get W_in',uvalue=[1,1,0],/align_center)
sw_bu_gwin=widget_button(sw_ba_up11,Font=ft_b_smaller,Frame=2,/menu,value='Get W_in',/align_center)
sw_bu_gwin_here=widget_button(sw_bu_gwin,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,1,0])
sw_bu_gwin_1to20=widget_button(sw_bu_gwin,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_win=widget_button(sw_bu_gwin_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,1,i])
sw_bu_gwin_21to40=widget_button(sw_bu_gwin,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_win=widget_button(sw_bu_gwin_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,1,i])
sw_bu_gwin_41to60=widget_button(sw_bu_gwin,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_win=widget_button(sw_bu_gwin_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,1,i])
;sw_bu_gwin=widget_button(sw_ba_up11,Font=ft_b_smaller,Frame=2,/menu,value='Get W_in',uvalue=[1,1,0],/align_center)
;for i=0,19 do sw_la_gwin=widget_button(sw_bu_gwin,Font=ft_b_smaller,Frame=2,/menu,value='W1 to W20',uvalue=[1,1,0],/align_center)
;sw_la_gwin=widget_button(sw_bu_gwin,Font=ft_b_smaller,Frame=2,/menu,value='W21 to W40',uvalue=[1,1,0],/align_center)
;sw_la_gwin=widget_button(sw_bu_gwin,Font=ft_b_smaller,Frame=2,/menu,value='W41 to W60',uvalue=[1,1,0],/align_center)


sw_bu_gres=widget_button(sw_ba_up21,Font=ft_b_smaller,Frame=2,/menu,value='Get Res_fun',/align_center,sensitive=0)
sw_bu_gres_here=widget_button(sw_bu_gres,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,2,0])
sw_bu_gres_1to20=widget_button(sw_bu_gres,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_res=widget_button(sw_bu_gres_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,2,i])
sw_bu_gres_21to40=widget_button(sw_bu_gres,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_res=widget_button(sw_bu_gres_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,2,i])
sw_bu_gres_41to60=widget_button(sw_bu_gres,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_res=widget_button(sw_bu_gres_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,2,i])
w_bu_gres_makeres=widget_button(sw_bu_gres,Font=ft_b_smaller,value='Make a Gaussian RF',/align_center,uvalue=[1,2,999])
;
sw_bu_wwou=widget_button(sw_ba_up31,Font=ft_b_smaller,Frame=2,/menu,value='Write Param',/align_center,sensitive=0)
sw_bu_wwou_here=widget_button(sw_bu_wwou,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,3,0])
sw_bu_wwou_1to20=widget_button(sw_bu_wwou,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_wou=widget_button(sw_bu_wwou_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,3,i])
sw_bu_wwou_21to40=widget_button(sw_bu_wwou,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_wou=widget_button(sw_bu_wwou_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,3,i])
sw_bu_wwou_41to60=widget_button(sw_bu_wwou,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_wou=widget_button(sw_bu_wwou_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,3,i])

sw_bu_wpara_in=widget_button(sw_ba_up31i,Font=ft_b_smaller,Frame=2,/menu,value='Import Param',/align_center,sensitive=0)
sw_bu_wpara_in_here=widget_button(sw_bu_wpara_in,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,33,0])
sw_bu_wpara_in_1to20=widget_button(sw_bu_wpara_in,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_para_in=widget_button(sw_bu_wpara_in_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,33,i])
sw_bu_wpara_in_21to40=widget_button(sw_bu_wpara_in,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_para_in=widget_button(sw_bu_wpara_in_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,33,i])
sw_bu_wpara_in_41to60=widget_button(sw_bu_wpara_in,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_para_in=widget_button(sw_bu_wpara_in_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,33,i])

sw_bu_wcur=widget_button(sw_ba_up31p,Font=ft_b_smaller,Frame=2,/menu,value='Write active spec.',/align_center,sensitive=0)
sw_bu_wcur_here=widget_button(sw_bu_wcur,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,4,0])
sw_bu_wcur_1to20=widget_button(sw_bu_wcur,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_cur=widget_button(sw_bu_wcur_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,4,i])
sw_bu_wcur_21to40=widget_button(sw_bu_wcur,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_cur=widget_button(sw_bu_wcur_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,4,i])
sw_bu_wcur_41to60=widget_button(sw_bu_wcur,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_cur=widget_button(sw_bu_wcur_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,4,i])

sw_bu_wsf=widget_button(sw_ba_up31pp,Font=ft_b_smaller,Frame=2,/menu,value='Write Sub_func',/align_center,sensitive=0)
sw_bu_wsf_here=widget_button(sw_bu_wsf,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,5,0])
sw_bu_wsf_1to20=widget_button(sw_bu_wsf,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_sf=widget_button(sw_bu_wsf_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,5,i])
sw_bu_wsf_21to40=widget_button(sw_bu_wsf,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_sf=widget_button(sw_bu_wsf_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,5,i])
sw_bu_wsf_41to60=widget_button(sw_bu_wsf,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_sf=widget_button(sw_bu_wsf_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,5,i])
;
sw_bu_wall=widget_button(sw_ba_up41,Font=ft_b_smaller,Frame=2,/menu,value='Write All',/align_center,sensitive=0)
sw_bu_wall_here=widget_button(sw_bu_wall,Font=ft_b_smaller,value='Here',/align_center,uvalue=[1,6,0])
sw_bu_wall_1to20=widget_button(sw_bu_wall,Font=ft_b_smaller,/menu,value='W1 to W20',/align_center)
for i=1,20 do sw_bu_sf=widget_button(sw_bu_wall_1to20,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,6,i])
sw_bu_wall_21to40=widget_button(sw_bu_wall,Font=ft_b_smaller,/menu,value='W21 to W40',/align_center)
for i=21,40 do sw_bu_sf=widget_button(sw_bu_wall_21to40,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,6,i])
sw_bu_wall_41to60=widget_button(sw_bu_wall,Font=ft_b_smaller,/menu,value='W41 to W60',/align_center)
for i=41,60 do sw_bu_sf=widget_button(sw_bu_wall_41to60,xsize=50,value='W'+strcompress(string(i),/remove_all),uvalue=[1,6,i])

sw_dl_wsf=widget_droplist(sw_ba_up32pp,Font=ft_b_smaller,xsize=100,value=listw,sensitive=0)

;sw_bu_winb=widget_button(sw_ba_up12,Font=ft_smaller,Frame=2,value='<<',uvalue=[2,1,1],/align_center)
sw_la_win=widget_label(sw_ba_up12,Font=ft_b_smaller,value=str_win,/align_center,xsize=50)
;sw_la_win=widget_text(sw_ba_up12,Font=ft_b_smaller,Frame=2,xsize=4,value=str_win,uvalue=[2,1,2],/align_center,/editable)
;sw_la_win=widget_droplist(sw_ba_up12,Frame=2,xsize=50,value=listw)
;sw_la_win=widget_droplist(sw_ba_up12,Frame=2,xsize=20,value=listw_1to20)
;sw_la_win=widget_droplist(sw_ba_up12,Frame=2,xsize=20,value=listw_21to40)
;sw_bu_winf=widget_button(sw_ba_up12,Font=ft_smaller,Frame=2,value='>>',uvalue=[2,1,3],/align_center)
;sw_bu_resb=widget_button(sw_ba_up22,Font=ft_smaller,Frame=2,value='<<',uvalue=[2,2,1],/align_center)
;sw_la_res=widget_label(sw_ba_up22,Font=ft_b_smaller,Frame=2,value=str_wres,uvalue=[2,2,2],/align_center)
;sw_la_res=widget_text(sw_ba_up22,Font=ft_b_smaller,Frame=2,xsize=4,value=str_res,uvalue=[2,2,2],/align_center,/editable)
;sw_la_res=widget_droplist(sw_ba_up22,Frame=2,xsize=50,value=listw)
sw_la_res=widget_label(sw_ba_up22,Font=ft_b_smaller,value=str_wres,/align_center,xsize=100)

;sw_bu_resf=widget_button(sw_ba_up22,Font=ft_smaller,Frame=2,value='>>',uvalue=[2,2,3],/align_center)
;sw_bu_woub=widget_button(sw_ba_up32,Font=ft_smaller,Frame=2,value='<<',uvalue=[2,3,1],/align_center)
;sw_la_wou=widget_label(sw_ba_up32,Font=ft_b_smaller,Frame=2,value=str_wou,uvalue=[2,3,2],/align_center)
;sw_la_wou=widget_text(sw_ba_up32,Font=ft_b_smaller,Frame=2,xsize=4,value=str_wou,uvalue=[2,3,2],/align_center,/editable)
;sw_la_wou=widget_droplist(sw_ba_up32,Frame=2,xsize=50,value=listw)
sw_la_wou=widget_label(sw_ba_up32,Font=ft_b_smaller,value=str_wou,/align_center,xsize=50)

;sw_la_wcur=widget_droplist(sw_ba_up32p,Frame=2,xsize=50,value=listw)
sw_la_wcur=widget_label(sw_ba_up32p,Font=ft_b_smaller,value=str_wcur,/align_center,xsize=50)
;
sw_la_wsf=widget_label(sw_ba_up31pp,Font=ft_b_smaller,value=str_wsf,/align_center,xsize=50)
sw_la_nsf=widget_label(sw_ba_up32pp,Font=ft_b_smaller,value='',/align_center,xsize=50,ysize=14)
sw_la_wall=widget_label(sw_ba_up42,Font=ft_b_smaller,value=str_wall,/align_center,xsize=50,ysize=14)

;
;sw_bu_rouf=widget_button(sw_ba_up32,Font=ft_smaller,Frame=2,value='>>',uvalue=[2,3,3],/align_center)
;sw_la_reslim=widget_label(sw_ba_up40,Font=ft_smaller,value='Limits for Res. fun.:+/-',/align_center)
;sw_la_resmin=widget_label(sw_ba_up41,Font=ft_smaller,value='xmin=  ',/align_center)
;sw_la_resmax=widget_label(sw_ba_up42,Font=ft_smaller,value='xmax=  ',/align_center)
;sw_tx_resmin=widget_text(sw_ba_up41,Font=ft_smaller,/editable,xsize=10,value='-1',/align_center)
;sw_tx_resmax=widget_text(sw_ba_up42,Font=ft_smaller,/editable,xsize=10,value=' 1',/align_center)
;sw_la_mev=widget_label(sw_ba_up41,Font=ft_smaller,value='  meV/mmeV',/align_center)
;sw_la_mev=widget_label(sw_ba_up42,Font=ft_smaller,value='  meV/mmeV',/align_center)

; ** Second mid : bases
sw_ba_mid=widget_base(sw_ba_leader,Title='Base row mid',frame=1,/row)
sw_ba_mid1=widget_base(sw_ba_mid,Title='Base row mid 1',/column)
sw_ba_mid2=widget_base(sw_ba_mid,Title='Base row mid 2',/column,/align_center)

;sw_ba_mid3=widget_base(sw_ba_mid,Title='Base row mid 3',/column)
sw_ba_mid10=widget_base(sw_ba_mid1,Title='Base row mid 10',/column,frame=1,/align_center)
sw_ba_mid11=widget_base(sw_ba_mid1,Title='Base row mid 11',/column,frame=1,/align_center)
;sw_ba_mid12=widget_base(sw_ba_mid1,Title='Base row mid 12',/column,frame=1,/align_center)
sw_ba_mid13=widget_base(sw_ba_mid1,Title='Base row mid 13',/column,frame=1,/align_center)

sw_ba_mid10up=widget_base(sw_ba_mid10,Title='Base row mid 11up',/row,/align_center)
sw_ba_mid10bot=widget_base(sw_ba_mid10,Title='Base row mid 11bot',/row,/align_center)

sw_ba_mid11upper=widget_base(sw_ba_mid11,Title='Base row mid 11upper',/row,/align_center)
sw_ba_mid11up=widget_base(sw_ba_mid11,Title='Base row mid 11up',/row,/align_center)
sw_ba_mid11midup=widget_base(sw_ba_mid11,Title='Base row mid 11bot',/row,/align_center)
sw_ba_mid11middw=widget_base(sw_ba_mid11,Title='Base row mid 11bot',/row,/align_center)
sw_ba_mid11bot=widget_base(sw_ba_mid11,Title='Base row mid 11bot',/row,/align_center,/nonexclusive)

;sw_ba_mid12up=widget_base(sw_ba_mid12,Title='Base row mid 12up',/row)
;sw_ba_mid12bot=widget_base(sw_ba_mid12,Title='Base row mid 12bot',/row,/nonexclusive)


;sw_ba_mid14=widget_base(sw_ba_mid1,Title='Base row mid 14',/row)
;sw_ba_mid15=widget_base(sw_ba_mid1,Title='Base row mid 15',/row)

; ** Second mid : button, labels and txtboxes
sw_la_reslim=widget_label(sw_ba_mid10up,Font=ft_b_smaller,value='--------** Range for Resolution Function-**---------',/align_center)
sw_la_range=widget_label(sw_ba_mid10bot,Font=ft_smaller,value='X range : +/-',/align_center)
sw_tx_resmax=widget_text(sw_ba_mid10bot,Font=ft_smaller,/editable,xsize=5,value='   1.000',/align_center)
sw_la_mev=widget_label(sw_ba_mid10bot,Font=ft_smaller,value='  meV/mmeV',/align_center)
;sw_la_resmin=widget_label(sw_ba_mid10bot,Font=ft_smaller,value='+/-',/align_center)
;sw_la_resmax=widget_label(sw_ba_up42,Font=ft_smaller,value='xmax=  ',/align_center)
;sw_tx_resmin=widget_text(sw_ba_up41,Font=ft_smaller,/editable,xsize=10,value='-1',/align_center)
;sw_tx_resmax=widget_text(sw_ba_up42,Font=ft_smaller,/editable,xsize=10,value=' 1',/align_center)
;sw_la_mev=widget_label(sw_ba_up41,Font=ft_smaller,value='  meV/mmeV',/align_center)
;sw_la_mev=widget_label(sw_ba_up42,Font=ft_smaller,value='  meV/mmeV',/align_center)
sw_la=widget_label(sw_ba_mid11upper,Font=ft_b_smaller,value='--------------** Range for Plots **--------------------',/align_center)
sw_la_xrange=widget_label(sw_ba_mid11up,Font=ft_smaller,value='X range:')
sw_tx_xmin=widget_text(sw_ba_mid11up,Font=ft_smaller,/editable,value='min',xsize=5,uvalue=[3,2,1])
sw_tx_xmax=widget_text(sw_ba_mid11up,Font=ft_smaller,/editable,value='max',xsize=5,uvalue=[3,2,1])
sw_la_mev=widget_label(sw_ba_mid11up,Font=ft_smaller,value='  meV/mmeV',/align_center)
sw_la_yrange=widget_label(sw_ba_mid11midup,Font=ft_smaller,value='Y range:')
sw_tx_ymin=widget_text(sw_ba_mid11midup,Font=ft_smaller,/editable,value='min',xsize=5,uvalue=[3,2,1])
sw_tx_ymax=widget_text(sw_ba_mid11midup,Font=ft_smaller,/editable,value='max',xsize=5,uvalue=[3,2,1])
sw_la_mev=widget_label(sw_ba_mid11midup,Font=ft_smaller,value='  meV/mmeV',/align_center)

sw_la_wsub=widget_label(sw_ba_mid11middw,Font=ft_smaller,value='Sp_#') & blabla=widget_label(sw_ba_mid11middw,Font=ft_smaller,value='',xsize=15)
sw_tx_wsub=widget_text(sw_ba_mid11middw,Font=ft_smaller,/editable,value=' 1',xsize=2,uvalue=[3,2,1]) & sw_tx_fwsub=sw_tx_wsub
blabla=widget_label(sw_ba_mid11middw,Font=ft_smaller,value='',xsize=15)
sw_bu_infsp=widget_button(sw_ba_mid11middw,value="<<",uvalue=[3,2,4],sensitive=0)
sw_bu_supsp=widget_button(sw_ba_mid11middw,value=">>",uvalue=[3,2,5],sensitive=0)
sw_bu_plot=widget_button(sw_ba_mid11middw,Font=ft_smaller,frame=2,value="OK",uvalue=[3,2,1],sensitive=0)

sw_bu_plre=widget_button(sw_ba_mid11bot,value="Res. Func.",uvalue=[3,2,2])
sw_bu_norm=widget_button(sw_ba_mid11bot,value="Norm.?",uvalue=[3,2,3])

; ** **********************************************************************************************
; ** Second up : button, labels and text boxes
;
sw_la_sdl=widget_label(sw_ba_mid13,Font=ft_b_smaller,value='----------------** Fitting Section **--------------------')
sw_ba_mid13up=widget_base(sw_ba_mid13,Title='Base row mid 13up',/row,/align_center)
sw_ba_mid13mid=widget_base(sw_ba_mid13,Title='Base row mid 13mid',/row,/align_center)
sw_ba_mid13bot=widget_base(sw_ba_mid13,Title='Base row mid 13bot',/row,/align_center)
sw_bu_done=widget_button(sw_ba_mid13bot,Font=ft_smaller,Frame=1,value='Done',xsize=100,uvalue=[6,2,0],sensitive=0)
sw_la_xrange=widget_label(sw_ba_mid13mid,Font=ft_smaller,value='Fit X range:')
sw_tx_fxmin=widget_text(sw_ba_mid13mid,/editable,value='min',xsize=5);,uvalue=[6,0,2])
sw_tx_fxmax=widget_text(sw_ba_mid13mid,/editable,value='max',xsize=5);,uvalue=[6,0,3])
sw_la_mev=widget_label(sw_ba_mid13mid,Font=ft_smaller,value='  meV/mmeV',/align_center)


blabla=widget_label(sw_ba_mid13up,value='Engine: ') & sw_dl_engine=widget_droplist(sw_ba_mid13up,value=['MPFIT','COOLFIT'],uvalue=[6,3,0]) & str_engine='MPFIT'
sw_bu_fit=widget_button(sw_ba_mid13up,Font=ft_b_smaller,Frame=2,value='   FIT !   ',uvalue=[6,0,1],sensitive=0)
;sw_tx_cycl=widget_text(sw_ba_mid13up,Font=ft_smaller,/editable,value=' 5',xsize=3);,uvalue=[6,0,1])
;sw_la_cycl=widget_label(sw_ba_mid13up,Font=ft_smaller,value='Cycles')

; ** *********************************************************************************************
; ** Second mid : graph
sw_pl_area=widget_draw(sw_ba_mid2,/Button_Events,Frame=5,Retain=2,Colors=-8, $
                 XSize=0.6*xsiz,YSize=1.5*ysiz ,uvalue=[7])
;                 XSize=420,YSize=400 ,uvalue=[7])             
;                 XSize=3*xsiz/4,YSize=18*ysiz/16 ,uvalue=[7])
;uvalue=[3,3,1],Colors=-8, $
;                XSize=3*xsiz/4,YSize=14*ysiz/16)
;sw_pl_areabot=widget_draw(sw_ba_mid2,/Button_Events,Frame=5, $
;                 XSize=3*xsiz/4,YSize= 4*ysiz/16 ,uvalue=[7])
;uvalue=[3,3,2],Colors=-8, $
;                XSize=3*xsiz/4,YSize=4*ysiz/16)
szsl=0 & str=''
;if sys_dep('VERSION') ge 4.0 then ii=execute('str=widget_info(peak_menu2,/geometry) & szsl=str.xsize')
bid=sys_dep('DYNLAB', sw_ba_leader, 1)
bid=str_fit(/none) ; to compile mp2cool ...

;widget_control, bad_id=i, sw_ba_leader, /Realize & put_logo

; ** Realization of the Base
widget_control, sw_ba_leader, /Realize; & put_logo
xmanager,'qens_fit',sw_ba_leader,/just_reg

; ** Get the window Id for the plotting area and store it in s_pl_wid
widget_control, bad_id=i, sw_pl_area, get_value=s_pl_wid    ; Store the number assigned to this window in s_pl_wid
wset,s_pl_wid                          ; Set the s_pl_wid active

; ** Get the window Id for the plotting area #2 and store it in s_pl_widbot
;widget_control, bad_id=i, sw_pl_areabot, get_value=s_pl_widbot  ; Store the number assigned to this window in s_pl_wid
;wset,s_pl_widbot                          ; Set the s_pl_wid active


endif
; Get drawable window index
;widget_control, bad_id=i, gw_plot_area, get_value=g_plot_wid
;wset,g_plot_wid

;if lamp_b1 gt 0 then xmanager, 'qens_fit', sw_ba_leader, /just_reg $
;else g_char=0.4

;if lamp_b1 le 0 then xmanager, 'qens_fit', sw_ba_leader

end

