;+
;    FUNCTION corr_tof, w_in, det_eff = efficiency, frameoverlap = frameoverlap, $
;         bkgd_trans = bkgd_trans, ana_eff = ana_eff,nf=nf,widthb=widthb,propb=propb,$
;         gexp=gexp,gprop=gprop $
;          ef1, fr1, bg1
;
;For IN4, IN5, IN6 and D7 data.
;
;KEYWORDS:
; /det_eff     : corrects for energy-variation of detector efficiency
; /frameoverlap: subtracts a t^-(nf) tail from the beginning
;                of the time frame (default nf = 4)
;                nf power of the frame when the frameoverlap correction is selected. Default is nf=4
; /bkgd_trans  : subtracts a flat background in each detector,
;                found using a moving filter (For IN4, IN5 and IN6 only)
; /ana_eff     : correct for energy dependence of analyser transmission
;                (For D7 only)
;    widthb is the width (in channel) of the moving window used to evaluate the background value
;    default is widthb = 30
;    propb is the proportion of background to remove from the spectra
;    default is propb = 1.
;
; (ef1, fr1, and bg1 are obsolete and kept for backwards compatability)
;
;DIMENSIONS
;   w_in=w_out(nchannels,nphases*nspectra)
;
;COMMAND SYNTAX
;   w6=corr_tof(w5[,/det_eff][,/frameoverlap][,/bkgd_trans][,/ana_eff][,nf=nf][,widthb=widthb][,propb=propb])
;
;   (optional keywords shown in square brackets)
;
;                        KHA,JRS 30/05/08
;
; modified by SR on 24/03/09 to allow correction for TOFTOF data
; modified by SR on 15/10/10 for correct evaluation of the frameoverlap for IN4
; modified by SR on 18/10/10 to incorporate the nf, widthb and propb keywords
; modified by SR on 06/11/13 to allow correction for FOCUS data
; modified by ARW on 3.2.15 to correct some bugs with D7 (detector efficiency, loading and correcting for analyser efficiency)
;
; par -> parp (due to a DIFF procedure having the name PAR !!!!)
; FIX -> long (FIX is a short integer !!!)
;-

FUNCTION read_trans, input, nchannels, lambda, wave, new
;+
;******* **********
;** cor_toff stuff
;-

    line = ''
    ON_IOERROR, end_of_file
    A = FLTARR(3,10000)
    B = FLTARR(3)
    OPENR, lun, input, /GET_LUN
    FOR iline = 0, 9999 DO BEGIN
        READF, lun, B
        A[*,iline] = B
    ENDFOR
    
    end_of_file:
    FREE_LUN, lun
    npts = iline
    A = A[*,0:npts - 1]
    Wax    = FLTARR(npts) & Trx    = Wax    & dTrx    = Wax
    Wax[*] = A[0,*]          & Trx[*] = A[1,*] & dTrx[*] = A[2,*]
    coeff  = POLY_FIT(Wax,Trx,9)
    trans1 = INTERPOL(Trx,Wax,wave)
    trans2 = FLOAT(POLY(wave,coeff))
    
    i1 = 0    & i2 = 0
    FOR i=1,nchannels-1 DO BEGIN
        IF (wave[i-1] LT 2.4) AND (wave[i] GE 2.4) THEN i1=i
        IF (wave[i-1] LT 3.5) AND (wave[i] GE 3.5) THEN i2=i
        IF (wave[i-1] LE lambda) AND (wave[i] GT lambda) THEN i0=i
    ENDFOR
    IF NOT NEW THEN BEGIN
        trans=trans2
        trans[i1:i2]=trans1[i1:i2]    ; interpolate to get bump and dip
    ; at lambda=2.5 to 3 Angstrom
    ENDIF ELSE BEGIN
        trans = trans2
    ENDELSE
    transm=trans[i0]+(lambda-wave[i0]) $
        *(trans[i0+1]-trans[i0])/(wave[i0+1]-wave[i0])
    trans=trans/transm
    RETURN, trans
END

; --------------------------------------------------------------------------
;***************************************************************************

FUNCTION corr_tof_tof, w_in, efficiency, frameoverlap, nf, bkgd, widthb, propb,gexp,fdelay,Wfplot
;+
;******* ************
;** cor_tof stuff
;-

    COMMON c_lamp_access, inst
    COMMON printing, iprint, outstring
    
    IF iprint THEN PRINT,'Start corr_tof_tof:'
    
    take_datp, datp
    
    insU=strupcase(inst)
    ; ------------------------------------------------------------------------------
    ;Set up input workspace and energy parameters
    
    parp   = datp.p
    
    lambda = parp[21]
    chw    = parp[18]
    chel   = parp[9]           ; elastic channel (if IN5, for high-angle detectors)
    L2     = parp[27]*1000.        ; sample-detector distance (mm)
    IF (insU EQ 'IN5') THEN BEGIN
        L2det = L2          ; sample-detector distance (mm)
        L2mul = L2det-300.    ; sample-multidetector distance
    ENDIF ELSE IF (insU EQ 'IN6') THEN $
        parp[14]=1.        ; rep period multiplier is 1 on IN6
        
    period = parp[13]*parp[14]    ; repetition period (mcs)
    IF (insU EQ 'IN4') THEN period = parp[13]*ROUND((parp[19]/parp[14]/4)) ; repetition period is FCspeed/(BCspeed*4) on IN4
    
    
    corr_e = efficiency
    corr_f = frameoverlap
    corr_b = bkgd
    
    IF iprint THEN BEGIN
        PRINT,'lambda=',lambda,'A, chw=',chw,'mcs, chel=',chel
        PRINT,' L2=',L2,'mm period=',period,'mcs'
        PRINT,' corr_e=',corr_e,' corr_f=',corr_f,' corr_b=',corr_b
    ENDIF
    
    sw = SIZE(w_in)
    IF iprint THEN PRINT,'SIZE(w_in)=',sw
    nchannels = sw[1]
    nruns     = 1
    IF sw[0] EQ 1 THEN nspectra = 1 ELSE nspectra = sw[2]
    IF sw[0] EQ 3 THEN nruns    = sw[3]
    
    IF iprint THEN PRINT,'nchannels=',nchannels,' nspectra=',nspectra,' nruns=',nruns
    x_in = datp.x
    y_in = datp.y
    e_in = datp.e
    
    se = SIZE(e_in)
    IF (se[0] NE sw[0] OR se[1] NE sw[1] OR se[2] NE sw[2]) THEN e_in=SQRT(w_in)
    
    IF(iprint GT 0) THEN PRINT,'corr_tof: End of check dimensions section'
    
    ; ------------------------------------------------------------------------------------
    ;   Set constants and prepare output arrays
    
    const1 = 5.22697    ; E(meV)=const1*V(m/ms)^2 for neutron
    const2 = 2.07193571    ; E(meV)=const2*k(A^-1)^2 for neutron
    const3 = 3.956076    ; V(m/ms)=const3/lambda(A) for neutron
    const4 = 81.8066    ; E(meV)=const4/lambda(A)^2 for neutron
    
    w_out = w_in*1.0 ; should be float variable
    e_out = e_in*1.0
    x_out = FLTARR(nchannels)
    y_out = y_in
    
    Ei=const4/lambda^2
    Vi=SQRT(Ei/const1)
    
    Tsam = chel*chw - L2/Vi
    
    IF (insU EQ 'IN5') THEN BEGIN
        cheldet = chel      & Teldet = L2det/Vi
        timedet = chw*FLOAT(x_in-cheldet)+Teldet
        Efdet   = const1*(L2det/timedet)^2
        Epsdet  = Ei-Efdet
        chelmul = chel-300./Vi    & Telmul=L2mul/Vi
        timemul = chw*FLOAT(x_in-chelmul)+Telmul
        Efmul   = const1*(L2mul/timemul)^2
        Epsmul  = Ei-Efmul
    ENDIF ELSE BEGIN
        Tel  = L2/Vi
        time = chw*FLOAT(x_in-chel) + Tel
        Ef   = const1*(L2/time)^2
        Eps  = Ei-Ef
        wave = SQRT(const4/Ef)
    ENDELSE
    
    ; ------------------------------------------------------------------------------------
    ;   Perform background correction
    
    IF corr_b THEN BEGIN
        iw=widthb
        FOR irun = 0, nruns - 1 DO BEGIN
            FOR ispec = 0,nspectra - 1 DO BEGIN
                S    = w_out[*,ispec,irun]*1.0
                Ss   = SMOOTH(S, 2*iw + 1)
                bkgd = MIN(Ss[iw:nchannels-iw-1]) > 0. & if (iprint and bkgd gt 0.) then print,'bkgd=',bkgd
                w_out[*,ispec,irun] = S - propb*bkgd
            ENDFOR
        ENDFOR
    ENDIF
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_tof: End of background correction section'
    ; ------------------------------------------------------------------------------------
    ;   Prepare for frame overlap correction
    
    IF corr_f THEN BEGIN
        nfit  = 10.
        ifit1 = nchannels - nfit
        ifit2 = nchannels - 1
        IF insU EQ 'IN5' THEN BEGIN
            Tdet = chw*(FLOAT(x_in[ifit1]+x_in[ifit2])/2.-cheldet)+Teldet
            time1det = timedet + period
            Tmul = chw*(FLOAT(x_in[ifit1]+x_in[ifit2])/2.-chelmul)+Telmul
            time1mul = timemul + period
        ENDIF ELSE BEGIN
            T=chw*(FLOAT(x_in[ifit1]+x_in[ifit2])/2.-chel)+Tel ; mean time of integrating region
            time1=time+period+fdelay*chw   ; frame-overlap time-of-flight
            Ef1   = const1*(L2/time1)^2 ; Final energy for second period
            EfT = const1*(L2/T)^2
        ENDELSE
    ENDIF ELSE BEGIN
        T=0.    & time1=0.
        Tdet=0. & time1det=0.
        Tmul=0. & time1mul=0.
    ENDELSE
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_tof: End of frame overlap correction section'
    ; ------------------------------------------------------------------------------------
    ;   Prepare for detector efficiency correction
    
    IF (corr_e EQ 0) THEN BEGIN
        eff=1.
        effdet=1.
        effmul=1.
    ENDIF ELSE BEGIN
        IF (insU EQ 'IN5') THEN BEGIN   ; use expressions for D7
            effdet=(1.-exp(-5.6/SQRT(Efdet)))/(1.-exp(-5.6/SQRT(Ei)))
            effmul=(1.-exp(-5.6/SQRT(Efmul)))/(1.-exp(-5.6/SQRT(Ei)))
        ENDIF ELSE IF (insU EQ 'IN6' OR insU EQ 'DCSASC') THEN BEGIN ; use expressions from Y. Blanc's report
            eff=0.94*(1.-EXP(-0.363*wave)) & i=WHERE(wave GE 4., n)
            IF (n GT 0) THEN eff[i]=EXP(-0.00625*wave[i])*(1.-EXP(-0.363*wave[i]))
            eff0=EXP(-0.00625*lambda)*(1.-EXP(-0.363*lambda))
            eff =eff/eff0    ; normalise to 1 at elastic peak
        ENDIF ELSE IF  (insU EQ 'TOFTOF') THEN BEGIN ; uses expression from T.Unruh published in Nuclear. Inst. and Methods in Physics A
            C1=-0.31 & C2=-9.3518
            eff =exp(C1/sqrt(Ef))*(1-exp(C2/sqrt(Ef)))
            eff0=exp(C1/sqrt(Ei))*(1-exp(C2/sqrt(Ei)))
            eff=eff/eff0
        ENDIF ELSE IF  (insU EQ 'IN4') THEN BEGIN ; uses expression from Rieutord's ILL Report concerning the INX software
            ;eff0=0.951*exp(-0.0887/sqrt(Ei))*(1.-exp(-5.597/sqrt(Ei)))
            ;eff=0.951*exp(-0.0887/sqrt(Ef))*(1.-exp(-5.597/sqrt(Ef)))
            A = 0.951 & B = -0.0887 & C = -5.597
            eff0 = A*exp(B/sqrt(Ei))*(1.-exp(C/sqrt(Ei)))
            eff = A*exp(B/sqrt(Ef))*(1.-exp(C/sqrt(Ef)))
            eff=eff/eff0 ; normalise to 1 @ elastic
        ENDIF ELSE IF  (insU EQ 'MIBEMOL') THEN BEGIN ; uses expression from Rieutord's ILL Report concerning the INX software
            eff0=(1.-exp(-2.49/sqrt(Ei)))
            eff=(1.-exp(-2.49/sqrt(e)))
            eff=eff/eff0 ; normalise to 1 @ elastic
        ENDIF ELSE $        ; use expressions for D7
            eff=(1.-exp(-5.6/SQRT(Ef)))/(1.-exp(-5.6/SQRT(Ei)))
    ENDELSE
    
    ; ------------------------------------------------------------------------------------
    ;   Perform frame-overlap and detector efficiency corrections
    
    FOR irun = 0, nruns - 1 DO BEGIN
        FOR ispec = 0, nspectra - 1 DO BEGIN
            S = w_out[*,ispec,irun]    & dS = e_out[*,ispec,irun]
            IF insU EQ 'IN5' THEN BEGIN
                IF y_in[ispec] LE 10. THEN BEGIN
                    T=Tmul & time1=time1mul & eff=effmul
                ENDIF ELSE BEGIN
                    T=Tdet & time1=time1det & eff=effdet
                ENDELSE
            ENDIF
            IF corr_f THEN BEGIN
                Y = TOTAL(S[ifit1:ifit2],1)/FLOAT(nfit)
                ;ratio_t=(time1^(-nf)-gexp*time1^(-nf-2)*Tel^2)/(T^(-nf)-gexp*T^(-nf-2)*Tel^2) & A=Y*ratio_t
                ratio_t=(time1^(-nf)*(1-Ef1/Ei)^gexp)/(T^(-nf)*(1-EfT/Ei)^gexp) & A=Y*ratio_t
                S=S-A
                if  (Wfplot GT 0) then begin
                    datp2={X:x_in,E:x_in*0.,  X_tit:'Channel', Y_tit:'Frame Overlap correction', W_tit:' ', Other_tit:' '}
                    GIVE_W,A,W=Wfplot & GIVE_DATP,datp2,W=Wfplot
                endif
                
            ;        A = Y*ratio_t^nf & S = S - A
            ;A = Y*(T^nf) & S = S - A/(time1^nf)
            ENDIF
            w_out[*,ispec,irun] = S/eff
            e_out[*,ispec,irun] = dS/eff
        ENDFOR
    ENDFOR
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_tof: End of main section'
    ; ------------------------------------------------------------------------------------
    s = ''
    IF corr_e THEN s = s + '/eff'
    IF corr_f THEN s = s + ',/frame'
    IF corr_b THEN s = s + ',/bkgd'
    
    datp.other_tit=datp.other_tit+' -ct('+s+')'
    
    IF corr_e THEN estring='with det eff corr'       ELSE estring='no det eff corr'
    IF corr_f THEN fstring='with frame overlap corr' ELSE fstring='no frame overlap corr'
    IF corr_b THEN bstring='with bkgd corr'          ELSE bstring='no bkgd corr'
    outstring = estring+', '+fstring+', '+bstring
    
    give_datp, datp
    
    finished:
    IF (iprint GT 0) THEN PRINT,'End corr_tof_tof:'
    
    RETURN, w_out
END


; ------------------------------------------------------------------------------
;*******************************************************************************

FUNCTION corr_tof_d7, w_in, ieff, iover, itrans, corr_b
;+
;******* ***********
;** corr_tof stuff for D7
;-

    COMMON c_lamp
    COMMON printing, iprint, outstring
    
    IF iprint THEN PRINT,'Start corr_tof_d7:'
    
    TAKE_DATP, datp
    
    ; ------------------------------------------------------------------------------
    ;Set up input file and energy parameters
    
    ;Find the analyser transmission files
    
    ii = ''
    ij = ''
    root = lamp_macro + '/D7'
    
    trans_file     = FILEPATH('transmission.dat',     ROOT_DIR = root)
    new_trans_file = FILEPATH('new_transmission.dat', ROOT_DIR = root)
    ii = file_search(trans_file)
    ij = file_search(new_trans_file)
    root = './'
    trans_file     = FILEPATH('transmission.dat',     ROOT_DIR = root)
    new_trans_file = FILEPATH('new_transmission.dat', ROOT_DIR = root)
    
    IF STRLEN(ii[0]) LT 1 THEN ii = file_search(trans_file)
    IF STRLEN(ij[0]) LT 1 THEN ij = file_search(new_trans_file)
    
    IF STRLEN(ii[0]) LT 1 AND STRLEN(ij[0]) LT 1 THEN BEGIN
        s = STRARR(2)
        s[0] = 'Corr_tof: cannot perform analyser efficiency correction'
        s[1] = '          Transmission files not found'
        ii = dialog_message(s, /ERROR)
        itrans = 0
    ENDIF ELSE BEGIN
        trans_file     = ii[0] ;analyser transmission from H2 experiment of Mompean
        new_trans_file = ij[0] ;new analyser transmission
    ENDELSE
    
    parp=datp.p
    
    lambda = parp[4]
    freq   = parp[5]
    chw    = parp[7]
    chel   = parp[9]
    TOF=long(parp[8])
    
    IF NOT TOF THEN BEGIN
        s = ' Corr_tof: Workspace data is not in TOF'
        ii = dialog_message(s, /ERROR)
        RETURN, w_in
    ENDIF
    
    nspectra  = long(parp[1])
    nphases   = long(parp[2])
    nchannels = long(parp[6])
    y_in      = datp.y
    e_in      = datp.e
    
    sw = SIZE(w_in)
    se = SIZE(e_in)
    
    IF (se[0] NE sw[0] OR se[1] NE sw[1]) THEN BEGIN
        PRINT,'Error - SIZE(w_in)=',sw
        PRINT,'      SIZE(e_in)=',se
        PRINT,'    - must be the same size'
        return,w_in
    ENDIF
    
    zeroed = WHERE(e_in LE -1.,n) ; zeroed channels
    IF iprint THEN PRINT,n/nchannels,' zeroed spectra'
    
    IF N_ELEMENTS(ieff)   NE 0 THEN eff0    = ieff   ELSE eff0   = 0
    IF N_ELEMENTS(itrans) NE 0 THEN trans0  = itrans ELSE trans0 = 0
    IF N_ELEMENTS(iover)  NE 0 THEN over0   = iover  ELSE over0  = 0
    IF N_ELEMENTS(corr_b) NE 0 THEN corr_b0 = corr_b ELSE corr_b0 = 0
    
    IF (iprint GT 0) THEN BEGIN
        PRINT,'efficiency   = ', eff0
        PRINT,'transmission = ', trans0
        PRINT,'overlap      = ', over0
        PRINT,'background   = ', corr_b0
    ENDIF
    
    IF (eff0 EQ 0 AND trans0 EQ 0 AND over0 EQ 0 AND corr_b0 EQ 0) THEN RETURN,w_in
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_d7: End of "w_in dimensions check etc." section'
    
    ; ------------------------------------------------------------------------------------
    ;   Set constants and prepare arrays
    
    eff  = FLTARR(nchannels)
    wave = FLTARR(nchannels)
    
    const1 = 5.22697    ; E(meV)=const1*V(m/ms)^2 for neutron
    const2 = 2.07193571    ; E(meV)=const2*Q(A^-1)^2 for neutron
    const3 = 3.956076    ; V(m/ms)=const3/lambda(A) for neutron
    const4 = 81.8066    ; E(meV)=const4/lambda(A)^2 for neutron
    
    L2 = 1500.0     ; sample-detector distance (mm)
    
    Ei  = const4/lambda^2
    Vi  = SQRT(Ei/const1)
    Tel = L2/Vi
    
    time = chw*FLOAT(INDGEN(nchannels) - chel) + Tel
    Ef   = const1*(L2/time)^2
    
;    eff = (1. - exp(-5.6/SQRT(Ef)))/(1.-exp(-5.6/SQRT(Ei)))
; ARW 3.2.15  Exponent = -0.51784*diameter(cm)*pressure(barr)	Bjorn Fak, from INX
; Diameter = 1 inch = 2.54 cm, pressure = 10 barr
    eff = (1. - exp(-0.51784*2.54*10.0/SQRT(Ef)))/(1.-exp(-0.51784*2.54*10.0/SQRT(Ei)))
    
    wave = SQRT(const4/Ef)
    
    IF (over0 EQ 1) THEN BEGIN
        freq   = freq/60.        ; convert to Hz
        period = 1.E+06/freq        ; period in mcs
        nfit   = 10            ; no.of points for fit of y=A*t^-4
        ifit1  = nchannels - nfit
        ifit2  = nchannels - 2
        T = chw*(FLOAT(ifit1 + ifit2)/2. - chel)+Tel ; mean time of integrating region
        time1  = time+period        ; frame-overlap time-of-flight
    ENDIF
    
    IF iprint THEN BEGIN
        PRINT,'lambda=',lambda,' Ei=',Ei,' freq=',freq,' chw=',chw
        PRINT,'Ef=',Ef
        PRINT,'eff=',eff
        PRINT,'wave=',wave
    ENDIF
    
    IF iprint THEN PRINT,'corr_tof_d7: End of "Set constants etc." section'
    
    ; ------------------------------------------------------------------------------
    ;Open and read from input file
    
    IF trans0 THEN BEGIN
        line=''
        IF STRLEN(trans_file) LT 5 THEN BEGIN
            s    = STRARR(2)
            s[0] = 'Corr_tof: Old analyser transmission curve not available'
            s[1] = '          Will use new analyser curve'
            old = 0
            ii = dialog_message(s)
        ENDIF ELSE BEGIN
            old = 1
            trans = read_trans(trans_file, nchannels, lambda, wave, 0)
        ENDELSE
        IF STRLEN(new_trans_file) LT 5 THEN BEGIN
            s    = STRARR(2)
            s[0] = 'Corr_tof: New analyser transmission curve not available'
            s[1] = '          Will use old analyser curve'
            new = 0
            ii = dialog_message(s)
        ENDIF ELSE BEGIN
            new = 1
            new_trans = read_trans(new_trans_file, nchannels, lambda, wave, 1)
        ENDELSE
        
        
        IF (iprint GT 0) THEN PRINT,'corr_tof_d7: End of "read from input files" section'
    ENDIF
    
    ; ------------------------------------------------------------------------------
    ;Perform correction
    
    w_out=w_in
    e_out=e_in
    
    ; ------------------------------------------------------------------------------------
    ;   Perform background correction
    IF corr_b0  THEN BEGIN
        iw=30
        FOR ispec=0,nphases*nspectra-1 DO BEGIN
            S=w_out[*,ispec]
            Ss=SMOOTH(S,2*iw+1)
            bkgd=MIN(Ss[iw:nchannels-iw-1])>0.
            IF (iprint GT 0) THEN BEGIN
                PRINT,'ispec=',ispec,' bkgd=',bkgd
            ENDIF
            w_out[*,ispec]=S-bkgd
        ENDFOR
    ENDIF
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_tof: End of background correction section'
    ; ------------------------------------------------------------------------------
    
    Ecorr=FLTARR(nchannels) & Ecorr[*]=1.
    
    IF eff0 EQ 1 THEN Ecorr=Ecorr/eff
    
    FOR ispec=0,nphases*nspectra-1 DO BEGIN
        S = w_out[*,ispec]    & dS = e_out[*,ispec]
        IF over0 EQ 1 THEN BEGIN
            Y=TOTAL(S[ifit1:ifit2],1)/FLOAT(nfit)
            A=Y*T^4
            S=S-A/time1^4
        ENDIF
        
        tr = 1.                        ; default not recommended!!!
        
        IF trans0 EQ 1 AND nspectra LE 50 THEN BEGIN    ;data previous to Oct 2006
            idet = ispec MOD nspectra
            IF idet LT 32 AND OLD THEN tr = trans
            IF NOT new THEN tr = trans
            IF NOT old THEN tr = new_trans
            IF idet GE 32 AND NEW THEN tr = new_trans
        ENDIF
; ARW 3.2.15  Changed to nspectra GE 106
;        IF trans0 EQ 1 AND nspectra EQ 106 THEN BEGIN    ;data after Oct 2006
        IF trans0 EQ 1 AND nspectra GE 106 THEN BEGIN    ;data after Oct 2006
            idet = ispec MOD nspectra
            IF idet LT 98 AND NEW THEN tr = new_trans
            IF NOT new THEN tr = trans
            IF NOT old THEN tr = new_trans
            IF idet GE 98 AND OLD THEN tr = trans
        ENDIF
        
;        w_out[*,ispec] = S*Ecorr*tr
;        e_out[*,ispec] = dS*Ecorr*tr
; ARW 3.2.15  Changed to S*Ecorr/tr
        w_out[*,ispec] = S*Ecorr/tr
        e_out[*,ispec] = dS*Ecorr/tr

    ENDFOR
    
    IF(n GT 0) THEN BEGIN
        w_out[zeroed]=0.    & e_out[zeroed]=-1.
    ENDIF
    
    IF (iprint GT 0) THEN PRINT,'corr_tof_d7: End of Correction section'
    
    ; ------------------------------------------------------------------------------
    ;Return parameters and exit
    
    datp.e=e_out
    
    s = ''
    IF eff0 THEN s = s + '/eff'
    IF over0 THEN s = s + ',/frame'
    IF trans0 THEN s = s + ',/trans'
    
    datp.other_tit = datp.other_tit+' -ct('+s+')'
    
    IF eff0    THEN estring='with det eff corr'    ELSE estring='no det eff corr'
    IF over0   THEN fstring='with frame ovlp corr' ELSE fstring='no frame ovlp corr'
    IF trans0  THEN tstring='with ana trans corr'  ELSE tstring='no ana trans corr'
    IF corr_b0 THEN bstring='with bkgd corr'       ELSE bstring='no bkgd corr'
    outstring = estring+', '+fstring+', '+tstring+', '+bstring
    
    finished:
    IF (iprint GT 0) THEN PRINT,'End corr_tof_d7:'
    
    give_datp, datp
    
    RETURN, w_out
END

; ------------------------------------------------------------------------------
;*******************************************************************************
;
FUNCTION corr_tof, w_in, det_eff = det_eff,frameoverlap = frameoverlap, $
                   bkgd_trans = bkgd_trans, ana_eff = ana_eff, nf=nf, gexp=gexp, fdelay=fdelay,widthb=widthb, propb=propb, rfplot=rfplot,$
                   ef1, fr1, bg1
;+
;******* ********
;** User callable
;** entry
;-
   
    COMMON c_lamp_access, inst
    COMMON c_lamp
    COMMON printing, iprint, outstring
    
    iprint = 0    ; if iprint>0, show debugging messages
    
    eff   = 0
    fo    = 0
    bkgd  = 0
    ana   = 0
    
    IF iprint THEN PRINT,'Start corr_tof:'
    IF N_ELEMENTS(ef1) GT 0 THEN eff  = ef1
    IF N_ELEMENTS(fr1) GT 0 THEN fo = fr1
    IF N_ELEMENTS(bg1) GT 0 THEN BEGIN
        ana   = bg1
        bkgd = 1
    ENDIF
    
    IF KEYWORD_SET(det_eff) THEN eff = 1
    IF KEYWORD_SET(frameoverlap) THEN BEGIN
        fo = 1 & Wfplot = 0.
        IF N_ELEMENTS(nf) EQ 0 THEN nf=4
        IF N_ELEMENTS(gexp) EQ 0 THEN gexp=0.
        IF N_ELEMENTS(fdelay) EQ 0 THEN fdelay=0.
        IF N_ELEMENTS(rfplot) NE 0 THEN Wfplot=abs(fix(rfplot))
        
    ENDIF
    IF KEYWORD_SET(bkgd_trans) THEN BEGIN
        bkgd = 1
        IF N_ELEMENTS(widthb) EQ 0 THEN widthb=30
        IF N_ELEMENTS(propb) EQ 0 THEN propb=1
    ENDIF
    IF KEYWORD_SET(ana_eff) THEN ana = 1
    
    ; ------------------------------------------------------------------------------
    ;Check instrument name and call appropriate function
    insU=strupcase(inst)
    IF (insU EQ 'IN4')     OR (insU EQ 'IN5')    OR (insU EQ 'IN6') OR $
        (insU EQ 'MIBEMOL') OR (insU EQ 'DCSASC') OR (insU EQ 'TOFTOF') OR (insU EQ 'FOCUS') THEN $
        w_out = corr_tof_tof(w_in, eff, fo, nf, bkgd, widthb, propb,gexp,fdelay,Wfplot) $
    ELSE IF (insU EQ 'D7') THEN $
    w_out = corr_tof_d7(w_in, eff, fo, ana, bkgd) $
ELSE BEGIN
s = 'Corr_tof: Instrument must be IN4, IN5, IN6, D7, mibemol, DCSasc or toftof'
ii = dialog_message(s, /ERROR)
RETURN, w_in
ENDELSE

; ------------------------------------------------------------------------------
;Return parameters and exit

PRINT,'corr_tof: '+outstring

finished:
RETURN, w_out
END
