FUNCTION gdos,w_in,Temp=temp,Emin=emin,Emax=emax,Inverse=inverse,LIMIT=limit,BREDOV=bredov,DW=dw
;+
;******* ****
;** User callable
;** S. Rols 11/01 srols@anl.gov
;** The call is w6=gdos(w_in,Temp=T)
;** Last modification 04.02.02 to include QENS_raw treatment
;** Last modification 09.09.09 to include the Limit keyword
;** Last modification 05 2010  to allow for Ostowskii formalism to be
;** performed instead of angle averaging...
;**
;-

COMMON c_lamp_access, inst
COMMON printing, iprint, outstring

take_datp, datp
par=datp.p
x_in=datp.x
y_in=datp.y
e_in=datp.e
; -------------------------------------------
; Donnees experimentales

;Print,'GDOS ## S.R 2002 ##'
lambda=0.

CASE inst OF
;        'MiBeMol':BEGIN
;        lambda=FLOAT(par(10))
;        temp=FLOAT(par(5))
;        ei=81.799/(lambda^2)+x_in*0.
;        END
        'DCSasc':BEGIN
        lambda=FLOAT(par[8])
        ei=81.799/(lambda^2)+x_in*0.
        END
        'QENS_raw':BEGIN
        ef=3.167 ;average final neutron energy in meV
        lambda=SQRT(81.799/ef)
        ei=x_in+ef
        END
        ELSE:BEGIN
        lambda=par[21]
        if (n_elements(temp) EQ 0) THEN temp=par[11]
        ei=81.799/(lambda^2)+x_in*0.
        END
ENDCASE
print,inst
IF (temp EQ 0.0) THEN temp=300.0
IF (n_elements(emax) EQ 0) THEN emax=MAX(x_in)
IF (n_elements(emin) EQ 0) THEN emin=MIN(x_in)
IF (n_elements(dw) EQ 0) THEN dw=0.

print, 'temp=',temp,'lambda=',lambda,'emin=',emin,'emax=',emax, 'DW =',dw

points=where(x_in ge emin and x_in le emax)

w_buf=w_in & e_buf=e_in & y_out=y_in
;
;Calcul de Q^2 et Debye Waller
;*****************************
q2=w_in*0.0 & deb_wal=q2
x_in=FLOAT(x_in)
FOR itet=0,n_elements(y_in)-1 DO q2[*,itet]=2*ei[*]-x_in[*]-2*sqrt(ei[*]*ABS(ei[*]-x_in[*]))*cos(y_in[itet]*!pi/180)
q2=q2/2.072
deb_wal=exp(-dw*q2)
;print,'q2=',q2
;
; Calcul de (Q_max^4 - Q_min^4) dans le cadre de l'approximation incoherente de Bredov
; on prend le cas IN4 ici donc Theta_min=15 et Theta_max=120
if (not keyword_set(bredov)) then goto, pas_bredov
if (n_elements(y_in) ne 1) then goto, pas_bredov
q4_bredov=x_in
theta_min=13. & theta_max=120.
x_in=FLOAT(x_in)
q4_bredov=(2*ei-x_in-2*sqrt(ei*ABS(ei-x_in))*cos(theta_max*!pi/180))^2 - (2*ei-x_in-2*sqrt(ei*ABS(ei-x_in))*cos(theta_min*!pi/180))^2
q4_bredov=q4_bredov/2.072/2.072
print,q4_bredov/q2
q2=q4_bredov
print,'** GDOS calculated using the incoherent approximation (BREDOV-OSTOWSKII formalism)'
goto, next
pas_bredov: print, '** GDOS calculated for an incoherent scatterer : averaged angle approximation, P(alpha, beta)'
next:
;
;Calcul de w/n(w) ou n(w) est le facteur de temperature dans le cas Stokes et anti Stokes
;****************************************************************************************
bosex=FLOAT(x_in)
bosex=bosex/(1-exp(-1.*bosex*11.6045/temp))
;
;Corrections
;******************************
indnul=WHERE(ABS(bosex) LE 1.e-12)
IF n_elements(indnul) GT 1 THEN BEGIN
w_buf[indnul,0:n_elements(y_in)-1]=0.
bosex[indnul]=1.
END
w_buf=w_in/q2/deb_wal
e_buf=e_buf/q2/deb_wal
FOR itet=0,n_elements(y_in)-1 DO BEGIN
w_buf[*,itet]=w_buf[*,itet]/bosex[*]
w_buf[*,itet]=w_buf[*,itet]*(x_in[*])^2
e_buf[*,itet]=e_buf[*,itet]/bosex[*]
e_buf[*,itet]=e_buf[*,itet]*(x_in[*])^2
ENDFOR
;
;output
x_out=x_in[points]
e_out=e_buf[points,*]
w_out=w_buf[points,*]
IF keyword_set(limit) THEN BEGIN
il=where(abs(w_out) ge limit)
if (il[0] ne -1) then begin
w_out[il]=0. & e_out[il]=0.
endif
ENDIF
IF (n_elements(inverse) EQ 0) THEN GOTO, output
IF (inverse EQ 1) THEN BEGIN
x_out=reverse(ABS(x_out))
e_out=reverse(e_out)
w_out=reverse(w_out)
ENDIF
output:
mod_datp, datp, "e", e_out
mod_datp, datp, "x", x_out
mod_datp, datp, "y", y_out
give_datp, datp
return, w_out
end
