;+    ****************************************************************
;     PROGRAMM MUPHOCOR/5       REICHARDT 5.11.1991
;     ****************************************************************
;     USE            : W1=muphocor("filename" [,/first_guess] [,multi_phonon=2])
;     INPUT          : "filename"
;     OUTPUT         : W1 is G(w) [first_guest if keyword is set]
;                    : multi_phonon=2 means get multi_phonon in Workspace W2

;     Version History: Fortran to Idl (Didier Richard Sept 2008) ";!!!"
;                      24 Sep 2008 V1 (tested with 2 inputs: almost ok)
;                             multi1.plot:   columns 1->6 identical (1000 pt)
;                             multi1.plot:   column  7    identical until pt 500 !!!
;                             multi1.plot:   column  8    identical until pt 500 !!!
;                             GDOS_plot.dat: columns 1->4 identical (1000 pt)
;                             GDOS_plot.dat: column  5    identical until pt 500 !!!
;                             DOS.dat:       columns 1->7 identical (463 pt)
;                             DSDO.plot:     columns 1->3 identical (1000 pt)
;                             DSDO.plot:     column  4    identical until pt 500 !!!
;                      26 Dec 2008 added output workspace for lamp
;
;-

pro muph_cor ,fifi, Wdos, Wmulti, title
;+
;** ********
;** main entry
;**   Dimensions
;**   **********
;**   muph_chn=1024   ;CHANNELS                           -> 600   ;!!! TO BE DYNAMIC
;**   muph_lm =1024   ;NUMBER OF DISCRET ENERGY INTERVALS -> 500*2 ;!!! TO BE DYNAMIC
;**   muph_cmo=10     ;DEBYE CUTOFF FREQUENCIES
;**   muph_cvr=100    ;TEMPERATURES etc...
;**   muph_trm=20     ;NUMBER OF MULTI-PHONON TERMS RESTRICTED TO 20 DUE TO DIMENSION
;**   muph_ers=3000   ;BUFFER filled in RESCOR, used in FOLD -> 2400 muph_lm*DELE*10 ;!!!
;**   muph_two=2      ;NSPEC
;**   ******
;-
      muph_chn=1024   ;CHANNELS                           -> 600   ;!!! TO BE DYNAMIC
      muph_lm =1024   ;NUMBER OF DISCRET ENERGY INTERVALS -> 500*2 ;!!! TO BE DYNAMIC
      muph_cmo=10     ;DEBYE CUTOFF FREQUENCIES
      muph_cvr=100    ;TEMPERATURES etc...
      muph_trm=20     ;NUMBER OF MULTI-PHONON TERMS RESTRICTED TO 20 DUE TO DIMENSION
      muph_ers=3000   ;BUFFER filled in RESCOR, used in FOLD -> 2400 muph_lm*DELE*10 ;!!!
      muph_two=2      ;NSPEC
;     ******
      debug=0

      COMMON muph_RESO,  ERES,IFOLD
      COMMON muph_MULTI, GND,LM,LM1,LM2,DELE
      COMMON muph_ROS,   NU,NO,E0,FP,CW,XNEL
      COMMON muph_AVAR,  Z0,Z1,HOT,NM,NDUM,IGLU,UN
      COMMON muph_BB1,   PI,PIR,FIMI,FIMA,ABK,AEMP,NPHO,ITM,IDW,IEMP,IRES,IPR,IVIT, $
                         IPLOT,HOX,TEMP,DW,UNT,FUN,NSPEC,RMA,T0,T0R,IE0,ILOSS
      COMMON muph_BB2,   HOE,EFIN,EXE,EXA,COM,COMA,COMI,QC,QMA,QMI,DQUP
      COMMON muph_PAR1,  AMASI,SIGI,CONCI,ALFI,RNOR,QFG0,QFG1,QFG2,AMEFF
      COMMON muph_PAR2,  PARQ,IQUO,LPAR

;     ******
      Z0   =fltarr(muph_chn+3) ;for muph_mean
      UN   =fltarr(muph_chn) & HOT  =UN   & PARQ=UN
      HOE  =fltarr(muph_lm)  & EFIN =HOE  & EXE =HOE  & EXA  =HOE & QC =HOE & QMA=HOE
                               QMI  =HOE  & Z1  =HOE  & GND  =HOE & QMQ=HOE
      SIGI =fltarr(muph_two) & CONCI=SIGI & ALFI=SIGI & AMASI=SIGI
      DQUP =fltarr(muph_lm,muph_trm)
;     ******
      FNU  =fltarr(muph_lm)  & QUO  =FNU  & GNU0  =FNU  & GNU  =FNU & GAV =FNU
                               GGES =FNU  & DSIG  =FNU  & FIN  =FNU & FOUT=FNU
      CMO  =fltarr(muph_cmo) & GNUM =FNU  & GNUP  =FNU
                               QUO1 =FNU  & FNU1  =FNU  & QUOFG=FNU & GNS =FNU
      CVR  =fltarr(muph_cvr) & TETAD=CVR  & TEMPUR=CVR  & CV   =CVR
      DWI  =fltarr(muph_two)
      FPI  =fltarr(muph_two,muph_lm)      & GAI   =FPI  & GESI =FPI
      RMAI =fltarr(muph_two,muph_trm)
      RM1  =fltarr(         muph_trm)     & C     =RM1  & RM2  =RM1
      GOM1 =fltarr(muph_two,muph_trm,muph_lm)
      ERES =fltarr(muph_ers)

      fmt12 ="(16I5)"
      fmt20 ="(8F10.4)"
      fmt26 ="(8F10.1)"
      fmt31 ="(10F12.3)"
      fmt222="(' ****************************************************')"
      fmt44 ="(' ******************************************************"+ $
             "****************************************************************')"

      IFOLD=0
      muph_INDATA, fifi,ok,u, title & if not ok then return  ;Input of data

;     Input of LTEM temperature values for which specific heat should be calculated
      LTEM=0L & READF,u,LTEM
      IF (LTEM GT 0) then begin & tmp=fltarr(LTEM) & READF,U,tmp & TEMPUR[0]=tmp & endif

;     END OF THE INPUT DATA  IF IRES=0 AND SECOND PART IS NOT USED

      IGR=ITM+1
      IF (IVIT GT 1) then IGR=IVIT

;     reminder: LM is the number of descret energy intervals
      LM2=2*LM
      LM1=LM+1
      LM5=LM2

;     In the case of energy gain spectra (IE0<0) LM5 is at maximum
;     equal to the incident energy divided by the width of the energy
;     interval
      IF (IE0 LE 0) then begin & LM5=E0/DELE & IF (LM5 GT LM2) then LM5=LM2 & endif

;     Reminder: NM = Number of TOF input data
      NPHO1=NPHO-1

      NGU=round(HOT[0]   /DELE) ;NGU=HOT(0)   /DELE+.5 ;!!!
      NGO=round(HOT[NM-1]/DELE) ;NGO=HOT(NM-1)/DELE+.5

      NGU1=NGU+1
      NGO1=NGO+1

      PRINT,NGU,NGO ,format="(2I5)"

;     In subroutine PREP1 the temperature factor plus several other
;     functions of Q or omega are calculated and stored in the
;     respective data arrays

      muph_PREP1, LM2,LM5,E0,DELE

      NGOO=MIN([LM5,NGO])

      PRINT,'' & PRINT,NGOO ,format="(' NGOO:',I5)"

;     * Extrapolation of the energy spectrum from the time spectrum ***
;     * This point is very important in the case of delta(w)< delta(t)*
;     L: energy channels, N: time channels

      FOR L=NGU1-1,NGOO-1 do begin
        FOR N=0,NM-1 do begin
          DIF=HOE[L]-HOT[N]
          IF (DIF LE 0) then begin ;.f
              Z1[L]=Z0[N-1]+(Z0[N]-Z0[N-1])*(HOE[L]-HOT[N-1])/(HOT[N]-HOT[N-1])
              break & endif
        ENDFOR
      ENDFOR

;     *****************************************************************

      if (NGO1 le LM5) then Z1[NGO1-1:LM5-1]=0.

      EF=EFIN[NGU1-1:LM5-1]
      HO=IE0*HOE[NGU1-1:LM5-1]
      EX=EXP(.5*HO/TEMP)
;     Reminder: delta(w) is proportional to (E_f*SQRT(E_f))^(-1)
      Z1[  NGU1-1:LM5-1]= Z1[NGU1-1:LM5-1]*(E0/EF)*SQRT(E0/EF)
      GNU0[NGU1-1:LM5-1]= Z1[NGU1-1:LM5-1]*EXE[NGU1-1:LM5-1]*EX/DQUP[NGU1-1:LM5-1,0]

;     ***** w^2 extrapolation for small w ****

      AFA=total(GNU0[NGU:NGU+2]/(HOE[NGU:NGU+2])^2) /3.
      GNU0[0:NGU-1]=AFA*HOE[0:NGU-1]^2
      Z1[  0:NGU-1]=0.

;     ***************************************

      IF (IVIT NE 0) then GNU0[0:LM5-1]=GNU0[0:LM5-1]>0.

;     ******** Calculation of M^(-n) ********
;     ***** in units of the neutron mass ****

      FOR N=1,NPHO do RMAI[0:NSPEC-1,N-1]=(1.0087/AMASI[0:NSPEC-1])^N

      GNU[LM1-1:LM2-1]=0.

;     ********* First guess for g(w) *********
;     Reichardt: L=1, LM5 !!!!
      GNU[0:LM-1]=GNU0[0:LM-1]*EXP(.625*QC[0:LM-1]*DW)

      vin=20 ;!!! muph_trm
      FOR L=LM-vin,LM do GNU[L-1]=(GNU[LM-vin-1]/400.)*(LM-L)^2

      SUM=total(GNU[0:LM-1]) *DELE

      GNU[ 0:LM2-1]=GNU[0:LM2-1]/SUM
      GNUM[0:LM2-1]=0.
      FNU1[0:LM2-1]=GNU[0:LM2-1]
      GNUM[0]      =GNU[0]*DELE
      GNUM[1:LM-1] =GNUM[0:LM-2]+GNU[1:LM-1]*DELE

      muph_CORRFA, LM,GNU,DELE,QUOFG,NSPEC
      muph_SEPAR , LM,GNU,DELE,QUOFG,FPI,NSPEC

      PRINT,'' & PRINT,format=fmt45 & print,''
      PRINT,'            OMEGA  TOF DISTR.   INPUT.DISTR.  GNU-0.APPR.'+ $
            '     INT(OM)     FNU/GNU      FNU1          FNU2         '

      IF debug then  pas=1 ELSE pas=LM5-1
      FOR L=0L,LM5-1,pas  do PRINT,L+1,HOE[L],Z1[L],GNU0[L],GNU[L],GNUM[L],QUOFG[L],FPI[0,L],FPI[1,L],$
                                   format="(I5,2F12.3,2E14.5,2F12.4,2E14.5)"

;**********************************************************************
;     BEGINNING OF THE ITERATIONS *************************************
;**********************************************************************

      ILAU=0
loop: repeat begin

      ILAU =ILAU+1
      PRINT,'' & PRINT,ILAU, format="(' ITERATION NUMBER:',I3)"

      IF (ILAU GT IGR) then IVIT=0
      IF (ILAU NE 1) then begin
;       ********* Normalization of GNU *********
        SUM=total(GNU[0:LM-1])*DELE
        GNU[0:LM-1]=GNU[0:LM-1]/SUM

        muph_CORRFA, LM ,GNU,DELE,QUOFG,NSPEC
        muph_SEPAR , LM,GNU,DELE,QUOFG,FPI,NSPEC

      ENDIF

;     ********* Calulation of T_1 = F(w)/(2hw*sinh(hw/2kT)) ********

;     Reichardt: L=1, LM2 !!!!
      REXE=1./EXE[0:LM-1]
      GNS[0:LM-1]=GNU[0:LM-1]*REXE
      FOR I=0,NSPEC-1 do FPI[I,0:LM-1]=FPI[I,0:LM-1]*REXE

;     ******** Calculation of the Deby-Waller factor **************
;     *** DW = Int(h^2/2M * F(w)/w * cosh(hw/2kT))/sinh(hw/2kT) ***
;     Attention: the integral runs from 1 to LM, i.e. the cut-off *
;                of the spectrum as defined by the user input *****

      DWI[0:NSPEC-1]=0.
      FOR I=0,NSPEC-1 do begin
        FOR  L=0,LM-1 do DWI[I]=DWI[I]+FPI[I,L]*EXA[L]
        DWI[I]=DWI[I]*DELE*1.0087/AMASI[I]
      ENDFOR

      DW1=total(GNS[0:LM-1]*EXA[0:LM-1]) *DELE*RMA

      PRINT, DW1,DWI[0],DWI[1] ,format="(' DEBYE-WALLER COEFF. FROM ITERATED DISTR.(GNU,FNU1,FNU2):',3E14.5)"

;     **************************************************************
      FOR I=0,NSPEC-1 do begin
;     **********Storage of T_1(-LM,+LM) in the array GND*************

        FOR L=1,LM    do GND[L-1]=FPI[I,LM-L]
        FOR L=LM1,LM2 do GND[L-1]=FPI[I,L-LM-1]

        GOM1[I,0,0:LM2-1]=FPI[I,0:LM2-1]

;       Here is the loop over the multiphonon terms.
;       GND is the so far determined density of states function devided
;       by h*omega and sinh(h*omega/2kT), i.e. T_1
;       GOM1(n) contains the different T_n terms.
        RFC=1.

        FOR N=1,NPHO1 do begin
;         RFC = 1 /(N+1)!
          RFC =RFC/(N+1.)
          muph_VIPHO, GNUM,GNUP,N
          GOM1[I,N,0:LM2-1]=GNUP[0:LM2-1]*RFC
          GNUM[0:LM2-1]    =GNUP[0:LM2-1]
        ENDFOR

        IF (IDW EQ 1) then DW=DWI[I]

        IF (IPR NE 0) and (ILAU EQ 1) then begin
          PRINT,'' & PRINT,'        OMEGA      F(OM)        G1(OM)       G2(OM)'+ $
                           ' G3(OM)       G4(OM)       G5(OM)       G6(OM)       G7(OM) G8(OM)'
          IF debug then  pas=1 ELSE pas=LM2-1
          FOR L=0L,LM2-1,pas do PRINT,L+1,HOE[L],FNU[L],GOM1[I,0:7,L], format="(I5,F9.3,9E13.4)"
          PRINT,'' & PRINT,format=fmt222
        ENDIF

;       Construction of the scattering law using eq.2.4

        GAV[ 0:LM5-1]=0.
        GGES[0:LM5-1]=0.

;       EX= detailed balance factor
        EX=EXP(.5*IE0*HOE[0:LM5-1]/TEMP)

;       Factors related to the integration over Q. Attention: w dependent!
        QRMA=DW*QMA[0:LM5-1]
        QRMI=DW*QMI[0:LM5-1]
        EXMA=EXP(-QRMA)
        EXMI=EXP(-QRMI)
        COF0=EXMI-EXMA  ;Here COFO = I_0

        FOR L=0,LM5-1 do begin
          FOR N=1,NPHO do begin
;           Calculation of (I_N)(x) = -x^N*exp(-x) - N*I_(N-1)
            COF0[L]=N*COF0[L]+EXMI[L]*QRMI[L]^N -EXMA[L]*QRMA[L]^N

            GAV[L] =GAV[L] + RMAI[I,N-1] * DQUP[L,N-1]   *GOM1[I,N-1,L]
            GOM1[I,N-1,L] = (1/EX[L])*COF0[L]*RMAI[I,N-1]*GOM1[I,N-1,L]/(2.*DW^(N+1))
            GGES[L]=GGES[L] + GOM1[I,N-1,L]
          ENDFOR
          GAV[L]=GAV[L]/(DQUP[L,0]*RMAI[I,0])
        ENDFOR

        IF (IFOLD EQ 1) THEN begin
          muph_FOLD, GAV ,FOUT,LM5,DELE
          GAV[ 0:LM5-1] = FOUT[0:LM5-1]
          muph_FOLD, GGES,FOUT,LM5,DELE
          GGES[0:LM5-1] = FOUT[0:LM5-1]
        ENDIF

        HO=IE0*HOE[0:LM5-1]
        EX=EXP(.5*HO/TEMP)
        GGES[  0:LM5-1]=GGES[0:LM5-1]/(DQUP[0:LM5-1,0]*RMAI[I,0])
        GAI[ I,0:LM5-1]=GAV[ 0:LM5-1]*  EXE[0:LM5-1]
        GESI[I,0:LM5-1]=GGES[0:LM5-1]*  EXE[0:LM5-1]  *EX

;     ****** Creation of plot file for multi-phonon contributions *****

        IF (ILAU GE ITM) then begin

          IF (IFOLD EQ 1) THEN begin
           FOR JJ=0,NPHO-1 do begin
            FIN[0:LM2-1] = reform(GOM1[I,JJ,0:LM2-1])
            muph_FOLD, FIN,FOUT,LM2,DELE
            GOM1[I,JJ,0:LM2-1] = FOUT[0:LM2-1]
           ENDFOR
          ENDIF

          SUMZ = total(Z1[0:LM2-1])*DELE
          SUMC = total(GOM1[I,0:NPHO-1,0:LM2-1])*DELE

          FOR L=0,LM5-1 do DSIG[L]=DSIG[L]+total(ALFI[I]*GOM1[I,0:NPHO-1,L])

          IF I LE 1 then begin & if I eq 0 then file='multi1_lamp.plot' else file='multi2_lamp.plot'
            uu=-1 & on_ioerror,mismult & OPENW,uu,file,/get_lun
            FOR L=0,LM2-1 do PRINTF,uu, L+1,HOE[L],Z1[L]/SUMZ,total(GOM1[I,0:3,L])/SUMC,GOM1[I,0:3,L]/SUMC, $
                                    format="(I5,F9.3,6E13.4)"
            mismult:if uu gt 0 then free_lun,uu
            Wmulti=[[HOE[0:LM2-1]],[Z1[0:LM2-1]/SUMZ],[reform(total(reform(GOM1[I,0:3,0:LM2-1]),1))],$
                    [reform(GOM1[I,0,0:LM2-1]/SUMC)] ,[reform(GOM1[I,1,0:LM2-1]/SUMC)],[reform(GOM1[I,2,0:LM2-1]/SUMC)],$
                    [reform(GOM1[I,3,0:LM2-1]/SUMC)]]
          ENDIF
        ENDIF
      ENDFOR

      ;     *** End of loop over I ******************************************

      QUO[ 0:LM5-1]=1.
      GAV[ 0:LM5-1]=reform(GAI[ 0,0:LM5-1]*ALFI[0]+GAI[ 1,0:LM5-1]*ALFI[1])
      GGES[0:LM5-1]=reform(GESI[0,0:LM5-1]*ALFI[0]+GESI[1,0:LM5-1]*ALFI[1])
      idx=where(GGES[0:LM5-1] ne 0.)
      IF idx[0] GE 0 then QUO[idx]=GNU0[idx]/GGES[idx]

;     Reichardt only normalized up to LM !!!!!!!!!!!
      SUM =total(GGES[0:NGOO-1])
      SUM2=total(GNU0[0:NGOO-1])
      QUAV=total(QUO[ 0:LM-1])

      RSUM=SUM/SUM2
      QUAV=RSUM*QUAV/LM

;     Reichardt: L=1, LM5 !!!!
      GNUM[0:LM-1]=0. & GNUP[0:LM-1]=0. & QUO1[0:LM-1]=0.

;     ******** New guess for f(w) using the quotient method ********
      IF (IVIT NE 0) then GNU[0:LM-1]=GNU[0:LM-1]*QUO[0:LM-1]*RSUM $

      ELSE BEGIN
;     ******* New guess for f(w) using the difference method *******
        GNU[0:LM-1]=GNU[0:LM-1]+GAV[0:LM-1]*(RSUM*QUO[0:LM-1]-1.)
        idx=where(GNU0[0:LM-1] EQ 0.)
        IF idx[0] GE 0 then GNU[idx]=0.
      ENDELSE

      idx=where(GNU[0:LM-1] GT 0.)
      if idx[0] GE 0 then begin & GNUPC=GNU[idx]/(3.*(HOE[idx]^2))
                                GNUP[idx]=1./GNUPC^.3333333  & endif

;     Reminder: FNU1 is the first guess for f(w)
;     Attention GNU not yet normalized

      idx=where(FNU1[0:LM-1] NE 0.)
      if idx[0] GE 0 then QUO1[idx]=GNU[idx]/FNU1[idx]

;     Reichardt: These loops are missing !!!!
;     for a smooth cut off
      GNU[LM1-1:LM2-1]=0.
      vin=20 ;!!! muph_trm
      FOR L=LM-vin,LM do begin
        GNU[L-1]=(GNU[LM-vin-1]/400.)*(LM-L)^2
      ENDFOR

;     ********** Calculation of first moment of f(w) **********
;     s.a. 248
      GNUM[0:LM5-1]=0.
      GNUM[0]=GNU[0]*DELE
      GNUM[1:LM-1]=GNUM[0:LM-2]+GNU[1:LM-1]*DELE

      SUM1=total(ABS(RSUM*QUO[0:LM-1]-QUAV)) / (LM*QUAV)

      PRINT,RSUM, format="(' Correction factor=',E14.5)"

      PRINT,QUAV,SUM1, format="(' AVERAGE OF (INP./CALC.)=',E14.5,'"+ $
                              "   MEAN RELATIVE  DEVIATION FROM AVERAGE=',E14.5)" & wait,.01
      IF (IPR NE 0) then break

;     **** ILAU is the number of the actual iteration; ****
;     **** ITM is the maximum number of iterations;    ****
;     **** Next iteration if ILAU less than ITM ****

      endrep until (ILAU GE ITM)

      muph_CORRFA, LM,GNU,DELE,QUOFG,NSPEC
      muph_SEPAR , LM,GNU,DELE,QUOFG,FPI,NSPEC

      FNU[0:LM5-1]=reform(CONCI[0]*FPI[0,0:LM5-1]+CONCI[1]*FPI[1,0:LM5-1])

      PRINT,'' & PRINT,'         OMEGA   INP. DIST.  CALC. DIST.   INP./CALC.'+ $
                       '    GNU[OM]       INT(OM)   DEBYE-OM  GNU-N/GNU-0  FNU/GNU      FNU'
      IF debug then  pas=1 ELSE pas=LM5-1
      FOR L=0L,LM5-1,pas do PRINT,L+1, HOE[L],GNU0[L]*RSUM,(GGES[L]/SUM),(RSUM*QUO[L]), $
                             GNU[L]/RSUM,GNUM[L],GNUP[L],QUO1[L],QUOFG[L],FNU[L] , $
                             format="(I5,F9.3,3E13.4, 2X,E13.4,F12.4,F10.2,2F11.3,E14.4)"

;     **** Next iteration if ILAU less than ITM ****
      IF (ILAU LT ITM) then goto,loop

      SDSIG = total(DSIG[0:LM5-1])*DELE
      DSIG[0:LM2-1] = DSIG[0:LM2-1]/SDSIG

      uu=-1 & on_ioerror,misgdos
      OPENW ,uu,'DSDO_lamp.plot',/get_lun
      FOR L=0,LM2-1 do PRINTF,uu,L+1,HOE[L],Z1[L]/SUMZ,DSIG[L],format="(I5,F9.3,2E13.4)" & free_lun,uu
      OPENW ,uu,'GDOS_plot_lamp.dat',/get_lun
      PRINTF,uu,LM5, format="(I5)"
      FOR L=0,LM5-1 do PRINTF,uu,L+1,HOE[L],GNU[L],GNU0[L]/(DELE*SUM2),GGES[L]/(DELE*SUM), format="(I5,F9.3,3E13.4)"
      misgdos:IF uu GT 0 then free_lun,uu

      IE=1.3*LM
      muph_SPLOT, 1,IE,HOE,GNU,0.,GNUM,1.

      GNUP[0:IE-1]=0.

      muph_MOMENT, GNU,HOE,LM,DELE,CMO

      PRINT,'' & PRINT,'DEBYE CUTOFF FREQUENCIES (IN MEV AND THZ)= (.33333*(N+3)*(OM**N)AV)**1/N'
      PRINT,'ARR.     -2        -1         0        +1        +2       +3        +4        +5        +6        +7'
      PRINT,CMO[0:9], format="(4X,10F10.3)"

      CMO[0:9]=CMO[0:9]/4.1355
      PRINT,CMO[0:9]

      IF (LTEM NE 0) then begin
        FOR L=0,LTEM-1 do begin
          TE=TEMPUR[L]
          muph_HEAT, LM,HOE,DELE,FNU,TE,CLV,TET
          CV[L]   =CLV
          TETAD[L]=TET
          CVR[L]  =CLV/(TE^3)
        ENDFOR

        PRINT,'' & PRINT,format=fmt45
        PRINT,'' & PRINT,' SPECIFIC HEAT IN MJ/(K*G-ATOM) : '
        PRINT,'' & PRINT,'       T(K)       T**2        CV[T]        CV[T]/T CV[T]/T**3     THETA-D(K)  '

        TTT2  =TEMPUR[0:LTEM-1]^2
        CVINVT=CV[0:LTEM-1]/TEMPUR[0:LTEM-1]
        IF debug then   pas=1 ELSE pas=LTEM-1
        FOR L=0L,LTEM-1,pas do PRINT,TEMPUR[L],TTT2[L],CV[L],CVINVT[L],CVR[L],TETAD[L],format="(2F12.2,4E14.4)"
      ENDIF

;     CALCULATION OF G(OM) FOR EQUIDISTANT TIME INTERVALS
      QUO[0:LM5-1]=GGES[0:LM5-1]-GNU[0:LM5-1]

      muph_EQUIT, LM5,QUO,QUAV/RSUM ,Wdos

;     ANALYSIS OF INCOMPLETE ENERGY LOSS SPECTRUM
      IF (ILOSS EQ 1) then muph_INCOMP, QUAV,GNU,GOM1,RMAI,DWI,GGES,GNUM,GNUP ,u
      free_lun,u
      END

;***********************************************************************
      pro muph_FOLD, FIN,FOUT,L,DELE
;+
;***********************************************************************
;** muphocor stuff
;-

      COMMON muph_RESO

      FOUT[0:L-1]=0.
      IEN=round((findgen(L)+1)*10.*DELE) ;!!!
      RES=ERES[IEN-1-1]
      idx=where(RES NE 0.)
      if idx[0] ge 0 then begin
         RES=RES/DELE
         M  =3*RES
         FOR loup=0,n_elements(idx)-1 do begin
           J=idx[loup]+1
           IF ((J-M) LT 1) THEN ML=1 ELSE ML=J-M
           IF ((J+M) GT L) THEN MU=L ELSE MU=J+M
           FNORM=0.
           FOR K=ML,MU do begin
             tmpex    =EXP(-((J-K)/RES[J-1])^2)
             FOUT[J-1]=FOUT[J-1]+FIN[K-1]*tmpex
             FNORM    =FNORM             +tmpex
           ENDFOR
           FOUT[J-1]=FOUT[J-1]/FNORM
         ENDFOR
       endif
      END

;***********************************************************************
      pro muph_INDATA, fifi, ok , u ,title
;+
;***********************************************************************
;** muphocor stuff
;-

      COMMON muph_BB1   & COMMON muph_AVAR & COMMON muph_ROS
      COMMON muph_MULTI & COMMON muph_PAR1 & COMMON muph_PAR2

      fmt10="('OUTPUT MUPHOCOR/5 *** EXTENDED VERSION  SEPT. 84      ')"
      fmt20="(' PROBLEM:',A)"
      fmt30="(16I5)"
      fmt40="(8F10.4)"
      fmt45="(' *********************************************************"+ $
            "*************************************************************')"

      OK=1
      PI=!pi ;3.1415927
      PIR=PI/180.
      NDIM=n_elements(PARQ)

      ALFI[0:1]=[1.,0.]
      PRINT ,format=fmt10
      nodebug=1
      if nodebug then begin
      catch,stat & if stat ne 0 then begin & OK=0 & if u gt 1 then free_lun,u
                                           print,!error_state.msg & RETURN & endif
      endif
      u=-1 & OPENR,u,fifi,/get_lun
      TITLE='' & READF,u,TITLE  & PRINT,'' & PRINT,TITLE,format=fmt20

      E0=0. & FP=0. & CW=0. & XNEL=0. & FIMI=0. & FIMA=0. & ABK=0. & AEMP=0.
      READF,u ,E0,FP,CW,XNEL,FIMI,FIMA,ABK,AEMP ;,format=fmt40
;     E0:INC. ENERGY(MEV)             FP:FLIGHTPATH(CM)
;     XNEL:CHANNEL OF ELASTIC LINE    CW:CHANNEL WIDTH(MYS)
;     FIMI:MIN.SCATT.ANGLE            FIMA:MAX.SCATT.ANGLE
;     UNT:CONST.BACKGROUND

      IF (AEMP EQ 0.) THEN AEMP=5.60

      LM=0L & NSPEC=0L & IQUO=0L & LPAR=0L
      READF,u,LM,NSPEC,IQUO,LPAR ;,format="(4I5)"
;     NSPEC: NUMBER OF DIFFERENT ATOMIC SPECIES

      HOX=0. & TEMP0=0. & DW=0.
      READF,u,HOX,TEMP0,DW ;,format="(3F10.4)"
;     HOX:   UPPER LIMIT OF DENSITY OF STATES
;     TEMPO: TEMPERATURE IN KELVIN
;     DW:    MEAN DEBY WALLER COEFFICIENT

      DELE=HOX/LM
;     DELE:  DELTA ENERGY FOR THE CALCULATION

      TEMP=TEMP0/11.605
;     TEMP:  TEMPERATURE IN MEV

      T0=SQRT(522.724/E0)*FP
;     T0:  TIME OF ARRIVAL OF ELASTICALLY SCATTERED NEUTRONS IN MYS

      T0R=T0/CW
;     T0R: T0 IN MULTIPLES OF CHANNELS

      tmp=[0.,0.,0.]
      for I=0,NSPEC-1 do begin & READF,u,tmp & AMASI[I]=tmp[0] & SIGI[I]=tmp[1]
                                             CONCI[I]=tmp[2] & endfor
;     AMASI: ATOMIC MASS
;     SIGI : SIGMA
;     CONC:  CONCENTRATION
;     ALFI:  SCATTERING POWER

      CONCI[1]=1.-CONCI[0]
      IF (NSPEC EQ 2) then begin & tmp=fltarr(LPAR) & READF,u,tmp & PARQ[0]=tmp & endif

      NPHO=0L & ITM=0L & IVIT=0L & IDW=0L & IEMP=0L & IRES=0L & IPR=0L & ILOSS=0L
      READF,u,NPHO,ITM,IVIT,IDW,IEMP,IRES,IPR,ILOSS
;     NPHO:     NUMBER OF MULTI PHONON TERMS,ITM:NUMBER OF ITERATIONS
;     ITM:      TOTAL NUMBER OF ITERATIONS
;     IVIT=0(1):ITERATION BY DIFFERENCE(QUOTIENT) METHOD
;     IDW=0:    DW COEFF. KEPT CONST.=INPUT VALUE,DW=1:DW COEFF.ITERATED
;     IEMP=1(0):CORRECTIONS FOR COUNTER EFFICIENCY WILL BE(NOT BE) DONE
;     IRES=1(0):DATA WILL BE(NOT BE) CORRECTED FOR SPECTR. RESOLUTION

;     ------------------------------------------------------------------
      IF (IEMP EQ 0) then begin & PRINT,''
          PRINT,'INPUT DISTRIBUTION HAS ALREADY BEEN CORRECTED FOR COUNTER EFFICIENCY' & endif
      IF (IEMP EQ 1) then begin & PRINT,''
          PRINT,'CORRECTION FOR COUNTER EFFICIENCY IS DONE IN THIS PROGRAM' & endif
      PRINT,E0,FP,CW,ABK,format="('INC. ENERGY=',F10.5,'   FLIGHT PATH=',F10.5,"+ $
                                "'   CHANNEL WIDTH=',F10.5,'   ABS.COEFF.=',F10.5)"
      PRINT,FIMI,FIMA   ,format="('RANGE OF SCATTERING ANGLES:',F10.3,' TO ',F10.3)"
      PRINT,HOX         ,format="('UPPER LIMIT OF PHONON DENSITY OF STATES:',F10.4)"
      PRINT,TEMP0,DW    ,format="(' TEMPERATURE:',F10.3,'   MEAN DEBYE-WALLER COEFF.:',E14.5)"
      PRINT,' IN THIS PROGRAM VALUES FOR DW-COEFF. ARE IN (1/MEV), TO '+ $
            'OBTAIN VALUES IN (ANG**2) MULTIPLY BY 2.0717 '
      IF (IDW EQ 0)   then PRINT,'DEBYE-WALLER COEF. FIXED=INPUT VALUE                '
      IF (IDW EQ 1)   then PRINT,'DEBYE-WALLER COEF. DETERM. FROM ITERATED DISTR.     '
      IF (NSPEC NE 1) then begin & PRINT,''
                      PRINT,' MULTI-PHONON CONTRIBUTIONS CALCULATED SEPARATELY FOR BOTH COMPONENTS '
                      muph_CONST & endif
      FOR I=0,NSPEC-1 do PRINT, AMASI[I],SIGI[I],CONCI[I],ALFI[I],format= $
                            "(' ATOMIC MASS:',F10.3,'   SIGMA:',F8.4,'   CONC:',F8.4,'   SCATT.POWER:',F8.4)"
      RMA=1.0087/AMASI[0]
      IF (NSPEC NE 1) then begin & PRINT,' PARAMETERS FOR FNU/GNU:'
                                 PRINT,PARQ[0:7],format=fmt40 & RMA=1.0087/AMEFF & endif
      PRINT,NPHO        ,format="('NUMBER OF MULTI-PHONON-TERMS:',I3)"
      PRINT,ITM         ,format="(' TOTAL NUMBER OF ITERATIONS :',I3)"
      IF (IVIT EQ 0) then PRINT,'     ITERATION BY DIFFERENCE METHOD'
      IF (IVIT GT 0) then PRINT,IVIT,format="('     ITERATION BY QUOTIENT METHOD UP TO IT. NR. :',I3)"

      muph_INTOF,u
      END

;***********************************************************************
;***********************************************************************
      pro muph_INTOF ,u
;+
;***********************************************************************
;***********************************************************************
;** muphocor stuff
;-

      COMMON muph_BB1   & COMMON muph_AVAR & COMMON muph_ROS
      COMMON muph_MULTI & COMMON muph_PAR1 & COMMON muph_PAR2

      Z00=fltarr(n_elements(GND)) & UN0=Z00

      fmt30="(16I5)"
      fmt40="(8F10.4)"
      fmt45="(' *********************************************************"+ $
            "*************************************************************')"
      debug=0

      ISOUR=0L & NUU=0L & NOO=0L & NU=0L & NO=0L & IGLU=0L
      READF,u,ISOUR,NUU,NOO,NU,NO,IGLU
;     NU:  First channel of spektrum NO: Last channel of spektrum
;     NUU: First channel of TOF distribution used for calculation
;     NOO: Last channel  of TOF distribution used for calculation
;
      NM=NO-NU+1
;     NM: Number of TOF input data

      UNT=0. & FUN=0. & READF,u,UNT,FUN
;     UNT: CONSTANT BACKGROUND
;     FUN: MULTIPLICATION FACTOR FOR TIME DEPENDENT BACKGROUND

      IF (ISOUR NE 1) then begin & tmp=fltarr(NOO-NUU+1) & READF,u,tmp,format=fmt40 & Z00[NUU-1]=tmp
;      Z00: TIME OF FLIGHT DISTRIBUTION

       IF (FUN NE 0.) then begin & READF,u,tmp,format=fmt40 & UN0[NUU-1]=tmp & endif
;      UN0: TIME DEPENDENT BACKGROUND
      ENDIF

;     Shift data to the left
      Z0[0:NM-1]=Z00[NU-1:NU+NM-2]
      UN[0:NM-1]=UN0[NU-1:NU+NM-2]

      PRINT,''
      PRINT,XNEL,NU,NO,format="('CHAN.OF EL.LINE:',F10.2,'   FIRST CHANNEL:',I5,'   LAST CHANNEL:',I5)"
      PRINT,UNT       ,format="('CONSTANT BACKGROUND:',F10.3)"
      PRINT,''
      PRINT,NUU,NOO   ,format="(' INPUT TIME OF FLIGHT DISTRIBUTION FROM CHANNEL :',I4,' TO CHANNEL :',I4)"
      IF debug then PRINT,Z00[NUU-1:NOO-1],format="(10F12.3)" ELSE PRINT,(Z00[NUU-1]),' -> ',Z00[NOO-1]

      IF (FUN NE 0.) then begin & PRINT,''
       PRINT,FUN,format="('TIME DEPENDENT BACKGROUND -- HAS TO BE MULTIPLIED BY A FACTOR:',F10.5)"
       IF debug then PRINT,UN0[NUU-1:NOO-1] ELSE PRINT,(UN0[NUU-1]),' -> ',UN0[NOO-1]

;      SMOOTHING OF TIME DEPENDENT BACKGROUND
       IF (IGLU NE 0) then begin
        FOR I=1,IGLU do begin
         muph_GLAT0,UN,Z1,NM
         UN[0:NM-1]=Z1[0:NM-1] & Z1[0:NM-1]=0.
        endfor
        PRINT,''
        PRINT,IGLU,format="('   TIME DEPENDENT BACKGROUND AFTER SMOOTHING -- NUMBER OF PROCESSES:',I2)"
        IF debug then PRINT,UN[0:NM-1]
       ENDIF

;      Subtraction of background
       Z0[0:NM-1]=Z0[0:NM-1]-FUN*UN[0:NM-1]
      ENDIF

      BAC=UNT
      idx=lindgen(NM)
      DELK=NU-1+idx+1-XNEL
;     Reminder: TOR IS THE TIME OF ARRIVAL OF ELASTICALLY SCATTERED
;     NEUTRONS IN MULTIPLES OF CHANNELS
;     HOT: Energy gain or loss respectively
      HOT[idx]=E0*(1.-(T0R/(T0R+DELK))^2)

;     Correction of constant backgroud for counter efficiency
      IF(IEMP EQ 0) then BAC=UNT/(1.0-EXP(-AEMP/SQRT(E0-HOT[idx])))
      Z0[idx]=Z0[idx]-BAC

;     Resolution corrections
      IF (IRES NE 0) then begin
       PRINT,'' & PRINT,format=fmt45 & PRINT,''
       PRINT,'INPUT DISTRIBUTION WILL BE CORRECTED FOR EXPERIMENTAL RESOLUTION'
       muph_RESCOR,Z0,Z1 ,u
       Z0[0:NM-1]=Z1[0:NM-1]
      ENDIF

      PRINT,'' & PRINT,format=fmt45
      IE0=-1
      VORZ=.5*(NU+NO)-XNEL

      ;.f
      IF VORZ LT 0. then begin ;Energy gain spektrum
                IE0=+1
;               Exchange first with last number and so forth
                Z0[ 0:NM-1]=    reverse(Z0[ 0:NM-1])
                HOT[0:NM-1]=abs(reverse(HOT[0:NM-1]))
                PRINT,'' & PRINT,' ENERGY GAIN '
                endif
      IF VORZ EQ 0. then   RETURN
      IF VORZ GT 0. then begin ;Energy loss spektrum
                PRINT,'' & PRINT,' ENERGY LOSS '
                endif

      idx =lindgen(NM)
      SQEF=SQRT(E0+IE0*HOT[idx])
      IF(IEMP EQ 1) then EMP=1.-EXP(-AEMP/SQEF) else EMP=1.
      Z0[idx]=Z0[idx]*EXP(5.025*ABK/SQEF)/EMP
      UN[idx]=Z0[idx]

      muph_MEAN,DELE

      PRINT,' ABSOLUTE ENERGY TRANSFER ',HOT[0],' -> ',HOT[NM-1]
      IF debug then PRINT,HOT[0:NM-1]
      PRINT,' INPUT DISTRIBUTION AFTER CORRECTIONS FOR BACKGROUND,RESOLUTION,COUNTER EFFICIENCY AND ABSORPTION  '
      IF debug then PRINT,Z0[0:NM-1]
      END

;***********************************************************************
;***********************************************************************
      pro muph_PREP1, LM2,LM5,E0,DELE
;+
;***********************************************************************
;***********************************************************************
;**   In subroutine PREP1 the temperature factor plus several other
;**   functions of Q or omega are calculated and stored in the
;**   respective data arrays
;-

      COMMON muph_BB1 & COMMON muph_BB2

      FOR L=1,LM2 do begin
;       HOE = energy in [meV]
        EN=(L-.5)*DELE
        HOE[L-1]=EN

;       EX = e^(energy/2kT)
        EX=EXP(.5*EN/TEMP)

;       EXE = energy * sinh(energy/2kT)
        EXE[L-1]=EN*(EX-1./EX)

;       EXA = energy * cosh(energy/2kT)
        EXA[L-1]=EX+1./EX
      ENDFOR

;     Reminder: PIR = pi in radiant, FIMA and FIMI: maximum and minimum
;               scattering angles

      FI0=.5*(FIMA+FIMI)*PIR
      DFI=.5*(FIMA-FIMI)*PIR

      COM =COS(FI0)*COS(DFI)
      COMA=COS(PIR*FIMA)
      COMI=COS(PIR*FIMI)

;     EF = energy of scattered neutrons
      EF = E0+IE0*HOE[0:LM5-1]

;     QC = h^2/2m * (ki^2 + kf^2 -2ki*kf*cos(theta)) =
;     momentum transfer squared times h^2/2m, m denoting the neutron mass

      QC[  0:LM5-1]= E0+EF-2.*SQRT(E0*EF)*COM
      QMA[ 0:LM5-1]= E0+EF-2.*SQRT(E0*EF)*COMA
      QMI[ 0:LM5-1]= E0+EF-2.*SQRT(E0*EF)*COMI
      EFIN[0:LM5-1]=    EF

      FOR N=1,NPHO do begin
          N1=N+1
          DQUP[0:LM5-1,N-1]=(QMA[0:LM5-1]^N1-QMI[0:LM5-1]^N1)/(2.*N1)
      ENDFOR

      END


;***********************************************************************
;***********************************************************************
      pro muph_EQUIT, LM5,QUO,QUAV ,Wdos
;+
;***********************************************************************
;***********************************************************************
;** muphocor stuff
;-

      COMMON muph_AVAR & COMMON muph_ROS & COMMON muph_MULTI
      COMMON muph_BB1  & COMMON muph_BB2

      fmt45="(' *********************************************************"+ $
            "*************************************************************')"
      debug=0
      NMA  =NM                            ;last channel - first channel
      HOMAX=HOE[LM5-1]

      IF (HOT[NM-1] GT HOMAX) then begin  ;lm = 2*no of energy intervals
       FOR N=1,NM do IF(HOT[N-1] GT HOMAX) then break
       NMA=N-1
      ENDIF

      FOR N=0,NMA-1 do begin
        FOR L=1,LM5-1 do begin
          DIF=HOT[N]-HOE[L]
          IF(DIF LE 0) then begin ;.f
             Z1[N]=QUO[L-1]+(QUO[L]-QUO[L-1])*(HOT[N]-HOE[L-1])/DELE
             break & endif
        ENDFOR
      ENDFOR

      HO =IE0*HOT[0:NMA-1]
      EF =E0+HO
      EX =EXP(.5*HO/TEMP)
      AFA=HO*(EX-1./EX)
      EN =(COMI-COMA)*(E0+EF-2.*SQRT(E0*EF)*COM)
      Z0[0:NMA-1]= UN[0:NMA-1]*AFA*EX*E0/(EN*EF*EF)
      Z1[0:NMA-1]=(Z0[0:NMA-1]/QUAV-Z1[0:NMA-1])>0.

      a=1 & b=(LM5/2-1)<(NMA-1)
      ZSUM=total((HOT[a:b]-HOT[a-1:b-1])*Z1[a:b])

      QMA[0:NMA-1]=HOT[0:NMA-1]/4.1355

      Z1[ 0:NMA-1]= Z1[0:NMA-1]/ZSUM
      QMI[0:NMA-1]= Z1[0:NMA-1]*4.1355

      PRINT,'' & PRINT,format=fmt45 & PRINT,''
      PRINT,'PHONON DENSITY OF STATES FOR EQUIDISTANT TIME-INTERVALS'
      PRINT,''
      PRINT,'            OMEGA    TOF DISTR.   INPUT DISTR.   G(OM)             NU[THZ]     G(NU)'
      IF debug then  pas=1  ELSE pas=NMA-1
      FOR N=0L,NMA-1,pas do PRINT,N+1,HOT[N],UN[N],Z0[N],Z1[N],QMA[N],QMI[N], $
                                  format="(I5,F12.3,F12.2,E16.5,E18.5,F16.3,E16.5)"

      uu=-1 & on_ioerror,misdos & OPENW,uu,'DOS_lamp.dat',/get_lun
      FOR N=0,NMA-1 do PRINTF,uu,N+1,HOT[N],UN[N],Z0[N],Z1[N],QMA[N],QMI[N], $
                             format="(I5,F12.3,F12.2,E16.5,E18.5,F16.3,E16.5)"
      misdos:IF uu GT 0 then free_lun,uu
      Wdos=[[HOT[0:NMA-1]],[Z0[0:NMA-1]],[Z1[0:NMA-1]]]
      END

;***********************************************************************
      pro muph_INCOMP, QUAV,GNU,GOM1,RMAI,DWI,GGES,GNUM,GNUP ,u
;+
;***********************************************************************
;**   THIS PART OF THE PROGRAM ALLOWS TO ANALYSE SPECTRA IN ENERGY LOSS
;**   IF ONE CANNOT DETERMINE THE TOTAL SPECTRUM DUE TO A TOO LOW PRIM. ENERGY
;**   THE E-GAIN SP. IS USED FOR NORMALISATION AND MULTI-PHONON CORR.
;-


      COMMON muph_AVAR & COMMON muph_ROS & COMMON muph_MULTI
      COMMON muph_BB1  & COMMON muph_BB2 & COMMON muph_PAR1

      GNU0=GNU*0. & GESI=fltarr(n_elements(ALFI),n_elements(GNU))

      fmt30="(16I5)"
      fmt40="(8F10.4)"
      fmt45="(' *********************************************************"+ $
            "*************************************************************')"
      debug=0
      PRINT,'' & PRINT,format=fmt45
      PRINT,' ANALYSIS OF THE ENERGY LOSS SPECTRUM               '

      IRES=0L & READF,u,IRES ;,format="(I5)"
      muph_INTOF, u

      LM5=E0/DELE
      PRINT,'' & PRINT,fmt45
      idx=findgen(LM5)
      EN =(idx+1.-.5)*DELE
      EF =E0-EN
      QC[idx]  =E0+EF-2.*SQRT(E0*EF)*COM
      QMA[idx] =E0+EF-2.*SQRT(E0*EF)*COMA
      QMI[idx] =E0+EF-2.*SQRT(E0*EF)*COMI
      EFIN[idx]=   EF

      FOR N=1,NPHO do begin & N1=N+1
          DQUP[idx,N-1]=(QMA[idx]^N1-QMI[idx]^N1)/(2.*N1) & ENDFOR

      NGU =round(HOT[0]   /DELE) ;NGU =HOT(0)   /DELE+.5 ;!!!
      NGO =round(HOT[NM-1]/DELE) ;NGO =HOT(NM-1)/DELE+.5
      NGU1=NGU+1
      NGO1=NGO+1
      NGOO=MIN0[[LM5,NGO]]
      IF NGU1 le NGOO then $
      FOR L=NGU1-1,NGOO-1 do begin
        FOR N=0,NM-1 do begin
            DIF=HOE[L]-HOT[N]
            IF (DIF LE 0) then begin ;.f
                Z1[L]=Z0[N-1]+(Z0[N]-Z0[N-1])*(HOE[L]-HOT[N-1])/(HOT[N]-HOT[N-1])
                break & ENDIF
        ENDFOR
      ENDFOR

      Z1[NGO1-1:LM5-1]=0.
      EF=EFIN[NGU1-1:LM5-1]
      EX=EXP(-.5*HOE[NGU1-1:LM5-1]/TEMP)
      GNU0[NGU1-1:LM5-1]=Z1[NGU1-1:LM5-1]*EXE[NGU1-1:LM5-1]*EX*(E0/EF)*SQRT(E0/EF)/(QUAV*DQUP[NGU1-1:LM5-1,0])

      AFA=total(GNU0[NGU:NGU+2]/(HOE[NGU:NGU+2])^2)/3.

      GNU0[0:NGU-1]=AFA*HOE[0:NGU-1]^2
      Z1[  0:NGU-1]=0.
      EF  =EFIN[0:LM5-1]
      HOT[ 0:LM5-1]=T0R*(SQRT(E0/EF)-1.)
      GNUM[0:LM5-1]=0.

      idx=lindgen(LM5)
      FOR I=0,NSPEC-1 do begin
        DW  =DWI[I]
        EX  =EXP(-.5*HOE[idx]/TEMP)
        GGES[idx]=0.
        QRMA=DW*QMA[idx]
        QRMI=DW*QMI[idx]
        EXMA=EXP(-QRMA)
        EXMI=EXP(-QRMI)
        COF0=EXMI-EXMA
        FOR N=1,NPHO do begin
          COF0=N*COF0+EXMI*QRMI^N-EXMA*QRMA^N
          IF (N EQ 1) then COF=COF0
          GGES[idx]=GGES[idx]+COF0*RMAI[I,N-1]*reform(GOM1[I,N-1,idx])/(2.*DW^(N+1))
        ENDFOR
        GGES[  idx]=GGES[idx]/(DQUP[idx,0]*RMAI[I,0])
        GESI[I,idx]=GGES[idx]*EXE[idx]
      ENDFOR

      GGES[idx]=reform(GESI[0,idx])*ALFI[0]+reform(GESI[1,idx])*ALFI[1]
;     GNU (idx)=2.*DW^2*(GNU0(idx)-GGES(idx))*DQUP(idx,0)/COF+GNU(idx)
      GNUP[idx]=GNU0[idx]/GGES[idx]
      GNU[ idx]=GNUP[idx]*GNU[ idx]

      GNUM[0]=GNU[0]*DELE
      GNUM[1:LM5-1]=GNUM[0:LM5-2]+GNU[1:LM5-1]*DELE
      LM=HOX/DELE
      RN=1./GNUM[LM-1]
;     GNUP(idx)=GNU(idx)*RN

      PRINT,'' & PRINT,'          OMEGA   INPUT DISTR. CALC.(E-GAIN)   INP/CALC'+ $
                       '     GNU[L]       INT(GNU)      CHAN.SHIFT  '
      IF debug then  pas=1  ELSE pas=LM5-1
      FOR L=0L,LM5-1,pas do PRINT,L+1,HOE[L],GNU0[L],GGES[L],GNUP[L],GNU[L],GNUM[L],HOT[L], $
                             format="(I5,F10.3,2E14.5,F11.4,E16.5,F12.5,F12.2)"

      END

;***********************************************************************
      pro muph_CONST
;+
;***********************************************************************
;**   Calculation of alpha(i)
;**   QFG0=S_0 : ratio of mean mass over effective mass
;**   QFG1=S_1 : c(1) over alpha(1)
;**   AMEFF: effective mass
;-

      COMMON muph_PAR1

;     Calculation of alpha(i)

      ALFI[0:1]=CONCI[0:1]*SIGI[0:1]/AMASI[0:1]

      ALFN=ALFI[0]+ALFI[1]
      ALFI[0:1]=ALFI[0:1]/ALFN

      RNOR=1./(ALFI[0]*CONCI[1]-ALFI[1]*CONCI[0])

      AC1=AMASI[0]*CONCI[0]
      AC2=AMASI[1]*CONCI[1]

;     QFG0=S_0 : ratio of mean mass over effective mass
      QFG0=(AC1+AC2)/(ALFI[0]*AMASI[0]+ALFI[1]*AMASI[1])

;     QFG1=S_1 : c(1) over alpha(1)
      QFG1=CONCI[0]/ALFI[0]
      QFG2=CONCI[1]/ALFI[1]

;     AMEFF: effective mass
      AMEFF=(SIGI[0]*CONCI[0]+SIGI[1]*CONCI[1])/ALFN

      QOMAV=(AMASI[1]*CONCI[0]+AMASI[0]*CONCI[1])/(AMASI[1]*ALFI[0]+AMASI[0]*ALFI[1])
      PRINT,''
      PRINT,QFG0,QFG1,QFG2,QUOMAV,format="(' LIMITS OF FNU/GNU(0,1,2):',3F12.4,'   RATIO OF <OM**2>:',F10.3)"
      PRINT,AMEFF,format="(' EFFECTIVE MASS:',F10.3)"
      END


;***********************************************************************
;***********************************************************************
      pro muph_CORRFA, LM,GNU,DELE,QUOFG,NSPEC
;+
;***********************************************************************
;***********************************************************************
;**   If IQUO is unequal 0 an external model for the ratio of F/G has
;**   to be provided. The model is given through the array PARQ
;**   LG is the cut-off energy in units of DELE
;**   QUOFG(L)  = S_0 +(S_1 - S_0) * L^2
;**   FINT:  Integrated G(omega)
;**   FINT1: Integrated S(omega)*G(omega)
;**   QUO0: S_@
;**   OMAV2: second moment of G(omega)
;-

      COMMON muph_PAR1 & COMMON muph_PAR2

      LM5=2*LM
      IF (NSPEC NE 2) then begin & QUOFG[0:LM5-1]=1. & RETURN & endif

;     If IQUO is unequal 0 an external model for the ratio of F/G has
;     to be provided. The model is given through the array PARQ

      IF (IQUO NE 0) then begin & QUOFG[0:LPAR-1]=PARQ[0:LPAR-1] & RETURN & endif

;     LG is the cut-off energy in units of DELE
      LG =long(PARQ[0]/DELE+1) ;!!!
      XLG=FLOAT(LG)
      IF (PARQ[1] NE 0.) then QFG1=PARQ[1]

      idx= findgen(XLG)
      XLR=(idx+1.)/XLG
      XL2=(idx+1-.5)
      XL2= XL2*XL2

;     QUOFG(L)  = S_0 +(S_1 - S_0) * L^2
      QUOFG[idx]=QFG0+(QFG1-QFG0)*XLR*XLR
      tmp  =GNU[idx]
      FINT =total(FINT +tmp)
      FINT1=total(FINT1+tmp*QUOFG[idx])
      FINT2=total(FINT2+tmp*XL2)
      FINT4=total(FINT4+tmp*XL2*XL2)

;     FINT:  Integrated G(omega)
;     FINT1: Integrated S(omega)*G(omega)
      FINT =FINT *DELE
      FINT1=FINT1*DELE

;     QUO0: S_@
      QUOO=(1.-FINT1)/(1.-FINT)

;     OMAV2: second moment of G(omega)
      idx  =findgen(LM)+1
      XL2  =(idx-.5)^2
      OMAV2=total(XL2*GNU[idx-1.])

      QUOFG[LG:LM5-1]=QUOO

      END


;***********************************************************************
;***********************************************************************
      pro muph_SEPAR, LM,GNU,DELE,QUOFG,FPI,NSPEC
;+
;***********************************************************************
;***********************************************************************
;**   FINT: Integrated S(omega)*G(omega)
;**   FPI: partial densities of state
;-

      COMMON muph_PAR1

      LM5=2*LM
      IF (NSPEC NE 2) then begin
        FPI[0,0:LM5-1]=GNU[0:LM5-1] & FPI[1,0:LM5-1]=0. & RETURN & endif

;     FINT: Integrated S(omega)*G(omega)
      FINT=total(GNU[0:LM-1]*QUOFG[0:LM-1])*DELE

;     FPI: partial densities of state
      FPI[0,0:LM5-1]= RNOR*(CONCI[1]-ALFI[1]*QUOFG[0:LM5-1]/FINT)*GNU[0:LM5-1]
      FPI[1,0:LM5-1]=-RNOR*(CONCI[0]-ALFI[0]*QUOFG[0:LM5-1]/FINT)*GNU[0:LM5-1]

      SUM1=total(FPI[0,0:LM-1])*DELE
      SUM2=total(FPI[1,0:LM-1])*DELE

      print,'' & PRINT,FINT,SUM1,SUM2, format="(' INTEGRALS (FNU,FNU1,FNU2):',3F12.5)"

      END

;***********************************************************************
;***********************************************************************
      pro muph_VIPHO, GNUM,GNUP,NCASE ;!!!
;+
;***********************************************************************
;***********************************************************************
;**   Folding of GNUM with GND. Output is stored in GNUP.
;**   Attention: The folding takes place in the range from 0 to 2 E_max
;-

      COMMON muph_MULTI

;
;     NCASE = 1 for first run

      IF (NCASE NE 1) then begin
;     *******************************
       GNUP[0:LM-1]=0.
       K1 =LM2+1-(lindgen(LM2)+1)
       tmp=GND[K1-1]
       K3=(lindgen(LM2)+1)-LM
       FOR I=0,LM-1 do begin
        K2=I+K3
        idx=where(K2 LE 0) & if idx[0] GE 0 then K2[idx]=abs(K2[idx])+1
        GNUP[I]=GNUP[I]+total(tmp[0:LM2-1]*GNUM[K2[0:LM2-1]-1])
       ENDFOR
       GNUP[0:LM-1]=GNUP[0:LM-1]*DELE

       GNUP[LM1-1:LM2-1]=0.
       LO=3*LM-(indgen(LM2)+1)+1
       FOR I=LM1,LM2 do begin
           GNUP[I-1]=GNUP[I-1]+total(reverse(GND[LM2-LO[I-1]:LM2-1])*GNUM[I-1-LM:I-LM+LO[I-1]-2-LM])
       ENDFOR
       GNUP[LM1-1:LM2-1]=GNUP[LM1-1:LM2-1]*DELE

      ENDIF ELSE begin
;     *******************************
       GNUP[0:LM2-1]=0.
       LO=LM2-(lindgen(LM2)+1)+1
       FOR I=1,LM2 do begin
           GNUP[I-1]=GNUP[I-1]+total(reverse(GND[LM2-LO[I-1]:LM2-1])*GND[I-1:I+LO[I-1]-2])
       ENDFOR
       GNUP[0:LM2-1]=GNUP[0:LM2-1]*DELE
      ENDELSE

      END

;***********************************************************************
;***********************************************************************
      pro muph_MEAN,DELE
;+
;***********************************************************************
;***********************************************************************
;**   Subroutine MEAN averages the time spectrum in the region where
;**   there are several time channels per corresponding energy channel
;-

      COMMON muph_AVAR

      Z0[NM+0]=Z0[NM-1]
      Z0[NM+1]=Z0[NM-1]
      Z0[NM+2]=Z0[NM-1]
      FOR N=4-1,NM-1 do begin
        RDH=(HOT[N]-HOT[N-1])/DELE
        RDH=1./ABS(RDH)
        FA1=0. & FA2=0. & FA3=0.
        IF (RDH GT 1.) then FA1=.5*(RDH-1.)
        IF (RDH GT 3.) then begin & FA1=1. & FA2=.5*(RDH-3.) & endif
        IF (RDH GT 5.) then begin & FA2=1. & IF(N GE 4) then FA3=1. & endif
        FNORM=1./(1.+2.*(FA1+FA2+FA3))
        Z1[N]=FNORM*(Z0[N]+FA1*(Z0[N-1]+Z0[N+1])+FA2*(Z0[N-2]+Z0[N+2])+FA3*(Z0[N-3]+Z0[N+3]))
      ENDFOR
      Z0[4-1:NM-1]=Z1[4-1:NM-1]
      END

;***********************************************************************
;***********************************************************************
      pro muph_MOMENT, GNU,HO,LMAX,DHO,CMO
;+
;***********************************************************************
;***********************************************************************
;** muphocor stuff
;-

      SUM=total(GNU[0:LMAX-1])*DHO
      GNU[0:LMAX-1]=GNU[0:LMAX-1]/SUM

      FOR M=0,9 do begin
        MEX=M+1-3
        CMO[M]=0.
        FOR L=0,LMAX-1 do CMO[M]=CMO[M]+GNU[L]*HO[L]^MEX
        CMO[M]=CMO[M]*DHO
      ENDFOR

      FOR M=1,10 do begin
        CMO[M-1]=(CMO[M-1]*M)/3.
        EX=M-3.
        IF (M EQ 3) then EX=1.
        EX=1./EX
        CMO[M-1]=CMO[M-1]^EX
      ENDFOR

      CMO[2]=total(GNU[0:LMAX-1]*ALOG(HO[0:LMAX-1]))
      CMO[2]=EXP(.3333333+CMO[2]*DHO)

      END

;******************************************************************
      pro muph_HEAT, LC,W,DVA,CTA,T,CV,TETAD
;+
;******************************************************************
;**   FREQUENCY SCALE IN MEV
;-

;     FREQUENCY SCALE IN MEV
      CV=0.
      FOR L=0,LC-1 do begin
        WR=11.605*W[L]/T
        IF (WR LE 100.) then begin
          EX=EXP(WR)
          IF (WR LE 50.) THEN CV=CV+EX*WR*WR*CTA[L]/((EX-1.)^2) $
                         ELSE CV=CV+WR*WR*CTA[L]/EX
        ENDIF
      ENDFOR

      CVR=CV*DVA
      CV =CVR*24.9429
      IF (CVR LE .25) then begin
        CVRI=CVR^(-.33333333)
        TETAD=4.27133*T*CVRI*(1.-9.679*EXP(-2.9576*CVRI))
      ENDIF ELSE begin
        IF (CVR LE .6) then begin
          DCVR=CVR-.26982
          X2=84.+158.238*DCVR+323.91*DCVR^3
          TETAD=500.*T/X2
        ENDIF ELSE begin
          DCVR=20.*(1.-CVR)
          X2=DCVR*(1.+.03571429*DCVR+1.448728E-3*DCVR^2+6.24974E-5*DCVR^3+3.8E-6*DCVR^4)
          TETAD=T*SQRT(X2)
        ENDELSE
      ENDELSE

      END

;***********************************************************************
;***********************************************************************
      pro muph_RESCOR,Z0,Z00 ,u
;+
;***********************************************************************
;***********************************************************************
;** muphocor stuff
;-

      COMMON muph_ROS & COMMON muph_RESO

      HO =fltarr(n_elements(Z0)) & QUO1=HO & QUO2=H0 & ERR =HO & DEN=HO
      Z1 =HO & Z2=H0  & TAU=HO   & Z01 =HO & NIT =HO & DEN2=HO
      RES=fltarr(100) & RIMP=RES ;!!! 100
      debug=0

      XEX=2.3
;     DELL = ratio of the distances Fermi-sample over sample detector
      DELL=70./300. ;!!!

;     NUS and NOS define the range of time channels where the correction is to be applied.
;     IGL: number of smoothing processes
;     ITM: number of iterations for the partial deconvolution


      NUS =0L & NOS=0L & IGL=0L & ITM=0L
      READF,u,NUS,NOS,IGL,ITM ;,format="(4I5)"
      ITM1=ITM+1

;     TAUE: primary resolution; TAUT: secondary resolution
      TAUE=0. & TAUT=0. & DFP=0. & UNT=0. & FERR=0.
      READF,u,TAUE,TAUT,DFP,UNT,FERR ;,format="(8F10.3)"
      IF (FERR EQ 0.) then FERR=1.

      NM =NO-NU+1
      NMS=NOS-NUS+1
      NUD=NUS-NU

      PRINT,TAUE,TAUT,UNT,FERR, format="(' DELTA-T(PRIM.)=',F8.2,'   DELTA-T(SEC.)=',F8.2,"+ $
                                       "'   EXP.BACKGROUND=',F10.2,'   FACTOR FOR DELTA-T=',F9.3)"

;     T0: time of arrival of elastically scattered neutrons, flight path
;     FP in cm
      T0=SQRT(522.724/E0)*FP
      T0=T0/CW

      FOR N=1,NM do begin
;       DELK: time channel with respect to elastic line
        DELK=NU-1+N-XNEL
;       RT3: conversion factor from delta(w) to delta(t)
;;      Reichardt:
;;      RT3=(1.+DELK/T0)^3+DELL
;;      TAU(N-1)=SQRT(TAUT^2+(TAUE*RT3)^2)
        RT3=(1.+DELK/T0)^3
        TAU[N-1]=SQRT(TAUT^2+(TAUE*(RT3+DELL))^2)
        DEN[N-1]=2. * (E0/(T0*CW)) * (TAU[N-1]/RT3)
;       H0: hw
        HO[N-1] =E0*(1.-(T0/(T0+DELK))^2)

        IHONEW = ABS(10*HO[N-1])
        IF(N EQ 1) THEN begin & IHOOLD=IHONEW & ERES[IHOOLD-1]=DEN[N-1]
        ENDIF ELSE begin
          IF(IHONEW GT IHOOLD) THEN begin
            F =  ABS(IHONEW-IHOOLD)
            FOR I=1,IHONEW-IHOOLD do ERES[IHOOLD+I-1]=ERES[IHOOLD-1] + I/F*(DEN[N-1]-ERES[IHOOLD-1])
            IHOOLD=IHONEW
          ENDIF
          IF(IHONEW LT IHOOLD) THEN begin
            F = ABS(IHONEW-IHOOLD)
            FOR I=1,IHOOLD-IHONEW do ERES[IHOOLD-I-1]=ERES[IHOOLD-1] + I/F*(DEN[N-1]-ERES[IHOOLD-1])
            IHOOLD=IHONEW
          ENDIF
        ENDELSE

        ZAB=ABS(Z0[N-1])
;       ZO: measured spectrum
        Z01[N-1]=Z0[N-1]
        IF (ZAB EQ 0.) then ZAB=1.
;       ERR: statistical error of measured spectrum
        ERR[N-1]=FERR*SQRT(ZAB+2.*UNT)/ZAB
      ENDFOR

      uu=-1 & on_ioerror,misres & OPENW,uu,'resolution_lamp.plot',/get_lun
      FOR N=0,NM-1 do PRINTF,uu, N+1, HO[N],DEN[N], format="(I5,2F9.3)"
      misres:IF uu GT 0 then free_lun,uu

      IF (ITM EQ 0) THEN begin & Z00[0:NM-1]=Z0[0:NM-1] & IFOLD=1 & RETURN & ENDIF

      PRINT,'' & PRINT,IGL ,format="('NUMBER OF SMOOTHING PROCESSES:',I3)"

      PRINT,ITM ,format="('MAXIMUM NUMBER OF ITERATIONS:',I3)"

;     *******RIMP(n): Rel. resolution obtained after n iterations *******

;     RIMP(n)=sqrt((RIMP(n-1)^4+RIMP(n-1)^8)/(1+RIMP(n-1)^4+RIMP(n-1)^8))
      RIMP[0]=1.
      FOR N=1,ITM1-1 do begin & RIM=RIMP[N-1] & RIMP[N]=RIM*(1.+RIM)/(1.+RIM+RIM*RIM) & endfor
      RIMP[0:ITM1-1]=SQRT(RIMP[0:ITM1-1])

      PRINT,'' & PRINT  ,format=' RESOLUTION TAU[N]/TAU[0] N=0,1,2,--,ITM'
      PRINT,RIMP[0:ITM1-1] ,format="(10F12.4)"

;     ******************************************************************
;     ********* Smoothing of input spectrum ****************************
      IF (IGL NE 0) then begin
        Z00[0:NMS-1]=Z0[NUD:NUD+NMS-1]
        FOR I=1,IGL do begin
          muph_GLAT0,Z00,Z01,NMS
          Z00[0:NMS-1]=Z01[0:NMS-1]
        ENDFOR

        Z01[0:NM-1]=Z0[0:NM-1]
        Z01[NUD:NUD+NMS-1]=Z00[0:NMS-1]

        PRINT,NUS,NOS, format="('SMOOTHING FROM CHANNEL NUMBER',I5,'TO ',I5)"
      ENDIF

;     ******************************************************************
;     ***** Special treatment of the limits of the spectrum ************

      FOR N=1,NM do begin
        NUG=TAU[N-1]/CW
        IF (NUG-N LT 0) then break & endfor ;.f
      NUG=NUG+1

      FOR N=1,NM do begin
        NOG=TAU[NM+1-N-1]/CW
        IF (NOG-N LE 0) then break & endfor ;.f
      NOG=NM-NOG-1

      PRINT,'' & PRINT,NM,NUG,NOG ,format="(3I10)"
      NUG1=NUG-1
      TAU[0:NUG1-1]=lindgen(NUG1)*CW+.00001

;     ******************************************************************

      NIT[0:NM-1] =0  & Z1[  0:NM-1]=Z01[0:NM-1] & Z2[0:NM-1]=0.
      QUO1[0:NM-1]=0. & QUO2[0:NM-1]=0.

;     *********** beginning of iterations ******************************

      IT=0
      repeat begin & IT=IT+1
       IDEV=0
       FOR N=1,NOG-1 do begin

;       RES: resolution function normalized to 1
        JM=TAU[N]/CW
        FOR J=1,JM do begin
          EX=(2.*J*CW/TAU[N])^XEX
          RES[J-1]=EXP(-.69315*EX)
        ENDFOR
        RSUR=1./(1.+2*total(RES[0:JM-1]))
        RES[0:JM-1]=RES[0:JM-1]*RSUR

;       ***** folding of Z1 and RES **********
        Z2[N]=Z1[N]*RSUR + total((Z1[N-JM:N-1]+Z1[N+1:N+JM])*RES[0:JM-1]) ;!!! N missing

        QUO1[N]=Z2[N]/Z01[N]
        DEV=ABS(QUO1[N]-1.)
;       Abbruchbedingung eq. (4.2)
        IF (DEV LE ERR[N]) then IDEV=IDEV+1 $
        ELSE begin
;       *** new partially deconvoluted spectrum **
          Z1[N] =Z1[N]/QUO1[N]
          NIT[N]=NIT[N]+1
        ENDELSE
        QUO2[N]=Z1[N]/Z01[N]
       ENDFOR

;     ************** End of iteration loop *****************************

      endrep until (IT-ITM NE 1)

      Z00[0:NM-1]=Z0[0:NM-1]+Z1[0:NM-1]-Z01[0:NM-1]
      DEN2[0:NM-1]=DEN[0:NM-1]*RIMP[NIT[0:NM-1]]

      PRINT,''
      PRINT,'   N    OMEGA    TAU    DEL-OM   Z-IMP   Z-IMP-SM'+ $
            ' Z-CALC   III/III   LIMIT     Z-CORR-SM  CORR.F.   Z-CORR  N-IT  DEL-OM'
      PRINT,''
      PRINT,'                                  I         II       III'+ $
            '                             IV      IV/II                     '
      IF debug then pas=1 ELSE pas=NM-1
      FOR N=0,NM-1, pas do $
      PRINT,N+1,HO[N],TAU[N],DEN[N],Z0[N],Z01[N],Z2[N],QUO1[N],ERR[N],Z1[N],QUO2[N],$
            Z00[N],NIT[N],DEN2[N],format="(I5,3F8.2,3F10.0,2F9.3,F13.0,F9.3,F10.0,I5,F8.2)"

     END

;***********************************************************************
;***********************************************************************
      pro muph_GLAT0,F0,F1,MAX
;+
;***********************************************************************
;***********************************************************************
;** muphocor stuff
;-

      MAX1=MAX-1
      MAX2=MAX-2
      FOR M=3-1,MAX2-1 do F1[M]=(-6.*F0[M-2]+24.*F0[M-1]+34.*F0[M]+24.*F0[M+1]-6.*F0[M+2])/70.
      F1[0]=62.*F0[0]+18.*F0[1]-6. *F0[2]-10.*F0[3]+6. *F0[4]
      F1[1]=18.*F0[0]+26.*F0[1]+24.*F0[2]+12.*F0[3]-10.*F0[4]
      F1[MAX1-1]=-10.*F0[MAX-5]+12.*F0[MAX-4]+24.*F0[MAX2-1]+26.*F0[MAX1-1]+18.*F0[MAX-1]
      F1[MAX-1] =6.  *F0[MAX-5]-10.*F0[MAX-4]-6. *F0[MAX2-1]+18.*F0[MAX1-1]+62.*F0[MAX-1]
      idx=[1,2,MAX1,MAX]-1
      F1[idx]=F1[idx]/70.
      END

;**********************************************************************
      pro muph_SPLOT, NU,NO,XAX,Y1,Y1M,Y2,Y2M
;+
;**********************************************************************
;** muphocor stuff
;-

      debug=0

      POS=strarr(121)

      Y1M =max(Y1[NU-1:NO-1])>0.
      idx =findgen(6)+1.
      YSC1=.2*Y1M*idx
      YSC2=.2*Y2M*idx

      PRINT,'' & PRINT,'                          G(OM)  (EQUIDISTANT ENERGY INTERVALS)'
      PRINT,YSC1, format="(22X,F8.3,5(12X,F8.3))"

      POS[0:119]='-'  & pos[120]=' '
      FOR I=1,7 do POS[20*(I-1)]='I'
      PRINT,POS,format="(9X,121A1)"
      IF debug then $
      FOR I=NU-1,NO-1 do begin
        POS[*]=' '
        POS[0]='I'
        FOR J=1,6 do POS[20*J]=']'
        IPOS=(100*Y2[I]/Y2M+.5)<120 ;!!! ROUND
        POS[IPOS]='*'
        IPOS=(100*Y1[I]/Y1M+.5)>0
        POS[IPOS]='+'
        PRINT,XAX[I],POS, format="(F8.2,1X,121A1)"
      ENDFOR

      POS[0:119]='-'
      FOR I=1,7 do POS[20*(I-1)]='I'
      PRINT,POS  ,format="(9X,121A1)"
      PRINT,YSC2 ,format="(22X,F8.3,5(12X,F8.3))"
      PRINT,'                                 INT ( G(OM) )    '
      END

;***********************************************************************
      function muphocor ,fifi, first_guess=fg, multi_phonon=Wmp
;+
;**   ******** ********
;** User callable
;** call muphocor ,fifi, first_guess=fg, multi_phonon=Wmp
;-

      common muph_dids, curfifi,curpath

      if n_elements(curfifi) ne 1 then begin & curfifi='' & curpath='' & endif
      if n_elements(fifi) ne 1 then begin
           fifi=dialog_pickfile(path=curpath,get_path=gpth,file=curfifi,resource_name='lamptouch')
           if fifi ne '' then begin
              curpath=gpth
              curfifi=strmid(fifi,strlen(gpth),80)
           endif
      endif
      Wout=0
      if fifi gt ' ' then begin & muph_cor,fifi, Wdos, Wmulti, title
       if n_elements(Wdos) gt 1 then begin
         if keyword_set(fg) then Wout=Wdos[*,1] else Wout=Wdos[*,2]
         if keyword_set(fg) then Wtit=strcompress(title)+" (first guess)" else Wtit=strcompress(title)
         datp1={X:Wdos[*,0],      X_tit:'Energy meV', Y_tit:'G(w) meV-1'  , W_tit:Wtit, Other_tit:fifi}

         if  (n_elements(Wmp) eq 1) and (n_elements(Wmulti) gt 1) then begin
           datp2={X:Wmulti[*,0],  X_tit:'Energy meV', Y_tit:'Multi phonon', W_tit:Wtit, Other_tit:fifi}
           GIVE_W,Wmulti[*,1:*],W=Wmp & GIVE_DATP,datp2,W=Wmp
         endif
         GIVE_DATP,datp1
       endif
      endif
      return,Wout
      END
