PRO hs_dos_write, w_2_write, DATP=datp,FILE_OUT=file_out,NSPECIES=n_sps,T2MIN=Tmin,T2MAX=Tmax,SANGLES=sangles $
                , CHANNELS=Channel,CNT_EFF=icnt_eff,DW=dw,AMASS=amass,XS=xs,NPH=n_ph,UNTER=unt,CONC2=conc2,EMAX=emax
;+
;** ************
;**
;** output workspace for Helmut Schober's DOS routine,
;** file header has a precise format
;** some values osbtained from parameters of workspace
;** others may be edited, use keywords for this
;-

IF not(keyword_set(file_out)) then file_out='input_mupho.txt'
IF not(keyword_set(n_sps)) THEN n_sps=1  ELSE n_sps=n_sps<3               ; number of species (NSPEC)       <3
IF not(keyword_set(amass)) then amass=[1.]
IF not(keyword_set(xs)) then xs=[1.]
IF n_elements(Tmin)      ne 1 THEN tmin =13                           ; smallest  scattering angle      13.075 IN4
IF n_elements(Sangles)   ge 1 THEN tmin =Sangles[0]                       ;
IF n_elements(Tmax)      ne 1 THEN tmax =120                             ; largest   scattering angle     120.125 IN4
IF n_elements(Sangles)   eq 2 THEN tmax =Sangles[1]                       ;
IF n_elements(Channel)   eq 2 THEN min_chan =Channel[0] ELSE min_chan=1   ; lower channel number             1
IF n_elements(Channel)   eq 2 THEN max_chan =Channel[1] ELSE max_chan=511 ; lower channel number           511
IF n_elements(icnt_eff)  eq 0 THEN cnt_eff  =3.3 ELSE cnt_eff=icnt_eff[0] ; counter efficiency               3.3
IF n_elements(icnt_eff)  lt 2 THEN i_cnt_eff=1 ELSE i_cnt_eff=icnt_eff[1] ; correct. for counter effic. 0/1  1
IF n_elements(dw)        eq 0 THEN dw       =0.03 ELSE      dw=dw[0]      ; Debye Waller coefficient [meV]   0.03
IF n_elements(iter_dw)   lt 2 THEN iter_dw  =1    ELSE iter_dw=dw[1]      ; correct. for DW-coefficient 0/1  1
IF n_elements(amass)    ne n_sps THEN BEGIN & Print,"PROBLEM: AMASS dimension doesn't match the number of species... input file not prepared" & GOTO, FINFIN & ENDIF   ; atomic mass [uma]
;IF n_elements(amass1)    ne 1 THEN amass1   =40                           ; atomic mass [a.u.] species 1    40
IF n_elements(xs)    ne n_sps THEN BEGIN & Print,"PROBLEM: XS dimension doesn't match the number of species... input file not prepared" & GOTO, FINFIN & ENDIF   ; scattering xs in barns
;IF n_elements(xs)       ne 1 THEN xs;      =7.64                         ; scat. cross-sect. [barn] spec1   7.8
IF n_elements(n_ph)      ne 1 THEN n_ph     =1   ELSE n_ph = n_ph<5       ; phonon terms included in GDOS   <5
IF n_elements(unt)       ne 1 THEN unt      =40                           ; UNT                             40

;***************************************************************
; set default parameters for header, descriptions from .par file
; if statements check for new values passed as parameters
; if parameter NOT passed set default
;***************************************************************
; datp parameters
;****************
IF n_elements(datp) eq 0 then take_datp,datp & P=datp.P
title=datp.w_tit     ;scientific case                :        CaFeAsF 6% doped IN6
e0=81.796/(P[21]^2)  ;energy [meV]                   :        81.796/(4.16^2)
fp        =P[27]*100 ;length of flight path [cm]     :        2.483 * 100
cw        =P[18]     ;channel width [mys]            :        4.8177
el        =P[9]      ;elastic line [channel number]  :      620
sam_temp  =P[11]     ;sample temperature [K]         :      300

abs_coeff=0.0        ;absorption coefficient         :        0
n_de     =500        ;number of energy intervals (LM):      500
iquo     =0          ;IQUO                           :        0
lpar     =1          ;LPAR                           :        1
if not(keyword_set(emax)) then maxEn=120 else maxEn=emax       ;cut-off energy of GDOS [meV]   :      120
amass1=amass[0] & xs1=xs[0]
amass2=0. & if n_sps eq 2 then amass2=amass[1]       ;atomic mass [a.u.] species 2   :        0
xs2=0. & if n_sps eq 2 then xs2=xs[1]                ;scat. cross-sect. [barn] spec.2:        0
if n_sps eq 1 then conc2=0.
if n_sps eq 2 then begin
IF not(keyword_set(conc2)) then begin
Print,"Conc2 keyword not set... Put Concentration for species 2 to 50%"
conc2=0.5
ENDIF
ENDIF
; conc2 = concentration species 2        :        0

iter_gdos=10         ;iterations for GDOS calcul.    :       10
diff_quot=10         ;difference/quotient method(0/n):       10
i_res_cor=0          ;resolution correction          :        0
ipr      =0          ;IPR                            :        0
iglu     =0          ;IGLU                           :        0
bgrd     =1          ;constant background            :      180
bgrd_sf  =0          ;time-dep. background scal. fac.:        0
min_chan_res=50      ;lower channel for resol. corr. :       50
max_chan_res=200     ;upper channel for resol. corr. :      200
i_smooth_res=0       ;smoothing proc. for reso. corr.:        0
iter_res =0          ;iterations for resolution corr.:        0
pt_res   =20.        ;primary time resolution [mys]  :       20
st_res   =25.        ;secondary time resolution [mys]:       25
d_fp     =2.         ;uncertainty in flight path [cm]:        2
ferr     =1          ;FERR                           :        1
ltem     =0          ;LTEM                           :        0
omega_g  =28         ;Omega_g                        :       28

print,strcompress(title),'-> Elastic peak channel:',el,' Temperature:',sam_temp
if el*sam_temp eq 0.  then RETURN

; start outputting info
openw, f_out, file_out,/get_lun ; open file
print,'fichier:',file_out
printf,f_out,title              ; header info
printf,f_out,format='(8f10.2)',e0,fp,cw,el,Tmin,Tmax,abs_coeff,cnt_eff
printf,f_out,format='(16i5)'  ,n_de,n_sps,iquo,lpar
printf,f_out,format='(8f10.2)',maxEn, sam_temp, dw
printf,f_out,format='(8f10.2)',amass1,xs1,1-conc2
;if (n_elements(n_sps) eq 2) then begin
if n_sps eq 2 then begin
    printf,f_out,format='(8f10.2)',amass2,xs2,conc2
    printf,f_out,format='(8f10.2)',omega_g
endif
printf,f_out,format='(16i5)'  ,n_ph,iter_gdos,diff_quot,iter_dw, i_cnt_eff, i_res_cor, ipr,0
printf,f_out,format='(16i5)'  ,0,min_chan,max_chan,min_chan,max_chan,0
printf,f_out,format='(8f10.4)',unt,0
; data
printf,f_out,format='(8f10.4)',w_2_write[min_chan:max_chan]
printf,f_out,format='(8f10.4)',0

free_lun, f_out
FINFIN:
end


FUNCTION sgroup2 ,w_in,NGROUPS=ngroups,THETA_MIN=theta_min,THETA_MAX=theta_max $
                 ,SINT=sint,MUPHO=mupho,nspecies=nsp,SCALE=scale $
                 ,SANGLES=sangles,CHANNELS=Channel,CNT_EFF=icnt_eff,DW=dw,AMASS=amass $
                 ,XS=xs,NPH=n_ph,UNTER=unt,FILE_OUT=file_out,CONC2=conc2,EMAX=emax
;+
;** ***********************************************************
;** User callable
;** S. Rols 21/02/08 rols@ill.fr
;** To group a certain number of spectra
;** The call is w6=sgroup(w_in,ngroups)
;** First version on 10.08.02
;**
;** New version to include modifications needed to replace RENK
;**
;** Added keywords for MUPHOCOR  D.Richard 11/01/2010
;**       SANGLES =[theta_min  , theta_max]
;**       CHANNELS=[min_chan   , max_chan ]
;**       CNT_EFF =[counter eff, 0/1 (evaluation)]
;**       DW      =[D.W.  coeff, 0/1 (evaluation)] (meV)
;**       AMASS   = Atomic mass                    (a.u.)
;**       XS      = scat. cross-sect.              (barn)
;**       NPH     = phonon terms included in GDOS  <5
;**       UNTER   = Untergrund
;**
;** ***********************************************************
;-

COMMON c_lamp_access, inst
COMMON printing, iprint, outstring

take_datp, datp
par =datp.p
x_in=datp.x
y_in=datp.y
e_in=datp.e

iprint=0

; -------------------------------------------
; Donnees experimentales

;Print,'SGROUP ## S.ROLS 2002 ##'

nchannels=n_elements(x_in) & nspectra=N_ELEMENTS(y_in)

if (nspectra eq 1) then begin
    str_mess='Data already grouped... we assume ng=1 and no /sint'
    y_buf=y_in
    w_buf=w_in
    e_buf=e_in
    goto, nogrp
endif

; -------------------------------------------
; tests sur la dimensions des erreurs

sein=n_elements(e_in) & swin=n_elements(w_in)

if (sein ne swin) then begin
    e_in=SQRT(w_in)
    print,"SGROUP:warning - errors were not calculated ... put them to sqrt(w)"
endif


IF (NOT KEYWORD_SET(scale)) THEN scale=1.

; ------------------------------------------
; Redimensionnement si THETA_MIN et THETA_MAX indiqu_s

IF n_elements(theta_min) eq 1 THEN tmin=theta_min ELSE tmin=y_in[0]
IF n_elements(theta_max) eq 1 THEN tmax=theta_max ELSE tmax=y_in[nspectra-1]

ind_in=Where((y_in le tmax) and (y_in ge tmin),ncount)
y_ini =y_in[ind_in]   & nspectra=ncount
w_ini =w_in[*,ind_in] & e_ini=e_in[*,ind_in]
tmin  =y_ini[0]       & tmax=y_ini[ncount-1]

if KEYWORD_SET(sint) THEN sintt=SIN(y_ini*!pi/180.) ELSE sintt=y_ini*0.+1.

for iy=0,ncount-1 do begin
    w_ini[*,iy]=w_ini[*,iy]*sintt[iy]
    e_ini[*,iy]=e_ini[*,iy]*sintt[iy]
endfor

; -------------------------------------------
; Qqes tests sur ngroups et nspectra

IF n_elements(ngroups) eq 1 THEN BEGIN

   itest=FIX(ngroups) - ngroups

   if(ngroups le 0) then begin
       PRINT,'SGROUP: Error - Number of groups MUST be > 0'
       w_out=w_buf
   endif
   if(itest ne 0) then begin
       PRINT,'SGROUP: Error - Number of groups MUST be integer'
       w_out=w_buf
   endif
   if(nspectra lt ngroups) then begin
       PRINT,'SGROUP: Error - Number of groups MUST be <= initial number of spectra'
       w_out=w_buf
   endif
ENDIF ELSE ngroups=1

ratio=FIX(nspectra/ngroups) & rest=nspectra-ratio*ngroups

if iprint ne 0 then print,inst,nspectra,ngroups,ratio,rest

imin=0 & imax=0

w_buf=FLTARR(nchannels,ngroups)
e_buf=FLTARR(nchannels,ngroups)
y_buf=FLTARR(ngroups)

for i=0,ngroups-1 do begin
  ajou=0
  if rest gt 0 then ajou=ratio else ajou=ratio-1
  if imin ne imax then imin=imax+1
  imax=imin+ajou
  if imax gt imin then begin
    w_buf[*,i]=total(w_ini[*,imin:imax],2)/(ajou+1)
    e_buf[*,i]=SQRT(total((e_ini[*,imin:imax])^2,2))/(ajou+1)
    y_buf[i]=total(y_ini[imin:imax])/(ajou+1)
    rest=rest-1
  endif else begin
    w_buf[*,i]=w_ini[*,imin]
    e_buf[*,i]=e_ini[*,imin]
    y_buf[i]=y_ini[imin]
    rest=0
  endelse
endfor

nogrp:e_out=e_buf*scale
w_out=w_buf*scale
if NOT(KEYWORD_SET(file_out)) THEN file_out='input_mupho.txt'
if (n_elements(nsp) ne 1) THEN nsp=1
if KEYWORD_SET(mupho)     THEN BEGIN
HS_DOS_WRITE, w_out, DATP=datp,NSPECIES=nsp,T2MIN=tmin,T2MAX=tmax,SANGLES=sangles   $
                                 , CHANNELS=Channel,CNT_EFF=icnt_eff,DW=dw,AMASS=amass,XS=xs,NPH=n_ph,UNTER=unt,$
                                 FILE_OUT=file_out,CONC2=conc2,EMAX=emax
ENDIF

str_mess='GROUPING Successfull !: '+'NG = '+strcompress(string(ngroups),/remove_all)+' Theta_min = ' $
         +strcompress(string(tmin),/remove_all)+' Theta_max = '+strcompress(string(tmax),/remove_all)
if KEYWORD_SET(sint)  then str_mess=str_mess+' With sin(theta) weighting'
if KEYWORD_SET(mupho) then str_mess=str_mess+' muphocor input file is : input_mupho.txt'

fin:print,str_mess
x_out=x_in
y_out=y_buf

mod_datp, datp, "e", e_out
mod_datp, datp, "y", y_out
give_datp, datp
return, w_out

end
