FUNCTION shoelace, a, debug
;+
;******* ********
;** User callable
;** Shoelace formula for calculating area of arbitrary convex polygon
;** a is an array of points for general shoelace formula.  
;** Polygon must be convex, but points don't need to be ordered
;**
;**                        JRS, 21/10/2009
;**
;** Return 1-point and 2-point input polygons as zero intersect area
;-

    dims      = (SIZE(a))[0]
    numpoints = (SIZE(a))[2]
    IF (dims LT 2)      THEN RETURN, 0
    IF (numpoints LT 3) THEN RETURN, 0

;check for, and remove duplicate points

    mod_a = a[0,*] * a[1,*]
    ii    = SORT(mod_a)
    a     = a[*,ii]
    FOR ipoint = 0, numpoints-2 DO BEGIN
        IF ABS(a[0,ipoint]-a[0,ipoint+1]) LT 1.e-5 AND $
                 ABS(a[1,ipoint]-a[1,ipoint+1]) LT 1.e-5 THEN a[*,ipoint] = [100,100]
    ENDFOR
    ii = WHERE(a[0,*] LT 99, numpoints)
    a  = a[*,ii]

;order polygon points
;choose startpoint
    ii       = WHERE(a[1,*] EQ MIN(a[1,*]), num_mins)
    IF num_mins GT 1 THEN ii = ii[0]
    unit_vec = FLTARR(numpoints) + 1.
    astart   = a[*,ii] # unit_vec
    a_rel    = a - astart

;find angles from startpoint and order
    angles = ATAN(a_rel[1,*]/a_rel[0,*])*180./!pi
    i0     = WHERE (a_rel[1,*] EQ 0 AND a_rel[0,*] EQ 0)        ; index of startpoint
    i1     = WHERE (a_rel[1,*] EQ 0 AND a_rel[0,*] NE 0, count)    ; indexes of points with angle = 0. or 180.
    angles[i0] = 0.0
    IF count GT 0 THEN BEGIN
        FOR isame = 0, N_ELEMENTS(i1) - 1 DO BEGIN        ; points to the right of the start point have angle = 0, to the left have angle = 180.
            IF a_rel[0,i1[isame]] GT 0 THEN angles[i1[isame]] = 0.0001 + isame*0.0001 ELSE angles[i1] = 180.0
        ENDFOR
    ENDIF
    ii     = WHERE(angles LT 0, count)
    IF count GT 0 THEN angles[ii] = angles[ii] + 180.
    ii     = SORT(angles)
    a      = a[*,ii]

    IF debug THEN FOR ip = 0, numpoints-1 DO XYOUTS, a[0,ip], a[1,ip], STRTRIM(STRING(ip),2)

;shoelace formula
    sum = 0
    FOR ipoint = 0, numpoints - 2 DO BEGIN
        sum = sum + a[0,ipoint]*a[1,ipoint+1] - a[1,ipoint]*a[0,ipoint+1]
    ENDFOR
    sum = sum + a[0,numpoints - 1]*a[1,0] - a[1,numpoints - 1]*a[0,0]
    area = 0.5 * ABS(sum)

    RETURN, area

    END

;**************************************************************************
; -------------------------------------------------------------------------

