function slice_hw, win, dQ=De ,sym=sym
;+
;******* ********
;** User callable
;** return the slice vector I(Q,hW=2.0725Q^2 +-dQ) for "Helmut Schober"
;** input: win is a 2D or 3D workspace containing S(Q,W) frame(s)
;**        Q(abcissa) & W(ordinate) are 1D vectors at this time
;**        dQ default is 0.05
;** call : w2=slice_hw( w1 [,dQ=0.05] [,/SYM] )
;**
;** D.R. May 2008 V1.2
;** D.R. Jun 2009 added /SYM to get the symetric slice (@-W)
;-

;Check for inputs
; ---------------
wout=0 & Hv=2.0725
sz=size(win)
if keyword_set(sym)    then sm=-1.  else  sm=1.
take_datp,datp & Q =datp.X & W=datp.Y*sm & E=datp.E^2           ;Get params Q,W,E
if n_elements(De) ne 1 then De=0.05
if n_elements(E)  eq n_elements(win) then eok=1 else eok=0
if sz[0] eq 3 then nz=sz[3] else nz=1
Wh=Hv*Q^2
if sz[0] le 1                                  then return,wout ;Bad inputs
if ((size(Q))[0]  ne 1) or ((size(W))[0] ne 1) then return,wout ;""
idh =where(Wh le max(W))    &  if idh[0] lt 0  then return,wout ;""

;Work in concerned W range
; ------------------------
 idw =where((W ge Wh[idh[0]]) and (W le Wh[idh[n_elements(idh)-1]]))
 W   =W[idw]  & nl=n_elements(W)
 wout=fltarr(nl,nz)
 eout=wout
 Qa  =sqrt(W/Hv)-De   &   Qb=Qa+2.*De

;Do the slice
; -----------
 for i=0,nl-1 do begin & j=idw[i]
     idx=where((Q ge Qa[i]) and (Q le Qb[i])) &  nc=n_elements(idx)
     if idx[0] ge 0 then begin & wout[i,*]=     reform(total(win[idx,j,*],1)) /nc
                   if eok then eout[i,*]=sqrt(reform(total(E[  idx,j,*],1)))/nc
     endif
 endfor

datp.x_tit=datp.y_tit            ;return results to Lamp
datp.y_tit='I[Q,hW]'
mod_datp ,datp,'X',W*sm
mod_datp ,datp,'Y',datp.Z
mod_datp ,datp,'E',reform(eout)
give_datp,datp
return,reform(wout)
end
