;
FUNCTION splice, w_in1, w_in2, min = xmin, max = xmax, xl
;+
;******* ******
;** User callable
;** Splices together two data sets together. Takes w_in1 outside the 
;** xmin->xmax region and w_in2 inside. 
;**
;** ARGUMENTS:
;** min    :min of w_in2 region
;** max    :max of w_in2 region
;**
;** DIMENSIONS
;** must be 2 1-d arrays input
;**
;** COMMAND SYNTAX:
;** w3=splice(w1,w2,min=<min>,max=<max>)
;**
;**                        KHA,JRS 15/11/02
;-

    iprint = 0        ;  if iprint > 0, show debugging messages

    IF(N_ELEMENTS(xl) GT 0) THEN xlimits = xl ELSE xlimits = [xmin,xmax]

    take_datp, dat1
    take_datp, dat2, /third

    xmin = xlimits[0]
    xmax = xlimits[1]

    IF ((SIZE(w_in1))[0] NE (SIZE(w_in2))[0]) THEN BEGIN
        s = 'Splice: Error - input workspaces must have same dimensions'
        i = dialog_message(s, /ERROR)
        RETURN, w_in1
    ENDIF
    nin1 = (SIZE(w_in1))[1]
    nin2 = (SIZE(w_in2))[1]
    IF iprint THEN PRINT, STRTRIM(STRING(nin1),2) + ' points in w_in1'
    IF iprint THEN PRINT, STRTRIM(STRING(nin2),2) + ' points in w_in2'

    n1 = 1
    n2 = 1
    IF ((SIZE(w_in1))[0] NE 1) THEN BEGIN
        IF ((SIZE(w_in1))[0] EQ 2) THEN n1=(SIZE(w_in1))[2] ELSE BEGIN
            n1=(SIZE(w_in1))[2]
            n2=(SIZE(w_in1))[3]
        ENDELSE
    ENDIF

    IF iprint THEN PRINT,'Splice: End of prepare arrays, etc. section.'

; ----------------------------------------------------------------------------
    
    iOK1 = WHERE(dat1.x LE xmin OR dat1.x GE xmax, nOK1)
    iOK2 = WHERE(dat2.x GE xmin AND dat2.x LE xmax, nOK2)
    IF (nOK1 LE 0) OR (nOK2 LE 0) THEN BEGIN
        s = 'Splice: Error - one of the workspaces has no data points to be kept'
        i = dialog_message(s, /ERROR)
        RETURN, w_in1
    ENDIF

    x_out=FLTARR(nOK1 + nOK2) & w_buf = x_out & e_buf = x_out
    w_out=FLTARR(nOK1 + nOK2,n1,n2) & e_out = w_out

    x_out = [dat1.x[iOK1], dat2.x[iOK2]]
    isort = SORT(x_out)
    x_out = x_out[isort]

    FOR i1=0,n1-1 DO BEGIN
        FOR i2=0,n2-1 DO BEGIN
            w_buf = [w_in1[iOK1,i1,i2], w_in2[iOK2,i1,i2]]
            w_buf = w_buf[isort]
            w_out[*,i1,i2] = w_buf
            e_buf = [dat1.e[iOK1,i1,i2], dat2.e[iOK2,i1,i2]]
            e_buf = e_buf[isort]
            e_out[*,i1,i2] = e_buf
        ENDFOR
    ENDFOR

    IF (n2 EQ 1) THEN e_out=REFORM(e_out,nOK1 + nOK2,n1)
    IF (n1 EQ 1) THEN e_out=REFORM(e_out,nOK1 + nOK2)

    IF iprint THEN PRINT,'Splice: End of splicing section.'

; --------------------------------------------------------------------------

    mod_datp, dat1, "x", x_out
    mod_datp, dat1, "e", e_out

        smin=STRTRIM(STRING(xmin),2) & smin = string_round(smin)
        smax=STRTRIM(STRING(xmax),2) & smax = string_round(smax)

    PRINT,'Splice: Splicing w_in1 outside ['+smin+','+smax+'] region, with w_in2 inside'
    dat1.other_tit=dat1.other_tit+' -sp(min='+smin+',max='+smax+')'

finished:
    give_datp, dat1

    RETURN, w_out
    END
