FUNCTION sqw_visu, w_in
;+
;******* ********
;** User callable
;** Uses IDL irregular griding to plot S(Q,w) from constant angle input data
;** Doesn't rebin, or interpolate.  Select "regular grid" in order to look
;** at output.  Cuts using SuperPlot are not possible.
;**
;**                        JRS 6/6/07
;-

    COMMON c_lamp_access, inst

    iprint = 0

    TAKE_DATP, datp

    IF inst EQ 'D7' THEN lambda = datp.p[4] ELSE lambda = datp.p[21]

    Ei   = 81.8066/lambda^2
    phi  = datp.y*!pi/180.
    Eps  = datp.x
    e_in = datp.e

    IF iprint THEN PRINT, 'Ei = ',Ei
    nphi = (SIZE(phi))[1]
    nEps = (SIZE(Eps))[1]

    IF iprint THEN PRINT, 'sqw_visu: Finished "check arrays" section'
; ----------------------------------------------------------------------
; Find gaps

    iphi = INDGEN(nphi-1) + 1
    diff = phi[iphi] - phi[iphi-1]
    mean_diff = MEAN(diff)
    i = WHERE(diff GT 2.*mean_diff)
    IF i[0] EQ -1 THEN BEGIN
        w_in1 = w_in
        e_in1 = e_in
        GOTO, nogaps
    ENDIF

    ngaps = (SIZE(i))[1]    

    FOR igap = 0,ngaps - 1 DO BEGIN
        IF iprint THEN PRINT, 'gap between ',phi[i[igap]],' and',phi[i[igap]+1]
        gapdiff = phi[i[igap]+1] - phi[i[igap]]
        ndets   = FIX(gapdiff/mean_diff)
        IF iprint THEN PRINT, ndets,' detectors to fill in'
        filldiff = gapdiff/FLOAT(ndets+1)
        gapphi = (INDGEN(ndets)+1.)*filldiff + phi[i[igap]]
        phi = [phi,gapphi]
        extraw = FLTARR(nEps,ndets)
        w_in1 = [[w_in],[extraw]]
        e_in1 = [[e_in],[extraw]]
    ENDFOR

    isort = SORT(phi)
    phi = phi[isort]
    w_in1 = w_in1[*,isort]
    e_in1 = e_in1[*,isort]
    nphi = (SIZE(phi))[1]

    IF iprint THEN PRINT, 'sqw_visu: Finished "check for gaps" section'
nogaps:
; ----------------------------------------------------------------------
; Form energy and Q 2d arrays

    Eco    = Eps # (FLTARR(1,nphi)+1.)
    COSphi = (FLTARR(nEps)+1.) # COS(phi)
    Qco    = SQRT((2.*Ei-Eco-2.*SQRT(Ei*(Ei-Eco))*COSphi)/2.07193571)

    w_out  = TRANSPOSE(w_in1)
    e_out  = TRANSPOSE(e_in1)

;adjust zero level
    i = WHERE(w_out NE 0)
    w_out[i] = w_out[i] + ABS(MIN(w_out))

    IF iprint THEN PRINT, 'sqw_visu: Finished "forming Q and E arrays" section'

; -----------------------------------------------------------------------
; Return workspaces

    datp.x_tit      = 'Wavevector Transfer (A^-1)'
    datp.y_tit      = 'Energy Transfer (meV)'
    datp.other_tit = datp.other_tit + ' -sv'

    MOD_DATP, datp, "x", TRANSPOSE(Qco)
    MOD_DATP, datp, "y", TRANSPOSE(Eco)
    MOD_DATP, datp, "e", e_out
    GIVE_DATP, datp

    RETURN, w_out
    END
