;
FUNCTION sumbank, w_in, dQ=dQ, qb
;+
;******* *******
;** User callable
;** For IN4, IN5 and IN6 data.
;**
;** ARGUMENTS
;** dQ:    Q-spacing with which to sum detectors. If dQ is not specified, nothing
;**    is done for IN5 data. For IN6 or IN4, adds data in centre, upper and lower
;**    detector banks by default.
;**
;** DIMENSIONS
;** w_in=w_out(nchannels,nspectra)
;**
;** COMMAND SYNTAX
;** w2=sumbank(w1,dQ=<dQ>)
;**
;**                            KHA,JRS 6/06/07
;-

    COMMON c_lamp_access, inst

    iprint = 0    ; if iprint>0, show debugging messages

    IF iprint THEN PRINT,'Start sumbank:'

    take_datp, datp
    IF(N_ELEMENTS(qb) GT 0) THEN dQ=qb

; ------------------------------------------------------------------------------
;Set up input workspace

    IF (SIZE(w_in))[0] EQ 2 THEN nspectra=(SIZE(w_in))[2] ELSE RETURN, w_in
    IF iprint THEN PRINT,' nspectra =',nspectra
    IF TOTAL(SIZE(datp.e)) NE TOTAL(SIZE(w_in)) THEN BEGIN
        s='Sumbank: errors incompatible with input workspace'
        ii=dialog_message(s,/ERROR)
        RETURN, 0
    ENDIF    
    
    e_in=datp.e
    
    IF N_ELEMENTS(dQ) EQ 0 THEN dQ = 0
    lambda = datp.p[21]
    IF iprint THEN PRINT,' dQ =',dQ,' lambda =', lambda

    y_out = datp.y
    w_out = w_in
    e_out = e_in

    IF iprint THEN PRINT,'End of input setup section'

; ------------------------------------------------------------------------------
;add detectors together with dQ spacing

    k    = 2.*!pi/lambda
    Qarr = 2.*k*SIN(y_out*!pi/360.)    ; convert scattering angle to Q

    Qmin = -1.
    iout = -1
    FOR ispec=0,nspectra-1 DO BEGIN
        Q = Qarr[ispec]
        IF (iprint) THEN PRINT,' ispec =',ispec,' Q =',Q
        IF (ABS(Q-Qmin) GT dQ) THEN BEGIN
            IF ispec NE 0 THEN BEGIN
                y_out[iout]   = y_out[iout]/nout
                w_out[*,iout] = w_out[*,iout]/nout
                e_out[*,iout] = e_out[*,iout]/nout^2
                IF (iprint) THEN PRINT,' Adding',nout,' detectors at 2theta =',y_out[iout]
            ENDIF
            Qmin = Q
            iout = iout + 1
            nout = 1
            y_out[iout]   = y_out[ispec]
            w_out[*,iout] = w_out[*,ispec]
            e_out[*,iout] = e_out[*,ispec]^2
        ENDIF ELSE BEGIN
            nout = nout + 1
            y_out[iout]   = y_out[iout]   + y_out[ispec]
            w_out[*,iout] = w_out[*,iout] + w_out[*,ispec]
            e_out[*,iout] = e_out[*,iout] + e_out[*,ispec]^2
        ENDELSE
    ENDFOR
    IF (iprint) THEN PRINT,' iout =',iout

    y_out[iout]   = y_out[iout]/nout
    w_out[*,iout] = w_out[*,iout]/nout
    e_out[*,iout] = e_out[*,iout]/nout^2

    y_out = y_out[0:iout]
    w_out = w_out[*,0:iout]
    e_out = SQRT(e_out[*,0:iout])
    nspectra = iout + 1

    IF (iprint GT 0) THEN PRINT, $
        'Sumbank: End of '+inst+' detector-adding section'

; ------------------------------------------------------------------------------
;Return parameters and exit

    mod_datp, datp, "y", y_out
    mod_datp, datp, "e", e_out

    s = STRTRIM(STRING(dQ),2) & s = string_round(s)
    CASE inst OF
        'IN5': Qstring = inst + ' sum over dQ = ' + s + ' A-1'
        ELSE: BEGIN
            IF dQ EQ 0.0 THEN Qstring = inst + ' - standard sum over angles'
            IF dQ NE 0.0 THEN Qstring = inst + ' sum over dQ = ' + s + ' A-1'
        END
    ENDCASE
    PRINT,'Sumbank: '+Qstring
    IF dQ EQ 0.0 THEN qs = '' ELSE qs = ':dQ=' + s
    datp.other_tit = datp.other_tit+' -sb('+inst+qs+')'

    give_datp, datp

finished:
    RETURN, w_out

    END
