;
FUNCTION tof_cutter, w_in, Qx=Qx, Qy=Qy, dQ=dQ, E=E, dE=dE
;+
;******* **********
;** User callable
;** Takes 2d cuts through tof single-crystal data (output from tof_qrebin.pro)
;**
;** Dimensions
;** ------------
;** w_in : (n_energies, nQx, nQy) -> w_out : (nQx, n_energies) - Qy cut
;**                 -> w_out : (nQy, n_energies) - Qx cut
;**                 -> w_out : (nQx, nQy)        - E  cut
;**
;**                            JRS 20/12/07
;-

    iprint = 0

    TAKE_DATP, datp

    e_in = datp.e

    IF N_ELEMENTS(Qx) GT 0 THEN dim = 2
    IF N_ELEMENTS(Qy) GT 0 THEN dim = 3
    IF N_ELEMENTS(E)  GT 0 THEN dim = 1
    IF dim GT 1 THEN IF N_ELEMENTS(dQ) LT 0 THEN BEGIN
        s = 'tof_cutter: dQ must be specified'
        ii = dialog_message(s,/ERROR)
        RETURN, w_in
    ENDIF
    IF dim EQ 1 THEN IF N_ELEMENTS(dE) LT 0 THEN BEGIN
        s = 'tof_cutter: dE must be specified'
        ii = dialog_message(s,/ERROR)
        RETURN, w_in
    ENDIF
    IF dim EQ 0 THEN RETURN, w_in

    IF iprint THEN PRINT, 'tof_cutter: End of prelim section'

; ---------------------------------------------------------------------------
; Cut through relevent dimension

    CASE dim OF
    1: BEGIN
        ii=WHERE(datp.x GT E-(dE/2.) AND datp.x LT E+(dE/2.))

        IF TOTAL(ii) EQ -1 THEN BEGIN
            s='tof_cutter: No data in cut'
            ii = dialog_message(s,/ERROR)
            RETURN, w_in
        ENDIF
        npts  = (SIZE(ii))[1]
        IF npts LE 1 THEN w_out = w_in[ii,*,*] ELSE $
                  w_out = TOTAL(w_in[ii,*,*],1)
        IF npts LE 1 THEN e_out = e_in[ii,*,*]^2 ELSE $
                  e_out = TOTAL(e_in[ii,*,*]^2,1)
        x_out = datp.y
        y_out = datp.z
        ytit  = datp.z_tit
        xtit  = datp.y_tit
        otit  = 'E='+STRING_ROUND(STRING(E))+',dE='+STRING_ROUND(STRING(dE))
       END
    2: BEGIN
        ii=WHERE(datp.y GT Qx-(dQ/2.) AND datp.y LT Qx+(dQ/2.))
        IF TOTAL(ii) EQ -1 THEN BEGIN
            s='tof_cutter: No data in cut'
            ii = dialog_message(s,/ERROR)
            RETURN, w_in
        ENDIF
        npts  = (SIZE(ii))[1]
        IF npts LE 1 THEN w_out = TRANSPOSE(w_in[*,ii,*]) ELSE $
                  w_out = TRANSPOSE(TOTAL(w_in[*,ii,*],2))
        IF npts LE 1 THEN e_out = TRANSPOSE(e_in[*,ii,*]^2) ELSE $
                  e_out = TRANSPOSE(TOTAL(e_in[*,ii,*]^2,2))
        x_out = datp.z
        y_out = datp.x
        ytit  = 'Energy Transfer (meV)'
        xtit = datp.z_tit
        otit  = 'Qx='+STRING_ROUND(STRING(Qx))+',dQ='+STRING_ROUND(STRING(dQ))
       END    
    3: BEGIN
        ii=WHERE(datp.z GT Qy-(dQ/2.) AND datp.z LT Qy+(dQ/2.))
        IF TOTAL(ii) EQ -1 THEN BEGIN
            s='tof_cutter: No data in cut'
            ii = dialog_message(s,/ERROR)
            RETURN, w_in
        ENDIF
        npts  = (SIZE(ii))[1]
        IF npts LE 1 THEN w_out = TRANSPOSE(w_in[*,*,ii]) ELSE $
                  w_out = TRANSPOSE(TOTAL(w_in[*,*,ii],3))
        IF npts LE 1 THEN e_out = TRANSPOSE(e_in[*,*,ii]^2) ELSE $
                  e_out = TRANSPOSE(TOTAL(e_in[*,*,ii]^2,3))
        x_out = datp.y
        y_out = datp.x
        ytit  = 'Energy Transfer (meV)'
        xtit  = datp.y_tit
        otit  = 'Qy='+STRING_ROUND(STRING(Qy))+',dQ='+STRING_ROUND(STRING(dQ))
       END
    ENDCASE

    IF iprint THEN PRINT, 'tof_cutter: End of data cut section'

; ----------------------------------------------------------------------------
;normalise and remove zeroes

    ii = WHERE(ABS(e_out) GE 1)

    w_out = w_out/npts
    e_out = SQRT(e_out)/npts

    IF N_ELEMENTS(ii) GT 1 THEN BEGIN
        w_out[ii] = 0
        e_out[ii] = -1
    ENDIF
    
    IF iprint THEN PRINT, 'tof_cutter: End of remove zeroes section'

; ----------------------------------------------------------------------------
;Return variables and exit

    MOD_DATP, datp, "x", x_out
    MOD_DATP, datp, "y", y_out
    MOD_DATP, datp, "e", e_out
    datp.y_tit = ytit
    datp.x_tit = xtit

    datp.other_tit = datp.other_tit + ' -tcut('+otit+')'

    ii = STRPOS(otit, ',')
    IF dim GT 1 THEN unit=' A^-1' ELSE unit = ' meV'
    PRINT, 'tof_cutter: slice at '+STRMID(otit,0,ii)+unit+', of width '+STRMID(otit,ii+1)+unit

    GIVE_DATP, datp

    RETURN, w_out

    END
