;**********************************************
;INPUT AND PLOTTING INTERFACE FOR MAGNETIC CELL
;**********************************************

;Didier.Richard and Bachir.Ouladiaff 2002-2003
;Institut Laue-Langevin GRENOBLE     (@ill.eu)

;**********************************************

;fun MAGNET_FGET  --> Get numeric fields from GUI
;fun MAGNET_GLAT  --> Get a  b  c fields from GUI
;fun MAGNET_SPLIT --> Separate symbol & bonds (c-3-5-8)
;pro MAGNET_CONX  --> Clear or create bonds   (c-3-5-8)
;pro MAGNET_CLALL --> Clear all atom inputs
;pro MAGNET_CLEAR --> Clear atom inputs by item
;pro MAGNET_SAV   --> Read all atom input fields and put in memory
;pro MAGNET_MOV   --> put values from memory into GUI atom fields
;pro MAGNET_PAR   --> Read-write files, fill lamp workspaces, plot the atoms
;pro MAGNET_EVENT --> Event handler for main GUI
;pro MAGNET_PROPA --> Handle the propagation fields
;pro MAGMOD_EVENT --> Event handler for MAGNET_MODCC
;pro MAGNET_MODCT --> atom color table GUI
;pro MAGNET_FREQ  --> Frequency selector GUI
;pro MAGMOV_EVENT --> Event handler for MAGNET_MOVIE
;pro MAGNET_MOVIE --> Rotate and snapshot GUI
;pro MAGPLA_EVENT --> Event handler for MAGNET_PLANE
;pro MAGNET_PLANE --> Add planes GUI
;pro MAGNET_WIN   --> GUI constructor

;External macros: lamp.cbk (optional link to Lamp) sys_dep('FONTS')
;                 liv_objet.pro (plotting procedures)
;                 read_hdf.pro, write_hdf.pro, read_xml.pro, write_xml.pro, read_mag.pro, write_pdb (i/o)

function magnet_fget, base, default
;+
;******* ***********
;**
;** Get numeric fields from GUI (pointed by base)
;-
res=base*0+default
for i=0,n_elements(base)-1 do begin
    widget_control,base[i],get_value=tmp & tmp=strtrim(tmp[0],2) & if tmp le ' ' then tmp='0'
    is=strpos(tmp,'/')
    if is gt 0 then begin & tmp='1.*('+strmid(tmp,0,is)+')'+strmid(tmp,is,50) & ii=execute('tmp='+tmp) & endif
    on_ioerror,misflt & ok=0 & res[i]=float(tmp) & ok=1 & misflt: if not ok then res[i]=default
endfor
return,res
end

function magnet_glat, base, def
;+
;******* ***********
;**
;** Get a b c fields from GUI (pointed by base)
;-

cells_d =[0.,def,0.,def,0.,def]
for i=0,2 do begin & v1=0. & v2=0.
    widget_control,base[i],get_value=tmp & tmp=strtrim(tmp,2) & if tmp[0] le ' ' then tmp=string(def)
    on_ioerror,misflt & reads,tmp+' 0', v1,v2
    if v2 lt v1 then begin & tmp=v1  & v1=v2  & v2=tmp & endif
    cells_d[i*2]=v1   & cells_d[i*2+1] =v2  & misflt:
endfor
return,cells_d
end

function magnet_split, atom, conx
;+
;******* ************
;**
;** Separate symbol & bonds (c-3-5-8)
;-

conx=0 & idx=where(byte(atom) le 57) & idx=idx[0] ; char <=9

if idx gt 0 then begin
   num=strsplit(/EXTRACT,strmid(atom,idx,20),'-',/preserve_null)    & nm =n_elements(num)
   if nm gt 1 then begin & on_ioerror,misnum & conx=fix(num[1:nm-1]) & misnum:
   endif else begin
      num=strsplit(/EXTRACT,strmid(atom,idx,20),'+',/preserve_null) & nm =n_elements(num)
      if nm gt 1 then begin & on_ioerror,misnup & conx=-fix(num[1:nm-1]) & misnup: & endif
   endelse
   return,strmid(atom,0,idx)
endif
   return,atom
end

pro magnet_conx, st
;+
;** ***********
;**
;** Create bonds (c-3-5-8)
;-

forward_function d_livSymbtoType ;,d_livConnex

nj=st.MAXSEQ
if nj gt 1 then begin
 widget_control, st.DATA  ,get_uvalue=atoms & atom_xyz=atoms[0:2,0:nj-1]
 widget_control, st.SYMBOL,get_uvalue=tsymb & atyp=d_livSymbtoType(tsymb,nj)
 propag =lonarr(12,nj)
 angl   =magnet_fget(st.ANGL, 90.)
 celz   =magnet_glat(st.LAT , 10.) & mina=min(atom_xyz,max=maxa)
 if (celz[0] ne 0) or (celz[2] ne 0)  or  (celz[4] ne 0) or (mina  lt -0.01) or (maxa  gt 1.5) then lat=[1.,1.,1.] $
 else lat=[celz[1],celz[3],celz[5]]

 d_livConnex, atyp, atom_xyz, propag, 0, ANGLE=angl, ABC=lat

 if total(propag) gt 0 then begin
   for i=0,nj-2 do begin & k=0
     while ((k lt 11) and (propag[k,i] gt 0)) do begin & tsymb[i]=tsymb[i]+'-'+strtrim(string(propag[k,i]),2) & k=k+1 & endwhile
   endfor
   widget_control, st.SYMBOL,set_uvalue=tsymb
   st.BUILD=1
 endif else st.BUILD=0
endif
end

pro magnet_clall, st
;+
;** ************
;**
;** Clear all atom inputs
;-
          magnet_clear,'clrat',st
          magnet_clear,'clrco',st
          magnet_clear,'rstsi',st
          magnet_clear,'clrmo',st
          magnet_clear,'clrph',st
          magnet_clear,'rstwd',st
          magnet_clear,'rstlg',st
          magnet_clear,'clrcc',st
          magnet_clear,'clrsy',st
end
pro magnet_clear,flog,st
;+
;** ************
;**
;** Clear atom inputs by item
;-
common c_magnet_win, bases, c_colnam, c_colval, ee,nn,snap, donit
common c_magnet_sup, pv_sup, pv_plan ,pv_mk

nj=n_elements(st.SEQ)

  case flog of
  'clrat':begin & widget_control, st.DATA    ,get_uvalue=atoms & atoms[*,*]=0.
                widget_control, st.DATA    ,set_uvalue=atoms              & st.RorI  =0
                widget_control, st.BUTKI[0],set_value='Real vectors ->K1' & st.CURMOM=1
          end
  'clrsy':begin & widget_control, st.SYMBOL,get_uvalue=table & table[*]    =' '
                widget_control, st.SYMBOL,set_uvalue=table
                for i=0,nj-1 do widget_control,st.ATOM[ i]  ,set_value   =' '
                for i=0,nj-1 do widget_control,st.NAM[  i]  ,set_value   ='          '
                for i=0,2    do widget_control,st.LAT[  i]  ,set_value   =' 10.0'
                st.CURSEQ=1 & magnet_mov,'all',st  & st.MAXSEQ=0 & st.BUILD=0
                ee=0 & nn=0 & pv_sup=0 & pv_plan=0 & st.NOEXT =0
          end
  'clrcx':begin & widget_control, st.SYMBOL,get_uvalue=table & idx=strpos(table,'-') & idy=where(idx gt 0)
                st.BUILD=0
                if idy[0] ge 0 then begin
		               for i=0,n_elements(idy)-1 do table[idy[i]]=strmid(table[idy[i]],0,idx[idy[i]])
                   widget_control, st.SYMBOL,set_uvalue  =table
                   for i=0,nj-1 do widget_control,st.ATOM[ i]  ,set_value=table[st.CURSEQ+i-1]
	              endif
          end
  'clrco':begin & widget_control, st.DATA  ,get_uvalue=atoms & atoms[0:2,*]=0.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.COOR[ i,0],set_value   =' '
                for i=0,nj-1 do widget_control,st.COOR[ i,1],set_value   =' '
                for i=0,nj-1 do widget_control,st.COOR[ i,2],set_value   =' '
          end
  'rstsi':begin & widget_control, st.DATA  ,get_uvalue=atoms & atoms[3,*]  =1.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.SITE[ i]  ,set_value   =' 1'
          end
  'clrmo':begin & m=st.CURMOM -1 & j=st.MK*st.RorI*3 ;Real or Imaginary
                widget_control, st.DATA  ,get_uvalue=atoms & atoms[7+j+m*3:7+j+m*3+2,*] =0.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.CMOM[ i,0],set_value   =' '
                for i=0,nj-1 do widget_control,st.CMOM[ i,1],set_value   =' '
                for i=0,nj-1 do widget_control,st.CMOM[ i,2],set_value   =' '
          end
  'clrph':begin & widget_control, st.DATA  ,get_uvalue=atoms & atoms[7+st.mk*3*2:7+st.mk*3*2+st.mk-1,*]=0.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.CPHA[ i]  ,set_value   =' '
          end
  'rstwd':begin & widget_control, st.DATA  ,get_uvalue=atoms & atoms[4,*]  =1.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.WIDTH[i]  ,set_value   =' 1.0'
          end
  'rstlg':begin & widget_control, st.DATA  ,get_uvalue=atoms & atoms[6,*]  =1.
                widget_control, st.DATA  ,set_uvalue=atoms
                for i=0,nj-1 do widget_control,st.WIDTH[i]  ,set_value   =' 1.0'
          end
  'clrcc':begin & widget_control, st.COLOR ,get_uvalue=table & table[*]    =' '
                widget_control, st.COLOR ,set_uvalue=table
                for i=0,nj-1 do widget_control,st.COL[  i]  ,set_value   =' '
          end
  else:
  endcase
end

pro magnet_sav, flog,st
;+
;** **********
;**
;** Read all atom input fields and put in memory
;** st.DATA   ->atoms  (0:2=coord 3=site 4=width 5=color 6=length 7:*=Rmoments+Imoment+Phase , index)
;** st.SYMBOL ->tsymbo (index)
;** st.COLOR  ->tcolor (index)
;-
forward_function d_livTypetoName, d_livSymbtoType

nj =n_elements(st.SEQ)
m  =st.CURMOM -1
cur=st.CURSEQ -1

widget_control, st.DATA  ,get_uvalue=atoms

;      Magn.moments
;      ------------
       res=magnet_fget(st.CMOM, 0.) & j=st.MK*st.RorI*3 ;Real or Imaginary
       for i=0,nj-1 do atoms[7+j+m*3:7+j+m*3+2,cur+i]=[res[i,0],res[i,1],res[i,2]]
       res=magnet_fget(st.CPHA, 0.) & j=st.MK*3*2       ;Phase
       for i=0,nj-1 do atoms[7+j+m,cur+i]=res[i]

if flog ne 'momag' then begin
   widget_control, st.SYMBOL,get_uvalue=tsymbo
   widget_control, st.COLOR ,get_uvalue=tcolor
   for i=0,nj-1 do begin
;      Symbols
;      -------
       widget_control,st.ATOM[i],get_value=sisi & tsymbo[cur+i]=strtrim(sisi[0],2)
       nam=d_livTypetoName(d_livSymbtoType(magnet_split(tsymbo[cur+i]),1))
       if nam eq 'Atom' then nam=' ' & widget_control,st.NAM[i] ,set_value=strmid(nam+'          ',0,10)
;      Atomique coordinates
;      --------------------
       atoms[0:2,cur+i]=magnet_fget(transpose(st.COOR[i,0:2]), 0.)
;      Sites
;      -----
       atoms[3  ,cur+i]=magnet_fget(st.SITE[i]    , 1.) >1
;      Widths/Length
;      -------------
       atoms[4+2*st.WorL  ,cur+i]=magnet_fget(st.WIDTH[i]   , 1.) >0
;      Colors
;      ------
       widget_control,st.COL[i] ,get_value=colo & tcolor[cur+i]=strtrim(colo[0],2)
   endfor
   widget_control, st.SYMBOL,set_uvalue=tsymbo
   widget_control, st.COLOR ,set_uvalue=tcolor

   if st.MAXSEQ le cur+nj then begin & st.MAXSEQ=cur
      for i=0,nj-1 do if tsymbo[cur+i] gt ' ' then st.MAXSEQ=cur+i+1
   endif
endif
widget_control, st.DATA  ,set_uvalue=atoms
end

pro magnet_mov, flog,st,pos
;+
;** **********
;**
;** put values from memory starting at st.CURSEQ into GUI atom fields
;-

forward_function d_livTypetoName, d_livSymbtoType

nj =n_elements(st.SEQ)
m  =st.CURMOM -1
cur=st.CURSEQ -1
sup=0
widget_control, st.DATA  ,get_uvalue=atoms  &  sz=size(atoms)
widget_control, st.SYMBOL,get_uvalue=tsymbo
widget_control, st.COLOR ,get_uvalue=tcolor

;Remove or insert a line
if (flog eq 'inser') then begin & atoms =[[atoms[*,0:pos]],[fltarr(sz[1],1)],[atoms[*,pos+1:sz[2]-1]]] & atoms[3:4,pos+1]=1
                                tsymbo=[tsymbo[0:pos] , '' , tsymbo[ pos+1:sz[2]-1]] & sup= 1
                                tcolor=[tcolor[0:pos] , '' , tcolor[ pos+1:sz[2]-1]] & sz =size(atoms) & endif
if (flog eq 'delet') then begin & sup=-1 & if pos gt 0 then begin
                                atoms =[[atoms[*,0:pos-1]] ,[atoms[*,pos+1:sz[2]-1]]]
                                tsymbo=[tsymbo[0:pos-1]    , tsymbo[ pos+1:sz[2]-1]]
                                tcolor=[tcolor[0:pos-1]    , tcolor[ pos+1:sz[2]-1]] & sz =size(atoms)
                                endif else begin & atoms =atoms[*,pos+1:sz[2]-1] & tsymbo=tsymbo[pos+1:sz[2]-1]
                                                 tcolor=tcolor[ pos+1:sz[2]-1] & sz =size(atoms) & endelse
endif
;Increase tables size if necessary
if cur+nj gt sz[2] then begin & sup=50 & atoms =[[atoms],[fltarr(sz[1],sup)]] & tsymbo=[tsymbo , strarr(sup)]
                                       tcolor=[tcolor , strarr(sup)]
endif
if sup ne 0 then begin & widget_control, st.DATA  ,set_uvalue=atoms
                       widget_control, st.SYMBOL,set_uvalue=tsymbo
                       widget_control, st.COLOR ,set_uvalue=tcolor
endif
;      Magn.moments
;      ------------
  j=st.MK*st.RorI*3    ;Real or Imaginary
  mom=atoms[7+j+m*3:7+j+m*3+2,cur:cur+nj-1] & tmp=strtrim(string(mom),2)
  idx=where(mom eq 0.) & if idx[0] ge 0 then  tmp[idx]=' '
  for i=0,nj-1 do begin & widget_control,st.CMOM[ i,0],set_value=tmp[0,i]
                        widget_control,st.CMOM[ i,1],set_value=tmp[1,i]
                        widget_control,st.CMOM[ i,2],set_value=tmp[2,i] & endfor
  j=st.MK*3*2          ;Phase
  mom=atoms[7+j+m,cur:cur+nj-1]             & tmp=strtrim(string(mom),2)
  idx=where(mom eq 0.) & if idx[0] ge 0 then  tmp[idx]=' '
  for i=0,nj-1 do widget_control,st.CPHA[ i],set_value=tmp[i]

  if flog ne 'momag' then begin
     for i=0,nj-1 do begin
;      Sequence
;      --------
       val=strtrim(string(cur+i+1),2) & if cur+i+1 le 9 then val=' '+val & if cur+i+1 le 99 then val=' '+val
       widget_control,st.SEQ[i] ,set_value=val+' '
;      Name
;      ----
       nam=d_livTypetoName(d_livSymbtoType(magnet_split(tsymbo[cur+i]),1))
       if nam eq 'Atom' then nam=' ' & widget_control,st.NAM[i] ,set_value=strmid(nam+'          ',0,10)
;      Symbols
;      -------
       widget_control,st.ATOM[i],set_value=tsymbo[cur+i]
;      Atomique coordinates
;      --------------------
       for j=0,2 do begin & if atoms[j,cur+i] eq 0 then if nam eq ' ' then res=' ' else res='0 ' $
                          else res=strtrim(string(atoms[j,cur+i]),2)
                          widget_control,st.COOR[i,j],set_value=res     & endfor
;      Sites
;      -----
       if atoms[3 ,cur+i] eq 0 then res='1' else $
       res=strtrim(string(fix(atoms[3 ,cur+i])),2)      & widget_control,st.SITE[i] ,set_value=res
;      Widths/Length
;      -------------
       if atoms[4+2*st.WorL ,cur+i] eq 0 then res=' ' else $
       res=strtrim(string(atoms[4+2*st.WorL ,cur+i]),2) & widget_control,st.WIDTH[i],set_value=res
;      Colors
;      ------
       widget_control,st.COL[i] ,set_value=tcolor[cur+i]
     endfor
  endif
end

pro magnet_par, st ,butid ,export, PCENT=pcent ,WDRAW=wdraw
;+
;** **********
;**
;** 0) if butid=-3 -> export plot image & return
;** 1) if butid=-1 -> read  a file and put atom & parameter values in GUI
;** 1) if butid=-4 -> import from Wks, put atom & parameter values in GUI
;** 2) Get back parameters  fields from GUI
;** 3) Put atom & parameter values into W,X,Y,Z,PV,PTXT,P and gives them to a workspace
;** 4) if butid=-2 -> write a NeXus file
;** 5) if butid> 0 -> do a plot
;-

;st={KV:kv,DUPL:[dupx,dupy,dupz],LAT:[la,lb,lc],MCOO:1,ANGL:[alpha,beta,gamma],MODU:1,COEF:1,MESS:bmes,BUTE:[cart,sphe,dyna,phas,ampl,coef1,coef2],BTRAN:btran,  $
;    SEQ:bseq,NAM:bnam,CURSEQ:1,MAXSEQ:0,ATOM:batom,COOR:[[bcox],[bcoy],[bcoz]],SITE:bsite,CURMOM:1,CMOM:[[bmox],[bmoy],[bmoz]],CPHA:bmph,RorI:0,WorL:0,awrat:.5,$
;    BUTKI:[butki,butri,butci,buthw],WIDTH:bwfct,COL:bcol,FILE:bfil,CURW:19,DATA:watom,SYMBOL:bdef2,COLOR:bdef8,WININF:wininf,BTIT:btit,BPROP:bprop,MK:mk,BUILD:0,NOEXT:0}
common c_magnet_win, bases, c_colnam, c_colval, ee,nn,snap, donit
common c_magnet_sup, pv_sup, pv_plan ,pv_mk
forward_function d_livTypetoSymb, d_livSymbtoType, read_mag

;EXPORT
;******
IF butid eq -3 then begin & if n_elements(wdraw) eq 0 then wdraw=bases[4]
                          liv_objet,WDRAW=wdraw,PS=export, PCENT=pcent & RETURN & endif

widget_control,st.FILE,get_value=bfil & bfil=strtrim(bfil[0],2) & lfil=strlowcase(bfil)
widget_control,/hourglass
ws='0'
if st.wininf[8] gt 0 then begin & widget_control,st.wininf[8],get_value=ws & ws=strmid(ws,1,2) & endif
wn=long(ws)

;READ a FILE or WORKSPACE
;************************
IF (butid eq -1) or (butid eq -4) then begin & snap=0
   w=0 & msg=' Bad file !!!'

   widget_control,st.btran,get_value = transl

   if butid eq -4 then begin  ;Read a workspace
    msg=' Bad workspace !!!'
    CALL_PROCEDURE,'take_datp',datp,W=wn
    ptxt=datp.par_txt & p=[0]
    if (strpos(ptxt[0],'OBJECT:') eq 0) then begin
        CALL_PROCEDURE,'take_w',w   , W=wn
        bfil=datp.other_tit     & wt  =datp.w_tit
        x= datp.x  & y =datp.y  & z   =datp.z
        ee=datp.e  & nn=datp.n
        p =datp.p  & pv=datp.pv
    endif
   endif else $
   if strpos(lfil,'.hdf') ge 0 then begin
      read_hdf, bfil, w, XC=x, YC=y, ZC=z, E=ee, N=nn, PR=p, PV=pv, PAR_TXT=ptxt, W_tit=wt, X_tit=xt, Y_TIT=yt, Z_tit=zt, OTHER_TIT=ot, SNAP=snap
   endif else $
   if strpos(lfil,'.xml') ge 0 then begin
      read_xml, bfil, w, XC=x, YC=y, ZC=z, E=ee, N=nn, PR=p, PV=pv, PAR_TXT=ptxt, W_tit=wt, X_tit=xt, Y_TIT=yt, Z_tit=zt, OTHER_TIT=ot, SNAP=snap
   endif else begin
      if transl eq 'Mat' then begin
         widget_control,bases[6],get_uvalue=MAT
         if total(abs(MAT-[[1,0,0],[0,1,0],[0,0,1]])) ne 0 then MATsym=MAT
      endif
      w=read_mag('inst','',bfil,status,datp, MATsym=MATsym)
      x=datp.x & y=datp.y & z=datp.z & ee=datp.e & nn=datp.n & p=datp.p & pv=datp.pv & ptxt=datp.par_txt & wt=datp.w_tit & xt=datp.x_tit & yt=datp.y_tit & zt=datp.z_tit & ot=datp.other_tit
   endelse
   if (strpos(ptxt[0],'OBJECT:') ne 0) or (p[0] eq 2) or (n_elements(w) eq 0) then begin
                    widget_control,st.MESS,set_value=msg & RETURN &    endif

   kp_nn=nn & kp_ee=ee & magnet_clall,st & nn=kp_nn & ee=kp_ee
                       ; ***************

   for i=0,2 do begin &  v1=p[(i)*2+1] & v2=p[(i)*2+2]
       if v1 eq 0 then tmp=strtrim(string(v2),2) else tmp=strtrim(string(v1),2)+','+ strtrim(string(v2),2)
                widget_control,st.LAT[i] ,set_value=tmp                             ;length abc
   endfor
   for i=0,2 do widget_control,st.ANGL[i],set_value=strtrim(string(p[ 7+i]),2)      ;angles
   for i=0,2 do widget_control,st.DUPL[i],set_value=strtrim(string(fix(p[11+i])),2) ;# cells
   for i=0,6 do widget_control,st.BUTE[i],set_button=0
   st.MCOO =p[14]
   st.BUILD=p[18]
   st.NOEXT=p[20]
   if p[14] eq 1. then widget_control,st.BUTE[0],set_button=1 else $                ;vector input
   if p[14] eq 2. then widget_control,st.BUTE[1],set_button=1 else $
   if p[14] ge 3. then widget_control,st.BUTE[2],set_button=1
   if p[14] ge 3. then widget_control,st.wininf[21],sensitive=0 else widget_control,st.wininf[21],sensitive=1
   if p[14] ge 3. then widget_control,st.wininf[22],sensitive=0 else widget_control,st.wininf[21],sensitive=1
   if p[15] eq 1. then widget_control,st.BUTE[3],set_button=1 else $                ;modulation mode
                       widget_control,st.BUTE[4],set_button=1      & st.MODU=p[15]
   if p[17] eq 1. then widget_control,st.BUTE[5],set_button=1 else $                ;Fourier coef.
                       widget_control,st.BUTE[6],set_button=1      & st.COEF=p[17]
   if p[19] gt 0. then if p[19] lt 1. then st.AWRAT=p[19]                           ;Arrow/Atom ratio

   widget_control,st.btran,get_uvalue=translet
   if n_elements(MATsym) ne 9 then transl=translet[p[16]-1] ;translations   
   widget_control,st.btran,set_value = transl
   mk= p[25]
   magnet_propa, 'clear', st &  if mk gt 0 then begin & Vpro=P[26:26+mk*3-1]
                                        magnet_propa, 'init', st, newV=Vpro & endif ;propagations

   na=n_elements(w) & st.MAXSEQ=na

   tsymbo=strarr(na>50)    & tsymbo[0:na-1]=d_livTypetoSymb(w)
   tcolor=strarr(na>50)    & for i=0,na-1 do  begin & v=long(pv[2,i])
                                              if v ge 0 then begin & n1=long(v/10000) & n2=long(v/100)-n1*100 & n3=v-n1*10000-n2*100
                                              tcolor[i]=strtrim(string(n1),2)+' '+strtrim(string(n2),2)+' '+strtrim(string(n3),2)
                                              endif
                             endfor
   if total(pv[3,0:na-1]) eq 0. then pv[3,0:na-1]=1.

   atoms =fltarr(7+st.mk*3*2+st.mk,na>50) & atoms[3:6 ,0:na-1]=pv[0:3,0:na-1]
                      if mk gt 0 then atoms[7:7 +mk*3-1                 ,0:na-1]=pv[4       :4+mk*3-1     ,0:na-1] ;Real
                      if mk gt 0 then atoms[7+st.mk*3  :7+st.mk*3+mk*3-1,0:na-1]=pv[4+mk*3  :4+mk*3*2-1   ,0:na-1] ;Imaginary
                      if mk gt 0 then atoms[7+st.mk*3*2:7+st.mk*3*2+mk-1,0:na-1]=pv[4+mk*3*2:4+mk*3*2+mk-1,0:na-1] ;Phase
                                      atoms[0   ,0:na-1]=transpose([x])
                                      atoms[1   ,0:na-1]=transpose([y])
                                      atoms[2   ,0:na-1]=transpose([z])
   pv_sup=0 & pv_plan=0  & pvsz=size(pv)
   if pvsz[0] gt 1  then $
   if pvsz[2] gt na then begin & pv_sup=pv[*,na:*] & tmp=pv_sup[0,*] & tmx=where(tmp lt 0)
                               if tmx[0] ge 0 then pv_plan=pv_sup[*,tmx]
			                         if n_elements(tmx) eq pvsz[2]-na then pv_sup=0 else pv_sup=pv_sup[*,where(tmp ge 0)]
	 endif

   idx=4+mk*3*2+mk ;Bonds if there
	 there=0
   for i=0,na-1 do begin & k=0
                 while ((k lt 11) and (pv[idx+k,i] ne 0)) do begin & there=1
                   if pv[idx+k,i] gt 0 then b='-' else b='+'
                   tsymbo[i]=tsymbo[i]+b+strtrim(string(abs(fix(pv[idx+k,i]))),2) & k=k+1 & endwhile & endfor

   widget_control,st.DATA  ,set_uvalue=atoms
   widget_control,st.COLOR ,set_uvalue=tcolor
   widget_control,st.SYMBOL,set_uvalue=tsymbo
   widget_control,st.BTIT  ,set_value =strtrim(wt,2)

   if (p[18] eq 1.) and (there ne 1) then magnet_conx,st ; Build bonds if p18

   magnet_mov,'all',st

ENDIF else begin & magnet_sav,  'all' , st
                 magnet_propa,'save', st & endelse

;READ ALL INPUTS FROM GUI
;************************
IF st.MAXSEQ gt 0 then begin
widget_control, st.btran,get_uvalue=translet,get_value=lab
transl_l=(where(translet eq lab)+1)[0]
if lab eq 'Mat' then transl_l=1
cells_n =magnet_fget(st.DUPL,  1.)
angles  =magnet_fget(st.ANGL, 90.)
cells_d =magnet_glat(st.LAT , 10.)
mom_coor=            st.MCOO
modulat =            st.MODU
f_coef  =            st.COEF
nbatoms =            st.MAXSEQ
build   =            st.BUILD
awrat   =            st.AWRAT
noext   =            st.NOEXT

widget_control, st.SYMBOL,get_uvalue=tsymbo & tsymbo= tsymbo[  0:nbatoms-1]
widget_control, st.COLOR ,get_uvalue=tcolor & tcolor=[tcolor[  0:nbatoms-1]]
widget_control, st.DATA  ,get_uvalue=atoms  & atoms = atoms[ *,0:nbatoms-1]
widget_control, st.BTIT  ,get_value =wt     & wt=strtrim(wt[0],2)
widget_control, st.BPROP ,get_uvalue=Vprop  & mk=st.MK

while (mk ge 2) and (total(abs(atoms[7+(mk-1)*3:7+mk*3-1,*])+abs(atoms[7+st.mk*3+(mk-1)*3:7+st.mk*3+mk*3-1,*])) eq 0) do mk=mk-1
;                                           Real                     Imaginary
pv_mk=mk
x   = reform(atoms[0,*])
y   = reform(atoms[1,*])
z   = reform(atoms[2,*])
ot  = bfil
pv  = fltarr(4+mk*3*2+mk+12,nbatoms)
pv[ 0:3,*]                   =atoms[3:6,*]                          ;Site, width, color, length
pv[ 4       :4+mk*3-1  ,*]   =atoms[7:7+mk*3-1,*]                   ;Real
pv[ 4+mk*3  :4+mk*3*2-1,*]   =atoms[7+st.mk*3  :7+st.mk*3+mk*3-1,*] ;Imaginary
pv[ 4+mk*3*2:4+mk*3*2+mk-1,*]=atoms[7+st.mk*3*2:7+st.mk*3*2+mk-1,*] ;Phase

idx=4+mk*3*2+mk
for i=0,nbatoms-1 do begin & tsymbo[i]=magnet_split(tsymbo[i],conx)   ;Bonds
      pv[idx,i]=conx[*] & endfor
w   = d_livSymbtoType(tsymbo,nbatoms)

n1=long(0) & n2=n1 & n3=n1 & tcoln=transpose([lonarr(nbatoms)-1])
for i=0,nbatoms-1 do begin & on_ioerror,misrds
    ccol=strtrim(strlowcase(tcolor[i]),2) & idx=where(c_colnam eq ccol)
    if idx[0] ge 0 then ccol=c_colval[idx[0]]
    if ccol gt ' ' then begin & reads,ccol+' 0 0',n1,n2,n3 & tcoln[i]=n1*10000+n2*100+n3 & endif
    misrds:
endfor
pv[2,*]=tcoln
if (size(pv_sup))[1]  eq (size(pv))[1] then pv=[[pv],[pv_sup]]
if (size(pv_plan))[1] ge (size(pv))[1] then pv=[[pv],[pv_plan[0:(size(pv))[1]-1,*]]]
ptxt=['OBJECT: '  $
     ,'coord a0' ,'coord a1'  $
     ,'coord b0' ,'coord b1'  $
     ,'coord c0' ,'coord c1'  $
     ,'alpha   ' ,'beta    ' ,'gamma   '  $
     ,'nb atoms'  $
     ,'# cells along a:','# cells along b:','# cells along c:' $
     ,'vectors input: 1:M.cartesian 2:M.spherical 3:Vibration' $
     ,'modulation mode:  1:phase 2:amplitude' $
     ,'translation P:1 C:2 A:3 B:4 I:5 F:6 Ro:7 Rv:8 H:9' $
     ,'Fourier coef (position):  1:cell 2:atom' $
     ,'Bonds: 0:given 1:to build' $
     ,'Maximum arrow/atom radius ratio' $
     ,'Do not extend molecule outside cell' $
     ,'free','free' $
     ,'free','free' $
     ,'nb propagations/modes on X,Y,Z'  $
     , strarr(mk*3) $
     ,'Var_params PV[i  ,*] site       i' $
     ,'Var_params PV[i  ,*] width      i' $
     ,'Var_params PV[i  ,*] color      i' $
     ,'Var_params PV[i  ,*] length     i' $
     ,'Var_params PV[i:j,*] R vectors  i' $
     ,'Var_params PV[i:j,*] I vectors  i' $
     ,'Var_params PV[i:j,*] phases     i' $
     ,'Var_params PV[i:j,*] bonds      i' $
     ]
p   =[ 1              $
     , cells_d[0:1]   $
     , cells_d[2:3]   $
     , cells_d[4:5]   $
     , angles[*]      $
     , nbatoms        $
     , cells_n        $
     , mom_coor       $
     , modulat        $
     , transl_l       $
     , f_coef         $
     , build          $
     , awrat          $
     , noext          $
     , 0,0            $
     , 0,0            $
     , mk             $
     , Vprop[0:(mk-1)*3+2] $
     , 0,1,2,3,4,4+mk*3,4+mk*3*2,4+mk*3*2+mk $
     ]
;SAVE IN WORKSPACE
;*****************
IF wn gt 0 then begin
    datp={w_tit:wt, other_tit:ot, p:p, par_txt:ptxt, x:x, y:y, z:z, pv:pv, N:NN, E:EE}
    CALL_PROCEDURE,'give_w'   ,w   ,W=wn
    CALL_PROCEDURE,'give_datp',datp,W=wn
ENDIF

;WRITE a FILE
;************
IF butid eq -2 then begin &     if donit then begin & liv_objet,WDRAW=st.wininf[0],PS='SNAP',SNAP=snap
                                                  if (size(snap))[0] eq 2 then snap=congrid(snap,192,192)
                              endif
    if (strpos(strlowcase(bfil),'.hdf') lt 0) and (strpos(strlowcase(bfil),'.xml') lt 0) and $
       (strpos(strlowcase(bfil),'.pdb') lt 0) $
        then begin & bfil=bfil+'.xml' & widget_control,st.FILE,set_value=bfil & endif

    if  strpos(strlowcase(bfil),'.hdf') ge 0 then begin
        write_hdf, bfil , w  , XC=x, YC=y ,     ZC=z , E=EE  , N=NN   $
                             , PR=p, PV=pv,     P_TXT=ptxt            $
                             , W_tit=wt   ,     X_tit=' ' , Y_TIT=' ' $
                             , Z_tit=' '  ,     OTHER_TIT=ot, SRC='MagDraw', HIST=' ' $
                             , PAR_TXT_ALL=' ', LIM=' ', MACH=' ',SNAP=snap
    endif else $
    if  strpos(strlowcase(bfil),'.xml') ge 0 then begin
        write_xml, bfil , w  , XC=x, YC=y ,     ZC=z , E=EE  , N=NN   $
                             , PR=p, PV=pv,     P_TXT=ptxt            $
                             , W_tit=wt   ,     X_tit=' ' , Y_TIT=' ' $
                             , Z_tit=' '  ,     OTHER_TIT=ot, SRC='MagDraw', HIST=' ' $
                             , PAR_TXT_ALL=' ', LIM=' ', MACH=' ',SNAP=snap
    endif else $
    if (strpos(strlowcase(bfil),'.pdb') ge 0) and (wn gt 0) then begin
        write_pdb, bfil , W=wn
    endif else $
    if  strpos(strlowcase(bfil),'.fst') ge 0 then begin
       ;write_atm, bfil , w  , XC=x, YC=y ,     ZC=z , E=EE  , N=NN   $
       ;                     , PR=p, PV=pv,     P_TXT=ptxt            $
       ;                     , W_tit=wt   ,     X_tit=' ' , Y_TIT=' ' $
       ;                     , Z_tit=' '  ,     OTHER_TIT=ot, SRC='MagDraw', HIST=' ' $
       ;                     , PAR_TXT_ALL=' ', LIM=' ', MACH=' '
    endif
ENDIF

;DO THE PLOT
;***********
IF butid gt 0 then begin
 beside=st.wininf[6] & wdd=st.wininf[0] & omen=st.wininf[1]
 odim  =st.wininf[2:3]+st.wininf[30:31]-st.wininf[28:29]

 if (not beside) or (nbatoms eq 1) then begin
   widget_control,st.wininf[4] ,map=0 & widget_control,st.wininf[5] ,map=1
   widget_control,st.wininf[9] ,map=0 & widget_control,st.wininf[10],map=1
   widget_control,butid,set_value='Editor',set_uvalue='input'
   widget_control,st.wininf[11],set_value=strmid(wt+' ('+ot+')                                   ',0,38)
   if st.wininf[7] gt 0 then widget_control,st.wininf[7] ,sensitive=0
   if st.wininf[8] gt 0 then widget_control,st.wininf[8] ,sensitive=0

   if strpos(wt,'Step:') eq 0 then begin ;Check step value (>1 if memory lack !!!)
      step=1L & tailwt=strmid(wt,5,3) & reads,tailwt+' 1',step
      if step gt 1 then begin & nA=n_elements(W) & d1=(size(pv))[1] & nF =((size(pv))[2])/nA
                              pv=reform(pv,d1,nA,nF) & nF=nF/step & idx=lindgen(nF)*step
                              pv=reform(pv[*,*,idx],d1,nA*nF)     & print,'Using 1 frame over ',step
      endif
   endif

   liv_objet,WDRAW=wdd,DIM=odim,MENUBAS=omen,L_MESS=st.wininf[11],PP=p,XX=x,YY=y,ZZ=z,WW=w,EE=EE,NN=NN,VV=pv,TITLE=wt
   donit=1
 endif else CALL_PROCEDURE,'see',w=wn,/beside
ENDIF

ENDIF
END

pro M_DYING,id
;+
;** *******
;** when dying, map lamp again
;-
@lamp.cbk
if lamp_b1 gt 0 then widget_control,lamp_b1,bad_id=ii,map=1,iconify=0 else exit
d_livClearAnim
end

pro magnet_event, ev
;+
;** ************
;**
;** The event handler
;-

common c_magnet_sup, pv_sup, pv_plan ,pv_mk
common c_fullprof  , filloc
forward_function d_livTypetoName, d_livSymbtoType

widget_control,ev.id,get_uvalue=uv

if (n_elements(uv) eq 2) and (ev.id ne ev.top) then if uv[0] eq -1  then begin
       on_ioerror,misFP & openr,u,filloc,/get_lun & str=fstat(u) & free_lun,u
	     ctime=str.ctime>str.mtime
		   if ctime gt uv[1] then begin & uv[1]=ctime & widget_control,ev.id,set_uvalue=uv
			                              magdraw,filloc,/plot   & endif
		   misFP: widget_control,ev.id,timer=4. & return & endif

if n_elements(uv) gt 0 then begin
 widget_control,ev.top ,get_uvalue=st
 widget_control,st.MESS,set_value =' '
 nj =n_elements(st.SEQ)
 nk =fix((st.MAXSEQ/10)>nj)

 catch,stat & if stat ne 0 then begin & print,!error_state.msg & return & endif

 if ev.id eq ev.top then begin          ;Resizing plotting window
  widget_control,st.wininf[16],get_uvalue=tmp
  if tmp eq 'input' then begin
     widget_control,ev.top, tlb_get_size=v2 & v2=long(v2)
     if (abs(st.wininf[30]-v2[0]) gt 30) or (abs(st.wininf[31]-v2[1]) gt 30) then begin
       if !version.os_family eq 'Windows' then ban=30 else ban=0
       st.wininf[30]=v2[0]>st.wininf[28] &  st.wininf[31]=(v2[1]>(st.wininf[29]-ban))+ban
       newsx=st.wininf[2]+st.wininf[30]-st.wininf[28]
       newsy=st.wininf[3]+st.wininf[31]-st.wininf[29]
       widget_control,st.wininf[0] , XSIZE=newsx, YSIZE=newsy
       widget_control,ev.top       , SCR_XSIZE=st.wininf[30], SCR_YSIZE=st.wininf[31]
       magnet_par,0,-3,'RESIZE', PCENT=[newsx,newsy]
       widget_control,ev.top, set_uvalue=st
     endif
  endif
  return

 endif else if n_elements(uv) eq 1 then begin
  case uv of
  'fcell':st.coef=1
  'fatom':st.coef=2
  'carte':begin & st.mcoo=1 & widget_control,st.wininf[21],sensitive=1 & widget_control,st.wininf[22],sensitive=1 & end
  'spher':begin & st.mcoo=2 & widget_control,st.wininf[21],sensitive=1 & widget_control,st.wininf[22],sensitive=1 & end
  'dynam':begin & st.mcoo=3 & widget_control,st.wininf[21],sensitive=0 & widget_control,st.wininf[22],sensitive=0 & end
  'phase':st.modu=1
  'ampli':st.modu=2

  'eback':if st.CURSEQ gt 1 then begin & magnet_sav,'all',st & st.CURSEQ=1
                                       magnet_mov,'all',st
          endif
  'fback':if st.CURSEQ gt 1 then begin & magnet_sav,'all',st & st.CURSEQ=(st.CURSEQ-nk*2)>1
                                       magnet_mov,'all',st
          endif
  'lback':if st.CURSEQ gt 1 then begin & magnet_sav,'all',st & st.CURSEQ=(st.CURSEQ-nj+1)>1
                                       magnet_mov,'all',st
          endif
  'lforw':begin & magnet_sav,'all',st
            if st.CURSEQ lt st.MAXSEQ then begin & st.CURSEQ=(st.CURSEQ+nj-1)<st.MAXSEQ
                                       magnet_mov,'all',st & endif
          end
  'fforw':begin & magnet_sav,'all',st
            if st.CURSEQ lt st.MAXSEQ then begin & st.CURSEQ=(st.CURSEQ+nk*2)<st.MAXSEQ
                                       magnet_mov,'all',st & endif
          end
  'eforw':begin & magnet_sav,'all',st
            if st.CURSEQ lt (st.MAXSEQ-nk+2)>1 then begin & st.CURSEQ=(st.MAXSEQ-nj+2)>1
                                       magnet_mov,'all',st & endif
          end

  'enter':
  'clrsy':magnet_clear,'clrsy',st
  'clrcx':magnet_clear,'clrcx',st
  'clral':magnet_clall,        st
  'clrco':magnet_clear,'clrco',st
  'rstsi':magnet_clear,'rstsi',st

  'bldcx':begin & magnet_sav,'all',st & magnet_clear,'clrcx',st & magnet_conx ,st & magnet_mov,'all',st & end

  'momki':begin & widget_control,ev.id,get_value=s & s=strmid(s,9,2)
          if st.RorI eq 1 then txt='Imag vectors ->K'+s else txt='Real vectors ->K'+s
          magnet_sav,  'momag',st & st.CURMOM=fix(s) & magnet_mov,'momag',st & widget_control,st.BUTKI[0],set_value=txt
          end
  'r-2-i':begin & s=strtrim(fix(st.CURMOM),2) & magnet_sav,  'momag',st & st.RorI=abs(st.RorI-1)
          if st.RorI eq 1 then txt='Imag vectors ->K'+s else txt='Real vectors ->K'+s & widget_control,st.BUTKI[0],set_value=txt
          if st.RorI eq 1 then txt='->Change from imaginary to real part' else txt='->Change from real to imaginary part' & widget_control,st.BUTKI[1],set_value=txt
          magnet_mov  ,'momag',st & end
  'clrmo':magnet_clear,'clrmo',st
  'clrph':magnet_clear,'clrph',st

  'w-2-l':begin & magnet_sav ,'all'  ,st & st.WorL=abs(st.WorL-1)
          if st.WorL eq 1 then txt='Length' else txt='Width' & widget_control,st.BUTKI[2],set_value=txt
          if st.WorL eq 1 then txt='->Change from moment Length to atom Width' else txt='->Change from atom Width to moment Length' & widget_control,st.BUTKI[3],set_value=txt
          magnet_mov  ,'all'  ,st & end

  'wd3/4':begin & magnet_sav ,'all'  ,st                          & widget_control, st.DATA  ,get_uvalue=atoms
                atoms[4+2*st.WorL,*]=atoms[4+2*st.WorL,*]*3./4. & widget_control, st.DATA  ,set_uvalue=atoms
                magnet_mov ,'all'  ,st      & end
  'wd4/3':begin & magnet_sav ,'all'  ,st                          & widget_control, st.DATA  ,get_uvalue=atoms
                atoms[4+2*st.WorL,*]=atoms[4+2*st.WorL,*]*4./3. & widget_control, st.DATA  ,set_uvalue=atoms
                magnet_mov ,'all'  ,st      & end

  'rstwd':if st.WorL eq 1 then magnet_clear,'rstlg',st else magnet_clear,'rstwd',st
  'clrcc':magnet_clear,'clrcc',st

  'modcc':begin & magnet_sav ,'all'  ,st & magnet_modct, st & end

  'fread':begin & widget_control,st.wininf[4] ,map=0 & widget_control,st.wininf[14],sensitive=0
                widget_control,st.wininf[17],map=1 & widget_control,st.FILE,get_value=bfil & bfil=strtrim(bfil[0],2)
                filt=['*.atm','*.fst','*.pdb','*.xml','*.hdf','*.d07','*.d23','*.g94','*_4.car','XDATCAR*','CONTCAR*','CHG*','*.cif','*.gout']
                if !version.release lt '5.4' then filt='*.xml'
                bfil=dialog_pickfile(dialog_parent=st.wininf[17],filter=filt,file=bfil,get_path=pth,/read)
                widget_control,st.wininf[17],map=0
                widget_control,st.wininf[4] ,map=1 & widget_control,st.wininf[14],sensitive=1

                if bfil ne '' then begin
                   widget_control,st.FILE,set_value=bfil
                   magnet_par,st,-1
                   UM_Progress ; Update Memory _ Progress Bar
                  endif
                end
  'dread':begin & widget_control,st.FILE,get_value=bfil & bfil=strtrim(bfil[0],2)
                if bfil ne '' then begin
                   widget_control,st.FILE,set_value=bfil
                   magnet_par,st,-1
                   UM_Progress ; Update Memory _ Progress Bar
                endif
                end
  'fwrit':begin & widget_control,st.wininf[4] ,map=0 & widget_control,st.wininf[14],sensitive=0
                widget_control,st.wininf[17],map=1 & widget_control,st.FILE,get_value=bfil & bfil=strtrim(bfil[0],2)
                filt='*.xml' & if !version.release ge '5.4' then filt=['*.xml','*.hdf','*.pdb']
                bfil=dialog_pickfile(dialog_parent=st.wininf[17],filter=filt,file=bfil,get_path=pth,/write)

                widget_control,st.wininf[17],map=0
                widget_control,st.wininf[4] ,map=1 & widget_control,st.wininf[14],sensitive=1
                if bfil ne '' then begin
                   widget_control,st.FILE,set_value=bfil
                   magnet_par,st,-2
                endif
                end

  'wrksp':begin & widget_control,ev.id,get_value=ws & widget_control,st.wininf[8],set_value=ws
          magnet_par,st,0
          UM_Progress ; Update Memory _ Progress Bar
          end
  'wrkin':begin & widget_control,ev.id,get_value=ws & widget_control,st.wininf[8],set_value=ws
          magnet_par,st,-4
          widget_control,st.wininf[16],get_uvalue=tmp
          if tmp eq 'input' then magnet_par,st,st.wininf[16]
          UM_Progress ; Update Memory _ Progress Bar
          end

  'below':begin & st.wininf[6]=abs(st.wininf[6]-1)
          if st.wininf[6] eq 1 then txt='Set plot below' else txt='Set plot beside'
          if st.wininf[7] gt 0 then widget_control,st.wininf[7],set_value=txt & END

  'xvrml':magnet_par,st,-3,'VRML'
  'xpost':magnet_par,st,-3,'PS'
  'xpngf':magnet_par,st,-3,'PNG'
  'xjpeg':magnet_par,st,-3,'JPEG'

  'panel':begin & magnet_movie ,ev.top ,mbas
                if mbas gt 0 then st.wininf[20]=mbas & end
  'plane':begin & magnet_plane ,ev.top ,mbas
                if mbas gt 0 then st.wininf[24]=mbas & end

  'ortho':magnet_par,0,-3,'ORTHOGONAL', PCENT=[0]
  
  'bk_th':begin & mapi=3-st.wininf[34] & st.wininf[34]=mapi & mapi=mapi-1 ;Toggle slider Background/Threshold
                widget_control,st.wininf[33],  map=  mapi
                widget_control,st.wininf[32],  map=1-mapi & end

  'bgrnd':magnet_par,0,-3,'BACKGROUND', PCENT=[ev.value,135,135,205]
  
  'cocth':begin & widget_control,ev.id, get_value=vpc                      ;Threshold slider from main window
                d_livGetLattice, WDRAW=wdraw ;???
                magnet_par,0,-3,'VOL_DENS', PCENT=[-1,vpc,0], WDRAW=wdraw
          end

  'resvw':begin & d_livGetLattice, WDRAW=wdraw ;???
                magnet_par,0,-3,'PLANE'   , PCENT=[6, 5 ,(1),0,0,lonarr(9) ,0,0, 13], WDRAW=wdraw ;View point 0=selected 1=reset 2=center
          end

  'do_it':begin ;in case we are in Help and controlled by FullProf
             widget_control,st.wininf[12],map=0 & widget_control,st.wininf[13],map=1 & widget_control,st.wininf[14] ,map=1
	           magnet_par,st, ev.id
             if st.MAXSEQ gt 0 then begin
	              widget_control,st.wininf[19],sensitive=1
	              if st.wininf[6] eq 0 then begin
	                 newsx=st.wininf[2]+st.wininf[30]-st.wininf[28]
                   newsy=st.wininf[3]+st.wininf[31]-st.wininf[29]
                   widget_control,st.wininf[0] , XSIZE=newsx, YSIZE=newsy
	                 widget_control,ev.top,scr_xsize=st.wininf[30],scr_ysize=st.wininf[31]
	                 if xregistered('Magnet_movie') gt 0 then widget_control,st.wininf[20],map=1
	                 if xregistered('Magnet_plane') gt 0 then widget_control,st.wininf[24],map=1, bad_id=ii
	                 widget_control,st.wininf[15],sensitive=1
	                 widget_control,st.wininf[18],sensitive=1 & widget_control,st.wininf[23],sensitive=1
	                 st.wininf[13] =st.wininf[5]
	              endif
	           endif
	           UM_Progress ; Update Memory _ Progress Bar
          end
  'input':begin ;in case we are in Help and controlled by FullProf
             widget_control,st.wininf[12],map=0 & widget_control,st.wininf[13],map=1 & widget_control,st.wininf[14] ,map=1
             widget_control,st.wininf[0] , XSIZE=st.wininf[2], YSIZE=st.wininf[3]
	           widget_control,ev.top       ,scr_xsize=st.wininf[28],scr_ysize=st.wininf[29]
             widget_control,st.wininf[5] ,map=0 & widget_control,st.wininf[4] ,map=1
             widget_control,st.wininf[10],map=0 & widget_control,st.wininf[9] ,map=1
             widget_control,ev.id,set_value='Plot cells',set_uvalue='do_it'
             st.wininf[13]=st.wininf[4]
             if st.wininf[7] gt 0 then widget_control,st.wininf[7] ,sensitive=1
             widget_control,st.wininf[18],sensitive=0 & widget_control,st.wininf[23],sensitive=0
             widget_control,st.wininf[19],sensitive=0
             widget_control,st.wininf[15],sensitive=0
	           if xregistered('Magnet_movie') gt 0 then widget_control,st.wininf[20],map=0 else st.wininf[27]=0
	           if xregistered('Magnet_plane') gt 0 then widget_control,st.wininf[24],map=0, bad_id=ii
             if st.wininf[8]  gt 0 then widget_control,st.wininf[8] ,sensitive=1
             if st.wininf[27] gt 0 then widget_control,st.wininf[20],bad_id=ii,/destroy    &  st.wininf[27]=0
	           magnet_par,0,-3,'GET_INFO', PCENT=pv_plan
	  end
  'helpr':begin & widget_control,st.wininf[13] ,map=0  & widget_control,st.wininf[12],map=1
                widget_control,st.wininf[14] ,map=0  & end
  'endhp':begin & widget_control,st.wininf[12] ,map=0  & widget_control,st.wininf[13],map=1
                widget_control,st.wininf[14] ,map=1  & end
  'slamp':begin & widget_control,ev.id   ,sensitive=0  & if st.wininf[25] eq 0 then lamp else $
                widget_control,st.wininf[25] ,map=1,iconify=0  & end
  'docmd':      ii=execute('don_do_cmd')
  'clost':begin & magnet_par,st,0 & widget_control,ev.top,/destroy & end
  else:
  endcase
  if uv ne 'clost' then widget_control,ev.top,set_uvalue=st

 endif else if n_elements(uv) ge 2 then begin
  case uv[0] of
       ;return on Atom symbols
  1:    if n_elements(pv_sup)+n_elements(pv_plan) le 2 then begin
              widget_control,st.ATOM[uv[1]],get_value=symb & symb=strtrim(symb[0],2)
              if symb le ' ' then begin &  magnet_sav ,'all'  ,st
                                         magnet_mov ,'delet',st, st.CURSEQ-1+uv[1] ;** remove a line **
              endif else begin & nam=d_livTypetoName(d_livSymbtoType(magnet_split(symb),1))
                if nam eq 'Atom' then nam=' '
                widget_control,st.NAM[ uv[1]],get_value=cmp & cmp=strtrim(cmp[0],2)
                if cmp eq nam then begin
                   magnet_sav ,'all'  ,st
                   if uv[1] eq nj-1 then begin & st.CURSEQ=(st.CURSEQ+1)<st.MAXSEQ
                                               magnet_mov ,'all'  ,st               ;** show next line **
                   endif else magnet_mov ,'inser',st, st.CURSEQ-1+uv[1]             ;** insert  a line **
                endif else widget_control,st.NAM[ uv[1]],set_value=strmid(nam+'          ',0,10)
              endelse
              widget_control,ev.top,set_uvalue=st
        endif
       ;event on Propagation parameter button
  2:    begin & if uv[1] lt 0 then begin & magnet_propa,'clear', st
              endif         else begin & widget_control,ev.id, get_value =newp
                                 magnet_propa,'change',st,newp & endelse
        end
       ;event on Translation parameter button
  3:    begin & widget_control,uv[2],get_uvalue=translet
              widget_control,uv[2],set_value =translet[uv[1]]
              if strtrim(translet[uv[1]],2) eq 'Mat' then magnet_MATsym,ev,uv ;'Mat'
        end
       ;event on arrow width limit
  4:    begin & widget_control,uv[2],get_value=labm & widget_control,ev.id,get_value=labl
              idx=strpos(labm,'(') & labm=strmid(labm,0,idx+1)+labl+')' & widget_control,uv[2],set_value=labm
	            st.awrat=uv[1]/10.   & widget_control,ev.top,set_uvalue=st
	      end
  else:
  endcase
 endif
endif
end

pro magnet_MATsym,ev,uv ,ok
;+
;** *************
;** makes little 3*3 input matrix gui
;-
@lamp.cbk
 widget_control,uv[2],tlb_get_offset=tlb_offs
 widget_control,uv[2],tlb_get_size  =tlb_size
 tlb_offs=tlb_offs+tlb_size/2
 colW=50 & rowH=30
 tmpbas= widget_base(  title='Trans Matrix',/column $
                      ,group_leader=ev.top,/modal,resource_name="lamp")
 matbas= widget_table( tmpbas,column_width=colW,row_height=rowH,font=ft_propor,Xsize=3,Ysize=3 $
                      ,format="(A)",alignment=1,/no_headers,/editable,uvalue=[0])
 butbas= widget_base(  tmpbas,/row)
 butOk = widget_button(butbas,value='Ok'          ,font=ft_b_normal,event_pro='magnet_Matsym_event')
 butLab= widget_label( butbas,value='            ',font=ft_propor) ;12 char
 butCan= widget_button(butbas,value='Cancel'      ,font=ft_b_normal,event_pro='magnet_Matsym_event')
 widget_control,uv[3] ,get_uvalue=MATsym ;use_table_select=[left, top, right, bottom]
 widget_control,matbas,set_value =MATsym ,use_table_select=[0,0,2,2]
 widget_control,tmpbas,tlb_set_Xoffset=tlb_offs[0],tlb_set_Yoffset=tlb_offs[1],/realize
 widget_control,tmpbas,set_uvalue=[uv,matbas,butLab]
 
 XMANAGER,'MATsym',tmpbas,event_handler='magnet_MATsym_event',/no_block,/just_reg
end

pro magnet_MATsym_event,ev
;+
;** *******************
;** event handler for input 3*3 matrix
;                                 uv  (0, 1 , 2              , 3             , 4     , 5     )
;-
widget_control,ev.top ,get_uvalue=uv ;[3, ii, btran->translet, lablat->MATsym, matbas, butLab]
widget_control,ev.id  ,get_value =cas
if n_elements(cas) ne 1 then return
ok=1
case cas of
'Ok'    :begin & widget_control,uv[4],get_value=MATsym ,use_table_select=[0,0,2,2]
               on_ioerror,misMAT & ok=0 & MATsym=fix(MATsym)
               if total(abs(MATsym)) ge 3 then ok=1 & misMAT:
               if ok then widget_control,uv[3],set_uvalue=MATsym ;Keep MATsym
         end
'Cancel':begin & widget_control,uv[2],get_uvalue=translet ;First entry is Primitive
               widget_control,uv[2],set_value =translet[0]
         end
else:
endcase
if ok then widget_control,ev.top,/destroy $
      else widget_control,uv[5],set_value =' Bad input! '
end

pro magnet_propa, flg, st, newp, newV=newV
;+
;** ************
;**
;** Handle the propagation fields
;-

widget_control,st.bprop, get_value =str, get_uvalue=Vprop
if flg eq 'save' then newp=str else if n_elements(newp) eq 0 then  newp='K:01'

case flg of
'clear': begin & for i=0,2 do widget_control,st.kv[i], set_value ='' & Vprop[*]=0.
         end
'init':  begin & n=n_elements(newV) & Vprop[0:n-1]=newV & Sprop=strtrim(string(Vprop[0:2]),2)
               idx=where(Vprop[0:2] eq 0.) & if idx[0] ge 0 then  Sprop[idx]=' '
               for i=0,2 do widget_control,st.kv[i], set_value=Sprop[i]
         end
 else:   begin & num=long(strmid(str ,2,2))-1 & Vprop[num*3:num*3+2]=magnet_fget(st.kv, 0.)
               num=long(strmid(newp,2,2))-1 & Sprop=strtrim(string(Vprop[num*3:num*3+2]),2)
               idx=where(Vprop[num*3:num*3+2] eq 0.) & if idx[0] ge 0 then  Sprop[idx]=' '
               for i=0,2 do widget_control,st.kv[i], set_value=Sprop[i]
         end
 endcase
widget_control,st.bprop, set_uvalue=Vprop, set_value=newp
end

pro magmod_event, ev
;+
;** ************
;**
;** Event handler for MAGNET_MODCT
;-

widget_control,ev.id ,get_uvalue=uv
widget_control,ev.top,get_uvalue=rows & w  =rows[0,*] & nat=n_elements(w)
col=magnet_fget( rows[1:3,*], 0)      & col=byte(reform(col,3,nat))
case uv of
'modify': d_liv_change_table, w, col ,/set
'reset':  begin & d_liv_change_table, /reset & widget_control,ev.top,/destroy & end
'cancel': widget_control,ev.top,/destroy
else:
endcase
end

pro magnet_modct, st
;+
;** ************
;**
;** GUI atom color table
;-
@lamp.cbk

if (xregistered('Magnet_modcc') eq 0) and (st.MAXSEQ gt 0) then begin & nat=st.MAXSEQ

    widget_control, st.SYMBOL,get_uvalue=tsymbo & tsymbo= tsymbo[  0:nat-1]
    for i=0,nat-1 do tsymbo[i]=magnet_split(tsymbo[i],conx)
    w = d_livSymbtoType(tsymbo,nat)
    idx=sort(w) & w=w[idx] & tsymbo=tsymbo[idx]
    idu=uniq(w) & w=w[idu] & tsymbo=strupcase(tsymbo[idu])
    nat=n_elements( w)     & rows  =lonarr(4,nat) & col=lonarr(3,nat)
    d_liv_change_table, w, col ,/get              & col=strtrim(string(col),2)

    bas  =widget_base(  title='Atom color table',/column,resource_name="lamp")
    for i=0,nat-1 do begin
        bos=widget_base(  bas,resource_name="mic" ,/row)
        rows[0,i]=w[i]
        bid      =widget_label(bos,value='R:'               ,font=ft_propor)
        rows[1,i]=widget_text( bos,value=col[0,i],xsize=4   ,font=ft_propor,/editable)
        bid      =widget_label(bos,value=' G:'              ,font=ft_propor)
        rows[2,i]=widget_text( bos,value=col[1,i],xsize=4   ,font=ft_propor,/editable)
        bid      =widget_label(bos,value=' B:'              ,font=ft_propor)
        rows[3,i]=widget_text( bos,value=col[2,i],xsize=4   ,font=ft_propor,/editable)
        bid      =widget_label(bos,value='  '+tsymbo[i]+'  ',font=ft_propor)
    endfor
    bos  =widget_base(  bas,resource_name="ben" ,/row)
   ;if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo',bos
    bid  =widget_button(bos,value='Cancel'         ,font=ft_b_normal,uvalue='cancel')
    bid  =widget_label( bos,value=' '              ,font=ft_propor)
    bid  =widget_button(bos,value='Set new values' ,font=ft_b_normal,uvalue='modify')
    bid  =widget_label( bos,value=' '              ,font=ft_propor)
    bid  =widget_button(bos,value='Reset'          ,font=ft_b_normal,uvalue='reset')

   widget_control,bas,set_uvalue=rows
   widget_control,bas,/realize ;& if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo'
   if lamp_b1 gt 0 then widget_control,bas,group_leader=lamp_b1
   XMANAGER,'Magnet_modcc',bas,event_handler='magmod_event',/just_reg,/no_block
endif
end

pro magnet_freq,bas0,flog,bas,dnu,evtop
;+
;** ***********
;**
;** Frequency selector GUI
;-
@lamp.cbk
common c_magnet_sup, pv_sup, pv_plan ,pv_mk

if xregistered('Magnet_freq') eq 1 then widget_control,bas,bad_id=ii,/destroy

widget_control, bas0     ,get_uvalue=st
widget_control, st.DATA  ,get_uvalue=atoms
mk   =pv_mk
szat =st.MAXSEQ
szpv =size(pv_sup) & if szpv[0] eq 1 then szpv[2]=1
lust =atoms[7+st.mk*3*2:7+st.mk*3*2+mk-1,0] ;Phase=Omega=Frequency
if n_elements(pv_sup) gt 1 then lust=[[lust],[pv_sup[4+mk*3*2:4+mk*3*2+mk-1,lindgen(szpv[2]/szat)*szat]]]
if flog eq 0 then         unit='THz'
if flog eq 1 then begin & unit='cm-1' & lust=lust*33.36 & endif
if flog eq 2 then begin & unit='meV'  & lust=lust*4.136 & endif
slust=strarr(n_elements(lust))
for i=0L,n_elements(lust)/mk-1 do begin & j=i*mk
                  slust[j  ]=string(i+1,format='(i5)')+'.1   -> '+string(lust[j  ])
  if mk gt 1 then slust[j+1]=string(i+1,format='(i5)')+'..2  -> '+string(lust[j+1])
  if mk gt 2 then slust[j+2]=string(i+1,format='(i5)')+'...3 -> '+string(lust[j+2])
endfor

bas  =widget_base(  title='Frequency list ('+unit+')',resource_name="lamp",uvalue=0,group_leader=bas0,/floating)
blst =widget_list(  bas,value=slust,font=ft_propor,xsize=32,ysize=8,uvalue=[10,bas0,evtop,dnu,mk])
widget_control,bas ,/realize
XMANAGER,'Magnet_freq',bas,event_handler='magmov_event',/just_reg,/no_block
end

pro magmov_event,ev
;+
;** ************
;**
;** Event handler for MAGNET_MOVIE
;-

catch,stat & if stat ne 0 then begin & print,!error_state.msg & return & endif

widget_control,ev.id,get_uvalue=uv & widget_control,ev.top,get_uvalue=cuv
if n_elements(uv) gt 0 then begin
   on_ioerror,misflt
   case uv[0] of
   1: begin & dxyz=magnet_fget(uv[2:4], 0.)                        ;Initial direction
	          magnet_par,0,-3,'DIRECTION', PCENT=dxyz
      end
   2: begin & dxyz=magnet_fget(uv[2:4], 0.)                        ;Increment
	          magnet_par,0,-3,'INCREMENT', PCENT=dxyz
            if uv[5] gt 0 then widget_control,ev.id,bad_id=ii,timer=cuv[0]/50.
      end
   3: begin & widget_control,uv[2],get_value=seq & seq=fix(seq[0]) ;jpeg
            widget_control,uv[3],get_value=siz & siz=siz[0]
	          idx=strpos(siz,'%') & if idx gt 0 then siz=strmid(siz,0,idx) & siz=fix(siz)
            magnet_par,0,-3,'JPEG', PCENT=[siz,seq]
	          seq=seq+1 & seq=strtrim(string(seq),2)
	          if strlen(seq) eq 1 then seq='00'+seq else if strlen(seq) eq 2 then seq='0'+seq
	          widget_control,uv[2],set_value=seq
      end
   4: begin & widget_control,ev.id,get_value=but                   ;Play
            if strlowcase(but) eq 'stop' then begin
	             widget_control,uv[2],get_uvalue=vv & vv[5]=-1            & widget_control,uv[2],set_uvalue=vv
	             widget_control,uv[2],timer=0
	             widget_control,ev.id,set_value='PLAY'
	             widget_control,uv[1],get_uvalue=st & st.wininf[27]= 0    & widget_control,uv[1],set_uvalue=st
	          endif else begin
	             widget_control,uv[2],get_uvalue=vv & vv[5]=cuv[0]        & widget_control,uv[2],set_uvalue=vv
	             widget_control,uv[2],timer=cuv[0]/100.
	             widget_control,ev.id,set_value='STOP'
	             widget_control,uv[1],get_uvalue=st & st.wininf[27]=uv[2] & widget_control,uv[1],set_uvalue=st
	          endelse
      end
   5:       magnet_par,0,-3,'ZOOM', PCENT=[ev.value]

   6: begin & if uv[2] eq 1 then begin & stat=ev.select & txt=uv[3] & uv[4]=stat    ;Cut button event
                                     widget_control,ev.id,set_uvalue=uv & endif
	          if uv[2] eq 2 then begin & widget_control,uv[3],get_uvalue=uv         ;Cut text event
                                     stat=uv[4]    & txt=ev.id          & endif
	          if stat  eq 0 then pcent=[100] $
	          else begin &  widget_control,txt,get_value=zc & pcent=[float(zc[0])] & endelse
            magnet_par,0,-3,'CUT', PCENT=pcent
      end
   7: begin & dxyz=magnet_fget(uv[2:4], 0.)                                       ;Direction
            dn  =magnet_fget(uv[7]  , 1 )
	          widget_control,uv[6],bad_id=ii,get_value=pt & pt=fix(  pt[0])       ;Step
	          if ii eq 0 then begin
             if uv[5] ne 0 then begin & inc =1
                if uv[5] eq -1 then begin & sign='+'
	                 widget_control,ev.id,get_value=sign,bad_id=ii & if sign[0] eq '-' then inc=-1 & endif
	              pt=(pt+inc)>1  & widget_control,uv[6],set_value=strtrim(string(pt),2) & endif
	           magnet_par,0,-3,'CELL_STEP', PCENT=[dxyz,pt,cuv[1],cuv[2],cuv[3],cuv[4],cuv[5],dn,cuv[7],cuv[8]]
             if uv[5] gt 0 then widget_control,uv[6],timer=cuv[0]/50.
	          endif
      end
   8: begin & dxyz=magnet_fget(uv[2:4], 0.)                        ;Direction
            dn  =magnet_fget(uv[7]  , 1 )
            widget_control,uv[6],set_value='1  '                 ;Step
	          magnet_par,0,-3,'CELL_STEP', PCENT=[dxyz,1 ,cuv[1],cuv[2],cuv[3],cuv[4],cuv[5],dn,cuv[7],cuv[8]]
      end
   9: begin & widget_control,uv[1],get_uvalue=st & ok=0
            case uv[3] of
            1: begin &                      ok=1 & cuv[0]=uv[4]    ;Speed
               end
            2: if st.mcoo eq 3 then begin & ok=1 & cuv[1]=uv[4]    ;Smoothing (Steps)
               endif
            3: if st.mcoo eq 3 then begin & ok=2 & cuv[2]=uv[4]    ;Amplitude
               endif
            4: if st.mcoo eq 3 then begin & ok=2 & cuv[3]=uv[4]    ;Displacement mode
               endif
            5: if st.mcoo eq 3 then begin & ok=2 & cuv[4]=uv[4]    ;Ball & Stick
               endif
            6: if st.mcoo eq 3 then begin & ok=2 & cuv[5]=uv[4]    ;Vectors
	                if uv[4] gt 1 then ok=-2
               endif
            7: if st.mcoo eq 3 then begin & ok=1 & cuv[6]=uv[4]    ;Frequency selector GUI
                  bas=st.wininf[26]
                  widget_control,uv[2],get_uvalue=dnu
                  magnet_freq,uv[1],uv[4],bas,dnu,ev.top & st.wininf[26]=bas
                  widget_control,uv[1],set_uvalue=st & endif
            8: if st.mcoo eq 3 then begin & ok=2 & cuv[7]=uv[4]    ;Bonds
                                        cuv[8]=uv[5] & endif
            9: if st.mcoo eq 3 then begin & ok=0                   ;Ellipsoids
	                widget_control,/hourglass
	                magnet_par,0,-3,'CLOUDS', PCENT=[0,0,0,0,uv[4],cuv[2],cuv[3]]
	                UM_Progress ; Update Memory _ Progress Bar
               endif
            else:
            endcase
            if ok gt 0 then begin
              widget_control,uv[2],get_value=labm & widget_control,ev.id,get_value=labl
              idx=strpos(labm,'(') & labm=strmid(labm,0,idx+1)+labl+')' & widget_control,uv[2],set_value=labm
              widget_control,ev.top,set_uvalue=cuv
            endif
            if abs(ok) eq 2 then begin & widget_control,uv[2],get_uvalue=dnu & dn=magnet_fget(dnu,1)
	               magnet_par,0,-3,'CELL_STEP', PCENT=[0,0,0 ,1 ,cuv[1],cuv[2],cuv[3],cuv[4],cuv[5],dn,cuv[7],cuv[8]]
	    endif
      end
  10: begin & widget_control,uv[2],get_uvalue=cuv                  ;Frequence index from list
            mk=uv[4]
            dn=ev.index/mk+1 & mode=ev.index mod mk
            cuv[3]=mode
            widget_control,uv[3],set_value=strtrim(string(dn),2)
            widget_control,uv[2],set_uvalue=cuv
	          magnet_par,0,-3,'CELL_STEP', PCENT=[0,0,0 ,1 ,cuv[1],cuv[2],cuv[3],cuv[4],cuv[5],dn,cuv[7],cuv[8]]
      end ;[10,bas0,evtop,dnu,mk]

  11: begin & widget_control,uv[1],get_uvalue=st & magnet_plane ,uv[1] ,mbas
            if mbas gt 0 then begin & st.wininf[24]=mbas & widget_control,uv[1],set_uvalue=st & endif
      end
  15: begin & widget_control,ev.id,get_value=txt                   ;Movie button
            if strpos(txt,'close') lt 0 then begin
              widget_control,uv[2],sens=0,get_value=seq & seq=strcompress(seq[0])
              widget_control,uv[3],sens=0,get_value=siz & siz=siz[0]
              widget_control,uv[4],sens=0,get_value=rnk , get_uvalue =fmo
              widget_control,uv[5],sens=1 & widget_control,uv[6],sens=1
              widget_control,ev.id,set_value ='close'
              widget_control,uv[1],get_uvalue=st
	            widget_control,fmo  ,get_value =ext
              idx=strpos(siz,'%') & if idx gt 0 then siz=strmid(siz,0,idx) & siz=fix(siz)
	            idx=strpos(rnk,':')
	            if idx ge 0 then rnk=fix(strmid(rnk,idx+1,4)) else rnk=100
              did_mpeg,'open',wdraw=st.wininf[0],file='magdraw'+seq+ext,qual=rnk,dims=siz,frame=uv[5]
            endif else begin                                     ;Close movie button
              widget_control,uv[3],sens=1 & widget_control,uv[4],sens=1
              widget_control,uv[5],sens=0,set_value ='frm=000'
              widget_control,uv[6],sens=0,set_button=1,set_value='paused '
              widget_control,ev.id,set_value='movie'
              widget_control,uv[2],get_value=seq & seq=fix(seq[0])
	            seq=seq+1 & seq=strtrim(string(seq),2)
	            if strlen(seq) eq 1 then seq='0'+seq
	            widget_control,uv[2],set_value=seq,sens=1
              did_mpeg,'save'
            endelse
      end
  16: begin & widget_control,ev.id,get_value=txt                   ;Mpeg rank (quality)
            widget_control,uv[2],set_value='rank:'+txt
      end
  17: begin & if ev.select then begin & did_mpeg,'pause' & widget_control,ev.id,set_value='paused '
            endif        else begin & did_mpeg,'start' & widget_control,ev.id,set_value='started' & endelse
      end
  18: begin & widget_control,ev.id,get_value=txt                   ;Movie format: .mpeg,.gif,.mj2,...
            widget_control,uv[2],set_value=txt,get_uvalue=fmo
            widget_control,fmo  ,set_value=txt
      end
   else:
   endcase
   misflt:
endif
end

pro magnet_movie,bas0,bas
;+
;** ************
;**
;** Rotate , atoms displacement , snapshot ... GUI.
;-
@lamp.cbk

if xregistered('Magnet_movie') eq 0 then begin                                    ;speed steps ampl mod b&s dir freq rbld rvdw
  if bas0 gt 0 then $
       bas =widget_base(  title='Animation panel',/column,resource_name="lamp",uvalue=[5   ,8    ,10  ,0  ,1  ,0  ,1   ,0   ,10],group_leader=bas0,/floating) $
  else bas =widget_base(  title='Animation panel',/column,resource_name="lamp",uvalue=[5   ,8    ,10  ,0  ,1  ,0  ,1   ,0   ,10])
  ;if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo',bas
   ba1  =widget_base(  bas,frame=2,resource_name="mic" ,/row)
     set=widget_button(ba1,value='View->'              ,font=ft_b_normal)
     bid=widget_label( ba1,value='h'                   ,font=ft_b_normal)
     dx =widget_text(  ba1,value='+0.0',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba1,value='k'                   ,font=ft_b_normal)
     dy =widget_text(  ba1,value='+0.0',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba1,value='l'                   ,font=ft_b_normal)
     dz =widget_text(  ba1,value='+1.0',xsize=4,/editab,font=ft_propor)
    ;bid=widget_label (ba1,value='Direction'           ,font=ft_b_normal)
     bid=widget_label( ba1,value=' '                   ,font=ft_propor)
     opt=widget_button(ba1,value=' MENU ',/menu        ,font=ft_b_normal)
      op1=widget_button(opt,value='Speed (medium)'        ,/menu,font=ft_b_normal)
       o11=widget_button(op1,value='slower'              ,font=ft_b_normal,uvalue=[9,bas0,op1,1,10])
       o12=widget_button(op1,value='medium'              ,font=ft_b_normal,uvalue=[9,bas0,op1,1,5])
       o13=widget_button(op1,value='faster'              ,font=ft_b_normal,uvalue=[9,bas0,op1,1,1])
;     bid=widget_button(opt,value='-----'                ,font=ft_b_normal)
      bid=widget_button(opt,value='--DISPLACEMENTS--'    ,font=ft_b_normal)
      op2=widget_button(opt,value=' Steps (8)'            ,/menu,font=ft_b_normal)
       o21=widget_button(op2,value='1'                   ,font=ft_b_normal,uvalue=[9,bas0,op2,2,1])
       o21=widget_button(op2,value='2'                   ,font=ft_b_normal,uvalue=[9,bas0,op2,2,2])
       o21=widget_button(op2,value='4'                   ,font=ft_b_normal,uvalue=[9,bas0,op2,2,4])
       o21=widget_button(op2,value='8'                   ,font=ft_b_normal,uvalue=[9,bas0,op2,2,8])
       o21=widget_button(op2,value='16'                  ,font=ft_b_normal,uvalue=[9,bas0,op2,2,16])
       o21=widget_button(op2,value='32'                  ,font=ft_b_normal,uvalue=[9,bas0,op2,2,32])
       o21=widget_button(op2,value='64'                  ,font=ft_b_normal,uvalue=[9,bas0,op2,2,64])
       o21=widget_button(op2,value='128'                 ,font=ft_b_normal,uvalue=[9,bas0,op2,2,128])
      op7=widget_button(opt,value=' Frequency list ()'    ,/menu,font=ft_b_normal)
       o71=widget_button(op7,value='THz'                 ,font=ft_b_normal,uvalue=[9,bas0,op7,7,0])
       o72=widget_button(op7,value='cm-1'                ,font=ft_b_normal,uvalue=[9,bas0,op7,7,1])
       o73=widget_button(op7,value='meV'                 ,font=ft_b_normal,uvalue=[9,bas0,op7,7,2])
      op3=widget_button(opt,value=' Amplitude ([xyz]*1.0)',/menu,font=ft_b_normal)
       o31=widget_button(op3,value='[xyz]*0.1'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,1])
       o31=widget_button(op3,value='[xyz]*0.2'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,2])
       o31=widget_button(op3,value='[xyz]*0.3'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,3])
       o31=widget_button(op3,value='[xyz]*0.5'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,5])
       o31=widget_button(op3,value='[xyz]*1.0'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,10])
       o31=widget_button(op3,value='[xyz]*1.5'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,15])
       o31=widget_button(op3,value='[xyz]*2.0'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,20])
       o31=widget_button(op3,value='[xyz]*3.0'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,30])
       o31=widget_button(op3,value='[xyz]*6.0'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,60])
       o31=widget_button(op3,value='[xyz]*10.'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,100])
       o31=widget_button(op3,value='[xyz]*20.'           ,font=ft_b_normal,uvalue=[9,bas0,op3,3,200])
      op6=widget_button(opt,value=' Directions (drawn )'  ,/menu,font=ft_b_normal)
       o61=widget_button(op6,value='drawn'               ,font=ft_b_normal,uvalue=[9,bas0,op6,6,0])
       o62=widget_button(op6,value='hidden'              ,font=ft_b_normal,uvalue=[9,bas0,op6,6,1])
       o63=widget_button(op6,value='------'              ,font=ft_b_normal)
       o64=widget_button(op6,value='Hide/Show lines'     ,font=ft_b_normal,uvalue=[9,bas0,op6,6,2])
       o65=widget_button(op6,value='Colors'               ,/menu,font=ft_b_normal)
       bid=widget_button(o65,value='Yellow'              ,font=ft_b_normal,uvalue=[9,bas0,op6,6,4])
       bid=widget_button(o65,value='White'               ,font=ft_b_normal,uvalue=[9,bas0,op6,6,5])
       bid=widget_button(o65,value='Black'               ,font=ft_b_normal,uvalue=[9,bas0,op6,6,6])
       bid=widget_button(o65,value='Grey'                ,font=ft_b_normal,uvalue=[9,bas0,op6,6,7])
       bid=widget_button(o65,value='Atom'                ,font=ft_b_normal,uvalue=[9,bas0,op6,6,8])
       bid=widget_button(o65,value='Red'                 ,font=ft_b_normal,uvalue=[9,bas0,op6,6,3])
;     op4=widget_button(opt,value=' Displacement mode (1)',/menu,font=ft_b_normal)
;      o41=widget_button(op4,value='1'                   ,font=ft_b_normal,uvalue=[9,bas0,op4,4,0])
;      o42=widget_button(op4,value='2'                   ,font=ft_b_normal,uvalue=[9,bas0,op4,4,1])
;      o43=widget_button(op4,value='3'                   ,font=ft_b_normal,uvalue=[9,bas0,op4,4,2])
      op5=widget_button(opt,value=' Atoms (Balls)      '  ,/menu,font=ft_b_normal)
       o51=widget_button(op5,value='Ball - Stick'        ,font=ft_b_normal,uvalue=[9,bas0,op5,5,0])
       o52=widget_button(op5,value='Balls'               ,font=ft_b_normal,uvalue=[9,bas0,op5,5,1])
       o53=widget_button(op5,value='Sticks'              ,font=ft_b_normal,uvalue=[9,bas0,op5,5,2])
      op8=widget_button(opt,value=' Bonds (Original)  '   ,/menu,font=ft_b_normal)
       o81=widget_button(op8,value='Original'            ,font=ft_b_normal,uvalue=[9,bas0,op8,8,0,10])
       o82=widget_button(op8,value='Live'                ,/menu,font=ft_b_normal)
       o821=widget_button(o82,value='Live rvdW'          ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,10])
       o822=widget_button(o82,value='Live r*1.2'         ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,12])
       o822=widget_button(o82,value='Live r*1.4'         ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,14])
       o822=widget_button(o82,value='Live r*1.6'         ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,16])
       o822=widget_button(o82,value='Live r*1.8'         ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,18])
       o822=widget_button(o82,value='Live r*2.0'         ,font=ft_b_normal,uvalue=[9,bas0,op8,8,1,20])
      op9=widget_button(opt,value=' Ellipsoids'           ,/menu,font=ft_b_normal)
       o91=widget_button(op9,value='Build'               ,font=ft_b_normal,uvalue=[9,bas0,op9,9,2])
       o91=widget_button(op9,value='Hide'                ,font=ft_b_normal,uvalue=[9,bas0,op9,9,0])
       o92=widget_button(op9,value='Show'                ,font=ft_b_normal,uvalue=[9,bas0,op9,9,1])
      bid =widget_button(opt,value='--PROPAGATION VECTORS--',font=ft_b_normal)
       bhd=widget_button(opt,value=' Help'               ,/menu,font=ft_b_normal)
       bid=widget_button(bhd,value='(If you have many cells use the right mouse button to select atoms in cell 1)' ,font=ft_b_normal)
       bid=widget_button(bhd,value='(To see movements of propagation vector use the PLAY button just on right)'    ,font=ft_b_normal)
       bid=widget_button(bhd,value='(To Change directions fill hkl indices just on left      )' ,font=ft_b_normal)
       bid=widget_button(bhd,value='(To smooth movements  fill fractional hkl indices        )' ,font=ft_b_normal)
       bid=widget_button(bhd,value='(The file modules.xml is created automaticaly            )' ,font=ft_b_normal)
;     bid=widget_button(opt,value='-----'                ,font=ft_b_normal)
      bid=widget_button(opt,value='--OTHER OBJECTS--'    ,font=ft_b_normal,uvalue=[11,bas0])
      bid=widget_button(opt,value='Planes - Polygons'    ,font=ft_b_normal,uvalue=[11,bas0])
      bid=widget_button(opt,value='Hide/Show Objects'    ,font=ft_b_normal,uvalue=[11,bas0])
      bid=widget_button(opt,value='View point - distance',font=ft_b_normal,uvalue=[11,bas0])
      bid=widget_button(opt,value='Projections - angles' ,font=ft_b_normal,uvalue=[11,bas0])

     go1=widget_button(ba1,value='1'                   ,font=ft_smaller)
     pre=widget_button(ba1,value='-'                   ,font=ft_smaller)
     stp=widget_text(  ba1,value='0001',xsize=4,/editab,font=ft_b_normal)
     nxt=widget_button(ba1,value='+'                   ,font=ft_smaller)
     flm=widget_button(ba1,value='PLAY'                ,font=ft_b_normal)
     widget_control,set,set_uvalue=[1,bas0,dx,dy,dz]
     widget_control,flm,set_uvalue=[4,bas0,stp]
    ;clp=widget_button(widget_base(ba1,/nonexclusive),value='clip',font=ft_b_normal)
    ;zz =widget_text  (ba1,value='0.50',xsize=4,/editab,font=ft_propor)
    ;widget_control,clp,set_uvalue=[6,bas0,1,zz,0]
    ;widget_control,zz ,set_uvalue=[6,bas0,2,clp]
   ba2  =widget_base(  bas,frame=2,resource_name="don" ,/row)
     rut=widget_button(ba2,value='Rota->'              ,font=ft_b_normal)
     bid=widget_label( ba2,value='H'                   ,font=ft_b_normal)
     ix =widget_text(  ba2,value='+.00',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba2,value='V'                   ,font=ft_b_normal)
     iy =widget_text(  ba2,value='+.00',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba2,value='z'                   ,font=ft_b_normal)
     iz =widget_text(  ba2,value='-.01',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba2,value='(2Pi*increment)'     ,font=ft_b_normal)
     bid=widget_label( ba2,value=' '                   ,font=ft_propor)
     flm=widget_button(ba2,value='PLAY'                ,font=ft_smaller)
     bid=widget_label( ba2,value='  d#'                ,font=ft_b_normal) & widget_control,bid,sensitive=0
     dnu=widget_text(  ba2,value='0001',xsize=4,/editab,font=ft_b_normal)
     	widget_control,stp,set_uvalue=[7,bas0,dx,dy,dz, 0,stp,dnu]
     	widget_control,pre,set_uvalue=[7,bas0,dx,dy,dz,-1,stp,dnu]
     	widget_control,nxt,set_uvalue=[7,bas0,dx,dy,dz,-1,stp,dnu]
     	widget_control,go1,set_uvalue=[8,bas0,dx,dy,dz,-1,stp,dnu]
     	widget_control,op1,set_uvalue=[dnu] & widget_control,op2,set_uvalue=[dnu]
     	widget_control,op3,set_uvalue=[dnu];& widget_control,op4,set_uvalue=[dnu]
     	widget_control,op5,set_uvalue=[dnu] & widget_control,op6,set_uvalue=[dnu]
     	widget_control,op7,set_uvalue=[dnu] & widget_control,op8,set_uvalue=[dnu]
     	widget_control,op9,set_uvalue=[dnu]
     widget_control,flm,set_uvalue=[4,bas0,rut]
     widget_control,rut,set_uvalue=[2,bas0,ix,iy,iz,0]
   ba3  =widget_base(  bas,frame=2,resource_name="ben" ,/row)
     wrt=widget_button(ba3,value='Jpeg->'              ,font=ft_b_normal)
     bid=widget_label( ba3,value='magdraw_'            ,font=ft_b_normal)
     seq=widget_text(  ba3,value='001',xsize=3,/editab ,font=ft_propor)
     bid=widget_label( ba3,value='.jpeg'               ,font=ft_b_normal)
     bid=widget_label( ba3,value=' size:'              ,font=ft_b_normal)
     siz=widget_text(  ba3,value=' 50%',xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba3,value='zoom:'               ,font=ft_b_normal)
     coc=widget_slider(ba3,value='100',xsize=120,minimum=40,maximum=300,title='')
     widget_control,coc,set_uvalue=[5,bas0,300]
     widget_control,wrt,set_uvalue=[3,bas0,seq,siz]
   ba4  =widget_base(  bas,frame=2,resource_name="did" ,/row)
     did_mpeg_reset, 0, table_ext = mforms
     def_form = mforms[0] & mforms = mforms[1:*]
     wrt=widget_button(ba4,value='movie'               ,font=ft_b_normal)
     bid=widget_label( ba4,value='magdraw_'            ,font=ft_b_normal)
     seq=widget_text(  ba4,value='01',xsize=2,/editab  ,font=ft_propor)
     fmo=widget_label( ba4,value=def_form              ,font=ft_b_normal)
     bid=widget_label( ba4,value=' size:'              ,font=ft_b_normal)
     siz=widget_text(  ba4,value='100%',xsize=4,/editab,font=ft_propor)
     if !version.release lt 6.2 then begin
        rnk=widget_button(ba4,value='rank:100',/menu      ,font=ft_b_normal,uvalue=fmo)
        for i=10,100,10 do $
        bid=widget_button(rnk,value=strtrim(string(i),2)  ,font=ft_b_normal,uvalue=[16,bas0,rnk])
     endif else begin
        rnk=widget_button(ba4,value=def_form ,/menu       ,font=ft_b_normal,uvalue=fmo)
        for i=0,n_elements(mforms)-1 do begin
          if mforms[i] gt ' ' then $
            bid=widget_button(rnk,value=mforms[i]         ,font=ft_b_normal,uvalue=[18,bas0,rnk])
        endfor
     endelse
     frm=widget_label( ba4,value='frm=000'                ,font=ft_b_normal)
     pau=widget_button(widget_base(ba4,/nonexclusive),value='Paused ',font=ft_b_normal)
     widget_control,wrt,set_uvalue=[15,bas0,seq,siz,rnk,frm,pau]
     widget_control,pau,sens=0,set_uvalue=[17,bas0],set_button=1
     widget_control,frm,sens=0
     widget_control,bas,/realize ;& if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo'
     XMANAGER,'Magnet_movie',bas,event_handler='magmov_event',/just_reg,/no_block
endif else bas=0
end

pro magpla_event,ev
;+
;** ************
;**
;** Event handler for MAGNET_PLANE
;-

widget_control,ev.id,get_uvalue=uv & widget_control,ev.top,get_uvalue=cuv
if n_elements(uv) gt 0 then begin
   on_ioerror,misflt
   app1=2+0       +cuv[1]-1
   app2=2+cuv[0]  +cuv[1]-1
   hid =2+cuv[0]*2+cuv[1]-1
   mov =2+cuv[0]*3+cuv[1]-1
   alo =2+cuv[0]*4+cuv[1]-1

   d_livGetLattice, get=lot, wdraw=wdraw & lat=lot[0+cuv[alo]*2:1+cuv[alo]*2]
   if wdraw eq 0 then begin & widget_control,ev.top,/destroy & return & endif

   case uv[0] of
   0: begin & widget_control,ev.id,get_value =labl & widget_control,uv[1],set_value=labl ; PLANE NUMBER
            widget_control,uv[1],get_uvalue=nide
            widget_control,uv[21],sens=0 ;bod <a><b><c>
            widget_control,uv[22],sens=0 ;ba2 [c1][c2][c3]
            cuv[1]=uv[2]   ;set new current plane#
            app1  =2+0       +cuv[1]-1
            app2  =2+cuv[0]  +cuv[1]-1
            hid   =2+cuv[0]*2+cuv[1]-1 & cuv[hid]=0
            mov   =2+cuv[0]*3+cuv[1]-1
            alo   =2+cuv[0]*4+cuv[1]-1
	    for i=0,8 do widget_control,uv[3+i],set_value=string(uv[12+i]/1000.,format='(f6.3)')
	    idx1=4 & idx2=4+nide[2] & idx3=4+nide[2]+nide[3] & idx4=idx3+1 & idx5=idx3+2 & idx6=idx3+3
	    widget_control,nide[idx1+cuv[app1]-1],get_value =lab & widget_control,nide[0],set_value=lab
	    widget_control,nide[idx2+cuv[app2]-1],get_value =lab & widget_control,nide[1],set_value='Select atoms'
	    widget_control,nide[idx3]            ,set_button=cuv[hid]
	    widget_control,nide[idx4]            ,set_value =round( cuv[mov]/10.)>(-100)<100        ;slider
	    widget_control,nide[idx5]            ,set_value =string(cuv[mov]/1000.*(lat[1]-lat[0])+lat[0],format='(f6.3)') ;value Angstrom
	    for i=0,2 do widget_control,nide[idx6+i],set_button=0
	    widget_control,nide[idx6+cuv[alo]]      ,set_button=1
      cxyz=magnet_fget(uv[3:11], 0.)
	    magnet_par,0,-3,'PLANE', PCENT=[3,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],0], WDRAW=wdraw
      end
   1: begin & widget_control,ev.id,get_value=labl & widget_control,uv[1],set_value=labl ; PLANE APPEARANCE1
            cxyz=magnet_fget(uv[3:11], 0.)
	    cuv[app1]=uv[2]
	    magnet_par,0,-3,'PLANE', PCENT=[1,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],0], WDRAW=wdraw
      end
   2: begin & widget_control,ev.id,get_value=labl & widget_control,uv[1],set_value=labl ; PLANE APPEARANCE2
            cxyz=magnet_fget(uv[3:11], 0.)
	    cuv[app2]=uv[2]
	    magnet_par,0,-3,'PLANE', PCENT=[2,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],0], WDRAW=wdraw
	    if (uv[2] eq 6) or (uv[2] eq 7) then $
	    magnet_par,0,-3,'PLANE', PCENT=[4,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],0], WDRAW=wdraw
      end
   3: begin & cxyz=magnet_fget(uv[3:11], 0.)                                            ; HIDE/SHOW
	    cuv[hid] =ev.select
	    magnet_par,0,-3,'PLANE', PCENT=[3,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],0], WDRAW=wdraw
      end
   4: begin & if uv[2] eq 8 then begin                                                  ; GET c1 c2 c3 from SELECTED ATOMS
               pcent=[6,cuv[1],cuv[hid],cuv[app1],cuv[app2],lonarr(9),cuv[mov],cuv[alo],uv[2]]
	             magnet_par,0,-3,'PLANE', PCENT=pcent, WDRAW=wdraw
	             if total(pcent[5:13]) ne 0 then $
	                for i=0,8 do widget_control,uv[3+i],set_value=string(pcent[5+i]/1000.,format='(f6.3)')
	             uv[2]=0
	             widget_control,uv[1], set_value='Plane c123'
	             widget_control,uv[18],sens=1       ;bod <a><b><c>
               widget_control,uv[19],sens=1,map=1 ;ba2 [c1][c2][c3]
               widget_control,uv[20],       map=0 ;
	          endif
      cxyz=magnet_fget(uv[3:11], 0.)                                            ; BUILD or MOVE or FIX
	    slid =uv[12+cuv[0]]
	    slvd =uv[12+cuv[0]+1]
	    if (slid ne ev.id) and (uv[2] eq 0) then begin    ;Reset
	      widget_control,slid ,set_value = 0 & cuv[mov]=0 ;build
	      widget_control,slvd ,set_value =string(lat[0],format='(f6.3)') ;Angstrom
	      magnet_par,0,-3,'PLANE', PCENT=[5,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw
	      magnet_par,0,-3,'PLANE', PCENT=[4,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw
	    endif else begin
	      if uv[2] eq 0 then begin ;Slider
	         widget_control,slid ,get_value =slv  & cuv[mov]=slv*10
		       widget_control,slvd ,set_value =string(slv/100.*(lat[1]-lat[0])+lat[0],format='(f6.3)') ;Angstrom
	      endif else begin         ;Input text
	         on_ioerror,mislv
	         widget_control,slvd ,get_value =slv & slv=(slv[0]-lat[0])/(lat[1]-lat[0]) & cuv[mov]=round(slv*1000) ;Angstrom
	         widget_control,slid ,set_value =round( slv*100)>(-100)<100 & mislv: ;Angstrom
	      endelse
	      magnet_par,0,-3,'PLANE', PCENT=[4,cuv[1],cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw
	    endelse
	    pliuv=uv[12+cuv[1]-1]
	    widget_control,pliuv,get_uvalue= pluv  & pluv[12:20]=round(cxyz*1000)
	    widget_control,pliuv,set_uvalue= pluv
      end
   5: begin & widget_control,ev.id,get_uvalue= uv                                       ; MOVE ALONG
            widget_control,uv[2],set_value = 0 & cuv[mov]=0 & cuv[alo]=uv[1]
            widget_control,uv[3],set_value =string(lot[0+cuv[alo]*2],format='(f6.3)') ; Angstrom
      end
   6: case uv[2] of                                                                   ; POLYGONS
         1: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Create poly from selected atoms
         2: begin & cxyz=magnet_fget(uv[3:11], 0.)
	          magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],round(cxyz*1000),cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Create poly from c1,c2,c3
            end
         3: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show last poly
         4: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show all poly
         5: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Remove last poly
         6: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Remove all poly

         7: magnet_par,0,-3,'PLANE', PCENT=[6, 4 , uv[3]  ,cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide (not)selected atoms
         8: magnet_par,0,-3,'PLANE', PCENT=[2, 4 ,cuv[hid],cuv[app1],   1     ,lonarr(9)       ,cuv[mov],cuv[alo], 0],    WDRAW=wdraw ;Show all atoms
         9: magnet_par,0,-3,'PLANE', PCENT=[6, 4 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],-1],    WDRAW=wdraw ;Remove poly from selected atoms
         10:magnet_par,0,-3,'PLANE', PCENT=[6, 5 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show boxes
         11:magnet_par,0,-3,'PLANE', PCENT=[6, 5 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show bonds
         12:magnet_par,0,-3,'PLANE', PCENT=[6, 5 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show color-bar
         13:magnet_par,0,-3,'PLANE', PCENT=[6, 5 , uv[3]  ,cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;View point 0=selected 1=reset 2=center
         14:magnet_par,0,-3,'PLANE', PCENT=[2, 4 ,cuv[hid],cuv[app1],   4     ,lonarr(9)       ,cuv[mov],cuv[alo], 0],    WDRAW=wdraw ;Hide all atoms

         15:begin & magnet_par,0,-3,'VOL_DENS', PCENT=[uv[3:5]], WDRAW=wdraw ;Hide/Show VOLUMES
	                if uv[4] gt 0 then begin
	                  widget_control,ev.id,get_value=txtpc
	                  widget_control,uv[6],get_value=txtmn
	                  txtmn=strmid(txtmn,0,strpos(txtmn,' ->')+1)+txtpc
                    widget_control,uv[6],set_value=txtmn   ;Update the menu text
                    widget_control,uv[7],set_value=uv[4]*2 ;Update the slider
                    widget_control,uv[8],map=0
                    widget_control,uv[9],map=1             ;Map the threshold slider
	                endif
	                UM_Progress ; Update Memory _ Progress Bar
	          end
         16:magnet_par,0,-3,'PLANE', PCENT=[6, 5 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show axis
         17:magnet_par,0,-3,'PLANE', PCENT=[6, 5 ,cuv[hid],cuv[app1],cuv[app2],lonarr(9)       ,cuv[mov],cuv[alo],uv[2]], WDRAW=wdraw ;Hide/Show title

         18:begin & widget_control,ev.id, get_value=vpc                      ;Hide/Show VOLUMES
                  magnet_par,0,-3,'VOL_DENS', PCENT=[uv[3],vpc/2.,uv[5]], WDRAW=wdraw
            end
         else:
         endcase
    7: if xregistered('Magnet_win') ne 0 then magnet_win,/ANIDR
    
    8: begin & widget_control,uv[1],map=0 & widget_control,uv[2],map=1 & end ;Map the threshold slider
    
   else:
   endcase
   widget_control,ev.top,set_uvalue=cuv
   misflt:
endif
end

pro magnet_plane, bas0,bas
;+
;** ************
;**
;** Add/remove planes , objects and change object attributes GUI
;-

@lamp.cbk
if xregistered('Magnet_plane') eq 0 then begin                  ;uvalue=#Planes, curP, app1,  app2,  hid    move  along
   if bas0 gt 0 then $
        bas =widget_base(  title='Planes panel',/column,resource_name="lamp",uvalue=[3,3, 3,3,3, 1,1,1, 1,1,0, 0,0,0 ,0,1,2],group_leader=bas0,/floating) $
   else bas =widget_base(  title='Planes panel',/column,resource_name="lamp",uvalue=[3,3, 3,3,3, 1,1,1, 1,1,0, 0,0,0 ,0,1,2])
  ;if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo',bas
   ini1 =[0,0,0, 0,1000,0, 0,1000,1000] & ini2=[0,0,0, 1000,0,0, 1000,0,1000] & ini3=[0,0,0, 0,1000,0, 1000,1000,0]
   dc1  =lonarr(3) & dc2=dc1 & dc3=dc2

   ba1  =widget_base(  bas,frame=2,resource_name="mic" ,/row) ;First buttons and menus
   ba2  =widget_base(  bas,frame=2,resource_name="don")       ;Junck
   ba21 =widget_base(  ba2,/row,map=0)                        ;C1 C2 C3 planes
   ba22 =widget_base(  ba2,/row,map=1)                        ;Thresholds for volume density
   ba3  =widget_base(  bas,frame=2,resource_name="ben" ,/row) ;Moving planes

    ;C1 C2 C3
     bid=widget_label( ba21,value='c1'                     ,font=ft_b_normal)
     for i=0,2 do dc1[i]=widget_text(  ba21,value=string(ini3[0+i]/1000.,format='(f5.3)'),xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba21,value='c2'                     ,font=ft_b_normal)
     for i=0,2 do dc2[i]=widget_text(  ba21,value=string(ini3[3+i]/1000.,format='(f5.3)'),xsize=4,/editab,font=ft_propor)
     bid=widget_label( ba21,value='c3'                     ,font=ft_b_normal)
     for i=0,2 do dc3[i]=widget_text(  ba21,value=string(ini3[6+i]/1000.,format='(f5.3)'),xsize=4,/editab,font=ft_propor)

    ;THRESHOLDS
     bid =widget_label( ba22,value='Threshold*2 (%)'                  ,font=ft_b_normal)
     ths =widget_slider(ba22,value=66,minimum=1,maximum=200,xsize=200 ,font=ft_b_normal,title='')
     bid =widget_label( ba22,value=' '                                ,font=ft_b_normal)
     ths1=widget_button(ba22,value='Pts-Wire-Solid'                   ,font=ft_b_normal)
     ths2=widget_button(ba22,value='Hide/Show'                        ,font=ft_b_normal)
    
     bid=widget_label( ba3,value='Move plane along'                   ,font=ft_b_normal)
     bod=widget_base(  ba3,/exclusive,/row)
     sli=widget_slider(ba3,value=0,minimum=-100,maximum=100,xsize=170 ,font=ft_b_normal,/drag)
     slv=widget_text(  ba3,value='  .  ',xsize=5,/editab,font=ft_propor)
     bxa=widget_button(bod,value='a',/no_release,uvalue=[5,0,sli,slv])
     bxb=widget_button(bod,value='b',/no_release,uvalue=[5,1,sli,slv])
     bxc=widget_button(bod,value='c',/no_release,uvalue=[5,2,sli,slv]) & widget_control,bxc,set_button=1
     fxi=widget_button(ba3,value='fixe it',font=ft_b_normal)

     pln  =widget_button(ba1,value='Plane->ab',/menu   ,font=ft_b_normal)
      pln1=widget_button(pln,value='Plane->bc',uvalue=[0,pln,1,dc1,dc2,dc3,ini1, bod,ba21,ba22])
      pln2=widget_button(pln,value='Plane->ac',uvalue=[0,pln,2,dc1,dc2,dc3,ini2, bod,ba21,ba22])
      pln3=widget_button(pln,value='Plane->ab',uvalue=[0,pln,3,dc1,dc2,dc3,ini3, bod,ba21,ba22])
      plcc=widget_button(pln,value='Plane c1 c2 c3 from 3 selected atoms')
      bid =widget_button(pln,value=' ')
      vold=widget_button(pln,value='Threshold value for volume densities',uvalue=[8,ba21,ba22]) ;Just to map the threshold slider

     uv   =[4,pln,0,dc1,dc2,dc3,pln1,pln2,pln3,sli,slv,0,bod,ba21,ba22]
     bld  =widget_button(ba1,value='Reset'              ,font=ft_b_normal,uvalue=uv)
     uv[2]=0 & widget_control,sli ,set_uvalue=uv
     uv[2]=1 & widget_control,fxi ,set_uvalue=uv
     uv[2]=2 & widget_control,slv ,set_uvalue=uv
     uv[2]=8 & widget_control,plcc,set_uvalue=uv
     
     widget_control,bod, sens=0
     widget_control,ba21,sens=0

     nex  =widget_button(ba1,value='Transp. blue  '    ,/menu,font=ft_b_normal)
      opb =widget_button(nex,value='Opaque  blue  '    ,font=ft_b_normal,uvalue=[1,nex,1,dc1,dc2,dc3])
      opw =widget_button(nex,value='Opaque  white '    ,font=ft_b_normal,uvalue=[1,nex,2,dc1,dc2,dc3])
      trb =widget_button(nex,value='Transp. blue  '    ,font=ft_b_normal,uvalue=[1,nex,3,dc1,dc2,dc3])
      trw =widget_button(nex,value='Transp. white '    ,font=ft_b_normal,uvalue=[1,nex,4,dc1,dc2,dc3])

     hid  =widget_button(widget_base(ba1,/nonexclusive),value='Hide',font=ft_b_normal,uvalue=[3,0,0,dc1,dc2,dc3]) & widget_control,hid,set_button=0

     nax  =widget_button(ba1,value='Atoms in cell '    ,/menu,font=ft_b_normal)
      trc =widget_button(nax,value='Atoms in cell '    ,font=ft_b_normal,uvalue=[2,nax,1,dc1,dc2,dc3])
      trs =widget_button(nax,value='Atoms in plane'    ,font=ft_b_normal,uvalue=[2,nax,2,dc1,dc2,dc3])
      trl =widget_button(nax,value='Atoms in bonds'    ,font=ft_b_normal,uvalue=[2,nax,5,dc1,dc2,dc3])
      trh =widget_button(nax,value='Atoms hidden  '    ,font=ft_b_normal,uvalue=[2,nax,4,dc1,dc2,dc3])
      bid =widget_button(nax,value='Hide/Show bonds'   ,font=ft_b_normal,uvalue=[6,0,11])
      bid =widget_button(nax,value=' '                 ,font=ft_b_normal)
      dib =widget_button(nax,value='Projections'       ,/menu,font=ft_b_normal)
      tre =widget_button(dib,value='Electronic charges',font=ft_b_normal,uvalue=[2,nax,6,dc1,dc2,dc3])
      trp =widget_button(dib,value='Magnetic   charges',font=ft_b_normal,uvalue=[2,nax,7,dc1,dc2,dc3])
      if xregistered('Magnet_win') ne 0 then begin
       bid=widget_button(nax,value='Animation panel'   ,font=ft_b_normal,uvalue=[7,0])
       bid=widget_button(nax,value='Snapshot panel'    ,font=ft_b_normal,uvalue=[7,0])
      endif
     pol  =widget_button(ba1,value='Other objects->'                       ,/menu,font=ft_b_normal)
      bid =widget_button(pol,value='(Select atoms with right mouse button)' ,font=ft_b_normal)
      bid =widget_button(pol,value='(symbol should be visible if selected)' ,font=ft_b_normal)
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      bid =widget_button(pol,value='->Create a polygon from selected atoms' ,font=ft_b_normal,uvalue=[6,0,1 ])
      bid =widget_button(pol,value='     (Give DISTANCE if 2 are selected)' ,font=ft_b_normal,uvalue=[6,0,1 ])
      bid =widget_button(pol,value='     (Give ANGLES   if 3 are selected)' ,font=ft_b_normal,uvalue=[6,0,1 ])
      bid =widget_button(pol,value='->Create a polygon from c1 c2 c3'       ,font=ft_b_normal,uvalue=[6,0,2,dc1,dc2,dc3])
      bid =widget_button(pol,value='->Hide/Show last created polygon'       ,font=ft_b_normal,uvalue=[6,0,3 ])
      bid =widget_button(pol,value='->Hide/Show  all polygons'              ,font=ft_b_normal,uvalue=[6,0,4 ])
      bid =widget_button(pol,value='->Remove polygon tied to selected atoms',font=ft_b_normal,uvalue=[6,0,9 ])
      bid =widget_button(pol,value='->Remove    last created polygon'       ,font=ft_b_normal,uvalue=[6,0,5 ])
      bid =widget_button(pol,value='->Remove     all polygons'              ,font=ft_b_normal,uvalue=[6,0,6 ])
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      b1d =widget_button(pol,value='->Hide/Show other objects'              ,font=ft_b_normal,/menu)
       bid=widget_button(b1d,value='->Hide/Show  axis'                      ,font=ft_b_normal,uvalue=[6,0,16])
       bid=widget_button(b1d,value='->Hide/Show  boxes'                     ,font=ft_b_normal,uvalue=[6,0,10])
       bid=widget_button(b1d,value='->Hide/Show  title'                     ,font=ft_b_normal,uvalue=[6,0,17])
       bid=widget_button(b1d,value='->Hide/Show  color-bar'                 ,font=ft_b_normal,uvalue=[6,0,12])
       bid=widget_button(b1d,value='->Hide/Show  bonds (sticks)'            ,font=ft_b_normal,uvalue=[6,0,11])
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      bid =widget_button(pol,value='->Hide     selected atoms type'         ,font=ft_b_normal,uvalue=[6,0,7 ,-1])
      bid =widget_button(pol,value='->Hide     selected atoms'              ,font=ft_b_normal,uvalue=[6,0,7 , 0])
      bid =widget_button(pol,value='->Hide not selected atoms'              ,font=ft_b_normal,uvalue=[6,0,7 , 1])
      bid =widget_button(pol,value='->Hide all atoms'                       ,font=ft_b_normal,uvalue=[6,0,14])
      bid =widget_button(pol,value='->Show all atoms'                       ,font=ft_b_normal,uvalue=[6,0,8 ])
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      bid =widget_button(pol,value='->View point: set eye on selected atom' ,font=ft_b_normal,uvalue=[6,0,13 ,0])
      bid =widget_button(pol,value='->View point: center the selected atom' ,font=ft_b_normal,uvalue=[6,0,13 ,2])
      bid =widget_button(pol,value='->View point: reset to default'         ,font=ft_b_normal,uvalue=[6,0,13 ,1])
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      dib =widget_button(pol,value='->Charges Projections'                 ,/menu,font=ft_b_normal)
       bid=widget_button(dib,value='Electronic charges'                     ,font=ft_b_normal,uvalue=[2,nax,6,dc1,dc2,dc3])
       bid=widget_button(dib,value='Magnetic   charges'                     ,font=ft_b_normal,uvalue=[2,nax,7,dc1,dc2,dc3])
      b2d =widget_button(pol,value='->Create densities threshold -> %'     ,font=ft_b_normal,/menu)
       bid=widget_button(b2d,value='-> 1 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1, 1,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='-> 2 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1, 2,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='-> 3 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1, 3,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='-> 4 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1, 4,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='-> 5 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1, 5,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->10 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,10,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->20 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,20,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->30 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,30,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->40 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,40,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->50 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,50,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->60 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,60,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->70 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,70,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->80 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,80,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->90 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,90,0,b2d,ths,ba21,ba22])
       bid=widget_button(b2d,value='->95 %'                                 ,font=ft_b_normal,uvalue=[6,0,15,-1,95,0,b2d,ths,ba21,ba22])
      bid =widget_button(pol,value='->Points->Wires->Solid densities'       ,font=ft_b_normal,uvalue=[6,0,15,-1,0 ,2,b2d,ths,ba21,ba22])
      bid =widget_button(pol,value='->Hide/Show densities'                  ,font=ft_b_normal,uvalue=[6,0,15,-1,0 ,1,b2d,ths,ba21,ba22])
      bid =widget_button(pol,value=' '                                      ,font=ft_b_normal)
      if xregistered('Magnet_win') ne 0 then $
       bid=widget_button(pol,value='->Animation - Snapshot panel'           ,font=ft_b_normal,uvalue=[7,0])

     widget_control,ths ,set_uvalue=[6,0,18,-1,95,0,b2d,ths] ;Threshold Slider
     widget_control,ths1,set_uvalue=[6,0,15,-1,0 ,2,b2d,ths] ;Points->Wires->Solid
     widget_control,ths2,set_uvalue=[6,0,15,-1,0 ,1,b2d,ths] ;Hide/Show
     widget_control,pln ,set_uvalue=[nex,nax,4,6  ,opb,opw,trb,trw   ,trc,trs,trl,trh,tre,trp   ,hid,sli,slv,bxa,bxb,bxc,0]

   widget_control,bas,/realize ;& if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo'
   XMANAGER,'Magnet_plane',bas,event_handler='magpla_event',/just_reg,/no_block
endif else bas=0
end

pro magnet_win, file, PLOT=plotit, W=work , timer=timi , none=none , anidr=anidr
;+
;** **********
;**
;** The GUI CONSTRUCTOR
;-
@lamp.cbk
common c_magnet_win, bases, c_colnam, c_colval, ee,nn,snap, donit
common c_fullprof  , filloc

if keyword_set(none) then return
Xman=0

if xregistered('Magnet_win') eq 0 then begin

if n_elements( lamp_b1) ne 1 then begin & lamp_b1=0 & fk=1
    lamp_font = sys_dep('FONTS')
    ft_propor    = lamp_font[0,fk]
    ft_biggest   = lamp_font[1,fk]
    ft_bigger    = lamp_font[2,fk]
    ft_b_bigger  = lamp_font[3,fk]
    ft_normal    = lamp_font[4,fk]
    ft_b_normal  = lamp_font[5,fk]
    ft_smaller   = lamp_font[6,fk]
    ft_smallest  = lamp_font[7,fk]
endif
liv_objet
donit=0
nj=6    ;Number of atoms lines in the gui.
mk=4    ;Number of Propagation vectors, minimum 3 (3 displacement modes)
        ;Translation label table
translab=['Primitive-> 0 0 0'   ,'C-face centred-> 0; 1/2 1/2 0'   ,'A-face centred-> 0; 0 1/2 1/2','B-face centred-> 0; 1/2 0 1/2' $
         ,'Body centred-> 0; 1/2 1/2 1/2','All-face centred-> 0; 1/2 1/2 0; 0 1/2 1/2; 1/2 0 1/2' ,'Rhombohedrally centred (obverse)-> 0; 2/3 1/3 1/3; 1/3 2/3 2/3'$
         ,'Rhombohedrally centred (reverse)-> 0; 1/3 2/3 1/3; 2/3 1/3 2/3','Hexagonally centred-> 0; 2/3 1/3 0; 1/3 2/3 0' $
         ,'','Transformation MATRIX applied on input file only (ie: Prim to Cryst cell)']
translet=[' P ',' C ',' A ',' B ',' I ',' F ',' Ro',' Rv',' H ',' P ','Mat']

c_colnam=['black' ,'grey' ,'white'   ,'red'   ,'green'  ,'blue'  ,'cyan' ,'magenta','yellow' ,'brown','purple','beige']
c_colval=['0 0 0' ,'5 5 5','10 10 10','10 0 0','0 10 0' ,'0 0 10','0 4 5','7 0 3'  ,'10 10 0','5 5 0','6 0 5' ,'8 7 1']

bas0 =widget_base(   title='Magnetic/Dynamic cell editor V1.7',/column,resource_name="lamp",/tlb_size_events)
junky=widget_base(   bas0)
baspp=widget_base(   junky,map=1) & basp0=widget_base(baspp,/column)
basdd=widget_base(   junky,map=0)
baspf=widget_base(   junky,map=0)
bashp=widget_base(   junky,map=0,/column)

wglob=widget_base(   basp0,frame=4,resource_name="mic",/column)
row1 =widget_base(   wglob,/row) & if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo',row1
labl =widget_label(  row1 ,value=' ... GLOBAL CELL PARAMETERS ...',font=ft_biggest)

;Propagation vectors
row0   =widget_base(  wglob,/row)
row1a  =widget_base(  row0 ,/column)
row1b  =widget_base(  row0 ,/column)
row2a  =widget_base(  row1a ,/row,/frame) & kv=lonarr(3)
lablp  =widget_label( row2a,value="Propagation vectors:",font=ft_b_normal)
bprop  =widget_button(row2a,value="K:01"     ,font=ft_b_normal,/menu,uvalue=[fltarr(3*mk)])
    kpr=widget_button(bprop,value="Clear all",font=ft_b_normal,uvalue=[2,-1,bprop])
for ii=0,mk-1 do begin & str=strtrim(string(ii+1),2) & if ii lt 9 then str='0'+ str
    bid=widget_button(bprop,value="K:"+str   ,font=ft_b_normal,uvalue=[2, ii,bprop]) & endfor
kv[0]  =widget_text(    row2a,xsize=5,ysize=1  ,font=ft_propor,/editable)
kv[1]  =widget_text(    row2a,xsize=5,ysize=1  ,font=ft_propor,/editable)
kv[2]  =widget_text(    row2a,xsize=5,ysize=1  ,font=ft_propor,/editable)
;Number of cells
labl =widget_label(row2a,value="    Number of cells along a:",font=ft_b_normal)
dupx =widget_text( row2a,value='1 ',xsize=2,ysize=1   ,font=ft_propor,/editable)
labl =widget_label(row2a,value="b:",font=ft_b_normal)
dupy =widget_text( row2a,value='1 ',xsize=2,ysize=1   ,font=ft_propor,/editable)
labl =widget_label(row2a,value="c:",font=ft_b_normal)
dupz =widget_text( row2a,value='1 ',xsize=2,ysize=1   ,font=ft_propor,/editable)

; -----

row  =widget_base( wglob,/row)
rowc1=widget_base( row  ,/column)
rowc2=widget_base( row  ,/column)
row31=widget_base( rowc1,/row,/frame)
row41=widget_base( rowc1,/row,/frame)
row51=widget_base( rowc1,/row,/frame)
row32=widget_base( rowc2,/row)
row42=widget_base( rowc2,/row)
row52=widget_base( rowc2,/row)
row62=widget_base( rowc2)

;Exclusive buttons & LATTICE PARAMETERS
;Cartesian,Spherical Moments or vibration
labl =widget_label(row31,value="Vectors input:",font=ft_b_normal)
bexc =widget_base( row31,/exclusive,/row)
      cart=widget_button(bexc,value='Cart. M',font=ft_b_normal,uvalue='carte',/no_release)
      sphe=widget_button(bexc,value='Sphe. M',font=ft_b_normal,uvalue='spher',/no_release)
      dyna=widget_button(bexc,value='Dynamic',font=ft_b_normal,uvalue='dynam',/no_release)
      widget_control,cart,set_button=1
      widget_control,sphe,sensitive =0

labl =widget_label(row32,value="     a=",font=ft_propor)
la   =widget_text( row32,value=' 10.0'  ,xsize=5,ysize=1,font=ft_propor,/editable)
labl =widget_label(row32,value="    b=" ,font=ft_propor)
lb   =widget_text( row32,value=' 10.0'  ,xsize=5,ysize=1,font=ft_propor,/editable)
labl =widget_label(row32,value="     c=",font=ft_propor)
lc   =widget_text( row32,value=' 10.0'  ,xsize=5,ysize=1,font=ft_propor,/editable)

;Fourier coef.
labl =widget_label(row41,value="Fourier coef. relative to cell,atom:",font=ft_b_normal)
bcoef=widget_base( row41,/exclusive,/row)
      coef1=widget_button(bcoef,value='Sk ',font=ft_b_normal,uvalue='fcell',/no_release)
      coef2=widget_button(bcoef,value='Tk ',font=ft_b_normal,uvalue='fatom',/no_release)
      widget_control,coef1,set_button=1

;Angles
labl =widget_label(row42,value=" alpha=",font=ft_propor)
alpha=widget_text( row42,value=' 90.0',xsize=5,ysize=1,font=ft_propor,/editable)
labl =widget_label(row42,value=" beta=",font=ft_propor)
beta =widget_text( row42,value=' 90.0',xsize=5,ysize=1,font=ft_propor,/editable)
labl =widget_label(row42,value=" gamma=",font=ft_propor)
gamma=widget_text( row42,value=' 90.0',xsize=5,ysize=1,font=ft_propor,/editable)

;Phase or Amplitude Modulations
labl =widget_label(row51,value="Modulation:",font=ft_b_normal)
bexc =widget_base( row51,/exclusive,/row)
      phas=widget_button(bexc,value='Phase',font=ft_b_normal    ,uvalue='phase',/no_release)
      ampl=widget_button(bexc,value='Amplitude',font=ft_b_normal,uvalue='ampli',/no_release)
      widget_control,phas,set_button=1
      widget_control,bexc,sensitive =0

;Title, Transations and Message field
labl  =widget_label( row52,value=' Title='       ,font=ft_propor)
btit  =widget_text(  row52,value=' ',xsize=21    ,ysize=1,font=ft_propor,/editable)
lablat=widget_label( row52,value=" Lattice=",font=ft_propor,uvalue=[[1,0,0],[0,1,0],[0,0,1]])
       btran  =widget_button(row52,value=translet[0],font=ft_b_normal,/menu,uvalue=translet)
         for ii=0,n_elements(translab)-1 do begin & str=strtrim(string(ii),2)
           bid=widget_button(btran,value=translab[ii],font=ft_b_normal,uvalue=[3, ii,btran,lablat]) & endfor

bmes  =widget_label( row62,value=' ',xsize=280 ,font=ft_b_normal)

watom=widget_base(  basp0,frame=4,resource_name="don",/column)
bid  =widget_base(  watom,/row)
labl =widget_label( bid  ,value=" ... ATOMS & MAGNETIC MOMENT / DISPLACEMENT INPUTS ...",font=ft_biggest)

;Atom symbol fields
bdef  =widget_base(  watom,/row)

bdef1 =widget_base(  bdef ,/column)
 bti1 =widget_base(  bdef1,/row) & bseq =lonarr(nj) & bnam =lonarr(nj)
 eback=widget_button(bti1 ,value=''   ,font=ft_smallest,uvalue='eback')
 backf=widget_button(bti1 ,value='<<' ,font=ft_smallest,uvalue='fback')
 back =widget_button(bti1 ,value='<'  ,font=ft_smallest,uvalue='lback')
 fwd  =widget_button(bti1 ,value='>'  ,font=ft_smallest,uvalue='lforw')
 fwdf =widget_button(bti1 ,value='>>' ,font=ft_smallest,uvalue='fforw')
 efwd =widget_button(bti1 ,value=''   ,font=ft_smallest,uvalue='eforw')
 for jj=0,nj-1 do begin & bid=widget_base(bdef1) & bid1=widget_base(bid,/row) & bid2=widget_base(bid,/row,map=0)
                 bseq[jj]  =widget_label( bid1 ,value=strtrim(string(jj+1),2)+'~~~',font=ft_propor)
                 bnam[jj]  =widget_label( bid1 ,value='~~~~~~~~~~',font=ft_propor,/align_left)
                 bod       =widget_text(  bid2 ,font=ft_propor,xsize=1)
 endfor
bdef2 =widget_base(  bdef ,/column)
 but  =widget_base(  bdef2,/row)
 butc =widget_button(but  ,value='Symbol',font=ft_b_normal,/menu) & batom=lonarr(nj)
 buth =widget_button(butc ,value='(Hit enter when symbol is blank to remove line)',font=ft_b_normal)
 buth =widget_button(butc ,value='(Hit enter when symbol exists to insert a line)',font=ft_b_normal)
 buth =widget_button(butc ,value=' '                        ,font=ft_b_normal)
 buth =widget_button(butc ,value='(Fe-2-3-7 iron is bound with atoms 2,3 and 7)',font=ft_b_normal)
 buth =widget_button(butc ,value='->Clear bonds'           ,font=ft_b_normal,uvalue='clrcx')
 buth =widget_button(butc ,value='->Build bonds'           ,font=ft_b_normal,uvalue='bldcx')
 buth =widget_button(butc ,value=' '                       ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Clear symbols'         ,font=ft_b_normal,uvalue='clrsy')
 buth =widget_button(butc ,value=' '                       ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Reset all'             ,font=ft_b_normal,uvalue='clral')
 for jj=0,nj-1 do begin & bid=widget_base(bdef2) & bid=widget_base(bid,/row)
                 batom[jj] =widget_text(  bid  ,value='   ' ,font=ft_propor,xsize=6,ysize=1,/editable,uvalue=[1,jj])
 endfor

;Atomic coordinates
bdef3=widget_base(   bdef ,/column)
 but  =widget_base(  bdef3,/row)
 butc =widget_button(but  ,value='Atomic coordinates',font=ft_b_normal,/menu)
 buth =widget_button(butc ,value='(Enter X Y Z values)'     ,font=ft_b_normal)
 buth =widget_button(butc ,value=' '                        ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Clear  coordinates'     ,font=ft_b_normal,uvalue='clrco')
 bcoX =lonarr(nj) & bcoY =lonarr(nj) & bcoZ =lonarr(nj)
 for jj=0,nj-1 do begin & bid=widget_base(bdef3) & bid=widget_base(bid,/row)
                 bcoX[jj]  =widget_text(  bid  ,value='    ',font=ft_propor,xsize=4,ysize=1,/editable)
                 bcoY[jj]  =widget_text(  bid  ,value='    ',font=ft_propor,xsize=4,ysize=1,/editable)
                 bcoZ[jj]  =widget_text(  bid  ,value='    ',font=ft_propor,xsize=4,ysize=1,/editable)
 endfor

;Atom sites
bdef4=widget_base(   bdef ,/column)
 but  =widget_base(  bdef4,/row)
 butc =widget_button(but  ,value='site',font=ft_b_normal,/menu) & bsite=lonarr(nj)
 buth =widget_button(butc ,value=' '   ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Reset sites',font=ft_b_normal,uvalue='rstsi')
 for jj=0,nj-1 do begin & bid=widget_base(bdef4) & bid=widget_base(bid,/row)
                 bsite[jj] =widget_text(  bid  ,value=' 1 ' ,font=ft_propor,xsize=3,ysize=1,/editable)
 endfor

;atom moments
bdef6 =widget_base(  bdef ,/column)
 bti6 =widget_base(  bdef6,/row)
 butki=widget_button(bti6 ,value='Real vectors ->K1'          ,font=ft_b_normal,/menu)
 buth =widget_button(butki,value='(mx my mz) or (x y z displacements) or' ,font=ft_b_normal)
 buth =widget_button(butki,value='(module phi theta [degree]) or',font=ft_b_normal)
 buth =widget_button(butki,value='(ellipsoid axis length)'       ,font=ft_b_normal)
 buth =widget_button(butki,value=' '                             ,font=ft_b_normal)
 butri=widget_button(butki,value='->Change from real to imaginary part' ,font=ft_b_normal,uvalue='r-2-i')
 buth =widget_button(butki,value='  (or ellipsoid axis versus angles)'  ,font=ft_b_normal,uvalue='r-2-i')
 buth =widget_button(butki,value=' '                             ,font=ft_b_normal)
 for ii=1,3  do buth= widget_button(butki,value='-->     k'+strtrim(string(ii),2)+'     (mode '+strtrim(string(ii),2)+')',font=ft_b_normal,uvalue='momki')
 for ii=4,mk do buth= widget_button(butki,value='-->     k'+strtrim(string(ii),2),font=ft_b_normal,uvalue='momki')
 buth =widget_button(butki,value=' '                          ,font=ft_b_normal)
 buth =widget_button(butki,value='->Clear selected vectors'   ,font=ft_b_normal,uvalue='clrmo')
 bmoX =lonarr(nj) & bmoY =lonarr(nj) & bmoZ =lonarr(nj)
 for jj=0,nj-1 do begin & bid=widget_base(bdef6) & bid=widget_base(bid,/row)
                 bmoX[jj]  =widget_text(  bid  ,value='    '   ,font=ft_propor,xsize=4,ysize=1,/editable)
                 bmoY[jj]  =widget_text(  bid  ,value='    '   ,font=ft_propor,xsize=4,ysize=1,/editable)
                 bmoZ[jj]  =widget_text(  bid  ,value='    '   ,font=ft_propor,xsize=4,ysize=1,/editable)
 endfor

;atom phase
bdef5 =widget_base(  bdef ,/column)
 bti5 =widget_base(  bdef5,/row)
 butpi=widget_button(bti5 ,value='Phase'          ,font=ft_b_normal,/menu) & bmph=lonarr(nj)
 buth =widget_button(butpi,value='(linked with selected vectors)'     ,font=ft_b_normal)
 buth =widget_button(butpi,value='(Gives in fraction of 2*Pi)'        ,font=ft_b_normal)
 buth =widget_button(butpi,value=' '                                  ,font=ft_b_normal)
 buth =widget_button(butpi,value='->Clear phases for selected vectors',font=ft_b_normal,uvalue='clrph')
 for jj=0,nj-1 do begin & bid=widget_base(bdef5) & bid=widget_base(bid,/row)
                 bmph[jj]  =widget_text(  bid  ,value='    '   ,font=ft_propor,xsize=4,ysize=1,/editable)
 endfor

;Atom width / moment length
bdef7=widget_base(   bdef ,/column)
 bti7 =widget_base(  bdef7,/row)
 butci=widget_button(bti7 ,value='Width'                   ,font=ft_b_normal,/menu)
 buth =widget_button(butci,value='(width/length factor linked with symbol+site)' ,font=ft_b_normal) & bwfct=lonarr(nj)
 buth =widget_button(butci,value='(=0 is invisible)'       ,font=ft_b_normal)
 buth =widget_button(butci,value='(#1 to change default)'  ,font=ft_b_normal)
 buth =widget_button(butci,value='(=1 use previous change)',font=ft_b_normal)
 buth =widget_button(butci,value=' '                       ,font=ft_b_normal)
 buthw=widget_button(butci,value='->Change from atom Width to moment Length'     ,font=ft_b_normal,uvalue='w-2-l')
 buth =widget_button(butci,value=' '                       ,font=ft_b_normal)
 buthr=widget_button(butci,value='->Maximum arrow/atom radius ratio (.5)'        ,font=ft_b_normal,/menu)
   thr=widget_button(buthr,value='.7'                      ,font=ft_b_normal,uvalue=[4,7,buthr])
   thr=widget_button(buthr,value='.6'                      ,font=ft_b_normal,uvalue=[4,6,buthr])
   thr=widget_button(buthr,value='.5'                      ,font=ft_b_normal,uvalue=[4,5,buthr])
   thr=widget_button(buthr,value='.4'                      ,font=ft_b_normal,uvalue=[4,4,buthr])
   thr=widget_button(buthr,value='.3'                      ,font=ft_b_normal,uvalue=[4,3,buthr])
   thr=widget_button(buthr,value='.2'                      ,font=ft_b_normal,uvalue=[4,2,buthr])
   thr=widget_button(buthr,value='.1'                      ,font=ft_b_normal,uvalue=[4,1,buthr])
 buth =widget_button(butci,value=' '                       ,font=ft_b_normal)
 buth =widget_button(butci,value='->f*3/4'                 ,font=ft_b_normal,uvalue='wd3/4')
 buth =widget_button(butci,value='->f*4/3'                 ,font=ft_b_normal,uvalue='wd4/3')
 buth =widget_button(butci,value='->Reset factors to 1.'   ,font=ft_b_normal,uvalue='rstwd')
 for jj=0,nj-1 do begin & bid=widget_base(bdef7) & bid=widget_base(bid,/row)
                 bwfct[jj] =widget_text(  bid  ,value=' 1.0',font=ft_propor,xsize=4,ysize=1,/editable)
 endfor

;Atom colors
bdef8=widget_base(   bdef ,/column)
 bti8 =widget_base(  bdef8,/row)
 butc =widget_button(bti8 ,value='Color'                   ,font=ft_b_normal,/menu)
 buth =widget_button(butc ,value='(use R,G,B values 0->10)',font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(5 5 5 is grey)'         ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='( -or- )'                ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(use by names:)'         ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(black grey white)'      ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(red green blue)'        ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(cyan magenta yellow)'   ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value='(brown purple beige)'    ,font=ft_b_normal) & bcol=lonarr(nj)
 buth =widget_button(butc ,value=' '                       ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Clear colors linked with symbol+site',font=ft_b_normal,uvalue='clrcc')
 buth =widget_button(butc ,value=' '                       ,font=ft_b_normal)
 buth =widget_button(butc ,value='->Modify atom color table',font=ft_b_normal,uvalue='modcc')
 for jj=0,nj-1 do begin & bid=widget_base(bdef8) & bid=widget_base(bid,/row)
                 bcol[jj]  =widget_text(  bid  ,value=' ',font=ft_propor,xsize=6,ysize=1,/editable)
 endfor
 
 ;Memory Progress-bar
 PUT_Progress, basp0

;Bottom fields and buttons
wdor  =widget_base(  bas0 ,resource_name="ben",/row)
junki =widget_base(  wdor)
wdod  =widget_base(  junki,map=1) & wdo   =widget_base(  wdod ,/row,/frame)
wdol  =widget_base(  junki,map=0) & wdom  =widget_base(  wdol ,/row)

 mes_co=widget_label(wdom ,value='~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',font=ft_propor,/align_left)

 labl =widget_label( wdo  ,value='File name=' ,font=ft_b_normal)
 bfil =widget_text(  wdo  ,value=' ',xsize=20 ,ysize=1,font=ft_propor,/editable,uvalue='dread')
 bread=widget_button(wdo  ,value='Browse'     ,font=ft_b_normal,uvalue='fread')
 bid  =widget_button(wdo  ,value='Write'      ,font=ft_b_normal,uvalue='fwrit')

labl  =widget_label( wdor ,value=' '          ,font=ft_propor)

wdo   =widget_base(  wdor ,/row,/frame)
 if lamp_b1 gt 0 then begin
  labl =widget_label( wdo  ,value='Workspace:' ,font=ft_b_normal)
  wks  =widget_button(wdo  ,value='W19'        ,font=ft_b_normal,/menu)
  for ii=1,20 do bid=widget_button(wks ,value='W'+strtrim(string(ii),2),font=ft_b_normal,uvalue='wrksp')
  sensb=1
 endif else begin & wks=0 & sensb=0 & bibi=0 &endelse

 doit  =widget_button(wdo  ,value='Plot cells'      ,font=ft_b_normal,uvalue='do_it')
 bido  =widget_button(wdo  ,value='Options'         ,font=ft_b_normal,/menu)
  if sensb then begin
   bibi=widget_button(bido ,value='Set plot beside' ,font=ft_b_normal,uvalue='below')
   zizi=widget_button(bido ,value='Import workspace',font=ft_b_normal,/menu)
   for ii=1,20 do bid=widget_button(zizi ,value='W'+strtrim(string(ii),2),font=ft_b_normal,uvalue='wrkin')
  endif
  bipi =widget_button(bido ,value='Export as...'    ,font=ft_b_normal,/menu) & widget_control,bipi,sensitive =0
   bid =widget_button(bipi ,value='VRML (.wrl)'     ,font=ft_b_normal,uvalue='xvrml') & widget_control,bid ,sensitive =0
   bid =widget_button(bipi ,value='PostScript (.ps)',font=ft_b_normal,uvalue='xpost')
   bid =widget_button(bipi ,value='PNG  (.png)'     ,font=ft_b_normal,uvalue='xpngf')
   bid =widget_button(bipi ,value='JPEG (.jpeg)'    ,font=ft_b_normal,uvalue='xjpeg')

  bid  =widget_button(bido ,value='')
  c_4dM=widget_button(bido ,value='Appearance'             ,font=ft_b_normal,/menu)
  bidf =widget_button(bido ,value='Mouse and View features',font=ft_b_normal,/menu)
   bid =widget_button(bidf ,value='Toggle Projection: Orthogonal / Perspective' ,font=ft_b_normal,uvalue='ortho')
   bid =widget_button(bidf ,value='Toggle Slider: Backround / Threshold'        ,font=ft_b_normal,uvalue='bk_th')
   bid =widget_button(bidf ,value='Reset to default view'                       ,font=ft_b_normal,uvalue='resvw')
   bid =widget_button(bidf ,value=''                       ,font=ft_propor)
   bid =widget_button(bidf ,value='(Mouse Actions:)'       ,font=ft_propor)
   bid =widget_button(bidf ,value='( -Rotating the view   -> click the Left  button and move)'            ,font=ft_propor)
   bid =widget_button(bidf ,value='( -Zooming  the view   -> click the Right button and move up / down)'  ,font=ft_propor)
   bid =widget_button(bidf ,value='( -Entering the view   -> Press the Shift key  while zooming)'         ,font=ft_propor)
   bid =widget_button(bidf ,value=' '                      ,font=ft_propor)
   bid =widget_button(bidf ,value='( -Selecting objects   -> click the Right button on an object)'        ,font=ft_propor)
   bid =widget_button(bidf ,value='( -Selecting e-LogBook -> click the Right button on an empty zone)'    ,font=ft_propor)
  
  bipr =widget_button(bido ,value='Animation - snapshot ->'      ,font=ft_b_normal,uvalue='panel') & widget_control,bipr,sensitive =0
  bpla =widget_button(bido ,value='Planes - polygons - other ->' ,font=ft_b_normal,uvalue='plane') & widget_control,bpla,sensitive =0
  bid  =widget_button(bido ,value=''                             ,font=ft_b_normal)
  bid  =widget_button(bido ,value='Do input commands ->'         ,font=ft_b_normal,uvalue='docmd')
  bid  =widget_button(bido ,value='Lamp ->'                      ,font=ft_b_normal,uvalue='slamp')
  widget_control,c_4dM,sensitive=0,set_uvalue=[bpla,0L]

 cocSLD=widget_base(  wdo)
 cocJK1=widget_base(cocSLD, map=1)  & cocV=1
 cocJK2=widget_base(cocSLD, map=0) ;& cocV=2
 coc   =widget_slider(cocJK1, value='1',/suppress_value,xsize=120,minimum=1,maximum=240,title='Background',/drag,uvalue='bgrnd')
 cocIso=widget_slider(cocJK2, value='33'               ,xsize=120,minimum=1,maximum=100,title=''                ,uvalue='cocth')
 widget_control,cocSLD,sensitive=0

wdo   =widget_base(  wdor ,/row)
;labl =widget_label (wdo  ,value='  '         ,font=ft_propor)
 bid  =widget_button(wdo  ,value='Help'       ,font=ft_b_normal,uvalue='helpr')
 bid  =widget_button(wdo  ,value='Done'       ,font=ft_b_normal,uvalue='clost')

;Add the help frame
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='FILE NAME'        ,font=ft_b_bigger)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='   The extension defines the file format'         ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' ".xml",".pdb" for  Ascii   in/out file (READ-WRITE)' ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' ".fst",".atm" from FullProf input file (READ)'         ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' ".d07","d23" from Phonon,".g94" from Gaussian (READ)',font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' "_4.car" from Cerius,".gout" from GULP        (READ)',font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' "XDATCAR*","CHGCAR*","CONTCAR*" from Vasp     (READ)',font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value=' ".hdf" for  NeXus    in/out file (READ-WRITE) '  ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='PLOT CELLS'    ,font=ft_b_bigger)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Use right mouse button vertically for zooming'       ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Use left mouse button for rotating'                  ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Click right mouse button on atom to select (symbol displayed)' ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='OPTIONS'      ,font=ft_b_bigger)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Set plot beside to create a plot in separate window' ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Animation: set a direction, rotate, see propagations/displacements, write jpeg' ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='ATOMS & MAGNETIC MOMENT/DISPLACEMENT EDITOR'                        ,font=ft_b_bigger)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Click on title items for informations'               ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Changing width-color apply for same subseq. symbol and site' ,font=ft_propor)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='MAGNETIC MOMENTS' ,font=ft_b_bigger)
tmd=widget_base(  bashp,/row) & whp=widget_label( tmd , value='  Reserve K01 for null moments'                        ,font=ft_propor)
tmd=widget_base(  bashp,/row)
tmd=widget_base(  bashp)      & whp=widget_button(tmd , value='--- RETURN from this HELP ----->' ,font=ft_b_bigger, uvalue='endhp')

;realize
widget_control,bas0,/realize & if lamp_b1 gt 0 then CALL_PROCEDURE,'put_logo'

if lamp_b1 gt 0 then begin & widget_control,lamp_b1,iconify=1
                             widget_control,bas0,iconify=0
endif else Xman=1

if !version.os eq 'darwin' then render=1 else render=0            ;bug MacOs/Xquarz

;Add the plot frame
gml=widget_info(  baspp ,/geometry) & stx=gml.scr_xsize & sty=gml.scr_ysize
wdd=widget_draw(  basdd , GRAPHICS_LEVEL=2,UVALUE='DRAW', RETAIN=0, RENDERE=render, $
                         XSIZE=stx, YSIZE=sty,/BUTTON_EVENTS,/EXPOSE_EVENTS)

widget_control,bas0, tlb_get_size=v2 & v2=long(v2)
if !version.os_family eq 'Windows' then v2[1]=v2[1]+30

;Put all identificators in a structure
wininf=lonarr(35)
wininf[0] =wdd & wininf[1]=c_4dM & wininf[2] =stx  & wininf[3]=sty & wininf[4]=baspp & wininf[5]=basdd & wininf[6]=0 & wininf[7]=bibi & wininf[8]=wks
wininf[9] =wdod  & wininf[10]=wdol & wininf[11]=mes_co & wininf[12]=bashp & wininf[13]=baspp & wininf[14]=wdor & wininf[15]=bipi & wininf[16]=doit
wininf[17]=baspf & wininf[18]=bipr & wininf[19]=cocSLD & wininf[21]=lablp & wininf[22]=row41 & wininf[23]=bpla & wininf[24]=0
wininf[25]=lamp_b1 & wininf[26]=0  & wininf[27]=0      & wininf[28]=v2[0] & wininf[29]=v2[1] & wininf[30]=v2[0] & wininf[31]=v2[1]
wininf[32]=cocJK1  & wininf[33]=cocJK2 & wininf[34]=cocV
st={KV:kv,DUPL:[dupx,dupy,dupz],LAT:[la,lb,lc],MCOO:1,ANGL:[alpha,beta,gamma],MODU:1,COEF:1,MESS:bmes,BUTE:[cart,sphe,dyna,phas,ampl,coef1,coef2],BTRAN:btran,  $
    SEQ:bseq,NAM:bnam,CURSEQ:1,MAXSEQ:0,ATOM:batom,COOR:[[bcox],[bcoy],[bcoz]],SITE:bsite,CURMOM:1,CMOM:[[bmox],[bmoy],[bmoz]],CPHA:bmph,RorI:0,WorL:0,awrat:.5,$
    BUTKI:[butki,butri,butci,buthw],WIDTH:bwfct,COL:bcol,FILE:bfil,CURW:19,DATA:watom,SYMBOL:bdef2,COLOR:bdef8,WININF:wininf,BTIT:btit,BPROP:bprop,MK:mk,BUILD:0,NOEXT:0}
na=200
nn=0
ee=0

;Put structure and tables in uvalues
atoms=fltarr(7+mk*3*2+mk,na) & atoms[3:4,*]=1. & atoms[6,*]=1.
widget_control,bas0 ,set_uvalue=st
widget_control,watom,set_uvalue=atoms
widget_control,bdef2,set_uvalue=strarr(na)
widget_control,bdef8,set_uvalue=strarr(na)
bases =[bas0,bfil,bread,doit,wdd,bipr,lablat]

if n_elements(timi) eq 1 then begin & widget_control,junki,set_uvalue=[-1,timi],timer=5.
                                    filloc=file  & endif

XMANAGER,'Magnet_win',bas0,event_handler='magnet_event',CLEANUP='M_DYING',/no_block,/just_reg
empty
widget_control,bas0,iconify=0,show=1
if lamp_b1 gt 0 then widget_control,bas0,group_leader=lamp_b1

UM_Progress ; Update Memory _ Progress Bar

endif ;xregistered

if n_elements(file) eq 1 then begin & widget_control,bases[3],get_uvalue=uval
                                    if uval eq 'input' then magnet_event, {WIDGET_BUTTON, ID:bases[3], TOP:bases[0], HANDLER:0L, SELECT:1}
                                    widget_control,bases[1],set_value =file
                                    widget_control,bases[0],get_uvalue=st
                                    magnet_par,st,-1 ;magnet_event, {WIDGET_BUTTON, ID:bases(2), TOP:bases(0), HANDLER:0L, SELECT:1}
                                    widget_control,bases[0],set_uvalue=st & endif
if n_elements(work) eq 1 then begin & widget_control,bases[3],get_uvalue=uval
                                    if uval eq 'input' then magnet_event, {WIDGET_BUTTON, ID:bases[3], TOP:bases[0], HANDLER:0L, SELECT:1}
                                    widget_control,bases[0],get_uvalue=st
                                    widget_control,st.wininf[8],set_value='W'+strtrim(string(work),2)
                                    magnet_par,st,-4
                                    widget_control,bases[0],set_uvalue=st & endif
if keyword_set(plotit)   then begin & widget_control,bases[3],get_uvalue=uval
                                    widget_control,bases[0],get_uvalue=st
				                            if st.wininf[6]    then magnet_event, {WIDGET_BUTTON, ID:st.wininf[7], TOP:bases[0], HANDLER:0L, SELECT:1}
                                    if uval eq 'do_it' then magnet_event, {WIDGET_BUTTON, ID:bases[3]    , TOP:bases[0], HANDLER:0L, SELECT:1}
                                    endif
if keyword_set(anidr)    then begin & widget_control,bases[3],get_uvalue=uval
                                    if uval eq 'input' then magnet_event, {WIDGET_BUTTON, ID:bases[5]    , TOP:bases[0], HANDLER:0L, SELECT:1}
                                    endif
if Xman then xmanager,/no_block
end
