FUNCTION make_volume, Win, thresh=thresh, W=wn ,hkl=hkl, projections=proj, datp=datp, squared=squared
;+
;******* ***********
;** User callable
;** from a 3D workspace, make a lamp object workspace to use in MAGDRAW
;** Call: w2 = make_volume(w1 , /proj)
;-

forward_function regular

argdp=(arg_present(datp))
argwi=(n_elements(wn) eq 1)
if argwi then TAKE_W, Win, W=wn
if argwi then TAKE_DATP,datp, W=wn else if (not argdp) then TAKE_DATP,datp
s=SIZE(Win)

if (s[0] eq 3) and (not keyword_set(hkl)) and (not keyword_set(proj)) then begin ;Raw volume
  mx=max(Win,min=mi)
  if n_elements(thresh) ne 1 then thresh=mi+(mx-mi)/6.
  if (thresh lt mi) or (thresh gt mx) then return,Win

  shade_volume,Win,thresh,v,p

  v=reform(v,3,n_elements(v)/3)

  x=reform(v[0,*]) & MOD_DATP,datp,'X',x
  y=reform(v[1,*]) & MOD_DATP,datp,'Y',y
  z=reform(v[2,*]) & MOD_DATP,datp,'Z',z

  MOD_DATP,datp,'PV',p
  MOD_DATP,datp,'PAR_TXT',['OBJECT: 1:Mol 2:Shape ','Thresh ']
  MOD_DATP,datp,'P'      ,[               2.       , Thresh  ]
  if (not argdp) then GIVE_DATP,datp
  return, x*0 +1
  
endif else $
if (s[0] eq 3) and (keyword_set(hkl) or keyword_set(proj)) then begin

  newsz=regular(Win,datp=datp,/GetNewSize) ;Must be on a regular grid
  if (newsz[0] lt 10.*s[1]) and (newsz[1] lt 10.*s[2]) and (newsz[2] lt 10.*s[3]) then begin
             Wreg = regular(Win, datp=datp)
  endif else Wreg = Win
  s  =  size(Wreg)
  
  if keyword_set(proj) then begin
   if n_elements(datp.x) ne s[1] then X=findgen(s[1]) else X=datp.x
   if n_elements(datp.y) ne s[2] then Y=findgen(s[2]) else Y=datp.y
   if n_elements(datp.z) ne s[3] then Z=findgen(s[3]) else Z=datp.z
   a0=min(X     ,max=a1) & b0=min(Y     ,max=b1)  & c0=min(Z     ,max=c1) ;Volume in a cube
   if keyword_set(squared) then begin
      a0=1  & b0=a0 & c0=a0
      a1=10 & b1=a1 & c1=a1
   endif
  endif else begin
   a0=min(datp.X,max=a1) & b0=min(datp.Y,max=b1)  & c0=min(datp.Z,max=c1) ;Volume in a* b* c*
  endelse
  wout=[0,0] & x=[a0,a1] & y=[b0,b1] & z=[c0,c1] & e =0.
  alpha=90. & beta=90.  & gamma=90.
  wa=2 & cells=[1,1,1]  & mk=1
  if n_elements(thresh) ne 1 then thresh=30 ;%

  p=[1.,a0,a1,b0,b1,c0,c1,alpha,beta,gamma,wa,cells, 4,1, 1,1,0 ,0,0, [0,0,0,0], mk,fltarr(mk*3)]
  ptxt=['OBJECT: '  $
     ,'coord a0' ,'coord a1'  $
     ,'coord b0' ,'coord b1'  $
     ,'coord c0' ,'coord c1'  $
     ,'alpha   ' ,'beta    ' ,'gamma   '  $
     ,'nb atoms'  $
     ,'# cells along a:','# cells along b:','# cells along c:' $
     ,'vectors input: 1:M.cartesian 2:M.spherical 3:Vibration' $
     ,'modulation mode:  1:phase 2:amplitude' $
     ,'translation P:1 C:2 A:3 B:4 I:5 F:6 Ro:7 Rv:8 H:9' $
     ,'Fourier coef (position):  1:cell 2:atom' $
     ,'Bounds: 0:given 1:to build' $
     ,'Maximum arrow/atom radius ratio' $
     ,'Do not extend molecule outside cell' $
     ,'free','free' $
     ,'free','free' $
     ,'nb propagations/modes on X,Y,Z'  $
     , strarr(mk*3) $
     ]

  pv=fltarr(4+mk*3*2+mk+12,wa+4)
  pv[0,0:wa-1]=1. & pv[1,0:wa-1]=0. & pv[2,0:wa-1]=-1. & pv[3,0:wa-1]=1.
  pv[*,wa+0]=[-4,3,255,255,255,1, 0.,0.,0.,0.,1000.,0.,0.,1000.,1000., 0   ,0,0,0,0,0,0,0]
  pv[*,wa+1]=[-4,3,255,255,255,2, 0.,0.,0.,1000.,0.,0.,1000.,0.,1000., 1000,1,0,0,0,0,0,0]
  pv[*,wa+2]=[-4,3,255,255,255,3, 0.,0.,0.,0.,1000.,0.,1000.,1000.,0., 0   ,2,0,0,0,0,0,0]
  pv[*,wa+3]=[-9,0,thresh,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
  
  datp={X:x, Y:y, Z:z, E:e, N:temporary(Wreg), $
        P:p, PV:pv, PAR_TXT:ptxt, W_TIT:datp.W_TIT, OTHER_TIT:datp.OTHER_TIT}
  if (not argdp) then GIVE_DATP,datp
  return,wout

endif else $  
if (s[0] eq 1) and (n_elements(datp.x) eq s[1]) and (n_elements(datp.y) eq s[1]) $
               and (n_elements(datp.z) eq s[1]) then begin   ; Scattered 3d points (xyzv)
	X=datp.X[sort(datp.X)] & X=X[uniq(X)] & maxX=max(X,min=minX) & nx=n_elements(X)	   
	Y=datp.Y[sort(datp.Y)] & Y=Y[uniq(Y)] & maxY=max(Y,min=minY) & ny=n_elements(Y)	   
	Z=datp.Z[sort(datp.Z)] & Z=Z[uniq(Z)] & maxZ=max(Z,min=minZ) & nz=n_elements(Z)
	
	if nx gt 1 then dX=total((X-shift(X,1))[1:nx-2])/(nx-2)  else return,win
	if ny gt 1 then dY=total((Y-shift(Y,1))[1:ny-2])/(ny-2)  else return,win
	if nz gt 1 then dZ=total((Z-shift(Z,1))[1:nz-2])/(nz-2)  else return,win
	dimx=round((maxX-minX)/dX+1) & dimy=round((maxY-minY)/dY+1) & dimz=round((maxZ-minZ)/dZ+1)
	
	z1X =round((datp.X-minX)/dX) & z1Y =round((datp.Y-minY)/dY) & z1Z =round((datp.Z-minZ)/dZ)

	XX =findgen(dimx)*dX+minX & XT='X'
	YY =findgen(dimy)*dY+minY & YT='Y'
	ZZ =findgen(dimz)*dZ+minZ & ZT='Z'
	Wtmp=fltarr(dimx,dimy,dimz)                              ;New space

	for i=0L,n_elements(Win)-1 do if Wtmp[z1X[i],z1Y[i],z1Z[i]] lt Win[i] then  $
			                 Wtmp[z1X[i],z1Y[i],z1Z[i]]=Win[i]
	MOD_DATP,datp,'X',XX
	MOD_DATP,datp,'Y',YY
	MOD_DATP,datp,'Z',ZZ
	if (not argdp) then GIVE_DATP,datp
	return,Wtmp

endif else return,win
end
