;+
; :Description:
;    Builds an OS independant path out of a list of directory names.
;
; :Params:
;    paths : in, required, type="strarr"
;       An array of directory names.
;
; :Returns:
;     The path resulting from the concatenation of the input directories.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 22 Feb. 2011
;
; :Categories: paths
;-

function build_path, dirs

    ; A list of directories must be provided as input.
    if n_elements(dirs) eq 0 then begin
        print, "No directories provided."
        return, -1    
    endif

    ; The output path.
    path = strtrim(dirs[0])
    
    ; Each directory of dirs array is appended with the corresponding OS path separator to the output path.
    for i = 1, n_elements(dirs) - 1 do path = path + path_sep() + strtrim(dirs[i],2)

    ; The output path is returned.
    return, path

end

;+
; :Description:
;    The module constructor.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 22 Feb. 2011
;
; :Categories: paths
;-
pro paths_utilities

end
