;+
; :Description:
;    Handles the event that opens an url adress in a browser.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 23 Feb. 2011
;
; :Categories: apps, elements database
;
; :private:
;-

pro periodic_table_about_openurl, event

  ; Get the uvalue of the button that will open the url when clicked.
  widget_control, event.id, get_uvalue = url

  ; Open the url adress.
  open_url, url

end

;+
; :Description:
;    Handles the events coming from the "about" dialog.
;
; :Params:
;    event : in, required, type="struct"
;       The event.
;
; :Returns:
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 23 Feb. 2011
;
; :Categories: apps, elements database
;
; :private:
;-
pro periodic_table_about_tlb_event, event

  forward_function build_path

  paths_utilities
    
  widget_control, event.top, get_uvalue = info, /no_copy

  ; The type of the event.  
  eventType = tag_names(event, /structure_name)
  
  ; Case of a WIDGET_TIMER event. The only one considered for that dialog.
  if eventType eq "WIDGET_TIMER" then begin
 
    p = file_dirname(file_which('periodic_table.pro'))  
    rcDir = build_path([p, "rc"])

    ; Increment the timer and mod it by 11 (the # of small images to be displayed in the about dialog).  
    info.timer = 1 + info.timer mod 5

    filename = build_path([rcDir, "about" + strtrim(info.timer,2) + ".jpg"])
    res = query_jpeg(filename)
    if (res eq 0) then return
      
    ; Read the image corresponding to the timer.
    read_jpeg, filename, image
      
    ; Store the IDL window ID of the drawing area in |winVis|  
    widget_control, info.aboutwin, get_value = winVis
  
    ; Select it as the active one for drawing.
    wset, winVis

    ; Display the loaded image in it.      
    tvscl, image, true = 1
      
    ; Generate another timer event that will the next image in 3s.
    widget_control, event.top, timer = 3
  
  endif
  
  widget_control, event.top, set_uvalue = info, /no_copy
        
end

;+
; :Description:
;    Builds up the "About" dialog.
;    It is made of a draw area where small images are displayed one after the other and of 
;    general informations about the application.
;    This dialog is displayed as a pop-up window with its own event handler.
;
; :Params:
;    group_leader : in, optional, type="long"
;       The widget id of the leading widget.
;       Default = 0L 
;
; :Author: pellegrini
;
; :History:
;    pellegrini, 23 Feb. 2011
;
; :Categories: apps, elements database
;-
pro periodic_table_about, group_leader = group_leader

  catch, theError
  if theError ne 0 then begin
  
    catch, /cancel
   ;ok = error_message(traceback = 1)
    print,!error_state.msg
  
  endif

  ; If the widget is already opened, then do nothing.
  if xregistered("periodic_table_about") then return

  ; If no group leader is given as input, use 0 as a default value (no group leader).
  if n_elements(group_leader) eq 0 then group_leader = 0L
      
  ; The main base. Will host all the child widgets.
  tlb = widget_base(group_leader = group_leader, /base_align_center, /row, title = "About", tlb_frame_attr = 1)
  
  ; The main base is actually displayed.
  widget_control, tlb, /realize

  ; The drawing area where the images will be displayed.    
  aboutwin = widget_draw(tlb, xsize = 200, ysize = 200)
    
  dummy = widget_base(tlb, /base_align_center, /column)
  
  ; The labels that will show general information about the application.
  l = widget_label(dummy, value = "", ysize = 10)  
  l = widget_label(dummy, value = "The LAMP elements database", font = "Times*20*Italic")  
  l = widget_label(dummy, value = "", ysize = 20)  
  l = widget_label(dummy, value = "A program developped at", font = "Times*18")
  ; Clicking on this button will open the CS webpage. 
  l = widget_label(dummy, value = "", ysize = 10)  
  b = widget_button(dummy, value = "Computing For Science Group", font = "Times*18*Bold", xsize = 250, $
                    uvalue = "http://www.ill.eu/instruments-support/computing-for-science/home/", $
                    event_pro = "periodic_table_about_openurl")
  l = widget_label(dummy, value = "", ysize = 20)  
  ; Clicking on this button will open the ILL webpage. 
  b = widget_button(dummy, value = "Institut Laue Langevin", font = "Times*18*Bold", xsize = 250, $
                    uvalue = "http://www.ill.eu/", event_pro = "periodic_table_about_openurl")
  l = widget_label(dummy, value = "", ysize = 10)  
  
  ; The structure storing the parameters to export.
  info = {aboutwin : aboutwin, timer : 0L}
    
  widget_control, tlb, set_uvalue = info, /no_copy 

  ; A timer event is generated at 0s.
  widget_control, tlb, timer = 0
        
  xmanager, "periodic_table_about", tlb, event_handler = "periodic_table_about_tlb_event", /just_reg
  
end

