pro psd_hkl, w_in, w_out, inst=inst, UB=ub, waveL = waveL, gamma=gamma , omega=omega , chi=chi , phi=phi ,$
                   EE, XX, YY, ZZ  , TOF=tof , z1H=z1H,z1K=z1K,z1L=z1L
;+
;** *******
;   Detector space to reciprocal space translation
;   Instruments: D9 , D10 , D19
;   Code taken from rdid.pro (written in 2004 D.R.)
;   Tof keyword: 0=normal scan , 1=3rd dimension is the energy transfer.
;   Return: w_out(3d space) , EE(errors) , XX(H) , YY(K) , ZZ(L)
;           z1H,z1K,z1L are the index translation (optionnaly)
;-

siz  = size(w_in)
if siz[0] lt 2            then return ; 1D input ?
if n_elements(waveL) ne 1 then return ; no wave length ?
insv = strlowcase(inst)

UB1=INVERT(ub,statub) & if statub ne 0 then return ; UB matrix incorrect ?

if siz[0] eq 2 then scan = 1 else scan = siz[3]  &  vect = fltarr(scan)

AX_g = vect +gamma*!pi/180.   & AX_o = vect +omega*!pi/180. ; Sample angles for each scan
AX_c = vect +chi  *!pi/180.   & AX_p = vect +phi  *!pi/180. ; ""

; Initiate some variables
; ***********************
  SIN_g = 2*sin(AX_g/2.)/waveL
  SIN_g =   sin(AX_g)   /waveL
  if insv eq 'd10' then COS_o = cos(AX_o-AX_g/2.) else COS_o = cos(AX_o)
  if insv eq 'd10' then SIN_o = sin(AX_o-AX_g/2.) else SIN_o = sin(AX_o)
  COS_c = cos(AX_c)  &  SIN_c = sin(AX_c)
  COS_p = cos(AX_p)  &  SIN_p = sin(AX_p)
  
         ;HMU  =[[(COS_o * COS_c * COS_p - SIN_o * SIN_p)* SIN_g] , $
         ;       [(COS_o * COS_c * SIN_p + SIN_o * COS_p)* SIN_g] , $
         ;       [(COS_o * SIN_c)* SIN_g]]
         
         ;HKL  = transpose((UB1)##(HMU))  &  HKL = round(HKL*d)/d

  z1H  = w_in*0. & z1K=z1H    & z1L=z1H ; Space for indexing each pixel inputs
  a1   =-1       & a3    =-1            ; No masked detectors
  xoff = 0.      & yoff  = 0. & zoff=0. ; No offset
  wave = waveL   & g_sign= 1  & d   =16         ; gamma origin axis is left

; Instrument specific
; *******************
case insv of
  'd19': begin & d = 16.          & flat  = 0      ; resulting space factor reduction & CURVE detector
         ncat  = 640              & nano  = 256    ; *** Detector size 640*256
         cgap  = .25              & agap  = .1565  ; *** Resolution (cm)
         a1    = 32    & a2=2     & a3= 1 & a4=1   ;     Masked detectors cat:a1_left,a2_right  ano:a3_down,a4_up
         cgap  = .1875            & agap  = .11719 ; *** Angular resolution (deg) for curved  detector ***
         radius= 76.4             & mpsd  = 1      ; (cm) & psd
         g_sign=-1                                 ; gamma origin axis is right
         end
  'd9':  begin & d = 16.          & flat  = 1      ; resulting space factor reduction & FLAT  detector
         ncat  = siz[1]           & nano  = siz[2] ; *** Detector size 32*32 , 128*128!
         cgap  = 2.               & agap  = 2.     ; *** Resolution (cm)
         radius= 488.             & mpsd  = 1      ; (cm) & psd
         if ncat ge 128 then begin& cgap=.1 & agap=.1 & radius=595. & endif
         end
  'd10': begin & d = 16.          & flat  = 1      ; resulting space factor reduction & FLAT  detector
         ncat  = 32               & nano  = 32     ; *** Detector size 32*32
         cgap  = 2.5              & agap  = 2.5    ; *** Resolution (cm)
         radius= 380.             & mpsd  = 1      ; (cm) & psd ;(radius= 437 ???)
         end
  'in5': begin & d = 16.          & flat  = 0      ; resulting space factor reduction & CURVE detector
         ncat  = 395              & nano  = 256    ; *** Detector size ncat=12*32+11=395 nano=256
         cgap  = 2.6              & agap  = 1.172  ; *** Resolution (cm)
         a1    = 0    & a2=33     & a3= 9 & a4=7   ;     Masked detectors cat:a1_left,a2_right  ano:a3_down,a4_up
         cgap  = .37242           & agap  = .16055 ; *** Angular resolution (deg) for curved  detector *** 147/ncat & 41.1/nano
         radius= 400.             & mpsd  = 1      ; (cm) & psd
         g_sign=-1                                 ; gamma origin axis is right
         end
else:    return
endcase
cmid = (ncat+1)/2. & amid=(nano+1)/2. ; middle detector pixel

;AMD
;***
cath= findgen(ncat)+1. & anod= findgen(nano)+1.
a   = g_sign*(cmid-cath)*cgap + xoff &   if a1 ge 0 then a = a[a1:ncat-a2-1]        ;masked detetectors
z   =       -(amid-anod)*agap + zoff &   if a3 ge 0 then z = z[a3:nano-a4-1]        ;""
b   =   radius          + yoff
if flat then   dd   = sqrt(a^2  + b^2)                                       ;Flat   detector
if flat then   gamo = atan(a,b) $                                            ;Flat   detector
else   begin & gamo = a*!pi/180.          & z    = z*!pi/180.                ;Curved detector
               cnu  = cos(z) & snu=sin(z) & snup = snu/wave   & endelse      ;Curved detector
scan_G = 1
if (scan gt 1) and (n_elements(uniq(AX_g)) eq 1) then begin                  ;Speed-up if one position detector
    gamp   = gamo + AX_g[0]  & gampS = sin(gamp)
    scan_G = 0               & gampC = cos(gamp)  &   endif
    
lamp_loop_so,SFdll ;We may have loops in C code
dimA = n_elements(a)  &  dimZ = n_elements(z)  &  dimS = n_elements(scan)

;Z1frmd HKL for each w_in pixel
;******
for k=0,scan-1 do begin
    cos_phim = COS_p[k]  &  cos_chim=COS_c[k]  &  cos_omem=COS_o[k]
    sin_phim = SIN_p[k]  &  sin_chim=SIN_c[k]  &  sin_omem=SIN_o[k]
    
    Mome = transpose([[cos_omem,sin_omem,0.] , [-sin_omem,cos_omem,0.] , [0.,0.,1.]])
    Mphi = transpose([[cos_phim,sin_phim,0.] , [-sin_phim,cos_phim,0.] , [0.,0.,1.]])
    Mchi = transpose([[cos_chim,0.,sin_chim] , [0.,1.,0.]              , [-sin_chim,0.,cos_chim]])
    zz   = UB1##Mphi##Mchi##Mome

    if scan_G then begin & gamp  = gamo + AX_g[k]      ; Detector is moving
                           gampS = sin(gamp)
                           gampC = cos(gamp)  &  endif

    if (flat) then begin                               ;*** Flat detector   ***
      for j=0, dimZ-1 do begin
          for i=0, dimA-1 do begin
              nup   = atan(z[j],dd[i]) & cnup  = cos(nup)  & snup=sin(nup)
              cnupS = gampS[i]*cnup    & cnupC =(gampC[i]*cnup -1.)
              hkl   = zz##([cnupS      , cnupC    , snup]/wave)
              z1H[i,j,k]=hkl[0] & z1K[i,j,k]=hkl[1] & z1L[i,j,k]=hkl[2] ;hkl for each indices ijk
          endfor ; i Gamma
      endfor ; j Z

    endif else begin                                   ;*** Curved detector ***
      if SFdll gt ' '  then begin ;Try the C code if there.
        bid = CALL_EXTERNAL(SFdll, 'HKL_calc', $
             (dimZ), (dimA), gampS, (dimA), gampC, (dimZ), cnu, $
              wave, (dimA), 3, 3, zz, (dimZ), snup, k, $
             (dimA), (dimZ), (dimS), z1H, z1K, z1L, /I_VALUE, /CDECL)
        if (bid ne 1) then begin & lamp_loop_so,/reset & SFdll='' & endif
      endif
      if SFdll le ' ' then begin  ;Use the IDL code (C code failed)
      for j=0, dimZ-1 do begin
          cnupS = gampS *  cnu[j]      / wave
          cnupC =(gampC *  cnu[j] -1.) / wave ; snup is already set
          for i=0, dimA-1 do begin
              hkl   = zz##([cnupS[i]  , cnupC[i] , snup[j]])
              z1H[i,j,k]=hkl[0] & z1K[i,j,k]=hkl[1] & z1L[i,j,k]=hkl[2] ;hkl for each indices ijk
          endfor ; i Gamma
      endfor ; j Z 
      endif
    endelse  ; curved detector
    
endfor       ; k scan

z1H=round(z1H*d)/d & z1K=round(z1K*d)/d   & z1L=round(z1L*d)/d ;Step=1./d
set_tolerance,1./d /2.
setmanip,/noraw

;Size of reciprocal space W_out
;******************************
scl2x = max(z1H,min=scl1x) & dimx = (scl2x-scl1x)*d+1 & XX = findgen(dimx)/d+scl1x & XT='H'
scl2y = max(z1K,min=scl1y) & dimy = (scl2y-scl1y)*d+1 & YY = findgen(dimy)/d+scl1y & YT='K'
scl2z = max(z1L,min=scl1z) & dimz = (scl2z-scl1z)*d+1 & ZZ = findgen(dimz)/d+scl1z & ZT='L'
W_out = fltarr(dimx,dimy,dimz)    ; New space
z1H   = round((z1H-scl1x)*d)      ; hkl -> index in new space
z1K   = round((z1K-scl1y)*d)      ; ""
z1L   = round((z1L-scl1z)*d)      ; ""
 
;Fill reciprocal space
;*********************          
if SFdll gt ' '  then begin ;Try the C code if there.
  bid = CALL_EXTERNAL(SFdll, 'HKL_vol', $
    n_elements(W_in), W_in, long(dimx), long(dimy), long(dimz), W_out, n_elements(z1H), z1H, z1K, z1L, $
    VALUE=[1, 0, 1, 1, 1, 0, 1, 0, 0, 0], /I_VALUE, /CDECL)
  if (bid ne 1) then begin & lamp_loop_so,/reset & SFdll='' & endif
endif

if SFdll le ' ' then $      ;Use the IDL code (C code failed)
  for i=0L,n_elements(w_in)-1 do if W_out[z1H[i],z1K[i],z1L[i]] lt w_in[i] then  $
           W_out[z1H[i],z1K[i],z1L[i]]  = w_in[i]

EE    = sqrt(w_out) ;error is used for futur scan additions
end
