;READ FILES CONTAINING ATOM COORDINATES AND SOME OF THEIR ATTRIBUTES.
;
;D.R. Started Oct 2003 -->
;D.R  added reading FST (Fullprof) files. (Feb 2010)
;Eric PELLEGRINI added reading atom coordinates from PDB files. (Mar 2010)
;D.R  new format XDATCAR. (Apr 2011)
;Stef Rols  passing from Primitive Cell to Cristal Cell, MATsym keyword added (Apr 2011)
;D.R  added reading GULP (.gout) and CIF files. (May 2011)
;S.R  correct problem with the matrix P (July 2012).
;S.R  adapt reading the polarization vectors to PHONON v5: needs the .d46 file to be in the
;     same directory as the .d07 file (January 2014) 

pro prim_to_crist_unitcell, Aprim,P,at_num,x,y,z,pv,mk
;+
;** **********************
;** S. ROLS May 2011
;** input : 
;** -------
;** pos_atp(3,Np)  = coordinates for the Np atom @ primitive cell : pos_atp(0,1)=x coordinate (0) of the second atom (1)
;** x,y,z(3*Np,Np)= cartesian coordinates of the polarization vectors (first index = mode, second index = atom)
;** P(3,3) = transfer matrix from primitive cell vectors to cristallographic cell
;** In mathematical notation, the P matrix is expressed as: Pij=<Ai|Aj>
;** Here, it should be constructed as P = [[P11,P21,P31],[P12,P22,P32],[P13,P23,P33]]
;** Mp = coordinate matrix of the primitive cell in cartesian
;** at_num = vector containing the atomic number of the atoms
;** A1p = [Mp(0,0), Mp(1,0), Mp(2,0) ]
;** A2p = [Mp(0,1), Mp(1,1), Mp(2,1) ]
;** A3p = [Mp(0,2), Mp(1,2), Mp(2,2) ]
;** 
;** output :
;** --------
;** pos_atp(3,Np) = coordinates for the Np atom @ primitive cell : pos_atp(0,1)=x coordinate (0) of the second atom (1)
;** x,y,z(3*Np,Np*mult) = cartesian coordinates of the polarization vectors (first index = mode, second index = atom)
;** at_num = vector containing the atomic number of the atoms
;** M = coordinate matrix of the new cell
;** A1 = [M(0,0), M(1,0), M(2,0) ]
;** A2 = [M(0,1), M(1,1), M(2,1) ]
;** A3 = [M(0,2), M(1,2), M(2,2) ]
;-

; Coordinates of the current cell vectors (primitive):
Mp=Aprim
A1p = [Mp[0,0], Mp[1,0], Mp[2,0] ]
A2p = [Mp[0,1], Mp[1,1], Mp[2,1] ]
A3p = [Mp[0,2], Mp[1,2], Mp[2,2] ]

;Did reads the matrix in the 'IDL' Manner... which is just the transpose of our logic here. Therefore transpose it before all.
P=transpose(P)
;Coordinates of the new cell vectors (cristallographic):
M = Mp#P

; inverse of the transformation matrix P
Pmoins1=invert(P)
Np=n_elements(x)  ; Np is the number of atoms in the origin cell

; compact the x,y,z vectors in a single array:
pos_atp=transpose([[x],[y],[z]])

; first determine the multiplicity of the New cell
; Volume = A.(BXC)
A1 = [M[0,0], M[1,0], M[2,0] ]
A2 = [M[0,1], M[1,1], M[2,1] ]
A3 = [M[0,2], M[1,2], M[2,2] ]

Vp=ABS(total(A1p*CROSSP(A2p, A3p)))
V =ABS(total(A1 *CROSSP(A2 , A3)))
mult=round(V/Vp)
N   =np*mult ; the new number of atoms

T=fltarr(3,4) & T[*,1]=[1.,0.,0.] & T[*,2]=[0.,1.,0.] & T[*,3]=[0.,0.,1.] ; translation vectors for the addition of atoms
print,'The multiplicity of the new cell is:',mult
print,'The number of atom per cell goes from', Np,' to ',N
mult_max=4
If ((mult lt 1.) or (mult gt mult_max)) then begin
  print,'The multiplicity of the new cell should be inferior to ',mult_max,'... Exiting without change'
  return
endif

; transform the fractional coordinates of the atoms in the new cell system:
pos_at=fltarr(3,Np*mult)
mat=at_num

for im=0,mult-1 do begin
  if im ne 0 then at_num=[mat,at_num]
  for i=0,Np-1 do begin
    pos_at[*,i+im*Np]=Pmoins1#(pos_atp[*,i]+T[*,im])
  endfor
endfor
pos_at=(round(pos_at*1000.)/1000.)
i=where(pos_at lt 0.) & if i[0] ge 0 then pos_at[i]=pos_at[i]+abs(long(pos_at[i]))+1. ;Stay fractional
i=where(pos_at gt 1.) & if i[0] ge 0 then pos_at[i]=pos_at[i]-abs(long(pos_at[i]))
x=reform(pos_at[0,*]) & y=reform(pos_at[1,*]) & z=reform(pos_at[2,*])                 ;Split coordinates

; Now deal with the vibrations vectors (polarization)
; stupid extension of Did's pv vectors as they are encoded in a quite complicated manner...
; PV ENTRIES MEANING for DISPLACEMENTS(MK=1) , VIBRATIONS(MK=3) , MAGNETIC PROPAGATION(MK=Nv)
; PV[0:3,*]   Reserved [1. , 1. , -1. , 1.] (molecule, width factor,  ! , ! )
; 
; PV[4:6,*]             dx ,dy ,dz                            DISPLACEMENT (XDATCAR Vasp)
; PV[4:12,*]  Real      vx1,vy1,vz1, vx2,vy2,vz2, vx3,vy3,vz3 VIBRATION 3modes (g94,d07,gout)
; PV[4:12,*]  Real      mx1,my1,mz1, mx2,my2,mz2, mx3,my3,mz3 MOMENT (here mk=3 propa vectors)(atm,fst)

; PV[13:21,*] Imaginary vx1,vy1,vz1, vx2,vy2,vz2, vx3,vy3,vz3 VIBRATION 3modes (d07)
; PV[13:21,*] Imaginary mx1,my1,mz1, mx2,my2,mz2, mx3,my3,mz3 MOMENT (here 3 propa vectors)(atm,fst)

; PV[22:24,*] Frequency f1 ,f2 ,f3                                      3modes (g94,d07,gout)
; PV[22:24,*] Phase     ph1,ph2,ph3                                  (here mk=3 propa vectors)(atm,fst) 

; PV[25->,*]  Reserved

; First Check that pv is correctly dimensioned

print, 'MK=',mk

CASE mk OF
3: Begin ; eigenvector case...
wn=Np & sz=size(pv) & sz=sz[2] & jjmax = wn+(wn-1)*wn

if jjmax le sz then begin

  rppv=fltarr(4+mk*3*2+mk+12,N*N)

  for jm=0,mult-1 do begin
    for i=0L,wn-1 do begin ; loop over the eigenmodes - The number of eigenmodes remains the same even after extension of the cell...
      for j=0L,wn-1 do begin & jj=j+i*wn & jj2=j+jm*wn+i*N   
        rppv[*,jj2] = pv[*,jj] ; & print,jj,j,i,wn,'jjmax=',wn-1+(wn-1)*wn
     endfor 
    endfor
  endfor
  pv=rppv

endif  ; jjmax le sz
end

1: Begin
; The file is an XDATCAR... we have to change the pv vector containing the dislacements i.e. pv[4:6,wn*nframe]
wn=Np & sz=size(pv) & sz=sz[2] & nfram=sz/wn ;& jjmax = wn+(wn-1)*wn
ifr=floor(nfram) & if(ifr ne nfram) then print,'Problem in nbre of frame:ifr nfram=',ifr,nfram
nfram=ifr & print,'The number of Frame is:',nfram

;if jjmax le sz then begin

;first change the pv vector...

; transform the fractional coordinates of the atoms in the new cell system:

for ifr=1,nfram-1 do begin
pos_atp=pv[4:6,ifr*wn:(ifr+1)*wn-1]
  for i=0,wn-1 do begin
    pos_at[*,i]=Pmoins1#(pos_atp[*,i])
  endfor
pos_at=(round(pos_at*1000.)/1000.)
i=where(pos_at lt -1.) & if i[0] ge 0 then pos_at[i]=pos_at[i]+1.;abs(long(pos_at[i]))+1. ;Stay fractional
i=where(pos_at gt 1.) & if i[0] ge 0 then pos_at[i]=pos_at[i]-1. ;-abs(long(pos_at[i]))
pv[4:6,ifr*wn:(ifr+1)*wn-1]=pos_at
endfor

if mult gt 1 then begin
  rppv=fltarr(4+mk*3*2+mk+12,N*nfram) ; new pv vectors augmented from the additional atom in the cell in case mult>1
  rppv[*,0:wn*nfram-1]=pv[4:6,*]
  for imult=1,mult-1 do rppv[*,imult*wn*nfram:(imult+1)*wn*nfram-1]=pv[4:6,*]
  pv=rppv
  rppv=0.
endif
  
end
ENDCASE
  
Aprim=M
end

pro atom_split, atom, num
;+
;** **********
;** separate an atom string such that atom=Mn34 gives atom=Mn & num=34
;-

num=0 & idx=where(byte(atom) le 57) & idx=idx[0] ; char <=9
on_ioerror,misnum
if idx gt 0 then begin & num=fix(strmid(atom,idx,4)) & atom=strmid(atom,0,idx) & endif
misnum:
end

Function PCell_To_Inv_Mat, a0, b0, c0, alpha0, beta0, gamma0
;+
;******* ****************
;**
;** This function performs the conversion from the six cell parameters a, b, c, alpha, beta and gamma
;** to the inverted cell matrix assuming that:
;**    - a //x
;**    - b in (x,y) plane
;** a, b, c in Angstrom
;** Alpha, beta and gamma in degree
;** The inverted cell matrix allows to convert from direct coordinates to fractional coordinates.
;-

    ; Local copy of the input parameters.
    a     = a0
    b     = b0
    c     = c0
    alpha = alpha0
    beta  = beta0
    gamma = gamma0

    ; The cell matrix is initialized.
    Mat = fltarr(3,3)
        
    ; Alpha, Beta and Gamma are converted in radian.
    alpha  = alpha*!pi/180.0
    beta   = beta *!pi/180.0
    gamma  = gamma*!pi/180.0
    
    ; Some useful intermediates.
    cA  = cos(alpha)
    cB  = cos(beta)
    cG  = cos(gamma)
    sG  = sin(gamma)
    tG  = tan(gamma)
    Phi = Sqrt(1.0 - cA^2 - cB^2 - cG^2 + 2.0*cA*cB*cG)
    
    ; The inverted cell matrix taken from Int. Table of Crys. Vol B p.345
    Mat[0,0] =  1.0/a 
    Mat[0,1] = -1.0/(a*tG) 
    Mat[0,2] =  (cA*cG - cB)/(a*Phi*sG)
    Mat[1,0] =  0.0
    Mat[1,1] =  1.0/(b*sG)
    Mat[1,2] =  (cB*cG - cA)/(b*Phi*sG)
    Mat[2,0] =  0.0
    Mat[2,1] =  0.0
    Mat[2,2] =  sG/(c*Phi)

    Return, Mat
End

function Mat_to_Pcell, m,unit
;+
;******* ************
;** pcell=Mat_to_Pcell(matrice,unit) -> pcell=[a,b,c,alpha,beta,gamma]
;-

a=sqrt(m[0,0]^2+m[1,0]^2+m[2,0]^2)
b=sqrt(m[0,1]^2+m[1,1]^2+m[2,1]^2)
c=sqrt(m[0,2]^2+m[1,2]^2+m[2,2]^2)

x=(m[0,1]*m[0,2]+m[1,1]*m[1,2]+m[2,1]*m[2,2]) / (b*c)
y=(m[0,0]*m[0,2]+m[1,0]*m[1,2]+m[2,0]*m[2,2]) / (a*c)
z=(m[0,0]*m[0,1]+m[1,0]*m[1,1]+m[2,0]*m[2,1]) / (a*b)
alpha=acos(x)*180./!pi
beta =acos(y)*180./!pi
gamma=acos(z)*180./!pi

return, [a*unit,b*unit,c*unit,alpha,beta,gamma]
end

function read_mag, inst,path,bfil,status,datp, MATsym=MATsym
;+
;******* ********
;** reads atom files comming from different dyn mol applications
;** and a lamp workspace "object" for use  by MagDraw application
;*******************
;Important Variables:
;*******************
;** WT                 'title'
;** W                  [atomic numbers]
;** X,Y,Z               atomic coordinates [0->1]
;** A,B,C               cell size Angst.
;** ALPHA,BETA,GAMMA    cell angles Deg.
;** BND                 =1 tell to build bounds
;** P20                 =1 tell to complete molecules outside the cell
;** WIDTH               fraction of real atom size (default=1.)
;** CSV,SPGR,IFO,MK,VMK miscellaneous magnetic,movements properties (xdatcar .G94 .D07 .GOUT .ATM .FST)
;** OFFS                set to 1 to eliminate first atom index
;** PV !
;-

forward_function d_livSymbtoType, d_livMasstoType, d_livTypetoName

CATCH,stat & if stat ne 0 then begin & lamp_error & if u gt 0 then free_lun,u & return,0 & endif

u=0 & liv_objet & d_livClearAnim ; Complile liv_objet & Clear some memory

w=0 & x=0. & y=0.   & z=0.  & e=0.  & n=0   & p=[0] & ptxt=['']   & pv=0. & wt='' & xt='' & yt='' & zt='' & ot=''
datp={X:x, Y:y, Z:z, E:e, N:n, P:p, PV:pv, PAR_TXT:ptxt, W_TIT:wt, X_TIT:xt, Y_TIT:yt, Z_TIT:zt, OTHER_TIT:ot}
if n_elements(bfil)  eq 0 then return,w
line =' '  & xc=0.  & yc=0. & zc=0.   & ec=0. & mk=1  & a=10. & b=10.    & c=10. & alpha=90. & beta=90.  & gamma=90.
phase=1    & ifo=1. & ph=0. & spgr=1. & xi=0. & yi=0. & zi=0. & modul=0. & phi=0 & theta=0   & imodul=0. & iphi=0 & itheta=0
Vmk=[0.,0.,0.] & curato=0   & offs=0  & csv=1 & bnd=0 & xyz=[xc,yc,zc]   & xyzP=[xyz,xyz]    & Alabel=''
pv0=fltarr(4+mk*3*2+mk+12)  & pv=pv0  & p20=0 & width=1. & a0 = 0.0 & b0 = 0.0 & c0 = 0.0    & has_pbc = 0

on_ioerror,misread & u=-1 & status=11
OPENR,u,path+bfil,/get_lun
status=13
ot  =bfil
pnt= strpos(/reverse_search,bfil,'.') & if pnt ge 0 then ext=strlowcase(strtrim(strmid(bfil,pnt,10),2)) else ext=''
pnt= strpos(/reverse_search,strlowcase(strtrim(bfil,2)),'xdatcar')
    if pnt ge 0 then ext='xdatcar' else begin
       pnt= strpos(/reverse_search,strlowcase(strtrim(bfil,2)),'chgcar')
       if pnt ge 0 then ext='chgcar' else begin
          pnt= strpos(/reverse_search,strlowcase(strtrim(bfil,2)),'contcar')
          if pnt ge 0 then ext='contcar' else begin
    endelse & endelse & endelse
    
; PV ENTRIES MEANING for DISPLACEMENTS(MK=1) , VIBRATIONS(MK=3) , MAGNETIC PROPAGATION(MK=Nv)
; PV[0:3,*]   Reserved [1. , 1. , -1. , 1.] (molecule, width factor,  ! , ! )
; 
; PV[4:6,*]             dx ,dy ,dz                            DISPLACEMENT (XDATCAR Vasp)
; PV[4:12,*]  Real      vx1,vy1,vz1, vx2,vy2,vz2, vx3,vy3,vz3 VIBRATION 3modes (g94,d07,gout)
; PV[4:12,*]  Real      mx1,my1,mz1, mx2,my2,mz2, mx3,my3,mz3 MOMENT (here mk=3 propa vectors)(atm,fst)

; PV[13:21,*] Imaginary vx1,vy1,vz1, vx2,vy2,vz2, vx3,vy3,vz3 VIBRATION 3modes (d07)
; PV[13:21,*] Imaginary mx1,my1,mz1, mx2,my2,mz2, mx3,my3,mz3 MOMENT (here 3 propa vectors)(atm,fst)

; PV[22:24,*] Frequency f1 ,f2 ,f3                                      3modes (g94,d07,gout)
; PV[22:24,*] Phase     ph1,ph2,ph3                                  (here mk=3 propa vectors)(atm,fst) 

; PV[25->,*]  Reserved

;*****************
;External function
;*****************
;Mat_to_Pcell      pcell=Mat_to_Pcell(matrice,unit) -> pcell=[a,b,c,alpha,beta,gamma]
;Atom_split        atom_split, atom, num  ->  atom=Mn34 gives atom=Mn & num=34
;d_livSymbtoType   NumAt=d_livSymbtoType([SymbAt])
;d_livMasstoType   NumAt=d_livMasstoType( MassAt )

;*****************
;Variables to set:
;*****************
;WT                 'title'
;W                  [atomic numbers]
;X,Y,Z               atomic coordinates [0->1]
;A,B,C               cell size Angst.
;ALPHA,BETA,GAMMA    cell angles Deg.
;BND                 =1 tell to build bounds
;P20                 =1 tell to complete molecules outside the cell
;WIDTH               fraction of real atom size (default=1.)
;CSV,SPGR,IFO,MK,VMK miscellaneous magnetic,movements properties (xdatcar .G94 .D07 .GOUT .ATM .FST)
;OFFS                set to 1 to eliminate first atom index
;PV !
;*****************
case ext of

; Case of a file with '.pdb' extension. Will be parsed as a 'Protein Data Bank' file.
'.pdb': begin

    ; Flag used to check whether PBC are stored in the PDB file.
    Has_PBC = 0
    
    ; The space group is set by default to "P".
    space_group = "P"

    ; Some PDB formats definitions.
    ; The PDB format for 'ATOM'/'HETATM' line without the last element not useful for us.
    atom_long_fmt  = '(I5,1X,A4,A1,A3,1X,A1,I4,A1,3X,3F8.3,2F6.2,10X,A2)'
    ; A shorter version without the last four elements. Tried if the previous one failed.
    atom_short_fmt = '(I5,1X,A4,A1,A3,1X,A1,I4,A1,3X,3F8.3)'
    
    ; The PDB format for 'CRYST1'.
    cryst1_long_fmt  = '(3F9.3,3F7.2,1X,A11,I4)'
    ; A shorter version without the last two elements. Tried if the previous one failed.
    cryst1_short_fmt = '(3F9.3,3F7.2)'
    
    ; This string will store the contents of each PDB file line.
    line = ''
    
    ; Loop over all the line of the file.
    While Not EOF(u) Do Begin
        
        ; Reads the line and store it in |line|.
        Readf, u, line
        
        ; The line is lowerized and right-trimed.  
        line = strlowcase(strtrim(line,2))
        
        ; Case where the line starts with 'TITLE'.
        ; This line has to be parsed as it stores the title.
        ; In PDB there can be several 'TITLE' lines so all of them
        ; will be concatenated to |WT| variable.        
        If strmid(line, 0, 5) eq "title" Then Begin
            wt  = wt + strtrim(strmid(line, 5),2) + ''
                
        ; Case where the 'CRYST1' PDB line.
        ; This line has to be parsed as it stores the PBC.
        EndIf Else If strmid(line, 0, 6) eq "cryst1" Then Begin
                                                                
            ; First tries to read the complete 'CRYST1' format
            On_IOError, Cryst1_Long_Fmt_Error
                        
            s_group = ""
            z_value = 0            
            
            OK_Long = 0
            
            ; If so, extract from it a, b, c, alpha, beta and gamma values.
            Reads, strmid(line, 6), a, b, c, alpha, beta, gamma, s_group, z_value, format = cryst1_long_fmt
            
            ; The line could be read.
            OK_long = 1
            
            ; The space group is extracted from the line and upperized.
            space_group = strupcase(strmid(strtrim(s_group,2),0,1))
                        
            ; The long-format read failed so tries to read the short 'CRYST1' format.
            Cryst1_Long_Fmt_Error: If Not OK_Long Then Begin
                On_IOError, Cryst1_Short_Fmt_Error
                
                OK_Short = 0
                                
                ; If so, extract from it a, b, c, alpha, beta and gamma values.
                Reads, strmid(line, 6), a, b, c, alpha, beta, gamma, format = cryst1_short_fmt
                
                OK_short = 1
                
                ; The short-format read failed. There must be something really wrong with that line.
                Cryst1_Short_Fmt_Error: If Not OK_Short Then Lamp_Error
                                                
            EndIf
            
            ; When PBC are found, set the Has_PBC flag to 1.
            Has_PBC = 1
            
        ; Case where the line contains the keywords 'ATOMS' or 'HETATM'.
        ; Those lines must be parsed, they contain the atomic cartesian coordinates.
        EndIf Else If (strmid(line, 0, 4) eq "atom") or (strmid(line, 0, 6) eq "hetatm") Then Begin
        
            ; The variable that will store the contents of the line are reset at each reading
            serial_num  = 0
            at_name     = ''
            alt_loc     = ''
            res_name    = ''
            ch_name     = ''
            res_seq_num = 0
            res_ins     = ''
            x_coord     = 0.0
            y_coord     = 0.0
            z_coord     = 0.0    
            occupancy   = 0.0
            temp_fact   = 0.0
            at_symbol   = 'XX'
        
            ; First tries to read the x, y, z and the atom symbol using the long format.
            On_IOError, Atom_Long_Fmt_Error
            
            OK_Long = 0
                        
            Reads, strmid(line, 6), serial_num, at_name, alt_loc, res_name, ch_name,        $
                   res_seq_num, res_ins, x_coord, y_coord, z_coord, occupancy, temp_factor, $
                   at_symbol, format = atom_long_fmt
                   
            OK_Long = 1
            
            ; The long-format read failed so tries to read the x, y and z using the short format.
            Atom_Long_Fmt_Error: If Not OK_Long Then Begin
                On_IOError, Atom_Short_Fmt_Error
                
                OK_Short = 0
                
                Reads, strmid(line, 6), serial_num, at_name, alt_loc, res_name, ch_name, $
                       res_seq_num, res_ins, x_coord, y_coord, z_coord, format = atom_short_fmt
                       
                OK_Short = 1
                
                ; The short-format read failed. There must be something really wrong with that line.
                Atom_Short_Fmt_Error: If Not OK_Short Then Lamp_Error
            EndIf
            
            ; Retrieve the atomic number from the atom symbol.
            Num_At = d_livSymbtoType([strtrim(strupcase(at_symbol),2)])
            If Num_At le 0 Then Begin
                Num_At = Guess_Atom_Number_From_Atom_Name[at_name]
            EndIf

            ; Case of the atom read. Initializes the LAMP array.
            ; W     --> the atomic numbers.            
            ; Coord --> the atomic coordinates.            
            If N_ELEMENTS(Coord) eq 0 Then Begin
            
                ; This array will store the atomic number of the atoms read.
                w = LonArr(1)
                
                ; It is initialized with the atomic number of the first atom.
                w[0] = Num_At
                
                ; This array will store the atomic coordinates (3,n_atoms).
                Coord = FltArr(3,1)
                
                ; It is initialized with the coordinates of the first atom read.
                Coord[*,0] = [x_coord, y_coord, z_coord]
                
                ; Not used in case of PDB file but necessary in the context of readmag.
                pv=pv0
                
            ; Otherwise, append them with the parsed values.
            EndIf Else Begin
            
                ; The atomic number and coordinates array are extended with respectively
                ; the atomic number and coordinates of the last read atom.
                w     = [w, Num_At]
                Coord = [[Coord], [x_coord, y_coord, z_coord]]
                pv    = [[pv],[pv0]]
                
            EndElse
            
        EndIf
                    
    EndWhile

    ; Case where the PDB file has no 'CRYST1' field.
    ; a, b and c are respectively set to max(x), max(y) and max(z)
    ; The coordinates will not be converted to fractional coordinates. 
    If not Has_PBC Then Begin
    
        a     = max(Coord[0,*], min = a0)
        b     = max(Coord[1,*], min = b0)
        c     = max(Coord[2,*], min = c0)
        alpha = 90.0   
        beta  = 90.0   
        gamma = 90.0

        ;Coord[0,*] = Coord[0,*] - a0
        ;Coord[1,*] = Coord[1,*] - b0
        ;Coord[2,*] = Coord[2,*] - c0

        ;a = a - a0
        ;b = b - b0
        ;c = c - c0

        ;a0 = 0.0
        ;b0 = 0.0
        ;c0 = 0.0

    EndIf Else begin
        
    ; The cell matrix is constructed out of the six cell parameters.
    Cell_Mat = PCell_To_Inv_Mat(a, b, c, alpha, beta, gamma)
        
    ; The direct coordinates are converted to fractional coordinates,
    ; and only three digits are kept.
    Coord = Round(Matrix_Multiply(Cell_Mat, Coord)*1000.0)/1000.0
  
    Endelse
      
    ; The x, y and z coordinates are extracted out of the coordinates matrix.
    x = Reform(Coord[0,*])
    y = Reform(Coord[1,*])
    z = Reform(Coord[2,*])
    
    ; The bond will be constructed when possible.
    bnd = 1

    ; Set the global variable |spgr| to an internal number coding for the space group.    
    Case space_group Of
        'P': spgr = 1
        'C': spgr = 2 
        'A': spgr = 3
        'B': spgr = 4
        'I': spgr = 5
        'F': spgr = 6
        'R': spgr = 7
        'H': spgr = 9
        Else: spgr=1
    EndCase   
   End

'.fst': begin & offs=1 ;** FullProf
    readf,u,line
    WHILE not eof(u) do begin
    readf,u,line & line=strtrim(line,2) & lin0=strcompress(line)
    head=strlowcase(line)  &  idx=strpos(head,' ')
    if idx gt 0 then begin &   head=strmid(head,0,idx) & line=strmid(line,idx+1,250) & endif
    CASE head of
    ''     :
    '!titl': wt=strmid(lin0,7,80)

    'cell' : reads,line,a,b,c, alpha,beta,gamma

    'spaceg':begin & spgr=strupcase(strmid(lin0,7,1)) & spaceG=strmid(lin0,9,80)
                   case spgr of 'P':spgr=1 & 'C':spgr=2 & 'A':spgr=3 & 'B':spgr=4 & 'I':spgr=5 & 'F':spgr=6 & 'R':spgr=7 & else:spgr=1 & endcase
                   end
    'gener': begin & if strpos(line,'/') gt 0 then line=strjoin(strsplit(/EXTRACT,line,'/'),'./')
                   line=strcompress(strlowcase(line),/remove_all)
                   line=strsplit(/EXTRACT,line,',') & if n_elements(gener) eq 0 then gener= [line] $        ;UN-USED!!!
                                                                                else gener=[[gener],[line]]
                   line='' & end

    'box'  : begin & box=fltarr(6) & reads,line,box & if total(abs(box)) gt 3. then begin & bnd=1 & endif & end

    'conn' : bnd=1

    'atom' : begin & line=strtrim(line,2)    & idx =where(byte(line) le 32) & idx=idx[0]
                   name=strmid(line,0,idx) & line=strmid(line,idx,110)    & Alabel=[Alabel,name] ;Label  of the atom
                   atom_split,name,num
                   line=strtrim(line,2)    & idx =where(byte(line) le 32) & idx=idx[0]           ;Symbol of the atom
                   name=strmid(line,0,idx) & line=strmid(line,idx,110)
                   type=d_livSymbtoType([name])
                   if (type gt 0) then begin & reads,line,xc,yc,zc & w=[w,type] & pv1=pv0
                                             if strpos(line,'nodisplay') gt 0 then pv1[1]=.0001 & pv=[[pv],[pv1]]
                                             if xc lt 0. then xc=(xc+1.)>0. & if xc gt 1. then xc=(xc-1.)<1. ;FRACTIONAL
                                             if yc lt 0. then yc=(yc+1.)>0. & if yc gt 1. then yc=(yc-1.)<1.
                                             if zc lt 0. then zc=(zc+1.)>0. & if zc gt 1. then zc=(zc-1.)<1.
                                             x=[x,round(xc*1000.)/1000.]
                                             y=[y,round(yc*1000.)/1000.] & z=[z,round(zc*1000.)/1000.] & endif
                   curato=n_elements(w)-1  & end

    'symm' : begin & if strpos(line,'/') gt 0 then line=strjoin(strsplit(/EXTRACT,line,'/'),'./')
                   line=strcompress(strlowcase(line),/remove_all)
                   line=strsplit(/EXTRACT,line,',')  & if n_elements(symm)  eq 0 then symm = [line] $
                                                                                 else symm =[[symm],[line]]
                                                       if n_elements(symk)  eq 0 then symk =  0     $
                                                                                 else symk =[ symk , 0]
                   line='' & end

    'msym' : begin & if strpos(line,'/') gt 0 then line=strjoin(strsplit(/EXTRACT,line,'/'),'./')
                   line=strcompress(strlowcase(line),/remove_all)
                   line=strsplit(/EXTRACT,line,',')  & if n_elements(msym)  eq 0 then msym = [line] $
                                                                                 else msym =[[msym],[line]]
                   szt =n_elements(symk)-1 & symk[szt]=symk[szt]+1
                   line='' & end

    'k'    : begin & reads,line,xc,yc,zc
                   if n_elements(veck) eq 0 then begin & veck=1            & Vmk=[    xc,yc,zc]
                   endif else begin & mk=mk+1 & pv0=fltarr(4+mk*3*2+mk+12) & Vmk=[Vmk,xc,yc,zc]  & endelse
                   if curato gt 0 then begin &  wth=pv[1,*] & pv=fltarr(n_elements(pv0),curato+1) & pv[1,*]=wth & endif
                   end

    'matom': begin & line=strtrim(line,2)    & idx =where(byte(line) le 32) & idx=idx[0] ;Label  of the atom
                   name=strmid(line,0,idx) & line=strmid(line,idx,110)
                   atom_split,name,num
                   line=strtrim(line,2)    & idx =where(byte(line) le 32) & idx=idx[0] ;Symbol of the atom
                   name=strmid(line,0,idx) & line=strmid(line,idx,110)
                   type=d_livSymbtoType([name])
                   if (type gt 0) then begin & reads,line,xc,yc,zc
                                             if xc lt 0. then xc=(xc+1.)>0. & if xc gt 1. then xc=(xc-1.)<1. ;FRACTIONAL
                                             if yc lt 0. then yc=(yc+1.)>0. & if yc gt 1. then yc=(yc-1.)<1.
                                             if zc lt 0. then zc=(zc+1.)>0. & if zc gt 1. then zc=(zc-1.)<1.
                                             xc=round(xc*1000.)/1000. & yc=round(yc*1000.)/1000. & zc=round(zc*1000.)/1000.
                                             idx=where((x eq xc) and (y eq yc) and (z eq zc)) &  idx=idx[n_elements(idx)-1]
                                             if idx le 0 then begin & w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]]
                                                                    curato=n_elements(w)-1 ;NEW ATOM
                                             endif else curato=idx                         ;OLD ATOM
                   endif & end

    'skp'  : begin & num=0L & symkn=0L & reads,line,num,symkn,xc,yc,zc, xi,yi,zi, ph & num=num-1
                   pv[4+num*3:4+num*3+2,curato]=[xc,yc,zc] & pv[4+mk*3+num*3:4+mk*3+num*3+2,curato]=[xi,yi,zi]
                   pv[4+mk*3*2+num,curato]=ph
                   if n_elements(symkt) eq 0 then symkt=[curato-1,num+1,symkn] else symkt=[[symkt],[curato-1,num+1,symkn]]
                   end
    else:
    ENDCASE
    ENDWHILE
        end

'chgcar': begin & csv=1 & p20=0 & bnd=1 & width=(3./4.)^3 ;** Vasp (densities)
      readf,u,wt   &  ips=strpos(strlowcase(wt),'atoms:')
      unit =0.          & readf,u,unit
      mat  =fltarr(3,3) & readf,u,mat
      nbatm=lonarr(12)  & readf,u,line & tmpline=line
                          readf,u,line
      if strpos(line,'Direct') eq -1 then begin      ;We have atom symbols in tmpline and nb atoms in line
         atoms= strsplit(/EXTRACT,strcompress(strtrim(tmpline,2)),' ') ;Atom symbols
         type = d_livSymbtoType(atoms)
         reads, line   +' 0 0 0 0 0 0 0 0 0 0 0',nbatm & nbatm=nbatm[where(nbatm gt 0)] ;Atoms numbers
         readf,u,line ;Read 'Direct'
      endif else if ips ge 0 then begin
         atoms=strcompress(strtrim(strmid(wt,ips+6,50),2)) & atoms=strsplit(/EXTRACT,atoms,' ')   ;Atom  symbols
         type=d_livSymbtoType(atoms)
         reads, tmpline+' 0 0 0 0 0 0 0 0 0 0 0',nbatm & nbatm=nbatm[where(nbatm gt 0)] ;Atoms numbers
      endif else begin
         type=indgen(n_elements(nbatm))+1
         reads, tmpline+' 0 0 0 0 0 0 0 0 0 0 0',nbatm & nbatm=nbatm[where(nbatm gt 0)] ;Atoms numbers
      endelse
      W=lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]
      pcell=Mat_to_Pcell(mat,unit)
      a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
      wn=n_elements(W)
      xyz=fltarr(3,wn)   & readf,u,xyz
      X=reform(xyz[0,*]) & Y=reform(xyz[1,*]) & Z=reform(xyz[2,*])
      PV=fltarr(n_elements(pv0),wn)
      readf,u,line       & readf,u,line & dim=[0,0,0]  & reads,line,dim & kpline=line
      N=fltarr(dim[0],dim[1],dim[2])    & readf,u,N
      ok=1
      while ok do begin & readf,u,line    & if strpos(line,kpline) ge 0 then ok=0 & endwhile
      M=fltarr(dim[0],dim[1],dim[2])    & readf,u,M
      N=[[[N]],[[M]]] & N=reform(N,dim[0],dim[1],dim[2],2,/overwrite)
      end

'xdatcar': begin & csv=3 & bnd=1 & p20=0 & width=(3./4.)^3    ;** Vasp (frames) atoms unknown
     wn=0L & wnb=0L & nfram=0L
     line5=strarr(5)
     readf,u,line5
     bugformat=strpos(line5[3],'Direct')
     oldformat=1
     
     info =FSTAT(u) & bsize=info.size-info.cur_ptr & crlf=2 ; CRLF IS TO BE ADJUSTED !!!! bsize and CRLF ARE USED TO CALCULATED AN AVERAGED NUMBER OF FRAME

     if bugformat lt 0 then begin
        tmp=strsplit(/EXTRACT, strtrim( strcompress(line5[1]) ,2) ,' ') ;check if we have an old format
        if n_elements(tmp) lt 5 then oldformat=0                        ;No
        if oldformat then begin
           reads,line5[0],wn,wnb,nfram,format='(i4,i4,i5)' ;nfram is in the file
           if (wn gt 0) and (wn eq wnb) then begin
              reads,line5[1],xc,a,b,c,yc & a=a*1.E10 &  b=b*1.E10 &  c=c*1.E10
              wt=line5[4]   & readf,u,line
           endif
        endif else begin ;new format
           wt =line5[0]                           ;title
           unit=0. & reads,line5[1],unit          ;cell scale
           mat=fltarr(3,3) & reads,line5[2:4],mat ;cell matrix
           readf,u,line & line=strtrim(line,2)    ;atoms
           atoms=strsplit(/EXTRACT,strcompress(line),' ')
           type =d_livSymbtoType(atoms)
           nbatm=lonarr(12)  & readf,u,line       ;nb atoms
           reads,line+' 0 0 0 0 0 0 0 0 0 0 0',nbatm
           nbatm=nbatm[where(nbatm gt 0)]
           W =lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]
           wn=n_elements(W) & wnb=wn
           nfram=round(bsize/(((37.+crlf)*wn)+crlf)) ;nfram is calculated from the size of the file and value of crlf "magic formula" to me SR 11/2013
        endelse
     endif else begin ;bugformat
        wt=line5[0] & atoms=strtrim(line5[1],2)
        atoms=strsplit(/EXTRACT,strcompress(atoms),' ')   & type =d_livSymbtoType(atoms)
        nbatm=lonarr(12)  & reads,line5[2]+' 0 0 0 0 0 0 0 0 0 0 0',nbatm  & nbatm=nbatm[where(nbatm gt 0)]
        W =lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]
        wn=n_elements(W) & wnb=wn
        nfram=round(bsize/(((37.+crlf)*wn)+crlf))   ;nfram is calculated
     endelse

     if (wn gt 0) and (wn eq wnb) then begin
      on_ioerror,frmeof & ok=1
      xyz=fltarr(3,wn,nfram)
      bid=fltarr(3,wn)
      for nf=0L,nfram-1 do begin & readf,u,line  &  readf,u,bid  &  xyz[0,0,nf]=bid  & endfor
      ok=0

      frmeof: if ok then begin & tot=total(total(xyz,1),1) & idx=where(tot ne 0.)
                               if n_elements(idx) ne n_elements(tot) then xyz=xyz[*,*,idx]
                               tot=0 & idx=0 & endif
      nfram=ulong((size(xyz))[3])
      print,'Frames:',nfram,' Step:1 ' & if ok then print,!error_state.msg

      mk=1 & pv=fltarr(4+mk*3*2+mk+12,1.*wn*(nfram)) & Vmk=fltarr(3*mk)
      corg=xyz[*,*,0]
      curd=corg*0.
      for i=1UL,nfram-1 do begin & disp=xyz[*,*,i]-xyz[*,*,i-1]
          idx=where(abs(disp) gt .5) & if idx[0] ge 0 then begin & tmp=disp[idx] & tmp=tmp-round(tmp) & disp[idx]=tmp & endif
          curd=disp+curd & pv[4:6,i*wn:(i+1)*wn-1]=curd
      endfor
      X=reform(xyz[0,*,0]) & Y=reform(xyz[1,*,0])
      Z=reform(xyz[2,*,0]) & if n_elements(W) ne wn then W=lonarr(wn)
      xyz=0 ;Free memory
      v=-1
      
      IF OLDFORMAT THEN BEGIN ;must read CONTCAR to have cell parameters
      
      vfil=strtrim(bfil,2) & xdat=strmid(vfil,pnt,4) & if xdat eq 'XDAT' then xdat='CONT' else xdat='cont'
      vfil=strmid(vfil,0,pnt)+xdat+strmid(vfil,pnt+4,100)
      on_ioerror,misCONTCAR
      OPENR,v,path+vfil,/get_lun
      readf,v,wt & ips=strpos(strlowcase(wt),'atoms:')
      unit =0.          & readf,v,unit
      mat  =fltarr(3,3) & readf,v,mat

      atoms=''          & readf,v,atoms
      didi =''          & readf,v,line
      if strpos(strlowcase(line),'direct') ge 0 then begin & line=atoms   & atoms=''               & endif $
                                                else begin & readf,v,didi & atoms=strtrim(atoms,2) & endelse
      if ips   ge 0  then atoms=strtrim(strmid(wt,ips+6,80),2)

      nbatm=lonarr(12)  & reads,  line+' 0 0 0 0 0 0 0 0 0 0 0',nbatm   & nbatm=nbatm[where(nbatm gt 0)]
      if atoms gt '' then begin & atoms=strsplit(/EXTRACT,strcompress(atoms),' ') & type =d_livSymbtoType(atoms)
      endif          else       type =indgen(n_elements(nbatm))+1

      W=lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]

      ENDIF
      
      pcell=Mat_to_Pcell(mat,unit)
      a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
      misCONTCAR: if v gt 0 then free_lun,v
      
      wt='Step:1 '+wt
     endif
     end

'contcar': begin & csv=1 & bnd=1 & p20=0 & width=1.    ;** Vasp (structure only)
      wn=0 & atoms='' & didi=''
      readf,u,wt & ips=strpos(strlowcase(wt),'atoms:')
      unit =0.          & readf,u,unit
      mat  =fltarr(3,3) & readf,u,mat

      readf,u,atoms     & readf,u,line
      if strpos(strlowcase(line),'direct') ge 0 then begin & line=atoms   & atoms=''               & endif $
                                                else begin & readf,u,didi & atoms=strtrim(atoms,2) & endelse
      if ips   ge 0  then atoms=strtrim(strmid(wt,ips+6,80),2)

      nbatm=lonarr(12)  & reads,  line+' 0 0 0 0 0 0 0 0 0 0 0',nbatm   & nbatm=nbatm[where(nbatm gt 0)]
      if atoms gt '' then begin & atoms=strsplit(/EXTRACT,strcompress(atoms),' ') & type =d_livSymbtoType(atoms)
      endif          else       type =indgen(n_elements(nbatm))+1

      W=lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]
      pcell=Mat_to_Pcell(mat,unit)
      a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
      wn=n_elements(W) & tmp=fltarr(3)
      xyz=fltarr(3,wn)
      for i=0,wn-1 do begin & readf,u,tmp & xyz[*,i]=tmp & endfor
      xyz=(round(xyz*1000.)/1000.)
      idx=where(xyz lt 0.) & if idx[0] ge 0 then xyz[idx]=(xyz[idx]+1.)>0.
      idx=where(xyz gt 1.) & if idx[0] ge 0 then xyz[idx]=(xyz[idx]-1.)<1.
      X=reform(xyz[0,*]) & Y=reform(xyz[1,*]) & Z=reform(xyz[2,*])
      PV=fltarr(n_elements(pv0),wn)
     end

'.d23': begin & csv=1 & bnd=0 & p20=1 & width=1.    ;** Phonon (structure only)
      wn=0
      readf,u,wt & ips=strpos(strlowcase(wt),'atoms:')
      unit =0.          & readf,u,unit
      mat  =fltarr(3,3) & readf,u,mat
      nbatm=lonarr(12)  & readf,u,line & reads,line+' 0 0 0 0 0 0 0 0 0 0 0',nbatm & nbatm=nbatm[where(nbatm gt 0)]
      if ips ge 0 then begin & atoms=strcompress(strtrim(strmid(wt,ips+6,50),2)) & atoms=strsplit(/EXTRACT,atoms,' ')
         type=d_livSymbtoType(atoms)
      endif else type=indgen(n_elements(nbatm))+1
      W=lonarr(nbatm[0])+type[0] & for i=1L,n_elements(nbatm)-1 do W=[W,lonarr(nbatm[i])+type[i]]
      mat  =transpose(mat)
      pcell=Mat_to_Pcell(mat,unit)
      a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
      wn=n_elements(W)   & tmp=fltarr(3)
      xyz=fltarr(3,wn)   & readf,u,line
      for i=0L,wn-1 do begin & readf,u,tmp & xyz[*,i]=tmp & endfor
      xyz=(round(xyz*1000.)/1000.)>0.<1.
      X=reform(xyz[0,*]) & Y=reform(xyz[1,*]) & Z=reform(xyz[2,*])
      PV=fltarr(n_elements(pv0),wn)
     end

'.car': if strpos(bfil,'_4.') ge 0 then begin & csv=0 & bnd=1 & offs=1 ;** Cerius
     readf,u,line & wt=strtrim(line,2)
     ok=1
     while ok do begin & readf,u,line  & if strpos(line,'!DATE') ge 0 then ok=0 & endwhile
     ok=1
     while ok do begin & readf,u,line  & if strpos(line,'end') ge 0 then ok=0 $
           else begin & name=strmid(line,0,7) & line=strmid(line,7,120)
                atom_split,name,num         & type=d_livSymbtoType([name])
                if (type gt 0) then begin & reads,line,xc,yc,zc
                    w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]] & curato=curato+1 & endif
           endelse
     endwhile
     if curato gt 0 then begin
        wn=curato & offs=0 & W=W[1:wn] & X=X[1:wn] & Y=Y[1:wn] & Z=Z[1:wn] & PV=PV[*,1:*]
     endif
     endif

;'.g94': begin & csv=3 & bnd=1 & a=1. & b=1. & c=1. & offs=1      ;** Gaussian 94 (vibrations) a,b,c,angles unknown
;    readf,u,line & wt=strtrim(line,2)
;    ok=1
;    while ok do begin & readf,u,line  & if strpos(line,'X           Y           Z') ge 0 then ok=0 & endwhile
;    ok=1
;    while ok do begin & readf,u,line  & if strpos(line,'-------------------------') ge 0 then ok=0 & endwhile
;    ok=1 & bid=0 & typ=0
;    while ok do begin & readf,u,line  & if strpos(line,'-------------------------') ge 0 then ok=0 $
;           else begin & reads,line+' 0',bid
;                if bid gt 0 then begin & reads,line, bid,typ,xc,yc,zc & W=[W,typ] & X=[X,xc] & Y=[Y,yc] & Z=[Z,zc]
;                                       curato=curato+1 & endif
;           endelse
;    endwhile
;    if curato gt 0 then begin
;     wn=curato & W=W[1:wn] & X=X[1:wn] & Y=Y[1:wn] & Z=Z[1:wn] & offs=0
;     mk=3 & pv=fltarr(4+mk*3*2+mk+12,1.*wn*wn) & Vmk=fltarr(3*mk)
;     cri3=fltarr(9)
;     
;     for i=0L,wn-1 do begin
;       ok=1
;       while ok do begin & readf,u,line  & if strpos(line,'Frequencies --') ge 0 then ok=0 & endwhile
;       line=strmid(line,15,80) & reads,line,xi,yi,zi
;       pv[4+mk*3*2+0,i*wn:i*wn+wn-1]=xi/33.36 ;Cm-1 -> THz
;       pv[4+mk*3*2+1,i*wn:i*wn+wn-1]=yi/33.36
;       pv[4+mk*3*2+2,i*wn:i*wn+wn-1]=zi/33.36
;       ok=1
;       while ok do begin & readf,u,line  & if strpos(line,'Atom AN') ge 0 then ok=0 & endwhile
;       for j=0L,wn-1 do begin
;         readf,u,line & reads,line+' 0',bid & if bid eq 0 then readf,u,line
;         line=strmid(line,9,120) & reads,line,cri3 & jj=j+i*wn
;           pv[4+0*3:4+0*3+2  ,jj]=cri3[0:2] ; XYZ Real mode1
;           pv[4+1*3:4+1*3+2  ,jj]=cri3[3:5] ; XYZ Real mode2
;           pv[4+2*3:4+2*3+2  ,jj]=cri3[6:8] ; XYZ Real mode3
;       endfor
;     endfor
;    endif
;        end
        
'.mkl': begin & csv=3 & bnd=1 & a=1. & b=1. & c=1. & offs=1      ;** Molekel (vibrations) a,b,c,angles unknown TO BE DONE BY SR
        readf,u,line & wt=strtrim(line,2)
        ok=1
        while ok do begin & readf,u,line  & if strpos(line,'$COORD') ge 0 then ok=0 & endwhile
        ok=1
        while ok do begin
          readf,u,line
          if strpos(line,'$END') ge 0 then begin
            ok=0
          endif else begin
         reads,line, typ,xc,yc,zc & W=[W,typ] & X=[X,xc] & Y=[Y,yc] & Z=[Z,zc] & curato=curato+1
         endelse 
        endwhile
        ok=1 & bid=0 & typ=0

    if curato gt 0 then begin
      wn=curato & W=W[1:wn] & X=X[1:wn] & Y=Y[1:wn] & Z=Z[1:wn] & offs=0
      mk=3 & pv=fltarr(4+mk*3*2+mk+12,1.*wn*wn) & Vmk=fltarr(3*mk)
      cri3=fltarr(9)
      
      while ok do begin & readf,u,line  & if strpos(line,'$FREQ') ge 0 then ok=0 & endwhile

      for i=0L,wn-1 do begin

        readf,u,line ; read the X    X    X line...
        readf,u,line & reads,line,xi,yi,zi
        pv[4+mk*3*2+0,i*wn:i*wn+wn-1]=xi/33.36 ;Cm-1 -> THz
        pv[4+mk*3*2+1,i*wn:i*wn+wn-1]=yi/33.36
        pv[4+mk*3*2+2,i*wn:i*wn+wn-1]=zi/33.36
;        ok=1
;        while ok do begin & readf,u,line  & if strpos(line,'Atom AN') ge 0 then ok=0 & endwhile
        for j=0L,wn-1 do begin
          readf,u,line & reads,line,cri3 & jj=j+i*wn
          pv[4+0*3:4+0*3+2  ,jj]=cri3[0:2] ; XYZ Real mode1
          pv[4+1*3:4+1*3+2  ,jj]=cri3[3:5] ; XYZ Real mode2
          pv[4+2*3:4+2*3+2  ,jj]=cri3[6:8] ; XYZ Real mode3
        endfor
      endfor
    endif
  end
        

'.gout': begin & csv=3 & bnd=1 & offs=0 ;** Gulp (vibrations) ~= .g94 , .d07
    Aprim=fltarr(3,3)
    while strpos(line,'Formula =') lt 0 do readf,u,line
    wt=strtrim(strmid(line,strpos(line,'=')+1,80),2) ;TITLE
    
    while strpos(line,'irreducible atoms')  lt 0 do readf,u,line & irrATO=1 & reads,strmid(line,strpos(line,'=')+1,10),irrATO
    
    while strpos(line,'Total number atoms') lt 0 do readf,u,line & totATO=1 & reads,strmid(line,strpos(line,'=')+1,10),totATO

    while strpos(line,'Cartesian lattice vectors')   lt 0 do readf,u,line ; CELL MATRIX
    readf,u,line & readf,u,Aprim

    while strpos(line,'Primitive cell parameters :') lt 0 do readf,u,line ; CELL PARAMETERS
    readf,u,line
   ;READ CELL PARAMETERS  PRIM and FULL
    readf,u,line & aP=strpos(line,'a =')       +3 & alphaP=strpos(line,'alpha =',aP)+7
                   aF=strpos(line,'a =',alphaP)+3 & alphaF=strpos(line,'alpha =',aF)+7
                   reads,strmid(line,aP,20),a     & reads,strmid(line,alphaP,20),alpha & aP=a & alphaP=alpha
                   reads,strmid(line,aF,20),a     & reads,strmid(line,alphaF,20),alpha & aF=a & alphaF=alpha
    readf,u,line & bP=strpos(line,'b =')       +3 & betaP =strpos(line,'beta  =',bP)+7
                   bF=strpos(line,'b =',betaP) +3 & betaF =strpos(line,'beta  =',bF)+7
                   reads,strmid(line,bP,20),b     & reads,strmid(line,betaP, 20),beta  & bP=b & betaP =beta
                   reads,strmid(line,bF,20),b     & reads,strmid(line,betaF, 20),beta  & bF=b & betaF =beta
    readf,u,line & cP=strpos(line,'c =')       +3 & gammaP=strpos(line,'gamma =',cP)+7
                   cF=strpos(line,'c =',gammaP)+3 & gammaF=strpos(line,'gamma =',cF)+7
                   reads,strmid(line,cP,20),c     & reads,strmid(line,gammaP,20),gamma & cP=c & gammaP=gamma   
                   reads,strmid(line,cF,20),c     & reads,strmid(line,gammaF,20),gamma & cF=c & gammaF=gamma   

    A=aP & B=bP & C=cp & ALPHA=alphaP & BETA=betaP & gamma=gammaP ;KEEP PRIMITIVE CELL

    while strpos(line,'No.  Atomic') lt 0 do readf,u,line
    readf,u,line
    readf,u,line
    
    vfil=strtrim(bfil,2) & vfil=strmid(vfil,0, strpos(/reverse_search,vfil,'.'))+'.cif'
    vfil=finefile(path+vfil,count=nc)
    symb='' & X=0. & Y=0. & Z=0. & PV=pv0
    if (irrATO ne totATO) and (nc gt 0) then begin ;read the structure from CIF  file
      OPENR,v,vfil,/get_lun
      while             strpos(line,'_atom_site_fract_z')    lt 0 do   readf,v,line &    ipos=9
      readf,v,line & if strpos(line,'_atom_site_occupancy')  ge 0 then readf,v,line else ipos=4
      on_ioerror,endCIF
      while (1) do if line gt ' ' then begin
        atom=strcompress(strmid(line,0,ipos),/remove_all) & reads,strmid(line,ipos,80),xyz
        atom_split,atom,num & symb=[symb,atom] & X=[X,xyz[0]] & Y=[Y,xyz[1]] & Z=[Z,xyz[2]] & PV=[[PV],[pv0]]
        readf,v,line
      endif
      endCIF: if v gt 0 then free_lun,v
      on_ioerror,misread
    endif else begin                               ;read the structure from GULP file
      irrXYZ=fltarr(irrATO) & irrTYP=fltarr(irrATO)
      for i=0,irrATO-1 do begin
        readf,u,line & atom=strcompress(strmid(line,8,6),/remove_all) & strline=strmid(line,17,80)
        if strpos(strline,'*') gt 0 then begin
           strline=strsplit(/EXTRACT,strline,'*') & xyz[0:2]=float(strline[0:2])
        endif else reads,strline,xyz
        atom_split,atom,num & symb=[symb,atom] & X=[X,xyz[0]] & Y=[Y,xyz[1]] & Z=[Z,xyz[2]] & PV=[[PV],[pv0]]
      endfor
    endelse
    if n_elements(X) gt 1 then begin & symb=symb[1:*] & X=X[1:*] & Y=Y[1:*] & Z=Z[1:*] & PV=PV[*,1:*]
     W=d_livSymbtoType(symb) & endif
    WN=n_elements(W)

    if totATO eq WN then begin ; GET PHONONS

    while strpos(line,'Eigenvectors :')     lt 0 do readf,u,line ; (cm-1) VECTOR POSITIONEMENT
    nml  =3*2L         ;IF 6 modes/line then ...
    frame=WN/(nml/3)
    if frame*(nml/3) ne WN then frame=frame+1
    Lfreq=strarr(frame)       ;1 line nml freq * frame
    Lvec =strarr(3L*WN,frame) ;3 lines xyz * WN atoms * frame
    tempV=strarr(3L*WN)       ;3 lines xyz * WN buffer

    for i=0L,frame-1 do begin ;READ ALL in this loop
      while strpos(line,'Frequency')          lt 0 do readf,u,line & Lfreq[i]=line
      while strpos(line,'Raman Intsty')       lt 0 do readf,u,line & readf,u, line
      readf,u,tempV & Lvec[0,i]=tempV
    endfor
    ;WE SUPPOSE LAST FRAME IS COMPLETED BY ZEROS !!!
    
    Lfreq=strmid(Lfreq,11,100) & Lvec=strmid(reform(Lvec,n_elements(Lvec)),9,100)
    DcripF=fltarr(3L*WN)    & reads,Lfreq,DcripF  ; HAVE FREQUENCIES (Cm-1)
    DcripF=DcripF/33.36                           ; Now THz
    DcripV=fltarr(9L*WN*WN) & reads,Lvec ,DcripV  ; HAVE VECTORS
    mk=3 & pv=fltarr(4+mk*3*2+mk+12,1.*wn*wn) & Vmk=fltarr(3*mk)

    for i=0L,WN-1 do begin       ;Split frequencies
      pv[4+mk*3*2+0,i*wn:i*wn+wn-1]=DcripF[i*3]
      pv[4+mk*3*2+1,i*wn:i*wn+wn-1]=DcripF[i*3+1]
      pv[4+mk*3*2+2,i*wn:i*wn+wn-1]=DcripF[i*3+2]
    endfor
    pas=nml/3
    kk =0
    for i=0L,WN-pas,pas do begin  ;Split vectors
      for j=0L,WN-1 do begin &  jj=j+i*wn
        for k=0L,pas-1 do begin & kkk=kk+3*k
           pv[4+0*3  ,jj+k*WN]=DcripV[kkk        ] ; X Real mode1
           pv[4+1*3  ,jj+k*WN]=DcripV[kkk      +1] ; X Real mode2
           pv[4+2*3  ,jj+k*WN]=DcripV[kkk      +2] ; X Real mode3
           pv[4+0*3+1,jj+k*WN]=DcripV[kkk+nml    ] ;   Y Real mode1
           pv[4+1*3+1,jj+k*WN]=DcripV[kkk+nml  +1] ;   Y Real mode2
           pv[4+2*3+1,jj+k*WN]=DcripV[kkk+nml  +2] ;   Y Real mode3
           pv[4+0*3+2,jj+k*WN]=DcripV[kkk+nml*2  ] ;     Z Real mode1
           pv[4+1*3+2,jj+k*WN]=DcripV[kkk+nml*2+1] ;     Z Real mode2
           pv[4+2*3+2,jj+k*WN]=DcripV[kkk+nml*2+2] ;     Z Real mode3
        endfor
        kk=kk+9*pas
      endfor
    endfor
    endif ; GETTING PHONONS  
    end
    
'.cif': begin & BND=1 & OFFS=0
    WT='Unknown'
    while strpos(line,'_cell_length_a')     lt 0 do readf,u,line & reads,strmid(line,15,80),A
    while strpos(line,'_cell_length_b')     lt 0 do readf,u,line & reads,strmid(line,15,80),B
    while strpos(line,'_cell_length_c')     lt 0 do readf,u,line & reads,strmid(line,15,80),C
    while strpos(line,'_cell_angle_alpha')  lt 0 do readf,u,line & reads,strmid(line,18,80),ALPHA
    while strpos(line,'_cell_angle_beta')   lt 0 do readf,u,line & reads,strmid(line,18,80),BETA
    while strpos(line,'_cell_angle_gamma')  lt 0 do readf,u,line & reads,strmid(line,18,80),GAMMA

    while             strpos(line,'_atom_site_fract_z')    lt 0 do   readf,u,line &    ipos=9
    readf,u,line & if strpos(line,'_atom_site_occupancy')  ge 0 then readf,u,line else ipos=4
    symb='' & X=0. & Y=0. & Z=0. & PV=pv0
    on_ioerror,endline
    while (1) do if line gt ' ' then begin
      atom=strcompress(strmid(line,0,ipos),/remove_all) & reads,strmid(line,ipos,80),xyz
      atom_split,atom,num & symb=[symb,atom] & X=[X,xyz[0]] & Y=[Y,xyz[1]] & Z=[Z,xyz[2]] & PV=[[PV],[pv0]]
      readf,u,line
    endif
    endline:
    if n_elements(X) gt 1 then begin & symb=symb[1:*] & X=X[1:*] & Y=Y[1:*] & Z=Z[1:*] & PV=PV[*,1:*]
     W=d_livSymbtoType(symb) & endif
    end
    
'.d07': begin & csv=3 & bnd=1 & offs=1 ;** Phonon (vibrations) ~= .g94
    ;
    b2fil=STRMID(bfil,0,strlen(bfil)-4)+'.d46'
    openr,u_d46,path+b2fil,/get_lun,ERROR=err_d46
    IF (err_d46 NE 0) then lamp_message,'.d46 file not found: will look for eigenvect. in .d07 file (PHONON v4)'
    IF (err_d46 EQ 0) then lamp_message,'.d46 file found: will look for eigenvect. in .d46 file (PHONON v5)'
    
    ;
    readf,u,line & readf,u,line & readf,u,line
    wt=strtrim(strmid(line,strpos(line,':')+1,80),2)
    ok=1
    while ok do begin & readf,u,line  & if strpos(line,'Lattice constants')    ge 0 then ok=0 & endwhile
    readf,u,line & reads,line,a,b,c & readf,u,line
    readf,u,line & reads,line,alpha,beta,gamma
    readf,u,line & UC=strtrim(strmid(line,strpos(line,':')+1,80),2) ; Unit Cell (spgr) to use with Acrys
    ok=1
    while ok do begin & readf,u,line  & if strpos(line,'Basic matrix')         ge 0 then ok=0 & endwhile
    readf,u,line & Acrys=fltarr(3,3) & Aprim=Acrys & Avec=fltarr(3) ; Not in used
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Acrys[*,0]=Avec
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Acrys[*,1]=Avec
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Acrys[*,2]=Avec
    readf,u,line
    readf,u,line ; READ SECOND MATRIX (Aprim)
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Aprim[*,0]=Avec
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Aprim[*,1]=Avec
    readf,u,line & line =strmid(line,strpos(line,'|')+1,60) & reads,line,Avec & Aprim[*,2]=Avec
    Aprim=transpose(Aprim)
    pcell=Mat_to_Pcell(Aprim,1.)
    a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
    ok=1
    while ok do begin & readf,u,line  & if strpos(line,'DISPLACIVE PARTICLES') ge 0 then ok=0 & endwhile
        readf,u,line & line=strmid(line,strpos(line,':')+1,80) & non_equi=0 & reads,line,non_equi
    ok=1
    while ok do begin & readf,u,line  & if strpos(line,'DISPLACIVE PARTICLES') ge 0 then ok=0 & endwhile
        readf,u,line & wn=0. & line=strmid(line,strpos(line,':')+1,80) & reads,line,wn
    offs=0 & equi=0
    if wn gt 0 then begin
     w=lonarr(wn) & x=fltarr(wn) & y=x & z=x & mk=3 & pv=fltarr(4+mk*3*2+mk+12,1.*wn*wn) & Vmk=fltarr(3*mk)
     for i=1L,non_equi do begin
       ok=1
       while ok do begin & readf,u,line  & if strpos(line,' equivalent site')  ge 0 then ok=0 & endwhile
       line=strmid(line,strpos(line,':')+1,80) & reads,line,equi
       ok=1
       while ok do begin & readf,u,line  & if strpos(line,'Name       X    ')  ge 0 then ok=0 & endwhile
      ;ATOMIC POSITIONS
       for j=1L,equi do begin
         readf,u,line & line=strmid(line,9,80) & reads,line,xyzP
         x[curato]=xyzP[3] & y[curato]=xyzP[4] & z[curato]=xyzP[5] & curato=curato+1
       endfor
       
       nmass=0.
       readf,u,line   & line=strmid(line,9,80) & reads,line,nmass & n=d_livMasstoType(nmass)
       for j=1L,equi do  w[curato-j]=n
     endfor
     cri3=fltarr(6)
;
CASE (err_d46 EQ 0) of
  1:begin ; read in the .d46 file
    tmp=fltarr(6*wn+1,3*wn)
       ok=1
       readf,u_d46,line
       readf,u_d46,line & line=strsplit(line,/extract) & n_d46=fix(line[0])
       if ABS(n_d46 - 3*wn) GT 0.5 then begin & lamp_error,'Problem: dimension in the .d07 and .d46 files do not match... exiting' & goto, s_exit & endif
       line ='' & readf,u_d46,line ; number of k points
       line ='' & readf,u_d46,line & line=strsplit(line,/extract) ; q value and label of first k point
       wt=wt+' Pol. vectors at '+ line[1] & line=''
       while ok do begin & readf,u_d46,line  & if strpos(line,'Omega') ge 0 then ok=0 & endwhile
       readf,u_d46,line & line=strsplit(line,/extract)
       wt=wt+' ('+line[1]+','+line[2]+','+line[3]+')'
       line=''
       readf,u_d46,tmp
     for i=0L,wn-1 do begin       
;       line=strmid(line,9,80) & reads,line,xi,yi,zi ; read the frequencies
       xi=tmp[0,i*3] & yi=tmp[0,i*3+1] & zi=tmp[0,i*3+2]
       pv[4+mk*3*2+0,i*wn:i*wn+wn-1]=xi
       pv[4+mk*3*2+1,i*wn:i*wn+wn-1]=yi
       pv[4+mk*3*2+2,i*wn:i*wn+wn-1]=zi
       for j=0L,wn-1 do begin &  jj=j+i*wn
;         readf,u,line & line=strmid(line,13,80) & reads,line,cri3
           cri3=[tmp[j*3+1,i*3:i*3+2],tmp[3*wn+j*3+1,i*3:i*3+2]]
           pv[4+0*3  ,jj]=cri3[0] & pv[4+mk*3+0*3  ,jj]=cri3[1] ; X Real & Imag mode1
           pv[4+1*3  ,jj]=cri3[2] & pv[4+mk*3+1*3  ,jj]=cri3[3] ; X Real & Imag mode2
           pv[4+2*3  ,jj]=cri3[4] & pv[4+mk*3+2*3  ,jj]=cri3[5] ; X Real & Imag mode3
           cri3=[tmp[j*3+2,i*3:i*3+2],tmp[3*wn+j*3+2,i*3:i*3+2]]
           pv[4+0*3+1,jj]=cri3[0] & pv[4+mk*3+0*3+1,jj]=cri3[1] ; Y Real & Imag mode1
           pv[4+1*3+1,jj]=cri3[2] & pv[4+mk*3+1*3+1,jj]=cri3[3] ; Y Real & Imag mode2
           pv[4+2*3+1,jj]=cri3[4] & pv[4+mk*3+2*3+1,jj]=cri3[5] ; Y Real & Imag mode3
           cri3=[tmp[j*3+3,i*3:i*3+2],tmp[3*wn+j*3+3,i*3:i*3+2]]
           pv[4+0*3+2,jj]=cri3[0] & pv[4+mk*3+0*3+2,jj]=cri3[1] ; Z Real & Imag mode1
           pv[4+1*3+2,jj]=cri3[2] & pv[4+mk*3+1*3+2,jj]=cri3[3] ; Z Real & Imag mode2
           pv[4+2*3+2,jj]=cri3[4] & pv[4+mk*3+2*3+2,jj]=cri3[5] ; Z Real & Imag mode3
       endfor
     endfor
       end
  ELSE:begin ; read in the .d07 file
     for i=0L,wn-1 do begin
       ok=1
       while ok do begin & readf,u,line  & if strpos(line,'Omega:') ge 0 then ok=0 & endwhile
       line=strmid(line,9,80) & reads,line,xi,yi,zi ; read the frequencies
       pv[4+mk*3*2+0,i*wn:i*wn+wn-1]=xi
       pv[4+mk*3*2+1,i*wn:i*wn+wn-1]=yi
       pv[4+mk*3*2+2,i*wn:i*wn+wn-1]=zi
       for j=0L,wn-1 do begin &  jj=j+i*wn
         readf,u,line & line=strmid(line,13,80) & reads,line,cri3
           pv[4+0*3  ,jj]=cri3[0] & pv[4+mk*3+0*3  ,jj]=cri3[1] ; X Real & Imag mode1
           pv[4+1*3  ,jj]=cri3[2] & pv[4+mk*3+1*3  ,jj]=cri3[3] ; X Real & Imag mode2
           pv[4+2*3  ,jj]=cri3[4] & pv[4+mk*3+2*3  ,jj]=cri3[5] ; X Real & Imag mode3
         readf,u,line & line=strmid(line,13,80) & reads,line,cri3
           pv[4+0*3+1,jj]=cri3[0] & pv[4+mk*3+0*3+1,jj]=cri3[1] ; Y Real & Imag mode1
           pv[4+1*3+1,jj]=cri3[2] & pv[4+mk*3+1*3+1,jj]=cri3[3] ; Y Real & Imag mode2
           pv[4+2*3+1,jj]=cri3[4] & pv[4+mk*3+2*3+1,jj]=cri3[5] ; Y Real & Imag mode3
         readf,u,line & line=strmid(line,13,80) & reads,line,cri3
           pv[4+0*3+2,jj]=cri3[0] & pv[4+mk*3+0*3+2,jj]=cri3[1] ; Z Real & Imag mode1
           pv[4+1*3+2,jj]=cri3[2] & pv[4+mk*3+1*3+2,jj]=cri3[3] ; Z Real & Imag mode2
           pv[4+2*3+2,jj]=cri3[4] & pv[4+mk*3+2*3+2,jj]=cri3[5] ; Z Real & Imag mode3
       endfor
     endfor
     end
   ENDCASE
    endif
s_exit:
    end

'.atm': begin & offs=1 ;** FullProf
    WHILE not eof(u) do begin
    readf,u,line & line=strtrim(line,2) & lin0=line
    head=strlowcase(strmid(line,0,4))   & line=strmid(line,5,120)
    CASE head of
    ''    :
    ' '   :
    'titl': wt=line

    'cell': reads,line,a,b,c, alpha,beta,gamma

    'spgr': begin & spgr=strtrim(line,2)    & spgr=strupcase(strmid(spgr,0,1))
                  case spgr of 'P':spgr=1 & 'C':spgr=2 & 'A':spgr=3 & 'B':spgr=4 & 'I':spgr=5 & 'F':spgr=6 & 'R':spgr=7 & else:spgr=1 & endcase
            end
    'jbtv': begin & reads,line,ifo & if ifo ne 1 then ifo=2 & end

    'veck': begin & reads,line,mk & mk=mk>1 & Vmk=fltarr(3*mk) & pv0=fltarr(4+mk*3*2+mk+12) & pv=pv0 & end

    'atom': begin & line=strtrim(line,2) & idx=where(byte(line) le 32) & idx=idx[0]
                  name=strmid(line,0,idx) & line=strmid(line,idx,110)
                  atom_split,name,num
                  type=d_livSymbtoType([name])
                  if (type gt 0) then begin & reads,line,xc,yc,zc & w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]] & endif
                  curato=n_elements(w)-1 & end

    'mk_p': begin & num=0 & reads,line,num,xc,yc,zc, xi,yi,zi, ph & num=num-1
                  pv[4+num*3:4+num*3+2,curato]=[xc,yc,zc] & pv[4+mk*3+num*3:4+mk*3+num*3+2,curato]=[xi,yi,zi]
                  pv[4+mk*3*2+num,curato]=ph
                  end

    else:   if strpos(head,'vk_') eq 0 then begin & line=strmid(lin0,3,100) & reads,line,num,xc,yc,zc & num=num-1
               Vmk[num*3:num*3+2]=[xc,yc,zc]
            endif else begin
               idx=strpos(head,'_')
               if (idx ge 2) then begin
                 name=strmid(head,0,idx-1)    & atom_split,name,num & type=d_livSymbtoType([name])
                 idx =where(byte(lin0) le 32) & idx=idx[0]
                 line=strmid(line,idx,110)    & reads,line,xc,yc,zc & w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]]
                 curato=n_elements(w)-1
               endif
            endelse
    ENDCASE
    ENDWHILE
    end

'.sum': begin & offs=1
    WHILE not eof(u) do begin
    readf,u,line
    if strpos(line,' =') eq 0 then begin & head=strmid(line,1,10)
    CASE head of
    '=> Title: ': wt=strmid(line,11,80)

    '=> Phase N': reads,strmid(line,14,2),phase

    '==> ATOM P': begin & readf,u,line & readf,u,line & readf,u,line & readf,u,line & line=strtrim(line,0)
                        while line gt ' ' do begin
                           name=strtrim(strmid(line,0,6),2) & line=strmid(line,6,80) & i=strpos(line,'(')
                           while i ge 0 do begin & strput,line,' ',i & i=strpos(line,'(') & endwhile & i=strpos(line,')')
                           while i ge 0 do begin & strput,line,' ',i & i=strpos(line,')') & endwhile
                           reads,line,xc,ec,yc,ec,zc
                           atom_split,name,num
                           type=d_livSymbtoType([name])
                           if (type gt 0) and (num eq 0) then begin & w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]] & endif
                           readf,u,line & line=strtrim(line,0)
                        endwhile
                  end
    '=> Cell pa': readf,u,a,ec,b,ec,c,ec, alpha,ec,beta,ec,gamma

    '==> MAGNET': begin & readf,u,line & readf,u,line & carte=0 & spher=0 & curato=0
                        if strpos(line,'Phi') gt 0 then begin &  spher=1 & readf,u,line & endif
                        if strpos(line,'Mx')  gt 0 then begin &  carte=1 & readf,u,line & endif
                        readf,u,line & line=strtrim(line,0)
                        while line gt ' ' do begin
                           name=strtrim(strmid(line,0,6),2) & line=strmid(line,6,80) & i=strpos(line,'(')
                           while i ge 0 do begin & strput,line,' ',i & i=strpos(line,'(') & endwhile & i=strpos(line,')')
                           while i ge 0 do begin & strput,line,' ',i & i=strpos(line,')') & endwhile
                           atom_split,name,num
                           type=d_livSymbtoType([name])
                           if (type gt 0) and (num gt 0) then begin
                               idx=where(w eq type) & curato=idx[num-1]
                               if spher then reads,line,modul,ec,phi,ec,theta,ec,ph $
                                        else reads,line,xc,ec,yc,ec,zc,ec,modul,ec,ph
                           endif
                           if strpos(line,'Imag') then begin & endif
                           readf,u,line & line=strtrim(line,0)
                        endwhile
                  end
    '=> Propaga': begin & readf,u,line & readf,u,line & line=strtrim(line,0) & num=0
                        while line gt ' ' do begin
                           name=strtrim(strmid(line,0,5),2)
                           if name gt ' ' then reads,line,num,xc,yc,zc
                           readf,u,line & line=strtrim(line,0)
                        endwhile
                  end
    else:
    ENDCASE
    endif
    ENDWHILE
    end

'.pcr': begin & readf,u,line & wt=strtrim(strmid(line,5,80),2) & offs=1
    WHILE not eof(u) do begin
    readf,u,line
    if strpos(line,'!') eq 0 then begin & head=strmid(line,0,10)
    CASE head of
    '!  Data fo': reads,strmid(line,26,4),phase

    '!Atom Typ ': begin & carte=0 & spher=0 & curato=0
                        if strpos(line,'Rx')    gt 0 then begin & carte=1 & readf,u,line & endif
                        if strpos(line,'Rphi')  gt 0 then begin & spher=1 & readf,u,line & endif
                        readf,u,line
                        while strpos(line ,'!') ne 0 do begin
                           name=strtrim(strmid(line,0,5),2) & line=strmid(line,11,80)
                           atom_split,name,num
                           type=d_livSymbtoType([name])
                           if type gt 0 then begin
                              if carte+spher eq 0 then begin & reads,line,xc,yc,zc  & readf,u,line ;Bid
                                      if num eq 0 then begin & w=[w,type] & x=[x,xc] & y=[y,yc] & z=[z,zc] & pv=[[pv],[pv0]] & endif
                              endif else begin
                                      idx=where(w eq type) & curato=idx[num-1]
                                      if carte then reads,line,ec,ec,ec,ec,ec,ec,ec,xc,yc,zc $
                                               else reads,line,ec,ec,ec,ec,ec,ec,ec,modul,phi,theta
                                      readf,u,line ;Bid
                                      readf,u,line ;Imaginary component
                                      if carte then reads,line,xi,yi,zi,ec,ec,ec,ph $
                                               else reads,line,imodul,iphi,itheta,ec,ec,ec,ph
                                      readf,u,line ;Bid
                              endelse
                           endif
                           readf,u,line
                        endwhile
                  end
    '!     a   ': readf,u,a,b,c, alpha,beta,gamma

    '! Propagat': begin & readf,u,line & num=0
                        while strpos(line,'Propagation') gt 0 do begin & num=num+1
                           reads,line,xc,yc,zc
                           readf,u,line ;Bid
                           readf,u,line
                        endwhile
                  end
    else:
    ENDCASE
    endif
    ENDWHILE
    end
else:   status=14
endcase

misread:if u gt 0 then free_lun,u & u=-1

wn=n_elements(w)
if (wn gt 1) or (w[0] gt 1) then begin
 if (offs) then begin & w=w[1:wn-1] & x=x[1:wn-1] & y=y[1:wn-1] & z=z[1:wn-1] & pv=pv[*,1:wn-1] & wn=wn-1 & endif

;SYMETRY atoms & moments      symkt=[[index atom, K_n, msym_n]] & symk=nb of msysm for each symm
;***********************
 nAsym=n_elements(symm)/3
 if nAsym gt 0 then begin & if (symm[0,0] eq 'x') and (symm[1,0] eq 'y') and (symm[2,0] eq 'z') then nA1=1 else nA1=0
    XX=X & YY=Y & ZZ=Z & WW=W & WWW=W & PVPV=PV & PVV=PV & tmp=0.

    for i =na1,nAsym-1 do begin
        ii=execute('tmp='+symm[0,i]) & XX=[XX,round(tmp*1000.)/1000.]
        ii=execute('tmp='+symm[1,i]) & YY=[YY,round(tmp*1000.)/1000.]
        ii=execute('tmp='+symm[2,i]) & ZZ=[ZZ,round(tmp*1000.)/1000.]
        WW =[WW,WWW]
        if i gt 0 then s=total(symk[0:i-1]) else s=0       ;MSYM OFFSET
        nMsym=symk[i]
        for j=0,nMsym-1 do begin
         idk=where(symkt[2,*] eq j+1) & cursym=symkt[*,idk];ATOMS CONCERED BY THIS MSYM
         for k=0,mk-1 do begin
           idk=where(cursym[1,*] eq k+1)                   ;ATOMS CONCERNS BY THIS K VECTOR
           if idk[0] ge 0 then begin
            idk=reform(cursym[0,idk])
            idx=k*3 & idx=[4+idx,4+mk*3+idx]               ;real + imaginary index
            U  =PVPV[idx,*] & U=U[*,idk] & ii=execute('tmp='+msym[0,j+s]) & PVV[idx[0],idk]=tmp[0,*] & PVV[idx[1],idk]=tmp[1,*] & idx=idx+1
            V  =PVPV[idx,*] & V=V[*,idk] & ii=execute('tmp='+msym[1,j+s]) & PVV[idx[0],idk]=tmp[0,*] & PVV[idx[1],idk]=tmp[1,*] & idx=idx+1
            W  =PVPV[idx,*] & W=W[*,idk] & ii=execute('tmp='+msym[2,j+s]) & PVV[idx[0],idk]=tmp[0,*] & PVV[idx[1],idk]=tmp[1,*]
            PVV[4+mk*3*2:4+mk*3*2+mk-1,idk] =float(msym[3,j+s]) ; phase
           endif
         endfor
        endfor
        PV=[[PV],[PVV]]
    endfor
    X=XX & Y=YY & Z=ZZ & W=WW & wn=n_elements(W) ;FRACTIONAL ->
    idx=where(x lt 0.) & if idx[0] ge 0 then x[idx]=(x[idx]+1.)>0. & idx=where(x gt 1.) & if idx[0] ge 0 then x[idx]=(x[idx]-1.)<1.
    idx=where(y lt 0.) & if idx[0] ge 0 then y[idx]=(y[idx]+1.)>0. & idx=where(y gt 1.) & if idx[0] ge 0 then y[idx]=(y[idx]-1.)<1.
    idx=where(z lt 0.) & if idx[0] ge 0 then z[idx]=(z[idx]+1.)>0. & idx=where(z gt 1.) & if idx[0] ge 0 then z[idx]=(z[idx]-1.)<1.
   ;Suppress identical symetries
    for i=0,wn-2 do begin
        idx=where((x eq x[i]) and (y eq y[i]) and (z eq z[i]))
        if n_elements(idx) gt 1 then w[idx[1:*]]=-1
    endfor
    idx=where(w ne -1) & w=w[idx] & x=x[idx] & y=y[idx] & z=z[idx] & pv=pv[*,idx] & wn=n_elements(w)
 endif
 
 if  n_elements(MATsym) eq 9 then begin ;Prim to Cryst matrice transformation
     Aprim=invert(PCell_To_Inv_Mat( a, b, c, alpha, beta, gamma))
     prim_to_crist_unitcell,Aprim,Matsym,w,x,y,z,pv,mk
     pcell=Mat_to_Pcell(Aprim,1.) 
     a=pcell[0] & b=pcell[1] & c=pcell[2] & alpha=pcell[3] & beta=pcell[4] & gamma=pcell[5]
     wn=n_elements(x)
 endif

 p=[1.,a0,a,b0,b,c0,c,alpha,beta,gamma,wn,1,1,1, csv,1, spgr, ifo,bnd,0,p20,0,0,0,0, mk,Vmk]
 ptxt=['OBJECT: '  $
     ,'coord a0' ,'coord a1'  $
     ,'coord b0' ,'coord b1'  $
     ,'coord c0' ,'coord c1'  $
     ,'alpha   ' ,'beta    ' ,'gamma   '  $
     ,'nb atoms'  $
     ,'# cells along a:','# cells along b:','# cells along c:' $
     ,'vectors input: 1:M.cartesian 2:M.spherical 3:Vibration' $
     ,'modulation mode:  1:phase 2:amplitude' $
     ,'translation P:1 C:2 A:3 B:4 I:5 F:6 Ro:7 Rv:8 H:9' $
     ,'Fourier coef (position):  1:cell 2:atom' $
     ,'Bounds: 0:given 1:to build' $
     ,'Maximum arrow/atom radius ratio' $
     ,'Do not extend molecule outside cell' $
     ,'free','free' $
     ,'free','free' $
     ,'nb propagations/modes on X,Y,Z'  $
     , strarr(mk*3) $
     ]
 pv[0,*]=1. & pv[2,*]=-1. & pv[3,*]=1. & pv[1,where(pv[1,*] ne .0001)]=width
 status=0
 datp={X:x, Y:y, Z:z, E:e, N:n, P:p, PV:pv, PAR_TXT:ptxt, W_TIT:wt, X_TIT:xt, Y_TIT:yt, Z_TIT:zt, OTHER_TIT:ot}
endif
return,w
end
