function rd_mari, INST , PATH , FILENAME , STATUS , DATP
;+
;******* ********
;**
;**  Read mari spe files into LAMP.
;**
;**  Return of the function
;**   DATA     is an array of any dimensions and type containing the data values (spectra).
;**
;**  Input  parameters:
;**   INST(0)  is the file_type  (or instrument_name ) (string defined in customize tables).
;**   INST(1)  is the file_group (or instrument_group) (string defined in customize tables).
;**   INST(2)  is '1' if raw  button is set.
;**   PATH     is the full path where to find the data (string defined in customize tables).
;**   FILENAME is the name of the data file.
;**             if FILENAME(1) exists, this is the requested image number in the file
;**                                    entered as  file.ext{3} (for image 3).
;**  Output parameters:
;**   STATUS   is the returned error code you can choose from the following list:
;**           0 =' Successfull read'
;**           1 =' Client/server on local  node not established'
;**           2 =' Client/server on router node not established'
;**           3 =' The local  node cannot access the server node'
;**           4 =' The router node cannot access the server node'
;**           5 =' VME memory read error'
;**           7 =' Sequence error in data transfer'
;**           9 =' Parameter error'
;**           10=' Router is busy with other transfer'
;**           11=' Cant open the file or file not found'
;**           13=' Data file incomplete'
;**           14=' Bad instrument data definition'
;**           24=' Cant read the file'.
;**
;**   DATP     is a structure defined as follow: (all tags are OPTIONAL)
;**           DATP.X        = vector of x coordinates.
;**           DATP.Y        = vector of y coordinates.
;**           DATP.Z        = vector of z coordinates.
;**           DATP.W_TIT    =   main title
;**           DATP.X_TIT    = x axis title
;**           DATP.Y_TIT    = y axis title
;**           DATP.Z_TIT    = z axis title
;**           DATP.OTHER_TIT=    sub title
;**           DATP.N        = monitors
;**           DATP.P        = vector of parameter values, max dim is defined in customize.
;**           DATP.PAR_TXT  = string array of text associated to DATP.P (same size)
;**           DATP.PV       = an array of any dimensions containing other parameter values
;**           DATP.E        = the errors associated to DATA (same size)
;**           DATP.TIME     = string date of the experiment.
;-

 DATA  =0
 STATUS=11
 CATCH,stat & if stat ne 0 then begin & print,!error_state.msg & RETURN, DATA & endif

 ON_IOERROR, no_file
 OPENR,unit, PATH+FILENAME,/get_lun                   ;Open the data file      
 ON_IOERROR, read_err
 STATUS=13
 
 nq=0L & nw=0L & txt=' ' & fmt='(8G10.4)'
 READF,unit,nq,nw
 

;set up arrays
 yv   =fltarr(nq)
 xv   =fltarr(nw)
 data =fltarr(nw,nq) & tmpd=fltarr(nw)
 e_out=fltarr(nw,nq)
 p_out=fltarr(30)
 ptxt =strarr(30)
 txt  =' '

;fiddle in the wavelength for S(Q,w) calc
 ei=150.0
 p_out[21]=sqrt(81.8/ei)
 ptxt[21] ='Wavelength'
 ON_IOERROR, eof_err

;first block is just angles
 READF,unit,txt
 READF,unit,yv
;Then get energies
 READF,unit,txt
 READF,unit,xv
 if round(nw/8.) eq nw/8. then READF,unit,txt ;once more
 
;Then blocks of data and errors
 for k=0,nq-1 do begin

;  first data
   READF,unit,txt
   READF,unit,tmpd,format=fmt & data[*,k] =tmpd

;  then errors
   READF,unit,txt
   READF,unit,tmpd,format=fmt & e_out[*,k]=tmpd
 endfor
 eof_err:

; That's it
      STATUS=0                                        ;Status is ok
;     ********
      wt       = FILENAME
      xt       =' Energy (meV) '
      yt       =' Angles (deg) '

      DATP={X:      xv,    $                          ;Pass into the DATP structure
            Y:      yv,    $
            W_TIT:  wt,    $                          ;those variables which were read-in
            X_TIT:  xt,    $
            Y_TIT:  yt,    $
            P:      p_out, $
            PAR_TXT:ptxt,  $
            E:      e_out  }
;     **********************

 read_err:  FREE_LUN,unit                             ;Free the unit number
 no_file:

 RETURN, DATA                                         ;Return the data values
;************
 END
