function rd_x, INST , PATH , FILENAME , STATUS , DATP
;+
;******* ****
;**
;**  Standard call for a data-read function interfacing LAMP.
;**      
;**  Return of the function
;**   DATA     is an array of any dimensions and type containing the data values (spectra).
;**
;**  Input  parameters:
;**   INST(0)  is the file_type  (or instrument_name ) (string defined in customize tables).
;**   INST(1)  is the file_group (or instrument_group) (string defined in customize tables).
;**   PATH     is the full path where to find the data (string defined in customize tables).
;**   FILENAME is the name of the data file.
;**   if FILENAME(1) exists,  this is the requested image number in the file.
;**
;**  Output parameters:
;**   STATUS   is the returned error code you can choose from the following list:
;**      0 =' Successfull read'  
;**      1 =' Client/server on local node not established'
;**      2 =' Client/server on router node not established'
;**      3 =' The local  node cannot access the server node'
;**      4 =' The router node cannot access the server node'
;**      5 =' VME memory read error'
;**      7 =' Sequence error in data transfer'
;**      9 =' Parameter error'
;**      10=' Router is busy with other transfer'
;**      11=' Cant open the file or file not found'
;**      13=' Data file incomplete'
;**      14=' Bad instrument data definition'
;**      24=' Cant read the file'.
;**      
;**   DATP     is a structure defined as follow: (all tags are OPTIONAL)
;**      DATP.X        = vector of x coordinates.
;**      DATP.Y        = vector of y coordinates.
;**      DATP.Z        = vector of z coordinates.
;**      DATP.W_TIT    =   main title
;**      DATP.X_TIT    = x axis title
;**      DATP.Y_TIT    = y axis title
;**      DATP.Z_TIT    = z axis title
;**      DATP.OTHER_TIT=    sub title
;**      DATP.N        = monitors
;**      DATP.P        = vector of parameter values up to 31
;**      DATP.PAR_TXT  = string array of text associated to DATP.P (same size)
;**      DATP.PV       = an array of any dimensions containing other parameter values
;**      DATP.E        = the errors associated to DATA (same size)
;**      DATP.TIME     = string date of the experiment.
;-

common c_edf  , pref, suf

 DATA  =0   &   headerSize=6 & dataLines=8192
 UNIT  =-1  &   YY=0
 STATUS=11
 CATCH,stat &  IF stat ne 0 then begin & print,!error_state.msg
               IF unit gt 0 THEN FREE_LUN,unit & RETURN, DATA & endif

 MYFILE=FILENAME
 MYPATH=PATH
 IF (strpos(MYFILE, '.')  lt 0) and (n_elements(pref) eq 1) then BEGIN
         ON_IOERROR, mis_mor & MYFILE=strtrim(fix(FILENAME),2) & mis_mor:
         if pref gt " " then MYFILE=pref+MYFILE
         if n_elements(suf) eq 1 then if suf  gt " " then MYFILE=MYFILE+suf
         ON_IOERROR, mis_num & YY=fix(FILENAME) & mis_num:
         endif

 ON_IOERROR, no_file
 OPENR,unit, MYPATH+MYFILE,/get_lun   ;Open the data file

      STATUS=13
      ON_IOERROR, read_err
      HEADER =STRARR(headerSize)
      READF,unit,HEADER               ;Read the header
      buf    =fltarr(2,dataLines)
      ON_IOERROR, mis_data
      READF,unit,buf & mis_data:      ;Read the XY data
      DATA   =reform(buf[1,*])
      XX     =reform(buf[0,*])

      STATUS=0                        ;Status is ok
;     ********


      DATP={X:     XX, X_TIT:'Angles',$
            Y:     YY, Y_TIT:' '     ,$
            Z_TIT: 'Numor'           ,$
            PAR_TXT: HEADER          ,$
            P:fltarr(headerSize)     ,$
            W_TIT: 'X_ray data'      ,$
            other_TIT:  MYFILE}
;     **********************            

 read_err:  FREE_LUN,unit          ;Free the unit number
 no_file:   IF unit lt 0 THEN print,!error_state.msg, string(7b)
 
 RETURN, DATA                      ;Return the data values
;************

 END
