;+
;************************************************************************
;* rdedf.pro                                                            *
;* Pupose:      rdedf is the function to read and display the ESRF data *
;*              files with LAMP from ILL.                               *  
;* Author:      Tobias Karrer                                           *
;* Created On:  10/03/97                                                *
;*----------------------------------------------------------------------*
;* Modifications:                                                    *
;* 30/05/97  changed the way to read the data, because LAMP wasn't   *
;*           able to read modified Images (p. ex. with saxs_add)     *
;* 11/06/97  added the variable 'pref' to specify the prefix of the  *
;*           filename                                                *
;* 12/06/97  modified the way to get the filename, after doing tests *
;* 02/07/97  removed code to display image number                    *
;* 30/07/97  added code to read format and dimension out of header   *
;* 21/06/01  status=11 before the OPENR (D.R.)             *
;* 21/06/01  imgNo =0  only when entering rdedf (D.R.)     *
;* 21/06/01  nofile:if c_unit le 0 then c_file='?' (D.R.)  *
;* 15/11/01  get parameters differently (D.R.)             *
;* 10/03/08  get counter_pos motor_pos ByteOrder (D.R.)    *
;* 14/10/11  add UnsignedLong + better swap (D.R)          *
;* 28/04/16  add SignedInteger (D.R)                       *
;***********************************************************
;-

function rdedf ,inst , path , filename , status , datp
;+
;******* *****
;** read and display the ESRF data
;-

common c_edf   , pref  ,suf
common c_edf_open, c_unit, c_file, c_imgNo

data  =0

;** if an error occurs, CATCH makes to quit the function, printing the error messages **
CATCH,stat & if stat ne 0 then begin & print,!error_stare.msg & RETURN, DATA & endif

imgNo=0                ;** variable to keep the image number
maxilen=169            ;** length of the maximal header in the edf-file
w=0 & s=0 & i=0 & j=0  ;** variables to count loops, arrays, etc...
flag=0                 ;** variable for title - 0:no title, 1:display title
title =''              ;** variable to keep the title of the image
head=STRARR(maxilen)   ;** whole header
par =STRARR(maxilen)   ;** value of header params
txxt=STRARR(maxilen)   ;** name of header params
line=''                ;** temporary variable for one headerline
formt='?'              ;** DataType
bytor='uni'            ;** ByteOrder

;** pref is a common block defined variable and is set with rdset,prefix='' or set_pref,'' ******
IF N_ELEMENTS(pref) EQ 1 THEN prefname=pref ELSE prefname=''

;** suf  is a common block defined variable and is set with rdset,suffix='' or set_suf,'' ******
IF N_ELEMENTS(suf)  EQ 1 THEN sufname=suf ELSE sufname=''

;** if filename is an array, first element = filename, second = image number ******
IF N_ELEMENTS(filename) GT 1 THEN BEGIN & imgNo=fix(filename[1])-1
                                        filename=filename[0] & ENDIF

IF N_ELEMENTS(c_unit)   EQ 0 THEN BEGIN & c_unit=0 & c_file=''
                                        c_imgNo=0 & ENDIF


;** looking for the extension '.edf'  ******
ext=STRPOS(strlowcase(filename),'.edf')

IF ext LT 0 THEN BEGIN &  on_ioerror,misnumor
      nfile=long(filename) & ln=strlen(filename)
      if ln gt 4 then filename=strmid(filename,ln-4,4)
      misnumor: file=prefname+filename+sufname
      if STRPOS(strlowcase(file),'.edf') lt 0 then file=file+'.edf'
ENDIF ELSE BEGIN & hypnorm =STRPOS(filename,'[',ext) ;** '[' given? ******
      hypshift=STRPOS(filename,'{',ext)              ;** '{' given? ******
      IF (hypnorm LT 0) AND (hypshift LT 0) THEN BEGIN
        file=filename
      ENDIF ELSE BEGIN              ;** get the given image number ******
        file=strmid(filename,0,ext+4)
        ON_IOERROR, mislong         ;** on convert error to 'long,
                                    ;** jump to label mislong
        imgNo=long(strmid(filename,ext+5,6))-1
        flag=1
        mislong: IF imgNo LT 0 THEN print,'No valid image number'
     ENDELSE
ENDELSE

OT=file & XT='X' & YT='Y' & X=0 & Y=0 & dim1=1 & dim2=1

ON_IOERROR,nofile  ;** on open error jump to label nofile

status=11          ;** display ' Cant open the file or file not found' ******

;** it is checked, if the file out of which to read the image is already open ******  
if (file ne c_file) or (imgNo lt c_imgNo) then begin
   if c_unit  gt 0 then FREE_LUN,c_unit
   c_file=file & c_unit=0      & c_imgNo=0
   OPENR,c_unit,path+file,/GET_LUN
   endif

ON_IOERROR,erread  ;** on read error jump to label erread
status=13          ;** display ' Data file incomplete' ******

;** read image data and header values ******
;**
FOR loop=c_imgNo,imgNo DO BEGIN

  j=0L
  ;** read each single line of the file and save it in head(j), ******
  ;** until line contains '}' ******
  REPEAT BEGIN & READF,c_unit,line & if j lt maxilen then head[j]=line & j=j+1

  ;** looking for 'DataType'  in the header ******
  IF STRPOS(line,'DataType') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    formt=STRCOMPRESS(STRMID(line,w+1,s-w-1), /REMOVE_ALL)
  ENDIF
  ;** looking for 'ByteOrder'  in the header ******
  IF STRPOS(line,'ByteOrder') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    blab=STRCOMPRESS(STRMID(line,w+1,s-w-1), /REMOVE_ALL)
    if blab eq 'HighByteFirst' then bytor='uni'
    if blab eq 'LowByteFirst'  then bytor='win'
  ENDIF
  ;** looking for 'Title'  in the header ******
  IF STRPOS(line,'Title') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    title=STRCOMPRESS(STRMID(line,w+1,s-w-1))
  ENDIF
  ;** looking for 'SubTitle'  in the header ******
  IF STRPOS(line,'SubTitle') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    XT=STRCOMPRESS(STRMID(line,w+1,s-w-1))
  ENDIF
  IF STRPOS(line,'scan_no') GT -1 THEN IF title eq '' THEN title=STRCOMPRESS(line)
  
  ;** looking for X index in the header ******
  IF STRPOS(line,'col_beg') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    X=long(STRMID(line,w+1,s-w-1))
  ENDIF
  ;** looking for Y index in the header ******
  IF STRPOS(line,'row_beg') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    Y=long(STRMID(line,w+1,s-w-1))
  ENDIF
  ;** looking for dimension 1 in the header ******
  IF STRPOS(line,'Dim_1') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    dim1=long(STRMID(line,w+1,s-w-1))
  ENDIF
  ;** looking for dimension 2  in the header ******
  IF STRPOS(line,'Dim_2') GT -1 THEN BEGIN
    w=STRPOS(line,'=')
    s=STRPOS(line,';')
    dim2=long(STRMID(line,w+1,s-w-1))
      CASE formt OF
      'ShortValue':      data=INTARR(dim1,dim2)
      'UnsignedShort':   data=INTARR(dim1,dim2)
      'IntegerValue':    data=LONARR(dim1,dim2) 
      'SignedInteger':   data=LONARR(dim1,dim2)
      'UnsignedInteger': data=ULONARR(dim1,dim2) 
      'UnsignedLong':    data=ULONARR(dim1,dim2) 
      'FloatValue':      data=FLTARR(dim1,dim2)
      'DoubleValue':     data=DBLARR(dim1,dim2)
      ELSE:
      ENDCASE
    IF N_ELEMENTS(data) EQ 1 THEN BEGIN
      print, "DataType:",formt," not known, using UnsignedShort as default"
      data=INTARR(dim1,dim2) & formt='UnsignedShort'
    ENDIF
  ENDIF
  ;** looking for image number in the header ******
  ;** if found, store value in 'titleVal' ******
  IF STRPOS(line,'Image') GT -1 THEN $
     titleVal=STRMID(line,0,STRPOS(line,';'))
  ENDREP UNTIL (StrPos(line,'}') NE -1)

  ;** read the image data *******
  READU,c_unit,data  & status=0 & swap=0  ;** 'Successful read' ******
  X=lindgen(dim1)+X & Y=lindgen(dim2)+Y
  ;swap=sys_dep('SWAPER',bytor)
  ;IF swap then CASE formt OF
  IF bytor eq 'uni' then CASE formt OF
      'ShortValue':      BYTEORDER,data,/sswap,/swap_if_little_endian
      'UnsignedShort':   BYTEORDER,data,/sswap,/swap_if_little_endian
      'IntegerValue':    BYTEORDER,data,/lswap,/swap_if_little_endian
      'SignedInteger':   BYTEORDER,data,/lswap,/swap_if_little_endian
      'UnsignedInteger': BYTEORDER,data,/lswap,/swap_if_little_endian
      'UnsignedLong':    BYTEORDER,data,/lswap,/swap_if_little_endian
      ELSE:
      ENDCASE
  IF bytor eq 'win' then CASE formt OF
      'ShortValue':     BYTEORDER,data,/sswap,/swap_if_big_endian
      'UnsignedShort':  BYTEORDER,data,/sswap,/swap_if_big_endian
      'IntegerValue':   BYTEORDER,data,/lswap,/swap_if_big_endian
      'SignedInteger':  BYTEORDER,data,/lswap,/swap_if_big_endian
      'UnsignedInteger':BYTEORDER,data,/lswap,/swap_if_big_endian
      'UnsignedLong':   BYTEORDER,data,/lswap,/swap_if_big_endian
      ELSE:
      ENDCASE
  ;if formt eq 'UnsignedShort' then positive,data
  
  DATA=FLOAT(DATA) ;anyway

  ;** the 'for' loop stores the names and the values of the headerlines ******
  ;** in their variables ******
  k=0L
  on_ioerror,misflt
  head=strcompress(head) & C_P="" & C_L="" & M_P="" & M_L=""
  FOR i=0L,(j<maxilen)-1 DO BEGIN
    line=head[i]
    w=STRPOS(line,'=')
    if w gt 0 then begin
       s=STRPOS(line,';')
       ok=-1 & ks=string(k,format='(I3)')
       if s gt w then begin
          if STRPOS(line,'counter_pos') GT -1 THEN BEGIN
             w=STRPOS(line,'=') & s=STRPOS(line,';')
             C_P=float(strsplit(/EXTRACT,strtrim(STRMID(line,w+1,s-w-1),2),' '))
          endif else $
          if STRPOS(line,'motor_pos')   GT -1 THEN BEGIN
             w=STRPOS(line,'=') & s=STRPOS(line,';')
             M_P=float(strsplit(/EXTRACT,strtrim(STRMID(line,w+1,s-w-1),2),' '))
          endif else $
          if STRPOS(line,'counter_mne') GT -1 THEN BEGIN
             w=STRPOS(line,'=') & s=STRPOS(line,';')
             C_L='c_p '+strsplit(/EXTRACT,strtrim(STRMID(line,w+1,s-w-1),2),' ')
          endif else $
          if STRPOS(line,'motor_mne')   GT -1 THEN BEGIN
             w=STRPOS(line,'=') & s=STRPOS(line,';')
             M_L='m_p '+strsplit(/EXTRACT,strtrim(STRMID(line,w+1,s-w-1),2),' ')
          endif else begin
             ok=0
             txxt[k]=STRMID(line,0,w-1)+' '+STRMID(line,s+1,100)
             num=STRMID(line,w+1,s-w-1)
             num=float(num) & ok=1 & misflt:
          endelse
       endif
       if (ok) eq 1 then par[k] =num
       if (ok) eq 0 then txxt[k]=txxt[k]+STRMID(line,w+1,100)
       if (ok) ge 0 then k=k+1
    endif
  ENDFOR
  txxt=txxt[0:k-1]
  par =par[ 0:k-1]
  ;help,C_P,C_L,M_P,M_L
  nk=n_elements(C_P)
  if (nk gt 1) and (nk eq n_elements(C_L)) then begin & txxt=[txxt,C_L] & par=[par,C_P] & endif
  nk=n_elements(M_P)
  if (nk gt 1) and (nk eq n_elements(M_L)) then begin & txxt=[txxt,M_L] & par=[par,M_P] & endif
  
ENDFOR ;** loop=c_imgNo,imgNo ******

erread:c_imgNo = imgNo+1

;** when flag=1, imagenumber is displayed as title ******
IF flag EQ 1 THEN title=titleVal

;** passing of header to lamp ******
datp={X:X,Y:Y,PAR_TXT:txxt,P:par,W_tit:title,OTHER_tit:OT,X_tit:XT,Y_tit:YT}

nofile:if c_unit le 0 then c_file='?'

if status ne 0 then if c_unit  gt 0 then FREE_LUN,c_unit
if status ne 0 then print,!error_state.msg

RETURN,data
END




