function rdgsas, INST , PATH , FILENAME , STATUS , DATP
;+
;******* ****** 
;**
;**    Standard call for a data-read function interfacing LAMP.
;**      
;**    Return of the function
;**     DATA     is an array of any dimensions and type containing the data values (spectra).
;**
;**    Input  parameters:
;**     INST(0)  is the file_type  (or instrument_name ) (string defined in customize tables).
;**     INST(1)  is the file_group (or instrument_group) (string defined in customize tables).
;**     PATH     is the full path where to find the data (string defined in customize tables).
;**     FILENAME is the name of the data file.
;**
;**    Output parameters:
;**     STATUS   is the returned error code you can choose from the following list:
;**          0 =' Successfull read'    
;**          1 =' Client/server on local node not established'
;**          2 =' Client/server on router node not established'
;**          3 =' The local  node cannot access the server node'
;**          4 =' The router node cannot access the server node'
;**          5 =' VME memory read error'
;**          7 =' Sequence error in data transfer'
;**          9 =' Parameter error'
;**          10=' Router is busy with other transfer'
;**          11=' Cant open the file or file not found'
;**          13=' Data file incomplete'
;**          14=' Bad instrument data definition'
;**          24=' Cant read the file'.
;**          
;**     DATP     is a structure defined as follow: (all tags are OPTIONAL)
;**          DATP.X        = vector of x coordinates.
;**          DATP.Y        = vector of y coordinates.
;**          DATP.Z        = vector of z coordinates.
;**          DATP.W_TIT    =   main title
;**          DATP.X_TIT    = x axis title
;**          DATP.Y_TIT    = y axis title
;**          DATP.Z_TIT    = z axis title
;**          DATP.OTHER_TIT=    sub title
;**          DATP.N        = monitors
;**          DATP.P        = vector of parameter values up to 31
;**          DATP.PAR_TXT  = string array of text associated to DATP.P (same size)
;**          DATP.PV       = an array of any dimensions containing other parameter values
;**          DATP.E        = the errors associated to DATA (same size)
;**          DATP.TIME     = string date of the experiment.
;-
    DATA  =0
    STATUS=11
    CATCH,stat & if stat ne 0 then begin & print,!error_state.msg & RETURN, DATA & endif
    ON_IOERROR, no_file

    OPENR,unit, PATH+FILENAME,/get_lun                ;Open the data file

    STATUS=13
    ON_IOERROR, read_err
    wt=' '
    line=' '
    prmfil=' '
    readf, unit , wt
    readf, unit , line
    if strpos(line,'Instrument parameter') ge 0 then prmfil=strmid(line,27,53)
    readf,unit,line
    if strpos(line,'Monitor') ge 0 then readf,unit,line
    while strpos(line,'#')    eq 0 do   readf,unit,line
    time_map=0
    IF strpos(line,'TIME_MAP') ge 0 THEN BEGIN 
        time_map=1 
        bid1=' '
        bid2=' '
        mapno=0
        nvals=0
        nrec=0
        clckwdt=0.0
        reads,line,format="(A8,3I5,A9,F5)",bid1,MAPNO,NVALS,NREC,bid2,CLCKWDT
        tmp=lonarr(nvals)
        readf,unit,tmp
        tmp1=reform(tmp[0:nvals-2],3,nvals/3)
        channels=tmp[nvals-4]+(tmp[nvals-1]-tmp[nvals-3])/tmp[nvals-2]
        front=fltarr(channels)
        width=fltarr(channels)
        j=0
        for i=1,channels do begin
            if j lt nvals/3 then if tmp1[0,j] le i then begin
                front[i-1]=tmp1[1,j]/clckwdt
                width[i-1]=tmp1[2,j]/clckwdt
                j=j+1            
            endif else begin
                front[i-1]=front[i-2]+width[i-2]
                width[i-1]=width[i-2]
            endelse else begin
                front[i-1]=front[i-2]+width[i-2]
                width[i-1]=width[i-2]
            endelse 
            ;print,i,front[i-1],width[i-1]
        endfor
        readf,unit,line
    ENDIF
    banks=0
    while not eof(unit) and strpos(line,'BANK') ge 0 do BEGIN 
        banks=banks+1 
        bid=' '
        ibank=0
        nchan=0
        nrec=0
        bcoef=0.0
        bintyp=' '
        reads,line,format="(A4,3I5,A9,F4)",bid,ibank,nchan,nrec,bintyp,bcoef
        IF strpos(line,'ESD') ge 0 THEN BEGIN  
             IF strpos(line,'TIME_MAP') ge 0 THEN BEGIN 
                tmp=fltarr(nchan*2)
                readf,unit,tmp
                tmp=reform(tmp,2,nchan)
                if banks eq 1 then begin
                    data=fltarr(nchan)
                    err =fltarr(nchan)
                    data=tmp[0,*]/width
                    err =tmp[1,*]/width
                    data=reform(data,nchan)
                    err =reform(err, nchan)
                endif else begin
                    data=[[data],[reform(tmp[0,*]/width)]]
                    err =[[err], [reform(tmp[1,*]/width)]]
                endelse
             ENDIF else print,'no time map given, not handled yet'
         ENDIF else print,'Either STD or ALT format, only ESD handled yet'
        if not eof(unit) then readf,unit,line
    ENDWHILE
    par      = [        3.0       ,        5.5        ]    ;      .
    ptxt     = ['First  parameter','Second parameter' ]    ;      .
    pall     =  INDGEN(20,8)
    xv = front+width/2
    xt = 'TOF/microseconds'
    STATUS=0                            ;Status is ok
;     ********
    DATP={X:      xv,    $                    ;Pass those variables which were 
          W_TIT:  wt,    $                 ;read-in into the DATP structure
          X_TIT:  xt,    $                   
          P:      par,   $
          PAR_TXT:ptxt,  $
          E:      err,   $
          N:      1.,    $
          PV:     pall   }
;     **********************                      
    RETURN, DATA
    read_err:FREE_LUN,unit
    no_file: print,!error_state.msg
    RETURN, DATA
END
