function read_mar, fifi, swap=swap
;+
;******* ********
;**
;**  Read a "classic" uncompressed Mar image file
;**  Use /swap to force byte swapping
;**  D.R. Jul 2011
;**
;**(http://www.marresearch.com/man/marcvt.htm)
;**(http://www.marresearch.com/man/mar345_formats.htm)
;**(http://www.marresearch.com/man/mar300_formats.htm)
;-

if n_elements(swap) ne 1 then swap=0
bytH=6900 ;After marcvt conversion?
wout=0

on_ioerror,misfil & u=-1
openr,u,fifi,/get_lun
  on_ioerror,misdat
  ;Checking the format
  ; ------------------
  l =0L & readu,u,l & s1=l & byteorder,l,/Lswap
  if (l eq 1234) or (s1 eq 1234) then begin
    lamp_message,'!!! Use a marcvt converter to uncompress this file.'
    free_lun,u & return,wout
  endif
  if swap then byteorder,s1,/Lswap & s1=s1<50000
  
  l =lonarr(9) & f=fltarr(15) & b=bytarr(24) & readu,u,l,f,b ;Read parameters
  if swap then byteorder,l ,/Lswap
  s2=l[0]<50000 & bytH=l[1]<50000
  h =bytarr(bytH-24-25*4)     & extra=l[3]   & readu,u,h     ;Read the gap

  wout=uintarr(s1,s2)         & readu,u,wout                 ;Read the data
  if swap then byteorder,wout,/Sswap  & wout=long(wout)
  
  if extra gt 0 then begin                                   ;Read high intensities
     high=lonarr(extra,2)     & readu,u,high
     if swap then byteorder,high,/Lswap
     Wout[high[*,0]-1]=high[*,1]
  endif
  
  Wt=fifi & Xt='X Detector' & Yt='Y Detector' & Ot=string(b)
  X =lindgen(s1)+1 & Y=lindgen(s2)+1
  P =[s1,l,f]
  P_txt=[ $
     '00 Number of pixels in x                     =',$
     '01 Number of pixels in y                     =',$
     '02 Record length in byte                     =',$
     '03 Max. no. of records                       =',$
     '04 No. of high intensity pixels in image.    =',$
     '05 No. of high intensity records in image.   =',$
     '06 No. of X-ray counts at start of exposure. =',$
     '07 No. of X-ray counts at end of exposure.   =',$
     '08 Exposure time in seconds.                 =',$
     '09 Exposure dose in units.                   =',$
     '10 Sum of X-ray counts.                      =',$
     '11 Maximum radius.                           =',$
     '12 Minimum radius.                           =',$
     '13 Unused.                                   =',$
     '14 Unused.                                   =',$
     '15 Unused.                                   =',$
     '16 Unused.                                   =',$
     '17 Centre in x.                              =',$
     '18 Centre in y.                              =',$
     '19 Used wavelength.                          =',$
     '20 Used distance crystal-detector.           =',$
     '21 PHI axis at start of exposure.            =',$
     '22 PHI axis at end of exposure.              =',$
     '23 OMEGA axis position.                      =',$
     '24 High intensity multiplier.                =']

  datp={X:X,Y:Y,W_tit:Wt,X_tit:Xt,Y_tit:Yt,Other_tit:Ot,P:P,PAR_txt:P_txt}
  give_datp,datp

  misdat:free_lun,u
  
  return,wout
  
misfil:lamp_error
return,wout

end
  
