function read_neat, INST , PATH , FILENAME , STATUS , DATP
;+
;*****************************************************************************
;** Reading Data files from NEAT data files
;** input file #1 : ????.INO file = contains reduced DATA (S(omega,2theta))
;**
;** Stephane Rols 11/2001 srols@anl.gov
;** modified by S. Rols (put free_lun ...) on 03/18/02
;** modified by D. Richard (read n_channels & n_spectra) on 14-Apr-05
;*****************************************************************************
;-

CATCH,stat & if stat ne 0 then begin & print,!error_state.msg & RETURN, DATA & endif
; ** Initializing of some parameters and arrays
; ** ******************************************
n_channels=512L       & n_spectra =141L ;Values will be read from the file
DATA      =0
str       ='' & str4=strarr(4) & str13=strarr(13) & str35=strarr(35) & bid4=lonarr(4)

; ** Second Step : Read the DATA of the run # FILENAME contained
; ** in the FILENAME.ino file
; ** ***********************************************************
ON_IOERROR, no_file
FIFI=FILENAME
if strpos(FILENAME,'.') eq -1 then begin
   while strpos(FIFI,'0') eq 0 do FIFI=strmid(FIFI,1,10)
   FIFI=FIFI+'NEAT.OUT' & endif

STATUS=11
OPENR,unit, PATH+FIFI,/get_lun       ;Open the ????.asc file
STATUS=13

readf,unit,str13                     ; reading the first 13 lines (not used)
readf,unit,bid4,n_spectra,n_channels
readf,unit,str4

readf,unit,str & str_title=str       ; reading the run title
readf,unit,str & readf,unit,str

a=fltarr(8*13) & readf,unit,a
SD_dist =a[12] & step_value=a[48] & lambda=a[18] & ch_w=a[52]/step_value
chop_per=a[44] & rep_period=a[43] & delay=a[50]/step_value

y=fltarr(n_spectra)
a=fltarr(8)    & readf,unit,a & y[0:7] =a
                 readf,unit,a & y[8:15]=a
readf,unit,str                       ;reading 'RRRRRRRRRRRRR'

a=fltarr(n_spectra-16) & readf,unit,a & y[16:n_spectra-1]=a

readf,unit,str35

DATA_LECT =FLTARR(n_channels,n_spectra) & readf,unit,DATA_LECT ;=S(t,2theta)

DATA=DATA_LECT[0:n_channels-1,1:n_spectra-1]
x_in=findgen(n_channels)+1

; ** Also find the elastic channel and pass it into the DATP str
; ** ****************************************************************

xtot=x_in & ytot=TOTAL(DATA,2) & etot=SQRT(ytot)
y0  =MAX(ytot,i0)   & xmin=xtot[(i0-20)>0] & xmax=xtot[(i0+20)<(n_channels-1)]
fitgauss, xtot, ytot, etot, xmin, xmax, gauss, dgauss
chel=gauss[2]  ; elastic channel

; ** Data are read ... Now extract the monitor and the error arrays
; ** and pass into the DATP structure
; ** ***************************************************************

DATPN        =DATA_LECT[0:n_channels-1,0]
DATAE        =SQRT(DATA)
DATPX        =x_in
DATPY        =y[1:n_spectra-1]
DATPW_TIT    =str_title+FILENAME
DATPX_TIT    ='Channel #'
DATPY_TIT    ='2 theta (degree)'
DATPOTHER_TIT=FILENAME
DATPTIME     ='01 janvier 3002'

; ** Pass into the DATP structure
; ** *****************************
; ** Now write the read parameters in the DATP.P corresponding array
; ** ***************************************************************
PAR_TXT =STRARR(31) & P=fltarr(31)
P[21]=lambda        &        PAR_TXT[21]='Wavelength (angstroms)       ='
P[27]=SD_dist/1000  &        PAR_TXT[27]='SD distance (mm)             ='
P[18]=ch_w          &        PAR_TXT[18]='channel width (us)           ='
P[9] =chel          &        PAR_TXT[9] ='Elastic channel(us)          ='
P[14]=rep_period    &        PAR_TXT[14]='Repetition period            ='
P[13]=chop_per      &        PAR_TXT[13]='Chopper 4 period (us)        ='

DATP={X:            DATPX,            $
      Y:            DATPY,            $
      W_TIT:        DATPW_TIT,        $
      X_TIT:        DATPX_TIT,        $
      Y_TIT:        DATPY_TIT,        $
      N:            DATPN,            $
      E:            DATAE,            $
      OTHER_TIT:    DATPOTHER_TIT,    $
      TIME:         DATPTIME,         $
      P:            P,                $
      PAR_TXT:      PAR_TXT }

STATUS=0
read_err:  FREE_LUN,unit
no_file:   if STATUS ne 0 then print,!error_state.msg

RETURN, DATA                                      ;Return the data values
;************
END

