function read_paxy, INST_GRP, PATH, FILENUME, STATUS, DATP
;+
;******* *********
;** readin function for instrument PAXY at LLB -> XY0123.32
;**
;** INST_GRP= [instrument , group , rawbut , '0'] ; rawbut is '1' or '0' after doing RDSET,/noraw
;** Returns WOUT detector counts , STATUS set to 0 if ok , DATP set to parameters structure
;-

fifi = FILENUME
if (strmid(fifi,0,1) le '9') then begin
  if (strpos(fifi,'XY')  lt 0) and (strpos(fifi,'PA')  lt 0) then while strlen(fifi) lt 4 do fifi = '0'+fifi ; Leading 0's
  if (strpos(fifi,'XY')  lt 0) and (strpos(fifi,'PA')  lt 0) then fifi = 'XY'+fifi
endif
if (strpos(fifi,'.32') lt 0) then fifi = fifi+'.32'

head = 256
siz  = [2,128,128]  &  WOUT = 0L

STATUS = 11
on_ioerror, misopen & u=-1

openr, u, PATH+fifi, /get_lun

  on_ioerror, miseof
  STATUS = 13
  rawbut = 1  &  if n_elements(INST_GRP) ge 3 then if INST_GRP[2] eq '0' then rawbut=0
  
  Header = bytarr(head)          & readu, u, Header
  WOUT   = lonarr(siz[1],siz[2]) & readu, u, WOUT   & byteorder, WOUT, /lswap, /swap_if_big_endian
  
  tail   ='' & line = ' '
  while not eof(u) do begin  readf, u, line  &  tail = [tail,line]  &  endwhile

  insv     =       string(Header[0:1])    & insv = 'PA'+insv
  numor    =       string(Header[2:5])
  user     =       string(Header[6:11])
  sample   =       string(Header[12:23])  & sample = strtrim(sample,2)
  time_s   = float(string(Header[24:29]))
  monitor  =  long(string(Header[30:37]))
  date     =       string(Header[38:47])  + ' ' + string(Header[48:55])
  imax     =  long(string(Header[56:60]))
  rpm_sel  =  long(string(Header[61:65]))
  pix_sz   = float(string(Header[66:67])) / 10.
  distance = float(string(Header[68:72])) /1000.
  counts   =  long(string(Header[102:108]))

  thick    =  0.1  & idx =strpos(tail,'Thickness Sample')       & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then thick  = float(strmid(para,idx+1)) / 10.
                     endif
  trans    =  0.9  & idx =strpos(tail,'Transmission')           & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then trans  = float(strmid(para,idx+1))
                     endif
  wl_set   =  5.01 & idx =strpos(tail,'Lambda (lect')           & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then wl_set = float(strmid(para,idx+1))
                     endif
  wl_ask   =  5.01 & idx =strpos(tail,'Lambda (Para')           & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then wl_ask = float(strmid(para,idx+1))
                     endif
  dl_l     = 11.71 & idx =strpos(tail,'Delta-Lambda')           & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then dl_l   = float(strmid(para,idx+1))
                     endif
  diaph0   =  7.61 & idx =strpos(tail,'Diaphragmes 0m')         & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then diaph0 = float(strmid(para,idx+1))
                     endif
  diaph2   = 20.01 & idx =strpos(tail,'Diaphragmes 2m')         & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then diaph2 = float(strmid(para,idx+1))
                     endif
  diaph5   =  0.01 & idx =strpos(tail,'Diaphragmes 5m')         & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then diaph5 = float(strmid(para,idx+1))
                     endif
  collim   =  2.24 & idx =strpos(tail,'D Coll(m)')              & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then collim = float(strmid(para,idx+1))
                     endif
  det_eff  =  0.09 & idx =strpos(tail,'Efficacite Detecteur')   & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then det_eff= float(strmid(para,idx+1))
                     endif
  dead_t   =  3.61 & idx =strpos(tail,'Temps Mort')             & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then dead_t = float(strmid(para,idx+1))
                     endif
  Cx       = 62.51 & idx =strpos(tail,'X0 ')                    & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then Cx         = float(strmid(para,idx+1))
                     endif
  Cy       = 62.51 & idx =strpos(tail,'Y0 ')                    & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then Cy         = float(strmid(para,idx+1))
                     endif
  pix_sz   =   .51 & idx =strpos(tail,'Resolution')             & idx = where(idx ge 0) & idx = idx[0]
                     if   idx ge 0 then begin para = tail[idx]  & idx = strpos(para,'=')
                       if idx gt 0 then pix_sz     = float(strmid(para,idx+1)) / 10.
                     endif
print,cx,cy
                   XT  ='Dectector X'       &  YT ='Detector Y'  & ZT  ='Numor' & Z=numor
                   WT  = sample
                   OT  = insv + ' Date ' + date + ' User ' + user + ' Run ' + numor
                   X   = indgen(siz[1])+1   &  Y  = indgen(siz[2])+1
                   N   = monitor
                   PV  = 0
                   E   = 0
 
                   P_TXT=[ $
                   '0)  PRESET 1                             =','1)  PRESET 2                             =',$
                   '2)  Run duration (sec.)                  =',$
                   '3)  Total detector counts                =','4)  Detector offset angle (deg.)         =',$
                   '5)  Coder 1: By (mm)                     =','6)  Coder 2: Bx (mm)                     =',$
                   '7)  Coder 3: Sample changer transl. (mm) =','8)  Coder 4: Detector distance (set) (m) =',$
                   '9)  Sample-Detector distance (calc.) (m) =','10) Sample Temperature (K)               =',$
                   '11) Value of IEEE-1 at start             =','12) Value of IEEE-1 at end               =',$
                   '13) Beam centre adress X0 (mm)           =','14) Beam centre adress Y0 (mm)           =',$
                   '15) Wavelength (angstroms)               =','16) Wavelength resolution                =',$
                   '17) Collimation  (m)                     =','18) Detector angle (set) (deg.)          =',$
                   '19) Detector translation (set) (mm)      =','20) Selector angle (deg.)                =',$
                   '21) Sample distance (mm)                 =','22) Sample rotation (deg.)               =',$
                   '23) Changer position                     =','24) Sample height (mm)                   =',$
                   '25) Shear speed (1/min.)                 =','26) Numor                                =',$
                   '27) Attenuator type                      =','28) Attenuator IN/OUT                    =',$
                   '29) Attenuation_value (not used)         =','30) Mode 1:Tof  3:Kin                    =',$
                   '31) Not used  ='                           ,'32) Not used  ='                           ,$
                   '33) Not used  ='                           ,'34) Attenuator2 factor (chopper)         =',$
                   '35) Not used  ='                           ,'36) Not used  =',$
                   '37) X pixel size (cm)                    =','38) Y pixel size (cm)                    =',$
                   '39) Not used  ='                           ,'40) Not used  =',$
                   '41) DeadTime by tube (ns)                =','42) Not used  =',$
                   '43) Selector (rpm)                       =','44) Sample thickness (cm)                =',$
                   '45) Transmission                         =','46) Wavelength asked (angstroms)         =',$
                   '47) Diaphragmes 0m                       =','48) Diaphragmes 2m                       =',$
                   '49) Diaphragmes 5m                       =','50) Detector efficiency                  =']

; Parameters used by sans_sheet
; *****************************                   
                      ;0         1         2       3           4          5           6      7           8
                      ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
              ;paridx=[15      , 9       , 19    , 18        , 17       , 2         , 26   , 27        , 28,$
              ;        24      , 22      , 10    , 16        , 6        , 5         , -1   , -1        , -1  , -1  , -1]
                      ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight, Xdet , Ydet      , Mode, att2, Tau
                      ;9         10        11      12          13         14          15     16        , 17  , 18  , 19
                   P=fltarr(n_elements(P_TXT))
                   p[ 2] = time_s
                   P[ 9] = distance
                   P[15] = wl_set
                   p[16] = dl_l
                   P[17] = collim
                   p[26] = numor
                   p[37] = pix_sz   &  p[38] = pix_sz
                   P[41] = dead_t*100.
                   N     = monitor
; Other parameters
; ****************
                   P[ 0] = time_s
                   P[ 1] = monitor
                   P[ 3] = counts
                   P[13] = Cx * pix_sz * 10.  &  P[14] = Cy * pix_sz * 10.
                   P[43] = rpm_sel
                   P[44] = thick
                   P[45] = trans
                   P[46] = wl_ask
                   P[47] = diaph0  &  P[48] = diaph2  &  P[49] = diaph5
                   P[50] = det_eff

  DATP   ={w_tit:WT ,x_tit:XT ,y_tit:YT ,z_tit:ZT ,other_tit:OT ,time:DATE,p:P,par_txt:P_TXT,$
           x:X      ,y:Y      ,z:Z      ,n:N      ,pv:PV        ,e:E}
  STATUS = 0
  miseof: free_lun, u
  return, float(WOUT)

misopen:
print,!error_state.msg
return, WOUT
end
