function read_toftof, INST , PATH , FILENOME , STATUS , DATP
;+
;*****************************************************************************
;** Reading Data files from TOFTOF FRMII data files
;** input file #1 : ??????_0000.raw file = contains raw DATA
;**
;** Stephane Rols 02/2009 rols@ill.eu
;**               02/2013 added a heading '0' in filename if necessary SR
;**                       change variable gauss by gaugau (idl8.1 confusion) DR
;**               09/2013 (by SR) to incorporate more parameters (tot_count, M1 and D rate, chopper info)
;**               05/2014 (by DR) accept also '.nxs. files
;*****************************************************************************
;-
forward_function read_nexus

CATCH,stat & if stat ne 0 then begin & lamp_error & if unit gt 0 then free_lun,unit & RETURN,DATA & endif

; ** First : Initializing of some parameters and arrays
; ** **************************************************
DATA     = 0
unit     =-1
str      ='' & n_fields=1L & num_records=1L & n_channels=1L & n_spectra=1L
FILENAME =FILENOME

if strpos(FILENAME,'_')   lt 0 then FILENAME=FILENAME+'_0000'         ; Complete FILENAME
if strpos(FILENAME,'raw') lt 0 then FILENAME=FILENAME+'.raw'          ; with default extension (or '_raw.nxs')
nl=strpos(FILENAME,'raw') - 6
while (nl le 5) do begin FILENAME='0'+FILENAME & nl = nl+1 & endwhile ; and leading 0's

fifi = file_search(PATH+FILENAME,count=nn)
if nn eq 0 then FILENAME = strmid(FILENAME,0,11) + '_raw.nxs'         ; Try '_raw.nxs' if '.raw' is not there
if strpos(FILENAME , '.nxs') gt 0 then begin & STATUS=-77
                  DATA = read_nexus(INST , PATH , FILENAME , STATUS , DATP)
                  return, temporary(DATA)
ENDIF

ON_IOERROR, no_file
STATUS=11
OPENR,unit, PATH+FILENAME,/get_lun     ; Open the ????.raw file
STATUS=13
ON_IOERROR, read_err

readf,unit,str & str_header=str & readf,unit,str
WHILE strpos(str,'aDetInfo(') ne 0 do begin & str_header=[str_header,str] & readf,unit,str & endwhile

; ** extract the header info
str   =strmid(str,9,20)  & reads,str, n_fields,num_records & det_arr=strarr(num_records)
readf,unit,str & readf,unit,str
readf,unit,det_arr
readf,unit,str & bid='aData('
str   =strmid(str,6,20)  & reads,str, n_channels,n_spectra & r_channels=n_channels
DATA  =FLTARR(n_channels,n_spectra) & readf,unit,DATA ;=S(t,2theta)

free_lun,unit

; ** Second Step : Read the Detector information  (STEF reads the file again using READ_ASCII function)
; ** *******************************************
; ** Third Step : Remove the 'None' detectors from Data
; ** **************************************************
;template_s={version:1. ,datastart:50L ,delimiter:byte(32) ,missingvalue:1e6 ,commentsymbol:'#' ,fieldcount:14L         ,$
;            fieldtypes:    [ 3L  , 3L  , 3L  , 3L  , 3L  , 4L     , 3L  , 3L  , 3L  , 3L  , 3L  , 3L  , 3L    , 7L    ],$
;            fieldnames:    ['f01','f02','f03','f04','f05','angles','f07','f08','f09','f10','f11','f12','index','state'],$
;            fieldlocations:[ 4L  , 10L , 16L , 22L , 28L , 32L    , 44L , 50L , 55L , 62L , 68L , 73L , 78L   , 85L   ],$
;            fieldgroups:   [ 0L  , 1L  , 2L  , 3L  , 4L  , 5L     , 6L  , 7L  , 8L  , 9L  , 10L , 11L , 12L   , 13L   ]}
;result=READ_ASCII(PATH+FILENAME, NUM_RECORDS=num_records,template=template_s)
;ind   =where(result.state ne "'None'",n_spectra) ; result.state indicates if the detector is installed or not (''None'' <=> absent)
;angles=abs(result.angles(ind))     & A=sort(angles) ; angles values are in the result.angles array (degree)
;DATA  =DATA(*,result.index(ind)-1) & DATA=DATA(*,A)
;x_in  =findgen(n_channels)+1
;y_in  =angles(A)

ind   =where(strpos(det_arr,"'None'") lt 0 ,n_spectra) ;(DIDS uses variable det_arr read in first step)
res   =strmid(det_arr[ind],0,82)
result=fltarr(n_fields-1,n_spectra) & reads,res,result
angles=abs(result[ 5,*])      & A=sort(angles)
DATA  =DATA[*,result[12,*]-1] & DATA=DATA[*,A]
x_in  =findgen(n_channels)+1
y_in  =angles[A]

; ** Also find the elastic channel and pass it into the DATP str
; ** ***********************************************************

xtot=x_in & ytot=TOTAL(DATA,2) & etot=SQRT(ytot)
y0  =MAX(ytot,i0)   & xmin=xtot[(i0-20)>0] & xmax=xtot[(i0+20)<(n_channels-1)]
fitgauss, xtot, ytot, etot, xmin, xmax, gaugau, dgauss
chel=gaugau[2]  ; elastic channel

; ** Data are read ... Now extract the monitor and the error arrays
; ** and pass into the DATP structure
; ** also deal with necessary input : wavelength,TOF (delay, channel width)
; ** temperature & distance ...
; ** *********************************************************************
;
; read title and other useful information
cretime=' ' & str_title=' ' & expteam=' ' & localc=' '
ind=WHERE(STRMATCH(str_header, 'File_Creation_Time:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & b=strpos(str,':') & cretime  =strcompress(strmid(str,b+2,80)) & endif
ind=WHERE(STRMATCH(str_header, 'Title:*'             , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & b=strpos(str,':') & str_title=strcompress(strmid(str,b+2,80)) & endif
ind=WHERE(STRMATCH(str_header, 'ExperimentTeam:*'    , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & b=strpos(str,':') & expteam  =strcompress(strmid(str,b+2,80)) & endif
ind=WHERE(STRMATCH(str_header, 'LocalContact:*'      , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & b=strpos(str,':') & localc   =strcompress(strmid(str,b+2,80)) & endif

; read monitor in the "MonitorCounts:" field
mon=1 & tot_count=0
ind=WHERE(STRMATCH(str_header, 'MonitorCounts:*'     , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & mon=long(str[1])     & endif
ind=WHERE(STRMATCH(str_header, 'TotalCounts:*'     , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & tot_count=float(str[1])     & endif
countm1=total([mon[*,0]])

; TOF parameter and wavelength
lambda=0. & tof_timeinterval=0. & tof_channelwidth=0. & tof_delay=0.
ind=WHERE(STRMATCH(str_header, 'Chopper_Wavelength:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & lambda=float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'TOF_TimeInterval:*'  , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & tof_timeinterval=float(str[1])*1.e6 & endif ; time frame in microseconds
ind=WHERE(STRMATCH(str_header, 'TOF_ChannelWidth:*'  , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & tof_channelwidth=float(str[1]) & endif ;>value in 50ns units...
ch_w=tof_channelwidth*50.*1e-3 ; >value in microseconds
ind=WHERE(STRMATCH(str_header, 'TOF_Delay:*'         , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & tof_delay=float(str[1]) & endif ; time frame in microseconds
delay=tof_delay*50.*1e-3 ; >value in microseconds

; Chopper info
Chopper_Speed=0. & Chopper_Ratio=1
ind=WHERE(STRMATCH(str_header, 'Chopper_Speed:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & Chopper_Speed=float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'Chopper_Ratio:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & Chopper_Ratio=float(str[1]) & endif

; Time and date
tof_timepreselection=0. & statis=0. & str_savingdate=' ' & str_startdate=' ' & str_starttime=' '
ind=WHERE(STRMATCH(str_header, 'TOF_TimePreselection:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & tof_timepreselection=float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'Status:*'              , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & statis=float(str[1])/100.          & endif
measuring_time=tof_timepreselection*statis
ind=WHERE(STRMATCH(str_header, 'SavingDate:*'          , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & str_savingdate=str[1] & endif
ind=WHERE(STRMATCH(str_header, 'StartDate:*'           , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & str_startdate =str[1] & endif
ind=WHERE(STRMATCH(str_header, 'StartTime:*'           , /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & str_starttime =str[1] & endif
measuring_time=tof_timepreselection*statis

; Temperatures
avgtemp=0. & stdvtemp=0. & mintemp=0. & maxtemp=0.
ind=WHERE(STRMATCH(str_header, 'AverageSampleTemperature:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & if (str[1] ne "'None'") then avgtemp =float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'StandardDeviationOfSampleTemperature:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & if (str[1] ne "'None'") then stdvtemp=float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'MinimumSampleTemperature:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & if (str[1] ne "'None'") then mintemp =float(str[1]) & endif
ind=WHERE(STRMATCH(str_header, 'MaximumSampleTemperature:*', /FOLD_CASE) EQ 1)
if ind[0] ge 0 then begin & str=str_header[ind] & str = STRSPLIT( str, ' ', /EXTRACT,count=co) & if (str[1] ne "'None'") then maxtemp =float(str[1]) & endif

; Calculate effective number of channels and channel_witdh in microseconds (see below)
n_channels=floor(tof_timeinterval/(ch_w))

DATA=DATA[0:n_channels-1,*] & x_in=x_in[0:n_channels-1]

DATPN        =[measuring_time,countm1,tot_count]
DATAE        =SQRT(DATA)
DATPX        =x_in
DATPY        =y_in ;(1:n_spectra-1)
DATPW_TIT    =str_title+' ('+FILENOME+')'
DATPX_TIT    ='Channel #'
DATPY_TIT    ='2 theta (degree)'
DATPOTHER_TIT=FILENAME+' Date:'+cretime+' User:'+expteam+' Contact:'+localc
DATPTIME     =str_savingdate

; ** Pass into the DATP structure
; ** *****************************
; ** Now write the read parameters in the DATP.P corresponding array
; ** ***************************************************************
PAR_TXT =STRARR(32) & P=fltarr(32)

P[0]=long(FILENOME)  &      PAR_TXT[0]= '00/ Numor                        ='
P[1]=measuring_time  &      PAR_TXT[1]= '01/ Duration (s)                 ='
P[2]=n_channels      &      PAR_TXT[2]= '02/ Number of reserved channels  ='
P[3]=countm1         &      PAR_TXT[3]= '03/ Tot. Counts in M1            ='
P[4]=tot_count       &      PAR_TXT[4]= '04/ Tot. Counts in Detectors     ='
P[5]=P[3]/P[1]       &      PAR_TXT[5]= '05/ M1 rate (c/s)                ='
P[6]=P[4]/P[1]       &      PAR_TXT[6]= '06/ D rate (c/s)                 ='
P[7]=0.              &      PAR_TXT[7]= '07/ Not used                     ='
P[8]=0.              &      PAR_TXT[8]= '08/ Not used                     ='
P[9]=chel            &      PAR_TXT[9]= '09/ Elastic peak position        ='
P[10]=long(FILENOME) &      PAR_TXT[10]='10/ Numor                        ='
P[11]=avgtemp        &      PAR_TXT[11]='11/ Avg. Temperature             ='
P[12]=stdvtemp       &      PAR_TXT[12]='12/ Stdv. Temperature            ='
P[13]=mintemp        &      PAR_TXT[13]='13/ Min. Temperature             ='
P[14]=maxtemp        &      PAR_TXT[14]='14/ Max. Temperature             ='
P[15]=0.             &      PAR_TXT[15]='15/ Not used                     ='
P[16]=0.             &      PAR_TXT[16]='16/ Not used                     ='
P[17]=0.             &      PAR_TXT[17]='17/ Not used                     ='
P[18]=ch_w           &      PAR_TXT[18]='18/ Channel Width (microsec.)    ='
P[19]=n_channels     &      PAR_TXT[19]='19/ Number of channels used      ='
P[20]=delay          &      PAR_TXT[20]='20/ TOF delay (microsec.)        ='
P[21]=lambda         &      PAR_TXT[21]='21/ Wavelength (angstroms)       ='
P[22]=Chopper_Speed  &      PAR_TXT[22]='22/ Chopper Speed (RPM)          ='
P[23]=Chopper_Ratio  &      PAR_TXT[23]='23/ Chopper Ratio                ='
P[24]=0.             &      PAR_TXT[24]='24/ Not used                     ='
P[25]=0.             &      PAR_TXT[25]='25/ Not used                     ='
P[26]=0.             &      PAR_TXT[26]='26/ Not used                     ='
P[27]=4.0            &      PAR_TXT[27]='27/ Dist. Det - Sample (m)       ='
P[28]=0.             &      PAR_TXT[28]='28/ Not used                     ='
P[29]=0.             &      PAR_TXT[29]='29/ Not used                     ='
P[30]=n_spectra      &      PAR_TXT[30]='30/ Number of angles             ='

P=[P,fltarr(n_elements(str_header))] & PAR_TXT=[PAR_TXT,str_header+' =']

tutobene: DATP={  X:             DATPX,     $
                  Y:             DATPY,     $
                  W_TIT:         DATPW_TIT, $
                  X_TIT:         DATPX_TIT, $
                  Y_TIT:         DATPY_TIT, $
                  N:             DATPN,     $
                  E:             DATAE,     $
                  OTHER_TIT: DATPOTHER_TIT, $
                  TIME:          DATPTIME,  $
                  P:             P,         $
                  PAR_TXT:       PAR_TXT }

STATUS=0
read_err:  FREE_LUN,unit
no_file:   if STATUS ne 0 then lamp_error

RETURN, DATA                                      ;Return the data values
;***********
END


; Parameter                  Unit   comment
;
; TOF_ChannelWidth           50 ns  width of one time channel (value x 50
;                                   ns)
; TOF_TimeInterval            1 s   time interval between two successive
;                                   n-pulses
; TOF_TimePreselection        1 s   preselected time of measurement
; AverageSampleTemperature:   1 K   this temp. should be read out as the
;                                   sample temperature. The value might be
;                                   "None" if no temp info was/is available
; TOF_MonitorInput                  channel number of monitor. The monitor
;                                   is a fission chamber right in front of

;the sample. It is calibrated and you can calculate the absolut flux of
;neutrons with the sensitivity of 1.48e-5 for 0.18 nm neutrons
;(sensitivity is proportional to wavelength). The crosssection of the
;beam is given by the horizontal and vertical gap of the sample slit which
;is position right in front of the fission chamber.;
;
;List of detectors:;

; 6th column: scattering angle 2 theta
;13th column: number of the detector in the data list below
;14th column: serial number of installed detectors; None means that no
;             detector is installed
;
;Data list:
;
;One detector per line, sorted according to the number in the 13th column
;of the detector list
;
;Each line is a space separated list of the intensity of successive time
;channels. The effective number of channels is given by:
;
;    TOF_TimeInterval/(TOF_ChannelWidth*50)
;
;That means for TOF_TimeInterval=0.01s and TOF_ChannelWidth=196*50ns as in
;the attached example data file you get an effective number of channels of
;1020.408... channels. That means that you should use the first 1020
;channels only. The rest of the channels will be empty except channel  1021
;which is however only "half filled". The time gap between the last
;channel (1020) and the first channel (1) is then given by:
;TOF_TimeInterval/(TOF_ChannelWidth*50) - 1020 * TOF_ChannelWidth*50 in ns!
; This are about 3.92 micro seconds in the example. Thus if you shift data
;from the energy gain side to the loss side you have to respect this time
;gap. No additional time gap has to be considdered!
