function read_v4, INST_GRP, PATH, FILENUME, STATUS, DATP
;+
;******* *********
;** reading function for instrument V4 at HZB -> D#######.001
;**
;** INST_GRP= [instrument , group , rawbut , '0'] ; rawbut is '1' or '0' after doing RDSET,/noraw
;** Returns WOUT detector counts , STATUS set to 0 if ok , DATP set to parameters structure
;**
;** To complete - missing parameters: lambda, pixel size, etc.
;-

insv = 'V4'

fifi = FILENUME
if (strmid(fifi,0,1) le '9') then begin
  if (strpos(fifi,'D')  lt 0)  then while strlen(fifi) lt 7 do fifi = '0'+fifi ; Leading 0's
  if (strpos(fifi,'D')  lt 0)  then fifi = 'D'+fifi
endif
if (strpos(fifi,'.001') lt 0) then fifi = fifi+'.001'

STATUS = 11
on_ioerror, misopen & u=-1

openr, u, PATH+fifi, /get_lun

on_ioerror, miseof
STATUS = 13
rawbut = 1  &  if n_elements(INST_GRP) ge 3 then if INST_GRP[2] eq '0' then rawbut=0

;Read whole file into array 
line = ''
array = ''
while not eof(u) do begin
  readf, u, line
  array = [array, line]
endwhile

;Determine where the counts block starts
icount = strpos(array, '%Counts') & icount = where(icount ge 0) & icount = icount[0]

;Get important parameters - TO DO: Need to define defaults or care about undefined variables 

;%File
tmp = 'FileName='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  tmp = strsplit(tmp[1], '.', /extract)
  numor = strmid(tmp[0], 1)
endif

tmp = 'FileDate='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  date = tmp[1]
endif

tmp = 'FileTime='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  time = tmp[1]
endif  

tmp = 'DataSize='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  size_total = long(tmp[1])
endif

tmp = 'DataSizeX='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  size_x = long(tmp[1])
endif

tmp = 'DataSizeY='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  size_y = long(tmp[1])
endif

tmp = 'Title='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  title = tmp[1]
endif

tmp = 'User='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  user = tmp[1]
endif  

;%Version

;%Sample
tmp = 'SampleName='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  sample_name = tmp[1]
endif

tmp = 'Position='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  position = long(tmp[1])
endif

tmp = 'Omega='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  omega = float(tmp[1])
endif

tmp = 'Chi='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  chi = float(tmp[1])
endif

tmp = 'Temperature='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  temp = float(tmp[1])
endif

tmp = 'Temp1='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  temp1 = float(tmp[1])
endif

tmp = 'Temp2='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  temp2 = float(tmp[1])
endif

;%Setup
tmp = 'Collimation='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  collim = float(tmp[1])
endif

tmp = 'SD=' ;sample-detector distance ???????
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  sampledet_distance = float(tmp[1])
endif

;%Counter
tmp = 'Sum='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  det_counts = float(tmp[1])
endif

tmp = 'Time='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) 
;Select line starting with Time= to discriminate from FileTime=, Moni1/Time=, etc.
if idx[0] ge 0 then begin
  for i = 0, n_elements(idx)-1 do begin
    if (strmid(array[idx[i]], 0, 5) eq tmp) then begin
      tmp2 = strsplit(array[idx[i]], '=', /extract)
      count_time = float(tmp2[1])
    endif
  endfor
endif

tmp = 'Moni1='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  monitor1 = float(tmp[1])
endif

tmp = 'Moni2='
idx =strpos(array[0:icount-1], tmp) & idx = where(idx ge 0) & idx = idx[0]
if idx ge 0 then begin
  tmp = strsplit(array[idx], '=', /extract)
  monitor2 = float(tmp[1])
endif

;%History

;%Counts
counts = []
for i = icount+1, n_elements(array)-1 do begin
  tmp = float(strsplit(array[i], ',', /extract))
  counts = [counts, tmp]
endfor

; Output array
WOUT   = reform(counts, size_x, size_y)

XT ='Detector X'       
YT ='Detector Y'  
ZT ='Numor' & Z=numor
WT = sample_name
OT = insv + ' Date ' + date + ' User ' + user + ' Run ' + numor
X  = indgen(size_x)+1   
Y  = indgen(size_y)+1
N   = monitor1
PV  = 0
E   = 0
 
P_TXT=[ $
        '0)  PRESET 1                             =','1)  PRESET 2                             =',$
        '2)  Run duration (sec.)                  =',$
        '3)  Total detector counts                =','4)  Detector offset angle (deg.)         =',$
        '5)  Coder 1: By (mm)                     =','6)  Coder 2: Bx (mm)                     =',$
        '7)  Coder 3: Sample changer transl. (mm) =','8)  Coder 4: Detector distance (set) (m) =',$
        '9)  Sample-Detector distance (calc.) (m) =','10) Sample Temperature (K)               =',$
        '11) Value of IEEE-1 at start             =','12) Value of IEEE-1 at end               =',$
        '13) Beam centre adress X0 (mm)           =','14) Beam centre adress Y0 (mm)           =',$
        '15) Wavelength (angstroms)               =','16) Wavelength resolution                =',$
        '17) Collimation  (m)                     =','18) Detector angle (set) (deg.)          =',$
        '19) Detector translation (set) (mm)      =','20) Selector angle (deg.)                =',$
        '21) Sample distance (mm)                 =','22) Sample rotation (deg.)               =',$
        '23) Changer position                     =','24) Sample height (mm)                   =',$
        '25) Shear speed (1/min.)                 =','26) Numor                                =',$
        '27) Attenuator type                      =','28) Attenuator IN/OUT                    =',$
        '29) Attenuation_value (not used)         =','30) Mode 1:Tof  3:Kin                    =',$
        '31) Not used  ='                           ,'32) Not used  ='                           ,$
        '33) Not used  ='                           ,'34) Attenuator2 factor (chopper)         =',$
        '35) Not used  ='                           ,'36) Not used  =',$
        '37) X pixel size (cm)                    =','38) Y pixel size (cm)                    =',$
        '39) Not used  ='                           ,'40) Not used  =',$
        '41) DeadTime by tube (ns)                =','42) Not used  =',$
        '43) Selector (rpm)                       =','44) Sample thickness (cm)                =',$
        '45) Transmission                         =','46) Wavelength asked (angstroms)         =',$
        '47) Diaphragmes 0m                       =','48) Diaphragmes 2m                       =',$
        '49) Diaphragmes 5m                       =','50) Detector efficiency                  =']

; Parameters used by sans_sheet
; *****************************                   
        ;0         1         2       3           4          5           6      7           8
        ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
;paridx=[15      , 9       , 19    , 18        , 17       , 2         , 26   , 27        , 28,$
;        24      , 22      , 10    , 16        , 6        , 5         , -1   , -1        , -1  , -1  , -1]
;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight, Xdet , Ydet      , Mode, att2, Tau
;9         10        11      12          13         14          15     16        , 17  , 18  , 19
P=fltarr(n_elements(P_TXT))
P[ 2] = count_time
P[ 9] = sampledet_distance
P[15] = 5.0                         ; MISSING WAVELENGTH IN FILE!!!!!!!!!!!!
p[16] = 0.1                         ; MISSING DELTA_LAMBDA/LAMBDA IN FILE!!!!!!!!!!!!
P[17] = collim
p[26] = long(numor)
p[37] = 1.0                         ; MISSING X PIXEL SIZE IN FILE!!!!!!!!!!!!
p[38] = 1.0                         ; MISSING Y PIXEL SIZE IN FILE!!!!!!!!!!!!
P[41] = 0.0                         ; MISSING DEAD_TIME IN FILE!!!!!!!!!!!!

; Other parameters
; ****************
P[ 0] = count_time
P[ 1] = monitor1
P[ 3] = det_counts
;                   P[13] = Cx * pix_sz * 10.  &  P[14] = Cy * pix_sz * 10.
;                   P[43] = rpm_sel
;                   P[44] = thick
;                   P[45] = trans
;                   P[46] = wl_ask
;                   P[47] = diaph0  &  P[48] = diaph2  &  P[49] = diaph5
;                   P[50] = det_eff

  DATP   ={w_tit:WT ,x_tit:XT ,y_tit:YT ,z_tit:ZT ,other_tit:OT ,time:DATE,p:P,par_txt:P_TXT,$
           x:X      ,y:Y      ,z:Z      ,n:N      ,pv:PV        ,e:E}
  STATUS = 0
  miseof: free_lun, u
  return, float(WOUT)

misopen:
print,!error_state.msg
return, WOUT
end
