function regular, Win, DX=dX, DY=dY, DZ=dZ, DATP=datp, GetNewSize=GNS, Xmin=Xmin,Xmax=Xmax,Ymin=Ymin,Ymax=Ymax, fillMin=fillMin
;+
;******* *******
;** User callable
;** Pixelize irregular 1d,2d,(3d) Win and return data in a regular grid,
;** (no interpolation in process).
;** Call:W2  =REGULAR(W1  [,DX=dX] [,DY=dY])            from command line.
;**      Wout=REGULAR(Win [,DX=dX] [,DY=dY], DATP=datp) from a  procedure.
;**      sxyz=REGULAR(Win [,DX=dX] [,DY=dY], DATP=datp,/GetNewSize).
;** 
;** dX>0.  : optional X bin width (<=0 do'nt touch X)
;** dY>0.  : optional Y bin width (<=0 do'nt touch Y)
;** datp   : ={X:x ,Y:y ,E:e}   new x,y,e will be returned in datp.
;** 
;** D.R. March 2006 (I.L.L.) for JR.S.
;** D.R. May   2006 added /GetNewSize  (return NewSize only)
;** D.R. Jun   2007 limits expand to MFCT times in each direction (default:9.5)
;** D.R. Oct   2008 revisited
;** D.R. Dec   2011 added Min,Max keywords. CARE WIN IS CHANGED FOR MEMORY SAVINGS.
;** D.R. Nov   2012 added fillMin and DZ. in case of expand, fill holes with the minimum value (to do ...)
;**                                       and take care of third dimension (used for hkl volumes)
;-

;Get workspace variables
;***********************
 MFCT=9.5
 okDP=1 & if n_elements(datp) eq 0 then TAKE_DATP,datp else okDP=0
 okE =(n_elements(datp.e) eq  n_elements(Win))
 siz = size(Win)
 tags= tag_names(datp) & idx=where(tags eq 'Z', Zok)
 
 X   = datp.X       & Y  =[datp.Y]       & E  = [datp.e]
 if Zok  then         Z  =[datp.Z]   else  Z  =  0
 if (siz[0] ge 1) and (n_elements(X) ne siz[1]) then X=findgen(siz[1])+1
 if (siz[0] ge 2) and (n_elements(Y) ne siz[2]) then Y=findgen(siz[2])+1
 if (siz[0] ge 3) and (n_elements(Z) ne siz[3]) then Z=findgen(siz[3])+1. ;Z may be used for other purpose
 dmx =(size(X))[0]  & dmy=(size(Y))[0]   & dmz=(size(Z))[0]
 mima= 0

;Check for MIN MAX keywords, CARE WIN IS CHANGED.
 if (((n_elements(Xmin) eq 1) or  (n_elements(Xmax) eq 1)) and dmx eq 1) or $
    (((n_elements(Ymin) eq 1) or  (n_elements(Ymax) eq 1)) and dmy le 1) then begin
   idx=-1 & idy=-1 & mima=1
   if (n_elements(Xmin) eq 1) and (n_elements(Xmax) eq 1) then idx=where((X ge Xmin) and (X le Xmax)) else $
   if (n_elements(Xmin) eq 1) then idx= where(X ge Xmin)  else $
   if (n_elements(Xmax) eq 1) then idx= where(X le Xmax)
   if (n_elements(Ymin) eq 1) and (n_elements(Ymax) eq 1) then idy=where((Y ge Ymin) and (Y le Ymax)) else $
   if (n_elements(Ymin) eq 1) then idy= where(Y ge Ymin)  else $
   if (n_elements(Ymax) eq 1) then idy= where(Y le Ymax)
   
   if (idx[0] ge 0) and (idy[0] ge 0) then begin
                               WIN=WIN[idx,idy,*] & X=X[idx] & Y=Y[idy] & if okE then E=E[idx,idy,*] & endif else $
   if (idx[0] ge 0) then begin & WIN=WIN[idx,*,*]   & X=X[idx] & if okE then E=E[idx,*,*]              & endif else $
   if (idy[0] ge 0) then begin & WIN=WIN[*,idy,*]   & Y=Y[idy] & if okE then E=E[*,idy,*]              & endif
 endif
 
 siz=size(Win)    &  sx=siz[1]
 if siz[0] gt 1 then sy=siz[2] else sy =1 & osx=sx & sx1=sx-1 & nx1=n_elements(X)-1
 if siz[0] gt 2 then sz=siz[3] else sz =1 & osy=sy & sy1=sy-1 & ny1=n_elements(Y)-1
                                            osz=sz & sz1=sz-1 & nz1=n_elements(Z)-1
 if X[nx1] lt X[0] then crx=-1 else crx=1
 if Y[ny1] lt Y[0] then cry=-1 else cry=1
 if Z[nz1] lt Z[0] then crz=-1 else crz=1

;Check for non-regular dimensions X  Y  Z
;****************************************
 maxx=max(X,min=minx) & bX=(maxx-minx)/sx1     &  endx=(X[1]    -X[0])*sx1+minx
 maxy=max(Y,min=miny) & bY=(maxy-miny)/(sy1>1) &  endy=(Y[1<sy1]-Y[0])*sy1+miny
 maxz=max(Z,min=minz) & bZ=(maxz-minz)/(sz1>1) &  endz=(Z[1<sz1]-Z[0])*sz1+minz
 if (endx le maxx+bX/10.) and (endx ge maxx-bX/10.)  then regX=1  else regX=0
 if (endy le maxy+bY/10.) and (endy ge maxy-bY/10.)  then regY=1  else regY=0
 if (endz le maxz+bZ/10.) and (endz ge maxz-bZ/10.)  then regZ=1  else regZ=0
 if n_elements(dX) eq 1 then if dX gt 0 then begin & if abs(1.-dX/bX) gt .1  then regX=-1 & bX=dX & endif else regX=1
 if n_elements(dY) eq 1 then if dY gt 0 then begin & if abs(1.-dY/bY) gt .1  then regY=-1 & bY=dY & endif else regY=1
 if n_elements(dZ) eq 1 then if dZ gt 0 then begin & if abs(1.-dZ/bZ) gt .1  then regZ=-1 & bZ=dZ & endif else regZ=1
 if  sy   eq 1  then  regY=1
 if  sz   eq 1  then  regZ=1
 if (regX eq 1)  and (regY eq 1) and (regZ eq 1) and (not keyword_set(GNS)) then begin
     if mima then begin & MOD_DATP,datp,'X',X & MOD_DATP,datp,'Y',Y & MOD_DATP,datp,'E',temporary(E) & endif
     if mima and okDP then GIVE_DATP, datp
     return,Win & endif
     
;Calculate Xout
;**************
 if regX ne 1 then begin
   if regX eq 0 then begin & difx=abs(X-shift(X,1)) & difx=difx[sort(difx)] ;** dX absent
                           idx =where(difx gt 0.) & bX  =difx[idx[1]]     ;limits to MFCT/3 times
			    osx =round((maxx-minx)/bX)+1. & bX=bX*(((osx/sx)/(MFCT/3.))>1.) & endif
   osx =round((maxx-minx)/bX)+1. & bX=bX*(((osx/sx)/MFCT)>1.)             ;limits to MFCT times
   osx =round((maxx-minx)/bX)+1. & Xout=findgen(osx)*bX+minx
 endif else Xout=X[*,0]

;Calculate Yout
;**************
 if regY ne 1 then begin
   if regY eq 0 then begin & if dmy eq 2 then dify=abs(Y-shift(Y,0,1)) else $
                           dify=abs(Y-shift(Y,1)) & dify=dify[sort(dify)] ;** dY absent
                           idy =where(dify gt 0.) & bY  =dify[idy[1]]     ;limits to MFCT/3 times
			    osy =round((maxy-miny)/bY)+1. & bY=bY*(((osy/sy)/(MFCT/3.))>1.) & endif
   osy =round((maxy-miny)/bY)+1. & bY=bY*(((osy/sy)/MFCT)>1.)             ;limits to MFCT times
   osy =round((maxy-miny)/bY)+1. & Yout=findgen(osy)*bY+miny
 endif else if (size(Y))[0] eq 1 then Yout=Y else Yout=reform(Y[0,*])

;Calculate Zout
;**************
 if regZ ne 1 then begin
   if regZ eq 0 then begin & if dmz eq 3 then difz=abs(Z-shift(Z,0,0,1)) else $
                           difz=abs(Z-shift(Z,1)) & difz=difz[sort(difz)] ;** dZ absent
                           idz =where(difz gt 0.) & bZ  =difz[idz[1]]     ;limits to MFCT/3 times
          osz =round((maxz-minz)/bZ)+1. & bZ=bZ*(((osz/sz)/(MFCT/3.))>1.) & endif
   osz =round((maxz-minz)/bZ)+1. & bZ=bZ*(((osz/sz)/MFCT)>1.)             ;limits to MFCT times
   osz =round((maxz-minz)/bZ)+1. & Zout=findgen(osz)*bZ+minz
 endif else if (size(Z))[0] eq 1 then Zout=Z else Zout=Z

;Just return the new size or Win unchanged
;*****************************************
 if keyword_set(GNS)            then return,float([osx,osy,osz])
 if (regX eq 1) and (regY eq 1) and (regZ eq 1) then begin
     if mima then begin & MOD_DATP,datp,'X',X & MOD_DATP,datp,'Y',Y
                        MOD_DATP,datp,'E',temporary(E) & endif
     if mima and okDP then GIVE_DATP, datp
     return,Win & endif

;Make output arrays. Do not treat the Z dimension here.
;******************  *********************************
 if (regX ne 1) or (regY ne 1) then begin
 
 Wout=reform(fltarr(osx,osy,sz)) & Wcnt=reform(fltarr(osx,osy)) & jx=0 & yw=0
 if okE then E2=E^2
 if okE then Eout=Wout else Eout=datp.e

;Loop for one, two (three dimensional) input
;*******************************************
 if (regX eq 1) and (dmy eq 1) then begin
;**************************************************************************
;******************************************** simplified loop, X is regular
 tt=(Y[0:sy1-1]+Y[1:*])/2. & ya=[Y[0]-by*cry,tt] & yb=[tt,Y[sy1]+by*cry]
 for j=0,sy1 do begin     ;** Loop on Y
        ;Find Yout indexes
                if ya[j] le yb[j] then yw=where((Yout ge ya[j]) and (Yout le yb[j])) $
                                  else yw=where((Yout ge yb[j]) and (Yout le ya[j]))
        ;Fill output
                if (yw[0] ge 0) then $
		             for n=0,n_elements(yw)-1 do begin
                   if okE then Eout[*,yw[n],*]=Eout[*,yw[n],*]+E2[ *,j,*]
                               Wout[*,yw[n],*]=Wout[*,yw[n],*]+Win[*,j,*]
                               Wcnt[*,yw[n]  ]=Wcnt[*,yw[n]  ]+1.
                 endfor
  endfor        ;**Y

;**************************************************************************
;******************************************** simplified loop, Y is regular
 endif else if (regY eq 1) and (dmx eq 1) then begin
    tt=(X[0:sx1-1]+X[1:*])/2. & xa=[X[0]-bx*crx,tt] & xb=[tt,X[sx1]+bx*crx]
    for i=0,sx1 do begin  ;** Loop on X
        ;Find Xout indexes
                if xa[i] le xb[i] then xw=where((Xout ge xa[i]) and (Xout le xb[i])) $
                                  else xw=where((Xout ge xb[i]) and (Xout le xa[i]))
        ;Fill output
                if (xw[0] ge 0) then $
		             for n=0,n_elements(xw)-1 do begin
                   if okE then Eout[xw[n],*,*]=Eout[xw[n],*,*]+E2[ i,*,*]
                               Wout[xw[n],*,*]=Wout[xw[n],*,*]+Win[i,*,*]
                               Wcnt[xw[n],*  ]=Wcnt[xw[n],*  ]+1.
                 endfor
    endfor      ;**X

;*************************************************************************
;******************************************** neither X and Y are regular!
 endif else if (regX ne 1) and (regY ne 1) then begin
 
  for j=0,sy1 do begin    ;** Loop on Y
    if dmx eq 2 then jx=j
    for i=0,sx1 do begin  ;** Loop on X
        ;Find Xout indexes
	        if regX ne 1 then begin
                  if i eq 0   then xa=X[i,jx]-bx*crx else xa=(X[i-1,jx]+X[i,jx])/2.
                  if i eq sx1 then xb=X[i,jx]+bx*crx else xb=(X[i+1,jx]+X[i,jx])/2.
                  if xb lt xa then begin & tt=xb  & xb=xa & xa=tt & endif
                  xw=where((Xout ge xa) and (Xout le xb))
		      endif else xw=i
        ;Find Yout indexes
                if sy gt 1  then begin
                 if dmy eq 2 then begin          ;** bi-dim Y
                  if j eq 0   then ya=Y[i,j] -by*cry else ya=(Y[i,j-1] +Y[i,j]) /2.
                  if j eq sy1 then yb=Y[i,j] +by*cry else yb=(Y[i,j+1] +Y[i,j]) /2.
                  if yb lt ya then begin & tt=yb  & yb=ya & ya=tt & endif
                  yw=where((Yout ge ya) and (Yout le yb))
                 endif else if i eq 0 then begin ;** vector Y
                  if j eq 0   then ya=Y[j]   -by*cry else ya=(Y[j-1]   +Y[j])   /2.
                  if j eq sy1 then yb=Y[j]   +by*cry else yb=(Y[j+1]   +Y[j])   /2.
                  if yb lt ya then begin & tt=yb  & yb=ya & ya=tt & endif
                  yw=where((Yout ge ya) and (Yout le yb))
                 endif
                endif
        ;Fill output
                if (xw[0] ge 0) and (yw[0] ge 0) then begin & for k=0,sz1 do begin ;**Z Loop
                   if okE then Eout[xw,yw,k]=Eout[xw,yw,k]+E2[ i,j,k]
                               Wout[xw,yw,k]=Wout[xw,yw,k]+Win[i,j,k]
                   endfor ;**Z
                               Wcnt[xw,yw,0]=Wcnt[xw,yw,0]+1.
                endif
    endfor  ;**X
  endfor  ;**Y
 endif

 Wcnt=Wcnt>1.
 for k=0,sz1 do Wout[*,*,k]=Wout[*,*,k]/Wcnt
 if okE then for k=0,sz1 do Eout[*,*,k]=sqrt(Eout[*,*,k]/Wcnt)
 Wcnt=0
 
 endif ;treatement of X and/or Y

 ;treat Z dimension now
 ;*********************
 if regZ ne 1 then begin
   if regX+regY ne 2 then Zonly = 0 else Zonly = 1
   Wouz=(fltarr(osx,osy,osz)) & Wcnt=fltarr(osz)
   if Zonly then Wout=Win
   if okE   then if Zonly then E2=E^2 else E2=temporary(Eout)^2
   if okE   then Eout=Wouz else Eout=datp.e
   
   tt=(Z[0:sz1-1]+Z[1:*])/2. & za=[Z[0]-bz*crz,tt] & zb=[tt,Z[sz1]+bz*crz]
   for i=0,sz1 do begin  ;** Loop on Z
        ;Find Zout indexes
                if za[i] le zb[i] then zw=where((Zout ge za[i]) and (Zout le zb[i])) $
                                  else zw=where((Zout ge zb[i]) and (Zout le za[i]))
        ;Fill output
                if (zw[0] ge 0) then $
                 for n=0,n_elements(zw)-1 do begin
                   if okE then Eout[*,*,zw[n]]=Eout[*,*,zw[n]]+E2[  *,*,i]
                               Wouz[*,*,zw[n]]=Wouz[*,*,zw[n]]+Wout[*,*,i]
                               Wcnt[zw[n]    ]=Wcnt[zw[n]  ]  +1.
                 endfor
   endfor
    
   Wout=temporary(Wouz)
   Wcnt=Wcnt>1.
   for k=0,osz-1 do Wout[*,*,k]=Wout[*,*,k]/Wcnt[k]
   if okE then for k=0,osz-1 do Eout[*,*,k]=sqrt(Eout[*,*,k]/Wcnt[k])
   wcnt=0
 endif ;treatment of Z

;Return parameters
;*****************
 if regX ne 1 then MOD_DATP,datp,'X',Xout
 if regY ne 1 then MOD_DATP,datp,'Y',Yout
 if regZ ne 1 then MOD_DATP,datp,'Z',Zout
                   MOD_DATP,datp,'E',temporary(Eout)
 if okDP then GIVE_DATP, datp
 return  ,temporary(Wout)
end
