function teepsd, w_in
;+
;******* *******
;** User callable
;** Converts a 3D array [2Theta , Height , Tof] to energy
;** Simplified version of in5_t2epsd (by JO)
;** ELP is Pn[9]-1 and must be OK
;-

if (size(w_in))[0] ne 3 then return, w_in         ; Only for Tof/Psd image 
nch     = (size(w_in))[3]

take_datp, datp & p_in = datp.p                   ; Get parameters of w_in

okE   = (n_elements(w_in) eq n_elements(datp.E))  ; Check if errors present

Ei         = 81.8049/p_in[21]^2                   ; Ei in meV
telast     = p_in[27]*p_in[21]/3956.035           ; TOF in sec
cwidth     = p_in[18]*1e-6                        ; Chw from mu-sec to sec
epp        = p_in[9]-1.                           ; Elastic Peak Position
flightTime = telast - (epp-findgen(nch))*cwidth
Ef         = Ei*(telast/flightTime)^2
dtdE       = flightTime/(2.*Ef) * 1e6             ; to be concistent with vnorm absolute scalling
kikf       = SQRT(Ei/Ef)                          ; all times above calculated in sec.
mult       = dtdE * kikf

z_out      = Ei - 81.8049/(flightTime*3956.035/p_in[27])^2
w_out      = w_in

for k = 0, nch-1 do begin                         ; For all channels
                w_out[ *,*,k] *= mult[k]
    if okE then datp.e[*,*,k] *= mult[k]
endfor

mod_datp, datp, "Z_tit", ' Energy [meV]'
mod_datp, datp, "Z", z_out
give_datp,datp
return,  w_out
end
