//
//  exploreForDoc.cpp
//  idl2xref
//
//  Created by Yannick Raoul on 5/13/13.
//  Copyright 2013 ILL/ADHOC. All rights reserved.
//

#include "exploreForDoc.h"

#include "tree.h"

#include <iostream>
#include <fstream>

#include <ctime>
#include <string>
#include <iterator>
#include <iostream>
#include <iomanip>
#include <algorithm>
#include <list>
#include <vector>
#include <map>
#include <set>

#ifdef BUILD_WITH_XCODE
#include "idl-parser.tab.h" /* on inclut les constantes definies dans le resultat de yacc */
#else
#include "idl-parser.h" /* pareil, mais avec autoconf/automake */
#endif

extern Node *root;			/*+ racine de l arbre abstrait +*/


//////////////////////////////
//
// DATA STRUCTURE
//
// as in "Algorithms + Data Structures = Programs"
//
// Using c++ std library
//
//////////////////////////////


// 1) Structure to keep track of parsed idl files

struct ParsedIDLFile
{
    std::string baseFileName;
    std::string sourceDirName;
    Node *root;// result of yyparse
};

typedef std::vector<ParsedIDLFile> TIDLFileArray ;
size_t currentFileID;// Index of current parsed file in TIDLFileArray, used to keep a link below



// 2) List of routine used in a locally defined routine.

struct OutsideRoutineReference
{
    std::string rname;  // name of the routine, used as key
    int rtype;          // Either PROCEDURE_CALL or FUNCTION_CALL
    int fileID;         // index in TIDLFileArray, see above
    int lineNumber;     // line of reference in that file
};

typedef std::list<OutsideRoutineReference> TOutsideRefList ;




// 3) Set of locally defined routine.
//    ie defined in one of the "ParsedIDLFile" above

struct RoutineDefinition
{
    std::string dname;      // name of the routine, used as key
    int dtype;              // Either DECLARATION_PROC or DECLARATION_FUNC
    int fileID;             // index in TIDLFileArray
    int lineNumber;         // line of definition in that file
    TOutsideRefList outsideRefList;  // list of routines referenced in this definition
};

// compare on RoutineDefinition, used to insert in set 
bool operator<(const RoutineDefinition& l, const RoutineDefinition& r) {
    // Has to distinguish proc and function with same name 
    return ( l.dname==r.dname ? l.dtype<r.dtype : l.dname<r.dname );
}

// No need to have ==, because std uses "!(a<b) && !(b<a)"  !

typedef std::set<RoutineDefinition> TRDefSet ;



// 4) Map of all references.

// Key of TRefIDTLocationListMap below
struct RoutineRefID
{
    std::string rname;  // name of the routine, used as key
    int rtype;           // Either PROCEDURE_CALL or FUNCTION_CALL
};

// compare on RoutineRefID, used to insert in map 
bool operator<(const RoutineRefID& l, const RoutineRefID& r) {
    // Has to distinguish proc and function with same name 
    return ( l.rname==r.rname ? l.rtype<r.rtype : l.rname<r.rname );
}

// TTokenLocationList is the Value of TRefIDTLocationListMap below
struct TokenLocation
{
    int fileID;         // index in TIDLFileArray
    int lineNumber;     // line of reference in that file
};
typedef std::list<TokenLocation> TTokenLocationList ;

typedef std::map <RoutineRefID, TTokenLocationList > TRefIDTLocationListMap;
typedef std::pair<RoutineRefID, TTokenLocationList > TRefIDTLocationListPair;


//////////////////////////////
//
// END OF DATA STRUCTURE
//
//////////////////////////////


// Global variable to keep track of parsed module
TIDLFileArray gidl_collection;

// Global map to keep track of definitions
TRDefSet gdef_set;
//RoutineDefinition gMainRoutineDefinition;
const RoutineDefinition * gCurrentRoutineDefinition=NULL;
TOutsideRefList * gCurrentOutsideRefList=NULL;

// Global variable to keep track of references
TRefIDTLocationListMap gref_list_map;




// internal C++ routine
void outputXRefReport(std::ostream & dest);
void outputDefsAndRefs(std::ostream & dest); 
void outputDefsAndCalls(std::ostream & dest);
void outputFileList(std::ostream & dest);

char* first_string_in_parameters_list(Node *n);


/* ecrit au terminal des informations sur le noeud */
void printCrossRefNode(Node *n) {
    switch(n->typeNode){
        case BLOCK:
            //printf("BLOCK               ");
            break;
        case DECLARATION_LIST:
            //printf("DECLARATION_LIST    ");
            break;
        case  DECLARATION_PROC:
            printf("DECL_PROC: %s\n",(n->valNode).uString);
            break;
        case  DECLARATION_FUNC:
            printf("DECL_FUNC: %s\n",(n->valNode).uString);
            break;
        case  COMMON_CASE :
            //printf("COMMON_CASE         ");
            break;
        case  SUITE_IDENT:
            //printf("SUITE_IDENT         ");
            break;
        case  SUITE_PARAM:
            printf("SUITE_PARAM         \n");
            break;
        case  Catch:
            //printf("CATCH               ");
            break;
        case  PARAM	:
            printf("PARAM               \n");
            break;
        case  PARAM_EXTRA:
            //printf("PARAM_EXTRA         ");
            break;
        case  PARAM_REF_EXTRA:
            //printf("PARAM_REF_EXTRA     ");
            break;
        case  STATEMENT:
            //printf("STATEMENT           ");
            break;
        case  STATEMENT_LIST:
            //printf("STATEMENT_LIST      ");
            break;
        case  STATEMENT_LIST_ET:
            //printf("STATEMENT_LIST_ET   ");
            break;
        case COMMENTSTATEMENT:
            //printf("COMMENTSTATEMENT      ");
            break;
        case  PARENTHESE:
            //printf("PARENTHESE          ");
            break;
        case  AROBASE:
            //("AROBASE             ");
            break;
        case  AROBASE_POINT	:
            //printf("AROBASE_POINT       ");
            break;
        case  RETURN:
            //printf("RETURN              ");
            break;
        case  PLUS	:
            //printf("PLUS                ");
            break;
        case  MINUS	:
            //printf("MINUS               ");
            break;
        case  PUISS	:
            //printf("PUISS               ");
            break;
        case  PROCEDURE_CALL:
            printf("PROCEDURE_CALL %s      \n",(n->valNode).uString);
            break;
        case  FUNCTION_CALL	:
            printf("FUNCTION_CALL %s      \n",(n->valNode).uString);
            break;
        case  SUITE_CALL_LIST:
            //printf("SUITE_CALL_LIST     ");
            break;
        case  SUITE_CALL:
            printf("SUITE_CALL          ");
            break;
        case  STRUCTURE:
            //printf("STRUCTURE           ");
            break;
        case  SUITE_CONS:
            //printf("SUITE_CONS          ");
            break;
        case  REF_STRUCT:
            //printf("REF_STRUCT          ");
            break;
        case  MATRIX:
            //printf("MATRIX              ");
            break;
        case  SUITE_MATRIX	:
            //printf("SUITE_MATRIX        ");
            break;
        case  REF_MATRIX	:
            //printf("REF_MATRIX %s       \n",(n->valNode).uString);
            break;
        case  PARAM_MATRIX_SUITE:
            //printf("PARAM_MATRIX_SUITE  ");
            break;
        case  PARAM_MATRIX_ETOILE	:
            //printf("PARAM_MATRIX_ETOILE ");
            break;
        case  INTERVALLE	:
            //printf("INTERVALLE          ");
            break;
        case  CROCHET	:
            //printf("CROCHET             ");
            break;
        case  SUITE_EXPRESSION:
            //printf("SUITE_EXPRESSION    ");
            break;
        case  FIN_IF	:
            //printf("FIN_IF              ");
            break;
        case  VAR_SYSTEM:
            //printf("VAR_SYSTEM          ");
            break;
        case  FIN_ELSE	:
            //printf("FIN_ELSE            ");
            break;
        case  CASE_STATEMENT:
            //printf("CASE_STATEMENT      ");
            break;
        case  CASE_STATEMENT_SUITE:
            //printf("CASE_STATEMENT_SUITE");
            break;
        case  CASE	:
            //printf("CASE                ");
            break;
        case  CASE_ELSE:
            //printf("CASE_ELSE           ");
            break;
        case  CASE_SUITE:
            //printf("CASE_SUITE          ");
            break;
        case FUNCTION_CALL_OU_REF_MATRIX:
            //printf("FUNCorMATR %s\n", (n->valNode).uString);
            break;
        case  NAME	: /* on ecrit ici l'identificateur contenu dans le noeud */
//            if ((n->fd==NULL) && (n->fg==NULL)) {
//                printf("NAME : %s      \n",(n->valNode).uString);
//            } else {
//                printf("NAME                \n");
//            }
            break;
        case  WHILE	:
            //printf("WHILE               ");
            break;
        case  FIN_WHILE:
            //printf("FIN_WHILE           ");
            break;
        case  REPEAT_STATEMENT:
            //printf("REPEAT_STATEMENT    ");
            break;
        case  FIN_REPEAT:
            //printf("FIN_REPEAT          ");
            break;
        case  FOR:
            //printf("FOR                 ");
            break;
        case  FIN_FOR:
            //printf("FIN_FOR             ");
            break;
        case  FOR_COND:
            //printf("FOR_COND            ");
            break;
        case  EXPRESSION_FAC:
            //printf("EXPRESSION_FAC      ");
            break;
        case EXP_OU_FUNCALL:
            printf("EXP_OU_FUNCALL      \n");
            break;
        case  ASSIGNMENT:
            //printf("ASSIGNMENT : %s",(n->valNode).uString);
            break;
        case 	IDENTIFIER:
            //printf("IDENT : %s",(n->valNode).uString);
            break;
        case 	INTEGER	:
            //printf("INTEGER : %d   ",(n->valNode).uInt);
            break;
        case 	REAL	:
            //printf("REAL                ");
            break;
        case 	DECIMAL	:
            //printf("DECIMAL             ");
            break;
        case 	HEXADECIMAL	:
            //printf("HEXADECIMAL         ");
            break;
        case 	OCTAL	:
            //printf("OCTAL               ");
            break;
        case 	STRING	:
            printf("STRING : %s\n",(n->valNode).uString);
            break;
        case 	End	:
            //printf("End                 ");
            break;
        case 	Begin	:
            //printf("Begin               ");
            break;
        case 	If	:
            //printf("If                  ");
            break;
        case 	Then	:
            //printf("Then                ");
            break;
        case 	Of	:
            //printf("Of                  ");
            break;
        case 	Or	:
            //printf("Or                  ");
            break;
        case ORSHORTCUT :
            //printf("||                  ");
            break;
        case 	 Case	:
            //printf("Case                ");
            break;
        case 	And	:
            //printf("And                 ");
            break;
        case ANDSHORTCUT :
            //printf("&&                  ");
            break;
        case 	LT	:
            //printf("LT                  ");
            break;
        case 	LE	:
            //printf("LE                  ");
            break;
        case 	UMINUS	:
            //printf("UMINUS              ");
            break;
        case 	UPLUS	:
            //printf("UPLUS               ");
            break;
        case PlusPlus :
            //printf("++                  ");
            break;
        case MoinsMoins :
            //printf("--                  ");
            break;
        case 	GT	:
            //printf("GT                  ");
            break;
        case GE	:
            //printf("GE                  ");
            break;
        case EQ	:
            //printf("EQ                  ");
            break;
        case NE	:
            //printf("NE                  ");
            break;
        case Pro	:
            printf("Pro                 ");
            break;
        case Not	:
            //printf("Not                 ");
            break;
        case TILDE	:
            //printf("~                 ");
            break;
        case TIMES	:
            //printf("TIMES               ");
            break;
        case SLASH	:
            //printf("SLASH               ");
            break;
        case Mod	:
            //printf("Mod                 ");
            break;
        case While	:
            //printf("While               ");
            break;
        case Repeat	:
            //printf("Repeat              ");
            break;
        case For	:
            //printf("For                 ");
            break;
        case Endrep	:
            //printf("Endrep              ");
            break;
        case Endwhile:
            //printf("Endwhile            ");
            break;
        case Endfor	:
            //printf("Endfor              ");
            break;
        case Do	:
            //printf("Do                  ");
            break;
        case Until	:
            //printf("Until               ");
            break;
        case Assign	:
            //printf("Assign              ");
            break;
        case Else	:
            //printf("Else                ");
            break;
        case Endelse	:
            //printf("Endelse             ");
            break;
        case Endif	:
            //printf("Endif               ");
            break;
        case Function:
            //printf("Function            ");
            break;
        case Common	:
            //printf("Common              ");
            break;
        case Extra	:
            //printf("Extra               ");
            break;
        case Ref_Extra	:
            //printf("Ref_Extra           ");
            break;
        case Endcase	:
            //printf("Endcase             ");
            break;
        case Return	:
            //printf("Return              ");
            break;
        case INF	:
            //printf("INF                 ");
            break;
        case SUP	:
            //printf("SUP                 ");
            break;
        case OBJECT	:
            //printf("OBJECT              ");
            break;
        case REF_STRUCT_PARENTHESE:
            //printf("REF_STRUCT_PARENT   ");
            break;
        case INTERRO:
            //printf("INTERRO             ");
            break;
        case DECL_PROC_OBJ:
            //printf("DECL_PROC_OBJ       ");
            break;
        case DECL_FUNC_OBJ:
            //printf("DECL_FUNC_OBJ       ");
            break;
        case METHODE_CALL:
            printf("METHODE_CALL %s      \n",(n->valNode).uString);
            break;
        case IDENTIFIER_PARENTHESE:
            //printf("IDENTIFIER_PARENTHESE");
            break;
        case FORWARD_FUNCTION:
            //printf("FORWARD_FUNCTION    ");
            break;
        case FLECHE:
            //printf("FLECHE              ");
            break;
        case GESTION_ERREUR:
            //printf("GESTION_ERREUR      ");
            break;
        case NAMED_STRUCTURE:
            //printf("NAMED_STRUCTURE     ");
            break;
        case DIESE :
            //printf("DIESE               ");
            break;
        case DIESE2:
            //printf("DIESE2              ");
            break;
        case COMPILE_OPT:
            //printf("COMPILE_OPT         ");
            break;
        case ACOMMENT:
            //printf("ACOMMENT : %s",(n->valNode).uString);
            break;
        case CR:
            //printf("CR :                ");
            break;
        case INCLUDE:
            //printf("INCLUDE : %s",(n->valNode).uString);
            break;
        case REF_STRUCT_LIST:
            //printf("REF_STRUCT_LIST     ");
            break;
        default:
            printf(" CAS NON TRAITE = %d\n", n->typeNode);
    }
    //(" l:%d ", n->lineInSource);
}




/*
 * Utility to find the first 'string' parameter in a parameter list
 * used in function "call_function" or proc "call_procedure" context
 */
char* first_string_in_parameters_list(Node *n) {
            
    if (n==NULL)
        return(NULL);

    switch(n->typeNode){
        
        case STRING:
            return((n->valNode).uString);
            break;
        
        case PARENTHESE:
            return(first_string_in_parameters_list(n->fg));

        case SUITE_CALL_LIST:
            return(first_string_in_parameters_list(n->fg));
            
        default:
            //printf("%d\n", n->typeNode);
            return(NULL);

    }
    
}



/* Extrait les routines definies ou appelees dans le noeud */
void extractCrossRefInfo(Node *n) {
    TRefIDTLocationListMap::iterator p_refloc;

    switch(n->typeNode){
        
            
        case  PROCEDURE_CALL:
        case  FUNCTION_CALL	:
        case  METHODE_PROCEDURE_CALL :
        case  METHODE_FUNCTION_CALL	:
        {
            
            // We collect all references to function or proc, here.
            // discrimination between internal (IDL) routine and user one will occur during report phase,
            // as we have to "collect them all" before.
        
            if (n->typeNode==METHODE_PROCEDURE_CALL || n->typeNode==METHODE_FUNCTION_CALL) {
//                if (displayMessage) {
//                    std::cerr << "Methode call " << (n->valNode).uString << std::endl;
//                }
            }
            
            TokenLocation tmpTokenLocation;
            tmpTokenLocation.fileID = (int)currentFileID;
            tmpTokenLocation.lineNumber = n->lineInSource ;
            RoutineRefID tmpRoutineRefID;
            tmpRoutineRefID.rname = (n->valNode).uString;
            tmpRoutineRefID.rtype = n->typeNode; 

            p_refloc = gref_list_map.find(tmpRoutineRefID);
            if (p_refloc == gref_list_map.end()) { // not found, this is the first occurence
                TTokenLocationList tmpList;
                tmpList.push_back(tmpTokenLocation);
                gref_list_map.insert(TRefIDTLocationListPair(tmpRoutineRefID, tmpList));
            } else {
                p_refloc->second.push_back(tmpTokenLocation);
            }
            
            // Insert this reference in the list of current PROC/FUNCTION
            OutsideRoutineReference tmpOutsideRoutineReference;
            tmpOutsideRoutineReference.rname = (n->valNode).uString;
            tmpOutsideRoutineReference.rtype = n->typeNode;
            tmpOutsideRoutineReference.fileID = (int)currentFileID;
            tmpOutsideRoutineReference.lineNumber = n->lineInSource ;
            
            if (gCurrentOutsideRefList!=NULL) {
                gCurrentOutsideRefList->push_back(tmpOutsideRoutineReference);
            }
            
            // Special case for "call_function" and "call_procedure"
            
            if (   std::string((n->valNode).uString) == "call_function"
                || std::string((n->valNode).uString) == "call_procedure" )
            {
                
                // Start a new tree exploration to find the first parameter.
                // It can be as simple as "q = CALL_FUNCTION('tosca', e_i)"
                // but also "call_function(self.func,..."
                // or "V= CALL_FUNCTION('dial_'+DD.generic+'_read',tipe)"
                // We only look for string litteral
                char* proc_or_func_name = first_string_in_parameters_list(n->fg);
                
                if (proc_or_func_name!=NULL)
                {
                    OutsideRoutineReference tmp2OutsideRoutineReference;
                    // strip ' from string to obtain routine name
                    tmp2OutsideRoutineReference.rname = std::string(proc_or_func_name, 1, strlen(proc_or_func_name)-2);
                    tmp2OutsideRoutineReference.rtype = n->typeNode;
                    tmp2OutsideRoutineReference.fileID = (int)currentFileID;
                    tmp2OutsideRoutineReference.lineNumber = n->lineInSource ;
                    
                    if (gCurrentOutsideRefList!=NULL) {
                        gCurrentOutsideRefList->push_back(tmp2OutsideRoutineReference);
                    }

                }
                
            }
        }
        break;

        case  DECLARATION_PROC:
        case  DECLARATION_FUNC: {
            RoutineDefinition tmpCurrentRoutineDefinition;
            tmpCurrentRoutineDefinition.dname = (n->valNode).uString;
            tmpCurrentRoutineDefinition.dtype = n->typeNode;
            tmpCurrentRoutineDefinition.fileID = (int)currentFileID;
            tmpCurrentRoutineDefinition.lineNumber = n->lineInSource ;

            std::pair<TRDefSet::iterator, bool> insert_result = gdef_set.insert(tmpCurrentRoutineDefinition);
            if (!insert_result.second) {
                // Already there, write a warning if asked
                if (writeWarning) {
                    std::cerr << "Warning, '" << (n->valNode).uString << "' seems to have double definition" << std::endl ;
                    std::cerr << "\tCheck "
                    << insert_result.first->lineNumber << ":" << gidl_collection[insert_result.first->fileID].baseFileName
                    << " and "
                    << tmpCurrentRoutineDefinition.lineNumber << ":" << gidl_collection[currentFileID].baseFileName << std::endl ;
                }
            }
            
            // Then update our global variable used to build a list of locally called proc and func
            gCurrentOutsideRefList = (TOutsideRefList *) &(insert_result.first->outsideRefList) ;// cast to non const !
            // It should be another way than this cast, but [14/10/2013] I am still learning

        }
        break;           
        default:;
    }
}

/* visite un noeud */
void visitNode (Node *n, int tab, int bfg) {
    /* tab indique le niveau de tabulation courant */
    /* bfg est un boolean indiquant si le node est un fils gauche de son pere */
    /* il sert pour la bonne marche de l'affichage */

//    int i;
    /* si c'est un fils gauche, on devra ecrire la tabulation pour l'aligne avec le reste */
//    if (bfg==1) { /* on aligne le node */
//        printf("  \t");
//        for(int i=1;i<3*tab;i++) {
//            printf("\t");
//        }
//    }

    if (n != NULL) {
        //printf("->");
        //if (displayMessage)
        //    printCrossRefNode(n); /* on sort ce qui nous interesse */
        
        extractCrossRefInfo(n);
        
        /* on affiche les fils par recurence */
        visitNode(n->fg, tab + 1, 0); /* on affiche le fils gauche */
        visitNode(n->fd, tab + 1, 1); /* on affiche le fils droit */
    }
    
}




/* lance la visite de l arbre abstrait */
void visitTree(Node *n) {
    if (n == NULL) {
        if (displayMessage)
            fprintf(stderr, "L'arbre abstrait est vide\n");
    } else {
        visitNode(n, 0, 0); /* on affiche la racine avec les bons parametres */
    }
}


void startIDLXRef(int nbFiles){
    
    gidl_collection.clear();
    gidl_collection.reserve(nbFiles);// avoid some reallocate
    gdef_set.clear();
    gref_list_map .clear();
    
}


void collectIDL_Def_and_Ref (char * baseFileName, char * sourceDirName) {  
    
    // Keep track of parsed files
    ParsedIDLFile tmpParsedIDLFile;
    
    tmpParsedIDLFile.baseFileName = baseFileName;
    tmpParsedIDLFile.sourceDirName =  sourceDirName;
    tmpParsedIDLFile.root = root;
    
    gidl_collection.push_back(tmpParsedIDLFile);
    currentFileID = gidl_collection.size()-1;
    gCurrentRoutineDefinition=NULL;// Just to say "no current routine definition"
    gCurrentOutsideRefList=NULL;

    visitTree(root);
    
}// collectIDL_Def_and_Ref


void printIDLXRef(bool full_list){

    TIDLFileArray::iterator fileIterator;

    if (full_list) {
        // Raw dump of our data
        std::cout << "Files :" << std::endl;

        for(fileIterator = gidl_collection.begin(); 
            fileIterator != gidl_collection.end();
            fileIterator++)
        {
            std::cout << fileIterator->baseFileName << std::endl;
        }
        
        std::cout << std::endl;

        std::cout << "Defs :" << std::endl;
        TRDefSet::iterator its;
        // Show key
        for(its = gdef_set.begin(); its!=gdef_set.end(); ++its)
        {
            std::cout << its->dname << std::endl;
        }    
        std::cout << std::endl;

        
        std::cout << "Refs :" << std::endl;
        TRefIDTLocationListMap::iterator p2;
        // Show key
        for(p2 = gref_list_map.begin(); p2!=gref_list_map.end(); ++p2)
        {
            std::cout << p2->first.rname << "\t (" << p2->second.size() << " refs)" << std::endl;
            std::cout << "\t";
            TTokenLocationList::iterator it;// Iter on the RoutineReference list
            for(it = p2->second.begin(); it!=p2->second.end(); ++it)
            {
                std::cout << " <" << it->fileID << "," << it->lineNumber << "> ";
            }
            std::cout << std::endl;
        }    
        std::cout << std::endl;
    
    } else {

        outputXRefReport(std::cout) ;
        
    }// if (full_list)

}// printIDLXRef
    



// Print report in a file
void outputFormattedXref(char * destFileName) {
    
    if (destFileName[0]=='\0') {
        outputXRefReport(std::cout) ;
    } else {
        std::fstream outfile(destFileName, std::ios::out);
        if (!outfile.is_open()) {
            std::cout << "Error: cannot write to file: " << destFileName << std::endl;
            return;
        }

        outputXRefReport(outfile);
        outfile.close();
    }
}


// Output report to an output stream
void outputXRefReport(std::ostream & dest){

    time_t t = time(0);
    struct tm * now = localtime( & t );

    dest << "IDL XREF REPORT" << std::endl ;
    dest << "===============" << std::endl << std::endl;

    dest << "generated on " << asctime(now) << std::endl;

    outputDefsAndRefs(dest);
    outputDefsAndCalls(dest);
    outputFileList(dest);
}


//
// Output Definitions and references, sorted alphabeticaly by routine name
//
void outputDefsAndRefs(std::ostream & dest)
{
    // Formatted and filtered by "Defined here" criteria
    
    dest << "Alphabetical list of defined routines, followed by references" << std::endl ;
    dest << "-------------------------------------------------------------" << std::endl ;
    // 1) Loop on "routines defined in the set of parsed files".
    TRDefSet::iterator it1;
    for(it1 = gdef_set.begin(); it1!=gdef_set.end(); ++it1) {
        dest << it1->dname ;// Name of defined routine
        
        if (it1->dtype == DECLARATION_PROC)
            dest << " P "; 
        else if (it1->dtype == DECLARATION_FUNC)
            dest << " F "; 
        else if (it1->dtype == DECL_PROC_OBJ)
            dest << " OP "; 
        else if (it1->dtype == DECL_FUNC_OBJ)
            dest << " OF ";
        
        dest    << "["
                << gidl_collection.at(it1->fileID).baseFileName
                 << ":" << it1->lineNumber
                << "]" ;
        
        
        RoutineRefID tmpRoutineRefID;
        tmpRoutineRefID.rname = it1->dname;
        if (it1->dtype == DECLARATION_PROC) {
            tmpRoutineRefID.rtype =PROCEDURE_CALL;
        } else if (it1->dtype == DECLARATION_FUNC) {
            tmpRoutineRefID.rtype =FUNCTION_CALL;
        } else if (it1->dtype == DECL_PROC_OBJ) {
            tmpRoutineRefID.rtype =PROCEDURE_CALL;
        } else if (it1->dtype == DECL_FUNC_OBJ) {
            tmpRoutineRefID.rtype =FUNCTION_CALL;
        }
        
        TRefIDTLocationListMap::iterator p2 = gref_list_map.find(tmpRoutineRefID);
        if (p2 == gref_list_map.end()) {
            dest << "\t(No reference)" << std::endl;
        } else {
            // Found a list of references
            if (p2->second.size()==1)
                dest << "\t(1 reference)";
            else
                dest << "\t(" << p2->second.size() << " references)";
            
            int currentFileID = -1;// none
            TTokenLocationList::iterator it;// Iter on the RoutineReference list
            for(it = p2->second.begin(); it!=p2->second.end(); ++it) {
                
                // By construct, fileID are grouped.
                // So we keep the same line until fileID change
                // Line number are not ordered for now
                if (it->fileID!=currentFileID) {
                    // Print path and name of file on a new line
                    dest << std::endl << "\t"
                        << gidl_collection.at(it->fileID).sourceDirName << PATHSEP
                        << gidl_collection.at(it->fileID).baseFileName << "\t" ;
                    
                    currentFileID = it->fileID;
                    dest << it->lineNumber;
                } else {
                    dest << ", " << it->lineNumber;
                }
                
            }// iter on list of references
            dest << std::endl;
        }
        dest << std::endl;
    }// iter on defined routines

    
}// outputDefsAndRefs







//
// Output Definitions and calls, sorted alphabeticaly by routine name
//
void outputDefsAndCalls(std::ostream & dest)
{
    using namespace std;

    
    dest << std::endl << std::endl;
    dest << "Alphabetical list of defined routines, followed by calls" << std::endl ;
    dest << "--------------------------------------------------------" << std::endl << std::endl;
    // 1) Loop on defined here.

    TRDefSet::iterator its;
    TOutsideRefList::const_iterator it_refs;// Why const ???
    
    for(its = gdef_set.begin(); its!=gdef_set.end(); ++its) {
        dest << its->dname ;// Name of defined routine
        
        if (its->dtype == DECLARATION_PROC)
            dest << " P "; 
        else if (its->dtype == DECLARATION_FUNC)
            dest << " F "; 
        else if (its->dtype == DECL_PROC_OBJ)
            dest << " OP "; 
        else if (its->dtype == DECL_FUNC_OBJ)
            dest << " OF ";
        
        dest << "["
            << gidl_collection.at(its->fileID).baseFileName
            << ":" << its->lineNumber 
            << "]"
            << std::endl;
        
        
        // First, count outside reference
        std::map<std::string, size_t>  word_map; // used to count words

        for(it_refs = its->outsideRefList.begin(); it_refs!=its->outsideRefList.end(); ++it_refs) {
            RoutineDefinition tmpRD;
            tmpRD.dname = it_refs->rname;
            if (it_refs->rtype==PROCEDURE_CALL)
                tmpRD.dtype = DECLARATION_PROC;
            else if ((it_refs->rtype==FUNCTION_CALL))
                tmpRD.dtype = DECLARATION_FUNC;

            
            // Count only if reference belongs to "defined here" set
            if (gdef_set.find(tmpRD) != gdef_set.end()){
                ++word_map[it_refs->rname];
            }
        }
        
        // Then output this condensed list, if not empty
        if (word_map.size()>0) {
            dest << "  Inside calls:" << std::endl;
            std::map<std::string, size_t>::const_iterator itr;
            for (itr = word_map.begin(); itr != word_map.end(); ++itr) {
                dest << "\t" << (*itr).first ;
                if ((*itr).second > 1)
                    dest << " (" << (*itr).second << ")";
                dest << std::endl;
            }
        }
        
        
        dest << std::endl;
    }// iter on defined routines

    
}// outputDefsAndCalls




//
// Output used file list
//
void outputFileList(std::ostream & dest)
{
    using namespace std;

    dest << std::endl << std::endl;
    dest << "List of parsed files" << std::endl ;
    dest << "--------------------" << std::endl << std::endl;

    TIDLFileArray::iterator fileIterator;
    int file_index = 1;
    
    for(fileIterator = gidl_collection.begin(); 
        fileIterator != gidl_collection.end();
        fileIterator++)
    {
        dest << "[" << file_index << "]\t" << fileIterator->baseFileName << std::endl;
        file_index++;
    }
    
}
