
/* %option header-file="idl-lexer.h" */


%{
/******************************************************************************
*                                IDL2XREF Project
*-----------------------------------------------------------------------------
*   ILL (Institut Laue Langevin)
*
*   38000 GRENOBLE Cedex
*-----------------------------------------------------------------------------
*   Module              :       Syntax description of the IDL Language
*   Auteurs             :       Azizi Mourier Karim
                                Benzeghioua Abdeslam
                                Gardon Lucien
                                Sylvestre Nadege
                                Bourtembourg Reynald
                                Yannick Raoul
*   Date                :       February 21 2002
*   Modifications       :       September 19 2003
*                               Novembre 11 2013
*
*****************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"

#ifdef BUILD_WITH_XCODE
#include "idl-parser.tab.h" /* on inclut les constantes definies dans le resultat de yacc */
#else
#include "idl-parser.h" /* pareil, mais avec autoconf/automake */
#endif


yy_size_t larger_string_size; /*+ taille de la plus grande chaine rencontrée +*/

extern YYLTYPE yylloc;

char* lexError=NULL;
    
int nbOccCharInString(char c, char * s);
int addComment(char* str, int nLine);
void my_flush_buffer();

#define YY_USER_INIT yylloc.first_line=1;

%}


space 		[ \t]
bl 		    {space}+
eol         (\r*\n)
digit 		[0-9]
character 	[a-zA-Z]
hexa 		[a-fA-F]
oct		    [0-7]
digit_char 	({digit}|{character}|"$"|"_")
real  		({digit}*"."(({digit}+([eEdD][-+]?({digit})+)?)|({digit}+[eEdD]))|{digit}+"."(({digit}*([eEdD][-+]?({digit})*)?)|({digit}+[eEdD])))
exposant  	{digit}+[eEdD][-+]?{digit}*
decimal		{digit}+[Uu]?([Bb]|[lL]|[sS]|[lL][lL])?
integer  	{digit}+
identifier 	[_]*{character}{digit_char}*
hexadecimal	[']({digit}|{hexa})+['][Xx][Uu]?[sS]?([Bb]|[Ll]|[lL][lL])?
octal		((["]{oct}+)|([']{oct}+['][Oo]))[uU]?([Ll]|[Bb]|[lL][lL])?
sep 		{bl}
strings		((['][^'\r\n]*['])*|(["][^"\r\n]*["])*)
comment     ;.*

/*------------------------------*/
%%
[eE][lL][sS][eE]{space}*":" {return(ElseCase);}
{sep} ;			/* saute les blancs et tabulations */


"##=" {return(DIESE2EQ);} /* Nouveaux operateurs d'IDL 6.0 */
"#=" {return(DIESEEQ);}
"*=" {return(TIMESEQ);}
"+=" {return(PLUSEQ);}
"-=" {return(MOINSEQ);}
"/=" {return(SLASHEQ);}
"<=" {return(MINEQ);}
">=" {return(MAXEQ);}
[aA][nN][dD]= {return(ANDEQ);}
[eE][qQ]= {return(EQEQ);}
[gG][eE]= {return(GEEQ);}
[gG][tT]= {return(GTEQ);}
[lL][eE]= {return(LEEQ);}
[lL][tT]= {return(LTEQ);}
[mM][oO][dD]= {return(MODEQ);}
[nN][eE]= {return(NEEQ);}
[oO][rR]= {return(OREQ);}
[xX][oO][rR]= {return(XOREQ);}
"^=" {return(POWEREQ);}
"&&" {return(ANDSHORTCUT);}   /* Nouveaux operateurs logiques */
"||" {return(ORSHORTCUT);}
"~" {return(TILDE);}
"++" {return(PlusPlus);}      /* operateur ++ */
"--" {return(MoinsMoins);}    /* operateur -- */
"**" {return('^');}/* to be confirmed */

[fF][Oo][Rr][Ww][Aa][Rr][Dd][_][Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn]  {return(Forward_function);}
[cC][oO][mM][pP][iI][lL][eE][_][Oo][pP][tT]    	{return(COMPILE_OPT);}
[bB][eE][gG][iI][nN] 			{return(Begin);}
[eE][nN][dD] 				{return(End);}
[iI][fF]	 			{return(If);}
[tT][hH][eE][nN] 			{return(Then);}
[oO][fF]	 			{return(Of);}
[cC][aA][sS][eE] 			{return(Case);}
[sS][wW][iI][tT][cC][hH]	{return(Switch);}
[oO][rR]	 			{return(Or);}
[aA][nN][dD]	 			{return(And);}
[lL][tT]	 			{return(LT);}
[gG][tT]	 			{return(GT);}
[lL][eE]	 			{return(LE);}
[gG][eE]	 			{return(GE);}
[eE][qQ]	 			{return(EQ);}
[nN][eE]	 			{return(NE);}
[pP][rR][oO]	 			{return(Pro);}
[nN][oO][tT]	 			{return(Not);}
[mM][oO][dD]	 			{return(Mod);}
[wW][hH][iI][lL][eE] 			{return(While);}
[rR][eE][pP][eE][aA][tT]		{return(Repeat);}
[fF][oO][rR]		 		{return(For);}
[eE][nN][dD][rR][eE][pP]		{return(Endrep);}
[eE][nN][dD][wW][hH][iI][lL][eE]	{return(Endwhile);}
[eE][nN][dD][fF][oO][rR]		{return(Endfor);}
[dD][oO]				{return(Do);}
[uU][nN][tT][iI][lL]			{return(Until);}
[eE][nN][dD][eE][lL][sS][eE]		{return(Endelse);}
[eE][lL][sS][eE]			{return(Else);}
[eE][nN][dD][iI][fF]			{return(Endif);}
[fF][uU][nN][cC][tT][iI][oO][nN]	{return(Function);}
[cC][oO][mM][mM][oO][nN]		{return(Common);}
"_EXTRA"              {return(Extra);}/* [_][eE][xX][tT][rR][aA]			{return(Extra);}*/
"_REF_EXTRA"          {return(Ref_Extra);}/* [_][rR][eE][fF][_][eE][xX][tT][rR][aA]  {return(Ref_Extra);}*/
[eE][nN][dD][cC][aA][sS][eE]            {return(Endcase);}
[eE][nN][dD][sS][wW][iI][tT][cC][hH] 	{return(Endswitch);}
[rR][eE][tT][uU][rR][nN]		{return(Return);}
[cC][aA][tT][cC][hH]			{return(Catch);}
[fF][oO][rR][eE][aA][cC][hH]                {return(Foreach);}
[eE][nN][dD][fF][oO][rR][eE][aA][cC][hH]	{return(Endforeach);}


[xX][oO][rR]  	      	      	      	{return(XOR);}
"##"        	      	      	      	{return(DIESE2);}
                                
INHERITS                        {return(INHERITS);}/* [iI][nN][hH][eE][rR][iI][tT][sS]      	{return(INHERITS);} */


"+" {return('+');}
"-" {return('-');}
"*" {return('*');}
"/" {return(SLASH);}
"=" {return('=');}
"," {return(',');}
"{" {return('{');}
"}" {return('}');}
"(" {return('(');}
")" {return(')');}
"[" {return('[');}
"]" {return(']');}
":" {return(':');}
"." {return('.');}
"!" {return('!');}
"#" {return(DIESE);}
">" {return('>');}
"<" {return('<');}
"^" {return('^');}
"->" {return(FLECHE);}
"?" {return('?');}

"["{space}*"]"      {return(NULL_ARRAY);}
"{"{space}*"}"      {return(NULL_STRUCT);}

"$"{space}*{eol}                        {yylloc.first_line++;}
"$"{space}*{comment}{eol}               {addComment(yytext, yylloc.first_line); yylloc.first_line++;}
"$"{space}*({eol})+{space}*({eol})*     {yylloc.first_line = yylloc.first_line + nbOccCharInString('\n',yytext);}
"$"{space}*({eol})+({space}*{comment}{eol})*  {yylloc.first_line = yylloc.first_line + nbOccCharInString('\n',yytext);addComment(yytext, yylloc.first_line);}
"$"{space}*{comment}({eol})+({space}*{comment}{eol})*  {yylloc.first_line = yylloc.first_line + nbOccCharInString('\n',yytext);addComment(yytext, yylloc.first_line);}


({comment}|{bl}|{eol})* { addComment(yytext, yylloc.first_line); yylloc.first_line = yylloc.first_line + nbOccCharInString('\n',yytext);return(CR);}


"&"                     {return(CR);}	/* on retourne un CR sur un & */

\@{identifier}(\.{identifier})*    {strcpy(yylval.uChar,yytext);return (INCLUDE);}


{comment}       {/*strcpy(yylval.uChar,yytext)*/ addComment(yytext, yylloc.first_line); /* return (CR); hide comments */ }
{exposant}      {strcpy(yylval.uReal,yytext);return(REAL);}
{real}          {strcpy(yylval.uReal,yytext);return(REAL);}
{decimal}       {strcpy(yylval.uChar,yytext);return (DECIMAL);}
{hexadecimal} 	{strcpy(yylval.uChar,yytext);return (HEXADECIMAL);}
{octal}         {strcpy(yylval.uChar,yytext);return (OCTAL);}
{identifier}  	{strcpy(yylval.uChar,yytext);return (IDENTIFIER);}
{integer}       {yylval.uInt=atoi(yytext);return(INTEGER);}
{strings}       {
                    if (yyget_leng()>larger_string_size)
                        larger_string_size = yyget_leng();
    
                    if (yyget_leng()<AST_MAX_STRING_SIZE-2) {
                        strncpy(yylval.uChar,yytext,yyget_leng());
                        yylval.uChar[yyget_leng()]='\0';
                        return (STRING);
                    } else {
                        strncpy(yylval.uChar,yytext,AST_MAX_STRING_SIZE-2);
                        yylval.uChar[AST_MAX_STRING_SIZE-1] = '\0';
                        return (STRING);
                    }
                }

. {
   if (lexError!=NULL) {
     lexError = (char*) realloc (lexError, strlen(lexError)+strlen(yytext)+40);
     sprintf(lexError,"%slexical error (line %d) : %s\n",lexError,yylloc.first_line,yytext);
   } else {
     lexError = (char*) malloc (strlen(yytext)+40);
     sprintf(lexError,"lexical error (line %d) : %s\n",yylloc.first_line,yytext);
   }
  }

%%


void my_flush_buffer()
{
    YY_FLUSH_BUFFER;
}

int yywrap()
{
        return(1);  	/* par defaut ... */
}

/*+ Renvoie le nombre d'occurences du caractere c dans la chaine de caracteres s +*/
int nbOccCharInString(char c, char * s)
{
int i;
int count;
 if (s != NULL) {
   i = 0;
   count = 0;
   for(i=0;i<strlen(s);i++){
     if(c == s[i]){ count++; }
   }
   return count;
 } else {
   return 0;
 }
}

/*+ Pour supprimer tout ce qui se trouve avant un ';' +*/
void removeBeforeComment(char* strInput, char *strResult) {
    char * tmp;
    tmp = strchr(strInput,';');
    if (tmp)
        strcpy(strResult,tmp);
    return;
}

int addComment(char* str, int nLine) {
    Comment *commentTmp;
    if (strchr(str,';')) {
        commentTmp = (Comment*) malloc (sizeof(Comment));
        commentTmp->commentString = (char*) malloc (strlen(str)+1);
        removeBeforeComment(str, commentTmp->commentString);
        /*  printf("%s\n", commentTmp->commentString);*/
        /*  strcpy(commentTmp->commentString, str);*/
        commentTmp->lineInSource = nLine;
        commentTmp->nextComment = NULL;

        if (lastComment == NULL) {
            commentTable = commentTmp;
        } else {
            lastComment->nextComment = commentTmp;
        }
        lastComment = commentTmp;
    }
    return(1);
}
