//
//  D19_Ycor.c
//  Test-Lamp-Loops
//
//  Created by Yannick Raoul on 5/27/13.
//  Copyright 2013 ILL/ADHOC. All rights reserved.
//

#include "lamp_loops.h"

#include <stdlib.h>





//help, L, z, mat, Wout
//L               LONG      = Array[32]
//Z               INT       =        0
//MAT             FLOAT     = Array[640, 640]
//WOUT            FLOAT     = Array[640, 256, 572]

//help, Wout(*,L,z)
//<Expression>    FLOAT     = Array[640, 32]

//help, mat#Wout(*,L,z)
//<Expression>    FLOAT     = Array[640, 32]



IDL_LONG D19_Xcor (int argc, void *argv[])
{
    
    if(argc != 10) return 0;
    
	return D19_Xcor_natural(
                            (IDL_LONG) argv[0], // sz
                            (IDL_LONG) argv[1], // dim_i_L
                            (IDL_LONG*) argv[2], // L
                            (IDL_LONG) argv[3], // dim_i_mat
                            (IDL_LONG) argv[4], // dim_j_mat
                            (float*) argv[5], // mat
                            (IDL_LONG) argv[6], // dim_i_Wout
                            (IDL_LONG) argv[7], // dim_j_Wout
                            (IDL_LONG) argv[8], // dim_k_Wout
                            (float*) argv[9] // Wout
                            );
    
}

//
//'D19_Xcor':   BEGIN ;CALLED FROM D19_COR TO CORRECT ELECTRONIC HORIZONTAL DETECTOR
//;Long32:  sz , L
//;Float32: Wout , mat
//for z=0,sz-1 do Wout(*,L,z) = reverse(mat#Wout(*,L,z))
//END

IDL_LONG D19_Xcor_natural (
                      IDL_LONG sz,
                      IDL_LONG dim_i_L,
                      const IDL_LONG* L,
                      IDL_LONG dim_i_mat,
                      IDL_LONG dim_j_mat,
                      const float* mat,
                      IDL_LONG dim_i_Wout,
                      IDL_LONG dim_j_Wout,
                      IDL_LONG dim_k_Wout,
                      float* Wout
)
{
    int res = 0;
    float * tmp_sub_Wout = NULL;
    float * tmp2_sub_Wout = NULL;
    int dim_i_j_Wout = dim_i_Wout * dim_j_Wout;
	int z, i_L, i;
	int colA, rowB, k;
	int nCol, nRow;

    // Check on size (for matrix product)
    if (dim_j_mat != dim_i_Wout)
        return(0);
    
    tmp_sub_Wout = (float*)malloc (sizeof(float) * dim_i_Wout * dim_i_L) ;
    if (tmp_sub_Wout==NULL)
        goto clean_exit ;

    tmp2_sub_Wout = (float*)malloc (sizeof(float) * dim_i_Wout * dim_i_L) ;
    if (tmp2_sub_Wout==NULL)
        goto clean_exit ;
    

    for (z=0 ; z<sz ; z++)
    {   
        // Wout(*,L,z)
        // Extract a sub array, should be 32 lines
        for (i_L=0 ; i_L<dim_i_L ; i_L++) {
            int start_wout_line = L[i_L]*dim_i_Wout + z*dim_i_j_Wout;
            int start_tmp_wout_line = i_L*dim_i_Wout;
            for (i=0 ; i<dim_i_Wout; i++)
            {
                tmp_sub_Wout[start_tmp_wout_line++] = Wout[start_wout_line++];
            }
        }
        
        // Matrix Product
        nCol = dim_j_mat;
        nRow = dim_i_L;
        for(colA=0; colA < nCol; colA++){
            for(rowB=0; rowB < nRow; rowB++) {
                float res_el = 0.0f;
                int tmp_sub_Wout_index = (rowB*dim_j_mat)/* +k */;
                int mat_index = /* (k*nCol)+ */ colA;
                for (k=0 ; k < dim_j_mat ; k++){
                    res_el += tmp_sub_Wout[tmp_sub_Wout_index] * mat[mat_index];
                    tmp_sub_Wout_index++;
                    mat_index += nCol;
                }
                tmp2_sub_Wout[ rowB * nCol + colA] = res_el;
            }
        }
                
        // Put back in reverse order (line only)
        for (i_L=0 ; i_L<dim_i_L ; i_L++) {
            int start_tmp_wout_line = i_L*dim_i_Wout;
            int end_of_wout_line = L[i_L]*dim_i_Wout + z*dim_i_j_Wout + dim_i_Wout - 1;
            for (i=0 ; i<dim_i_Wout; i++)
            {
                Wout[end_of_wout_line--] = tmp2_sub_Wout[start_tmp_wout_line++];
            }
        }
        
    }// for (int z=0
    
    res = 1;
    
clean_exit:
    if (tmp_sub_Wout!=NULL) free(tmp_sub_Wout);
    if (tmp2_sub_Wout!=NULL) free(tmp2_sub_Wout);
    
    return(res);

}









//help, cor_Yd
//COR_YD (COR2_D19)
//FLOAT     = Array[640, 256]
//SX              INT       =      640
//SY              INT       =      256
//WIN             FLOAT     = Array[640, 256, 572]
//REB             LONG      =        25600
//ZD              FLOAT     = Array[1, 256]

//sx, dims_cor_Yd[0], dims_cor_Yd[1], cor_Yd, dims_Win[0], dims_Win[1], dims_Win[2], Win, reb, dims_Wout[0], dims_Wout[1], dims_Wout[2], Wout

IDL_LONG D19_Ycor (int argc, void *argv[])
{
    
    if(argc != 15) return 0;
    
	return D19_Ycor_natural(
                            (IDL_LONG) argv[0], // sx
                            (IDL_LONG) argv[1], // sy
                            (IDL_LONG) argv[2], // sz
                            (IDL_LONG) argv[3], // dim_i_cor_Yd
                            (IDL_LONG) argv[4], // dim_j_cor_Yd
                            (float*) argv[5], // cor_Yd                            
                            (IDL_LONG) argv[6], // dim_i_Win
                            (IDL_LONG) argv[7], // dim_j_Win
                            (IDL_LONG) argv[8], // dim_k_Win
                            (float*) argv[9], // Win                    
                            (IDL_LONG) argv[10], // reb
                            (IDL_LONG) argv[11], // dim_i_Wout
                            (IDL_LONG) argv[12], // dim_j_Wout
                            (IDL_LONG) argv[13], // dim_k_Wout
                            (float*) argv[14] // Wout
                            );
    
}


//'D19_Ycor':   BEGIN ;CALLED FROM D19_COR TO CORRECT ELECTRONIC VERTICAL DETECTOR
//;Long32:  sx , sy , sz , reb
//;Float32: cor_Yd , Win , Wout
//for j=0,sx-1 do begin
//zd   = cor_Yd[j,*]
//wini = rebin(reform(Win[j,*,*]),reb,sz,/sample)
//for y=1,sy-1 do Wout[j,y,*] = total(wini[zd[y-1]+1 : zd[y] , *] ,1)
//endfor
//END


IDL_LONG D19_Ycor_natural (
                      IDL_LONG sx,
                      IDL_LONG sy,
                      IDL_LONG sz,                
                      IDL_LONG dim_i_cor_Yd,
                      IDL_LONG dim_j_cor_Yd,
                      float* cor_Yd,
                      IDL_LONG dim_i_Win,
                      IDL_LONG dim_j_Win,
                      IDL_LONG dim_k_Win,
                      float* Win,                      
                      IDL_LONG reb,
                      IDL_LONG dim_i_Wout,
                      IDL_LONG dim_j_Wout,
                      IDL_LONG dim_k_Wout,
                      float * Wout
                      )
{
	int j, zd_j;
	int dim_i_j_Wout = dim_i_Wout*dim_j_Wout;
    IDL_LONG nb_element_wini = reb * dim_k_Win;
	float * zd = NULL;
	float * wini = NULL;

    zd = (float*)malloc (sizeof(float) * dim_j_cor_Yd);
    if (zd==NULL)
        return(0);

    wini = (float*)malloc (sizeof(float) * nb_element_wini);
    if (wini==NULL) {
        free(zd);
        return(0);
    }
    // j loop
    for (j=0 ; j<sx ; j++) {
		int replicate_factor, dim_i_j_Win, i_wini, k_Win, j_Win;
        //zd   = cor_Yd(j,*)
        for (zd_j=0 ; zd_j<dim_j_cor_Yd ; zd_j++) {
            zd[zd_j] = cor_Yd[j + zd_j*dim_i_cor_Yd];
        }
        
        //wini = rebin(reform(Win[j,*,*]),reb,sz,/sample)
        
        // Source are all j columns
        replicate_factor = reb / dim_j_Win;// should be 100
        dim_i_j_Win = dim_i_Win*dim_j_Win;
        //int dim_j_k_Win = dim_j_Win*dim_k_Win;

        i_wini=0;// flat index in wini destination
        for  (k_Win = 0 ; k_Win < dim_k_Win ; k_Win++) {
            for (j_Win = 0 ; j_Win < dim_j_Win ; j_Win++) {
                
                int i_source = j + j_Win*dim_i_Win + k_Win*dim_i_j_Win;
                float to_be_replicated = Win[i_source];
                int i_rep;
                // rebin
                for (i_rep = 0 ; i_rep < replicate_factor ; i_rep++) {
                    wini[i_wini++] = to_be_replicated;
                }
            }
        }

            
        //for y=1,sy-1 do Wout[j,y,*] = total(wini[zd[y-1]+1 : zd[y] , *] ,1)
		{
        int y, wini_count;
        for (y=1 ; y<sy ; y++) {
            int wout_index = j + y*dim_i_Wout;// will add dim_i_j_Wout below for third dim.
			int wout_k_index;
            for (wout_k_index=0 ; wout_k_index<dim_k_Wout ; wout_k_index++) {
                
                float total = 0.0f;
                // total(wini[zd[y-1]+1 : zd[y] , *] ,1)
                
                int first_i_wini_index = (int)zd[y-1]+1;
                int last_i_wini_index = (int)zd[y];
				
                // Note, extract from idlm doc 
                // "For example, if the variable vec is a 10-element vector, vec[4:8] is a five-element vector composed of vec[4] through vec[8]."

                // wini is a two dimension array  [ reb, dim_k_Win ]
                int wini_index = first_i_wini_index + wout_k_index*reb;
                for (wini_count=first_i_wini_index ; wini_count <= last_i_wini_index ; wini_count++)
                {
                    total += wini[wini_index++];
                }
                
                Wout[wout_index] = total;
                wout_index += dim_i_j_Wout;// progress in flat space
            }
        }
		}
        
    }// j loop

    free(zd);
    free(wini);
	return (1);
    
}// D19_Ycor_natural
