

#include "lamp_loops.h"



IDL_LONG D33_panels_loop (int argc, void *argv[])
{

	
	/* Insure that the correct number of arguments were passed in */
	if(argc != 13) return 0;
		
	return D33_panels_loop_natural(
		(IDL_LONG) argv[0], // NxV
		(IDL_LONG) argv[1], // sz
		(IDL_LONG*) argv[2], // xVa
		(IDL_LONG*) argv[3], // yVa
		(IDL_LONG*) argv[4], // xVb
		(IDL_LONG*) argv[5], // yVb
        (IDL_LONG) argv[6], // dim_i_mergCNT
		(float*) argv[7], // mergCNT
        (IDL_LONG) argv[8], // dim_i_merDET
        (IDL_LONG) argv[9], // dim_j_merDET                                   
		(float*) argv[10], // merDET
        (IDL_LONG) argv[11], // dim_i_curPan                                   
		(float*) argv[12] // curPan
		);
        
}// D33_panels_loop
	




/*
 * Version with natural C interface. This version can be called directly
 * by IDL using the AUTO_GLUE keyword to CALL_EXTERNAL.
 */
IDL_LONG D33_panels_loop_natural (
	IDL_LONG NxV,
	IDL_LONG sz,
	IDL_LONG * xVa,
	IDL_LONG * yVa,
	IDL_LONG * xVb,
	IDL_LONG * yVb,
    IDL_LONG dim_i_mergCNT,
	float * mergCNT,
    IDL_LONG dim_i_mergDET,
    IDL_LONG dim_j_mergDET,
	float * mergDET,
    IDL_LONG dim_i_curPan,
	float * curPan 
	)
{

// IDL code
// ;p1 = NxV      &  p2 = sz
// ;p3 = xVa      &  p4 = yVa     &  p5 = xVb    &  p6 = yVb
// ;p7 = mergCNT  &  p8 = merDET  &  p9 = curPan
//for i= 0,p1-1 do begin
//   ii1 = p3[i] & ii2 = p4[i]
//   jj1 = p5[i] & jj2 = p6[i]
//   p7[ii1:ii2 , jj1:jj2]+= 1.
//   for z= 0,p2-1 do p8[ii1:ii2 , jj1:jj2 ,z]+= p9[i,z]
//endfor

	int dim_i_j_mergDET = dim_i_mergDET*dim_j_mergDET;
			
	int i, ii, jj, z;// loop index
	
	int ii1, ii2;
	int jj1, jj2;
	for (i=0 ; i<NxV ; i++) {
		ii1 = xVa[i] ; ii2 = yVa[i] ;
		jj1 = xVb[i] ; jj2 = yVb[i] ;
        

		for (ii=ii1; ii<=ii2 ; ii++) {
			for (jj=jj1 ; jj<=jj2 ; jj++) {
				mergCNT[ii+jj*dim_i_mergCNT] += 1.0;
			}	
		}

		for (z = 0; z<sz ; z++) {
			for (ii=ii1; ii<=ii2 ; ii++) {
				for (jj=jj1 ; jj<=jj2 ; jj++) {
					mergDET[ii + jj*dim_i_mergDET + z*dim_i_j_mergDET] += curPan[i + z*dim_i_curPan];
				}	
			}
		}

    }// for (i=0

    return 1;
        
}// D33_panels_loop_natural
	
	
	


/*
 * Trying to optimize
 */
IDL_LONG D33_panels_loop_natural_v2 (
                                  IDL_LONG NxV,
                                  IDL_LONG sz,
                                  IDL_LONG * xVa,
                                  IDL_LONG * yVa,
                                  IDL_LONG * xVb,
                                  IDL_LONG * yVb,
                                  IDL_LONG dim_i_mergCNT,
                                  float * mergCNT,
                                  IDL_LONG dim_i_mergDET,
                                  IDL_LONG dim_j_mergDET,
                                  float * mergDET,
                                  IDL_LONG dim_i_curPan,
                                  float * curPan 
                                  )
{
    
    // IDL code
    // ;p1 = NxV      &  p2 = sz
    // ;p3 = xVa      &  p4 = yVa     &  p5 = xVb    &  p6 = yVb
    // ;p7 = mergCNT  &  p8 = merDET  &  p9 = curPan
    //for i= 0,p1-1 do begin
    //   ii1 = p3[i] & ii2 = p4[i]
    //   jj1 = p5[i] & jj2 = p6[i]
    //   p7[ii1:ii2 , jj1:jj2]+= 1.
    //   for z= 0,p2-1 do p8[ii1:ii2 , jj1:jj2 ,z]+= p9[i,z]
    //endfor
    
	int dim_i_j_mergDET = dim_i_mergDET*dim_j_mergDET;
    
	int i, ii, jj, z;// loop index
	
	int ii1, ii2;
	int jj1, jj2;
	for (i=0 ; i<NxV ; i++) {
		ii1 = xVa[i] ; ii2 = yVa[i] ;
		jj1 = xVb[i] ; jj2 = yVb[i] ;
        
        for (jj=jj1 ; jj<=jj2 ; jj++) {
            int jj_index = jj*dim_i_mergCNT;
            for (ii=ii1; ii<=ii2 ; ii++) {
				mergCNT[jj_index++] += 1.0;
			}	
		}
        
        for (jj=jj1 ; jj<=jj2 ; jj++) {
			for (ii=ii1; ii<=ii2 ; ii++) {
                int ii_jj_z_merDET_index = ii + jj*dim_i_mergDET;
                int i_z_curPan_index = i;
                for (z = 0; z<sz ; z++) {
					mergDET[ii_jj_z_merDET_index] += curPan[i_z_curPan_index];
                    ii_jj_z_merDET_index += dim_i_j_mergDET;
                    i_z_curPan_index += dim_i_curPan;              
				}	
			}
		}

    }// for (i=0
    
    return 1;
    
}// D33_panels_loop_natural_v2


