
#include "lamp_loops.h"

/*****
 ;CALLED FROM READ_HDF TO CONSTRUCT A UNIQ IMAGE FROM TOF FRAMES
 ;p1 = NFix
 ;p2 = SFix1        &  p3 = SFix2  &  p4 = SFiy1  &  p5 = SFiy2
 ;p6 = fullimage    &  p7 = Data
 ;p8 = SFfullCount  &  p9 = SFweight
 for i=0L,p1-1 do begin
 ii1=p2[i] & ii2=p3[i]
 jj1=p4[i] & jj2=p5[i]
 p6[ii1:ii2,jj1:jj2] += p7[i]
 p8[ii1:ii2,jj1:jj2] += p9[i]
 endfor 
 ******/

int gOptimize_tof2 = 0;// [25/06/2013] Work in progress

IDL_LONG D33_tof2 (int argc, void *argv[])
{
    
    if(argc != 10) return 0;
    
	return D33_tof2_natural(
                (IDL_LONG) argv[0], // NFix
                (IDL_LONG*) argv[1], // SFix1
                (IDL_LONG*) argv[2], // SFix2
                (IDL_LONG*) argv[3], // SFiy1
                (IDL_LONG*) argv[4], // SFiy2
                (IDL_LONG) argv[5], // First dimension of fullimage
                (float*) argv[6], // fullimage
                (float*) argv[7], // Data
                (float*) argv[8], // SFfullCount
                (float*) argv[9] // SFweight
               );
            
}// D33_tof2



IDL_LONG D33_tof2_natural( IDL_LONG NFix,
                        IDL_LONG * SFix1,
                        IDL_LONG * SFix2,
                        IDL_LONG * SFiy1,
                        IDL_LONG * SFiy2,
                        IDL_LONG dim_i_fullimage,   // First dimension of fullimage
                        float * fullimage,          // out
                        float * Data,               // in
                        float * SFfullCount,        // in
                        float * SFweight            // in
                        )
{
int i, ii, jj;

    for (i = 0 ; i < NFix ; i++) {
        
        int ii1, ii2;
        int jj1, jj2;
        
        ii1 = SFix1[i] ; ii2 = SFix2[i] ;
        jj1 = SFiy1[i] ; jj2 = SFiy2[i] ;
        
        for (ii=ii1; ii<=ii2 ; ii++) {
            for (jj=jj1 ; jj<=jj2 ; jj++) {
                int fi_index = ii+jj*dim_i_fullimage;
                fullimage[fi_index] += Data[i];
                SFfullCount[fi_index] += SFweight[i];
            }	
        }
    
    }

    return 1;
    
}// D33_tof2_natural






/*
 * Second version, addition only.
 * not really faster on 2013 processors
 *
 *
 */
//IDL_LONG D33_tof2_natural_v2( IDL_LONG NFix,
//                          IDL_LONG * SFix1,
//                          IDL_LONG * SFix2,
//                          IDL_LONG * SFiy1,
//                          IDL_LONG * SFiy2,
//                          IDL_LONG dim_i_fullimage,   // First dimension of fullimage
//                          float * fullimage,          // out
//                          float * Data,               // in
//                          float * SFfullCount,        // in
//                          float * SFweight            // in
//                          )
//{
//    int i, ii, jj, jj_index;
//    
//    for (i = 0 ; i < NFix ; i++) {
//        
//        int ii1, ii2;
//        int jj1, jj2;
//        
//        ii1 = SFix1[i] ; ii2 = SFix2[i] ;
//        jj1 = SFiy1[i] ; jj2 = SFiy2[i] ;
//        
//        
//        int jj_index = ii1+jj1*dim_i_fullimage;
//        for (jj=jj1 ; jj<=jj2 ; jj++) {
//            int ii_index = jj_index;
//            for (ii=ii1; ii<=ii2 ; ii++) {
//                fullimage[ii_index] += Data[i];
//                SFfullCount[ii_index] += SFweight[i];
//                ii_index++;
//            }
//            jj_index += dim_i_fullimage;
//        }
//            
//    }
//    
//    return 1;
//    
//}// D33_tof2_natural_v2

