//
//  HKL_calc.c
//  Test-Lamp-Loops
//
//  Created by Yannick Raoul on 5/24/13.
//  Copyright 2013 ILL/ADHOC. All rights reserved.
//

#include "lamp_loops.h"
#include <stdlib.h>




IDL_LONG HKL_calc (int argc, void *argv[])
{
    
    if(argc != 21) return 0;
    
	return HKL_calc_natural(
                            *(IDL_LONG*)argv[0], // n_elements_z
                            *(IDL_LONG*)argv[1], // n_elements_sgamp
                            (float*) argv[2], // sgamp
                            *(IDL_LONG*)argv[3], // n_elements_cgamp
                            (float*) argv[4], // cgamp
                            *(IDL_LONG*)argv[5], // n_elements_cnu
                            (float*) argv[6], // cnu
                            *(float*)argv[7], // wave
                            *(IDL_LONG*)argv[8], // n_elements_a
                            *(IDL_LONG*)argv[9], // dim_i_zz
                            *(IDL_LONG*)argv[10], // dim_j_zz
                            (float*) argv[11], // zz
                            *(IDL_LONG*)argv[12], // n_elements_snup
                            (float*) argv[13], // snup
                            *(IDL_LONG*)argv[14], // k
                            *(IDL_LONG*)argv[15], // dim_i_z1H
                            *(IDL_LONG*)argv[16], // dim_j_z1H
                            *(IDL_LONG*)argv[17], // dim_k_z1H
                            (float*) argv[18], // z1H
                            (float*) argv[19], // z1K
                            (float*) argv[20] // z1L
                            );
    
}// HKL_calc


//        'HKL_calc':   BEGIN ;CALLED FROM RDID D19 ENTRY TO CALCULATE EACH HKL FOR EACH IJK
//        ;Long32:  n_elements(z)-1 , n_elements(a)-1 , k , z1H , z1K , z1L
//        ;Float32: sgamp , cgamp , cnu , snup , wave , zz
//        for  j=0,n_elements(z)-1 do begin
//          cnupS = sgamp * cnu[j]      / wave
//          cnupC =(cgamp * cnu[j] -1.) / wave
//          for i=0,n_elements(a)-1 do begin
//              hkl  =zz##([ cnupS[i] , cnupC[i] , snup[j] ])
//              z1H[i,j,k] = hkl[0] & z1K[i,j,k] = hkl[1] & z1L[i,j,k] = hkl[2]
//          endfor ;i gamma
//        endfor  ;j z
//        ;k is in the calling code
//        END



IDL_LONG HKL_calc_natural(
                     IDL_LONG n_elements_z,
                     IDL_LONG n_elements_sgamp,
                     float* sgamp,
                     IDL_LONG n_elements_cgamp,
                     float* cgamp,
                     IDL_LONG n_elements_cnu,
                     float* cnu,
                     float wave,
                     IDL_LONG n_elements_a,
                     IDL_LONG dim_i_zz,
                     IDL_LONG dim_j_zz,
                     float* zz,
                     IDL_LONG n_elements_snup,
                     float* snup,
                     IDL_LONG k,
                     IDL_LONG dim_i_z1H,
                     IDL_LONG dim_j_z1H,
                     IDL_LONG dim_k_z1H,
                     float* z1H,
                     float* z1K,
                     float* z1L
                    )
{
    int res = 0;
    float * cnupS = NULL;
    float * cnupC = NULL;
    int dim_i_j_z1H = dim_i_z1H*dim_j_z1H;
    int i, j, cnup_index, z1HKL_index, i_hkl, i_v_nup;
    
    if ((dim_i_zz!=3) || (dim_j_zz!=3))
        goto clean_exit ;
    
    cnupS = (float*)malloc (sizeof(float) * n_elements_sgamp);
    if (cnupS==NULL)
        goto clean_exit ;
    
    cnupC = (float*)malloc (sizeof(float) * n_elements_cgamp);
    if (cnupC==NULL)
        goto clean_exit ;
    
    
//    FILE * hkl_log = fopen("/Users/raoul/hkl_log.txt", "a");
//    fprintf(hkl_log, "k: %d nz: %d nsgamp: %d wave: %g cnu: %p\n", k, n_elements_z, n_elements_sgamp, wave, cnu);
//    fclose(hkl_log);
//    goto clean_exit ;

    
    //        for  j=0,n_elements(z)-1 do begin   
    for (j=0 ; j<n_elements_z ; j++) {
        
        //          cnupS = sgamp * cnu[j]      / wave
        //          cnupC =(cgamp * cnu[j] -1.) / wave
        for (cnup_index=0 ; cnup_index<n_elements_sgamp ; cnup_index++)
        {
            cnupS[cnup_index] =  sgamp[cnup_index] * (cnu[j] / wave);
            cnupC[cnup_index] = (cgamp[cnup_index] * cnu[j] - 1.0f) / wave;
        }

//        float out_1 = cnu[j] / wave;
//        float out_2 = 1.0f / wave;
//        for (int cnup_index=0 ; cnup_index<n_elements_sgamp ; cnup_index++)
//        {
//            cnupS[cnup_index] =  sgamp[cnup_index] * out_1;
//            cnupC[cnup_index] =  cgamp[cnup_index] * out_1 - out_2;
//        }

        
        // for i=0,n_elements(a)-1 do begin
        z1HKL_index = /* i + */ j*dim_i_z1H + k*dim_i_j_z1H ;
        for (i=0 ; i<n_elements_a ; i++) {
            float v_nup[3];
            float hkl[3];
          
            // hkl  =zz##([ cnupS[i] , cnupC[i] , snup[j] ])
            v_nup[0] = cnupS[i];
            v_nup[1] = cnupC[i];
            v_nup[2] = snup[j];
            for (i_hkl = 0 ; i_hkl<3 ; i_hkl++) {
                float sum = 0.0f;
                for (i_v_nup = 0 ; i_v_nup<3 ; i_v_nup++) {
                    sum += zz[i_v_nup + i_hkl*3] * v_nup[i_v_nup];
                }
                hkl[i_hkl] = sum;            
            }
            
            // z1H[i,j,k] = hkl[0] & z1K[i,j,k] = hkl[1] & z1L[i,j,k] = hkl[2]
            z1H[z1HKL_index] = hkl[0];
            z1K[z1HKL_index] = hkl[1];
            z1L[z1HKL_index] = hkl[2];
            z1HKL_index++; // progress along first dimension
        }
    }
    res = 1;
    
clean_exit:
    if (cnupS!=NULL) free(cnupS);
    if (cnupC!=NULL) free(cnupC);
    
    return(res);
}