//
//  HKL_calc.c
//  Test-Lamp-Loops
//
//  Created by Yannick Raoul on 5/24/13.
//  Copyright 2013 ILL/ADHOC. All rights reserved.
//

#include "lamp_loops.h"


/*****
 'HKL_vol':    BEGIN ;CALLED FROM RDID D19 ENTRY TO FILL THE HKL VOLUME
 ;Long32:  n_elements(WOUT) , z1H , z1K , z1L
 ;Float32: Wtmp , WOUT
 for i=0L,n_elements(WOUT)-1 do if Wtmp[z1H[i] , z1K[i] , z1L[i]] lt WOUT[i] then  $
 Wtmp[z1H[i] , z1K[i] , z1L[i]] =  WOUT[i]
 END ******/


IDL_LONG HKL_vol (int argc, void *argv[])
{
    
    if(argc != 10) return 0;
    
	return HKL_vol_natural(
                            (IDL_LONG) argv[0], // n_elements_WOUT
                            (float*) argv[1], // WOUT
                            (IDL_LONG) argv[2], // dim_i_Wtmp
                            (IDL_LONG) argv[3], // dim_j_Wtmp
                            (IDL_LONG) argv[4], // dim_k_Wtmp
                            (float*) argv[5], // Wtmp
                            (IDL_LONG) argv[6], // n_elements_z1H
                            (IDL_LONG*) argv[7], // z1H
                            (IDL_LONG*) argv[8], // z1K
                            (IDL_LONG*) argv[9] // z1L
                            );
    
}// HKL_vol


//help, Wtmp
//WTMP            FLOAT     = Array[89, 158, 155]
//help, WOUT
//WOUT            FLOAT     = Array[606, 254, 572]
//help, z1H
//Z1H             LONG      = Array[606, 254, 572]

IDL_LONG HKL_vol_natural(
                    IDL_LONG n_elements_WOUT,
                    float * WOUT,
                    IDL_LONG dim_i_Wtmp,
                    IDL_LONG dim_j_Wtmp,
                    IDL_LONG dim_k_Wtmp,
                    float * Wtmp,
                    IDL_LONG n_elements_z1H,
                    IDL_LONG * z1H,
                    IDL_LONG * z1K,
                    IDL_LONG * z1L
                    )
{
    IDL_LONG dim_i_j_Wtmp = dim_i_Wtmp * dim_j_Wtmp;
    IDL_LONG i = 0;

    for (i = 0 ; i < n_elements_WOUT ; i++) {
        IDL_LONG indexWtmp = z1H[i] + z1K[i]*dim_i_Wtmp + z1L[i]*dim_i_j_Wtmp;
        //if (indexWtmp>=0 && indexWtmp<n_elements_WOUT) {
        if (Wtmp[indexWtmp] < WOUT[i]) Wtmp[indexWtmp] = WOUT[i];
        //} else {
        //    printf("Aie [i=%d]", i);
        //}
    }

    return 1;

}





// The following code is an attempt to optimize by unrolling loop, but the gain is minimal

//
//#define TOGETHER (3)
//
//int HKL_vol_natural_unrolled(
//                    IDL_LONG n_elements_WOUT,
//                    float * WOUT,
//                    IDL_LONG dim_i_Wtmp,
//                    IDL_LONG dim_j_Wtmp,
//                    IDL_LONG dim_k_Wtmp,
//                    float * Wtmp,
//                    IDL_LONG n_elements_z1H,
//                    IDL_LONG * z1H,
//                    IDL_LONG * z1K,
//                    IDL_LONG * z1L
//                    )
//{
//    
//    IDL_LONG dim_i_j_Wtmp = dim_i_Wtmp * dim_j_Wtmp;
//    IDL_LONG indexWtmp;
//    IDL_LONG i = 0;
//        
//    IDL_LONG repeat = (n_elements_WOUT / TOGETHER);  /* number of times to repeat   */
//    IDL_LONG left  =  (n_elements_WOUT % TOGETHER);  /* calculate remainder         */
//    
//    while (repeat-- > 0)
//    {
//        indexWtmp = z1H[i] + z1K[i]*dim_i_Wtmp + z1L[i]*dim_i_j_Wtmp;
//        if (Wtmp[indexWtmp] < WOUT[i]) Wtmp[indexWtmp] = WOUT[i];
//
//        indexWtmp = z1H[i+1] + z1K[i+1]*dim_i_Wtmp + z1L[i+1]*dim_i_j_Wtmp;
//        if (Wtmp[indexWtmp] < WOUT[i+1]) Wtmp[indexWtmp] = WOUT[i+1];
//
//        indexWtmp = z1H[i+2] + z1K[i+2]*dim_i_Wtmp + z1L[i+2]*dim_i_j_Wtmp;
//        if (Wtmp[indexWtmp] < WOUT[i+2]) Wtmp[indexWtmp] = WOUT[i+2];
//
//        i += TOGETHER;
//    }
//    
//    /* Use a switch statement to process remaining by jumping to the case label     */ 
//    /* at the label that will then drop through to complete the set                 */ 
//    switch (left) 
//    {
//        case 2 : /* two left                                      */
//            indexWtmp = z1H[i+1] + z1K[i+1]*dim_i_Wtmp + z1L[i+1]*dim_i_j_Wtmp;
//            if (Wtmp[indexWtmp] < WOUT[i+1]) Wtmp[indexWtmp] = WOUT[i+1];
//
//        case 1 :  /* just one left to process                      */ 
//            indexWtmp = z1H[i] + z1K[i]*dim_i_Wtmp + z1L[i]*dim_i_j_Wtmp;
//            if (Wtmp[indexWtmp] < WOUT[i]) Wtmp[indexWtmp] = WOUT[i];
//            
//        case 0 : /* none left                                     */
//            ;
//    } 
//
//    return 1;
//    
//}// HKL_vol_natural_unrolled
