//
//  IN5_dist.c
//  LAMP_Loops
//
//  Created by Yannick Raoul on 3/8/13.
//  Copyright 2013 ILL/CS. All rights reserved.
//

#include "lamp_loops.h"


//bid = CALL_EXTERNAL(SFdll, 'IN5_dist',$
//                    n_elements(indLim), indLim, dims_DATA[0], dims_DATA[1], dims_DATA[2], DATA, $
//                    j, W_out, ind, ind1, unMoinsPA, PA, $
//                    Eok, E, E_out, $
//                    VALUE=[1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0], /I_VALUE, /CDECL)



IDL_LONG IN5_dist (int argc, void *argv[])
{
    
    if(argc != 15) return 0;
    
	return IN5_dist_natural(
                            (IDL_LONG)          argv[0], // n_elements_indLim
                            (const IDL_LONG *)  argv[1], // indLim
                            (IDL_LONG)          argv[2], // dim_i_DATA
                            (IDL_LONG)          argv[3], // dim_j_DATA
                            (IDL_LONG)          argv[4], // dim_k_DATA
                            (const float *)     argv[5], // DATA
                            (IDL_LONG)          argv[6], // j
                            (float *)           argv[7], // W_out
                            (const IDL_LONG *)  argv[8], // ind
                            (const IDL_LONG *)  argv[9], // ind1
                            (float *)           argv[10], // unMoinsPA
                            (float *)           argv[11], // PA
                            (IDL_LONG)          argv[12], // Eok
                            (const float *)     argv[13], // E
                            (float *)           argv[14] // E_out    
                            );
    
}// IN5_dist






/*****
 
 'IN5_dist':   BEGIN ;CALLED FROM READ_HDF FOR IN5 DISTANCE CORRECTION
 ;Long32:  n_elements(indLim) , indLim , j , ind , ind1
 ;Float32: W_out , E_out , DATA , PA , unMoinsPA
 for k=0,n_elements(indLim)-1 do begin
 vv                 = DATA [*,j,indLim[k]]
 W_out[*,j,ind[k] ] = W_out[*,j,ind[k] ] + vv * unMoinsPA[k]
 W_out[*,j,ind1[k]] = W_out[*,j,ind1[k]] + vv * PA[k]
 if Eok then begin  ; Only add errors
 ee                 = E    [*,j,indLim[k]]
 E_out[*,j,ind[k] ] = E_out[*,j,ind[k] ] + ee * unMoinsPA[k]
 E_out[*,j,ind1[k]] = E_out[*,j,ind1[k]] + ee * PA[k]
 endif
 endfor
 END
 
 ******/


IDL_LONG IN5_dist_natural(IDL_LONG n_elements_indLim,
                          const IDL_LONG * indLim,
                          IDL_LONG  dim_i_DATA,
                          IDL_LONG  dim_j_DATA,
                          IDL_LONG  dim_k_DATA,
                          const float * DATA,
                          IDL_LONG  j,
                          float * W_out,
                          const IDL_LONG * ind,
                          const IDL_LONG * ind1,
                          float * unMoinsPA,
                          float * PA,
                          IDL_LONG Eok,
                          const float * E,
                          float * E_out
                          )
{
    IDL_LONG k;
    IDL_LONG dim_i_j_DATA = dim_i_DATA * dim_j_DATA;
    
    //for k=0,n_elements(indLim)-1 do begin
    for (k=0 ; k<n_elements_indLim ; k++) {

        IDL_LONG i;
        
        IDL_LONG DATA_offset = j*dim_i_DATA + indLim[k]*dim_i_j_DATA;
        IDL_LONG out_offset1 = j*dim_i_DATA + ind[k]*dim_i_j_DATA;
        IDL_LONG out_offset2 = j*dim_i_DATA + ind1[k]*dim_i_j_DATA;

        // Loop to progress along first dimension of DATA, W_out and E_out array
        for (i=0 ; i<dim_i_DATA ; i++)
        {
            float vv = DATA[DATA_offset];
            W_out[out_offset1] += vv * unMoinsPA[k];
            W_out[out_offset2] += vv * PA[k];

            if (Eok) {
                float ee = E[DATA_offset];
                E_out[out_offset1] += ee * unMoinsPA[k];
                E_out[out_offset2] += ee * PA[k];
            }
            
            DATA_offset++;
            out_offset1++;
            out_offset2++;
        }// for i
    
    }// for k
    
    return 1;
    
}

