//
//  STR_proj.c
//  LAMP_Loops
//
//  Created by Yannick Raoul on 3/8/13.
//  Copyright 2013 ILL/CS. All rights reserved.
//

#include "lamp_loops.h"

#include <stdlib.h>


IDL_LONG STR_proj (int argc, void *argv[])
{
    
    if(argc != 19) return -1;
    
	return STR_proj_natural(
                            (IDL_LONG)          argv[0], // n_elements_sk
                            (const IDL_LONG *)  argv[1], // ind
                            (const IDL_LONG *)  argv[2], // ik
                            (const IDL_LONG *)  argv[3], // ik1
                            (IDL_LONG)          argv[4], // j
                            (IDL_LONG)          argv[5], // Eok
                            (IDL_LONG)          argv[6], // dim_i_out
                            (IDL_LONG)          argv[7], // dim_j_out
                            (IDL_LONG)          argv[8], // dim_k_out                            
                            (float *)           argv[9], // Wout
                            (float *)           argv[10], // Eout
                            (float *)           argv[11], // IWeight
                            (IDL_LONG)          argv[12], // dim_v_e_1
                            (IDL_LONG)          argv[13], // dim_v_e_2
                            (IDL_LONG)          argv[14], // dim_v_e_3                            
                            (const float *)     argv[15], // v 
                            (const float *)     argv[16], // e 
                            (const float *)     argv[17], // PA
                            (const float *)     argv[18] // PA1
                            );
    
}// STR_proj




/*****
 
 'STR_proj':   BEGIN ;CALLED FROM STRAIGHT_2D WITHOUT KEYWORD TEXTURE
 ;Long32:  n_elements(sk) , ind , Eok
 ;Float32: Wout , Eout , v , e , PA , PA1
 for k=0L,n_elements(sk)-1 do begin
     kk=ind[k]
     vv[*]=v[kk,*,*] & if Eok then ee[*]=e[kk,*,*]
     wout[ik[ k], j ,*]=wout[ik[ k], j ,*]+vv*PA1[k]
     wout[ik[ k], j ,*]=wout[ik1[k], j ,*]+vv*PA[ k]
     if Eok then begin
         Eout[ik[ k], j ,*]=Eout[ik[ k], j ,*]+ee*PA1[k]        ;Increase errors
         Eout[ik1[k], j ,*]=Eout[ik1[k], j ,*]+ee*PA[ k]
     endif
     Iweight[ik[ k]]      =Iweight[ik[ k]]   +   PA1[k]        ;Calculate weights for integration
     Iweight[ik1[k]]      =Iweight[ik1[k]]   +   PA[ k]
 endfor
 
 ******/


IDL_LONG STR_proj_natural(
                          IDL_LONG n_elements_sk,
                          const IDL_LONG * ind,
                          const IDL_LONG * ik,
                          const IDL_LONG * ik1,
                          IDL_LONG j,
                          IDL_LONG Eok,
                          IDL_LONG  dim_i_out,
                          IDL_LONG  dim_j_out,
                          IDL_LONG  dim_k_out,
                          float * Wout,
                          float * Eout,
                          float * Iweight,
                          IDL_LONG  dim_v_e_1,
                          IDL_LONG  dim_v_e_2,
                          IDL_LONG  dim_v_e_3,
                          const float * v,
                          const float * e,
                          const float * PA,
                          const float * PA1
                          )
{
    int res = -2;
    IDL_LONG k;
    IDL_LONG dim_i_j_out = dim_i_out * dim_j_out; // used to progress along third dimension
    IDL_LONG dim_1_2_v_e = dim_v_e_1 * dim_v_e_2;
    float * vv = NULL;// tmp
    float * ee = NULL;// tmp
    
    //vv[*]=v[kk,*,*]
    IDL_LONG dim_vv_ee = 1 * dim_v_e_2 * dim_v_e_3;// 1 * 1 * 512, for example
    
    if (dim_vv_ee!=dim_v_e_3)
        goto clean_exit ;// must be equal, else will crash below
    
    // alloc temporary
    vv = (float*)malloc (sizeof(float) * dim_vv_ee) ;
    if (vv==NULL)
        goto clean_exit ;
    
    if (Eok) {
        ee = (float*)malloc (sizeof(float) * dim_vv_ee) ;
        if (ee==NULL)
            goto clean_exit ;
    }
    
    //for k=0,n_elements(sk)-1 do begin
    for (k=0 ; k<n_elements_sk ; k++) {
        
        IDL_LONG index_dim3, index_v, kk;
        
        kk = ind[k];
        
        // Extract vv and ee
        //vv[*]=v[kk,*,*] & if Eok then ee[*]=e[kk,*,*]
        IDL_LONG v_e_offset = kk;
        for (index_v=0 ; index_v<dim_vv_ee ; index_v++) {
            vv[index_v] = v[v_e_offset];
            if (Eok) {
                ee[index_v] = e[v_e_offset];
            }

            v_e_offset += dim_1_2_v_e;
        }
        
        
        // Loop to progress along third dimension of Wout, Eout and Iweight array

        //wout[ik[ k], j ,*]
        IDL_LONG out_offset1 = ik [k] + j*dim_i_out;// + index_dim3*dim_i_j_out;
        //wout[ik[ k], j ,*]
        IDL_LONG out_offset2 = ik1[k] + j*dim_i_out;// + index_dim3*dim_i_j_out;
        
        for (index_dim3=0 ; index_dim3<dim_k_out ; index_dim3++)
        {
            Wout[out_offset1] += vv[index_dim3] * PA1[k];
            Wout[out_offset2] += vv[index_dim3] * PA[ k];
            
            if (Eok) {
                Eout[out_offset1] += ee[index_dim3] * PA1[k];
                Eout[out_offset2] += ee[index_dim3] * PA[ k];
            }
            
            out_offset1 += dim_i_j_out;
            out_offset2 += dim_i_j_out;

        }// for index_dim3
        
        Iweight[ ik[ k] ] += PA1[k];
        Iweight[ ik1[k] ] += PA[ k];

    }// for k
    
    res = 1;
    
clean_exit:
    if (vv!=NULL) free(vv);
    if (ee!=NULL) free(ee);
    
    return(res);

    
}// STR_proj_natural

