//
//  STR_textu.c
//  LAMP_Loops
//
//  Created by Yannick Raoul on 14/8/13.
//  Copyright 2013 ILL/CS. All rights reserved.
//

#include "lamp_loops.h"

#include <stdlib.h>


IDL_LONG STR_textu (int argc, void *argv[])
{
    
    if(argc != 21) return -1;
    
	return STR_textu_natural(
                             (IDL_LONG)         argv[0], // n_elements_sk
                             (IDL_LONG *)       argv[1], // ind[]
                             (IDL_LONG)         argv[2], // Eok
                             (IDL_LONG *)       argv[3], // jof1[]
                             (IDL_LONG *)       argv[4], // jof2[]
                             (IDL_LONG *)       argv[5], // ik[]
                             (IDL_LONG *)       argv[6], // ik1[]
                             (IDL_LONG)         argv[7], // dim_i_out
                             (IDL_LONG)         argv[8], // dim_j_out
                             (IDL_LONG)         argv[9], // dim_k_out
                             (float *)          argv[10], // Wout
                             (float *)          argv[11], // Eout
                             (IDL_LONG)         argv[12], // dim_v_e_1
                             (IDL_LONG)         argv[13], // dim_v_e_2
                             (IDL_LONG)         argv[14], // dim_v_e_3                            
                             (const float *)    argv[15], // v 
                             (const float *)    argv[16], // e 
                             (const float *)    argv[17], // PA01
                             (const float *)    argv[18], // PA02
                             (const float *)    argv[19], // PA11
                             (const float *)    argv[20]  // PA12
                           );
    
}// STR_textu




/*****
 
 'STR_textu':  BEGIN ;CALLED FROM STRAIGHT_2D WITH KEYWORD TEXTURE SET
 ;Long32:  n_elements(sk) , ind , Eok , jof1 , jof2
 ;Float32: Wout , Eout , v , e , PA01 , PA02 , PA11 , PA12
 for k=0L,n_elements(sk)-1 do begin kk = ind[k]
 jj1 =jof1[kk] & jj2 =jof2[kk] & vv[*] =v[kk,*,*] & if Eok then ee[*] =e[kk,*,*]
 wout[ik [k],jj1,*] = wout[ik [k],jj1,*] +vv*PA11[k]
 wout[ik1[k],jj1,*] = wout[ik1[k],jj1,*] +vv*PA01[k]
 wout[ik [k],jj2,*] = wout[ik [k],jj2,*] +vv*PA12[k]
 wout[ik1[k],jj2,*] = wout[ik1[k],jj2,*] +vv*PA02[k]
 if Eok then begin
 Eout[ik [k],jj1,*] = Eout[ik [k],jj1,*] +ee*PA11[k]    ;Increase errors
 Eout[ik1[k],jj1,*] = Eout[ik1[k],jj1,*] +ee*PA01[k]
 Eout[ik [k],jj2,*] = Eout[ik [k],jj2,*] +ee*PA12[k]
 Eout[ik1[k],jj2,*] = Eout[ik1[k],jj2,*] +ee*PA02[k]
 endif
 endfor
 END
 
 ******/



IDL_LONG STR_textu_natural(
                           IDL_LONG n_elements_sk,
                           IDL_LONG * ind,
                           IDL_LONG Eok,
                           IDL_LONG * jof1,
                           IDL_LONG * jof2,
                           IDL_LONG * ik,
                           IDL_LONG * ik1,
                           IDL_LONG  dim_i_out,
                           IDL_LONG  dim_j_out,
                           IDL_LONG  dim_k_out,
                           float * Wout,
                           float * Eout,
                           IDL_LONG  dim_v_e_1,
                           IDL_LONG  dim_v_e_2,
                           IDL_LONG  dim_v_e_3,
                           const float * v,
                           const float * e,
                           const float * PA01,
                           const float * PA02,
                           const float * PA11,
                           const float * PA12
                         )
{
    int res = -2;

    IDL_LONG k;
    IDL_LONG dim_i_j_out = dim_i_out * dim_j_out;
    IDL_LONG dim_1_2_v_e = dim_v_e_1 * dim_v_e_2;
    float * vv = NULL;// tmp
    float * ee = NULL;// tmp

    
    //vv[*]=v[kk,*,*]
    IDL_LONG dim_vv_ve = 1 * dim_v_e_2 * dim_v_e_3;// 1 * 1 * 512, for example
    
    if (dim_vv_ve!=dim_v_e_3)
        goto clean_exit ;// must be equal, else will crash below

    // alloc temporary
    vv = (float*)malloc (sizeof(float) * dim_vv_ve) ;
    if (vv==NULL)
        goto clean_exit ;
    
    if (Eok) {
        ee = (float*)malloc (sizeof(float) * dim_vv_ve) ;
        if (ee==NULL)
            goto clean_exit ;
    }
    
    //for k=0L,n_elements(sk)-1 do
    for (k=0 ; k<n_elements_sk ; k++) {
        IDL_LONG index_v, kk, jj1, jj2;
        
        kk = ind[k];
        jj1 = jof1[k];
        jj2 = jof2[k];
        
        // Extract vv and ee
        IDL_LONG v_e_offset = kk;
        for (index_v=0 ; index_v<dim_vv_ve ; index_v++) {
            vv[index_v] = v[v_e_offset];
            if (Eok) {
                ee[index_v] = e[v_e_offset];
            }
            
            v_e_offset += dim_1_2_v_e;
        }
    
        IDL_LONG out_offset1 = ik  [k] + jj1*dim_i_out;// + index_dim3*dim_i_j_out;
        IDL_LONG out_offset2 = ik1 [k] + jj1*dim_i_out;// + index_dim3*dim_i_j_out;
        IDL_LONG out_offset3 = ik  [k] + jj2*dim_i_out;// + index_dim3*dim_i_j_out;
        IDL_LONG out_offset4 = ik1 [k] + jj2*dim_i_out;// + index_dim3*dim_i_j_out;

        // Loop to progress along third dimension of Wout and Eout array
        for (IDL_LONG index_dim3=0 ; index_dim3<dim_k_out ; index_dim3++)
        {
            
//            wout[ik [k],jj1,*] = wout[ik [k],jj1,*] +vv*PA11[k]
//            wout[ik1[k],jj1,*] = wout[ik1[k],jj1,*] +vv*PA01[k]
//            wout[ik [k],jj2,*] = wout[ik [k],jj2,*] +vv*PA12[k]
//            wout[ik1[k],jj2,*] = wout[ik1[k],jj2,*] +vv*PA02[k]
            
            Wout[out_offset1] += vv[index_dim3]*PA11[k];
            Wout[out_offset2] += vv[index_dim3]*PA01[k];
            Wout[out_offset3] += vv[index_dim3]*PA12[k];
            Wout[out_offset4] += vv[index_dim3]*PA02[k];
            
            
            if (Eok) {
                Eout[out_offset1] += ee[index_dim3]*PA11[k];
                Eout[out_offset2] += ee[index_dim3]*PA01[k];
                Eout[out_offset3] += ee[index_dim3]*PA12[k];
                Eout[out_offset4] += ee[index_dim3]*PA02[k];
            }
            
            out_offset1 += dim_i_j_out;
            out_offset2 += dim_i_j_out;
            out_offset3 += dim_i_j_out;
            out_offset4 += dim_i_j_out;
            
        }// for (IDL_LONG index_dim3=0 ; index_dim3<dim_k_out
        
    }// for (k=0 ; k<n_elements_sk ; k++)
    
    res = 1;

clean_exit:
    if (vv!=NULL) free(vv);
    if (ee!=NULL) free(ee);
    
    return(res);
    
}

