;+
;** ***************************
;** Test several modules of the Lamp application.
;** You should start Lamp first, only the main lamp window is present.
;** then launch lamp_fast_unit_test without any parameter.
;** See results in the file lamp_fast_unit_test.txt
;-

pro lamp_test_clean, W_managed
;+
;** ***************
;** Clean new managed base for lamp_fast_unit_test
;-
cur = widget_info(/managed)
for i = 0,n_elements(cur)-1 do $
    if (where(W_managed eq cur[i]))[0] eq -1 then widget_control,cur[i], bad_id=ii, /destroy
end

pro lamp_fast_unit_test
;+
;** ***************************
;** You should start Lamp first,
;** then launch this procedure without parameter.
;** See results in the file lamp_fast_unit_test.txt
;-
@lamp.cbk

W_STORE, /ALL                      ; Will use worspaces for tests
SEE,W=1, /below                    ; initiate and manage "SnapShotWks" window
cd, current= Mee                   ; Working directory
RDSET, base='Current Path'
P_to_DATA  = lamp_dir+lamp_dvd+'test_unit'+lamp_dvd
KP_path    = path_for_online
KP_inst    = inst_value
nbE        = 0
nbT        = 0B
W_managed  = widget_info(/managed) ; Keep already managed widgets

on_ioerror,miswrite
OPENW,u,'lamp_fast_unit_test.txt', /get_lun

  lamp_message,'Output test in lamp_fast_unit_test.txt'

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************************
  st = 'GIVE_W , MOD_DATP , SEE , RDSET ...'       & nbT+=1 & st = st+string(nbT)
; ******************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    W_test    = dist(10)
    W_tot     = total(W_test)
    GIVE_W ,    W_test, W=1
    GIVE_W ,    W_test, W=63
    GIVE_DATP,  {E:sqrt(W_test), N:1, W_tit:'Test'}, W=1
    SEE,/beside,W=1
    RDSET,path=P_to_DATA, inst='NeXus'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if !error_state.msg gt ' ' then return
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *******************************
  st = 'AVAILABLE WORKSPACES ...'                  & nbT+=1 & st = st+string(nbT)
; *******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    ShowFreeW
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************************
  st = 'EXPORT A WORKSPACE (GUI) ...'              & nbT+=1 & st = st+string(nbT)
; ***********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_DID_SAVE_MENU,1
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************************
  st = 'IMPORT A WORKSPACE (GUI) ...'              & nbT+=1 & st = st+string(nbT)
; ***********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_ICK_INIT,0
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **************************************
  st = 'HDF EXPORT API (+WRITE_LAMP)...'           & nbT+=1 & st = st+string(nbT)
; **************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    WRITE_LAMP,'lamp_fast_hdf_test.hdf', W=1, format='HDF'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *************************************
  st = 'HDF IMPORT API (+READ_LAMP)...'            & nbT+=1 & st = st+string(nbT)
; *************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    READ_LAMP,'lamp_fast_hdf_test.hdf', W=2
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if long(total(W2)) ne long(W_tot) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  file_delete,'lamp_fast_hdf_test.hdf', /ALLOW_NON, /QUIET

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'ASCII COLUMN EXPORT ...'                   & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    WRITE_LAMP,'lamp_fast_col_test.txt', W=1, format='Column'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'ASCII COLUMN IMPORT ...'                   & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    xicute, "W2 = READ_COLUMN('lamp_fast_col_test.txt')"
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if long(total(W2)) ne long(W_tot) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  file_delete,'lamp_fast_col_test.txt', /ALLOW_NON, /QUIET

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***************************************
  st = 'RDFILTER (MULTIPLE READ GUI) ...'          & nbT+=1 & st = st+string(nbT)
; ***************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    RDFILTER
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ****************************************************
  st = 'RDFILTER (USING X Y Z RANGES PROJECTIONS) ...' & nbT+=1 & st = st+string(nbT)
; ****************************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    W20 = 0
    RDFILTER, XRANGE =[2,9], YRANGE=[2,9], ZRANGE=[2,3],$
             /XPROJ,/YPROJ,/ZPROJ, SELECT='000003.nxs'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if long(total(W20)) ne long(total(W_test[1:8,1:8])*2.) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
    TO_DID_CUR,'W1'
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *******************************
  st = 'DO INPUT COMMAND GUI ...'                  & nbT+=1 & st = st+string(nbT)
; *******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    DON_DO_CMD
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************
  st = 'XBU GUI ...'                               & nbT+=1 & st = st+string(nbT)
; ******************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    DIALINIT,'xbu'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************************
  st = 'XBU READ EXECUTE (GEORGE)...'              & nbT+=1 & st = st+string(nbT)
; ***********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    XBUCONTROL  ,'Read'  ,FILE=P_to_DATA+'fifi.xbu' ,NAME='xbu'
    XBUCONTROL  ,'Start' ,NAME='xbu'
    DIALTAG,     'xbu'   ,TAG ='FREQUENCY',set=.01
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **********************************
  st = 'PROX_SHEET GUI and TREAT...'               & nbT+=1 & st = st+string(nbT)
; **********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    PROX_SHEET, P_to_DATA+'prox_sheet_unitest.prox', /Unitest
    if (strpos(!error_state.msg,'INIT_ERROR_STRING') lt 0) then $
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if cell.sn ne W20 then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
    TO_DID_CUR,'W1'
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *************************************
  st = 'BROWSE INSTRUMENT PATH GUI ...'            & nbT+=1 & st = st+string(nbT)
; *************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_MAC_EVENT, {ID:0L}, [-88,562,0,0,0 ,-1,-1,0]
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **********************************
  st = 'CALIBRATION GUI AND LIST...'               & nbT+=1 & st = st+string(nbT)
; **********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    CALIBRATION
    CALIBRATION, /LIST
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed
 
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
; **************************
  st = 'USER MACROS GUI ...'                       & nbT+=1 & st = st+string(nbT)
; **************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    macro_files,{ID:0L}, [-88,203,0,0]
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************
  st = 'SAVE SESSION ...'                          & nbT+=1 & st = st+string(nbT)
; ***********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    SAVESESSION
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  file_delete,'lamp.ses', /ALLOW_NON, /QUIET

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *******************************
  st = 'PLOTTING OPTIONS GUI ...'                  & nbT+=1 & st = st+string(nbT)
; *******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_DID_CREATE_BEGOOD, 1, 70, 24, 0, intarr(4,2), lonarr(21)
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ************************
  st = 'CUSTOMIZE GUI ...'                         & nbT+=1 & st = st+string(nbT)
; ************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    CUSTOMIZ,1
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'DATA PARAMETERS GUI ...'                   & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    PAR_DISP, {ID:0L}
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **********************
  st = 'JOURNAL GUI ...'                           & nbT+=1 & st = st+string(nbT)
; **********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_DID_JOURNAL, 0L, [-88,396,0,0]
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *****************************
  st = 'PERIODIC TABLE GUI ...'                    & nbT+=1 & st = st+string(nbT)
; *****************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    PERIODIC_TABLE
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'E_LogBOOK (LIV_LAMP)...'                   & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    SEE,w=1, /surface, /beside
    SEE,w=1, /image,  /LOGBOOK
    if strpos(!error_state.msg,'e-LogBook.html') eq -1 then $
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *****************************
  st = 'LAMP GEORGE LAYOUT ...'                    & nbT+=1 & st = st+string(nbT)
; *****************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_LAYOUT, 'george'
    P_LAYOUT, 'lamp'
    P_LAYOUT, 'full'
    P_LAYOUT, 'light'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ********************
  st = 'SUPERPLOT GUI ...'                         & nbT+=1 & st = st+string(nbT)
; ********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    SUPRPLOT,1
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *****************************
  st = 'SCROLL SPECTRA GUI ...'                    & nbT+=1 & st = st+string(nbT)
; *****************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_BEN_EVENT,{ID:0L}, [-88,402,0]
    P_DID_SETWIN0
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***************************************
  st = 'BASIC SECTOR INTEGRATION GUI ...'          & nbT+=1 & st = st+string(nbT)
; ***************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P_BEN_EVENT,{ID:0L}, [-88,401,0]
    P_DID_SETWIN0
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
; *******************************************
  st = 'MAGDRAW GUI + XDATCAR (LIV_OBJET)...'      & nbT+=1 & st = st+string(nbT)
; *******************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    MAGNET_WIN, P_to_DATA+'XDATCAR', /PLOT
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (round(total(W19))  ne 744)   or $ ;Frames calcul <Cr><Lf>!
      ((round(total(PV19)) ne 27804) and (round(total(PV19)) ne 27440)) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
; **********************************
  st = 'RDOPR RDID W_ACCU (D16) ...'               & nbT+=1 & st = st+string(nbT)
; **********************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    RDSET, INST='D16', /RAW, PATH=p_to_data
    SET_TOLERANCE, 0.
    W = RDOPR('92557+92670', W=19, DATP=rdp)
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (long(total(W19)/1000.) ne 255) or ((size(W19))[1] ne 502)  then  begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'STRAIGHTENING (D16) ...'                    & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    Y19 = findgen(320)/319.*31.9 + .05 - 16. ;Y in cm and keep X in 2theta
    dst = rdp.p[12]                          ;Sample distance (cm)
    XICUTE, 'W20 = STRAIGHT_2D(W19, DISTANCE='+string(dst)+', /INTEGR)'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (long(total(W20)/1000.) ne 493) or ((size(W20))[1] ne 969) then $
    if (long(total(W20)/1000.) ne 503) or ((size(W20))[1] ne 989) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **********************
  st = 'STR_FIT GUI ...'                           & nbT+=1 & st = st+string(nbT)
; **********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    STR_FIT
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
; *****************************
  st = 'STR_FIT PSEUDO FIT ...'                    & nbT+=1 & st = st+string(nbT)
; *****************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    P = STR_FIT(0, Win=1, Bkgd="flat", /pseudo, Fit=f,Res_Dfit=d,Sigma=s,ConvFunc=c, Out_SubF=o)
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(F)) ne round(W_tot) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
; ***************************
  st = 'STR_FIT FIT MODEL...'                      & nbT+=1 & st = st+string(nbT)
; ***************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++ & cd, mee
  endif else begin
    cd, P_to_DATA
    P = STR_FIT(0, Win=1 ,Bkgd="flat",User_model="model_user_last.pro", Fit=f)
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(F)) ne round(W_tot) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  file_delete,P_to_DATA+'str_params.txt', /ALLOW_NON, /QUIET

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
; *********************
  st = 'GK_FIT GUI ...'                            & nbT+=1 & st = st+string(nbT)
; *********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    GFIT
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************
  st = 'QENS_FIT GUI ...'                          & nbT+=1 & st = st+string(nbT)
; ***********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    QENS_FIT
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************
  st = 'INX GUI ...'                               & nbT+=1 & st = st+string(nbT)
; ******************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    INX
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
; ***********************************************
  st = 'DIDLINE(IN6) ,LINEUP ,GROUPX ,GROUPY ...'  & nbT+=1 & st = st+string(nbT)
; ***********************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    RDSET, inst='IN6'
    READ_LAMP,p_to_data+'sample-Cd6Tb_300k.hdf' , W=2
    READ_LAMP,p_to_data+'empty-can_300k.hdf'    , W=3
    READ_LAMP,p_to_data+'vanadium_300k.hdf'     , W=4
    XICUTE,'W5 = DIDLINE(samp=2,vana=4,empty=3,canva=0,cadm=0,resol=0,'     + $
                        'trans_abs=[0.600,0.850,0.000,0.100,0.000,0.000,'   + $
                        '0.000,1.000,0.000,1.000,1.000,0.000,-4.000,1.500,' + $
                        '300.000,0.000,0.000])'
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(W5*10000.)) ne 437 then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************************
  st = 'UNROLLING IMAGE GUI ...'                   & nbT+=1 & st = st+string(nbT)
; ******************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    WDIAG
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *************************
  st = 'TOMOGRAPHY GUI ...'                        & nbT+=1 & st = st+string(nbT)
; *************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    TOMOGRAPHY
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ***********************************************
  st = 'LIVC SURFACE/CONTOUR/BESIDE PLOT GUI ...'  & nbT+=1 & st = st+string(nbT)
; ***********************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    SEE, W=1, /SURFACE,/CONTOUR,/BESIDE
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************
  st = 'REGULAR ...'                               & nbT+=1 & st = st+string(nbT)
; ******************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    W20 = [[[W1]],[[W1]],[[W1]],[[W1]]]
    X20 = alog(X1)
    Y20 = alog(Y1)
    W20 = REGULAR(W20, DATP={X:X20, Y:Y20, Z:[1,2,3,4],E:0})
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (long(total(W20)/n_elements(W20)) ne long(W_tot/n_elements(W1))) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **************************************
  st = 'MAKE_VOLUME->LIV_OBJET , SEE...'           & nbT+=1 & st = st+string(nbT)
; **************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    W20 = [[[W1]],[[W1]],[[W1]],[[W1]]]
    SEE, W=20, /beside
    XICUTE,'W20=MAKE_VOLUME(W20,/proj)'
    SEE, W=20, /beside
    if strpos(!error_state.msg,'100.CTB') eq -1 then $
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (abs( total(N20)/n_elements(N20) - W_tot/n_elements(W1) ) gt 1.)  $
        or ( total(PV20) ne 12325. ) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *********************
  st = 'COSMOS GUI ...'                            & nbT+=1 & st = st+string(nbT)
; *********************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    RDSET, inst='D17'
    COSMOS
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
; ***************************
  st = 'BS (IN10) MACROS ...'                      & nbT+=1 & st = st+string(nbT)
; ***************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++ & cd, mee
  endif else begin
    RDSET, inst='IN10'
    cd, P_to_DATA
    XICUTE,'@in10_elastic_scan'
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(W33)) ne 191. then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  file_delete,P_to_DATA+'attenuation_factors.dat', /ALLOW_NON, /QUIET
  file_delete,P_to_DATA+'temperatures.dat',      /ALLOW_NON, /QUIET
  file_delete,P_to_DATA+'backthrough.dat',     /ALLOW_NON, /QUIET
  file_delete,P_to_DATA+'str_params.txt',    /ALLOW_NON, /QUIET
  file_delete,P_to_DATA+'lamp.jou',        /ALLOW_NON, /QUIET
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  ; ***************************
  st = 'BS (IN16B) MACROS ...'                      & nbT+=1 & st = st+string(nbT)
  ; ***************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++ & cd, mee
  endif else begin
    RDSET, inst='IN16B'
    cd, P_to_DATA
    XICUTE,'@in16b_qens_treat'
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(W7)*1000) ne 680093 then begin
      lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

  ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
; ***************************
  st = 'TOF (IN4) MACROS ...'                      & nbT+=1 & st = st+string(nbT)
; ***************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++ & cd, mee
  endif else begin
    RDSET, inst='IN4'
    cd, P_to_DATA
    XICUTE,'@in4_gdos'
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(W20)*1000.) ne 28. then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  file_delete,P_to_DATA+'gdos_Hydgraphene_0.85A.dat',        /ALLOW_NON, /QUIET

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  ; ***************************
  st = 'D16 MACROS ...'                      & nbT+=1 & st = st+string(nbT)
  ; ***************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++ & cd, mee
  endif else begin
    RDSET, inst='D16'
    cd, P_to_DATA
    XICUTE,'@d16_dif_scan'
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if round(total(W21)) ne 294 then begin
      lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse

  ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ************************************
  st = 'SANS_SHEET GUI + TREAT D22...'             & nbT+=1 & st = st+string(nbT)
; ************************************
  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
  catch,stat
  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
  endif else begin
    SANS_SHEET, savfile = P_to_DATA+'d22_sheet_unit_test.sav'   ;Start from a saved file
    TRAP_SET, fill=1
    SANS_SETTINGS_GETMASK, size(w9),[0.,0.],M,/nobeam,/noWbeam  ;Get the detector mask
    GIVE_W, W=61, M                                             ;Re-calculate all transmissions
    xicuter,'t=SANS_TRANSM(Sbeam=5,wat=1,Ewat=2,sam=3,Esam=4,Wbea=6,Xr=[59,70],yr=[57,72],Mask=w61)'
    common c_sans_base0   , base0
    widget_control, base0, get_uvalue= baset
    qdep = '.14*exp(-4.*alog(2.)*(((THETAX-.588)<0.)/.414)^2)'
    
    SANS_SETTINGS_SWAP, baset, LMH="low"   , /get    ;Set to low    angle
    xicuter,"c=SANS_CENTER(W9,Xr=[52.,76.],yr=[51.,75.],/Fit,Mask=w61)" ;Re-calculate center
    xicuter,"w18=SANS_COR(sam=12,wat=10,Ewat=11,Esam=13,scadm=14,wcadm=15,Qdep='"+qdep+"')" ;Cor
    xicuter,"w19=SANS_RADIAL(W18,minInt=5,incr=1.0)" ;Integration
    totLow =total(w19)
    SANS_SETTINGS_SWAP, baset, LMH="middle", /get    ;Set to middle angle
    xicuter,"c=SANS_CENTER(W9,Xr=[59.,70.],yr=[58.,71.],/Fit,Mask=w61)" ;Re-calculate center
    xicuter,"w18=SANS_COR(sam=12,wat=10,Ewat=11,Esam=13,scadm=14,wcadm=15,Qdep='"+qdep+"')" ;Cor
    xicuter,"w19=SANS_RADIAL(W18,minInt=5,incr=1.0)" ;Integration
    totMid =total(w19)
    SANS_SETTINGS_SWAP, baset, LMH="high"  , /get    ;Set to high   angle
    xicuter,"c=SANS_CENTER(W9,Xr=[22.,32.],yr=[58.,70.],/Fit,Mask=w61)" ;Re-calculate center
    xicuter,"w18=SANS_COR(sam=12,wat=10,Ewat=11,Esam=13,scadm=14,wcadm=15,Qdep='"+qdep+"')" ;Cor
    xicuter,"w19=SANS_RADIAL(W18,minInt=5,incr=1.0)" ;Integration
    totHigh=total(w19)
    
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if (long(totLow*10) ne 2204) or  $
       (long(totMid*10) ne 470 ) or (long(totHigh*10) ne 165) then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
  endelse
  lamp_test_clean, W_managed

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ************************
st = 'TEST V1 (Berlin) ...'                        & nbT+=1 & st = st+string(nbT)
; ************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
   if !version.os ne 'Win32' then begin ;No 'so' for Windows
    RDSET, inst='V1'
    cd, P_to_DATA
    W = RDOPR('51284', W=19, DATP=rdp)    
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if max(W19[*,*,4]) ne 690. then begin
       lamp_message,'!Different result' & printf,u,'!Different result'   & nbE++ & endif
   endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ****************************
st = 'TEST PELICAN (Ansto) ...'                    & nbT+=1 & st = st+string(nbT)
; ****************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
    RDSET, inst='PELICAN'
    cd, P_to_DATA
    W = RDOPR('4677', W=19, DATP=rdp)
    cd, mee
    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
    if total(W19[*,199,59]) ne 2. then begin
      lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; **************************
st = 'TEST SPEC{} (ESRF) ...'                       & nbT+=1 & st = st+string(nbT)
; **************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='Spec'
  cd, P_to_DATA
  W = RDOPR('spec_Id31.dat{1:20}', W=19, DATP=rdp)
  cd, mee
  if strpos(!error_state.msg, 'Type conversion error') lt 0 then $ ;Happen sometimes!
  if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  if (total(W19[*,19]) ne 1439.) or (round(total(X19[*,19])*10.) ne 9246) then begin
    lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ********************************
st = 'TEST PREFIX,SUFFIX JOINs ...'                 & nbT+=1 & st = st+string(nbT)
; ********************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='Column'
  cd, P_to_DATA
  RDSET, prefix ='TiO2_01C_14keV_'     ;Set file prefix
  RDSET, suffix ='_mar3450.txt'        ;Set file suffix
  W = RDOPR('001:004', W=19, DATP=rdp) ;Read 4 scan files
  RDSET, prefix='', suffix=''          ;Reset prefix and suffix
  cd, mee
  if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  if (W19[2499,3] ne 624.75) or (round(X19[2499,3]) ne 62) then begin
    lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ******************
st = 'TEST D2B  ...'                                & nbT+=1 & st = st+string(nbT)
; ******************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='D2B', /noraw ; therefore import DIF plugins
  cd, P_to_DATA
  CALIBRATION, file='d2b_unittest.cal'
  W  = RDOPR('514500', W=30)
  W  = RDOPR('514501', W=31)
  W  = RDOPR('514500>514501', W=32)
  Wt = total(W32,1)
  cd, mee
  if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  if (X30[0] ne -10.0) or (X32[3224] ne 151.2) or (round(Wt[127]) ne 10033) then begin
    lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *************************
st = 'TEST READING D7  ...'                         & nbT+=1 & st = st+string(nbT)
; *************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='D7' ; therefore import TOF_LSS plugins
  cd, P_to_DATA
  W  = RDOPR('204405', W=28)
  Wt = total(W28,1)
  cd, mee
  if (!error_state.msg gt ' ') and (strpos(!error_state.msg,'arithmetic') lt 0) then begin
                                    lamp_error & printf,u,!error_state.msg & nbE++ & endif
  if (Wt[0] ne 109.) or (Wt[1] ne 89.) then begin
    lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; *************************
st = 'TEST READING IN5  ...'                        & nbT+=1 & st = st+string(nbT)
; *************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='IN5', /raw ; therefore import IN5 plugins
  cd, P_to_DATA
  W  = RDOPR('133280', W=27)
  cd, mee
  if (!error_state.msg gt ' ') then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
  if (total(W27) ne 60663.) or (round(x27[383]*10.) ne -119) or (round(y27[240]*10.) ne 1469) then begin
      lamp_message,'!Different result' & printf,u,'!Different result'    & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ********************************
st = 'JUST LOAD BRISP PLUGIN  ...'                  & nbT+=1 & st = st+string(nbT)
; ********************************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  RDSET, inst='BRISP' ; therefore import IN5 plugins
  if (!error_state.msg gt ' ') then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
endelse

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

; ********************
st = 'LIVE UPDATE ...'                              & nbT+=1 & st = st+string(nbT)
; ********************
printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
catch,stat
if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
endif else begin
  lu_lamp, /silent, oBridge=d0, /test
  if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & nbE++ & endif
endelse
lamp_test_clean, W_managed


; *******************
  st = 'SCAN GUI ...'                               & nbT+=1 & st = st+string(nbT)
; *******************
;  printf,u, '' & printf,u,st & print,'' & print,st & message,/reset
;  catch,stat
;  if stat ne 0 then begin & lamp_error & printf,u,!error_state.msg & nbE++
;  endif else begin
;    P_DID_EVENT,0,[-88,306,0,1] ; or SCAN,1
;    if !error_state.msg gt ' ' then begin & lamp_error & printf,u,!error_state.msg & endif
;  endelse
;  lamp_test_clean, W_managed


; ********************
st = 'LAMP SERVER ...'                              & nbT+=1 & st = st+string(nbT)
; ********************
; START_lamp -nws
; Lamp> W2=dist(80)
; Lamp> see,w=2
; A lamp.png file must be produced ...
; 
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


  cd, Mee
  printf,u, '' & printf,u, '*********** ' & printf,u, 'End of test ',nbE,' error(s)'
                 printf,u, '*********** '
  print   , '' & print,    '*********** ' & print,    'End of test ',nbE,' error(s)'
                 print,    '*********** '
  free_lun,u
  file_delete,P_to_DATA+'lamp.jou',        /ALLOW_NON, /QUIET
  W_EXCHANGE, /ALL  ; Reset all changes
  RDSET,path=KP_path, inst=KP_inst
  TRAP_SET, fill=1
  xdisplayfile,'lamp_fast_unit_test.txt', GROUP=lamp_b1, TITLE='lamp_fast_unit_test.txt'
  return
 
miswrite: lamp_error
end
