function discover_read,file_seed,n_atoms
;+
;******* *************
;** User callable
;** routine to read Discover MD files from Materials Studio
;** (no fixed atoms, not a continuation trajectory)
;**
;** output is a workspace in LAMP and ascii input files for nMoldyn
;**
;** call is w1=discover_read('seed',number of atoms)
;** where 'seed' is the name of the MS simulation
;** NOTE a shelx structure file called seed.res and
;** saved from the first frame of the simulation
;** must be in the same working directory
;**
;** return matrix (time,xyz,atoms) to a workspace in lamp
;** could read .inp and .out files in addition to .his to get all pars
;** need to write a file of fractional coords for a periodic universe, or
;** a file of continuous Cartesian coordinates for an infinite universe
;**
;** cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
;** acquired from Don as a subroutine in a bigger prog
;** Fortran routine is probably called one frame at a time
;** hence the IF statements in Don's routine
;** his file stays open all the time for DON hence no open or close here
;** cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
;**       following format described in
;**               http://www.accelrys.com/doc/life/insight2000.1/
;**               formats980/File_Formats_1998.html#483003
;** ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
;** variable definitions following web page (above) and the Don/Lambert routine
;** many variables are defined "on the fly"
;-

;character variables
versioninfo=bytarr(80) & title=bytarr(80)
;double prec variables
Vershn=0D
a=0D & b=0D & c=0D & alpha=0D & beta=0D & gamma=0D
lattvec=dblarr(3,3) & cryst2cart=dblarr(3,3) & cart2cryst=dblarr(3,3)
translmatrix=dblarr(3,196) & rotmatrix=dblarr(3,3,196) & symmmatrix=dblarr(3,3,196)
framestep_fsec=0D
;integer variables
NAtTyp=0L & NNmRes=0L & Natoms=0L & res1=0L & NAtMov=0L
NMol=0L & NRes=0L & NBonds=0L & NSymOp=0L & NEner=0L
framewritefreq=0L & framestartnum=0L & Nskip=0L
tmp=bytarr(15)
i=0L & j=0L & k=0L

;******************************************************
;read discover inp file to get atom order and
;parameters for nMoldyn
;******************************************************
junk=' '
inp_file=file_seed+'.inp'
openr,iunit,inp_file,/get_lun
i_rd=0
while i_rd eq 0 do begin
  readf,iunit,junk & junk=strtrim(junk)
  if (junk eq '# Dynamics Section:') then i_rd=1
endwhile
readf,iunit,junk
readf,iunit,junk ; read total time
line_con=strsplit(junk,' ',/extract) & tot_time=line_con[2] & print,tot_time
readf,iunit,junk ; read time step
line_con=strsplit(junk,' ',/extract) & time_step=line_con[2] & print, time_step
free_lun,iunit
print,'inp file read'
;******************************************************
;discover inp file read
;******************************************************

;******************************************************
;read discover his file
;******************************************************
; open his file, specify f77 unformatted, endian convention to read on Pentium
; read records in single read statements
his_file=file_seed+'.his'
openr,iunit,his_file,/get_lun,/f77,/swap_if_little
;cccccccccc READ HEADER FRAME
;cccccccccc  GENERAL INFO
;records 1-4
  readu,iunit,i
  readu,iunit,versioninfo,Vershn & print, string(versioninfo), Vershn
  readu,iunit,title & readu,iunit,title

;cccccccccc FORCE FIELD TYPING INFORMATION
  NAtTyp=1L
;  readu,iunit,NAtTyp & print, NAtTyp
  Typename=bytarr(4*NAtTyp) & Typemass=dblarr(NAtTyp)
;record 5, 3 variables
  readu,iunit,NAtTyp,Typename,Typemass

;cccccccccc MODEL INFORMATION (N.O. ATOMS, MOLECULES, ETC)
  NNmRes=1L &  nameshort=bytarr(4*NNmRes)
;record 6, 2 variables
  readu,iunit,NNmRes,nameshort

  NAtoms=long(n_atoms) & n=lonarr(NAtoms) & nameshort=bytarr(4*NAtoms)
;record 7, 3 variables
  readu,iunit,NAtoms,n,nameshort

  reserved_int=0L & NAtMov=NAtoms & n=lonarr(NAtMov)
;record 8, 3 variables
  readu,iunit,reserved_int,NAtMov,n

  NMol=1L
  NAtpMol=lonarr(NMol) & NRespMol=lonarr(NMol) & NpMol=lonarr(2,NMol)
;record 9, 3 variables
  readu,iunit,NMol,NpMol

  NRes=1L
  first=lonarr(NRes) & last=lonarr(NRes) & firstlast=lonarr(2,NRes)
;record 10, 3 variables
  readu,iunit,NRes,firstlast

;record 11a
  readu,iunit,NBonds
  if ( NBonds gt 0 ) then begin
    Atom1=lonarr(NBonds) & Atom2=lonarr(NBonds) & Atom12=lonarr(2,NBonds)
    ;record 11b
    readu,iunit,Atom12
  endif

;cccccccccc CRYSTAL STRUCTURE INFORMATION; parameters then matrices
;record 12
  res_real4=fltarr(4) & res_real6=fltarr(6) & res_int0=0L & res_int6=lonarr(6)
  readu,iunit,a,b,c,alpha,beta,gamma,$
              lattvec,cryst2cart,cart2cryst,symmmatrix,translmatrix,rotmatrix, $
              NSymOp, $
              res_real4,res_int0,res_real6,res_int6
  print,'Cell information, a,b,c,alpha,beta,gamma'
  print,a,b,c,alpha,beta,gamma

;cccccccccc DYNAMICS INFORMATION
;record 13
  readu,iunit,NEner,framestep_fsec,framewritefreq,framestartnum
  print,'Frames info, NEner,framestep_fsec,framewritefreq'
  print,NEner,framestep_fsec,framewritefreq

;cccccccccc ENERGY AND THERMODYNAMICS  INFORMATION
  bla_real=dblarr(59+5*NMol+NEner+NMol*NEner)
;record 14
  readu,iunit,bla_real

;ccccccc END READ HEADER FRAME

;calculate values from inp file and his header
n_frames=round(float(tot_time/framestep_fsec)/float(framewritefreq))
frame_time=float(time_step)*float(framewritefreq)

;ccccccc BEGIN READ FRAMES
;c
;cccccccccc STARTING COORDINATES AND VELOCITIES (angstrom and angstrom/step)
;c       "section=1"
;c
  xyz_all=fltarr(n_frames,3,NAtoms)
  xyz4=fltarr(3,NAtoms)
;record 15
  readu,iunit,xyz4
  xyz_all[0,*,*]=xyz4[*,*]
  print,xyz4[*,0]

  cart_velo=fltarr(3,NAtoms)
;record 16
  readu,iunit,cart_velo
;c

;cccccccccc START OF FRAME LOOP
;c......Fixed atoms are not included in data frames here
;c       "section=2"
;c
for i_frame=1,n_frames-1 do begin
  readu,iunit,i
;record n+1
  readu,iunit,bla_real
;record n+2
  readu,iunit,a,b,c,alpha,beta,gamma,lattvec
;record n+3
  readu,iunit,xyz4
  xyz_all[i_frame,*,*]=xyz4[*,*]
;record n+4
  readu,iunit,cart_velo
endfor

free_lun,iunit
print,'.his file read'
;******************************************************
;finished his read
;******************************************************

;******************************************************
;read shelx file to get atom order and
;parameters for nMoldyn
;******************************************************
shelx_file=file_seed+'.res'
openr,iunit,shelx_file,/get_lun
junk=' ' & readf,iunit,junk ; skip title
print,'shelx file open'

readf,iunit,junk ; read cell
  line_con=strsplit(junk,' ',/extract)
  a=float(line_con[2]) & print, a; parameter a
  b=float(line_con[3]) & print, b; parameter b
  c=float(line_con[4]) & print, c; parameter c

readf,iunit,junk ; skip LATTICE

readf,iunit,junk ; read atom types
  line_con=strsplit(junk,' ',/extract)
  n_types=n_elements(line_con)-1 & print, n_types
  atoms=strarr(n_types)
  for i=0,n_types-1 do begin
    atoms[i]=line_con[i+1] & print, atoms[i]
  endfor

;read block of atom data
atom_label=intarr(n_atoms)
for i=0,n_atoms-1 do begin
  readf,iunit,junk
  line_con=strsplit(junk,' ',/extract)
  atom_label[i]=line_con[1]
endfor

free_lun,iunit
print,'shelx file read'

n_atoms_types=intarr(n_types)
for i=0,n_types-1 do begin
  n_atoms_types[i]=n_elements(where(atom_label eq i+1)) & print, n_atoms_types[i]
endfor

;write input file for nMoldyn
openw,iunit,'traj.inp',/get_lun
printf,iunit,'MAXSTEP'
printf,iunit,n_frames
printf,iunit,'TIMESTEP'
printf,iunit,frame_time/1000.0
printf,iunit,'CELL'
printf,iunit,a,b,c
printf,iunit,'CUTOFF'
printf,iunit,'80'
printf,iunit,'ATOMS'
for i=0,n_types-1 do begin
  printf,iunit,atoms[i]
  printf,iunit,n_atoms_types[i]
endfor

free_lun,iunit
print,'nMoldyn inp file written'
;******************************************************
;finished SHELX read
;******************************************************

;******************************************************
; write coordinate trajectory file
; in order of atom types
;******************************************************
print, 'writing coordinate file'
;fractional coords for output
xyz_all[*,0,*]=xyz_all[*,0,*]/a
xyz_all[*,1,*]=xyz_all[*,1,*]/b
xyz_all[*,2,*]=xyz_all[*,2,*]/c
openw,iunit,'traj.xyz',/get_lun
printf,iunit,'xdatcar-type dummy header - 6 lines'
printf,iunit,'xdatcar-type dummy header - 6 lines'
printf,iunit,'xdatcar-type dummy header - 6 lines'
printf,iunit,'xdatcar-type dummy header - 6 lines'
printf,iunit,'xdatcar-type dummy header - 6 lines'
printf,iunit,'xdatcar-type dummy header - 6 lines'
coord=fltarr(3) & coord_all=xyz_all
for i_frame=0,n_frames-1 do begin
  i_count=0
  for i_type=0,n_types-1 do begin
    atom_list=where(atom_label eq i_type+1)
    for i_atom=0,n_atoms_types[i_type]-1 do begin
      coord=xyz_all[i_frame,*,atom_list[i_atom]]
      printf,iunit,format='(3f12.6)', coord[0],coord[1],coord[2]
      coord_all[i_frame,*,i_count]=coord
      i_count=i_count+1
    endfor
  endfor
  printf,iunit,' '
endfor
free_lun, iunit
print, 'coordinate file done'

; return atoms in right order to Lamp
return,coord_all

end


