;TS 2018-06-04 Removed the restriction that om_fwhm is disregarded for polarized runs since the resolution calculation now should be based only on the UP polarization. 
;PG 2017-08-07 Added wavelength resolution smearing of wide beam at mid-chopper position (incoherent case), updated gravity correction, mainly openoffset correction, added chopper radius into c_params.cr 
;TS 2016-10-25 Deleted om_fwhm^2 since this messes up in comparison between different runs when intensities change and 
;              therefore the bendiness calculation gives a different result. NOTE: this should not be changed in the general cosmos
;TS 2016-10-21 COSMOS was writing too many files if only two channels were measured
;TS 2016-10-21 Had a problem that the wavelength is sometimes one point longer or shorter in different pol runs... I don't know why this happens
;TS 2016-10-21 Identify the pol channels that are empty since the message pops up if only tow or three channels measured
;TS 2016-10-21 changed reflect1[0] to reflect1[0,0] to accomodate multiple files
;
;;Modified on 27/07/15 by PG:
;If the mean chopper opening is not recorded use requested one in this case
;dan option integrated for FIGARO
;the coherent option uses the wavelength corrected twotheta in case of FIGARO
;Modified on 27/06/01 by TS:
;Compare and distinguish between cases of S2/C larger or smaller than 2Dw
;Fixed a bug in the coherent summation case concerning lam_top/bot and lam_min/max.
;Modified on 06/05/15 by PG:
;Different slit settings for direct and reflected beam are now possible for constant q COSMOS.
;Modified on 28/04/15 by PG:
;Error in divergent case coherent COSMOS corrected. Check for same slit settings for direct and reflected beam for constant q COSMOS. 
;Modified on 26/04/15 by PG:
; Check for large difference in actual and mean chopper opening and use actual in this case (NoMad bug)
;Modified on 12/03/15 by PG:
; Small correction for error calculation for constant q version and no binning for 0 count and add a threshhold for bending check
;Modified 12/3/15: RC removed a factor (bins) from coherent error calculation
;Modified 9/3/15: RC added special case for figaro bounce up coherent as the pixel sense is inverted cf D17 and bounce down
;Modified on 11/03/15 by PG:
; Corrected error calculation for constant q version and to apply the search for 0 error at the very end
;Modified on 06/03/15 by PG:
; Small correction to take the distances for the reflected beam rather than the direct beam for constant q version
;Modified on 04/03/15 by PG:
; Correcting the possible theta-offset in the constant q version of FIGARO
;Modified on 16/02/15 by PG:
; Some changes in pixel-range in the constant q version
;Modified on 13/02/15 by PG:
; Corrected an error in the resolution calculation (interslit distance instead of sdd for the incoming angular spread)
;Modified on 10/02/15 by PG:
; Improvement in the binning procedure
;Modified on 03/10/14 by PG:
; If foreground width and s3w produce a smaller angular divergence than s2w and s3w, the former is used to calculate the final resolution
;Modified on 12/02/13 by PG:
; Angular resolution calculation updated
;Modified on 01/02/13 by PG:
; Added option to have different chopper openings for direct and reflected beam
;Modified on 31/01/13 by PG:
; Statistical error calculation updated (error of 0 counts is 1)

pro cosmos_analysis_init

    forward_function cosmos_str_make
    forward_function cosmos_str_parse
    forward_function cosmos_str_theta
    forward_function cosmos_anal_sort
    forward_function cosmos_anal_mean
    forward_function cosmos_get_table_labels
    forward_function cosmos_getnumors    
   
    cosmos_str

end

; append datasets [ x, err_x, y, err_y ]
function cosmos_anal_append, array1, array2

    return, transpose([ transpose(array1), transpose(array2) ])
  
end

; subtract averaged background on array [ x, t ] with error [ x, t ]
pro cosmos_anal_backgroundaverage_old, arr, err, maskbk

    a=size(maskbk)
    bkgndnum = a[1]
    
    for t = 0, n_elements(arr[0, *]) - 1 do begin
      bkgnd = mean(arr[maskbk, t])
      bkgnderrorsquared = total(err[maskbk, t]^2) / (bkgndnum^2)
      arr[*, t] = arr[*, t] - bkgnd
      err[*, t] = sqrt(err[*, t]^2 + bkgnderrorsquared) ; add error in quadrature
    endfor

    return
  
end

; subtract average but 2d background --> Not used since v.3.2.13 (problem with error bars)
pro cosmos_anal_backgroundaverage, arr, err, maskbk, str
  ; new version of average written by bob 29-5-15 to subtract background from each pixel
  ; good for both coherent and incoherent.  take mean intensity and mean pixel from background
  ;strips and extrapolates to foreground pixels and subtracts

  tsize = n_elements(arr[0, *])
  xsize = n_elements(arr[*, 0])
  tlim = tsize - 1
  msize=n_elements(maskbk)
  parr=fltarr(xsize)
  parre=fltarr(xsize)

  err2=fltarr(xsize)
  gradoff=fltarr(2,tsize)
  maskbk_t=msize-1
  ;search for a break in the maskbk
  
  for i=0,msize-2 do begin
    if maskbk[i+1]-maskbk[i] gt 1 then maskbk_t=i
  endfor
  
  for t = 0, tlim do begin   ;loop over all time channels

    for k=0,xsize-1 do begin
      p=arr[k,t]*float(k)
      parr[k]=p
      parre[k]=(err[k,t]*float(k))^2
      err2[k]=(err[k,t])^2
    endfor
    
  
    ;maskbk_t=msize-1 means there was only one background left or right but not both
    if maskbk_t ne msize-1 then begin
      
      maskbkt=maskbk[(maskbk_t+1):(msize-1)]
      maskbkb=maskbk[0:maskbk_t]
  
      i1=mean(arr[maskbkb,t])
      i1e=sqrt(total(err2[maskbkb])/n_elements(maskbkb))
      p1=float(total(parr[maskbkb])/total(arr[maskbkb,t]))
  
      if i1 eq 0 then p1=mean(float(maskbkb))
      
      i2=mean(arr[maskbkt,t])
      i2e=sqrt(total(err2[maskbkt])/n_elements(maskbkb))
      p2=float(total(parr[maskbkt])/total(arr[maskbkt,t]))
  
      if i2 eq 0 then p2=mean(float(maskbkt))
  
      gradoff[0,t]=(i2-i1)/(p2-p1)
      gradoff[1,t]=i1-gradoff[0,t]*p1
      
      gradmax=(i2+(0.5*i2e)-(i1-0.5*i1e))/(p2-p1)
      offmax=i1-gradmax*p1
      gradmin=(i2-(0.5*i2e)-(i1+0.5*i1e))/(p2-p1)
      offmax=i1-gradmin*p1
      backgmax = findgen(xsize) * gradmax + gradmax
      backgmin = findgen(xsize) * gradmin + gradmin
      backgerr=0.5*abs(backgmax-backgmin)
      ;if t eq 100 or t eq 500 or t eq 600 or t eq 700 or t eq 800 then begin
        ;print,(t),gradoff(0,t),gradoff(1,t)
        ;print,arr(maskbk,t)
      ;endif
      backg = findgen(xsize) * gradoff[0,t] + gradoff[1,t] ; 1D array in x
      arr[*, t] = arr[*, t] - backg
      ;if t eq 100 or t eq 500 or t eq 600 or t eq 700 or t eq 800 then begin
      ;print,arr(maskbk,t)
      ;print,backg(maskbk)
      ;endif
      ; add error in quadrature
      err[*, t] = sqrt(err[*, t]^2 +backgerr^2)
      ;cosmos_logmessage, ' Background average slope to ' + str + ' runs with regression factors  y = a x + b  :'
  ;    cosmos_logmessage, string(9b) + 'min' + string([9b, 9b]) + 'max'
  ;    cosmos_logmessage, '  a' + string(9b) + cosmos_str_make(min(gradoff[1, *])) + string(9b) + cosmos_str_make(max(gradoff[1, *]))
  ;    cosmos_logmessage, '  b' + string(9b) + cosmos_str_make(min(gradoff[0, *])) + string(9b) + cosmos_str_make(max(gradoff[0, *]))
    endif else begin
     ;only one background strip so must subtract a constant -the eman of that background strip
    bk=mean(arr[maskbk,t])
    arr[*, t] = arr[*, t] - bk
    bkgnderrorsquared = total(err[maskbk, t]^2) / (msize^2)
    err[*, t] = sqrt(err[*, t]^2 + bkgnderrorsquared)
    endelse

  endfor

  return

end
; subtract fitted background
pro cosmos_anal_backgroundfit, arr, err, maskbk, str
  
    compile_opt idl2
  
    tsize = n_elements(arr[0, *])
    xsize = n_elements(arr[*, 0])
    tlim = tsize - 1
    fit = fltarr(2, tsize)
    sigz = fltarr(2, tsize)
    backg=findgen(xsize)
    for i = 0, tlim do begin
  
        fit[*, i] = linfit(maskbk, arr[maskbk, i],sigma=a);,measure_errors=err(maskbk,i) weight with errors doesn't work if they are zero
        sigz[*,i]=a
        backg= findgen(xsize) * fit[1,i] + fit[0,i]
        backgmax = findgen(xsize) * (fit[1,i]+sigz[1, i]) + fit[0,i]-sigz[0, i] ; 1D array in x
        backgmin = findgen(xsize) * (fit[1,i]-sigz[1, i]) + fit[0,i]+sigz[0, i] ; 1D array in x
        backgerr = 0.5*abs(backgmax-backgmin)
        
       ;print,i,fit(1,i),sigz(1,i),fit(0,i),sigz(0,i)
        arr[*, i] = arr[*, i] - backg
        ; add error in quadrature
        
        err[*, i] = sqrt(err[*, i]^2 + backgerr^2)
  
    endfor
   
    ;cosmos_logmessage, ' Background fitted to ' + str + ' runs with regression factors  y = a x + b  :'
    cosmos_logmessage, string(9b) + 'min' + string([9b, 9b]) + 'max'
    cosmos_logmessage, '  a' + string(9b) + cosmos_str_make(min(fit[1, *])) + string(9b) + cosmos_str_make(max(fit[1, *]))
    cosmos_logmessage, '  b' + string(9b) + cosmos_str_make(min(fit[0, *])) + string(9b) + cosmos_str_make(max(fit[0, *]))

    return

end

; calculate angle offset for flat detector surface
function cosmos_anal_calcangleoffset, peakpos, xoff, density, distance

    common cosmos_params, c_params
    
    return, atan(((peakpos - xoff) * density - (c_params.pixels_x / 2.)-0.5) * c_params.pixelwidth, distance)
  
end

; calculate corrected angle for flat detector surface
function cosmos_anal_correctangle, angle, dirpeakpos, refpeakpos, dirxoff, refxoff, density, dirdist, refdist

    dirOffset = cosmos_anal_calcangleoffset(dirpeakpos, dirxoff, density, dirdist)
    
    refOffset = cosmos_anal_calcangleoffset(refpeakpos, refxoff, density, refdist)

    return, (2. * angle + dirOffset - refOffset)/2.0
    
end

; calculate corrected time of flight distance for flat detector surface
function cosmos_anal_correctdistance, peakpos, xoff, density, distance

    return, distance / cos(cosmos_anal_calcangleoffset(peakpos, xoff, density, distance))
  
end

;+
; :Description:
;    Factors and common calculations required by TOF polarization correction routines
;
; :Params:
;
; :Author: M. A. Gonzalez from T. Saerbeck's code
;
; :History:
;    M. A. Gonzalez, 29 April 2015
;
; :Categories: cosmos
;
;-
pro cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L,  InMat

  common cosmos_params, c_params
  common cosmos_settings, c_settings

  ;Default values:

  ;Efficiencies fitted linearly in five different regions
  F1_limits  = [8.12986, 12.11868, 15.0000, 20.09632]
  F2_limits  = [6.43601, 8.70000, 16.92311, 20.09632]
  Phi_limits = [8.129857, 13.000000, 16.107501, 20.096323]
  P1_limits  = [6.900000, 9.758882, 12.567910, 15.376938]
  P2_limits  = [6.949854, 9.758882, 12.567910, 15.376938]

  ;Factors (from fits)
  F1K  = [1.0, 0.0, 0.0, 0.0, 0.0, 0.0]
  F2K  = [1.0, 0.0, 0.0, 0.0, 0.0, 0.0]
  PhiK = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
  P1K  = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
  P2K  = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

  ;If file with polarization correction factors is given then read the factors from it
  if c_settings.polcorrfile gt '' then begin
    openr, u, c_settings.polcorrfile, /get_lun
    line = ''
    while ~EOF(u) do begin
      readf, u, line
      tmp = strsplit(line, '=', /extract, count=count)
      if count eq 2 then begin ;all lines without equal sign will be ignored
        ;remove all spaces and initial/final brackets or parentheses
        name = strtrim(tmp[0], 2)
        content = strtrim(tmp[1], 2)
        if strmid(content, 0, 1) eq '[' or strmid(content, 0, 1) eq '(' or strmid(content, 0, 1) eq '{' $
          then content = strmid(content, 1, strlen(content)-2)
        values = float(strsplit(content, ',', /extract))

        case name of
          'F1_limits':  F1_limits = values
          'F2_limits':  F2_limits = values
          'Phi_limits': Phi_limits = values
          'P1_limits':  P1_limits = values
          'P2_limits':  P2_limits = values
          'F1K':        F1K = values
          'F2K':        F2K = values
          'PhiK':       PhiK = values
          'P1K':        P1K = values
          'P2K':        P2K = values
        endcase
      endif
    endwhile

    close, u & free_lun, u
  endif

  F1A = fltarr(5) & F1A[0] = F1K[0]
  F1A[1] = F1A[0] + F1K[1] * F1_limits[0]
  F1A[2] = F1A[1] + F1K[2] * (F1_limits[1]-F1_limits[0])
  F1A[3] = F1A[2] + F1K[3] * (F1_limits[2]-F1_limits[1])
  F1A[4] = F1A[3] + F1K[4] * (F1_limits[3]-F1_limits[2])

  F2A = fltarr(5) & F2A[0] = F2K[0]
  F2A[1] = F2A[0] + F2K[1] * F2_limits[0]
  F2A[2] = F2A[1] + F2K[2] * (F2_limits[1]-F2_limits[0])
  F2A[3] = F2A[2] + F2K[3] * (F2_limits[2]-F2_limits[1])
  F2A[4] = F2A[3] + F2K[4] * (F2_limits[3]-F2_limits[2])

  PhiA = fltarr(5) & PhiA[0] = PhiK[0]
  PhiA[1] = PhiA[0] + PhiK[1] * Phi_limits[0]
  PhiA[2] = PhiA[1] + PhiK[2] * (Phi_limits[1]-Phi_limits[0])
  PhiA[3] = PhiA[2] + PhiK[3] * (Phi_limits[2]-Phi_limits[1])
  PhiA[4] = PhiA[3] + PhiK[4] * (Phi_limits[3]-Phi_limits[2])

  P1A = fltarr(5) & P1A[0] = P1K[0]
  P1A[1] = P1A[0] + P1K[1] * P1_limits[0]
  P1A[2] = P1A[1] + P1K[2] * (P1_limits[1]-P1_limits[0])
  P1A[3] = P1A[2] + P1K[3] * (P1_limits[2]-P1_limits[1])
  P1A[4] = P1A[3] + P1K[4] * (P1_limits[3]-P1_limits[2])

  P2A = fltarr(5) & P2A[0] = P2K[0]
  P2A[1] = P2A[0] + P2K[1] * P2_limits[0]
  P2A[2] = P2A[1] + P2K[2] * (P2_limits[1]-P2_limits[0])
  P2A[3] = P2A[2] + P2K[3] * (P2_limits[2]-P2_limits[1])
  P2A[4] = P2A[3] + P2K[4] * (P2_limits[3]-P2_limits[2])

  ;Wavelength dependence
  F1L = fltarr(n_elements(wavelength))
  L1 = where(wavelength le F1_limits[0])
  L2 = where(wavelength gt F1_limits[0] and wavelength le F1_limits[1])
  L3 = where(wavelength gt F1_limits[1] and wavelength le F1_limits[2])
  L4 = where(wavelength gt F1_limits[2] and wavelength le F1_limits[3])
  L5 = where(wavelength gt F1_limits[3])
  F1L[L1] = F1A[0] + F1K[1] * wavelength[L1]
  F1L[L2] = F1A[1] + F1K[2] * (wavelength[L2] - F1_limits[0])
  F1L[L3] = F1A[2] + F1K[3] * (wavelength[L3] - F1_limits[1])
  F1L[L4] = F1A[3] + F1K[4] * (wavelength[L4] - F1_limits[2])
  F1L[L5] = F1A[4] + F1K[5] * (wavelength[L5] - F1_limits[3])

  F2L = fltarr(n_elements(wavelength))
  L1 = where(wavelength le F2_limits[0])
  L2 = where(wavelength gt F2_limits[0] and wavelength le F2_limits[1])
  L3 = where(wavelength gt F2_limits[1] and wavelength le F2_limits[2])
  L4 = where(wavelength gt F2_limits[2] and wavelength le F2_limits[3])
  L5 = where(wavelength gt F2_limits[3])
  F2L[L1] = F2A[0] + F2K[1] * wavelength[L1]
  F2L[L2] = F2A[1] + F2K[2] * (wavelength[L2] - F2_limits[0])
  F2L[L3] = F2A[2] + F2K[3] * (wavelength[L3] - F2_limits[1])
  F2L[L4] = F2A[3] + F2K[4] * (wavelength[L4] - F2_limits[2])
  F2L[L5] = F2A[4] + F2K[5] * (wavelength[L5] - F2_limits[3])

  PhiL = fltarr(n_elements(wavelength))
  L1 = where(wavelength le Phi_limits[0])
  L2 = where(wavelength gt Phi_limits[0] and wavelength le Phi_limits[1])
  L3 = where(wavelength gt Phi_limits[1] and wavelength le Phi_limits[2])
  L4 = where(wavelength gt Phi_limits[2] and wavelength le Phi_limits[3])
  L5 = where(wavelength gt Phi_limits[3])
  PhiL[L1] = PhiA[0] + PhiK[1] * wavelength[L1]
  PhiL[L2] = PhiA[1] + PhiK[2] * (wavelength[L2] - Phi_limits[0])
  PhiL[L3] = PhiA[2] + PhiK[3] * (wavelength[L3] - Phi_limits[1])
  PhiL[L4] = PhiA[3] + PhiK[4] * (wavelength[L4] - Phi_limits[2])
  PhiL[L5] = PhiA[4] + PhiK[5] * (wavelength[L5] - Phi_limits[3])

  P1L = fltarr(n_elements(wavelength))
  L1 = where(wavelength le P1_limits[0])
  L2 = where(wavelength gt P1_limits[0] and wavelength le P1_limits[1])
  L3 = where(wavelength gt P1_limits[1] and wavelength le P1_limits[2])
  L4 = where(wavelength gt P1_limits[2] and wavelength le P1_limits[3])
  L5 = where(wavelength gt P1_limits[3])
  P1L[L1] = P1A[0] + P1K[1] * wavelength[L1]
  P1L[L2] = P1A[1] + P1K[2] * (wavelength[L2] - P1_limits[0])
  P1L[L3] = P1A[2] + P1K[3] * (wavelength[L3] - P1_limits[1])
  P1L[L4] = P1A[3] + P1K[4] * (wavelength[L4] - P1_limits[2])
  P1L[L5] = P1A[4] + P1K[5] * (wavelength[L5] - P1_limits[3])

  P2L = fltarr(n_elements(wavelength))
  L1 = where(wavelength le P2_limits[0])
  L2 = where(wavelength gt P2_limits[0] and wavelength le P2_limits[1])
  L3 = where(wavelength gt P2_limits[1] and wavelength le P2_limits[2])
  L4 = where(wavelength gt P2_limits[2] and wavelength le P2_limits[3])
  L5 = where(wavelength gt P2_limits[3])
  P2L[L1] = P2A[0] + P2K[1] * wavelength[L1]
  P2L[L2] = P2A[1] + P2K[2] * (wavelength[L2] - P2_limits[0])
  P2L[L3] = P2A[2] + P2K[3] * (wavelength[L3] - P2_limits[1])
  P2L[L4] = P2A[3] + P2K[4] * (wavelength[L4] - P2_limits[2])
  P2L[L5] = P2A[4] + P2K[5] * (wavelength[L5] - P2_limits[3])

  ;Errors are of the order of 0.2%;TS 2016-10-21 Changed the error of the factors to 0.033% since the polarization is now known much better. -- Is it??
  eF1L  = F1L/3000.
  eF2L  = F2L/3000.
  ePhiL = PhiL/500.
  eP1L  = P1L/500.
  eP2L  = P2L/500.

  ;plot = plot(wavelength, F1L, name='F1', xtitle='Wavelength', ytitle='Eff', xrange=[0,30])
  ;plot = plot(wavelength, F2L, name='F2', /overplot, color='red')
  ;leg = legend()
  ;plot = plot(wavelength, PhiL, name='Phi', xtitle='Wavelength', ytitle='Eff', xrange=[0,30])
  ;plot = plot(wavelength, P1L,  name='P1', /overplot, color='red')
  ;plot = plot(wavelength, P2L,  name='P2', /overplot, color='blue')
  ;leg = legend()

  InMat = 1.0 - P2L - P1L + 2.0*P1L*P2L


  ;  P1A  = [0., 0., 0., 0., 0., 0., 0.]
  ;  P2A  = [0., 0., 0., 0., 0., 0., 0.]
  ;  PhiA = [2.65102, -1.34576, 0.44757, -0.07965, 8.20919E-3, -4.90504E-4, 1.57318E-5, -2.09134E-7]
  ;
  ;  ; arrays to store wavelength dependence of efficiencies
  ;  P1L   = fltarr(n_elements(wavelength))
  ;  P2L   = fltarr(n_elements(wavelength))
  ;  PhiL  = fltarr(n_elements(wavelength))
  ;  eP1L  = fltarr(n_elements(wavelength)) ;errors???
  ;  eP2L  = fltarr(n_elements(wavelength))
  ;  ePhiL = fltarr(n_elements(wavelength))
  ;
  ;  P1L  = poly(wavelength, P1A)  & eP1L  = P1L/1000.
  ;  P2L  = poly(wavelength, P2A)  & eP2L  = P2L/1000.
  ;
  ;  ;*This is currently not PHI but P*A calculated over PA=1-2*PHI =>PHIL[j]=0.5*(1-PA[j])
  ;  ;How phi has to be calculated:: PHI = P1+P2+-2P1P2 => p1=1/2+-1/2sqrt(1-2PHI)
  ;  PhiL = poly(wavelength, PhiA) & ePhiL = PhiL/1000.
  ;
  ;  phigiven = total(P1A)
  ;  if phigiven eq 0 then begin
  ;    cosmos_logmessage, 'Cosmos_TofPolCorr_factors: Only phi was supplied!'
  ;    PhiL = 0.5*(1-PhiL) & ePhiL = 0.5*ePhiL ;phil between 0 and 1, normally close to 0
  ;    tmp = (1.-2.*PhiL) > 1.0e-6
  ;    P1L  = 0.5-0.5*sqrt(tmp) & eP1L = 0.5*ePhiL/(sqrt(tmp))
  ;    P2L  = p1L & eP2L = eP1L
  ;  endif

end
;+
; :Description:
;    Corrects analyzer efficiency in direct beam measurement.
;
; :Params:
;   array = 1D vector containing the integrated DB as a function of wavelength
;   array_err = 1D vector with the errors associated to array
;   params = parameters of the DB measuremens
;   lambda = 1D vector containing the wavelengths
;   lambda_range = 1D vector containing the indexes of the useful wavelengths
;
; :Author: M. A. Gonzalez from T. Saerbeck's code
;
; :History:
;    M. A. Gonzalez, 29 April 2015
;
; :Categories: cosmos
;
;-
pro cosmos_tofpolcorr_db, array, array_err, wavelength

  cosmos_logmessage, 'Direct beam polarization correction done.', /warning
  cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L,  InMat

  ;* Calculate the corrected counts & error (do first errors because original array needed)

  array_err /= (1.0 - P1L - P2L + 2.*P1L*P2L)^2
  array_err += (eP1L*array*(2.*P1L-1.)/(1.-P2L-P1L+2.*P2L*P1L)^2)^2
  array_err += (eP2L*array*(2.*P2L-1.)/(1.-P2L-P1L+2.*P2L*P1L)^2)^2
  array_err = sqrt(array_err)

  array /= InMat ; InMAT ## VInt

end

function cosmos_tofpolcorr_noana, array

  cosmos_logmessage, 'Performing polarization correction without analyzer...', /warning

  s = size(array)
  if s[0] ne 3 and s[0] ne 4 then begin
    cosmos_logmessage, 'Unexpected number of dimensions in array sent to cosmos_tofpolcorr_noana.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check that wavelengths of runs with different polarizations are compatible
  wavel_tol = 0.2 ;TS increased the tolerance from 0.03 to 0.2
  if s[0] eq 3 then wavelength = reform(array[4,*,0]) else wavelength = reform(array[4,*,0,0])
  if s[0] eq 3 then wavel_I11 = reform(array[4,*,3]) else wavel_I11 = reform(array[4,*,0,3])
  wavel_diff = abs(wavelength-wavel_I11) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    ;cosmos_logmessage, 'wavel_diff = '+string(wavel_diff)+' ', /warning
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_noana.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L, InMat

  if s[0] eq 3 then nslices = 1 else nslices = s[3]

  for islice = 0, nslices-1 do begin

    if s[0] eq 3 then begin
      mI00  = reform(array[2,*,0]) & meI00 = reform(array[3,*,0])
      mI11  = reform(array[2,*,3]) & meI11 = reform(array[3,*,3])
    endif else begin
      mI00  = reform(array[2,*,islice,0]) & meI00 = reform(array[3,*,islice,0])
      mI11  = reform(array[2,*,islice,3]) & meI11 = reform(array[3,*,islice,3])
    endelse

    ;*Prepare an output array containing the corrected counts:
    wout = [[mI00], [mI11]]
    eout = [[meI00],[meI11]]

    ;loop over each wavelength value (need to think how to do this directly with arrays)
    for j = 0, n_elements(wavelength)-1 do begin

      ;;************************************************
      ;;* Calculate the matrices
      ;*The input vector:
      VInt  = [[mI00[j]],[mI11[j]]]
      VeInt = [[meI00[j]],[meI11[j]]]

      ;**Inverse of the efficiency matrizes::
      InF1M = 1/(F1L[j])*[[F1L[j], 0], [F1L[j]-1, 1]];
      InTPM = 1/(2*p1L[j]-1)*[[p1L[j]-1,p1L[j]], [p1L[j], p1L[j]-1]];

      ;**The error matrizes below are the input errors times the element-wise algebraic derivative of the matrix
      eInF1M = eF1L[j]*(1/(F1L[j])^2*[[0, 0], [1, -1]]);
      eInTPM = eP1L[j]*(1/(2*p1L[j]-1)^2*[[1, -1], [-1, 1]]);

      ;InF1M=INVERT(F1M)
      ;Mat=F1M ## F2M ## TPM ## RAM;
      InMat = InTPM##InF1M;INVERT(MAT);

      ;* Calculate the corrected counts
      wout[j,*] = InMAT ## VInt
      eout[j,*] = (eInTPM ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InTPM ## eInF1M ## VInt)^2
      eout[j,*] = eout[j,*]+InMat^2 ## VeInt^2
      eout[j,*] = sqrt(eout[j,*])

    endfor ;end loop over wavelength values

    if s[0] eq 3 then begin
      array[2,*,0] = wout[*,0] & array[3,*,0] = eout[*,0] ;mI00
      array[2,*,3] = wout[*,1] & array[3,*,3] = eout[*,1] ;mI11
    endif else begin
      array[2,*,islice,0] = wout[*,0] & array[3,*,islice,0] = eout[*,0] ;mI00
      array[2,*,islice,3] = wout[*,1] & array[3,*,islice,3] = eout[*,1] ;mI11
    endelse

  endfor ;end loop over kinetic slices

  return, array

end

function cosmos_tofpolcorr_two, array

  cosmos_logmessage, 'Performing polarization correction of two states with Analyzer.', /warning

  s = size(array)
  if s[0] ne 3 and s[0] ne 4 then begin
    cosmos_logmessage, 'Unexpected number of dimensions in array sent to cosmos_tofpolcorr_two.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check that wavelengths of runs with different polarizations are compatible
  wavel_tol = 0.2
  if s[0] eq 3 then wavelength = reform(array[4,*,0]) else wavelength = reform(array[4,*,0,0])
  if s[0] eq 3 then wavel_I11 = reform(array[4,*,3]) else wavel_I11 = reform(array[4,*,0,3])
  wavel_diff = abs(wavelength-wavel_I11) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_two.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L, InMat

  if s[0] eq 3 then nslices = 1 else nslices = s[3]

  for islice = 0, nslices-1 do begin

    if s[0] eq 3 then begin
      mI00  = reform(array[2,*,0]) & meI00 = reform(array[3,*,0])
      mI01  = reform(array[2,*,0]) & meI01 = reform(array[3,*,0]) ;I01 set equal to I00?
      mI10  = reform(array[2,*,0]) & meI10 = reform(array[3,*,0]) ;I10 set equal to I00?
      mI11  = reform(array[2,*,3]) & meI11 = reform(array[3,*,3])
    endif else begin
      mI00  = reform(array[2,*,islice,0]) & meI00 = reform(array[3,*,islice,0])
      mI01  = reform(array[2,*,islice,0]) & meI01 = reform(array[3,*,islice,0]) ;I01 set equal to I00?
      mI10  = reform(array[2,*,islice,0]) & meI10 = reform(array[3,*,islice,0]) ;I10 set equal to I00?
      mI11  = reform(array[2,*,islice,3]) & meI11 = reform(array[3,*,islice,3])
    endelse

    ;*Prepare an output array containing the corrected counts:
    wout = [[mI00], [mI01], [mI10], [mI11]]
    eout = [[meI00],[meI01],[meI10],[meI11]]

    ;loop over each wavelength value (need to think how to do this directly with arrays)
    for j = 0, n_elements(wavelength)-1 do begin

      TRANS10 = -(-mI11[j]*p2L[j]^2*f1L[j]+2*f1L[j]*p1L[j]*mI11[j]*p2L[j]^2-2*mI00[j]*f1L[j]*f2L[j]*p2L[j]-8*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]+2*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]+8*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]+2*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]-8*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]*p1L[j]-2*f1L[j]*p1L[j]*mI11[j]*p2L[j]-2*mI00[j]*f2L[j]*p1L[j]*p2L[j]+8*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]+mI11[j]*p2L[j]*f1L[j]+mI00[j]*f1L[j]*f2L[j]-mI00[j]*f2L[j]*p1L[j]^2+4*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2+4*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]-4*mI00[j]*f1L[j]*f2L[j]*p1L[j]+mI00[j]*f2L[j]*p1L[j]-4*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2-mI00[j]*f1L[j]^2*f2L[j])/(-f1L[j]*f2L[j]+2*f2L[j]*p1L[j]*p2L[j]+3*f1L[j]*f2L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*p2L[j]^2*f1L[j]*p1L[j]+2*p2L[j]*f1L[j]*p1L[j]+3*f1L[j]*f2L[j]*p2L[j]-p2L[j]*f1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])
      dI00 = -f2L[j]*(-2*p2L[j]*f1L[j]+2*p1L[j]^2*p2L[j]+8*p2L[j]*f1L[j]*p1L[j]-2*p1L[j]*p2L[j]+2*p2L[j]*f1L[j]^2-8*p2L[j]*f1L[j]^2*p1L[j]-8*p2L[j]*f1L[j]*p1L[j]^2+8*p2L[j]*f1L[j]^2*p1L[j]^2-4*f1L[j]*p1L[j]-f1L[j]^2+4*f1L[j]^2*p1L[j]+p1L[j]+f1L[j]-p1L[j]^2+4*f1L[j]*p1L[j]^2-4*f1L[j]^2*p1L[j]^2)/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])
      dI11 = -p2L[j]*f1L[j]*(1-2*p1L[j]-p2L[j]+2*p1L[j]*p2L[j])/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])
      dIf1 = -f2L[j]*(-p1L[j]*mI11[j]*p2L[j]+4*mI00[j]*f2L[j]*p1L[j]*p2L[j]^2-mI00[j]*f2L[j]*p1L[j]^2*p2L[j]-10*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2-8*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]^2+2*mI00[j]*f1L[j]*f2L[j]*p1L[j]-mI00[j]*f2L[j]*p1L[j]*p2L[j]-32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^4*p2L[j]+32*mI00[j]*f2L[j]*p1L[j]^4*p2L[j]*f1L[j]-32*mI00[j]*f2L[j]*p1L[j]^4*p2L[j]^2*f1L[j]+32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^4*p2L[j]^2+32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3*p2L[j]^3+2*mI00[j]*f2L[j]*p1L[j]^4+4*mI00[j]*p1L[j]^3*p2L[j]^3-4*p1L[j]^3*mI11[j]*p2L[j]^3-8*mI00[j]*f2L[j]*p1L[j]^3*p2L[j]^3-16*mI00[j]*p2L[j]^3*f1L[j]^2*p1L[j]^3+8*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^4-8*mI00[j]*f2L[j]*p1L[j]^4*p2L[j]+8*mI00[j]*f2L[j]*p1L[j]^4*p2L[j]^2-8*mI00[j]*f2L[j]*p1L[j]^4*f1L[j]+10*mI00[j]*f2L[j]*p1L[j]^3*p2L[j]-4*mI00[j]*f2L[j]*p1L[j]^3*p2L[j]^2+16*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3-4*mI00[j]*f2L[j]*p1L[j]*p2L[j]^3+12*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]^3+18*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]-20*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3-36*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^2+24*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^3-6*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]-5*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]+8*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2-8*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^3+12*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^2+18*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2-7*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]-12*mI00[j]*p2L[j]^3*f1L[j]^2*p1L[j]+24*mI00[j]*p2L[j]^3*f1L[j]^2*p1L[j]^2-4*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^3-3*mI00[j]*p1L[j]*p2L[j]^2+mI00[j]*f2L[j]*p1L[j]^2-3*mI00[j]*p1L[j]^2*p2L[j]+3*p1L[j]^2*mI11[j]*p2L[j]-9*p1L[j]^2*mI11[j]*p2L[j]^2+9*mI00[j]*p1L[j]^2*p2L[j]^2+mI00[j]*p1L[j]*p2L[j]+3*p1L[j]*mI11[j]*p2L[j]^2-8*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]+8*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+40*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]-40*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]^2-64*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3*p2L[j]+64*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3*p2L[j]^2+34*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]*p1L[j]-52*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2*p1L[j]-84*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]+120*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]^2+88*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3*p2L[j]-112*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3*p2L[j]^2+24*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^3*p1L[j]-48*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]^3+2*mI00[j]*p1L[j]^3*p2L[j]-6*mI00[j]*p1L[j]^3*p2L[j]^2-3*mI00[j]*f2L[j]*p1L[j]^3+2*mI00[j]*p1L[j]*p2L[j]^3-6*mI00[j]*p1L[j]^2*p2L[j]^3+mI00[j]*p2L[j]*f1L[j]^2-3*mI00[j]*p2L[j]^2*f1L[j]^2+mI00[j]*f1L[j]^2*f2L[j]+2*mI00[j]*p2L[j]^3*f1L[j]^2-2*p1L[j]^3*mI11[j]*p2L[j]+6*p1L[j]^3*mI11[j]*p2L[j]^2+6*p1L[j]^2*mI11[j]*p2L[j]^3-2*p1L[j]*mI11[j]*p2L[j]^3)/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])^2
      dIf2 = p2L[j]*f1L[j]*(3*p1L[j]*mI11[j]*p2L[j]-12*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]-36*mI00[j]*p2L[j]*f1L[j]*p1L[j]^2+24*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]^2+18*mI00[j]*p2L[j]*f1L[j]*p1L[j]+12*mI00[j]*f1L[j]*p1L[j]^2+24*mI00[j]*p2L[j]*f1L[j]*p1L[j]^3-16*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]^3+12*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]-24*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^2+16*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^3-18*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]-24*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^3+36*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^2-19*f1L[j]*p1L[j]*mI11[j]*p2L[j]+28*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]-12*f1L[j]*p1L[j]^3*mI11[j]*p2L[j]+22*f1L[j]*p1L[j]*mI11[j]*p2L[j]^2-28*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]^2+8*f1L[j]*p1L[j]^3*mI11[j]*p2L[j]^2-8*f1L[j]*p1L[j]*mI11[j]*p2L[j]^3+8*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]^3-mI00[j]*f1L[j]^2+2*mI00[j]*p1L[j]^3-2*p1L[j]^3*mI11[j]-mI11[j]*f1L[j]+2*mI00[j]*p1L[j]*p2L[j]^2+9*mI00[j]*p1L[j]^2*p2L[j]-9*p1L[j]^2*mI11[j]*p2L[j]+6*p1L[j]^2*mI11[j]*p2L[j]^2-6*mI00[j]*p1L[j]^2*p2L[j]^2-3*mI00[j]*p1L[j]*p2L[j]-2*p1L[j]*mI11[j]*p2L[j]^2-6*mI00[j]*f1L[j]*p1L[j]+2*mI00[j]*p2L[j]^2*f1L[j]-3*mI00[j]*p2L[j]*f1L[j]-p1L[j]*mI11[j]+mI00[j]*p1L[j]-3*mI00[j]*p1L[j]^2+mI00[j]*f1L[j]+3*p1L[j]^2*mI11[j]-6*mI00[j]*p1L[j]^3*p2L[j]+4*mI00[j]*p1L[j]^3*p2L[j]^2+3*mI00[j]*p2L[j]*f1L[j]^2-2*mI00[j]*p2L[j]^2*f1L[j]^2+5*f1L[j]*p1L[j]*mI11[j]+6*mI00[j]*f1L[j]^2*p1L[j]-8*f1L[j]*p1L[j]^2*mI11[j]-12*f1L[j]^2*p1L[j]^2*mI00[j]-8*mI00[j]*f1L[j]*p1L[j]^3+6*p1L[j]^3*mI11[j]*p2L[j]+4*f1L[j]*p1L[j]^3*mI11[j]+8*f1L[j]^2*p1L[j]^3*mI00[j]-4*p1L[j]^3*mI11[j]*p2L[j]^2-5*mI11[j]*p2L[j]^2*f1L[j]+2*mI11[j]*p2L[j]^3*f1L[j]+4*mI11[j]*p2L[j]*f1L[j])/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])^2
      dIp1 = -f1L[j]*f2L[j]*(-2*p1L[j]*mI11[j]*p2L[j]-2*mI00[j]*f1L[j]*f2L[j]*p2L[j]+8*mI00[j]*f2L[j]*p1L[j]*p2L[j]^2+24*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]+8*mI00[j]*f1L[j]*f2L[j]*p2L[j]^2+8*mI00[j]*p2L[j]*f1L[j]*p1L[j]^2+6*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]+4*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2-24*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]^2-12*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]^2-8*mI00[j]*p2L[j]*f1L[j]*p1L[j]-2*mI00[j]*f1L[j]*f2L[j]*p1L[j]-2*mI00[j]*f2L[j]*p1L[j]*p2L[j]+mI00[j]*f2L[j]*p2L[j]-4*mI00[j]*f2L[j]*p2L[j]^2-8*mI00[j]*f2L[j]*p1L[j]*p2L[j]^3-16*mI00[j]*p2L[j]^3*f1L[j]*p1L[j]-8*mI00[j]*f1L[j]*f2L[j]*p2L[j]^3+16*mI00[j]*p2L[j]^3*f1L[j]*p1L[j]^2+8*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]^3-24*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]+24*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^2+8*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]+6*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]-12*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2-8*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^2-4*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2+4*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]+16*mI00[j]*p2L[j]^3*f1L[j]^2*p1L[j]-16*mI00[j]*p2L[j]^3*f1L[j]^2*p1L[j]^2+8*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^3+4*f1L[j]*p1L[j]*mI11[j]*p2L[j]-4*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]-12*f1L[j]*p1L[j]*mI11[j]*p2L[j]^2+12*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]^2+8*f1L[j]*p1L[j]*mI11[j]*p2L[j]^3-8*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]^3+2*mI11[j]*p2L[j]^3-2*mI00[j]*p2L[j]^3+4*mI00[j]*f2L[j]*p2L[j]^3-6*mI00[j]*p1L[j]*p2L[j]^2-mI00[j]*f2L[j]*p1L[j]^2-2*mI00[j]*p1L[j]^2*p2L[j]+2*p1L[j]^2*mI11[j]*p2L[j]-6*p1L[j]^2*mI11[j]*p2L[j]^2+6*mI00[j]*p1L[j]^2*p2L[j]^2+2*mI00[j]*p1L[j]*p2L[j]-mI00[j]*p2L[j]+6*p1L[j]*mI11[j]*p2L[j]^2-6*mI00[j]*p2L[j]^2*f1L[j]+2*mI00[j]*p2L[j]*f1L[j]+3*mI00[j]*p2L[j]^2+16*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]-40*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2-24*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]+48*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]^2+mI11[j]*p2L[j]-3*mI11[j]*p2L[j]^2+32*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]^3-32*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]^3-24*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]*p1L[j]+48*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2*p1L[j]+24*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]-48*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]^2-32*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^3*p1L[j]+32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]^3+4*mI00[j]*p1L[j]*p2L[j]^3+4*mI00[j]*p2L[j]^3*f1L[j]-4*mI00[j]*p1L[j]^2*p2L[j]^3-2*mI00[j]*p2L[j]*f1L[j]^2+6*mI00[j]*p2L[j]^2*f1L[j]^2-mI00[j]*f1L[j]^2*f2L[j]-4*mI00[j]*p2L[j]^3*f1L[j]^2+4*p1L[j]^2*mI11[j]*p2L[j]^3-4*p1L[j]*mI11[j]*p2L[j]^3+3*mI11[j]*p2L[j]^2*f1L[j]-2*mI11[j]*p2L[j]^3*f1L[j]-mI11[j]*p2L[j]*f1L[j])/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])^2
      dIp2 = f1L[j]*f2L[j]*(-2*p1L[j]*mI11[j]*p2L[j]-4*mI00[j]*f1L[j]*f2L[j]*p2L[j]+4*mI00[j]*f2L[j]*p1L[j]*p2L[j]^2+12*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]+4*mI00[j]*f1L[j]*f2L[j]*p2L[j]^2+24*mI00[j]*p2L[j]*f1L[j]*p1L[j]^2+12*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]+12*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2-24*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]^2-12*mI00[j]*f2L[j]*p1L[j]^2*p2L[j]^2-12*mI00[j]*p2L[j]*f1L[j]*p1L[j]-6*mI00[j]*f1L[j]*f2L[j]*p1L[j]-4*mI00[j]*f2L[j]*p1L[j]*p2L[j]-12*mI00[j]*f1L[j]*p1L[j]^2-16*mI00[j]*p2L[j]*f1L[j]*p1L[j]^3+16*mI00[j]*p2L[j]^2*f1L[j]*p1L[j]^3-8*mI00[j]*f2L[j]*p1L[j]^3*p2L[j]+8*mI00[j]*f2L[j]*p1L[j]^3*p2L[j]^2-8*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3-12*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]+8*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3+24*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^2-16*mI00[j]*p2L[j]^2*f1L[j]^2*p1L[j]^3+12*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]+4*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]-4*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2+16*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^3-24*mI00[j]*p2L[j]*f1L[j]^2*p1L[j]^2-12*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2+6*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]+10*f1L[j]*p1L[j]*mI11[j]*p2L[j]-16*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]+8*f1L[j]*p1L[j]^3*mI11[j]*p2L[j]-6*f1L[j]*p1L[j]*mI11[j]*p2L[j]^2+12*f1L[j]*p1L[j]^2*mI11[j]*p2L[j]^2-8*f1L[j]*p1L[j]^3*mI11[j]*p2L[j]^2+mI00[j]*f1L[j]^2-2*mI00[j]*p1L[j]^3+2*p1L[j]^3*mI11[j]+mI11[j]*f1L[j]-2*mI00[j]*p1L[j]*p2L[j]^2+mI00[j]*f2L[j]*p1L[j]-3*mI00[j]*f2L[j]*p1L[j]^2-6*mI00[j]*p1L[j]^2*p2L[j]+6*p1L[j]^2*mI11[j]*p2L[j]-6*p1L[j]^2*mI11[j]*p2L[j]^2+6*mI00[j]*p1L[j]^2*p2L[j]^2+2*mI00[j]*p1L[j]*p2L[j]+mI00[j]*f1L[j]*f2L[j]+2*p1L[j]*mI11[j]*p2L[j]^2+6*mI00[j]*f1L[j]*p1L[j]-2*mI00[j]*p2L[j]^2*f1L[j]+2*mI00[j]*p2L[j]*f1L[j]+24*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]-24*mI00[j]*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2-48*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]+48*mI00[j]*f1L[j]*f2L[j]*p1L[j]^2*p2L[j]^2+p1L[j]*mI11[j]-mI00[j]*p1L[j]+3*mI00[j]*p1L[j]^2-mI00[j]*f1L[j]-3*p1L[j]^2*mI11[j]+32*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3*p2L[j]-32*mI00[j]*f1L[j]*f2L[j]*p1L[j]^3*p2L[j]^2-24*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]*p1L[j]+24*mI00[j]*f1L[j]^2*f2L[j]*p2L[j]^2*p1L[j]+48*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]-48*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^2*p2L[j]^2-32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3*p2L[j]+32*mI00[j]*f1L[j]^2*f2L[j]*p1L[j]^3*p2L[j]^2+4*mI00[j]*p1L[j]^3*p2L[j]-4*mI00[j]*p1L[j]^3*p2L[j]^2+2*mI00[j]*f2L[j]*p1L[j]^3-2*mI00[j]*p2L[j]*f1L[j]^2+2*mI00[j]*p2L[j]^2*f1L[j]^2-mI00[j]*f1L[j]^2*f2L[j]-5*f1L[j]*p1L[j]*mI11[j]-6*mI00[j]*f1L[j]^2*p1L[j]+8*f1L[j]*p1L[j]^2*mI11[j]+12*f1L[j]^2*p1L[j]^2*mI00[j]+8*mI00[j]*f1L[j]*p1L[j]^3-4*p1L[j]^3*mI11[j]*p2L[j]-4*f1L[j]*p1L[j]^3*mI11[j]-8*f1L[j]^2*p1L[j]^3*mI00[j]+4*p1L[j]^3*mI11[j]*p2L[j]^2+mI11[j]*p2L[j]^2*f1L[j]-2*mI11[j]*p2L[j]*f1L[j])/(-p2L[j]*f1L[j]+3*f1L[j]*f2L[j]*p2L[j]-2*p2L[j]^2*f1L[j]*p1L[j]-2*f1L[j]*f2L[j]*p2L[j]^2-2*f2L[j]*p1L[j]^2*p2L[j]-2*f1L[j]*f2L[j]*p1L[j]^2+2*p2L[j]*f1L[j]*p1L[j]+p2L[j]^2*f1L[j]+f2L[j]*p1L[j]^2+3*f1L[j]*f2L[j]*p1L[j]+2*f2L[j]*p1L[j]*p2L[j]-f1L[j]*f2L[j]-f2L[j]*p1L[j]-8*f1L[j]*f2L[j]*p1L[j]*p2L[j]+4*f1L[j]*f2L[j]*p1L[j]*p2L[j]^2+4*f1L[j]*f2L[j]*p1L[j]^2*p2L[j])^2

      mI10[j] = TRANS10
      meI10[j] = sqrt((dI00*meI00[j])^2+(dI11*meI11[j])^2+(dIf1*eF1L[j])^2+(dIf2*eF2L[j])^2+(dIp1*ep1L[j])^2+(dIp2*eP2L[j])^2);


      TRANS01 = -(mI00[j]*P2L[j]*f1L[j]-2*mI00[j]*f2L[j]*P1L[j]*P2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]*P1L[j]+2*P1L[j]*mI10[j]*f2L[j]*P2L[j]+mI00[j]*P1L[j]*P2L[j]-P1L[j]*mI10[j]*P2L[j]+4*mI00[j]*f1L[j]*f2L[j]*P1L[j]*P2L[j]+P1L[j]*mI11[j]*P2L[j]-mI00[j]*f1L[j]+2*mI00[j]*f1L[j]*P1L[j]-P1L[j]*mI11[j]-P1L[j]*mI10[j]*f2L[j]+mI00[j]*f2L[j]*P1L[j]+mI00[j]*f1L[j]*f2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P1L[j]+P1L[j]*mI10[j]-mI00[j]*P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))
      dI00 = -(-P1L[j]+P1L[j]*P2L[j]+f2L[j]*P1L[j]-2*f2L[j]*P1L[j]*P2L[j]+2*f1L[j]*P1L[j]-2*P2L[j]*f1L[j]*P1L[j]-2*f1L[j]*f2L[j]*P1L[j]+4*f1L[j]*f2L[j]*P1L[j]*P2L[j]+f1L[j]*f2L[j]-f1L[j]+P2L[j]*f1L[j]-2*f1L[j]*f2L[j]*P2L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))
      dI11 = -(P1L[j]*P2L[j]-P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))
      dI10 = -(P1L[j]-P1L[j]*P2L[j]-f2L[j]*P1L[j]+2*f2L[j]*P1L[j]*P2L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))
      dIf1 = -(mI00[j]*P2L[j]-2*mI00[j]*f2L[j]*P2L[j]-2*mI00[j]*P1L[j]*P2L[j]+4*mI00[j]*f2L[j]*P1L[j]*P2L[j]-mI00[j]+2*mI00[j]*P1L[j]+mI00[j]*f2L[j]-2*mI00[j]*f2L[j]*P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))+(mI00[j]*P2L[j]*f1L[j]-2*mI00[j]*f2L[j]*P1L[j]*P2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]*P1L[j]+2*P1L[j]*mI10[j]*f2L[j]*P2L[j]+mI00[j]*P1L[j]*P2L[j]-P1L[j]*mI10[j]*P2L[j]+4*mI00[j]*f1L[j]*f2L[j]*P1L[j]*P2L[j]+P1L[j]*mI11[j]*P2L[j]-mI00[j]*f1L[j]+2*mI00[j]*f1L[j]*P1L[j]-P1L[j]*mI11[j]-P1L[j]*mI10[j]*f2L[j]+mI00[j]*f2L[j]*P1L[j]+mI00[j]*f1L[j]*f2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P1L[j]+P1L[j]*mI10[j]-mI00[j]*P1L[j])*(-1+2*P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])^2*(-1+P2L[j]))
      dIf2 = -(-2*mI00[j]*P1L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]+2*P1L[j]*mI10[j]*P2L[j]+4*mI00[j]*P2L[j]*f1L[j]*P1L[j]-P1L[j]*mI10[j]+mI00[j]*P1L[j]+mI00[j]*f1L[j]-2*mI00[j]*f1L[j]*P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))
      dIP1 = -(-2*mI00[j]*f2L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]+2*mI10[j]*f2L[j]*P2L[j]+mI00[j]*P2L[j]-mI10[j]*P2L[j]+4*mI00[j]*f1L[j]*f2L[j]*P2L[j]+mI11[j]*P2L[j]+2*mI00[j]*f1L[j]-mI11[j]-mI10[j]*f2L[j]+mI00[j]*f2L[j]-2*mI00[j]*f1L[j]*f2L[j]+mI10[j]-mI00[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))+(mI00[j]*P2L[j]*f1L[j]-2*mI00[j]*f2L[j]*P1L[j]*P2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]*P1L[j]+2*P1L[j]*mI10[j]*f2L[j]*P2L[j]+mI00[j]*P1L[j]*P2L[j]-P1L[j]*mI10[j]*P2L[j]+4*mI00[j]*f1L[j]*f2L[j]*P1L[j]*P2L[j]+P1L[j]*mI11[j]*P2L[j]-mI00[j]*f1L[j]+2*mI00[j]*f1L[j]*P1L[j]-P1L[j]*mI11[j]-P1L[j]*mI10[j]*f2L[j]+mI00[j]*f2L[j]*P1L[j]+mI00[j]*f1L[j]*f2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P1L[j]+P1L[j]*mI10[j]-mI00[j]*P1L[j])*(-1+2*f1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])^2*(-1+P2L[j]))
      dIP2 = -(mI00[j]*f1L[j]-2*mI00[j]*f2L[j]*P1L[j]-2*mI00[j]*f1L[j]*f2L[j]-2*mI00[j]*f1L[j]*P1L[j]+2*P1L[j]*mI10[j]*f2L[j]+mI00[j]*P1L[j]-P1L[j]*mI10[j]+4*mI00[j]*f1L[j]*f2L[j]*P1L[j]+P1L[j]*mI11[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j]))+(mI00[j]*P2L[j]*f1L[j]-2*mI00[j]*f2L[j]*P1L[j]*P2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P2L[j]-2*mI00[j]*P2L[j]*f1L[j]*P1L[j]+2*P1L[j]*mI10[j]*f2L[j]*P2L[j]+mI00[j]*P1L[j]*P2L[j]-P1L[j]*mI10[j]*P2L[j]+4*mI00[j]*f1L[j]*f2L[j]*P1L[j]*P2L[j]+P1L[j]*mI11[j]*P2L[j]-mI00[j]*f1L[j]+2*mI00[j]*f1L[j]*P1L[j]-P1L[j]*mI11[j]-P1L[j]*mI10[j]*f2L[j]+mI00[j]*f2L[j]*P1L[j]+mI00[j]*f1L[j]*f2L[j]-2*mI00[j]*f1L[j]*f2L[j]*P1L[j]+P1L[j]*mI10[j]-mI00[j]*P1L[j])/((-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])*(-1+P2L[j])^2)

      mI01[j] = TRANS01
      meI01[j] = sqrt((dI00*meI00[j])^2+(dI11*meI11[j])^2+(dI10*meI10[j])^2+(dIf1*eF1L[j])^2+(dIf2*eF2L[j])^2+(dIp1*ep1L[j])^2+(dIp2*eP2L[j])^2);

      ;;************************************************
      ;;* Calculate the matrices
      ;*The input vector:
      VInt  = [[mI00[j]],[mI01[j]],[mI10[j]],[mI11[j]]]
      VeInt = [[meI00[j]],[meI01[j]],[meI10[j]],[meI11[j]]]

      ;**Inverse of the efficiency matrizes::
      InF1M = 1/(F1L[j])*[[F1L[j], 0, 0, 0], [0, F1L[j], 0, 0], [F1L[j]-1, 0, 1, 0], [0, F1L[j]-1, 0, 1]];
      InF2M = 1/(F2L[j])*[[F2L[j], 0, 0, 0], [F2L[j]-1, 1, 0, 0], [0, 0, F2L[j], 0], [0, 0, F2L[j]-1, 1]];
      InTPM = 1/(2*p1L[j]-1)*[[p1L[j]-1, 0, p1L[j], 0], [0, p1L[j]-1, 0, p1L[j]], [p1L[j], 0, p1L[j]-1, 0], [0, p1L[j], 0, p1L[j]-1]];
      InRAM = 1/(2*p2L[j]-1)*[[p2L[j]-1, p2L[j], 0, 0], [p2L[j], p2L[j]-1, 0, 0], [0, 0, p2L[j]-1, p2L[j]], [0, 0, p2L[j], p2L[j]-1]];

      ;**The error matrizes below are the input errors times the element-wise algebraic derivative of the matrix
      eInF1M = eF1L[j]*(1/(F1L[j])^2*[[0, 0, 0, 0], [0, 0, 0, 0], [1, 0, -1, 0], [0, 1, 0, -1]]);
      eInF2M = eF2L[j]*(1/(F2L[j])^2*[[0, 0, 0, 0], [1, -1, 0, 0], [0, 0, 0, 0], [0, 0, 1, -1]]);
      eInTPM = eP1L[j]*(1/(2*p1L[j]-1)^2*[[1, 0, -1, 0], [0, 1, 0, -1], [-1, 0, 1, 0], [0, -1, 0, 1]]);
      eInRAM = ep2L[j]*(1/(2*p2L[j]-1)^2*[[1, -1, 0, 0], [-1, 1, 0, 0], [0, 0, 1, -1], [0, 0, -1, 1]]);

      ;InF1M=INVERT(F1M)
      ;Mat=F1M ## F2M ## TPM ## RAM;
      InMat = InRAM##InTPM##InF2M##InF1M ;INVERT(MAT)

      ;* Calculate the corrected counts
      wout[j,*] = InMAT ## VInt
      eout[j,*] = (eInRAM ## InTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## eInTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## eInF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## InF2M ## eInF1M ## VInt)^2
      eout[j,*] = eout[j,*]+InMat^2 ## VeInt^2
      eout[j,*] = sqrt(eout[j,*])

    endfor ;end loop over wavelength values

    if s[0] eq 3 then begin
      array[2,*,0] = wout[*,0] & array[3,*,0] = eout[*,0] ;mI00
      array[2,*,1] = wout[*,1] & array[3,*,1] = eout[*,1] ;mI01
      array[2,*,2] = wout[*,2] & array[3,*,2] = eout[*,2] ;mI10
      array[2,*,3] = wout[*,3] & array[3,*,3] = eout[*,1] ;mI11
    endif else begin
      array[2,*,islice,0] = wout[*,0] & array[3,*,islice,0] = eout[*,0] ;mI00
      array[2,*,islice,1] = wout[*,1] & array[3,*,islice,1] = eout[*,1] ;mI00
      array[2,*,islice,2] = wout[*,2] & array[3,*,islice,2] = eout[*,2] ;mI00
      array[2,*,islice,3] = wout[*,3] & array[3,*,islice,3] = eout[*,3] ;mI11
    endelse

  endfor ;end loop over kinetic slices

  return, array

end

function cosmos_tofpolcorr_three, array

  s = size(array)

  if s[0] ne 3 and s[0] ne 4 then begin
    cosmos_logmessage, 'Unexpected number of dimensions in array sent to cosmos_tofpolcorr_three.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check that wavelengths of runs with different polarizations are compatible
  wavel_tol = 0.2
  if s[0] eq 3 then wavelength = reform(array[4,*,0]) else wavelength = reform(array[4,*,0,0])

  ;check 11 run
  if s[0] eq 3 then wavel_I11 = reform(array[4,*,3]) else wavel_I11 = reform(array[4,*,0,3])
  wavel_diff = abs(wavelength-wavel_I11) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I11).', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;determine if 01 or 10 used and check wavelengths
  if s[0] eq 3 then begin
    wavel_I01 = reform(array[4,*,1])
    wavel_I10 = reform(array[4,*,2])
  endif else begin
    wavel_I01 = reform(array[4,*,0,1])
    wavel_I10 = reform(array[4,*,0,2])
  endelse
  if max(wavel_I01) gt 0 then begin
    I01flag = 1 & I10flag = 0
    wavel_diff = abs(wavelength-wavel_I01) / wavelength
    if max(wavel_diff) gt wavel_tol then begin
      cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I01).', /warning
      cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
      return, array
    endif 
  endif else begin
    I01flag = 0 & I10flag = 1
    wavel_diff = abs(wavelength-wavel_I10) / wavelength
    if max(wavel_diff) gt wavel_tol then begin
      cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I10).', /warning
      cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
      return, array
    endif
  endelse

  cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L, InMat

  if s[0] eq 3 then nslices = 1 else nslices = s[3]

  for islice = 0, nslices-1 do begin

    if s[0] eq 3 then begin
      mI00  = reform(array[2,*,0]) & meI00 = reform(array[3,*,0])
      mI01  = reform(array[2,*,1]) & meI01 = reform(array[3,*,1])
      mI10  = reform(array[2,*,2]) & meI10 = reform(array[3,*,2])
      mI11  = reform(array[2,*,3]) & meI11 = reform(array[3,*,3])
      if I10flag eq 1 then array[*,*,1] = array[*,*,2] else array[*,*,2] = array[*,*,1]
    endif else begin
      mI00  = reform(array[2,*,islice,0]) & meI00 = reform(array[3,*,islice,0])
      mI01  = reform(array[2,*,islice,1]) & meI01 = reform(array[3,*,islice,1])
      mI10  = reform(array[2,*,islice,2]) & meI10 = reform(array[3,*,islice,2])
      mI11  = reform(array[2,*,islice,3]) & meI11 = reform(array[3,*,islice,3])
      if I10flag eq 1 then array[*,*,1] = array[*,*,2] else array[*,*,2] = array[*,*,1]
    endelse

    ;*Prepare an output array containing the corrected counts:
    wout = [[mI00], [mI01], [mI10], [mI11]]
    eout = [[meI00],[meI01],[meI10],[meI11]]

    ;loop over each wavelength value (need to think how to do this directly with arrays)
    for j = 0, n_elements(wavelength)-1 do begin

      ;prepare the missing intensity:
      if I10flag eq 1 then begin
        mI01[j] = -(2*MI00[j]*f2L[j]*P2L[j]-P2L[j]*MI11[j]-2*MI10[j]*f2L[j]*P2L[j]+MI10[j]*P2L[j]-MI00[j]*P2L[j]+P1L[j]*MI11[j]-2*MI00[j]*f1L[j]*P1L[j]+MI00[j]*P1L[j]-P1L[j]*MI10[j]+MI00[j]*f1L[j]+MI10[j]*f2L[j]-MI00[j]*f2L[j])/(P2L[j]-P1L[j]+2*f1L[j]*P1L[j]-f1L[j])
        meI01[j] = 0
      endif else begin
        mI10[j] = (-MI00[j]*P2L[j]+P2L[j]*MI01[j]-P2L[j]*MI11[j]+2*MI00[j]*f2L[j]*P2L[j]-MI01[j]*P1L[j]+P1L[j]*MI11[j]+MI00[j]*P1L[j]-2*MI00[j]*f1L[j]*P1L[j]+2*MI01[j]*f1L[j]*P1L[j]+MI00[j]*f1L[j]-MI00[j]*f2L[j]-MI01[j]*f1L[j])/(-P2L[j]+2*f2L[j]*P2L[j]+P1L[j]-f2L[j])
        meI10[j] = 0
      endelse

      ;;************************************************
      ;;* Calculate the matrices
      ;;*The input vector:
      VInt  = [[mI00[j]],[mI01[j]],[mI10[j]],[mI11[j]]]
      VeInt = [[meI00[j]],[meI01[j]],[meI10[j]],[meI11[j]]]

      ;**Inverse of the efficiency matrizes::
      InF1M = 1/(F1L[j])*[[F1L[j], 0, 0, 0], [0, F1L[j], 0, 0], [F1L[j]-1, 0, 1, 0], [0, F1L[j]-1, 0, 1]];
      InF2M = 1/(F2L[j])*[[F2L[j], 0, 0, 0], [F2L[j]-1, 1, 0, 0], [0, 0, F2L[j], 0], [0, 0, F2L[j]-1, 1]];
      InTPM = 1/(2*p1L[j]-1)*[[p1L[j]-1, 0, p1L[j], 0], [0, p1L[j]-1, 0, p1L[j]], [p1L[j], 0, p1L[j]-1, 0], [0, p1L[j], 0, p1L[j]-1]];
      InRAM = 1/(2*p2L[j]-1)*[[p2L[j]-1, p2L[j], 0, 0], [p2L[j], p2L[j]-1, 0, 0], [0, 0, p2L[j]-1, p2L[j]], [0, 0, p2L[j], p2L[j]-1]];

      ;**The error matrizes below are the input errors times the element-wise algebraic derivative of the matrix
      eInF1M = eF1L[j]*(1/(F1L[j])^2*[[0, 0, 0, 0], [0, 0, 0, 0], [1, 0, -1, 0], [0, 1, 0, -1]]);
      eInF2M = eF2L[j]*(1/(F2L[j])^2*[[0, 0, 0, 0], [1, -1, 0, 0], [0, 0, 0, 0], [0, 0, 1, -1]]);
      eInTPM = eP1L[j]*(1/(2*p1L[j]-1)^2*[[1, 0, -1, 0], [0, 1, 0, -1], [-1, 0, 1, 0], [0, -1, 0, 1]]);
      eInRAM = ep2L[j]*(1/(2*p2L[j]-1)^2*[[1, -1, 0, 0], [-1, 1, 0, 0], [0, 0, 1, -1], [0, 0, -1, 1]]);

      ;InF1M=INVERT(F1M)
      ;Mat=F1M ## F2M ## TPM ## RAM;
      InMat = InRAM##InTPM##InF2M##InF1M;INVERT(MAT);

      ;* Calculate the corrected counts
      wout[j,*] = InMAT ## VInt
      eout[j,*] = (eInRAM ## InTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## eInTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## eInF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## InF2M ## eInF1M ## VInt)^2
      eout[j,*] = eout[j,*]+InMat^2 ## VeInt^2
      eout[j,*] = sqrt(eout[j,*])

    endfor ;end loop over wavelength values

    if s[0] eq 3 then begin
      array[2,*,0] = wout[*,0] & array[3,*,0] = eout[*,0] ;mI00
      array[2,*,1] = wout[*,1] & array[3,*,1] = eout[*,1] ;mI01
      array[2,*,2] = wout[*,2] & array[3,*,2] = eout[*,2] ;mI10
      array[2,*,3] = wout[*,3] & array[3,*,3] = eout[*,3] ;mI11
    endif else begin
      array[2,*,islice,0] = wout[*,0] & array[3,*,islice,0] = eout[*,0] ;mI00
      array[2,*,islice,1] = wout[*,1] & array[3,*,islice,1] = eout[*,1] ;mI01
      array[2,*,islice,2] = wout[*,2] & array[3,*,islice,2] = eout[*,2] ;mI10
      array[2,*,islice,3] = wout[*,3] & array[3,*,islice,3] = eout[*,3] ;mI11
    endelse

  endfor ;end loop over kinetic slices

  return, array

end

function cosmos_tofpolcorr_full, array

  s = size(array)
  if s[0] ne 3 and s[0] ne 4 then begin
    cosmos_logmessage, 'Unexpected number of dimensions in array sent to cosmos_tofpolcorr_full.', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check that wavelengths of runs with different polarizations are compatible
  wavel_tol = 0.2
  if s[0] eq 3 then wavelength = reform(array[4,*,0]) else wavelength = reform(array[4,*,0,0])

  ;check 11 run
  if s[0] eq 3 then wavel_I11 = reform(array[4,*,3]) else wavel_I11 = reform(array[4,*,0,3])
  wavel_diff = abs(wavelength-wavel_I11) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I11).', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check 01 run
  if s[0] eq 3 then wavel_I01 = reform(array[4,*,1]) else wavel_I01 = reform(array[4,*,0,1])
  wavel_diff = abs(wavelength-wavel_I01) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I01).', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  ;check 10 run
  if s[0] eq 3 then wavel_I10 = reform(array[4,*,2]) else wavel_I10 = reform(array[4,*,0,2])
  wavel_diff = abs(wavelength-wavel_I10) / wavelength
  if max(wavel_diff) gt wavel_tol then begin
    cosmos_logmessage, 'Found wavelengths too different in cosmos_tofpolcorr_three (I10).', /warning
    cosmos_logmessage, 'No polarization efficiency corrections done!', /warning
    return, array
  endif

  cosmos_tofpolcorr_factors, wavelength, F1L, F2L, PhiL, P1L, P2L, eF1L, eF2L, ePhiL, eP1L, eP2L, InMat

  if s[0] eq 3 then nslices = 1 else nslices = s[3]

  for islice = 0, nslices-1 do begin

    if s[0] eq 3 then begin
      mI00  = reform(array[2,*,0]) & meI00 = reform(array[3,*,0])
      mI01  = reform(array[2,*,1]) & meI01 = reform(array[3,*,1])
      mI10  = reform(array[2,*,2]) & meI10 = reform(array[3,*,2])
      mI11  = reform(array[2,*,3]) & meI11 = reform(array[3,*,3])
    endif else begin
      mI00  = reform(array[2,*,islice,0]) & meI00 = reform(array[3,*,islice,0])
      mI01  = reform(array[2,*,islice,1]) & meI01 = reform(array[3,*,islice,1])
      mI10  = reform(array[2,*,islice,2]) & meI10 = reform(array[3,*,islice,2])
      mI11  = reform(array[2,*,islice,3]) & meI11 = reform(array[3,*,islice,3])
    endelse

    ;*Prepare an output array containing the corrected counts:
    wout = [[mI00], [mI01], [mI10], [mI11]]
    eout = [[meI00],[meI01],[meI10],[meI11]]

    ;loop over each wavelength value (need to think how to do this directly with arrays)
    for j = 0, n_elements(wavelength)-1 do begin

      ;;************************************************
      ;;* Calculate the matrices
      ;*The input vector:
      VInt  = [[mI00[j]],[mI01[j]],[mI10[j]],[mI11[j]]]
      VeInt = [[meI00[j]],[meI01[j]],[meI10[j]],[meI11[j]]]

      ;**Inverse of the efficiency matrizes::
      InF1M = 1/(F1L[j])*[[F1L[j], 0, 0, 0], [0, F1L[j], 0, 0], [F1L[j]-1, 0, 1, 0], [0, F1L[j]-1, 0, 1]];
      InF2M = 1/(F2L[j])*[[F2L[j], 0, 0, 0], [F2L[j]-1, 1, 0, 0], [0, 0, F2L[j], 0], [0, 0, F2L[j]-1, 1]];
      InTPM = 1/(2*p1L[j]-1)*[[p1L[j]-1, 0, p1L[j], 0], [0, p1L[j]-1, 0, p1L[j]], [p1L[j], 0, p1L[j]-1, 0], [0, p1L[j], 0, p1L[j]-1]];
      InRAM = 1/(2*p2L[j]-1)*[[p2L[j]-1, p2L[j], 0, 0], [p2L[j], p2L[j]-1, 0, 0], [0, 0, p2L[j]-1, p2L[j]], [0, 0, p2L[j], p2L[j]-1]];

      ;**The error matrizes below are the input errors times the element-wise algebraic derivative of the matrix
      eInF1M = eF1L[j]*(1/(F1L[j])^2*[[0, 0, 0, 0], [0, 0, 0, 0], [1, 0, -1, 0], [0, 1, 0, -1]]);
      eInF2M = eF2L[j]*(1/(F2L[j])^2*[[0, 0, 0, 0], [1, -1, 0, 0], [0, 0, 0, 0], [0, 0, 1, -1]]);
      eInTPM = eP1L[j]*(1/(2*p1L[j]-1)^2*[[1, 0, -1, 0], [0, 1, 0, -1], [-1, 0, 1, 0], [0, -1, 0, 1]]);
      eInRAM = ep2L[j]*(1/(2*p2L[j]-1)^2*[[1, -1, 0, 0], [-1, 1, 0, 0], [0, 0, 1, -1], [0, 0, -1, 1]]);

      ;InF1M=INVERT(F1M)
      ;Mat=F1M ## F2M ## TPM ## RAM;
      InMat = InRAM##InTPM##InF2M##InF1M;INVERT(MAT);

      ;* Calculate the corrected counts
      wout[j,*] = InMAT ## VInt
      eout[j,*] = (eInRAM ## InTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## eInTPM ## InF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## eInF2M ## InF1M ## VInt)^2
      eout[j,*] = eout[j,*]+(InRAM ## InTPM ## InF2M ## eInF1M ## VInt)^2
      eout[j,*] = eout[j,*]+InMat^2 ## VeInt^2
      eout[j,*] = sqrt(eout[j,*])

    endfor ;end loop over wavelength values

    if s[0] eq 3 then begin
      array[2,*,0] = wout[*,0] & array[3,*,0] = eout[*,0] ;mI00
      array[2,*,1] = wout[*,1] & array[3,*,1] = eout[*,1] ;mI01
      array[2,*,2] = wout[*,2] & array[3,*,2] = eout[*,2] ;mI10
      array[2,*,3] = wout[*,3] & array[3,*,3] = eout[*,3] ;mI11
    endif else begin
      array[2,*,islice,0] = wout[*,0] & array[3,*,islice,0] = eout[*,0] ;mI00
      array[2,*,islice,1] = wout[*,1] & array[3,*,islice,1] = eout[*,1] ;mI01
      array[2,*,islice,2] = wout[*,2] & array[3,*,islice,2] = eout[*,2] ;mI10
      array[2,*,islice,3] = wout[*,3] & array[3,*,islice,3] = eout[*,3] ;mI11
    endelse

  endfor ;end loop over kinetic slices

  return, array

end

; discards points on the dark side - algorithm copied from pro bundle below
pro cosmos_anal_discard, result
  
    iteration = 1
    totaldiscards = 0
  
    repeat begin
  
        discards = 0
        i = 0
        newpt = 0
    
        while (i le (n_elements(result[0, *]) - 2)) do begin
    
            if (result[1, i] gt result[1, i+1]) then begin
                mini = i+1
                maxi = i        
            endif else begin      
                mini = i
                maxi = i+1        
            endelse
      
            if (((result[0, maxi] - result[1, maxi]) lt (result[0, mini] - result[1, mini])) $
               and ((result[0, maxi] + result[1, maxi]) gt (result[0, mini] + result[1, mini]))) then begin
        
                if (result[3, i] gt result[3, i+1]) then begin
        
                    result[*, newpt] = result[*, i+1]
          
                endif else begin
        
                    result[*, newpt] = result[*, i]
          
                endelse
        
                discards = discards + 1
                i = i + 2
        
            endif else begin
      
                result[*, newpt] = result[*, i]
                i = i + 1
        
            endelse
            
            newpt = newpt + 1
      
        endwhile
    
        result = result[*, 0 : newpt - 1]
        totaldiscards = totaldiscards + discards
        iteration = iteration + 1
    
    endrep until (discards eq 0)
  
    cosmos_logmessage, ' Discarded ' + cosmos_str_make(totaldiscards) + ' points with low statistics.'
  
    return
  
end

;new cosmos_anal_factor routine from Bob Cubitt (copied to fcosmos02 in May 15th 2009)
function cosmos_anal_factor, array_a, array_b
  
    ;new version added 1/12/2008 as noisy data didnt seem to work with the mean method
    ; this rebins one set onto another, divides one by the other and then takes a weighted mean
    ; to find the factor between angles.
    
    result = fltarr(2)
    
    ; sort arrays
    a = cosmos_anal_sort(array_a)
    b = cosmos_anal_sort(array_b)
    
    overa = where(a[0, *] ge min(b[0, *]), count)
    
    if count gt 0 then a = a[*, overa] else return, result
    
    b = b[*, where(b[0, *] le max(a[0, *]))]
    
    ;fractional error in the b array overlap region
    bf_err=b[3,*]/b[2,*]
    
    ;fractional error in the a array overlap region
    af_err=a[3,*]/a[2,*]
    
    ; produce interpolation arrays
    ;if keyword_set(l) then begin
    if 1 eq 1 then begin
    
        inter = interpol(a[2, *], a[0, *], b[0, *])
        intererr = interpol(a[3, *], a[0, *], b[0, *])
      
    endif else begin
    
        deriv = spl_init(a[0, *], a[2, *])
        inter = spl_interp(a[0, *], a[2, *], deriv, b[0, *])
      
    endelse
    
    ;fractional error squared in ratio at each point
    ffac_err=(bf_err^2+(intererr/inter)^2)
    
    fac=b[2,*]/inter
    
    ;true error
    fac_err=sqrt(ffac_err)*fac
    
    n=n_elements(fac)
    
    ffac=fltarr(4,n)
    ffac[2,*]=fac
    ffac[3,*]=fac_err
    result[0]=cosmos_anal_mean(ffac,1)
      
    return, result
  
end

; new grouping procedure - groups integer number of q points into each new bin
; modified 07/05 to group integral numbers of q points subject to constant multiplier of instrument resolution (given by groupnum)
; MG 28.01.2015 --> average over number of grouped points (needed to treat both coherent/incoherent cases)
; MG 6.2.2015 --> routine rewritten because BC and PG arrived to the conclusion that the Q resolution was  not OK
;function cosmos_anal_group, result, groupnum
; 
;    ; display min and max fractional resolutions for dataset
;    fractionalerror = result[1, *] / result[0, *]
;    minfracerror = min(fractionalerror)
;    maxfracerror = max(fractionalerror)
;    
;    if (groupnum le 0.0) then begin
;    
;        groupnum = 1.0
;        cosmos_logmessage, ' Grouping data to calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%) (Auto)'
;        
;    endif else begin
;    
;        cosmos_logmessage, ' Grouping data to ' + cosmos_str_make(groupnum) + ' times calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + $
;        '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%)'
;        
;    endelse
;    
;    i = 0
;    newpt = 0
;    sortedresult = cosmos_anal_sort(result)
;    groupedresult = sortedresult ; preserve unbundled data!
;    
;    j = 0 ; number of points in new bin
;    minj = 0;
;    maxj = 0;
;    
;    for i = 0, n_elements(result[0, *]) - 1 do begin
;    
;        if (j eq 0) then begin
;      
;            ; groupedresult[0, *] are min values
;            groupedresult[0, newpt] = sortedresult[0, i] - sortedresult[1, i]
;            ; groupedresult[1, *] are max values
;            groupedresult[1, newpt] = sortedresult[0, i] + sortedresult[1, i]
;            groupedresult[2:4, newpt] = sortedresult[2:4, i]
;            currentthreshold = sortedresult[0, i] + (sortedresult[1, i] * groupnum)
;            j = j + 1
;        
;        endif else begin
;      
;            if (sortedresult[0, i] ge currentthreshold) then begin
;        
;                if (maxj eq 0) then begin
;                    minj = j
;                    maxj = j
;                endif else begin          
;                    if (j lt minj) then minj = j
;                    if (j gt maxj) then maxj = j            
;                endelse
;                
;                ;average before going to next point
;                if j gt 0 then groupedresult[2:4, newpt] = groupedresult[2:4, newpt] / float(j)
;                
;                newpt = newpt + 1
;                groupedresult[0, newpt] = sortedresult[0, i] - sortedresult[1, i]
;                groupedresult[1, newpt] = sortedresult[0, i] + sortedresult[1, i]
;                groupedresult[2:4, newpt] = sortedresult[2:4, i]
;                currentthreshold = sortedresult[0, i] + (sortedresult[1, i] * groupnum)
;                j = 1
;          
;            endif else begin
;        
;                groupedresult[0, newpt] = groupedresult[0, newpt] < (sortedresult[0, i] - sortedresult[1, i])
;                groupedresult[1, newpt] = groupedresult[1, newpt] > (sortedresult[0, i] + sortedresult[1, i])
;                groupedresult[2, newpt] = groupedresult[2, newpt] + sortedresult[2, i]
;                groupedresult[3, newpt] = sqrt(groupedresult[3, newpt]^2 + sortedresult[3, i]^2)
;                groupedresult[4, newpt] = groupedresult[4, newpt] + sortedresult[4, i]
;                j = j + 1
;          
;            endelse
;        
;        endelse
;      
;    endfor
;    
;    ;average for last point 
;    if j gt 0 then groupedresult[2:4, newpt] = groupedresult[2:4, newpt] / float(j)
;
;    groupedresult = groupedresult[*, 0 : newpt]
;    
;    ; turn min, max back into value, error
;    for i = 0, n_elements(groupedresult[0, *]) - 1 do begin
;    
;        groupedresult[1, i] = (groupedresult[1, i] - groupedresult[0, i]) / 2.
;        groupedresult[0, i] = groupedresult[0, i] + groupedresult[1, i]
;      
;    endfor
;    
;    cosmos_logmessage, '  Min/max numbers of old points in new bins = ' + cosmos_str_make(minj) + ', ' + cosmos_str_make(maxj)
;    
;    return, groupedresult
;  
;end

function cosmos_anal_group, result, groupnum

    ; display min and max fractional resolutions for dataset
    fractionalerror = result[1, *] / result[0, *]
    minfracerror = min(fractionalerror)
    maxfracerror = max(fractionalerror)

    if (groupnum le 0.0) then begin

        groupnum = 1.0
        cosmos_logmessage, ' Grouping data to calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%) (Auto)'

    endif else begin

        cosmos_logmessage, ' Grouping data to ' + cosmos_str_make(groupnum) + ' times calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + $
        '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%)'

    endelse

    sortedresult = cosmos_anal_sort(result)
    groupedresult = sortedresult ; preserve unbundled data!

    i = 0
    newpt = 0
    minj = 0
    maxj = 0
    startpoint = 0
    old_threshold = 0.0
    
    for i = 0, n_elements(result[0, *]) - 1 do begin
      
        if i lt startpoint then continue
        
        new_threshold =  sortedresult[0, i] + (sortedresult[1, i] * groupnum)
        
        ;determine how many points have Q < threshold
        points = where(sortedresult[0,*] gt old_threshold and sortedresult[0,*] lt new_threshold)
        q_values = sortedresult[0,points]
        ;MG: need to think about good way to compute weights
        weights = (sortedresult[2,points] / sortedresult[3,points])^2 ;weight = 1 / dI^2 --> problem with counts = 1
        ;PG: I would use the absolute value, not the square
        ;weights = abs(sortedresult[2,points] / sortedresult[3,points])
        weights[*] = 1 ;PG: actually, maybe it shouldn't be weighted at all
        ;print, sortedresult[0, i]
        ;print, old_threshold, new_threshold
        ;help, q_values, weights
        ;print, q_values
        ;print, weights
        
        ;average Q (weighted by errors)
        groupedresult[0, newpt] = total(weights*q_values) / total(weights)  ;PG=Maybe the final q shouldn't be weighted
        
        ;Q-resolution
        groupedresult[1, newpt] = sqrt( sortedresult[1, i]^2 + 0.68^2 * (max(q_values) - min(q_values))^2 )
        
        ;average reflectivity (weighted by errors)
        groupedresult[2, newpt] = total(weights*sortedresult[2,points]) / total(weights) ;PG: I think the reflectivity does not have to be weighted
        
        ;error in reflectivity
        groupedresult[3, newpt] = sqrt( total(weights^2*sortedresult[3,points]^2) ) / total(weights) ;PG: The same for the error in reflectivity

        ;average wavelength (weighted by errors)
        groupedresult[4, newpt] = total(weights*sortedresult[4,points]) / total(weights) ;PG: The same for the wavelength of the reflectivity
        
        ;check if min/max number of points need to be updated
        if (maxj eq 0) then begin
          minj = n_elements(q_values)
          maxj = n_elements(q_values)
        endif else begin
          if (n_elements(q_values) lt minj) then minj = n_elements(q_values)
          if (n_elements(q_values) gt maxj) then maxj = n_elements(q_values)
        endelse
        
        ;prepare for next point
        startpoint = i + n_elements(q_values)
        if startpoint lt n_elements(result[0, *]) then begin
           newpt = newpt + 1
           old_threshold = new_threshold
        endif else break   
     
    endfor   
    
    groupedresult = groupedresult[*, 0 : newpt]

    cosmos_logmessage, '  Min/max numbers of old points in new bins = ' + cosmos_str_make(minj) + ', ' + cosmos_str_make(maxj)

    return, groupedresult

end

function cosmos_anal_group_pol, result, groupnum

  ;Grouping is done using the Q/dQ data for the first set (flippers = 00) and the first slice (kinetic)
  ;The other 3 sets are grouped in the same way.

  s = size(result)
  if s[0] ne 3 and s[0] ne 4 then begin
    cosmos_logmessage, 'Unexpected number of dimensions in array sent to cosmos_anal_group_pol.', /warning
    cosmos_logmessage, 'No grouping done!', /warning
    return, result
  endif

  ; get min and max fractional resolutions for dataset
  ; uses only the first set (flippers 00), as it is assumed that the Qs are the same for all 4 states
  if s[0] eq 3 then begin
    nslices = 1
    fractionalerror = reform(result[1,*,0]) / reform(result[0,*,0])
  endif else begin
    nslices = s[3]
    fractionalerror = reform(result[1,*,0,0]) / reform(result[0,*,0,0])
  endelse
  minfracerror = min(fractionalerror)
  maxfracerror = max(fractionalerror)

  if (groupnum le 0.0) then begin
    groupnum = 1.0
    cosmos_logmessage, ' Grouping pol data to calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%) (Auto)'
  endif else begin
    cosmos_logmessage, ' Grouping pol data to ' + cosmos_str_make(groupnum) + ' times calculated Q resolution of instrument = (' + cosmos_str_make(minfracerror * 100.0) + $
      '%, ' + cosmos_str_make(maxfracerror * 100.0) + '%)'
  endelse

  if s[0] eq 3 then begin
    sortedresult = result[*,sort(result[0,*,0]),*]
    sortedq = reform(sortedresult[0,*,0])
  endif else begin
    sortedresult = result[*,sort(result[0,*,0,0]),*,*]
    sortedq = reform(sortedresult[0,*,0,0])
  endelse

  groupedresult = sortedresult

  i = 0
  newpt = 0
  minj = 0
  maxj = 0
  startpoint = 0
  old_threshold = 0.0

  for i = 0, n_elements(sortedq)-1 do begin

    if i lt startpoint then continue

    if s[0] eq 3 then begin
      new_threshold =  sortedq[i] + (sortedresult[1, i, 0] * groupnum)
      points = where(sortedq gt old_threshold and sortedq lt new_threshold)
      q_values = sortedq[points]
      ; Average Q
      groupedresult[0, newpt, *] = total(q_values) / n_elements(q_values)
      ;Q-resolution
      groupedresult[1, newpt, *] = sqrt(sortedresult[1,i,0]^2 + 0.68^2 * (max(q_values) - min(q_values))^2 )
      ;average reflectivity
      for ipol = 0,3 do begin
        groupedresult[2, newpt, ipol] = total(sortedresult[2,points,ipol]) / n_elements(q_values)
      endfor
      ;error in reflectivity
      for ipol = 0,3 do begin
        groupedresult[3, newpt, ipol] = sqrt( total(sortedresult[3,points,ipol]^2) ) / n_elements(q_values)
      endfor
      ;average wavelength
      groupedresult[4, newpt, *] = total(sortedresult[4,points,0]) / n_elements(q_values)
    endif else begin
      new_threshold =  sortedresult[0, i, 0, 0] + (sortedresult[1, i, 0, 0] * groupnum)
      points = where(sortedq gt old_threshold and sortedq lt new_threshold)
      q_values = sortedq[points]
      ; Average Q
      groupedresult[0, newpt, *, *] = total(q_values) / n_elements(q_values)
      ;Q-resolution
      groupedresult[1, newpt, *, *] = sqrt(sortedresult[1,i,0,0]^2 + 0.68^2 * (max(q_values) - min(q_values))^2 )
      ;average reflectivity
      for ipol = 0,3 do begin
        for islice = 0,nslices-1 do begin
          groupedresult[2, newpt, islice, ipol] = total(sortedresult[2,points,islice,ipol]) / n_elements(q_values)
        endfor
      endfor
      ;error in reflectivity
      for ipol = 0,3 do begin
        for islice = 0,nslices-1 do begin
          groupedresult[3, newpt, islice, ipol] = sqrt( total(sortedresult[3,points,islice,ipol]^2) ) / n_elements(q_values)
        endfor
      endfor
      ;average wavelength
      groupedresult[4, newpt, *, *] = total(sortedresult[4,points,0,0]) / n_elements(q_values)
    endelse

    ;check if min/max number of points need to be updated
    if (maxj eq 0) then begin
      minj = n_elements(q_values)
      maxj = n_elements(q_values)
    endif else begin
      if (n_elements(q_values) lt minj) then minj = n_elements(q_values)
      if (n_elements(q_values) gt maxj) then maxj = n_elements(q_values)
    endelse

    ;prepare for next point
    startpoint = i + n_elements(q_values)
    if startpoint lt n_elements(sortedq) then begin
      newpt = newpt + 1
      old_threshold = new_threshold
    endif else break

  endfor

  ;Cut arrays to the number of grouped points
  if s[0] eq 3 then groupedresult = groupedresult[*, 0:newpt, *] else groupedresult = groupedresult[*, 0:newpt, *, *]

  cosmos_logmessage, '  Min/max numbers of old points in new bins = ' + cosmos_str_make(minj) + ', ' + cosmos_str_make(maxj)

  return, groupedresult

end

; calculate log of dataset [x, err_x, y, err_y]
function cosmos_anal_log, array

    ;MAG (19/11/2009) Modified to remove only the negative points, but keep those where y < dy
    pts = where(array[2, *] gt 0., temp1)
    
    if temp1 gt 0 then begin
    
        resultlog = fltarr(5, temp1)
        resultlog[0, *] = reform(array[0, pts])
        resultlog[1, *] = reform(array[1, pts])
        resultlog[4, *] = reform(array[4, pts])

        resultlog[2, *] = alog10(reform(array[2, pts]))
      
        ;MAG (19/11/2009): Logarithmic error bars: If plotted log(y)+log(y+dy) and log(y)-log(ydy) one
        ;gets asymmetric error bars. Plot z=log(y) and the error dz = d[log(y)]
        ;And differential analysis leads to dz = (1/2.303)*(dy/y) and plot z+dz and z-dz.
        ;Note that the log errors are plotted on the log(y) scale and not on the y scale.
        resultlog[3, *] = abs( reform(array[3, pts]) / (reform(array[2, pts]) * alog(10.0)))
      
    endif else begin
    
        resultlog = fltarr(5, 1)
        resultlog[*, 0] = -1.0
      
    endelse
    
    return, resultlog
  
end

; works out mean in y weighted by y error (or not) for array[x, xerr, y, yerr] with error
function cosmos_anal_mean, array, weightoption

    sumy = 0.0
    sumyerrsq = 0.0
    
    if (weightoption eq 0) then begin
    
        ; no weighting
        divisor = 0.0
      
        for i = 0, (n_elements(array[0, *]) - 1) do begin
      
            sumy = sumy + array[2, i]
            sumyerrsq = sumyerrsq + (array[3, i]^2)
            divisor = divisor + 1.0
        
        endfor
      
        unweightedmean = sumy / divisor
        unweightedmeanerr = sqrt(sumyerrsq) / divisor
       
        return, [unweightedmean, unweightedmeanerr]
      
    endif else begin
    
        ; weight by y error
        sumweights = 0.0
        sumerrsq = 0.0
        divisor = 0.0
      
        for i = 0, (n_elements(array[0, *]) - 1) do begin
      
            if array[2,i] gt 0 then begin        
                weightfactor = array[2,i]^2/ abs(array[3, i]^2)
                sumy = sumy + (array[2, i] * weightfactor)
                sumerrsq = sumerrsq + (array[3, i]/array[2,i])^2  ;sum of frac err sq
                sumweights = sumweights + weightfactor
                divisor=divisor+1
          
            endif
        
        endfor
      
        weightedmean = sumy / (sumweights)
        weightedmeanerr = weightedmean*sqrt(sumerrsq)
      
        return, [weightedmean, weightedmeanerr]
      
    endelse
  
end

; searches for peak in array, returning thresholds of peak
function cosmos_anal_peaksearchbasic, array

    result = lonarr(2)
    temp2 = max(array, temp1)
    result[0] = temp1 ; start at maximum
    temp2 = (temp2 + min(array)) / 2. ; threshold of peak
  
    while (array[result[0]] ge temp2) and (result[0] gt 0) do result[0] = result[0] - 1
  
    result[1] = temp1
  
    while (array[result[1]] ge temp2) and (result[1] lt n_elements(array) - 1) do result[1] = result[1] + 1
  
    return, result
  
end

; even better peak search routine!! (to be used if wished!)
pro cosmos_anal_peaksearchpro_newer, cnt, xrange, usefulx, maximum, peakrange

    detectorx = total(cnt, 2)
    fit = gaussfit(indgen(xrange[1] - xrange[0] + 1) + xrange[0], detectorx[xrange[0]:xrange[1]], fitparams)
    peak = fitparams[1]
    fwhm = fitparams[2]
    maximum = fix(peak + 0.5)
  
    ; need to insure we enclose the entire reflection / direct beam...
    peakrange = indgen(fix(fwhm * 8 + 0.5)) + fix(peak - (fwhm * 4) + 0.5)
  
    if ((min(peakrange) lt xrange[0]) or (max(peakrange) gt xrange[1])) then cosmos_logmessage, 'Wide peak detected - may be poor statistics!', /WARNING
  
    peakrangeindex = where((peakrange gt xrange[0]) and (peakrange lt xrange[1]), count)
  
    if (count ne 0) then peakrange = peakrange[peakrangeindex] else maximum = -1
  
    return
  
end


; better peak search in count(x, t) array
pro cosmos_anal_peaksearchpro, cnt, xrange, usefulx, maximum, peakrange

    outsidepeakleft = cnt[usefulx[0]:xrange[0],*]
    outsidepeakright = cnt[xrange[1]:usefulx[1], *]
    outsidepeak = [outsidepeakleft, outsidepeakright]
  
    ; mean over x outside x range
    timeaverage = total(outsidepeak, 1) / n_elements(outsidepeak[*, 0])
    normalisedcounts = cnt
  
    for t = 0, n_elements(cnt[0, *]) - 1 do normalisedcounts[*, t] = normalisedcounts[*, t] - timeaverage[t]
    
    ; can't divide by timeaverage[t] since this may be zero!
    ; note for future modifications this could be improved to take into account the usually (~linearly) sloping background in x
    detectorx = total(normalisedcounts, 2)
    background = mean([detectorx[usefulx[0]:xrange[0]], detectorx[xrange[1]:usefulx[1]]])
    foreground = detectorx - background
    thirdmaximum = max(foreground[xrange[0]:xrange[1]], maximum) / 3.
    maximum = maximum + xrange[0]
    abovethirdmax = where(foreground gt thirdmaximum, count)
  
    ; ensure peak is actually in area (if not this is due to poor background subtraction (see above) / poor statistics)
    if (count ne 0) then peakrangeindex = where((abovethirdmax gt xrange[0]) and (abovethirdmax lt xrange[1]), count)
  
    if (count ne 0) then begin
  
        fwthm = max(abovethirdmax) - min(abovethirdmax)
    
        if fwthm eq 0 then fwthm=1
    
        ;MAG: added on 25/5/09 to deal with problem of having only 1 point > thirdmaximum
        ; need to insure we enclose the entire reflection / direct beam...
        peakrange = indgen(fwthm * 6) + maximum - (fwthm * 3)
        peakrangeindex = where((peakrange gt xrange[0]) and (peakrange lt xrange[1]), count)
    
    endif
  
    if (count ne 0) then peakrange = peakrange[peakrangeindex] else maximum = -1
  
    return
  
end

; refined search for peak in array using previous search and foreground range
; (0 is no range) and guess, returning peak centre
function cosmos_anal_peaksearchrefined, array, search, foregd

    temp = max(array[search[0]:search[1]], guess_new)
    guess_new = guess_new + search[0]
    
    repeat begin
    
        guess_old = guess_new
        low = (guess_old - fix(1 + foregd[0])) > search[0]
        ;high = ((guess_old + fix(1 + foregd[1])) < search[1]) > (low + 1)
        high = ((guess_old + fix(1 + foregd[1])) < search[1]) > low 
      
        ; calculated weighted mean
        temp = total(abs(array[low:high]))
        if temp gt 0. then guess_new = round(total(abs(array[low:high]) * (lindgen(high - low + 1) + low)) / temp)
      
    endrep until guess_old eq guess_new
    
    return, guess_new
  
end

; (re)check search and detector ranges - added 04/05 to handle changing detector size
pro cosmos_anal_ranges
  
  common cosmos_params, c_params
      
  common cosmos_settings, c_settings
  
  ; parse detector ranges
  ; MAG: Changed to have the possibility of having different search ranges for each angle.
  if (c_settings.usefulpixelmin1 lt 0L) or (c_settings.usefulpixelmin1 ge (c_params.pixels_x - 1)) or $
     (c_settings.usefulpixelmax1 le 0L) or (c_settings.usefulpixelmax1 gt (c_params.pixels_x - 1)) or $
     (c_settings.usefulpixelmax1 lt c_settings.usefulpixelmin1) then begin

    c_settings.usefulpixelmin1 = 0L
    c_settings.usefulpixelmax1 = (c_params.pixels_x - 1)
    
    cosmos_logmessage, 'Useful range for angle 1 in x pixels off detector.'
    
  endif
  
  cosmos_logmessage, 'Useful pixel range for angle 1 read as ' + cosmos_str_make(c_settings.usefulpixelmin1) + ' to ' + cosmos_str_make(c_settings.usefulpixelmax1) + '.'

  if c_settings.nangles ge 1 then begin

    ; parse detector ranges
    ; MAG: Changed to have the possibility of having different search ranges for each angle.
    if (c_settings.usefulpixelmin2 lt 0L) or (c_settings.usefulpixelmin2 ge (c_params.pixels_x - 1)) or $
       (c_settings.usefulpixelmax2 le 0L) or (c_settings.usefulpixelmax2 gt (c_params.pixels_x - 1)) or $
       (c_settings.usefulpixelmax2 lt c_settings.usefulpixelmin2) then begin

      c_settings.usefulpixelmin2 = 0L
      c_settings.usefulpixelmax2 = (c_params.pixels_x - 1)
    
      cosmos_logmessage, 'Useful range for angle 2 in x pixels off detector.'
    
    endif

    cosmos_logmessage, 'Useful pixel range for angle 2 read as ' + cosmos_str_make(c_settings.usefulpixelmin2) + ' to ' + cosmos_str_make(c_settings.usefulpixelmax2) + '.'

    if c_settings.nangles ge 2 then begin

      ; parse detector ranges
      ; MAG: Changed to have the possibility of having different search ranges for each angle.
      if (c_settings.usefulpixelmin3 lt 0L) or (c_settings.usefulpixelmin3 ge (c_params.pixels_x - 1)) or $
         (c_settings.usefulpixelmax3 le 0L) or (c_settings.usefulpixelmax3 gt (c_params.pixels_x - 1)) or $
         (c_settings.usefulpixelmax3 lt c_settings.usefulpixelmin3) then begin

        c_settings.usefulpixelmin3 = 0L
        c_settings.usefulpixelmax3 = (c_params.pixels_x - 1)
    
        cosmos_logmessage, 'Useful range for angle 3 in x pixels off detector.'
    
      endif
      
      cosmos_logmessage, 'Useful pixel range for angle 3 read as ' + cosmos_str_make(c_settings.usefulpixelmin3) + ' to ' + cosmos_str_make(c_settings.usefulpixelmax3) + '.'

    endif
    
  endif
    
  if c_settings.autosearchpixel1 eq 1 then begin
    c_settings.searchpixelmin1 = 0L
    c_settings.searchpixelmax1 = -1L
  endif
    
  if c_settings.searchpixelmin1 le c_settings.usefulpixelmin1 then c_settings.searchpixelmin1 = c_settings.usefulpixelmin1 + 1
  if (c_settings.searchpixelmax1 ge c_settings.usefulpixelmax1) then c_settings.searchpixelmax1 = c_settings.usefulpixelmax1 - 1
  
  if c_settings.searchpixelmax1 gt 0L then begin
  
    if c_settings.searchpixelmax1 lt c_settings.searchpixelmin1 then begin
      dummy = c_settings.searchpixelmax1
      c_settings.searchpixelmax1 = c_settings.searchpixelmin1 
      c_settings.searchpixelmin1 = dummy    
    endif
  
    cosmos_logmessage, ' Search pixel range for angle 1 read as ' + cosmos_str_make(c_settings.searchpixelmin1) + ' to ' + cosmos_str_make(c_settings.searchpixelmax1) + '.'
    
  endif

  if c_settings.nangles ge 1 then begin

    if c_settings.autosearchpixel2 eq 1 then begin
      c_settings.searchpixelmin2 = 0L
      c_settings.searchpixelmax2 = -1L
    endif
    
    if c_settings.searchpixelmin2 le c_settings.usefulpixelmin2 then c_settings.searchpixelmin2 = c_settings.usefulpixelmin2 + 1
    if (c_settings.searchpixelmax2 ge c_settings.usefulpixelmax2) then c_settings.searchpixelmax2 = c_settings.usefulpixelmax2 - 1
  
    if c_settings.searchpixelmax2 gt 0L then begin
  
      if c_settings.searchpixelmax2 lt c_settings.searchpixelmin2 then begin
        dummy = c_settings.searchpixelmax2
        c_settings.searchpixelmax2 = c_settings.searchpixelmin2 
        c_settings.searchpixelmin2 = dummy    
      endif
  
      cosmos_logmessage, ' Search pixel range for angle 2 read as ' + cosmos_str_make(c_settings.searchpixelmin2) + ' to ' + cosmos_str_make(c_settings.searchpixelmax2) + '.'
    
    endif

    if c_settings.nangles ge 2 then begin

      if c_settings.autosearchpixel3 eq 1 then begin
        c_settings.searchpixelmin3 = 0L
        c_settings.searchpixelmax3 = -1L
      endif
    
      if c_settings.searchpixelmin3 le c_settings.usefulpixelmin3 then c_settings.searchpixelmin3 = c_settings.usefulpixelmin3 + 1
      if (c_settings.searchpixelmax3 ge c_settings.usefulpixelmax3) then c_settings.searchpixelmax3 = c_settings.usefulpixelmax3 - 1
  
      if c_settings.searchpixelmax3 gt 0L then begin
  
        if c_settings.searchpixelmax3 lt c_settings.searchpixelmin3 then begin
          dummy = c_settings.searchpixelmax2
          c_settings.searchpixelmax3 = c_settings.searchpixelmin3 
          c_settings.searchpixelmin3 = dummy    
        endif
  
        cosmos_logmessage, ' Search pixel range for angle 3 read as ' + cosmos_str_make(c_settings.searchpixelmin3) + ' to ' + cosmos_str_make(c_settings.searchpixelmax3) + '.'
    
      endif
    
    endif
    
  endif
    
  c_settings.searchtimemin1 = c_settings.searchtimemin1 > 0L
  c_settings.searchtimemax1 = c_settings.searchtimemax1 < (c_params.pixels_t - 1)
  
  if c_settings.searchtimemax1 gt 0L then begin
  
    if c_settings.searchtimemax1 lt c_settings.searchtimemin1 then begin
      dummy = c_settings.searchtimemax1
      c_settings.searchtimemax1 = c_settings.searchtimemin1
      c_settings.searchtimemin1 = dummy    
    endif 
  
    cosmos_logmessage, ' Search time range for angle 1 read as ' + cosmos_str_make(c_settings.searchtimemin1) + ' to ' + cosmos_str_make(c_settings.searchtimemax1) + '.'
    
  endif

  if c_settings.nangles ge 1 then begin

    c_settings.searchtimemin2 = c_settings.searchtimemin2 > 0L
    c_settings.searchtimemax2 = c_settings.searchtimemax2 < (c_params.pixels_t - 1)
  
    if c_settings.searchtimemax2 gt 0L then begin
  
      if c_settings.searchtimemax2 lt c_settings.searchtimemin2 then begin
        dummy = c_settings.searchtimemax2
        c_settings.searchtimemax2 = c_settings.searchtimemin2
        c_settings.searchtimemin2 = dummy    
      endif 
  
      cosmos_logmessage, ' Search time range for angle 2 read as ' + cosmos_str_make(c_settings.searchtimemin2) + ' to ' + cosmos_str_make(c_settings.searchtimemax2) + '.'
    
    endif

    if c_settings.nangles ge 2 then begin

      c_settings.searchtimemin3 = c_settings.searchtimemin3 > 0L
      c_settings.searchtimemax3 = c_settings.searchtimemax3 < (c_params.pixels_t - 1)
  
      if c_settings.searchtimemax3 gt 0L then begin
  
        if c_settings.searchtimemax3 lt c_settings.searchtimemin3 then begin
          dummy = c_settings.searchtimemax3
          c_settings.searchtimemax3 = c_settings.searchtimemin3
          c_settings.searchtimemin3 = dummy    
        endif 
  
        cosmos_logmessage, ' Search time range for angle 3 read as ' + cosmos_str_make(c_settings.searchtimemin3) + ' to ' + cosmos_str_make(c_settings.searchtimemax3) + '.'
    
      endif

    endif
    
  endif
    
  ; parse calculation ranges
  
  ;MAG: Changed to include the different foreground, background, and lambda ranges for each angle.
  
  cosmos_logmessage, ' '
      
  ; width of foreground for angle 1 (0 is default)
  if total(c_settings.foregdwidth1) lt 0 then c_settings.foregdwidth1 = [0,0]
  cosmos_logmessage, ' Peak width (in foreground) for angle 1: ' + cosmos_str_make(c_settings.foregdwidth1[0]) + ' + peak_pixel + ' + cosmos_str_make(c_settings.foregdwidth1[1]) + '.'

  if c_settings.nangles ge 1 then begin
    
    if total(c_settings.foregdwidth2) lt 0 then c_settings.foregdwidth2 = [0,0]
    cosmos_logmessage, ' Peak width (in foreground) for angle 2: ' + cosmos_str_make(c_settings.foregdwidth2[0]) + ' + peak_pixel + ' + cosmos_str_make(c_settings.foregdwidth2[1]) + '.'

    if c_settings.nangles ge 2 then begin
  
      if total(c_settings.foregdwidth3) lt 0 then c_settings.foregdwidth3 = [0,0]
      cosmos_logmessage, ' Peak width (in foreground) for angle 3: ' + cosmos_str_make(c_settings.foregdwidth3[0]) + ' + peak_pixel + ' + cosmos_str_make(c_settings.foregdwidth3[1]) + '.'
    
    endif
    
  endif
    
  cosmos_logmessage, ' '
    
  if c_settings.backgdlowoffset1 lt 0L then c_settings.backgdlowoffset1 = 0L
  if c_settings.backgdhighoffset1 lt 0L then c_settings.backgdhighoffset1 = 0L
       
  cosmos_logmessage, ' Background for angle 1 read as : ' + string(9b) + 'low' + string(9b) + 'high'
  cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Offset : ' + string(9b) + cosmos_str_make(c_settings.backgdlowoffset1) + string(9b) + cosmos_str_make(c_settings.backgdhighoffset1)
  cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Width : ' + string(9b) + cosmos_str_make(c_settings.backgdlowwidth1) + string(9b) + cosmos_str_make(c_settings.backgdhighwidth1)
  
  if c_settings.nangles ge 1 then begin

    if c_settings.backgdlowoffset2 lt 0L then c_settings.backgdlowoffset2 = 0L
    if c_settings.backgdhighoffset2 lt 0L then c_settings.backgdhighoffset2 = 0L
      
    cosmos_logmessage, ' Background for angle 2 read as : ' + string(9b) + 'low' + string(9b) + 'high'
    cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Offset : ' + string(9b) + cosmos_str_make(c_settings.backgdlowoffset2) + string(9b) + cosmos_str_make(c_settings.backgdhighoffset2)
    cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Width : ' + string(9b) + cosmos_str_make(c_settings.backgdlowwidth2) + string(9b) + cosmos_str_make(c_settings.backgdhighwidth2)
  
    if c_settings.nangles ge 2 then begin

      if c_settings.backgdlowoffset3 lt 0L then c_settings.backgdlowoffset3 = 0L
      if c_settings.backgdhighoffset3 lt 0L then c_settings.backgdhighoffset3 = 0L
      
      cosmos_logmessage, ' Background for angle 3 read as : ' + string(9b) + 'low' + string(9b) + 'high'
      cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Offset : ' + string(9b) + cosmos_str_make(c_settings.backgdlowoffset3) + string(9b) + cosmos_str_make(c_settings.backgdhighoffset3)
      cosmos_logmessage, string(9b) + string(9b) + string(9b) + 'Width : ' + string(9b) + cosmos_str_make(c_settings.backgdlowwidth3) + string(9b) + cosmos_str_make(c_settings.backgdhighwidth3)
    
    endif
    
  endif
  
  cosmos_logmessage, ' '
  
  ; lower limit of wavelength window
  if c_settings.lambdamin1 lt 0. then c_settings.lambdamin1 = 0.
  ; upper limit of wavelength window (0 is no limit)
  if c_settings.lambdamax1 lt 0. then c_settings.lambdamax1 = 0.
  if c_settings.lambdamax1 lt c_settings.lambdamin1 then begin
    dummy = c_settings.lambdamax1
    c_settings.lambdamax1 = c_settings.lambdamin1
    c_settings.lambdamin1 = dummy  
  endif
  cosmos_logmessage, ' Wavelength range for angle 1 read as ' + cosmos_str_make(c_settings.lambdamin1) + ' to ' + cosmos_str_make(c_settings.lambdamax1) + '.'
  
  if c_settings.nangles ge 1 then begin

    ; lower limit of wavelength window
    if c_settings.lambdamin2 lt 0. then c_settings.lambdamin2 = 0.
    ; upper limit of wavelength window (0 is no limit)
    if c_settings.lambdamax2 lt 0. then c_settings.lambdamax2 = 0.
    if c_settings.lambdamax2 lt c_settings.lambdamin2 then begin
      dummy = c_settings.lambdamax2
      c_settings.lambdamax2 = c_settings.lambdamin2
      c_settings.lambdamin2 = dummy  
    endif
    cosmos_logmessage, ' Wavelength range for angle 2 read as ' + cosmos_str_make(c_settings.lambdamin2) + ' to ' + cosmos_str_make(c_settings.lambdamax2) + '.'
  
    if c_settings.nangles ge 2 then begin

      ; lower limit of wavelength window
      if c_settings.lambdamin3 lt 0. then c_settings.lambdamin3 = 0.
      ; upper limit of wavelength window (0 is no limit)
      if c_settings.lambdamax3 lt 0. then c_settings.lambdamax3 = 0.
      if c_settings.lambdamax3 lt c_settings.lambdamin3 then begin
        dummy = c_settings.lambdamax3
        c_settings.lambdamax3 = c_settings.lambdamin3
        c_settings.lambdamin3 = dummy  
      endif
      cosmos_logmessage, ' Wavelength range for angle 3 read as ' + cosmos_str_make(c_settings.lambdamin3) + ' to ' + cosmos_str_make(c_settings.lambdamax3) + '.'
      
    endif
    
  endif
  
  cosmos_logmessage, '----------------------------------------------------------------------------------------------'
  
end

;+
; :Description:
;    Main analysis routine.
;    Reads numors corresponding to reflected and direct beam, performs the required corrections,
;    and computes the reflectivity.
;
; :Params:
;    result        : out, required, type="float array" - Calculated reflectivity
;    groupedpresult: out, required, type="float array" - Calculated reflectivity (grouped data)
;    direct        : in, required, type="string array" - List of DB numors
;    reflect       : in, required, type="string array" - List of reflect numors
;    instr         : in, required, type="string array" - List of instrument background numors (often empty)
;    theta         : in, required, type="float" - Incident angle (in degrees or - 1 for san or -2 for dan) 
;    runno         : in, required, type="integer" - Angle position (1, 2 or 3)
;    nodirect_run  : in, optional, type="logical" - Set if there is not a DB measurement, so the monitor of 
;                                                   the reflected beam will be used to estimate the DB.
;
; :Author: E. Pellegrini, M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, 4 March 2015
;
; :Categories: cosmos
;
;-
pro cosmos_anal_run, result, groupedresult, direct, reflect, instr, theta, runno, nodirect_run = nodirect_run, pol_run = pol_run, tra = tra, f1 = f1, f2 = f2, store_polparams = store_polparams, pol_lambda = pol_lambda, pol_parref = pol_parref

  forward_function cosmos_str_make
  common cosmos_params, c_params
  common cosmos_settings, c_settings
  common cosmos_water, water_file, water_array, water_lowerlimit
    
  ;Set type of scan (standard = DB in detector, nodirect = DB as monitor of RB)
  if keyword_set(nodirect_run) then begin
     normal_run = 0
     nodirect_run = 1
  endif else begin
     normal_run = 1  
     nodirect_run = 0
  endelse

  ;Check if pol defined or not
  if n_elements(pol_run) gt 0 then begin
    if pol_run ge 1 then pol_run = 1 else pol_run = 0
  endif else pol_run = 0

  if n_elements(store_polparams) gt 0 then begin
    if store_polparams ge 1 then store_polparams = 1 else store_polparams = 0
  endif else store_polparams = 0
    
  ;Local variables (needs to be updated):
  ;  tsize, xsize : size of count arrays
  ;  trangedir, trangeref, xrangedir, xrangeref : ranges for peak searching
  ;  usefulx[2], searchx[2], searchy[2]
  ;  xlim : useful limits of detector
  ;  foreground = anal.foregd1[0] or anal.foregd2[0] or anal.foregd3[0]
  ;  backgroundleft1, backgroundleft2
  ;  peakdir, peakref : centre of peaks in x direction
  ;  fgdsizedir, fgdsizeref : size of foreground window
  ;  fgddir, fgdref : ranges of foreground
  ;  bgddir, bgdref : arrays of background pixels
  ;  bgddirno, bgdrefno : number of background pixels
  ;  angle_centre : angle to centre of detector
  ;  angle_bragg : Bragg angle
  
  case runno of
  
     1 : begin
        rusefulx = lonarr(2)
        rusefulx[0] = c_settings.usefulpixelmin1
        rusefulx[1] = c_settings.usefulpixelmax1
        rsearchpixel = lonarr(2)
        rsearchpixel[0] = c_settings.searchpixelmin1
        rsearchpixel[1] = c_settings.searchpixelmax1
        rsearchtime = lonarr(2)
        rsearchtime[0] = c_settings.searchtimemin1
        rsearchtime[1] = c_settings.searchtimemax1
        foregd = c_settings.foregdwidth1
        backgdlowoffset = c_settings.backgdlowoffset1
        backgdlowwidth = c_settings.backgdlowwidth1
        backgdhighoffset = c_settings.backgdhighoffset1
        backgdhighwidth = c_settings.backgdhighwidth1
        lambda_ini = c_settings.lambdamin1
        lambda_end = c_settings.lambdamax1
        grouping_value = c_settings.groupvar1
        c_params.poff = c_settings.poff1
        c_params.d0 = c_settings.d01
        c_params.openoff = c_settings.openoff1
        c_params.offset = c_settings.spoff1
        c_params.pixelwidth = c_settings.pixelwidth
        c_params.sdetd = c_settings.sdetd
        c_params.edelay = c_settings.edelay1
        c_params.xdh1 = c_settings.xdh1
        c_params.xdh2 = c_settings.xdh2
        c_params.ydet = c_settings.ydet
        calc_method = strtrim(c_settings.method1,2)
        figaro_dan_ref_pixel = c_settings.detpixel1
     end
        
     2 : begin
        rusefulx = lonarr(2)
        rusefulx[0] = c_settings.usefulpixelmin2
        rusefulx[1] = c_settings.usefulpixelmax2
        rsearchpixel = lonarr(2)
        rsearchpixel[0] = c_settings.searchpixelmin2
        rsearchpixel[1] = c_settings.searchpixelmax2
        rsearchtime = lonarr(2)
        rsearchtime[0] = c_settings.searchtimemin2
        rsearchtime[1] = c_settings.searchtimemax2
        foregd = c_settings.foregdwidth2
        backgdlowoffset = c_settings.backgdlowoffset2
        backgdlowwidth = c_settings.backgdlowwidth2
        backgdhighoffset = c_settings.backgdhighoffset2
        backgdhighwidth = c_settings.backgdhighwidth2
        lambda_ini = c_settings.lambdamin2
        lambda_end = c_settings.lambdamax2
        grouping_value = c_settings.groupvar2
        c_params.poff = c_settings.poff2
        c_params.d0 = c_settings.d02
        c_params.openoff = c_settings.openoff2
        c_params.offset = c_settings.spoff2
        c_params.pixelwidth = c_settings.pixelwidth
        c_params.sdetd = c_settings.sdetd
        c_params.edelay = c_settings.edelay2
        c_params.xdh1 = c_settings.xdh1
        c_params.xdh2 = c_settings.xdh2
        c_params.ydet = c_settings.ydet
        calc_method = strtrim(c_settings.method2,2)
        figaro_dan_ref_pixel = c_settings.detpixel2
     end
    
     3 : begin
        rusefulx = lonarr(2)
        rusefulx[0] = c_settings.usefulpixelmin3
        rusefulx[1] = c_settings.usefulpixelmax3
        rsearchpixel = lonarr(2)
        rsearchpixel[0] = c_settings.searchpixelmin3
        rsearchpixel[1] = c_settings.searchpixelmax3
        rsearchtime = lonarr(2)
        rsearchtime[0] = c_settings.searchtimemin3
        rsearchtime[1] = c_settings.searchtimemax3
        foregd = c_settings.foregdwidth3
        backgdlowoffset = c_settings.backgdlowoffset3
        backgdlowwidth = c_settings.backgdlowwidth3
        backgdhighoffset = c_settings.backgdhighoffset3
        backgdhighwidth = c_settings.backgdhighwidth3
        lambda_ini = c_settings.lambdamin3
        lambda_end = c_settings.lambdamax3
        grouping_value = c_settings.groupvar3
        c_params.poff = c_settings.poff3
        c_params.d0 = c_settings.d03
        c_params.openoff = c_settings.openoff3
        c_params.offset = c_settings.spoff3
        c_params.pixelwidth = c_settings.pixelwidth
        c_params.sdetd = c_settings.sdetd
        c_params.edelay = c_settings.edelay3
        c_params.xdh1 = c_settings.xdh1
        c_params.xdh2 = c_settings.xdh2
        c_params.ydet = c_settings.ydet
        calc_method = strtrim(c_settings.method3,2)
        figaro_dan_ref_pixel = c_settings.detpixel3
     end
    
  endcase ;end case runno --> values for corresponding angle stored in common variables

  grouping = c_settings.group ;Grouping option (yes button applies to all angles)
  
  ;Read data for reflected beam
  cosmos_raw_read, reflect, monref, cntref, parref, c_settings.pathdir, c_settings.override, /reflect
  
  ;Polarization case: Use same parameters (from 1st numor in list) for all polarization states

  if pol_run then begin
    if (store_polparams) then begin
       pol_parref = parref
    endif else if n_elements(pol_parref) gt 0 then begin
       parref.dan = pol_parref.dan
       parref.san = pol_parref.san
       parref.openangle = pol_parref.openangle
       parref.period = pol_parref.period
       parref.slits = pol_parref.slits
       parref.slitsproduct = pol_parref.slitsproduct
       parref.tofd = pol_parref.tofd
       parref.tofm = pol_parref.tofm
       parref.delayangle = pol_parref.delayangle
       parref.delay = pol_parref.delay
       parref.channelwidth = pol_parref.channelwidth
       parref.sdetd = pol_parref.sdetd
       parref.refdown = pol_parref.refdown
       parref.offset = pol_parref.offset
       parref.slit3 = pol_parref.slit3
       parref.slit2 = pol_parref.slit2
       parref.s2w = pol_parref.s2w
       parref.s3w = pol_parref.s3w
       parref.tra = pol_parref.tra
    endif
  endif

  ;Now we know the instrument and we can check if requested option is valid, as only Figaro can accept nodirect option. 
  if nodirect_run then begin
     if (~strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin 
        cosmos_logmessage, 'Not prepared to use monitor as direct beam (option only ready for figaro).', /WARNING
        result = -1
        groupedresult = -1
        return
      endif
  endif 

  ;Check that monitor or time for this numor are not zero (unless nodirect is used) and then read DB
 
  if normal_run then begin
    
     if (parref.monitor le 0) and (parref.runtime le 0) then begin
        cosmos_logmessage, ' Warning: No reflect counts read. Unable to calculate for run ' + cosmos_str_make(runno) + '.'
        result = -1
        groupedresult = -1
        return
     endif

     ;Read data for direct beam and check that it is not empty          
     cosmos_direct_read, direct, mondir, cntdir, pardir, runno
     if (pardir.monitor le 0) and (pardir.runtime le 0) then begin
        cosmos_logmessage, ' Error: No direct counts read. Unable to calculate for run ' + cosmos_str_make(runno) + '.'
        result = -1
        groupedresult = -1
        return
     endif
     
     ;coherent method requires same slit openings for DB and RB	
     ;PG on 06/05/15 This is not needed anymore as we can quite accurately calculate the theoretical direct beam width from the reflected beam slit sizes 
     ;slits_tolerance = 0.01
     ;if calc_method eq 'coherent' then begin
     ;  if abs(parref.s2w-pardir.s2w) gt slits_tolerance or $
     ;     abs(parref.s3w-pardir.s3w) gt slits_tolerance then begin
     ;       cosmos_logmessage, ' Error: Cannot use coherent analysis because slits openings are not the same in direct and reflected beam!'
     ;       cosmos_logmessage, ' Using standard incoherent analysis instead!'
     ;       cosmos_logmessage, ' Note that in the output file the method employed will still appear incorrectly as coherent'
     ;       calc_method = 'incoherent'
     ;  endif  
     ;endif     
     
  endif   
  
  ;Foreground width for DB and RB (they are the same for D17, but in the case of Figaro
  ;the width for the DB is larger by 6 pixels (3 on each side of the peak). 
  if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
     foregddir = foregd
     foregdref = foregd
  endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
     foregddir = foregd + 3
     foregdref = foregd
  endif
 
  ;Check if numor read is kinetic and if so get the number of slices
  kinetic = 0
  refsize = size(cntref)
  if refsize[0] eq 3 then begin
     kinetic = 1 
     slices = refsize[3]
  endif   

  ;Dimensions: detector pixels (xsize) and TOF channels (tsize) 
  xsize = refsize[1]
  tsize = refsize[2]

  ;If /pol_run option, get values corresponding to analyzer translation and state of flippers 1 and 2
  if pol_run then begin
    tra = parref.tra
    f1  = parref.f1
    f2  = parref.f2
  endif
  
  ;Read data for instrument background
  cosmos_raw_read, instr, monins, cntins, parins, c_settings.pathdir, c_settings.override
    
  ;Generalize by making cntref a 3D array. For a non kinetic scan set 3rd dimension to 1 time slice.
  if not kinetic then begin
     slices = 1
     cntref=reform(cntref, xsize, tsize, 1)
  endif   

  ;Loop over slices
  for islice = 0, slices-1 do begin

     cntrefi = cntref[*,*,islice]
     if normal_run then cntdiri = cntdir ;in order to not overwrite cntdir if slices>1
     cntinsi = cntins                    ;in order to not overwrite cntins if slices>1
  
     ;Check size of DB and resize if necessary (MAG: Can this really happen or only if cntdir empty?)
     if normal_run then begin
        if n_elements(cntdiri) ne n_elements(cntrefi) then begin
           cosmos_logmessage, ' Incompatible size of data array for direct runs - resizing.'
           cntdiri = congrid(cntdiri, xsize, tsize, /minus_one, /interp)
        endif
        cntdiri = float(cntdiri)
        cntdiri_err = sqrt(cntdiri) > 1.0 ;Set arbitrarily error = 1 for pixels with 0 counts
     endif
  
     ;Check size of instrument background numor and resize if necessary 
     if n_elements(cntrefi) ne n_elements(cntinsi) then begin
        if parins.monitor gt 0 then cosmos_logmessage, ' Incompatible size of data array for instrument background runs- resizing.'
        cntinsi = congrid(cntinsi, xsize, tsize, /minus_one, /interp)
     endif
    
     ;Error assignment
     cntrefi = float(cntrefi)
     cntinsi = float(cntinsi)
     cntrefi_err = sqrt(cntrefi) 
     cntinsi_err = sqrt(cntinsi) 
  
     ;Normalize DB and RB by slitsproduct in order to take into account possible differences
     ;in the opening of the slits. Each numor is divided by (S2X * S3X).
     if normal_run then begin
        if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
           if total(cntdiri) gt 0 then cntdiri /= pardir.slitsproduct
           if total(cntrefi) gt 0 then cntrefi /= parref.slitsproduct
           if total(cntinsi) gt 0 then cntinsi /= parins.slitsproduct
           if total(cntdiri) gt 0 then cntdiri_err /= pardir.slitsproduct
           if total(cntrefi) gt 0 then cntrefi_err /= parref.slitsproduct
           if total(cntinsi) gt 0 then cntinsi_err /= parins.slitsproduct
        endif
     endif
     
     ;Correct detector efficiency
     if n_elements(cntrefi[*, 0]) eq n_elements(water_array) then begin
       for i = 0, tsize - 1 do begin
         cntrefi[*, i]     /= water_array
         cntrefi_err[*, i] /= water_array
       endfor
     endif ;else cosmos_logmessage, ' Run no. ' + cosmos_str_make(runno) + ': Water incompatible with direct/reflect runs so not used.', /WARNING
     
     ;TS 2015-06-01: DB water correction should be done here as well, before cntinsi is subtracted to cntdiri
     if n_elements(cntdiri[*, 0]) eq n_elements(water_array) then begin
        for i = 0, tsize - 1 do begin
           cntdiri[*, i]     /= water_array
           cntdiri_err[*, i] /= water_array
        endfor
     endif ;else cosmos_logmessage, ' Run no. ' + cosmos_str_make(runno) + ': Water incompatible with direct/reflect runs so not used.', /WARNING

     ;MG 2016-07-05: Added also efficiency correction for cntinsi for completeness
     if n_elements(cntinsi[*, 0]) eq n_elements(water_array) then begin
       for i = 0, tsize - 1 do begin
         cntinsi[*, i]     /= water_array
         cntinsi_err[*, i] /= water_array
       endfor
     endif ;else cosmos_logmessage, ' Run no. ' + cosmos_str_make(runno) + ': Water incompatible with direct/reflect runs so not used.', /WARNING

     ;NoDirect --> Subtract instrument background to RB after normalization by monitor or time. 
     if nodirect_run then begin
        ;Determine scaling factor
        if c_settings.normalise eq 0 then begin ;counting time
           if float(parref.runtime) le 0. then begin
              cosmos_logmessage, 'Unable to normalise. Reflect beam runtime read as ' + cosmos_str_make(parref.runtime) + '.', /WARNING
              temp = 1.0
           endif else temp = float(parins.runtime) / float(parref.runtime)
        endif else begin ;monitor
           if float(parref.monitor) le 0. then begin
              cosmos_logmessage, 'Unable to normalise. Reflect beam monitor read as ' + cosmos_str_make(parref.monitor) + '.', /WARNING
              temp = 1.0
           endif else temp = float(parins.monitor) / float(parref.monitor)
        endelse
        ;Scale instrument background and subtract to RB
        if (parins.monitor gt 0) or (parins.runtime gt 0) then begin
           cntinsi     /= temp
           cntinsi_err /= temp
           cntrefi      = cntrefi - cntinsi
           cntrefi_err  = sqrt(cntrefi_err^2 + cntinsi_err^2) ;add errors in quadrature
        endif  
     ;Normal_run --> Normalize RB and instrument background to DB + subtract bgr from DB and RB
     endif else begin
        ;Determine scaling factor
        if c_settings.normalise eq 0 then begin ;counting time
           if float(pardir.runtime) le 0. then begin
               cosmos_logmessage, 'Unable to normalise. Direct beam runtime read as ' + cosmos_str_make(pardir.runtime) + '.', /WARNING
               temp1 = 1.0 & temp2 = 1.0
           endif else begin
               temp1 = float(parref.runtime) / float(pardir.runtime)
               temp2 = float(parins.runtime) / float(pardir.runtime)
           endelse
        endif else begin ; monitor
           if float(pardir.monitor) le 0. then begin
               cosmos_logmessage, 'Unable to normalise. Direct beam monitor read as ' + cosmos_str_make(pardir.monitor) + '.', /WARNING
               temp1 = 1.0 & temp2 = 1.0
           endif else begin
              temp1 = float(parref.monitor) / float(pardir.monitor)
              temp2 = float(parins.monitor) / float(pardir.monitor)
           endelse
        endelse
        ;Scale RB (reflected beam)
        cntrefi     /= temp1
        cntrefi_err /= temp1
        ;Scale instrument background and subtract to RB and DB (for normal run)
        if (parins.monitor gt 0) or (parins.runtime gt 0) then begin
           cntinsi     /= temp2
           cntinsi_err /= temp2
           cntrefi      = cntrefi - cntinsi
           cntrefi_err  = sqrt(cntrefi_err^2 + cntinsi_err^2) ;add errors in quadrature
           if normal_run then begin
              cntdiri     = cntdiri - cntinsi
              cntdiri_err = sqrt(cntdiri_err^2 + cntinsi_err^2) ;add errors in quadrature
           endif
        endif
     endelse ;end time/monitor normalization and background subtraction   
        
     ;Standard case: Define peakdir = DB search ranges (only once if kinetic scan) 
     if normal_run and islice eq 0 then begin

        ;Useful pixel detector limits
        xlim = [ (rusefulx[0] - pardir.x_min) > 0L, (rusefulx[1] - pardir.x_min) < (xsize - 1) ] ;assumes x_min/max are the same for DB and RB
        
        ;TOF channels range
        if rsearchtime[1] lt 0 then trangedir = [0, tsize - 1] else trangedir = rsearchtime
      
        ;The search limits defined in the table are only applied to the peak search for the RB.
        ;For the DB the search is done over the full useful range.
        xrangedir = cosmos_anal_peaksearchbasic(total(cntdiri[xlim[0]:xlim[1],*], 2)) + xlim[0]
        cosmos_logmessage, ' Auto searching for peak; preliminary limits found: Direct = [' + cosmos_str_make(xrangedir[0]) + ', ' + cosmos_str_make(xrangedir[1]) + ']'
           
        ;Peak search ranges for DB 
        xrangedir = [ xrangedir[0] > xlim[0], xrangedir[1] < xlim[1] ]
          
        ;Search for DB peak
        cosmos_anal_peaksearchpro, cntdiri[*, trangedir[0]:trangedir[1]], xrangedir, xlim, peakdir, peakdirrange
          
        if (peakdir lt 0L) then begin
           cosmos_logmessage, ' Unable to fit direct peak. This may be due to poor statistics.'
           if total(foregddir) le 0 then begin
              cosmos_logmessage, ' Defaulting to (foreground) peak width of 13 (Auto)'
              peakdir = cosmos_anal_peaksearchrefined(total(cntdiri, 2), xrangedir, [6,6])
              peakdirrange = indgen(13) + peakdir - 6
           endif else peakdir = cosmos_anal_peaksearchrefined(total(cntdiri, 2), xrangedir, foregddir)
        endif
        
     endif ;end if normal_run --> peakdir defined (peakdir does not exist for nodirect_run)
     
     ;Now define search ranges for RB --> peakref
     xlim = [ (rusefulx[0] - parref.x_min) > 0L, (rusefulx[1] - parref.x_min) < (xsize - 1) ] ;useful detector limits
     if rsearchtime[1] lt 0 then trangeref = [0, tsize - 1] else trangeref = rsearchtime ;TOF channels range
     ;Pixel range or set to table values if Auto option is not selected
     if rsearchpixel[1] lt 0 then begin ;'auto' option
       xrangeref = cosmos_anal_peaksearchbasic(total(cntrefi[xlim[0]:xlim[1],*], 2)) + xlim[0]
       cosmos_logmessage, ' Auto searching for peak; preliminary limits found: Reflect = [' + cosmos_str_make(xrangeref[0]) + ', ' + cosmos_str_make(xrangeref[1]) + ']'
     endif else xrangeref = rsearchpixel
     xrangeref = [ xrangeref[0] > xlim[0], xrangeref[1] < xlim[1] ]
     cosmos_anal_peaksearchpro, cntrefi[*, trangeref[0]:trangeref[1]], xrangeref, xlim, peakref, peakrefrange

     if (peakref lt 0L) then begin
        cosmos_logmessage, ' Unable to fit reflect peak. This may be due to poor statistics.'
        if total(foregdref) le 0 then begin
           peakref = cosmos_anal_peaksearchrefined(total(cntrefi, 2), xrangeref, [6,6])
           peakrefrange = indgen(13) + peakref - 6
        endif else peakref = cosmos_anal_peaksearchrefined(total(cntrefi, 2), xrangeref, foregdref)
     endif

     ;Patch to use the same peak position in all the polarization runs
     ;Relies on c_settings.peakref being 0 for all the angles before 1st calculation
     if pol_run then begin
        if  c_settings.peakref[runno-1] eq 0 then c_settings.peakref[runno-1] = peakref $ 
                                             else peakref = c_settings.peakref[runno-1]
     endif
     cosmos_logmessage, ' '
  
     if normal_run then begin
        cosmos_logmessage, ' Direct peak at pixel = ' + cosmos_str_make(peakdir) + '. Reflect peak at pixel = ' + cosmos_str_make(peakref)
     endif else begin
        cosmos_logmessage, ' No direct beam used (only monitor). Reflect peak at pixel = ' + cosmos_str_make(peakref)
     endelse

     ;Patch to force peak to value if limits are the same
     ;Remember to also specify foreground window size if this is the case!!
     if normal_run then begin
        if rsearchpixel[0] eq rsearchpixel[1] then peakdir = rsearchpixel[0]
     endif
     if rsearchpixel[0] eq rsearchpixel[1] then peakref=rsearchpixel[0]
  
     ;Check foreground window size
     if total(foregd) le 0 then begin ;'auto' foreground
        cosmos_logmessage, ' Auto foreground pixel window found as'
     endif else begin
        if normal_run then peakdirrange = indgen(total(foregddir)+1) + peakdir - foregddir[0]
        peakrefrange = indgen(total(foregdref)+1) + peakref - foregdref[0] ;PG: Is this where the foreground pixels are defined? ;MG: Yes!
        cosmos_logmessage, ' Manual foreground pixel requested, size ' + cosmos_str_make(foregddir[0]) + $
                                                             ' + 1 + ' + cosmos_str_make(foregddir[1]) + $
                                                      ' (direct) and ' + cosmos_str_make(foregdref[0]) + $
                                                             ' + 1 + ' + cosmos_str_make(foregdref[1]) + $
                                                   ' (reflect), giving'
     endelse
  
     if normal_run then begin
       cosmos_logmessage, '  Direct: (' + cosmos_str_make(min(peakdirrange)) + $ 
                                    ',' + cosmos_str_make(max(peakdirrange)) + $
                        ')  Reflect: (' + cosmos_str_make(min(peakrefrange)) + $
                                    ',' + cosmos_str_make(max(peakrefrange)) + ')'
     endif else begin
        cosmos_logmessage, '  No direct beam. Reflect: (' + cosmos_str_make(min(peakrefrange)) + ',' + cosmos_str_make(max(peakrefrange)) + ')'  
     endelse
      
     ;Foreground ranges
     if normal_run then begin
        peakdirrange = peakdirrange[where((peakdirrange ge xlim[0]) and (peakdirrange le xlim[1]))]
        fgddir = [min(peakdirrange), max(peakdirrange)]
        bgddir = intarr(xsize) ; maskbk for background
     endif
     peakrefrange = peakrefrange[where((peakrefrange ge xlim[0]) and (peakrefrange le xlim[1]))]  
     fgdref = [min(peakrefrange), max(peakrefrange)]  
     bgdref = intarr(xsize)
      
     ;Background ranges 
     if (backgdlowoffset le 0L) and (backgdlowwidth lt 0L) then begin
        if normal_run then if fgddir[0] gt xlim[0] then bgddir[xlim[0]:fgddir[0] - 1] = 0
        if                fgdref[0] gt xlim[0] then bgdref[xlim[0]:fgdref[0] - 1] = 0
        cosmos_logmessage, ' No left background specified.'
     endif else begin
        if normal_run then begin
           temp1 = fgddir[0] - (backgdlowoffset > 0L) - 1
           if (temp1 ge xlim[0]) and (backgdlowwidth gt 0) then bgddir[(temp1-backgdlowwidth+1)>xlim[0]:temp1] = 1
           cosmos_logmessage, ' Direct background range, low = (' + cosmos_str_make(temp1-backgdlowwidth + 1) + ',' + cosmos_str_make(temp1) + ')'
        endif
        temp1 = fgdref[0] - (backgdlowoffset > 0L) - 1
        if (temp1 ge xlim[0]) and (backgdlowwidth gt 0) then bgdref[(temp1-backgdlowwidth+1)>xlim[0]:temp1] = 1
        cosmos_logmessage, ' Reflect background range, low = (' + cosmos_str_make(temp1-backgdlowwidth + 1) + ',' + cosmos_str_make(temp1) + ')'
     endelse
    
     if (backgdhighoffset le 0L) and (backgdhighwidth lt 0L) then begin
        if normal_run then if fgddir[1] lt xlim[1] then bgddir[fgddir[1]+1:xlim[1]] = 0
        if fgdref[1] lt xlim[1] then bgdref[fgdref[1]+1:xlim[1]] = 0
        cosmos_logmessage, ' No right background specified.'
     endif else begin
        if normal_run then begin 
           temp1 = fgddir[1] + (backgdhighoffset > 0L) + 1
           if (temp1 le xlim[1]) and (backgdhighwidth gt 0) then bgddir[temp1:(temp1+backgdhighwidth-1)<xlim[1]] = 1
           cosmos_logmessage, ' Direct background range, high = (' + cosmos_str_make(temp1) + ',' + cosmos_str_make(temp1+backgdhighwidth - 1) + ')'
        endif
        temp1 = fgdref[1] + (backgdhighoffset > 0L) + 1
        if (temp1 le xlim[1]) and (backgdhighwidth gt 0) then bgdref[temp1 :(temp1+backgdhighwidth-1)<xlim[1]] = 1
        cosmos_logmessage, ' Reflect background range, high = (' + cosmos_str_make(temp1) + ',' + cosmos_str_make(temp1+backgdhighwidth - 1) + ')'
     endelse
  
     ;Background arrays
     if normal_run then begin
        bgddir = where(bgddir eq 1, bgddirno)  
        if bgddirno le 0 then cosmos_logmessage, 'No background to subtract on direct runs.', /WARNING
     endif
     bgdref = where(bgdref eq 1, bgdrefno)
     if bgdrefno le 0 then cosmos_logmessage, 'No background to subtract on reflect runs.', /WARNING
  
     if normal_run then begin
        cosmos_logmessage, ' Total background: Direct: ' + cosmos_str_make(bgddirno) + '. Reflect:' + cosmos_str_make(bgdrefno)
     endif else begin
        cosmos_logmessage, ' Total background: No direct beam. Reflect:' + cosmos_str_make(bgdrefno)
     endelse
  
     ;Subtract background
     if bgdrefno gt 0 then begin
        if c_settings.backmeth eq 0 then begin ;subtract 'average' background
           if normal_run then if bgddirno gt 0 then cosmos_anal_backgroundaverage_old, cntdiri, cntdiri_err, bgddir
           cosmos_anal_backgroundaverage_old, cntrefi, cntrefi_err, bgdref
;           if normal_run then if bgddirno gt 0 then cosmos_anal_backgroundaverage, cntdiri, cntdiri_err, bgddir
;           cosmos_anal_backgroundaverage, cntrefi, cntrefi_err, bgdref
        endif else begin ;subtract 'fitted' background
           if normal_run then if bgddirno gt 0 then cosmos_anal_backgroundfit, cntdiri, cntdiri_err, bgddir, 'direct'
           cosmos_anal_backgroundfit, cntrefi, cntrefi_err, bgdref, 'reflect'
        endelse
     endif
  
     ;Determine theta and lambda (only for 1st frame if kinetic)
     if islice eq 0 then begin  

        ;DET option only valid for Figaro
        ;For other cases, return to SAN option whenever DET was requested
        if theta eq -3. then begin
          if ~(strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
            cosmos_logmessage, ' DET option only for Figaro. Using SAN to calculate theta!'
            theta = -1.
          endif
        endif
    
        case theta of
          
           -1. : begin ;SAN option
            
              if calc_method eq 'coherent' then begin   

                 ;coherent method --> the scattering angle is to be determined from the pixel selected
                 ;to project the (2theta,lambda) data, so we need to correct the difference between this
                 ;pixel and the exact position of the reflected peak.
                 ;2-theta = 2*san + [angle(newpeakref,pcen) - angle(peakref,pcen)] (signs will depend on geometry)

                 ;newpeakref is the exact position of the reflected peak
                 ;peakref is the pixel number (integer) where the peak was found
                 ;MAG: check calculation in DAN - is it necessary to add half a pixel to peakref when computing angle_bragg?
                 
                 ;sdr = parref.tofd - c_params.d0 + c_params.chopsep/2. ;sample-detector distance --> substituted by parref.sdetd
                 
                 if n_elements(peakrefrange) ge 7 then fit_range = peakrefrange else fit_range = indgen(7) + peakref - 3
                 fit_gauss = gaussfit(fit_range, total(cntrefi[fit_range,*],2), fit_result, nterms=3)
                 newpeakref = fit_result[1]
                 cosmos_logmessage, ' Constant q-binning option. Reflect peak position (from gaussian fit) = ' + cosmos_str_make(newpeakref)

                 if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
                 	  trueSampleDetDistance = parref.sdetd - parref.offset	;PG on 27/7/15

                    if parref.refdown eq 0 then begin  ;determine if reflection down
                       angle_bragg = (parref.san * !pi / 180.) + $
                          0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) - $ ;PG on 06/3/15: One too many half pixels... (9x)	;PG on 27/7/15
                          0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)	;PG on 27/7/15
                    endif else begin
                       angle_bragg = (parref.san * !pi / 180.) - $
                          0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) + $	;PG on 27/7/15
                          0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)	;PG on 27/7/15
                    endelse
                 endif else begin ;D17
                    angle_bragg = (parref.san * !pi / 180.) - $
                       0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/parref.sdetd) + $
                       0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/parref.sdetd)
                 endelse
                 cosmos_logmessage, ' Using SAN to calculate theta = ' + cosmos_str_make(angle_bragg * 180. / !pi)+' deg.'

              endif else begin ;incoherent method 

                 angle_bragg = parref.san * !pi / 180. &  ;convert to radians
                 cosmos_logmessage, ' '
                 cosmos_logmessage, ' Using SAN to calculate theta = ' + cosmos_str_make(angle_bragg * 180. / !pi)+' deg.'

              endelse
           end
      
           -2. : begin ;DAN option 
            
            if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
              
              ;sdd = pardir.tofd - c_params.d0 + c_params.chopsep/2.0 ;sample-detector distance DB --> use pardir.sdetd 
              ;sdr = parref.tofd - c_params.d0 + c_params.chopsep/2.0 ;sample-detector distance RB --> use parref.sdetd
              angle_centre = ((parref.dan - pardir.dan) / 2.) * !pi / 180. 
                
              ; Refit peak positions because we need a better accuracy (using peakref/peakdir accuracy is just 1 pixel)
              if n_elements(peakdirrange) ge 7 then fit_range = peakdirrange else fit_range = indgen(7) + peakdir - 3
              fit_gauss = gaussfit(fit_range, total(cntdiri[fit_range,*],2), fit_result, nterms=3)
              newpeakdir =fit_result[1]
              if n_elements(peakrefrange) ge 7 then fit_range = peakrefrange else fit_range = indgen(7) + peakref - 3
              fit_gauss = gaussfit(fit_range, total(cntrefi[fit_range,*],2), fit_result, nterms=3)
              newpeakref = fit_result[1]
                
              cosmos_logmessage, ' D17: DAN option. Direct peak position (new from gaussian fit) = ' + cosmos_str_make(newpeakdir)
              cosmos_logmessage, ' D17: DAN option. Reflect peak position (new from gaussian fit) = ' + cosmos_str_make(newpeakref)
                
              if calc_method eq 'coherent' then begin
                 ;Coherent analysis --> detector pixels projected onto a given pixel, so we need
                 ;to select middle of specular pixel (peakref) instead of value from fit
                 angle_bragg = angle_centre + $
                    0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/pardir.sdetd) - $
                    0.5*atan(((float(peakref)+0.5)-c_params.pcen)*c_params.pixelwidth/parref.sdetd)  ;PG on 13/2/15: I think we have to use the middle of the coherent center pixel
              endif else begin
                 angle_bragg = angle_centre + $
                    0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/pardir.sdetd) - $
                    0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/parref.sdetd)
              endelse
                
              cosmos_logmessage, ' D17: Using DAN to calculate theta = ' + cosmos_str_make(angle_bragg * 180. / !pi) +$
                                 ' deg. (angle to centre of detector = ' + cosmos_str_make(angle_centre * 180. / !pi) + ')'
          
            endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
  
              angle_centre = ((parref.dan - pardir.dan) / 2.) * !pi / 180. 

              ; Refit peak positions because we need a better accuracy (using peakref/peakdir accuracy is just 1 pixel)
              if n_elements(peakdirrange) ge 7 then fit_range = peakdirrange else fit_range = indgen(7) + peakdir - 3
              fit_gauss = gaussfit(fit_range, total(cntdiri[fit_range,*],2), fit_result, nterms=3)
              newpeakdir =fit_result[1]
              if n_elements(peakrefrange) ge 7 then fit_range = peakrefrange else fit_range = indgen(7) + peakref - 3
              fit_gauss = gaussfit(fit_range, total(cntrefi[fit_range,*],2), fit_result, nterms=3)
              newpeakref = fit_result[1]

              cosmos_logmessage, ' Figaro: DAN option. Direct peak position (new from gaussian fit) = ' + cosmos_str_make(newpeakdir)
              cosmos_logmessage, ' Figaro: DAN option. Reflect peak position (new from gaussian fit) = ' + cosmos_str_make(newpeakref)

              trueSampleDetDistance = parref.sdetd - parref.offset

              if calc_method eq 'coherent' then begin
                ;Coherent analysis --> detector pixels projected onto a given pixel, so we need
                ;to select middle of specular pixel (peakref) instead of value from fit
                if parref.refdown then angle_bragg = -1.0 * angle_centre + $	;PG on 27/7/15: dan value has a sign in the raw data file
                  0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) - $
                  0.5*atan(((float(peakref)+0.5)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) $ 
                else angle_bragg = angle_centre - $
                  0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) + $
                  0.5*atan(((float(peakref)+0.5)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) 
              endif else begin
                if parref.refdown then angle_bragg = -1.0 * angle_centre + $	;PG on 27/7/15: dan value has a sign in the raw data file
                  0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) - $
                  0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) $
                else angle_bragg = angle_centre - $
                  0.5*atan((newpeakdir-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) + $
                  0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance) 
              endelse

              cosmos_logmessage, ' Figaro: Using DAN to calculate theta = ' + cosmos_str_make(angle_bragg * 180. / !pi) 
              
            endif
            
           end ;end case = -2 (DAN)

           -3. : begin ;DET option (only Figaro)

               ;Refine theta from difference between real peak position and wanted position
               if figaro_dan_ref_pixel gt 0 and figaro_dan_ref_pixel lt (c_params.pixels_x-1) then $
                 reference_pixel = figaro_dan_ref_pixel else reference_pixel = parref.wantedpixel

               ; Refit peak positions because we need a better accuracy (using peakref/peakdir accuracy is just 1 pixel)
               if n_elements(peakrefrange) ge 7 then fit_range = peakrefrange else fit_range = indgen(7) + peakref - 3
               fit_gauss = gaussfit(fit_range, total(cntrefi[fit_range,*],2), fit_result, nterms=3)
               newpeakref = fit_result[1]

               trueSampleDetDistance = parref.sdetd - parref.offset

               ;correction based on assuming that theta = parref.wantedtheta when
               ;reflection is in parref.wantedpixel and that the distance from sample
               ;to parref.wantedpixel is trueSampleDetDistance and that the detector is
               ;placed perpendicular to the axis between the sample and parref.wantedpixel

               if parref.refdown then angle_bragg = parref.wantedtheta * !pi / 180. - $
                 0.5*atan((newpeakref-reference_pixel)*c_params.pixelwidth/trueSampleDetDistance) $
               else angle_bragg = parref.wantedtheta * !pi / 180. + $
                 0.5*atan((newpeakref-reference_pixel)*c_params.pixelwidth/trueSampleDetDistance)

               cosmos_logmessage, ' Figaro: Using DET to calculate theta = ' + cosmos_str_make(angle_bragg * 180. / !pi) + ' deg. ' + $
                 ' (Reference detector position = ' + cosmos_str_make(reference_pixel) + $
                 ' , Fitted reflected peak position = ' + cosmos_str_make(newpeakref) + $
                 ' , Wanted Theta = ' + cosmos_str_make(parref.wantedtheta) +')'

           end ;end case = -3 (DET)
      
           else : begin ;using theta value from table
            
              if calc_method eq 'coherent' then begin
                 
                 ;coherent method --> correct difference between exact position of reflected peak
                 ;and integer pixel used to project the detector data (as for SAN method)
                 
                 ;sdr = parref.tofd - c_params.d0 + c_params.chopsep/2. --> use parref.sdetd
                 
                 if n_elements(peakrefrange) ge 7 then fit_range = peakrefrange else fit_range = indgen(7) + peakref - 3
                 fit_gauss = gaussfit(fit_range, total(cntrefi[fit_range,*],2), fit_result, nterms=3)
                 newpeakref = fit_result[1]

                 cosmos_logmessage, ' Constant q-binning option. Reflect peak position (from gaussian fit) = ' + cosmos_str_make(newpeakref)

                 if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
                 	  trueSampleDetDistance = parref.sdetd - parref.offset
                    if parref.refdown eq 0 then begin  ;determine if reflection down
                       angle_bragg = (theta * !pi / 180.) + $
                          0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)- $	;PG on 27/7/15:
                          0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)	;PG on 27/7/15:
                   endif else begin
                       angle_bragg = (theta * !pi / 180.) - $
                          0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)+ $	;PG on 27/7/15:
                          0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/trueSampleDetDistance)	;PG on 27/7/15:
                   endelse
                 endif else begin ;D17
                    angle_bragg = (theta * !pi / 180.) - $
                        0.5*atan((float(peakref)-c_params.pcen)*c_params.pixelwidth/parref.sdetd)+ $
                        0.5*atan((newpeakref-c_params.pcen)*c_params.pixelwidth/parref.sdetd)
                 endelse
                 cosmos_logmessage, ' Using user supplied figure for theta = ' + cosmos_str_make(angle_bragg * 180. / !pi)+' deg.'
              
              endif else begin ;incoherent
                 angle_bragg = theta*!pi/180.
                 cosmos_logmessage, ''
                 cosmos_logmessage, ' Using user supplied figure for theta = ' + cosmos_str_make(angle_bragg * 180. / !pi)+' deg.'
              endelse
              
           end
           
        endcase
  
        case runno of
            1: c_params.angle_bragg_1 = angle_bragg / !dtor
            2: c_params.angle_bragg_2 = angle_bragg / !dtor 
            3: c_params.angle_bragg_3 = angle_bragg / !dtor 
        endcase
  
        ;Determine wavelength (in angstroms) from TOF channels --> lambda[t] (using RB)
        corrected_sdetd = cosmos_anal_correctdistance(peakref, parref.x_min, parref.pixeldensity, parref.sdetd)
        ref_total_tofd = parref.tofd - parref.sdetd + corrected_sdetd
        if normal_run then begin
           corrected_sdetd = cosmos_anal_correctdistance(peakdir, parref.x_min, parref.pixeldensity, pardir.sdetd)
           dir_total_tofd = pardir.tofd - pardir.sdetd + corrected_sdetd
           dir_ref_diff = abs(ref_total_tofd - dir_total_tofd)
           cosmos_logmessage, ' Difference in corrected TOF distance between direct and reflect beams is ' + cosmos_str_make(dir_ref_diff)
           if (dir_ref_diff / ref_total_tofd) gt 0.01 then cosmos_logmessage, ' Run no. ' + cosmos_str_make(runno) + ': Different TOF distances from direct and reflect runs.', /WARNING
        endif
        lambda = 1e10 * (c_params.planckperkg * ((findgen(tsize) + 0.5) * parref.channelwidth + parref.delay) / ref_total_tofd)
    
        cosmos_logmessage, ' '
        cosmos_logmessage, ' Constants used in calculating lambda :'
        cosmos_logmessage, '  poff         = ' + cosmos_str_make(c_params.poff)
        cosmos_logmessage, '  openoff      = ' + cosmos_str_make(c_params.openoff)
        cosmos_logmessage, '  d0           = ' + cosmos_str_make(c_params.d0) + ' m'
        cosmos_logmessage, '  pixelwidth   = ' + cosmos_str_make(c_params.pixelwidth) + ' m'
        cosmos_logmessage, '  TOF distance = ' + cosmos_str_make(ref_total_tofd)
        cosmos_logmessage, '  constants.planckperkg = ' + cosmos_str_make(c_params.planckperkg)
        cosmos_logmessage, '  parref.channelwidth = ' + cosmos_str_make(parref.channelwidth)
        cosmos_logmessage, '  parref.delay = ' + cosmos_str_make(parref.delay)
        cosmos_logmessage, '  parref.offset = ' + cosmos_str_make(parref.offset)  

        ;Reevaluation of lambda to take into account gravity effects (only for Figaro - horizontal reflectometer)
        openofflam=lambda*0.0  ;PG on 7/8/17 for consistency with FIGARO      
        if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
           newlambda = lambda
           newtheta = fltarr(tsize)
           newdelayangle = fltarr(tsize)
           newdelay = fltarr(tsize)    
           xchopper = c_params.d0 ;distance mid-chopper to sample
           xslits3 = parref.slit3 + parref.offset
           xslits2 = parref.slit2 + parref.offset
           cosmos_logmessage, '  Gravity correction with xslits2 = ' + cosmos_str_make(xslits2)
           cosmos_logmessage, '  Gravity correction with xslits3 = ' + cosmos_str_make(xslits3)
           cosmos_gravnew, angle_bragg, lambda, newtheta, delta, dz, poffoff, openofflam, xchopper, xslits3, xslits2, parref.refdown,c_params.chopsep,c_params.cr
           newdelayangle = parref.delayangle - poffoff/2.
           newdelay = parref.delay + ((parref.delayangle - newdelayangle)/360.)*parref.period
           newlambda = 1e10 * (c_params.planckperkg * ((findgen(tsize)+0.5) * parref.channelwidth + newdelay) / ref_total_tofd)
        endif
  
        ;Write lambda[t] array to file
        temp1 = 0
        lambda_lun = -1
        openw, lambda_lun, 'cosmos_lambdaconv' + cosmos_str_make(runno) + '.dat', error = temp1, /get_lun
        if temp1 eq 0 then begin
           on_ioerror, lambda_exit
           if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
              printf, lambda_lun, 'Time Pixel,   Wavelength (Angstroms),   Wavelength after gravity corr,   Theta after gravity corr'
              for i = 0, n_elements(lambda) - 1 do printf, lambda_lun, i, ',', lambda[i], ',', newlambda[i], ',', newtheta[i]
           endif else begin
              printf, lambda_lun, 'Time Pixel,   Wavelength (Angstroms)'
              for i = 0, n_elements(lambda) - 1 do printf, lambda_lun, i, ',', lambda[i]
           endelse
        endif
        lambda_exit: if lambda_lun gt 0 then free_lun, lambda_lun

        ;For Figaro, use gravity corrected wavelength
        if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then lambda = newlambda 

        ;Determine useful lambda range (from table or simply lambda > 0 if lambda_max = 0 in table)
        if lambda_end eq 0 then lambda_range = where(lambda gt 0., result_size) else $
           lambda_range = where((lambda gt lambda_ini) and (lambda lt lambda_end), result_size)
         ;TS 2015-08-24 NOTE: HERE IS A PROBLEM IN POLARIZED DATA (226299,226300,226301,226302) when lambda size is one point longer...

     endif ;end of if islice = 0 (angle_bragg and lambda[t] computed)
  
     ;Sum over 2-theta (in peakdir range) for direct beam  ---> cntdiri = DB[lambda]
     if normal_run then begin
        tempdb = cntdiri ;Full DB needed later on for new resolution calculation
        cntdiri = total(cntdiri[fgddir[0]:fgddir[1], *], 1)
        cntdiri_err = sqrt(total(cntdiri_err[fgddir[0]:fgddir[1], *]^2, 1)) 

        ;Polarized DB --> check if analyzer used or not and if used, correct analyzer efficiency
        if pol_run then begin
          cosmos_logmessage, ' '
          ;First check that both flippers are off
          if pardir.f1 eq 0 and pardir.f2 eq 0 then begin
            if pardir.tra gt 200. then $
              cosmos_logmessage, 'Direct Beam measured without analyzer.' $ ;No corrections to do
            else if pardir.tra lt 0.0 then $
              cosmos_logmessage, 'TRA < 0 in direct beam measurement. Assuming analyzer is out!' $
            else begin
              cosmos_logmessage, 'Correcting Direct Beam for analyzer efficiency!' ;TS 2015-08-24 Added "Direct Beam" for clarity
              cosmos_tofpolcorr_db, cntdiri, cntdiri_err, lambda
            endelse
          endif else begin
            cosmos_logmessage, 'Flipper state does not correspond to 00. Please use appropriate run for DB00 correction', /WARNING
          endelse
        endif ;endif polarized DB
     endif
  
     ;Calculating the rms angular variation of the specular
     ;Need this for the resolution calculation for bent samples in the conventional incoherent sum method.  
     ;angd is the angular distribution on the detector in pixels
     ;det_fwhm is the fwhm variation on the detector of specular signal [bc]
     angd = total(cntrefi[fgdref[0]:fgdref[1], *], 2)
     temp = 0 & for i = 0, n_elements(angd)-1 do temp = temp + angd[i]*float(i)
     pref = temp/total(angd) + fgdref[0]
     angd_cen = pref - fgdref[0]
     tt = 0 & for i = 0, n_elements(angd)-1 do tt = tt + angd[i]*(float(angd_cen-i))^2
     det_fwhm = 2.3548 * c_params.pixelwidth * sqrt(tt/(total(angd))) ;fwhm in m distribution 2th on detector

     ;Do the same calculation for the direct beam, the difference in the rms values is from the waviness [RC]
     if normal_run then begin
       angdb = total(tempdb[fgddir[0] : fgddir[1], *], 2)
       temp = 0 & for i = 0, n_elements(angdb)-1 do temp = temp + angdb[i]*float(i)
       pref = temp/total(angdb) + fgddir[0]
       angdb_cen = pref - fgddir[0]
       tt = 0 & for i = 0,n_elements(angdb)-1 do tt = tt + angdb[i]*(float(angdb_cen-i))^2
       detdb_fwhm = 2.3548 * c_params.pixelwidth * sqrt(tt/(total(angdb))) ;db fwhm in m distribution 2th on detector
     endif


     ;PG on 16/2/15: This had to be moved up, such that the information whether the sample is bent exists for Bob's procedure which follows
     ratio = parref.s3w/parref.s2w
     ;coll = c_params.interslit
     ;sdd =pardir.sdetd; 
     ;sdr =parref.sdetd; 
     sdr = parref.slit3 + parref.sdetd ;s3 to detector dist in m
     vs = sdr+(ratio*c_params.interslit*0.001)/(1+ratio) ;virtual source position for the collimation [bc]
     ;print,'vs=',vs
     da = 0.68 * sqrt((parref.s2w^2 + parref.s3w^2)/c_params.interslit^2) ;fwhm incoming angular width
     s2_fwhm = (0.68*parref.s2w)/c_params.interslit  ;fwhm angles of first slit PG:on 13/2/15 corrected for interslit distance, this has to be anticipated to choose the case for coherent COSMOS
     detres = (2.2)*0.001  ;FWHM  of detector resolution assuming a gaussian (m) same for both figaro and d17
     da_det = sqrt((da*vs)^2+detres^2)    ;fwhm on detector of da from virtual source in m
     print,'da_det=',da_det
     print,'det_fwhm=',det_fwhm
     print,'detdb_fwhm=',detdb_fwhm
     print, 'pixelwidth=', c_params.pixelwidth
     ; (CARE: should be an instrument parameter, CHECK: same resolution for Figaro/D17? BioRef in future?)
     ;MAG: Patch to make nodirect option to work, but this has to be corrected
     ;I guess coherent method makes no sense without DB - need to be able to calculate resol without DB
     ;To revise when last version of resolution calculation is defined by Bob and Philipp
     
     if ((abs(parref.s2w-pardir.s2w) ge 0.04) or (abs(parref.s3w-pardir.s3w) ge 0.04)) then begin	;PG on 28/04/15: This case is needed if different slit settings were used for the direct and reflected beams
        if (det_fwhm-da_det) ge 0 then begin
           om_fwhm_real = 0.5*sqrt((det_fwhm)^2-da_det^2)/pardir.sdetd
           if sqrt((det_fwhm)^2-da_det^2) ge c_params.pixelwidth then om_fwhm = 0.5*sqrt((det_fwhm)^2-da_det^2)/pardir.sdetd	;PG on 27/7/15: I think the threshhold should be the pixel size (4x)
           if sqrt((det_fwhm)^2-da_det^2) lt c_params.pixelwidth then om_fwhm = 0
           print, 'firstif=', sqrt((det_fwhm)^2-da_det^2)
        endif else begin
           om_fwhm_real = 0
           om_fwhm = 0
        endelse   
     endif else begin        
     	if normal_run then begin
        	if ((det_fwhm)^2-detdb_fwhm^2) ge 0 then begin
           		om_fwhm_real = 0.5*sqrt((det_fwhm)^2-detdb_fwhm^2)/pardir.sdetd
           		if sqrt((det_fwhm)^2-detdb_fwhm^2) ge c_params.pixelwidth then om_fwhm = 0.5*sqrt((det_fwhm)^2-detdb_fwhm^2)/pardir.sdetd
           		if sqrt((det_fwhm)^2-detdb_fwhm^2) lt c_params.pixelwidth then om_fwhm = 0
           		print, 'secondif=', sqrt((det_fwhm)^2-detdb_fwhm^2)
        	endif else begin
           		om_fwhm_real = 0
           		om_fwhm = 0
        	endelse   
     	endif else begin
        	om_fwhm_real = 0
        	om_fwhm = 0
     	endelse
     endelse
     twotheta = 2.0 * angle_bragg ;2theta of the specular
     print, 'om_fwhm (deg)=', om_fwhm*180/!pi
     print, 'om_fwhm_real (deg)=', om_fwhm_real*180/!pi
     ;Coherent treatment (only for normal_run)
     if normal_run and calc_method eq 'coherent' then begin
     
     
        cosmos_logmessage, ' '
        cosmos_logmessage, 'Coherent analysis'
        cosmos_logmessage, 'Coherent analysis - binning to specular pixel : ' + cosmos_str_make(peakref)
  
        ;b_lam = width of one time channel  ;PG on 6/2/2015
        b_lam=c_params.planckperkg * parref.channelwidth*1e10 / parref.tofd ;definition of the bin size in wavelength

        ;b_tth = angular size of a pixel from the sample
        b_tth = atan(c_params.pixelwidth/parref.sdetd)    ;definition of the angular size of a pixel from the sample

        ;define the extremes of 2theta in the chosen data range [bob 14-5-15]
        if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) and parref.refdown eq 0  then begin
          ; if figaro and reflecting up we need to change the signs of 2theta inside the data area [bob 9/3/15]
          tth_max = twotheta + float(fgdref[1]-peakref) * b_tth  ; 2th foreground max
          tth_min = twotheta + float(fgdref[0]-peakref) * b_tth  ; 2th foreground min
        endif else begin  ;D17 or figaro reflecting down
          tth_max = twotheta + float(peakref-fgdref[0]) * b_tth  ; 2th foreground max
          tth_min = twotheta + float(peakref-fgdref[1]) * b_tth  ; 2th forground min
        endelse

        ;find the real limits of wavelength needed outside the requested range [bob 14-5-15]
        ;TS 2015-06-01 ((detres/(2*parref.sdetd)) changed to ((detres/(parref.sdetd))
        ;since the comparison to S2 does not need the factor of 2 in the detector resolution
        if ((om_fwhm gt 0) AND ((detres/(parref.sdetd)) gt s2_fwhm)) then begin ;bent case
          cosmos_logmessage, 'Bent case Lam max. ', /WARNING
          lam_max_p = (sin((tth_max)*0.5)/sin(twotheta/2.))*lambda_end ; max lam needed for bent case  [bob 13/4/15]
          lam_min_p = (sin((tth_min)*0.5)/sin(twotheta/2.))*lambda_ini ; min lam needed for bent case  [bob 13/4/15]
        endif else begin  ;divergent case
          ;print,'divvy case'
          lam_max_p = (sin((tth_max-twotheta/2.0)*0.5)/sin(twotheta/4.))*lambda_end ; max lam needed for divergent case  [bob 13/4/15]
          lam_min_p = (sin((tth_min-twotheta/2.0)*0.5)/sin(twotheta/4.))*lambda_ini ; min lam needed for divergent case  [bob 13/4/15]
          cosmos_logmessage, 'Divergent case Lam max. ', /WARNING
        endelse

        ;print,lambda_ini,lambda_end,lam_min_p,lam_max_p
  

        aa = floor((lam_min_p-lambda[0])/b_lam)  ; wavelength bin of the lowest wavelength needed for lammin
        bb = floor((lam_max_p-lambda[0])/b_lam)  ; wavelength bin of highest wavelength needed for lammax

        cosmos_logmessage, 'Coherent analysis - tth_min, tth_max, ... = ' + cosmos_str_make(tth_min) + ' ' + $
          cosmos_str_make(tth_max) + ' ' + cosmos_str_make(b_tth) + ' ' + cosmos_str_make(peakref) + ' ' +  $
          cosmos_str_make(fgdref[0]) + ' ' + cosmos_str_make(fgdref[1])
        cosmos_logmessage, 'Coherent analysis - lamt, lamb = ' + cosmos_str_make(lam_max_p) + ' ' + $
          cosmos_str_make(lam_min_p) + ' ' + cosmos_str_make(lambda[0]) + ' ' + cosmos_str_make(b_lam) + ' ' + $
          cosmos_str_make(lambda_end) + ' ' + cosmos_str_make(lambda_ini) + ' ' + cosmos_str_make(twotheta)
        cosmos_logmessage, 'Coherent analysis - foreground pixels: ' + cosmos_str_make(fgdref[0]) + '-' + cosmos_str_make(fgdref[1])
        cosmos_logmessage, 'Coherent analysis - aa, bb = ' + cosmos_str_make(aa) + ' ' + cosmos_str_make(bb)

        refBeam2D = cntrefi
        cntrefitemp = cntrefi ;PG on 12/03/15: Need the "raw" counts for the 0 count error correction at the end
        refBeam2D_err = cntrefi_err
        counter = fltarr(n_elements(refBeam2D[0,*]))
        cntrefi = fltarr(n_elements(refBeam2D[0,*]))
        cntrefi_err = fltarr(n_elements(refBeam2D[0,*]))
        ss = n_elements(refBeam2D[0,*])-1

        detres_a = detres/parref.sdetd

        for i = fgdref[0], fgdref[1] do begin

           ; divide by DB[lambda] - Needs to be done now, before mixing constant-Q pixels (with different 2theta/lambda)

           refBeam2D_err[i,*] = sqrt( (refBeam2D_err[i,*]/cntdiri)^2 + $
              (refBeam2D[i,*]*cntdiri_err/cntdiri^2)^2 )  ;PG on 11/03/15 error for DB has to be included and it has to happen before the DB normalization
           
           refBeam2D[i,*] = refBeam2D[i,*]/cntdiri

           for j = aa, bb do begin

              if j ge 0 and j le ss then begin
                 
                 lam = lambda[j] ;wavelength at pixel i,j
                 twotheta_arr = fltarr(n_elements(refBeam2D[0,*]))	;PG on 27/07/15 corrected theta needed in case of FIGARO:
                 if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then twotheta_arr=2*newtheta*!dtor else twotheta_arr[*]=twotheta

                 if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) and parref.refdown eq 0  then begin
                   ; if figaro and reflecting up need to change signs
                   gama = -1.0*float(peakref-i)*b_tth          ;2th shift from specular
                 endif else begin ;for d17 and reflecting down on figaro
                   gama = float(peakref-i)*b_tth          ;2th shift from specular
                 endelse
                 ;TS 2015-06-01 ((detres/(2*parref.sdetd)) changed to ((detres/(parref.sdetd))
                 ;since the comparison to S2 does not need the factor of 2 in the detector resolution
                 ;otherwise the slope is calculated wrong in the second angle
                 if ((om_fwhm gt 0) AND ((detres/(parref.sdetd)) gt s2_fwhm)) then begin
                   lam_bot = sin((twotheta_arr[j])*0.5)*(lam-0.5*b_lam)/(sin((twotheta_arr[j]+gama+b_tth/2.)*0.5)) ;bot lam  ;specular wavelength at same qz  ;PG on 16/02/15: only valid for bent sample and collimated incoming beam
                   lam_top = sin((twotheta_arr[j])*0.5)*(lam+0.5*b_lam)/(sin((twotheta_arr[j]+gama-b_tth/2.)*0.5)) ;top lam  ;specular wavelength at same qz  ;PG on 16/02/15: only valid for bent sample and collimated incoming beam
                   ;if j eq 300 then cosmos_logmessage, 'Bent sample coherent lam_bot + lam_top calculation ', /WARNING
                   ;if j eq 400 then print, 'bent'
                 endif else begin
                   lam_bot = sin((twotheta_arr[j]+gama)*0.5)*(lam-0.5*b_lam)/(sin((twotheta_arr[j]+3.*gama+b_tth)*0.5))  ;For divergent incoming beam; PG on 28/04/15: b_tth should not be halfed here (see paper)
                   lam_top = sin((twotheta_arr[j]+gama)*0.5)*(lam+0.5*b_lam)/(sin((twotheta_arr[j]+3.*gama-b_tth)*0.5))
                   ;if j eq 300 then cosmos_logmessage, 'Divergent sample coherent lam_bot + lam_top calculation ', /WARNING
                   ;cosmos_logmessage, 'Divergent beam coherent lambda calculation ', /WARNING
                   ;if j eq 400 then print, 'div'
                   ;if j eq 500 then print,i,peakref,lam,lam_bot,lam_top,gamma*180./!pi,twotheta*180./!pi
                   ;these are lines of constant q with the origin shifted by gamma
                 endelse
                 ;calculate the limits in lambda on the chosen 2th line in pixels
                 newcb = floor(((lam_bot-lambda[0])/b_lam)+0.5)
                 newct = floor(((lam_top-lambda[0])/b_lam)+0.5)

                 bins = newct-newcb+1

                 ;loop over all the specular bins that fit the pixel width and share counts between them
                 for k = newcb, newct do begin
                   if (newcb ge 0) and (newct le ss) then begin ;[bob]  ss is the largest time channel number- normally 999
                     cntrefi[k] = cntrefi[k] + refBeam2D[i,j]/float(bins)
                     cntrefi_err[k] = sqrt(cntrefi_err[k]^2 + (refBeam2D_err[i,j])^2/float(bins) )
                     counter[k] = counter[k]+1
                   endif
                 endfor

              endif ;end if j ge 0 and j le ss

           endfor ;end for j = aa, bb

        endfor ;end for i = fgdref[0], fgdref[1]
        
        if grouping eq 0 then begin
          temp = where(cntrefi_err eq 0)
          if (temp[0] ne -1) then cntrefi_err[temp]=sqrt( (1/cntdiri[temp])^2 + $
            (cntrefitemp[temp]*cntdiri_err[temp]/cntdiri[temp]^2)^2 ) ;PG on 12/03/15: In case of 0 counts the statistical count error is 1
        endif
  
     endif else begin ;incoherent case --> just sum reflected beam over foreground pixels
          
        cntrefi = total(cntrefi[fgdref[0] : fgdref[1], *], 1)
        cntrefi_err = sqrt(total(cntrefi_err[fgdref[0] : fgdref[1], *]^2, 1))
        if grouping eq 0 then begin ;PG on 11/03/15:Only needed if no grouping is applied
           temp = where(cntrefi_err eq 0)
           if (temp[0] ne -1) then cntrefi_err[temp]=1 ;PG: In case of 0 counts the statistical count error is 1
        endif
     endelse
     ; Care: At this point the content of cntrefi is different depending on the calculation method
     ; Coherent: cntrefi[lambda] = Reflected_Beam / Direct_Beam
     ; Incoherent: cntrefi[lambda] = Reflected_Beam
  
     ; Calculate Q and reflectivity
     if result_size gt 0 then begin
        
  	    ;Calculate resulting lambda resolution
        ;PG on 7/8/17: added wavelength dependent opening
  	    chop_res = (c_params.chopsep + (c_params.planckperkg * (parref.openangle+openofflam[lambda_range]) * parref.period / $
  	       (360. * lambda[lambda_range] * 1.e-10))) / (2 * parref.tofd) ;0.5*max chopper lambda resolution
  
        ;chop_res = (c_params.chopsep + (c_params.planckperkg * parref.openangle * parref.period / $
  	     ;  (360. * lambda[lambda_range] * 1.e-10))) / (2 * parref.tofd) ;0.5*max chopper lambda resolution
  
  	    det_res = c_params.planckperkg * parref.channelwidth / $;calculate max detector lambda resolution
  	       ( lambda[lambda_range] * 1.e-10) / (2.0 * parref.tofd)
  
  	    err_res = 0.98 * (3*chop_res^2 + det_res^2 + 3*chop_res*det_res)/(2*chop_res + det_res) ;FWHM of resulting trapezoid
  	    
  	    
  	    ;PG on 07/08/2017: Added wavelength smearing due to finite beam size at chopper 1 position:
  	    tempratio=(parref.tofd-sdr)*1000/c_params.interslit
  	    tempa=tempratio*abs(parref.s2w-parref.s3w)+parref.s2w
  	    tempb=tempratio*(parref.s2w+parref.s3w)+parref.s2w
  	    tempwidthfwhm=0.49*(tempb^3-tempa^3)/(tempb^2-tempa^2)

  	    width_res = tempwidthfwhm/1000.0 * parref.period / (2 * !pi * c_params.cr)* c_params.planckperkg / $;calculate beam width lambda resolution
           ( lambda[lambda_range] * 1.e-10) / parref.tofd
         err_res = sqrt(err_res^2+width_res^2)
  
  	    ;s3_fwhm=(0.68*parref.s3w)/c_params.interslit  ; fwhm angles of second slit  PG on 16/02/15: corrected for interslit distance
		    s3_fwhm=(0.68*parref.s3w)/(parref.slit3*1000.0+parref.sdetd*1000.0)  ; fwhm angles of second slit  PG on 28/04/15: corrected for interslit distance
  	    ;print,'s2 s3= ',s2_fwhm*180./!pi,s3_fwhm*180./!pi


        ;Resolution calculation (different depending on coherent/incoherent treatment)
        if calc_method eq 'coherent' then begin
           if om_fwhm gt 0 then begin
            ;TS 2015-06-01: corrected da for S3-det distance
            ;;TS 2015-06-01: distinguish between cases of divergence larger or smaller than bendiness of sample 
            if s2_fwhm ge 2*om_fwhm then err_ray = sqrt(  (detres/(parref.slit3+parref.sdetd))^2 + s3_fwhm^2   + om_fwhm^2  )/angle_bragg  ;PG on 16/2/15: I don;t think this is correct
            if s2_fwhm ge 2*om_fwhm then cosmos_logmessage, 'Bent sample resolution with s2fwhm>2*om_fwhm ', /WARNING ;2018-06-06 TS
            if s2_fwhm lt 2*om_fwhm then err_ray = sqrt(  (detres/(2*(parref.slit3+parref.sdetd)))^2 + s3_fwhm^2   + s2_fwhm^2  )/angle_bragg  ;PG on 16/2/15: I don;t think this is correct 
            if s2_fwhm lt 2*om_fwhm then cosmos_logmessage, 'Bent sample resolution with s2fwhm<2*om_fwhm ', /WARNING ;2018-06-06 TS
            ;cosmos_logmessage, 'detres ' + cosmos_str_make(detres), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'slit3 ' + cosmos_str_make(parref.slit3), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'sdet ' + cosmos_str_make(parref.sdetd), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 's2w ' + cosmos_str_make(parref.s2w), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 's3w ' + cosmos_str_make(parref.s3w), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'interslit ' + cosmos_str_make(c_params.interslit), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'om_fwhm ' + cosmos_str_make(om_fwhm), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, '2x om_fwhm ' + cosmos_str_make(2*om_fwhm), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 's2_fwhm ' + cosmos_str_make(s2_fwhm), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 's3_fwhm ' + cosmos_str_make(s3_fwhm), /WARNING ;2018-06-06 TS for debugging
            ;tester=detres/(2*(parref.slit3+parref.sdetd)) ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'detr ' + cosmos_str_make(tester), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'da ' + cosmos_str_make(da*180/!pi), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'angle_bragg ' + cosmos_str_make(angle_bragg), /WARNING ;2018-06-06 TS for debugging
            ;cosmos_logmessage, 'n err_ray ' + cosmos_str_make(err_ray), /WARNING ;2018-06-06 TS for debugging
            ;err_ray = sqrt((detres/(2*parref.sdetd))^2 + s3_fwhm^2+s2_fwhm^2)/angle_bragg  ;This is the case for a bent sample
           endif else begin
              if s2_fwhm gt (detres/parref.sdetd) then begin
                 err_ray = sqrt((detres/(parref.slit3+parref.sdetd))^2 + s3_fwhm^2)/angle_bragg  ;This is the case for a flat sample and a divergent incoming beam
                 cosmos_logmessage, 'Divergent beam resolution ', /WARNING
              endif else begin
                 cosmos_logmessage, 'Constant q foreground COSMOS for angle ' + cosmos_str_make(runno) + ' is not appropriate!', /WARNING  ;This is the case were the detector resolution is worse than the incoming divergence for a flat sample
                 err_ray = sqrt(da^2 + (detres/(parref.slit3+parref.sdetd))^2)/angle_bragg
              endelse
           endelse
           
        endif else begin ;incoherent case
          
           ;TS 2016-10-25 Deleted om_fwhm^2 (only for polarized case) since this messes up in comparison between 
           ;different runs when bendiness may slightly change. Only for polarized case.
           ;if pol_run then err_ray = sqrt(da^2)/angle_bragg else err_ray = sqrt(da^2+om_fwhm^2)/angle_bragg
           ;TS 2018-06-04 removed restriction for polarized data to ignore om_fwhm
           err_ray = sqrt(da^2+om_fwhm^2)/angle_bragg
           if pol_run then begin
            cosmos_logmessage, ' '
            cosmos_logmessage, ' POLARIZED RUN: Make sure resolution calculation is the same, if not, check grouping factors are the same.'
            cosmos_logmessage, ' '
           endif
           ; see paper - PG on 16/2/2015 decided to not use the sample bendiness in the incoherent case
           ;print, om_fwhm
           if om_fwhm gt 0 then begin
                if ((abs(parref.s2w-pardir.s2w) lt 0.04) and (abs(parref.s3w-pardir.s3w) lt 0.04)) then cosmos_logmessage, 'The sample is bent or there is a lot of diffuse scattering for angle ' + cosmos_str_make(runno) + '!'	;PG on 28/04/15
           endif
           ; PG: In case foreground width is smaller than incoming angular resolution
           err_ray_temp = 0.68*sqrt((((total(foregdref)+1)*c_params.pixelwidth)^2 + (parref.s3w/1000)^2)/parref.sdetd^2)/angle_bragg ;PG on 16/2/2015
           ; PG: Choose the smaller of the two angular resolutions
           err_ray=min([err_ray,err_ray_temp])

        endelse ;end of resolution calculation (coherent/incoherent)
        
        totf = fgdref[1]-fgdref[0]
        cosmos_logmessage, ' '
        cosmos_logmessage, ' Constants used in calculating wavelength resolution :'
        cosmos_logmessage, '  constants.chopsep = ' + cosmos_str_make(c_params.chopsep)
        cosmos_logmessage, '  constants.planckperkg = ' + cosmos_str_make(c_params.planckperkg)
        cosmos_logmessage, '  parref.openangle = ' + cosmos_str_make(parref.openangle)
        cosmos_logmessage, '  parref.period = ' + cosmos_str_make(parref.period)
        cosmos_logmessage, '  parref.tofd = ' + cosmos_str_make(parref.tofd)
        cosmos_logmessage, '  constants.chopsep / tofd= ' + cosmos_str_make(c_params.chopsep/parref.tofd)
        cosmos_logmessage, '  err_res (min) = ' + cosmos_str_make(min(err_res))
        cosmos_logmessage, '  err_res (max) = ' + cosmos_str_make(max(err_res))
        cosmos_logmessage, ' '
        cosmos_logmessage, ' Constants used in calculating angular resolution :'
        cosmos_logmessage, '  parref.slits = ' + cosmos_str_make(parref.s2w)+'  '+ cosmos_str_make(parref.s3w)
        if normal_run then cosmos_logmessage, '  fwhm db (mm) ref (mm) = ' + cosmos_str_make(detdb_fwhm*1000.)+'  '+ cosmos_str_make(det_fwhm*1000.)
        cosmos_logmessage, '  constants.interslit = ' + cosmos_str_make(c_params.interslit)
        cosmos_logmessage, '  sample waviness (fwhm) = ' + cosmos_str_make(om_fwhm_real*180./!pi) + 'deg, incoming angular divergence (fwhm) = ' + cosmos_str_make(da*180/!pi) + 'deg'
        cosmos_logmessage, '  angle_bragg = ' + cosmos_str_make(angle_bragg) + ' rad = ' + cosmos_str_make(angle_bragg*180./3.1416) + 'deg'
        ;cosmos_logmessage, '  twotheta of chosen pixel line = ' + cosmos_str_make(twotheta*180./3.1416) + 'deg'
        cosmos_logmessage, '  err_ray = ' + cosmos_str_make(err_ray)
        cosmos_logmessage, ' '
  
        result = fltarr(5, result_size) ;[Q, dQ, I, dI, lambda]
  
        if (strcmp(strtrim(c_params.inst,2), 'figa', /FOLD_CASE)) then $
              result[0,*] = 4. * !pi * sin(newtheta[lambda_range]*!dtor) / lambda[lambda_range] $
        else  result[0,*] = 4. * !pi * sin(angle_bragg) / lambda[lambda_range]
    
        ; Propagation of Gaussian error
        result[1,*] = result[0,*] * sqrt(err_res^2 + err_ray^2)
       
        ; Wavelength
        result[4,*] = lambda[lambda_range]
    
        if normal_run then begin
            
           if max(cntdiri[lambda_range]) le 0. then begin
            
              result[2, *] = cntrefi[lambda_range]
              result[3, *] = cntrefi_err[lambda_range]
              if grouping eq 1 then groupedresult = cosmos_anal_group(result, grouping_value) else groupedresult = result
  
           endif else begin
  
              ;If coherent treatment, result = RB/DB already
              if calc_method eq 'coherent' then result[2, *] = cntrefi[lambda_range] $             
                 else result[2, *] = cntrefi[lambda_range] / cntdiri[lambda_range]
             
              ;PG on 7/8/17: correction if the chopper opening is different for the reflected and direct beams
              result[2, *] = result[2, *] $
                 * (c_params.chopsep / c_params.planckperkg / pardir.period * lambda[lambda_range] * 1.e-10 + (pardir.openangle+openofflam[lambda_range]) / 360. ) $
                 / (c_params.chopsep / c_params.planckperkg / parref.period * lambda[lambda_range] * 1.e-10 + (parref.openangle+openofflam[lambda_range]) / 360. )	
          
              ;MAG 08.07.09: Error calculated as square root of variance = sqrt [ (dR/D)^2 + (-R*dD/D^2)^2 ], where
              ; R is the reflected beam and D the direct beam, and dR and dD their corresponding errors.
              if calc_method eq 'coherent' then result[3,*] = cntrefi_err[lambda_range] $
                else result[3, *] = sqrt( (cntrefi_err[lambda_range]/cntdiri[lambda_range])^2 + $
                                          (cntdiri_err[lambda_range]*cntrefi[lambda_range]/cntdiri[lambda_range]^2)^2 )
        	   
              ;PG on 7/8/17: correction if the chopper opening is different for the reflected and direct beams
        	    result[3, *] = result[3, *] $
        	       * (c_params.chopsep / c_params.planckperkg / pardir.period * lambda[lambda_range] * 1.e-10 + (pardir.openangle+openofflam[lambda_range]) / 360. )$
                 / (c_params.chopsep / c_params.planckperkg / parref.period * lambda[lambda_range] * 1.e-10 + (parref.openangle+openofflam[lambda_range]) / 360. )	
 
              ;If requested, group data and store them in groupedresult
              if grouping eq 1 then begin

                 ;for polarization data, need to ensure that Q, dQ, lambda are the same for all
                 ;the polarized states, otherwise the grouping will give a different number of bins
                 if store_polparams then begin
                   pol_lambda = fltarr(3, result_size)
                   pol_lambda[0:1,*] = result[0:1,*]
                   pol_lambda[2,*] = result[4,*]
                 endif

                 if n_elements(pol_lambda) gt 0 then begin ;replace Q, dQ, lambda by reference values
                   if n_elements(result[0,*]) eq n_elements(pol_lambda[0,*]) then begin
                     result[0:1,*] = pol_lambda[0:1,*]
                     result[4,*] = pol_lambda[2,*]
                   endif else begin
                     cosmos_logmessage, 'Reference Q values cannot be used. Different number of points!', /warning
                     cosmos_logmessage, 'Try to slightly adjust the Min/Max Lambda', /warning ;TS 2016-10-21 Had a problem that the wavelength is sometimes one point longer...
                     ;print, 'lambda length problem'
                   endelse
                 endif

                 if calc_method eq 'coherent' then begin
                    groupedresult = cosmos_anal_group(result, grouping_value)
                 endif else begin   
                    groupeddir = fltarr(5, result_size)
                    groupeddir[0:1, *] = result[0:1, *]
                    groupeddir[2, *] = cntdiri[lambda_range]
                    groupeddir[3, *] = cntdiri_err[lambda_range]
                    groupeddir[4, *] = lambda[lambda_range]
                    groupeddir = cosmos_anal_group(groupeddir, grouping_value)
                    groupedref = fltarr(5, result_size)
                    groupedref[0:1, *] = result[0:1, *]
                    groupedref[2, *] = cntrefi[lambda_range] $
                       * (c_params.chopsep / c_params.planckperkg / pardir.period * lambda[lambda_range] * 1.e-10 + pardir.openangle / 360. ) $
                       / (c_params.chopsep / c_params.planckperkg / parref.period * lambda[lambda_range] * 1.e-10 + parref.openangle / 360. )	
                    groupedref[3, *] = cntrefi_err[lambda_range] $
                       * (c_params.chopsep / c_params.planckperkg / pardir.period * lambda[lambda_range] * 1.e-10 + pardir.openangle / 360. ) $
                       /(c_params.chopsep / c_params.planckperkg / parref.period * lambda[lambda_range] * 1.e-10 + parref.openangle / 360. )	 
                    groupedref[4, *] = lambda[lambda_range]
                    groupedref = cosmos_anal_group(groupedref, grouping_value)
                    groupedresult = groupedref
                    groupedresult[2, *] = groupedref[2, *] / groupeddir[2, *]
                    groupedresult[3, *] = sqrt( (groupedref[3, *]/groupeddir[2,*])^2 + $
                                                (groupeddir[3,*]*groupedref[2,*]/groupeddir[2,*]^2)^2 )
                 endelse 
                 
              endif else groupedresult = result ;no grouping 
            
           endelse
           
           ;PG on 11/03/15: In case of 0 counts the statistical count error is 1
           temp = where(groupedresult[3,*] eq 0)
           if (temp[0] ne -1) then groupedresult[3,temp] = 1.0   

        endif else begin ;nodirect 
  
           ;Calculate corresponding wavelengths for monitor 2
           monitor_tofd = parref.tofm
           monitor_beam = fltarr(tsize)
        
           monitor_beam = monref[1,*]
        
           monitor_lambda = fltarr(tsize)
           monitor_lambda = 1e10 * (c_params.planckperkg * ((findgen(tsize) + 0.5) * parref.channelwidth + parref.delay) / monitor_tofd)
      
           ;Interpolate monitor_beam to wavelengths measured in detector
           monitor_beam2 = fltarr(tsize)
           monitor_beam2 = interpol(monitor_beam,monitor_lambda,lambda)
    
           a = 5000.0    ;from visual fit to #136565-136567 (cycle 093, using re-gassed monitor)
           b = 0.4
           c = 4000.
           d = 0.15
           eff = ( a * exp(-b*lambda[lambda_range]) + c * exp(-d*lambda[lambda_range]) )
           direct_beam = monitor_beam2[lambda_range] * eff
           err_direct_beam = sqrt(monitor_beam2[lambda_range]) * eff
    
           result[2,*] = cntrefi[lambda_range] / (direct_beam > 1)
  
           ;MAG 08.07.09: Error calculated as square root of variance = sqrt [ (dR/D)^2 + (-R*dD/D^2)^2 ], where
           ; R is the reflected beam and D the direct beam, and dR and dD their corresponding errors.
           result[3,*] = sqrt((cntrefi_err[lambda_range]/(direct_beam>1))^2 + (err_direct_beam*cntrefi[lambda_range]/(direct_beam>1)^2)^2 )
      
           if grouping eq 1 then begin
              groupeddir = fltarr(5, result_size)
              groupeddir[0:1, *] = result[0:1, *]
              groupeddir[2, *] = direct_beam > 1
              groupeddir[3, *] = err_direct_beam
              groupeddir[4, *] = lambda[lambda_range]
              groupeddir = cosmos_anal_group(groupeddir, grouping_value)
              groupedref = fltarr(5, result_size)
              groupedref[0:1, *] = result[0:1, *]
              groupedref[2, *] = cntrefi[lambda_range]
              groupedref[3, *] = cntrefi_err[lambda_range]
              groupedref[4, *] = lambda[lambda_range]
              groupedref = cosmos_anal_group(groupedref, grouping_value)
              groupedresult = groupedref
              groupedresult[2, *] = groupedref[2, *] / groupeddir[2, *]
              groupedresult[3, *] = sqrt( (groupedref[3, *]/groupeddir[2,*])^2 + (groupeddir[3,*]*groupedref[2,*]/groupeddir[2,*]^2)^2 )
           endif else groupedresult = result
    
        endelse ;end if normal_run ... else nodirect ...
      
        if islice eq 0 then keeppoints = where((finite(result[2, *]) eq 1) and (finite(result[3, *]) eq 1), nbkeeppoints)
        if nbkeeppoints gt 0 then result = result[*, keeppoints] else result = -1

     endif else result = -1 ; end if result_size gt 0
     
     if result_size eq 0 then begin
       cosmos_logmessage, 'Empty array returned as result!', /WARNING
     endif else begin
       if islice eq 0 then begin
          result_slices = fltarr(5, nbkeeppoints, slices)
          groupedsize = size(groupedresult)
          groupedresult_slices = fltarr(5, groupedsize[2], slices)
       endif 
       result_slices[*,*,islice] = result
       groupedresult_slices[*,*,islice] = groupedresult
     endelse
     
  endfor ;end of loop over slices

  if slices eq 1 then begin
     result = result_slices[*,*,0]
     groupedresult = groupedresult_slices[*,*,0]
  endif else begin
     result = result_slices
     groupedresult = groupedresult_slices
  endelse

  return
  
end

; sort dataset [ x, err_x, y, err_y ] by x value
function cosmos_anal_sort, dataset

  result = dataset[*, sort(dataset[0, *])]
  return, result
  
end

; normalise 1D array
function cosmos_array_norm, array

  return, array / (mean(array))
  
end

; read direct beam files, using previously read values if any
pro cosmos_direct_read, newfiles, mondir, cntdir, pardir, runno

  common cosmos_direct, direct1_files, direct2_files, direct3_files, direct1_monitor, direct2_monitor, $
         direct3_monitor, direct1_array, direct2_array, direct3_array, direct1_params, direct2_params, $
         direct3_params
         
  common cosmos_settings, c_settings
    
  case runno of
    1 : temp2 = direct1_files
    2 : temp2 = direct2_files
    3 : temp2 = direct3_files
  endcase
  
  ; compare existing direct files
  temp1 = where(newfiles ne temp2, count)
     
  ; read new direct files
  if count gt 0 then begin
    
    cosmos_raw_read, newfiles, monitors, counts, params, c_settings.pathdir, c_settings.override
    
    case runno of
      1 : begin
        direct1_files = newfiles
        direct1_monitor = monitors
        direct1_array = counts
        direct1_params = params
      end
      2 : begin
        direct2_files = newfiles
        direct2_monitor = monitors
        direct2_array = counts
        direct2_params = params
      end
      3 : begin
        direct3_files = newfiles
        direct3_monitor = monitors
        direct3_array = counts
        direct3_params = params
      end
    endcase
  endif
  
  ; return appropriate values
  case runno of
    1 : begin
      mondir = direct1_monitor
      cntdir = direct1_array
      pardir = direct1_params
    end
    2 : begin
      mondir = direct2_monitor
      cntdir = direct2_array
      pardir = direct2_params
    end
    3 : begin
      mondir = direct3_monitor
      cntdir = direct3_array
      pardir = direct3_params
    end
  endcase
  
  return
  
end

;+
; :Description:
;    Interprets a string and returns a string array of numors.
;    The input string should be formatted as follows: An individual numors can 
;    be given alone or more than one numor can be given separated by a plus sign (+).
;    A range of numors can be specified by givin the first and last numors separated 
;    by a comma (,) or a larger than sign (>).
;    For example, the string '1+5+10,13+20' will return the following array of numors: 
;    [1,5,10,11,12,13,20].
;
;    27/4/2015: Added 'p' delimiter to separate numors measured with different polarizations.
;    Use '1+2+3 p 4 p 6>10 p 11+15>20' to read the numors with 4 different polarizations
;    (1,2,3: R00; 4: R10; 6,7,8,9,10: R01; and 11,15,16,17,18,19,20: R11).
;
; :Params:
;    intlist : in, required, type="String"
;       Contains the string to interpret.
;
; :Returns:
;     A string array containing the list of numors corresponding to the input or
;     an empty string if the input is wrong or a list composed by string arrays
;     if polarized data are read (p delimiter)
;
; :Author: M. A. Gonzalez
;
; 29 April 2015
;-
function cosmos_getnumors, intlist

   common cosmos_params, c_params

   intlist = strcompress(intlist, /remove_all) ;remove all spaces
   
   ;if cell is empty, return empty string
   if intlist eq '' then return, ''

   ;split string and identify position and length of each item
   item_pos = strsplit(intlist, '+,>pP', count=number_items, length=item_length)
   
   ;check if all items are valid numbers
   on_ioerror, conversion_error
   for i = 0, number_items-1 do begin
       tmp = long(strmid(intlist, item_pos[i], item_length[i]))   ;item is an integer
       if tmp gt 999999 then goto, conversion_error               ;numor has only 6 numbers
   endfor

   ;create array to contain the list of numors with first item
   temp_list = [string(strmid(intlist, item_pos[0], item_length[0]), format='(I06)')]

   if number_items gt 1 then begin ;check delimiter and create numor_list array accordingly
       for i = 0, number_items-2 do begin
           delimiter = strmid(intlist, item_pos[i]+item_length[i], 1)
           case 1 of
               delimiter eq '+': temp_list = [temp_list, string(strmid(intlist, item_pos[i+1], item_length[i+1]), format='(I06)')]
               delimiter eq ',' or delimiter eq '>': begin
                   first_numor = long(strmid(intlist, item_pos[i], item_length[i])) + 1
                   last_numor  = long(strmid(intlist, item_pos[i+1], item_length[i+1]))
                   for numor = first_numor, last_numor do temp_list = [temp_list, string(numor, format='(I06)')]
                end
                delimiter eq 'p' or delimiter eq 'P': begin
                  if n_elements(numor_list) eq 0 then begin
                    numor_list = list(temp_list)
                    temp_list = [string(strmid(intlist, item_pos[i+1], item_length[i+1]), format='(I06)')]
                  endif else begin
                    numor_list.Add, temp_list
                    temp_list = [string(strmid(intlist, item_pos[i+1], item_length[i+1]), format='(I06)')]
                  endelse
                end
           else: begin
               cosmos_logmessage, 'Unknown delimiter. Returning empty string!', /WARNING
               return, ''
               end
           endcase
       endfor
    
   endif 

   if n_elements(numor_list) eq 0 then numor_list = temp_list else numor_list.Add, temp_list
   
   return, numor_list
   
   conversion_error: 
       cosmos_logmessage, 'Non valid numor number. Returning empty string!', /WARNING
       return, ''
       
end


pro cosmos_grav,theta0,lambda,newtheta,delta,dz,poffoff,openofflam,xchopper,xslits3,xslits2,refdown

;macro when given straight line theta (theta0 (deg)), wavelength (lambda) and sam-midchop 
; distance xchopper(m) calculates
; true theta (newtheta (deg)), the displacement for the beam along the sample (delta (m))
; the drop of the beam at the sample centre (dz (m)) the offset in poff (poffoff (deg))
; and the offset in the opening (openofflam (deg))

;choppers turn clockwise from the reactor by default so poff=poff-poffoff
; opening offset only changes the apparent opening and does NOT make a time delay
; the normal delay associated with opening the chopper comes from changing the phase of
; the second controling chopper

g = 9.81       ;gravity

cr = 305.e-3;c_params.cr  ;PG on 07/08/2017 read global value 305.e-3   ;chopper radius to the middle of guide
yslits3 = xslits3 * tan(theta0) ;x1*tan(theta) in the paper
yslits2 = xslits2 * tan(theta0)


;origin is the centre of the sample

v = 3956./lambda  ;neutron velocity
k = g/(2.*v^2) ;a characteristic inverse length


;define parabola y = y0 - k * (x-x0)^2 passing by (xslits3,yslits3) and (xslits2,yslits2)
x0 = ( k*(xslits2^2-xslits3^2) + (yslits2-yslits3) ) / (2*k*(xslits2-xslits3))
y0 = yslits3 + k * (xslits3-x0)^2

delta = x0 - sqrt(y0/k)   ; shift in x along sample due to gravity
dz = y0 - k*x0^2          ; if beam carrys on to x=0 what height would it be below sample


grad = (2.*k)*(x0-delta)  ;gradient of parabola ( to find true theta at y=0)
newtheta = atan(grad)


chopz = xchopper*tan(theta0) - (y0-k*(xchopper-x0)^2)
poffoff = (chopz/cr) * !radeg
gradc = (2.*k)*(x0-xchopper)  ;gradient at the midchopper distance
openofflam = (atan(gradc)-theta0) * !radeg


deltath = newtheta - theta0 ;difference between nominal angle theta0 and effective incident angle (in rad)
if (refdown eq 0) then begin
   ;reflection from upwards
   newtheta = newtheta * !radeg ;effective incident angle in deg
endif else begin
   ;reflection from downwards
   newtheta = (theta0 - deltath) * !radeg ;effective incident angle in deg
  endelse
print,'newtheta',(theta0* !radeg),newtheta
print,'poffoff',poffoff
print,'openofflam',openofflam

end

pro cosmos_gravnew,theta0,lambda,newtheta,delta,dz,poffoff,openofflam,xchopper,xslits3,xslits2,refdown,cht,cr

  ;macro when given straight line theta (theta0 (deg)), wavelength (lambda) and sam-midchop
  ; distance xchopper(m) calculates
  ; true theta (newtheta (deg)), the displacement for the beam along the sample (delta (m))
  ; the drop of the beam at the sample centre (dz (m)) the offset in poff (poffoff (deg))
  ; and the offset in the opening (openofflam (deg))

  ;choppers turn clockwise from the reactor by default so poff=poff-poffoff
  ; opening offset only changes the apparent opening and does NOT make a time delay
  ; the normal delay associated with opening the chopper comes from changing the phase of
  ; the second controling chopper

  g = 9.81       ;gravity

  if (refdown ne 0) then theta0 = -1.0 * theta0
  yslits3 = xslits3 * tan(theta0) ;x1*tan(theta) in the paper
  yslits2 = xslits2 * tan(theta0)


  ;origin is the centre of the sample

  v = 3956./lambda  ;neutron velocity
  k = g/(2.*v^2) ;a characteristic inverse length


  ;define parabola y = y0 - k * (x-x0)^2 passing by (xslits3,yslits3) and (xslits2,yslits2)
  x0 = ( k*(xslits2^2-xslits3^2) + (yslits2-yslits3) ) / (2*k*(xslits2-xslits3))
  y0 = yslits3 + k * (xslits3-x0)^2

  delta = x0 - sqrt(y0/k)   ; shift in x along sample due to gravity
   if (refdown ne 0) then delta = x0 + sqrt(y0/k)
  dz = y0 - k*x0^2          ; if beam carrys on to x=0 what height would it be below sample


  grad = (2.*k)*(x0-delta)  ;gradient of parabola ( to find true theta at y=0)
  newtheta = atan(grad)


  chopz = xchopper*tan(theta0) - (y0-k*(xchopper-x0)^2)
  poffoff = (chopz/cr) * !radeg

  gradc = (2.*k)*(x0-xchopper)  ;gradient at the midchopper distance
 ; openofflam = (atan(gradc)-theta0) * !radeg
  openofflam = cht*(gradc-tan(theta0))/cr * !radeg


  if (refdown ne 0) then begin
    newtheta = -1.0 * newtheta * !radeg
    theta0 = -1.0 * theta0
 endif else newtheta = newtheta * !radeg
  
;  print,'newtheta',(theta0* !radeg),newtheta
; print,'poffoff',poffoff
; print,'openofflam',openofflam
  
end


pro cosmos_init_detector

  common cosmos_params, c_params
  
  common cosmos_water, water_file, water_array, water_lowerlimit
  
  common cosmos_direct, direct1_files, direct2_files, direct3_files, direct1_monitor, $
    direct2_monitor, direct3_monitor, direct1_array, direct2_array, direct3_array, $
    direct1_params, direct2_params, direct3_params
    
  water_array = make_array(c_params.pixels_x, /FLOAT, VALUE = 1.)
  direct1_array = lonarr(c_params.pixels_x, c_params.pixels_t)
  direct2_array = lonarr(c_params.pixels_x, c_params.pixels_t)
  direct3_array = lonarr(c_params.pixels_x, c_params.pixels_t)
  
end

function cosmos_get_machine_params, path, numor

  on_ioerror, bad_numor
    
  valid = 0
  
  file_name = string(numor, format = "(I06)")
  file_name = path + strtrim(file_name,2)

  raw_lun=-1
  openr, raw_lun, file_name, /get_lun, /compress

  tempstr = ''
          
  readf, raw_lun, tempstr ;RRRR line
  readf, raw_lun, tempstr ;Read detector format
  temp = strsplit(tempstr,' ',/extract)
  detector_format = temp[2]
  readf, raw_lun, tempstr ;AAAA line
  readf, raw_lun, tempstr ;number of ASCII characters in following line
  readf, raw_lun, tempstr ;Read instrument
  temp = strmid(tempstr,0,4)
  inst = strupcase(strtrim(temp,2))

  ; read in next lines of text until first block of parameters
  for i = 6, 34 do readf, raw_lun, tempstr
        
  ; read 1st set of parameters
  par1 = fltarr(128)
  readf, raw_lun, par1

  if inst eq 'D17' then begin
    spoff    = 0.0
    poff     = par1[54]
    d0       = par1[55]
    openoff  = par1[56]
    ; The pixel width are converted from mm to m.
    pixelwidth = par1[103]*1.0e-3   
    sdetd = par1[25]*1.0e-3   
    edelay = par1[96]*1.0e-6 ;TOF delay (in s)
    valid = 1
  endif else if inst eq 'FIGA' then begin
    spoff    = par1[23] * 1.0e-3
    poff     = par1[54]
    d0       = par1[55] * 1.0e-3
    openoff  = par1[56]
    ; The pixel width are converted from mm to m.
    pixelwidth = par1[104]*1.0e-3    
    sdetd = par1[18]*1.0e-3   
    edelay = (par1[96]+par1[93]) * 1.0e-6 ;TOF delay (in s) 26.4.13 MG added extra correction required for Figaro
    valid = 1
  endif else valid = 0
            
  bad_numor:
      if raw_lun gt 0 then free_lun, raw_lun
      if valid then return, [poff, d0, openoff, pixelwidth, sdetd, edelay, spoff] else return, -1
      
end

function sample_to_detector_distance, sht1, dh1, dh2, pixel, xdh1, xdh2, xdet, ydet, mmpy   
                 
    dh1A = dh1 - ydet
    dh2A = dh2 - ydet

    ;first calculate theta based on DH1 and DH2 positions
    theta = atan((dh2-dh1)/(xdh2-xdh1))

    ;calculate distances from DH1 bearing to point on the rail directly below detector wires
    ya = sin(theta)*(xdet-xdh1)
    xa = cos(theta)*(xdet-xdh1)

    ;define position of point on the rail below detector wires
    xdetrail = xa+xdh1
    ydetrail = ya+dh1a

    ;calculate distance from point on the rail below detector wires to beam position
    ypos = ydet-(128.0*mmpy)+(pixel*mmpy)

    ;calculate position of beam
    xbeam = xdetrail-(sin(theta)*ypos)
    ybeam = ydetrail+(cos(theta)*ypos)

    det = sqrt( xbeam^2 + (ybeam-sht1)^2)

    return, det

end

; reads from raw data files (specified by array of strings), returning
;  summed detector counts (int array (x, t))
;  parameters :
;   params.monitor : summed monitor counts (long) which is 0 if no files found
;   params.runtime : summed runtime (seconds) (float)
;   params.dan : detector angle (dan) (degrees) (float)
;   params.san : sample angle (san) (degrees) float)
;   params.size_x : number of pixels in x direction (long)
;   params.size_t : number of time channels (long)
;   params.x_min : lower limit of pixels used
;   params.x_max : upper limit of pixels used
;   params.tofd : time of flight distance (float)
;   params.delay : time delay (float)
;   params.channelwidth : channel width (float)
;   params.pixeldensity : pixel density
;   params.tra : translation of analyzer (for polarization measurements, only D17)
;   params.f1 : state of flipper 1 (for polarization measurements, only D17)
;   params.f2 : state of flipper 2 (for polarization measurements, only D17)

pro cosmos_raw_read, files, monitors, counts, params, path, override, reflect=reflect
  
  common cosmos_params, c_params
          
  flag_defined = 0
              
  testfile = files[n_elements(files)-1]
  testfile = string(testfile, FORMAT = "(I06)")
  testfile = path + path_sep() + strtrim(testfile,2)
  ok = file_test(testfile)

  if ok then begin
    
    for f = 0, n_elements(files) - 1 do begin    
        
        if strcompress(files[f]) gt ' ' then begin
    
            file_name = string(files[f], FORMAT = "(I06)")
          
            file_name = path + path_sep() + strtrim(file_name,2)
      
            on_ioerror, cosmos_raw_read_exit2
            raw_lun=-1
            openr, raw_lun, file_name, error = tempval, /get_lun, /compress
      
            if tempval eq 0 then begin
      
                cosmos_logmessage, ''
                cosmos_logmessage, ' Reading ' + file_name
                tempstr = ''
          
                readf, raw_lun, tempstr ;RRRR line
                readf, raw_lun, tempstr ;Read detector format
                temp = strsplit(tempstr,' ',/extract)
                detector_format = temp[2]

                c_params.data_format = detector_format
                readf, raw_lun, tempstr ;AAAA line
                readf, raw_lun, tempstr ;number of ASCII characters in following line
                readf, raw_lun, tempstr ;Read instrument

                temp = strmid(tempstr,0,4)
                c_params.inst = strtrim(temp,2)

                if (keyword_set(reflect)) then begin
                    ; read in next lines of text until first block of parameters
                    for i = 6, 25 do readf, raw_lun, tempstr

                    readf, raw_lun, tempstr ; The user, local contact ...

                    c_params.user = strmid(tempstr,0,6)
                    c_params.local_contact = strmid(tempstr,6,4)
                    c_params.title = strmid(tempstr,10,50)
                    c_params.subtitle = strmid(tempstr,60)

                    readf, raw_lun, tempstr ; The starting and finishing date and time

                    ;Check if start and end time need to be updated                    
                    if long(files[f]) lt c_params.min_refl_numor then begin
                       c_params.min_refl_numor = long(files[f])
                       c_params.start_time = strmid(tempstr,0,18)     
                    endif                 
                    if long(files[f]) gt c_params.max_refl_numor then begin
                       c_params.max_refl_numor = long(files[f])
                       c_params.end_time = strmid(tempstr,20,18)
                    endif   

                    ; read in next lines of text until first block of parameters
                    for i = 28,34 do readf, raw_lun, tempstr
            
                endif else begin
                    ; read in next lines of text until first block of parameters
                    for i = 6, 34 do readf, raw_lun, tempstr
                endelse
        
                ; read 1st set of parameters
                par1 = fltarr(128)
                readf, raw_lun, par1
        
                ;MAG: Reading machine options (if override not selected)
        
                if (override eq 0) then begin
        
                  if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
                    
                    ; poff
                    tmp = par1[54]            
                    if (tmp gt 0.0) then begin            
                      c_params.poff = tmp
                    endif else begin
                      cosmos_logmessage, 'No valid value for poff in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.poff), /WARNING
                    endelse
              
                    ; d0
                    tmp = par1[55]            
                    if (tmp gt 0.0) then begin            
                      ;distance chopper-sample written in m in D17 numor
                      c_params.d0 = tmp 
                    endif else begin            
                      cosmos_logmessage, 'No valid value for d0 in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.d0), /WARNING
                    endelse
              
                    ; openoff
                    tmp = par1[56]            
                    if (tmp ne 0.0) then begin
                      c_params.openoff = tmp
                    endif else begin            
                      cosmos_logmessage, 'No valid value for openoff in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.openoff), /WARNING              
                    endelse
              
                    ; pixelwidth
                    tmp = par1[103]
                    if (tmp gt 0.0) then begin            
                      ; The pixelwidth is converted from mm to m.
                      c_params.pixelwidth = tmp*1.0e-3
                    endif else begin            
                      cosmos_logmessage, 'No valid value for pixelwidth in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.pixelwidth), /WARNING              
                    endelse
              
                    ; sample to detector distance
                    tmp = par1[25]
                    if (tmp gt 0.0) then begin            
                      ; The sample to detector distance is converted from mm to m.
                      c_params.sdetd = tmp*1.0e-3
                    endif else begin            
                      cosmos_logmessage, 'No valid value for sample to detector distance in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.sdetd), /WARNING              
                    endelse
               
                    ; TOF delay
                    tmp = par1[96]
                    if (tmp gt 0.0) then begin            
                      ; The TOF delay is converted from microseconds to seconds
                      c_params.edelay = tmp*1.0e-6
                    endif else begin            
                      cosmos_logmessage, 'No valid value for TOF delay in numor ' + file_name, /WARNING
                      cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.edelay), /WARNING              
                    endelse
            
                  endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
                    
                    ; poff
                    tmp = par1[54]                        
                    if (tmp gt 0.0) then begin            
                        c_params.poff = tmp
                    endif else begin            
                        cosmos_logmessage, 'No valid value for poff in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.poff), /WARNING              
                    endelse
                
                    ; d0
                    tmp = par1[55]
                    if (tmp gt 0.0) then begin            
                        ;distance chopper-sample written in mm in FIGARO numor. Converted in m.
                        c_params.d0 = tmp * 1e-3               
                    endif else begin            
                        cosmos_logmessage, 'No valid value for d0 in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.d0), /WARNING              
                    endelse
                
                    ; openoff
                    tmp = par1[56]
                    if (tmp ne 0.0) then begin            
                        c_params.openoff = tmp
                    endif else begin            
                        cosmos_logmessage, 'No valid value for openoff in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.openoff), /WARNING              
                    endelse
                
                    ; pixelwidth
                    tmp = par1[104]
                    if (tmp gt 0.0) then begin            
                        c_params.pixelwidth = tmp*1.0e-3
                    endif else begin            
                        cosmos_logmessage, 'No valid value for pixelwidth in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.pixelwidth), /WARNING              
                    endelse
                
                    ; sample to detector distance
                    tmp = par1[18]
                    if (tmp gt 0.0) then begin            
                        ; The sample to detector distance is converted from mm to m.
                        c_params.sdetd = tmp*1.0e-3
                    endif else begin            
                        cosmos_logmessage, 'No valid value for sample to detector distance in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.sdetd), /WARNING              
                    endelse
                 
                    ; sample position offset
                    tmp = par1[23]
                    c_params.offset = tmp*1.0e-3
                    if (abs(tmp) gt 1.e-6) then cosmos_logmessage, 'Not zero offset for sample position = ' + cosmos_str_make(c_params.offset)
                
                    ; TOF delay
                    tmp = par1[96] + par1[93] ;26.4.2013 MG added extra correction term required for Figaro
                    if (tmp gt 0.0) then begin            
                        ; The TOF delay is converted from microseconds to seconds
                        c_params.edelay = tmp*1.0e-6
                    endif else begin            
                        cosmos_logmessage, 'No valid value for TOF delay in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.edelay), /WARNING              
                    endelse
    
                    ; xdh1
                    tmp = par1[90]
                    if (tmp gt 0.0) then begin            
                        c_params.xdh1 = tmp*1.0e-3
                    endif else begin            
                        cosmos_logmessage, 'No valid value for xdh1 in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.xdh1), /WARNING              
                    endelse
    
                    ; xdh2
                    tmp = par1[91]
                    if (tmp gt 0.0) then begin            
                        c_params.xdh2 = tmp*1.0e-3
                    endif else begin            
                        cosmos_logmessage, 'No valid value for xdh2 in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.xdh2), /WARNING              
                    endelse
    
                    ; ydet
                    tmp = par1[92]
                    if (tmp gt 0.0) then begin            
                        c_params.ydet = tmp*1.0e-3
                    endif else begin            
                        cosmos_logmessage, 'No valid value for ydet in numor ' + file_name, /WARNING
                        cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.ydet), /WARNING              
                    endelse
            
                  endif else begin          
                    cosmos_logmessage, 'Instrument name not recognized!', /WARNING            
                  endelse
          
                endif ;end override
        
                ;MAG: Setting arrays sizes from numor
                if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
            
                    ; New and Old detector
                    if c_params.data_format gt 1 then  c_params.pcen=132.5 else c_params.pcen=135.79   
                  
                    ;pixels_x
                    tmp = long(par1[98]-par1[97]+1)
                    if (tmp gt 1) then begin
                      c_params.pixels_x = tmp
                    endif else begin
                      c_params.pixels_x = c_params.pixels_x_d17
                    endelse
                    
                    ;pixels_t
                    tmp = long(par1[94])
                    if (tmp gt 1) then begin
                      c_params.pixels_t = tmp
                    endif else begin
                      c_params.pixels_t = c_params.pixels_t_d17
                    endelse
                    
                    ;check if kinetic numor --> New for D17 (not tested yet, MAG 21/02/2013)
                    kinetic_d17 = 0
                    tmp = long(par1[84])
                    if tmp eq 3 then begin
                      kinetic_d17 = 1 
                      tmp = long(par1[26])
                      slices_d17 = tmp
                    endif  
              
                endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
            
                    ;PG: introduced for consistency with D17
                    c_params.pcen = (float(par1[100]-par1[99])/2)+float(par1[99]);-0.5  ;PG on 07/08/2017
                
                    ;pixels_x
                    tmp = long(par1[100]-par1[99]+1) ;Y1,Y2
                    if (tmp gt 1) then begin
                      c_params.pixels_x = tmp
                    endif else begin
                      c_params.pixels_x = c_params.pixels_x_fig
                    endelse
                    
                    ;pixels_t
                    tmp = long(par1[94])
                    if (tmp gt 1) then begin
                      c_params.pixels_t = tmp
                    endif else begin
                      c_params.pixels_t = c_params.pixels_t_fig
                     endelse
          
                    ;check if kinetic numor
                    kinetic_figaro = 0
                    tmp = long(par1[84])
                    if tmp eq 3 then begin
                      kinetic_figaro = 1 
                      tmp = long(par1[26])
                      slices_figaro = tmp
                    endif  
              
                endif else begin
            
                    cosmos_logmessage, 'Instrument name not recognized!', /WARNING
              
                endelse ;end setting array sizes
                
                ; read 2 line spacer
                for i = 1, 2 do readf, raw_lun, tempstr
                
                ; read 2nd set of parameters
                par2 = fltarr(256)
                readf, raw_lun, par2
                
                ; useful parameter information :
                ;  params1[ ] :
                ;   2 : run time (in tenths of a second)
                ;   4 : total monitor counts
                ;   94 : number of channels
                ;   95 : channel width (micro seconds)
                ;   96 : tof delay (micro seconds)
                ;   97 : x1
                ;   98 : x2
                ;   99 : y1
                ;   100 : y2
                ;   101 : nx, detector grouping factor in x (ie nx = 1 => base detector has 286 pixels in x, nx = 2 => base detector has 144 pixels in x)
                ;   102 : ny, detector grouping factor in y (ny = 1 => 276 pixels)
                ;  params2[ ] :
                ;   2 : san, sample angle
                ;   15 : sample to detector distance (millimetres)
                ;   16 : dan, detector angle
                ;   40 : requested chopper 1 speed (revs / min)
                ;   41 : requested chopper 1 phase (degrees)
                ;   42 : requested chopper 2 speed
                ;   43 : requested chopper 2 phase
                ;   44 : actual chopper 1 speed
                ;   45 : actual chopper 1 phase
                ;   46 : actual chopper 2 speed
                ;   47 : actual chopper 2 phase
                ;   93 : slit value
                ;   95 : slit value
                        
                if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
                
                  if float(par2[14]) gt 0.0 then begin
                    if float(par2[14]) lt 10.0 then begin
                      c_params.chopsep = float(par2[14])*1.e-2   ;separation given in cm in numor file
                    endif else begin
                      c_params.chopsep = float(par2[14])*1.e-3   ;separation given in mm in numor file
                    endelse
                  endif else begin
                    c_params.chopsep = c_params.chopsep_d17
                  endelse
                  ;check if separation between choppers is reasonable - if not, use default value
                  if c_params.chopsep lt 79e-3 or c_params.chopsep gt 87e-3 then begin
                    ;separation between choppers = 85 mm before 2012 and 81.1 mm since 2012
                    if detector_format ge 2 then c_params.chopsep = c_params.chopsep_d17 else c_params.chopsep = 85.e-3
                    cosmos_logmessage, 'No valid value for chopsep in numor ' + file_name, /WARNING
                    cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.chopsep), /WARNING
                  endif
                  
                  c_params.chopwin = c_params.chopwin_d17
                  
                  ;PG on 07/08/2017: Added chopper radius
                  c_params.cr = c_params.cr_d17  ;
        
                  ;PG: Corrected for reading in actual interslit distance
                  c_params.interslit = float(par1[58])-float(par1[59]) 
                  if c_params.interslit lt 3000.0 then c_params.interslit = c_params.interslit_d17
        
                  temppar = { cosmos_parameters }
                  temppar.monitor = float(par1[4])
                  temppar.runtime = float(par1[2] / 10.)
                  temppar.dan = float(par2[16])
                  temppar.san = float(par2[2])
                  temppar.size_x = long(par1[98] - par1[97] + 1) ;X1,X2
                  
                  temppar.size_t = long(par1[94])
                  
                  temppar.x_min = long(par1[97]) ;X1
                  temppar.x_max = long(par1[98]) ;X2
        
                  temppar.s2w = par2[8]+par2[9]
                  temppar.s3w = par2[12]+par2[13]
                  temppar.slits = (temppar.s2w + temppar.s3w) / 2.
                  ;MAG: Parameter added in order to normalise by slits opening values (in case they are different for reflected and direct beam)
                  temppar.slitsproduct = temppar.s2w*temppar.s3w
                  temppar.slit3=float(par1[59])*1e-3	;PG on 28/04/15: Added for consistency with FIGARO and needed for constant-q COSMOS
                  
                  temppar.sdetd = c_params.sdetd ;It should not use the numor parameter to allow for override to work
                  ;temppar.tofd = float(par2[15] * 1e-3) + c_params.d0 - (c_params.chopsep / 2.)
                  temppar.tofd = c_params.sdetd + c_params.d0 - (c_params.chopsep / 2.)
                  
                  if par2[47] gt 360. then par2[47]=par2[47]/100.	;PG: Put actual opening back in
                  if par2[45] gt 360. then par2[45]=par2[45]/100.
                  
                  if par2[43] gt 360. then par2[43]=par2[43]/100.	;PG: changed from "actual" opening which means last value to requested opening that is closer to the mean value
                  if par2[41] gt 360. then par2[41]=par2[41]/100. 
                  if par2[110] gt 360. then par2[110]=par2[110]/100.	;PG: added mean value
                  if par2[114] gt 360. then par2[114]=par2[114]/100.
                  
                  open_actual= c_params.chopwin - (par2[47] - par2[45]) - c_params.openoff	;PG: Put actual opening back in
                  open_req= c_params.chopwin - (par2[43] - par2[41]) - c_params.openoff	;PG on 27/07/15: Calculate requested opening
                  
                  if par2[114] eq 0. then begin	;PG: using mean chopper values if possible, if not requested ones
                  temppar.openangle = c_params.chopwin - (par2[43] - par2[41]) - c_params.openoff
                  temppar.period = 60. / par2[40]
                  endif else begin
                  temppar.openangle = c_params.chopwin - (par2[114] - par2[110]) - c_params.openoff
                  temppar.period = 60. / par2[109]
                  endelse 
        	        ;if (abs(temppar.openangle-open_actual) gt 1.0) then temppar.openangle = open_actual	;PG: Due to recent bug in NoMad
        	        if (abs(temppar.openangle-open_actual) gt 1.0) then temppar.openangle = open_req		;PG on 27/07/15: The use of the requested is better as the actual one may fluctuate alot
        
                  temppar.delayangle = (c_params.poff - temppar.openangle) / 2. ;add phase of chopper 1 if ne 0?
                  temppar.delay = c_params.edelay - (temppar.delayangle / 360.) * temppar.period
                  temppar.channelwidth = par1[95] * 1e-6
                  temppar.pixeldensity = par1[101]
                  size_y = long(par1[100] - par1[99] + 1) ;Y1,Y2

                  temppar.tra = par2[24]       ;analyzer translation
                  temppar.f1  = fix(par2[99])  ;flipper 1 state (0=off, 1=on)
                  temppar.f2  = fix(par2[100]) ;flipper 2 state (0=off, 1=on)
                  
                endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin
                
                  c_params.chopwin = c_params.chopwin_fig
                  c_params.interslit = c_params.interslit_fig
                  
                  ;PG on 07/08/2017: Added chopper radius
                  c_params.cr = c_params.cr_fig  
                  
                  ;distance between slits
                  tmp = par1[105] 
                  if (tmp gt 0.0) then begin
                  
                    c_params.interslit = tmp
                    
                  endif else begin
                  
                    cosmos_logmessage, 'No valid value for interslit in numor ' + file_name, /WARNING
                    cosmos_logmessage, 'Using default value = ' + cosmos_str_make(c_params.interslit), /WARNING
                    
                  endelse
                  
                  temppar = { cosmos_parameters }
                  ;monitor
                  temppar.monitor = float(par1[4]) 
                  ;measuring time
                  temppar.runtime = float(par1[2])
                  ;detector angle 
                  temppar.dan = float(par1[14])
                  ;bragg angle to compute q read from Theta from Figaro Settings parameter 
                  temppar.san = float(par1[51]) 
                  ;MAG: Note that for figaro the x dimension corresponds to the vertical plane
                  temppar.size_x = long(par1[100] - par1[99] + 1) ;Y1,Y2
                  temppar.size_t = long(par1[94])
                  ;Y1
                  temppar.x_min = long(par1[99]) 
                  ;Y2
                  temppar.x_max = long(par1[100])
                  
                  ;PG: added for consistency with D17
                  temppar.s2w = float(par2[111]) 
                  temppar.s3w = float(par2[117])
                  
                  ;MG (18/6/2015): correction to slits opening depending on BGS3 actual value
                  case 1 of
                    (par2[13] ge 150)                       : temppar.s3w += 0.08
                    (par2[13] ge 50) and (par2[13] lt 150)  : temppar.s3w += 0.06
                    (par2[13] ge -50) and (par2[13] lt 50)  : temppar.s3w += 0.00
                    (par2[13] ge -150) and (par2[13] lt -50): temppar.s3w -= 0.12
                    (par2[13] lt -150)                      : temppar.s3w -= 0.24
                  endcase
                  
                  ;MG (19/6/2015): two new params needed for DAN option
                  temppar.wantedtheta = par1[51]
                  temppar.wantedpixel = par1[83]
                   
                  ;Defined using s2h and s3h
                  temppar.slits = (temppar.s2w + temppar.s3w) / 2.
                  ;sample to detector distance. Apply a correction only for numor version >= 9.
                  if (fix(par1[89]) ge 9) then begin
                      cosmos_logmessage, "Applying sample to detector distance correction."
                      ;temppar.sdetd = sample_to_detector_distance(par2[18] * 1e-3, par2[27] * 1e-3, par2[28] * 1e-3, c_params.pcen, c_params.xdh1, c_params.xdh2, par1[18] * 1e-3, c_params.ydet, c_params.pixelwidth)      
                      temppar.sdetd = sample_to_detector_distance(par2[18] * 1e-3, par2[27] * 1e-3, par2[28] * 1e-3, c_params.pcen, c_params.xdh1, c_params.xdh2, c_params.sdetd, c_params.ydet, c_params.pixelwidth)      
                  endif else begin
                      ;temppar.sdetd = float(par1[18] * 1e-3)
                      temppar.sdetd = c_params.sdetd ;It should not use the numor parameter to allow for override to work
                  endelse 
                  ;tof distance from mid-chopper to detector (d0=mid-chopper to sample in case of FIGARO!)
                  temppar.tofd = temppar.sdetd + c_params.d0 
                  ;tof distance from chopper to monitor 2 in figaro
                  temppar.tofm = float(par1[25] * 1e-3) + c_params.d0
                  
                  ;MAG (19/04/11) Added temppar.offset to contain the sample offset position (needed for gravity corrections)
                  ;temppar.offset = float(par1[23] * 1e-3) ;(12/05/11) par1[24] previously, commented to allow override to work
                  temppar.offset = c_params.offset
                  temppar.slit3 = float(par1[106] * 1e-3) ;S3-sample distance without sample offset
                  temppar.slit2 = temppar.slit3 + float(par1[105] * 1e-3) ;S2-sample distance without sample offset       
                  
                  chopper1 = par2[200] ;first controlling chopper
                  chopper2 = par2[201] ;second controlling chopper
                
                  case chopper1 of
                
                    1 : begin
                    
                      speed1 = par2[180]
                      phase1 = par2[181]
                      case chopper2 of
                        2 : c_params.chopsep = c_params.chopsep12_fig
                        3 : c_params.chopsep = c_params.chopsep13_fig
                        4 : c_params.chopsep = c_params.chopsep14_fig
                      endcase
                      
                    end
                    
                    2 : begin
                    
                      speed1 = par2[185]
                      phase1 = par2[186]
                      case chopper2 of
                        3 : c_params.chopsep = c_params.chopsep23_fig
                        4 : c_params.chopsep = c_params.chopsep24_fig
                      endcase
                      
                    end
                    
                    3 : begin
                    
                      speed1 = par2[190]
                      phase1 = par2[191]
                      case chopper2 of
                        4 : c_params.chopsep = c_params.chopsep34_fig
                      endcase
                      
                    end
                    
                    else : cosmos_logmessage, 'No right value in par2[200] for controlling chopper 1', /WARNING
                    
                  endcase
                  
                  case chopper2 of
                  
                    2 : begin
                    
                      speed2 = par2[185]
                      phase2 = par2[186]
                      
                    end
                    
                    3 : begin
                    
                      speed2 = par2[190]
                      phase2 = par2[191]
                      
                    end
                    
                    4 : begin
                    
                      speed2 = par2[195]
                      phase2 = par2[196]
                      
                    end
                    
                    else : cosmos_logmessage, 'No right value in par2[201] for controlling chopper 2', /WARNING
                    
                  endcase
              
                  ;virtual chopper 2 phase (scaling by 100 needed?)
                  if phase2 gt 360. then phase2=phase2/100. 
                  
                  ;if phase1 gt 360. then phase1=phase1/100. ;virtual chopper 1 phase (scaling by 100 needed?)
                  ;Set to zero the phase of first controlling chopper if it is 1
                  if (chopper1 eq 1) then phase1 = 0.0 
                  
                  temppar.openangle = c_params.chopwin - (phase2 - phase1) - c_params.openoff
                  ;chopwin = 45
                  ;period calc. using virtual chopper 1 speed
                  temppar.period = 60. / speed1 
                  temppar.delayangle = phase1 + (c_params.poff - temppar.openangle) / 2.
                  ;using TOF delay (in microsec)
                  temppar.delay = c_params.edelay - (temppar.delayangle / 360.) * temppar.period
                  ;using TOF channel width (in microsec) 
                  temppar.channelwidth = par1[95] * 1e-6
                  ;using ny, binning denominator 
                  temppar.pixeldensity = par1[102]
                   
                  size_y = long(par1[98] - par1[97] + 1) ;X1,X2
              
                  temppar.refdown = 0
                  if par2[13] gt 45 then temppar.refdown = 1 ;determine if reflection down
                    
                endif else begin
                
                  cosmos_logmessage, 'Instrument name not recognized!', /WARNING
            
                endelse
    
                ;MAG: Check if numor sizes are compatible! Only one change allowed.
                ;     Now that constants.pixels_x is not taken as a constant, but
                ;     read from the numor it could be changed.
                if flag_defined eq 0 then begin
                        
                  params = temppar
                  params.monitor = 0.
                  params.runtime = 0.
        
                  monitors = lonarr(2, temppar.size_t)
                  counts = lonarr(temppar.size_x, temppar.size_t)
        
                  if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) && kinetic_d17 then begin
                    monitors = lonarr(2, temppar.size_t, slices_d17)
                    counts = lonarr(temppar.size_x, temppar.size_t, slices_d17)
                  endif
                  if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) && kinetic_figaro then begin
                    if c_params.data_format le 9 then monitors = lonarr(2,slices_figaro) else monitors = lonarr(2, temppar.size_t, slices_figaro)
                    counts = lonarr(temppar.size_x, temppar.size_t, slices_figaro)
                  endif
                  
                  if c_params.pixels_x ne params.size_x then begin
                  
                    if c_params.sizeset ne 0 then begin
                  
                      cosmos_logmessage, 'Incompatible detector size found in file.', /WARNING
                    
                    endif else begin
                  
                      cosmos_logmessage, 'Detected ' + cosmos_str_make(params.size_x) + ' x-pixels in file.'
                      cosmos_logmessage, 'Changed detector size: useful pixel and peak search ranges may be invalid.', /WARNING
                      cosmos_logmessage, 'Also check machine c: 276 x-pixel detector has 1.0213mm pixel width, 4 x-tube detector has 25.6mm pixel width.', /WARNING
                    
                      c_params.pixels_x = params.size_x
                    
                      cosmos_init_detector
                      cosmos_logmessage, 'Reparsing ranges:'
                      cosmos_anal_ranges
                      c_params.sizeset = 1
                    
                    endelse
                  
                  endif
        
                  flag_defined = 1
      
                endif
    
                ; compare with old parameters and output warning if different
                if abs(temppar.dan - params.dan) gt 0.1 then cosmos_logmessage, 'Variation in detector angle detected between files.', /WARNING
              
                if abs(temppar.san - params.san) gt 0.1 then cosmos_logmessage, 'Variation in sample angle detected between files.', /WARNING
              
                if (temppar.size_x ne params.size_x) or (temppar.size_t ne params.size_t) then begin
              
                  cosmos_logmessage, '  Error : Unable to read file. Incompatible data array dimensions.'
                
                endif else begin
              
                  ; augment parameters
                  
                  params.monitor = params.monitor + temppar.monitor
                  params.runtime = params.runtime + temppar.runtime
            
                  if (strcmp(strtrim(c_params.inst,2),'d17',/FOLD_CASE)) then begin
            
                    if (c_params.data_format eq 1) then begin
              
                      ; read 3 line spacer
                      for i = 1, 3 do readf, raw_lun, tempstr
                    
                      ; read total number of pixels in detector
                      readf, raw_lun, tempvar
                    
                      ; construct detector array of specified size
                      if size_y le 1 then detarr = lonarr(params.size_x, params.size_t) else detarr = lonarr(size_y, params.size_x, params.size_t)
                    
                      ; read detector data
                      readf, raw_lun, detarr
                
                    endif else if (c_params.data_format eq 2) then begin
              
                      ; construct detector array of specified size
                      if size_y le 1 then detarr = lonarr(params.size_x, params.size_t) else cosmos_logmessage, 'Option to work with [X,Y,TOF] detector not ready yet.', /WARNING
                  
                      detline = lonarr(params.size_t)
                    
                      for i = 0,params.size_x-1 do begin
                    
                        ;skip 3 spacer lines
                        for j = 1, 3 do readf, raw_lun, tempstr  
                  
                        temp = 0
                      
                        readf, raw_lun, temp
                      
                        if (temp ne params.size_t) then cosmos_logmessage, 'Different number of TOF channels found.', /WARNING
                      
                        readf, raw_lun, detline
                        detarr[i,*] = detline
                      
                      endfor
                
                    endif else if (c_params.data_format eq 3) then begin
                    
                       ; New data format (version 3) produced by new kinetic acquisition card (used from 21/02/2013)
                               
                       if kinetic_d17 then begin  
        
                          ; read 3 line spacer (SSSSS ... IIIII)
                          for j = 1, 3 do readf, raw_lun, tempstr
                          tempvar = 0L
                          readf, raw_lun, tempvar
                          tmparr = fltarr(tempvar)                      
                          readf, raw_lun, tmparr
                          tmpmonarr = reform(tmparr[0:2*slices_d17*c_params.pixels_t-1], c_params.pixels_t, slices_d17, 2)
        
                          monarr = transpose(tmpmonarr,[2,1,0])
                          tmpdetarr = reform(tmparr[2*slices_d17*c_params.pixels_t:*], c_params.pixels_t, slices_d17, params.size_x)
                          detarr = transpose(tmpdetarr, [2,0,1])
        
                       endif else begin
        
                          ;create array for two monitors
        
                          monarr = lonarr(2,params.size_t)
                          
                          ; read 3 line spacer (SSSSS ... IIIII)
                          for j = 1, 3 do readf, raw_lun, tempstr
                          tempvar = 0L
                          readf, raw_lun, tempvar
                          tmparr = fltarr(tempvar)                      
                          readf, raw_lun, tmparr
        
                          monarr[0,*] = tmparr[0:c_params.pixels_t-1]
                          monarr[1,*] = tmparr[c_params.pixels_t:2*c_params.pixels_t-1]
                          tmpdetarr = reform(tmparr[2*c_params.pixels_t:*], c_params.pixels_t, params.size_x)
                          detarr = transpose(tmpdetarr, [1,0])
                          
                       endelse
                    
                    endif
      
                  endif else if (strcmp(strtrim(c_params.inst,2),'figa',/FOLD_CASE)) then begin

                    if kinetic_figaro then begin
                     
                        ;create array for two monitors and data for each time slice
                        if c_params.data_format le 9 then monarr = lonarr(2,slices_figaro) else monarr = lonarr(2, c_params.pixels_t, slices_figaro)                  
                        if size_y le 1 then detarr = lonarr(params.size_x, params.size_t,slices_figaro) $
                          else cosmos_dialogue_warn, 'Do not know how to read 3D data. Format?'
                          
                        ;read each time slice
                        if c_params.data_format le 9 then begin ;Old Format
                         
                            for i = 0,slices_figaro-1 do begin 
                               ; read 3 line spacer (SSSSS ... FFFFF)
                               for j = 1, 3 do readf, raw_lun, tempstr
                               ; read monitor values
                               tempvar1=0L
                               tempvar2=0L
                               readf, raw_lun, tempvar1, tempvar2
                               if tempvar2 ge 1 then for j = 1,tempvar2 do readf, raw_lun, tempstr 
                               tmparr = fltarr(tempvar1)                      
                               readf, raw_lun, tmparr
                               monarr[*,i] = long(tmparr[1:2])
                               ;skip IIIII line
                               readf, raw_lun, tempstr
                               ;read block of data
                               readf, raw_lun, tempvar
                               tmparr = lonarr(tempvar)
                               readf, raw_lun, tmparr
                               tmparr = reform(tmparr,1000,256)
                               tmparr = transpose(tmparr)
                               detarr[*,*,i] = tmparr
                            endfor
                            
                        endif else begin ;New Card with new format
      
                            ; read 3 line spacer (SSSSS ... IIIII)
                            for j = 1, 3 do readf, raw_lun, tempstr
                            tempvar = 0L
                            readf, raw_lun, tempvar
                            tmparr = fltarr(tempvar)                      
                            readf, raw_lun, tmparr
                            tmpmonarr = reform(tmparr[0:2*slices_figaro*c_params.pixels_t-1], c_params.pixels_t, slices_figaro, 2)
                            monarr = transpose(tmpmonarr,[2,1,0])
                            tmpdetarr = reform(tmparr[2*slices_figaro*c_params.pixels_t:*], c_params.pixels_t, slices_figaro, params.size_x)
                            detarr = transpose(tmpdetarr, [2,0,1])
                         
                        endelse   
                          
                    endif else begin ;Standard TOF
      
                        ;create array for two monitors
                 
                        monarr = lonarr(2,params.size_t)
                 
                        if c_params.data_format le 9 then begin ;Old Format
                  
                           ; read two monitors
                           for i = 0,1 do begin
                 
                              ; read 3 line spacer
                              for j = 1, 3 do readf, raw_lun, tempstr
                   
                              ; read total number of channels in monitor
                              readf, raw_lun, tempvar
                   
                              ; read monitor data
                              tmparr = lonarr(params.size_t)
                   
                              readf, raw_lun, tmparr
                   
                              monarr[i,*] = tmparr
                   
                           endfor
                 
                           ; construct detector array of specified size
                           if size_y le 1 then begin
                 
                              detarr = lonarr(params.size_x, params.size_t)
                   
                           endif else begin
                 
                              cosmos_logmessage, 'Do not know how to read 3D data. Format?', /WARNING
                   
                           endelse
                 
                           ; read detector data
                           for i = 0,params.size_x-1 do begin
                 
                              ; read 3 line spacer
                              for j = 1, 3 do readf, raw_lun, tempstr
                   
                              ; read total number of channels in detector
                              readf, raw_lun, tempvar
                   
                              ; read detector data
                              tmparr = lonarr(params.size_t)
                   
                              readf, raw_lun, tmparr
                              detarr[i,*] = tmparr
                   
                           endfor
                  
                        endif else begin ;New Card with new format
                        
                           ; read 3 line spacer (SSSSS ... IIIII)
                           for j = 1, 3 do readf, raw_lun, tempstr
                           tempvar = 0L
                           readf, raw_lun, tempvar
                           tmparr = fltarr(tempvar)                      
                           readf, raw_lun, tmparr
                           monarr[0,*] = tmparr[0:c_params.pixels_t-1]
                           monarr[1,*] = tmparr[c_params.pixels_t:2*c_params.pixels_t-1]
                           tmpdetarr = reform(tmparr[2*c_params.pixels_t:*], c_params.pixels_t, params.size_x)
                           detarr = transpose(tmpdetarr, [1,0])
                                            
                        endelse
      
                    endelse
      
                  endif else begin
         
                    cosmos_logmessage, 'Instrument name not recognized!', /WARNING
           
                  endelse
    
                  ; augment counts
                  if size_y le 1 then counts = counts + detarr else counts = counts + total(detarr, 1)
            
                  ; augment monitors for FIGARO or D17 (if stored in monarr)
                  if n_elements(monarr) gt 0 then monitors = monitors + monarr
    
                endelse ;finish main reading block
  
            endif else begin
      
                cosmos_logmessage, "File: " + file_name + " " + !error_state.msg, /WARNING
        
            endelse ;block if tempval eq 0 ... (file opened without error or with error)

            cosmos_raw_read_exit2 : if raw_lun gt 0 then free_lun, raw_lun
    
        endif
    
    endfor ;end loop over files

  endif else begin ;last file not readable --> error in table input?
     
     if strcompress(files[n_elements(files)-1]) gt ' 'then cosmos_logmessage, "File: " + testfile + " not readable!", /WARNING
  
  endelse
  
  if flag_defined eq 0 then begin
    
    counts = fltarr(c_params.pixels_x, c_params.pixels_t)
    params = { cosmos_parameters }
    params.monitor = 0.
    params.runtime = 0.
    params.size_x = c_params.pixels_x
    params.size_t = c_params.pixels_t
  
  endif

  return
  
end

pro cosmos_run_setup

  ;forward_function cosmos_str_parse
  
  common cosmos_params, c_params
     
  common cosmos_direct, direct1_files, direct2_files, direct3_files, direct1_monitor, direct2_monitor, direct3_monitor, $
                        direct1_array, direct2_array, direct3_array, direct1_params, direct2_params, direct3_params

  common cosmos_results, c_results
  
  common cosmos_settings, c_settings
        
  common cosmos_water, water_file, water_array, water_lowerlimit
                                               
  ; flag for resizing of detector (can only do once per analysis run!)
  c_params.sizeset = 0

  ; water run(s)
  temp2 = cosmos_str_parse(c_settings.waterrun)
  
  ; compare existing water file
  temp1 = where(temp2 ne water_file, temp3)
        
  if temp3 gt 0 then begin
          
    cosmos_raw_read, temp2, monitors, counts, params, c_settings.pathdir, c_settings.override    
    
    if params.runtime gt 0 then begin
      water_file = temp2
      cosmos_logmessage, ' Using ' + water_file + ' as water file.'
      
      ;MAG: cosmos_init_detector not used. Create water_arrays here.
      if size(counts, /n_dimensions) eq 1 then begin
      
        water_array = make_array(c_params.pixels_x, /float, value = 1.)
        water_array = counts
        
      endif else begin
      
        if size(counts, /n_dimensions) eq 2 then begin
        
          water_array = make_array(c_params.pixels_x, /float, value = 1.)
          water_array = total(counts, 2)
          
        endif else begin
        
          cosmos_logmessage, 'Unable to interpret water file. Water not used.'
          
        endelse
        
      endelse
      
      ; get useful detector and peak search ranges
      cosmos_anal_ranges
      
      minusefulx = long(c_settings.usefulpixelmin1)
      if c_settings.nangles ge 1 then begin
        minusefulx = minusefulx < long(c_settings.usefulpixelmin2)
        if c_settings.nangles ge 2 then minusefulx = minusefulx < long(c_settings.usefulpixelmin3)
      endif
      
      maxusefulx = long(c_settings.usefulpixelmax1)
      if c_settings.nangles ge 1 then begin
        maxusefulx = maxusefulx > long(c_settings.usefulpixelmax2)             
        if c_settings.usefulpixelmax2 gt maxusefulx then maxusefulx = long(c_settings.usefulpixelmax2)
        if c_settings.nangles ge 2 then maxusefulx = maxusefulx > long(c_settings.usefulpixelmax3)             
      endif
      
      water_array[minusefulx:maxusefulx] = cosmos_array_norm(water_array[minusefulx : maxusefulx])
      
      if min(water_array[minusefulx : maxusefulx]) le water_lowerlimit then $
        cosmos_logmessage, 'Low efficiency pixels detected in water - check file and useful detector range.', /WARNING
        
      water_array = (water_array > water_lowerlimit)
      
    endif else begin
    
      ; get useful detector and peak search ranges
      cosmos_anal_ranges
      
    endelse
    
  endif else begin
    
    ; get useful detector and peak search ranges
    cosmos_anal_ranges
    
  endelse
    
  c_results = !NULL

end

;+
; :Description:
;    Read content of selected rows and call cosmos_anal_run to get reflectivities.
;    Then normalizes and joins data from different angles and write output files.
;    
; :Params:
;
; :Returns:
;
; :Author: E. Pellegrini, M. A. Gonzalez
;
; :History:
;    M. A. Gonzalez, 4 March 2015
;
; :Categories: cosmos
;
;-
pro cosmos_run_background

  common cosmos_params, c_params
  common cosmos_direct, direct1_files, direct2_files, direct3_files, direct1_monitor, direct2_monitor, direct3_monitor, $
                        direct1_array, direct2_array, direct3_array, direct1_params, direct2_params, direct3_params
  common cosmos_results, c_results
  common cosmos_settings, c_settings
  common cosmos_water, water_file, water_array, water_lowerlimit
      
  ;Clear math errors
  temp1 = check_math()
              
  error_index = 0
    
  catch, error_index
    
  if error_index ne 0 then begin
     cosmos_logmessage, 'Error while analysing row ' + cosmos_str_make(c_params.row+1) + ":", /WARNING
     cosmos_logmessage, !ERROR_STATE.MSG, /WARNING
     cosmos_logmessage, 'Check detector ranges, offsets etc.', /WARNING
     cosmos_logmessage, 'Please report bugs to <[pellegrini,gonzalez,richard,gutfreund]@ill.fr>', /WARNING
     c_params.row = c_params.row + 1     
     return
  endif
  
  if (c_params.row lt c_params.rowfinish) then begin
  
     cosmos_logmessage, ' '
     cosmos_logmessage, '##############################################################'
     cosmos_logmessage, 'Analysing Row ' + cosmos_str_make(c_params.row + 1) + ':'
     cosmos_logmessage, '##############################################################'
     cosmos_logmessage, ' '
               
     t_ini = systime(1)
     
     ;Reset min/max numors for the current line
     c_params.min_refl_numor = 1000000
     c_params.max_refl_numor = 0 
                
     ;Create and initialize needed arrays and variables
     direct1  = strarr(1) & direct2  = strarr(1) & direct3  = strarr(1)
     reflect1 = strarr(1) & reflect2 = strarr(1) & reflect3 = strarr(1)
     instr1   = strarr(1) & instr2   = strarr(1) & instr3   = strarr(1)
     factor1 = fltarr(1)  & factor2 = fltarr(1)
     theta1 = 0.          & theta2 = 0.          & theta3 = 0.
     
     ;Reset to 0 the peakref position for each new row (needed for polarization patch)
     c_settings.peakref = [0, 0, 0]
    
     nCols = n_elements((*c_settings.table)[*, 0])
     tLabels = cosmos_get_table_labels(nCols)
    
     for col = 0, n_elements((*c_settings.table)[*, 0]) - 1 do begin
                
        case tLabels[col] of
      
           'Direct 1' : direct1 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
           'Direct 2' : direct2 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
           'Direct 3' : direct3 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
       
           'Reflect 1' : reflect1 = cosmos_getnumors((*c_settings.table)[col, c_params.row]) 
           'Reflect 2' : reflect2 = cosmos_getnumors((*c_settings.table)[col, c_params.row]) 
           'Reflect 3' : reflect3 = cosmos_getnumors((*c_settings.table)[col, c_params.row]) 
        
           'Instr 1' : instr1 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
           'Instr 2' : instr2 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
           'Instr 3' : instr3 = cosmos_getnumors((*c_settings.table)[col, c_params.row])
        
           'Normalise' : nurm = cosmos_str_parse((*c_settings.table)[col, c_params.row], /FLOAT)
           'Factor' : factor1 = cosmos_str_parse((*c_settings.table)[col, c_params.row], /FLOAT)
           'Factor 1-2' : factor1 = cosmos_str_parse((*c_settings.table)[col, c_params.row], /FLOAT)
           'Factor 2-3' : factor2 = cosmos_str_parse((*c_settings.table)[col, c_params.row], /FLOAT)
        
           'Theta 1' : theta1 = cosmos_str_theta((*c_settings.table)[col, c_params.row])
           'Theta 2' : theta2 = cosmos_str_theta((*c_settings.table)[col, c_params.row])
           'Theta 3' : theta3 = cosmos_str_theta((*c_settings.table)[col, c_params.row])
        
           'Out file' : begin 
              outfile  = cosmos_str_parse((*c_settings.table)[col, c_params.row], /STRING_SET)
              if (strlen(strtrim(outfile,2)) eq 0) then begin
                 ;name for output file generated from numors for reflect
                 ;check numors (strings) present
                 check = [0, 0, 0]
                 if reflect1[0,0] gt '' then check[0] = 1 ;TS 2016-10-21 changed reflect1[0] to reflect1[0,0] to accomodate multiple files 
                 if reflect2[0,0] gt '' then check[1] = 1 ;TS 2016-10-21 changed reflect1[0] to reflect1[0,0] to accomodate multiple files
                 if reflect3[0,0] gt '' then check[2] = 1 ;TS 2016-10-21 changed reflect1[0] to reflect1[0,0] to accomodate multiple files
                 refs = strtrim([string(reflect1[0]),string(reflect2[0]),string(reflect3[0])],2)
                 outfile = strjoin(refs[where(check eq 1)],"_")
              endif
              outfile= filepath(outfile,ROOT_DIR=c_settings.outputdir)
           end
        
           'Out LAMP' : outlamp = cosmos_str_parse((*c_settings.table)[col, c_params.row], /INTEGER)
        
        else :
      
        endcase ;end case tLabels[col]
    
     endfor ;end for col = 0, n_elements((*c_settings.table)[*, 0]) - 1
                          
     ;Correct data and extract reflectivity for each angle
     
     ;Angle 1
     if (size(reflect1, /type)) eq 11 then polOpt = n_elements(reflect1) else polOpt = 0 ;check if polarized data

     if n_elements(reflect1[0]) gt 1 then checkReflect1 = (reflect1[0])[0] else checkReflect1 = reflect1[0]
     if n_elements(direct1[0]) gt 1 then checkDirect1 = (direct1[0])[0] else checkDirect1 = direct1[0]

     if (checkReflect1 ne '') then begin      
     
        if checkDirect1 ne '' then begin

          errorPol = 0

          if polOpt eq 0 then begin

            cosmos_logmessage, 'TSNEW =====  Reflect1: ' + cosmos_str_make(reflect1) + ':' + cosmos_str_make(n_elements(reflect1))
            cosmos_anal_run, result1, gpresult1, direct1, reflect1, instr1, theta1[0], 1

          endif else begin

            for i = 0, n_elements(reflect1)-1 do begin

              cosmos_logmessage, 'TSNEW ==== Reflect1: ' + cosmos_str_make(reflect1) + ':' + cosmos_str_make(n_elements(reflect1))
              tmp = reflect1[i]
              cosmos_logmessage, 'TSNEW ==== TMP: ' + cosmos_str_make(tmp)
              if i eq 0 then cosmos_anal_run, tmpresult, tmpgpresult, direct1, tmp, instr1, theta1[0], 1, /POL_RUN, tra=tra, f1=f1, f2=f2, /STORE_POLPARAMS, pol_lambda = pol_lambda, pol_parref = pol_parref $
              else cosmos_anal_run, tmpresult, tmpgpresult, direct1, tmp, instr1, theta1[0], 1, /POL_RUN, tra=tra, f1=f1, f2=f2, pol_lambda = pol_lambda, pol_parref = pol_parref

              if i eq 0 then begin

                ;check if analyzer is in (done only for the first numor)
                if tra gt 200. then begin
                  cosmos_logmessage, 'Reflected Beam (angle 1) measured without analyzer.'
                  polOpt = 1
                endif else if tra lt 0.0 then begin
                  cosmos_logmessage, 'TRA < 0 in reflected beam measurement (angle 1). Assuming analyzer is out!'
                  polOpt = 1
                endif else cosmos_logmessage, 'Correcting reflected beam (angle 1) for analyzer efficiency!'

                ;define array to contain the full set of data including different polarizations
                s = size(tmpresult) & gps = size(tmpgpresult)
                ;print, 'size ', s[0]
                case s[0] of
                  2: begin
                    polresult = fltarr(s[1], s[2], 4)
                    gppolresult = fltarr(gps[1], gps[2], 4)
                  end
                  3: begin
                    polresult = fltarr(s[1], s[2], s[3], 4)
                    gppolresult = fltarr(gps[1], gps[2], gps[3], 4)
                  end
                  else: begin
                    cosmos_logmessage, 'Polarized data - Wrong number of dimensions of result returned by cosmos_anal_run!', /warning
                    errorPol = 1
                  end
                endcase

                npoints = s[2]

              endif ;defined polOpt and s from first numor

              ;check dimensions (number of reflectivity points) are consistent
              if n_elements(tmpresult[0,*]) ne npoints then begin
                cosmos_logmessage, 'Polarized data - Number of datapoints is not the same in all the sets!', /warning
                errorPol = 1
              endif

              ;check flipper values are consistent and store results in polresult in the right flipper order [00, 01, 10, 11]
              if not errorPol then begin
                
                if polOpt eq 1 then begin                    ;no analyzer --> state 0X = index 0, state 1X = index 3
                  if f1 eq 0 then index = 0 else index = 3
                endif else index = 2*f1 + f2                 ;with analyzer --> 0 = 00, 1 = 01, 2 = 10, 3 = 11

                case s[0] of
                  2: begin
                    polresult[*,*,index] = tmpresult
                    gppolresult[*,*,index] = tmpgpresult
                  end
                  3: begin
                    polresult[*,*,*,index] = tmpresult
                    gppolresult[*,*,*,index] = tmpgpresult
                  end
                endcase
              endif

            endfor

            if errorPol then result1 = -1 else begin
              case polOpt of
                1: begin
                  ;TS 2015-08-27 Added a message for me
                  cosmos_logmessage, '!'
                  cosmos_logmessage, '_noana Efficiency Correction!'
                  cosmos_logmessage, '!'
                  result1 = cosmos_tofpolcorr_noana(polresult)
                  gpresult1 = cosmos_tofpolcorr_noana(gppolresult)
                end
                2: begin
                  ;TS 2015-08-27 Added a message for me
                  cosmos_logmessage, '!'
                  cosmos_logmessage, '_two Efficiency Correction!'
                  cosmos_logmessage, '!'
                  result1 = cosmos_tofpolcorr_two(polresult)
                  gpresult1 = cosmos_tofpolcorr_two(gppolresult)
                end
                3: begin
                  ;TS 2015-08-27 Added a message for me
                  cosmos_logmessage, '!'
                  cosmos_logmessage, '_three Efficiency Correction!'
                  cosmos_logmessage, '!'
                  result1 = cosmos_tofpolcorr_three(polresult)
                  gpresult1 = cosmos_tofpolcorr_three(gppolresult)
                end
                4: begin
                  ;TS 2015-08-27 Added a message for me
                  cosmos_logmessage, '!'
                  cosmos_logmessage, '_full Efficiency Correction!'
                  cosmos_logmessage, '!'
                  result1 = cosmos_tofpolcorr_full(polresult)
                  gpresult1 = cosmos_tofpolcorr_full(gppolresult)
                end
              endcase
            endelse

          endelse ;end treat of polarized data

        endif else begin
           cosmos_logmessage, ' No direct beam for angle 1. Analyse reflectivity using monitor 2!'
           cosmos_anal_run, result1, gpresult1, direct1, reflect1, instr1, theta1[0], 1, /NODIRECT_RUN
        endelse
        
     endif else result1 = -1
    
     ;Angle 2

     ;First check if polOpt > 0 and if so, if data are compatible
     if polOpt gt 0 then begin
       if polOpt eq 1 then begin
         if (size(reflect2, /type)) eq 11 and n_elements(reflect2) ne 2 then begin
           cosmos_logmessage, 'Numors in angle 2 incompatible with existing polOpt', /warning
           errorPol = 1
         endif else errorPol = 0
       endif else begin
         if (size(reflect2, /type)) eq 11 and n_elements(reflect2) ne polOpt then begin
           cosmos_logmessage, 'Numors in angle 2 incompatible with existing polOpt', /warning
           errorPol = 1
         endif else errorPol = 0
       endelse
     endif else begin
       if checkDirect1 eq '' then begin
         if (size(reflect2, /type)) eq 11 then polOpt = n_elements(reflect2) else polOpt = 0 ;check if polarized data
         errorPol = 0
       endif else begin
         if (size(reflect2, /type)) eq 11 then begin
            cosmos_logmessage, 'Polarized and non polarized runs cannot be mixed', /warning
            errorPol = 1
         endif else errorPol = 0 
       endelse
     endelse

     if n_elements(reflect2[0]) gt 1 then checkReflect2 = (reflect2[0])[0] else checkReflect2 = reflect2[0]
     if n_elements(direct2[0]) gt 1 then checkDirect2 = (direct2[0])[0] else checkDirect2 = direct2[0]

     if not errorPol and (checkReflect2 ne '') then begin
       if checkDirect2 ne '' then begin

         if polOpt eq 0 then $
           cosmos_anal_run, result2, gpresult2, direct2, reflect2, instr2, theta2[0], 2 $
         else begin

           for i = 0, n_elements(reflect2)-1 do begin

             tmp = reflect2[i]
             ;cosmos_anal_run, tmpresult, tmpgpresult, direct2, tmp, instr2, theta2[0], 2, /POL_RUN, tra=tra, f1=f1, f2=f2
             ;TS 03/08/2015 Make use of the use_lambda flag for second angle
             if i eq 0 then cosmos_anal_run, tmpresult, tmpgpresult, direct2, tmp, instr2, theta2[0], 2, /POL_RUN, tra=tra, f1=f1, f2=f2, /STORE_POLPARAMS, pol_lambda = pol_lambda, pol_parref = pol_parref $
             else cosmos_anal_run, tmpresult, tmpgpresult, direct2, tmp, instr2, theta2[0], 2, /POL_RUN, tra=tra, f1=f1, f2=f2, pol_lambda = pol_lambda, pol_parref = pol_parref

             if i eq 0 then begin
                ;check if analyzer is in(done only for the first numor)
                ;TS 2015-08-27 Added the check for Analyzer to the second angle
               if tra gt 200. then begin
                 cosmos_logmessage, 'Reflected Beam (angle 2) measured without analyzer.'
                 polOpt = 1
               endif else if tra lt 0.0 then begin
                 cosmos_logmessage, 'TRA < 0 in reflected beam measurement (angle 2). Assuming analyzer is out!'
                 polOpt = 1
               endif else cosmos_logmessage, 'Correcting reflected beam (angle 2) for analyzer efficiency!'
               ;define array to contain the full set of data including different polarizations
               s = size(tmpresult) & gps = size(tmpgpresult)
               case s[0] of
                 2: begin
                      polresult = fltarr(s[1], s[2], 4)
                      gppolresult = fltarr(gps[1], gps[2], 4)
                    end  
                 3: begin
                      polresult = fltarr(s[1], s[2], s[3], 4)
                      gppolresult = fltarr(gps[1], gps[2], gps[3], 4)
                    end
                 else: begin
                   cosmos_logmessage, 'Polarized data - Wrong number of dimensions of result returned by cosmos_anal_run!', /warning
                   errorPol = 1
                 end
               endcase

               npoints = s[2]

             endif ;defined polOpt and s from first numor

             ;check dimensions (number of reflectivity points) are consistent
             if n_elements(tmpresult[0,*]) ne npoints then begin
               cosmos_logmessage, 'Polarized data - Number of datapoints is not the same in all the sets!', /warning
               errorPol = 1
             endif

             ;check flipper values are consistent and store results in polresult in the right flipper order [00, 01, 10, 11]
             if not errorPol then begin

               if polOpt eq 1 then begin                    ;no analyzer --> state 0X = index 0, state 1X = index 3
                 if f1 eq 0 then index = 0 else index = 3
               endif else index = 2*f1 + f2                 ;with analyzer --> 0 = 00, 1 = 01, 2 = 10, 3 = 11

               case s[0] of
                 2: begin
                      polresult[*,*,index] = tmpresult
                      gppolresult[*,*,index] = tmpgpresult
                    end  
                 3: begin
                      polresult[*,*,*,index] = tmpresult
                      gppolresult[*,*,*,index] = tmpgpresult
                    end  
               endcase
             endif

           endfor

           if errorPol then result2 = -1 else begin
             case polOpt of
               1: begin
                ;TS 2015-08-27 Added a message for me
                cosmos_logmessage, '!'
                cosmos_logmessage, '_noana Efficiency Correction!'
                cosmos_logmessage, '!'
                 result2 = cosmos_tofpolcorr_noana(polresult)
                 gpresult2 = cosmos_tofpolcorr_noana(gppolresult)
               end
               2: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_TWO Efficiency Correction!'
                 cosmos_logmessage, '!'
                 result2 = cosmos_tofpolcorr_two(polresult)
                 gpresult2 = cosmos_tofpolcorr_two(gppolresult)
               end
               3: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_three Efficiency Correction!'
                 cosmos_logmessage, '!'
                 result2 = cosmos_tofpolcorr_three(polresult)
                 gpresult2 = cosmos_tofpolcorr_three(gppolresult)
               end
               4: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_full Efficiency Correction!'
                 cosmos_logmessage, '!'
                 result2 = cosmos_tofpolcorr_full(polresult)
                 gpresult2 = cosmos_tofpolcorr_full(gppolresult)
               end
             endcase
           endelse

         endelse ;end treat of polarized data

       endif else begin
         cosmos_logmessage, ' No direct beam for angle 2. Analyse reflectivity using monitor 2!'
         cosmos_anal_run, result2, gpresult2, direct2, reflect2, instr2, theta2[0], 2, /NODIRECT_RUN
       endelse

     endif else result2 = -1
          
    
     ;Angle 3

     ;First check if polOpt > 0 and if so, if data are compatible
     if polOpt gt 0 then begin
       if polOpt eq 1 then begin
         if (size(reflect3, /type)) eq 11 and n_elements(reflect3) ne 2 then begin
           cosmos_logmessage, 'Numors in angle 3 incompatible with existing polOpt', /warning
           errorPol = 1
         endif else errorPol = 0
       endif else begin
         if (size(reflect3, /type)) eq 11 and n_elements(reflect3) ne polOpt then begin
           cosmos_logmessage, 'Numors in angle 3 incompatible with existing polOpt', /warning
           errorPol = 1
         endif else errorPol = 0
       endelse
     endif else begin
       if checkDirect1 eq '' and checkDirect2 eq '' then begin
         if (size(reflect3, /type)) eq 11 then polOpt = n_elements(reflect3) else polOpt = 0 ;check if polarized data
         errorPol = 0
       endif else begin
         if (size(reflect3, /type)) eq 11 then begin
           cosmos_logmessage, 'Polarized and non polarized runs cannot be mixed', /warning
           errorPol = 1
         endif else errorPol = 0
       endelse
     endelse

     if n_elements(reflect3[0]) gt 1 then checkReflect3 = (reflect3[0])[0] else checkReflect3 = reflect3[0]
     if n_elements(direct3[0]) gt 1 then checkDirect3 = (direct3[0])[0] else checkDirect3 = direct3[0]

     if not errorPol and (checkReflect3 ne '') then begin

       if checkDirect3 ne '' then begin

         if polOpt eq 0 then $
           cosmos_anal_run, result3, gpresult3, direct3, reflect3, instr3, theta3[0], 3 $
         else begin

           for i = 0, n_elements(reflect3)-1 do begin

             tmp = reflect3[i]
             ;cosmos_anal_run, tmpresult, tmpgpresult, direct3, tmp, instr3, theta3[0], 3, /POL_RUN, tra=tra, f1=f1, f2=f2
             ;TS 03/08/2015 Make use of the use_lambda flag for third angle
             if i eq 0 then cosmos_anal_run, tmpresult, tmpgpresult, direct3, tmp, instr3, theta3[0], 3, /POL_RUN, tra=tra, f1=f1, f2=f2, /STORE_POLPARAMS, pol_lambda = pol_lambda, pol_parref = pol_parref $
             else cosmos_anal_run, tmpresult, tmpgpresult, direct3, tmp, instr3, theta3[0], 3, /POL_RUN, tra=tra, f1=f1, f2=f2, pol_lambda = pol_lambda, pol_parref = pol_parref

             if i eq 0 then begin
               ;check if analyzer is in(done only for the first numor)
               ;TS 2015-08-27 Added the check for Analyzer to the third angle
               if tra gt 200. then begin
                 cosmos_logmessage, 'Reflected Beam (angle 3) measured without analyzer.'
                 polOpt = 1
               endif else if tra lt 0.0 then begin
                 cosmos_logmessage, 'TRA < 0 in reflected beam measurement (angle 3). Assuming analyzer is out!'
                 polOpt = 1
               endif else cosmos_logmessage, 'Correcting reflected beam (angle 3) for analyzer efficiency!'
               ;define array to contain the full set of data including different polarizations
               s = size(tmpresult) & gps = size(tmpgpresult)
               case s[0] of
                 2: begin
                      polresult = fltarr(s[1], s[2], 4)
                      gppolresult = fltarr(gps[1], gps[2], 4)
                    end
                 3: begin
                      polresult = fltarr(s[1], s[2], s[3], 4)
                      gppolresult = fltarr(gps[1], gps[2], gps[3], 4)
                    end
                 else: begin
                   cosmos_logmessage, 'Polarized data - Wrong number of dimensions of result returned by cosmos_anal_run!', /warning
                   errorPol = 1
                 end
               endcase

               npoints = s[2]

             endif ;defined polOpt and s from first numor

             ;check dimensions (number of reflectivity points) are consistent
             if n_elements(tmpresult[0,*]) ne npoints then begin
               cosmos_logmessage, 'Polarized data - Number of datapoints is not the same in all the sets!', /warning
               errorPol = 1
             endif

             ;check flipper values are consistent and store results in polresult in the right flipper order [00, 01, 10, 11]
             if not errorPol then begin

               if polOpt eq 1 then begin                    ;no analyzer --> state 0X = index 0, state 1X = index 3
                 if f1 eq 0 then index = 0 else index = 3
               endif else index = 2*f1 + f2                 ;with analyzer --> 0 = 00, 1 = 01, 2 = 10, 3 = 11

               case s[0] of
                 2: begin
                      polresult[*,*,index] = tmpresult
                      gppolresult[*,*,index] = tmpgpresult
                    end
                 3: begin
                      polresult[*,*,*,index] = tmpresult
                      gppolresult[*,*,*,index] = tmpgpresult
                    end
               endcase
             endif

           endfor

           if errorPol then result3 = -1 else begin
             case polOpt of
               1: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_noana Efficiency Correction!'
                 cosmos_logmessage, '!'
                    result3 = cosmos_tofpolcorr_noana(polresult)
                    gpresult3 = cosmos_tofpolcorr_noana(gppolresult)
                  end
               2: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_two Efficiency Correction!'
                 cosmos_logmessage, '!'
                    result3 = cosmos_tofpolcorr_two(polresult)
                    gpresult3 = cosmos_tofpolcorr_two(gppolresult)
                  end
               3: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_three Efficiency Correction!'
                 cosmos_logmessage, '!'
                    result3 = cosmos_tofpolcorr_three(polresult)
                    gpresult3 = cosmos_tofpolcorr_three(gppolresult)
                  end
               4: begin
                 ;TS 2015-08-27 Added a message for me
                 cosmos_logmessage, '!'
                 cosmos_logmessage, '_full Efficiency Correction!'
                 cosmos_logmessage, '!'
                    result3 = cosmos_tofpolcorr_full(polresult)
                    gpresult3 = cosmos_tofpolcorr_full(gppolresult)
                  end
             endcase
           endelse

         endelse ;end treat of polarized data

       endif else begin
         cosmos_logmessage, ' No direct beam for angle 3. Analyse reflectivity using monitor 2!'
         cosmos_anal_run, result3, gpresult3, direct3, reflect3, instr3, theta3[0], 3, /NODIRECT_RUN
       endelse

     endif else result3 = -1
          
     rescheck1 = n_elements(result1) gt 1
     rescheck2 = n_elements(result2) gt 1
     rescheck3 = n_elements(result3) gt 1

     if polOpt eq 0 then begin ;standard case

       ;Check if result1 is a kinetic run. If so check if result2 and result3 contain the same number of time slices.
       res1size = size(result1)
       if res1size[0] eq 3 then begin
          kinetic = 1
          slices = res1size[3]
          if rescheck2 then begin
             res2size = size(result2)
             if (res2size[0] ne 3 or res2size[3] ne slices) then begin
                cosmos_dialogue_warn, 'Different number of time slices in reflect1 and reflect2. Only reflect1 used!'
                rescheck2 = 0
             endif  
          endif    
          if rescheck3 then begin
             res3size = size(result3)
             if (res3size[0] ne 3 or res3size[3] ne slices) then begin
                cosmos_dialogue_warn, 'Different number of time slices in reflect1 and reflect3. Only reflect1 used!'
                rescheck3 = 0
             endif  
          endif    
       endif else begin
          kinetic = 0
          slices = 1
       endelse
     
       for islice = 0,slices-1 do begin
     
          if rescheck1 then res1 = result1[*,*,islice] else res1=-1
          if rescheck2 then res2 = result2[*,*,islice] else res2=-1
          if rescheck3 then res3 = result3[*,*,islice] else res3=-1
          if rescheck1 then gpres1 = gpresult1[*,*,islice]
          if rescheck2 then gpres2 = gpresult2[*,*,islice]
          if rescheck3 then gpres3 = gpresult3[*,*,islice]
  
          factor1 = factor1[0]
          factor2 = factor2[0]
        
          ;Match results
          if rescheck1 and rescheck2 then begin
             if factor1 eq 0. then begin
                temp1 = cosmos_anal_factor(res1, res2)
                temp1g = cosmos_anal_factor(gpres1, gpres2)

                if temp1g[0] ne 0. then begin
                   factor1 = 1.0 / temp1g[0]
                   cosmos_logmessage, ' Auto factor 1-2 taken from grouped data = ' + cosmos_str_make(factor1) + ' (factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
                   cosmos_logmessage, ' '
                endif else begin
                   if temp1[0] ne 0. then begin
                      factor1 = 1.0 / temp1[0]
                      cosmos_logmessage, ' Auto factor 1-2 taken from ungrouped data = ' + cosmos_str_make(factor1) + ' (no overlap in grouped data)'
                      cosmos_logmessage, ' '
                   endif else begin
                      factor1 = 1.0
                      cosmos_logmessage, ' No q overlap between 1st and 2nd runs.', /WARNING
                      cosmos_logmessage, ' '
                   endelse
                endelse
             endif
          endif
    
          if rescheck2 and rescheck3 then begin
             if factor2 eq 0. then begin
                temp1 = cosmos_anal_factor(res2, res3)
                temp1g = cosmos_anal_factor(gpres2, gpres3)
                if temp1g[0] ne 0. then begin
                   factor2 = 1.0 / temp1g[0]
                   cosmos_logmessage, ' Auto factor 2-3 taken from grouped data = ' + cosmos_str_make(factor2) + '(factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
                endif else begin
                   if temp1[0] ne 0. then begin
                      factor2 = 1.0 / temp1[0]
                      cosmos_logmessage, ' Auto factor 2-3 taken from ungrouped data = ' + cosmos_str_make(factor2) + ' (no overlap in grouped data)'
                   endif else begin
                      factor2 = 1.0
                      cosmos_logmessage, ' No q overlap between 2nd and 3rd runs.', /WARNING
                   endelse
                endelse
             endif
          endif
    
          if rescheck1 and rescheck3 then begin
             if (factor1 * factor2) eq 0. then begin
                temp1 = cosmos_anal_factor(res1, res3)
                temp1g = cosmos_anal_factor(gpres1, gpres3)
                factor2 = 1.0
                if temp1g[0] ne 0. then begin
                   factor1 = 1.0 / temp1g[0]
                   cosmos_logmessage, ' Auto factor 1-3 taken from grouped data = ' + cosmos_str_make(factor1) + ' (factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
                endif else begin
                   if temp1[0] ne 0. then begin
                      factor1 = 1.0 / temp1[0]
                      cosmos_logmessage, ' Auto factor 1-3 taken from ungrouped data = ' + cosmos_str_make(factor1) + ' (no overlap in grouped data)'
                   endif else begin
                      factor1 = 1.0
                      cosmos_logmessage, ' No q overlap between 1st and 3rd runs.', /WARNING
                   endelse
                endelse
             endif
          endif
  
          if (factor1 eq 0) then factor1 = 1.0
          c_settings.factor1 = factor1 ;Send value to c_settings for mft output
          if (factor2 eq 0) then factor2 = 1.0
          c_settings.factor2 = factor2 ;Send value to c_settings for mft output
          fact = factor1*factor2

          case (ishft(rescheck1, 2) + ishft(rescheck2, 1) + rescheck3) of
    
             0 : result = -1
      
             1 : begin
                result = res3
                result[0:1,*] = res3[0:1,*]
                result[2:3,*] = fact*res3[2:3,*]
                gpresult = gpres3
                gpresult[0:1,*] = gpres3[0:1,*]
                gpresult[2:3,*] = fact*gpres3[2:3,*]
             end
      
             2 : begin
                result = res2
                result[0:1,*] = res2[0:1,*]
                result[2:3,*] = factor1*res2[2:3,*]
                gpresult = gpres2
                gpresult[0:1,*] = gpres2[0:1,*]
                gpresult[2:3,*] = factor1*gpres2[2:3,*]
             end
      
             3 : begin
                res2[2:3, *] = factor1*res2[2:3, *]
                res3[2:3, *] = fact*res3[2:3, *]
                result = cosmos_anal_append(res2, res3)
                gpres2[2:3, *] = factor1*gpres2[2:3, *]
                gpres3[2:3, *] = fact*gpres3[2:3, *]
                gpresult = cosmos_anal_append(gpres2, gpres3)
                ;set back non-normalized values to gpres2 and gpres3 (needed for /lam output files)
                res2[2:3, *] /= factor1
                res3[2:3, *] /= fact
                gpres2[2:3, *] /= factor1
                gpres3[2:3, *] /= fact
             end
      
             4 : begin
                result = res1
                gpresult = gpres1
             end
      
             5 : begin
                res3[2:3, *] = fact*res3[2:3, *]
                result = cosmos_anal_append(res1, res3)
                gpres3[2:3, *] = fact*gpres3[2:3, *]
                gpresult = cosmos_anal_append(gpres1, gpres3)
                ;set back non-normalized values to gpres3 (needed for /lam output files)
                res3[2:3, *] /= fact
                gpres3[2:3, *] /= fact
             end
      
             6 : begin
                res2[2:3, *] = factor1*res2[2:3, *]
                result = cosmos_anal_append(res1, res2)
                gpres2[2:3, *] = factor1*gpres2[2:3, *]
                gpresult = cosmos_anal_append(gpres1, gpres2)
                ;set back non-normalized values to gpres2 (needed for /lam output files)
                res2[2:3, *] /= factor1
                gpres2[2:3, *] /= factor1
             end
      
             7 : begin
                res2[2:3, *] = factor1*res2[2:3, *]
                res3[2:3, *] = fact*res3[2:3, *]
                result = cosmos_anal_append(cosmos_anal_append(res1, res2), res3)
                gpres2[2:3, *] = factor1*gpres2[2:3, *]
                gpres3[2:3, *] = fact*gpres3[2:3, *]
                gpresult = cosmos_anal_append(cosmos_anal_append(gpres1, gpres2), gpres3)
                ;set back non-normalized values to gpres2 and gpres3 (needed for /lam output files)
                res2[2:3, *] /= factor1
                res3[2:3, *] /= fact
                gpres2[2:3, *] /= factor1
                gpres3[2:3, *] /= fact
             end
      
          else : result = -1
    
          endcase ;end case (ishft(rescheck1, 2) + ishft(rescheck2, 1) + rescheck3) = available angles
  
          if n_elements(result) gt 1 then begin
  
             result = cosmos_anal_sort(result)
             gpresult = cosmos_anal_sort(gpresult)
   
             ;Write unbundled data to .out file
             if kinetic then begin
                if islice eq 0 then begin
                   outfile0 = outfile
                   outfile=file_dirname(outfile0)+path_sep()+'slice'+strtrim(string(islice),2)+'_'+file_basename(outfile0)
                endif else begin
                   outfile=file_dirname(outfile0)+path_sep()+'slice'+strtrim(string(islice),2)+'_'+file_basename(outfile0)
                endelse
             endif
  
             if c_settings.outputfmt[3] eq 1 then begin
                cosmos_file_write, outfile, result, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /out
             endif
                       
             ;Group data
             ;MAG: Here I use the grouping variables defined for the first angle. Not sure what to do if
             ;different values used for each angle. TO CHECK THIS!
             ;Note that the call to cosmos_anal_bundle was already commented in Cosmos, so the only decision here
             ;is to call or not the cosmos_anal_discard procedure.
             if c_settings.group eq 1 then begin
                result = gpresult
                if c_settings.groupdiscard ne 0 then cosmos_anal_discard, result
             endif
    
             ;Multiply data by overall normalisation factor
             if nurm[0] eq 0. then nurm[0] = 1.0
             c_settings.normvar = nurm[0] ;Send the normalization value to c_settings for mft output
             cosmos_logmessage, ' Normalization factors used :'
             cosmos_logmessage, '  Normalization Reflect 1 = ' + cosmos_str_make(nurm[0])
             cosmos_logmessage, '  Factor 1-2 = ' + cosmos_str_make(factor1)
             cosmos_logmessage, '  Factor 2-3 = ' + cosmos_str_make(factor2)
             cosmos_logmessage, ' '
        
             result[2,*] = result[2,*] * nurm[0]
             result[3,*] = result[3,*] * nurm[0]
  
             if slices eq 1 then begin
                tagname = 'Row ' + cosmos_str_make(c_params.row+1)
             endif else begin
                tagname = 'Row ' + cosmos_str_make(c_params.row+1) + ' time slice ' + cosmos_str_make(islice+1)          
             endelse
        
             if reflect1[0] eq 0 then reflect1 = '' ;Comparison also valid for empty string
             if reflect2[0] eq 0 then reflect2 = ''
             if reflect3[0] eq 0 then reflect3 = ''
             arr = [reflect1[0], reflect2[0], reflect3[0]]
             numor = where(strlen(arr) ne 0)
             numor = arr[numor[0]]
             tagname = tagname + '$' + cosmos_str_make(numor)
             c_results = create_struct(c_results, tagname, result)
          
             ;Write bundled data to file(s) and logged bundled data to lamp
             ;AFT
             if c_settings.outputfmt[0] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /aft
             ;DAT
             if c_settings.outputfmt[1] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /dat
             ;MFT
             if c_settings.outputfmt[2] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /mft
             ;Lambda format --> one different file per angle
             if c_settings.outputfmt[4] eq 1 then begin            
                if c_settings.group eq 1 then begin
                   if rescheck1 then begin
                      arr = gpres1
                      arr[2,*] = arr[2,*]*nurm[0]
                      arr[3,*] = arr[3,*]*nurm[0]
                      outfilelam = outfile + '_angle1'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                   if rescheck2 then begin
                      arr = gpres2
                      arr[2,*] = arr[2,*]*nurm[0]*factor1
                      arr[3,*] = arr[3,*]*nurm[0]*factor1
                      outfilelam = outfile + '_angle2'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                   if rescheck3 then begin
                      arr = gpres3
                      arr[2,*] = arr[2,*]*nurm[0]*factor1*factor2
                      arr[3,*] = arr[3,*]*nurm[0]*factor1*factor2
                      outfilelam = outfile + '_angle3'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                endif else begin
                   if rescheck1 then begin
                      arr = res1
                      arr[2,*] = arr[2,*]*nurm[0]
                      arr[3,*] = arr[3,*]*nurm[0]
                      outfilelam = outfile + '_angle1'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                   if rescheck2 then begin
                      arr = res2
                      arr[2,*] = arr[2,*]*nurm[0]*factor1
                      arr[3,*] = arr[3,*]*nurm[0]*factor1
                      outfilelam = outfile + '_angle2'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                   if rescheck3 then begin
                      arr = res3
                      arr[2,*] = arr[2,*]*nurm[0]*factor1*factor2
                      arr[3,*] = arr[3,*]*nurm[0]*factor1*factor2
                      outfilelam = outfile + '_angle3'
                      cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, reflect1, reflect2, reflect3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                   endif
                endelse
             endif ;end writing lambda format
              
             resultlog = cosmos_anal_log(result)
             
             if resultlog[0, 0] ge 0.0 then begin
                if c_params.rowfinish ge 500 then $
                  cosmos_lamp_write, outlamp[0], resultlog, file_basename(outfile) else $
                  cosmos_lamp_write, outlamp[0], resultlog, file_basename(outfile), /saveall
             endif else begin
                cosmos_logmessage, 'Log dataset for Lamp empty.', /WARNING
             endelse
    
          endif ;end if n_elements(result) gt 1 --> output files
  
       endfor ;end for islice = 0,slices-1
     
     endif else begin ;merging and writing of polarized data

       ;Check if result1 is a kinetic run. If so check if result2 and result3 contain the same number of time slices.
       res1size = size(result1)
       if res1size[0] eq 4 then begin
         kinetic = 1
         slices = res1size[3]
         if rescheck2 then begin
           res2size = size(result2)
           if (res2size[0] ne 4 or res2size[3] ne slices) then begin
             cosmos_dialogue_warn, 'Different number of time slices in reflect1 and reflect2. Only reflect1 used!'
             rescheck2 = 0
           endif
         endif
         if rescheck3 then begin
           res3size = size(result3)
           if (res3size[0] ne 4 or res3size[3] ne slices) then begin
             cosmos_dialogue_warn, 'Different number of time slices in reflect1 and reflect3. Only reflect1 used!'
             rescheck3 = 0
           endif
         endif
       endif else begin
         kinetic = 0
         slices = 1
       endelse

       polstring = ['00', '01', '10', '11']

       for ipol = 0, 3 do begin

         ;TS 2016-10-21 COSMOS was writing too many files if only two channels were measured. For THREE it is OK since I am calculating the remaining in the correction procedure.
         if polopt le 2 then begin
           if ipol ne 0 then ipol=3
           if ipol ne 0 then cosmos_logmessage, 'Skipped writing of Pol[01] and POL[10]', /WARNING
         endif
         ;TS 2016-10-21 END: COSMOS was writing too many files if only two channels were measured

         for islice = 0,slices-1 do begin

           if kinetic then begin
             if rescheck1 then res1 = result1[*,*,islice,ipol] else res1=-1
             if rescheck2 then res2 = result2[*,*,islice,ipol] else res2=-1
             if rescheck3 then res3 = result3[*,*,islice,ipol] else res3=-1
             if rescheck1 then gpres1 = gpresult1[*,*,islice,ipol]
             if rescheck2 then gpres2 = gpresult2[*,*,islice,ipol]
             if rescheck3 then gpres3 = gpresult3[*,*,islice,ipol]
           endif else begin
             if rescheck1 then res1 = result1[*,*,ipol] else res1=-1
             if rescheck2 then res2 = result2[*,*,ipol] else res2=-1
             if rescheck3 then res3 = result3[*,*,ipol] else res3=-1
             if rescheck1 then gpres1 = gpresult1[*,*,ipol]
             if rescheck2 then gpres2 = gpresult2[*,*,ipol]
             if rescheck3 then gpres3 = gpresult3[*,*,ipol]
           endelse

           factor1 = factor1[0]
           factor2 = factor2[0]

           ;Match results
           if rescheck1 and rescheck2 then begin
             if factor1 eq 0. then begin
               temp1 = cosmos_anal_factor(res1, res2)
               temp1g = cosmos_anal_factor(gpres1, gpres2)
               if temp1g[0] ne 0. then begin
                 factor1 = 1.0 / temp1g[0]
                 cosmos_logmessage, ' Auto factor 1-2 taken from grouped data = ' + cosmos_str_make(factor1) + ' (factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
                 cosmos_logmessage, ' '
               endif else begin
                 if temp1[0] ne 0. then begin
                   factor1 = 1.0 / temp1[0]
                   cosmos_logmessage, ' Auto factor 1-2 taken from ungrouped data = ' + cosmos_str_make(factor1) + ' (no overlap in grouped data)'
                   cosmos_logmessage, ' '
                 endif else begin
                   factor1 = 1.0
                   cosmos_logmessage, ' No q overlap between 1st and 2nd runs.', /WARNING
                   cosmos_logmessage, ' '
                 endelse
               endelse
             endif
           endif

           if rescheck2 and rescheck3 then begin
             if factor2 eq 0. then begin
               temp1 = cosmos_anal_factor(res2, res3)
               temp1g = cosmos_anal_factor(gpres2, gpres3)
               if temp1g[0] ne 0. then begin
                 factor2 = 1.0 / temp1g[0]
                 cosmos_logmessage, ' Auto factor 2-3 taken from grouped data = ' + cosmos_str_make(factor2) + '(factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
               endif else begin
                 if temp1[0] ne 0. then begin
                   factor2 = 1.0 / temp1[0]
                   cosmos_logmessage, ' Auto factor 2-3 taken from ungrouped data = ' + cosmos_str_make(factor2) + ' (no overlap in grouped data)'
                 endif else begin
                   factor2 = 1.0
                   cosmos_logmessage, ' No q overlap between 2nd and 3rd runs.', /WARNING
                 endelse
               endelse
             endif
           endif

           if rescheck1 and rescheck3 then begin
             if (factor1 * factor2) eq 0. then begin
               temp1 = cosmos_anal_factor(res1, res3)
               temp1g = cosmos_anal_factor(gpres1, gpres3)
               factor2 = 1.0
               if temp1g[0] ne 0. then begin
                 factor1 = 1.0 / temp1g[0]
                 cosmos_logmessage, ' Auto factor 1-3 taken from grouped data = ' + cosmos_str_make(factor1) + ' (factor from ungrouped data = ' + cosmos_str_make(1.0 / temp1[0]) + ')'
               endif else begin
                 if temp1[0] ne 0. then begin
                   factor1 = 1.0 / temp1[0]
                   cosmos_logmessage, ' Auto factor 1-3 taken from ungrouped data = ' + cosmos_str_make(factor1) + ' (no overlap in grouped data)'
                 endif else begin
                   factor1 = 1.0
                   cosmos_logmessage, ' No q overlap between 1st and 3rd runs.', /WARNING
                 endelse
               endelse
             endif
           endif

           if (factor1 eq 0) then factor1 = 1.0
           c_settings.factor1 = factor1 ;Send value to c_settings for mft output
           if (factor2 eq 0) then factor2 = 1.0
           c_settings.factor2 = factor2 ;Send value to c_settings for mft output
           fact = factor1*factor2

           case (ishft(rescheck1, 2) + ishft(rescheck2, 1) + rescheck3) of

             0 : result = -1

             1 : begin
               result = res3
               result[0:1,*] = res3[0:1,*]
               result[2:3,*] = fact*res3[2:3,*]
               gpresult = gpres3
               gpresult[0:1,*] = gpres3[0:1,*]
               gpresult[2:3,*] = fact*gpres3[2:3,*]
             end

             2 : begin
               result = res2
               result[0:1,*] = res2[0:1,*]
               result[2:3,*] = factor1*res2[2:3,*]
               gpresult = gpres2
               gpresult[0:1,*] = gpres2[0:1,*]
               gpresult[2:3,*] = factor1*gpres2[2:3,*]
             end

             3 : begin
               res2[2:3, *] = factor1*res2[2:3, *]
               res3[2:3, *] = fact*res3[2:3, *]
               result = cosmos_anal_append(res2, res3)
               gpres2[2:3, *] = factor1*gpres2[2:3, *]
               gpres3[2:3, *] = fact*gpres3[2:3, *]
               gpresult = cosmos_anal_append(gpres2, gpres3)
               ;set back non-normalized values to gpres2 and gpres3 (needed for /lam output files)
               res2[2:3, *] /= factor1
               res3[2:3, *] /= fact
               gpres2[2:3, *] /= factor1
               gpres3[2:3, *] /= fact
             end

             4 : begin
               result = res1
               gpresult = gpres1
             end

             5 : begin
               res3[2:3, *] = fact*res3[2:3, *]
               result = cosmos_anal_append(res1, res3)
               gpres3[2:3, *] = fact*gpres3[2:3, *]
               gpresult = cosmos_anal_append(gpres1, gpres3)
               ;set back non-normalized values to gpres3 (needed for /lam output files)
               res3[2:3, *] /= fact
               gpres3[2:3, *] /= fact
             end

             6 : begin
               res2[2:3, *] = factor1*res2[2:3, *]
               result = cosmos_anal_append(res1, res2)
               gpres2[2:3, *] = factor1*gpres2[2:3, *]
               gpresult = cosmos_anal_append(gpres1, gpres2)
               ;set back non-normalized values to gpres2 (needed for /lam output files)
               res2[2:3, *] /= factor1
               gpres2[2:3, *] /= factor1
             end

             7 : begin
               res2[2:3, *] = factor1*res2[2:3, *]
               res3[2:3, *] = fact*res3[2:3, *]
               result = cosmos_anal_append(cosmos_anal_append(res1, res2), res3)
               gpres2[2:3, *] = factor1*gpres2[2:3, *]
               gpres3[2:3, *] = fact*gpres3[2:3, *]
               gpresult = cosmos_anal_append(cosmos_anal_append(gpres1, gpres2), gpres3)
               ;set back non-normalized values to gpres2 and gpres3 (needed for /lam output files)
               res2[2:3, *] /= factor1
               res3[2:3, *] /= fact
               gpres2[2:3, *] /= factor1
               gpres3[2:3, *] /= fact
             end

             else : result = -1

           endcase ;end case (ishft(rescheck1, 2) + ishft(rescheck2, 1) + rescheck3) = available angles

           if n_elements(result) gt 1 then begin

             result = cosmos_anal_sort(result)
             gpresult = cosmos_anal_sort(gpresult)

             ;Write unbundled data to .out file
             if kinetic then begin
               if islice eq 0 then begin
                 if ipol eq 0 then outfile0 = outfile
                 outfile=file_dirname(outfile0)+path_sep()+'pol'+polstring[ipol]+'_slice'+strtrim(string(islice),2)+'_'+file_basename(outfile0)
               endif else begin
                 outfile=file_dirname(outfile0)+path_sep()+'pol'+polstring[ipol]+'_slice'+strtrim(string(islice),2)+'_'+file_basename(outfile0)
               endelse
             endif else begin
               if ipol eq 0 then outfile0 = outfile
               outfile=file_dirname(outfile0)+path_sep()+'pol'+polstring[ipol]+'_'+file_basename(outfile0)
             endelse

             if c_settings.outputfmt[3] eq 1 then begin
               ;MG: to think: cannot send numor string because order may be different and also not enough
               ;number of numors, as here I always have 4 polarizations (by replicating polarizations)
               ;but number of numors can be also 2 or 3 instead of 4
               ;if n_elements(reflect1) gt 1 then rrr1 = reflect1[ipol] else rrr1 = reflect1
               ;if n_elements(reflect2) gt 1 then rrr2 = reflect2[ipol] else rrr2 = reflect2
               ;if n_elements(reflect3) gt 1 then rrr3 = reflect3[ipol] else rrr3 = reflect3
               if n_elements(reflect1) gt 1 then rrr1 = reflect1[0] else rrr1 = reflect1
               if n_elements(reflect2) gt 1 then rrr2 = reflect2[0] else rrr2 = reflect2
               if n_elements(reflect3) gt 1 then rrr3 = reflect3[0] else rrr3 = reflect3
               cosmos_file_write, outfile, result, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /out
             endif

             ;Group data
             ;MAG: Here I use the grouping variables defined for the first angle. Not sure what to do if
             ;different values used for each angle. TO CHECK THIS!
             ;Note that the call to cosmos_anal_bundle was already commented in Cosmos, so the only decision here
             ;is to call or not the cosmos_anal_discard procedure.
             if c_settings.group eq 1 then begin
               result = gpresult
               if c_settings.groupdiscard ne 0 then cosmos_anal_discard, result
             endif

             ;Multiply data by overall normalisation factor
             if nurm[0] eq 0. then nurm[0] = 1.0
             c_settings.normvar = nurm[0] ;Send the normalization value to c_settings for mft output
             cosmos_logmessage, ' Normalization factors used :'
             cosmos_logmessage, '  Normalization Reflect 1 = ' + cosmos_str_make(nurm[0])
             cosmos_logmessage, '  Factor 1-2 = ' + cosmos_str_make(factor1)
             cosmos_logmessage, '  Factor 2-3 = ' + cosmos_str_make(factor2)
             cosmos_logmessage, ' '

             result[2,*] = result[2,*] * nurm[0]
             result[3,*] = result[3,*] * nurm[0]

             if slices eq 1 then begin
               tagname = 'Row ' + cosmos_str_make(c_params.row+1)
             endif else begin
               tagname = 'Row ' + cosmos_str_make(c_params.row+1) + ' time slice ' + cosmos_str_make(islice+1)
             endelse

             ;TS 23/06/2016:: Problem that reflect1[0] needs to be a 1 elment array, but in case that there are more runnubmers, it has more
             cosmos_logmessage, 'TSNEW ==== NN reflect1 ' + cosmos_str_make(reflect1[0])
             if reflect1[0,0] eq 0 then reflect1 = '' ;Comparison also valid for empty string
             if reflect2[0,0] eq 0 then reflect2 = ''
             if reflect3[0,0] eq 0 then reflect3 = ''
             arr = [reflect1[0], reflect2[0], reflect3[0]]
             numor = where(strlen(arr) ne 0)
             numor = arr[numor[0]]
             tagname = tagname + '$' + cosmos_str_make(numor) + '$Pol' + cosmos_str_make(ipol)
             c_results = create_struct(c_results, tagname, result)

             ;MG: to think: cannot send numor string because order may be different and also not enough
             ;number of numors, as here I always have 4 polarizations (by replicating polarizations)
             ;but number of numors can be also 2 or 3 instead of 4
             ;if n_elements(reflect1) gt 1 then rrr1 = reflect1[ipol] else rrr1 = reflect1
             ;if n_elements(reflect2) gt 1 then rrr2 = reflect2[ipol] else rrr2 = reflect2
             ;if n_elements(reflect3) gt 1 then rrr3 = reflect3[ipol] else rrr3 = reflect3
             if n_elements(reflect1) gt 1 then rrr1 = reflect1[0] else rrr1 = reflect1
             if n_elements(reflect2) gt 1 then rrr2 = reflect2[0] else rrr2 = reflect2
             if n_elements(reflect3) gt 1 then rrr3 = reflect3[0] else rrr3 = reflect3

             ;Write bundled data to file(s) and logged bundled data to lamp
             ;AFT
             if c_settings.outputfmt[0] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /aft
             ;DAT
             if c_settings.outputfmt[1] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /dat
             ;MFT
             if c_settings.outputfmt[2] eq 1 then cosmos_file_write, outfile, result, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /mft
             ;Lambda format --> one different file per angle
             if c_settings.outputfmt[4] eq 1 then begin
               if c_settings.group eq 1 then begin
                 if rescheck1 then begin
                   arr = gpres1
                   arr[2,*] = arr[2,*]*nurm[0]
                   arr[3,*] = arr[3,*]*nurm[0]
                   outfilelam = outfile + '_angle1'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
                 if rescheck2 then begin
                   arr = gpres2
                   arr[2,*] = arr[2,*]*nurm[0]*factor1
                   arr[3,*] = arr[3,*]*nurm[0]*factor1
                   outfilelam = outfile + '_angle2'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
                 if rescheck3 then begin
                   arr = gpres3
                   arr[2,*] = arr[2,*]*nurm[0]*factor1*factor2
                   arr[3,*] = arr[3,*]*nurm[0]*factor1*factor2
                   outfilelam = outfile + '_angle3'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
               endif else begin
                 if rescheck1 then begin
                   arr = res1
                   arr[2,*] = arr[2,*]*nurm[0]
                   arr[3,*] = arr[3,*]*nurm[0]
                   outfilelam = outfile + '_angle1'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
                 if rescheck2 then begin
                   arr = res2
                   arr[2,*] = arr[2,*]*nurm[0]*factor1
                   arr[3,*] = arr[3,*]*nurm[0]*factor1
                   outfilelam = outfile + '_angle2'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
                 if rescheck3 then begin
                   arr = res3
                   arr[2,*] = arr[2,*]*nurm[0]*factor1*factor2
                   arr[3,*] = arr[3,*]*nurm[0]*factor1*factor2
                   outfilelam = outfile + '_angle3'
                   cosmos_file_write, outfilelam, arr, direct1, direct2, direct3, rrr1, rrr2, rrr3, instr1, instr2, instr3, theta1, theta2, theta3, water_file, /lam
                 endif
               endelse
             endif ;end writing lambda format

             resultlog = cosmos_anal_log(result)

             if resultlog[0, 0] ge 0.0 then begin
               ;TS 2016-10-21 Changed where the pol files get written
               if polopt le 2 then begin
                 if ipol eq 0 then $
                   cosmos_lamp_write, outlamp[0], resultlog, file_basename(outfile)
                 if ipol eq 3 then $
                   cosmos_lamp_write, outlamp[0]+1, resultlog, file_basename(outfile)
               endif else begin
                 if ipol eq 0 then $
                   cosmos_lamp_write, outlamp[0], resultlog, file_basename(outfile)
                 if ipol eq 1 then $
                   cosmos_lamp_write, outlamp[0]+1, resultlog, file_basename(outfile)
                 if ipol eq 2 then $
                   cosmos_lamp_write, outlamp[0]+2, resultlog, file_basename(outfile)
                 if ipol eq 3 then $
                   cosmos_lamp_write, outlamp[0]+3, resultlog, file_basename(outfile)
               endelse
               ;TS 2016-10-21 END:: Changed where the pol files get written
             endif else begin
               cosmos_logmessage, 'Log dataset for Lamp empty. --> POL:: ' + cosmos_str_make(ipol) , /WARNING ;TS 2016-10-21 Identify the pol channels that are empty since the message pops up if only two or three channels measured
             endelse

           endif ;end if n_elements(result) gt 1 --> output files

         endfor ;end for islice = 0,slices-1
       endfor ;end for ipol = 0,3

     endelse
     
     t_elapsed = systime(1) - t_ini
     cosmos_logmessage, 'Row treated in ' + cosmos_str_make(t_elapsed) + ' seconds'
     ;print, c_params.row, t_elapsed, systime(1) - 1427188740.
    
  endif ;end if (c_params.row lt c_params.rowfinish)
  
  c_params.row = c_params.row + 1

end


pro cosmos_analysis
  
  return
  
end
