;+
;d.R. and M.A.G started april 2007
;Por MIGUEL see:  sans_cor , sans_radial , sans_settings_event(case 72) , sans_settings_runs
;
;sans_merge_fields      Show/Hide fields of the merging tool
;sans_merge_find_scal   Determine Qfct and margins between Q vectors of two configs (called by sans_merge_find_scaling)
;sans_merge_in_files    Having Qfactors and margins, merge all configs in files
;sans_merging           Handle GUI and Batch mode merging
;sans_sav_rest_results  Save/Restore and merge in/from binary idl file_result.sav (call sans_merge_results for that)
;sans_accu              Do operations between Q vectors
;sans_settings_swap     Swap between low middle high configs, share commons for 3 configs used also by sans_sheet
;sans_settings_choice_water Choice between alternatives water runs
;sans_settings_struc    Modify a structure when size of tag value is different
;sans_settings_beamST   Toggle between direct beam and transm beam (historical confusion) used by sans_sheet
;sans_settings_event    Event GUI Handler used also by sans_sheet
;sans-settings_model    Use STR_FIT to overplot a model
;sans_settings_read     Import settings from a sav file
;sans_settings_write    Export settings in a sav file
;sans_settings_text     Check gui input fields
;sans_settings_trans    Check sans_trans and fill thick or transm, used by sans_sheet
;sans_settings_dist     Get  cells distances from center
;sans_settings_getmask  Get  current mask as a binary matrix
;sans_settings_GetMaskBeam Give sans_beam  to sans_sheet
;sans_settings_GetTrans    Give sans_trans to sans_sheet
;sans_settings_GetSupl     Give sans_supl  to sans_sheet
;sans_settings_mask     Show current mask and center
;sans_settings_series   Format a list into lamp series syntax, used mainly by sans_sheet
;sans_settings_phi0     Calculate the flux from the transmission beam
;sans_settings_runs_live   Read runs for batch process
;sans_settings_runs        Read runs for the GUI
;sans_settings_par      Keep some data parameters in datp from current read run
;sans_settings_WNI      Keep info on actually loaded workspaces
;sans_settings_current  Fill current settings into common block C_SANS_SETTINGS / put memory data into workspaces
;sans_settings          GUI constructor
;
;independent functions
;*********************
;sans_center            return center
;sans_cor               return corrected data
;sans_radial            return integrations
;sans_read_grasp        return grasp output file
;sans_read_gt           return gt (ron) file
;sans_transm            return transmissions
;sans_sigmaQ            return sigma Q
;
;independent procedures
;**********************
;sans_tau               dead time correction
;sans_mask              set instrument masks
;sans_read_mask         read a saved instrument mask
;sans_read_settings     will be used (batch)
;sans_reform_series     keep transmissions and results
;sans_save_gt           save gt (ron) files
;sans_save_Qs           save 2D Qvalues
;sans_sector            interface with bens.pro
;sans_write_settings    will be used (batch)
;
;***************************************************************************************************************
;***************************************************************************************************************
;-

function sans_center, W_in, beam=nbeam, Xrange=xrange, Yrange=yrange, Fit=fit, Gravity=gravity, Mask=M, Center=Cxy, $
                      watermask=watermask,ewatw=ewatw, samplemask=samplemask, no_ctr=no_ctr
;+
;******* ***********
;** User callable
;** Return the beam center coordinates from a 2D (regular) transmission data run.
;** Either use str_fit (/fit) or/and calculate the gravity center (/gravity).
;** Optional xrange=[1,n] and yrange=[1,m] in pixel unit
;** Take care of Errors and Mask when passed
;**(This function is also called by "TRAPC")
;** watermask=1,2,3 -> auto calculation of water  beam mask (we have only water in W_in with a beamstop in place)
;**                    ewatw is an optional empty water cell.
;** samplemask  -> auto calculation of sample beam mask (W_in is the direct beam , samplemask is sample with a beamstop in place)
;**                D.R. Feb 2009
;**                D.R  May 2014 accept a structure from a batch command C = sans_center({BeamTrC:'99', FLUX:'26'}, /MASK)
;-

;sans_supl[1] is the sample-detector distance used to calculate the beam width sample(for sigmaQ)

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

Cxy =sans_ctr[0:1] & eW=0
if n_elements(W_in) eq 0 then W_in=0 & siz =size(W_in)

if siz[siz[0]+1] eq 8 then begin     ;we have a structure, so it is a batch command
   sans_settings_runs_live, W_in     ;read the beam run in correct workspace
   return, sans_center(beam=9, Xrange=xrange, Yrange=yrange, Fit=fit, Gravity=gravity, Mask=M)
endif


if siz[0] eq 2 then TAKE_DATP,datp $
else if n_elements(nbeam) eq 1 then begin & TAKE_W,W_in,W=nbeam & TAKE_DATP,datp ,W=nbeam & siz=size(W_in) & endif
if      n_elements(ewatw) eq 1 then begin & TAKE_W,eW  ,W=ewatw & TAKE_DATP,datpe,W=ewatw                  & endif

if siz[0] eq 2 then begin
   Win=W_in
   if n_elements(sans_supl)   le 1 then sans_settings,/nowind  ; init sans_settings   ! (unprobable case)
   if            sans_supl[0] eq 0 then sans_settings_par,datp ; get  data parameters ! (unprobable case)

   FX      =long(sans_supl[30])
   supl49 = 0
   tau=sans_supl[6]*sans_supl[27] & ftau=sans_supl[7] & tim=datp.p[sans_supl[85]] ; tim=sans_supl[23]
  ;if sans_supl[66] gt 1 then tau = 0. ;disable dead times whith multi-panels data (commented for Tbeam!)
   if tau*tim gt 0. then sans_tau, Win ,0 ,ftau ,tau ,tim ;dead times correction

   if not keyword_set(fit) then fit    =0
   if not keyword_set(fit) then gravity=1 ;Set a default

   if (n_elements(samplemask) eq 0) and (n_elements(watermask) ne 1) then Dtheta=1 else Dtheta=0 ;keeping beam width (radian)

   keepX=findgen(siz[1])+1. & keepY=findgen(siz[2])+1.  ;keepX=datp.X(*,0) & keepY=datp.Y(*,0)
       X=keepX              &     Y=keepY
   E=datp.E
   if n_elements(E) ne n_elements(Win) then if sans_supl[66] gt 1 then E=sqrt(Win>0.) else E=sqrt(Win) ;NegMask!

   if n_elements(M) ne n_elements(Win) then if n_elements(M) eq 1 then $
              if M  eq 1 then sans_settings_getmask,siz,[0.,0.],M,/nobeam,/noWbeam ;Detector only
   if n_elements(M) ne n_elements(Win) then Mok=0 else Mok=1

   if Mok then if max(M) eq 1 then begin ;TAKE CARE OF THE DETECTOR MASK
       Em =max(E)
       idx=where(M eq 1, complement=icx) ;& icx=where(M eq 0)
       E[idx]=Em*100.   ;Increase error of masked data
       avg=total(Win[icx])/n_elements(Win[icx]) & Win[idx]=avg
       if n_elements(ewatw) eq 1 then begin
       ave=total(eW[ icx])/n_elements(ew[ icx]) & eW[ idx]=ave & endif
   endif else Mok=0

   if not Mok then avg=total(Win)/n_elements(Win)

   s1=siz[1]-1 & s2=siz[2]-1 & a1=0 & a2=0 & b1=0 & b2=0 & slope=0
   ;if FX eq -1 we have added 1 in xyrange (bad for negative values)
   ;if FX eq  0 we have correct index+1
   if n_elements(xrange) eq 2 then begin & if xrange[0] gt 0 then a1=(xrange[0]-1)  ;care fixed negative values
                                         if xrange[0] lt 0 then a1=(xrange[0]+1+2*FX)
                                         if xrange[1] lt 0 then a2=(xrange[0]+1+2*FX)
                                         if xrange[1] gt 0 then a2=(xrange[1]-1) & endif
   if n_elements(yrange) eq 2 then begin & if yrange[0] gt 0 then b1=(yrange[0]-1) ;care fixed negative values
                                         if yrange[0] lt 0 then b1=(yrange[0]+1+2*FX)
                                         if yrange[1] lt 0 then b2=(yrange[0]+1+2*FX)
                                         if yrange[1] gt 0 then b2=(yrange[1]-1) & endif
   if (  (a1*a2*b1*b2 ne 0.) and (a1 eq a2) and (b1 eq b2)) $
   or ((((a1 eq 0) and (a2 ne 0)) or ((a2 eq 0) and (a1 ne 0)))  and  $
       (((b1 eq 0) and (b2 ne 0)) or ((b2 eq 0) and (b1 ne 0)))) then begin  ;We have a center fixed by the user!!!
      if abs(a2) gt abs(a1) then a1=a2
      if abs(b2) gt abs(b1) then b1=b2
     ;sans_supl(i:j)=[a1,b1]
      Cxy=[a1,b1]
      if a1 lt 0 then Cxy[0]=Cxy[0]-1 else Cxy[0]=Cxy[0]+1 ;as for sans_ctr
      if b1 lt 0 then Cxy[1]=Cxy[1]-1 else Cxy[1]=Cxy[1]+1
      fit=0 & gravity=0 & Dtheta=0 & nX0=0 & nY0=0
   endif else begin
      a1=a1>0. & b1=b1>0.
      a2=a2<s1 & b2=b2<s2
      if a2 le a1 then begin & a1=0 & a2=s1 & endif
      if b2 le b1 then begin & b1=0 & b2=s2 & endif

      Xproj=total(Win[a1:a2,b1:b2],2) & X=X[a1:a2] & EXproj=sqrt(total(E[a1:a2,b1:b2]^2,2)) & nX0=n_elements(X)-1
      Yproj=total(Win[a1:a2,b1:b2],1) & Y=Y[b1:b2] & EYproj=sqrt(total(E[a1:a2,b1:b2]^2,1)) & nY0=n_elements(Y)-1
   endelse

   if (nX0 lt 4) or (nY0 lt 4) then begin
      Dtheta=0 & if fit then gravity=1 & fit=0 & endif ;No possibility to fit!!!

   if n_elements(watermask) eq 1 then begin & slope=2
      Xsproj=total(Win,2) & Ysproj=total(Win,1)
      Xproj =abs(total(win-shift(win,0,-2) ,2)) & Xproj[s1-1:*]=0 ;Derivative anhence the hole
      Yproj =abs(total(win-shift(win,-2,0) ,1)) & Yproj[s2-1:*]=0
      Xproj =smooth(Xproj,3) & Yproj=smooth(Yproj,3) & Xproj[0]=0 & Yproj[0]=0
      if n_elements(ewatw) eq 1 then begin                        ;We have an empty cell
         ;if sans_trans(0,0)*sans_trans(1,0) gt 0. then trae=sans_trans(0,0)/sans_trans(1,0) else trae=1.
         ;wen=win - trae *eW *datp.N(0)/datpe.N(0)
         ;wen=wen - total(Wen)/n_elements(Wen)
      endif ;else wen=win-avg
      win=win-avg
   endif

   if n_elements(samplemask) eq n_elements(Win) then begin
      Xsproj=total(samplemask,2)
      Ysproj=total(samplemask,1) & endif

  ;Gravity
   if keyword_set(gravity) then begin
      Xweight=Xproj*0.+1. & Yweight=Yproj*0.+1.
      if Mok then begin
         idx=where(EXproj gt 0.) & Xweight[idx]=Xproj[idx]/(EXproj[idx])^2
         idx=where(EYproj gt 0.) & Yweight[idx]=Yproj[idx]/(EYproj[idx])^2
      endif
      Cxy[0]=total(Xproj*Xweight*X)/(total(Xproj*Xweight))
      Cxy[1]=total(Yproj*Yweight*Y)/(total(Yproj*Yweight))
   endif

  ;Fit a gaussian with a flat background (quad for water mask?)
   if keyword_set(fit) or (Dtheta) then begin
      Xpar = str_fit(Xproj,Bkgd=slope,Xin=X,Ein=EXproj,win=-1,/gauss);,/p2_mod);,p_extra=1) ;win=-1 to not write str_params.txt
      Ypar = str_fit(Yproj,Bkgd=slope,Xin=Y,Ein=EYproj,win=-1,/gauss);,/p2_mod);,p_extra=1)
      if keyword_set(fit) then begin
         Cxyf =[Xpar[2],Ypar[2]]
         if keyword_set(gravity) then Cxy=(Cxy+Cxyf)/2. else Cxy=Cxyf ;(Fit & Gravity) or Fit or gravity
      endif
     ;if Dtheta then supl49=atan(sqrt(total((([Xpar(3),Ypar(3)]/2.)*sans_supl(8:9))^2))/sqrt(2)/(sans_supl(1)*100.)) ;beam width (radian)
     ;resultX = gaussfit(X, Xproj, paramX, nterms=4)
     ;resultY = gaussfit(Y, Yproj, paramY, nterms=4)
      if Dtheta then supl49=atan(     total(  [Xpar[3],Ypar[3]]/2. *sans_supl[8:9]) /2./2.3548 /(sans_supl[1]*100.)) ;beam width (radian)

   endif


   if nX0*nY0 gt 0 then begin
      a1=where(keepX ge Cxy[0])>0 & a1=a1[0] & a2=(a1-1)>0
      b1=where(keepY ge Cxy[1])>0 & b1=b1[0] & b2=(b1-1)>0
      q1=abs(keepX[a1]- Cxy[0])   & q2=abs(keepX[a2]-Cxy[0]) & a1=round((a1-q1/(q1+q2))*100.)/100.
      q1=abs(keepY[b1]- Cxy[1])   & q2=abs(keepY[b2]-Cxy[1]) & b1=round((b1-q1/(q1+q2))*100.)/100.
      if (X[1] eq round(X[1]))  and  (Y[1] eq round(Y[1])) then Cxy=round(Cxy*100.)/100.
   endif

   ; ------------
   if keyword_set(fit) then begin ;Unused
    if n_elements(watermask) eq 1  then f=0.75 else f=2.
    ma =Xpar[9]/Xpar[1]*f    & mb =Ypar[9]/Ypar[1]*f            ;Fct_integral/High  or  ma=Xpar(3) & mb=Ypar(3) ;FWMH
    ma1=floor(Cxy[0]-ma)>0   & ma2=ceil(Cxy[0]+ma-ma1+1)        ;Auto mask X suggested
    mb1=floor(Cxy[1]-mb)>0   & mb2=ceil(Cxy[1]+mb-mb1+1)        ;Auto mask Y suggested
   endif

   if (n_elements(watermask) eq 1) or (n_elements(samplemask) ge 1) then begin          ;AUTO_MASK ASKED FOR WATER OR SAMPLE
      if n_elements(Xsproj) gt 1  then begin
                 devx1 =Xsproj-shift(Xsproj,-1)             & devx2 =Xsproj-shift(Xsproj,+1)  ;Take highest derivatives left,right
                 devy1 =Ysproj-shift(Ysproj,-1)             & devy2 =Ysproj-shift(Ysproj,+1)
                 maxx1 =max(devx1[3:ceil(Cxy[0]<s1)],ix1)   & maxx2 =max(devx2[floor(Cxy[0]>0):s1-3],ix2)
                 maxy1 =max(devy1[3:ceil(Cxy[1]<s2)],iy1)   & maxy2 =max(devy2[floor(Cxy[1]>0):s2-3],iy2)
                 ix1=ix1+1 & ix2=floor(Cxy[0]>0)+ix2        & iy1=iy1+1 & iy2=floor(Cxy[1]>0)+iy2
                 ma1=ix1   & ma2=ix2-ix1+1                  & mb1=iy1   & mb2=iy2-iy1+1
                 stp1=0 & stp2=1
                 Xaproj=total(win[*,iy1:iy2],2) & Yaproj=total(win[ix1:ix2,*],1)
                 while (stp1 ne stp2) do begin & stp1=ma2+mb2
                  if Xaproj[(ix1-1)>0]  gt Xaproj[ix1] then begin & ma1=ma1-1 & ma2=ma2+1 & ix1=(ix1-1)>0 & endif ;go to edges
                  if Yaproj[(iy1-1)>0]  gt Yaproj[iy1] then begin & mb1=mb1-1 & mb2=mb2+1 & iy1=(iy1-1)>0 & endif
                  if Xaproj[(ix2+1)<s1] gt Xaproj[ix2] then begin & ma2=ma2+1 & ix2=(ix2+1)<s1            & endif
                  if Yaproj[(iy2+1)<s2] gt Yaproj[iy2] then begin & mb2=mb2+1 & iy2=(iy2+1)<s2            & endif
                  stp2=ma2+mb2
                 endwhile
                 if (n_elements(watermask) eq 1) then begin
                 if (n_elements (Ew) gt 1)   then begin & stp1=0 & stp2=1
                 Xwproj=total(Ew[*,iy1:iy2],2) & Ywproj=total(Ew[ix1:ix2,*],1)
                 while (stp1 ne stp2) do begin & stp1=ma2+mb2
                  if Xwproj[(ix1-1)>0]  gt Xwproj[ix1] then begin & ma1=ma1-1 & ma2=ma2+1 & ix1=(ix1-1)>0 & endif ;go to flat zone
                  if Ywproj[(iy1-1)>0]  gt Ywproj[iy1] then begin & mb1=mb1-1 & mb2=mb2+1 & iy1=(iy1-1)>0 & endif ;for Water
                  if Xwproj[(ix2+1)<s1] gt Xwproj[ix2] then begin & ma2=ma2+1 & ix2=(ix2+1)<s1            & endif
                  if Ywproj[(iy2+1)<s2] gt Ywproj[iy2] then begin & mb2=mb2+1 & iy2=(iy2+1)<s2            & endif
                  stp2=ma2+mb2
                 endwhile
                 endif
                 endif else begin
                 ma1=(ma1-1)>0 & mb1=(mb1-1)>0 & ma2=ma2+1 & mb2=mb2+1                        ;+one down left for Sample
                 if  (ma1+ma2-1 lt s1) then ma2=ma2+1 & if (mb1+mb2-1 lt s2) then mb2=mb2+1   ;+one top right for Sample
                 endelse
      endif
      if n_elements(watermask) eq 1 then sans_mask,waterbox=[ma1+1,mb1+1,ma2,mb2] $               ;Set water  mask
                                    else sans_mask,xybeam=  [ma1+1,mb1+1,ma2,mb2]                 ;Set sample mask

   endif else if not keyword_set(no_ctr) then begin
                    sans_ctr[0:1]=[a1 , b1]                                          ;else keep center (see supl for negative values)
                    if sans_ctr[0] lt 0 then sans_ctr[0]=sans_ctr[0]-1 else sans_ctr[0]=sans_ctr[0]+1
                    if sans_ctr[1] lt 0 then sans_ctr[1]=sans_ctr[1]-1 else sans_ctr[1]=sans_ctr[1]+1
                    sans_supl[49] = supl49 ;Dtheta
              endif
endif

return, Cxy
end

;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
function sans_cor, water=nwater,Ewater=newater,sample=nsample,Esample=nEsample,scadmium=nscadmium,wcadmium=nwcadmium, efficiency=effic,$
                   transmission=trans, no_centr=no_centr, geteff=woter, Qdep=Qdep, message=Mess, XWdep=XWdep, suplMASK=suplMASK
;+
;******* ********
;** User callable
;** Call w8=sans_cor(water=1, Ewater=2, sample=3, Esample=4, scadmium=5, (/efficiency, transm=[1.,1.,1.,1.])
;**
;** trans=[TRwater, TRewater, TRsample, TResample] in this order , DEFAULT is in SANS_TRANS common variable
;** sample Background can be a series
;** returns corrected sample.
;**
;** What is done:
;** correction  for Q dependent response of detector (Instrument specific)
;** dead_time   correction  & monitor normalisation
;** empty cell  and background(cadmium) subtraction (transmission calculated for each Q)
;** correction  for detectors efficiencies(water)
;** solid angle (thickness) correction.
;**
;** geteff=woter return detector efficiencies
;** Qdep  =Qdep  add an instrumental dependent function
;** D.R Jun 2009
;**
;** Added Q dependent transmission (sample thickness)
;** Better use of different absolute scaling
;** Miguel Gonzalez Oct 2009->
;**
;** Use water center calculated at beam water read
;** Journaling
;** D.R Oct 2010
;**
;** Use Phi0 sample for Phi0 water when concistant
;** D.R Dec 2010
;**
;** Use water distance if Dis-Disw <= 5 mmm
;** Just for the eye when we devide water by sample water
;** D.R Mar 2012
;**
;** Introduction of XWdep: a water cross section formula (as Qdep)
;** D.R Oct 2012
;**
;** Increment PARIDX with Xdetsiz, Ydetsiz, Mode (for D33 TOF merging)
;** New common c_sans_tofl
;** New table baset.attenuaF (factor of attenuation factor versus collimation)
;** D.R Feb Mar 2013
;**
;** Add an offset for distance (D11 special Can)
;** D.R Apr 2013
;**
;** Add an alternative tr.Beam for Phi_0 calculation in sans_supl[60]
;** D.R Feb 2014
;**
;** Add structure input nSample or nWater (Batch/Prox/Web input)
;** D.R May 2014
;;**
;** added suplMASK, suplement mask index for WATER comming from negative counts at read time
;** D.R Oct 2014
;-

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl
common c_sans_tofl,      TofFlux, LambdaVq

;SANS_TRANS: transmission   [ [Water , WaterCell , Sample1 , SampleCell1 , Sample1 , Sample2 , ...],
;            thickness        [      , 0         ,         , 0           ,         ,         , ...],
;            tr.error Sample  [Water , WaterCell , Sample1 , SampleCell1 , Sample1 , Sample2 , ...],
;            tr. SampleCell   [0     , 0         , 0       , SampleCell1 , SampleCell1 , SampleCell2 , ...]  (if series)
;            tr.error S.Cell  [0     , 0         , 0       ,             ,             ,             , ...]

;** sans_supl(0)  Wavelength
;** sans_supl(1)  Distance sample->detector sample
;** sans_supl(2)  Offset detector (mm) sample
;** sans_supl(3)  Angle detector (deg) sample
;** sans_supl(4)  Collimation sample
;** sans_supl(6)  TAU factor
;** sans_supl(7)  TAU function to be used
;** sans_supl(8)  X pixel size in Cm
;** sans_supl(9)  Y pixel size in Cm

;** sans_supl(10) WATER  thickness if >0.                             or trans(0,1)
;** sans_supl(11) SAMPLE thickness if >0.                             or trans(2,1)
;** sans_supl(12) SAMPLE Tr error                                     or trans(2,2)
;** FOR SAMPLE SERIES
;** sans_trans(4:*,1) SAMPLE thickness
;** sans_trans(4:*,2) SAMPLE Tr error

;** sans_supl(26) Attenuator factor                                   Unused here
;** sans_supl(27) enable or disable dead_time correction
;** sans_supl(28) enable or disable solid angle correction
;** sans_supl(29) 1=provided water cross section in (42)
;** sans_supl(42) WATER scattering cross section input user
;** sans_supl(43) enable or disable thickness as f(Q)
;** sans_supl(44) 1=un_scale in absolute scaling (but correct from efficiencies, not from thickness)
;** sans_supl(45) rotation axe (cm)

;** sans_supl(32) Water distance
;** sans_supl(33) Water duration ****(unused as sample duration(23)-> taken in datp)****
;** sans_supl(34) Water collimation

;** sans_supl(31) Phi_0 sample (N/Sec) corrected, masked
;** sans_supl(36) Beam sample distance
;** sans_supl(37) Beam sample duration
;** sans_supl(38) Beam sample collimation

;** sans_supl(35) Phi_0 water  (N/Sec) corrected, masked
;** sans_supl(39) Beam water  distance
;** sans_supl(40) Beam water  duration
;** sans_supl(41) Beam water  collimation
;** sans_supl(51) Offset detector (mm) water
;** sans_supl(56) Water X center (calculated at beam water read)
;** sans_supl(57) Water Y center (calculated at beam water read)
;== sans_supl(58) =1 if water can't be used for correction (set by sans_cor)
;** sans_supl(59) Water Angle detector (deg)
;== sans_supl(60) (alternative Phi_0 sample supl[31])
;== sans_supl(61) =1 disable Qdep correction
;RD sans_supl(62) Beam water  monitor related to sans_supl(40) in sans_settings_runs
;RD sans_supl(63) Beam sample monitor related to sans_supl(37) in sans_settings_runs
;RD sans_supl(64) Mode 1:Tof  3:Kin  4:TOFmerged
;** sans_supl(65) 0:disable offset  1:enable offset detector  2:enable offset distance
;** sans_supl(66) >1 means at read negative counts must be masked (inter panels)

;** sans_supl(48) UseTime=1 , UseMonitor=0

;** sans_supl(85) Index of time duration in datp

debug = 0
if debug then openw, debugunit, 'debug_sans_cor.txt', /GET_LUN

sizS = size(nSample)  &  sizW = size(nWater); If Batch/Prox/Web command then ...
if (sizS[sizS[0]+1] eq 8) or (sizW[sizW[0]+1] eq 8) then begin 
 if sizW[sizW[0]+1] eq 8  then SANS_SETTINGS_RUNS_LIVE, nWater
 if sizS[sizS[0]+1] eq 8  then SANS_SETTINGS_RUNS_LIVE, nSample
 Qdep  = sans_inst_T.Qdep
 XWdep = sans_inst_T.XWdep
 return, SANS_COR( water=10, Ewater=11,sample=12,Esample=13,scadmium=14,wcadmium=15, efficiency=effic,$
         transmission=trans, no_centr=no_centr, geteff=woter, Qdep=Qdep, message=Mess, XWdep=XWdep, suplMASK=suplMASK)
endif

UseTime=sans_supl[48]

Mess ='??? Bad inputs'
;Be sure to have same precision for sample/water distance,offset,angle,collimation
sans_supl[[1,32,2,51,3,59,4,34]]=round(sans_supl[[1,32,2,51,3,59,4,34]]*1000.)/1000.

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Distances: '
  printf, debugunit, '    sans_supl[1]  = ', sans_supl[1], ' (Sample)'
  printf, debugunit, '    sans_supl[32] = ', sans_supl[1], ' (Water)'
  printf, debugunit, '    sans_supl[2]  = ', sans_supl[1], ' (Offset Sample)'
  printf, debugunit, '    sans_supl[51] = ', sans_supl[1], ' (Offset Water)'
  printf, debugunit, '    sans_supl[3]  = ', sans_supl[1], ' (Angle Sample)'
  printf, debugunit, '    sans_supl[59] = ', sans_supl[1], ' (Angle Water)'
  printf, debugunit, '    sans_supl[4]  = ', sans_supl[1], ' (Collimation Sample)'
  printf, debugunit, '    sans_supl[34] = ', sans_supl[1], ' (Collimation Water)'
endif

tau    =sans_supl[6]*sans_supl[27] & ftau=sans_supl[7]
if sans_supl[66] gt 1 then tau = 0. ;disable dead times correction whith multi-panels data

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Dead time: '
  printf, debugunit, '    sans_supl[66]  = ', sans_supl[66]
  printf, debugunit, '    tau            = ', tau  
endif

Dis    =sans_supl[1]
Disw   =sans_supl[32]
DisWB  =sans_supl[39]
timS   =sans_supl[23] ;unused -> taken from datp
timW   =sans_supl[33] ;unused -> taken from datp
alphaDs=sans_supl[3]  ;detector tilt sample
alphaDw=sans_supl[59] ;detector tilt water
batch  =sans_supl[79] ;batch mode

wout   =0
woter  =0
sameWS =0             ;=1 if water & sample are in same configuration DIST(m), OFFSET(mm), ANGLE(deg)

difdWS =abs(DisW-Dis)
if (difdWS le .1) and (abs(sans_supl[2]-sans_supl[51]) le 1.) and (abs(alphaDs-alphaDw) le 1.) then sameWS=1
if (sameWS) and (difdWS le .005) then begin ;Make Water and Sample the same distance if difference <= 5mm
    Dis=Disw ;Just for the eye when we divide water by sample water
    if debug then begin
      printf, debugunit, ''
      printf, debugunit, 'Water and Sample distances adjusted to ', Dis
    endif 
endif

;Get transmissions from sans_trans common variable 0=water 1=Ewater 2=sample 3=Esample
;*************************************************
if n_elements(trans) lt 4 then if n_elements(sans_trans) ge 4 then trans=sans_trans else return,wout
if n_elements(sans_supl)  le 1 then sans_settings,/nowind  ; init sans_settings   ! (unprobable case)

idx = where(trans[*,0] le 0.) & if idx[0] ge 0 then trans[idx,0]=1.
A0 = trans[1,0]  ;trans Wempty  (/Wbeam)
A1 = trans[0,0]  ;trans Water   (/emptyCan)
A2 = trans[2,0]  ;trans Sample  (/emptyCan) !!! /BeamS if no empty cell
A3 = trans[3,0]  ;trans Sempty  (/Dbeam)
AA1= A1
AA2= A2  ;Keep for test

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Transmissions: '
  printf, debugunit, '    A0  = ', A0, ' (Bkg for water)'
  printf, debugunit, '    A1  = ', A1, ' (Water)'
  printf, debugunit, '    A2  = ', A2, ' (Sample)'
  printf, debugunit, '    A3  = ', A3, ' (Bkg for sample)'
endif 

;get sample and water counts
;***************************
if n_elements(nwater)   eq 1   then TAKE_W,woter   ,W=nwater
if n_elements(nsample)  eq 1   then TAKE_W,wout    ,W=nsample
if n_elements(wout)     gt 1   then s=size(wout)    else s=size(woter)
if n_elements(woter)    gt 1   then TAKE_DATP,datpw,W=nwater
if n_elements(wout)     gt 1   then TAKE_DATP,datp ,W=nsample

if n_elements(datpW) eq 0 then if n_elements(datp ) gt 0 then datpW=datp  ;!!! Change this (for sans_settings_dist)
if n_elements(datp ) eq 0 then if n_elements(datpW) gt 0 then datp =datpW

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Total counts: '
  printf, debugunit, '    Water  = ', total(woter)
  if s[0] eq 3 then begin
     for i=0,s[3]-1 do printf, debugunit, '    Sample = ', total(wout[*,*,i]), ' [', i, ']'
  endif else printf, debugunit, '    Sample = ', total(wout)
endif

;Theta2 , QdepS , QdepW (Q dependent response, instrument specific)
;******
theta2S=0 & theta2W=0 & QdepS=1. & QdepW=1.
;Qdep='' ;Remove instrument Q-dependence <- *** FOR TESTS ***
if n_elements(Qdep) eq 1 then begin
  ;Qdep  ='(cos(THETA^3/dist*2)^2)*(1-(radius/(max(radius)>(1e-9))*.095))';D11 'COOLFIT'  ACTUAL FORMULA (EMPIRICAL)
  ;Qdep  ='.14*exp(-4.*alog(2.)*(((THETAX-.588)<0.)/.414)^2)'             ;D22 'COOLFIT'  ACTUAL FORMULA (EMPIRICAL)
  ;Qdep =''                                                               ;D33 done in read_hdf
endif

Xdet=sans_supl[8] & Ydet=sans_supl[9]

if Xdet gt 0. then begin & xyPsize=[Xdet,Ydet] & xy0=sans_ctr[0:1]-1.       ;***Sample center
   if xy0[0] lt 0 then xy0[0]=xy0[0]+2.
   if xy0[1] lt 0 then xy0[1]=xy0[1]+2.
   sans_settings_dist, s, Dst , Center=xy0, xyPsize=xyPsize, distance=Dis, DforT=DforTs, aX=datp.X, aY=datp.Y, thetrad=thetrad
   if n_elements(thetrad) gt 1 then theta2S=thetrad else $
   if n_elements(DforTs)  gt 1 then theta2S=atan(DforTs/Dis/100.) else theta2S=atan(Dst/Dis/100.)  ;***sample Radian
   xyW=xy0
   xyT=sans_supl[56:57]-1. & if xyT[0] lt 0 then xyT[0]=xyT[0]+2. & if xyT[1] lt 0 then xyT[1]=xyT[1]+2.

   a=(sans_beam[3:6]-1)>0
   if (a[2]*a[3] gt 0) and (n_elements(woter) gt 1) then begin & xyW=[a[0]+a[2]/2. , a[1]+a[3]/2.]
                                                                          ;***Water  center = Sample center (same dist, offset, tilt)
       if sameWS then xyW=xy0 else $
                                                                          ;***Water distW  = Water Beam distWB (center inside Mask)
       if (abs(DisW-DisWB) le .1) and (xyT[0]*xyT[1] gt 0.) and $
          (xyT[0] gt a[0]) and (xyT[0] lt a[0]+a[2]) and (xyT[1] gt a[1]) and (xyT[1] lt a[1]+a[3]) then xyW=xyT $
                                                                          ;***Water  center from Mask then from Fit!!!
       else if (not(keyword_set(no_centr))) then begin
               Wroi =woter[a[0]:a[0]+a[2],a[1]:a[1]+a[3]] & Wroi=max(Wroi)-Wroi
               if ((size(Wroi))[1] ge 5) and ((size(Wroi))[2] ge 5) then begin
                 WXpar=str_fit(total(Wroi,2),win=-1)
                 WYpar=str_fit(total(Wroi,1),win=-1) ;win=-1 to not write str_params.txt
                 xyT=[WXpar[2]+a[0],WYpar[2]+a[1]]
                 if (xyT[0] gt a[0]) and (xyT[0] lt a[0]+a[2]) and (xyT[1] gt a[1]) and (xyT[1] lt a[1]+a[3]) then xyW=xyT
               endif
       endif
   endif
   sans_settings_dist, s,DstW,Center=xyW,xyPsize=xyPsize, distance=DisW, DforT=DforTw, aX=datpw.X, aY=datpw.Y, thetrad=thetrad, /water
   if n_elements(thetrad) gt 1 then theta2W=thetrad else $
   if n_elements(DforTw)  gt 1 then theta2W=atan(DforTw/DisW/100.) else theta2W=atan(DstW/DisW/100.) ;***water  Radian
endif

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Angles: '
  printf, debugunit, '    min(theta2S)  = ', min(theta2S, idx)*!radeg, ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
  printf, debugunit, '    max(theta2S)  = ', max(theta2S, idx)*!radeg, ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
  printf, debugunit, '    min(theta2W)  = ', min(theta2W, idx)*!radeg, ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
  printf, debugunit, '    max(theta2W)  = ', max(theta2W, idx)*!radeg, ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
endif

costhetaThinS=cos(theta2S) ;sample
costhetaThinW=cos(theta2W) ;water

;Solid angle correction matrix determination
;*******************************************
if (not keyword_set(no_centr)) and (sans_supl[28] eq 1) then begin
   if n_elements(DforTs) gt 1 then begin & alphas=abs(alphaDs *!pi/180.) ;We may have a detector angle
                                         alphaw=abs(alphaDw *!pi/180.)

      Unit=float(max(xyPsize,min=mu))
      if xyPsize[0]  eq mu  then ratioxy=[mu/Unit,1.] else ratioxy=[1.,mu/Unit]

     ;a   =abs(sans_supl(2)/10.) * tan(alpha)                  ;Use param  offset (Raxe=0)
      aS  =abs(xy0[0]-s[1]/2.+.5)* ratioxy[0]*sin(alphas)*Unit ;Use center offset
      aW  =abs(xyW[0]-s[1]/2.+.5)* ratioxy[0]*sin(alphaw)*Unit ;Use center offset
      rs  =sans_supl[45]*(1.-cos(alphas))
      rw  =sans_supl[45]*(1.-cos(alphaw))
      Ds  =Dis *100.+aS+rs
      Dw  =DisW*100.+aW+rw
      if alphaDs gt 0.     then sgnS=-1 else sgnS=1    ;Bluffing with a beam normal to the detector (new Xcenter)
      if alphaDw gt 0.     then sgnW=-1 else sgnW=1    ;alpha > 0   !!
      if xy0[0] gt s[1]/2. then sgnS=-1 else sgnS=1    ;=offset < 0
      if xyW[0] gt s[1]/2. then sgnW=-1 else sgnW=1    ;Bluffing with a beam normal to the detector (new Xcenter)
      offcs=sin(alphas)*Ds*sgnS & xy0s=[xy0[0]+offcs/ratioxy[0]/Unit,xy0[1]]
      offcw=sin(alphaw)*Dw*sgnW & xy0w=[xyW[0]+offcw/ratioxy[0]/Unit,xyW[1]]
      newDs=abs(offcs)/tan(alphas)
      newDw=abs(offcw)/tan(alphaw)

      sans_settings_dist, s, Dst, Center=xy0s, xyPsize=xyPsize, aX= datp.X, aY=datp.Y, thetrad=thetrad
      if n_elements(thetrad) gt 1 then theta2S=thetrad else theta2s=atan(Dst /newDs)
      if abs(alphaDw) gt 1. then begin
         sans_settings_dist, s, Dst, Center=xy0w, xyPsize=xyPsize, aX=datpw.X, aY=datpw.Y, thetrad=thetrad, /water
         if n_elements(thetrad) gt 1 then theta2W=thetrad else theta2w=atan(Dst /newDw)
      endif
   endif
   costheta2Scub=cos(theta2s)^3.  ;Calculate costheta2Scub sample (taking angle detector)
   costheta2Wcub=cos(theta2w)^3.  ;Calculate costheta2Wcub water  (taking angle detector)
endif else begin
   costheta2Scub=1. & costheta2Wcub=1. & endelse

if debug then begin
   printf, debugunit, ''
   printf, debugunit, 'Solid angle correction: '
   printf, debugunit, '    min(costheta2Scub)  = ', min(costheta2Scub, idx), ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
   printf, debugunit, '    max(costheta2Scub)  = ', max(costheta2Scub, idx), ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
   printf, debugunit, '    min(costheta2Wcub)  = ', min(costheta2Wcub, idx), ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
   printf, debugunit, '    max(costheta2Wcub)  = ', max(costheta2wcub, idx), ' deg at ', idx-fix(idx/256)*256, fix(idx/256)
endif

;QDEP FUNCTION EVALUATION (THETA ,IF USED, IS CALCULATED FROM THE NORMAL TO THE DETECTOR)
if n_elements(Qdep) eq 1  then begin & QDEPU=strupcase(Qdep)
      if (not sans_supl[61]) then if (strpos(QDEPU,'Q')      ge 0) or $
                                     (strpos(QDEPU,'DIST')   ge 0) or $
                                     (strpos(QDEPU,'WAVE')   ge 0) or $
                                     (strpos(QDEPU,'THETA')  ge 0) or $
                                     (strpos(QDEPU,'RADIUS') ge 0) then begin
       szthS=size(theta2S) & szthW=size(theta2W)
       radius=0 & theta=0 & thetaX=0 & thetaY=0 & dist=1 & wave=sans_supl[0] & q=0. & radiusY=0
       ii=execute('mQdep='+Qdep) ; suppose minimum at Q=0 or theta=0 or radius=0 ...
       
       if (strpos(QDEPU,'Q')  ge 0) then Q=4.*!pi*sin(theta2S/2.)/sans_supl[0]
       if (strpos(QDEPU,'DIST') ge 0) then    dist    =Dis
       if (strpos(QDEPU,'THETA') ge 0) then   theta   =theta2S
       if (strpos(QDEPU,'THETAX') ge 0) then  thetaX  =theta2S[*,xy0[1]>0<(szthS[2]-1)] # (fltarr(1,szthS[2])+1.)
       if (strpos(QDEPU,'THETAY') ge 0) then  thetaY  =(fltarr(szthS[1])+1.) # theta2S[xy0[0]>0<(szthS[1]-1),*]
       if (strpos(QDEPU,'RADIUS') ge 0) then  radius  =Dst
       if (strpos(QDEPU,'RADIUSY') ge 0) then radiusY =(fltarr(szthS[1])+1.) # Dst[    xy0[0]>0<(szthS[1]-1),*]
       ii=execute('QdepS='+Qdep) & QdepS=1./(QdepS-mQdep+1.) ;***Qdep sample

       if (strpos(QDEPU,'Q')  ge 0) then Q=4.*!pi*sin(theta2W/2.)/sans_supl[0]
       if (strpos(QDEPU,'DIST') ge 0) then    dist    =DisW
       if (strpos(QDEPU,'THETA') ge 0) then   theta   =theta2W
       if (strpos(QDEPU,'THETAX') ge 0) then  thetaX  =theta2W[*,xyW[1]>0<(szthW[2]-1)] # (fltarr(1,szthW[2])+1.)
       if (strpos(QDEPU,'THETAY') ge 0) then  thetaY  =(fltarr(szthW[1])+1.) # theta2W[xyW[0]>0<(szthW[1]-1),*]
       if (strpos(QDEPU,'RADIUS') ge 0) then  radius  =DstW
       if (strpos(QDEPU,'RADIUSY') ge 0) then radiusY =(fltarr(szthW[1])+1.) # DstW[   xyW[0]>0<(szthW[1]-1),*]
       ii=execute('QdepW='+Qdep) & QdepW=1./(QdepW-mQdep+1.) ;***Qdep water
      endif
endif

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Qdep correction: '
  printf, debugunit, '    min(QdepS)  = ', min(QdepS)
  printf, debugunit, '    max(QdepS)  = ', max(QdepS)
  printf, debugunit, '    min(QdepW)  = ', min(QdepW)
  printf, debugunit, '    max(QdepW)  = ', max(QdepW)
endif

showplot=0 & if showplot then begin
window,0 & if n_elements(costheta2Scub) gt 1 then tvscl,bytscl(congrid(costheta2Scub,256,256))
           if n_elements(costhetaThinS) gt 1 then tvscl,bytscl(congrid(costhetaThinS,256,256)),256,256
           if n_elements(QdepS)         gt 1 then tvscl,bytscl(congrid(QdepS        ,256,256)),512,0
           Qdept=QdepS[*,0] & plot,thetaX[*,0],Qdept
           endif
           
;Thickness correction: matrix determination or new transmissions as f(q)
;***********************************************************************
miguel=1 ;*** =0 using thickness matrix  =1 using thickness correction for transmission ***
         ;*********************************************************************************
if (not keyword_set(no_centr)) and (sans_supl[43] eq 1) then begin
   if not miguel then begin
      costhetaThinS=2. *costhetaThinS/(1.+costhetaThinS) ; !!! Scattering at e/2
      costhetaThinW=2. *costhetaThinW/(1.+costhetaThinW)
   endif else begin & migW=double((1./costhetaThinW -1.)) & costhetaThinW=1.
                    migS=double((1./costhetaThinS -1.)) & costhetaThinS=1. & endelse
endif else begin
      costhetaThinS=1. & costhetaThinW=1. & miguel=0 & endelse

sizt  =size(trans)
thickW=sans_supl[10] & if (thickW eq 0.) and (sizt[0] eq 2) then thickW=trans[0,1] ;WATER  thickness


;CALCULATE DETECTORS EFFICIENCIES
;********************************
if n_elements(woter)    gt 1   then begin
      timW=datpw.p[sans_supl[85]]

      if UseTime then N_W=timW else N_W=datpw.N[0]
      sxy=n_elements(woter)
      if tau*timW gt 0. then sans_tau, woter, 0, ftau, tau, timW ;dead times for Water

      if miguel then begin ;Compute Transmission dependence on 2theta
         if A0 gt 0. then atheta = alog(A0<1.) * migW  &  A0=A0 *float( 1. + atheta/2. + atheta^2/6. + atheta^3/24. + atheta^4/120.) ;water can
         if A1 gt 0. then atheta = alog(A1<1.) * migW  &  A1=A1 *float( 1. + atheta/2. + atheta^2/6. + atheta^3/24. + atheta^4/120.) ;water
      endif
      
     ;I = [ (I_w - I_Cd)/(Tw*Tew) - (I_ew - I_Cd)/Tew ] =
     ;    [ (I_w/Tw - I_ew) - I_Cd * (1/Tw - 1) ]/Tew

     ;SUBTRACT EMPTY CAN FROM WATER
      if n_elements(nEwater)  eq 1   then TAKE_W,Ewater,W=nEwater
      if n_elements(Ewater)   eq sxy then begin
        
         TAKE_DATP, datps, W=nEwater & tim=datps.p[sans_supl[85]]

         if UseTime then N_EW=tim else N_EW=datps.N[0]
         if tau*tim gt 0. then sans_tau, Ewater, 0, ftau, tau, tim ;dead times for water Can

         woter=woter/A1 -  Ewater *N_W/N_EW             ;(I_w/Tw - I_ew)
         
         if debug then begin
           printf, debugunit, ''
           printf, debugunit, 'Water - Water_bkg: '
           printf, debugunit, '    A1   = ', min(A1), max(A1)
           printf, debugunit, '    N_W  = ', N_W
           printf, debugunit, '    N_EW = ', N_EW
         endif

      endif else A0=A1 ;SHOULD DIVIDE by Tw ANYWAY !!!

     ;SUBTRACT CADMIUM FROM WATER
      if n_elements(nwcadmium) eq 1   then TAKE_W,wcadmium,W=nwcadmium
      if n_elements(wcadmium)   eq sxy then begin
         
         TAKE_DATP, datps, W=nwcadmium & tim=datps.p[sans_supl[85]]

         if UseTime then N_C=tim else N_C=datps.N[0]

         woter=woter - (1./A1 -1.)*wcadmium *N_W/N_C    ;(I_w/Tw - I_ew) - I_Cd * (1/Tw - 1)

         if debug then begin
           printf, debugunit, ''
           printf, debugunit, 'Water - Cd: '
           printf, debugunit, '    A1  = ', min(A1), max(A1)
           printf, debugunit, '    N_W = ', N_W
           printf, debugunit, '    N_C = ', N_C
         endif
         
      endif
      
      ;if we have at least a water transmission
      if AA1 gt 0. then woter=woter/A0                  ;[ (I_w/Tw - I_ew) - I_Cd * (1/Tw - 1) ] /Tew
      if debug then begin
        printf, debugunit, ''
        printf, debugunit, 'Water/T: '
        printf, debugunit, '    A0  = ', min(A0), max(A0)
      endif
      
      woter=woter /costheta2Wcub /costhetaThinW *QdepW  ;Detector corrections
      if debug then begin
        printf, debugunit, ''
        printf, debugunit, 'Water corrections: '
        printf, debugunit, '    min(costheta2Wcub)  = ', min(costheta2Wcub)
        printf, debugunit, '    max(costheta2Wcub)  = ', max(costheta2Wcub)
        printf, debugunit, '    min(costhetaThinW)  = ', min(costhetaThinW)
        printf, debugunit, '    max(costhetaThinW)  = ', max(costhetaThinW)
        printf, debugunit, '    min(QdepW)          = ', min(QdepW)
        printf, debugunit, '    max(QdepW)          = ', max(QdepW)
      endif

     ;EXCLUDE MASKED POINTS
      sans_settings_getmask,size(woter),xyW,M,/nobeam
      if n_elements(M) eq sxy then begin
         if n_elements(suplMASK) gt 0 then M[suplMASK] =1 ;NEGATIVE COUNTS AT READ TIME MUST BE MASKED
         idx = where( (M eq 0), complement=idy)
      endif else begin
         idx = [-1]  &  idy = [-1]
      endelse
      if idx[0] ge 0 then avg = median(woter[idx],/even) else avg=median(woter,/even)
      if idy[0] ge 0 then woter[idy] = avg
      if debug then begin
        printf, debugunit, ''
        printf, debugunit, 'Water median: '
        printf, debugunit, '    avg  = ', avg
      endif

      ;EFFICIENCIES
      ;************
      waterKP=woter>1.
      woter  =(waterKP)/avg

      if keyword_set(effic) then begin & Mess=''
         MOD_DATP  ,datpw,'E',0
         MOD_DATP  ,datpw,'N',1
         datpw.x_tit=datpw.x_tit+' Efficiencies'
         GIVE_DATP ,datpw
         return, woter  ;round(woter*100.)/100.
      endif
      
endif else begin & woter=1. & waterKP=woter & endelse

if debug then begin
  printf, debugunit, ''
  printf, debugunit, 'Water used in following corrections: '
  printf, debugunit, '    min(woter)     = ', min(woter)
  printf, debugunit, '    max(woter)     = ', max(woter)
  if n_elements(woter) gt 1 then printf, debugunit, '    median(woter)  = ', median(woter) $
                            else printf, debugunit, '    woter = ', woter
endif

Mon=1.

;GET SAMPLE SIGMAS, MONITOR, TIME
;********************************
if n_elements(wout)  gt 1 then begin
   eout=datp.E    & pZ=datp.Z
   s= size(wout)  & if (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2) THEN RETURN,wout
   sxy=s[1]*s[2]  & if  s[0] gt 2  then nz=s[3]-1 else nz=0

   if n_elements(eout)  ne n_elements(wout) then if sans_supl[66] gt 1 then eout=sqrt(wout>0.) else eout=sqrt(wout) ;NegMask!
   if n_elements(pZ)    ne nz+1 then pZ=lindgen(nz+1)
   if n_elements(woter) ne sxy  then sans_supl[58]=1 else sans_supl[58]=0
   if nz eq 0 then timS=datp.p[sans_supl[85]] else timS=datp.pv[sans_supl[85],*]
   if UseTime then N_S=timS else N_S=datp.N
   Mon=N_S

   ;Get thickness and Tr errors for Sample (as for series)
   ;**************************************
   thickS=sans_supl[11] & if (thickS eq 0.) and (sizt[0] eq 2) then thickS=trans[2,1] ;SAMPLE thickness
   XsectW=sans_supl[42]                                                               ;WATER  Xsection
   XsectS=sans_supl[12] & if (XsectS eq 0.) and (sizt[0] eq 2) then XsectS=trans[2,2] ;SAMPLE tr error Unused at this time in error propag

   if (sizt[0] eq 2) and (sizt[1]-4 ge nz+1) then begin
                           if sizt[2] gt 1 then thickS=trans[4+pZ,1]
                           if sizt[2] gt 2 then XsectS=trans[4+pZ,2] & endif
   ;if SERIES: sample transmission
   ;******************************
   if (sizt[1]-4  ge nz+1) then A2=trans[4+pZ,0] ;[/emptyCan]
   n2 =n_elements(a2)-1
   n3 =0

   if tau*timS[0] gt 0. then sans_tau, Wout, Eout, ftau, tau, timS ;dead times for Sample

  ;CHECK IF WE HAVE CADMIUM
  ;************************
   haveCAD=0
   if n_elements(nscadmium) eq 1   then TAKE_W,scadmium,W=nscadmium
   if n_elements(scadmium)  eq sxy then begin
    
      TAKE_DATP, datpC, W=nscadmium & E_C=datpC.E    & timC=datpC.p[sans_supl[85]]

      if UseTime then N_C=timC  else N_C=datpC.N[0]
      if n_elements(E_C)   ne sxy then if sans_supl[66] gt 1 then E_C=sqrt(scadmium>0.) else E_C=sqrt(scadmium) ;NegMask!
      haveCAD=1
   endif

  ;I = [ (I_s - I_Cd)/(Ts*Tes) - (I_es - I_Cd)/Tes ] =
  ;    [ (I_s/Ts - I_es) - I_Cd * (1/Ts - 1) ]/Tes

  ;SUBTRACT EMPTY CAN and CADMIUM FROM SAMPLE
  ;******************************************
   if n_elements(nEsample) eq 1   then TAKE_W,Esample,W=nEsample
   sizc=size(Esample)
   sxyc=sizc[1]*sizc[2] & if sizc[0] gt 2 then nzc=sizc[3]-1 else nzc=0
   if sxyc  eq sxy then begin
      if nzc gt 0 then if nzc lt nz  then begin & nzc=0 & Esample=Esample[*,*,0] & endif
      TAKE_DATP, datps, W=nEsample  & E_ES=datps.E      & tim=datps.p[sans_supl[85]]

      if nzc gt 0 then tim=datps.pv[sans_supl[85],*]
      if UseTime then N_ES=tim else N_ES=datps.N
      if n_elements(E_ES)  ne n_elements(Esample) then if sans_supl[66] gt 1 then E_ES=sqrt(Esample>0.) else E_ES=sqrt(Esample) ;NegMask!

     ;if SERIES: emptyC transmission
     ;******************************
      if (nzc gt 0) then begin
         if (sizt[0] eq 2) and (sizt[1]-4 ge nzc+1) then A3=trans[4+pZ,3] ;[/emptyCan]
      endif

      n3 =n_elements(a3)-1

      if tau*tim[0] gt 0. then sans_tau, Esample ,E_ES ,ftau ,tau ,tim ;dead times for Sample Can

      for i=0L,nz do begin & T2=A2[i<n2]
      
        if miguel then begin ;Compute Transmission dependence on 2theta
           atheta = alog(T2<1.) * migS & T2=T2 *float( 1. + atheta/2. + atheta^2/6. + atheta^3/24. + atheta^4/120.) ;sample
        endif
        
        if haveCAD then begin ;(I_s/Ts - I_es) - I_Cd * (1/Ts - 1)
           wout[*,*,i]=      wout[*,*,i]/T2    -  Esample[*,*,i<nzc] *N_S[i]/N_ES[i<nzc]    $
                                               -  (1./T2-1.)*scadmium*N_S[i]/N_C
           eout[*,*,i]=sqrt((eout[*,*,i]/T2)^2 + (E_ES[   *,*,i<nzc] *N_S[i]/N_ES[i<nzc])^2 $
                                               + ((1./T2-1.)*E_C     *N_S[i]/N_C)^2)
           if debug then begin
             printf, debugunit, ''
             printf, debugunit, '(I_s/Ts - I_es) - I_Cd * (1/Ts - 1): '
             printf, debugunit, '    T2   = ', min(T2), max(T2)
             printf, debugunit, '    N_S  = ', N_S[i]
             printf, debugunit, '    N_ES = ', N_ES[i<nzc]
             printf, debugunit, '    N_C  = ', N_C
           endif
           
        endif      else begin ;(I_s/Ts - I_es)
          
           wout[*,*,i]=      wout[*,*,i]/T2    -  Esample[*,*,i<nzc] *N_S[i]/N_ES[i<nzc]
           eout[*,*,i]=sqrt((eout[*,*,i]/T2)^2 + (E_ES[   *,*,i<nzc] *N_S[i]/N_ES[i<nzc])^2)
           
           if debug then begin
             printf, debugunit, ''
             printf, debugunit, '(I_s/Ts - I_es): '
             printf, debugunit, '    T2   = ', min(T2), max(T2)
             printf, debugunit, '    N_S  = ', N_S[i]
             printf, debugunit, '    N_ES = ', N_ES[i<nzc]
           endif
           
        endelse
      endfor
      haveCAD=0 ;Prevent to subtract cadmium again

   endif else begin & A3=A2 & n3=n2 & endelse ;SHOULD DIVIDE by Ts ANYWAY !!!

  ;SUBTRACT CADMIUM FROM SAMPLE when EMPTY CELL IS MISSING
  ;*******************************************************
   if haveCAD then begin                  ;(I_s/Ts - I_es) - I_Cd * (1/Ts - 1)
      for i=0L,nz do wout[*,*,i]=      wout[*,*,i]    -  (1./a2[i<n2]-1.)*scadmium*N_S[i]/N_C
      for i=0L,nz do eout[*,*,i]=sqrt((eout[*,*,i])^2 + ((1./a2[i<n2]-1.)*E_C     *N_S[i]/N_C)^2)
      if debug then printf, debugunit, ' Subtracting Cd from sample, no emty cell!'
   endif

   if not keyword_set(no_centr) then begin ;CONTINUE IF WE HAVE A CENTER
   ;*******

   if AA2 gt 0.   then  $                 ;We have at least one sample transmission
   for i=0L,nz do begin & T33=A3[i<n3]      ;[ (I_s/Ts - I_es) - I_Cd * (1/Ts - 1) ]/Tes
        if miguel and (n3 ge i) then begin;Compute Transmission dependence on 2theta
           atheta = alog(T33<1.) * migS  &  T3=T33 *float( 1. + atheta/2. + atheta^2/6. + atheta^3/24. + atheta^4/120.) ;sample can
        endif
        if not miguel then T3=T33
        wout[*,*,i]=      wout[*,*,i]/T3  ;[ (I_s/Ts - I_es) - I_Cd * (1/Ts - 1) ]/Tes
        eout[*,*,i]=      eout[*,*,i]/T3
        if debug then begin
          printf, debugunit, ''
          printf, debugunit, '[ (I_s/Ts - I_es) - I_Cd * (1/Ts - 1) ]/Tes: '
          printf, debugunit, 'Possible transmission(2-theta): '
          printf, debugunit, '    T3   = ', min(T3), max(T3)
        endif
   endfor

  ;SOLID ANGLE CORRECTION + (THICKNESS CORRECTION) + QDEPS
  ;*****************************************************
   if n_elements(theta2S) eq sxy then for i=0L,nz do wout[*,*,i]=wout[*,*,i]/costheta2Scub/costhetaThinS*QdepS
   if n_elements(theta2S) eq sxy then for i=0L,nz do eout[*,*,i]=eout[*,*,i]/costheta2Scub/costhetaThinS*QdepS
   if debug and n_elements(theta2S) eq sxy then begin
     printf, debugunit, ''
     printf, debugunit, 'Solid angle + thickness + Qdep correction: '
     printf, debugunit, '    min(costheta2Scub)   = ', min(costheta2Scub)
     printf, debugunit, '    max(costheta2Scub)   = ', max(costheta2Scub)
     printf, debugunit, '    min(costhetaThinS)   = ', min(costhetaThinS)
     printf, debugunit, '    max(costhetaThinS)   = ', max(costhetaThinS)
     printf, debugunit, '    min(QdepS)           = ', min(QdepS)
     printf, debugunit, '    max(QdepS)           = ', max(QdepS)
   endif

   ;timS=sample duration    (series)
   ;timW=water  duration
   ;Dis =sample distance
   ;Disw=water  distance
   ;thickS=Thickness sample (series)
   ;thickW=Thickness water
    BtimS =sans_supl[37]    ;beam sample duration
    BmonS =sans_supl[63]    ;beam sample monitor
    BtimW =sans_supl[40]    ;beam water  duration
    BmonW =sans_supl[62]    ;beam water  monitor
    BdisS =sans_supl[36]>.1 ;beam sample distance
    BdisW =sans_supl[39]>.1 ;beam water  distance
    
    COLLs =sans_supl[4]                ; Collimation sample
    COLLw =sans_supl[34]               ; Collimation water
    COLLsB=sans_supl[38]               ; Collimation sample beam
    COLLwB=sans_supl[41]               ; Collimation water  beam
    Phi0S =sans_supl[31]               ; Flux at sample: Neutron/Sec * attenuation
    Phi0W =sans_supl[35]               ; Flux at water:  Neutron/Sec * attenuation
    SAfW  =(Xdet*Ydet)/((BdisW*100.)^2); Solid angle water
    SAfS  =(Xdet*Ydet)/((BdisS*100.)^2); Solid angle sample
    if debug  then begin
      printf, debugunit, ''
      printf, debugunit, 'Flux and solid angle: '
      printf, debugunit, '    Phi0S   = ', Phi0S
      printf, debugunit, '    Phi0W   = ', Phi0W
      printf, debugunit, '    SAfS   = ', SAfS
      printf, debugunit, '    SAfW   = ', SAfW
    endif

    ;Check for Phi0 integrity (same distance and same collimation)
    PhiS=Phi0S
    PhiW=Phi0W
    Mess=''
    if (abs(Dis -BDisS) gt .1) OR (abs(COLLs-COLLsB) gt .1) Then begin & PhiS=0.
       Mess='WARNING: Different Distances or Collimations for Sample and Dbeam Sample' & endif
    if Mess gt ' ' then begin & lamp_journal,Mess & lamp_message,Mess  & Mess='' & endif

    if (abs(DisW-BDisW) gt .1) OR (abs(COLLw-COLLwB) gt .1) Then begin & PhiW=0.   ;Water # Water  beam
       if (abs(DisW-BDisS) le .1) AND (abs(COLLw-COLLsB) le .1) Then begin       ;but...
                                                                       PhiW=PhiS ;Water = Sample beam
                                  SAfW=(Xdet*Ydet)/((DisW*100.)^2) & endif       ;Correct Solid angle water
       if (n_elements(woter) gt 1) and (not sans_supl[44]) and (sans_supl[29]) Then $
         if PhiW le 0 then Mess='WARNING: Different Distances or Collimations for Water and Tr.Beam' $
                      else Mess='WARNING: PHI0 for Water is set with Sample PHI0 Tr.Beam'
    endif
    if Mess gt ' ' then begin & lamp_journal,Mess & lamp_message,Mess  & Mess='' & endif

    ;Compute cross section for water(if possible)
    if (n_elements(idx) gt 1) and (PhiW gt 0.) and (thickW gt 0.) then $
         XsectWater =(total(Waterkp[idx])/(thickW*PhiW*SAfW*timW)/n_elements(idx))>0.001<99. $
    else XsectWater =1.
    if debug  then begin
      printf, debugunit, ''
      printf, debugunit, 'Water cross section: '
      printf, debugunit, '    XsectWater   = ', XsectWater
    endif

    ;Determine scaling factor between sample and water measurements
    FscTXT=' '
    if  (abs(Dis -DisW) le .1)  and (abs(COLLs-COLLw) le .1) then Fsc =1.  $
    else if (PhiS*PhiW  gt 0.)  then begin 
        Fsc = ( PhiS*SAfS)  / (PhiW*SAfW) & FscTXT='( PhiS*SAfS)  / (PhiW*SAfW) ='
        ;print, 'Fsc using time ', Fsc, ', Time ratio = ', BtimS/BtimW, ', Monitor ratio = ', BmonS/BmonW
        ;print, 'Fsc using monitor ', Fsc * (BtimS/BtimW) * (BmonW/BmonS)
        if not(UseTime) and (BmonS*BmonW gt 0.) then Fsc = Fsc * (BtimS/BtimW) * (BmonW/BmonS) ;eliminate use of times if monitor required    
        
    endif else begin & Fsc  =((COLLw*DisW) / (COLLs*Dis))^2 & FscTXT='((COLLw*DisW) / (COLLs*Dis))^2 =' & endelse
    if debug  then begin
      printf, debugunit, ''
      printf, debugunit, 'Scaling factor Sample/Water: '
      printf, debugunit, '    Fsc   = ', Fsc
    endif

   ;Variables:
   ;*********
    lamp_journal,' BdisS='+strcompress(BdisS)+' disS='+strcompress(dis )+' BcollS='+strcompress(COLLsB)+' CollS='+strcompress(COLLs)+' PhiS='+strcompress(PhiS)+' thickS1='+strcompress(thickS[0])+' timS1='+strcompress(timS[0])+' NS1='+strcompress(N_S[0])
    if n_elements(woter) gt 1 then $
    lamp_journal,' BdisW='+strcompress(BdisW)+' disW='+strcompress(disW)+' BcollW='+strcompress(COLLwB)+' CollW='+strcompress(COLLw)+' PhiW='+strcompress(PhiW)+' thickW ='+strcompress(thickW   )+' timW ='+strcompress(timW   )+' nW ='+strcompress(N_W)

   ;What can we do:
   ;**************
    ;Absolute calibration (cm^-1): use instrumental constant (flux measurement) or standard sample (water)
    func = 2 ;Default value --> Apply efficiency scaling + flux measurement. func = 1 --> Water scaling

    ;Check if water scaling demanded and a correct value (positive) is given for the water scattering cross section
    if (sans_supl[29]) and (sans_supl[42] gt 0.) then begin & XsectWater=sans_supl[42] & func=1 & endif

    ;No water or not absolute scaling demanded
    if n_elements(Waterkp) eq 1 then func=2
    if sans_supl[44]      then begin & func=3 & Mess='No scaling demanded' & endif

    if Mess gt ' ' then begin & lamp_journal,Mess & lamp_message,Mess  & Mess='' & endif

    ;No PhiS so try water scaling
    if (PhiS le 0.) and (func eq 2) then  if n_elements(Waterkp) gt 1 $
      then begin & func=1 & Mess='WARNING: No incident flux for sample measurement !!!!' & endif $
      else begin & func=3 & Mess='WARNING: No absolute scaling performed! No incident flux for sample nor water measurement.' & endelse
    ;Verify thickness
    if (func lt 3) and (thickS[0] le 0.) then begin & func=3 & Mess='WARNING: No absolute scaling performed! No Thickness for Sample.' & endif
    if (func eq 1) and (thickW    le 0.) then begin & func=3 & Mess='WARNING: No absolute scaling performed! No Thickness for Water.'  & endif
                                                    
    if Mess gt ' ' then begin & lamp_journal,Mess & lamp_message,Mess  & Mess='' & endif
    
    if debug then begin
       printf, debugunit, ''
       printf, debugunit, 'Subtraction corrections done: '
       if s[0] ge 3 then begin
         for i = 0, s[3] - 1 do printf, debugunit,  '    median(wout) = ', median(wout[*,*,i]), ' [', i, ']'
       endif else printf, debugunit, '    median(wout) = ', median(wout)
    endif

   ;EFFICIENCY SCALING
   ;******************

    if func ge 2 then begin
       if n_elements(woter) eq sxy then for i=0L,nz do wout[*,*,i] = wout[*,*,i]/woter
       if n_elements(woter) eq sxy then for i=0L,nz do eout[*,*,i] = eout[*,*,i]/woter
       if debug then begin
         printf, debugunit, ''
         printf, debugunit, 'Absolute scaling - Dividing by woter: '
         printf, debugunit, '    min(woter)   = ', min(woter)
         printf, debugunit, '    max(woter)   = ', max(woter)
         if n_elements(woter) gt 1 then printf, debugunit, '    median(woter)   = ', median(woter)
         if s[0] ge 3 then begin
           for i = 0, s[3] - 1 do printf, debugunit, '    median(wout) = ', median(wout[*,*,i]), ' [', i, ']'
         endif else printf, debugunit, median(wout)
       endif
       if func eq 2 then begin
          if (usetime) or (BmonS le 0.) then TMnorm=timS   else TMnorm=N_S* (BtimS/BmonS) ;eliminate use of times if monitor required
          if (usetime) or (BmonS le 0.) then TorM=' (norm by Time)' else TorM=' (norm by Monitor)'
          Mess='#Efficiency scaling with PhiS='+strtrim(PhiS,2)+'/s' + TorM
          
          for i=0L,nz do wout[*,*,i]=wout[*,*,i]/thickS[i<n2]/PhiS/SafS /TMnorm[i]
          for i=0L,nz do eout[*,*,i]=eout[*,*,i]/thickS[i<n2]/PhiS/SafS /TMnorm[i]
          if debug then begin
            printf, debugunit, ''
            printf, debugunit, 'Absolute scaling - Dividing by thickS, PhiS, SafS, TMnorm: '
            for i=0L,nz do begin
               printf, debugunit, '    Sample ', i
               printf, debugunit, '        thickS[i<n2] = ', thickS[i<n2]
               printf, debugunit, '        PhiS         = ', PhiS
               printf, debugunit, '        SafS         = ', SafS
               printf, debugunit, '        TMnorm[i]    = ', TMnorm[i]
               if s[0] ge 3 then begin
                 for i = 0, s[3] - 1 do printf, debugunit, '    median(wout) = ', median(wout[*,*,i]), ' [', i, ']'
               endif else printf, debugunit, median(wout)
            endfor
          endif
             ;for i=0L,nz do print, 'Dividing by sample time = ',timS(i)
             ;for i=0L,nz do print, 'Dividing by sample monitor * BtimS/BmonS = ',N_S(i)*BtimS/BmonS
          Mon[*]=1.
       endif else if not sans_supl[44] then begin
          if UseTime then Mess='#Normalised by the Time' else Mess='#Normalised by the Monitor'
          for i=0L,nz do wout[*,*,i]=wout[*,*,i]/N_S[i]
          for i=0L,nz do eout[*,*,i]=eout[*,*,i]/N_S[i]
          if debug then begin
            printf, debugunit, ''
            printf, debugunit, 'Absolute scaling - Dividing by N_S: '
            for i=0L,nz do begin
              printf, debugunit, '    N_S[i] = ', N_S[i]
              if s[0] ge 3 then begin
                for i = 0, s[3] - 1 do printf, debugunit, '    median(wout) = ', median(wout[*,*,i]), ' [', i, ']'
              endif else printf, debugunit, median(wout)
            endfor
          endif
          Mon[*]=1.
       endif
    endif

   ;WATER SCALING
   ;*************
    if func eq 1 then begin
      if PhiW gt 0. then fluxratio = PhiS/PhiW else fluxratio=1.
      if not(UseTime) and (BmonS*BmonW gt 0.) then fluxratio = fluxratio * (BtimS/BtimW) * (BmonW/BmonS) ;eliminate use of times if monitor required
      if not(UseTime) and (BmonS*BmonW gt 0.) then TorM=' (norm by Monitor)' else TorM=' (norm by Time)'
      
      if (Fsc ne 1.) and (PhiW*PhiS gt 0.) then  PhiTXT='  Phi0S/Phi0W ='+strtrim(fluxratio,2) else PhiTXT=''
      Mess='#Water scaling with Fsc='+FscTXT+strtrim(fsc,2)+'  XsectWater='+strtrim(XsectWater,2)  +  PhiTXT + TorM
      Waterdv=(Waterkp) /N_W /thickW *Fsc /XsectWater
      ;***EVENTUALLY UNDO THE SOLID ANGLE CORRECTION (same dist and offset)***
      if sameWS then Waterdv= (Waterdv) *(costheta2Wcub/costheta2Scub) *(QdepS/QdepW)
      
      if n_elements(Waterdv) eq sxy then begin
         for i=0L,nz do wout[*,*,i]=wout[*,*,i]/thickS[i<n2]/N_S[i]/Waterdv
         for i=0L,nz do eout[*,*,i]=eout[*,*,i]/thickS[i<n2]/N_S[i]/Waterdv
         if debug then begin
           printf, debugunit, ''
           printf, debugunit, 'Absolute scaling - Dividing by thickS, N_S and waterdv: '
           for i=0L,nz do begin
             printf, debugunit, '    thickS[i<n2] = ', thickS[i<n2]
             printf, debugunit, '    N_S[i]       = ', N_S[i]
             printf, debugunit, '    min(waterdv) = ', min(waterdv)
             printf, debugunit, '    max(waterdv) = ', max(waterdv)
             if s[0] ge 3 then begin
               for i = 0, s[3] - 1 do printf, debugunit, '    median(wout) = ', median(wout[*,*,i]), ' [', i, ']'
             endif else printf, debugunit, median(wout)
           endfor
         endif
         Mon[*]=1.
      endif else Mess='WARNING: Sample & Water Have different dimensions !!!!'
    endif

    ;Should return Xsect water !!!
    if (PhiW gt 0.) and (XsectWater ne 1.) and (not sans_supl[29]) then sans_supl[42]=XsectWater

   endif else Mess='' ;NO_CENTER (just do intermediate corrections)

  ;Poissonian problem!!!
  ;******************
  ;idw0=where(Wout le 1.)

  if sans_supl[52] eq 0 then begin ;See sans_save_Qs
    ;szthS=size(theta2S)
    ;thetaX=       theta2S(*,xy0(1)>0<(szthS(2)-1)) *180./!pi
    ;thetaY=reform(theta2S(xy0(0)>0<(szthS(1)-1),*) *180./!pi)
    ;MOD_DATP,datp,'X',thetaX  &  datp.X_TIT='2Theta centered'
    ;MOD_DATP,datp,'Y',thetaY  &  datp.Y_TIT='2Theta centered'
  endif

   MOD_DATP  ,datp,'E',eout
   MOD_DATP  ,datp,'N',Mon
   if Mess gt ' ' then datp.x_tit=Mess+" (corrected)"
   if Mess gt ' ' then datp.time =Mess
   GIVE_DATP ,datp
   
   if batch then if sans_supl[78] gt 0 then begin
       wi = long(30+sans_supl[78]) ;-> w31, w32, w33 (L, M, H)
       GIVE_W,    Wout, W=wi
       GIVE_DATP, datp, W=wi
   endif

endif else Mess='' ;NO DATA
if Mess gt ' ' then begin & lamp_journal,Mess & lamp_message,Mess & print,'' & endif

if debug then free_lun, debugunit

return,wout
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

;******* ***********
function sans_radial, M , CENTER=xy0, INCREMENT=incr, MASK=masq, NOMASK=nomask, PixSize=xypsiz, distance=dst ,SECTOR=sector $
                        , X=xx, ERROR=error, AZIMUTAL=azimut, suplMASK=suplMASK, minInt=minInt
;+
;******* ***********
;** User callable
;**     Call:   w2=sans_radial(W1, ...)
;**     ****
;**     Input:      M       =2D or 3D data matrix.
;**     *****       xy0     =[x0,y0] or determined by "sans_center" [1:n]
;**                 incr    =Radius increment (width) in pixels units   (default  1.0)
;**                 masq    =same as M(n,m), contains 1's for masked values otherwise determined by "sans_mask.pro"
;**                 xypsiz  =size of the pixel in Cm. Default is [1.,1.]
;**                 dst     =Distance sample-detector in Cm (data parameter).
;**                 sector  =[inner_radius, outer_radius, angle1_deg, angle2_deg, squared, totsec]
;**                 suplMASK=suplement mask index comming from negative counts at read time
;**                 minInt  =minimum points to integrate
;**
;**     Output:     Wout   =result of Radial integration.
;**     ******      xx     =diagram abscissa: radius in pixel unit.
;**                 error  =diagram errors
;-
forward_function sans_sigmaQ
common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

Wout=0
;*****************************CHECK FOR PARAMETERS******************************
s=size(M)   & s12= s[1]* s[2]
if (s[0] lt 2) or (s[1] lt 2) or (s[2] lt 2)                                          then RETURN, Wout
if n_elements(xy0)  lt 2   then if  n_elements(sans_ctr)  ge 2 then xy0=sans_ctr[0:1] else RETURN, Wout
xy0=xy0-1
if xy0[0] lt 0 then xy0[0]=xy0[0]+2.
if xy0[1] lt 0 then xy0[1]=xy0[1]+2.

if n_elements(sans_supl) le 1 then sans_settings,/nowind  ; init sans_settings   ! (unprobable case)

   wavl   =sans_supl[0]
   dista  =sans_supl[1]
   offset =sans_supl[2] ;Unused here
   angle  =sans_supl[3]
   Raxe   =sans_supl[45] & if Raxe eq 0. then angle=0.
   XYunit =sans_supl[52] ;(0:pixel 1:2theta)
   noWbeam=sans_supl[58]
   batch  =sans_supl[79] ;batch mode
   if sans_supl[8] gt 0. then xypsiz=sans_supl[8:9] ;size pixel detector X,Y (Cm)

if n_elements(masq) ne s12 then $
if not keyword_set(nomask) then sans_settings_getmask,s,xy0,masq ,noWbeam=noWbeam
if not keyword_set(nomask) and (n_elements(suplMASK) gt 0) then masq[suplMASK] =1

if n_elements(dst)    ne 1 then if n_elements(dista)  eq 1 then dst=dista*100.  else dst=100.
if n_elements(incr)   ne 1 then incr  = 1
if n_elements(minInt) ne 1 then minInt= 1

TAKE_DATP, datp                                ;workspace variables

;thetrad=2theta matrice(radian). If present means calculate radius from this (M is not a plane!!)
thetrad =0
if XYunit eq 1 then sans_settings_dist, s,DstS,Center=xy0,xyPsize=xypsiz, DforT=DforT, aX=datp.X, aY=datp.Y ,thetrad=thetrad, atanGG=atanGG

;***** The input Weights due to previous corrections *****
s_e =size( datp.e)
in_t=0
if s_e[0] eq 3 then begin;M2=total(M,3) & E2=total(datp.e,3)   & ide=where(E2 eq 0.) ;average weights 3D->2D
                                          E2=datp.e            & ide=where(E2 eq 0.)
                             if ide[0] ge 0 then E2[ide]=total(E2)/n_elements(E2)
                            ;in_e=    1D/E2^2  & in_t=2 & endif
                             in_e=    1D/E2    & in_t=3 & endif
                            ;in_e=abs(M /E2^2) & in_t=1 & endif
                            ;in_e=abs(M2/E2^2) & in_t=1 & endif ;average weights
if n_elements(datp.e) eq s12 then begin &   E2=datp.e            & ide=where(E2 eq 0.)
                             if ide[0] ge 0 then E2[ide]=total(E2)/n_elements(E2)
                            ;in_e=    1D/E2^2  & in_t=2 & endif
                             in_e=    1D/E2    & in_t=3 & endif
                            ;in_e=abs(M /E2^2) & in_t=1 & endif
;in_t=0


;***** Call radial integrations *****
radial_AT, M, CENTER=xy0, INCREMENT=incr, MASK=masq, PixSize=xypsiz, AZIMUTAL=azimut, thetrad=thetrad, $
           atanGG=atanGG, weight_mode=in_t, input_weight=in_e, DIAGRAM=Wout, Sector=sector, X=xx, $
           ERROR=error, Dangle=angle, distance=dst, Raxe=Raxe, minInt=minInt
Z = 0
X = xx
txt='Radius'
if keyword_set(azimut) then begin & txt='Degrees'

endif else if n_elements(xypsiz) eq 2 then if xypsiz[0] gt 0. then begin
   alpha=abs(angle)*!pi/180.
   r=Raxe*(1.-cos(alpha)) ;Make unit angles 2theta radian ;*!RADEG for degres

   if XYunit eq 1 then X=xx/dst*min(xypsiz)
   if XYunit eq 0 then X=atan(xx/(dst))

   txt='2Theta'
   X  = 4.*!pi*sin(X/2.)/wavl  ; Make unit in Q
   txt='Q'
   Z  = sans_sigmaQ(X=X)       ; DEFAULT SIGMAQ
endif

;***** Return All *****
sw=size(Wout)
if sw[0] eq 2 then begin
   datp.x_tit=txt
   datp.y_tit="Numors"
   datp.z_tit="Integrated counts"
   Y=findgen(sw[2])+1
endif else begin
   datp.x_tit=txt
   datp.y_tit="Integrated counts"
   datp.z_tit=""
   Y=0.
endelse

MOD_DATP ,datp,"X",X
MOD_DATP ,datp,"Y",Y
MOD_DATP ,datp,"Z",Z
MOD_DATP ,datp,"E",error
GIVE_DATP,datp

if batch then if sans_supl[78] gt 0 then begin ;NOTE: for 360 radial integration only
    wi = long(33+sans_supl[78]) ;-> w34, w35, w36
    GIVE_W,    Wout, W=wi
    GIVE_DATP, datp, W=wi
    common c_sans_batch,  parconf, bid1, bid2, bid3, bid4
    sans_reform_series, Wout, dp=datp, PARCONF=parconf, RESUME=2 ;keep results in memory
endif

return,Wout
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

function sans_read_graspr, fifi ,datp=datp
;+
;******* ****************
;** read Grasp data, should be a bit old
;**
;-
Wout=0 & line='' & WT=fifi & numor=0 & trans=0. & thick=0. & cx0=0. & cy0=0.
tab =fltarr(4)
on_ioerror,misgrasp & u=-1 & ok=0

openr,u,fifi,/get_lun

readf,u,line & header=[line]
while (strpos(line,'* Analysis *') lt 0) do begin & readf,u,line & header=[header,line] & endwhile
readf,u,line & header=[header,line]  & OT=line
while (strpos(line,'Mod_') ne 0)         do begin & readf,u,line & header=[header,line] & endwhile
Mod_=strsplit(/EXTRACT,strcompress(line),' ')    & XT=Mod_[0] & YT=Mod_[1]
line='' & while (line le '  ') do readf,u,line

reads,line,tab & Wout =[tab]          ;First line of values
while not eof(u) do begin
readf,u,tab    & Wout =[[Wout],[tab]] ;folowing lines
endwhile
ok=1
misgrasp:if u gt 0 then free_lun,u
if not ok then lamp_error
if n_elements(Wout) ge 4 then begin
   Q   =reform(Wout[0,*])
   I   =reform(Wout[1,*])
   E   =reform(Wout[2,*])
   dQ  =reform(Wout[3,*]) & ZT='Delta Q'
   
   idx =where(strpos(Header,'Subtitle:')   eq 0) & if idx[0] gt 0 then WT=strmid(Header[idx],10,80)
   idx =where(strpos(Header,'Foreground:') eq 0) & if idx[0] gt 0 then begin & tmp=Header[idx]
        pos=  strpos(tmp   ,'Loaded:') &   if pos gt 0 then numor=long(strmid(tmp,pos+7,10))
        endif
   idx =where(strpos(Header,'Transmissions:') eq 0) & if idx[0] gt 0 then begin & tmp=Header[idx]
        pos=  strpos(tmp   ,'Ts =')    &   if pos gt 0 then trans=float(strmid(tmp,pos+4,10))
        endif
   idx =where(strpos(Header,'Data: Divided') eq 0)  & if idx[0] gt 0 then begin & tmp=Header[idx]
        pos=  strpos(tmp   ,'Volume')  &   if pos gt 0 then thick=float(strmid(tmp,pos+6,10))
        endif
   idx =where(strpos(Header,'Beam Centre1')  eq 0) & if idx[0] gt 0 then begin & tmp=Header[idx]
        pos=  strpos(tmp   ,'Cx =')    &   if pos gt 0 then cx0  =float(strmid(tmp,pos+4,10))
        pos=  strpos(tmp   ,'Cy =')    &   if pos gt 0 then cy0  =float(strmid(tmp,pos+4,10))
        endif
;;*IDXP*
 ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
 ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight ->** 0->14 **

;*OTHER PARAMS FOR SUPL*
 ;center X , center Y , thickness , dTh , typ , transm , group , X Psize , Y Psize , instrument
 ;title , sector open , sector rotat , flux ** 15->28 **

   P=fltarr(31) & Ptxt=strarr(31)
   P[6] =numor                  & Ptxt[6] ='6)  Numor       '
   P[15]=cx0                    & Ptxt[15]='15) X center    '
   P[16]=cy0                    & Ptxt[16]='16) Y center    '
   P[17]=thick                  & Ptxt[17]='17) Thickness Cm'
   p[19]=2.                     & Ptxt[19]='19) Typ !!!     '
   P[20]=trans                  & Ptxt[20]='20) Transmission'
   P[25]=0.                     & Ptxt[25]='25) '+Wt

   ptxt=[ptxt,'',header]+'=' & P=[P,0,fltarr(n_elements(header))]

   datp={X:Q, Y:0, E:E, Z:dQ, P:p, par_TXT:Ptxt, x_tit:XT, y_tit:YT, w_tit:WT, other_tit:fifi+' '+OT}
   give_datp,datp
   wout=I
endif

return,Wout
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

function sans_read_gt, fifi, datp=datp
;+
;******* ************
;**
;** Read file in Ron format ILL-SANS treated data
;-

treatment=[  $
'UNKNOWN_0', $                    ;typ=0
'CORRECTED DATA', $                ;typ=1  Xt='X detector'    Xt='Y detector'
'RADIAL INTEGRATIONS', $           ;typ=2  Xt='Q'             Yt='Frames' or 'counts'
'UNKNOWN_3',           $           ;typ=3
'TOTAL AZIMUTHAL',     $           ;typ=4  Xt='Degree'        Yt='Total'
'SECTOR TOTAL  SYMMETRICAL', $     ;typ=5  Xt='Sectors (deg)' Yt='numors' or 'Total'
'SECTOR TOTAL  NON_SYMMETRICAL', $ ;typ=6  Xt='Sectors (deg)' Yt='numors' or 'Total'
'SECTOR RADIAL SYMMETRICAL'    , $ ;typ=7  Xt='Q' Yt='S(Q)' or 'Numors' or 'Sectors' or 'Numors by sectors'
'SECTOR RADIAL NON_SYMMETRICAL']   ;typ=8  Xt='Q' Yt='S(Q)' or 'Numors' or 'Sectors' or 'Numors by sectors'

wout=0. & E=0.
on_ioerror,misGT & u=-1 & line=' ' & head=lonarr(6) & flt=0. & ok=0
openr,u,fifi,/get_lun
  readf,u,line & Wt=strcompress(line) ;READ THE HEADER
  readf,u,line & instr=strsplit(/EXTRACT,strcompress(line)+' a b c',' ') & instr=instr[2]
  readf,u,head & numor=head[0]  & dx=head[2]    & dy=head[3]   & skip=head[4]-1
  readf,u,head & commt=head[1]  & ierr=head[5]                 & skip=skip-1
  readf,u,line & skip=skip-1
  if strpos(line,'Lamp') eq 0 then wlamp=1 else wlamp=0

  Ot=fifi
  if dy gt 1 then begin & Xt='X detector' & Yt='Y detector' & wout=fltarr(dx,dy) & typ=1 & endif
  if dy le 1 then begin & Xt='Q'          & Yt='S(Q)'       & wout=fltarr(3,dx)  & typ=2 & endif

  if commt gt 0 then for i=1,commt do begin ;READ THE COMMENTS
     readf,u,line & skip=skip-1
     if wlamp then if i eq 1 then Ot=line+' '+fifi else $
                   if i eq 2 then Xt=line          else $
                   if i eq 3 then Yt=line
     if i eq 1 then begin
        idxtyp=where(treatment eq line) & if idxtyp ge 0 then typ=idxtyp
     endif
  endfor

  ptxt=strarr(skip) & p=fltarr(skip) ;READ THE PARAMETERS
  j=0
  for i=0,skip-1 do begin
      readf,u,line & k=strpos(line,' !')
      if k gt 0 then begin & reads,line,flt & p[j]=flt & ptxt[j]=strcompress(strmid(line,k+2,80)) & j=j+1 & endif
  endfor
 ;*IDXP*
 ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
 ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight
  idx2P=[5,4,0,28,6,23,28,28,28,16,19,20,12,13,14] ;0->14
 ;*OTHER PARAMS FOR SUPL*
 ;center X , center Y , thickness , dTh , typ , transm , group , X Psize , Y Psize , instrument
 ;title , sector open , sector rotat , flux
  idx2S=[1,2,22,31,28,21,3,17,18,28,28,10,11,9]    ;15->28

  P=P[[idx2P,idx2S]]           & Ptxt=Ptxt[[idx2P,idx2S]]
  
  P[3] =0.                     & Ptxt[3] =' Detector angle'
  P[6] =numor                  & Ptxt[6] =' Numor'
  P[7] =0.                     & Ptxt[7] =' Attenuator index'
  P[8] =0.                     & Ptxt[8] =' Attenuator On/Off'
  P[12]=P[12]/100.
  P[15]=(P[15]+1)/(P[22]/10)   & Ptxt[15]=' X center'
  P[16]=(P[16]+1)/(P[23]/10)   & Ptxt[16]=' Y center'
  P[17]=P[17]/10.              & Ptxt[17]=' Thickness Cm'
  P[18]=P[18]/1000.            & Ptxt[18]=' Delta Theta' 
  P[19]=typ                    & Ptxt[19]=' '+treatment[typ]
  P[21]=P[21]/max(P[22:23]/10) & Ptxt[21]=' Grouping Pix'
  P[22]=P[22]/10.              & Ptxt[22]=' X pixel size Cm'
  P[23]=P[23]/10.              & Ptxt[23]=' Y pixel size Cm'
  P[24]=0.                     & Ptxt[24]=' '+instr
  P[25]=0.                     & Ptxt[25]=' '+Wt
  ptxt=ptxt+'='


  readf,u,wout ;READ THE DATA
  if (dy gt 1) and (ierr) then begin & E=wout*0. & readf,u,E & endif

 ;REFORM THE DATA
  if (dy eq 1) then begin & X=reform(Wout[0,*]) & E=reform(Wout[2,*]) & Y=numor & Wout=reform(Wout[1,*])
                          idx=where( (Wout ne 0.) or (E ne 0.) )   ;Remove 0 points!
                          if n_elements(idx) ne n_elements(E) then begin & Wout=Wout[idx] & E=E[idx] & X=X[idx] & endif
  endif
  if (dy gt 1) then begin & X=findgen(dx) & Y=findgen(dy) & endif

  datp={w_tit:Wt, x_tit:Xt, y_tit:Yt, Other_tit:Ot,P:p,par_txt:ptxt,X:X,Y:Y,E:E}
  if not arg_present(datp) then GIVE_DATP,datp
  ok=1
misGT:if u gt 0 then free_lun,u
if not ok then lamp_error
return,wout
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

function sans_transm, rd_struc, Sbeam=nsbeam,water=nwater,Ewater=newater,sample=nsample,Esample=nEsample,Wbeam=nwbeam,$
                      xrange=xrange,yrange=yrange, MASk=M
;+
;******* ***********
;** User callable
;** calculate transmission from empty can or direct beam
;** Call t=sans_transm(Sbeam=5, water=1, Ewater=2, sample=3 ,Esample=4, xrange=[10,50],yrange=[11,49])
;**   or t=sans_transm(structure, /MASk)
;**       {Tsample:'33:35', Tscell:'44', Tsbeam:'55', Twater:'11', Twcell:'22', Twbeam:'66'}
;**
;** !beam must by supplied, the others are optionals.
;** returns an array of transmission values.
;**                       ----------------
;** D.R. Feb   2009 prefere empty can to calculate transmission
;** D.R  Oct   2009 accept sample empty can series
;** D.R  May   2014 accept a structure from a batch command
;-

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

;SANS_TRANS: transmission   [ [Water , WaterCell , Sample1 , SampleCell1 , Sample1 , Sample2 , ...],
;            thickness        [      , 0         ,         , 0           ,         ,         , ...],
;            tr.error Sample  [Water , WaterCell , Sample1 , SampleCell1 , Sample1 , Sample2 , ...],
;            tr. SampleCell   [0     , 0         , 0       , SampleCell1 , SampleCell1 , SampleCell2 , ...]  (if series)
;            tr.error S.Cell  [0     , 0         , 0       ,             ,             ,             , ...]

ntr = n_elements(sans_trans[*,0]) & trans=fltarr(ntr,2)
siz = size(rd_struc)

if siz[siz[0]+1] eq 8 then begin     ;we have a structure, so it is a batch command
   sans_settings_runs_live, rd_struc ;read the runs in correct workspaces
   return, sans_transm(Sbeam=5, water=1, Ewater=2, sample=3, Esample=4, Wbeam=6,$
                       xrange=xrange, yrange=yrange, MASk=M)
endif

if   n_elements(nsbeam) eq 1 then begin & TAKE_W,beam,W=nsbeam & TAKE_DATP,datp,W=nsbeam & siz=size(beam) & endif
if siz[0] ne 2 then $
  if n_elements(nwbeam) eq 1 then begin & TAKE_W,beam,W=nwbeam & TAKE_DATP,datp,W=nwbeam & siz=size(beam) & endif

if siz[0] eq 2 then begin ; one of the beams must be there
   s1=siz[1]-1 & s2=siz[2]-1 & a1=0 & a2=0 & b1=0 & b2=0
   if n_elements(xrange) eq 2 then begin & a1=(xrange[0]-1)>0<s1 & a2=(xrange[1]-1)>0<s1 & endif
   if n_elements(yrange) eq 2 then begin & b1=(yrange[0]-1)>0<s2 & b2=(yrange[1]-1)>0<s2 & endif
   if a2 le a1 then begin & a1=0 & a2=s1 & endif
   if b2 le b1 then begin & b1=0 & b2=s2 & endif

   if n_elements(sans_supl)   le 1 then sans_settings,/nowind    ; init sans_settings   ! (unprobable case)
   if            sans_supl[0] eq 0 then sans_settings_par,datp   ; get  data parameters ! (unprobable case)
   
   UseTime=sans_supl[48] ;0 to use monitors , 1 to use time
   TimeIDX=sans_supl[85] ;index of time in DATP.p
   
   tau=sans_supl[6]*sans_supl[27] & ftau=sans_supl[7] & tim=datp.p[TimeIDX]
  ;if sans_supl[66] gt 1 then tau = 0. ;disable dead times correction whith multi-panels data (commented for transmissions!)
   if UseTime then NB=tim else NB=datp.N[0]

   if n_elements(M) ne n_elements(beam) then if n_elements(M) eq 1 then $
              if M  eq 1 then sans_settings_getmask,siz,[0.,0.],M,/nobeam,/noWbeam
   if n_elements(M) ne n_elements(beam) then masq=1 else masq=1-M[a1:a2,b1:b2]

   tbeam=beam
   if tau*tim gt 0. then sans_tau, tbeam ,0 ,ftau ,tau ,tim ;dead times Beam sample

   tbeam=total(tbeam[a1:a2,b1:b2]*masq)

   if n_elements(nwbeam)   eq 1 then TAKE_W,wbeam ,W=nwbeam
   if n_elements(wbeam)    eq n_elements(beam) then begin
      TAKE_DATP,datpbw,W=nwbeam  & tim=datpbw.p[TimeIDX]
      if UseTime then NW=tim else NW=datpbw.N[0]
      if tau*tim gt 0. then sans_tau, wbeam ,0 ,ftau ,tau ,tim ;dead times Beam water

      wbeam=total(wbeam[a1:a2,b1:b2]*masq)
   endif else begin & wbeam=tbeam & NW=NB & endelse

;EMPTY WATER TRANSMISSION
;************************
   if n_elements(nEwater)  eq 1 then TAKE_W,Ewater,W=nEwater
   if n_elements(Ewater)   eq n_elements(beam) then begin
      TAKE_DATP,datpew,W=nEwater & tim=datpew.p[TimeIDX]
      if UseTime then NWe=tim else NWe=datpew.N[0]
      if tau*tim gt 0. then sans_tau, Ewater ,0 ,ftau ,tau ,tim ;dead times Water can

      tEwater=total(Ewater[a1:a2,b1:b2]*masq)

      sans_trans[1,0]=tEwater*NW/NWe/wbeam
      sans_trans[1,2]=sqrt((sqrt(tEwater)/wbeam)^2 + (sqrt(wbeam)*tEwater/wbeam^2)^2)*NW/NWe ;sigma
      trans[1,0]     =sans_trans[1,0]
   endif

;WATER TRANSMISSION
;******************
   if n_elements(nwater)   eq 1 then TAKE_W,woter ,W=nwater
   if n_elements(woter)    eq n_elements(beam) then begin
      TAKE_DATP,datpw,W=nwater  & tim=datpw.p[TimeIDX]
      if UseTime then NS=tim else NS=datpw.N[0]
      if tau*tim gt 0. then sans_tau, woter ,0 ,ftau ,tau ,tim ;dead times Water

      twater=total(woter[a1:a2,b1:b2]*masq)

     ;*EITHER USE BEAM OR WATER CELL*
      if n_elements(tEwater) eq 1 then begin & direct=tEwater & ND=NWe
      endif else begin & direct=wbeam & ND=NW & sans_trans[1,1]=0. & endelse

      sans_trans[0,0]=twater*ND/NS/direct
      sans_trans[0,2]=sqrt((sqrt(twater)/direct)^2 + (sqrt(direct)*twater/direct^2)^2)*ND/NS ;sigma
      trans[0,0]     =sans_trans[0,0]
   endif

;EMPTY SAMPLE TRANSMISSION
;*************************
   if n_elements(nEsample) eq 1 then TAKE_W,Esample,W=nEsample
   if n_elements(Esample)  gt 1 then $
   if n_elements(Esample[*,*,0]) eq n_elements(beam) then begin
      TAKE_DATP,datpes,W=nEsample
      s =size(Esample) & if  s[0] gt 2  then nz=s[3]-1 else nz=0
      if nz eq 0 then tim=datpes.p[TimeIDX] else tim=datpes.pv[TimeIDX,*]
      if UseTime then NSe=tim[0:nz] else NSe=datpes.N[0:nz]
      if tau*tim[0] gt 0. then sans_tau, Esample ,0 ,ftau ,tau ,tim ;dead times Sample can

      tEsample=fltarr(nz+1) & for j=0L,nz do tEsample[j]=total(Esample[a1:a2,b1:b2,j]*masq)

      tmp     =tEsample*NB/NSe/tbeam
      tmpE    =sqrt((sqrt(tEsample)/tbeam)^2 + (sqrt(tbeam)*tEsample/tbeam^2)^2)*NB/NSe ;sigma

      if nz eq 0 then begin & tEsample=tEsample[0] & NSe=NSe[0] & endif
      sizt=size(sans_trans)
      if ((nz gt 0) and (sizt[2] lt 5)) $
      or (sizt[1] lt nz+1+4) then begin & arr=fltarr(max([nz+1+4,sizt[1]]),sizt[2]>5) & arr[0:sizt[1]-1,0:sizt[2]-1]=sans_trans
                                        if sizt[1] lt nz+1+4 then arr[sizt[1]:*,1]=arr[2,1]
                                        sans_trans=arr & trans=sans_trans[*,[0,3]] & endif
      sans_trans[3,0] = tmp[ 0]
      sans_trans[3,2] = tmpE[0]
      if nz gt 0 then begin & sans_trans[3,3]=tmp[ 0]  & sans_trans[4:4+nz,3] = tmp
                              sans_trans[3,4]=tmpE[0]  & sans_trans[4:4+nz,4] = tmpE
      endif
      trans[3,0]      = tmp[ 0]  & if nz gt 0 then trans[4:4+nz,1] = tmp
   endif

;SAMPLE TRANSMISSION
;*******************
   if n_elements(nsample)  eq 1 then TAKE_W,sample,W=nsample
   if n_elements(sample)   gt 1 then $
   if n_elements(sample[*,*,0]) eq n_elements(beam) then begin
      TAKE_DATP,datps,W=nsample
      s =size(sample) & if  s[0] gt 2  then nz=s[3]-1 else nz=0
      if nz eq 0 then tim=datps.p[TimeIDX] else tim=datps.pv[TimeIDX,*]
      pZ=datps.Z & npZ=n_elements(pZ) & if (npZ ne nz+1) or (pZ[0] eq pZ[npZ-1]) then pZ=lindgen(nz+1)
      if UseTime then NS=tim[0:nz] else NS=datps.N[0:nz]
      if tau*tim[0] gt 0. then sans_tau, sample ,0 ,ftau ,tau ,tim ;dead times Sample

      tsample=fltarr(nz+1) & for j=0L,nz do tsample[j]=total(sample[a1:a2,b1:b2,j]*masq)

      sizt=size(sans_trans)
      if sizt[1] lt nz+1+4 then begin & arr=fltarr(nz+1+4,sizt[2]) &  arr[0:sizt[1]-1,*]=sans_trans[0:sizt[1]-1,*]
                                      arr[sizt[1]:*,1]=arr[2,1]  &  sans_trans=arr & trans=sans_trans[*,[0,3]] & endif
     ;*EITHER USE BEAM OR SAMPLE CELL*
      nzc=n_elements(tEsample) & if nzc gt 1 then pZ=pZ[0:(nzc-1)<nz]
      if nzc ge 1 then begin & direct=tEsample[pZ] & ND=NSe[pZ]
      endif else begin & direct=tbeam & ND=NB & sans_trans[3,1]=0. & endelse

      tmp    =tsample*ND/NS/direct
      tmpE   =sqrt((sqrt(tsample)/direct)^2 + (sqrt(direct)*tsample/direct^2)^2)*ND/NS ;sigma
      if (pZ[0] eq 0) then begin & sans_trans[2,0]=tmp[0] & sans_trans[2,2]=tmpE[0] & endif

      sans_trans[4+pZ,0]=tmp & sans_trans[4+pZ,2]=tmpE
      trans[2,0]=tmp[0] & trans[4:4+n_elements(tmp)-1,0]=tmp
   endif

   sans_trans[*,0]=round(sans_trans[*,0]*1000.)/1000.
   sans_trans[*,2]=round(sans_trans[*,2]*1000.)/1000.
endif
return,trans
end

function sans_sigmaQ, X=Qx , dL_L=dL_L , Lambda=Lambda , dTh=dTh
;+
;******* ***********
;** Return sigma Q
;** Used by sans_merge_in_file
;**         sans_radial
;**         sans_settings_event-> 72: radial integration 360deg and sectors
;**                               76: plot dQ or Fit
;**                               78:-> 5: save selected curves
;-
common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl
common c_sans_tofl,     TofFlux, LambdaVq

if n_elements(dL_L)   eq 0 then dL_L  = sans_supl[50] ;delta_lambda/lambda
if n_elements(Lambda) eq 0 then Lambda= sans_supl[0]  ;lambda
if n_elements(dTh)    eq 0 then dTh   = sans_supl[49] ;delta theta calculated in sans_center

; If TOF merged data as D33. Un_consistant if calculated from result vectors
okTOFmerged = ((sans_supl[64] eq 4.) and ((size(LambdaVq))[2] eq 2))

F1 = (1./(2.*sqrt(2.*alog(2.)))*dL_L)^2
;                                         lambda        Q             Qout
if okTOFmerged then F2 = (4.*!pi/interpol(LambdaVq[*,1],LambdaVq[*,0],Qx))^2 $
               else F2 = (4.*!pi/Lambda)^2

if dTh gt 0. then dQ = sqrt(Qx^2   *F1 + (F2-Qx^2)   *dTh^2) $ ; sigmaQ
             else dQ = Qx*0.
return,dQ
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************


;independent procedures
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_tau, Wout ,Eout ,ftau ,tau ,timS
;+
;** ********
;** User callable
;** dead time correction
;** Wout input/output data  (2D or 3D)
;** Eout input/output error (2D or 3D or 0)
;** ftau function index [None , I/(1-tau*tot/time) , I/exp(-tau*tot/time) , by vert tubes , by hori tubes]
;** tau  tau value
;** tims acquisition time (sec)
;-

    siz=size(Wout) & if siz[0] eq 3 then nz=siz[3]-1 else nz=0
    Eok=(n_elements(Eout) eq n_elements(Wout))

    if ftau eq 1 then begin & Tproj=total(Wout,2) & Tproj=total(Tproj,1)
                            for j=0L,nz do begin & fY= (1.-tau*Tproj[j]/timS[j])  & Wout[*,*,j]=Wout[*,*,j]/fY
                                                                 if Eok then      Eout[*,*,j]=Eout[*,*,j]/fy & endfor & endif
    if ftau eq 2 then begin & Tproj=total(Wout,2) & Tproj=total(Tproj,1)
                            for j=0L,nz do begin & fY=exp(-tau*Tproj[j]/timS[j])  & Wout[*,*,j]=Wout[*,*,j]/fY
                                                                 if Eok then      Eout[*,*,j]=Eout[*,*,j]/fy & endfor & endif
    if ftau eq 3 then begin & Xproj=total(Wout,2)
                            for j=0L,nz do begin & fY=(1.-tau*Xproj[*,j]/timS[j]) & fY=fY#(fltarr(siz[2])+1.)
                                                                                  Wout[*,*,j]=Wout[*,*,j]/fY
                                                                 if Eok then      Eout[*,*,j]=Eout[*,*,j]/fy & endfor & endif
    if ftau eq 4 then begin & Yproj=total(Wout,1)
                            for j=0L,nz do begin & fY=(1.-tau*Yproj[*,j]/timS[j]) & fY=(fltarr(siz[1])+1.)#fY
                                                                                  Wout[*,*,j]=Wout[*,*,j]/fY
                                                                 if Eok then      Eout[*,*,j]=Eout[*,*,j]/fy & endfor & endif
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_mask, baset, instrument=inst, clear=clear, back=back, Xlines=xlin, Ylines=ylin, XYbeam=xyb, box=box, $
                      ellipse=ellipse,rectangle=rectang, waterbox=wbox,  noplot=noplot,plot=plotit, Win=Win ,center=ctr
;+
;** *********
;** User callable
;** set instrument mask
;** Instrument set default instrument mask
;**/Clear      reset the  detector mask (to reset beamsize set xybeam to [0,0])
;**/Back       undo  last detector mask setting (xline,yline,box)
;**/ellipse    the shape of the beam_Stop
;**/rectangle  the shape of the beam_Stop
;** Xline      mask  the  vertical   detector line at X (x=1>m)
;** Yline      mask  the  horizontal detector line at Y (y=1>n)
;** XYbeam     Sample beamsize to mask [nx,ny] around the center or [xmin,ymin,nx,ny]
;** Wat
;** Box        [x,y , nx,ny] means mask detector from x to x+nx-1 and y to y+ny-1  (x=1>n , y=1,m)
;** If plot    is not set then pass a 2D data run in Win just in case the size of the mask is unknown
;** center     [cx,cy] is called by SANS_CENTER to fill the sans_ctr common variable
;** D.R. April 2007
;-

;** baset      added when XYbeam is set, we check if water beam can also be filled. D.R. Mar 2011

@lamp.cbk
common c_trap,  trap_x1,trap_x2,trap_y1,trap_y2,trap_ws, trap_current
common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

;sans_masq = detector mask [[1,1,Sx,Sy] , [x1,y1,Nx,Ny] , [....]]
;sans_beam = beam     mask [samplNX,samplNY , rect|elli , watrX1,watrY1 , watrNX,watrNY , samplX1,samplY1]
;                              0       1          2         3        4      5      6         7       8

forward_function sans_settings_RegSheet, sans_inst_init

if n_elements(sans_supl)   le 1 then sans_settings,inst=inst,/nowind    ; init sans_settings   ! (unprobable case)
s = -1
if n_elements(inst) eq 1 then begin
   InstList  = sans_inst_init('List1')
   s = where(InstList eq strupcase(inst)) & s = s[0]
   sans_masq = sans_inst_init('Mask' , s, detres=sans_supl[8:9]) ; Get the default mask for the selected instrument (first call)
endif
   
if s ge 0 then sans_supl[5]=s
if n_elements(ctr) ge 2 then sans_ctr=ctr

;Get the size of the mask
; -----------------------
siz=[0L] & if (n_elements(trap_ws) ne 1) or (not keyword_set(plotit)) then noplot=1
if n_elements(Win) gt 1 then siz=size(Win) else if not keyword_set(noplot) then ii=execute('siz=size(w'+trap_ws+')')
if n_elements(sans_masq) eq 0 then if siz[0] eq 2 then sans_masq=[1,1,siz[1],siz[2]] else return
if n_elements(Win) gt 1 then sans_masq[*,0]=[1,1,siz[1],siz[2]]
s1=sans_masq[2]-1  & s2=sans_masq[3]-1
sm=size(sans_masq) & if sm[0] eq 2 then sm2=sm[2] else sm2=1


;Usefull supl variables ONLY to fill sample or water beamStop when similar
FX    =long(sans_supl[30])
Dis   =sans_supl[1]   ; Distance    Sample
Disw  =sans_supl[32]  ; Distance    Water
OffS  =sans_supl[2]   ; Offset      Sample
OffW  =sans_supl[51]  ; Offset      Water
COLLs =sans_supl[4]   ; Collimation sample
COLLw =sans_supl[34]  ; Collimation water


;Process all keywords
; -------------------
if keyword_set(ellipse)   then sans_beam[2]=1
if keyword_set(rectang)   then sans_beam[2]=0
if keyword_set(clear)     then begin & sans_masq=[1,1,s1+1,s2+1] & endif ;& kp=sans_beam(2) & sans_beam(*)=0. & sans_beam(2)=kp

if keyword_set(back)      then if n_elements(sans_masq) gt 4 then sans_masq=sans_masq[*,0:(size(sans_masq))[2]-2]


if n_elements(xyb)  eq 2  then begin & sans_beam[0]=xyb[0]>0<s1 & sans_beam[1]=xyb[1]>0<s2   & endif
                                      ;sans_beam[7]=0           & sans_beam[8]=0   & endif
if n_elements(xyb)  eq 4  then begin & sans_beam[0]=xyb[2]>0<s1 & sans_beam[1]=xyb[3]>0<s2
                                       sans_beam[7]=xyb[0]      & sans_beam[8]=xyb[1]
   dodo=1 & if dodo then $
   if sans_beam[3]*sans_beam[5] le 0 then $ ;SHOULD BEAM WATER INHERITES FROM SAMPLE!
      if (abs(Dis -DisW) le .1) and (abs(OffS-OffW) le .1) then begin
          sans_beam[3:6]=sans_beam[[7,8,0,1]]
          if n_elements(baset) gt 0 then begin
             sbox =sans_beam[3:6] & stbox=long([sbox[0],sbox[0]+sbox[2]-1,sbox[1],sbox[1]+sbox[3]-1]+FX)
             stbox=strtrim(stbox[0],2)+','+strtrim(stbox[1],2)+','+strtrim(stbox[2],2)+','+strtrim(stbox[3],2)
             sans_settings_swap,baset,waterbox=sbox ,forWater=baset.curWATER
             if sans_settings_RegSheet() then sans_sheet_receive,waterbox=stbox
          endif
      endif
endif

if n_elements(xlin) ge 1  then if xlin[0] gt 0 then $
   for i=0,n_elements(xlin)-1 do begin & newi=[xlin[i]>1,1,1,s2+1] & ok=1 ; xlin(i)>1<(s1+1) but image can be wider (D33,D16)
       if sm2 gt 1 then for j=1,sm2-1 do if (sans_masq[0,j] eq newi[0]) and $
                                            (sans_masq[1,j] eq newi[1]) and $
                                            (sans_masq[2,j] eq newi[2]) and $
                                            (sans_masq[3,j] eq newi[3]) then ok=0
       if ok then sans_masq=[[sans_masq],[newi]]
   endfor
if n_elements(ylin) ge 1  then if ylin[0] gt 0 then $
   for i=0,n_elements(ylin)-1 do begin & newi=[1,ylin[i]>1,s1+1,1] & ok=1 ; ylin(i)>1<(s2+1) but image can be wider (D33,D16)
       if sm2 gt 1 then for j=1,sm2-1 do if (sans_masq[0,j] eq newi[0]) and $
                                            (sans_masq[1,j] eq newi[1]) and $
                                            (sans_masq[2,j] eq newi[2]) and $
                                            (sans_masq[3,j] eq newi[3]) then ok=0
       if ok then sans_masq=[[sans_masq],[newi]]
   endfor
if n_elements(box)  eq 4  then begin & ok=1
       if sm2 gt 1 then for j=1,sm2-1 do if (sans_masq[0,j] eq box[ 0]) and $ 
                                            (sans_masq[1,j] eq box[ 1]) and $ 
                                            (sans_masq[2,j] eq box[ 2]) and $ 
                                            (sans_masq[3,j] eq box[ 3]) then ok=0
       if ok then sans_masq=[[sans_masq],[box]]
endif

if n_elements(wbox) eq 4  then begin & sans_beam[3:6]=wbox
   if sans_beam[0]*sans_beam[1] le 0 then $ ;SHOULD BEAM SAMPLE INHERITES FROM WATER!
      if (abs(Dis -DisW) le .1) and (abs(OffS-OffW) le .1) then begin
          sans_beam[[0,1,7,8]]=sans_beam[[5,6,3,4]]
          sbox =sans_beam[3:6] & stbox=long([sbox[0],sbox[0]+sbox[2]-1,sbox[1],sbox[1]+sbox[3]-1]+FX)
          stbox=strtrim(stbox[0],2)+','+strtrim(stbox[1],2)+','+strtrim(stbox[2],2)+','+strtrim(stbox[3],2)
          if sans_settings_RegSheet() then sans_sheet_receive,beambox=stbox
      endif
endif

;Show the mask over last plotted workspace
; ----------------------------------------
if not keyword_set(noplot) then if (keyword_set(clear)) or (keyword_set(back)) or $
      (n_elements(sans_masq) gt 4) or (sans_beam[0]*sans_beam[1] gt 0) then begin
    TAKE_W , Wxx ,W=long(trap_ws)
    m=min(wxx)
    k=sans_masq-1
    b=sans_beam[0:1]/2.
    c=sans_ctr[0:1] -1.
    if (n_elements(k) gt 4) then for i=1,(size(k))[2]-1 do $
                                 wxx[k[0,i]<s1:(k[0,i]+k[2,i])<s1 , k[1,i]<s2:(k[1,i]+k[3,i])<s2]=m

    if b[0]*b[1] gt 0 then begin & a1=round(c[0]-b[0])>0<s1 & a2=round(c[0]+b[0])>0<s1
                                 b1=round(c[1]-b[1])>0<s2 & b2=round(c[1]+b[1])>0<s2
                                 wxx[a1:a2,b1:b2]=m & endif

    kpt=[trap_x1,trap_x2,trap_y1,trap_y2] & kp_ws=trap_ws
    trap_set,smoo=0,getfu=fu
    sans_gt_par ,sys=sys
    GIVE_W , Wxx,  W=sys+3
    if fu then see,W=sys+3,/image,/replot else see,W=sys+3,/image
    trap_ws=kp_ws
    trap_x1=kpt[0] & trap_x2=kpt[1] & trap_y1=kpt[2] & trap_y2=kpt[3]
endif
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_read_mask, fifi
;+
;** **************
;** User callable (to verify)
;** read a saved instrumen mask file
;-

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

if n_elements(sans_supl) le 1 then sans_settings,/nowind    ; init sans_settings   ! (unprobable case)
dx=long(sans_masq[2])
dy=long(sans_masq[3])
on_ioerror,mismsk & u=-1 & ok=0 & tit=' ' & nb=0L
openr,u,fifi,/get_lun
readf,u,tit
readf,u,nb
if nb ne dx*dy then begin & dx=long(sqrt(nb)) & dy=dx & endif

M=strarr(dy) & readf,u,M                     ;READ SYMBOL MASK

readf,u,nb
MM=lonarr(4,nb) & readf,u,MM & sans_masq=MM  ;READ ARRAY MASK
ok=1
mismsk:if u gt 0 then free_lun,u
if not ok then lamp_error
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_del_select_results, getlist=gtlst, index=index
;+
;** ***********************
;** delete selected Q vector results from gui
;-

common resume_iso, iso_conf, iso_npt, iso_WQE, iso_NTC,    iso_p, iso_supl
common resume_azi, azi_conf, azi_npt, azi_WQE, azi_NTC,    azi_p, azi_supl
ok=0
if (n_elements(gtlst) eq 1) and (n_elements(index) gt 0) then begin
  if (gtlst eq 5) or (gtlst eq 6) then gtlist=4 else $            ; TOTAL  SECTOR = AZIMUTAL
  if (gtlst eq 7) or (gtlst eq 8) then gtlist=2 else gtlist=gtlst ; RADIAL SECTOR = RADIAL

  if gtlist eq 2 then begin ; ISO
    idx=lindgen(n_elements(iso_conf))
    idx[index]=-1 & idx=where(idx ge 0)
    if idx[0] ge 0 then begin
       iso_conf=iso_conf[ idx] & iso_npt=iso_npt[idx] & iso_WQE =iso_WQE[ *,idx]
       iso_NTC =iso_NTC[*,idx] & iso_p  =iso_p[*,idx] & iso_supl=iso_supl[*,idx]
       sans_reform_series,getlist =gtlist
       gtlst=gtlist & ok=1 ;Get new list
    endif else sans_reform_series,clear  =(gtlist)

  endif else $
  if gtlist eq 4 then begin ; AZI
    idx=lindgen(n_elements(azi_conf))
    idx[index]=-1 & idx=where(idx ge 0)
    if idx[0] ge 0 then begin
       azi_conf=azi_conf[ idx] & azi_npt=azi_npt[idx] & azi_WQE =azi_WQE[ *,idx]
       azi_NTC =azi_NTC[*,idx] & azi_p  =azi_p[*,idx] & azi_supl=azi_supl[*,idx]
       sans_reform_series,getlist =gtlist
       gtlst=gtlist & ok=1 ;get new list
    endif else sans_reform_series,clear  =(gtlist)
  endif
  if not ok then sans_reform_series,getlist =gtlst
endif
end

pro sans_merge_fields, baset, clear=Clmh, show=Siqm
;+
;** *****************
;** Clear and/or Map Qrange and factor fields
;-

;      Action  Qr     Scl   Lab     Qrange IDs ....................       Factor IDs ..      Bkg    4thBut 4thQr....    4thScl
;      2       3      4     5       6     7     8     9     10    11      12    13    14     15     16     17    18     19
;[76,0,butAct, QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld , bcbg , WSbas, WSfla,WSflb, WSfld]
widget_control,baset.bases4[17],get_uvalue=bRSshoUV
if n_elements(bRSshoUV) lt 14 then return ;NoTABS available

if n_elements(Clmh) ge 4 then begin ;Clear Qrange & Factor fields
   if Clmh[0] eq 1 then begin & widget_control,bRSshoUV[6]  ,set_value='' ; low
                                widget_control,bRSshoUV[7]  ,set_value=''
                                widget_control,bRSshoUV[12] ,set_value='' & endif
   if Clmh[1] eq 1 then begin & widget_control,bRSshoUV[8]  ,set_value='' ; middle
                                widget_control,bRSshoUV[9]  ,set_value=''
                                widget_control,bRSshoUV[13] ,set_value='' & endif
   if Clmh[2] eq 1 then begin & widget_control,bRSshoUV[10] ,set_value='' ; high
                                widget_control,bRSshoUV[11] ,set_value=''
                                widget_control,bRSshoUV[14] ,set_value='' & endif
   if Clmh[3] eq 1 then begin & widget_control,bRSshoUV[17] ,set_value='' ; 4TH config
                                widget_control,bRSshoUV[18] ,set_value=''
                                widget_control,bRSshoUV[19] ,set_value='' & endif

endif

if n_elements(Siqm) eq 1 then begin & mapi=[0,0,0,0] & mapi[Siqm]=1 ;Map Qrange or Scaling or Text base
   widget_control,bRSshoUV[5] ,map=mapi[0] ;Label
   widget_control,bRSshoUV[3] ,map=mapi[1] ;Qrange
   widget_control,bRSshoUV[4] ,map=mapi[2] ;Scale
   widget_control,bRSshoUV[16],map=mapi[3] ;4TH config
   if mapi[1] then actlab='Qrange' else if mapi[2] then actlab='Scales' $
                                   else if mapi[3] then actlab='4th Cf' else actlab='Action'
   widget_control,bRSshoUV[2],set_value=actlab
endif
end
  

function sans_merge_find_scal   , Lista, Listb, idxa, idxb, Clst
;+
;******* ********************
;**
;** For 2 config SORTED results Lista(lowerQ) , Listb(higherQ): 
;** returns [Scale for Lista ,
;**          right detector Qmargin for Lista ;
;**          left  beam     Qmargin for Listb ]
;** Does the work only for sans_merge_find_scaling()
;-

  aqqua=0. & aqqub=0.     & erra=0.  & errb=0.
  nla=n_elements(Lista)   & nlb=n_elements(Listb)
  if nlb le nla then Listb=[listb,strarr(nla-nlb+1)+'100000']
  kb=-1 & sampla=0L & samplb=0L & nfound=0.
  Freturn=[1., 0.,0.] ;SFactor, detector and beam margins
  F4=4.
  forcb=1 ;variable=1 if we just read both Lista and Listb

  FOR ka=0,nla-1 do begin ; Go through the two lists in parallel finding the same samples
      reads,Lista[ka],sampla
      if forcb then begin & kb=kb+1 & reads,Listb[kb],samplb & forcb=0     & endif
      
      while (samplb lt sampla) do begin & kb=kb+1 & reads,Listb[kb],samplb & endwhile
      
      if sampla eq samplb  then begin & forcb=1
         rpa=Clst & sans_reform_series,getlist=rpa,index=idxa[ka] ;Sample in first list
         rpb=Clst & sans_reform_series,getlist=rpb,index=idxb[kb] ;Corresponding sample in second list
         aqqua=aqqua+rpa.W & erra=erra+rpa.E                      ;Accumulate
         aqqub=aqqub+rpb.W & errb=errb+rpb.E & nfound=nfound+1.
      endif
  ENDFOR
  
  IF nfound gt 0. then begin                        ; We have found two curves
     aqqua=aqqua/nfound   & aqqub=aqqub/nfound
     erra =erra /nfound   & errb =errb /nfound
     Xa   =rpa.X          & Xb   =rpb.X
     nbXa =n_elements(Xa) & nbXb =n_elements(Xb)

     m=0 & n=nbXa-1 & OKrga=0
     if rpa.NTC[5] ne rpa.NTC[6] then begin         ;margins LIMIT INDEX first curve defined by the User
        if rpa.NTC[5] gt 0. then $
        m=(where(rpa.X ge rpa.NTC[5]))[0] > 0
        if rpa.NTC[6] gt 0. then $
        n=(where(rpa.X ge rpa.NTC[6]))[0] > 0
        if n le m then n=n_elements(Xa)-1
        if (m ne 0) or (n ne n_elements(Xa)-1) then OKrga=1
        aqqua=aqqua[m:n] & erra=erra[m:n] & Xa=Xa[m:n]
     endif
     
     m=0 & n=nbXb-1 & OKrgb=0
     if rpb.NTC[5] ne rpb.NTC[6] then begin         ;margins LIMIT INDEX second curve defined by the User
        if rpb.NTC[5] gt 0. then $
        m=(where(rpb.X ge rpb.NTC[5]))[0] > 0
        if rpb.NTC[6] gt 0. then $
        n=(where(rpb.X ge rpb.NTC[6]))[0] > 0
        if n le m then n=n_elements(Xb)-1
        if (m ne 0) or (n ne n_elements(Xa)-1) then OKrgb=1
        aqqub=aqqub[m:n] & errb=errb[m:n] & Xb=Xb[m:n]
     endif
     
     ddx=where((aqqua gt 1E-7) and (aqqua>1. gt F4*erra))
     if (ddx[0] ge 0) and (n_elements(ddx) ne n_elements(Xa)) then begin ;Discard low resolution points LRP
         Xa=Xa[ddx] & aqqua=aqqua[ddx]   &  erra=erra[ddx]  & endif
     ddx=where((aqqub gt 1E-7) and (aqqub>1. gt F4*errb))
     if (ddx[0] ge 0) and (n_elements(ddx) ne n_elements(Xb)) then begin ;Discard low resolution points LRP
         Xb=Xb[ddx] & aqqub=aqqub[ddx]   &  errb=errb[ddx]  & endif

     Qmin=Xb[0] & Qmax=Xa[n_elements(Xa)-1]          ;Find portion of curves in same Qrange
     idQa=where((Xa ge Qmin) and (Xa le Qmax))
     idQb=where((Xb ge Qmin) and (Xb le Qmax))
     
     if (idQa[0] ge 0) and (idQb[0] ge 0) then begin ;We have a common Qrange for both curves
       na=n_elements(idQa) & nb=n_elements(idQb)
       if  OKrga             then plima=0 else plima=5
       if  OKrgb             then plimb=0 else plimb=5
       if (OKrga or na le 3) then plsma=0 else plsma=1
       if (OKrgb or nb le 3) then plsmb=0 else plsmb=1
       lrpa=(na*10/100)>plsma<plima   ;Discard some points at the limit of the detector curve A
       lrpb=(nb*10/100)>plsmb<plimb   ;Discard some points at the limit of the beam     curve B
       aqqua=aqqua[idqa[0:na-1-lrpa]] & erra =erra[idqa[0:na-1-lrpa]] & Xa=Xa[idqa[0:na-1-lrpa]]
       aqqub=aqqub[idqb[lrpb:nb-1]]   & errb =errb[idqb[lrpb:nb-1]]   & Xb=Xb[idqb[lrpb:nb-1]]
       newNa=n_elements(Xa)           & newNb=n_elements(Xb)

       if na ge nb then begin ;Interpolate onto the minimal curve
                        Qya=interpol(aqqua,Xa,Xb,/QUAD) & Qyb=aqqub
                        Eya=interpol(erra ,Xa,Xb,/QUAD) & Eyb=errb
       endif else begin & Qyb=interpol(aqqub,Xb,Xa,/QUAD) & Qya=aqqua
                        Eyb=interpol(errb ,Xb,Xa,/QUAD) & Eya=erra & endelse
 
       Freturn[0]=median(Qyb/Qya, /EVEN) ;Errors are not used here
       if newNa ne nbXa then Freturn[1]=Xa[n_elements(Xa)-1]
       if newNb ne nbXb then Freturn[2]=Xb[0]
       
     endif else begin ;No common Qrange so try an other solution !!
       Qinter =(Qmax+Qmin)/2.
       Qya=interpol(aqqua,Xa,[Xa,Qinter],/QUAD) & Ainter=Qya[n_elements(Qya)-1]
       Qyb=interpol(aqqub,Xb,[Qinter,Xb],/QUAD) & Binter=Qyb[0]
       if Binter/Ainter gt 0. then Freturn[0]=Binter/Ainter
     endelse
  ENDIF ;nfound gt 0
  return,Freturn
end   

function sans_merge_find_scaling, Liste, Clst, idxl, idxm, idxh, WaveSuplement=WaveSuplement
;+
;******* ***********************
;** returns Fscale & Qmargin calculated from the sorted list indexed by idxl,idxm,idxh
;** Called by event 77 (sans_merging('SCALE'), comming from the suggestion GUI item or from Batch.
;-

Fscale=[1.,1.,1.]
Marges=[0.,0. ,0.,0. ,0.,0.]
Flm=0. & Fmh=0.

IF (idxl[0] ge 0) and (idxm[0] ge 0) then begin ;Find factor between Low & Middle  configuration
  Lista=Liste[idxl]       & Listb=Liste[idxm]
  Flm=sans_merge_find_scal(Lista, Listb, idxl, idxm, Clst)
  Fscale[0]=round(Flm[0]*1000. )/1000.          ;Scale is applied to the LOW configuration
  Marges[1]=round(Flm[1]*10000.)/10000.         ;Margin detector applied to LOW
  Marges[2]=round(Flm[2]*10000.)/10000.         ;Margin beam     applied to MIDDLE
ENDIF

IF (idxm[0] ge 0) and (idxh[0] ge 0) then begin ;Find factor between Middle & High configuration
  Lista=Liste[idxm]       & Listb=Liste[idxh]
  Fmh=sans_merge_find_scal(Lista, Listb, idxm, idxh, Clst)
  Fscale[2]=round(1./Fmh[0] *1000.)/1000.       ;Scale is applied to the HIGH configuration
  Marges[3]=round(   Fmh[1]*10000.)/10000.      ;Margin detector applied to MIDDLE
  Marges[4]=round(   Fmh[2]*10000.)/10000.      ;Margin beam     applied to HIGH
ENDIF

IF (idxl[0] ge 0) and (idxh[0] ge 0) and (Flm[0]+Fmh[0] eq 0.) then begin ;Find factor between Low & High configuration
  Lista=Liste[idxl]       & Listb=Liste[idxh]
  Flh=sans_merge_find_scal(Lista, Listb, idxl, idxh, Clst)
  Fscale[0]=round(Flh[0]*1000. )/1000.          ;Scale is applied to the LOW configuration
  Marges[1]=round(Flh[1]*10000.)/10000.         ;Margin detector applied to LOW
  Marges[4]=round(Flh[2]*10000.)/10000.         ;Margin beam     applied to HIGH
ENDIF

IF (idxl[0] ge 0) and (WaveSuplement.waveSL[0] ge 0) then begin ;Find factor between 4th & low configuration
  Lista=Liste[WaveSuplement.waveSL]      & Listb=Liste[idxl]
  F4l=sans_merge_find_scal(Lista, Listb, WaveSuplement.waveSL, idxl, Clst)
  WaveSuplement.marg_F4th[2] = round(F4l[0]*Fscale[0]*1000. )/1000.          ;Scale is applied to the 4th configuration
  WaveSuplement.marg_F4th[1] = round(F4l[1]          *10000.)/10000.         ;Margin detector applied to 4th
ENDIF

return,[Marges,Fscale]
end

pro sans_merge_in_files, Liste, Clst, idxl, idxm, idxh, oneMerge=oneMrg, background=bcbg, instrument=instru, WaveSuplement=WaveSuplement
;+
;** *******************
;** Merge sorted configurations (having Qranges and Qfactors)
;** Comming from event 76 (sans_merging('MERGE_file'): merge one sample for superplot or a fit
;** Comming from event 77 (sans_merging('MERGE_file'): merge all selected and write in text files
;** If oneMrg is present then return {WW:ww,datp:datp}
;-
forward_function sans_sigmaQ

if n_elements(instru) eq 1 then begin
   ids =strpos(instru,'<') & if ids[0] ge 0 then strput,instru,'_',ids[0]
   ids =strpos(instru,'>') & if ids[0] ge 0 then strput,instru,'_',ids[0]
endif else instru='I'

Listc=['300000'] ; We have not necessarily a third curve
Badpt=1          ; nb of bad points to extract when margins are not defined by the User

IF (idxl[0] ge 0) and (idxm[0] ge 0) then begin
  Lista=Liste[idxl]       & Listb=Liste[idxm]
  idxa =idxl              & idxb =idxm
  if idxh[0] ge 0 then begin
                            Listc=Liste[idxh]
                            idxc =idxh
                            endif
ENDIF ELSE IF (idxm[0] ge 0) and (idxh[0] ge 0) then begin
  Lista=Liste[idxm]       & Listb=Liste[idxh]
  idxa =idxm              & idxb =idxh

ENDIF ELSE IF (idxl[0] ge 0) and (idxh[0] ge 0) then begin
  Lista=Liste[idxl]       & Listb=Liste[idxh]
  idxa =idxl              & idxb =idxh

ENDIF ELSE return ;None in the Lists ...

;WE MAY HAVE A 4TH suplementary liste with a different wavelength.
 ;idx4 = WaveSuplement.waveSLall
 idx4 = WaveSuplement.waveSL
 if idx4[0] gt -1 then begin & List4 = Liste[idx4]
                               samp4 = lonarr(n_elements(idx4)) & kL=0L
                               for i=0,n_elements(idx4)-1 do begin
                                   reads,  List4[i], kL & samp4[i] = kL & endfor
endif

;DO THE WORK
  nla=n_elements(Lista)   & nlb=n_elements(Listb) & nlc=n_elements(Listc)
  nmx=max([nla,nlb,nlc])
  if nla le nmx then Lista=[lista,strarr(nmx-nla+1)+'100000']
  if nlb le nmx then Listb=[listb,strarr(nmx-nlb+1)+'200000']
  if nlc le nmx then Listc=[listc,strarr(nmx-nlc+1)+'300000']
  samplmin=0L
  sampla=0L & samplb=0L & samplc=0L
      ka=-1 &     kb=-1 &     kc=-1 & F4=4.
    fcta='' &   fctb='' &   fctc='' & fct4=''
  forca=1  ;variable=1 if we just read Lista
  forcb=1  ;variable=1 if we just read Listb
  forcc=1  ;variable=1 if we just read Listc
  
  WHILE samplmin lt 100000 do begin ; Go through the lists in parallel finding the same samples (same line)
      rpa=0 & rpb=0 & rpc=0
      if forca then begin & ka=ka+1 & reads,Lista[ka],sampla & forca=0 & endif
      if forcb then begin & kb=kb+1 & reads,Listb[kb],samplb & forcb=0 & endif
      if forcc then begin & kc=kc+1 & reads,Listc[kc],samplc & forcc=0 & endif
      samplmin=min([sampla,samplb,samplc])

      while (sampla lt samplmin) do begin & ka=ka+1 & reads,Lista[ka],sampla & endwhile
      while (samplb lt samplmin) do begin & kb=kb+1 & reads,Listb[kb],samplb & endwhile
      while (samplc lt samplmin) do begin & kc=kc+1 & reads,Listc[kc],samplc & endwhile
      samplmin=min([sampla,samplb,samplc])

      IF            (sampla eq samplb) and (samplmin eq sampla) then begin
         rpa=Clst & sans_reform_series,getlist=rpa,index=idxa[ka] ;Sample in first list
         rpb=Clst & sans_reform_series,getlist=rpb,index=idxb[kb] ;same sample in second list
         forca=1  & forcb=1
         IF samplc eq samplb then begin
         rpc=Clst & sans_reform_series,getlist=rpc,index=idxc[kc] ;same sample in third list
         forcc=1
         ENDIF

      ENDIF ELSE IF (samplb eq samplc) and (samplmin eq samplb) then begin
         rpa=Clst & sans_reform_series,getlist=rpa,index=idxb[kb] ;Sample in second list
         rpb=Clst & sans_reform_series,getlist=rpb,index=idxc[kc] ;same sample in third list
         forcb=1  & forcc=1

      ENDIF ELSE IF (sampla eq samplc) and (samplmin eq sampla) then begin
         rpa=Clst & sans_reform_series,getlist=rpa,index=idxa[ka] ;Sample in first list
         rpb=Clst & sans_reform_series,getlist=rpb,index=idxc[kc] ;same sample in third list
         forca=1  & forcc=1

      ENDIF ELSE IF samplmin lt 100000 then begin
         idm=where([sampla,samplb,samplc] eq samplmin)            ;One sample only
         idm=idm[0] & rpa=Clst
         if idm eq 0 then begin & sans_reform_series,getlist=rpa,index=idxa[ka] & forca=1 & endif
         if idm eq 1 then begin & sans_reform_series,getlist=rpa,index=idxb[kb] & forcb=1 & endif
         if idm eq 2 then begin & sans_reform_series,getlist=rpa,index=idxc[kc] & forcc=1 & endif
      ENDIF
      
; *** NOW WE MERGE ***

      IF samplmin lt 100000 then begin ; *** First  curve ***

         sans_merge_in_f, rpa, titW, fcta, fifia, Wa, Xa, Ea, dQa
         WW = Wa & EE = Ea
         XX = Xa & dQ = dQa
         titsamp = titW
         fifi    = fifia

        ;********************* 
         IF n_tags(rpb) gt 0 then begin ; *** Second curve ***
            sans_merge_in_f, rpb, titW, fctb, fifib, Wb,Xb,Eb,dQb            
            Qmax=XX[n_elements(XX)-1]     ; Find portion of curves in same Qrange
            idQb=where(Xb gt Qmax) & idQb=idQb[0]
            if idQb ge 0 then begin       ; PREFER THE BEST RESOLUTION
              WW=[WW, Wb[idQb:*]] & EE=[EE, Eb[ idQb:*]]
              XX=[XX, Xb[idQb:*]] & dQ=[dQ, dQb[idQb:*]]
              titsamp = titsamp+' + '+titW
              fifi    = fifi   + '_' +fifib
            endif
         ENDIF
         
        ;*********************          
         IF n_tags(rpc) gt 0 then begin ; *** Third curve ***
            sans_merge_in_f, rpc, titW, fctc, fific, Wc,Xc,Ec,dQc
            Qmax=XX[n_elements(XX)-1]     ; Find portion of curves in same Qrange
            idQc=where(Xc gt Qmax) & idQc=idQc[0]
            if idQc ge 0 then begin       ; PREFER THE BEST RESOLUTION
              WW=[WW, Wc[idQc:*]] & EE=[EE, Ec[ idQc:*]]
              XX=[XX, Xc[idQc:*]] & dQ=[dQ, dQc[idQc:*]]
              titsamp = titsamp+' + '+titW
              fifi    = fifi   + '_' +fific
            endif
         ENDIF

        ;*********************
        ;SUPLEMENTARY 4TH CURVE (wavelength should be different) IDX4 AND IDXL > 0
         IF (idx4[0] gt -1) and (idxl[0] gt -1) then begin
          idx=where(samp4 eq samplmin)   ; SAME SAMPLE = SAME TABLE LINE
          IF idx[0] ge 0 then begin
           rp4=Clst & sans_reform_series,getlist=rp4,index=idx4[idx]
           lambda123 = round(rpa.P[0]*10.)/10. ; Main wavelength
           lambda4   = round(rp4.P[0]*10.)/10. ; 4th  wavelength

           IF (lambda4 ge lambda123) then begin ;AT THIS TIME PROCESS ONLY IF LAMBDA4 IS GREATER            
            sans_merge_in_f, rp4, titW, fct4, fifi4, W4,X4,E4,dQ4
            Qmax=X4[n_elements(X4)-1]     ; Find portion of curves in same Qrange
            idQ4=where(XX gt Qmax) & idQ4=idQ4[0]
            if idQ4 ge 0 then begin       ; PREFER THE BEST RESOLUTION 4th CURVE
              WW=[W4, WW[idQ4:*]] & EE=[E4, EE[idQ4:*]]
              XX=[X4, XX[idQ4:*]] & dQ=[dQ4,dQ[idQ4:*]]
              Fct4    = Fct4 + ' '
              titsamp = titW +' + '+titsamp
              fifi    = fifi4+ '_' +fifi
            endif
         ENDIF & ENDIF & ENDIF
        
        ;*********************
        ;WRITE THE FILE
         if n_elements(bcbg) eq 1 then begin
            WW=WW-bcbg
           ;if bcbg gt 1. then EE=EE-sqrt(bcbg)
         endif
         Wt    =titsamp+' ['+instru+' '+fifi+']*['+Fct4+fcta+' '+fctb+' '+fctc+']'

         filcol=instru+'_merge_'+fifi+'.txt'
         datp  ={W_tit:Wt, X_tit:'Q', Y_tit:'I(Q)', Z_tit:'Delta Q', E:EE, X:XX, Y:samplmin,Z:float(dQ), P:0, PV:0}
         give_W,WW, W=22 & give_datp,datp,W=22
         if not arg_present(oneMrg) then write_lamp,filcol,W=22,format='column'
         
      ENDIF ;samplmin<100000
  ENDWHILE
  if arg_present(oneMrg) then oneMrg={WW:ww,datp:datp}
end

pro sans_merge_in_f, rpb, titW, fctb, fifib, Wb,Xb,Eb,dQb
;+
;** ********************
;** Works for sans_merge_in_file
;**    Input:  rpb
;**    Output: titW, fctb, fileb, Wb,Xb,Eb,dQb
;-
  Badpt= 1                  ; nb of bad points to extract when margins are not defined by the User
  F4   = 4.                 ; Used to discard low resolution points
  Wb   = rpb.W * rpb.NTC[4] ; Factor defined by the User
  Xb   = rpb.X & Eb=rpb.E * rpb.NTC[4]
  fctb = string(rpb.NTC[4],format='(F4.2)')

  DL_L=rpb.P[12] & Lambda=rpb.P[0] & dth=rpb.dth[0] ; for sigmaQ
  
  m=0 & n=n_elements(Xb)-1

  if rpb.NTC[5] ne rpb.NTC[6] then begin ;margins LIMIT INDEX defined by the User
    if rpb.NTC[5] gt 0. then m=(where(Xb ge rpb.NTC[5]))[0] > 0
    if rpb.NTC[6] gt 0. then n=(where(Xb ge rpb.NTC[6]))[0] > 0
    if n le m then n=n_elements(Xb)-1
    Wb=Wb[m:n] & Eb=Eb[m:n] & Xb=Xb[m:n]
  endif else begin
    ddx=where((Wb gt 1E-7) and (Wb>1. gt F4*Eb))
    if (ddx[0] ge 0) and (n_elements(ddx) ne n_elements(Xb)) then begin ;Discard low resolution points LRP
      Xb=Xb[ddx]  &  Wb=Wb[ddx]  &  Eb=Eb[ddx]  & endif
      if Badpt gt 0 then begin
        m=Badpt & n=n_elements(Xb)-1-Badpt
        Wb=Wb[m:n] & Eb=Eb[m:n] & Xb=Xb[m:n]
      endif
  endelse

  dQb= sans_sigmaQ(X=Xb , dL_L=dL_L , Lambda=Lambda , dTh=dTh)

  px=max([strpos(rpb.TIT,'mM '),strpos(rpb.TIT,'pL '),strpos(rpb.TIT,'gH ') ,$
          strpos(rpb.TIT,'mM.'),strpos(rpb.TIT,'pL.'),strpos(rpb.TIT,'gH.')])
  titW   =strmid(rpb.TIT,px+3,strpos(rpb.TIT,' _')-px-3)
  fifib  =strtrim(long(rpb.NTC[0]),2)

  ps=strpos(rpb.TIT,'.Sect') ;Complete the filename if necessary
  if ps gt px then fifib = fifib+'s'+strmid(rpb.TIT,ps+5,3)
  pb= strpos(rpb.TIT,' (-')
  ps= strpos(rpb.TIT,' [' )
  if ps gt px then begin
     bc=strpos(rpb.TIT,']',ps)
     if strpos(rpb.TIT,' [-') gt px then fifib = fifib+'minus'+strmid(rpb.TIT,ps+3,bc-ps-3) else $
     if strpos(rpb.TIT,' [+') gt px then fifib = fifib+'plus' +strmid(rpb.TIT,ps+3,bc-ps-3) else $
     if strpos(rpb.TIT,' [/') gt px then fifib = fifib+'over' +strmid(rpb.TIT,ps+3,bc-ps-3) else $
     if strpos(rpb.TIT,' [*') gt px then fifib = fifib+'by'   +strmid(rpb.TIT,ps+3,bc-ps-3)
  endif else $
     if pb gt px then fifib = fifib+'bg'+strmid(rpb.TIT,pb+3,strpos(rpb.TIT,')',pb)-pb-3)

end


pro sans_merging, flag, typ, idxll,idxmm,idxhh, gtlist=gtlist, suggestion=marg_Flmh, oneMerge=oneMrg $
                            ,background=bcbg, instrument=instru, WaveSuplement=WaveSuplement
;** ************
;+
;** Works for Batch or GUI mode. Get/Set gtlist, Set suggestion scaling, merge in files.
;** typ is integration mode to search correct sans_reform_series memory.
;** idxll , idxmm , idxhh return/get all index of LMH integrations (sectors include).
;** suggestion keyword is dedicated for flag = 'SCALE','SCALE_set'
;** oneMerge, background, instrument keywords are dedicated for flag='MERGE_file'
;**                            If oneMrg is present then return {WW:ww,datp:datp}
;-

; Set gtlist and Set idxll,idxmm,idxhh
; ************************************
if flag eq 'LMH.AllIndex' then begin
   gtlist=typ
   sans_reform_series,getlist =gtlist, /rawConf
   idxll=where( (strpos(gtlist,'pL ') gt 4) or (strpos(gtlist,'pL.Sect') gt 4) )
   idxmm=where( (strpos(gtlist,'mM ') gt 4) or (strpos(gtlist,'mM.Sect') gt 4) )
   idxhh=where( (strpos(gtlist,'gH ') gt 4) or (strpos(gtlist,'gH.Sect') gt 4) )

   sans_settings_GetSupl, supl                                                        ; Actual waveLength
   waveP = ' _WavL'+strtrim(string(round(supl[0]*10.)/10. ,format='(F4.1)'),2)
   waveSL = -1  &  waveSM = -1  &  waveSH = -1 & waveP = waveP & marg_F4th=[0.,0.,0.] ; Care for Wave Suplement in Low  [M,H]

   if (idxll[0] ge 0) and ( (idxmm[0] ge 0) or (idxhh[0] ge 0) ) then begin
     if   idxmm[0] ge 0 then curW = idxmm[0] else $ ; check for principal wavelength
       if idxhh[0] ge 0 then curW = idxhh[0] else $ ; ""
       if idxll[0] ge 0 then curW = idxll[0]        ; ""
     idxW1 = strpos(gtlist[curW],' _WavL') & idxW2 = strpos(gtlist[curW],'_',idxW1+2)
     waveP = strmid(gtlist[curW],idxW1,idxW2-idxW1)
     
     tmpConff = gtlist[idxll]
     idxwaveP = where(strpos(tmpConff , waveP)  gt 0 , complement=WS) ;WS 4th config
     if (WS[0] gt -1) and (idxwaveP[0] gt -1) then waveSL = idxll[WS] ;We have a WaveSuplement !!!
   endif
   WaveSuplement = {waveSL:waveSL ,  waveSM:waveSM , waveSH:waveSH , waveP:waveP    , waveSLall:waveSL , marg_F4th:marg_F4th}
    ; WaveSuplement selectable Low,  [M H]                         , main wavelength, total 4th config   marge and scale
endif

; Get gtlist and Set sorted idxll,idxmm,idxhh
; *******************************************
if flag eq 'SORT' then begin                ;SORT BY LINE NUMBER (same line->same sample)
                                            ;"001.08??.Sect180.0Sym"

   waveP     = WaveSuplement.waveP
   waveSL    = -1 & waveSM = -1 & waveSH = -1
   waveSLall = WaveSuplement.waveSLall & marg_F4th= WaveSuplement.marg_F4th

   if idxll[0] ge 0 then begin
      tmpConff= gtlist[idxll] & tmpConf=strmid(tmpConff,0,6)+strmid(tmpConff,9,9)
      idxsect = sort(tmpConf) & idxll  =idxll[idxsect] & tmpConff = tmpConff[idxsect]
      
      idxwaveP= where(strpos(tmpConff , waveP) gt 0 , complement=WS)
      if (WS[0] gt -1) and (idxwaveP[0] gt -1) then begin ; We have a WaveSuplement !!!
          waveSL = idxll[WS]                              ; 4th sorted, selected
          idxll  = idxll[idxwaveP]                        ; Low sorted, selected
      endif ; ELSE WE HAVE ONLY ONE WAVEL IN THAT LOW CONFIG.
   endif

   if idxmm[0] ge 0 then begin
      tmpConff= gtlist[idxmm] & tmpConf=strmid(tmpConff,0,6)+strmid(tmpConff,9,9)
      idxsect = sort(tmpConf) & idxmm  =idxmm[idxsect];& tmpConff = tmpConff[idxsect]
   endif

   if idxhh[0] ge 0 then begin
      tmpConff= gtlist[idxhh] & tmpConf=strmid(tmpConff,0,6)+strmid(tmpConff,9,9)
      idxsect = sort(tmpConf) & idxhh  =idxhh[idxsect];& tmpConff = tmpConff[idxsect]
   endif   
   WaveSuplement = {waveSL:waveSL ,  waveSM:waveSM , waveSH:waveSH , waveP:waveP    , waveSLall:waveSLall , marg_F4th:marg_F4th}
    ; WaveSuplement selectable Low,  [M H]                         , main wavelength, total 4th config      marge and scale
endif

; Get Qrange and factors suggestion
; *********************************
if flag eq 'SCALE' then begin
   marg_Flmh=sans_merge_find_scaling(gtlist, typ, idxll, idxmm, idxhh, WaveSuplement=WaveSuplement)
endif

; Set Qrange and factors suggestion in memory result
; **************************************************
if flag eq 'SCALE_set' then begin
   if idxll[0] ge 0 then sans_reform_series,getlist=typ, index=idxll[0], setfct=marg_Flmh[6], setmarg=marg_Flmh[0:1]
   if idxmm[0] ge 0 then sans_reform_series,getlist=typ, index=idxmm[0], setfct=marg_Flmh[7], setmarg=marg_Flmh[2:3]
   if idxhh[0] ge 0 then sans_reform_series,getlist=typ, index=idxhh[0], setfct=marg_Flmh[8], setmarg=marg_Flmh[4:5]
   if WaveSuplement.waveSLall[0] ge 0 then $
      sans_reform_series,getlist=typ, index =WaveSuplement.waveSLall,  /setindex, $
                                      setfct=WaveSuplement.marg_F4th[2], setmarg=WaveSuplement.marg_F4th[0:1]
endif

if flag eq 'MERGE_file' then begin
   if arg_present(oneMrg) then sans_merge_in_files, gtlist, typ, idxll, idxmm, idxhh ,background=bcbg, instrument=instru, oneMerge=oneMrg $
                          else sans_merge_in_files, gtlist, typ, idxll, idxmm, idxhh ,background=bcbg, instrument=instru, WaveSuplement=WaveSuplement
endif

end

pro sans_merge, reset=reset, background=bcbg, instrument=instru
;+
;** **********
;** Sans_merging entry for batch mode
;-

typ = 2 ; Radial integration

if keyword_set(reset) then begin
   sans_reform_series, clear=typ ;or -1
   return
endif

; Get the full result list(config texts) and indexes
  sans_merging, 'LMH.AllIndex', typ, idxll,idxmm,idxhh, gtlist=gtlist, WaveSuplement=WaveSuplement
  
; Sort the result list
  sans_merging, 'SORT',         typ, idxll,idxmm,idxhh, gtlist=gtlist, WaveSuplement=WaveSuplement

; Get a suggestion for scaling and marging between configs
  sans_merging, 'SCALE',        typ, idxll,idxmm,idxhh, gtlist=gtlist, suggestion=marg_Flmh, WaveSuplement=WaveSuplement

; Set this suggestion in result list memory
  sans_merging, 'SCALE_set',    typ, idxll,idxmm,idxhh, gtlist=gtlist, suggestion=marg_Flmh, WaveSuplement=WaveSuplement

; Do the merge and write in files
  sans_merging, 'MERGE_file',   typ, idxll,idxmm,idxhh, gtlist=gtlist, background=bcbg, instrument=instru, WaveSuplement=WaveSuplement
end

pro sans_merge_results,flg, fufu, newISO=newISO
;+
;** ******************
;**
;** Merge result List (do not confuse with merging configs)
;-

common resume_iso, iso_conf, iso_npt, iso_WQE, iso_NTC,    iso_p, iso_supl
common resume_azi, azi_conf, azi_npt, azi_WQE, azi_NTC,    azi_p, azi_supl

common tempor_iso, tmi_conf, tmi_npt, tmi_WQE, tmi_NTC,    tmi_p, tmi_supl
common tempor_azi, tma_conf, tma_npt, tma_WQE, tma_NTC,    tma_p, tma_supl

common keep_fifi, fifi

if flg eq 'fput' then fifi=fufu
if flg eq 'fget' then fufu=fifi

if flg eq 'newi' then begin
   typ=newISO.supl[21] & npt=n_elements(newISO.W)
   if (typ eq 4) or (typ eq 5) or(typ eq 6) then begin ;AZIMUTAL or TOTAL RADIAL
       tma_conf=newISO.TIT & tma_npt=npt & tma_WQE=[newISO.W,newISO.X,newISO.E] & tma_NTC=newISO.NTC & tma_P=newISO.P & tma_supl=newISO.supl
   endif else begin                                    ;RADIAL
       tmi_conf=newISO.TIT & tmi_npt=npt & tmi_WQE=[newISO.W,newISO.X,newISO.E] & tmi_NTC=newISO.NTC & tmi_P=newISO.P & tmi_supl=newISO.supl
   endelse
endif

if flg eq 'temp' then begin & npi=n_elements(iso_NTC) & npa=n_elements(azi_NTC) ;keep current before restore
   if npi gt 1 then begin & tmi_conf=iso_conf & tmi_npt=iso_npt & tmi_WQE=iso_WQE & tmi_NTC=iso_NTC & tmi_p=iso_p & tmi_supl=iso_supl & endif
   if npa gt 1 then begin & tma_conf=azi_conf & tma_npt=azi_npt & tma_WQE=azi_WQE & tma_NTC=azi_NTC & tma_p=azi_p & tma_supl=azi_supl & endif
endif

if (flg eq 'mrge') or (flg eq 'newi') then begin & npi=n_elements(iso_NTC) & npa=n_elements(azi_NTC) ;restored file
                                                 nti=n_elements(tmi_NTC) & nta=n_elements(tma_NTC) ;current result
     if (npi le 1) and (nti gt 1) then begin
        iso_conf=tmi_conf & iso_npt=tmi_npt & iso_WQE=tmi_WQE & iso_NTC=tmi_NTC & iso_p=tmi_p & iso_supl=tmi_supl & endif
     if (npa le 1) and (nta gt 1) then begin
        azi_conf=tma_conf & azi_npt=tma_npt & azi_WQE=tma_WQE & azi_NTC=tma_NTC & azi_p=tma_p & azi_supl=tma_supl & endif

     if (npi gt 1) and (nti gt 1) then begin ;merge radial
       ;RESIZE DATA IF NECESSARY
       ; -----------------------
        difnp=(size(tmi_WQE))[1] - (size(iso_WQE))[1]
        if difnp gt 0 then iso_WQE=[iso_WQE,dblarr( difnp,n_elements(iso_npt))]
        if difnp lt 0 then tmi_WQE=[tmi_WQE,dblarr(-difnp,n_elements(tmi_npt))]
       ;KEEP UNIQ CONFIGS
       ; ----------------
        for i=0L,n_elements(tmi_conf)-1 do begin
            idm=where(iso_conf eq tmi_conf[i]) & j=idm[0]
            if j ge 0 then begin
              iso_npt[j]=tmi_npt[i] & iso_WQE[*,j]=tmi_WQE[*,i] & iso_NTC[*,j]=tmi_NTC[*,i] & iso_p[*,j]=tmi_P[*,i]
              iso_supl[*,j]=tmi_Supl[*,i]
            endif else begin
              iso_conf=[iso_conf,tmi_conf[i]]      & iso_npt=[iso_npt,tmi_npt[i]]   & iso_WQE =[[iso_WQE] ,[tmi_WQE[*,i]]]
              iso_NTC =[[iso_NTC] ,[tmi_NTC[*,i]]] & iso_p  =[[iso_p],[tmi_P[*,i]]] & iso_supl=[[iso_supl],[tmi_Supl[*,i]]]
            endelse
        endfor
     endif
     if (npa gt 1) and (nta gt 1) then begin ;merge azimutal
       ;RESIZE DATA IF NECESSARY
       ; -----------------------
        difnp=(size(tma_WQE))[1] - (size(azi_WQE))[1]
        if difnp gt 0 then azi_WQE=[azi_WQE,dblarr( difnp,n_elements(azi_npt))]
        if difnp lt 0 then tma_WQE=[tma_WQE,dblarr(-difnp,n_elements(tma_npt))]
       ;KEEP UNIQ CONFIGS
       ; ----------------
        for i=0L,n_elements(tma_conf)-1 do begin
            idm=where(azi_conf eq tma_conf[i]) & j=idm[0]
            if j ge 0 then begin
              azi_npt[j]=tma_npt[i] & azi_WQE[*,j]=tma_WQE[*,i] & azi_NTC[*,j]=tma_NTC[*,i] & azi_p[*,j]=tma_P[*,i]
              azi_supl[*,j]=tma_Supl[*,i]
            endif else begin
              azi_conf=[azi_conf,tma_conf[i]]      & azi_npt=[azi_npt,tma_npt[i]]   & azi_WQE =[[azi_WQE] ,[tma_WQE[*,i]]]
              azi_NTC =[[azi_NTC] ,[tma_NTC[*,i]]] & azi_p  =[[azi_p],[tma_P[*,i]]] & azi_supl=[[iso_supl],[tma_Supl[*,i]]]
            endelse
        endfor
     endif
endif

if flg eq 'zero' then begin & npi=n_elements(iso_NTC) & npa=n_elements(azi_NTC) ;restored file
                            nti=n_elements(tmi_NTC) & nta=n_elements(tma_NTC) ;current result
   if (nti gt 1) then begin
        iso_conf=tmi_conf & iso_npt=tmi_npt & iso_WQE=tmi_WQE & iso_NTC=tmi_NTC & iso_p=tmi_p & iso_supl=tmi_supl & endif
   if (nta gt 1) then begin
        azi_conf=tma_conf & azi_npt=tma_npt & azi_WQE=tma_WQE & azi_NTC=tma_NTC & azi_p=tma_p & azi_supl=tma_supl & endif
endif

if (flg eq 'zoro') or (flg eq 'newi') then begin
   tmi_conf='' & tmi_npt=0 & tmi_WQE=0 & tmi_NTC=0 & tmi_p=0 & tmi_supl=0
   tma_conf='' & tma_npt=0 & tma_WQE=0 & tma_NTC=0 & tma_p=0 & tma_supl=0
endif

end

pro sans_sav_rest_results,flg, path=gpth, file=filhere
;+
;** *********************
;** save/restore and merge in/from binary idl file_result.sav
;-

forward_function   finefile
common resume_iso, iso_conf, iso_npt, iso_WQE, iso_NTC,    iso_p, iso_supl
common resume_azi, azi_conf, azi_npt, azi_WQE, azi_NTC,    azi_p, azi_supl

if n_elements(gpth)    ne 1 then gpth=''
if n_elements(filhere) ne 1 then fifi='sans_result_list.sav' else fifi=gpth+filhere
defext='sav' & filt='*.sav'
if (flg eq 'rest') or (flg eq 'mrge') then if n_elements(filhere) ne 1 then $
   fifi=dialog_pickfile(get_path=gpth,file=fifi,default_ext=defext,/read ,resource_name='lamptouch',filter=filt)
if (flg eq 'save')                    then if n_elements(filhere) ne 1 then $
   fifi=dialog_pickfile(get_path=gpth,file=fifi,default_ext=defext,/write,resource_name='lamptouch',filter=filt,/overwrite_prompt)

if fifi ne '' then begin
   fifo  =strmid(fifi,strlen(gpth))
   npi   =n_elements(iso_NTC)  &  npa=n_elements(azi_NTC)

   if flg eq 'save' then if npi+npa gt 2 then save,/variables,filename=fifi,/compress
   if flg eq 'rest' then begin
                    sans_merge_results,'temp'                 ;store   current commons
                    restore,filename=fifi                     ;read    fifi
                    sans_merge_results,'mrge'                 ;merge   commons
                    sans_merge_results,'zoro'                 ;clear   stored commons
                    endif
   if flg eq 'mrge' then if npi+npa gt 2 then begin
                    sans_merge_results,'temp'                 ;store   current commons
                    sans_merge_results,'fput',fifi            ;care renamed fifi
                    restore,filename=fifi                     ;read    fifi
                    sans_merge_results,'fget',fifi            ;care renamed fifi
                    sans_merge_results,'mrge'                 ;merge   commons
                    save,/variables,filename=fifi,/compress   ;save in fifi
                    sans_merge_results,'zero'                 ;restore common
                    sans_merge_results,'zoro'                 ;clear   stored commons
                 endif
endif
end

pro sans_reform_series, Win, dp=dp, titles=parx, parconf=part, resume=typ, getlist=gtlst, rawConf=rawConf   ,$
                          clear=clear, index=index, save=sav, restore=rest, merge=mrge            ,$
                          getfct=getfct, setfct=setfct, getmarg=getmarg, setmarg=setmarg, setindex=setindex, idxZ=pZ ,$
                          savtrs=savtrs, nothick=nothick, gettrs=gettrs, add=add, curLMH=curLMH ,INSTRU=inst,$
                          transmission=transmission,thickness=thickness,allsupl=allsupl, sort=sortL, S_IDs=Sample_IDs
;+
;** ******************
;** using specific memory:
;** 1)If keyword resume(typ) is not present then add numor series having same title_formula and same config
;**   Output: Win with parameters:
;**           dp  (updated PV,E,N,Z)
;**           parx(title_formula)
;**           part(title_formula+config)
;**
;** 2)If keyword resume is present then keep results (replace numors having same title_formula and same config)
;-

forward_function   finefile
common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

common resume_iso, iso_conf, iso_npt, iso_WQE, iso_NTC,    iso_p, iso_supl
common resume_azi, azi_conf, azi_npt, azi_WQE, azi_NTC,    azi_p, azi_supl
common resume_trs, trs_tits

fifi="sans_transmissions.txt" & maxN=2000 & if n_elements(trs_tits) eq 0 then trs_tits=['-']
;****************************************


;SAVING TRANSMISSIONS THICKNESS WAVE TITS in sans_transmissions.txt
; -----------------------------------------------------------------
if n_elements(savtrs) ge 4 then begin
   if n_elements(sans_trans) lt 4 then return
   trans =sans_trans[*,0] & ntr=n_elements(trans)
   if n_elements(sans_trans) gt 5 then thick=sans_trans[*,1] else thick=trans*0. & thick[[1,3]]=0. ;Can
   if keyword_set(nothick) then thick[*]=0.
   if ntr gt 4 then begin & trans=[trans[0:3],trans[4+pZ]] & thick=[thick[0:3],thick[4+pZ]] & ntr=n_elements(trans)
                          if pZ[0] ne 0 then trans[2]=0. & endif
   wavlen=sans_supl[0] ;  Assume we are in correct config if multiple waveL!!!
   tits  =strcompress(strupcase(savtrs),/remove_all) & idx=where(tits ne '')
   if idx[0] ge 0 then begin & tits=tits[idx] & trans=trans[idx<(ntr-1)] & thick=thick[idx<(ntr-1)]
   endif          else return
   ntr=n_elements(trans)  & ntt=n_elements(tits) & ntm=min([ntr,ntt])
   if ntm lt ntt then tits =tits[ 0:ntm-1]
   if ntm gt ntt then trans=trans[0:ntm-1]
   if ntm gt ntt then thick=thick[0:ntm-1]

   tits  ='Wavl:'+strtrim(string(round(wavlen  *10)  /10  ,format='(F4.1)'),2)+'  '+tits
   for i=0L,ntm-1 do begin
     tits[i]='Es:'  +strtrim(string(round(thick[i]*100.) /100. ,format='(F5.3)'),2)+'  '+tits[i]
     tits[i]='TR:'  +strtrim(string(round(trans[i]*1000.)/1000.,format='(F5.3)'),2)+'  '+tits[i]
   endfor

   if trs_tits[0] eq '-' then begin
      bid=finefile(fifi,count=cnt)
      if cnt gt 0 then begin & on_ioerror,mistrs1 & u=-1
         openr,u,fifi,/get_lun & readf,u,cnt & trs_tits=strarr(cnt) & readf,u,trs_tits & mistrs1:
         if u gt 0 then free_lun,u
   endif & endif
   if trs_tits[0] eq '-' then trs_tits=tits else begin
     ;KEEP UNIQ TRS_TITS
     ; -----------------
     tmpk=strmid(trs_tits,10,200) & tmpc=strmid(tits,10,200)
     for i=0L,ntm-1 do begin
         idm=where(tmpk eq tmpc[i]) & j=idm[0]
         if j ge 0 then trs_tits[j]=tits[i] $
                   else trs_tits=[trs_tits,tits[i]]
     endfor
   endelse
   cnt=n_elements(trs_tits)<maxN
   on_ioerror,mistrs2 & u=-1
   openw,u,fifi,/get_lun & printf,u,cnt & for i=0L,cnt-1 do printf,u,trs_tits[i] & mistrs2:
   if u gt 0 then free_lun,u
   return
endif

;GETTING TRANSMISSIONS from sans_transmissions.txt
; ------------------------------------------------
if n_elements(gettrs) ge 4 then begin
  ;if trs_tits(0) eq '-' then begin
      gpth='' & filter=fifi
      result=dialog_pickfile(get_path=gpth,filter=filter,file=fifi,/read,resource_name='lamptouch')
      if result gt '' then begin & on_ioerror,mistrs3 & u=-1
         openr,u,result,/get_lun & readf,u,cnt & trs_tits=strarr(cnt) & readf,u,trs_tits & mistrs3:
         if u gt 0 then free_lun,u
      endif
  ;endif
   if n_elements(sans_trans) gt 5 then thick=sans_trans[*,1] else thick=sans_trans*0. & thick[[1,3]]=0. ;Can
   wavlen=sans_supl[0] ;  Assume we are in correct config if multiple waveL!!!
   tits  =strcompress(strupcase(gettrs),/remove_all) & idx=where(tits ne '') & ntt=n_elements(tits)
   gettrs='No transmission updated !!!'
   if trs_tits[0] eq '-' or (n_elements(sans_trans[*,0]) lt ntt) or (idx[0] lt 0) then return

   tmptrs =strmid(trs_tits,10,200)
   nbfound=0
   for i=0L,ntt-1 do begin
     if tits[i] ne '' then begin
       tit='Wavl:'+strtrim(string(round(wavlen  *10)  /10    ,format='(F4.1)'),2)+'  '+tits[i]
       ti0='Es:'  +strtrim(string(round(0.      *100.) /100. ,format='(F5.3)'),2)+'  '+tit
       tit='Es:'  +strtrim(string(round(thick[i]*100.) /100. ,format='(F5.3)'),2)+'  '+tit
       idx=where(tmptrs eq tit)
       if idx[0] ge 0 then begin
          tmp=float(strmid(trs_tits[idx[0]],3,5)) & nbfound=nbfound+1
          if (i eq 2) and (pZ[0] eq 0) then sans_trans[i,0]=tmp else $
          if  i lt 4                   then sans_trans[i,0]=tmp else sans_trans[4+pZ[i-4]]=tmp
       endif else begin
          idx=where(tmptrs eq ti0) ;gives a second chance
          if idx[0] ge 0 then begin
          tmp=float(strmid(trs_tits[idx[0]],3,5)) & nbfound=nbfound+1
          if (i eq 2) and (pZ[0] eq 0) then sans_trans[i,0]=tmp else $
          if  i lt 4                   then sans_trans[i,0]=tmp else sans_trans[4+pZ[i-4]]=tmp & endif
       endelse
     endif
    endfor
    if nbfound gt 0 then gettrs=string(nbfound)+ ' transmission[s] updated'
   return
endif

;"RESUMING" OPERATIONS save , restore , give back a list entry , give back all list , clear , scale , margin
; --------------------
if keyword_set(sav)  then begin & sans_sav_rest_results,'save' & return & endif
if keyword_set(mrge) then begin & sans_sav_rest_results,'mrge' & return & endif
if keyword_set(rest) then begin & sans_sav_rest_results,'rest' & return & endif

if n_elements(gtlst) eq 1 then if (gtlst eq 5) or (gtlst eq 6) then gtlist=4 else $            ; TOTAL  SECTOR = AZIMUTAL
                               if (gtlst eq 7) or (gtlst eq 8) then gtlist=2 else gtlist=gtlst ; RADIAL SECTOR = RADIAL
if n_elements(clear) eq 1 then if (clear eq 5) or (clear eq 6) then clear =4 else $            ; TOTAL  SECTOR = AZIMUTAL
                               if (clear eq 7) or (clear eq 8) then clear =2 else clear =clear ; RADIAL SECTOR = RADIAL
if n_elements(sortL) ne 1 then sortL=-1

if (arg_present(getfct) or arg_present(getmarg)) then begin
 if arg_present(getfct)       then begin & getfct=1.
   if (gtlist eq 2) and (n_elements(iso_npt) gt index) then getfct=iso_NTC[4,index]
   if (gtlist eq 4) and (n_elements(azi_npt) gt index) then getfct=azi_NTC[4,index]
 endif
 if arg_present(getmarg)      then begin & getmarg=[0.,0.]
   if (gtlist eq 2) and (n_elements(iso_npt) gt index) then getmarg=(iso_NTC[5:6,index])
   if (gtlist eq 4) and (n_elements(azi_npt) gt index) then getmarg=(azi_NTC[5:6,index])
 endif
 return
endif

if (n_elements (setfct)  eq 1) or (n_elements (setmarg) eq 2) then begin
 if n_elements( setfct)  eq 1 then begin                                                       ;SCALING
   if (gtlist eq 2) and (n_elements(iso_npt) gt index[n_elements(index)-1]) then begin
                    if keyword_set(setindex) then begin
                               iso_NTC[4,index]=setfct                                         ;Update using index
                    endif else begin
                               tmp=strmid(iso_CONF[index[0]],4,4)                              ;Update same config as index[0]
                               idx=strpos(iso_CONF,tmp) & idx=where(idx eq 4)
                               iso_NTC[4,idx]=setfct
                    endelse  & endif
   if (gtlist eq 4) and (n_elements(azi_npt) gt index[n_elements(index)-1]) then begin
                    if keyword_set(setindex) then begin
                               azi_NTC[4,index]=setfct                                         ;Update using index
                    endif else begin
                               tmp=strmid(azi_CONF[index[0]],4,4)                              ;Update same config as index[0]
                               idx=strpos(azi_CONF,tmp) & idx=where(idx eq 4)
                               azi_NTC[4,idx]=setfct
                    endelse  & endif
 endif
 if n_elements( setmarg) eq 2 then begin                                                       ;MARGING
   if (gtlist eq 2) and (n_elements(iso_npt) gt index[n_elements(index)-1]) then begin
                    if keyword_set(setindex) then begin
                               for ii=0L,n_elements(index)-1 do iso_NTC[5:6,index[ii]]=setmarg ;Update using index
                    endif else begin
                               tmp=strmid(iso_CONF[index[0]],4,4)                              ;Update same config as index[0]
                               idx=strpos(iso_CONF,tmp) & idx=where(idx eq 4)
                               for ii=0L,n_elements(idx)-1 do iso_NTC[5:6,idx[ii]]=setmarg
                    endelse  & endif
   if (gtlist eq 4) and (n_elements(azi_npt) gt index[n_elements(index)-1]) then begin
                    if keyword_set(setindex) then begin
                               for ii=0L,n_elements(index)-1 do azi_NTC[5:6,index[ii]]=setmarg ;Update using index
                    endif else begin
                               tmp=strmid(azi_CONF[index[0]],4,4)                              ;Update same config as index[0]
                               idx=strpos(azi_CONF,tmp) & idx=where(idx eq 4)
                               for ii=0L,n_elements(idx)-1 do azi_NTC[5:6,idx[ii]]=setmarg
                    endelse  & endif
 endif
 return

endif

if (n_elements(index) eq 1) and (n_elements(gtlist) eq 1) then begin ;return content List(index)
   if (gtlist eq 2) and (n_elements(iso_npt) gt index) then begin & i=index & np=iso_npt[i]
      if np gt 1 then begin
       MTx=strpos(iso_conf[i],' #')
       if MTx gt 0 then MTmess=strmid(iso_conf[i],MTx+1) else MTmess=''
       if keyword_set(allsupl) then $
       gtlst={TIT:iso_conf[i],NTC:iso_NTC[*,i],W:iso_WQE[0:np-1,i],X:iso_WQE[np:np*2-1,i],E:iso_WQE[np*2:np*3-1,i],SUPL:iso_supl[*,i],P:iso_p[*,i],TIME:MTmess} else $
       gtlst={TIT:iso_conf[i],NTC:iso_NTC[*,i],W:iso_WQE[0:np-1,i],X:iso_WQE[np:np*2-1,i],E:iso_WQE[np*2:np*3-1,i],Es:iso_supl[11,i] ,P:iso_p[*,i],TIME:MTmess,dTh:iso_supl[49,i]}
      endif
   endif else $
   if (gtlist eq 4) and (n_elements(azi_npt) gt index) then begin & i=index & np=azi_npt[i]
      if np gt 1 then begin
       MTx=strpos(azi_conf[i],' #')
       if MTx gt 0 then MTmess=strmid(azi_conf[i],MTx+1) else MTmess=''
       if keyword_set(allsupl) then $
       gtlst={TIT:azi_conf[i],NTC:azi_NTC[*,i],W:azi_WQE[0:np-1,i],X:azi_WQE[np:np*2-1,i],E:azi_WQE[np*2:np*3-1,i],SUPL:azi_supl[*,i],P:azi_p[*,i],TIME:MTmess} else $
       gtlst={TIT:azi_conf[i],NTC:azi_NTC[*,i],W:azi_WQE[0:np-1,i],X:azi_WQE[np:np*2-1,i],E:azi_WQE[np*2:np*3-1,i],Es:azi_supl[11,i] ,P:azi_p[*,i],TIME:MTmess,dTh:azi_supl[49,i]}
      endif
   endif
   return
endif

if n_elements(gtlist) eq 1 then begin ;return the List (sorted): index numor Conf (only Conf if rawconf is set)
   if gtlist eq 2 then begin & No=n_elements(iso_npt)
     if n_elements(iso_NTC) gt 1 then begin
      if No ge 1000 then fmt='(I04)' else if No ge 100 then fmt='(I03)' else if No ge 10 then fmt='(I02)' else fmt='(I1)'
      Ni=string(lindgen(No)+1,format=fmt) & if No gt 1 then Ni=transpose(Ni)

      if sortL ge 0 then begin
        if sortL eq 1 then begin            ;sort conf1 conf2 conf3  "001.08mM.Sect180.0Sym" 
         idxp =where(strpos(iso_conf,'pL ') gt 3) & idx=-1
         idxpS=where(strpos(iso_conf,'pL.') gt 3)
         idxm =where(strpos(iso_conf,'mM ') gt 3)
         idxmS=where(strpos(iso_conf,'mM.') gt 3)
         idxg =where(strpos(iso_conf,'gH ') gt 3)
         idxgS=where(strpos(iso_conf,'gH.') gt 3)
         idxu =where(strpos(iso_conf,'uU ') gt 3)
         if idxp[ 0] ge 0 then idx=[idx,idxp ]
         if idxpS[0] ge 0 then idx=[idx,idxpS]
         if idxm[ 0] ge 0 then idx=[idx,idxm ]
         if idxmS[0] ge 0 then idx=[idx,idxmS]
         if idxg[ 0] ge 0 then idx=[idx,idxg ]
         if idxgS[0] ge 0 then idx=[idx,idxgS]
         if idxu[ 0] ge 0 then idx=[idx,idxu ]   & nid=n_elements(idx)-1
         if nid gt 0 then begin
           idx=idx[1:nid]
           if nid ne  n_elements(iso_conf) then begin ;Complete with reminder entries
             idxr=where( (strpos(iso_conf,'pL ') le 3) and (strpos(iso_conf,'mM ') le 3) $
                     and (strpos(iso_conf,'gH ') le 3) and (strpos(iso_conf,'uU ') le 3) $
                     and (strpos(iso_conf,'pL.') le 3) and (strpos(iso_conf,'mM.') le 3) $
                     and (strpos(iso_conf,'gH.') le 3))
             if idxr[0] ge 0 then idx=[idx,idxr]
           endif
         endif

        endif else if sortL eq 2 then begin ;sort by Sectors  "001.08mM.Sect180.0Sym" 
         idxpS=where(strpos(iso_conf,'pL.Sect') gt 3) & idx=-1
         idxmS=where(strpos(iso_conf,'mM.Sect') gt 3)
         idxgS=where(strpos(iso_conf,'gH.Sect') gt 3)
         if idxpS[0] ge 0 then idx=[idx,idxpS]
         if idxmS[0] ge 0 then idx=[idx,idxmS]
         if idxgS[0] ge 0 then idx=[idx,idxgS]   & nid=n_elements(idx)-1
         if nid gt 0 then idx=idx[1:nid]
         if nid gt 1 then begin
           tmpConf=iso_conf[idx]                             ;"001.08mM.Sect180.0Sym" 
           tmpConf=strmid(tmpConf,0,6)+strmid(tmpConf,9,9)
           idxsect=sort(tmpConf) & idx=idx[idxsect]
         endif
         if nid gt 0 then begin
           if nid ne  n_elements(iso_conf) then begin
             idxr=where( (strpos(iso_conf,'pL.Sect') le 3) and (strpos(iso_conf,'mM.Sect') le 3) $
                     and (strpos(iso_conf,'gH.Sect') le 3))
             if idxr[0] ge 0 then idx=[idx,idxr]
           endif
         endif

        endif else idx=sort(iso_conf)       ;sort by Samples  "001.08mM.Sect180.0Sym" 

        if idx[0] ge 0 then begin
          iso_conf=iso_conf[idx]  & iso_npt =iso_npt[idx] & iso_WQE =iso_WQE[*,idx]
          iso_NTC =iso_NTC[*,idx] & iso_p   =iso_p[*,idx] & iso_supl=iso_supl[*,idx]
        endif
      endif ;SORT
      
      if keyword_set(rawConf) then gtlst=iso_conf else $
      gtlst=Ni+' '+strtrim(long64(iso_NTC[0,*]),2)+' '+iso_conf ;Return the List
     endif else gtlst='None'

   endif else $
   if gtlist eq 4 then begin & No=n_elements(azi_npt)
     if n_elements(azi_NTC) gt 1 then begin
      if No ge 1000 then fmt='(I04)' else if No ge 100 then fmt='(I03)' else if No ge 10 then fmt='(I02)' else fmt='(I1)'
      Ni=string(lindgen(No)+1,format=fmt) & if No gt 1 then Ni=transpose(Ni)

      if sortL ge 0 then begin
        if sortL eq 1 then begin            ;sort conf1 conf2 conf3  "001.08mM.Sect180.0Sym" 
         idxp =where(strpos(azi_conf,'pL ') gt 3) & idx=-1
         idxpS=where(strpos(azi_conf,'pL.') gt 3)
         idxm =where(strpos(azi_conf,'mM ') gt 3)
         idxmS=where(strpos(azi_conf,'mM.') gt 3)
         idxg =where(strpos(azi_conf,'gH ') gt 3)
         idxgS=where(strpos(azi_conf,'gH.') gt 3)
         idxu =where(strpos(azi_conf,'uU ') gt 3)
         if idxp[ 0] ge 0 then idx=[idx,idxp ]
         if idxpS[0] ge 0 then idx=[idx,idxpS]
         if idxm[ 0] ge 0 then idx=[idx,idxm ]
         if idxmS[0] ge 0 then idx=[idx,idxmS]
         if idxg[ 0] ge 0 then idx=[idx,idxg ]
         if idxgS[0] ge 0 then idx=[idx,idxgS]
         if idxu[ 0] ge 0 then idx=[idx,idxu ]   & nid=n_elements(idx)-1
         if nid gt 0 then begin
           idx=idx[1:nid]
           if nid ne  n_elements(azi_conf) then begin ;Complete with reminder entries
             idxr=where( (strpos(azi_conf,'pL ') le 3) and (strpos(azi_conf,'mM ') le 3) $
                     and (strpos(azi_conf,'gH ') le 3) and (strpos(azi_conf,'uU ') le 3) $
                     and (strpos(azi_conf,'pL.') le 3) and (strpos(azi_conf,'mM.') le 3) $
                     and (strpos(azi_conf,'gH.') le 3))
             if idxr[0] ge 0 then idx=[idx,idxr]
           endif
         endif

        endif else if sortL eq 2 then begin ;sort by Sectors  "001.08mM.Sect180.0Sym" 
         idxpS=where(strpos(azi_conf,'pL.Sect') gt 3) & idx=-1
         idxmS=where(strpos(azi_conf,'mM.Sect') gt 3)
         idxgS=where(strpos(azi_conf,'gH.Sect') gt 3)
         if idxpS[0] ge 0 then idx=[idx,idxpS]
         if idxmS[0] ge 0 then idx=[idx,idxmS]
         if idxgS[0] ge 0 then idx=[idx,idxgS]   & nid=n_elements(idx)-1
         if nid gt 0 then idx=idx[1:nid]
         if nid gt 1 then begin
           tmpConf=azi_conf[idx]
           tmpConf=strmid(tmpConf,0,6)+strmid(tmpConf,9,9)
           idxsect=sort(tmpConf) & idx=idx[idxsect]
         endif
         if nid gt 0 then begin
           if nid ne  n_elements(azi_conf) then begin
             idxr=where( (strpos(azi_conf,'pL.Sect') le 3) and (strpos(azi_conf,'mM.Sect') le 3) $
                     and (strpos(azi_conf,'gH.Sect') le 3))
             if idxr[0] ge 0 then idx=[idx,idxr]
           endif
         endif

        endif else idx=sort(azi_conf)       ;sort by Samples  "001.08mM.Sect180.0Sym" 

        if idx[0] ge 0 then begin
          azi_conf=azi_conf[idx]  & azi_npt =azi_npt[idx] & azi_WQE =azi_WQE[*,idx]
          azi_NTC =azi_NTC[*,idx] & azi_p   =azi_p[*,idx] & azi_supl=azi_supl[*,idx]
        endif
      endif ;SORT
      
      if keyword_set(rawConf) then gtlst=azi_conf else $
      gtlst=Ni+' '+strtrim(long64(azi_NTC[0,*]),2)+' '+azi_conf ;Return the List
     endif else gtlst='None'
     
   endif else gtlst='None' ;neither iso nor azi
   return
endif

if n_elements(clear)  eq 1 then begin ;Clear a content List
   if (clear eq 2) or (clear eq -1) then begin & iso_conf='' & iso_npt=0 & iso_WQU=0 & iso_NTC=0 & iso_p=0 & iso_supl=0 & endif
   if (clear eq 4) or (clear eq -1) then begin & azi_conf='' & azi_npt=0 & azi_WQU=0 & azi_NTC=0 & azi_p=0 & azi_supl=0 & endif
   if (clear eq -1)                 then       trs_tits='-'
   return
endif

;RESUME PRESENT OR NOT (typ)
; --------------------
if n_elements(dp) eq 0 then begin & TAKE_DATP,dp & okdp=1 & endif else okdp=0

swin =size(Win) & if swin[0] eq 2 then ns =swin[2] else ns =1
                  if swin[0] eq 3 then nl =swin[3] else nl =1
if n_elements(sans_supl) le 1 then sans_settings,/nowind    ; init sans_settings   ! (unprobable case)
if ((size(dp.pv))[0] ne 2) or (ns*nl eq 1) then pv=dp.p else pv=dp.pv
if n_elements(pv) eq 1 then pv=dp.pv
npv  =size(PV)  & if npv[0]  eq 2 then npv=npv[2]  else npv=1

MTmess=''
if n_elements(dp.time) eq 1 then if strpos(dp.time,'#') eq 0 then MTmess=' '+dp.time

if n_elements(pZ) eq 0 then pZ=[dp.Z] ;Bof!

if n_elements(typ) eq 0 then if n_elements(pZ) ne nl  then pZ=lindgen(nl)  ;Make par titles
if n_elements(typ) ne 0 then if n_elements(pZ) ne npv then pZ=lindgen(npv) ;Resume (keep results)
idxp  = sans_supl[80:*]
wavlen= reform(pv[idxp[0] ,*])  &  distan=reform(pv[idxp[1] ,*])   ;GET CONFIG
offset= reform(pv[idxp[2] ,*])  &  angled=reform(pv[idxp[3] ,*])
collim= reform(pv[idxp[4] ,*])  &  attenu=reform(pv[idxp[7] ,*])
numors=[reform(pv[idxp[6] ,*])] &  Aonoff=reform(pv[idxp[8] ,*]) & timx  =idxp[5]
Srotat= reform(pv[idxp[10],*])  &  Stempk=reform(pv[idxp[11],*])
Srotat= Srotat mod 360.         &  Stimes=reform(pv[timx    ,*])
 
if n_elements(transmission)    gt 0 then trans=transmission      else $
if n_elements(sans_trans[*,0]) gt 5 then trans=sans_trans[4:*,0] else trans=[sans_trans[2]]
if n_elements(thickness)       gt 0 then thick=thickness         else $
if n_elements(sans_trans[*,0]) gt 5 then thick=sans_trans[4:*,1] else thick=[sans_supl[11]]
if n_elements(parx) eq 0 then parx=strtrim(string(long64(numors)),2)+' '+dp.w_tit
parx =strcompress(reform(strtrim(parx,2)))

;CASE 1 -> REFORM SERIES (Just return parconf=part  from  titles=parx)
;RESUME NOT PRESENT
; ----------------------
if n_elements(typ) eq 0 then begin
  part=strupcase(parx)
  for i=0L,nl-1 do begin & idn=strpos(parx[i],' ') & parx[i]=strmid(parx[i],idn+1,200)    ;Remove numor
                   if nl ge 100 then fmt='(I03)' else if nl ge 10 then fmt='(I03)' else fmt='(I03)'
                   idxi=string(inst,format='(I02)') & ilmh='.'+idxi+['uU','pL','mM','gH']
                   part[i]=string(pZ[i]+1,format=fmt) + (ilmh)[curLMH] + ' '+strupcase(parx[i])
                   if Srotat[i] lt 0 then Srotat[i]=360.+Srotat[i]
                   part[i]=part[i]+' _WavL'+strtrim(string(round(wavlen[i]*10.)/10. ,format='(F4.1)'),2) $
                                  +'_Dist' +strtrim(string(round(distan[i]*10.)/10. ,format='(F5.1)'),2) $
                                  +'_Offs' +strtrim(long(  round(offset[i])),2) $
                                  +'_CoLL' +strtrim(string(round(collim[i]*10.)/10. ,format='(F5.1)'),2) $
                                  +'_Srot' +strtrim(string(round(Srotat[i]*10.)/10. ,format='(F5.1)'),2) $
                                  +'_Stmp' +strtrim(string(round(Stempk[i]*10.)/10. ,format='(F6.1)'),2) $
                                  +'_Stim' +strtrim(string(round(Stimes[i]*10.)/10. ,format='(F7.1)'),2)

                                 ;+'_Es'   +strtrim(string(round(thick(pZ(i))*100.)/100.,format='(F5.3)'),2) $
                                 ;+'_Angl' +strtrim(string(round(angled(i)*10.)/10. ,format='(F4.1)'),2) $
                                 ;+'_Attn' +strtrim(string(long (attenu(i)*Aonoff(i))),2)
 ;print,part(i)
  endfor
  if (nl gt 1) and (keyword_set(add)) then begin ;*** WE HAVE A SERIES ***
     idx =sort(part)
     park=part[idx] & un=uniq(park) & nu=n_elements(un)

     if nu lt nl then begin ;*** WE HAVE IDENTICAL SAMPLES ***

      E=dp.E & if n_elements(E) eq n_elements(Win) then begin & E=E[*,*,idx] & Eok=1 & endif else Eok=0
      Z=dp.Z & if n_elements(Z) eq nl              then begin & Z=Z[idx]     & Zok=1 & endif else Zok=0
      Win =Win[*,*,idx]  & pv=dp.pv[*,idx]    & N=dp.N[idx]
      parx=parx[   idx]  & numors=numors[idx] & part=park
      j=0
      for i=0L,n_elements(un)-1 do begin & cur=un[i] ;*** ADD NUMORS ***
          if cur ne j  then begin & idu=lindgen(cur-j+1)+j
                         Win[*,*,cur]=     total(Win[*,*,idu]  ,3)
             if Eok then E[  *,*,cur]=sqrt(total(E[  *,*,idu]^2,3))
                         pv[timx,cur]=     total(pv[timx,idu],2)
                         N[      cur]=     total(N[      idu])
          endif
          j= cur+1
      endfor

      Win=Win[*,*,un] & if Eok then E=E[*,*,un]
      pv =pv[   *,un] & if Zok then Z=Z[    un] & N=N[un] ; *** REDUCE SERIES AND GIVE BACK TO LAMP ***
      parx=parx[  un] &           part=part[un]

      dp.P=pv[*,nu-1] & dp.W_tit=parx[nu-1] ; P and w_tit for the "new" last numor

      mod_datp ,dp,'PV',pv & mod_datp,dp,'N' ,N & mod_datp ,dp,'E' ,E  & mod_datp,dp,'Z' ,Z
      if okdp then $
      give_datp,dp
     endif
  endif

;CASE 2 -> KEEP RESULTS FOR RESUMING
;RESUME IS PRESENT
; ----------------------------------
;typ=2 RADIAL   360
;    4 AZIMUTAL 360
;    5 TOTAL  SECTORS SYMMETRIC , 6 NOT SYMMETRIC
;    7 RADIAL SECTORS SYMMETRIC , 8 NOT SYMMETRIC
endif else begin
  if (typ eq 2) or ((typ ge 4) and (typ le 8)) then begin
                         ;title   n_point  W X E    num tr ctr  param  supl
   ;common resume_iso, iso_conf, iso_npt, iso_WQE, iso_NTC,    iso_p, iso_supl
   ;common resume_azi, azi_conf, azi_npt, azi_WQE, azi_NTC,    azi_p, azi_supl
   ;PREPARE NEW RESULTS
   ; ------------------
    parts =[part]
    Y     =[dp.Y]
    ttrans=trans[pZ]
    tthick=thick[pZ]
    npt   =replicate(swin[1],ns)
    if ns gt 1   then vec=transpose(dblarr(ns)+1.)
    if ns gt 1   then X  =dp.X     #vec         else X  =dp.X
    if ns gt 1   then NTC=dblarr(7)#vec         else NTC=dblarr(7) ; 4,5,6 fct,Lmargin,Rmargin
    if ns gt 1   then Sup=sans_supl#vec         else Sup=sans_supl

   ;NEXT LINES CONCERN TYP 5 or 6 when we have Total Sectors Rocking...
    if ((typ eq 5) or (typ eq 6)) and (strpos(dp.X_tit,'Numor') ge 0) then rocking=1 else rocking=0
   ;NEXT LINES CONCERN TYP 7 or 8 when we have Numors by Sectors ...
    if (rocking) or (typ eq 7) or (typ eq 8) then begin
     if ns ne npv then begin & if npv eq 1 then PV=[[pv],[pv]] ;Never!!! because ns=npv/nsectors
                       PV =congrid(PV,(size(pv))[1],ns,/center) & endif
     if ns ne n_elements(parts)  then parts =congrid(parts ,ns,/center)
     if ns ne n_elements(numors) then numors=congrid(numors,ns,/center)
     if ns ne n_elements(ttrans) then ttrans=congrid(ttrans,ns,/center)
     if ns ne n_elements(tthick) then tthick=congrid(tthick,ns,/center)
     if ns ne n_elements(Y)      then Y     =fltarr( ns)
     if rocking then begin
        if strpos(dp.X_tit,'Time') ge 0 then rocking=2 else $
        if strpos(dp.X_tit,'Temp') ge 0 then rocking=3 else $
        if strpos(dp.X_tit,'Rot')  ge 0 then rocking=4
        for i=0,ns-1 do PV[*,i]=PV[*,0]
        parts[*]=parts[0] & numors[*]=numors[0] & ttrans[*]=ttrans[0] & tthick[*]=tthick[0] & endif
    endif

    NTC[0 ,*]=numors & NTC[1,*]=ttrans & NTC[2,*]=sans_ctr[0] & NTC[3,*]=sans_ctr[1] & NTC[4,*]=1.
    WQE      =[Win,X,dp.E]
    Sup[11,*]=tthick
    Sup[21,*]=typ

   ;WE MAY ADD THE SAMPLE IDS (Apr 2013)    
    if n_elements(Sample_IDs) ge n_elements(pZ)    then Sample_IDs = Sample_IDs[pZ]
    if n_elements(Sample_IDs) eq n_elements(parts) then $
       for i=0L,n_elements(parts)-1 do begin
           bl=strpos(parts[i],' ') & parts[i]=strmid(parts[i],0,bl) + ' '+Sample_IDs[i] + strmid(parts[i],bl,200) & endfor

    if (typ eq 7) or (typ eq 8) or (rocking gt 0) then begin  ; Sectors: add the angle in parconf
       if (typ eq 5) or (typ eq 7) then secM='Sym' else secM='unS'
       if rocking gt 0 then Rck='.Rck'+strtrim(rocking,2) else Rck=''
       for i=0L,n_elements(parts)-1 do begin
           bl=strpos(parts[i],' ') & sect='.Sect'+strtrim(string(round(Y[i]*10.)/10.,format='(F05.1)'),2)+secM
           parts[i]=strmid(parts[i],0,bl) + sect + Rck + strmid(parts[i],bl,200)
       endfor
    endif

    if (typ eq 2) or (typ eq 7) or (typ eq 8) then begin ;RADIAL CURVES
     if n_elements(iso_NTC) le 1 then begin
        iso_conf=parts+MTmess & iso_npt=npt & iso_WQE=WQE     & iso_NTC=NTC & iso_p=PV[idxp,*] & iso_supl=Sup
     endif else begin
       ;RESIZE DATA IF NECESSARY
       ; -----------------------
        difnp=(size(WQE))[1] -(size(iso_WQE))[1]
        if difnp gt 0 then iso_WQE=[iso_WQE,dblarr( difnp,n_elements(iso_npt))]
        if difnp lt 0 then     WQE=[    WQE,dblarr(-difnp,ns)]

       ;KEEP UNIQ CONFIGS
       ; ----------------
        for i=0L,n_elements(parts)-1 do begin
            idm=where(strmid(iso_conf,0,strlen(parts[i])) eq parts[i]) & j=idm[0]
            if j ge 0 then begin
              iso_conf[j]=parts[i]+MTmess
              iso_npt[j] =npt[i] & iso_WQE[*,j]=WQE[*,i] & iso_NTC[*,j]=NTC[*,i] & iso_p[*,j]=PV[idxp,i] & iso_supl[*,j]=Sup[*,i]
            endif else begin
              iso_conf=[iso_conf,parts[i]+MTmess] & iso_npt=[iso_npt,npt[i]]       & iso_WQE =[[iso_WQE] ,[WQE[*,i]]]
              iso_NTC =[[iso_NTC] ,[NTC[*,i]]]    & iso_p  =[[iso_p],[PV[idxp,i]]] & iso_supl=[[iso_supl],[Sup[*,i]]]
            endelse
        endfor
     endelse
    ;SORT by LMH
     if n_elements(iso_conf) gt 1 then begin & idx=sort(iso_conf)
        iso_conf=iso_conf[idx]  & iso_npt =iso_npt[idx] & iso_WQE =iso_WQE[*,idx]
        iso_NTC =iso_NTC[*,idx] & iso_p   =iso_p[*,idx] & iso_supl=iso_supl[*,idx]
     endif
    endif ;RADIAL

    if (typ eq 4) or (typ eq 5) or (typ eq 6) then begin ;AZIMUTAL CURVES ; TOTAL SECTORS
     if n_elements(azi_NTC) le 1 then begin
        azi_conf=parts+MTmess & azi_npt=npt & azi_WQE=WQE     & azi_NTC=NTC & azi_p=PV[idxp,*] & azi_supl=Sup
     endif else begin
       ;RESIZE DATA IF NECESSARY
       ; -----------------------
        difnp=(size(WQE))[1] - (size(azi_WQE))[1]
        if difnp gt 0 then azi_WQE=[azi_WQE,dblarr( difnp,n_elements(azi_npt))]
        if difnp lt 0 then     WQE=[    WQE,dblarr(-difnp,ns)]

       ;KEEP UNIQ CONFIGS
       ; ----------------
        for i=0L,n_elements(parts)-1 do begin
            idm=where(strmid(azi_conf,0,strlen(parts[i])) eq parts[i]) & j=idm[0]
            if j ge 0 then begin
              azi_conf[j]=parts[i]+MTmess
              azi_npt[j] =npt[i] & azi_WQE[*,j]=WQE[*,i] & azi_NTC[*,j]=NTC[*,i] & azi_p[*,j]=PV[idxp,i] & azi_supl[*,j]=Sup[*,i]
            endif else begin
              azi_conf=[azi_conf,parts[i]+MTmess] & azi_npt=[azi_npt,npt[i]]       & azi_WQE =[[azi_WQE] ,[WQE[*,i]]]
              azi_NTC =[[azi_NTC] ,[NTC[*,i]]]    & azi_p  =[[azi_p],[PV[idxp,i]]] & azi_supl=[[azi_supl],[Sup[*,i]]]
            endelse
        endfor
     endelse
    ;SORT by LMH
     if n_elements(azi_conf) gt 1 then begin & idx=sort(azi_conf)
        azi_conf=azi_conf[idx]  & azi_npt =azi_npt[idx] & azi_WQE =azi_WQE[*,idx]
        azi_NTC =azi_NTC[*,idx] & azi_p   =azi_p[*,idx] & azi_supl=azi_supl[*,idx]
     endif

    endif ;AZIMUTAL

  endif ;TYP ok

endelse
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_save_gt, Win, datp=datp, par=par, pZ=pZ, Fincr=Fincr, Inst=instru, fromList=fromList
;+
;** ************
;**
;** Write file in Ron format ILL-SANS treated data
;-


;** sans_supl: 0 =wavl 1=distance 2=offset 3=angle 4=collimation 8=Xpixsize 9=Ypixsize
;**            11=thickness 20=delta_R 21=typ_group 80:*=index_of_parameters
;**

forward_function   finefile
common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl

;Check for inputs
;****************
sz=size(Win) & if sz[0]  ge 2 then sy=sz[2] else sy=1 & sx=sz[1]
               if sz[0]  eq 3 then nz=sz[3] else nz=1
;if sz(0)                gt 2 then return
if n_elements(Win)       le 1 then return
if n_elements(sans_supl) le 1 then sans_settings,/nowind  ; init sans_settings   ! (unprobable case)
if n_elements(datp)      eq 0 then TAKE_DATP,datp

E=datp.E & ierrs=0 & if n_elements(E) ne n_elements(Win) then E=Win*0. else ierrs=1
X=datp.X & Y=datp.Y & Xt=datp.X_TIT & Yt=datp.Y_TIT & Pv=datp.Pv & Z=datp.Z
MTmess = datp.time

xp=sans_supl[80:*] ;index of parameters
         ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
         ;0         1         2       3           4          5           6      7           8
         ;9         10        11      12          13         14          15     16
         ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight, Xdet , Ydet

;Check for data type
;*******************
treatment=[  $
'UNKNOWN_0', $                     ;typ=0
'CORRECTED DATA', $                ;typ=1  Xt='X detector'    Yt='Y detector'
'RADIAL INTEGRATIONS', $           ;typ=2  Xt='Q'             Yt='Frames' or 'counts'
'UNKNOWN_3',           $           ;typ=3
'TOTAL AZIMUTHAL',     $           ;typ=4  Xt='Degrees'       Yt='Total'
'SECTOR TOTAL  SYMMETRICAL', $     ;typ=5  Xt='Sectors (deg)' Yt='numors' or 'Total'
'SECTOR TOTAL  NON_SYMMETRICAL', $ ;typ=6  Xt='Sectors (deg)' Yt='numors' or 'Total'
'SECTOR RADIAL SYMMETRICAL'    , $ ;typ=7  Xt='Q'   Yt='S(Q)' or 'Numors' or 'Sectors' or 'Numors by sectors'
'SECTOR RADIAL NON_SYMMETRICAL']   ;typ=8  Xt='Q'   Yt='S(Q)' or 'Numors' or 'Sectors' or 'Numors by sectors'
typp   = sans_supl[21]>0<(n_elements(treatment)-1)
treatyp= treatment[typp]
if (size(MTmess))[1] ne 7 then MTmess='' & if strpos(MTmess,'#') ne 0 then MTmess=''
if MTmess gt ' ' then plus1=1 else plus1=0

if (Xt le ' ') then if (typp eq 2) or (typp eq 7) or (typp eq 8) then Xt='Q'
if (Yt le ' ') then if (typp eq 2) or (typp eq 7) or (typp eq 8) then Yt='Counts'
if (Xt le ' ') then if (typp eq 4) or (typp eq 5) or (typp eq 6) then Xt='Degrees'
if (Yt le ' ') then if (typp eq 4) or (typp eq 5) or (typp eq 6) then Yt='Total'
rocking=0
isDQ   =0
bsl    =180.
if (strpos(strlowcase(Yt),'detector')  ge 0) or (sz[0] eq 3) or $
   (strpos(strlowcase(Xt),'corrected') ge 0) then begin                         ;2D CORRECTED DATA (DETECTOR)
   typ=1 & gtf='t' & dx=sx & dy=sy & nskip=39+plus1 & nskipp=0        & npdfx=0
   treatyp=treatment[typ]
endif else begin
   typ=2 & gtf='g' & dx=sx & dy=1  & nskip=42+plus1 & nskipp=38+plus1 & npdfx=3 ;RADIAL OR TOTAL INTEGRATION DATA
   if strlowcase(Yt) eq 'numors' then nz=sy else dx=sx*sy
   if strlowcase(Yt) eq 'numors' then Yt='Counts'
   if strpos(strlowcase(Yt),' by ')   gt 0 then begin  ;Numors by sectors
      nz=sy & dx=sx
   endif
   if strpos(strlowcase(Xt),'numors') ge 0 then begin  ;Rocking
      nz=sy & dx=sx & rocking=1
   endif
   if strpos(strlowcase(Xt),'degree') ge 0 then begin  ;Total azimutal
      nz=sy & dx=sx
   endif
   if (n_elements(Z) eq sx)  and Xt eq 'Q' then begin
      isDQ=1 & Xt=XT+' (dQ)'                           ;DeltaQ should be printed
   endif
endelse

if ((typp eq 7) or (typp eq 8)) and (typ eq 2) then sectors=Y else sectors=fltarr(nz)
if strpos(strlowcase(Yt),'sectors') ge 0 then Yt='Counts'
if total(sectors) gt 0. then sect=1 else sect=0 & strsec=''
if sect then bsl=sans_supl[15] ;Opening sectors

if typ eq 1 then Win=Win>0.    ;PB E10.3 ON WINDOWS (exponent is 3char instead of 2)

Strans=sans_trans[2]
Sthick=sans_supl[11]
DeltaR=sans_supl[20]*max(sans_supl[8:9])
Offset=sans_supl[2] & if sans_supl[65] ne 1 then Offset=0.

if ((size(Pv))[0] eq 2) and (nz gt 1) then begin ;We may have many numors
    numor  =reform(long64(Pv[xp[6] ,*]))
    Sheight=reform(       Pv[xp[9] ,*])
    Srot   =reform(       Pv[xp[10],*])
    Ktemp  =reform(       Pv[xp[11],*])
    Ctime  =reform(       Pv[xp[5] ,*])
    if n_elements(sans_trans) gt 5 then begin
       if (n_elements(pZ) ne n_elements(numor)) then pZ=lindgen(n_elements(numor))
       Strans=sans_trans[4+pZ,0]
       Sthick=sans_trans[4+pZ,1] & endif
    if n_elements(par) eq n_elements(numor) then  titles   =par $
    else begin & titles=strarr(n_elements(numor)) & titles[*]=datp.w_tit & endelse

endif else begin                 ;Or only one
    numor  =long64(datp.P[xp[6]])
    Sheight=       datp.P[xp[9]]
    Srot   =       datp.P[xp[10]]
    Ktemp  =       datp.P[xp[11]]
    Ctime  =       datp.P[xp[5]]
    titles =       datp.w_tit
endelse

if n_elements(numor) lt nz then begin ;Case 'Numors by Sectors' or rocking
    numor  =congrid(numor  ,nz,/center)
    Sheight=congrid(Sheight,nz,/center)
    Srot   =congrid(Srot   ,nz,/center)
    Ktemp  =congrid(Ktemp  ,nz,/center)
    Ctime  =congrid(Ctime  ,nz,/center)
    Strans =congrid(Strans ,nz,/center)
    Sthick =congrid(Sthick ,nz,/center)
    titles =congrid(titles ,nz,/center)
    sectors=congrid(sectors,nz,/center)
endif
if rocking gt 0 then begin & numor[*]=numor[0] & endif

on_ioerror,mis
for i=0L,nz-1 do begin & u=-1
  snumor=strtrim(string(numor[i]),2) & l=strlen(snumor)
  while l lt 6 do begin & snumor='0'+snumor & l=l+1 & endwhile
  nn=1
  if sect then strsec='_'+string(round(sectors[i]),format='(I3.3)')
  if n_elements(fromList) eq 1 then seq=fromList else seq=100
  seqstr='.'+string(seq,format='(I3.3)')
  fif=gtf+snumor+strsec+seqstr
  if keyword_set(Fincr) then $
  while (nn gt 0) do begin & seq=string(seq,format='(I3.3)') & fif=gtf+snumor+strsec+'.'+seq
                           nam=finefile(fif,count=nn) & if nn gt 0 then seq=seq+1 & endwhile
  openw,u,fif,/get_lun
    printf,u,strmid(titles[i],0,80)
    printf,u,'ILL  SANS '+instru+'  Written by Lamp'
    printf,u,numor[i],seq,dx,dy,nskip,nskipp, format='(6(I10))'
    printf,u,1,4+plus1,32,0,npdfx,ierrs     , format='(6(I10))'
    printf,u,'Lamp '+stime(dum)
    printf,u,treatyp
    printf,u,Xt & printf,u,Yt
    if plus1 then printf,u,MTmess
    printf,u,' '

    fmt='(F10.4,A)'
    printf,u,Offset         ,' ! Theta-0 Detector offset angle',format=fmt
    printf,u,sans_ctr[0]*sans_supl[8]-1,' ! X0 cms Beam center',format=fmt
    printf,u,sans_ctr[1]*sans_supl[9]-1,' ! Y0 cms Beam center',format=fmt
    printf,u,DeltaR         ,' ! Delta-R cms regrouping step'  ,format=fmt
    printf,u,sans_supl[1]   ,' ! SD m Sample-detector distance',format=fmt
    printf,u,sans_supl[0]   ,' ! Angstroms incident wavelength',format=fmt
    printf,u,sans_supl[4]   ,' ! m collimation distance'       ,format=fmt
    printf,u,1.             ,' ! concentration'                ,format=fmt
    printf,u,0.             ,' ! ISUM central window sum'      ,format=fmt
    printf,u,1.             ,' ! flux monitor counts'          ,format=fmt
    printf,u,bsl            ,' ! degrees detector sector width',format=fmt
    printf,u,sectors[i]     ,' ! degrees sector orientation'   ,format=fmt
    printf,u,datp.P[xp[12]]*100. ,' ! % wavelength spread'     ,format=fmt
    printf,u,datp.P[xp[13]]      ,' ! mm source slit width x'  ,format=fmt
    printf,u,datp.P[xp[14]]      ,' ! mm source slit height y' ,format=fmt
    printf,u,0.             ,' ! mm sample width x'            ,format=fmt
    printf,u,Sheight[i]     ,' ! mm sample height y'           ,format=fmt
    printf,u,sans_supl[8]*10,' ! mm detector x pixel size'     ,format=fmt
    printf,u,sans_supl[9]*10,' ! mm detector y pixel size'     ,format=fmt
    printf,u,Srot[   i]     ,' ! degrees sample normal/beam'   ,format=fmt
    printf,u,Ktemp[  i]     ,' ! K sample temperature'         ,format=fmt
    printf,u,Strans[ i]     ,' ! sample transmission'          ,format=fmt
    printf,u,Sthick[ i]*10  ,' ! mm sample thickness'          ,format=fmt
    printf,u,Ctime[  i]     ,' ! counting time secs'           ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,0.             ,' ! reserved'                     ,format=fmt
    printf,u,sans_supl[49]*1000. ,' ! Delta Theta *1000.'      ,format=fmt

    if typ ne 1 then begin ;G format
       printf,u,dx,0,0,0,0,0,0,6,                        format='(8(I9,1X))'
       printf,u,1.,sans_supl[1]*100.,0.,1.,sans_supl[0], format='(5(E14.6,1X))'
       printf,u,0.,0.               ,0.,0.,0.,           format='(5(E14.6,1X))'
       if nz gt 1 then      for j=0L,sx-1 do printf,u,X[j],Win[j,i],E[j,i], format='(3(E14.6,1X))' $
       else for k=0,sy-1 do for j=0L,sx-1 do printf,u,X[j],Win[j,k],E[j,k], format='(3(E14.6,1X))'

       if isDQ then for j=0L,sx-1 do printf,u,Z[j], format='(E14.6)' ;Should be deltaQ

    endif else begin       ;T format
       printf,u,Win[*,*,i], format='(8(E10.3,1X))'
       if ierrs then printf,u,E[  *,*,i], format='(8(E10.3,1X))'
    endelse
  flush,u & free_lun,u & u=-1
endfor
return
mis:if u gt 0 then free_lun,u
    lamp_error
end

pro sans_save_Qs, Qfile, sz ,getQ=Q, getQx=Qx, getQy=Qy, noWrite=noWrite
;+
;** ************
;** save 2D Q ascii file after correction
;-

@lamp.cbk
common c_sans_settings

if sans_supl[0]*sans_supl[1] gt 0 then begin
  xy0=sans_ctr-1 & if xy0[0] lt 0 then xy0[0]=xy0[0]+2.
                   if xy0[1] lt 0 then xy0[1]=xy0[1]+2.
  Cx=strtrim(string(xy0[0],format='(f7.2)'),2)
  Cy=strtrim(string(xy0[1],format='(f7.2)'),2)
  Wl=strtrim(string(sans_supl[0],format='(f7.2)'),2)
  D =strtrim(string(sans_supl[1],format='(f7.2)'),2)
  Qfile=Qfile+'_d'+D+'cx'+Cx+'cy'+Cy+'wl'+Wl+'.txt'
  tmp=finefile(Qfile,count=cnt)
  if cnt gt 0 then noWrite=1 ;File already written
  
  sans_settings_dist, [2,sz],Dst,Center=xy0,xyPsize=sans_supl[8:9], DforT=DforT ,thetrad=thetrad

  if n_elements(thetrad) gt 1 then theta2=thetrad $ ;We have theta coordinates (ex:D16)
  else begin
    if (abs(sans_supl[3])  ge 1.) and (n_elements(DforT) gt 1) then R=DforT $;We have a detector angle
    else R=Dst
    theta2=atan(R/(sans_supl[1]*100.))
  endelse
  
  Cab= round( xy0 )
  Q  = round( 4.*!pi*sin(theta2/2.)/sans_supl[0] *100000. )/100000.
  
  if (Cab[0] ge 0.) and (Cab[0] lt sz[0]) and $
     (Cab[1] ge 0.) and (Cab[1] lt sz[1]) then begin                ;center is in Q matrix
      Qx =        Q[*,Cab[1]]    & Qx[0:(Cab[0]-1)>0] = -Qx[0:(Cab[0]-1)>0]
      Qy = reform(Q[  Cab[0],*]) & Qy[0:(Cab[1]-1)>0] = -Qy[0:(Cab[1]-1)>0]
      if Cab[0] gt xy0[0]   then   Qx[Cab[0]]         = -Qx[Cab[0]] ;center over middle pixel
      if Cab[1] gt xy0[1]   then   Qy[Cab[1]]         = -Qy[Cab[1]] ;center over middle pixel
  endif else begin
      Qx = lindgen(sz[0])+1
      Qy = lindgen(sz[1])+1
  endelse
 ;USING WORKSPACE W29.
  if not keyword_set(noWrite) then begin
    XICUTE,'W29=[1,2,3]'
    W29 = Q
    E29 = 0 & Z29=0 & X29=lindgen(sz[0]) & Y29=lindgen(sz[1])
    X_tit[29]='X detector' & Y_tit[29]='Y detector' & W_tit[29]=Qfile
    write_lamp,Qfile,W=29,format='column'
  endif
endif
end

pro sans_save_NIST, file, qx, qy, data, err
  openw, u, file, /get_lun
  printf, u, 'Data columns Qx - Qy - I(Qx,Qy) - err(I)'
  printf, u, 'ASCII data'
  for ix = 0, n_elements(qx)-1 do begin
    for iy = 0, n_elements(qy)-1 do begin
      printf, u, format='(f7.3,2x,f7.3,2x,e16.8,2x,e16.8)', qx[ix], qy[iy], data[ix,iy], err[ix,iy]
    endfor
  endfor
  close, u
  free_lun, u
end

;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

;pro int_event,ev,uv
;;+
;;** *********
;;** event parser for ben_int_create sectors (also in bens)
;;-
;
;   common c_sans_to_ben, sansBen
;
;   widget_control,get_uvalue=varAccess,ev.top
;   idx=where(tag_names(varAccess) eq 'DATACCESS') ; If linked to sans_settings
;   if idx[0] lt 0 then widget_control,bad_id=ii,get_uvalue=varAccess,sansBen
;
;   case uv[1] of
;      400:  begin & wait,.05 & widget_control,/destroy,ev.top & end
;      401:  BEN_CHANGE_WORKSPACE,uv
;      402:  int_help,varAccess,ev.top
;      403:  int_loadWorkspace,varAccess,uv[3],uv[4]
;      405:  int_plotMenu,varAccess,uv
;      406:  int_updateRay,varAccess,ev,uv[3]
;      407:  int_changeZone,varAccess,ev,uv[3],uv[4]
;      408:  int_miniPix,varAccess,1
;      409:  int_changeCenter,varAccess
;      410:  int_zonePlot,varAccess,ev.select
;      411:  int_updateRes,varAccess,uv[3],uv[4],uv[5]
;      412:  int_updateMini,varAccess,ev,uv
;      else: print,"INT_EVENT:",varAccess,uv
;   endcase
;   if (idx[0] lt 0) then $
;   if ((uv[1] eq 406) or (uv[1] eq 407) or (uv[1] eq 412)) then sans_settings_mask_event
;end
;
;pro int_loadWorkspace,varAccess,k,labelID
;;+
;;** *****************
;;** for sectors (also in bens)
;;** Tests if the workspace Wk has good dimensions. If it is
;;** the case, the procedure `int_loadRequest' is called. If the
;;** workspace does not fit dimension requirement, an error
;;** message is displayed in the widget_label `labelID'.
;;-
;
;forward_function desk_name, NUMtSTR, int_loadRequest, BEN_WK_SIZE
;
;   temp=BEN_WK_SIZE(k)
;   if (temp[0] ne 2) and (temp[0] ne 3) then begin
;      widget_control,set_value=("'W" + NUMtSTR(k,0) + "' is invalid."),labelID
;      print,string(7b)
;      return
;   end
;   widget_control,set_value=("Loading 'W" + NUMtSTR(k,0) + "'."),labelID
;   BEN_READ_WK,k,data,datx,daty,datn,datp,datpar
;   if temp[0] eq 3 then data=data[*,*,temp[3]-1] ;Two dimensions only !!!DIDS
;   if n_elements(datx) ne temp[1] then datx=indgen(temp[1])+1
;   if n_elements(daty) ne temp[2] then daty=indgen(temp[2])+1
;   temp=int_loadRequest(varAccess,k,data,datx,daty,datn,datp,datpar)
;   widget_control,set_value="Ready.",labelID
;end
;
;pro int_poly,a1,a2,a3,c,c_x,c_y,r1,r2,deg2rad,test,squared
;;+
;;** ********
;;** for sectors (also in bens)
;;** Procedure 'int_poly' draws the section enclosed in angles
;;** a1 and a3, and radia r1 and r2, in the current window.
;;-
;
;forward_function int_middle
;
;   a12=int_middle(varAccess,a1,a2)
;   a23=int_middle(varAccess,a2,a3)
;   
;   temp1=[a1,a12,a2,a23,a3]*deg2rad
;   temp2=[a3,a23,a2,a12,a1]*deg2rad
;   
;   if not squared then begin
;      x=[r1*cos(temp1),r2*cos(temp2),r1*cos(a1*deg2rad)]
;      y=[r1*sin(temp1),r2*sin(temp2),r1*sin(a1*deg2rad)]
;   endif else begin
;      a3a1= a3-a1 & if a3a1 lt 0. then a3a1=360.+a3a1
;      dx=( (a3a1*r2/(r1>1.) -a3a1)<90. )/2.
;      b1=a1-dx & b12=a12-dx/2. & b23=a23+dx/2. & b3=a3+dx
;      temp3=[b1,b12,a2,b23,b3]*deg2rad
;      temp3=[b1,           b3]*deg2rad ; Modification for Bruno D...
;      temp2=[a3,           a1]*deg2rad ; Modification for Bruno D...
;      x=[r1*cos(temp3),r2*cos(temp2),r1*cos(b1*deg2rad)]
;      y=[r1*sin(temp3),r2*sin(temp2),r1*sin(b1*deg2rad)]
;   endelse
;   
;   oplot,c_x+x,c_y+y,color=c,thick=2,psym=0
;   oplot,c_x-x,c_y-y,color=c,thick=2,psym=0,lineStyle=2*test
;end
;
;pro int_drawAxes,varAccess
;;+
;;** ************
;;** for sectors (also in bens)
;;** This procedure updates the mini widget_draw.
;;-
;
;common c_trap,  trap_x1,trap_x2,trap_y1,trap_y2,trap_ws, trap_current
;
;   widget_control,get_uvalue=dims,varAccess.datAccess.dims
;   widget_control,get_uvalue=set,varAccess.settings
;   widget_control,get_uvalue=pix,varAccess.pixmap
;   widget_control,get_uvalue=cst,varAccess.constants
;   widget_control,get_uvalue=col,varAccess.colors
;   widget_control,get_uvalue=IDs,varAccess.IDs
;   widget_control,get_uvalue=opt,varAccess.options
;   c_x=set.cx+0.5
;   c_y=set.cy+0.5
;   if (dims.dim eq dims.dimx) $
;   then c_y=c_y+(dims.dim-dims.dimy)/2.0*(dims.dimx/dims.dim) $
;   else c_x=c_x+(dims.dim-dims.dimx)/2.0*(dims.dimy/dims.dim)
;   cur=set.alpha
;   window,0,xsize=cst.minSize,ysize=cst.minSize,/pixmap,retain=2 & trap_current=0
;   tvscl,pix.img,pix.xmarg,pix.ymarg,order=0
;   if ((pix.xmarg ne 0) or (pix.ymarg ne 0)) then begin
;      x=pix.xmarg*[1,1,-1,-1,1]+[-1,-1,cst.minSize,cst.minSize,-1]
;      y=pix.ymarg*[1,-1,-1,1,1]+[-1,cst.minSize,cst.minSize,-1,-1]
;      plot,x,y,xrange=[0,cst.minSize],yrange=[0,cst.minSize], $
;   /device,/noerase,pos=[0,0,cst.minSize,cst.minSize], $
;   lineStyle=1,xstyle=5,ystyle=5,psym=0
;   end
;   temp=[-dims.dim,dims.dim]*sqrt(2)
;   plot,[0,dims.dim],[0,dims.dim],/nodata,/noerase,/device, $
;      xstyle=5,ystyle=5,pos=[0,0,cst.minSize,cst.minSize],psym=0
;   for i=0,(set.zones-1) do begin
;      if set.zones le 8 then $
;      oplot   , cos(cur*cst.deg2rad)*temp+c_x,sin(cur*cst.deg2rad)* $
;          temp+c_y,color=col[i],thick=1,psym=0
;      int_poly,(cur-set.beta),cur,(cur+set.beta),col[i], $
;          c_x,c_y,set.r1,set.r2,cst.deg2rad,opt.plotZone,varAccess.squared
;      cur=cur+set.gamma
;   end
;   polyfill,c_x+set.mx*[-1,-1,1,1],c_y+set.my*[1,-1,-1,1],color=0
;   wset,IDs.mini
;   device,copy=[0,0,cst.minSize,cst.minSize,0,0,0]
;end
;
;pro int_updateMini,varAccess,ev,uv
;;+
;;** **************
;;** for sectors (also in bens)
;;** This procedure is called to respond to an event from
;;** an angle (alpha,beta or gamma) widget_slider.
;;-
;
;   widget_control,get_uvalue=set,varAccess.settings
;   if (uv[3] eq 0) then begin
;      set.alpha=ev.value
;      widget_control,ev.id, get_value=tmp & set.alpha=tmp ;DIDS we can have a FUNC_GET_VALUE
;   endif else $
;   if (uv[3] eq 1) then begin
;      set.beta =ev.value
;      widget_control,ev.id, get_value=tmp & set.beta=tmp  ;DIDS we can have a FUNC_GET_VALUE
;      widget_control,uv[4],SET_SLIDER_MAX=(180/(ev.value>1))<36
;   endif else begin
;      set.gamma=ev.value
;      widget_control,ev.id, get_value=tmp & set.gamma=tmp ;DIDS we can have a FUNC_GET_VALUE
;      uv[4]=set.gamma
;      widget_control,set_uvalue=uv,ev.id
;   endelse
;   widget_control,set_uvalue=set,varAccess.settings
;   int_drawAxes,varAccess
;end
;
;pro int_changeCenter,varAccess
;;+
;;** ****************
;;** for sectors (also in bens)
;;** This procedure reads the four widget_texts, and updates
;;** the corresponding variables, and the mini picture.
;;** It is called by the 'updateRes' procedure, and when the
;;** return key is pressed within a widget_text.
;;-
;
;forward_function BEN_READTEXT
;
;   widget_control,get_uvalue=IDs,varAccess.IDs
;   widget_control,get_uvalue=set,varAccess.settings
;   set.cx=BEN_READTEXT(IDs.cx)-1.0
;   widget_control,set_value=NUMtSTR(set.cx+1.0,2),IDs.cx
;   set.cy=BEN_READTEXT(IDs.cy)-1.0
;   widget_control,set_value=NUMtSTR(set.cy+1.0,2),IDs.cy
;
;  ;DID!! Correct Inner & Outer radius
;   widget_control,get_uvalue=temp,varAccess.datAccess.dims
;   dim   =temp.dim
;   if set.cx lt 0 then mx=-set.cx+dim else mx=max([dim,set.cx])
;   if set.cy lt 0 then my=-set.cy+dim else my=max([dim,set.cy])
;
;   mm    =1+round(sqrt(2)*max([mx,my]))
;   widget_control,set_slider_max=mm,IDs.r1
;   widget_control,set_slider_max=mm,IDs.r2
;
;   set.mx=BEN_READTEXT(IDs.mx)
;   widget_control,set_value=NUMtSTR(set.mx,2),IDs.mx
;   set.my=BEN_READTEXT(IDs.my)
;   widget_control,set_value=NUMtSTR(set.my,2),IDs.my
;   widget_control,set_uvalue=set,varAccess.settings
;   int_drawAxes,varAccess
;end
;
;pro ben_int_create,desk,n,m,base,request
;;+
;;** **************
;;** for sectors (also in bens)
;;** Modified to work with sans_settings (everywhere you find "sans")
;;-
;
;@lamp.cbk
;common c_sans_to_ben, sansBen
;
;forward_function desk_name, NUMtSTR, sys_dep, BEN_COMMANDS, int_inits
;
;kplamp_siz=lamp_siz & sans=0
;if n_elements(sansBen) gt 1 then begin & benSans=sansBen & sansBen=0L & sans=1 & lamp_siz=lamp_siz>800 & endif
;
;;;;;; Creation of the top-level base, and the top `command bar'.
;   if (m eq 0) then winTitle=desk_name(n,1) $
;   else winTitle=(desk_name(n,1) + " " + NUMtSTR(1+m,0))
;   if sans then $
;   base=widget_base(title=winTitle,/column,group_leader=long(bensans[0]),resource_name='lampben',map=0) else $
;   base=widget_base(title=winTitle,/column,group_leader=desk            ,resource_name='lampben')
;   if sans then sansBen=base
;
;   topRow=widget_base(base,/row)
;   comID =BEN_COMMANDS(topRow,n,[0,0])
;
;;;;;; Creation of the widgets containing all the `common'
;;;;;; variables used by the program. The IDs of those widgets
;;;;;; are stored into the top-level base user-value.
;   varBase  =widget_base(title=(winTitle + " vars"),map=0,group=base)
;   datAccess=widget_base(varBase,/row)
;   data     =widget_label(datAccess,value="")
;   datx     =widget_label(datAccess,value="")
;   daty     =widget_label(datAccess,value="")
;   datn     =widget_label(datAccess,value="")
;   datp     =widget_label(datAccess,value="")
;   datppp   =widget_label(datAccess,value="")
;   dims     =widget_label(datAccess,value="")
;   dat      ={w:data,x:datx,y:daty,n:datn,p:datp,par:datppp,dims:dims}
;   constants=int_inits(0,winTitle)
;   if sans then constants.minSize=long(benSans[10])
;   options  =int_inits(2)
;   set      =widget_label(varBase,value="",uvalue=int_inits(3))
;   opt      =widget_label(varBase,value="",uvalue=options)
;   cst      =widget_label(varBase,value="",uvalue=constants)
;   IDs      =widget_label(varBase,value="")
;   col      =widget_label(varBase,value="",uvalue=int_inits(1))
;   pix      =widget_label(varBase,value="")
;   p0       =widget_label(varBase,value="",uvalue=0)
;   p1       =widget_label(varBase,value="",uvalue=0)
;   p2       =widget_label(varBase,value="",uvalue=0)
;   varAccess={datAccess:dat,settings:set,options:opt,constants:cst,IDs:IDs,colors:col,pixmap:pix,plots:[p0,p1,p2],squared:0}
;   widget_control,set_uvalue=varAccess,base
;   int_initData,varAccess,request
;   widget_control,get_uvalue=set ,varAccess.settings
;   widget_control,get_uvalue=temp,varAccess.datAccess.dims
;   dim      =temp.dim
;   lamp_siz =kplamp_siz
;
;;;;;; The main draw window, to plot the integration result.
;;;;;; If the screen ysize is lower than 900 pixels, then the window
;;;;;; is created horizontal; otherwise, it has a more vertical aspect.
;   if (lamp_siz lt 1000) then begin
;      temp    =widget_base(base,/row)
;      settings=widget_base(temp,/row,/frame)
;      result  =widget_draw(temp,retain=2, /button_event,uvalue=[-88,0], $
;               xsize=constants.drawSize*(constants.drawSize/503.37),ysize=constants.drawSize)
;   end else begin
;      result  =widget_draw(base,  xsize=constants.drawSize*(constants.drawSize/503.37),  $
;                          ysize=constants.drawSize,retain=2,/button_event,uvalue=[-88,0])
;      settings=widget_base(base,/row,/frame)
;   end
;
;;;;;; Creation of the column of sliders for setting angles and radius.
;   if sans then begin
;      widget_control,long(benSans[1]),set_uvalue=[-88,412,n,0]
;      widget_control,long(benSans[2]),set_uvalue=[-88,412,n,2,90.]
;      widget_control,long(benSans[3]),set_uvalue=[-88,412,n,1,long(benSans[4])]
;      widget_control,long(benSans[4]),set_uvalue=[-88,407,n,long(benSans[2]),long(benSans[3])]
;      widget_control,long(benSans[5]),set_uvalue=[-88,406,n,0]
;      widget_control,long(benSans[6]),set_uvalue=[-88,406,n,1]
;      r1ID=long(benSans[5]) & r2ID=long(benSans[6])
;
;   endif else begin & mx=1+round(sqrt(2)*dim)
;      sliders=widget_base(  settings,/column,xsize=constants.minSize/2.0)
;              put_logo,widget_base(sliders,/row)
;      asl    =widget_slider(sliders,value=set.alpha,title="Total rotation" ,min=0,max=90,uvalue=[-88,412,n,0] ,/drag)
;      gsl    =widget_slider(sliders,value=set.gamma,title="Single rotation",min=0,max=90,uvalue=[-88,412,n,2,set.gamma] ,/drag)
;      bsl    =widget_slider(sliders,value=set.beta ,title="Opening"        ,min=0,max=45,uvalue=[-88,412,n,1,0L] ,/drag)
;      zoneID =widget_slider(sliders,value=set.zones,title="# of sectors"   ,min=1,max=8 ,uvalue=[-88,407,n,gsl,bsl] ,/drag)
;      widget_control,bsl,set_uvalue=[-88,412,n,1,zoneID]
;      r1ID   =widget_slider(sliders,value=set.r1,title="Inner radius"      ,min=0,max=mx,uvalue=[-88,406,n,0] ,/drag)
;      r2ID   =widget_slider(sliders,value=set.r2,title="Outer radius"      ,min=0,max=mx,uvalue=[-88,406,n,1] ,/drag)
;   endelse
;
;;;;;; The right column contains the mini draw window, where
;;;;;; zones are represented, and the fields for center and
;;;;;; mask settings, plus the plots buttons.
;   others=widget_base(  settings,/column)
;   plots =widget_base(  others,/row)
;   tmpdid=widget_base(  others)
;   mini  =widget_draw(  tmpdid,retain=2, /button_event,uvalue=[-88,0], $
;                        xsize=constants.minSize,ysize=constants.minSize)
;
;;;; Toggle button to set the mini to logAspect or not
;   tmpdid=widget_base(  plots,/nonexclusive)
;   logBtn=widget_button(tmpdid,value="Log aspect",uvalue=[-88,408,n,0])
;
;   update=widget_button(plots,value="I[R]",uvalue=[-88,411,n,comID[7],logBtn,1],font=ft_b_normal)
;   update=widget_button(plots,value="I[S]",uvalue=[-88,411,n,comID[7],logBtn,2],font=ft_b_normal)
;;;; Creation of the 'plot' menu.
;   menuID   =lonarr(5)
;   menuID[0]=widget_button(plots      ,value="Plot",menu=2)
;   temp     =widget_button(menuID[0]  ,value="Over last plot",uvalue=[-88,405,n,0,0,comID[7],logBtn,0])
;   menuID[1]=widget_button(menuID[0]  ,value="Over plot #1"  ,uvalue=[-88,405,n,0,1,comID[7],logBtn,0])
;   menuID[2]=widget_button(menuID[0]  ,value="Over plot #2"  ,uvalue=[-88,405,n,0,2,comID[7],logBtn,0])
;   temp     =widget_button(menuID[0]  ,value="Into Mem 1"    ,uvalue=[-88,405,n,1,1])
;   temp     =widget_button(menuID[0]  ,value="Into Mem 2"    ,uvalue=[-88,405,n,1,2])
;   menuID[3]=widget_button(menuID[0]  ,value="Plot Mem 1"    ,uvalue=[-88,405,n,2,1])
;   menuID[4]=widget_button(menuID[0]  ,value="Plot Mem 2"    ,uvalue=[-88,405,n,2,2])
;;DID!!
;   temp     =widget_button(menuID[0]  ,value=" ")
;   temp     =widget_button(menuID[0]  ,value="To PS File...",uvalue=[-88,405,n,3,0,comID[7],0,0])
;   temp     =widget_button(menuID[0]  ,value="To Wsp W20",uvalue=[-88,405,n,3,0,comID[7],1,0])
;
;;;; Creation of the center and mask widget_texts.
;   temp    =widget_base(  others,/row)
;   temp_lab=widget_label( temp,value="Cx = ")
;   cxID    =widget_text(  temp,value=NUMtSTR(set.cx+1.0,2),/edit,/no_newline,uvalue=[-88,409,n],xsize=6,font=ft_b_bigger)
;   temp_lab=widget_label( temp,value="Cy = ")
;   cyID    =widget_text(  temp,value=NUMtSTR(set.cy+1.0,2),/edit,/no_newline,uvalue=[-88,409,n],xsize=6,font=ft_b_bigger)
;   temp    =widget_base(  others,/row)
;   temp_lab=widget_label( temp,value="Mask X:")
;   mxID    =widget_text(  temp,value=NUMtSTR(set.mx,2),uvalue=[-88,409,n],/edit,/no_newline,xsize=5,font=ft_b_bigger)
;   temp_lab=widget_label( temp,value=" Y:")
;   myID    =widget_text(  temp,value=NUMtSTR(set.my,2),uvalue=[-88,409,n],/edit,/no_newline,xsize=5,font=ft_b_bigger)
;   tmpdid  =widget_base(  others,/nonexclusive)
;   plotBtn =widget_button(tmpdid,value="Separate Symmetrical Sectors",uvalue=[-88,410,n])
;
;;;;;; All the widgets have been declared. The two-state buttons
;;;;;; are now set to their correct value, widget IDs are stored,
;;;;;; some widgets are made non-sensitive (the plot menu), and
;;;;;; the xmanager is called.
;   bid=sys_dep(      'DYNLAB',base,1)
;   widget_control,/realize,base
;   widget_control,set_button=options.plotZone,plotBtn
;   widget_control,set_button=options.logAspect,logBtn
;   widget_control,get_value=minID,mini
;   widget_control,get_value=drawID,result
;
;   if sans then begin & minID=long(benSans[7]) ;& cxID=long(benSans(8)) & cyID=long(benSans(9))
;   endif else put_logo
;
;   uv={menu:menuID,cx:cxID,cy:cyID,mx:mxID,my:myID,r1:r1ID,r2:r2ID,draw:drawID,mini:minID}
;   widget_control,set_uvalue=uv,varAccess.IDs
;   for i=1,4 do widget_control,sensitive=0,menuID[i]
;   int_numor  ,varAccess
;   int_miniPix,varAccess,0
;   widget_control,set_value="<- You must load W",comID[7]
;   if not sans then xmanager,desk_name(n,0),base,event_handler="lamp_event_parser",/just_reg,/no_block
;end
;
;pro ben_int_slider_set, id ,value
;;+
;;** ******************
;;** for sectors (also in bens)
;;-
;
;    widget_control,id,bad_id=ii, pro_set_value=''
;    widget_control,id,bad_id=ii, set_value=strtrim(round(float(value)*10),2)
;    widget_control,id,bad_id=ii, pro_set_value='ben_int_slider_set'
;end
;
;function ben_int_slider_get, id
;;+
;;******* ******************
;;** for sectors (also in bens)
;;-
;
;    widget_control,id,bad_id=ii, func_get_value=''
;    widget_control,id,bad_id=ii, get_value=value
;    widget_control,id,bad_id=ii, func_get_value='ben_int_slider_get'
;    return,float(value)/10.
;end

pro sans_sector
;+
;** ***********
;** Junk
;-
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_write_settings, filename
;+
;** *******************
;** *******************
;** User callable
;** save current sans_settings state for batch mode
;-
@lamp.cbk
common for_users,       a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z ; lamp

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl ; sans_settings
common c_sans_tofl    ; tof merged

common resume_iso     ; sans_reform_series
common resume_azi     ; sans_reform_series
common resume_trs     ; sans_reform_series

if n_elements(filename) ne 1 then begin
    filename = strupcase(inst_value)+'_Batch_'+(['Unk','Low','Middle','High'])[sans_supl[78]>0<3]+'.sav'
endif
lamp_message,'Saving '  +filename
SAVE,/variables,filename=filename,/compress

end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_read_settings, filename, success
;+
;**********************
;** User callable
;** restore a saved session for batch mode
;-
@lamp.cbk
common for_users,       a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z ; lamp

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl ; sans_settings
common c_sans_tofl    ; tof merged

common resume_iso     ; sans_reform_series
common resume_azi     ; sans_reform_series
common resume_trs     ; sans_reform_series

success=0
if n_elements(filename) ne 1 then begin & print,'!!! Bad filename ' & return & endif

lamp_message,'Restoring '  +filename
RESTORE, filename

RDSET, INST=inst_value
success=1
end
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************





;SANS_SETTINGS
;***************************************************************************************************************
;***************************************************************************************************************
;***************************************************************************************************************

pro sans_dying,ev
;+
;** **********
;** **********
;** sans dying
;-
@lamp.cbk
if lamp_b1 gt 0 then widget_control,lamp_b1,bad_id=ii,map=1 else exit
end

pro sans_gt_par ,plus ,get=get ,set=set ,sys=sys
;+
;** ***********
;** ***********
;** get or set PARi lamp global variables (avoid the use of lamp.cbk anywhere)
;-

@lamp.cbk
sys=lamp_sys
if arg_present(get) then ii=execute('get=par'+strtrim(sys+plus,2))
if arg_present(set) then ii=execute(    'par'+strtrim(sys+plus,2)+'=set')
end

pro sans_accu ,wka,rpa ,opr, wki,rpu ,newrpu=newrpu
;+
;** *********
;** *********
;** handle operations on Q vector results
;-
@lamp.cbk

if n_elements(opr) eq 0 then W_ACCU, accu=wka $  ;set accumulator to 0
else begin
     ii=execute('W'+wka+'=rpa.W') & ii=execute('X'+wka+'=rpa.X') ;W_ACCU works with workspaces !!!
     ii=execute('E'+wka+'=rpa.E') & ii=execute('N'+wka+'=1')
     if keyword_set(newrpu) then begin
     ii=execute('W'+wki+'=rpu.W') & ii=execute('X'+wki+'=rpu.X')
     ii=execute('E'+wki+'=rpu.E') & ii=execute('N'+wki+'=1')
     endif
     nQ=n_elements(rpa.X) & nX=nQ

     if opr eq '+' then W_ACCU, accu=fix(wka) , add=fix(wki)
     if opr eq '-' then W_ACCU, accu=fix(wka) , sub=fix(wki)
     if opr eq '/' then begin & Wa=0 & Ea=0 & Xa=0 & ii=execute('Wa=W'+wka+' & Ea=E'+wka+' & Xa=X'+wka)
                              Wi=0 & Ei=0 & Xi=0 & ii=execute('Wi=W'+wki+' & Ei=E'+wki+' & Xi=X'+wki)
                              idx=where(Wi ne 0.)  ; Xa & Xi SHOULD BE IDENTICAL ***
                              if n_elements(idx) ne nQ then begin
                                 Wa=Wa[idx] & Wi=Wi[idx] & Ea=Ea[idx] & Ei=Ei[idx] & Xa=Xa[idx] & Xi=Xi[idx]
                              endif
                              Ea=SQRT((ea/wi)^2+(ei*wa/wi^2)^2)
                              Wa=(Wa/Wi)>0. ; NaN>0 = 0
                              ii=execute('W'+wka+'=Wa & E'+wka+'=Ea & X'+wka+'=Xa')
                              endif
     if opr eq '*' then begin & Wa=0 & Ea=0 & Xa=0 & ii=execute('Wa=W'+wka+' & Ea=E'+wka+' & Xa=X'+wka)
                              Wi=0 & Ei=0 & Xi=0 & ii=execute('Wi=W'+wki+' & Ei=E'+wki+' & Xi=X'+wki)
                              Ea=SQRT((ea*wi)^2+(ei*wa)^2)
                              Wa=(Wa*Wi)>0. ; NaN>0 = 0  ,  Xa & Xi SHOULD BE IDENTICAL ***
                              ii=execute('W'+wka+'=Wa & E'+wka+'=Ea & X'+wka+'=Xa')
                              endif
                              
     ii=execute('nX=n_elements(W'+wka+')')
     if nX eq nQ then $
        ii=execute('rpa.W=W'+wka+' & rpa.X=X'+wka+' & rpa.E=E'+wka) $
     else $
        ii=execute('rpa={W:W'+wka+',X:X'+wka+',E:E'+wka+',TIT:rpa.TIT,NPT:nX,NTC:rpa.NTC,P:rpa.P,SUPL:rpa.SUPL}')

     numor=strtrim(long(rpu.NTC[0]),2) & CONF=rpa.TIT & bl=strpos(CONF,' ')
     CONF=strmid(CONF,0,bl) + ' ['+opr+numor+']' + strmid(CONF,bl,200) ;MODIFY TITLE
     rpa.TIT=CONF
endelse
end

function sans_settings_RegSheet,dum
;+
;******* **********************
;******* **********************
;** check if the sans spreadsheet is present or not
;-

return, ((xregistered('sans_sheet_gui',/noshow) gt 0) or (xregistered('reflecto_sheet_gui',/noshow) gt 0) or $
         (xregistered('difsans_sheet_gui',/noshow) gt 0))
end

pro sans_settings_swap, baset ,LMH=lmh ,get=get ,put=put ,reset=reset ,file=file ,take=strucT ,give=strucG ,waterbox=wbox ,forWater=forWn $
                      , shareSupl=idsupl, supl=supl, transm=transm, masq=masq ,no_toggles=no_toggles ,memswap=memswap ,nodata=nodata
;+
;** ******************
;** ******************
;** using memory, load current config and restore wanted config (among 3)
;-

forward_function   finefile
common c_sans_swapLMH, sans_struct_low, sans_struct_middle, sans_struct_high, sans_nodata_low, sans_nodata_middle, sans_nodata_high

if (not keyword_set(no_toggles)) and (not keyword_set(reset)) then $
if (n_elements(baset) gt 0) and (n_tags(sans_struct_low)+n_tags(sans_struct_middle)+n_tags(sans_struct_high) gt 3) then begin
  if n_tags(sans_struct_low)    le 1 then baset.savedLMH[0]=0 else $
  if baset.savedLMH[0] then sans_struct_low.history.mods   =baset.bases6[4:n_elements(sans_struct_low.history.mods)-3]    ;4:* share toggle buttons
  if n_tags(sans_struct_middle) le 1 then baset.savedLMH[1]=0 else $
  if baset.savedLMH[1] then sans_struct_middle.history.mods=baset.bases6[4:n_elements(sans_struct_middle.history.mods)-3] ;4:* share toggle buttons
  if n_tags(sans_struct_high)   le 1 then baset.savedLMH[2]=0 else $
  if baset.savedLMH[2] then sans_struct_high.history.mods  =baset.bases6[4:n_elements(sans_struct_high.history.mods)-3]   ;4:* share toggle buttons
endif else if n_elements(baset) gt 0 then baset.savedLMH=[0,0,0]

if  n_elements( lmh) eq 1   then  fifiswap=".sans_swap_"+lmh   else lmh=""
if  keyword_set(file)  then begin & fifiswap=fifiswap+".sav" & is=finefile(fifiswap,count=cnt) & endif else cnt=1

if (keyword_set(get) and (cnt gt 0)) then begin ;GET A CONFIG FROM FILE OR MEMORY and FILL GUI
                                          sans_settings_read,  baset, fifiswap ,memswap=memswap
                                          sans_settings_choice_water, baset, callflag='swap' ;BE SURE TO HAVE THE CORRECT WATER!!!
                                          endif
if (keyword_set(put))                then begin ;GET GUI INFO and STORE CONFIG IN FILE OR MEMORY
                                          sans_settings_choice_water, baset, callflag='swap' ;BE SURE TO HAVE THE CORRECT WATER!!!
                                          sans_settings_write, baset, fifiswap
                                          endif

if  keyword_set(reset)       then begin & sans_struct_low={bid:0} & sans_struct_middle=sans_struct_low & sans_struct_high=sans_struct_low
                                        sans_nodata_low={bid:0} & sans_nodata_middle=sans_struct_low & sans_nodata_high=sans_struct_low
                                        sans_settings_choice_water, baset, callflag='reset' & endif

if  arg_present(strucG) gt 0 then begin & if lmh eq "low"    then if keyword_set(nodata) then strucG=sans_nodata_low    else strucG=sans_struct_low
                                        if lmh eq "middle" then if keyword_set(nodata) then strucG=sans_nodata_middle else strucG=sans_struct_middle
                                        if lmh eq "high"   then if keyword_set(nodata) then strucG=sans_nodata_high   else strucG=sans_struct_high
                                  endif
if  n_elements( strucT) gt 0 then begin & if lmh eq "low"    then if keyword_set(nodata) then sans_nodata_low=strucT    else sans_struct_low   =strucT
                                        if lmh eq "middle" then if keyword_set(nodata) then sans_nodata_middle=strucT else sans_struct_middle=strucT
                                        if lmh eq "high"   then if keyword_set(nodata) then sans_nodata_high=strucT   else sans_struct_high  =strucT
                                  endif
if  n_elements( transm) gt 1 then begin & if baset.savedLMH[0] then sans_settings_struc,'sans_struct_low'   ,"TRANSMISSION",transm ;share transmission for all
                                        if baset.savedLMH[1] then sans_settings_struc,'sans_struct_middle',"TRANSMISSION",transm
                                        if baset.savedLMH[2] then sans_settings_struc,'sans_struct_high'  ,"TRANSMISSION",transm & endif

if  n_elements( masq)   gt 1 then begin & masqS=['']       & if n_elements(masq) gt 4 then begin & masqt=strtrim(string(long(masq)),2)
                                        for i=1,(size(masqt))[2]-1 do masqS=[masqS,'['+masqt[0,i]+','+masqt[1,i]+','+masqt[2,i]+','+masqt[3,i]+']']
                                        masqS=masqS[1:*] & endif
                                        if baset.savedLMH[0] then sans_settings_struc,'sans_struct_low'   ,"MASK",masqS   ;share masq for all
                                        if baset.savedLMH[1] then sans_settings_struc,'sans_struct_middle',"MASK",masqS
                                        if baset.savedLMH[2] then sans_settings_struc,'sans_struct_high'  ,"MASK",masqS & endif

if  n_elements( forWn)  ne 1 then forWn=-1
if  n_elements( wbox)   eq 4 then begin & if (forWn eq 1) and (wbox[2]*wbox[3] gt 0) then begin   ;share default water beam for all?
                                          water1=''
                                          if baset.savedLMH[1] then begin & water1=sans_struct_middle.water
                                                                          sans_struct_middle.XYFbeam[3:6]=wbox & endif
                                          if baset.savedLMH[0] then begin & water3=sans_struct_low.water
                                                                          if (water3 eq water1) or (water3 le ' ') then $
                                                                          sans_struct_low.XYFbeam[   3:6]=wbox & endif
                                          if baset.savedLMH[2] then begin & water2=sans_struct_high.water
                                                                          if (water2 eq water1) or (water2 le ' ') then $
                                                                          sans_struct_high.XYFbeam[  3:6]=wbox & endif
                                        endif
                                        sans_settings_choice_water, baset, callflag='wbox',  wbox=wbox & endif ;Dispatch correctly

if  n_elements( supl)   gt 1 then begin & if forWn gt 0 then begin ;share water info (ok # waters).
                                          if baset.savedLMH[0] then if forWn eq 3 then $ ;!! un_necessary
                                                                    sans_struct_low.SUPL[   idsupl]=supl[idsupl]
                                          if baset.savedLMH[2] then if forWn eq 2 then $ ;!! un_necessary
                                                                    sans_struct_high.SUPL[  idsupl]=supl[idsupl]
                                        endif else begin
                                          if baset.savedLMH[0] then sans_struct_low.SUPL[   idsupl]=supl[idsupl] ;share toggles for all
                                          if baset.savedLMH[1] then sans_struct_middle.SUPL[idsupl]=supl[idsupl]
                                          if baset.savedLMH[2] then sans_struct_high.SUPL[  idsupl]=supl[idsupl]
                                        endelse
                                  endif
end

pro sans_settings_choice_water, baset, callflag=flag, wbox=wbox
;+
;** **************************
;** **************************
;** handle the choice of the water confif (default , high , low)
;-

common c_sans_settings
common c_sans_water_all, bididx, bidD, bid1, bid3, WboxD, Wbox1, Wbox3, WatD,WceD,WcdD, bbid1, bbid2, bbid3

;DEFAULT row1         HIGH row2             LOW row3            MASK
;bases3( 0)=Water ,   bases3(29)=Water2 ,   bases3(30)=Water3 , bases3(25)=Water
;bases3( 1)=WaterC ,  bases3(31)=Water2C ,  bases3(32)=Water3C
;bases3( 5)=WaterCd , bases3(33)=Water2Cd , bases3(34)=Water3Cd

;callflag= 'sync'  -> call from sans_sheet_sync
;          'swap'  -> call from sans_settings_swap (put)
;          'swap'  -> call from sans_settings_swap (get)
;          'reset' -> call from sans_settings_swap (reset)
;          'wbox'  -> call from sans_settings_swap (water beam)
;          'conf'  -> call from sans_settings_event(show config [90])

if n_elements(baset) eq 0 then return
if flag eq 'reset' then begin & WboxD=0 & Wbox1=0 & Wbox3=0 & WatD='' & WceD='' & WcdD=''
                              widget_control,baset.bases3[0] ,set_value=''
                              widget_control,baset.bases3[29],set_value=''
                              widget_control,baset.bases3[30],set_value='' & return & endif

curLMH=baset.curLMH     ; 0,1,2,3
curWATER=baset.curWATER ;   1,2,3 USED FOR WBOX ONLY

;GET WATER1(default)  WATER2(high)  WATER3(low)              **SET IN SHEET_SYNC**
widget_control,baset.bases3[ 0],get_value=Wat1 & if Wat1 le ' ' then begin & WboxD=0 & endif ;Default water run
widget_control,baset.bases3[29],get_value=Wat2 & if Wat2 le ' ' then begin & Wbox3=0 & endif ;High
widget_control,baset.bases3[30],get_value=Wat3 & if Wat3 le ' ' then begin & Wbox1=0 & endif ;Low

;GET E-C-WATER1(default)  E-C-WATER2(high)  E-C-WATER3(low)  **SET IN SHEET_SYNC**
widget_control,baset.bases3[ 1],get_value=Wce1 & widget_control,baset.bases3[ 5],get_value=Wcd1 ;Row1
widget_control,baset.bases3[31],get_value=Wce2 & widget_control,baset.bases3[33],get_value=Wcd2 ;Row2
widget_control,baset.bases3[32],get_value=Wce3 & widget_control,baset.bases3[34],get_value=Wcd3 ;Row3

if n_elements(WatD) ne 1 then begin & WatD=Wat1 & WceD=Wce1 & WcdD=Wcd1 & endif ;Init common if empty

if Wat1 gt ' ' then begin & WatD=Wat1 & WceD=Wce1 & WcdD=Wcd1 & endif else $    ;Keep always default Water in common.
if Wat2 gt ' ' then begin & WatD=Wat2 & WceD=Wce2 & WcdD=Wcd2 & endif else $    ;
                    begin & WatD=Wat3 & WceD=Wce3 & WcdD=Wcd3 & endelse         ;Case user has a lowQ Water !!!

WatD=Wat1 & WceD=Wce1 & WcdD=Wcd1; Anyway, Isabelle says no default

if flag eq 'wbox' then begin ; Keep Water beam box in common.
   if curWater eq 1 then begin & WboxD=wbox ;Def
      if (Wat2 le ' ') then    Wbox3=wbox ;bof!!!
      if (Wat3 le ' ') then    Wbox1=wbox & endif
   if curWater eq 2 then       Wbox3=wbox ;High
   if curWater eq 3 then       Wbox1=wbox ;Low
endif

;PUT CORRECT WATER RUNS IN CURRENT SAMPLE SCATTERING SET **MANDATORY AFTER GET SWAPPING**
if (curLMH gt 0) then begin & ok=0
          if (curLMH eq 1) and (Wat3 gt ' ') then begin & ok=1     ;LOW  water is defined in row3
                widget_control,baset.bases3[ 0],set_value=Wat3 & widget_control,baset.bases3[25],set_value=Wat3 ;Mask=25
                widget_control,baset.bases3[ 1],set_value=Wce3 & widget_control,baset.bases3[ 5],set_value=Wcd3
                if n_elements(Wbox1) eq 4 then sans_beam[3:6]    =Wbox1
          endif else $
          if (curLMH eq 3) and (Wat2 gt ' ') then begin & ok=1     ;HIGH water is defined in row2
                widget_control,baset.bases3[ 0],set_value=Wat2 & widget_control,baset.bases3[25],set_value=Wat2 ;Mask=25
                widget_control,baset.bases3[ 1],set_value=Wce2 & widget_control,baset.bases3[ 5],set_value=Wcd2
                if n_elements(Wbox3) eq 4 then sans_beam[3:6]    =Wbox3
          endif
          if not(ok) then begin  ;Use Default water for the moment
                widget_control,baset.bases3[ 0],set_value=WatD & widget_control,baset.bases3[25],set_value=WatD ;Mask=25
                widget_control,baset.bases3[ 1],set_value=WceD & widget_control,baset.bases3[ 5],set_value=WcdD
                if n_elements(WboxD) eq 4 then sans_beam[3:6]    =WboxD
          endif
endif

case flag of
'sync':
'wbox':
'swap':   sans_settings_runs, baset ,[9,6 ,10,baset.bases3[0 ]],/noread,/haveall ;Be sure to have correct distance Water before/after swap !!
'conf':   sans_settings_runs, baset ,[9,6 ,10,baset.bases3[0 ]],/noread          ;Be sure to have correct distance Water showing a config
else:
endcase

end

pro sans_settings_struc, struc, TAG, val
;+
;** *******************
;** *******************
;** modify size of a structure
;-

common c_sans_swapLMH, sans_struct_low, sans_struct_middle, sans_struct_high, sans_nodata_low, sans_nodata_middle, sans_nodata_high

      sv=n_elements(val)
      sz=0L & ii=execute('sz=n_elements('+struc+'.'+TAG+')')
      if sz ne sv then begin
         elsa = '' & tlist=[''] & ii=execute('tlist=tag_names('+struc+')') & vg=''
         for k=0,n_elements(tlist)-1 do begin
          CASE tlist[k] of
           TAG:       elsa =elsa+vg+tlist[k]+':val'
           ELSE:      elsa =elsa+vg+tlist[k]+':'+struc+'.'+tlist[k]
          ENDCASE
          vg=','
         endfor
                 ii=execute(struc+'={'+elsa+'}')
      endif else ii=execute(struc+'.' +tag+'=val')
end

pro sans_settings_beamST, baset, Sbeam=srun, Tbeam=trun, get_Sbeam=gsrun, get_Tbeam=gtrun, Tog_STbeam=Tog_STbeam, $
                                 FLXbeam=FLXbeam
;+
;** ********************
;** ********************
;** center and tranmission beam use the same input (confusion when starting the project)
;** this procedure help to swap correctly between both direct beam
;-

;EQUIVALENCE with sans_settings
; -----------------------------
;          bases3 =[bwatr ,bwatre,bsampl,bsampe,bcadmi,bcadmw,  blabw,blabwe,blabs,blabse,blabc ,blabcw,$   ;0 ->11 sample
;                   awatr ,awatre,asampl,asampe,abeamS,abeamW,  alabw,alabwe,alabs,alabse,alabbS,alabbW,$   ;12->23 transmission
;                                 msampl,mwater,                             mlabs,mlabw ,confus,$          ;24->28 mask ,center
;                   Wwatr2,Wwatr3,Wwate2,Wwate3,Wcadc2,Wcadc3,  Wlabw2,Wlabw3,Wlabe2,Wlabe3,Wlabc2,Wlabc3,$ ;29->40 water alternatives
;                                                      SampFX,                                     SlabFX,$ ;41->42 FLUX  alternatives

widget_control,baset.bases3[28],get_uvalue=confuC ;CONFUS: Center<-->Transmisssion beam use same inputs !!!
if n_elements(srun) eq 1 then if confuC.T eq 'Ctr' then begin & confuC.V=srun & widget_control,baset.bases3[28],set_uvalue=confuC
                                                   endif else                 widget_control,baset.bases3[16],set_value =srun

if n_elements(trun) eq 1 then if confuC.T eq 'Trs' then begin & confuC.V=trun & widget_control,baset.bases3[28],set_uvalue=confuC
                                                   endif else                 widget_control,baset.bases3[16],set_value =trun

if arg_present(gsrun)    then if confuC.T eq 'Ctr' then begin & gsrun=confuC.V
                                                   endif else                 widget_control,baset.bases3[16],get_value =gsrun

if arg_present(gtrun)    then if confuC.T eq 'Trs' then begin & gtrun=confuC.V
                                                   endif else                 widget_control,baset.bases3[16],get_value =gtrun

if (n_elements(srun) eq 1) or (n_elements(trun) eq 1) then widget_control,baset.bases1[6],set_uvalue=baset ;No need!!!


if (n_elements(Tog_STbeam) eq 1) then if Tog_STbeam eq confuC.T then begin
            if Tog_STbeam  eq 'Trs' then begin & newtog='Ctr' & v1=5 & v2=9 & endif else begin & newtog='Trs' & v1=9 & v2=5 & endelse
            widget_control,baset.bases3[16],get_value =tmpV,get_uvalue=tmpS         & widget_control,baset.bases3[22],get_uvalue=tmpL
            confuC.S[2]=v1 & tmpS[2]=v2 ;NOOP !!!
            widget_control,baset.bases3[28],set_uvalue={V:tmpV,S:tmpS,T:newtog,L:tmpl} ;set new CONFU
            widget_control,baset.bases3[16],set_value =confuC.V,set_uvalue=confuC.S & widget_control,baset.bases3[22],set_uvalue=confuC.L
endif
end

pro sans_settings_lampies, SETpth=SETpth, GETpth=GETpth
;+
;** *********************
;** *********************
;** ** set or get the path_for_online lamp global variable (avoid use of lamp.cbk)
;-
@lamp.cbk
if arg_present(GETpth) then GETpth=path_for_online
if arg_present(SETpth) then path_for_online=SETpth
end

pro sans_settings_event, ev
;+
;** *******************
;** *******************
;** event handle for sans settings, also called in sans_sheet synchronisation
;-

forward_function sans_center, sans_cor, sans_transm, sans_radial, sans_read_graspr, sans_read_gt, sans_sigmaQ

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl
common c_sans_tofl,     TofFlux, LambdaVq
common c_sans_cur_run , cur_run , cur_mask ,cur_wn, cur_log, cur_menu, cur_cxy, cur_fct, cur_zoom, cur_eff
common c_sans_cur_aXY , cur_aX  , cur_aY   ,cur_SamCor
common c_lamp_par

COMMON COLORS, R_orig, G_orig, B_orig, R_curr, G_curr, B_curr

CATCH,stat & if stat ne 0 then begin & lamp_error & return & endif

widget_control,ev.top,get_uvalue=baset  &  bmess=baset.bases1[0]
widget_control,ev.id ,get_uvalue=uv
widget_control,bmess ,set_value ='   '
FX  =long(sans_supl[30])
idxp=sans_supl[80:*]

if n_elements(uv) gt 0 then begin
;TOP BASE RESIZING
if ev.id eq baset.bases1[6] then begin
   new_tbl=[ev.X,ev.Y]
   min_tbl=cur_fct[5:6]
   cur_tbl=cur_fct[7:8]
   act_drw=cur_fct[3:4]
   if (total(abs(new_tbl-cur_tbl)) gt 30) then begin & ok=0
     if (total(new_tbl-cur_tbl) gt 0) and (act_drw[0] lt act_drw[1]) then ok=1  ;Want to increase
     if (total(new_tbl-cur_tbl) lt 0) and (act_drw[0] gt act_drw[1]) then ok=2  ;Want to decrease
     if ok gt 0 then begin
        widget_control,ev.top,update=1,/hourglass
        win =((!version.os_family ne 'Windows') and (!version.os ne 'darwin'))
        wins= (!version.os_family ne 'Windows')
        cur_fct[3:4]=reverse(act_drw)
        if ok eq 1 then widget_control,ev.top,scr_Xsize=min_tbl[0]+cur_fct[3]-cur_fct[4] $ ;increase
                                             ,scr_Ysize=min_tbl[1]+cur_fct[3]-cur_fct[4]
        widget_control,baset.bases1[20],draw_Ysize=cur_fct[3],draw_Xsize=cur_fct[3]           ;large view
        widget_control,baset.bases1[21],draw_Ysize=cur_fct[3]                                 ;bar
        widget_control,baset.bases1[22],draw_Ysize=cur_fct[3],draw_Xsize=cur_fct[3]+94+win*30 ;plot zone

        widget_control,baset.bases1[23],Ysize     =cur_fct[3],Xsize     =cur_fct[3]           ;sectors scrolled
        DR=widget_info(baset.bases1[23],/geometry)
        widget_control,baset.bases1[23],set_draw_view=[((DR.draw_Xsize-DR.Xsize)/2)>0,((DR.draw_Ysize-DR.Ysize)/2)>0]

        widget_control,baset.bases1[24],draw_Ysize=cur_fct[3]                                 ;bar
        widget_control,cur_zoom[1],draw_xsize=cur_fct[3]*cur_zoom[0],draw_ysize=cur_fct[3]*cur_zoom[0];pixmap

        strj=widget_info(baset.bases1[8],/geometry) ;!!!
        if wins then newx=strj.Xsize else newx=strj.scr_xsize+2*strj.margin+strj.xpad
        if ok eq 1 then widget_control,ev.top,scr_Xsize=newx                                  ;increase
        if ok eq 2 then widget_control,ev.top,scr_Xsize=min_tbl[0],scr_Ysize=min_tbl[1]       ;decrease
        widget_control,ev.top,update=1

        sans_settings_mask, baset
        stri=widget_info(ev.top,/geometry)
        cur_fct[7:8]=[stri.scr_xsize+2*stri.margin , stri.scr_ysize+2*stri.margin]
     endif else widget_control,ev.top,scr_Xsize=cur_tbl[0],scr_Ysize=cur_tbl[1]
  endif    else widget_control,ev.top,scr_Xsize=cur_tbl[0],scr_Ysize=cur_tbl[1]

endif else case uv[0] of

 ;cur_menu(0)  Ranges/box input-> 0:Center Rg 1:Transm Rg 2:Mask box/beam 3:Scatt data
 ;             6:Batch macros 7:Sectors 9:Config 3buts 19:we are in LIST of RESULTS
 ;cur_menu(1)  Views 4:2d view 5:Averaging 8:Sector view
 ;cur_menu(2)  Averaging option 2:2PI Rad 4:2PI Azim 5:Sec tot sym 6:Sec tot asym 7:Sec rad sym 8:Sec rad asym
 ;cur_menu(3:5)Hide mask if 1. 3:Sample 4:Detector 5:Water
 ;cur_menu(6)  While in Mask 1:Sample beam 2:Water beam 3:Detect all conf 4:Detect cur conf
 ;cur_menu(7)  Workspace number i.e. 3,12,7,8,17,18
 ;cur_menu(8)  Enable efficiencies correct =1
 ;cur_menu(9)  cur_menu(2) curlist  71->List of Results 77->Restoring Results !!!
 ;cur_menu(10) data input-> 0:Center menu 1:Transmition menu 2:Mask menu 3:Data menu -1:Averaging

;SELECT A DATA PATH
1: begin & case uv[1] of
             2:begin ;PATH EXPLORER
                     result=dialog_pickfile(path='',/directory,resource_name='lamptouch')
                     if result gt ' ' then begin & sans_settings_lampies,SETpth =result
                        widget_control,bmess,set_value ="Data path: "+result  & endif
               end
             3:begin ;PATH LAMP
                     widget_control,baset.bases9[4],map=1,iconify=0,xoff=1,yoff=1 & ii=xregistered('LAMP')
                     widget_control,bmess,set_value ="Data path is taken from Lamp"
               end
             4:begin ;SHOW PATH
                     sans_settings_lampies,GETpth=result
                     if result eq '' then CD,current=result
                        widget_control,bmess,set_value ="Data path: "+result
                        re=dialog_message(result,/information,title="Path for the Data",dialog_parent=ev.top)
               end
             5: begin ;SET WORKING PATH
                     cd,current=mee
                     pptthh=dialog_pickfile(/directory,resource_name='lamptouch')
                     if pptthh gt ' ' then begin & P_SET_PATH,pptthh & mee=pptthh & endif
                     widget_control,bmess,set_value ="Working path: "+mee
                end
             else:
             endcase
         end
;RESTORE CONFIG FILE
2: begin & if (uv[1] ge 1) and (uv[1] le 3) then begin ;SWAP
            if sans_settings_RegSheet() then begin   ;Sans_sheet present?
               sans_sheet_receive, topget=topSheet
               widget_control,topSheet,get_uvalue=sbaset
            endif else topSheet=0L

            if topSheet gt 0 then begin ;Let sans_sheet do the work because it knows what water to determine.
               sans_sheet_receive, topget=topSheet, LMH=(["low","middle","high"])[uv[1]-1]
               widget_control,ev.top  ,get_uvalue= baset
            endif else begin
            
              if baset.curLMH gt 0 then begin          ;Update current first
                 sans_settings_swap, baset ,/put, LMH=(["low","middle","high"])[baset.curLMH-1]
                 baset.savedLMH[baset.curLMH-1]=1
              endif

              if baset.savedLMH[uv[1]-1] then begin    ;We can Swap
                 baset.curLMH=(uv[1])
                 sans_settings_swap, baset ,/get,LMH =(["low","middle","high"])[uv[1]-1],/memswap
              endif else widget_control,ev.id,set_button=0 ;CAN't Swap
              
              widget_control,ev.top  ,set_uvalue= baset
            endelse

            widget_control,ev.top,base_set_title=(["Conf 1","Conf 2","Conf 3"])[uv[1]-1]+" is current"

            cur_menu[0]=9 ;!!!!!
            if cur_menu[0] eq 9 then $
               ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:baset.bases5[27],TOP:ev.top,HANDLER:0L,SELECT:1}')

         endif else sans_settings_read, baset            ;restore from a file
         end
;EXIT
3: begin & if uv[1] eq 0 then                  widget_control,ev.top,/destroy   else $
         if uv[1]-1 ne baset.idxi then begin & widget_control,ev.id,get_value=cinst
                                             widget_control,ev.top,/destroy & wait,.3
                                             widget_control,/hourglass
                                             sans_settings,instrument=cinst & endif
         return & end
;SAVE CONFIG FILE
4: begin & widget_control,/hourglass
        ;widget_control,bmess ,set_value ='Reading data files...' & kperr=!error_state.msg
        ;sans_settings_runs,  baset ,[9,6,0,baset.bases3(0:5)], /HaveAll
        ;if kperr eq !error_state.msg then widget_control,bmess ,set_value ='   '
         if uv[1] eq 1 then begin & sans_settings_swap, baset ,/put ,LMH="low"    & baset.savedLMH[0]=1 & baset.curLMH=1 ;in memory
         endif else $
         if uv[1] eq 2 then begin & sans_settings_swap, baset ,/put ,LMH="middle" & baset.savedLMH[1]=1 & baset.curLMH=2 ;in memory
         endif else $
         if uv[1] eq 3 then begin & sans_settings_swap, baset ,/put ,LMH="high"   & baset.savedLMH[2]=1 & baset.curLMH=3 ;in memory

         endif else sans_settings_write,baset ;save to a file
         end
;CENTER MODIFIED IN CONFIG
5:       sans_settings_mask , baset

;COLLIM MODIFIED IN CONFIG
6: begin & collim=strsplit(/EXTRACT,baset.collim,',')
         if uv[1] then begin & baset.colidx=ev.index & baset.colval=float(collim[ev.index])
         endif    else begin & sans_settings_text,ev.id,farr,sarr
                             colidx=where(round(float(collim)*10.)/10. eq round(farr*10.)/10.) & colidx=colidx[0]>0
                             baset.colidx=colidx & baset.colval=farr & endelse
         sans_supl[4]=baset.colval
         end
;XYBEAM MODIFIED IN CONFIG
7:       sans_settings_mask , baset

;LARGE VIEW
8: begin & sans_settings, refresh=baset & if cur_menu[1] eq 5 then cur_menu[1]=4 & k=cur_menu[1]
         widget_control,baset.bases1[7],map=0    & widget_control,baset.bases1[8],map=1
         widget_control,baset.bases5[k],map=1    & if k ne 5 then $
                                                   widget_control,baset.bases5[5],map=0
         if sans_beam[2] eq 1 then txtf='rectang' else txtf='ellipse'
         widget_control,baset.bases1[13],set_value=    txtf
         widget_control,baset.bases1[12],set_value=' '
         end
;READ A DATA FILE
9: begin & widget_control,/hourglass & if uv[3] eq 0 then uv[3]=ev.id
         widget_control,baset.bases3[uv[1]],get_uvalue=sdata
         curTRS=0.
         if sans_settings_RegSheet() then sans_sheet_receive, gettrans=sdata.opr, mess=curTRS
         baset.curSAMtrs=curTRS
         sans_settings_runs , baset ,uv
         end
;MODIFIED WAVL, DISTANCE, OFFSET, ANGLE,
10:begin & sans_settings_text,ev.id,farr,sarr      & sans_supl[uv[1]]= farr[0]           & end

;MODIFIED Thickness WATER, SAMPL
11:begin & sans_settings_text,ev.id,farr,sarr      & sans_supl[uv[1]]= farr[0] & if uv[2] gt 0 then widget_control,uv[2],sens=1
         if ev.id eq baset.bases7[4]  then widget_control,baset.bases2[20],set_value=sarr[0] ;thin water
         if ev.id eq baset.bases2[20] then widget_control,baset.bases7[4] ,set_value=sarr[0]
         if ev.id eq baset.bases7[5]  then widget_control,baset.bases2[21],set_value=sarr[0] ;thin sample
         if ev.id eq baset.bases2[21] then widget_control,baset.bases7[5] ,set_value=sarr[0]
         if uv[1] eq 10 then sans_trans[0,1]   =farr[0]
         if uv[1] eq 11 then sans_trans[2,1]   =farr[0]
         if uv[1] eq 11 then if n_elements(sans_trans[*,1]) gt 4 then $
                             sans_trans[4:*,1] =farr[0] ;care swap!!! sans_supl,sans_trans
         end

;MODIFIED ATTENUATOR, TAU, XsectW
12:begin & sans_settings_text,ev.id,farr,sarr      & sans_supl[uv[1]]= farr[0]           & end

;EXPORT FILES
13:begin & pth =baset.path & parent=baset.bases1[6] & widget_control,baset.bases1[3],get_value=instru

         widget_control,/hourglass & gpth=''
         ids =strpos(instru,'<') & if ids[0] ge 0 then strput,instru,'_',ids[0]
         ids =strpos(instru,'>') & if ids[0] ge 0 then strput,instru,'_',ids[0]

         if uv[1] eq 6 then filext = ".png" else if uv[1] eq 7 then filext = ".dat" else filext = ".txt"
         if uv[1] eq 6 then begin & PNG=1 & uv[1] = 3 & endif else PNG = 0 ; PNG corrected data output

         if uv[1] eq 4 then begin ;EXPORT THE MASK
           xy0=sans_ctr-1 & if xy0[0] lt 0 then xy0[0]=xy0[0]+2. & if xy0[1] lt 0 then xy0[1]=xy0[1]+2.

           sans_settings_getmask,[2,sans_masq[2],sans_masq[3]],xy0 , M & Ms=size(M)
           M=M+34 &  idx=where(M eq 34) & if idx[0] ge 0 then M[idx]=M[idx]+12 & M=byte(M)
           TAKE_DATP,dp,W=9  & snumor='000'
           if n_elements(dp.P) gt idxp[6] then snumor=strtrim(string(long64(dp.P[idxp[6]])),2)
           defext='.msk'
           result=dialog_pickfile(path=pth,get_path=gpth,file=instru+'_'+snumor+defext,filter='*'+defext ,$
                  default_ext=defext,dialog=parent,/write,/overwrite_prompt,resource_name='lamptouch')
           if result ne '' then begin
              file =strmid(result,strlen(gpth))
              title=snumor+' '+strcompress(dp.w_tit) & np=strtrim(string(Ms[1]),2)
              on_ioerror,mismsq & u=-1 & mess='!!! Unable to write '+file
              openw ,u,result,/get_lun
              printf,u,title,','+stime(dum),format='(A20,A21)'    & printf,u,n_elements(M),format='(I8)'
              for i=0L,Ms[2]-1 do printf,u,string(M[*,i]),format='(A'+np+')'    ;PRINT SYMBOL MASK

              printf,u,(size(sans_masq))[2],format='(I8)' & printf,u,sans_masq ;PRINT ARRAY MASK
              mess=file+' is saved'
              mismsq:if u gt 0 then free_lun,u
              widget_control,bmess,           set_value=mess
              widget_control,baset.bases1[12],set_value=mess
              if sans_settings_RegSheet() then sans_sheet_receive, msk=file
           endif

         endif else begin         ;EXPORT RESULT DATA
          wk=uv[2] & wks=strtrim(long(wk),2)
          TAKE_W,tmpcor,W=wk & sz89=size(tmpcor) & TAKE_DATP,datp,W=wk

         ;uv(1) = 0 export gnum   integration results
         ;      = 1 export 1Dtext integration results, 4 columns
         ;      = 2 export tnum   corrected data 2D
         ;      = 3 export 2Dtext corrected data 2D, columns
         ;      = 6 export PNG    corrected data 2D, images
         ;      = 7 export NIST   corrected data 2D, list of qx, qy, I, error
         
          if (uv[1] eq 0) or (uv[1] eq 2) or (sz89[0] gt 1) then $ ;Ron or 2D
              widget_control,baset.bases3[8],get_uvalue=sdata

          if (n_tags(sdata) gt 1) then pr=sdata.par else pr=''
          if (uv[1] eq 0) or (uv[1] eq 2) then $ ;Only for sans_save_gt ***
              pZ=sdata.datp.Z ;INDEX USED TO FIND TRANSMISSION AND THICKNESS IN sans_trans ***

          if (sz89[0] eq 1) or (sz89[0] eq 2) or (sz89[0] eq 3) then begin ;if size is correct
          
           if sz89[0] eq 1  then nf=1 else if sz89[0] eq 2  then nf=sz89[2] else nf=sz89[3]
           if sz89[0] eq 2  then if (uv[1] eq 2) or (uv[1] eq 3) then nf=1 ;we have only one 2D to export
           
           if sz89[0] eq 1  then pr=strcompress(datp.w_tit)
                                 pr=strcompress(pr,/remove_all)

           if ((size(datp.pv))[0] eq 2) and (nf gt 1) then snumor=reform(strtrim(string(long64(datp.PV[idxp[6],*])),2)) $
                                                      else snumor=       strtrim(string(long64(datp.P[ idxp[6]<(n_elements(datp.P)-1)])),2)
           if uv[1] eq 1 then begin
              snumor=congrid([snumor],nf,/center) ;1D text files (Numors by Sectors)
              pr    =congrid([pr]    ,nf,/center) ;and sectors in datp.y
           endif
           if n_elements(snumor) eq nf then okn=nf-1 else okn=0 ; Check for integrity
           if n_elements(pr)     eq nf then okt=nf-1 else okt=0 ; 

           if (uv[1]  eq 1) or (uv[1] eq 3)   then begin & filter='*.column'      & defext= '.column'
                                                         result='2Dcolumn file' & endif
           if (uv[1]  eq 0) or (uv[1] eq 2)   then begin & filter='*.nil'         & defext= '.nil'
                                                         result='g/t file'      & endif
           if (uv[1]  eq 7) then begin & filter='*.dat' & defext= '.dat' & result='NIST file'      & endif

           if (nf ne okn+1) and (uv[1] ne 0) then begin & uv[1]=5 ;??? should not happend
             result=dialog_pickfile(path=pth,get_path=gpth,file=instru+'_result'+defext,filter=filter ,$
                    default_ext=defext,dialog=parent,/write,/overwrite_prompt,resource_name='lamptouch')
           endif
           if result ne '' then begin
              file =strmid(result,strlen(gpth))
              Fincr=baset.bases6[17]
             ;if (nf ne okn+1) then Fincr=1 ;to see!
       
              if (uv[1] eq 5)  then begin & write_lamp,result,W=wk,format='column' & endif ; .nil files
              
              W_store,W=29 ;code below and sans_saveQs are using W29
              
             ;Write file of Qvalues if not there for 2D outputs.
              if not PNG then $
              if (uv[1] eq 2) or (uv[1] eq 3) then begin
                  sans_save_Qs, instru+'_2D_Qvalues', [sz89[1],sz89[2]], getQx=Qx , getQy=Qy
                  mod_datp, datp,'X',Qx
                  mod_datp, datp,'Y',Qy ;  Do not retrieve Qx Qy in W18!!!
                  give_datp,datp, W=wk  &  endif

              if (uv[1] eq 1) or (uv[1] eq 3)  then begin   ; lamp 1D,2D column files, PNG 2d

                                 if  uv[1] eq 1  then Dim='1D' else Dim='2D'
                                 if (uv[1] eq 1) and (n_elements(datp.y) eq nf) then begin
                                     strY=strtrim(round(datp.y),2)
                                     if nf gt 1 then if strY[0] eq strY[nf-1] then $
                                            strY=strtrim(lindgen(nf),2)
                                 endif else strY=strtrim(lindgen(nf),2)

                                 for i=0L,nf-1 do begin & is=strtrim(i,2)
                                     if nf eq 1 then iss='_W'+wks else iss='_'+is
                                     result=instru+'_'+Dim+'_'+snumor[i<okn]+'_'+strY[i]+filext
                                     file  =result
                                     if (uv[1] eq 1) then XICUTE,'W29=W'+wks+'[*,'  +is+']' $
                                                     else XICUTE,'W29=W'+wks+'[*,*,'+is+']'
                                     tit29=pr[i<okt]
                                     XICUTE,"W_tit[29]='" + tit29+ "'"
                                     if not PNG then write_lamp,result,W=29,format='column'
                                     if     PNG then begin
                                       SEE,W=29,/beside,/PNG
                                       file_move, 'lamp.png' , result , /overwrite
                                      ;write_lamp,'snapshot_'+result,W=29,format='png'
                                      ;XICUTE,"other_tit[29]='Numor " + snumor[i<okn] + "'"
                                      ;SEE,W=29,/below,/PNG
                                      ;file_move, 'lamp.png' , 'small_'+result , /overwrite

                                     endif
                                 endfor
              endif
              if (uv[1] eq 0) or (uv[1] eq 2) then begin ; G or T files
                                 sans_save_gt,tmpcor,datp=datp, par=pr, pZ=pZ, Fincr=fincr, Inst=instru
                                 endif

              if uv[1] eq 7 then begin ; NIST 2D files
                Dim = '2D' & strY=  strtrim(lindgen(nf),2)
                ; Get Qx and Qy values
                sans_save_Qs, 'tmp', [sz89[1],sz89[2]], getQx=Qx , getQy=Qy, /nowrite
                for i=0L,nf-1 do begin
                  is=strtrim(i,2)
                  file = instru + '_' + Dim + '_' + snumor[i<okn] + '_' + strY[i] + filext
                  XICUTE, 'W29=W'+wks+'[*,*,'+is+']'
                  TAKE_W, data, W=29
                  TAKE_DATP, datp, W=29
                  sans_save_NIST, file, qx, qy, data, datp.e
                endfor
              endif
              W_restore,W=29
              widget_control,bmess,           set_value=file+' is saved'
              widget_control,baset.bases1[12],set_value=file+' is saved'
           endif
          endif else if sz89[0] ne 0 then begin 
              widget_control,bmess,           set_value='Wk is a bad dimensional result !'
              widget_control,baset.bases1[12],set_value='Wk is a bad dimensional result !' & endif
         endelse
         end

;DO FIELD INPUT
14:begin & widget_control,ev.id,get_value=opr     & XICUTER,opr[0]
         if uv[1] then sans_settings,refresh=baset & end

;IMPORT RESULT FILES G-T
15:begin & pth =baset.path & parent=baset.bases1[6] & widget_control,baset.bases1[3],get_value=instru
         ids =strpos(instru,'<') & if ids[0] ge 0 then strput,instru,'_',ids[0]
         ids =strpos(instru,'>') & if ids[0] ge 0 then strput,instru,'_',ids[0]
         gpth='' & filter='g*.*0*'
         result=dialog_pickfile(path=pth,get_path=gpth,filter=filter,dialog=parent,/read,resource_name='lamptouch')
         if result ne '' then XICUTER,"W25=sans_read_gt('"+result+"')"
   end
;IMPORT MASK FILE
16:begin & pth =baset.path & parent=baset.bases1[6] & widget_control,baset.bases1[3],get_value=instru
         ids =strpos(instru,'<') & if ids[0] ge 0 then strput,instru,'_',ids[0]
         ids =strpos(instru,'>') & if ids[0] ge 0 then strput,instru,'_',ids[0]
         gpth='' & filter=instru+'*.msk'
         result=dialog_pickfile(path=pth,get_path=gpth,filter=filter,dialog=parent,/read,resource_name='lamptouch')
         if result ne '' then begin
            XICUTER,"sans_read_mask,'"+result+"'" & sans_settings , refresh=baset
            file =strmid(result,strlen(gpth))
            if sans_settings_RegSheet() then sans_sheet_receive, msk=file
           ;sans_settings_swap,baset,masq=(sans_masq) ; current or all configs!!!
         endif
   end
;CHANGE FIRST INDEX 0 OR 1
17:begin & sans_supl[30]=-(sans_supl[30]+1) & sans_settings , refresh=baset & end

18:begin & if uv[1] eq 1 then p_layout,prox='sans_center' ;Unused
         if uv[1] eq 2 then p_layout,prox='sans_mask'   ;Unused
         if uv[1] eq 3 then p_layout,prox='sans_transm' ;Unused
         end
;MOUSE DRAW EVENTS
19:begin  ; uv1=1 for large square view  uv2=2 for Bens sectors else uv2=1
          ; uv1=2 for plot integration view
   curc =[ev.X,ev.Y]
   kpwin=!D.window
   if (cur_zoom[0] gt 1) and  (cur_menu[0] le 2) then begin              ;prevent clipboard while zooming
       if ev.type eq 0 then if ev.press   eq 4 then ev.press  =2         ;and no middle button available
       if ev.type eq 1 then if ev.release eq 4 then ev.release=2 & endif ;so right stand for middle...

   if (ev.type eq 0) and (cur_menu[1] eq 4) and  (cur_menu[0] le 2)  then $
     if ((ev.press eq 1) and (cur_zoom[0] eq 1)) or (ev.press eq 2)  then begin
                  ;we are in Center or Transmission to define XYranges: cur_menu(0)=0,1
                  ;we are in Mask to define Box or beam sample: cur_menu(0)=2
                   wset,baset.bases1[9]
                   p_zoom,curc[0],curc[1],xd,yd,bb
                   wset,kpwin
                   if (bb eq 1) or (bb eq -2) or (bb eq -3) then begin & d=cur_fct[3]/cur_fct[2]
                      curc=curc+cur_zoom[6:7] & curd=[xd,yd]+cur_zoom[6:7]
                      X =ceil((curc[0]+1)/d/cur_fct[9]  /cur_zoom[0]) ;baset.factor(0) when constant data size
                      Y =ceil((curc[1]+1)/d/cur_fct[10] /cur_zoom[0]) ;baset.factor(1)
                      Xd=ceil((curd[0]+1)/d/cur_fct[9]  /cur_zoom[0]) ;baset.factor(0)
                      Yd=ceil((curd[1]+1)/d/cur_fct[10] /cur_zoom[0]) ;baset.factor(1)
                      xr=[x,xd] & xr=long([min(xr),max(xr)]+FX)
                      yr=[y,yd] & yr=long([min(yr),max(yr)]+FX)

                     if cur_menu[0] eq 0 then targ=baset.bases6
                     if cur_menu[0] eq 1 then targ=baset.bases7

                     if cur_menu[0] eq 2 then begin
                       box=[xr[0]-FX,yr[0]-FX,xr[1]-xr[0]+1,yr[1]-yr[0]+1]
                       if cur_menu[6] eq 1 then begin                                      ;Sample Beam
                         if baset.bases8[1] ne 0 then begin ;beam sizes
                            widget_control,baset.bases8[0],set_value=strtrim(xr[1]-xr[0]+1,2)
                            widget_control,baset.bases8[1],set_value=strtrim(yr[1]-yr[0]+1,2)
                            sans_mask,xybeam=box[2:3]
                            if sans_settings_RegSheet() then sans_sheet_receive,beambox=strtrim(box[2],2)+' '+strtrim(box[3],2)
                         endif else begin                   ;beam ranges
                            idx=where([xr,yr] lt 100) & if n_elements(idx) ge 2 then mil=' , ' else mil=','
                            bbox=strtrim(xr[0],2)+','+strtrim(xr[1],2)+mil+strtrim(yr[0],2)+','+strtrim(yr[1],2)
                            widget_control,baset.bases8[0],/input_focus,set_value=bbox
                            sans_mask,baset,xybeam=box
                            if sans_settings_RegSheet() then sans_sheet_receive,beambox=bbox
                         endelse
                         cur_menu[3]=0
                       endif else begin
                          idx=where([xr,yr] lt 100) & if n_elements(idx) ge 2 then mil=' , ' else mil=','
                          sbox=strtrim(xr[0],2)+','+strtrim(xr[1],2)+mil+strtrim(yr[0],2)+','+strtrim(yr[1],2)
                          widget_control,baset.bases8[4],/input_focus,set_value=sbox
                          if cur_menu[6] eq 2 then begin                                   ;Water Beam Box
                              sans_mask,waterbox=box & cur_menu[5]=0
                              sans_settings_swap,baset,waterbox=box ,forWater=baset.curWATER
                              if sans_settings_RegSheet() then sans_sheet_receive,waterbox=sbox  & endif
                          if (cur_menu[6] eq 3) or (cur_menu[6] eq 4) then begin           ;Detector Box
                              sans_mask,box  =box & cur_menu[4]=0
                              if cur_menu[6] eq 3 then sans_settings_swap,baset,masq=(sans_masq) & endif ;!bug!
                       endelse
                       cur_cxy[2]=-1 ;***force Ben to change**
                       sans_settings, refresh=baset
                     endif else begin
                         widget_control,targ[0],set_value=strtrim(xr[0],2)
                         widget_control,targ[1],set_value=strtrim(xr[1],2)
                         widget_control,targ[2],set_value=strtrim(yr[0],2)
                         widget_control,targ[3],set_value=strtrim(yr[1],2)
                         sans_settings_mask,baset & endelse
                  endif
   endif
   if (ev.type ne 0) and (ev.type  ne 2) then cur_zoom[3]=0        ;clear moving anyway
   if (ev.type eq 0) and (ev.press eq 1) and (uv[1] eq 1)    $
                     and (uv[2] eq 1) and (cur_zoom[0] gt 1) $
                    then begin & cur_zoom[3]=1 & cur_zoom[4:5]=curc ;Press -> start moving
                    endif
   if (ev.type eq 2) and (cur_zoom[3] eq 1) then begin             ;Motion while zooming
     ;if (curc(0) ge 0) and (curc(1)) ge 0  then begin
          disp=curc-cur_zoom[4:5] & cur_zoom[4:5]=curc & dcs=cur_fct[3]
          newc=(cur_zoom[6:7]-disp*cur_zoom[0])>0<(cur_zoom[0]*dcs-dcs)
          if (newc[0] ne cur_zoom[6]) or (newc[1] ne cur_zoom[7]) then begin
              wset,baset.bases1[9]
              device,copy=[cur_zoom[6],cur_zoom[7],dcs,dcs,0,0,cur_zoom[2]]
              cur_zoom[6:7]=newc & wset,kpwin & endif
     ;endif else cur_zoom(3)=0
   endif $
   else  $
   if (ev.type eq 1) and (ev.release eq 4) then call_procedure,'lamp_event_parser',ev,[-88,0] $ ;Clipboard
   else $
   if (ev.type eq 2) then begin ;Motion -> show coordinates
         V=0 & sz=size(cur_run) & d=cur_fct[3]/cur_fct[2]
         if uv[1] eq 1 then $   ;DETECTORS
         if sz[0] eq 2 then begin
            if uv[2] eq 2 then begin ;Ben coordinates
               cur_fct3=cur_cxy[7] ;DRAW_SIZE while cur_fct(3) is the XSIZE
               X=ceil((curc[0]+1-(cur_fct3-cur_cxy[5])/2.)/cur_fct[0])>1<sz[1]
               Y=ceil((curc[1]+1-(cur_fct3-cur_cxy[6])/2.)/cur_fct[1])>1<sz[2]
            endif else begin
               curc= curc+cur_zoom[6:7]
               X=ceil((curc[0]+1)/d/cur_fct[9] /cur_zoom[0]) ;baset.factor(0)
               Y=ceil((curc[1]+1)/d/cur_fct[10]/cur_zoom[0]) ;baset.factor(1)
            endelse
            widget_control,baset.bases2[2],get_value=cx & cx=strtrim(cx[0],2)
            widget_control,baset.bases2[3],get_value=cy & cy=strtrim(cy[0],2)
            widget_control,baset.bases2[8],get_value=bx & bx=strtrim(bx[0],2)
            widget_control,baset.bases2[9],get_value=by & by=strtrim(by[0],2)
            V=cur_run[(x-1)>0<(sz[1]-1) , (y-1)>0<(sz[2]-1)]
            if V lt 10. then V=string(V,format='(F5.3)') else  V=strtrim(long(round(V)),2)
            Q=0 & theta2=0 & R=sqrt(total(((sans_ctr[0:1]-[X,Y])*sans_supl[8:9])^2))
            if R  lt min(sans_supl[8:9]) then R =0. $
            else begin
                xy0=sans_ctr-1 & if xy0[0] lt 0 then xy0[0]=xy0[0]+2. & if xy0[1] lt 0 then xy0[1]=xy0[1]+2.

                sans_settings_dist, sz,Dst,Center=xy0,xyPsize=sans_supl[8:9], DforT=DforT ,thetrad=thetrad
                R=Dst[(x-1)>0<(sz[1]-1),(y-1)>0<(sz[2]-1)]
            endelse
            if (abs(sans_supl[3]) ge 1.)  and (R gt 0.) then begin ;We have a detector angle
                if n_elements(DforT) gt 1 then R=DforT[(x-1)>0<(sz[1]-1),(y-1)>0<(sz[2]-1)]
            endif
            Rq=R
            if (sans_supl[1] gt 0) and (R gt 0.) then $
               if n_elements(thetrad)    gt 1 then theta2=thetrad[(x-1)>0<(sz[1]-1),(y-1)>0<(sz[2]-1)] $
                                              else theta2=atan(Rq/(sans_supl[1]*100.))
            if sans_supl[0]*sans_supl[1] gt 0 then Q=4.*!pi*sin(theta2/2.)/sans_supl[0]
            R=strtrim(string((round(R*10.)/10.),format='(F7.1)'),2) & Q=strtrim(round(Q*10000.)/10000.,2)
            X=long(X+FX) & Y=long(Y+FX)
            cx=strtrim(string(cx,format='(F8.2)'),2)
            cy=strtrim(string(cy,format='(F8.2)'),2)
            txt='X:'+strtrim(X,2)+'  Y:'+strtrim(Y,2)+'  I:'+V+' C['+cx+','+cy+'] R[cm]:'+R+' Q:'+Q
            widget_control,baset.bases1[12],set_value=strmid(txt,0,56)
         endif
         if uv[1] eq 2 then begin ;RESULTS motion
            wset,baset.bases1[14]
            resIQ=convert_coord(curc,/device,/to_data)
            wset,kpwin
            Q   =resIQ[0]
            I   =resIQ[1]
            txt =' ' & titF=''
            if (I ge 0.) and (Q ge 0.) and (baset.FIQ) then begin &     titF='(Q)'
                if baset.bases6[13] eq 2 then begin & Q=sqrt(Q)       & titF='(Q^2)'     & endif
                if baset.bases6[13] eq 3 then begin & Q=sqrt(sqrt(Q)) & titF='(Q^4)'     & endif
                if baset.IofQ       eq 0 then begin & I=I             & titF='I'   +titF & endif
                if baset.IofQ       eq 1 then begin & I=I/Q           & titF='IQ'  +titF & endif
                if baset.IofQ       eq 2 then begin & I=I/Q^2         & titF='IQ^2'+titF & endif
                if baset.IofQ       eq 3 then begin & I=I/Q^4         & titF='IQ^4'+titF & endif
                Q=round(Q*10000.)/10000.
                txt='Q= '+strtrim(Q,2)+'   I= '+strtrim(I,2) +'  -->  '+titF
            endif
            widget_control,baset.bases1[12],set_value=txt
         endif
   endif
   if (ev.type eq 1) and (ev.release eq 1) and (cur_menu[1] eq 5) and (cur_menu[0] ne 19) then begin
      ;widget_control,ev.top,set_uvalue=baset ; LIST OF RESULTS
       ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:baset.bases7[27],TOP:ev.top,HANDLER:0L,SELECT:1}')
   endif
   end

;LARGE VIEW (return to previous)
20:if ev.type eq 0 then begin & sans_settings, refresh=baset & k=cur_menu[1] ;button press
         widget_control,baset.bases1[7],map=0   & widget_control,baset.bases1[8],map=1
         widget_control,baset.bases5[k],map=1   & if k ne 5 then $
                                                  widget_control,baset.bases5[5],map=0
         if sans_beam[2] eq 1 then txtf='rectang' else txtf='ellipse'
         widget_control,baset.bases1[13],set_value=    txtf
         widget_control,baset.bases1[12],set_value=' '
         widget_control,baset.bases5[9] ,map=0  & widget_control,baset.bases5[3],map=1 ;Replace Option by the Treatment options button !
         endif
24:begin & baset.bases6[8]=ev.select & widget_control,ev.top,set_uvalue=baset ;Log display (Averaging and resume list)
         if cur_menu[1] eq 5 then begin ;Averaging
            if cur_menu[0] ne 19 then begin
              if baset.bases1[15] gt 0L then begin
               widget_control,baset.bases1[15],set_uvalue=[72,cur_menu[2],0]
               widget_control,ev.top,set_uvalue=baset
               ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:baset.bases1[15],TOP:ev.top,HANDLER:0L,SELECT:1}')
               widget_control,baset.bases1[15],set_uvalue=[72,2,0]
              endif
            endif else begin
               widget_control,ev.top,set_uvalue=baset
               ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L,CLICKS:0L}')
            endelse
         endif else sans_settings_mask, baset
         end
25:begin & mxzoom=round(min(cur_fct[3]/(cur_fct[2]/cur_fct[9:10]))) & ok=0 ;baset.factor 256/(128/fct) -> zoom original
         cur_zoom[3]=0
         if uv[1] eq 1 then if mxzoom*cur_zoom[0] lt 5 then begin &  ok=1  ;Zoom In
                                      cur_zoom[0]=cur_zoom[0]+1  & endif

         if uv[1] eq 2 then if        cur_zoom[0] gt 1 then begin &  ok=1  ;Zoom Out
                                      cur_zoom[0]=cur_zoom[0]-1  & endif
         if ok then begin &   widget_control,cur_zoom[1],draw_xsize=cur_fct[3]*cur_zoom[0] $
                                                      ,draw_ysize=cur_fct[3]*cur_zoom[0]
                            cur_zoom[6]=cur_zoom[6]<(cur_fct[3]*cur_zoom[0]-cur_fct[3])
                            cur_zoom[7]=cur_zoom[7]<(cur_fct[3]*cur_zoom[0]-cur_fct[3])


                            cur_zoom[4:5]=cur_fct[3]/(cur_zoom[0]+1)*2/3 & curc=[0,0]     ;re-center after Zoom In
                            if uv[1] eq 2 then begin & curc=cur_zoom[4:5] & cur_zoom[4:5]=0 ;re-center after Zoom Out
                            endif
                            disp=curc-cur_zoom[4:5] & cur_zoom[4:5]=curc & dcs=cur_fct[3]
                            newc=(cur_zoom[6:7]-disp*cur_zoom[0])>0<(cur_zoom[0]*dcs-dcs)
                            if (newc[0] ne cur_zoom[6]) or (newc[1] ne cur_zoom[7]) then begin
                              kpwin=!D.window & wset,baset.bases1[9]
                              device,copy=[cur_zoom[6],cur_zoom[7],dcs,dcs,0,0,cur_zoom[2]]
                              cur_zoom[6:7]=newc & wset,kpwin & endif

                            sans_settings_mask,baset  & endif
         if uv[1] eq 3 then sans_settings_mask,baset, /surf ;                       SURF
         if uv[1] eq 4 then begin & p_did_getW_cur, Widx ;UV(2)= 1:detector  2:sector IMAGE
                            if baset.bases6[8] then seem,/log
                            see,W=Widx,/beside,/image & seem,/reset & endif
         end
26:begin & setcol,uv[1] & sans_settings_mask,baset    & end      ;New color

27:begin & widget_control,ev.id           ,get_value=vz & pz=vz  ;Slider for numors
         if ev.id eq baset.bases9[9]  then begin & widget_control,baset.bases9[11],get_value=pz
                                                 widget_control,baset.bases9[11],set_value=vz & endif
         if ev.id eq baset.bases9[11] then begin & widget_control,baset.bases9[9] ,get_value=pz
                                                 widget_control,baset.bases9[9] ,set_value=vz & endif
         if pz ne vz then begin
          if cur_wn eq 18 then begin ;CORRECTED DATA

;            widget_control,  baset.bases3(8)  ,get_uvalue=sdata & TAKE_W,tmpcor,W=cur_wn     ;corrected data
;            if n_elements(tmpcor) eq n_elements(sdata.data) then begin kpdata=sdata.data     ; ""
;               sdata.data=tmpcor  &    widget_control,  baset.bases3(8)  ,set_uvalue=sdata   ;(bluf with sdata)
;               sdata.data=kpdata  &    catch,stat
;               if stat ne 0 then begin widget_control,  baset.bases3(8)  ,set_uvalue=sdata & CATCH,/cancel & endif $
;               else begin              sans_settings_runs, baset ,[9,8,12,baset.bases3(2)]  & cur_wn=18
;                                       widget_control,  baset.bases4(4)  ,set_value='CORRECTED'
;                                       widget_control,  baset.bases3(8)  ,set_uvalue=sdata  & endelse
;            endif

             sz=size(cur_SamCor.data)
             if sz[0] ge 2 then begin
                 if sz[0] eq 3 then sz=sz[3]-1 else sz=0
                 sans_settings_mask , baset, run=cur_SamCor.data[*,*,vz<sz]     ;Show corrected data
                 widget_control,baset.bases4[4] ,set_value='CORRECTED -> W16'
                 if sz eq 0  then title=cur_SamCor.datp.w_tit else title=cur_SamCor.par[vz<sz]
                 if sz gt 0  then begin & svz=strtrim(vz<sz,2) & XICUTER,'W16=W18[*,*,'+svz+']'
                                                               XICUTE ,'W_tit[16]="<'+title+'>"' & endif
                 title=strmid(strcompress(title) +' ->W16',0,55)
                 widget_control,baset.bases1[11],set_value=title
                 widget_control,baset.bases1[12],set_value=' '
                 SEE,W=16,/below,/image & to_did_cur,'W16'
                 num=0
                 if sz  gt 0 then ii=execute('num=long(cur_SamCor.datp.pv[idxp[6],vz<sz])')
                 if num le 0 then num=vz+1
                 widget_control,baset.bases9[10],set_value=strtrim(num,2)
                 widget_control,baset.bases9[12],set_value=strtrim(num,2)
             endif

          endif else $ ;TRANSMISSION OR SAMPLE SERIES
          if (cur_menu[0] eq 1) or (cur_menu[7] eq 3)  then sans_settings_runs , baset ,[9,20,3,baset.bases3[14]] else $ ;transmission series
          if (cur_menu[0] ne 1) or (cur_menu[7] eq 12) then sans_settings_runs , baset ,[9,8,12,baset.bases3[2] ]        ;sample series
         endif ;pz ne vz (slider changed)
         end
;CENTER
30:begin & widget_control,baset.bases1[7] ,map=0   & widget_control,baset.bases1[8] ,map=1 & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4] ,map=1   & widget_control,baset.bases5[5] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1] ,map=0   & widget_control,baset.bases4[0] ,map=1 & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0] ,map=1   & widget_control,baset.bases5[1] ,map=0 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2] ,map=0   & widget_control,baset.bases5[3] ,map=0 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6] ,map=0   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[5] ,sens=0  & widget_control,baset.bases4[6] ,sens=0
         widget_control,baset.bases4[14],sens=0  & widget_control,baset.bases4[15],sens=0
         widget_control,baset.bases4[3] ,map=0   & widget_control,baset.bases4[2] ,sens=0,map=1
         widget_control,baset.bases4[4] ,set_value='CENTER' & cur_menu[[0,1,10]]=[0,4,0]
         widget_control,baset.bases1[12],set_value=' [Xrg,Yrg]:drag with left or middle button'
         widget_control,baset.bases5[16],set_value=' '

         sans_settings_beamST,baset, Tog_STbeam='Ctr'   ;TOGGLE BEAM FOR Center<-->Transmisssion  (confus)

         cur_menu[[3,5]]=1 ;hide beam sample,water
         loadBeam=1
         if n_elements(uv) ge 2 then loadBeam=uv[1] ;if 0 do not replace previous image (unused)
         if loadBeam then begin 
            if baset.bases6[6] then sans_settings_mask, baset  $       ;hide beam sample,water
                               else sans_settings_mask, baset, /nomask ;and  detector mask
            cur_menu[4]=0                                              ;but  just for now
            sans_settings_runs, baset ,[9,22,9,baset.bases3[16]],/noread
                               ;                             9  event
                               ;                             22 is bases3 label index (data, datp, ...)
                               ;                             9  is Wi
                               ;                             16 is bases3 text  index (run number)
                                
            if (sans_supl[52] eq 1) then begin ;if xy unit is 2theta (D16) and no direct beam try to fill background data
                widget_control,  baset.bases3[22] ,get_uvalue=bdata ;to have correct units in cur_aX (common)
                if bdata.opr le ' ' then sans_settings_runs, baset ,[9,9 ,0,baset.bases3[3]], /noread
            endif
         endif
         end
31:      baset.bases6[4]=ev.select ;Fit
32:      baset.bases6[5]=ev.select ;Gravity
33:      begin & baset.bases6[6]=ev.select ;Use Mask
               if ev.select then sans_settings_mask, baset $
                            else sans_settings_mask, baset,/nomask & cur_menu[4]=0 ;no detector mask just for now
         end

39:begin & sans_settings_text, baset.bases6[0:3], farr, tarr, mess ;Calculate Center
         sans_settings_beamST,baset, Tog_STbeam='Ctr'
         sans_settings_runs, baset ,[9,22,0,baset.bases3[16]]
         sans_settings_text, baset.bases7[16]  ,Tauv  & sans_supl[6] =Tauv  ;Get Tau value
         widget_control,  baset.bases3[22] ,get_uvalue=bdata & ws=strtrim(string(bdata.wi),2)

         sarr=strtrim(string(farr-float(FX)),2)
         xr='['+sarr[0]+','+sarr[1]+']' & yr='['+sarr[2]+','+sarr[3]+']'
         if baset.bases6[4] then Fit =',/Fit'  else Fit =''
         if baset.bases6[5] then Grav=',/Grav' else Grav=''
         masq=''
         okC =0
         if bdata.opr gt ' ' then $
         if baset.bases6[6]  then begin & sans_settings_getmask,size(bdata.data),[0.,0.],M,/nobeam,/noWbeam ;size(cur_mask)!
                                        sans_gt_par ,sys=sys
                                        GIVE_W,w=sys+1 ,M & masq=',Mask=w'+strtrim(sys+1,2)
                                  endif
        ;ASK FOR AUTO CALCULATION SAMPLE MASK
         if (uv[1] eq 1) and (bdata.opr gt ' ') then begin
             widget_control,baset.bases3[26],get_uvalue=bdata        ;get also a sample from mask window
             if (bdata.opr gt ' ') and (n_elements(bdata.data) gt 1) then begin & sans_gt_par ,sys=sys
                                                                                GIVE_W,w=sys+2,bdata.data
                                                                                samplemask=',samplemask=w'+strtrim(sys+2,2)
                                                                     endif else samplemask=',samplemask=1'

             if ((size(bdata.data))[1] eq sans_masq[2]) and ((size(bdata.data))[2] eq sans_masq[3]) then begin
                 XICUTER,'c=sans_center(W'+ws+',Xr='+xr+',yr='+yr+ Fit+Grav+masq+samplemask+')'
                 sbox =sans_beam[[7,8,0,1]] & stbox=long([sbox[0],sbox[0]+sbox[2]-1,sbox[1],sbox[1]+sbox[3]-1]+FX)
                 stbox=strtrim(stbox[0],2)+','+strtrim(stbox[1],2)+','+strtrim(stbox[2],2)+','+strtrim(stbox[3],2)
                 if sans_settings_RegSheet() then sans_sheet_receive,beambox=stbox
                 sans_settings, refresh=baset
                 sans_settings_mask,baset
             endif
        ;CALCULATION OF SAMPLE CENTER (AUTO OR NOT)
         endif else if (uv[1] ne 1) then begin
           narr=where(tarr ne '0')
           if (bdata.opr gt ' ') or (n_elements(narr) eq 2) then begin
             if (bdata.opr gt ' ') then XICUTER,'c=sans_center(W'+ws+',Xr='+xr+',yr='+yr+ Fit+Grav+masq+')' $
                                   else begin & newCtr=float(sarr[narr])
                                              if FX ne 0 then begin
                                                if newCtr[0] lt 0 then newCtr[0]=newCtr[0]-2
                                                if newCtr[1] lt 0 then newCtr[1]=newCtr[1]-2
                                              endif
                                              sans_mask,center=NewCtr
                                   endelse
             cur_menu[3]=1 ;hide beam
             okC=1

          ;WE HAVE XY IN 2THETA UNIT !
           endif else if (sans_supl[52] eq 1) and (n_elements(cur_aX) gt 1) then begin
             Cx=interpol(findgen(n_elements(cur_ax)),cur_ax,0.) & if Cx lt 0 then Cx=Cx-1 else Cx=Cx+1
             Cy=interpol(findgen(n_elements(cur_ay)),cur_ay,0.) & if Cy lt 0 then Cy=Cy-1 else Cy=Cy+1
             Cx=long(Cx*100)/100. & Cy=long(Cy*100)/100.
             sans_mask,center=[Cx,Cy]
             okC=1
           endif
        ;CALULATION OK
           if okC then begin
             sans_settings,refresh=baset
             widget_control,baset.bases2[2],get_value=cx & cx=strtrim(cx[0],2)
             widget_control,baset.bases2[3],get_value=cy & cy=strtrim(cy[0],2)
             phi0=strtrim(string(sans_supl[31],format='(I10)'),2)
             dTh =strtrim(string(round(sans_supl[49]*10000.)/10000.,format='(F6.4)'),2)
             widget_control,baset.bases5[16],set_value='Cx='+cx+' Cy='+cy+' Flx='+phi0
             if sans_settings_RegSheet() then sans_sheet_receive, ctr=cx+","+cy+" dTh:"+dTh
           endif
         endif
         end

;ATTENUATION
40:begin & widget_control,baset.bases1[7] ,map=0   & widget_control,baset.bases1[8] ,map=1 & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4] ,map=1   & widget_control,baset.bases5[5] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1] ,map=0   & widget_control,baset.bases4[0] ,map=1 & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0] ,map=0   & widget_control,baset.bases5[1] ,map=1 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2] ,map=0   & widget_control,baset.bases5[3] ,map=0 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6] ,map=0   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[5] ,sens=1  & widget_control,baset.bases4[6] ,sens=1
         widget_control,baset.bases4[14],sens=1  & widget_control,baset.bases4[15],sens=1
         widget_control,baset.bases4[3] ,map=0   & widget_control,baset.bases4[2] ,sens=1,map=1
         widget_control,baset.bases4[4] ,set_value='TRANSMISSION' & cur_menu[[0,1,10]]=[1,4,1]
         widget_control,baset.bases1[12],set_value=' '
         widget_control,baset.bases7[29],set_value=' [XYrg]:drag left or middle button'

         sans_settings_beamST,baset, Tog_STbeam='Trs'   ;TOGGLE BEAM FOR Center<-->Transmisssion    confus

         cur_menu[[3,5]]=1                                    ;hide beam sample,water
         if sans_supl[66] gt 1 then $                         ;we may have panels On
            if (baset.curLMH ge 1) and (sans_supl[67+baset.curLMH]) then begin  
                widget_control,baset.bases7[21] ,set_button=0 ;so, hide all masks (NegMask)
                baset.bases6[7] = 0   & endif

         sans_settings_runs, baset ,[9,22,5,baset.bases3[16]],/noread

         if baset.bases6[7] then sans_settings_mask,baset  $       ;hide beam sample,water
                            else sans_settings_mask,baset, /nomask ;and  detector mask
                            cur_menu[4]=0                          ;but  just for now
         end

43:      begin & baset.bases6[7]=ev.select ;Use Mask
               if ev.select then sans_settings_mask, baset  $
                            else sans_settings_mask, baset, /nomask & cur_menu[4]=0 ;no detector mask just for now
         end

44:begin & widget_control,baset.bases4[3],map=1   & widget_control,baset.bases4[2],map=0 ;Manual transmission
         widget_control,baset.bases4[3],set_uvalue=uv[1]
         widget_control,uv[2],set_value=string(sans_trans[uv[1],0],format='(f5.3)'),/input_focus
         end
45:begin & widget_control,baset.bases4[3],map=0   & widget_control,baset.bases4[2],map=1 ;Manual transmission
         widget_control,baset.bases4[3],get_uvalue=trx
         sans_settings_text, ev.id, farr
         transm=string(farr,format='(f5.3)') ;error!
         sans_trans[trx,0]=farr & sans_settings,refresh=baset
         sans_settings_swap, baset, transm=(sans_trans)
         end
46:begin & if uv[1] eq 0 then begin & widget_control,baset.bases4[8] ,map=1 ;Show Sample series Transmissions
                                  widget_control,baset.bases4[0] ,map=0
                                  widget_control,baset.bases3[20],get_uvalue=sdata ;transmission data
                                  transm=string(sans_trans[2,0]  ,format='(f5.3)') & nt=n_elements(sans_trans[*,0])
                                  if nt gt 5 then for i=5L,nt-1 do transm=[transm,string(sans_trans[i,0],format='(f5.3)')]
                                  txt='Ts '+transm+'->'+sdata.par
                                  widget_control,baset.bases4[9] ,set_value =[txt] & endif
         if uv[1] eq 1 then begin & widget_control,baset.bases4[8] ,map=0
                                  widget_control,baset.bases4[0] ,map=1            & endif

         if uv[1] eq 2 then begin & widget_control,baset.bases4[10],map=1 ;Verify Sample series Transmissions with sample data
                                  widget_control,baset.bases4[1] ,map=0
                                  widget_control,baset.bases3[20],get_uvalue=sdata
                                  transm=string(sans_trans[2,0]  ,format='(f5.3)') & nt=n_elements(sans_trans[*,0])
                                  if nt gt 5 then for i=5L,nt-1 do transm=[transm,string(sans_trans[i,0],format='(f5.3)')]
                                  widget_control,baset.bases3[8 ],get_uvalue=vdata
                                  ns =n_elements(sdata.par) & nv=n_elements(vdata.par) & nn=n_elements(transm)
                                  pZ=vdata.datp.Z & if n_elements(pZ) ne nv then pZ=lindgen(nv) & j=pZ[0]<(nn-1)
                                  txt=['Ts '+transm[j]+'->'+sdata.par[j<(ns-1)],'Ts '+transm[j]+'->'+vdata.par[0],'---']

                                  if nv gt 1 then for i=1L,nv-1 do begin & j=pZ[i]
                                     if i lt nn  then begin
                                      if i lt ns then txt=[txt,'Ts '+transm[j]+'->'+sdata.par[j],'Ts '+transm[j]+'->'+vdata.par[i],'---'] $
                                                 else txt=[txt,'Ts '+'?? transmission'          ,'Ts '+transm[j]+'->'+vdata.par[i],'---']
                                     endif       else txt=[txt,vdata.par[i],'---']
                                  endfor
                                  widget_control,baset.bases4[11],set_value =[txt] & endif
         if uv[1] eq 3 then begin & widget_control,baset.bases4[10],map=0
                                  widget_control,baset.bases4[1] ,map=1            & endif

         if uv[1] eq 4 then begin & widget_control,baset.bases4[uv[2]],map=0 ;Show Transmission, Thickness and X-section Sample series
                            widget_control,baset.bases4[12]   ,map=1,set_uvalue=uv[2]
                            widget_control,baset.bases3[20],get_uvalue=sdata & ns=n_elements(sdata.par) ;transm
                            widget_control,baset.bases3[8 ],get_uvalue=vdata & nv=n_elements(vdata.par) ;data
                           ;if uv(2) eq 0 then $
                            if (ns gt nv) or (vdata.par[0] eq '') then begin & vdata=sdata & nv=ns & endif
                            transm=string(sans_trans[2,0]  ,format='(f5.3)') & nt=n_elements(sans_trans[*,0])
                            thickW=string(sans_supl[10]    ,format='(f5.3)')
                            thickS=string(sans_supl[11]    ,format='(f5.3)') & if sans_supl[11] ge 10. then thickS=string(sans_supl[11],format='(f5.2)')
                            XsectS=string(sans_supl[12]    ,format='(f5.3)')
                                  if nt gt 4 then begin
                                     sizt=size(sans_trans) & trs5=5
                                     if sizt[0] eq 1 then begin & sans_trans=[[sans_trans],[sans_trans*0.+thickS],[sans_trans*0.+XsectS],[sans_trans*0.+1],[sans_trans*0.]]
                                                                sans_trans[[0,1,3],1]=[thickW,'0.00','0.00'] & endif
                            endif
                            if nt gt 5 then for i=5L,nt-1 do begin & transm=[transm,string(sans_trans[i,0],format='(f5.3)')]
                                                                   if sans_trans[i,1] eq 0. then sans_trans[i,1]=sans_supl[11]
                                                                   thickS=[thickS,string(sans_trans[i,1],format='(f5.3)')]
                                                                   XsectS=[XsectS,string(sans_trans[i,2],format='(f5.3)')] & endfor
                                  nn =n_elements(transm)
                                  txt=['ts: '+transm[0]+' es: '+thickS[0]+' er: '+XsectS[0]+' ->'+vdata.par[0]]
                                  if nv gt 1 then for i=1L,nv-1 do begin
                                     if i lt nn then txt=[txt,'ts: '+transm[i]+' es: '+thickS[i]+' er: '+XsectS[i]+' ->'+vdata.par[i]] $
                                                else txt=[txt,'->'+vdata.par[i]]   & endfor
                                  widget_control,baset.bases4[13],set_value =[txt] & endif

         if uv[1] eq 5 then begin & widget_control,baset.bases4[13],get_value=txt ;Edit Transmission, Thickness and X-section Sample series
                                  widget_control,baset.bases3[20],get_uvalue=sdata & ns=n_elements(sdata.par)
                                  widget_control,baset.bases3[8 ],get_uvalue=vdata & nv=n_elements(vdata.par)
                                  nt=n_elements(sans_trans[*,0]) & sizt=size(sans_trans)
                                  ok=0 & k=0 & on_ioerror,misesxs
                                  for i=0L,n_elements(txt)-1 do begin
                                      jt=strpos(txt[i],'ts:') & je=strpos(txt[i],'es:')
                                      jx=strpos(txt[i],'er:') & jf=strpos(txt[i],'->' )
                                      tt=strmid(txt[i],jf+2,100) & tt=strcompress(tt)
                                      if (k lt nt) and (jt ge 0) and (je gt jt) and (jx gt je) and (jf gt jx) then begin
                                          vt=float(strmid(txt[i],jt+3,je-jt))
                                          ve=float(strmid(txt[i],je+3,jx-je))
                                          vx=float(strmid(txt[i],jx+3,jf-jx))
                                          if i eq 0 then begin & sans_trans[2,0]=vt & sans_supl[11]  =ve & sans_supl[12]  =vx & k=4
                                                         if sizt[0] eq 2 then begin & sans_trans[2,1]=ve & sans_trans[2,2]=vx & endif & endif
                                      endif
                                      if k lt nt  then begin & sans_trans[k,0]=vt & sans_trans[k,1]=ve & sans_trans[k,2]=vx & endif
                                if (ns gt nv) then if i lt ns  then sdata.par[i]=tt
                                if (ns le nv) then if i lt nv  then vdata.par[i]=tt
                                      k= k+1
                                  endfor
                                  widget_control,baset.bases3[20],set_uvalue=sdata
                                  widget_control,baset.bases3[8 ],set_uvalue=vdata
                                  ok=1 & sans_settings,refresh=baset
                                  sans_settings_swap, baset, transm=(sans_trans) ,shareSupl=[11,12] ,supl=(sans_supl)
                                  widget_control,baset.bases4[12],map=0,get_uvalue=vf
                                  widget_control,baset.bases4[vf],map=1
                                  misesxs: if not ok then widget_control,baset.bases1[12],set_value='? input error line'+string(i+1)
                                  endif
         end
47:
48:

49:begin & if n_elements(uv) gt 1 then widget_control,  uv[1],set_value=' '  ;Calculate Transmission
         widget_control,/hourglass
         kperr=!error_state.msg
         sans_settings_runs, baset ,[9,18,0,baset.bases3[12:17]], /HaveAll

         if kperr eq !error_state.msg then begin
         sans_settings_text, baset.bases7[16]  ,Tauv  & sans_supl[6] =Tauv  ;Get Tau value
         widget_control,  baset.bases3[18] ,get_uvalue=wdata  & wws =strtrim(string(wdata.wi) ,2)
         widget_control,  baset.bases3[19] ,get_uvalue=ewdata & ewws=strtrim(string(ewdata.wi),2)
         widget_control,  baset.bases3[20] ,get_uvalue=sdata  & sws =strtrim(string(sdata.wi) ,2)
         widget_control,  baset.bases3[21] ,get_uvalue=esdata & esws=strtrim(string(esdata.wi),2)
         widget_control,  baset.bases3[22] ,get_uvalue=bdataS & bsws=strtrim(string(bdataS.wi),2)
         widget_control,  baset.bases3[23] ,get_uvalue=bdataW & bwws=strtrim(string(bdataW.wi),2)

         widget_control,  baset.bases3[8]  ,get_uvalue=vdata

         if (bdataS.opr gt ' ') or (bdataW.opr gt ' ') then begin
           sizv=size(vdata.data) & if sizv[0] eq 3 then siv=sizv[3] else siv=1
           sizs=size(sdata.data) & if sizs[0] eq 3 then sis=sizs[3] else sis=1
           if sdata.opr gt ' ' then begin
              pZ=sdata.datp.Z & if n_elements(pZ) ne sis then pZ=findgen(sis)

              sizt=n_elements(sans_trans[*,0]) & mmt=max([siv,sis,pZ+1])
              if sizt-4 lt mmt then sans_trans=fltarr(mmt+4,5)
              if sizt-4 gt mmt then sans_trans=sans_trans[0:mmt-1+4,*] ;keep a good size for sans_trans
           endif
           if bdataS.opr gt ' ' then sizMask=size(bdataS.data) else sizMask=size(bdataW.data)
           if baset.bases6[7] then begin & sans_settings_getmask,size(sizMask),[0.,0.],M,/nobeam,/noWbeam ;size(cur_mask)!
                                         sans_gt_par ,sys=sys
                                         GIVE_W,w=sys+1,M & masq=',Mask=w'+strtrim(sys+1,2)
                                   endif else               masq=',Mask=0'
           sans_settings_text, baset.bases7[0:3], farr, sarr, mess
           sarr=strtrim(string(sarr-FX),2)
           xr ='['+sarr[0]+','+sarr[1]+']' & yr='['+sarr[2]+','+sarr[3]+']'
           ok =[0,0,0,0]  & savtrs=strarr(4)
           if bdataS.opr gt ' ' then cmd='Sbeam='+bsws else cmd='0'
           if wdata.opr  gt ' ' then begin & cmd=cmd+',wat=' + wws & savtrs[0]= wdata.par[0] & endif else ok[0]=1
           if ewdata.opr gt ' ' then begin & cmd=cmd+',Ewat='+ewws & savtrs[1]=ewdata.par[0] & endif else ok[1]=1
           if sdata.opr  gt ' ' then begin & cmd=cmd+',sam=' + sws & savtrs[2]= sdata.par[0] & endif else ok[2]=1
           if esdata.opr gt ' ' then begin & cmd=cmd+',Esam='+esws & savtrs[3]=esdata.par[0] & endif else ok[3]=1
           if bdataW.opr gt ' ' then         cmd=cmd+',Wbeam='+bwws & if n_elements(sdata.par) gt 1 then savtrs=[savtrs,sdata.par]

           XICUTER,'t=sans_transm('+cmd+',Xr='+xr+',yr='+yr +masq+')'

           if sdata.opr gt ' ' then sans_reform_series,savtrs=savtrs,/nothick,idxZ=pZ ;Keep transmissions, we may not have thickness

           widget_control,baset.bases4[3],map=0 & widget_control,baset.bases4[2],sens=1,map=1
           sans_settings,refresh=baset
           if n_elements(uv) gt 1 then widget_control,  uv[1],set_value=' Done'
          ;for i=0,3 do widget_control,baset.bases9(i+5),sens=ok(i) ;Asked by Peter Lindner !!!
           if baset.bases6[19] eq 0 then trtxt='Normalised by monitors' else trtxt='Normalised by times'

           if sans_settings_RegSheet() then sans_sheet_receive, trans=sans_trans, mess=trtxt
           sans_settings_swap, baset, transm=(sans_trans)
         endif ;opr gt ' '
         endif ;read error
         end

;MASK
50:begin & widget_control,baset.bases1[7], map=0   & widget_control,baset.bases1[8] ,map=1 & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4], map=1   & widget_control,baset.bases5[5] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=0   & widget_control,baset.bases4[0] ,map=0 & widget_control,baset.bases4[7] ,map=1
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=1   & widget_control,baset.bases5[3] ,map=0 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[3], map=0   & widget_control,baset.bases4[2] ,sens=0,map=1
        ;Hide CENTER adjustment for SECTORS
         widget_control,baset.bases4[19],map=0   & widget_control,baset.bases4[18],map=1
         widget_control,baset.bases4[21],map=0   & widget_control,baset.bases4[20],map=1

         widget_control,baset.bases4[4], set_value='MASK' & cur_menu[[0,1,10]]=[2,4,2]
         if sans_beam[2] eq 1 then txtf='rectang' else txtf='ellipse'
         widget_control,baset.bases1[13],set_value=    txtf
         widget_control,baset.bases1[12],set_value=' '
         cur_menu[4]=0
         if cur_menu[6] eq 1 then begin & cur_menu[5]=1 & cur_menu[3]=0 & endif ;sample
         if cur_menu[6] eq 2 then begin & cur_menu[5]=0 & cur_menu[3]=1 & endif ;water
         if cur_menu[6] eq 3 then begin & cur_menu[5]=1 & cur_menu[3]=0 & endif ;detector all configs
         if cur_menu[6] eq 4 then begin & cur_menu[5]=1 & cur_menu[3]=0 & endif ;detector cur config
         sans_settings_mask,baset      ;show beam sample,water
         if cur_menu[6] eq 1 then       sans_settings_runs, baset ,[9,26,7,baset.bases3[24]],/noread else $ ;sample
         if cur_menu[6] eq 2 then       sans_settings_runs, baset ,[9,27,8,baset.bases3[25]],/noread $       ;water
                             else       begin & endelse                                                   ;detector
         end
51:begin ;Change Mask
         widget_control,baset.bases8[2],get_value=xlin   & xlin=strsplit(/EXTRACT,strcompress(xlin[0] ,/remove_all),',')
         widget_control,baset.bases8[3],get_value=ylin   & ylin=strsplit(/EXTRACT,strcompress(ylin[0] ,/remove_all),',')
         widget_control,baset.bases8[4],get_value=stbox  & stbox=strcompress(stbox[0],/remove_all) & sbox=strsplit(/EXTRACT,stbox,',')
          ok=0 & if xlin[0] gt '' then begin & on_ioerror,misxl & xlin=long(xlin-FX) & ok=1 & misxl: & endif & if not ok then xlin=-1
          ok=0 & if ylin[0] gt '' then begin & on_ioerror,misyl & ylin=long(ylin-FX) & ok=1 & misyl: & endif & if not ok then ylin=-1
          ok=0 & if sbox[0] gt '' then begin & on_ioerror,missb & sbox=long(sbox-FX) & ok=1 & missb: & endif & if not ok then sbox=-1
          on_ioerror,mismask
         if uv[1] le 2 then begin & sans_settings_text, baset.bases8[0:1], farr, sarr, mess                       ;Sample Beam sizes
                                  sans_mask,xybeam=farr[0:1]  & cur_menu[3]=0
                                  if sans_settings_RegSheet() then sans_sheet_receive,beambox=strtrim(farr[0],2)+' '+strtrim(farr[1],2) & endif
         if uv[1] eq 3 then if xlin[0] ge 0 then begin & sans_mask,xline=xlin & cur_menu[4]=0 & endif
         if uv[1] eq 4 then if ylin[0] ge 0 then begin & sans_mask,yline=ylin & cur_menu[4]=0 & endif
         if uv[1] eq 5 then if sbox[0] ge 0 then begin & if n_elements(sbox) eq 2 then sbox=[sbox,sbox]
                   if n_elements(sbox) eq 4 then begin & xr=[sbox[0],sbox[1]] & xr=long([min(xr),max(xr)])
                                                       yr=[sbox[2],sbox[3]] & yr=long([min(yr),max(yr)])
                                                       sbox=[xr[0],yr[0] , xr[1]-xr[0]+1,yr[1]-yr[0]+1]
                                                       if cur_menu[6] eq 3 then sans_mask,box =sbox             ;Detector all configs
                                                       if cur_menu[6] eq 4 then sans_mask,box =sbox             ;Detector cur config
                                                       if cur_menu[6] eq 2 then sans_mask,waterbox=sbox         ;Water Beam
                                                       if cur_menu[6] eq 2 then begin
                                                          sans_settings_swap,baset,waterbox=sbox ,forWater=baset.curWATER ;share water for 3 configs?
                                                          if sans_settings_RegSheet() then sans_sheet_receive,waterbox=stbox & endif
                   endif
         endif
         if cur_menu[6] eq 3 then sans_settings_swap,baset,masq=(sans_masq) ;share detector changes for 3 configs !bug!

         if uv[1] eq 6 then begin & widget_control,baset.bases8[0],get_value=bbox                                 ;Sample Beam ranges
          stbox=strcompress(bbox[0],/remove_all) & bbox=strsplit(/EXTRACT,stbox,',')
          ok=0 & if bbox[0] gt '' then begin & on_ioerror,misbb & bbox=long(bbox-FX) & ok=1 & misbb: & endif & if not ok then bbox=-1
          on_ioerror,mismask
          if bbox[0] ge 0 then $
                   if n_elements(bbox) eq 4 then begin & xr=[bbox[0],bbox[1]] & xr=long([min(xr),max(xr)])
                                                       yr=[bbox[2],bbox[3]] & yr=long([min(yr),max(yr)])
                                                       bbox=[xr[0],yr[0] , xr[1]-xr[0]+1,yr[1]-yr[0]+1]
                                                       sans_mask,baset,xybeam=bbox  & cur_menu[3]=0
                                                       if sans_settings_RegSheet() then sans_sheet_receive,beambox=stbox & endif
         endif
         if cur_menu[6] eq 1 then cur_menu[4]=0
         if cur_menu[6] eq 2 then cur_menu[5]=0
         if cur_menu[6] eq 3 then cur_menu[4]=0
         if cur_menu[6] eq 4 then cur_menu[4]=0
         cur_cxy[2]=-1 ;***force Ben to change**
         sans_settings, refresh=baset
         mismask:
         end
52:begin & if cur_menu[6] eq 3 then sans_mask,/back                       ; Detector    Back all configs
         if cur_menu[6] eq 4 then sans_mask,/back                       ; Detector    Back
         if cur_menu[6] eq 3 then sans_settings_swap,baset,masq=(sans_masq) ;!bug!
         if cur_menu[6] eq 2 then sans_mask,waterbox=[0,0,0,0]          ; Water beam  Back
         if cur_menu[6] eq 2 then begin
            sans_settings_swap,baset,waterbox=[0,0,0,0] ,forWater=baset.curWATER & cur_eff=1.
            if sans_settings_RegSheet() then sans_sheet_receive,waterbox='0,0 0,0' & endif
         if cur_menu[6] eq 1 then sans_mask,xybeam=[0,0,0,0]            ; Sample beam Back
         if cur_menu[6] eq 1 then $
            if sans_settings_RegSheet() then sans_sheet_receive,beambox ='0,0 0,0'
         if cur_menu[6] eq 1 then cur_menu[4]=0
         if cur_menu[6] eq 2 then cur_menu[5]=0
         if cur_menu[6] eq 3 then cur_menu[4]=0
         if cur_menu[6] eq 4 then cur_menu[4]=0
         cur_cxy[2]=-1 & sans_settings, refresh=baset  & end
53:begin & if uv[1] eq 1 then begin & sans_mask,/clear & cur_menu[4]=0 & endif else sans_masq=0 & cur_cxy[2]=-1
         sans_settings, refresh=baset
         if cur_menu[6] eq 3 then sans_settings_swap,baset,masq=(sans_masq) ;!bug!
         end
54:begin & if sans_beam[2] eq 1 then txtf='rectang' else txtf='ellipse'
         case uv[1] of
         1: begin & sans_beam[2]=1 & sans_settings_mask,baset & widget_control,baset.bases1[13],set_value='rectang' & cur_menu[3]=0 & end
         2: begin & sans_beam[2]=2 & sans_settings_mask,baset & widget_control,baset.bases1[13],set_value='ellipse' & cur_menu[3]=0 & end
         3: begin & cur_menu[3]=abs(cur_menu[3]-1) & sans_settings_mask,baset          & end
         4: begin & cur_menu[4]=abs(cur_menu[4]-1) & sans_settings_mask,baset          & end
         5: begin & cur_menu[5]=abs(cur_menu[5]-1) & sans_settings_mask,baset          & end
         6: begin & sans_settings_mask,baset,/nobeam,/nomask,/nowbeam  & end
         7: begin & sans_settings_mask,baset,/showall         & end

         8: begin & cur_menu[3]=abs(cur_menu[3]-1) & sans_settings, refresh=baset & end ;toggle beam sample
         9: begin & cur_menu[5]=abs(cur_menu[5]-1) & sans_settings, refresh=baset & end ;toggle beam water
         10:begin & cur_menu[4]=abs(cur_menu[4]-1) & sans_settings, refresh=baset & end ;toggle mask unused!!!
         else:
         endcase
         end
55:begin & cur_menu[6]=uv[1]
         parent=widget_info(baset.bases3[26],/parent) & widget_control,parent,get_uvalue=uvp
         widget_control,uvp[uv[1]],bad_id=ii,set_button=1,sens=1 ;be sure we have correct (sample,detector,water) button selected
         if cur_menu[6] eq 1 then begin & cur_menu[5]=1 & cur_menu[3]=0  & endif  ;sample
         if cur_menu[6] eq 2 then begin & cur_menu[5]=0 & cur_menu[3]=1           ;water
            s=long(sans_beam[3:6]) & s=[s[0],s[0]+s[2]-1,s[1],s[1]+s[3]-1]+long(FX)
            sbox=strtrim(s[0],2)+','+strtrim(s[1],2)+' , '+strtrim(s[2],2)+','+strtrim(s[3],2)
            widget_control,baset.bases8[4],/input_focus,set_value=sbox & endif
         if cur_menu[6] eq 3 then begin & cur_menu[5]=1 & cur_menu[3]=0  & endif  ;detector
         if cur_menu[6] eq 4 then begin & cur_menu[5]=1 & cur_menu[3]=0  & endif  ;detector
         sans_settings_mask,baset
         if uv[1] eq 1 then begin ;SAMPLE
         if baset.bases5[23] gt 0 then begin
         widget_control,baset.bases5[23] ,sens=1  & widget_control,baset.bases5[24] ,sens=1 & endif ;Sample
         widget_control,baset.bases5[21] ,sens=1  & widget_control,baset.bases5[22] ,sens=1
         widget_control,baset.bases5[25] ,sens=0  & widget_control,baset.bases5[26] ,sens=0
         widget_control,baset.bases5[11] ,sens=0
         widget_control,baset.bases5[12] ,sens=1  & widget_control,baset.bases5[13] ,sens=0
         widget_control,baset.bases5[14] ,sens=1  & widget_control,baset.bases5[15] ,sens=0

         sans_settings_runs, baset ,[9,26,7,baset.bases3[24]],/noread

         ;ASKED FOR AUTO SAMPLE MASK CALCULATION
         ; -------------------------------------
         if (n_elements(uv) ge 3) then if uv[2] eq 1 then begin
            widget_control,baset.bases3[26],get_uvalue=bdata
            if (bdata.opr gt ' ') and (n_elements(bdata.data) gt 1) then $
             if ((size(bdata.data))[1] eq sans_masq[2]) and ((size(bdata.data))[2] eq sans_masq[3]) then begin
                sans_settings_getmask,size(bdata.data),[0.,0.],M,/nobeam,/noWbeam ;size(cur_mask)!
                c=sans_center(beam=bdata.wi,mask=M,/samplemask,/fit)
            endif
         endif
         endif

         if uv[1] eq 2 then begin ;WATER
         if n_elements(uv) ge 3 then if uv[2] eq -1 then begin
         widget_control,baset.bases3[24] ,sens=0  & widget_control,uvp[1] ,sens=0 & endif ;come from sheet (hide sample)

         if baset.bases5[23] gt 0 then begin
         widget_control,baset.bases5[23] ,sens=0  & widget_control,baset.bases5[24] ,sens=0 & endif ;Water
         widget_control,baset.bases5[21] ,sens=0  & widget_control,baset.bases5[22] ,sens=0
         widget_control,baset.bases5[25] ,sens=1  & widget_control,baset.bases5[26] ,sens=1
         widget_control,baset.bases5[11] ,sens=0
         widget_control,baset.bases5[12] ,sens=1  & widget_control,baset.bases5[13] ,sens=0
         widget_control,baset.bases5[14] ,sens=0  & widget_control,baset.bases5[15] ,sens=1
         sans_settings_runs, baset ,[9,27,8,baset.bases3[25]],/noread

         ;ASKED FOR AUTO WATER MASK CALCULATION
         ; ------------------------------------
         if (n_elements(uv) ge 3) then if uv[2] eq 1 then begin & cur_eff=1.
            widget_control,baset.bases3[27],get_uvalue=bdata
            widget_control,baset.bases3[ 7],get_uvalue=edata
            if (bdata.opr gt ' ') and (n_elements(bdata.data) gt 1) then begin
             if(edata.opr gt ' ') and (n_elements(bdata.data) eq n_elements(edata.data)) then ewatw=edata.wi
             sans_settings_getmask,size(bdata.data),[0.,0.],M,/nobeam,/noWbeam

             if ((size(bdata.data))[1] eq sans_masq[2]) and ((size(bdata.data))[2] eq sans_masq[3]) then begin
                corW=0
               ;a) Take beam mask from sample if we have a same config (d'nt care collimation!)
                if (abs(sans_supl[32]-sans_supl[1]) le .1) AND (abs(sans_supl[51]-sans_supl[2]) le .1) and $
                   (sans_beam[0]*sans_beam[1] gt 0.) Then begin
                    sans_beam[3:6]=[sans_beam[7],sans_beam[8],sans_beam[0],sans_beam[1]]
                endif else $
               ;b) Find water beam mask using sans_center: substract eWater or not!
                if (sans_trans[0,0]*sans_trans[1,0] gt 0.) and (n_elements(ewatw) eq 1) and (corW) then begin

                   trans=sans_trans[0:3] & trans='['+string(trans[0])+','+string(trans[1])+','+string(trans[0])+','+string(trans[1])+']'
                  ;Water is Sample
                   wws=strtrim(bdata.wi,2) & ews =strtrim(ewatw,2)
                   sans_gt_par ,sys=sys    & sys3=strtrim(sys+3,2)

                   XICUTE,"w"+sys3+"=sans_cor(sample="+wws+",Esample="+ews+",/no_centr,transm="+trans+")"

                           c=sans_center(beam=sys+3   ,mask=M,watermask=baset.curWATER,/fit)
                endif else c=sans_center(beam=bdata.wi,mask=M,watermask=baset.curWATER,/fit)

                sbox =sans_beam[3:6] & stbox=long([sbox[0],sbox[0]+sbox[2]-1,sbox[1],sbox[1]+sbox[3]-1]+FX)
                stbox=strtrim(stbox[0],2)+','+strtrim(stbox[1],2)+','+strtrim(stbox[2],2)+','+strtrim(stbox[3],2)
                sans_settings_swap,baset,waterbox=sbox ,forWater=baset.curWATER ;sans_beam(3:6)
                ;share water for 3 configs?
                if sans_settings_RegSheet() then sans_sheet_receive,waterbox=stbox
                sans_settings_mask,baset
             endif

            endif


         endif
         endif
         if (uv[1] eq 3) or (uv[1] eq 4) then begin
         if baset.bases5[23] gt 0 then begin
         widget_control,baset.bases5[23] ,sens=0  & widget_control,baset.bases5[24] ,sens=0 & endif ;Detector
         widget_control,baset.bases5[21] ,sens=0  & widget_control,baset.bases5[22] ,sens=0
         widget_control,baset.bases5[25] ,sens=1  & widget_control,baset.bases5[26] ,sens=1
         widget_control,baset.bases5[11] ,sens=1
         widget_control,baset.bases5[12] ,sens=1  & widget_control,baset.bases5[13] ,sens=1
         widget_control,baset.bases5[14] ,sens=1  & widget_control,baset.bases5[15] ,sens=1
         endif
   end
56:begin & cur_menu[8]=ev.select & cur_cxy[2]=-1 ;***force Ben to change**
         sans_settings_mask,baset & end ;Enable efficiencies display correction

;CONFIG
60:begin & if  uv[1] eq 0 then  sans_settings, refresh=baset
         widget_control,baset.bases1[7], map=1   & widget_control,baset.bases1[8] ,map=0 & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4], map=1   & widget_control,baset.bases5[5] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=1   & widget_control,baset.bases4[0] ,map=0 & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=0   & widget_control,baset.bases5[3] ,map=1 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[4], set_value='DATA' & cur_menu[[0,1,10]]=[3,4,3]
         widget_control,baset.bases1[12],set_value=' '
         cur_menu[3:5]=0 & sans_settings_mask,baset ;show beam sample,water
         if  uv[1] eq 1 then  sans_settings_read,baset
         end
;OTHER SETTINGS
61:begin & widget_control,baset.bases1[7], map=0   & widget_control,baset.bases1[8] ,map=1 & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4], map=1   & widget_control,baset.bases5[5] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=1   & widget_control,baset.bases4[0] ,map=0 & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=0   & widget_control,baset.bases5[3] ,map=1 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[4], set_value='DATA' & cur_menu[[0,1,10]]=[3,4,3]
         widget_control,baset.bases1[12],set_value=' '
         cur_menu[3:5]=0 & sans_settings_mask,baset ;show beam sample,water

         if n_elements(uv) eq 2 then if uv[1] eq 26 then begin ; read and calculate alternative flux
            sans_supl[60] = 0 ; Flux value to calculate
            sans_settings_runs, baset ,[9,42,26,baset.bases3[41]] ; we are here because (flux) has changed
                               ;                             9  event
                               ;                             42 is bases3 label index (data, datp, ...)
                               ;                             26 is Wi
                               ;                             41 is bases3 text  index (run number)
            widget_control, baset.bases3[42], get_uvalue=sdata
            if sdata.opr le ' ' then begin ; alternative flux is empty! So re-calculate normal sdbeam
               re_r = sans_supl[53]
               sans_supl[53] = 1           ; force re-read sdbeam to calculate PHI0
               sans_settings_runs, baset ,[9,22,9,baset.bases3[16]]
               sans_supl[53] = re_r
            endif
         endif
         end

;ANALYSIS
70:begin & s=size(cur_run) ;Show Q range
         if s[0] ge 2 then begin
            xy0=sans_ctr-1 & if xy0[0] lt 0 then xy0[0]=xy0[0]+2. & if xy0[1] lt 0 then xy0[1]=xy0[1]+2.

            sans_settings_dist, s,Dst,Center=xy0,xyPsize=sans_supl[8:9], DforT=DforT ,thetrad=thetrad
            if n_elements(DforT) gt 1 then maxR=max(DforT) else maxR=max(Dst)
            minR  =min(sans_beam[0:1]*sans_supl[8:9]/2.) ;~if d_angle
            theta2=atan([minR,maxR]    /(sans_supl[1]*100.))
            if n_elements(thetrad) gt 1 then theta2[1]=max(abs(thetrad))
            Qab   =4.*!pi*sin(theta2/2.)/sans_supl[0]
            txtq  =strtrim(string(round(Qab*10000.)/10000.),2)
            widget_control,baset.bases1[12],set_value=' Qrange: '+txtq[0]+' < Q < '+txtq[1]
            widget_control,bmess           ,set_value=' Qrange: '+txtq[0]+' < Q < '+txtq[1]
         endif
         end
71:begin ;LIST OF RESULTS...
         if uv[1] eq 0 then begin
            uv[1]=cur_menu[2] & if where([2,4,5,6,7,8] eq uv[1]) eq -1 then return & endif
         if uv[2] eq 1 then begin
         widget_control,/hourglass
         widget_control,baset.bases1[7], map=0   & widget_control,baset.bases1[8] ,map=1  & widget_control,baset.bases4[16],map=1
         widget_control,baset.bases5[4], map=0   & widget_control,baset.bases5[5] ,map=1  & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=0   & widget_control,baset.bases4[0] ,map=0  & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0  & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=0   & widget_control,baset.bases5[3] ,map=0  & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=0  & widget_control,baset.bases5[8] ,map=0
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=1
         widget_control,baset.bases4[4] ,set_value='LIST OF RESULTS ->W19 [w34,w35,w36]' & cur_menu[[0,1,2,10]]=[19,5,uv[1],-1]
        ;widget_control,baset.bases1(11),set_value='     '
         widget_control,baset.bases1[12],set_value='Use CTRL or SHIFT key to select entries in the LIST'

         gtlist=uv[1] & sans_reform_series,getlist =gtlist
         widget_control,baset.bases4[17],set_value =gtlist
         widget_control,baset.bases8[5] ,set_button=baset.bases6[12]
         widget_control,baset.bases8[9] ,set_button=baset.bases6[16]
         if baset.bases6[13] eq 0 then widget_control,baset.bases8[6] ,set_button=1 ;Q
         if baset.bases6[13] eq 1 then widget_control,baset.bases8[7] ,set_button=1 ;Log Q
         if baset.bases6[13] eq 2 then widget_control,baset.bases8[8] ,set_button=1 ;Q^2
         if baset.bases6[13] eq 3 then widget_control,baset.bases8[10],set_button=1 ;Q^4
         cur_menu[9]=uv[1]
         if n_elements(gtlist) ge 1 then begin & widget_control,baset.bases4[17],set_list_select=0
            if strpos( gtlist[0],'None') eq 0 then begin
            kpwin=!D.window & wset,baset.bases1[14] & erase & wset,kpwin
            endif
         endif
         endif
         ;CLEAR
         if uv[2] eq 0 then begin ; Clear all
            sans_reform_series,clear=cur_menu[2]  ; uv(1)!!!
            kpwin=!D.window & wset,baset.bases1[14] & erase & wset,kpwin
            for i=0,5 do XICUTE,(['W31','W32','W33','W34','W35','W36'])[i]+'=0'
            widget_control,baset.bases4[17],set_value='None'
            sans_merge_fields, baset, clear=[1,1,1 ,1] ;Clear all merging fields [L,M,H ,4th]
         endif
         
         if uv[2] eq 1 then sans_merge_fields, baset, show=0 ;We may have to fill merge fields with new values
         
         if uv[2] eq 2 then begin ; Clear selected
            idx=widget_info(baset.bases4[17],/list_select)
            if idx[0] ge 0 then begin
              list=cur_menu[2] ; RADIAL or AZI
              sans_del_select_results, getlist=list, index=idx
              if (size(list,/type)) eq 7 then $
                widget_control,baset.bases4[17],set_value=list
            endif
         endif
         end

72:begin;CORRECTIONS (INTEGRATIONS)

         widget_control,ev.top,map=1,/hourglass
         kperr=!error_state.msg
                                   ;[9,35,0,baset.bases3(29:34)]     alternative Water
         sans_settings_runs, baset ,[9,6 ,0,baset.bases3[0 : 5]], /HaveAll ;Have Water   and Data

         if (kperr eq !error_state.msg) or (!error_state.msg le ' ') then begin
        ;HAVE DATA AND PARAMETERS
        ; -----------------------
         keepUV = uv
         if uv[1] lt 4 then begin & m1=1 & m2=0
         endif         else begin & m1=0 & m2=1
                                  widget_control,baset.bases7[7] ,get_value=asl   & sans_supl[13]=asl          ;Total rotation
                                  widget_control,baset.bases7[8] ,get_value=gsl   & sans_supl[14]=gsl          ;Single rotation
                                  widget_control,baset.bases7[9] ,get_value=bsl   & sans_supl[15]=bsl          ;Opening
                                  widget_control,baset.bases7[10],get_value=zonID & sans_supl[16]=zonID        ;# of sectors
                                  widget_control,baset.bases7[11],get_value=r1ID  ; r1ID=round(r1ID/cur_fct[0]);X factor!
                                                                                    sans_supl[17]=r1ID         ;Inner radius
                                                                                    r1ID=round(r1ID/cur_fct[0]);
                                  widget_control,baset.bases7[12],get_value=r2ID  ; r2ID=round(r2ID/cur_fct[0]);X factor!
                                                                                    sans_supl[18]=r2ID         ;Outer radius
                                                                                    r2ID=round(r2ID/cur_fct[0]);relation pixel detector - pixel image
         endelse
         widget_control,baset.bases1[7], map=0   & widget_control,baset.bases1[8] ,map=1  & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4], map=m1  & widget_control,baset.bases5[5] ,map=0  & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=1   & widget_control,baset.bases4[0] ,map=0  & widget_control,baset.bases4[7] ,map=0
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0  & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=0   & widget_control,baset.bases5[3] ,map=0  & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=m2 & widget_control,baset.bases5[8] ,map=m2
         widget_control,baset.bases5[9], map=m1  & widget_control,baset.bases5[19],map=0
         if m1 then begin
         widget_control,baset.bases4[4] ,set_value='OPTIONS FOR DETECTOR CORRECTIONS'  & cur_menu[0]=9 & endif
        ;widget_control,baset.bases1(11),set_value='     '
         widget_control,baset.bases1[12],set_value='     '

         widget_control,  baset.bases3[6]  ,get_uvalue=wdata  & wws =strtrim(string(wdata.wi) ,2)
         widget_control,  baset.bases3[7]  ,get_uvalue=ewdata & ewws=strtrim(string(ewdata.wi),2)
         widget_control,  baset.bases3[8]  ,get_uvalue=sdata  & sws =strtrim(string(sdata.wi) ,2)
         widget_control,  baset.bases3[9]  ,get_uvalue=esdata & esws=strtrim(string(esdata.wi),2)
         widget_control,  baset.bases3[10] ,get_uvalue=csdata & csws=strtrim(string(csdata.wi),2)
         widget_control,  baset.bases3[11] ,get_uvalue=cwdata & cwws=strtrim(string(cwdata.wi),2)

         widget_control,baset.bases7[14],get_value=radin & if strpos(radin[0],'*') lt 0 then sans_supl[20]=radin
        ;widget_control,baset.bases7(15),get_value=Phi0W & if strpos(Phi0W(0),'*') lt 0 then sans_supl(35)=Phi0W
         widget_control,baset.bases7[16],get_value=Tauv  & if strpos(Tauv[ 0],'*') lt 0 then sans_supl[6] =Tauv
         widget_control,baset.bases7[6] ,get_value=crosW & if strpos(crosW[0],'*') lt 0 then sans_supl[42]=crosW
         widget_control,baset.bases2[20],get_value=thinW & if strpos(thinW[0],'*') lt 0 then sans_supl[10]=thinW
         widget_control,baset.bases2[21],get_value=thinS & if strpos(thinS[0],'*') lt 0 then sans_supl[11]=thinS

         bases=[baset.bases2[0:1],baset.bases2[6:7]] ;in case wavl,dista,offset,angle changed in config (no <c.r>)
         sans_settings_text, bases, farr, sarr, mess & sans_supl[0:3]=farr

        ;SEARCH FOR TRANSMISSIONS
        ; -----------------------
        if uv[1] eq 0 then begin & gettrs=strarr(4)
           if wdata.opr  gt ' ' then gettrs[0]= wdata.par[0]
           if ewdata.opr gt ' ' then gettrs[1]=ewdata.par[0]
           if sdata.opr  gt ' ' then gettrs[2]= sdata.par[0] & sis=n_elements(sdata.par)
           if esdata.opr gt ' ' then gettrs[3]=esdata.par[0]
           if sis gt 1 then begin & gettrs=[gettrs,sdata.par]  & pZ=sdata.datp.Z & endif else pZ=0
           if n_elements(pZ) ne sis then pZ=findgen(sis)

           sans_reform_series,gettrs=gettrs,idxZ=pZ

           widget_control,baset.bases1[12],set_value=gettrs[0]
           if strpos(gettrs[0],'No') ne 0 then begin
              transm=string(sans_trans[0:3,0],format='(F5.3)')
              txt1water='  '+baset.txtwater & txt2water =txt1water+'  cell'
              widget_control,baset.bases2[11],set_value =transm[0]+txt1water
              widget_control,baset.bases2[12],set_value =transm[1]+txt2water
              widget_control,baset.bases2[13],set_value =transm[2]+'  Sample 1..'
              widget_control,baset.bases2[14],set_value =transm[3]+'  Sample cell'
              widget_control,baset.bases2[15],set_value =transm[0]+txt1water
              widget_control,baset.bases2[16],set_value =transm[1]+txt2water
              widget_control,baset.bases2[17],set_value =transm[2]+'  Sample 1..'
              widget_control,baset.bases2[18],set_value =transm[3]+'  Sample cell'
              if sans_settings_RegSheet() then sans_sheet_receive, trans=sans_trans ,mess=gettrs[0] $
              else begin & widget_control,ev.top,set_uvalue=baset
                         ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:baset.bases9[13],TOP:ev.top,HANDLER:0L,SELECT:1}')
              endelse
           endif
        endif
        if uv[1] ge 1 then begin

        ;SHOW EFFICIENCIES
        ; ----------------
        if uv[1] eq 3 then begin & cur_eff=1. & tmpcor=0
          if wdata.opr gt ' ' then begin ;We have Water
            cmd='wat='  + wws +',/efficienc'
            if ewdata.opr gt ' ' then cmd=cmd+',Ewat=' +ewws
            if cwdata.opr gt ' ' then cmd=cmd+',wcadm='+cwws
            if not sans_supl[61] and (baset.Qdep gt ' ') then cmd=cmd+",Qdep='"+baset.Qdep+"'"

            sans_gt_par ,sys=sys & sys3=strtrim(sys+3,2)

            XICUTER,'w17=sans_cor('+cmd+',message=par'+sys3+')'
            TAKE_W,tmpcor,W=17
            if (strpos(cmd,'Ewat') gt 0) and (sans_trans[0,0] ne sans_trans[1,0]) then begin
                cur_eff=tmpcor & endif ;widget_control,baset.bases5(20),sens=1 & endif
            tit11 = strmid(wdata.datp.w_tit+' ('+wdata.opr+' ->W17)',0,55)
            
          endif else begin ; else we may have a calibration file
            CALIBRATION ,GET_CAL=tmpcor, INF=f_inf
            if (n_elements(tmpcor) gt 1) and (n_elements(f_inf) eq 1) then begin
                XICUTE,'w17="'+f_inf+'"'
                GIVE_W,tmpcor,W=17
                tit11 = strmid(f_inf+' (->W17)',0,55)
            endif
          endelse
          if n_elements(tmpcor) gt 1 then begin & cur_wn=17 & cur_menu[7]=17
                 sans_settings_mask , baset, run=(tmpcor) ;,/nobeam
                 widget_control,baset.bases4[4] ,set_value='EFFICIENCIES' & cur_menu[1]=4
                 widget_control,baset.bases1[11],set_value=tit11
                 widget_control,baset.bases1[12],set_value=' '
                 SEE,W=17,/below,/image & to_did_cur,'W17'
          endif

        ;DO CORRECTIONS
        ; -------------
        endif else if sdata.opr gt ' ' then begin & widget_control,baset.bases1[12] ,set_value ='CORRECTIONS ...'
           savtrs=strarr(4)
           cmd='sam='+sws & geteff=',geteff=W17'                  & savtrs[2]= sdata.par[0]
           if wdata.opr  gt ' ' then begin & cmd=cmd+',wat='  + wws & savtrs[0]= wdata.par[0] & endif else geteff=''
           if ewdata.opr gt ' ' then begin & cmd=cmd+',Ewat=' +ewws & savtrs[1]=ewdata.par[0] & endif
           if esdata.opr gt ' ' then begin & cmd=cmd+',Esam=' +esws & savtrs[3]=esdata.par[0] & endif
           if csdata.opr gt ' ' then       cmd=cmd+',scadm='+csws
           if cwdata.opr gt ' ' then       cmd=cmd+',wcadm='+cwws
           sis=n_elements(sdata.par)
           if sis gt 1 then savtrs=[savtrs,sdata.par]
           if not sans_supl[61] and (baset.Qdep gt ' ') then cmd=cmd+ ",Qdep='"+baset.Qdep+"'"
           if baset.XWdep gt ' ' then cmd=cmd +",XWdep='"+baset.XWdep+"'"

           suplMASK = ''
           PAR20    = [-1]
           if sans_supl[66] gt 1 then begin ;NEGATIVE COUNTS AT READ TIME MUST BE MASKED
              par20 = where(SData.data[*,*,0] lt 0.)
              if par20[0] ne -1 then suplMASK = ',suplMASK=par20' & endif ;c_lamp_par is defined in sans_settings_event

           sans_gt_par ,sys=sys & sys3=strtrim(sys+3,2)

           XICUTE,'w18=sans_cor('+cmd+geteff+',message=par'+sys3 +suplMASK+')'
           par18 = par12 ;Transfer also all titles defined in par12 and not in DATP (reading wrksp is W12) (common c_lamp_par is defined).

           sans_gt_par ,3 ,get=resMess
           if resMess gt ' ' then widget_control,baset.bases1[11],set_value=strmid(resMess,0,50)
           TAKE_W,tmpcor, W=18

           if n_elements(tmpcor) gt 1 then begin

              pZ=sdata.datp.Z & if n_elements(pZ) ne sis then pZ=findgen(sis)
              sans_reform_series,savtrs=savtrs,idxZ=pZ                 ;Keep transmissions, we must have thickness

              szCOR   = size(tmpcor) & if szCOR[0] eq 3 then sz=szCOR[3]-1 else sz=0
              noCount =-1e-19
              if (sans_supl[64] ne 1) and (sans_supl[64] ne 3) then begin ;not TOF neither KINETIC
              ;NEW: mask here the corrected 2d result for any use! (17 Oct 2014)
               sans_settings_getmask, szCOR ,[0.,0.] ,M ,/noWbeam
               idxM = where( M eq 1 ,complement=idxNoM)
               if (idxM[0] ge 0) or (PAR20[0] ge 0) then begin
                 for iz = 0,sz do begin
                   tmpD = tmpcor[*,*,iz]
                   minD = min(tmpD[idxNoM]) + noCount
                   if idxM[0]  ge 0 then tmpD[idxM]  = minD ; mask beam and bad detectors
                   if PAR20[0] ge 0 then tmpD[par20] = minD ; mask negative counts coming from readin procedure
                   tmpcor[*,*,iz] = tmpD
                 endfor
                 GIVE_W,tmpcor, W=18
               endif
              endif
              PAR18 = resMess                                          ;c_lamp_par is defined in sans_settings_event
              widget_control,baset.bases7[6],set_value=strtrim(string(sans_supl[42],format='(f6.3)'),2) ;Water Xsection
              if uv[1] eq 1 then begin
                 sans_settings_runs, baset ,[9,8,12,baset.bases3[2]],/noread   ;Show original data (no re_read)

                 cur_wn=18 & cur_menu[7]=18
                 widget_control,baset.bases9[11],get_value=vz          ;Get slider value
                 sans_settings_mask , baset, run=tmpcor[*,*,vz<sz]     ;Show corrected data
                 widget_control,baset.bases4[4] ,set_value='CORRECTED ->W18 [w31,w32,w33]' & cur_menu[1]=4
                 if sz eq 0  then title=sdata.datp.w_tit else title=sdata.par[vz<sz]
                 if sz gt 0  then begin & svz=strtrim(vz<sz,2) & XICUTER,'W16=W18[*,*,'+svz+']'
                                                               XICUTE ,'W_tit[16]="<'+title+'>"' & endif
                 title=strmid(strcompress(title)+' ('+sdata.opr+' ->W16)',0,55)
                 widget_control,baset.bases1[11],set_value=title
                 widget_control,baset.bases1[12],set_value=' '
                 if uv[1] eq 1 then if sz eq 0 then SEE,W=18,/below,/image else SEE,W=18,/below,/image
                 to_did_cur,'W18'
              endif
              WsCor=(['18','31','32','33'])[(baset.curLMH)>1]
              XICUTER,'W'+WsCor+'=W18' ;Dispatch results in workspace and c_sans_cur_Axy common
              cur_SamCor=SData & cur_SamCor.Wi=fix(WsCor) & cur_SamCor.DATA=tmpcor

              if (strpos(cmd,'Ewat') gt 0) and (sans_trans[0,0] ne sans_trans[1,0]) then begin
                  TAKE_W,cur_eff,W=17 & if n_elements(cur_eff) gt 1 then sans=1 else sans=0
                 ;widget_control,baset.bases5(20),sens=sans
              endif else cur_eff=1.

             ;DO INTEGRATIONS
             ; --------------        
              if uv[1] ge 2 then begin & widget_control,baset.bases1[12] ,set_value ='INTEGRATIONS ...'
                 sans_settings_text, baset.bases7[14],farr & incr=farr[0]>.5  ;Get radius increment
                 sincr    =',incr='+strtrim(string(incr),2)
                 sans_supl[21]=uv[1]
                 rocking=0
                 
                 dL_L=sans_supl[50] & Lambda=sans_supl[0] & dTh=sans_supl[49] ;To calculate dQ in Z19

                 if uv[1] eq 2 then begin                                     ;360  RADIAL INTEGRATIONS
                    XICUTE,'w19=sans_radial(W18,minInt=5'+sincr+suplMASK+')'  ;************************
                    TAKE_W,tmprad,W=19 & TAKE_DATP,rpa,W=19 & Ysec=rpa.Y
                                                              Xsec=rpa.X
                   ;dQ is calculated in sans_radial
                   ;dQ = sans_sigmaQ(X=rpa.X , dL_L=dL_L , Lambda=Lambda , dTh=dTh)
                   ;MOD_DATP ,rpa,'Z',dQ

                    if (size(tmprad))[0] eq 2   then  rpa.y_tit='Numors'
                    GIVE_DATP,rpa,W=19
                 endif ;Radial integration ->W19

                 if uv[1] eq 4 then begin                                     ;360  AZIMUTAL INTEGRATIONS
                    r1=min([r1ID,r2ID],max=r2)                                ;**************************
                    sector='['+string(r1)+','+string(r2)+']'
                    XICUTE,'w19=sans_radial(W18,sector='+sector+sincr+suplMASK+',/AZIMUTAL)'
                    TAKE_W,tmprad,W=19 & TAKE_DATP,rpa,W=19 & Ysec=rpa.Y
                                                              Xsec=rpa.X
                    if (size(tmprad))[0] eq 2   then  rpa.y_tit='Numors'
                 endif ;Azimutal integration ->W19

                ;************************                                      *******************
                 if uv[1] ge 5 then begin                                     ;SECTOR INTEGRATIONS
                ;************************                                      *******************
                    rocking = baset.bases6[22]    ; or UV(2)?
                    if (rocking gt 0) and (szCOR[0] eq 3) then begin
                       if uv[1] eq 7 then uv[1]=5 ;Force radial  sym -> total  sym
                       if uv[1] eq 8 then uv[1]=6 ;Force radial asym -> total asym
                    endif else rocking=0
                    
                    r1=min([r1ID,r2ID],max=r2) & cur=asl
                    if zonID eq 2 then car=gsl else car=180./zonID
                    if (r1 eq r2) then if (r2 eq 0) then r2=1 else r1=r1-1
                    squared=strtrim(long(baset.bases6[21]),2)
                    if (uv[1] eq 5) or (uv[1] eq 6) then totsec='1' else totsec='0'
                    for k=0,(zonID-1) do begin
                      for p=0,1 do begin
                          bet1=(cur-bsl+360.+ p*180.) mod 360
                          bet2=(cur+bsl     + p*180.) mod 360
                          kpbet1=bet1
                          kpbet2=bet2
                          sector='['+string(r1)+','+string(r2)+','+string(bet1)+','+string(bet2)+', '+squared+','+totsec+']'

                          XICUTE,'w19=sans_radial(W18,sector='+sector+sincr+suplMASK+')'
                          TAKE_W,tmpsec,W=19 & TAKE_DATP,rpa,W=19 & nn=n_elements(rpa.x)  & tmpY=cur+p*180

                          if (uv[1] eq 5) or (uv[1] eq 6) then begin &  tmpsec=transpose([total(tmpsec,1)/nn])
                           tmpE=transpose([sqrt(total(rpa.e^2,1))/nn])   & tmpX=tmpY & tmpy=total(rpa.x)/nn
                          endif else begin & tmpE=rpa.e          & tmpX=rpa.x & endelse

                          if (p eq 0) then begin & tm0sec=tmpsec & tm0E=tmpE  & tm0X=tmpX & tm0Y= tmpY
                          endif else begin
                           if uv[1] eq 5 then begin ;**** SECTOR TOTAL  SYMMETRICAL     <----------------------<<
                              idxt=where( ((tm0sec ne -1E-9) and (tmpsec ne -1E-9)) or $
                                          ((tm0sec eq -1E-9) and (tmpsec eq -1E-9)) )
                              tmpsec=(tm0sec+tmpsec)>0.  & tmpE=sqrt(tm0E^2+tmpE^2)    & tmpX= tm0X
                              if idxt[0] ge 0 then begin & tmpsec[idxt]=tmpsec[idxt]/2
                                                         tmpE[  idxt]=tmpE[  idxt]/2 & endif & endif
                                                         
                           if uv[1] eq 6 then begin ;**** SECTOR TOTAL  NON_SYMMETRICAL <----------------------<<
                              tmpsec=[tm0sec,tmpsec]>0.  & tmpE=    [tm0E  ,tmpE  ]    & tmpX=[tm0X,tmpX] & endif
                              
                           if uv[1] eq 7 then begin ;**** SECTOR RADIAL SYMMETRICAL     <----------------------<<
                              idxt=where( ((tm0sec ne -1E-9) and (tmpsec ne -1E-9)) or $
                                          ((tm0sec eq -1E-9) and (tmpsec eq -1E-9)) )
                              tmpsec=(tm0sec+tmpsec)>0.  & tmpE=sqrt(tm0E^2+tmpE^2)
                              if idxt[0] ge 0 then begin & tmpsec[idxt]=tmpsec[idxt]/2
                                                         tmpE[  idxt]=tmpE[  idxt]/2 & endif & endif
                                                         
                           if uv[1] eq 8 then begin ;**** SECTOR RADIAL NON_SYMMETRICAL <------------------....<<
                              tmpsec=[[[tm0sec]],[[tmpsec]]]>0. & tmpE=[[[tm0E]],[[tmpE]]] & tmpY=[tm0Y,tmpY]
                           endif
                           
                          ;KEEP SECTOR RESULTS
                          ; ------------------
                           tmptot=total(tmpsec)/n_elements(tmpsec)
                           if (tmptot ne -1E-9) and (tmptot ne 0.) then begin
                            if n_elements(tmprad) eq 0 then begin & tmprad=tmpsec & Esec=tmpE & Xsec=tmpX & Ysec=tmpY
                            endif else begin
                             if (uv[1] eq 5) or (uv[1] eq 6) then begin
                                tmprad=[tmprad,tmpsec] & Esec= [Esec,tmpE]  & Xsec=[Xsec,tmpX] & endif
                             if (uv[1] eq 7) or (uv[1] eq 8) then begin
                                tmprad=[[[tmprad]],[[tmpsec]]] & Esec=[[[Esec]],[[tmpE]]] & Ysec=[Ysec,tmpY] & endif
                            endelse
                           endif
                          endelse ;P=1
                     endfor & cur=cur+car
                   endfor

                   if n_elements(tmprad) gt 0 then begin

                   tmprad=reform(tmprad) & sz=size(tmprad) & Esec=reform(Esec) & KPYsec=Ysec & KPXsec=Xsec

                   ; ---SECTOR TOTAL (RE-ARANGE AND PUT TITLES)
                   if (uv[1] eq 5) or (uv[1] eq 6) then begin
                     if (sz[0] eq 1) and (szCOR[0] eq 3) then begin ;ROCKING ONE SECTOR (what else!)
                       X_tit ='Numors'  & Y_tit='Sect'+string(Xsec[0],format='(F5.1)')
                       KPYsec=Xsec[0]
                       Xsec  =indgen(sz[1])+1 & rpa.x_tit=X_tit & rpa.y_tit=Y_tit
                       KPXsec=Xsec
                       if rocking eq 0 then rocking=1
                    ;End rocking one sector
                    
                     endif else begin
                       if uv[1] eq 6 then begin & idx=sort(Xsec) & Xsec=Xsec[idx] & Esec=Esec[idx,*]
                                                tmprad=tmprad[idx,*] & KPXsec=Xsec & endif   ;Sort if non_symmetrical
                       rpa.x_tit='Sectors (deg)' & rpa.y_tit='Total'
                       if sz[0] eq 2 then begin
                         rpa.z_tit='Total'
                         if (rocking gt 0) and (szCOR[0] eq 3) then begin    ;ROCKING MANY SECTORS (TO TEST !!!!)
                           X_tit='Numors'  & Y_tit='Sectors (deg)'
                           rpa.x_tit=X_tit & rpa.y_tit=Y_tit
                           tmprad=transpose(tmprad) & Esec=transpose(Esec)
                           Ysec=Xsec     & Xsec=indgen(sz[2])+1
                           KPXsec=Xsec & KPYsec=Ysec
                        ;End rocking many sectors
                        
                         endif else begin
                           Ysec=findgen(sz[2])+1
                           rpa.y_tit='Numors'
                         endelse
                       endif
                     endelse
                   endif
                   ; ---SECTOR RADIAL (RE-ARANGE AND PUT TITLES)
                   if (uv[1] eq 7) or (uv[1] eq 8) then begin
                    if sz[0] gt 1 then tmp=total(tmprad,2)/sz[2] else tmp=tmprad    ;reduce to non_masked data
                    if sz[0] gt 2 then tmp=total(tmp   ,2)/sz[3]                    ;            ""
                    idx=where(tmp ne -1E-9) & idx1=idx[0]>0 & idx2=idx[n_elements(idx)-1]>0 ;    ""
                    tmprad=tmprad[idx1:idx2,*,*] & Esec=Esec[idx1:idx2,*,*]                 ;    ""
                    Xsec=Xsec[idx1:idx2]         & sz=size(tmprad) & KPXsec=Xsec            ;    ""

                   ;if (uv(1) eq 8) and (sz(0) le 2) then begin idy=sort(Ysec) & Ysec=Ysec(idy) ;Sort if non_symmetrical BOF!
                   ;   tmprad=tmprad(*,idy,*) & Esec=Esec(*,idy,*) & KPYsec=Ysec & endif

                    if (n_elements(Ysec) eq 1) and (sz[0] eq 1) then begin         ; one  sector
                        rpa.y_tit='S[Q]'    & endif else $
                    if (n_elements(Ysec) eq 1) and (sz[0] eq 2) then begin         ; one  sector  + series
                        rpa.y_tit='Numors'  & rpa.z_tit='S[Q]'
                        Ysec  =findgen(sz[2])+1 + Ysec/1000.
                        KPYsec=Ysec & endif else $
                    if (n_elements(Ysec) gt 1) and (sz[0] eq 2) then begin         ; many sectors
                        rpa.y_tit='Sectors' & rpa.z_tit='S[Q]' & endif else $
                    if (n_elements(Ysec) gt 1) and (sz[0] eq 3) then begin         ; many sectors + series
                        tmprad=transpose(tmprad,[0,2,1])
                        Esec  =transpose(Esec  ,[0,2,1])

                        tmprad=reform(tmprad,sz[1],sz[2]*sz[3])
                        Esec  =reform(Esec  ,sz[1],sz[2]*sz[3])
                        rpa.y_tit='Numors by sectors'   & rpa.z_tit='S[Q]'
                        for i=2L,sz[2] do KPYsec=[KPYsec,Ysec]
                        Ysec  =findgen(sz[2]*sz[3])+1   & endif
                   endif
                   if (uv[1] eq 7) or (uv[1] eq 8) then begin
                     dQ = sans_sigmaQ(X=rpa.X , dL_L=dL_L , Lambda=Lambda , dTh=dTh)
                     MOD_DATP ,rpa,'Z',dQ
                   endif

                   sz=size(tmprad)
                   if rocking ge 1 then begin
                    if rocking eq 2 then begin & rpa.X_tit='Numors Time'
                                               KPXsec=reform(rpa.pv[idxp[5] ,*])
                             for i=1L,sz[1]-1 do KPXsec[i]=KPXsec[i-1]+KPXsec[i] & endif
                    if rocking eq 3 then begin & rpa.X_tit='Numors Temperature'
                                               KPXsec=reform(rpa.pv[idxp[11],*]) & endif
                    if rocking eq 4 then begin & rpa.X_tit='Numors Rotation'
                                               KPXsec=reform(rpa.pv[idxp[10],*]) & endif
                    if total(KPXsec)/sz[1] eq KPXsec[0] then KPXsec=lindgen(sz[1])+1 ;Miss
                    Xsec=KPXsec
                   endif

                   MOD_DATP,rpa,'E',Esec & MOD_DATP ,rpa,'X',KPXsec & MOD_DATP,rpa,'Y',KPYsec
                   GIVE_W,tmprad,W=19    & GIVE_DATP,rpa,W=19

                 endif ;TMPRAD NOT EMPTY
                ;*****  *************************
                 endif ;SECTOR INTEGRATIONS ->W19
                ;*****  *************************

                 sz=size(tmprad)
                ;PLOT RESULTS
                ; -----------
                if sz[0] ge 1 then begin
                    widget_control,baset.bases5[4] ,map=0   & widget_control,baset.bases5[5],map=1
                    widget_control,baset.bases4[4] ,set_value='AVERAGING CURVE(s) ->W19 [w34,w35,w36]' & cur_menu[1:2]=[5,uv[1]]
                    kpwin=!D.window
                   wset,baset.bases1[14]
                   ilog=baset.bases6[8]
                   Ierr =baset.bases6[12]
                   if sz[0] eq 1 then begin
                      xma=max(Xsec,min=xmi) & xdif=1e-5 & yma=max(tmprad,min=ymi)  & ydif=1e-5
                      xrg =[xmi-xdif,xma+xdif]>0.        & yrg =[ymi-ydif,yma+ydif] & if ilog then yrg=[yrg[0]>(1e-7) , yrg[1]>(1e-5)]
                      if ilog then  plot   ,Xsec ,tmprad,xmargin=[7,1],ymargin=[3,1],title='',xtitle=rpa.x_tit,line=1,psym=1,xrang=xrg,yrang=yrg,ytitle='Log '+rpa.y_tit,/ylog $
                              else  plot   ,Xsec ,tmprad,xmargin=[7,1],ymargin=[3,1],title='',xtitle=rpa.x_tit,line=1,psym=1,xrang=xrg,yrang=yrg,ytitle=rpa.y_tit
                      if Ierr then  errplot,Xsec ,tmprad+rpa.e,tmprad-rpa.e
                   endif
                   if sz[0] eq 2 then begin & ilog=0
                     if sz[2] lt 100 then begin
                      if ilog then  surface,   tmprad>1e-7,Xsec,Ysec,az=10.,ax=70.,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=rpa.x_tit,title=rpa.y_tit,ztitle='log '+rpa.z_tit,/Zlog,/hori $
                      else          surface,   tmprad     ,Xsec,Ysec,az=10.,ax=70.,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=rpa.x_tit,title=rpa.y_tit,ztitle=rpa.z_tit,/hori
                     endif else begin
                      if ilog then  shade_surf,tmprad>1e-7,Xsec,Ysec,az=10.,ax=70.,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=rpa.x_tit,title=rpa.y_tit,ztitle='log '+rpa.z_tit,/Zlog $
                      else          shade_surf,tmprad     ,Xsec,Ysec,az=10.,ax=70.,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=rpa.x_tit,title=rpa.y_tit,ztitle=rpa.z_tit
                     endelse
                   endif
                   trap,current=!D.window
                   wset,kpwin

                   if sz[0] eq 1 then SEE,W=19,/below else SEE,W=19,/below,/surface ;show also in lamp
                   to_did_cur,'W19'
                   WsInt=(['19','34','35','36'])[(baset.curLMH)>1]
                   XICUTER,'W'+WsInt+'=W19' ;Dispatch results in config workspace

                   if sans_settings_RegSheet() then sans_sheet_receive, mess=Sample_IDs, /give_IDs

                  ;KEEP RESULTS FOR RESUMING
                  ; ------------------------
                  ;                    W19     datp
                   sans_reform_series, tmprad, dp=rpa, titles=sdata.par, idxZ=sdata.datp.Z, parconf=sdata.parconf, resume=uv[1], S_IDs=Sample_IDs
                    
                  ;CLEAR CORRECT MERGING FIELDS
                  ; ---------------------------
                   Clmh=[0,0,0 , 0] & Clmh[(baset.curLMH-1)>0]=1 ;[L,M,H , 4th unused here]
                   sans_merge_fields, baset, clear=Clmh
                   
                  ;SHOW PLOT LIST (not necessary for sans_sheet)
                  ;--------------
                   if not sans_settings_RegSheet() then begin
                     widget_control, ev.id, set_uvalue=[71,uv[1],1] ;New uvalue for current event
                     sans_settings_event,ev                         ;re-enter in this event handler
                     widget_control, ev.id, set_uvalue=keepUV       ;re-load correct uvalue
                   endif
                   
                endif ;PLOT RESULTS

              endif ;DO INTEGRATIONS
           endif ;tmpcor ok so do corrections
         endif ;DO CORRECTIONS

         endif ;DO CORRECTIONS OR INTEGRATIONS
         endif ;read error
         end
73:      DON_DO_CMD

;BATCH MACROS
74:begin & widget_control,baset.bases1[7] ,map=0   & widget_control,baset.bases1[8] ,map=1
         widget_control,baset.bases5[4] ,map=1   & widget_control,baset.bases5[5] ,map=0
         if uv[1] gt 0  then begin
         widget_control,baset.bases4[1] ,map=0   & widget_control,baset.bases4[0] ,map=0 & widget_control,baset.bases4[7] ,map=0 & endif
         widget_control,baset.bases4[5] ,sens=1  & widget_control,baset.bases4[6] ,sens=1
         widget_control,baset.bases4[14],sens=1  & widget_control,baset.bases4[15],sens=1
         widget_control,baset.bases4[3] ,map=0   & widget_control,baset.bases4[2] ,sens=1,map=1
         if uv[1] eq 1  then  widget_control,baset.bases4[0],map=1
         if uv[1] eq 2  then  widget_control,baset.bases4[7],map=1
         if uv[1] eq 3  then  widget_control,baset.bases4[1],map=1
         widget_control,baset.bases5[0] ,map=0   & widget_control,baset.bases5[1] ,map=0 & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2] ,map=0   & widget_control,baset.bases5[3] ,map=0 & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6] ,map=1   & widget_control,baset.bases5[7] ,map=0 & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases5[8] ,map=0   & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[9] ,map=0   & widget_control,baset.bases5[19],map=0
         widget_control,baset.bases4[4] ,set_value='BATCH MACROS' & cur_menu[0:1]=[6,4]
         widget_control,baset.bases1[12],set_value=' '
         end
75:      suprplot, cur_wn

76:begin & idx=widget_info(ev.id,/list_select) ;PLOT LIST OF RESULTS

         on_ioerror,mis76
         P_did_get_etc, d_etc, d_repr
         if d_repr[21] then begin & tvlct,  0    ,  0    ,  0    ,255 ; Force Black & White axes
                                    tvlct,255    ,255    ,255    ,  0 & endif

         if ev.index eq -100 then begin & ev.index=0 & widget_control, uv[15] , get_value=bcbg
                                        bcbg=float(strtrim(bcbg[0],2)) ;background is uses comming from Mergin Tab ...
         endif               else       bcbg=0.

         if ev.index lt 0 then toW21=abs(ev.index) else toW21=0
         if (toW21 ge 3)  and (toW21 le 5) then toW=22 else toW=21 & toWs=strtrim(toW,2)
         if idx[0] ge 0 then begin
         
          widget_control,/hourglass
          widget_control,baset.bases1[3],get_value=instru
          
         ;GET FIRST CURVE
         ;***************
          rpa=cur_menu[2] & sans_reform_series,getlist=rpa,index=idx[0]
          
          curmenu=cur_menu[2]
          if (curmenu eq 2) or (curmenu eq 7) or (curmenu eq 8) then curmenu=2 ;RADIAL
          if (curmenu eq 4) or (curmenu eq 5) or (curmenu eq 6) then curmenu=4 ;AZIMUTAL

         ;TOW21 0 PLOT
         ;      1 PLOT    -> W21              unused!
         ;      2         -> W21            + Suprplot
         ;      3         -> W22 with merge                   + dQ in Z22
         ;      4         -> W22 with merge + Suprplot        + dQ in Z22
         ;      5 PLOT dQ -> W23 with merge + W22             + dQ in Z22 + dQ in W23 (-1 no   model)
         ;      5 PLOT    -> W22 with merge + FIT (no F[I,Q]) + dQ in Z22 + dQ in W23 (0+ with model)
         ;      6         -> W21            + surface
         ;      7         -> W21            + surface time scaling
         
          if n_tags(rpa) gt 1 then begin & Xtit='' & Ytit='I'
     
           ;INIT Toggles and Titles
           ;***********************
            if curmenu eq 2 then         Xtit='Q'       & rocking=0
            if curmenu eq 4 then begin &   Xtit='Degrees'
               Rck=strpos(rpa.TIT,'.Rck')
               if Rck gt 0 then begin &    Xtit='Unit'    & rocking=1
                  Rck=strmid(rpa.TIT,Rck+4,1)
                  if Rck eq 1 then Xtit='Index' else if Rck eq 2 then Xtit='Time'
                  if Rck eq 3 then Xtit='Temp'  else if Rck eq 4 then Xtit='Rot'
               endif
            endif
            ilog =baset.bases6[8]    &   b11tit=strmid(rpa.TIT,0,55)
            styl =baset.bases6[16]   &   SIDs  =''
            discd=baset.bases6[18]   &   F4=4.
            Ierr =baset.bases6[12]   &   Qlog=0    & Qsqr=0
            IofQ =baset.IofQ         &   symPZ =1L & rdL =1L
            FIQ  =0
            FIQf =0
            QrgON=baset.Qrange[0]
            IrgON=baset.Irange[0]
            PrtON=baset.PrintON
            Itool=baset.ITOOLS
            RepSt=baset.RepStyle
            Uscal=1-sans_supl[44]
            sym  =[1 ,6  ,7  ,5 ,2, 3  ,4  ]
            cool =[50,100,200,30,75,150,230]
            if curmenu eq 2 then begin
               if baset.bases6[13] eq 3 then begin & Xtit ='Q^4'    & Qsqr=2 & endif
               if baset.bases6[13] eq 2 then begin & Xtit ='Q^2'    & Qsqr=1 & endif
               if baset.bases6[13] eq 1 then begin & if toW21 ne 2 then $            ;Not for Suprplot
                                                   Xtit ='log(Q)' & Qlog=1 & endif
               if IofQ eq 1 then Ytit='IQ'
               if IofQ eq 2 then Ytit='IQ^2'
               if IofQ eq 3 then Ytit='IQ^4'
               if ilog then if toW21 ne 2 then Ytit='Log '+Ytit                    ;Not for Suprplot
            endif
            Xtitp=Xtit
            Ytitp=Ytit

            widget_control,baset.bases5[4] ,map=0 & widget_control,baset.bases5[5],map=1 & cur_menu[1]=5 ;be sure that plot window
            widget_control,baset.bases5[19],map=1                                                        ;and TABS are visible
            widget_control,baset.bases1[7] ,map=0 & widget_control,baset.bases5[9],map=0                 ;CONFIG+Options not visible.

           ;INIT window titles b12tit,b11tit,TIT,SIDs,symPZ
           ;GET  all numors, time,dth reading all other curves
            nums=long64(rpa.NTC[0]) & TIT =strtrim(nums,2) & b12tit=TIT+' TRs='+string(rpa.NTC[1],format='(F5.3)')
            b12tit=b12tit+' , Es='+string(rpa.Es,format='(F5.3)')
            txtq  =strtrim(string(round([rpa.X[0],rpa.X[n_elements(rpa.X)-1]]*1000.)/1000.),2)
            b12tit=b12tit+'  ,  '+txtq[0]+' <Q< '+txtq[1]

            nx=n_elements(idx) & npt=lonarr(nx) & tim=fltarr(nx)         ;Keep usefull infos in tables
            TITlong=strarr(nx) & dTh=tim & dL_L=dTh & Lambda=dTh         ;sigmaQ

            npt[0]=n_elements(rpa.X) & tim[0]=rpa.P[5] & dTh[0]=rpa.dTh
            TITlong[0]=rpa.TIT  & dL_L[0]=rpa.P[12] & Lambda[0]=rpa.P[0] ;sigmaQ

            if nx gt 1 then begin
               px=max([strpos(b11tit,'mM '),strpos(b11tit,'pL '),strpos(b11tit,'gH ')  ,$
                       strpos(b11tit,'mM.'),strpos(b11tit,'pL.'),strpos(b11tit,'gH.')])+3
               b11tit =strmid(b11tit,px,strpos(b11tit,' _')-px)
               SIDs   =strmid(b11tit,0 ,strpos(b11tit,' '))
               on_ioerror,misPZ0 & rdl=0 & reads, rpa.TIT, rdL & misPZ0: symPZ = rdL
               b12tit='Runs-'+strtrim(TIT,2) & TIT=''
               for i=1L,nx-1 do begin & rpc=cur_menu[2] & sans_reform_series,getlist=rpc,index=idx[i]
                   px=max([strpos(rpc.TIT,'mM '),strpos(rpc.TIT,'pL '),strpos(rpc.TIT,'gH ')  ,$
                           strpos(rpc.TIT,'mM.'),strpos(rpc.TIT,'pL.'),strpos(rpc.TIT,'gH.')])+3
                   b11tit =b11tit+' + '+strmid(rpc.TIT,px,strpos(rpc.TIT,' _')-px)
                   SIDs   =[SIDs,strmid(rpc.TIT,px,strpos(rpc.TIT,' ',px)-px)]
                   on_ioerror,misPZ1 & rdl=0 & reads, rpc.TIT, rdL & misPZ1: symPZ = [symPZ, rdL]
                   num=long64(rpc.NTC[0]) & b12tit=b12tit+ '-' +strtrim(num,2) & nums=[nums,num]
                   npt[i]=n_elements(rpc.X)
                   tim[i]=rpc.P[5] ;toW21 eq 7
                   dTh[i]=rpc.dTh  & dL_L[i]=rpc.P[12] & Lambda[i]=rpc.P[0] ;toW21 = 3,4,5 sigmaQ
                   TITlong[i]=rpc.TIT                                       ;toW21 = 3,4,5
               endfor
               on_ioerror,mis76
            endif

            mxnpt=max(npt) & WW=fltarr(mxnpt,nx) & XX=ww & EE=ww ;INIT A MAXIMUM BUFFER LENGTH WW,XX,EE
            Otitp =strmid(b11tit,0,120)
            Wtitp =strmid(b12tit,0,120)
            b11tit=strmid(b11tit,0,56) & b12tit=strmid(b12tit,0,56)
            if toW21 gt 0 then b12tit='W'+toWs+'<- '+b12tit
            widget_control,baset.bases1[11],set_value=b11tit
            widget_control,baset.bases1[12],set_value=b12tit

           ;GETTING AGAIN ALL CURVES TO HAVE W X E variables + margins and factors
           ;**********************************************************************
                                  m=0 & n=npt[0]-1
                                  if rpa.NTC[5] ne rpa.NTC[6] then begin                ;margins LIMIT INDEXs
                                     if rpa.NTC[5] gt 0. then $
                                     m=(where(rpa.X ge rpa.NTC[5]))[0] > 0
                                     if rpa.NTC[6] gt 0. then $
                                     n=(where(rpa.X ge rpa.NTC[6]))[0] > 0
                                     if n le m then n=npt[0]-1
                                  endif
                                  W=rpa.W[m:n]*rpa.NTC[4] & E=rpa.E[m:n]*rpa.NTC[4]     ;rescale
                                  X=rpa.X[m:n]
                                  if discd then begin                                   ;discard LRP
                                     ddx=where((W gt 1E-7) and (W>1. gt F4*E))
                                     if ddx[0] ge 0 then begin & X=X[ddx] & W=W[ddx] & E=E[ddx] & endif
                                  endif
                                  np=n_elements(X)
                                  difnpt=mxnpt-np & if difnpt gt 0 then tw21=fltarr(difnpt) & da1=[difnpt] & da2=[np]
                                                           if difnpt gt 0 then WW[*,0]=[tw21+W[0],W] else WW[*,0]=W
                                                           if difnpt gt 0 then XX[*,0]=[tw21+X[0],X] else XX[*,0]=X
                                                           if difnpt gt 0 then EE[*,0]=[tw21+E[0],E] else EE[*,0]=E

            if nx gt 2 then for i=1L,nx-2 do begin
                                  rpb=cur_menu[2] & sans_reform_series,getlist=rpb,index=idx[i]
                                  m=0 & n=npt[i]-1
                                  if rpb.NTC[5] ne rpb.NTC[6] then begin                ;margins LIMIT INDEXs
                                     if rpb.NTC[5] gt 0. then $
                                     m=(where(rpb.X ge rpb.NTC[5]))[0] > 0
                                     if rpb.NTC[6] gt 0. then $
                                     n=(where(rpb.X ge rpb.NTC[6]))[0] > 0
                                     if n le m then n=npt[i]-1
                                  endif
                                  W=rpb.W[m:n]*rpb.NTC[4] & E=rpb.E[m:n]*rpb.NTC[4]     ;rescale
                                  X=rpb.X[m:n]
                                  if discd then begin                                   ;discard LRP
                                     ddx=where((W gt 1E-7) and (W>1. gt F4*E))
                                     if ddx[0] ge 0 then begin & X=X[ddx] & W=W[ddx] & E=E[ddx] & endif
                                  endif
                                  np=n_elements(X)
                                  difnpt=mxnpt-np & if difnpt gt 0 then tw21=fltarr(difnpt) & da1=[da1,difnpt] & da2=[da2,np]
                                                           if difnpt gt 0 then WW[*,i]=[tw21+W[0],W] else WW[*,i]=W
                                                           if difnpt gt 0 then XX[*,i]=[tw21+X[0],X] else XX[*,i]=X
                                                           if difnpt gt 0 then EE[*,i]=[tw21+E[0],E] else EE[*,i]=E  & endfor

            if nx gt 1 then begin & i=nx-1
                                  m=0 & n=npt[i]-1
                                  if rpc.NTC[5] ne rpc.NTC[6] then begin                ;margins LIMIT INDEXs
                                     if rpc.NTC[5] gt 0. then $
                                     m=(where(rpc.X ge rpc.NTC[5]))[0] > 0
                                     if rpc.NTC[6] gt 0. then $
                                     n=(where(rpc.X ge rpc.NTC[6]))[0] > 0
                                     if n le m then n=npt[i]-1
                                  endif
                                  W=rpc.W[m:n]*rpc.NTC[4] & E=rpc.E[m:n]*rpc.NTC[4]     ;rescale
                                  X=rpc.X[m:n]
                                  if discd then begin                                   ;discard LRP
                                     ddx=where((W gt 1E-7) and (W>1. gt F4*E))
                                     if ddx[0] ge 0 then begin & X=X[ddx] & W=W[ddx] & E=E[ddx] & endif
                                  endif
                                  np=n_elements(X)
                                  difnpt=mxnpt-np & if difnpt gt 0 then tw21=fltarr(difnpt) & da1=[da1,difnpt] & da2=[da2,np]
                                                           if difnpt gt 0 then WW[*,i]=[tw21+W[0],W] else WW[*,i]=W
                                                           if difnpt gt 0 then XX[*,i]=[tw21+X[0],X] else XX[*,i]=X
                                                           if difnpt gt 0 then EE[*,i]=[tw21+E[0],E] else EE[*,i]=E  & endif
           ;KEEP W X E before eventual F(I,Q)
           ;*********************************
            if bcbg ne 0. then begin
               WW=WW-bcbg            ;Background if we come from Merging (77:)
              ;if bcbg gt 1. then EE=EE-sqrt(bcbg)
            endif
            WWp=WW & EEp=EE & XXp=XX
            
           ;Read Qrange now even if not selected (for models)
           ;*************************************************
            widget_control,baset.Qrange[1],get_value=Qmi & widget_control,baset.Qrange[2],get_value=Qma
            Qmi=strcompress(Qmi[0])
            Qma=strcompress(Qma[0]) & on_ioerror,misQRG
            if Qmi gt ' ' then Qmi=float(Qmi) else Qmi=min(XX)
            if Qma gt ' ' then Qma=float(Qma) else Qma=max(XX)
            if Qma le Qmi then Qmi=min(XX)
            if Qma le Qmi then Qma=max(XX)
            misQRG: & on_ioerror,mis76
            
           SAMuniq=(nx eq 1)

           ;(I) VERSUS (Q) PLOTTING FUNCTIONS (only for radial integrations)
           ;*********************************
            if curmenu eq 2 then $
            if (toW21 eq 5) and (baset.Fmodel[1] ge 0) and not(SAMuniq) then begin
              Xtit='Q' & Xtitp=Xtit     ;Plot I(Q) if we have MANY fits (radius) to show
              Ytit='I' & Ytitp=Ytit
              FIQf=1
            endif else begin            ;else use the functions
              for i=0L,nx-1 do begin
                if IofQ eq 1 then begin & WW[*,i]=WW[*,i] * XX[*,i]    &  EE[*,i]= EE[*,i] * XX[*,i]    & endif
                if IofQ eq 2 then begin & WW[*,i]=WW[*,i] * XX[*,i]^2  &  EE[*,i]= EE[*,i] * XX[*,i]^2  & endif
                if IofQ eq 3 then begin & WW[*,i]=WW[*,i] * XX[*,i]^4  &  EE[*,i]= EE[*,i] * XX[*,i]^4  & endif
                if Qsqr eq 1 then XX[*,i]=XX[*,i]^2
                if Qsqr eq 2 then XX[*,i]=XX[*,i]^4
              endfor
              FIQ=1 & FIQf=1
            endelse

           ;INIT PLOT WINDOW RANGES ANYWAY (XYranges Xlog Ylog ...)
           ;******************************
            yma =max(WW,min=ymi)
            xdif=1e-5    & ydif=1e-5

            if QrgON then begin
                  xmi=Qmi & xma=Qma
                  if FIQ then begin
                     if Qsqr eq 1 then begin & xmi=Qmi^2 & xma=Qma^2 & xdif=xdif^2 & endif
                     if Qsqr eq 2 then begin & xmi=Qmi^4 & xma=Qma^4 & xdif=xdif^4 & endif
                  endif
            endif else xma =max(XX,min=xmi)

            idQx=indgen(n_elements(XX))
            if QrgON then begin
               idQx=where( (XX ge xmi) and (XX le xma) )
               if idQx[0] ge 0 then yma =max(WW[idQx], min=ymi) else idQx=0
            endif

            if IrgON then begin & IrgON=0
               widget_control,baset.Irange[1],get_value=Imi & widget_control,baset.Irange[2],get_value=Ima
               Imi=strcompress(Imi[0])
               Ima=strcompress(Ima[0]) & on_ioerror,misIRG
               if Imi gt ' ' then Imi=float(Imi) else Imi=ymi
               if Ima gt ' ' then Ima=float(Ima) else Ima=yma
               if Imi lt Ima then begin & IrgON=1
                  if (not FIQ) or (IofQ eq 0) then begin & ymi=Imi & yma=Ima & endif
                  if FIQ then begin
                     if IofQ eq 1 then begin & ymi=min((WWp[idQx]>Imi)*XXp[idQx]  )
                                               yma=max((WWp[idQx]<Ima)*XXp[idQx]  ) & endif ;IQ
                     if IofQ eq 2 then begin & ymi=min((WWp[idQx]>Imi)*XXp[idQx]^2)
                                               yma=max((WWp[idQx]<Ima)*XXp[idQx]^2) & endif ;IQ^2
                     if IofQ eq 3 then begin & ymi=min((WWp[idQx]>Imi)*XXp[idQx]^4)
                                               yma=max((WWp[idQx]<Ima)*XXp[idQx]^4) & endif ;IQ^4
                  endif
               endif & misIRG: & on_ioerror,mis76
            endif

            xrg  =[xmi-xdif,xma+xdif]>0.     & if Qlog then xrg[1]= xrg[1]>(1e-5)
            yrg  =[ymi-ydif,yma+ydif]        & if ilog then yrg   =[yrg[0]>(1e-7) , yrg[1]>(1e-5)]
            if yrg[1]-yrg[0] lt 1e-3 then begin & yrg[0]=yrg[0]-5e-4
                                                yrg[1]=yrg[1]+5e-4 & endif
            tlog =ilog*Qlog
            xylW3=[Qlog,Ilog,0, Ierr,Styl, nx]
            
            if toW21 eq 2 then Xtitp=Ytit+' [ '+Xtit+' ]'
            
            TIT=TIT+' -> '+Ytit+' VERSUS '+Xtit+' <-'
            if discd then TIT=TIT+ ' (LRP discarded)'

            ;PLOT ALL CURVES WHEN NECESSARY toW21 <2 or ONE FIT ???
            ;***************
            if (toW21 lt 2) or ((toW21 eq 5) and (baset.Fmodel[1] ge 0) and (SAMuniq)) then begin ;Plot also if we have one fit to show
             kpwin=!D.window & wset,baset.bases1[14] & !p.linestyle=0  & !p.psym=0
            ;PRINTER
             if PrtON then begin
                     tvlct ,pos_r  ,pos_g ,pos_b ,/get
                     tmp=fix(pos_r)+pos_g +pos_b
                     pop=min(tmp) & ccol=!C
                     pup=max(tmp) & bcol=!C

                     kpBackground =!P.background
                     kpcolor      =!P.color
                     !P.subtitle  ='' ;Otitp

                     nnums=n_elements(nums)
                     if nnums le 10 then nnums=nums else nnums=nums[[0,nnums-1]]
                     PsFile=strjoin(strcompress(string(long(nnums))),'-')+'.ps'
                     wplot = !D.name
                     on_ioerror, iferr
                     set_plot,'PS' & sx=7. & sy=11.5
                     device, yoffset=sy-0.5,xoffset=.5,/inches,/Landscape
                     device, filename=PsFile
                     device, bits_per_pixel=8,/color
                     TIT=Wtitp
                     !P.background=bcol
                     !P.color     =ccol
             endif
             if (RepSt eq 0) or (nx le 3) then begin ;PLOT ON A 2D AXIS
               if tlog then plot,[0,0],/nodata,xmargin=[7,1],ymargin=[3,2],title=TIT,xtitle=Xtit,line=1,xrang=xrg,yrang=yrg,ytitle=Ytit,/ylog,/xlog else $
               if ilog then plot,[0,0],/nodata,xmargin=[7,1],ymargin=[3,2],title=TIT,xtitle=Xtit,line=1,xrang=xrg,yrang=yrg,ytitle=Ytit,/ylog else $
               if Qlog then plot,[0,0],/nodata,xmargin=[7,1],ymargin=[3,2],title=TIT,xtitle=Xtit,line=1,xrang=xrg,yrang=yrg,ytitle=Ytit,/xlog $
                       else plot,[0,0],/nodata,xmargin=[7,1],ymargin=[3,2],title=TIT,xtitle=Xtit,line=1,xrang=xrg,yrang=yrg,ytitle=Ytit

               lgndCOL=lonarr(nx) & lgndSYM=lonarr(nx)
               if nx ge 20 then styl=0
               for i=0L,nx-1 do begin & if styl then !p.psym=sym[symPZ[i] mod 6] & col=cool[idx[i] mod 6]
                                   WWi=WW[*,i]
                                   EEi=EE[*,i]
                                   XXi=XX[*,i]
                                   oplot,XXi ,WWi ,symsize=.5, col=col
                                   if nx lt 20 then $
                                   if ierr then errplot,XXi ,WWi+EEi,WWi-EEi, col=col
                                   lgndCOL[i]=col
                                   lgndSYM[i]=!p.psym
               endfor
               !p.linestyle=0 & !p.psym=0
               if ((nx ge 2) and (nx le 9)) or (Itool) then begin
                  legende={NPT:lonarr(nx)+(size(WW))[1],COL:lgndCOL,STY:lonarr(nx), $
                           SYM:lgndSYM,NAM:SIDs+' '+strtrim(nums,2),WRK:idx+1}

                  if (nx ge 2) and (nx le 9) then $
                     if (IofQ eq 1) or (IofQ eq 2) or (IofQ eq 3)            then $
                     ligend, legende.NAM, psym=legende.SYM, colors=legende.COL, textcolors=legende.COL, $
                             linestyle=legende.STY, thick=2,/TOP,/LEFT,/box  else $
                     ligend, legende.NAM, psym=legende.SYM, colors=legende.COL, textcolors=legende.COL, $
                             linestyle=legende.STY, thick=2,/TOP,/RIGHT,/box
               endif
             endif else if RepSt eq 2 then begin ;PLOT ON A 3D AXIS
                   YY=lindgen(nx)+1
                   if nx lt 100 then begin
                      if tlog then  surface,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Xlog,/Zlog,/hori else $
                      if ilog then  surface,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Zlog,/hori else $
                      if Qlog then  surface,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Xlog,/hori $
                      else          surface,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/hori
                   endif else begin
                      if tlog then  shade_surf,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Xlog,/Zlog else $
                      if ilog then  shade_surf,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Zlog else $
                      if Qlog then  shade_surf,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit,/Xlog $
                      else          shade_surf,WW,XX,YY,az=10.,ax=70.,xrang=xrg,zrang=yrg,xmargin=[8,1],ymargin=[1,1],ytitle='',xtitle=Xtit,title=TIT,ztitle=Ytit
                   endelse
             endif
             if PrtON then begin
                     ii=execute('P_DID_PS_HEADER,7.,1,PsFile')
                     widget_control,baset.bases1[12],set_value='FILE--> '+PsFile
                     iferr : DEVICE ,/close & set_plot, wplot
                     !P.background=kpBackground
                     !P.color     =kpcolor
                     !P.subtitle  =''
             endif else if Itool then begin  ;ITOOLS
                     kp_ptit =!p.title & kp_xtit=!x.title & kp_ytit=!y.title
                     !p.title=b11tit   &!x.title=Xtit     & !y.title=Ytit
                     ii=execute('LIVE_LAMP_PLOT, 1, XX, WW, xrange=xrg, yrange=yrg, title=b11tit, error=EE'+ $
                                                 ', legende=legende, xylW=xylW3, okT=okT')
                     if not okT then $
                     ii=execute('LIVE_LAMP_PLOT, 1, XX, WW, xrange=xrg, yrange=yrg, title=b11tit, error=EE'+ $
                                                 ', legende=legende, xylW=xylW3')
                     !p.title=kp_ptit & !x.title=kp_xtit & !y.title=kp_ytit
             endif

             trap,current=!D.window
             wset,kpwin
            endif ;(PLOT)
           ;*************
           ;*************

           ;WHAT ELSE
           ;**** ****
            if toW21 gt 0 then begin
               sz=size(WW) & if sz[0] eq 2 then sz=sz[2] else sz=1
               if (toW21 eq 7) and (sz ge 3) then begin    ;Show surface (Y is Time)
                   Yunit=tim  & Ytitp='Time' & for i=1L,sz-1 do Yunit[i]=Yunit[i-1]+Yunit[i] ;& XX=XX(*,0)
               endif else begin
                   Yunit=nums & Ytitp='Runs'
               endelse
               if (toW21 eq 6) or (toW21 eq 7) then begin
                  if tlog then begin & Xtitp=Xtitp+'(log)' & Wtitp=Wtitp+'(log)'  & XX=alog10(XX>1e-5) & WW=alog10(WW>1e-5) & endif else $
                  if ilog then begin &                       Wtitp=Wtitp+'(log)'                       & WW=alog10(WW>1e-5) & endif else $
                  if Qlog then begin & Xtitp=Xtitp+'(log)'                        & XX=alog10(XX>1e-5)                      & endif
                  if toW21 eq 6 then Yunit=0
               endif

               if toW21 eq 5 then begin & WW=WWp & EE=EEp & XX=XXp & endif   ;RESTORE ORIGINAL REDUCTION FOR FIT

         ;TOW21 0 PLOT
         ;      1 PLOT    -> W21              unused!
         ;      2         -> W21            + Suprplot
         ;      3         -> W22 with merge                   + dQ in Z22
         ;      4         -> W22 with merge + Suprplot        + dQ in Z22
         ;      5 PLOT dQ -> W23 with merge + W22             + dQ in Z22 + dQ in W23 (-1 no   model)
         ;      5 PLOT    -> W22 with merge + FIT (no F[I,Q]) + dQ in Z22 + dQ in W23 (0+ with model)
         ;      6         -> W21            + surface
         ;      7         -> W21            + surface time scaling

           ;TITlong is "vecXnn numor lineNNN.instNN[gH,mM,pL] title etc..."
           ;there is a '.'  after [gH,mM,pL] when sector/rocking (pointP+5)
           ;there is a ' [' after [gH,mM,pL] when operated curve (pointP+6)

              ;toW USED FOR PLOTTING SURFACE , SUPERPLOT , STR_FIT input
               datp={X:XX,Y:Yunit,E:EE,X_TIT:Xtitp,Y_TIT:Ytitp,W_TIT:Wtitp,OTHER_TIT:Otitp,P:float(nums)}
               GIVE_W,WW,W=toW & GIVE_DATP,datp,W=toW

               if toW21 ne 5 then begin & WW=WWp & EE=EEp & XX=XXp & endif   ;RESTORE ORIGINAL REDUCTION ANYWAY

              ;WE HAVE TO MERGE?
              ;****************
               merged =0
               if ((toW21 eq 3) or  (toW21 eq 4)) or $                 ;Merge in W22 , show in suprplot
                  ((toW21 eq 5) and (baset.Fmodel[1] eq 0)) then begin ;MERGE for one FIT testing
                  
                  ;USE THE MACRO **SANS_MERGE_IN_FILES**               ;idx is the selected List
                   gtlist=cur_menu[2] & sans_reform_series,getlist =gtlist, /rawConf
                   idxl=-1 & idxm=-1 & idxh=-1
                   for i=0L,n_elements(idx)-1 do begin
                       if (strpos(gtlist[idx[i]],'pL ') gt 4) or (strpos(gtlist[idx[i]],'pL.') gt 4) then idxl=[idxl,idx[i]] else $
                       if (strpos(gtlist[idx[i]],'mM ') gt 4) or (strpos(gtlist[idx[i]],'mM.') gt 4) then idxm=[idxm,idx[i]] else $
                       if (strpos(gtlist[idx[i]],'gH ') gt 4) or (strpos(gtlist[idx[i]],'gH.') gt 4) then idxh=[idxh,idx[i]]
                   endfor
                   if n_elements(idxl) gt 1 then idxl=idxl[1]
                   if n_elements(idxm) gt 1 then idxm=idxm[1]
                   if n_elements(idxh) gt 1 then idxh=idxh[1]
                   
                   sans_merging, 'LMH.AllIndex', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, WaveSuplement=WaveSuplement
                   sans_merging, 'MERGE_file', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, instrument=instru, WaveSuplement=WaveSuplement
                   
                   if n_tags(oneMrg) gt 0 then begin
                      newW   =oneMrg.WW  &  newDATP=oneMrg.datp
                      newX   =newDATP.X  &  newE=newDATP.E  &  newdQ=newDATP.Z  & dQ=newdQ

                     ;Realy want to MERGE (3=merge in W22 , 4=merge in W22 & suprplot , or one FIT)
                      if (toW21 eq 3) or (toW21 eq 4) or (baset.Fmodel[1] eq 0) then begin
                         WW=newW & EE =newE   & datp=newDATP & XX=newX
                         merged=1  &  sz=1  &  xylW3[5]=sz
                      endif
                   endif ;oneMrg not empty
               endif ;toW21>3<5
               to_did_cur,'W'+toWs

               if (toW21 eq 2) or (toW21 eq 4) then suprplot,toW, xylW=xylW3

              ;Plot dQ or FIT a Model
               if (toW21 eq 5) then begin

                   if merged then nTh=1 else nTh=n_elements(dTh)

                   if baset.Fmodel[1] eq -1 then begin & if nTh gt 8 then begin & dTh=dTh[0:7] & nTh=8 & endif ;if show dQ max 8 configs !!!
                                                       XX=XX[*,0:nTh-1] & endif

                  ;Re-calculate dQ for PLOT or for MULTI FIT -> 2D deltaQ even in W23 if not merged
                   if (baset.Fmodel[1] ne 0) or not (merged) then begin
                     dQ=XX
                     for i=0L,nTh-1  do $
                         dQ[*,i] = sans_sigmaQ(X=XX[*,i] , dL_L=dL_L[i] , Lambda=Lambda[i] , dTh=dTh[i])

                     newdQ=dQ & newX =XX
                   endif

                   if baset.Fmodel[1] eq -1 then begin ;PLOT DELTA Q
                      b11tit='Lambda='+strtrim(string(Lambda[0],format='(F5.2)'),2)+' dL/L='+strtrim(long(dL_L[0]*100.),2)+'% dTh='
                      kpwin=!D.window & wset,baset.bases1[14] & !p.linestyle=0  & !p.psym=0
                      plot,[0,0],/nodata,xmargin=[7,1],ymargin=[3,2],title='dQ Configuration Resolution *100 (W23)',xtitle='Q',line=1,$
                                 xrang=[min(XX,max=Qmax),Qmax],yrang=round([min(dQ,max=dQmax),dQmax]*1000000)/10000.,ytitle='dQ*100'
                      for i=0L,nTh-1 do begin & !p.psym=sym[i mod 6] & col=cool[i mod 6]
                                 oplot,XX[*,i] ,dQ[*,i]*100. ,symsize=.5, col=col
                                 b11tit=b11tit+strtrim(string(dTh[i],format='(F8.6)'),2)+' '
                      endfor &   wset,kpwin & !p.linestyle=0 & !p.psym=0
                      widget_control,baset.bases1[11],set_value=b11tit
                   endif

                  ;Put newdQ in W23 (Used as input by STR_FIT)
                   WdQ='dQ Configuration Resolution'+strjoin(strcompress(string(long(nums))))
                   if (size(newX))[0] eq 1 then begin & YdQ=0.          & TdQ='dQ'  & ZdQ=''
                   endif                   else begin & YdQ=float(nums) & TdQ='RUN' & ZdQ='dQ'
                   endelse
                   datp={X:newX,Y:YdQ,E:0,X_TIT:'Q',Y_TIT:TdQ,Z_TIT:ZdQ,W_TIT:WdQ,OTHER_TIT:Otitp,P:float(nums)}
                  ;if (baset.Fmodel(1) ne -1)  or (n_elements(YdQ) eq 1) then begin ; Do not write in W23 if Show DeltaQ
                      GIVE_W,newdQ,W=23 & GIVE_DATP,datp,W=23 & to_did_cur,'W23'
                  ;endif
                   
                  ;MODELS Fmodel(0)=model  Fmodel(1)=0:one merged fit 1:none merged fits 2:2D fits
                   if baset.Fmodel[1] ge  0 then begin
                   
                    kpwin=!D.window & wset,baset.bases1[14] & !p.linestyle=0  & !p.psym=0
                    maxWW=[max(WW[*,0]) , max(WW[*,sz-1]) , sz]
                    Qrang=[Qmi,Qma]
                    if QrgON then QrangOK=Qrang

                    sans_settings_model, baset, Wtitp, toW, Qrang,QrangOK, maxWW, $
                                        [FIQf,IofQ,Qsqr], IDX, [Ilog,Qlog], SCALED=Uscal

                    if (baset.Fmodel[1] eq 1) and (sz gt 1) then FIQ=0 ;DONT REMEMBER WY ???
                   
                    wset,kpwin & !p.linestyle=0 & !p.psym=0

                   endif ;FIT (baset.Fmodel(1) ge 0)

               endif ;(toW21 eq 5)

               if sz ge 3 then $
               if (toW21 eq 6) or (toW21 eq 7) then SEE,W=toW,/surface,/contour,/beside

            endif ;(toW21 gt 0)

            baset.PrintON=0
            baset.ITOOLS =0
            baset.FIQ    =FIQ   &   widget_control,ev.top,set_uvalue=baset

          endif ;N_TAGS(RPA)

         endif ;LIST_SELECT
         mis76:
         if d_repr[21] then begin & tvlct,R_curr[255]    ,G_curr[255]    ,B_curr[255]    ,255
                                    tvlct,d_etc.curBg[0] ,d_etc.curBg[1] ,d_etc.curBg[2] ,  0 & endif
         return
         end

77:begin ;*** MERGE LIST IN AN EXISTING FILE * SAVE LIST IN A FILE * RESTORE A RESULT FILE ***
         ;*** -------------------------------- * --------------------- * ----------------- ***
         if  uv[1] eq 0  then begin & if uv[2] eq  1 then sans_reform_series,/MERGE $
                                                     else sans_reform_series,/SAVE
                                      if uv[2] eq -1 then begin & widget_control,ev.top,/destroy & return & endif & endif
         if  uv[1] eq 1  then begin &                     sans_reform_series,/RESTORE
                                                          gtlist=cur_menu[9] & sans_reform_series,getlist =gtlist
                                                          widget_control,baset.bases4[17],set_value =gtlist, get_uvalue=brSsho
                                                         ;Show actual IQfactor scaling (see few lines below)
                                                          uv=[77,7,2,brSsho[3:*]] ;  (sans_merge_fields)
                                                         ;***********************    *******************
                                                          endif
         
         ;*** CHANGE PRESENTATION  ERROR , LINE/SYMBOL , LRP ***
         ;*** -------------------  -----   -----------   --- ***
         if (uv[1] ge 2) and (uv[1] le 6) then begin   ;Change presentation...
           if uv[1] eq 2 then baset.bases6[12]=ev.select else $ ;error
           if uv[1] eq 6 then begin
              if uv[2] eq 0 then baset.bases6[16]=ev.select     ;style (symbol)
              if uv[2] eq 1 then baset.bases6[18]=ev.select     ;Lrp
           endif else baset.bases6[13]=uv[2]-3
           widget_control,ev.top,set_uvalue=baset
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         
         ;*** EVENTS CONCERCING MERGING CONFIGS ******************************************************************************

         ;*** GET/SET FACTORS SCALING AND MARGINS , MERGE ***
         ;*** ------- ------- ------- --- --------------- ***
         if (uv[1] eq 7) or (uv[1] eq 8) or (uv[1] eq 9) then begin
          ;CASE WE HAVE TO FILL ALL FIELDS FROM ACTUAL CONFIGURATIONS (come also from RESTORE)                                         16 4th configuration.....
          ;Qmargin button get fields     [77,7,1 ,QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld]
          ;IQ factor button get fields   [77,7,2 ,...
          ;4th config button get fields  [77,7,16,...
          ;
          ;IQ factor suggestion ALL      [77,7,3 ,...
          ;IQ factor suggestion SELECTED [77,7,4 ,...
          
          ;MERGE ALL      in files       [77,7,5 ,...
          ;MERGE SELECTED in files       [77,7,6 ,...
          
          ;CASE WE HAVE JUST TO SEND BACK CLEARED SCALINGS AND MARGINS, EMPTY THE FIELDS
          ;Clear Qmargin , IQ factor     [77,7,0 ,...
          
          ;CASE WE HAVE TO SEND BACK IN RESULTS WHAT ARE IN ALL THE FIELDS
          ;Factor field1 field2 field3     [77,8,1 2 3       ,...
          ;Qrange left1 right1 l2 r2 l3 r3 [77,9,1 2 3 4 5 6 ,...

          ;FIRST GET ALL LIST IN idxll, idxmm, idxhh (having "pL", "mM" or "gH" in the string list)
          ;*****************************************

           sans_merging, 'LMH.AllIndex', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, WaveSuplement=WaveSuplement

           if WaveSuplement.waveSLall[0] ge 0 then sens4th = 1 else sens4th = 0
           widget_control, uv[20] , sensitive = sens4th ; Activate WSbut if we have a 4th config
;widget_control, uv[20] , sensitive = 0 ; Not yet ready............

          ;REDUCE to SELECTED IF ASKED
          ;***************************
           if (uv[1] eq 7) and ((uv[2] eq 4) or (uv[2] eq 6)) then begin        ;SELECTED LIST for SUGGESTION or MERGE
               idx=widget_info(baset.bases4[17],/list_select)
               if n_elements(idx) lt 2 then return                              ;NONE SELECTED
               newll=-1 & newmm=-1 & newhh=-1
               for i=0,n_elements(idx)-1 do begin
                 ok=where(idxll eq idx[i]) & if ok[0] ge 0 then newll=[newll,idxll[ok[0]]]   else begin
                 ok=where(idxmm eq idx[i]) & if ok[0] ge 0 then newmm=[newmm,idxmm[ok[0]]]   else begin
                 ok=where(idxhh eq idx[i]) & if ok[0] ge 0 then newhh=[newhh,idxhh[ok[0]]] & endelse & endelse
               endfor
               if n_elements(newll) gt 1 then idxll=newll[1:*] else idxll=-1
               if n_elements(newmm) gt 1 then idxmm=newmm[1:*] else idxmm=-1
               if n_elements(newhh) gt 1 then idxhh=newhh[1:*] else idxhh=-1
           endif
           
          ;To use "sans_merge_in_files" or "sans_merge_find_scaling" ->  be sure to be sorted by sample and sectors
          ;uv[2]=3 get a suggestion from all the list
          ;uv[2]=4 get a suggestion from selected items in the list
          ;uv[2]=5 merge all      in files
          ;uv[2]=6 merge selected in files
          ;WaveSuplement = {waveSL:waveSL ,  waveSM:waveSM , waveSH:waveSH , waveP , waveSLall:waveSLall , marg_F4th:marg_F4th}
           sorted_ = 0
          ;SORT BY LINE NUMBER (same line->same sample "001.08??.Sect180.0Sym") SORT for MERGE or SUGGESTION
          ;********************************************************************
           if (uv[1] eq 7) and (uv[2] ge 3) and (uv[2] le 6) then begin ;SORT for MERGE and SUGGESTION

             sans_merging, 'SORT', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, WaveSuplement=WaveSuplement
             sorted_ = 1

           endif
           
          ;We will Get/Put the same infos for each config represented by there first index
           idxl=idxll[0]  &  idxm=idxmm[0]  &  idxh=idxhh[0]  &  idxl4th=WaveSuplement.waveSLall[0]
           
          ;But if we have a 4th config and sans_merging,'SORT' as not be called then be sure to get a low index from the main wavelength list
           if sens4th then if (not sorted_) then begin
              idx=0 & while ((where(WaveSuplement.waveSLall eq idxll[idx]))[0] ge 0) do idx +=1 & idxl=idxll[idx] & endif

          ;INIT ARRAY for Qrange, Scales, Background GUI values FOR FURTHER USE
           farr=fltarr(10) & sarr=strarr(10) & farr[6:8]=1.                 ;Qrange and Scale array LMH conf
           f4th=fltarr(3)  & s4th=strarr(3)  & f4th[2]  =1.                 ;Qrange and Scale array 4th conf
           widget_control, uv[15] , get_value=bcbg  & farr[9]=float(strtrim(bcbg[0],2)) ;background
           
          ;MERGE IN FILE CONFORMING TO ACTUAL QRANGE, SCALE, BACKGROUND
          ;************************************************************
           if (uv[1] eq 7) and ((uv[2] eq 5) or (uv[2] eq 6)) then begin ;MERGE (sorted)
               widget_control,baset.bases1[3],get_value=instru
               widget_control,/hourglass

               sans_merging, 'MERGE_file', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, background=farr[9], instrument=instru, WaveSuplement=WaveSuplement
           endif

          ;GET QRANGEs and SCALEs SUGGESTION, PUT IN GUI, THEN IN LIST MEMORY
          ;******************************************************************
           if  uv[1] eq 7  then if (uv[2] eq 3) or (uv[2] eq 4) then begin ; Qranges and Factor scaling suggestions (sorted)
            
                  sans_merging, 'SCALE', cur_menu[2], idxll,idxmm,idxhh, gtlist=gtlist, suggestion=marg_Flmh, WaveSuplement=WaveSuplement
            
                 ;Update the GUI fields
                  Flmh=marg_Flmh[6:8]
                  Mlmh=marg_Flmh[0:5]
                  if Flmh[0] gt 0. then if Flmh[0] ne 1. then sarr[6]=strtrim(string(Flmh[0],format='(f8.3)'),2)
                  if Flmh[1] gt 0. then if Flmh[1] ne 1. then sarr[7]=strtrim(string(Flmh[1],format='(f8.3)'),2)
                  if Flmh[2] gt 0. then if Flmh[2] ne 1. then sarr[8]=strtrim(string(Flmh[2],format='(f8.3)'),2)
                  
                  if Mlmh[0] gt 0. then sarr[0]=string(Mlmh[0],format='(f6.4)')
                  if Mlmh[1] gt 0. then sarr[1]=string(Mlmh[1],format='(f6.4)')
                  if Mlmh[2] gt 0. then sarr[2]=string(Mlmh[2],format='(f6.4)')
                  if Mlmh[3] gt 0. then sarr[3]=string(Mlmh[3],format='(f5.3)')
                  if Mlmh[4] gt 0. then sarr[4]=string(Mlmh[4],format='(f5.3)')
                  if Mlmh[5] gt 0. then sarr[5]=string(Mlmh[5],format='(f5.3)')
                  for k=0,5 do if strmid(sarr[k],0,1) eq '0' then sarr[k]=strmid(sarr[k],1) ;Suppress leading '0'
                  
                  for k=6,8 do widget_control,uv[6+k], set_value=sarr[k] ;Fill the Factor scaling in the fields
                  for k=0,5 do widget_control,uv[6+k], set_value=sarr[k] ;Fill the Qranges in the fields

                 ;Refresh the 4th config
                  IF sens4th then begin
                     f4th[2]  = WaveSuplement.marg_F4th[2]
                     f4th[0:1]= WaveSuplement.marg_F4th[0:1]
                     if f4th[2] gt 0. then     if f4th[2] ne 1. then s4th[2]=strtrim(string(f4th[2],format='(f8.3)'),2)
                     if f4th[0] gt 0. then                           s4th[0]=        string(f4th[0],format='(f7.5)')
                     if f4th[1] gt 0. then                           s4th[1]=        string(f4th[1],format='(f7.5)')
                     for k=0,1 do if strmid(s4th[k],0,1) eq '0' then s4th[k]=strmid(s4th[k],1) ;Suppress leading '0'
                     widget_control,uv[19] , set_value=s4th[2] ; =sarr[6] ;or assume Low scale
                     widget_control,uv[17] , set_value=s4th[0] ; =''
                     widget_control,uv[18] , set_value=s4th[1] ; =''
                  ENDIF

                 ;Show these fields in the GUI                  
                  sans_merge_fields, baset, show=2

                 ;Will put Qranges and Scales in sans_reform_series MEMORY
                  uv[1]=8 & uv[2]=1 ;GO SETTING THE RESULTS....
           endif ;                         6                                     12                 15      16                         20
          ;uv=[77,?,? ,QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
          
          ;FROM LIST MEMORY, SET GUI Qranges, Scales
          ;*****************************************
           if  uv[1] eq 7  then begin ;From results, get factor scaling 2, margins 1, reset all 0  then SHOW in merging fields

            ;Clear GUI Qrange, Scale / Background
             if (uv[2] le 2) or (uv[2] eq 16) then sans_merge_fields, baset, clear=[1,1,1,1]
             if  uv[2] eq 0 then widget_control, uv[15]  , set_value='' ; background
             
            ;Then set fields to the GUI from sans_reform_series MEMORY
             if (uv[2] eq 1) or (uv[2] eq 2) or (uv[2] eq 16) then begin

               if idxl ge 0 then begin
                  sans_reform_series,getlist=cur_menu[2],index=idxl    ,getfct =getfct ,getmarg=getmarg ;Get info for low
                  farr[0:1]=getmarg & farr[6]=getfct & endif
               if idxm ge 0 then begin
                  sans_reform_series,getlist=cur_menu[2],index=idxm    ,getfct =getfct ,getmarg=getmarg ;Get info for middle
                  farr[2:3]=getmarg & farr[7]=getfct & endif
               if idxh ge 0 then begin
                  sans_reform_series,getlist=cur_menu[2],index=idxh    ,getfct =getfct ,getmarg=getmarg ;Get info for high
                  farr[4:5]=getmarg & farr[8]=getfct & endif
               if sens4th   then begin
                  sans_reform_series,getlist=cur_menu[2],index=idxl4th ,getfct =getfct ,getmarg=getmarg ;Get info for 4th config
                  f4th[0:1]=getmarg & f4th[2]=getfct & endif

               if farr[0] ne farr[1] then begin & if farr[0] gt 0. then sarr[0]=string(farr[0],format='(f6.4)')
                                                  if farr[1] gt 0. then sarr[1]=string(farr[1],format='(f6.4)') & endif
               if farr[2] ne farr[3] then begin & if farr[2] gt 0. then sarr[2]=string(farr[2],format='(f6.4)')
                                                  if farr[3] gt 0. then sarr[3]=string(farr[3],format='(f5.3)') & endif
               if farr[4] ne farr[5] then begin & if farr[4] gt 0. then sarr[4]=string(farr[4],format='(f5.3)')
                                                  if farr[5] gt 0. then sarr[5]=string(farr[5],format='(f5.3)') & endif
               if f4th[0] ne f4th[1] then begin & if f4th[0] gt 0. then s4th[0]=string(f4th[0],format='(f7.5)')
                                                  if f4th[1] gt 0. then s4th[1]=string(f4th[1],format='(f7.5)') & endif
                                                  
               for k=0,5 do if strmid(sarr[k],0,1) eq '0' then sarr[k]=strmid(sarr[k],1) ;Suppress leading '0'
               for k=0,1 do if strmid(s4th[k],0,1) eq '0' then s4th[k]=strmid(s4th[k],1) ;Suppress leading '0'
               
               if farr[6] gt 0. then if farr[6] ne 1. then sarr[6]=strtrim(string(farr[6],format='(f8.3)'),2)
               if farr[7] gt 0. then if farr[7] ne 1. then sarr[7]=strtrim(string(farr[7],format='(f8.3)'),2)
               if farr[8] gt 0. then if farr[8] ne 1. then sarr[8]=strtrim(string(farr[8],format='(f8.3)'),2)
               if f4th[2] gt 0. then if f4th[2] ne 1. then s4th[2]=strtrim(string(f4th[2],format='(f8.3)'),2)

               for k=0,8 do widget_control,uv[6 +k], set_value=sarr[k] ;Fill the fields LMH
               for k=0,2 do widget_control,uv[17+k], set_value=s4th[k] ;Fill the fields 4th config

              ;and show these fields in the GUI
               sans_merge_fields, baset, show=uv[2] < 3 ; 1 is Qrange, 2 is Scales, 16<3 is 4th Cf
             endif
           endif ;                         6                                     12                 15      16                         20
          ;uv=[77,?,? ,QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

           
          ;PUT GUI QRANGEs, SCALEs TO sans_reform_series MEMORY <CR> entered
          ;COMES ALSO FROM SUGGESTIONS.
          ;*****************************************************************
           if ((uv[1] eq 7) and uv[2] eq 0) or $       ;uv[2]=0 means all was cleared and not refreshed
               (uv[1] eq 8) or (uv[1] eq 9) then begin ;8=set factor scaling  9=set margins (<CR> in one of the fields)

               sans_settings_text,uv[6:15] ,farr        ;Get from GUI (15 is background, not saved in list memory)
               if farr[6] le 0. then farr[6]=1. & if farr[7] le 0. then farr[7]=1. & if farr[8] le 0. then farr[8]=1.
               sans_settings_text,uv[17:19],f4th        ;Get from GUI (4th config)
               if f4th[2] le 0. then f4th[2]=1.
               
              ;Decide to set all fields for ALL "inst" + "pL" | "mM" | "gH"
              ;************************************************************
               marg_Flmh               = [farr[0:5] , farr[6:8]] ; [Qranges , Scales]
               WaveSuplement.marg_F4th = [f4th[0:1] , f4th[2]]
               sans_merging, 'SCALE_set', cur_menu[2], idxl,idxm,idxh, gtlist=gtlist, suggestion=marg_Flmh, WaveSuplement=WaveSuplement
           endif

          ;SHOW AGAIN THE SELECTED CURVES in the GUI                                                    -100 means using background
          ;*****************************************
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:-100L ,CLICKS:0L}') ;Show selected
         endif ;uv(1)= 7 or 8 or 9

         ;***** END CONCERCING MERGING CONFIGS ******************************************************************************


         ;*** F(I,Q) , QRANGE , IRANGE ***
         ;*** ------   ------   ------ ***
         if  uv[1] eq 10 then begin ;f(IQ)
           baset.IofQ=uv[2]
           widget_control,ev.top,set_uvalue=baset
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         if  uv[1] eq 11 then begin ;Qrange ON/OFF
           if uv[2] then begin & baset.Qrange[0]=ev.select
                               widget_control,ev.top,set_uvalue=baset & endif
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         if  uv[1] eq 13 then begin ;Irange ON/OFF
           if uv[2] then begin & baset.Irange[0]=ev.select
                               widget_control,ev.top,set_uvalue=baset & endif
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         
         ;*** PRINT , ITOOLS , RepStyle ***
         ;*** -----   ------   -------- ***
         if  uv[1] eq 12 then begin ;PRINT
           baset.PrintON=1
           widget_control,ev.top,set_uvalue=baset
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         if  uv[1] eq 14 then begin ;ITOOLS , RepStyle
           if uv[2] eq 1 then  baset.ITOOLS=1 else $
           if uv[2] ge 2 then begin ;Toogle RepStyle
              widget_control,ev.id,get_value=repText & p=strpos(repText,'(')
              if strpos(repText,'(Off)') ge 0 then newT='(On)' else newT='(Off)'
              widget_control,ev.id,set_value=strmid(repText,0,p)+newT
              if newT eq '(On)' then baset.RepStyle=uv[2] else baset.RepStyle=0
           endif
           widget_control,ev.top,set_uvalue=baset
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L ,CLICKS:0L}') ;Show selected
         endif
         
         ;*** SORT ***
         ;*** ---- ***
         if  uv[1] eq 15 then begin ;SORT the List 0=sample    1=conf 1,2,3    2=sectors
           gtlist=cur_menu[2] & sans_reform_series,getlist =gtlist ,sort=uv[2]
           widget_control,baset.bases4[17],set_value =gtlist
         endif
         
         ;*** SELECT OPTION ***
         ;*** ------ ------ ***
         if  uv[1] eq 16 then begin ;SELECT in the List 1=conf1 2=conf2 3=conf3
           gtlist=cur_menu[2] & sans_reform_series,getlist =gtlist
           if uv[2] eq 0 then idx=indgen(n_elements(gtlist))         else $
           if uv[2] eq 1 then idx=where( (strpos(gtlist,'pL ') gt 7) or (strpos(gtlist,'pL.') gt 7) ) else $
           if uv[2] eq 2 then idx=where( (strpos(gtlist,'mM ') gt 7) or (strpos(gtlist,'mM.') gt 7) ) else $
           if uv[2] eq 3 then idx=where( (strpos(gtlist,'gH ') gt 7) or (strpos(gtlist,'gH.') gt 7) )
           if idx[0] ge 0 then begin
           widget_control,baset.bases4[17],set_list_select=idx
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:0L, CLICKS:0L}') ;Show selected
           endif
         endif
         
         ;*** STORE DATA IN WORKSPACES ***
         ;*** ------------------------ ***
         if  uv[1]  ge 21 then begin & id=20L-long(uv[1]) ;Store selected entries using W21 , W22 or W23
           if uv[1] eq 25 then if uv[2] ge 0 then baset.Fmodel=[uv[2],uv[3]] else baset.Fmodel[1]=uv[3]      ;Fitting or showing DeltaQ
           widget_control,ev.top,set_uvalue=baset
           ii=execute('sans_settings_event,{WIDGET_LIST,ID:baset.bases4[17],TOP:ev.top,HANDLER:0L,INDEX:id, CLICKS:0L}') ;Show selected
         endif
         
         widget_control,ev.top,get_uvalue=baset
         end

78:      begin          ;OPERATIONS , GET/SAVE SELECTED CURVES
        ;OPERATIONS
        ;**********
          case uv[1] of

          -1:begin & widget_control,uv[4],get_value =opr ;Just show the operation
                   widget_control,uv[8],get_value =lab
                   widget_control,uv[7],get_uvalue=idxv1 & idx1=strsplit(idxv1,',',/extract) & nx1=n_elements(idx1) ;Operand index 1
                   widget_control,uv[8],get_uvalue=idxv2 & idx2=strsplit(idxv2,',',/extract) & nx2=n_elements(idx2) ;Operand index 2
                   widget_control,uv[3],get_value =fxva  & fxva=strtrim(fxva[0],2) & fxLa=float(fxva)
                   widget_control,uv[6],get_value =fxvu  & fxvu=strtrim(fxvu[0],2) & fxLu=float(fxvu) & fxkp=fxvu
               if fxLa ne 1. then fxva='*'+fxva else fxva=''
               if fxLu ne 1. then fxvu='*'+fxvu else fxvu=''
               if lab ne '*' then mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ('+fxkp+')' $ ;Minus Constant
                             else mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ['+strmid(idxv2[0],0,12)+']'+fxvu
               widget_control,baset.bases1[12],set_value=mesout
             end 

          0: begin & widget_control,uv[7],set_uvalue=''  ;Operand 1
               idx=widget_info(baset.bases4[17],/list_select)
               idxv1=''
               if idx[0] ge 0 then begin & idxv1=strjoin(strtrim(string(idx+1),2),',')
                                          widget_control,uv[7],set_uvalue=idxv1 & endif
               widget_control,uv[8],get_value =lab
               widget_control,uv[4],get_value =opr
               widget_control,uv[8],get_uvalue=idxv2
               widget_control,uv[3],get_value =fxva  & fxva=strtrim(fxva[0],2) & fxLa=float(fxva)
               widget_control,uv[6],get_value =fxvu  & fxvu=strtrim(fxvu[0],2) & fxLu=float(fxvu) & fxkp=fxvu
               if fxLa ne 1. then fxva='*'+fxva else fxva=''
               if fxLu ne 1. then fxvu='*'+fxvu else fxvu=''
               if lab ne '*' then mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ('+fxkp+')' $ ;Minus Constant
                             else mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ['+strmid(idxv2[0],0,12)+']'+fxvu
               widget_control,baset.bases1[12],set_value=mesout
             end

          1: begin & widget_control,ev.id,get_value=opr ;Operator
                  ;uv=[78,1,bopr1,bopr1T, opropr ,bopr2,bopr2T,blabop]
                   if strpos(opr,'Constant') gt 0  then lab=' ' else lab='*'
                   widget_control,uv[8],set_value=lab
                   opr=strmid(opr,0,1)
                   widget_control,uv[4],set_value=opr

               if lab ne '*' then begin ;Minus a Constant
                          widget_control,uv[5],sens=0,set_value='Constant' ;un-sens Select 2
               endif else widget_control,uv[5],sens=1,set_value='Select 2'
               
               widget_control,uv[7],get_uvalue=idxv1
               widget_control,uv[8],get_uvalue=idxv2
               widget_control,uv[3],get_value =fxva  & fxva=strtrim(fxva[0],2) & fxLa=float(fxva)
               widget_control,uv[6],get_value =fxvu  & fxvu=strtrim(fxvu[0],2) & fxLu=float(fxvu) & fxkp=fxvu
               if fxLa ne 1. then fxva='*'+fxva else fxva=''
               if fxLu ne 1. then fxvu='*'+fxvu else fxvu=''
               if lab ne '*' then mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ('+fxkp+')' $ ;Minus Constant
                             else mesout='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ['+strmid(idxv2[0],0,12)+']'+fxvu
               widget_control,baset.bases1[12],set_value=mesout
             end

          2: begin & widget_control,uv[8],set_uvalue=''  ;Operand 2
               idx=widget_info(baset.bases4[17],/list_select)
               idxv2=''
               if idx[0] ge 0 then begin & idxv2=strjoin(strtrim(string(idx+1),2),',')
                                          widget_control,uv[8],set_uvalue=idxv2 & endif
               widget_control,uv[4],get_value =opr
               widget_control,uv[7],get_uvalue=idxv1
               widget_control,uv[3],get_value =fxva  & fxva=strtrim(fxva[0],2) & fxLa=float(fxva)
               widget_control,uv[6],get_value =fxvu  & fxvu=strtrim(fxvu[0],2) & fxLu=float(fxvu)
               if fxLa ne 1. then fxva='*'+fxva else fxva=''
               if fxLu ne 1. then fxvu='*'+fxvu else fxvu=''
               widget_control,baset.bases1[12],set_value='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ['+strmid(idxv2[0],0,12)+']'+fxvu
             end

          3: begin & widget_control,uv[4],get_value =opr ;DO
                   widget_control,uv[8],get_value =lab
                   widget_control,uv[7],get_uvalue=idxv1 & idx1=strsplit(idxv1,',',/extract) & nx1=n_elements(idx1) ;Operand index 1
                   widget_control,uv[8],get_uvalue=idxv2 & idx2=strsplit(idxv2,',',/extract) & nx2=n_elements(idx2) ;Operand index 2
                   widget_control,uv[3],get_value =fxva  & fxva=strtrim(fxva[0],2)
                   widget_control,uv[6],get_value =fxvu  & fxvu=strtrim(fxvu[0],2) & fxkp=fxvu
                   if lab ne '*' then begin & idx2  =['0'] & nx2 =1 & endif ;Minus Constant

                   if idx1[0] gt ' ' then begin
                   if idx2[0] gt ' ' then begin
                   if (nx2 eq 1) or (nx2 eq nx1) then begin
                      messb12='! Bad curves...'
                      idxL1=long(idx1)-1 & idxL2=long(idx2)-1 & Tlst=cur_menu[2]
                      fxLa =float(fxva)  & fxLu =float(fxvu)
                      if fxLa ne 1. then fxva='*'+fxva else fxva=''
                      if fxLu ne 1. then fxvu='*'+fxvu else fxvu=''

                      if lab eq '*' then begin ;GET FIRST CURVE operand 2
                         rpu=Tlst & sans_reform_series,getlist=rpu,index=idxL2[0]
                         if fxLu ne 1. then begin & rpu.W=rpu.W*fxLu & rpu.E=rpu.E*fxLu & endif
                      endif else rpu={Minus:0,Constant:0}
                      if n_tags(rpu) gt 1 then begin
                         sans_gt_par,sys=wka  & wka=wka+1
                         newrpu=1 & wka=strtrim(wka,2) & wki=strtrim(wka+1,2)
                         sans_accu ,wka ;set accumulator to 0.

                         for k=0,nx1-1 do begin
                           rpa=Tlst & sans_reform_series,getlist=rpa,index=idxL1[k],/allsupl ;Get curves operand 1
                           if fxLa ne 1. then begin & rpa.W=rpa.W*fxLa & rpa.E=rpa.E*fxLa & endif

                           if nx2 gt 1 then begin
                           rpu=Tlst & sans_reform_series,getlist=rpu,index=idxL2[k]          ;Get curves operand 2
                           if fxLu ne 1. then begin & rpu.W=rpu.W*fxLu & rpu.E=rpu.E*fxLu & endif
                           endif else newrpu=(k eq 0)

                           if lab ne '*' then begin
                              rpa.W=rpa.W - float(fxkp)          ;Minus constant
                             ;if float(fxkp) gt 1. then rpa.E=rpa.E - sqrt(float(fxkp))
                              CONF=rpa.TIT & bl=strpos(CONF,' ') ;MODIFY TITLE
                              CONF=strmid(CONF,0,bl)+ ' ('+opr+fxkp+')'+ strmid(CONF,bl,200)
                              rpa.TIT=CONF
                           endif else begin
                             sans_accu, wka,rpa ,opr, wki,rpu ,/newrpu ;Do the operation
                           endelse
                           
                           sans_merge_results,'newi', newISO=rpa     ;Put result in the List

                          ;Pbid=fltarr(max(idpx)+1) & Pbid(idxp)=rpa.P ;POUR MEMO...
                          ;sans_reform_series, rpa.W, dp={X:rpa.Q,E:rpa.E,P:p}, titles='', parconf=rpa.tit, resume=Tlst, transmission=rpa.NTC(1), thickness=rpa.supl(11)

                         endfor
                         gtlist=Tlst & sans_reform_series,getlist  =gtlist
                         widget_control,baset.bases4[17],set_value =gtlist,set_list_top=(n_elements(gtlist)-6)>0
                         if lab ne '*' then $
                              messb12='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ('+fxkp+')' $ ;Minus Constant
                         else messb12='  ['+strmid(idxv1[0],0,12)+']'+fxva+' '+opr+' ['+strmid(idxv2[0],0,12)+']'+fxvu+' is done...'
                      endif
                   endif else messb12='! Incompatible selected lists...'
                   endif else messb12='! Operand 2 is missing...'
                   endif else messb12='! Operand 1 is missing...'

                   widget_control,baset.bases1[12],set_value=messb12
             end

          4: begin
            ;GET GNUM files
            ;**************
               fifi=dialog_pickfile(get_path=gpth,/read ,resource_name='lamptouch',filter='g*.*',/multi,/fix)
               if fifi[0] gt ' ' then begin
                  for f=0,n_elements(fifi)-1 do begin
                      if strpos(fifi[f],'gr') ge 0 then W=sans_read_graspr(fifi[f],datp=dp) $
                                                   else W=sans_read_gt(    fifi[f],datp=dp)
                      supl=sans_supl
                      supl[0] =dp.p[0]  & supl[1] =dp.p[1]  & supl[2] =dp.p[2]  & supl[3] =dp.p[3]
                      supl[4] =dp.p[4]  & supl[8] =dp.p[22] & supl[9] =dp.p[23] & supl[11]=dp.p[17]
                      supl[13]=dp.p[26] & supl[15]=dp.p[27] & supl[20]=dp.p[21] & supl[21]=dp.p[19]
                      supl[22]=dp.p[6]  & supl[23]=dp.p[5]  & supl[24]=dp.p[7]  & supl[25]=dp.p[8]
                      supl[31]=dp.p[28] & supl[46]=dp.p[10] & supl[47]=dp.p[11] & supl[49]=dp.p[18]
                      supl[50]=dp.p[12]
                      P  =fltarr(20) & P[0]=dp.P[0:14]
                      ntc=dblarr(7)
                      ntc[0]  =dp.p[6]  & ntc[1]  =dp.p[20] & ntc[2]  =dp.p[15] & ntc[3]  =dp.p[16]
                      ntc[4]  =1.
                      fufu    =strmid(fifi[f],strlen(gpth),20)
                      pntF    =strpos(fufu,'.') & if pntF gt 0 then fufu=strmid(fufu,0,pntF)
                      conf    =strtrim(dp.par_txt[25],2)   & pntB = strpos(conf,' ')
                      conf    =strmid(conf,0,pntB) + ' '+fufu+' ' + strmid(conf,pntB+1)

                      rpa={TIT:conf,npt:n_elements(dp.X),W:W,X:dp.X,E:dp.E,NTC:ntc,P:P,SUPL:supl}
                      sans_merge_results,'newi', newISO=rpa     ;Put gnum file in the List

                      gtlist=supl[21] & sans_reform_series,getlist =gtlist
                      widget_control,baset.bases4[17],set_value =gtlist,set_list_top=(n_elements(gtlist)-6)>0
                  endfor
               endif
              ;*IDXP*
              ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
              ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight
              ;idx2P=[5,4,0,28,6,23,28,28,28,16,19,20,12,13,14] ;0->14
              ;*OTHER PARAMS FOR SUPL*
              ;center X , center Y , thickness , dTh , typ , transm , group , X Psize , Y Psize , instrument
              ;title , sector open , sector rotat , flux
              ;idx2S=[1,2,22,31,28,21,3,17,18,28,28,10,11,9]    ;15->28
             end
             
          5: begin
            ;SAVE SELECTED CURVES uv(2): 1->gnum  2->4col
            ;********************************************
             idx=widget_info(baset.bases4[17],/list_select) & if idx[0] ge 0 then begin 
               widget_control,baset.bases1[3],get_value=instru
               kpSUPL=sans_supl ;KEEP CURRENT CONFIG
               kpCTR =sans_ctr
               kpTRS2=sans_trans[2]
               if uv[2] ne 1 then W_store,W=29

               for i=0,n_elements(idx)-1 do begin
                 rpa=cur_menu[2] & sans_reform_series,getlist=rpa,index=idx[i],/allsupl ;GET A CURVE
                 sans_supl    =rpa.supl
                 sans_ctr[0:1]=rpa.NTC[2:3] ;x y center
                 sans_trans[2]=rpa.NTC[1]   ;transmission
                 
                 npt =n_elements(rpa.X)
                 m=0 & n=npt-1
                 if rpa.NTC[5] ne rpa.NTC[6] then begin                ;margins  FIRST INDEX
                               if rpa.NTC[5] gt 0. then $
                               m=(where(rpa.X ge rpa.NTC[5]))[0] > 0
                               if rpa.NTC[6] gt 0. then $
                               n=(where(rpa.X ge rpa.NTC[6]))[0] > 0
                               if n le m then n=npt-1
                 endif
                 Win =rpa.W[m:n]*rpa.NTC[4] & E=rpa.E[m:n]*rpa.NTC[4]     ;rescale
                 X   =rpa.X[m:n]

                 P   =dblarr(max(idxP)+1)
                 P[idxP]=rpa.P[0:n_elements(rpa.P)-1]
                 P[idxP[6]]=rpa.NTC[0] ;Numor is double !!!
                 
                 ;*******TITLES DEFINED IN sans_save_gt
                 Wt    =rpa.TIT & Xt='' & Yt='' & seq=200L
                 Y     =0
                 MTmess=rpa.time
                 fifiS ='200'
                 px    =max([strpos(Wt,'mM '),strpos(Wt,'pL '),strpos(Wt,'gH ')  ,$
                             strpos(Wt,'mM.'),strpos(Wt,'pL.'),strpos(Wt,'gH.')])
                             
                 isec1 =strpos(Wt,'.Sect') ;Complete the filename if necessary
                 if isec1 gt px then begin & isec2=strpos(Wt,'.',isec1+1)
                                           seq  =strmid(Wt,isec1+5,isec2-isec1-5) & Y=long(seq)
                                           fifiS=seq  & endif
                 pb= strpos(Wt,' (-')
                 ps= strpos(Wt,' [' )
                 if ps gt px then begin & bc=strpos(Wt,']',ps) & seq=seq+400
                    if strpos(Wt,' [-') gt px then fifiS='minus'+strmid(Wt,ps+3,bc-ps-3) else $
                    if strpos(Wt,' [+') gt px then fifiS='plus' +strmid(Wt,ps+3,bc-ps-3) else $
                    if strpos(Wt,' [/') gt px then fifiS='over' +strmid(Wt,ps+3,bc-ps-3) else $
                    if strpos(Wt,' [*') gt px then fifiS='by'   +strmid(Wt,ps+3,bc-ps-3)
                 endif else $
                 if pb gt px then fifiS='bg'+strmid(Wt,pb+3,strpos(Wt,')',pb)-pb-3)

                 typ=rpa.supl[21] & dQ=0
                 if (typ eq 2) or (typ eq 7) or (typ eq 8) then begin ;Calculate dQ if RADIAL integration
                   dL_L=rpa.supl[50] & Lambda=rpa.supl[0] & dTh=rpa.supl[49]
                   dQ = sans_sigmaQ(X=X , dL_L=dL_L , Lambda=Lambda , dTh=dTh)
                 endif

                 datp={W_tit:Wt, X_tit:Xt, Y_tit:Yt, E:E, X:X, Y:Y,Z:dQ, P:P, PV:0, TIME:MTmess}
                 filcol='gnum file' 
                 if uv[2] eq 1 then $
                    sans_save_gt, win, datp=datp, par=Wt, pZ=0, Inst=instru, fromlist=seq $
                 else begin
                    idinf=strpos(instru,'<') & if idinf gt 0 then instru=strmid(instru,0,idinf)
                    filcol=instru+'_1D_'+strtrim(long64(rpa.NTC[0]),2)+'_'+fifiS+'.txt'
                    give_W,Win, W=29 & give_datp,datp,W=29
                    write_lamp,filcol,W=29,format='column'
                 endelse
               endfor
                        
               sans_trans[2]=kpTRS2 ;RESTORE CURRENT CONFIG
               sans_ctr     =kpCTR
               sans_supl    =kpSUPL
               if uv[2] ne 1 then W_restore,W=29
               
               widget_control,baset.bases1[12],set_value=filcol+' saved ...'
             endif
             end
          else:
          endcase
         end

79:begin & widget_control,ev.id,get_value=polyF & blc=strpos(polyF,' ')
         if blc gt 0 then polyF=strmid(polyF,0,blc)
         if uv[3] eq 1 then begin & baset.Fdisper=uv[1] & polyM='P.Dispersity ('+polyF+')' & endif ;POLY DISPERSITY
         if uv[3] eq 2 then begin & baset.Fstruct=uv[1] & polyM='F.Structure ('+polyF+')'  & endif ;STRUCTURE FACTOR
         widget_control,uv[2],set_value =polyM
         widget_control,uv[2],get_uvalue=FFmenu
         if uv[3] eq 2 then if uv[1] gt 0 then widget_control,FFmenu,sens=0 else widget_control,FFmenu,sens=1
   end

;SECTORS
80:begin & if uv[1] eq 0 then begin & m1=1 & m2=0    & tit='DEFINING SECTORS' & endif else begin & m1=0 & m2=1 & tit='CONFIG 3' & endelse
         widget_control,baset.bases1[7], map=0   & widget_control,baset.bases1[8] ,map=1  & widget_control,baset.bases4[16],map=0
         widget_control,baset.bases5[4], map=0   & widget_control,baset.bases5[5] ,map=0  & widget_control,baset.bases4[12],map=0
         widget_control,baset.bases4[1], map=m2  & widget_control,baset.bases4[0] ,map=0  & widget_control,baset.bases4[7] ,map=m1
         widget_control,baset.bases5[0], map=0   & widget_control,baset.bases5[1] ,map=0  & widget_control,baset.bases4[8] ,map=0
         widget_control,baset.bases5[2], map=0   & widget_control,baset.bases5[3] ,map=0  & widget_control,baset.bases4[10],map=0
         widget_control,baset.bases5[6], map=0   & widget_control,baset.bases5[7] ,map=1  & widget_control,baset.bases5[8] ,map=1
         widget_control,baset.bases5[9], map=0   & widget_control,baset.bases5[19],map=0
        ;higlight NUMOR
         widget_control,baset.bases5[14],sens=1  & widget_control,baset.bases5[15],sens=0
        ;Show CENTER adjustment for SECTORS
         widget_control,baset.bases4[18],map=0   & widget_control,baset.bases4[19],map=1
         widget_control,baset.bases4[20],map=0   & widget_control,baset.bases4[21],map=1
         widget_control,baset.bases4[19],get_uvalue=idC
         sans_settings_text, baset.bases2[2:3],farr, sarr
         widget_control,idC[0],set_value=sarr[0] & widget_control,idC[1],set_value=sarr[1]
         begtxt=' '
        ;Check if symmetric or asymmetric sectors
         if n_elements(uv) ge 3 then begin
            if (uv[2] eq 8)                   then INT_event,{TOP:baset.bases1[6],SELECT:1},[-88,410,1] ; non-symmetrical (AZIMUTHAL)
            if (uv[2] eq 9)  or (uv[2] eq 11) then INT_event,{TOP:baset.bases1[6],SELECT:0},[-88,410,1] ; symmetrical     (RADIAL)
            if (uv[2] eq 10) or (uv[2] eq 12) then INT_event,{TOP:baset.bases1[6],SELECT:1},[-88,410,1] ; non-symmetrical (RADIAL)
            sans_supl[21]=uv[2]-4
                        secMD =(sans_supl[21]-4)>0<5
                        sectxt=[    '2*Pi Azimuthal integration'      ,$
                                    'Total  integration   symmetrical',$
                                    'Total  integration non-symmetric',$
                                    'Radial integration   symmetrical',$
                                    'Radial integration non-symmetric', ' ']
                        if  baset.bases6[22] gt 0  then begtxt='Rocking: ' else begtxt=''
                        if (baset.bases6[22] gt 0) and ((secMD ne 1) and (secMD ne 2)) then begtxt='WARNING!!! '+begtxt
                        begtxt=begtxt+sectxt[secMD]
         endif

         widget_control,baset.bases4[4], set_value=tit & cur_menu[[0,1,10]]=[7,8,3] & if m1 then cur_menu[10]=-1

         cur_cxy[2]=-1
         cur_menu[3:5]=0 & sans_settings_mask,baset,/noWbeam ;show beam sample,(water)
         widget_control,baset.bases1[12],set_value=begtxt
         end
;SECTORS CENTER ADJUSTMENT
81:begin & if uv[1] eq 0 then begin & NewCtr=[0.,0.]
            if (sans_supl[52] eq 1) and (n_elements(cur_aX) gt 1) then begin
             Cx=interpol(findgen(n_elements(cur_ax)),cur_ax,0.) & if Cx lt 0 then NewCtr[0]=Cx-1 else NewCtr[0]=Cx+1
             Cy=interpol(findgen(n_elements(cur_ay)),cur_ay,0.) & if Cy lt 0 then NewCtr[1]=Cy-1 else NewCtr[1]=Cy+1
             Cx=long(Cx*100)/100. & Cy=long(Cy*100)/100.
             if FX eq 0 then if Cx ge 0 then Cx=Cx+1 & Cx=strcompress(Cx,/remove_all)
             widget_control,uv[2], set_value=Cx
             if FX eq 0 then if Cy ge 0 then Cy=Cy+1 & Cy=strcompress(Cy,/remove_all)
             widget_control,uv[3], set_value=Cy
            endif
         endif else begin
            widget_control,uv[2],get_value=Cx & widget_control,uv[3],get_value=Cy
            Cx=strcompress(Cx[0],/remove_all) & Cy=strcompress(Cy[0],/remove_all)
            on_ioerror,mis81
            newCtr=float([Cx,Cy])-FX
            if newCtr[0] lt 0 then newCtr[0]=newCtr[0]+2*FX & if newCtr[1] lt 0 then newCtr[1]=newCtr[1]+2*FX
            mis81:
         endelse
         sans_mask,center=NewCtr
         sans_settings,refresh=baset
         dTh =strtrim(string(round(sans_supl[49]*10000.)/10000.,format='(F6.4)'),2)
         if sans_settings_RegSheet() then sans_sheet_receive, ctr=cx+","+cy+" dTh:"+dTh
   end
;SQUARED SECTORS , ROCKING , Mode
82:begin & if uv[1] eq 1 then begin & baset.bases6[21]=ev.select
                                  cur_cxy[2]=-1   ;***force Ben to change**
                                  sans_settings_mask,baset   & endif   ;SQUARED
         if uv[1] eq 2 then begin & widget_control,ev.id,get_value=RKtxt ;ROCKING
                                  if uv[2] ge 0 then begin
                                    baset.bases6[22]=uv[2]
                                    RKtxt=strmid(RKtxt,0,strpos(RKtxt,' '))
                                    widget_control,baset.bases7[34],set_value='ROCKING: '+RKtxt
                                  endif
                                  secMD =(sans_supl[21]-4)>0<5
                                  sectxt=['2*Pi Azimuthal integration'      ,$
                                          'Total  integration   symmetrical',$
                                          'Total  integration non-symmetric',$
                                          'Radial integration   symmetrical',$
                                          'Radial integration non-symmetric', ' ']
                                  if  baset.bases6[22] gt 0  then begtxt='Rocking: ' else begtxt=''
                                  if (baset.bases6[22] gt 0) and ((secMD ne 1) and (secMD ne 2)) then begtxt='WARNING!!! '+begtxt
                                  widget_control,baset.bases1[12],set_value=begtxt+sectxt[secMD]
         endif
   end

85:begin ;SHOW CORRECTED DATA
          XICUTER,'W18='+(['W18','W31','W32','W33'])[uv[1]]
          TAKE_W,tmpcor,W=18
          if n_elements(tmpcor) gt 1 then begin
             widget_control,  baset.bases3[8]  ,get_uvalue=SData
             sz=size(tmpcor) & if sz[0] eq 3 then sz=sz[3]-1 else sz=0

             sans_settings_runs, baset ,[9,8,12,baset.bases3[2]],/noread   ;Show original data (no re_read)

             cur_wn=18 & cur_menu[7]=18
             widget_control,baset.bases9[11],get_value=vz
             sans_settings_mask , baset, run=tmpcor[*,*,vz<sz]     ;Show corrected data
             widget_control,baset.bases4[4] ,set_value='CORRECTED ->W18 [w31,w32,w33]' & cur_menu[1]=4
             if sz eq 0  then title=sdata.datp.w_tit else title=sdata.par[vz<sz]
             if sz gt 0  then begin & svz=strtrim(vz<sz,2) & XICUTER,'W16=W18[*,*,'+svz+']'
                                                           XICUTE ,'W_tit[16]="<'+title+'>"' & endif
             title=strmid(strcompress(title)+' ('+sdata.opr+' ->W16)',0,55)
             widget_control,baset.bases1[12],set_value=' '
             SEE,W=18,/below,/image & to_did_cur,'W18'
             cur_SamCor=SData & cur_SamCor.Wi=18 & cur_SamCor.DATA=tmpcor
          endif
   end

;GET and USE Water cross section
86:begin & Lambda=sans_supl[0] & XWv=1. & ii=execute('XWv='+baset.XWdep)
         sans_supl[42] = XWv
         widget_control,uv[1], set_value  =string(XWv,format='(F5.3)')
         widget_control,uv[2], set_button =1
   end

;OPTIONS FOR CORRECTIONS and CONFIG
90:begin & widget_control,baset.bases1[7],map=0   & widget_control,baset.bases1[8] ,map=1
         widget_control,baset.bases5[0],map=0   & widget_control,baset.bases5[1] ,map=0
         widget_control,baset.bases5[2],map=0   & widget_control,baset.bases5[3] ,map=0
         widget_control,baset.bases5[6],map=0   & widget_control,baset.bases5[7] ,map=0
         widget_control,baset.bases5[9],map=1   & widget_control,baset.bases5[19],map=0
         memes='OPTIONS FOR DETECTOR CORRECTIONS'
         widget_control,baset.bases4[4],set_value=memes  & cur_menu[0]=9
         widget_control,baset.bases1[12],set_value=' '

         sans_settings_beamST, baset,get_Sbeam=oldDbeam
         sans_settings_beamST, baset, Tog_STbeam='Ctr'  ;Better if we have
         sans_settings_beamST, baset,get_Sbeam=curDbeam        ;the Center
         if oldDbeam ne curDbeam then $
         sans_settings_runs, baset ,[9,22, 9,baset.bases3[16]],/noread,/haveall ;Center

        ;Be sure we have the correct water readed in scattering data
         sans_settings_choice_water, baset, callflag='conf'

        if sans_settings_RegSheet() then begin
            geo=widget_info(baset.bases1[8] ,/geo)
            widget_control, baset.bases1[7] ,Xsize=geo.Xsize
            widget_control, baset.bases1[11],set_value=' '
            widget_control, baset.bases1[7] ,map=1 ;CONFIG
            widget_control, baset.bases1[0] ,set_value=memes
            cur_menu[3:5]=0 & sans_settings_mask,baset    ;show beam for sample and water
        endif
         end
91:begin & baset.bases6[9] =ev.select          & sans_supl[27]=ev.select             ;Treat dead_times
         sans_settings_swap, baset, shareSupl=[27] ,supl=(sans_supl)         & end
92:begin & baset.bases6[10]=ev.select          & sans_supl[28]=ev.select             ;Treat solid_angles
         sans_settings_swap, baset, shareSupl=[28] ,supl=(sans_supl)         & end
93:begin & baset.bases6[11]=ev.select          & sans_supl[29]=ev.select             ;absolute scaling water cross section
         sans_settings_swap, baset, shareSupl=[29] ,supl=(sans_supl)         & end
94:begin & sans_supl[7]    =uv[1]              & sans_settings, refresh=baset        ;Tau function
         sans_settings_swap, baset, shareSupl=[7]  ,supl=(sans_supl)         & end
95:begin & baset.bases6[14]=ev.select          & sans_supl[43]=ev.select             ;Treat thick as f(Q)
         sans_settings_swap, baset, shareSupl=[43] ,supl=(sans_supl)         & end
96:begin & baset.bases6[15]=ev.select          & sans_supl[44]=ev.select             ;Un_scale
         sans_settings_swap, baset, shareSupl=[44] ,supl=(sans_supl)         & end
97:begin & baset.bases6[19]=uv[1]              & sans_supl[48]=baset.bases6[19]      ;Normalise Monitor/Time
         sans_settings_swap, baset, shareSupl=[48] ,supl=(sans_supl)         & end
98:begin & sans_supl[61]   =1-ev.select        & sans_settings, refresh=baset        ;Qdep
         sans_settings_swap, baset, shareSupl=[61] ,supl=(sans_supl)         & end

99:begin & if uv[1] eq 1 then p_did_journal,0L,[-88,396,0,0]
         if uv[1] eq 2 then par_disp,0
         if uv[1] eq 3 then CALIBRATION
         if uv[1] eq 4 then begin & CALIBRATION ,GET_CAL=g_cal, INF=f_inf
                                    if (n_elements(g_cal) gt 1) and (n_elements(f_inf) eq 1) then mesinf=f_inf else mesinf='None'
                                    widget_control,baset.bases1[12],set_value=mesinf
                                    widget_control,bmess           ,set_value=mesinf
         endif
         end
else:
endcase
widget_control,ev.top,set_uvalue=baset
endif
end

pro sans_settings_model, baset, Wtitp, toW, Qrang,QrangOK, maxWW, IQfunc, IDX, IQlog, SCALED=Uscal
;+
;** *******************
;** fitting routine, call str_fit
;-

;MODELS   baset.Fmodel(0)= model number
;         baset.Fmodel(1)= 0:one merged fit   1:none merged fits    2:2D fits

;Wtitp    Title
;toW      Input workspace number
;Qrang    What is in the "Q rg" fields
;QrangOK  What is in the "Q Rg" fields but selected (should be undefined)
;maxWW[0] max of first curve (follow= 1) (fit 1 to N)
;maxWW[1] max of last  curve (follow=-1) (fit N to 1)
;maxWW[2] number of curves
;IQfunc   [FIQf , IofQ , Qsqr] we may have f(Q) and/or f(I(Q))
;IDX      Selected curve index
;IQlog    1 for log(I(Q))
;Uscal    1 for unit scaled result

 cool =[50,100,200,30,75,150,230]
 if Uscal then Bkgds='none' else Bkgds='flat' ;Small Qrange -> try 'none' !!!
         
;Some initial parameters
 maxw0= maxWW[0] ;first curve maximum follow= 1 (fit 1 to N)
 maxw = maxWW[1] ;last  curve maximum follow=-1 (fit N to 1)
 sz   = maxWW[2]
 FIQf = IQfunc[0] & IofQ=IQfunc[1] & Qsqr=IQfunc[2]
 
 R1=200. & R2=R1*.7 & dR=R1*.2 & Rg=50.
 
 ;************************************
 ;TRY A GUINIER for initial parameters
 ;************************************
 Tgui=baset.Fmodel[0]
 if (Tgui eq 0) or (Tgui eq 1) or (Tgui eq 41) or (Tgui eq 46) or (Tgui eq 47) then begin
            formula='Y = P1* EXP(-((((X*P2)^2)/3.)<70.))'
            name   ='Guignier_approx.'
            Um={NAME:name,NPAR:2,PARAMS:[maxw,Rg] ,FITA:[0,0] ,BOUNDA:[1,1] ,LIMITA:[0.,1.] ,FORMULA:formula,$
                                                               BOUNDB:[0,1] ,LIMITB:[0.,9000.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd=Bkgds,User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
                        
            R1=median([par[*,2]]) & R2=R1*.7 & dR=R1*.2 & Rg=R1
 endif

;************************************
;DO THE FIT and get parameters in PAR
;************************************

 case baset.Fmodel[0] of
                    
 ;FORM FACTOR MODELS , FORM STRUCTURE is in baset.Fstruct 0:None  1:Hard Sphere
 ;******************   **************
 ;****************** , POLY DISPERS   is in baset.Fdisper 0:None  1:Gaussian  2:LogNorm  3:Shultz
 ;****************** , ************
 0:   begin & formula='t=x*p2 & y= (4.*!Pi*P1*P2^3*(sin(t) - t*cos(t))/(t)^3)^2'
            name   ='SANS_Sphere' ;*********************
            params =[.1 ,R1,dR]   ;[Scale,Radius,DeltaR]
            
            if baset.Fstruct eq 1 then begin
               formula= [formula,'Y= STR_HARD_SPHERE( X, P1, P2 ) ;(*)SANS HARD SPHERE structure']
               name   = [name   ,'SANS_str_HS']
               params =[[maxw ,R1,dR] , [.5,R1,0.]] ;[ [Scale,Radius,DeltaR] , [VolFrac,Rhs] ]
               Um={NAME:name,NPAR:[3,2],PARAMS:params   , FITA:lonarr(3,2) , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA:[[1,1,1],[0,1,0]],LIMITA:[[0.,1.   ,0.]  ,[0. ,1.   ,0.]] , $
                   BOUNDB:[[0,1,1],[1,1,0]],LIMITB:[[0.,9000.,100.],[.99,9000.,0.]] }
            endif else $
               Um={NAME:name,NPAR: 3   ,PARAMS:params   , FITA:lonarr(3)   , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA: [1,1,1]         ,LIMITA: [0.,1.   ,0.]  ,$
                   BOUNDB: [0,1,1]         ,LIMITB: [0.,9000.,100.] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 1:   begin & formula='t1=X*P2 & t2=X*P3 & Y=(4.*!Pi*(P1*P2^3*(sin(t1)-t1*cos(t1))/t1^3) - (P1*P4*P3^3*(sin(t2)-t2*cos(t2))/t2^3) )^2'
            name   ='SANS_H_Sphere'        ;***********************************
            params =[.1 ,R1,R2,1.,dR]      ;[Scale,Radius1,Radius2,1-Mu,DeltaR] 
            
            if baset.Fstruct eq 1 then begin
               formula= [formula,'Y= STR_HARD_SPHERE( X, P1, P2 ) ;(*)SANS HARD SPHERE structure']
               name   = [name   ,'SANS_str_HS']
               params =[[maxw^.5 ,R1,R2,1.,dR] , [.5,R1,0,0,0]] ;[ [Scale,Radius1,Radius2,1-Mu,DeltaR], [VolFrac,Rhs] ]
               Um={NAME:name,NPAR:[5,2],PARAMS:params   , FITA:lonarr(5,2) , FORMULA:formula , FOLLOW:-1 , $
                   BOUNDA:[[1,1,1,0,1],[0,1,0,0,0]],LIMITA:[[0.,1.   ,1.   ,0.,0.]  ,[0. ,1.   ,0.,0,0]] , $
                   BOUNDB:[[0,1,1,0,1],[1,1,0,0,0]],LIMITB:[[0.,9000.,9000.,0.,100.],[.99,9000.,0.,0,0]] }
            endif else $
               Um={NAME:name,NPAR: 5   ,PARAMS:params   , FITA:lonarr(5)   , FORMULA:formula , FOLLOW:-1 , $
                   BOUNDA: [1,1,1,0,1]             ,LIMITA: [0.,1.   ,1.   ,0.,0.] , $
                   BOUNDB: [0,1,1,0,1]             ,LIMITB: [0.,9000.,9000.,0.,100.] }

            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 41:  begin & formula='Y= STR_VESICLE(X, P1, P2, P3, P4, P5, P6, P7) ;SANS Bilayer VESICLE'
            name   ='SANS_Bilayer_VESICLE' ;*****************************************************
            params =[R1 ,1 ,1 ,1 ,1 ,1 ,1] ;[Radius,ThickIN,ThickLayer,sldSOLV,sldIN,sldLAYER,dR]
            
               Um={NAME:name,NPAR: 7   ,PARAMS:params   , FITA:lonarr(7)   , FORMULA:formula , FOLLOW:-1 , $
                   BOUNDA: [1,1,1,1,1,1,1]         ,LIMITA: [0.,0.   ,0.   ,0. ,0,0,0] , $
                   BOUNDB: [0,0,0,0,0,0,0]         ,LIMITB: [0.,0.   ,0.   ,0. ,0,0,0] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 44:  begin & formula='Y= STR_ELLIPSOID(X, P1, P2, P3, P4, LOOP=50) ;SANS ELLIPSOID'
            name   ='SANS_ELLIPSOID' ;***************************
            params =[.01 ,R1 ,1 ,dR] ;[sldensity,Radius,Ratio,dR]
            
               Um={NAME:name,NPAR: 4   ,PARAMS:params   , FITA:lonarr(4)   , FORMULA:formula , FOLLOW:-1 , $
                   BOUNDA: [1,1,1,1]               ,LIMITA: [0.,0.   ,0.   ,0.] , $
                   BOUNDB: [0,0,0,0]               ,LIMITB: [0.,0.   ,0.   ,0.] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 46:  begin & formula='Y = STR_STARPOLYGAUSS(X, P1, P2, P3, P4) ;SANS STAR Polymer'
            name   ='SANS_STAR_Polymer'   ;**************************
            params =[maxw ,Rg ,16 ,dR]    ;[I0,RadiusGyr,Arms,DeltaR]
             
               Um={NAME:name,NPAR: 4   ,PARAMS:params   , FITA:lonarr(4)   , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA: [1,1,1,1]         ,LIMITA: [0., 1., 2, 0]  ,$
                   BOUNDB: [0,0,0,0]         ,LIMITB: [0., 0., 0, 0] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 47:  begin & formula='Y = STR_STARPOLYPOLYG(X, P1, P2, P3, P4) ;SANS STAR Polydisperse Polymer'
            name   ='SANS_STAR_PolyPoly' ;**************************
            params =[maxw ,Rg ,16 ,dR]   ;[I0,RadiusGyr,Arms,DeltaR]

               Um={NAME:name,NPAR: 4   ,PARAMS:params   , FITA:lonarr(4)   , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA: [1,1,1,1]         ,LIMITA: [0., 1., 2, 0]  ,$
                   BOUNDB: [0,0,0,0]         ,LIMITB: [0., 0., 0, 0] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end
 48:  begin & formula='Y=STR_DISK(X, P1, P2, P3)'
            name   ='SANS_DISK' ;****************************
            params =[1. ,Rg,dR] ;[ScatContrast,Radius,DeltaR]
               Um={NAME:name,NPAR: 3   ,PARAMS:params   , FITA:lonarr(3)   , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA: [1,1,1]         ,LIMITA: [0.,1.   ,0.]  ,$
                   BOUNDB: [0,1,1]         ,LIMITB: [0.,1000.,100.] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end      
 49:  begin & formula='Y=STR_ROD(X, P1, P2, P3)'
            name   ='SANS_ROD'  ;****************************
            params =[1. ,Rg,dR] ;[ScatContrast,Length,DeltaL]
               Um={NAME:name,NPAR: 3   ,PARAMS:params   , FITA:lonarr(3)   , FORMULA:formula , FOLLOW:-1 ,$
                   BOUNDA: [1,1,1]         ,LIMITA: [0.,1.   ,0.]  ,$
                   BOUNDB: [0,1,1]         ,LIMITB: [0.,1000.,100.] }
            
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc,Res_dfit=fitd ,Res_disp=baset.Fdisper,Res_dti=23)
      end      

 100: begin ;LAUNCH STR_FIT WITH INITIAL SETTINGS
            if n_elements(QrangOK) ne 2 then Frng=[' Min',' Max'] $
                                        else Frng=strtrim(QrangOK,2)
            iniGUI={WIN:toW, FRNG:Frng, DTI:23, MODL:1, DISP:baset.Fdisper, IQlg:IQlog, $
                    S_STRU:baset.Fstruct-1}
            str_fit,iniGUI=iniGUI
            return
      end
 ;DISTRIBUTION MODELS
 ;*******************
 2:   begin & name   ='Gaussian'
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',/Gauss,$
                        Xrange=QrangOK  ,fit=fitc) ;,Res_dfit=fitd) 
      end
 3:   begin & name   ='Lorentz'
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',/Lorentz,$
                        Xrange=QrangOK  ,fit=fitc) ;,Res_dfit=fitd) 
      end
 4:   begin & name   ='Pseudo-Voigt'
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',/Pseudo,Res_pvf=0.,$
                        Xrange=QrangOK  ,fit=fitc) ;,Res_dfit=fitd) 
      end

 ;SHAPE-INDEPENDENT MODELS
 ;************************
 11:  begin & formula='Y = P1* EXP(-((((X*P2)^2)/3.)<70.))'
            name   ='Guignier_approx.'
            Um={NAME:name,NPAR:2,PARAMS:[maxw,Rg] ,FITA:[0,0] ,BOUNDA:[1,1] ,LIMITA:[0.,1.] ,FORMULA:formula,$
                                                               BOUNDB:[0,1] ,LIMITB:[0.,9000.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
      end
 12:  begin & formula='Y = P1/(X^4)'
            name   ='Porod_limit'
            Um={NAME:name,NPAR:1,PARAMS:[1.] ,FITA:[0] ,BOUNDA:[1] ,LIMITA:[0.] ,FORMULA:formula,$
                                                        BOUNDB:[0] ,LIMITB:[0.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
      end
 13:  begin & formula='Y = P1/(X^P2)'
            name   ='Power_law'
            Um={NAME:name,NPAR:2,PARAMS:[1.,4.] ,FITA:[0,0] ,BOUNDA:[1,1] ,LIMITA:[0.,0.] ,FORMULA:formula,$
                                                             BOUNDB:[0,0] ,LIMITB:[0.,0.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
      end
 14:  begin & formula='QRg=(X*P2)^2 & Y = P1*2.*(exp(-(QRg<70.))+QRg-1.)/QRg^2'
            name   ='Debye'
            Um={NAME:name,NPAR:2,PARAMS:[maxw,Rg] ,FITA:[0,0] ,BOUNDA:[1,1] ,LIMITA:[0.,1.] ,FORMULA:formula,$
                                                               BOUNDB:[0,1] ,LIMITB:[0.,9000.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
      end
 15:  begin & formula='Y = P1 /(1.+ P2*X^2 + P3*X^4)'
            name   ='Teubner_Strey'
            C1=-(total(Qrang)/2.)^2
            C2= (total(Qrang)/2.)^4
            Um={NAME:name,NPAR:3,PARAMS:[maxw,C1,C2] ,FITA:[0,0,0] ,BOUNDA:[1,0,1] ,LIMITA:[0.,0.,0.] ,FORMULA:formula,$
                                                                    BOUNDB:[0,1,0] ,LIMITB:[0.,0.,0.] ,FOLLOW:-1}
            par=str_fit(0,'Sans_fit.txt',Wtitp,Win=toW,Bkgd='flat',User_model=Um,$
                        Xrange=Qrang    ,fit=fitc) ;,Res_dfit=fitd)
      end
 16:  begin & name   ='Invariant' ; just an integral
            TAKE_W, WW, W=toW
            TAKE_datp, datp, W=toW  &  XX=datp.X
            par=fltarr(sz)
            szX=size(XX) & if szX[0] eq 2 then szX=szX[2]-1 else szX=0
            for k=0,sz-1 do begin
                W=WW[*,k] & W=W-min(W) ;Background should be fitted !!!
                X=XX[*,k<szX]
                Un=uniq(X)
                if n_elements(X) ne n_elements(Un) then begin & X=X[Un] & W=W[Un] & endif
                par[k]= int_tabulated(X , W*X^2)
            endfor
      end
 else:
 endcase

;************************************************************
;SHOW WANTED PARAMETERS IN STRING b11tit FOR ONE (merged) FIT
;************************************************************

 if (baset.Fmodel[1] eq 0)  or  (sz eq 1) then begin ;One merged Fit

 case baset.Fmodel[0] of
 
 ;FORM FACTOR MODELS
 ;******************
 0:   begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** SPHERE **
                           P3=strtrim(round(par[3] *10)/10.,2)
                           if baset.Fdisper eq 0 then P3='0'
            b11tit='Radius =' +P2+' DeltaR ='+P3+' Density ='+P1
      end
 1:   begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** H_SPHERE **
                           P3=strtrim(round(par[3] *10)/10.,2) & P4=strtrim(round(par[10]*10)/10.,2)
                           P5=strtrim(round(par[11]*10)/10.,2)
                           if baset.Fdisper eq 0 then P5='0'
            b11tit='Radius1=' +P2+' Radius2='+P3+' DeltaR='+P5+' 1-Mu ='+P4
      end 
 46:  begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** START Polymer **
                           P3=strtrim(round(par[3] *10)/10.,2) & P4=strtrim(round(par[10]*10)/10.,2)
                           if baset.Fdisper eq 0 then P4='0'
            b11tit='Radius gyr=' +P2+' Arms ='+P3+' DeltaR ='+P4+' Scale ='+P1
      end
 47:  begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** STAR Poly Polymer **
                           P3=strtrim(round(par[3] *10)/10.,2) & P4=strtrim(round(par[10]*10)/10.,2)
                           if baset.Fdisper eq 0 then P4='0'
            b11tit='Radius gyr=' +P2+' Arms ='+P3+' DeltaR ='+P4+' Scale ='+P1
      end
 48:  begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** DISK **
                           P3=strtrim(round(par[3] *10)/10.,2)
                           if baset.Fdisper eq 0 then P3='0'
            b11tit='Radius =' +P2+' DeltaR ='+P3+' Scale ='+P1
      end
 49:  begin & bkgrd=par[4] & P1=strtrim(round(par[1]),2)         & P2=strtrim(round(par[2] *10)/10.,2) ;** ROD **
                           P3=strtrim(round(par[3] *10)/10.,2)
                           if baset.Fdisper eq 0 then P3='0'
            b11tit='Length =' +P2+' DeltaL ='+P3+' Scale ='+P1
      end
                               
 ;DISTRIBUTION MODELS
 ;*******************
 2:   begin & bkgrd=par[4]
            b11tit='Integral='+strtrim(par[9],2) & end
 3:   begin & bkgrd=par[4]
            b11tit='Integral='+strtrim(par[9],2) & end
 4:   begin & bkgrd=par[4]
            b11tit='Integral='+strtrim(par[9],2) & end

 ;SHAPE-INDEPENDENT MODELS
 ;************************
 11:  begin & bkgrd=par[4]
            b11tit='Radius (Guinier)='+strtrim(round(par[2]*10.)/10.,2)
      end
 12:  begin & bkgrd=par[4] & P1=strtrim(par[1],2) & P2=strtrim(par[4],2)
            b11tit='Scale='+P1+' m=4'+' Bck='+P2
      end
 13:  begin & bkgrd=par[4] & P1=strtrim(par[1],2);& P2=strtrim(round(par(2)*10.)/10.+1.,2)
                           P3=strtrim(par[4],2) & P2=strtrim(round(par[2]*10.)/10.,2)
           ;b11tit='Scale(Ang-'+P2+')='+P1+' Bck='+P3
            b11tit='Scale='+P1+' m='+P2+' Bck='+P3
      end
 14:  begin & bkgrd=par[4] & P1=strtrim(par[1],2) & P2=strtrim(round(par[2]*10.)/10.,2)
                           P3=strtrim(par[4],2)
            b11tit='Radius='+P2+' Bck='+P3
      end
 15:  begin & bkgrd=par[4] & P1=strtrim(par[1],2) & P2=strtrim(par[2],2)
                           P3=strtrim(par[3],2) & P4=strtrim(par[4],2)
                           Xi=strtrim((.5*(1./par[3])^(.5) + .25*par[2]/par[3])^(.5),2)
                           Dm=strtrim((.5*(1./par[3])^(.5) - .25*par[2]/par[3])^(.5),2)
            b11tit='Scale(Cm-1)='+P1+' C1[A^2]='+P2+' C2[A^4]='+P3+' Xi[A]'+Xi+' dm[A]'+Dm+' Bck='+P4
            b11tit=strmid(b11tit,0,70)
      end
 16:  begin & bkgrd=0.
            b11tit='Invariant:'+strtrim(par[0],2)
      end
 else:
 endcase

 if n_elements(fitd) le 1 then fitd=0.
 fitd=fitd+bkgrd

;****************
;NOW PLOT THE FIT
;****************
 if n_elements(fitc) gt 1 then begin
     TAKE_datp, datp, W=toW  &  XX=datp.X
     XF=XX
     if FIQf  then begin
        if IofQ eq 1 then begin & fitc=fitc * XX   & fitd=fitd * XX   & endif
        if IofQ eq 2 then begin & fitc=fitc * XX^2 & fitd=fitd * XX^2 & endif
        if IofQ eq 3 then begin & fitc=fitc * XX^4 & fitd=fitd * XX^4 & endif
        if Qsqr eq 1 then XF=XX^2
        if Qsqr eq 2 then XF=XX^4
     endif
                                   oplot,XF,fitc,col=cool[4],thick=2 ;Fit convoluted
     if n_elements(fitd) gt 1 then oplot,XF,fitd,col=cool[5],thick=1 ;Model
     if n_elements(fitd) gt 1 then $
        ligend,['Fit','Model'],colors=cool[4:5],textcolors=cool[4:5],linestyl=[0,0],thick=[2,1],/BOTTOM,/LEFT,/box
 endif
 
 endif ;One Fit merged PLOT

;*****************************************
;PLOT WANTED PARAMETERS WHEN MULTIPLE FITS
;*****************************************

 if (baset.Fmodel[1] eq 1) and (sz gt 1) then begin ;Fit one by one PLOT

 case baset.Fmodel[0] of
 0:   begin & b11tit='Radius'    & toplot1=par[*,2] & end
 1:   begin & b11tit='Radius'    & toplot1=par[*,2] & toplot2=par[*,3] & end
                               
 ;DISTRIBUTION MODELS
 2:   begin & b11tit='Integrals' & toplot1=par[*,9] & end
 3:   begin & b11tit='Integrals' & toplot1=par[*,9] & end
 4:   begin & b11tit='Integrals' & toplot1=par[*,9] & end

 ;SHAPE-INDEPENDENT MODELS
 11:  begin & b11tit='Radius'              & toplot1=par[*,2] & end
 12:  begin & b11tit='Scale (A-5)'         & toplot1=par[*,1] & end
 13:  begin & b11tit='Scale (A-(m+1)) , m' & toplot1=par[*,1]
                                           toplot2=par[*,2] & end
 14:  begin & b11tit='Radius , Bck'        & toplot1=par[*,2]
                                           toplot2=par[*,4] & end
 15:  begin & b11tit='Scale (Cm-1) , Bck'  & toplot1=par[*,2]
                                           toplot2=par[*,4] & end
 16:  begin & b11tit='Invariant'           & toplot1=par[*]   & end
 else:
 endcase

 plot,idx+1,toplot1,col=cool[0],psym=0,thick=2,Xtit='Curve index',Ytit=b11tit,title=name
 if n_elements(toplot2) gt 1 then oplot,idx+1,toplot2,col=cool[1],psym=2,symsize=.5

 endif ;MULTIPLE FITS
 
 if baset.Fmodel[1] eq  2 then if (sz eq 2) or (sz eq 3) then begin ;2D Fit PLOT                     ;Fit2D
 endif ; NOT USED AT THIS TIME
 
 widget_control,baset.bases1[11],set_value=b11tit
 
end ;SANS_SETTINGS_MODEL

 ;cur_menu(0)  Ranges/box input-> 0:Center Rg 1:Transm Rg 2:Mask box/beam 3:Scatt data
 ;             6:Batch macros 7:Sectors 9:Config 3buts 19:we are in LIST of RESULTS
 ;cur_menu(1)  Views 4:2d view 5:Averaging 8:Sector view
 ;cur_menu(2)  Averaging option 2:2PI Rad 4:2PI Azim 5:Sec tot sym 6:Sec tot asym 7:Sec rad sym 8:Sec rad asym
 ;cur_menu(3:5)Hide mask if 1. 3:Sample 4:Detector 5:Water
 ;cur_menu(6)  While in Mask 1:Sample beam 2:Water beam 3:Detect all conf 4:Detect cur conf
 ;cur_menu(7)  Workspace number i.e. 3,12,7,8,17,18
 ;cur_menu(8)  Enable efficiencies correct =1
 ;cur_menu(9)  cur_menu(2) curlist  71:List of Results 77:Restoring Results !!!
 ;cur_menu(10) data input-> 0:Center menu 1:Transmition menu 2:Mask menu 3:Data menu -1:Averaging

pro sans_settings_read,  baset ,fifiswap ,memswap=memswap
;+
;** ******************
;** ******************
;** GET A CONFIG FROM FILE OR MEMORY and FILL sans_settings GUI
;-

common c_sans_tofl,      TofFlux, LambdaVq
common c_sans_cur_run
common c_sans_swapLMH

if n_elements(fifiswap) eq 1 then begin & result=fifiswap & gpth=''
endif else begin
; ----->Ask for an input file
  widget_control,baset.bases1[4],get_value=filtr & filtr=strcompress(filtr[0],/remove_all)
  filtr=[filtr,'*.sav']
  widget_control,baset.bases1[1],get_value=filer & filer=strcompress(filer[0],/remove_all)
  if filer eq '' then $
  widget_control,baset.bases1[2],get_value=filer & filer=strcompress(filer[0],/remove_all)
  pth   =baset.path
  parent=baset.bases1[6]
  result=dialog_pickfile(filter=filtr,path=pth,get_path=gpth,file=filer,dialog=parent,/read,/must_exist,resource_name='lamptouch')
endelse
if result ne '' then begin
   file=strmid(result,strlen(gpth))

   widget_control,/hourglass & cur_run=0 & cur_wn=0

; ----->Get the structure
   mess=1
   if n_elements(fifiswap) eq 1 then begin
      if strpos( fifiswap,".sav") lt 0 then begin & mess=0 ;Read from memory in settings
         if strpos(fifiswap,"low")    ge 0 then begin & settings=sans_struct_low    & baset.curLMH=1 & endif
         if strpos(fifiswap,"middle") ge 0 then begin & settings=sans_struct_middle & baset.curLMH=2 & endif
         if strpos(fifiswap,"high")   ge 0 then begin & settings=sans_struct_high   & baset.curLMH=3 & endif
      endif else       restore,result ;Unused
   endif    else begin & restore,result & cur_eff=1. & endelse

   if n_elements(settings) gt 0 then tags=tag_names(settings) else tags=['']
   if tags[0] ne "INSTRUMENT" then begin
      if n_elements(RLstruct) gt 0 then tags=tag_names(RLstruct) else tags=['']

      if tags[0] eq "SBASET"  then begin & msg="Loading from a Sans input sheet saved file"
         sans_settings_swap, LMH="low"   , take=RLstruct.low_struct
         sans_settings_swap, LMH="middle", take=RLstruct.middle_struct
         sans_settings_swap, LMH="high"  , take=RLstruct.high_struct
         baset.savedLMH =RLstruct.sbaset.savedLMH
         baset.curLMH   =RLstruct.sbaset.curLMH
        ;if RLstruct.sbaset.DATApth gt ' ' then sans_sheet_lampies,SETpth=RLstruct.sbaset.DATApth
         if RLstruct.sbaset.curLMH gt 0 then begin & k=RLstruct.sbaset.curLMH-1
                                        widget_control,baset.bases1[6],base_set_title="  "+(["LOW","MIDDLE","HIGH"])[k]+ " current Angle  "
                                        sans_settings_swap, baset,/get,LMH=(["low","middle","high"])[k] & endif
      endif else msg="!!! bad input setting file"
      if n_elements(iso_NTC)+n_elements(azi_NTC) gt 2 then begin
                 msg="results"
                 sans_reform_series,clear=-1
                 sans_sav_rest_results,'rest', path=gpth, file=file
                 ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:baset.bases7[27],TOP:baset.bases1[6],HANDLER:0L,SELECT:1}')
      endif
      if mess then widget_control,baset.bases1[0],set_value=msg
      return
   endif

; ----->Fill the GUI
   tags=tag_names(settings)
   for i=0,n_elements(tags)-1 do case tags[i] of
   'INSTRUMENT':  begin & widget_control,baset.bases1[3] ,set_value =settings.instrument   & end
   'ATTENUA':     begin &                                                                    baset.attenu=settings.attenua & end
   'COLLIMATION': begin & collim=strsplit(/EXTRACT,settings.collimation,',')
                        widget_control,baset.bases1[5] ,set_value =collim                  & baset.collim=settings.collimation & end
   'COLIDX':      begin & widget_control,baset.bases1[5] ,set_list_select=settings.colidx  & baset.colidx=settings.colidx  & end
   'COLVAL':      begin &                                                                    baset.colval=settings.colval  & end
   'FILTER':      begin & widget_control,baset.bases1[4] ,set_value =settings.filter       & end
   'FACTOR':      begin &                                                                    baset.factor=settings.factor  & end
   'QDEP':        begin &                                                                    baset.Qdep=settings.Qdep      & end

   'WATER':       begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[0]  ,set_value =settings.water    & end
   'WATERE':      begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[1]  ,set_value =settings.waterE   & end
   'CADMI_W':     begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[5]  ,set_value =settings.cadmi_w  & end

   'SAMPLE':      begin &                                    widget_control,baset.bases3[2]  ,set_value =settings.sample   & end
   'SAMPLEE':     begin &                                    widget_control,baset.bases3[3]  ,set_value =settings.sampleE  & end
   'CADMI_S':     begin &                                    widget_control,baset.bases3[4]  ,set_value =settings.cadmi_s  & end
   'RUNFLUX':     begin &                                    widget_control,baset.bases3[41] ,set_value =settings.runflux  & end

   'DATAW':       begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[6]  ,set_uvalue=settings.dataw    & end
   'DATAWE':      begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[7]  ,set_uvalue=settings.datawE   & end
   'DATACW':      begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                                                           widget_control,baset.bases3[11] ,set_uvalue=settings.datacw   & end

   'DATAS':       begin &                                    widget_control,baset.bases3[8]  ,set_uvalue=settings.datas    & end
   'DATASE':      begin &                                    widget_control,baset.bases3[9]  ,set_uvalue=settings.datasE   & end
   'DATACS':      begin &                                    widget_control,baset.bases3[10] ,set_uvalue=settings.datacs   & end
   'TRFLUX':      begin &                                    widget_control,baset.bases3[42] ,set_uvalue=settings.trflux   & end

   'HISTORY':     begin & if (not keyword_set(memswap)) then begin
                         trang  =settings.history.transmRG
                      widget_control,baset.bases7[0]  ,set_value =trang[0]
                      widget_control,baset.bases7[1]  ,set_value =trang[1]
                      widget_control,baset.bases7[2]  ,set_value =trang[2]
                      widget_control,baset.bases7[3]  ,set_value =trang[3]

                         mods   =settings.history.mods
                         baset.bases6[4:n_elements(mods)+3]=mods ;4:*

                         attdt  =settings.history.attdata
                      widget_control,baset.bases3[12] ,set_value =attdt[0]
                      widget_control,baset.bases3[13] ,set_value =attdt[1]
                      widget_control,baset.bases3[14] ,set_value =attdt[2]
                      widget_control,baset.bases3[15] ,set_value =attdt[3]
                      widget_control,baset.bases3[17] ,set_value =attdt[5]

                      adataw =settings.history.trwatr   & adatawE=settings.history.trwatrE
                      adatas =settings.history.trsampl  & adatasE=settings.history.trsamplE
                      adatabw=settings.history.trbeamw
                      widget_control,baset.bases3[18] ,set_uvalue=adataw
                      widget_control,baset.bases3[19] ,set_uvalue=adatawE
                      widget_control,baset.bases3[20] ,set_uvalue=adatas
                      widget_control,baset.bases3[21] ,set_uvalue=adatasE
                      widget_control,baset.bases3[23] ,set_uvalue=adatabw
                      endif

                         crang  =settings.history.centerRG
                      widget_control,baset.bases6[0]  ,set_value =crang[0]
                      widget_control,baset.bases6[1]  ,set_value =crang[1]
                      widget_control,baset.bases6[2]  ,set_value =crang[2]
                      widget_control,baset.bases6[3]  ,set_value =crang[3]

                         mskdt  =settings.history.mskdata
                      widget_control,baset.bases3[24] ,set_value =mskdt[0]
                      widget_control,baset.bases3[25] ,set_value =mskdt[1]

                      mdatas =settings.history.mksampl  & mdataw =settings.history.mkwatr
                      widget_control,baset.bases3[26] ,set_uvalue=mdatas
                      widget_control,baset.bases3[27] ,set_uvalue=mdataw

                        widget_control,baset.bases3[28] ,get_uvalue=CONFU & okt=(CONFU.T ne settings.CONFUS.T)
                        if okt then sans_settings_beamST,baset, Tog_STbeam=CONFU.T ;TOGGLE BEAM FOR Center<-->Transmisssion
                        widget_control,baset.bases3[28] ,get_uvalue=CONFU
                        widget_control,baset.bases3[16] ,set_value =settings.history.attdata[4]
                        widget_control,baset.bases3[22] ,set_uvalue=settings.history.trbeams
                        if okt then sans_settings_beamST,baset, Tog_STbeam=CONFU.T ;Reset
                  end
   'CONFUS':      begin & widget_control,baset.bases3[28] ,get_uvalue=CONFU & okt=(CONFU.T ne settings.CONFUS.T)
                        if okt then sans_settings_beamST,baset, Tog_STbeam=CONFU.T ;TOGGLE BEAM FOR Center<-->Transmisssion
                        widget_control,baset.bases3[28] ,get_uvalue=CONFU
                        settings.CONFUS.S=CONFU.S ;!!!
                        widget_control,baset.bases3[28] ,set_uvalue=settings.CONFUS
                        if okt then sans_settings_beamST,baset, Tog_STbeam=CONFU.T ;Reset
                  end
   'TOFFLUX':     begin & TofFlux  = settings.TOFFLUX
                        if n_tags(TofFlux) gt 1 then begin & read_hdf,/Compile & hdf_set_SansTof, TofFlux & endif
                  end
   'LAMBDAVQ':    begin & LambdaVq = settings.LAMBDAVQ
                  end
   'LOW_STRUCT':        sans_struct_low   =settings.low_struct
   'MIDDLE_STRUCT':     sans_struct_middle=settings.middle_struct
   'HIGH_STRUCT':       sans_struct_high  =settings.high_struct
   else:
   endcase
   sans_settings_current, settings,baset, /setrun ,memswap=memswap
   sans_settings,         refresh =baset, /param

   if n_elements(fifiswap) ne 1 then begin & baset.path=gpth & widget_control,baset.bases1[1],set_value=file
   endif else begin & widget_control,baset.bases1[6],set_uvalue=baset
                   ;sans_settings_runs,baset,[9,9 ,0,baset.bases3(3)],/config ; SHOW SOMETHING !!!
   endelse
endif
end

pro sans_settings_write, baset, fifiswap
;+
;** *******************
;** *******************
;** GET GUI INFO and STORE CONFIG IN FILE OR MEMORY
;-

common c_sans_settings
common c_sans_tofl,      TofFlux, LambdaVq
common c_sans_cur_run
common c_sans_swapLMH

bmess= baset.bases1[0] & FX=sans_supl[30]

; ----->Get all GUI fields
bases=[baset.bases2[0:3],baset.bases2[6:9],baset.bases2[22:23]]
sans_settings_text, bases, farr, sarr, mess
farr[2:3]=farr[2:3]-FX & farr[2:3]=sans_ctr[0:1] ;I do prefer sans_ctr !!!
if mess ne '' then begin
   bad=(['Wavelength','Distance','X center','Y center','Offset','Angle','X beam','Y beam','Wcoll','Wdist'])[fix(mess)]
   widget_control,bmess,set_value='!!! bad input in "'+bad+'" field' & return
endif
transv=strarr(4) & widget_control,baset.bases2[11],get_value=strt & transv[0]=strmid(strt,0,5) ;unused
                   widget_control,baset.bases2[12],get_value=strt & transv[1]=strmid(strt,0,5) ;unused
                   widget_control,baset.bases2[13],get_value=strt & transv[2]=strmid(strt,0,5) ;unused
                   widget_control,baset.bases2[14],get_value=strt & transv[3]=strmid(strt,0,5) ;unused

;sans_settings_mask,  baset ;NECESSARY ?

collima=baset.collim & colidx=baset.colidx & colval=baset.colval & fct=baset.factor & idxi=baset.idxi
attenua=baset.attenu & Qdep  =baset.Qdep
widget_control,baset.bases1[3] ,get_value=instru
widget_control,baset.bases2[4] ,get_value=masq    & num=0 & ii=0
; ----->Data run# to 
widget_control,baset.bases1[4] ,get_value=filtr   & filtr =strcompress(filtr[ 0] ,/remove_all) & i1=1 & i2=1 & i3=1  & i4=1 & i5=1 & i6=1
widget_control,baset.bases3[0] ,get_value=water   & water =strcompress(water[ 0] ,/remove_all) & i=strpos(water ,'=') & if water  ne '' then ii=execute("i1=0 & on_ioerror,mis1 & num=long('"+strmid(water ,i+1,50)+"')  & i1=1 & mis1:")
widget_control,baset.bases3[1] ,get_value=waterE  & waterE=strcompress(waterE[0] ,/remove_all) & i=strpos(waterE,'=') & if waterE ne '' then ii=execute("i2=0 & on_ioerror,mis2 & num=long('"+strmid(waterE,i+1,50)+"')  & i2=1 & mis2:")
widget_control,baset.bases3[2] ,get_value=sampl   & sampl =strcompress(sampl[ 0] ,/remove_all) & i=strpos(sampl ,'=') & if sampl  ne '' then ii=execute("i3=0 & on_ioerror,mis3 & num=long('"+strmid(sampl ,i+1,50)+"')  & i3=1 & mis3:")
widget_control,baset.bases3[3] ,get_value=samplE  & samplE=strcompress(samplE[0] ,/remove_all) & i=strpos(samplE,'=') & if samplE ne '' then ii=execute("i4=0 & on_ioerror,mis4 & num=long('"+strmid(samplE,i+1,50)+"')  & i4=1 & mis4:")
widget_control,baset.bases3[4] ,get_value=cadmis  & cadmiS=strcompress(cadmis[0] ,/remove_all) & i=strpos(cadmiS,'=') & if cadmiS ne '' then ii=execute("i5=0 & on_ioerror,mis5 & num=long('"+strmid(cadmiS,i+1,50)+"')  & i5=1 & mis5:")
widget_control,baset.bases3[5] ,get_value=cadmiw  & cadmiW=strcompress(cadmiw[0] ,/remove_all) & i=strpos(cadmiW,'=') & if cadmiW ne '' then ii=execute("i6=0 & on_ioerror,mis6 & num=long('"+strmid(cadmiW,i+1,50)+"')  & i6=1 & mis6:")
if i1+i2+i3+i4+i5+i6 ne 6 then widget_control,bmess,set_value='warning: Data files input not in syntax or not run numbers'

; ----->Sectors
widget_control,baset.bases7[7] ,get_value=asl   & sans_supl[13]=asl
widget_control,baset.bases7[8] ,get_value=gsl   & sans_supl[14]=gsl
widget_control,baset.bases7[9] ,get_value=bsl   & sans_supl[15]=bsl
widget_control,baset.bases7[10],get_value=zonID & sans_supl[16]=zonID
widget_control,baset.bases7[11],get_value=r1ID  & sans_supl[17]=r1ID  ;round(r1ID/cur_fct[0])!
widget_control,baset.bases7[12],get_value=r2ID  & sans_supl[18]=r2ID  ;round(r2ID/cur_fct[0])!

widget_control,baset.bases7[14],get_value=radin & if strpos(radin[0],'*') lt 0 then sans_supl[20]=radin
;widget_control,baset.bases7(15),get_value=Phi0W & if strpos(Phi0W(0),'*') lt 0 then sans_supl(35)=Phi0W
widget_control,baset.bases7[16],get_value=Tauv  & if strpos(Tauv[0] ,'*') lt 0 then sans_supl[6] =Tauv
widget_control,baset.bases7[6] ,get_value=crosW & if strpos(crosW[0],'*') lt 0 then sans_supl[42]=crosW
widget_control,baset.bases2[20],get_value=thinW & if strpos(thinW[0],'*') lt 0 then sans_supl[10]=thinW
widget_control,baset.bases2[21],get_value=thinS & if strpos(thinS[0],'*') lt 0 then sans_supl[11]=thinS

sans_settings_swap, baset, shareSupl=[6,42] ,supl=(sans_supl) ;Tau & water Xsection for all configs
sans_settings_swap, baset, transm=(sans_trans)                ;Trans for all configs

; ----->Get other miscellaneous fields & transmission run#
widget_control,baset.bases6[0] ,get_value=centXRa & widget_control,baset.bases6[1] ,get_value=centXRb
widget_control,baset.bases6[2] ,get_value=centYRa & widget_control,baset.bases6[3] ,get_value=centYRb
widget_control,baset.bases7[0] ,get_value=tranXRa & widget_control,baset.bases7[1] ,get_value=tranXRb
widget_control,baset.bases7[2] ,get_value=tranYRa & widget_control,baset.bases7[3] ,get_value=tranYRb
widget_control,baset.bases3[24],get_value=Smsampl & widget_control,baset.bases3[25],get_value=Smwater
widget_control,baset.bases3[12],get_value=Sawatr  & widget_control,baset.bases3[13],get_value=Sawatre
widget_control,baset.bases3[14],get_value=Sasampl & widget_control,baset.bases3[15],get_value=Sasampe
widget_control,baset.bases3[16],get_value=Sabeams & widget_control,baset.bases3[17],get_value=Sabeamw
widget_control,baset.bases3[41],get_value=runflux

tmp={data:0,datp:0, opr:' ', wi:0, par:'', opt:0}

; ----->In memory sample data
widget_control,baset.bases3[6] ,get_uvalue=dataw  & widget_control,baset.bases3[7] ,get_uvalue=datawE
widget_control,baset.bases3[8] ,get_uvalue=datas  & widget_control,baset.bases3[9] ,get_uvalue=datasE
widget_control,baset.bases3[10],get_uvalue=datacs & widget_control,baset.bases3[11],get_uvalue=datacw

; ----->Null sample data for settinND (sans_nodata_lmh)
tmpw  =tmp & tmpw.wi  =dataw.wi      & tmpwE=tmp & tmpwE.wi=datawE.wi     & tmps =tmp & tmps.wi =datas.wi
tmpsE =tmp & tmpsE.wi =datasE.wi     & tmpcs=tmp & tmpcs.wi=datacs.wi     & tmpcw=tmp & tmpcw.wi=datacw.wi

; ----->In memory transmission data
widget_control,baset.bases3[18],get_uvalue=adataw  & widget_control,baset.bases3[19],get_uvalue=adatawE
widget_control,baset.bases3[20],get_uvalue=adatas  & widget_control,baset.bases3[21],get_uvalue=adatasE
widget_control,baset.bases3[22],get_uvalue=adatabs & widget_control,baset.bases3[23],get_uvalue=adatabw
widget_control,baset.bases3[42],get_uvalue=trflux

; ----->Null transmission data for settinND (sans_nodata_lmh)
tmaw  =tmp & tmaw.wi =adataw.wi     & tmawE=tmp & tmawE.wi=adatawE.wi    & tmas =tmp & tmas.wi =adatas.wi
tmasE =tmp & tmasE.wi=adatasE.wi    & tmabs=tmp & tmabs.wi=adatabs.wi    & tmabw=tmp & tmabw.wi=adatabw.wi
tmpflx=tmp & tmpflx.wi=trflux.wi

; ----->Transmission<-->center add-on
widget_control,baset.bases3[28],get_uvalue=confuC  & confuC.S[3]=0
; ----->Null confu for settinND (sans_nodata_lmh)
tmpcon={V:confuC.V, S:confuC.S, T:confuC.T, L:tmp} & tmpcon.L.wi=confuC.L.wi

; ----->In memory data for mask
widget_control,baset.bases3[26],get_uvalue=mdatas  & widget_control,baset.bases3[27],get_uvalue=mdataw
; ----->Null data for mask for settinND (sans_nodata_lmh)
tmms =tmp & tmms.wi =mdatas.wi     & tmmw =tmp & tmmw.wi =mdataw.wi

CTDmod=baset.bases6[4:*]
history={centerRG:[centXRa,centXRb,centYRa,centYRb],transmRG:[tranXRa,tranXRb,tranYRa,tranYRb],mods:CTDmod,$
         mskdata: [Smsampl,Smwater],attdata:[Sawatr,Sawatre,Sasampl,Sasampe,Sabeams,Sabeamw],$
         trwatr:adataw,trwatrE:adatawE,trsampl:adatas,trsamplE:adatasE,trbeams:adatabs,trbeamw:adatabw,$
         mksampl:mdatas,mkwatr:mdataw}
histoND={centerRG:[centXRa,centXRb,centYRa,centYRb],transmRG:[tranXRa,tranXRb,tranYRa,tranYRb],mods:CTDmod,$
         mskdata: [Smsampl,Smwater],attdata:[Sawatr,Sawatre,Sasampl,Sasampe,Sabeams,Sabeamw],$
         trwatr:tmaw  ,trwatrE:tmawE  ,trsampl:tmas  ,trsamplE:tmasE  ,trbeams:tmabs  ,trbeamw:tmabw,$
         mksampl:tmms  ,mkwatr:tmmw}

if n_elements(fifiswap) eq 1 then if strpos( fifiswap,".sav") lt 0 then memo=1 else memo=0 else memo=0

; ----->Make the structure to save
if memo then $
 settings={instrument:instru,idxi:idxi,wavel:farr[0],distance:farr[1],Xcenter:farr[2],Ycenter:farr[3],$
           offset:farr[4],angle:farr[5],XYFbeam:[farr[6:7],sans_beam[2:*]],collimation:collima,colidx:colidx ,attenua:attenua,$
           colval:colval,mask:masq,filter:filtr,factor:fct,Qdep:Qdep,supl:sans_supl,dims:sans_masq[2:3],history:history,confus:confuC,$
           water:water,waterE:waterE,sample:sampl,sampleE:samplE,cadmi_s:cadmis,cadmi_w:cadmiw, runflux:runflux, $
           dataw:dataw,datawE:datawE,datas:datas ,datasE:datasE ,datacs:datacs,datacw:datacw  ,transmission:sans_trans,$
           TofFlux:TofFlux, LambdaVq:LambdaVq, trflux:trflux} $
else $
 settings={instrument:instru,idxi:idxi,wavel:farr[0],distance:farr[1],Xcenter:farr[2],Ycenter:farr[3],$
           offset:farr[4],angle:farr[5],XYFbeam:[farr[6:7],sans_beam[2:*]],collimation:collima,colidx:colidx ,attenua:attenua,$
           colval:colval,mask:masq,filter:filtr,factor:fct,Qdep:Qdep,supl:sans_supl,dims:sans_masq[2:3],history:history,confus:confuC,$
           water:water,waterE:waterE,sample:sampl,sampleE:samplE,cadmi_s:cadmis,cadmi_w:cadmiw, runflux:runflux, $
           dataw:dataw,datawE:datawE,datas:datas ,datasE:datasE ,datacs:datacs,datacw:datacw  ,transmission:sans_trans,$
           TofFlux:TofFlux, LambdaVq:LambdaVq, trflux:trflux, $
           low_struct:sans_struct_low,middle_struct:sans_struct_middle,high_struct:sans_struct_high}

 settinND={instrument:instru,idxi:idxi,wavel:farr[0],distance:farr[1],Xcenter:farr[2],Ycenter:farr[3],$
           offset:farr[4],angle:farr[5],XYFbeam:[farr[6:7],sans_beam[2:*]],collimation:collima,colidx:colidx ,attenua:attenua,$
           colval:colval,mask:masq,filter:filtr,factor:fct,Qdep:Qdep,supl:sans_supl,dims:sans_masq[2:3],history:histoND,confus:tmpcon,$
           water:water,waterE:waterE,sample:sampl,sampleE:samplE,cadmi_s:cadmis,cadmi_w:cadmiw, runflux:runflux, $
           dataw:tmpw,datawE:tmpwE,datas:tmps ,datasE:tmpsE ,datacs:tmpcs,datacw:tmpcw  ,transmission:sans_trans,$
           TofFlux:TofFlux, LambdaVq:LambdaVq, trflux:tmpflx}

;print,settings.confus.T, settings.confus.L.opr, settings.confus.L.wi

txtsav=''
if n_elements(dataw.data)  gt 1 then txtsav=txtsav+'Water  '
if n_elements(datawE.data) gt 1 then txtsav=txtsav+'W_cell '
if n_elements(datas.data)  gt 1 then txtsav=txtsav+'Sample '
if n_elements(datasE.data) gt 1 then txtsav=txtsav+'S_cell '
if n_elements(datacs.data) gt 1 then txtsav=txtsav+'Cadm_S '
if n_elements(datacw.data) gt 1 then txtsav=txtsav+'Cadm_W '

if n_elements(fifiswap) eq 1 then begin & result=fifiswap & gpth=''
endif else begin
; ----->Ask for a file name
  widget_control,baset.bases1[2],get_value=filer  & filer =strcompress(filer[ 0] ,/remove_all)
  if filer eq '' then $
  widget_control,baset.bases1[1],get_value=filer  & filer =strcompress(filer[ 0] ,/remove_all)
  pth   =baset.path
  parent=baset.bases1[6]
  result=dialog_pickfile(path=pth,get_path=gpth,file=filer,filter='*.sav',default_ext='sav',dialog=parent,/write,/overwrite_prompt,resource_name='lamptouch')
endelse
if result ne '' then begin
   file =strmid(result,strlen(gpth))

   widget_control,/hourglass

; ----->Save the structure
   if n_elements(fifiswap) eq 1 then begin
      if strpos( fifiswap,".sav") lt 0 then begin
         if strpos(fifiswap,"low")    ge 0 then begin & sans_struct_low   =settings & baset.savedLMH[0]=1
                                                      sans_nodata_low   =settinND & endif
         if strpos(fifiswap,"middle") ge 0 then begin & sans_struct_middle=settings & baset.savedLMH[1]=1
                                                      sans_nodata_middle=settinND & endif
         if strpos(fifiswap,"high")   ge 0 then begin & sans_struct_high  =settings & baset.savedLMH[2]=1
                                                      sans_nodata_high  =settinND & endif
         sans_settings_swap, baset, shareSupl=[6,42] ,supl=(sans_supl) ;Tau & water Xsection for all configs
         sans_settings_swap, baset, transm=(sans_trans)                ;Trans for all configs
      endif else save,settings,filename=result,/compress ;Unused
   endif    else begin
         a=strpos(file,'.sav') & if a gt 0 then curlst=strmid(file,0,a)+'_result.sav' else curlst=file+'_result.sav'
         sans_sav_rest_results,'save', path=gpth, file=curlst
         save,settings,filename=result,/compress
   endelse

   sans_settings_current, settings

   if n_elements(fifiswap) ne 1 then begin
     baset.path=gpth
     widget_control,baset.bases1[2],set_value=file
     if txtsav gt '' then widget_control,bmess,set_value=txtsav+ 'Config saved'
   endif else widget_control,baset.bases1[6],set_uvalue=baset
endif
end

pro sans_settings_text, bases, farr, sarr, mess
;+
;** ******************
;** ******************
;** get string fields, return as string and float
;-

mess='' & farr=fltarr(n_elements(bases)) & sarr=strarr(n_elements(bases)) & sarr[*]='0'

; ----->Get the string fields
for i=0,n_elements(bases)-1 do begin & ok=0
    on_ioerror,misflt & widget_control,bases[i],get_value=tmp & tmp=strtrim(tmp[0],2) & farr[i]=float(tmp) & sarr[i]=tmp & ok=1
    if sarr[i] le ' ' then sarr[i]='0'
    misflt: if not ok then mess=string(i)
endfor
end

pro sans_settings_trans, baset, thickWater=thinW, thickSample=thinS, trans4=trans4, transSample=trnsS
;+
;** *******************
;** *******************
;** update the sans_trans 2D global variable 
;** first  dim contains transmission for [water,Ewater,sample,Esample , sample series...]
;** second dim contains thickness
;** third  dim contains error
;** fourth dim contains Esample series if necessary
;-

common c_sans_settings

;trans: water, waterCell, sample, sampleCell, sampleSeries........
;thick: 0.1    0          0.2     0           0.2  0.3  0.2 ......
;error:

ntran4=n_elements(trans4) & nthinW=n_elements(thinW) & nthinT=0
nthinS=n_elements(thinS)  & ntrnsS=n_elements(trnsS) & ntrans=n_elements(sans_trans[*,0]) & kptrans=sans_trans
newsiz=max([nthinS,ntrnsS])+4
if newsiz gt ntrans then begin & sans_trans=fltarr(newsiz,5)                   & sans_trans[0:ntrans-1,*]=kptrans
                               sans_trans[ntrans:newsiz-1,0]=sans_trans[2,0] & sans_trans[ntrans:newsiz-1,1]=sans_trans[2,1]
                               sans_trans[ntrans:newsiz-1,2]=sans_trans[2,2] & ntrans=newsiz                        & endif

if nthinS  gt 0 then begin & thinT =reform(thinS) & thin0=thinS[0] & if nthinS lt ntrans-4 then thinT=[thinT,fltarr(ntrans-nthinS-4)]
                           nthinT=n_elements(thinT) & thz=where(thinT le 0.) & if thz[0] gt 0 then thinT[thz]=thin0 & endif

if ntran4  eq 4 then begin & sans_trans[0:4,0]=[trans4,trans4[2]]  & sans_trans[4,3]=trans4[3] & endif ;sans_trans(4,3) case we have Empty series
if ntrnsS  gt 0 then begin & sans_trans[2,0]=trnsS[0] & sans_trans[4:4+ntrnsS-1,0]=trnsS & endif
if nthinT  gt 0 then begin & sans_trans[2,1]=thinT[0] & sans_trans[4:4+nthinT-1,1]=thinT & sans_supl[11]=thinT[0]     & endif
if nthinW  eq 1 then begin & sans_trans[0,1]=thinw                                       & sans_supl[10]=thinW        & endif
end

pro sans_settings_dist, s, Dst,Center=xy0,xyPsize=xyPsize,pixel=pixel, distance=Dista, DforT=DforT, aX=aX, aY=aY ,thetrad=thetrad, atangGG=atanGG, water=woter
;+
;** ******************
;** ******************
;** returns current pixel distances in detector from the center
;-


;** sans_supl(1) =detector Distance (m)   sample
;** sans_supl(2) =detector Offset   (mm)  sample (not used)
;** sans_supl(3) =detector Angle    (deg) sample
;** sans_supl(32)=detector Distance (m)   water
;** sans_supl(51)=detector Offset   (mm)  water  (not used)
;** sans_supl(59)=detector Angle    (deg) water
;
;** sans_supl(45)=Rotation axe      (cm)
;** sans_supl(52)=XYunit            (0:pixel 1:2theta)
;** sans_supl(54)=detector centeX   (pixel)
;** sans_supl(55)=detector centeY   (pixel)
;** sans_supl(65) 0:disable offset  1:enable offset detector  2:enable offset distance

common c_sans_settings
common c_sans_cur_aXY

if n_elements(Dista) ne 1 then if keyword_set(woter) then Dista=sans_supl[32] else Dista=sans_supl[1]
if keyword_set(woter) then alphaD=sans_supl[59] else alphaD=sans_supl[3]
Raxe = sans_supl[45]

Unit=float(max(xyPsize,min=mu))
if xyPsize[0]  eq mu  then ratioxy=[mu/Unit,1.] else ratioxy=[1.,mu/Unit]
if keyword_set(pixel) then unit   = 1.

if (sans_supl[52] eq 1) and not keyword_set(pixel) then begin                           ;we have aX and aY in 2theta unit
         dthX=atan( (sans_supl[54]-xy0[0])*xyPsize[0]/(dista*100.) )
         dthY=atan( (sans_supl[55]-xy0[1])*xyPsize[1]/(dista*100.) )
        ;print,'dthX=',dthX,' dthY=',dthY
         dthX=0.
         if n_elements(aX) gt 0 then thX=aX*!pi/180.+dthX else thX=cur_aX*!pi/180.+dthX ;Correct thetaX with new center
         if n_elements(aY) gt 0 then thY=aY*!pi/180.+dthY else thY=cur_aY*!pi/180.+dthY ;Correct thetaY with new center

         Rx  = thX         *dista*100. & Rx=   [[Rx],[Rx]]   ;Approximate a circle on X !!!!!
         Ry  = tan(  thY)  *dista*100. & Ry=transpose(Ry)
         x   = congrid(Rx,s[1],s[2])
         y   = congrid(Ry,s[1],s[2])
         Dst =sqrt(x^2+y^2)

         if arg_present(atanGG) then atanGG=atan(y,x)

        ;thetrad= 2theta matrix = acos( cTX * cTY )
         if arg_present(thetrad) then begin
          thX=cos(abs(thX))
          thY=cos(abs(thY))
          cTX=congrid([[thx],[thx]] ,s[1],s[2])                 ;cos(2theta) on X
          cTY=congrid(transpose(thY),s[1],s[2])                 ;cos(2theta) on Y
          thetrad =acos( cTX * cTY )                            ;FORMULAE
         endif
         DforT=0
endif else begin
         x  =findgen(s[1])+.5 & x =[[x],[x]]
         y  =transpose(findgen(s[2]))+.5
         x  =(congrid(x,s[1],s[2])-xy0[0])*ratioxy[0]*Unit
         y  =(congrid(y,s[1],s[2])-xy0[1])*ratioxy[1]*Unit
         Dst=sqrt(x^2+y^2)

         if arg_present(DforT) then $
         if (abs(alphaD) ge 1.) and (Raxe gt 0.) then begin
            alpha=abs(alphaD)*!pi/180.                              ;We have a detector angle.
           ;a    =abs(sans_supl(2)/10.)            *tan(alpha)      ;Use param  offset (Raxe=0)
            a    =abs(xy0[0]-s[1]/2.+.5)*ratioxy[0]*sin(alpha)*Unit ;Use center offset
            cosa =cos(alpha)
            r    =Raxe*(1.-cosa)
            D    =Dista*100. & Dra=D+r+a
            if alphaD gt 0.      then sgn=[-1,1] else  sgn=[1,-1] ;alpha > 0   !!
            if xy0[0] gt s[1]/2. then sgn=[-1,1] else  sgn=[1,-1] ;=offset < 0 !!
            xoff=findgen(s[1])+.5 & idxl=where(xoff lt xy0[0]) & idxr=where(xoff ge xy0[0])
            sina=sin(sgn*alpha)
            if idxl[0] ge 0 then begin & xt=abs(x[idxl,*]) & x[idxl,*]=(xt*Dra*cosa)/(xt*sina[0]+Dra) & endif
            if idxr[0] ge 0 then begin & xt=abs(x[idxr,*]) & x[idxr,*]=(xt*Dra*cosa)/(xt*sina[1]+Dra) & endif
            DforT=sqrt(x^2+y^2)*D/Dra
            
         endif else DforT=0
         thetrad=0
endelse
end

pro sans_settings_getmask, s, xyc, masq ,nobeam=nobeam, noWbeam=noWbeam
;+
;** *********************
;** return current 2D mask in masq
;-

common c_sans_settings

masq=bytarr(s[1],s[2]) & s1=s[1]-1 & s2=s[2]-1
if (n_elements(sans_masq) gt 4) or  $
  ((n_elements(sans_beam) ge 3) and (not keyword_set(nobeam))) then begin
 if n_elements(sans_masq) gt 4 then begin
    msk=(sans_masq[*,1:*]-1)>0
    for i=0,n_elements(msk)/4 -1 do begin & a1=msk[0,i]<s1 & a2=(a1+msk[2,i])<s1      ;Instrument mask
                                          b1=msk[1,i]<s2 & b2=(b1+msk[3,i])<s2
                                          masq[a1:a2,b1:b2]=1 & endfor
 endif

 if n_elements(sans_beam) gt 6 then begin
  if not keyword_set(noWbeam)  then begin
   a  =(sans_beam[3:6]-1)>0
   xyW=[a[0]+a[2]/2. , a[1]+a[3]/2.]
   if a[2]*a[3] gt 0 then begin                                                       ;Water beam
      a1=a[0]>0<s1 & a2=(a[0]+a[2])>0<s1 & b1=a[1]>0<s2 & b2=(a[1]+a[3])>0<s2
      if (a1 lt a2) and (b1 lt b2) then begin
       if sans_beam[2] eq 1 then masq[a1:a2,b1:b2]=1 $                              ;squared beam
       else begin                                                                     ;elliptical
         sans_settings_dist, s, Dst,Center=xyW,xyPsize=reverse((a[2:3]+1)/2.),/pixel,/water
         idx=where(round(Dst) le round(min((a[2:3]+1)/2.)))
         if idx[0] ge 0 then masq[idx]=1
       endelse
      endif
   endif
  endif
  if not keyword_set(nobeam)  then begin
   xy0=xyc
   farr=[xy0,sans_beam[0:1]]                                                          ;Sample beam
   if n_elements(sans_beam) ge 9 then if (sans_beam[7] ne 0) or (sans_beam[8] ne 0) then begin
      xy0=[sans_beam[7]+(sans_beam[0]-1.)/2. , sans_beam[8]+(sans_beam[1]-1.)/2.]-1.
      a1=(sans_beam[7]-1)>0<s1 & a2=(sans_beam[7]+sans_beam[0]-2)>0<s1
      b1=(sans_beam[8]-1)>0<s2 & b2=(sans_beam[8]+sans_beam[1]-2)>0<s2
   endif else begin
      a1=round(xy0[0]-farr[2]/2)>0<s1 & a2=round(xy0[0]+farr[2]/2)>0<s1
      b1=round(xy0[1]-farr[3]/2)>0<s2 & b2=round(xy0[1]+farr[3]/2)>0<s2
   endelse
   if (a1 lt a2) and (b1 lt b2) then begin
      if sans_beam[2] eq 1 then masq[a1:a2,b1:b2]=1 $                                ;Squared beam
      else begin                                                                       ;elliptical
         sans_settings_dist, s, Dst,Center=xy0,xyPsize=reverse(farr[2:3]/2.),/pixel
         idx=where(round(Dst) le round(min(farr[2:3]/2.)))
         if idx[0] ge 0 then masq[idx]=1
      endelse
   endif
  endif
 endif
endif
end

pro sans_settings_GetMaskBeam, Mbeam
;+
;** *************************
;** *************************
;** return sans_beam global variable for sans_sheet
;-

common c_sans_settings
Mbeam=sans_beam            ;called by sans_sheet ***
end

pro sans_settings_GetTrans, trans
;+
;** **********************
;** **********************
;** return sans_trans global variable for sans_sheet
;-

common c_sans_settings
trans=sans_trans           ;called by sans_sheet ***
end

pro sans_settings_GetSupl, supl
;+
;** *********************
;** *********************
;** return sans_supl global variable for sans_sheet
;-

common c_sans_settings
supl =sans_supl            ;called by sans_sheet ***
end

pro sans_settings_SetSupl, supl
;+
;** *********************
;** *********************
;** update sans_supl global variable for sans_sheet
;-

common c_sans_settings
sans_supl=supl             ;called by sans_sheet only for panels/resolution ***
end

pro sans_settings_GetBgrd, Tbase, existSbg, existWbg
;+
;** *********************
;** *********************
;** return true for sans_sheet if empty can and/or cadmium are present
;-

existSbg=0 & existWbg=0    ;called by sans_sheet ***
widget_control,bad_id=ii,Tbase,get_uvalue=baset
widget_control,bad_id=ii,baset.bases3[1],get_value=waterC
widget_control,bad_id=ii,baset.bases3[5],get_value=waterCd & if waterC+waterCd gt '  ' then existWbg=1
widget_control,bad_id=ii,baset.bases3[3],get_value=samplC
widget_control,bad_id=ii,baset.bases3[4],get_value=samplCd & if samplC+samplCd gt '  ' then existSbg=1
end

pro sans_settings_mask, baset, run=data ,nobeam=nobeam ,nomask=nomask, nowbeam=nowbeam, showall=showall ,surf=surf
;+
;** ******************
;** ******************
;** plot the 2D selected data with or without masks
;-

common c_sans_settings
common c_sans_cur_run
common c_sans_cur_aXY
common c_sans_to_ben, sansBen

FX=sans_supl[30]
bmess   =baset.bases1[0]   &  bview   =baset.bases2[5]   & bview2=baset.bases1[9]  & bview3=baset.bases1[10]
bcenters=baset.bases2[2:3] &  bbeams  =baset.bases2[8:9] & bview5=baset.bases1[16] & bview6=baset.bases1[17]
bmask   =baset.bases2[4]

if n_elements(data) ne 0 then cur_run=data
if n_elements(data) gt 1 then newdata=1 else newdata=0
if n_elements(data) eq 0 then if n_elements(cur_run) gt 1   then data=cur_run

sizd=size(data) & cvh =cur_fct[2] ;128.
if sizd[0] ge 2  then  cur_fct[9:10]=cvh/[sizd[1],sizd[2]] ;FCT is the factor which map the data to a 128*128 window
fct =cur_fct[9:10] ; = baset.factor when data size is constant

titsurf='w'+strtrim(cur_wn,2)
if (newdata) and (baset.bases2[10] gt 0L) then widget_control,baset.bases2[10],set_value='Large View '+titsurf

if keyword_set(nobeam)  then cur_menu[3]=1
if keyword_set(nomask)  then cur_menu[4]=1
if keyword_set(nowbeam) then cur_menu[5]=1
if keyword_set(showall) then cur_menu[3:5]=0
if cur_menu[3] then nobeam=1 & if cur_menu[4] then nomask=1 & if cur_menu[5] then nowbeam=1


; ----->Check for center and beamsize
if not newdata then begin
 sans_settings_text, [bcenters,bbeams], farr, sarr, mess
 farr[0:1]=farr[0:1]-FX
 if farr[0] lt 0 then farr[0]=farr[0]+2*FX
 if farr[1] lt 0 then farr[1]=farr[1]+2*FX
 if mess ne '' then begin & widget_control,bmess,set_value='!!! bad input in Center or Beamsize field' & return & endif
 sans_mask, center=farr[0:1], xybeam=farr[2:3] ; be sure we have same values in common block
endif else farr=[sans_ctr[0:1],sans_beam[0:1]]

; ----->Check for mask
widget_control,bmask,get_value=masq   & sizm=size(masq) & sizm1=sizm[1]
msk=fltarr(4,sizm1) & tmp=fltarr(4)
for i=0,sizm1-1 do if masq[i] gt ' ' then begin
                      txt=strmid(strtrim(masq[i],2),1,20) & on_ioerror,mismasq & reads,txt,tmp
                      msk[*,i]=tmp & mismasq: & endif
on_ioerror,misetmask
msk=(msk-1)>0
dcs=cur_fct[3] ;256 or 384 the current draw size
sf1=round(cvh/fct[0]) & sf2=round(cvh/fct[1]) & s1=sf1-1 & s2=sf2-1 ; sf1,sf2 = sizd(1),sizd(2) ROUNDED
M  =bytarr(sf1,sf2)
K  =M
for i=0,sizm1-1 do if masq[i] gt ' ' then begin & a1=msk[0,i]<s1 & a2=(a1+msk[2,i])<s1     ;MASK detector
                                                b1=msk[1,i]<s2 & b2=(b1+msk[3,i])<s2
                                   if not keyword_set(nomask) then M[a1:a2,b1:b2]=100
                                                                   K[a1:a2,b1:b2]=1 & endif


a  =(sans_beam[3:6]-1)>0                                                                 ;WATER BEAM
xyW=[a[0]+a[2]/2. , a[1]+a[3]/2.]
if a[2]*a[3] gt 0 then begin
   a1=a[0]>0<s1 & a2=(a[0]+a[2])>0<s1 & b1=a[1]>0<s2 & b2=(a[1]+a[3])>0<s2
   if (a1 lt a2) and (b1 lt b2) then begin
       if sans_beam[2] eq 1 then begin
          if not keyword_set(nowbeam) then M[a1:a2,b1:b2]=114 & K[a1:a2,b1:b2]=1         ;Squared beam
       endif else begin                                                                  ;elliptical
         sans_settings_dist, [2,sf1,sf2], Dst,Center=xyW,xyPsize=reverse((a[2:3]+1)/2.),/pixel,/water
         idx=where(round(Dst) le round(min((a[2:3]+1)/2.)))
         if idx[0] ge 0 then begin & if not keyword_set(nowbeam) then M[idx]=114 & K[idx]=1 & endif
       endelse
   endif
endif else cur_menu[5]=0

farr[0:1]=farr[0:1]-1.                                                                   ;SAMPLE BEAM
if farr[0] lt 0 then farr[0]=farr[0]+2
if farr[1] lt 0 then farr[1]=farr[1]+2
xy0=farr[0:1] ;0->n-1
if n_elements(sans_beam) ge 9 then if (sans_beam[7] ne 0) or (sans_beam[8] ne 0) then begin
     xy0=[sans_beam[7]+(sans_beam[0]-1.)/2. , sans_beam[8]+(sans_beam[1]-1.)/2.]-1.
     a1=(sans_beam[7]-1)>0<s1 & a2=(sans_beam[7]+sans_beam[0]-2)>0<s1
     b1=(sans_beam[8]-1)>0<s2 & b2=(sans_beam[8]+sans_beam[1]-2)>0<s2
endif else begin
     a1=round(xy0[0]-farr[2]/2)>0<s1 & a2=round(xy0[0]+farr[2]/2)>0<s1
     b1=round(xy0[1]-farr[3]/2)>0<s2 & b2=round(xy0[1]+farr[3]/2)>0<s2
endelse
isbeam=0 &  bco=128
if (a1 lt a2) and (b1 lt b2) then begin
   isbeam=1
   if sans_beam[2] eq 1 then begin                                              ;SQUARED BEAM
      if not keyword_set(nobeam) then M[a1:a2,b1:b2]=bco & K[a1:a2,b1:b2]=1
   endif else begin                                                             ;ELLIPTICAL BEAM
      sans_settings_dist, [2,sf1,sf2], Dst,Center=xy0,xyPsize=reverse(farr[2:3]/2.),/pixel
      idx=where(round(Dst) le round(min(farr[2:3]/2.)))
      if idx[0] ge 0 then begin & if not keyword_set(nobeam) then M[idx]=bco & K[idx]=1 & endif
   endelse
endif else cur_menu[3]=0
cur_mask=K & idm=where(M gt 0)>0                                                ;Logical mask
idk=M[idm] ;keep mask & beam to put over center


a1=floor(farr[0])>0<s1           & a2=ceil(farr[0])>0<s1                                 ;CENTER
if farr[0]-a1 ge .75 then a1=a2  & if a2-farr[0] ge .75 then a2=a1
if farr[0]-a1 lt .45 then a2=a1  & if a2-farr[0] lt .45 then a1=a2
b1=floor(farr[1])>0<s2           & b2=ceil(farr[1])>0<s2
if farr[1]-b1 ge .75 then b1=b2  & if b2-farr[1] ge .75 then b2=b1
if farr[1]-b1 lt .45 then b2=b1  & if b2-farr[1] lt .45 then b1=b2
s4=s1/4 & s5=s2/4
ra=bytarr(a2-a1+1,(b2+s5)<s2 - ((b1-s5)>0) +1) & for i=0L,(size(ra))[2]-1,2 do ra[*,i]=bco
rb=bytarr((a2+s4)<s1 - ((a1-s4)>0) +1,b2-b1+1) & for i=0L,(size(rb))[1]-1,2 do rb[i,*]=bco
;Show center if not surf and not sectoring...
chere=0
if  (not keyword_set(surf)) and (cur_menu[0] ne 7) and (cur_menu[1] ne 8) then begin
 if (    keyword_set(nobeam)) or (not isbeam) or (total(cur_menu[3:5]) eq 0) then $
 if (not keyword_set(nomask)) or (not keyword_set(nobeam)) then begin & M[a1:a2,(b1-s5)>0:(b2+s5)<s2]=ra & chere=1
                                                                      M[(a1-s4)>0:(a2+s4)<s1,b1:b2]=rb & endif
endif else chere=1

if (cur_menu[10] eq 0) or (cur_menu[10] eq 1) then begin        ; Show RANGES for center or transmission if set
   if cur_menu[10] eq 0 then sans_settings_text, baset.bases6[0:3], forr, sorr, mess
   if cur_menu[10] eq 1 then sans_settings_text, baset.bases7[0:3], forr, sorr, mess
   xr=([forr[0],forr[1]]-FX-1)>0<s1 & yr=([forr[2],forr[3]]-FX-1)>0<s2
   if (xr[0] lt xr[1]) and (yr[0] lt yr[1]) then begin
       xr=(xr+[-1,+1])>0<s1 & yr=(yr+[-1,+1])>0<s2
       M[xr[0]:xr[1],yr]=bco/2 & M[xr,yr[0]:yr[1]]=bco/2
endif & endif

M[idm]=idk ;put center below mask & beam
if not chere then begin & M[a1:a2,0:s2]=byte(M[a1:a2,0:s2]*1.1) ;show small center
                        M[0:s1,b1:b2]=byte(M[0:s1,b1:b2]*1.1) & endif

; ----->Show view
kpwin=!D.window
Cbar =[findgen(dcs)]  & Cbar=congrid(transpose([[Cbar],[Cbar]]),25,dcs)
if baset.bases6[8] then Cbar=bytscl( alog(Cbar>1))

if  cur_wn eq 17 then data=round(data*10.)/10. else $              ;Show efficiencies
if (cur_menu[7] eq 7) or (cur_menu[7] eq 8) then $                 ;7=sample 8=water (mask menu)
if  n_elements(data)   gt 1 then if cur_menu[8] then begin         ;if asked, show corrected data

     trans=sans_trans[0:3] & trans[2]=baset.curSAMtrs
     if cur_menu[7] eq 8 then trans[[2,3]]=trans[[0,1]] ;sample is water
    ;Enable efficiencies and/or background correction
     if (trans[0]*trans[1] gt 0) or (trans[2]*trans[3] gt 0) then begin
         if trans[2] eq 0 then begin & trans[3]=0 & trans[2]=1 & endif
         transs='['+string(trans[0])+','+string(trans[1])+','+string(trans[2])+','+string(trans[3])+']'
         sans_settings_runs, baset ,[9,6 ,0,baset.bases3[0:1]], /noread, /haveall ;Be sure having Water Ewater
         sans_settings_runs, baset ,[9,9 ,0,baset.bases3[3:5]], /noread, /haveall ;Be sure having Esample Cadmium
         widget_control,  baset.bases3[6]  ,get_uvalue=wdata  & wws =strtrim(string(wdata.wi) ,2) ;water
         widget_control,  baset.bases3[7]  ,get_uvalue=ewdata & ewws=strtrim(string(ewdata.wi),2) ;Ewater
         widget_control,  baset.bases3[9]  ,get_uvalue=esdata & esws=strtrim(string(esdata.wi),2) ;Esample
         widget_control,  baset.bases3[10] ,get_uvalue=csdata & csws=strtrim(string(csdata.wi),2) ;Scadm
         widget_control,  baset.bases3[11] ,get_uvalue=cwdata & cwws=strtrim(string(cwdata.wi),2) ;Wcadm

         ESopr=esdata.opr & CSopr=csdata.opr
         if cur_menu[7] eq 8 then begin & esws=ewws & ESopr= ewdata.opr  ; SAMPLE IS WATER
                                        csws=cwws & CSopr= cwdata.opr  & endif
        ;print,wdata.opr+'-'+ewdata.opr+'-'+esdata.opr+'-'+csdata.opr+'-'+cwdata.opr+'-'
         cmd='sam='+strtrim(string(cur_wn),2)+',transm='+transs
         if (wdata.opr  gt ' ') and (trans[0] gt 0) then cmd=cmd+',wat='   +wws
         if (ewdata.opr gt ' ') and (trans[1] gt 0) then cmd=cmd+',Ewat='  +ewws
         if (ESopr      gt ' ') and (trans[3] gt 0) then cmd=cmd+',Esam='  +esws
         if (CSopr      gt ' ') and (trans[2] gt 0) then cmd=cmd+',scadm=' +csws
         if (cwdata.opr gt ' ') and (trans[0] gt 0) then cmd=cmd+',wcadm=' +cwws
        ;print,trans,cmd

         sans_gt_par ,sys=sys & sys3=strtrim(sys+3,2)
         XICUTE,'w'+sys3+'=sans_cor('+cmd+',/no_centr,geteff=W17)' ;Correction
         TAKE_W,data,W=sys+3 & TAKE_W,cur_eff,W=17
         if n_elements(data) eq n_elements(cur_eff) then data=round(data/cur_eff)
         newdata=1  ;for bens
     endif
endif
if n_elements(data)  eq n_elements(M) then begin & idx=where(M eq 0) & mimi=min(data[idx],max=mama)
                                                 datav=data & if n_elements(idm) gt 1 then datav[idm]=mimi
                                                 if mama-mimi gt 2. then sup=1. else sup=mama/100.
                                                 if baset.bases6[8] then datav=alog(data>sup)
                                                 dats=bytscl(datav)
                                                 if idx[0] ge 0 then M[idx]=dats[idx]
                                                 titsurf=titsurf+' min='+strtrim(mimi,2)+' max='+strtrim(mama,2)
      sector=0 & as_is=0
      if (cur_wn ne 17) and (cur_wn ne 18) and $
        ((cur_menu[0] eq 7) or (cur_menu[1] eq 8)) then begin ;SECTOR input/current view

; ----->Gives data to Ben.
        F=cur_fct[3]/max([sf1,sf2])
        cur_fct[0:1]=sans_supl[8:9] ;FOR BEN
        if cur_fct[0] gt cur_fct[1] then cur_fct[0:1]=[cur_fct[0]/cur_fct[1],1.] else $
        if cur_fct[0] lt cur_fct[1] then cur_fct[0:1]=[1.,cur_fct[1]/cur_fct[0]]      $
                                    else cur_fct[0:1]=[1.,1.]
        databen = M                 ;MASKED VALUES
        sector=1 & Fhow=2.
        if (sf1 ge Fhow*cur_fct[3]) or (sf2 ge Fhow*cur_fct[3]) then begin
           databen   = congrid(databen,round(sf1*cur_fct[0]),round(sf2*cur_fct[1]))
           szB       = size(databen)
           BenDrawSiz= max([szB[1],szB[2]]) ;KEEP LARGE DATA WHEN BIGGER THAN DRAW SECTOR VIEW
        endif else begin
           databen   = congrid(databen,round(sf1*cur_fct[0]*F),round(sf2*cur_fct[1]*F))
           szB       = size(databen)
           BenDrawSiz= long(cur_fct[3])     ;KEEP CURRENT DRAW SECTOR VIEW
           cur_fct[0:1]=cur_fct[0:1]*F
        endelse
        cur_cxy[5]=szB[1] ;databen siz1
        cur_cxy[6]=szB[2] ;databen siz2
        cur_cxy[7]=BenDrawSiz
        as_is     =1      ;ANYWAY TO ALLOW SURF ALTERNATIVE
    
        widget_control,sansBen,get_uvalue=varAccess
        widget_control,get_uvalue=IDs,varAccess.IDs
        cxID=IDs.cx & cyID=IDs.cy ;ID of centers
        r1ID=IDs.r1 & r2ID=IDs.r2 ;ID of radius slider
        widget_control,get_uvalue=cst,varAccess.constants & cst.minSize=BenDrawSiz
        widget_control,set_uvalue=cst,varAccess.constants

       ;ADJUST ORIGINAL DRAW SECTOR
        widget_control,baset.bases1[23],draw_Xsize=BenDrawSiz,draw_Ysize=BenDrawSiz
        DR=widget_info(baset.bases1[23],/geometry)
        widget_control,baset.bases1[23],set_draw_view=[((DR.draw_Xsize-DR.Xsize)/2)>0,((DR.draw_Ysize-DR.Ysize)/2)>0]
        
        szbYF=long(szB[1]/szB[2])>1<5  & szbYF=1                               ;unused!!
        if szbYF ge 2 then databen=rebin(databen,szB[1],szB[2]*szbYF ,/sample) ;unused!!
        ;print,cur_menu([0,1,10]),newdata,BenDrawSiz,cur_fct(3)
        ;help,cur_run
        ;help,/trace
        ;print,cur_fct(0:1)
        
           ;RELOAD DATABEN ANYWAY! "int_loadWorkspace in Bens"
            W_store  ,W=cur_wn   &   GIVE_W,W=cur_wn ,databen
            if (farr[0] lt 0) or (farr[1] lt 0) then begin
            endif
            INT_event,{TOP:baset.bases1[6]},[-88,403,1,cur_wn,baset.bases1[12]] ;Load workspace
            if (farr[0] lt 0) or (farr[1] lt 0) then begin
            endif
            W_restore,W=cur_wn   &   cur_cxy[0:1]=0.

        if cur_cxy[2] ne sans_beam[2] then begin                                ;Rectangl<->Ellips
           cur_cxy[2]  = sans_beam[2]                                           ; or
           varAccess.squared =baset.bases6[21]                                  ;(un)squared sectors
           widget_control,sansBen,set_uvalue=varAccess
           INT_miniPix,varAccess,0  & endif                                     ;replot masked data

        if (cur_cxy[0] ne  farr[0]) or (cur_cxy[1] ne farr[1]) then begin       ;Change center
            cur_cxy[0:1] =[farr[0:1]] ;0->n-1
            widget_control,cxID,set_value=strtrim(string((farr[0])*szB[1]/sf1+1),2)
            widget_control,cyID,set_value=strtrim(string((farr[1])*szB[2]/sf2*szbYF+szbYF/2.+1),2)
            INT_event,{TOP:baset.bases1[6]},[-88,409,1]
            
            if (farr[0] lt 0.) or (farr[1] lt 0.) then begin ; Redo radius if center negative because MinMax slider was wrong.
                secIR=sans_supl[17] ;*cur_fct[0]!
                secOR=sans_supl[18] ;*cur_fct[0]!
                widget_control,baset.bases7[11], set_value =secIR, get_uvalue=u_r1ID
                widget_control,baset.bases7[12], set_value =secOR, get_uvalue=u_r2ID
                INT_event,{TOP:sansBen,ID:baset.bases7[11],VALUE:long(secIR)},u_r1ID ;!!
                INT_event,{TOP:sansBen,ID:baset.bases7[12],VALUE:long(secOR)},u_r2ID ;!!
            endif
        endif

        if baset.bases6[8] ne cur_log then begin
            INT_miniPix,varAccess,0                                             ;replot masked data
            cur_log=baset.bases6[8] & endif $                                   ;Toggle log aspect or...
        else if not newdata then $
            INT_miniPix,varAccess,0                                             ;replot masked data
            
        if as_is then sans_settings_mask_event, Z=databen
      endif
endif

if (sf1 ne cvh) or (sf2 ne cvh) then pM=congrid(M,cvh,cvh) else pM=M
wset,bview  & tvscl,pM   ;Mini view (config)
wset,bview3 & tvscl,Cbar ;color Bar Detector
wset,bview6 & tvscl,Cbar ;color Bar Sectors

;bview2 = Detector view
;bview5 - Sectors view
if cur_zoom[0] gt 1 then begin & wset,cur_zoom[2]  & resu=bytscl(congrid(M,dcs*cur_zoom[0],dcs*cur_zoom[0])) & tv,resu
                               wset,bview2       & device,copy=[cur_zoom[6:7],dcs,dcs,0,0,cur_zoom[2]]
endif else begin &               wset,bview2       & resu=bytscl(congrid(M,dcs,dcs)) & tv,resu & endelse

if keyword_set(surf) then begin & tvlct,r,g,b,/get & sans_gt_par ,sys=sys & sys1=strtrim(sys+1,2)
                                if (sector) and (as_is) then resu=smooth(databen,3,/edge_t) else $
                                if cur_zoom[0] le 1 then resu=M
                               ;                      Xicute,'W'+sys1+'=0'
                               ;give_w,resu,w=sys+1 & Xicute,"w_tit["+sys1+"]='"+titsurf+"'"
                               ;if (sans_supl(52) eq 1) and (n_elements(cur_aX) gt 1) and (cur_zoom(0) le 1) then begin
                               ;  take_datp,datp,W=sys+1 & mod_datp,datp,'X',cur_aX & mod_datp,datp,'Y',cur_aY
                               ;  give_datp,datp,W=sys+1
                               ;endif
                               ;see ,w=sys+1,/beside,/surface,/contour
                                livc_lamp, resu, GROUP=baset.bases1[6], APPTLB=appTLB, AS_IS =as_is ,$
                                           TIT=titsurf, XTIT=xtit, YTIT=ytit, ZTIT=ztit, XX=cur_aX, YY=cur_aY ,WI=sys+1
                                if (sector) and (as_is) then begin & Sbend=size(resu)
                                                                   cur_cxy[3]=appTLB   ;LIVC
                                                                   cur_cxy[4]=bView5   ;SECT VIEW
                                                                   cur_cxy[5]=Sbend[1] ;databen siz1
                                                                   cur_cxy[6]=Sbend[2] ;databen siz2
                                                                   empty ;!!!
                                                                   sans_settings_mask_event
                                                                   endif
                                tvlct,r,g,b & Xicute,'w_tit['+sys1+']=""'
endif
wset,kpwin
misetmask:
end

pro sans_settings_mask_event, Z=databen
;+
;** ************************
;** ************************
;** Updates the 3d plot representation 
;** Comes also from Ben int_event (sector updated)
;-

common c_sans_cur_run

appTLB=long(cur_cxy[3])   ;LIVC_LAMP app
bView5=long(cur_cxy[4])   ;SECTOR DIRECT VIEW
szData=long(cur_cxy[5:6]) ;DATABEN SIZE
cur_cxy[3]=0
if (appTLB gt 1) then $
  if (widget_info(appTLB,/valid)) then $
    if (widget_info(appTLB,/type) eq 0) then begin
      widget_control,appTLB,get_uvalue=pState
      if n_elements(pState) eq 1 then begin
         DrawSiz=max(szData)
         a1=0 & a2=0 & b1=szData[0] & b2=szData[1]
         if DrawSiz  ne   szData[0] then a1=round((DrawSiz-szData[0])/2.)
         if DrawSiz  ne   szData[1] then a2=round((DrawSiz-szData[1])/2.)
         kpWin =!D.window      & wset,bView5
         VxNew =tvrd(a1,a2,b1,b2,true=1) & wset,kpWin
         if (*pState).rTerrainSurface eq OBJ_NEW() then cur_cxy[3]=appTLB $ ;appTLB NOT YET READY
         else (*pState).rTerrainSurface->GetProperty,VERT_COLORS=VxOld      ;GET VERTEX COLORS
        ;help,vxold,vxnew
         if n_elements(VxOld) eq n_elements(VxNew) then begin
           if n_elements(databen) eq szData[0]*szData[1] then $
          (*pState).rTerrainSurface->SetProperty,DATAZ=databen     ;REPLACE Z if asked
           VxNew=reform(VxNew,(size(VxOld))[1],(size(VxOld))[2])
          (*pState).rTerrainSurface->SetProperty,VERT_COLORS=VxNew ;SET VERTEX COLORS
           WIDGET_CONTROL, (*pState).wDraw[1], GET_VALUE=rWindow   ;GET LIVC WINDOW
           rWindow->Draw, (*pState).rTerrainView                   ;REDRAW
           cur_cxy[3]=appTLB
         endif
      endif
    endif
end

pro sans_settings_series, lyst, Splace=Splace, Tplace=Tplace
;+
;** ********************
;** ********************
;** handle holes in spreadsheet concerning samples and transmission series
;** TRplace and SPplace = [[first numor in cells] , [line index] , [position of ':']]
;-

common c_sans_place,  TRplace, SPplace

lyst=strcompress(lyst,/remove_all)  & idx =where(lyst ne '')
if idx[0] lt 0 then begin & lyst=[''] & return & endif
jc=n_elements(lyst)-1
while ((lyst[jc] eq '') and (jc gt 0)) do jc=jc-1                                                    ;do'nt care with trailing empty cells
lysti=long(lyst)
if keyword_set(Splace) then SPplace=[[lysti[idx]],[idx],[strpos(lyst[idx],':')]]
if keyword_set(Tplace) then TRplace=[[lysti[idx]],[idx],[strpos(lyst[idx],':')]]
bid=where(((strpos(lyst,'+')) + (strpos(lyst,'>')) + (strpos(lyst,'-')) + (strpos(lyst,':'))) gt -4) ;case we have operators in cells
if bid[0] ge 0 then lysti[bid]=-4                                  ;""
txt=lyst[0]
for i=1L,jc do begin
    if lysti[i] ne lysti[i-1]+1 then $
       if lysti[i-1] lt 0 then txt=txt+',' +lyst[i] else $
       if lysti[i-1] eq 0 then txt=txt+':,'+lyst[i] else txt=txt+':'+lyst[i-1]+','+lyst[i]
    if i eq jc then if lysti[i] ge 0 then txt=txt+':'+lyst[i]
endfor
lyst=txt
end

pro sans_settings_phi0, baset, sans_supl, tmpbeam, attFact=supl26, phi0=flux
;+
;** ******************
;** return attenuator factor and calculate flux phi0 from a transmission beam
;-
                                tau = sans_supl[6] * sans_supl[27]       ; enable * TAUvalue
                                ftau= sans_supl[7] & tim = sans_supl[23] ; tau function and time
                               ;if sans_supl[66] gt 1 then tau = 0. ; commented for Tbeam (sans_supl no defined here!)
                                err = 0
                                if tau*tim gt 0. then sans_tau, tmpbeam ,err ,ftau ,tau ,tim       ; dead times correction

                                sans_settings_getmask,size(tmpbeam),[0.,0.],M,/nobeam,/noWbeam     ; get detector mask then
                                if n_elements(M) eq n_elements(tmpbeam) then tmpbeam=tmpbeam*(1-M) ; mask correction
                                
                                attenu =float(strsplit(/EXTRACT,baset.attenu,','))            ;attenuator factor of reference
                                attenuF=baset.attenuF                                        ;attenuator factors / all possible collimations
                                supl26=(attenu[sans_supl[24]] *sans_supl[25]) > 1            ;att[number] * OnOff >1
                                if (supl26 gt 1.) and ((size(attenuF))[0] gt 0) then begin
                                    collim=strsplit(/EXTRACT,baset.collim,',')               ;Calculate collimation index for that transmission
                                    colidx=where(round(float(collim)*10.)/10. eq round(sans_supl[4]*10.)/10.) & colidx=colidx[0]>0
                                    supl26 = ( supl26 * attenuF[colidx,sans_supl[24]] )>1      ;Real attenuator factor
                                endif
                                if sans_supl[71] gt 1. then supl26 *= sans_supl[71] ; Attenuator 2 given by the instrument
                                flux  = total(tmpbeam) / tim * supl26               ; return PHI0

end

pro sans_settings_runs_live, toRead
;+
;** ***********************
;** ******************
;** Using batch mode:
;** reads the numors (calling RDOPR) and update parameters in sans_supl
;-
@lamp.cbk
common c_sans_settings
common c_sans_tofl,   TofFlux, LambdaVq
common c_sans_cur_run
common c_sans_batch,  parconf, bid1, bid2, bid3, bid4
common c_sans_base0,  base0

batch = sans_supl[79] ;1=Batch mode , 0=Gui mode

;Transm = {Tsample:'31290:31299' , Tscell:'31288' , Tsbeam:'31287' , Twater:'31289' , Twcell:'31288' , Twbeam:'31287'}
;          W3 sample               W4 sample cell   W5 sample beam   W1 water         W2 water cell    W6 water beam

;DBeam  = {CONFIG:'middle' , BeamTrC:'31285', FLUX:''}
;                            W9 direct Beam   W26 

;Water  = {Water:'31314' , WaterCell:'31313' , WaterB4C:'31312', WThickness:0.1}
;          W10             W11                 W15

;Sample = {Sample:'31315:31324' , SampleCell:'31313' , SampleB4C:'31312', SThickness:0.2}
;          W12                    W13                  W14

getWone, Wone ,second=second ;Keep workspaces# in use during current Do command

idxp = sans_supl[80:*] & danibl=",danibl=["+string(idxp[5])+"]" ;We may have add runs, so add times also
tags = tag_names(toRead)

IF BATCH THEN BEGIN ;TO IMPLEMENT IN THE FUTUR: TOF-MERGE and PANELS
;******************
for i = 0,n_elements(tags)-1 do begin
  case tags[i] of
; TRANSMISSIONS
; *************
  'TSAMPLE': begin & oprx = toRead.Tsample & ws = '3'
                     if oprx gt ' ' then begin
                       if (string(par3[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W3) gt 4 then begin ;read should be ok
                         par3 = oprx                     ;no need to keep parameters
                       endif else par3 = ''
                     endif else W_clear, W=3
             end
  'TSCELL':  begin & oprx = toRead.Tscell  & ws = '4'
                     if oprx gt ' ' then begin
                       if (string(par4[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W4) gt 4 then begin ;read should be ok
                         par4 = oprx                     ;no need to keep parameters
                       endif else par4 = ''
                     endif else W_clear, W=4
             end
  'TSBEAM':  begin & oprx = toRead.Tsbeam  & ws = '5'
                     if oprx gt ' ' then begin
                       if (string(par5[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W5) gt 4 then begin ;read should be ok
                         par5 = oprx                     ;no need to keep parameters
                       endif else par5 = ''
                     endif else W_clear, W=5
             end
  'TWATER':  begin & oprx = toRead.Twater  & ws = '1'
                     if oprx gt ' ' then begin
                       if (string(par1[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W1) gt 4 then begin ;read should be ok
                         par1 = oprx                     ;no need to keep parameters
                       endif else par1 = ''
                     endif else W_clear, W=1
             end
  'TWCELL':  begin & oprx = toRead.Twcell  & ws = '2'
                     if oprx gt ' ' then begin
                       if (string(par2[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W2) gt 4 then begin ;read should be ok
                         par2 = oprx                     ;no need to keep parameters
                       endif else par2 = ''
                     endif else W_clear, W=2
             end
  'TWBEAM':  begin & oprx = toRead.Twbeam  & ws = '6'
                     if oprx gt ' ' then begin
                       if (string(par6[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W6) gt 4 then begin ;read should be ok
                         par6 = oprx
                         kpsupl=sans_supl
                         take_datp, datp, W=6
                         tmpbeam = W6
                         sans_settings_par, datp , transm=1
                           sans_settings_phi0, sans_inst_T, sans_supl, tmpbeam, attFact=supl26, phi0=flux
                           kpsupl[35]= flux
                           kp_ctr    = sans_ctr        ;keep sample center
                           kpsupl[56:57]=sans_center(beam=6,/fit,/Mask) ;Should help for water auto mask
                           sans_ctr  = kp_ctr
                           kpsupl[39]= sans_supl[1]    ;Tr. water distance
                           kpsupl[40]= sans_supl[23]   ;Tr, Beam water duration
                           kpsupl[41]= sans_supl[4]    ;Tr. water collimation
                           kpsupl[62]= datp.N[0]       ;Tr, Beam water monitor
                                                       ;Note: we keep delta_theta [49]
                         sans_supl=kpsupl
                       endif else par6 = ''
                     endif else W_clear, W=6
             end
; DIRECT BEAM
; ***********
  'BEAMTRC': begin & oprx = toRead.BeamTrC & ws = '9'
                     if oprx gt ' ' then begin
                       if (string(par9[0]) ne oprx) then begin ;or sans_supl[53]
                         RDSET, /TofFlux  ;In case of TOFmerged
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W9) gt 4 then begin ;read should be ok
                         par9 = oprx
                         take_datp, datp, W=9
                         tmpbeam = W9
                         sans_settings_par,  datp, Wks=9
                         sans_settings_phi0, sans_inst_T, sans_supl, tmpbeam, attFact=supl26, phi0=flux
                         sans_supl[26]=supl26         ;ATT factor
                         sans_supl[31]=flux           ;PHI0
                         sans_supl[36]=sans_supl[1]   ;Beam sample distance
                         sans_supl[37]=sans_supl[23]  ;Beam sample duration
                         sans_supl[38]=sans_supl[4]   ;Beam sample collimation
                         sans_supl[63]=datp.N[0]      ;Beam sample monitor
                       endif else par9 = ''
                     endif else W_clear, W=9
             end
  'FLUX':    begin & oprx = toRead.Flux    & ws = '26'
                     if oprx gt ' ' then begin
                       if (string(par26[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W26) gt 4 then begin ;read should be ok
                         par26  = oprx
                         kpsupl = sans_supl    ; save sans_supl
                         take_datp, datp, W=26
                         tmpbeam = W26
                         sans_settings_par,  sdata.datp , transm=1
                         sans_settings_phi0, sans_inst_T, sans_supl, tmpbeam, attFact=supl26, phi0=flux
                           kpsupl[26] = supl26
                           kpsupl[31] = flux     ; MODIFIED PHI0
                           kpsupl[60] = flux
                         sans_supl  = kpsupl   ; restore original sans_supl with [31,60] modified
                       endif else par26 = ''
                     endif else W_clear, W=26
             end
; WATER
; *****
  'WATER':   begin  & oprx = toRead.Water     & ws = '10'
                     if oprx gt ' ' then begin
                       if (string(par10[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W10) gt 4 then begin ;read should be ok
                         par10 = oprx
                         kpsupl= sans_supl
                         take_datp, datp, W=10
                         sans_settings_par, datp

                           kpsupl[32]=sans_supl[1]  ;Water distance
                           kpsupl[33]=sans_supl[23] ;Water duration
                           kpsupl[34]=sans_supl[4]  ;Water collimation
                           kpsupl[51]=sans_supl[2]  ;Water offset
                           kpsupl[59]=sans_supl[3]  ;Water angle detector
                           
                         sans_supl=kpsupl
                       endif else par10 = ''
                     endif else W_clear, W=10
             end
  'WATERCELL':begin & oprx = toRead.WaterCell & ws = '11'
                     if oprx gt ' ' then begin
                       if (string(par11[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W11) gt 4 then begin ;read should be ok
                         par11 = oprx                     ;no need to keep parameters
                       endif else par11 = ''
                     endif else W_clear, W=11
             end
  'WATERB4C':begin  & oprx = toRead.WaterB4C  & ws = '15'
                     if oprx gt ' ' then begin
                       if (string(par15[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W15) gt 4 then begin ;read should be ok
                         par15 = oprx                     ;no need to keep parameters
                       endif else par15 = ''
                     endif else W_clear, W=15
             end
; SAMPLE 
; ******
  'SAMPLE':  begin   & oprx = toRead.Sample     & ws = '12'
                     if oprx gt ' ' then begin
                       if (string(par12[0]) ne oprx) or sans_supl[53] then begin
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"

                         if n_elements(W12) gt 4 then begin ;read should be ok
                          ;CARE WITH KINETIC SAMPLES -> construct Z (the pointer to transmission and thickness)
                           sizS = size(W12) & if sizS[0] eq 3 then nzS = sizS[3]-1 else nzS = 0 ;samples
                           sizT = size(W3)  & nzT=nZs
                           if (sizT[1] eq sizS[1]) and (sizT[2] eq sizS[2]) then $
                                              if sizT[0] eq 3 then nzT = sizT[3]-1 else nzT = 0 ;transmissions
                           pZ=lindgen(nzS+1)
                           if nzS ne nzT then begin
                             take_datp, datp, W=12
                             Ni = strtrim(long64(datp.pv[idxp[6],*]),2) ; Readed list of numors can be mixed kinetic or not
                             oprrg = strsplit(/EXTRACT,oprx,',')
                             Na = 0L                                    ; Construct asked list of numors
                             for k=0,n_elements(oprrg)-1 do begin
                               ncol = 0 & idxcol = strpos(oprrg[k],':')
                               while (idxcol gt 0) do begin & ncol+=1 & idxcol = strpos(oprrg[k],':',idxcol+1) & endwhile
                               tmp  = strsplit(/EXTRACT,oprrg[k],strjoin(replicate(':',ncol>1)))
                               run1 = long(tmp[0])
                               if n_elements(tmp) eq 1 then Na = [ Na , run1]
                               if n_elements(tmp) eq 2 then Na = [ Na , lindgen( (long(tmp[1])-run1+ncol)/ncol )*ncol + run1 ]
                             endfor
                             Na = strtrim(long(Na[1:*]),2)
                             iNa= 0
                             ipZ= 0
                             for iNi = 1,nzS do begin
                               if strpos(Ni[iNi] , Na[iNa]) ne 0 then begin iNa+=1 & ipZ+=1 & endif
                               pZ[iNi] = ipZ
                             endfor
                           endif ;nzS ne nzT
                           Z12 = pZ < nzT
                         endif   ;n_elements(W12) gt 4
                        ;GET PARCONF. par12 contains a numor-title table set by RDAND only
                         take_datp, datp, W=12
                         if n_elements(par12) eq 1 then par12 = strtrim(long(datp.p[idxp[6]]),2)+' '+strcompress(datp.w_tit)
                           
                         sans_reform_series,W12,dp=datp,titles=par12, PARCONF=parconf,curLMH=(sans_supl[78]>0<3),$
                                                instru=sans_supl[5]
                       endif
                       if n_elements(W12) gt 4 then begin ;read should be ok
                         par12 = oprx
                         take_datp, datp, W=12
                         sans_settings_par, datp, Wks=12          ;keep parameters of the sample
                       endif else par12 = ''
                     endif else W_clear, W=12
             end
  'SAMPLECELL':begin & oprx = toRead.SampleCell & ws = '13'
                     if oprx gt ' ' then begin
                       if (string(par13[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W13) gt 4 then begin ;read should be ok
                         par13 = oprx                     ;no need to keep parameters
                       endif else par13 = ''
                     endif else W_clear, W=13
             end
  'SAMPLEB4C':begin  & oprx = toRead.SampleB4C  & ws = '14'
                     if oprx gt ' ' then begin
                       if (string(par14[0]) ne oprx) then begin ;or sans_supl[53]
                         XICUTER,"w"+ws+"=RDOPR('"+oprx+"'"+danibl+")"
                       endif
                       if n_elements(W14) gt 4 then begin ;read should be ok
                         par14 = oprx                     ;no need to keep parameters
                       endif else par14 = ''
                     endif else W_clear, W=14
             end
  else:
  endcase
endfor
ENDIF ;BATCH

for i = 0,n_elements(tags)-1 do begin
  case tags[i] of
    'CONFIG':    begin & sans_supl[78] = where(['' , 'low', 'middle' , 'high'] eq strlowcase(toRead.Config)) > 0
                 end
    'WTHICKNESS':begin & sans_trans[0,1] = toRead.Wthickness[0]
                         sans_supl[10]   = toRead.Wthickness[0]
                 end
    'STHICKNESS':begin & Sthick = toRead.Sthickness
                         ns = size(W12)
                         if ns[0] eq 3 then ns = ns[3] else ns = 1
                         if n_elements(Sthick) ne ns then Sthick = fltarr(ns)+Sthick[0]
                         sans_supl[11]   = Sthick[0]
                         sans_trans[2,1] = Sthick[0]
                         sans_trans[4:4+ns-1,1] = Sthick
                 end
    else:
    endcase
endfor

IF not BATCH then begin
;**********************
;bases3 =[bwatr ,bwatre,bsampl,bsampe,bcadmi,bcadmw,  blabw,blabwe,blabs,blabse,blabc ,blabcw,$   ;0 ->11 sample
;         awatr ,awatre,asampl,asampe,abeamS,abeamW,  alabw,alabwe,alabs,alabse,alabbS,alabbW,$   ;12->23 transmission
;                       msampl,mwater,                             mlabs,mlabw ,confus,$          ;24->28 mask ,center
;         Wwatr2,Wwatr3,Wwate2,Wwate3,Wcadc2,Wcadc3,  Wlabw2,Wlabw3,Wlabe2,Wlabe3,Wlabc2,Wlabc3,$ ;29->40 water alternatives
;                                            SampFX,                                     SlabFX,$ ;41->42 FLUX  alternative

sans_supl[53] = 1 ;Force re-read
widget_control,base0, bad_id=ii, get_uvalue=baset ;get the main structure
widget_control,/hourglass

for i = 0,n_elements(tags)-1 do begin
  chMen = -1
  uv    = -1
  case tags[i] of
    ;TRANSMISSIONS      ;uv = [9, bases3_index_for_LABEL, Wrkspace#, bases3_index_for_text_opr]
    ;*************
    'TSAMPLE':   begin & uv = [9,20,3 ,baset.bases3[14]] & opr = toRead.TSAMPLE    & if cur_menu[0] ne 1 then chMen = 1 & end
    'TSCELL':    begin & uv = [9,21,4 ,baset.bases3[15]] & opr = toRead.TSCELL     & if cur_menu[0] ne 1 then chMen = 1 & end
    'TSBEAM':    begin & uv = [9,22,5 ,baset.bases3[16]] & opr = toRead.TSBEAM     & if cur_menu[0] ne 1 then chMen = 1
                                                                           sans_settings_beamST,baset, Tog_STbeam='Trs' & end
    'TWATER':    begin & uv = [9,18,1 ,baset.bases3[12]] & opr = toRead.TWATER     & if cur_menu[0] ne 1 then chMen = 1 & end
    'TWCELL':    begin & uv = [9,19,2 ,baset.bases3[13]] & opr = toRead.TWCELL     & if cur_menu[0] ne 1 then chMen = 1 & end
    'TWBEAM':    begin & uv = [9,23,6 ,baset.bases3[17]] & opr = toRead.TWBEAM     & if cur_menu[0] ne 1 then chMen = 1 & end
    
    ;DIRECT BEAM
    ;***********
    'BEAMTRC':   begin & uv = [9,22,9 ,baset.bases3[16]] & opr = toRead.BEAMTRC    & if cur_menu[0] ne 0 then chMen = 0
                                                                           sans_settings_beamST,baset, Tog_STbeam='Ctr' & end
    'FLUX':      begin & uv = [9,42,26,baset.bases3[41]] & opr = toRead.FLUX       & if cur_menu[0] ne 3 then chMen = 3 & end
    
    ;WATER
    ;*****
    'WATER':     begin & uv = [9,6 ,10,baset.bases3[0]]  & opr = toRead.WATER      & if cur_menu[0] ne 3 then chMen = 3 & end
    'WATERCELL': begin & uv = [9,7 ,11,baset.bases3[1]]  & opr = toRead.WATERCELL  & if cur_menu[0] ne 3 then chMen = 3 & end
    'WATERB4C':  begin & uv = [9,11,15,baset.bases3[5]]  & opr = toRead.WATERB4C   & if cur_menu[0] ne 3 then chMen = 3 & end
    
    ;SAMPLE
    ;******
    'SAMPLE':    begin & uv = [9,8 ,12,baset.bases3[2]]  & opr = toRead.SAMPLE     & if cur_menu[0] ne 3 then chMen = 3 & end
    'SAMPLECELL':begin & uv = [9,9 ,13,baset.bases3[3]]  & opr = toRead.SAMPLECELL & if cur_menu[0] ne 3 then chMen = 3 & end
    'SAMPLEB4C': begin & uv = [9,10,14,baset.bases3[4]]  & opr = toRead.SAMPLEB4C  & if cur_menu[0] ne 3 then chMen = 3 & end
    else:
  endcase
  if uv[0] eq 9 then begin ;we have a numor to read
    widget_control,uv[3], set_value='W'+strtrim(uv[2],2)+'= '+opr ;put the numor in its text widget
    sans_settings_runs, baset , uv      ;read the numor
  endif

  if chMen eq 0 then newUV = [30,1]   ;Check and have the correct layout (Center)
  if chMen eq 1 then newUV = [40]     ;"" (Transmissions)
  if chMen eq 7 then newUV = [80,0,0] ;"" (Sectors mask)
  if chMen eq 3 then newUV = [61]     ;"" (scattering samples)
  if chMen ge 0 then begin & bidbas = baset.bases7[22] ;Use the "Log" button for bluffing.
     widget_control, bidbas , get_uvalue=keepUV
     widget_control, bidbas , set_uvalue=newUV
     ii=execute('sans_settings_event,{WIDGET_BUTTON,ID:bidbas,TOP:base0,HANDLER:0L,SELECT:1}')
     widget_control, bidbas , set_uvalue=keepUV & endif
endfor
widget_control,base0, bad_id=ii, set_uvalue=baset ;save the main structure
ENDIF ;NOT BATCH

one = Wone & two = second ;Restore workspaces# in use during current Do command
end

pro sans_settings_runs, baset ,uv, haveall=haveall, config=config, noread=noread
;+
;** ******************
;** ******************
;** Using the GUI:
;** reads the numors (calling RDOPR) and update parameters in sans_supl
;-

common c_sans_settings
common c_sans_tofl,   TofFlux, LambdaVq
common c_sans_cur_run
common c_sans_cur_aXY
common c_sans_place,  TRplace, SPplace
common c_lamp_par
common c_lamp2par

frst=3
FX  =sans_supl[30]
idxp=sans_supl[80:*]
if  n_elements(uv) gt frst+1 then readall=1 else readall=0 ;HAVEALL -> DO NOT SHOW DATA ***
if  keyword_set(haveall)     then readall=1                ;/NOREAD is USED WHEN FORCE READ IS SET (means no re-read)
for i=frst,n_elements(uv)-1  do begin
    widget_control,uv[i],get_value=txt,get_uvalue=uvt & uv[2]=uvt[2] & txt=strcompress(txt[0],/remove_all)
    idx=strpos(txt,'=')
    wi =0
    if idx lt 2 then begin & wi=uv[2] & ws=strtrim(string(wi),2) & txt='w'+ws+'='+strmid(txt,idx+1) & idx=strlen(ws)+1 & endif
    if idx ge 2 then begin
       ws =strmid(txt,1,idx-1) & wi=long(ws)
       opr=strmid(txt,idx+1)
       widget_control,baset.bases3[uv[1]+i-frst],get_uvalue=sdata & oprlast=sdata.opr & wilast=sdata.wi & parlast=sdata.par

       isTRANS = ( (cur_menu[10] eq 1) or ((uv[2] ge 1) and (uv[2] le 6)) or (uv[2] eq 26))
       isWATER = ( (uv[2] eq 8) or ((uv[2] eq 10) or (uv[2] eq 11) or (uv[2] eq 15) or (uv[2] eq 24)) )

       curCONF = baset.curLMH
       if isWATER then if baset.curWATER eq 2 then curCONF=3 else if baset.curWATER eq 3 then curCONF=1
     ; print,baset.curLMH, curCONF, baset.curWATER, uv[2]

     ; Check if read option has changed: resulution + panel*10 + etc ...*100 + ...
       curOPT  = sans_supl[67] + sans_supl[67+curCONF]*10
       sdataTAG= tag_names(sdata)
       if (where(sdataTAG eq 'OPT'))[0] ge 0 then $ ;to be compatible with old Sheets
        if curOPT ne sdata.OPT then $
         oprlast = '*****READ BECAUSE READING OPTION CHANGED*****'
     ; end check reading option changed
     
       par='' & ii=execute("par=par"+ws)
       Kerr_string=""

       if (opr le ' ') then begin & widget_control,baset.bases3[uv[1]+i-frst],set_uvalue={data:0, datp:0, opr:opr, wi:wi, par:'', opt:curOPT}
                                  XICUTE,"w"+ws+"=0"
                                  if (uv[2] eq 9)  then sans_supl[[31,36]]=0. ;Reset phi0 Sample
                                  if (uv[2] eq 26) then sans_supl[60]     =0. ;Reset alternative flux
                                  if (uv[2] eq 6)  then sans_supl[[35,39]]=0. ;Reset phi0 Water
       endif else begin

         if (sans_supl[53]) or ((baset.bases6[20]) and (not keyword_set(noread))) then $
                  oprlast='*****READ ANYWAY EVEN IN MEMORY*****' $
         else $
         ;Workspace modified outside sans_settings!!!
         if (opr eq oprlast) and (wi eq wilast) and (string(par[0]) ne string(parlast[0])) and (n_elements(sdata.data) gt 1) then begin
                  print,opr+' restored->W' +ws
                  GIVE_W,sdata.data, W=wi  & ii=execute("par"+ws+"=parlast") & GIVE_DATP,sdata.datp, W=wi & endif
       endelse

       if (opr ne oprlast) or  (wi ne wilast) then begin ;SHOULD READ

                  iln=strlen(opr) & add='0'
                  if  strpos(opr,'+') eq iln-1 then begin & oprx=strmid(opr,0,iln-1) & add='1' & endif else oprx=opr

                  oprrg=strsplit(/EXTRACT,oprx,',')   & pZ=0L & off=1  ;Construct pZ for the fun (will be reconstructed below....)
                  for k=0,n_elements(oprrg)-1 do begin
                       ncol = 0 & idxcol = strpos(oprrg[k],':')
                       while (idxcol gt 0) do begin & ncol+=1 & idxcol = strpos(oprrg[k],':',idxcol+1) & endwhile
                       tmp  = strsplit(/EXTRACT,oprrg[k],strjoin(replicate(':',ncol>1)))
                       if tmp[0] le ' '        then       off=off+1 else $
                       if n_elements(tmp) eq 1 then begin & pZ =[pZ,             off+pZ[n_elements(pZ)-1]] & off=1
                       endif                   else begin & tmp=( (long(tmp[1])-long(tmp[0])+ncol)/ncol )>1
                                                            pZ =[pZ,lindgen(tmp)+off+pZ[n_elements(pZ)-1]] & off=1 & endelse
                  endfor
                  if n_elements(pZ) gt 1 then pZ=pZ[1:*]-1 & npZ=n_elements(pZ)

                  danibl=""                                     ;and added runs
                  if (strpos(oprx,'+') gt 0) or (strpos(oprx,'>') gt 0) or (strpos(oprx,'-') gt 0) then danibl=",danibl=["+string(idxp[5])+"]"
                  if npZ gt 1 then danibl=danibl+",labid=-"+strtrim(baset.bases1[12],2)+"L"

                  Kerr_string=!error_state.msg
                  
                  if (opr ne oprlast) then begin ;MUST READ
                      ii=execute("par"+ws+"='"+oprx+"'")
                     ;Right click and wrksp
                     
                      if  (uv[2] eq 9) $ ;and (sans_supl[60] le 0)) or (uv[2] eq 26) $
                                       then RDSET, /TofFlux  ;In case it is a TOFmerged file, ask to keep the Flux somewhere

                      if (sans_supl[66] gt 1) then begin                   ;Instrument with panels
                          if isTRANS then $                                ;Transmission runs  so
                             RDSET, panels=[1,lonarr(sans_supl[66]-1)] $   ;panels are not necessary (NegMask)
                          else if (curCONF ge 1) and (sans_supl[67+curCONF]) then $ ;else panels On?
                                   RDSET, panels=lonarr(sans_supl[66])+1     else RDSET, panels=[1,lonarr(sans_supl[66]-1)]
                      endif
                      
                      if (sans_supl[64] eq 4.) and (istrans) and (n_tags(TofFlux) gt 1) then begin
                        TofFlux.Request = 2    ; TOFmerge read + transmission file + Flux already there so ...
                        RDSET, TofFlux=TofFlux ; so tell the read procedure to use current flux even if config is different.
                      endif
                      
                      widget_control,/hourglass
                     ;**********************************************
                      XICUTER,"w"+ws+"=rdopr('"+oprx+"'"+danibl+")" ;******************... READING ...*******************
                     ;**********************************************

                      if (sans_supl[64] eq 4.) and (istrans) and (n_tags(TofFlux) gt 1) then begin
                        TofFlux.Request = 0    ; TOFmerge read + transmission file + Flux already there so ...
                        RDSET, TofFlux=TofFlux ; so tell the read procedure to use current flux only if config is identical.
                      endif

                      if (sans_supl[66] gt 1) then begin                   ;Instrument with panels
                          if (isTRANS) or (curCONF ne baset.curLMH) then $ ;Transmission or Water run so
                             if (baset.curLMH ge 1) then begin
                                if (sans_supl[67+baset.curLMH]) then RDSET, panels=[lonarr(sans_supl[66])+1] $ ;put panels On (NegMask)
                                                                else RDSET, panels=[1,lonarr(sans_supl[66]-1)] ;put panels Off
                                 cur_menu[4]=1                             ;hide all masks
                             endif
                      endif                                                
                      if  (uv[2] eq 9) $ ;and (sans_supl[60] le 0)) or (uv[2] eq 26) $
                                       then RDSET, TofFlux=keep_Flux ;In case it is a TOFmerged file, get back the Flux structure
                  endif
                  
                  tseq=systime() & tseq=strmid(tseq,11,2)+'_'+strmid(tseq,14,2)+'_'+strmid(tseq,17,2)

                  if (Kerr_string ne !error_state.msg) and (!error_state.msg gt ' ') and (strpos(!error_state.msg,'conversion') lt 0) then begin
                     Kerr_string=!error_state.msg & ii=execute('Reading_Error_'+tseq) ;change string error for next time
                     if (uv[2] eq 7) or (uv[2] gt 0) then begin ;a numor in mask or else , try to find it in our tables and put in wi
                        wii=wi & sans_settings_WNI, wii, get=oprx
                     endif else wii=0
                     if wii eq 0 then begin
                        widget_control,baset.bases1[12],set_value='!!! Error reading '+opr
                        widget_control,baset.bases1[0] ,set_value='!!! Error reading '+opr & opr=' ' &  endif
                        lamp_journal,Kerr_string
                  endif else Kerr_string=""

                  TAKE_W ,data ,W=wi & par=' ' & parconf='' & TAKE_DATP,dp ,W=wi

                  sz=size(data) & if sz[0] eq 3 then if sz[3] lt npZ then begin & opr=' ' & Kerr_string='!!! Missing runs' & endif
                                  if sz[0] eq 2 then if npZ   gt 1   then begin & opr=' ' & Kerr_string='!!! Missing runs' & endif

                  sz=size(data) & if sz[0] eq 3 then npZ=sz[3] else npZ=1
                  npP=n_elements(dp.p)                                                       ;WE HAVE STRANGE DATA !!!!
                  if  n_elements(dp.pv) lt npP*npZ then begin & if npZ eq 1 then pv=dp.p else pv=congrid([[dp.p],[dp.p]],npP,npZ) & mod_datp,dp,'PV',pv
                  endif else  if npP eq 1      then mod_datp,dp,'P',dp.pv
                  if  n_elements(dp.N)  lt npZ then mod_datp,dp,'N',replicate(dp.N[0],npZ)

                  if (uv[2] eq  3) and (not sans_settings_RegSheet()) then sans_settings_series,[oprx], /Tplace
                  if (uv[2] eq 12) and (not sans_settings_RegSheet()) then sans_settings_series,[oprx], /Splace

                  if (uv[2] eq 12) and (npZ gt 0) and (opr gt ' ') then begin & pZ=lindgen(npZ)  ;***CHECK FOR HOLES AND/OR KINETICS !!!

                    ;at this time par12 contains a numor-title table set by RDAND only

                    ;TRplace and SPplace = [[first numor in cells] , [line index] , [position of ':']]
                     if n_elements(SPplace) gt 1 then begin & ni=strtrim(long64(dp.pv[idxp[6],*]),2) ;Numors in PV (means runs+idx for frames)
                                                            nj=strtrim(long(  SPplace[*,0]),2)       ;Numors first in table cells
                        ii  =execute('ppar=par'+ws)
                        if (n_elements(ppar) ne npZ) or (ppar[0] le ' ') then ppar = strtrim(string(long(nj)),2)+' '+dp.w_tit
                                                            nr=strtrim(long(  ppar),2)             ;Numors readded
                        if n_elements(TRplace) le 1 then TRplace=SPplace*(-1)
                        colx=where(TRplace[*,2] gt 0) & if colx[0] ge 0 then colx=TRplace[colx,1]  ;We may have syntax "runa:runb" in transmissions
                        npl =0 & njl=n_elements(nj)-1 & nrl=n_elements(nr)-1 & off=0

                        pZ  =pZ+ SPplace[npl,1]       & pZS =' '+strtrim(10001+pZ,2)
                        npP =0 & kpar=strarr(npZ)     & kpar[0]=ppar[0]+pZS[0]
                                                      ; k  -> ni from PV      (all runs in data) goes in kpar
                                                      ; npl-> nj from SPplace (first run in table cells)
                                                      ; npP-> nr from ppar    (list of numors)
                                                      ; npl->                 (line number)
                        for k=1,n_elements(ni)-1 do begin ;
                            if (strpos(ni[k],nr[npP]) eq 0) and (ni[k] ne nr[npP]) then pZ[k]=pZ[k-1] else $  ;*** This is Kinetic frames (k>npP) (keep same index line)
                            if  strpos(ni[k],nr[(npP+1)<nrl]) eq 0  then begin & npP=npP+1                      ;*** else we go forward numor in nr (npP)
                                      kpnpl=npl
                                      if npl lt njl then if nj[npl+1] eq nr[npP] then npl=npl+1 ;*** We change line cell also
                                      if kpnpl eq npl then begin
                                                ok=where(colx eq SPplace[npl,1])                ;*** No change line but !
                                                if ok[0] ge 0 then off=off+1                    ;    TR  syntax "runa:runb"
                                      endif
                                      pZ[k]=SPplace[npl,1]+off
                            endif
                            kpar[k]=ppar[npP]+pZS[k]
                        endfor
                        ii=execute('par'+ws+'=kpar')
                     endif
                  endif ;uv[2] eq 12 (Samples)

                  if (uv[2] eq 9) then keep_Z = dp.Z ;In case it is a TOFmerged file, we have Lambda versus Q
                  
                  mod_datp,dp,'Z',pZ & give_datp,dp,W=wi

                  if n_elements(data) le 1 then opr=' ' & if opr le ' ' then data=0

;                 if opr gt ' ' then if (sz(1) lt sans_masq(2,0)) or (sz(2) lt sans_masq(3,0)) then begin
                  if opr gt ' ' then if (sz[1] lt 2)              or (sz[2] lt 2)              then begin ;Changed for D33 panels
                     Kerr_string='!!! bad file size for this Instrument '+opr
                     widget_control,baset.bases1[12],set_value=Kerr_string
                     widget_control,baset.bases1[0] ,set_value=Kerr_string & opr=' '
                     ii=execute('Reading_Error_'+tseq) ;change string error for next time
                     endif
                  if opr gt ' ' then begin ; Make parconf(titles) and put in sdata
                     sans_gt_par ,sys=sys & sys1=strtrim(sys+1,2)

                     XICUTE,"sans_reform_series,W"+ws+",titles=par"+ws+",parconf=par"+sys1+",add="+add+",curLMH="+string(baset.curLMH)+",instru="+string(baset.IDXI)
                     sans_gt_par ,1 ,get=parconf ,set=' '
                     if add eq '1' then begin & TAKE_W ,data ,W=wi & TAKE_DATP,dp,W=wi & endif
                             sans_settings_WNI, dp, set=wi ;put   wi,numor in our tables
                  endif else sans_settings_WNI,     set=wi ;clear wi,numor in our tables

                  iii=execute('par='+'par'+ws)
                  sdata={data:data, datp:dp, opr:opr, wi:wi, par:par, parconf:parconf, opt:curOPT}
                  widget_control,baset.bases3[uv[1]+i-frst],set_uvalue=sdata

;** sans_supl(0)  Wavelength
;** sans_supl(1)  Distance sample->detector sample
;** sans_supl(2)  Offset detector (mm) sample
;** sans_supl(3)  Angle detector (deg) sample
;** sans_supl(4)  Collimation sample

;RD sans_supl(23) Beam Duration
;RD sans_supl(24) Beam attenuation index
;RD sans_supl(25) Beam att on/off
;** sans_supl(32) Water distance
;** sans_supl(33) Water duration (unused as sample -> taken in datp)
;** sans_supl(34) Water collimation

;** sans_supl(31) C Phi_0 sample (N/Sec) corrected, masked
;** sans_supl(36) C Beam sample distance
;** sans_supl(37) C Beam sample duration
;** sans_supl(38) C Beam sample collimation

;** sans_supl(35) Phi_0 water  (N/Sec) corrected, masked
;** sans_supl(39) Beam water  distance
;** sans_supl(40) Beam water  duration
;** sans_supl(41) Beam water  collimation
;** sans_supl(51) Offset detector (mm) water
;** sans_supl(56) Water X center (calculated at beam water read)
;** sans_supl(57) Water Y center (calculated at beam water read)
;== sans_supl(58) =1 if water can't be used for correction (set by sans_cor)
;** sans_supl(59) Water Angle detector (deg)
;== sans_supl(60) (alternative Phi_0 sample supl[31])
;== sans_supl(61) =1 disable Qdep correction
;RD sans_supl(62) Beam water  monitor related to sans_supl(40) in sans_settings_runs
;RD sans_supl(63) Beam sample monitor related to sans_supl(37) in sans_settings_runs
;RD sans_supl(64) Mode 1:Tof  3:Kin  4:TOFmerged
;** sans_supl(65) 0:disable offset  1:enable offset detector  2:enable offset distance
;** sans_supl(66) >=1 means at read negative counts must be masked (inter panels)
;$$ sans_supl(67) 2:low resolution panels  1:high resolution
;$$ sans_supl(68) 1:enable all panels conf 1
;$$ sans_supl(69) 1:enable all panels conf 2
;$$ sans_supl(70) 1:enable all panels conf 3
;$$ sans_supl(71) Attenuator factor comming from data parameters

                                   ;cur_menu(10): 0=center window 1=transmission window 2=mask 3=data
                 ;SAMPLE OR BEAM -> get parameters  3=Tsample  5=TbeamS  9=directbeamS  12=Sample  13=CellSample

                  if (opr gt ' ') and ((uv[2] eq 3) or (uv[2] eq 5) or (uv[2] eq 9) or (uv[2] eq 12) or (uv[2] eq 13)) then begin
                            kpsupl=sans_supl
                            sans_settings_par, sdata.datp , transm = isTRANS , Wks=uv[2]

                            if (uv[2] eq 12) then begin & sz=size(sdata.data) & tz=size(sans_trans)   ;Sample->size transmission
                               if sz[0] eq 3 then if tz[1] lt sz[3]+4 then begin & kptrans=sans_trans ;Kinetics!!!!
                                  sans_trans=fltarr(4+sz[3],5)
                                  sans_trans[0:tz[1]-1,0]=kptrans[*,0] & sans_trans[tz[1]:*,0]=sans_trans[2,0]
                                  if tz[0] gt 1 then begin & sans_trans[0:tz[1]-1,1]= kptrans[*,1]
                                                             sans_trans[tz[1]:*  ,1]= sans_supl[11]
                                  endif         else begin & sans_trans[      *  ,1]= sans_supl[11]
                                                             sans_trans[[0,1],1]    = sans_supl[10] & endelse
                            endif & endif

                           ;if (uv(2) eq 12) or (uv(2) eq 13) or (uv(2) eq 9) then begin ;Sample,cell,Dbeam -> dist+collim
                            if (uv[2] eq 12)                  or (uv[2] eq 9) then begin ;Sample     ,Dbeam -> dist+collim
                                     collim=strsplit(/EXTRACT,baset.collim,',')
                                     colidx=where(round(float(collim)*10.)/10. eq round(sans_supl[4]*10.)/10.) & colidx=colidx[0]>0
                                     baset.colidx=colidx
                                     baset.colval=sans_supl[4]
                                     widget_control,baset.bases2[1] ,set_value=string(sans_supl[1] ,format='(f7.4)') ;Distance sample->detector
                                     widget_control,baset.bases2[19],set_value=string(sans_supl[4] ,format='(f7.4)') ;Collimation sample
                                     widget_control,baset.bases1[5] ,set_list_select=baset.colidx
                            endif

                            if (uv[2] eq 9) then begin ;calculate phi_0 (N/Sec) * Attenuator factor
                                     flux   = sans_supl[60]
                                     supl26 = sans_supl[26]
                                     if flux le 0 then sans_settings_phi0, baset, sans_supl, sdata.data, attFact=supl26, phi0=flux
                                     widget_control,baset.bases5[16],set_value=' Phi0 (N/sec)= '+strtrim(string(flux,format='(I10)'),2)
                                     sans_supl[26]=supl26         ;ATT factor
                                     sans_supl[31]=flux           ;PHI0
                                     sans_supl[36]=sans_supl[1]   ;Beam sample distance
                                     sans_supl[37]=sans_supl[23]  ;Beam sample duration
                                     sans_supl[38]=sans_supl[4]   ;Beam sample collimation
                                     sans_supl[63]=sdata.datp.N[0];Beam sample monitor
                                     
                                     if (sans_supl[64] eq 4.) and ((size(keep_Z))[0] eq 2) then begin ;We have a TOFmerged image
                                         TofFlux = keep_Flux & LambdaVq = keep_Z ; Lambda versus Q in keep_Z , Flux structure in keep_Flux
                                         widget_control,baset.bases7[23],set_button=0,sens=0 & sans_supl[27]=0 ;No Dead Times
                                         widget_control,baset.bases7[24],set_button=0,sens=0 & sans_supl[28]=0 ;No Solid angles
                                         widget_control,baset.bases7[26],set_button=0,sens=0 & sans_supl[43]=0 ;No Tr(2th)
                                     endif else begin
                                         widget_control,baset.bases7[23],sens=1 ;Dead Times   sensitive
                                         widget_control,baset.bases7[24],sens=1 ;Solid angles sensitive
                                         widget_control,baset.bases7[26],sens=1 ;Tr(2th)      sensitive
                                     endelse
                            endif
                           ;Just to have a Q value on images (before having sample/cell)
                            if (uv[2] eq 9) and (kpsupl[1] eq 0.) then kpsupl[1:2]=sans_supl[1:2]

                            if (uv[2] ne 12) and (uv[2] ne 13) and (kpsupl[1] ne 0) then begin
                                       sans_supl[1] =kpsupl[1]    ;keep distance    for sample
                                       sans_supl[2] =kpsupl[2]    ;keep offset      for sample
                                       sans_supl[3] =kpsupl[3]    ;keep angle det   for sample
                                       sans_supl[4] =kpsupl[4]    ;keep collimation for sample
                            endif
                           ;WARNING: we lost att for sample!!! (retreive from datp)****************
                            if (uv[2] ne 12) then begin
                                       sans_supl[22]=kpsupl[22]   ;keep numor       for sample
                                       sans_supl[23]=kpsupl[23]   ;keep duration    for sample
                                       sans_supl[46]=kpsupl[46]   ;keep Srot        for sample
                                       sans_supl[47]=kpsupl[47]   ;keep Stemp       for sample
                            endif
                            widget_control,baset.bases2[0] ,set_value=string(sans_supl[0] ,format='(f6.3)') ;Wavelength
                            widget_control,baset.bases2[6] ,set_value=string(sans_supl[2] ,format='(f7.3)') ;Offset detector (mm)
                            widget_control,baset.bases2[7] ,set_value=string(sans_supl[3] ,format='(f7.3)') ;Angle detector (deg)
                  endif

                 ;BEAM ALTERNATIVE FLUX: 26=FLXbeam -> calculate phi_0 (N/Sec) * Attenuator factor
                  if (opr gt ' ') and (uv[2] eq 26) then begin
                            kpsupl = sans_supl    ; save sans_supl
                            sans_settings_par,  sdata.datp , transm = isTRANS
                            sans_settings_phi0, baset, sans_supl, sdata.data, attFact=supl26, phi0=flux

                            widget_control,baset.bases5[16],set_value=' Phi0 (N/sec)= '+strtrim(string(flux,format='(I10)'),2)
                            widget_control,baset.bases1[12],set_value=' Phi0 (N/sec)= '+strtrim(string(flux,format='(I10)'),2)
                            kpsupl[26] = supl26        ; ATT factor used
                            kpsupl[31] = flux          ; MODIFIED PHI0
                            kpsupl[60] = flux
                            sans_supl = kpsupl         ; restore original sans_supl with [31,60] modified
                  endif

                 ;WATER -> get parameters  1=Twater  6=TbeamW  10=Water
                  if (opr gt ' ') and ((uv[2] eq 1) or (uv[2] eq 6) or (uv[2] eq 10)) then begin
                               kpsupl=sans_supl
                               sans_settings_par, sdata.datp , transm = isTRANS

                               if (uv[2] eq 10) then begin
                                   kpsupl[32]=sans_supl[1]  ;Water distance
                                   kpsupl[33]=sans_supl[23] ;Water duration
                                   kpsupl[34]=sans_supl[4]  ;Water collimation
                                   kpsupl[51]=sans_supl[2]  ;Water offset
                                   kpsupl[59]=sans_supl[3]  ;Water angle detector
                                   widget_control,baset.bases2[23],set_value=string(sans_supl[1] ,format='(f7.4)') ;Distance water->detector
                                   widget_control,baset.bases2[22],set_value=string(sans_supl[4] ,format='(f7.4)') ;Collimation water
                               endif
                               if (uv[2] eq 6) then begin ;calculate Water phi_0 (N/Sec) * Attenuator factor
                                     tmpbeam = sdata.data
                                     sans_settings_phi0, baset, sans_supl, tmpbeam, attFact=supl26, phi0=flux
                                     
                                     widget_control,baset.bases7[15],set_value=' Phi0 (N/sec)= '+strtrim(string(flux,format='(I10)'),2)
                                     kpsupl[35]   =flux
                                    ;kpsupl[26]   =supl26 ;??? used for SDbeam
                                     kp_ctr  =sans_ctr
                                     kpsupl[56:57]=sans_center(beam=6,/fit,/Mask) ;Should help for water auto mask
                                     sans_ctr=kp_ctr
                               endif

                               if (uv[2] eq 1) or (uv[2] eq 6) then begin ;Water transmission
                                   kpsupl[39]=sans_supl[1] ;Tr. water distance
                                   kpsupl[41]=sans_supl[4] ;Tr. water collimation
                                   if (uv[2] eq 6) then begin 
                                       kpsupl[40]=sans_supl[23]   ;Tr, Beam water duration
                                       kpsupl[62]=sdata.datp.N[0] ;Tr, Beam water monitor
                                   endif
                                  ;SHARE TRANSMISSION WATER WITH ANY CONFIGS :
                                   sans_settings_swap, baset, shareSupl=[     35,39,40,41   ,56,57   ,62], supl=kpsupl ;!!!
                               endif

                               sans_supl=kpsupl
                              ;SHARE WATER WITH OTHER CONFIGS relative to curWATER:
                               sans_settings_swap, baset, shareSupl=[32,33,34,35,39,40,41,51,56,57,59,62], supl=sans_supl, forwater=baset.curWATER
                  endif
       endif ;new read

       if (opr ne oprlast) or (wi ne wilast) or (opr le ' ') or (not readall) then $
       if sans_settings_RegSheet()  then begin
                      if opr le ' ' then begin & sans_sheet_receive, wave_dist_cnt={wavel:' ',dist:' ',colli:' ',offs:' ',time:' ',$
                                                                                  count:'<0>',opr:opr,cell:uv[2],watrN:baset.curWATER}
                                               sans_sheet_receive, ERROR=Kerr_string
                      endif else begin
                         kpsupl=sans_supl & sans_settings_par, sdata.datp , transm = isTRANS
                         supl  =sans_supl
                         if kpsupl[1] ne 0. then sans_supl=kpsupl ;Case of a first read !!!
                         totcnt=sdata.data
                         sz=size(totcnt) & if sz[0] eq 3 then nz=sz[3] else nz=1
                         sans_settings_getmask,size(totcnt[*,*,0]),[0.,0.],M,/nobeam,/noWbeam
                         if  n_elements(M) eq sz[1]*sz[2] then for n=0,nz-1 do totcnt[*,*,n]=totcnt[*,*,n]*(1-M)
                         pZ= sdata.datp.Z & npZ=n_elements(pZ) & upZ=n_elements(uniq(pZ))
                         if nz eq 1 then totcnt=total(totcnt) else $
                         if nz gt 1 then if upZ eq 1  then  begin &  totcnt=total(totcnt) & nz=1 & endif
                         if nz gt 1 then begin & totcnt=total(totcnt,1) & totcnt=total(totcnt,1) & endif
                         totcnt=strtrim(string(totcnt,format='(I10)'),2)
                        ;if (uv(2) eq 12) then begin
                           if nz gt 1 then begin & titou=sdata.par
                                      for k=0,n_elements(titou)-1 do titou[k]=strmid(titou[k],0,(strlen(titou[k])-6)>1)
                           endif else titou=strtrim(strmid(strcompress(sdata.datp.w_tit),0,20),2)
                           totcnt=titou+',N='+totcnt ;+strtrim(sdata.datp.p(idxp(5)),2)+'s'
                        ;endif
                         if nz gt 1 then begin & tmp=strarr(max(pZ+1))+'-' & tmp[pZ]=totcnt[0:(n_elements(pZ)<nz)-1] & totcnt=tmp & endif
                         wdm={wavel:string(supl[0],format='(F5.2)'),dist:string(supl[1],format='(F5.2)'),colli:string(supl[4],format='(F5.2)'),$
                              offs:string(supl[2],format='(F6.2)'),time:strtrim(string(supl[23],format='(I6)'),2),count:totcnt,opr:opr,$
                              cell:uv[2],watrN:baset.curWATER}
                         sans_sheet_receive, wave_dist_cnt=wdm
                      endelse
       endif
    endif ;idx ge 2
endfor
if (not readall) and (wi gt 0) and (opr gt ' ') then begin
                               siz=size(sdata.data) & if siz[0] eq 3 then sz=siz[3]-1 else sz=0 & vz=sz
                              ;Sliders
                               if sz eq 0 then begin & widget_control,baset.bases9[9] ,sens=0
                                                       widget_control,baset.bases9[11],sens=0
                                                       widget_control,baset.bases9[10],set_value=' '
                                                       widget_control,baset.bases9[12],set_value=' '
                               endif else begin &      widget_control,baset.bases9[9] ,sens=1,set_slider_max=sz
                                                       widget_control,baset.bases9[11],sens=1,set_slider_max=sz
                                                       widget_control,baset.bases9[9] ,get_value=vz
                                                       vz=vz<sz & num=0
                                      ii=execute('num=long(sdata.datp.pv[idxp[6],vz])')
                                      if num le 0 then num=vz+1
                                                     widget_control,baset.bases9[9] ,set_value=vz
                                                     widget_control,baset.bases9[11],set_value=vz
                                                     widget_control,baset.bases9[10],set_value=strtrim(num,2)
                                                     widget_control,baset.bases9[12],set_value=strtrim(num,2)
                               endelse
                               cur_wn=wi & cur_menu[7]=uv[2]
                               cur_aX=sdata.datp.X
                               cur_aY=sdata.datp.Y
                              ;Do not mask the 2D image if alternative flux
                               if uv[2] eq 26 then sans_settings_mask, baset, run=sdata.data[*,*,vz],/nobeam,/nowbeam $
                                              else sans_settings_mask, baset, run=sdata.data[*,*,vz]
                               if opr gt ' ' then begin
                                  if sz eq 0   then title=sdata.datp.w_tit else title=sdata.par[vz]
                                  title=strmid(strcompress(title)+' ('+sdata.opr+')',0,55)
                                  widget_control,baset.bases1[11],set_value=title
                                  if vz eq 0 then SEE,w=wi,/below,/image
                                  to_did_cur,'W'+ws

                                if (uv[2] ge 1) and (uv[2] le 6) then begin; Gives total counts transmissions
                                    sans_settings_text, baset.bases7[0:3], farr, sarr, mess
                                    sarr=strtrim(string(sarr-FX),2)
                                    sans_settings_getmask,size(cur_run),[0.,0.],M,/nobeam,/noWbeam
                                    if  n_elements(M) eq n_elements(cur_run) then begin
                                        totcnt=cur_run*(1-M)
                                        if (farr[0] lt farr[1]) and (farr[2] lt farr[3]) then totcnt=totcnt[sarr[0]:sarr[1],sarr[2]:sarr[3]]
                                        totcnt=strtrim(string(total(totcnt),format='(I10)'),2)
                                        widget_control,baset.bases7[29],set_value=' '+opr+' counts='+totcnt
                                endif & endif
                                if cur_menu[1] eq 5 then cur_menu[1]=4 & k=cur_menu[1]
                                widget_control,baset.bases5[5],map=0   & widget_control,baset.bases5[k],map=1
                                if not keyword_set(config) then begin
                                widget_control,baset.bases1[7],map=0   & widget_control,baset.bases1[8],map=1 & endif
                                widget_control,baset.bases4[4],set_value=strmid(opr,0,55)+"  (W"+ws+")"
                               endif
endif
end

pro sans_settings_par ,datp , transm=isTRANS , Wks=wi
;+
;** *****************
;** *****************
;** map wanted parameters in datp.p using specific instrument table index
;-


;** sans_supl(65) 0:disable offset  1:enable offset detector  2:enable offset distance
common c_sans_settings
common c_sans_base0   , base0
common c_lamp_p

idxp=sans_supl[80:*]

sans_supl[0] =datp.p[idxp[0]]  &  sans_supl[1] =datp.p[idxp[1]]  ;wavl     , distance
if sans_supl[65] eq 1 then        sans_supl[2] =datp.p[idxp[2]]  ;offset detector if supported
sans_supl[3] =datp.p[idxp[3]]                                    ;angle detector
sans_supl[4] =datp.p[idxp[4]]  &  sans_supl[22]=datp.p[idxp[6]]  ;collima  , numor
sans_supl[23]=datp.p[idxp[5]]  &  sans_supl[24]=datp.p[idxp[7]]  ;duration , attenuator index
sans_supl[25]=datp.p[idxp[8]]                                    ;att in/out
sans_supl[46]=datp.p[idxp[10]] &  sans_supl[47]=datp.p[idxp[11]] ;S rotat  , S temp
sans_supl[50]=datp.p[idxp[12]]                                   ;Dlambda/lambda
sans_supl[64]=0.                                                 ;default Mode
sans_supl[71]=1.                                                 ;default attenuation given by the instrument

if idxp[16] ne -1 then if   idxp[16] lt n_elements(datp.p)   $   ;XY_det_resolution                   (back compatible)
                  then begin
                       resoXY = [datp.p[idxp[15]],datp.p[idxp[16]]]
                       if (sans_supl[8] ne resoXY[0]) $
                       or (sans_supl[9] ne resoXY[1]) $
                       then begin                                ;new resolution -> new Mask + new center
                         if (sans_supl[66] le 1) or (not keyword_set(isTRANS)) then begin ; if no panels or not a transm file
                           sans_masq = sans_inst_init('Mask' , sans_supl[5] , detres=resoXY)
                           sans_ctr[0:1]  = sans_ctr[0:1] * (sans_supl[8:9] / resoXY)
                           sans_supl[8:9] = resoXY
                           if sans_supl[79] eq 0 then begin  ;GUI mode
                             widget_control,base0,get_uvalue=baset
                             sans_settings, refresh=baset
                           endif
                         endif ;instrument without panels
                       endif ;resolution changed
                       endif
if idxp[17] ne -1 then if   idxp[17] lt n_elements(datp.p)   $
                  then sans_supl[64]  = datp.p[idxp[17]]         ;Mode 1:Tof  3:Kin  4:TOFmerged      (back compatible)

if idxp[18] ne -1 then if   idxp[18] lt n_elements(datp.p)   $
                  then sans_supl[71]  = datp.p[idxp[18]] > 1     ;Attenuation given by the instrument (back compatible)

if idxp[19] ne -1 then if   idxp[19] lt n_elements(datp.p)   $   ;DeadTime (Tau)                      (back compatible)
                  then if n_elements(wi) eq 1.           $
                  then if strpos(strlowcase(par_txt[wi,idxp[19]]),'deadtime') ge 0 $
                  then if datp.p[idxp[19]]   gt 1.           $
                  then sans_supl[6]  = datp.p[idxp[19]]*1e-9     ;get parameter value of Tau (ns -> s)
end

pro sans_settings_WNI, datp , set=swi , get=numor , clear=clear
;+
;** **************
;** **************
;** retrieve a lost numor among workspaces (avoid reread operation)
;-

common c_sans_settings
common c_work_numo ,work_numo  ;0=numor  1=workspace  2=frame index  (0,*)=-# of valid entries

idxp=sans_supl[80:*] & nwn=(size(work_numo))[1] & wo=-work_numo[0]

if keyword_set(clear) then work_numo[*]=0

if n_elements(swi) eq 1 then begin ;swi=workspace            ***PUT OR/AND REPLACE ENTRIES USING SWI***
   idx=where( work_numo[*,1] ne swi) & nidx=n_elements(idx)
   if (idx[0] ge 0) and (nidx lt nwn) then begin & work_numo=[work_numo[idx,*],lonarr(nwn-nidx,3)] & work_numo[0,*]=-((wo-(nwn-nidx))>0) & endif
   if n_elements(datp) gt 0 then if n_tags(datp) gt 1 then begin
      wo=-work_numo[0]
      ni=(size(datp.pv))[0] & np=n_elements(datp.p)-1
      if ni eq 2 then ni=datp.pv[idxp[6]<np,*] else ni=datp.p[idxp[6]<np] & nn=n_elements(ni) ;get the numor(s) may be frames
      if wo+nn ge nwn then work_numo=[work_numo,lonarr(wo+nn-nwn+1000,3)]
      work_numo[wo+1:wo+nn,0]=ni & work_numo[wo+1:wo+nn,1]=swi & work_numo[wo+1:wo+nn,2]=lindgen(nn)  ;insert
      work_numo[0,*]=-(wo+nn)
   endif
endif

if n_elements(numor) eq 1 then begin ;datp=out_workspace , numor=run   ***TRANSFER WORKSPACE IN NEW WORKSPACE***
   idx=where(work_numo[*,0] eq numor) & idx=idx[0]
   if idx ge 0 then begin
      wn=datp                & wi=work_numo[idx,1] & ids=work_numo[idx,2]
      TAKE_W ,data ,W=wi     & TAKE_DATP,dp ,W=wi  & sz =size(data)
      if (sz[0] ge 2) then if ((ids eq 0) or (ids lt sz[3])) then begin
         data=data[*,*,ids]  & ni=(size(dp.pv))[0]
         if ni eq 2 then begin & dp.w_tit='' & p=dp.pv[*,ids] & MOD_DATP,dp,'P',p & MOD_DATP,dp,'PV',p & MOD_DATP,dp,'N',dp.N[0] & endif
         GIVE_W ,data ,W=wn  & GIVE_DATP,dp ,W=wn
      endif else datp=0 else datp=0
   endif else datp=0
endif
end

pro sans_settings_current, settings ,baset, setrun=setrun, nowind=nowind ,memswap=memswap
;+
;** *********************
;** *********************
;** Coming from sans_settings_read and sans_settings_write
;** to update all lamp workspaces
;-

common c_sans_settings
common c_sans_cur_run

sans_supl  =settings.SUPL

;sans_supl(0)= float(settings.WAVEL)    ;SANS_SUPL should already be ok !!!
;sans_supl(1)= float(settings.DISTANCE)
;sans_supl(2)= float(settings.OFFSET)
;sans_supl(3)= float(settings.ANGLE)
;sans_supl(4)= float(settings.COLVAL)
;sans_supl(5)=       settings.IDXI

sans_trans  = float(settings.TRANSMISSION)
sans_ctr    =[float(settings.XCENTER),float(settings.YCENTER)]
sans_beam   = float(settings.XYFBEAM)

masq        =       settings.MASK
    ;cvh=cur_fct(2) ;128
    ;sf1=long(round(cvh/settings.FACTOR(0))) & sf2=long(round(cvh/settings.FACTOR(1)))
    ;sans_masq  =[1,1,sf1,sf2]
sans_masq   =[1,1,settings.DIMS] & sizm=size(masq) & sizm1=sizm[1]
tmp=lonarr(4)
for i=0,sizm1-1 do if masq[i] gt ' ' then begin
                      txt=strmid(strtrim(masq[i],2),1,20) & on_ioerror,mismasq & reads,txt,tmp
                      sans_masq=[[sans_masq],[tmp]] & mismasq: & endif
on_ioerror,misetcur

;COMING ONLY FROM SANS_SETTINGS_READ to fill all workspaces
if keyword_set(setrun) then begin
   if (not keyword_set(memswap)) then begin & for i=0,5 do XICUTE,(['W31','W32','W33','W34','W35','W36'])[i]+'=0'
                                            sans_settings_WNI, /clear  &  endif
   tags=tag_names(settings)
   for i=0,n_elements(tags)-1 do case tags[i] of
   'DATAW':       begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                        if n_elements( settings.dataw.data)  gt 0 then begin & wi=settings.dataw.wi  & par=settings.dataw.par[0]
                             GIVE_W   ,settings.dataw.data ,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings Water'
                             GIVE_DATP,settings.dataw.datp ,W=wi  & sans_settings_WNI, settings.dataw.datp,  set=wi
                             endif & end
   'DATAWE':      begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                        if n_elements( settings.datawE.data) gt 0 then begin & wi=settings.datawE.wi & par=settings.datawE.par[0]
                             GIVE_W   ,settings.datawE.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings Water cell'
                             GIVE_DATP,settings.datawE.datp,W=wi  & sans_settings_WNI, settings.datawE.datp, set=wi
                             endif & end
   'DATACW':      begin;if (not keyword_set(memswap)) then $ ;commented to enable many WATER
                        if n_elements( settings.datacw.data) gt 0 then begin & wi=settings.datacw.wi & par=settings.datacw.par[0]
                             GIVE_W   ,settings.datacw.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings Cadmium W'
                             GIVE_DATP,settings.datacw.datp,W=wi  & sans_settings_WNI, settings.datacw.datp, set=wi
                             endif  & end
   'DATAS':       begin & if n_elements( settings.datas.data)  gt 0 then begin & wi=settings.datas.wi  & par=settings.datas.par[0]
                             GIVE_W   ,settings.datas.data ,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings Sample'
                             GIVE_DATP,settings.datas.datp ,W=wi  & sans_settings_WNI, settings.datas.datp,  set=wi
                            endif & end
   'DATASE':      begin & if n_elements( settings.datasE.data) gt 0 then begin & wi=settings.datasE.wi & par=settings.datasE.par[0]
                             GIVE_W   ,settings.datasE.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"       ;sans_read_settings Sample cell'
                             GIVE_DATP,settings.datasE.datp,W=wi  & sans_settings_WNI, settings.datasE.datp, set=wi
                         ;if not keyword_set(nowind) then $
                         ;   sans_settings_runs,baset,[9,9 ,0,baset.bases3(3)],/config ; ??? to put at the end of sans_settings_READ ???
                             endif & end
   'DATACS':      begin & if n_elements( settings.datacs.data) gt 0 then begin & wi=settings.datacs.wi & par=settings.datacs.par[0]
                             GIVE_W   ,settings.datacs.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings Cadmium S'
                             GIVE_DATP,settings.datacs.datp,W=wi  & sans_settings_WNI, settings.datacs.datp, set=wi
                             endif  & end

   'TRFLUX':      begin & if n_elements( settings.trflux.data) gt 0 then begin & wi=settings.trflux.wi & par=settings.trflux.par[0]
                             GIVE_W   ,settings.trflux.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'" ;sans_read_settings alternative FLUX'
                             GIVE_DATP,settings.trflux.datp,W=wi  & sans_settings_WNI, settings.trflux.datp, set=wi
                             endif  & end

   'HISTORY':     begin & if (not keyword_set(memswap)) then begin
                        if n_elements( settings.history.trwatr.data)   gt 1 then begin & wi=settings.history.trwatr.wi   & par=settings.history.trwatr.par[0]
                             GIVE_W   ,settings.history.trwatr.data , W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TR WATER'
                             GIVE_DATP,settings.history.trwatr.datp , W=wi  & sans_settings_WNI, settings.history.trwatr.datp,   set=wi
                             endif
                        if n_elements( settings.history.trwatrE.data)  gt 1 then begin & wi=settings.history.trwatrE.wi  & par=settings.history.trwatrE.par[0]
                             GIVE_W   ,settings.history.trwatrE.data, W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TR WATER cell'
                             GIVE_DATP,settings.history.trwatrE.datp, W=wi  & sans_settings_WNI, settings.history.trwatrE.datp,  set=wi
                             endif
                        if n_elements( settings.history.trsampl.data)  gt 1 then begin & wi=settings.history.trsampl.wi  & par=settings.history.trsampl.par[0]
                             GIVE_W   ,settings.history.trsampl.data, W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TR SAMPLE'
                             GIVE_DATP,settings.history.trsampl.datp, W=wi  & sans_settings_WNI, settings.history.trsampl.datp,  set=wi
                             endif
                        if n_elements( settings.history.trsamplE.data) gt 1 then begin & wi=settings.history.trsamplE.wi & par=settings.history.trsamplE.par[0]
                             GIVE_W   ,settings.history.trsamplE.data,W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TR SAMPLE cell'
                             GIVE_DATP,settings.history.trsamplE.datp,W=wi  & sans_settings_WNI, settings.history.trsamplE.datp, set=wi
                             endif
                        if n_elements( settings.history.trbeamw.data)  gt 1 then begin & wi=settings.history.trbeamw.wi  & par=settings.history.trbeamw.par[0]
                             GIVE_W   ,settings.history.trbeamw.data, W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TR BEAM water'
                             GIVE_DATP,settings.history.trbeamw.datp, W=wi  & sans_settings_WNI, settings.history.trbeamw.datp,  set=wi
                             endif
                        endif

                        if n_elements( settings.history.mksampl.data)  gt 1 then begin & wi=settings.history.mksampl.wi  & par=settings.history.mksampl.par[0]
                             GIVE_W   ,settings.history.mksampl.data, W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings DATA used for mask'
                             GIVE_DATP,settings.history.mksampl.datp, W=wi  & endif
                        if n_elements( settings.history.mkwatr.data)   gt 1 then begin & wi=settings.history.mkwatr.wi   & par=settings.history.mkwatr.par[0]
                             GIVE_W   ,settings.history.mkwatr.data,  W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings WATER used for mask'
                             GIVE_DATP,settings.history.mkwatr.datp,  W=wi  & endif

                        if (not keyword_set(memswap)) or (settings.CONFUS.T eq 'Trs') then $
                        if n_elements( settings.history.trbeams.data)  gt 1 then begin & wi=settings.history.trbeams.wi  & par=settings.history.trbeams.par[0]
                             GIVE_W   ,settings.history.trbeams.data, W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings TRS or CENTER BEAM sample '
                             GIVE_DATP,settings.history.trbeams.datp, W=wi  & sans_settings_WNI, settings.history.trbeams.datp,  set=wi
                             endif
                  end
   'CONFUS':      begin & if (not keyword_set(memswap)) or (settings.CONFUS.T eq 'Ctr') then $
                        if n_elements( settings.CONFUS.L.data)         gt 1 then begin & wi=settings.CONFUS.L.wi         & par=settings.CONFUS.L.par[0]
                             GIVE_W   ,settings.CONFUS.L.data,        W=wi  & XICUTE,"PAR"+strtrim(wi,2)+"='"+par+"'"  ;sans_read_settings ' Beam Center Ctr or Trs
                             GIVE_DATP,settings.CONFUS.L.datp,        W=wi  & sans_settings_WNI, settings.CONFUS.L.datp, set=wi
                             endif
                  end
   else:
   endcase
endif
misetcur:
end

pro sans_settings ,instrument=inst, refresh=baset ,none=none ,param=param ,nowind=nowind, map=mappi, baseSANS=baseSANS, $
                   Options=options, InstList=InstList, InstIsOn=InstIsOn, SansRef=SansRef
;+
;** ************* 
;** *************
;** sans_setting GUI creation
;** and update after certain events
;-

@lamp.cbk
forward_function sans_inst_init, int_loadRequest

common c_sans_settings, sans_inst_T, sans_masq, sans_beam, sans_ctr, sans_trans, sans_supl
common c_sans_tofl    , TofFlux, LambdaVq
common c_sans_cur_run
common c_sans_to_ben  , sansBEN
common c_sans_base0   , base0
common c_work_numo

version =' V4.7 (Dec 2015)'

if keyword_set(none) then return ;Just compile
fresh=0 & sheet=0
if n_elements(baset) ge 1 then begin & fresh=1 & mappi=-1 & endif ;Refresh gui with correct values
if n_elements(mappi) ne 1 then mappi=1 else if mappi eq 0 then sheet=1 ;mappi=0 means start sans_settings in background for SANS_SHEET
kpwin=!D.window

InstList = sans_inst_init('List1') ;Instrument names in Menu
InstLamp = sans_inst_init('List2') ;Real instrument names
InstIsOn = sans_inst_init('LstOn') ;1=On
SansRef  = sans_inst_init('LstSR') ;0=reflectometry 1=sans 2=scan

if n_elements(inst) ne 1 then if (not fresh) and  (n_elements(inst_value) eq 1) then begin
                              idx=where(InstLamp eq strupcase(inst_value)) & idx=idx[0]>0
                              inst=InstList[idx]                           & endif

if (xregistered('sans_settings_win',/noshow) eq 0) or (fresh) or keyword_set(nowind) then begin

  if fresh then idx=sans_supl[5] else idx=0
  instru  =InstList[idx]
  instrul =InstLamp[idx]
  if n_elements(inst) eq 1 then begin &  idx=where(InstList eq strupcase(inst))       & idx=idx[0]>0
                                       instru=InstList[idx] & instrul=InstLamp[idx] & endif
  idxi=idx
  if lamp_siz le 900 then sizpref=[256.,384.] else sizpref=[384.,256.] ;****************************** plot size 256<->384
                                              ; For_BEN(fct dep.)  Base_fct  draw_pref    base_size_var  cur_data_fct(#baset.factor)
  if n_elements(cur_fct)   le 1 then cur_fct  =[1.,1.             ,128.     ,sizpref     ,0,0,0,0       ,0.,0.       ]
                                               cvh=cur_fct[2] & dcs=cur_fct[3]

  if n_elements(sans_supl) le 1 then sans_supl=fltarr(100)
  if n_elements(work_numo) le 1 then work_numo=lonarr(5000,3)  ;0=numor  1=workspace  2=frame index  (0,*)=# of valid entries
  newi=(sans_supl[5] ne idxi) ;We may change instrument !!!
  
  if (not fresh) and (newi) then       sans_masq =0
  if (not fresh) or  (newi) then       if strlowcase(instrul) eq strlowcase(inst_value) then rdset,/raw else rdset,inst=instrul,/raw
  
  if (not fresh)            then begin & sans_beam=0 & sans_ctr=0 & sans_trans=0 & sans_supl[*]=0 & sans_masq=0 & endif
  if (not fresh)            then begin & TofFlux={bid:0}          & LambdaVq=0   & endif
  if     (fresh) then if idxi ne baset.idxi then begin & rdset,inst=instrul & baset.idxi=idxi     & endif

  masq=[''] ;Get default mask for instrument or get currently defined mask
  if n_elements(sans_masq) lt 4 then SANS_MASK,instrument=instru,/noplot
  if n_elements(sans_masq) gt 4 then begin & masqt=strtrim(string(long(sans_masq)),2)
     for i=1,(size(masqt))[2]-1 do masq=[masq,'['+masqt[0,i]+','+masqt[1,i]+','+masqt[2,i]+','+masqt[3,i]+']']
     masq=masq[1:n_elements(masq)-1] & endif
  if fresh then widget_control,baset.bases2[4],set_value=masq

 ;Default variables
  offset  =' 0.0 ' & angle=' 0.0 ' & beamx=' 0.0' & beamy=' 0.0'  & wavl='  .   '  & dista  ='  .   ' & Sphi0='00000000'
  colidx  =1  & transm=['1.00 ','1.00 ','1.00 ','1.00 ']          & attenua='1'    & fscal  =0        & Wphi0='    0'
  thinW   =.1 & thinS =.1 & crosS=1. & beambx=' ' & distW=dista   & vcoll=dista    & Wcoll  =dista
  collima ='1.' & centex=1. & centey=1. & Xdet=1. & Ydet=1.       & crosW=1.       & UnScal =0
  forceRead  =0 ;1 re-read the data each time
  beammod    =1 ;1 XYrange , 0 XYsize
  useCfit    =1 ;show/hide gauss fit for center

  fortau=['    no need for that detector','    I=I/(1.-tau*Itotal/time)','    I=I/exp(-tau*Itotal/time)',$
          '    Ivtub=Ivtub/(1.-tau*Itotalvtub/time)','    Ihtub=Ihtub/(1.-tau*Itotalhtub/time)']

  Qdep  =''        ;Qdep formula should contain 'Q' and/or 'DIST'(m) and/or 'THETHA' and/or 'RADIUS'(cm) and/or 'WAVE'
  XWdep =''        ;cross section formula        ********************************************************************
  txtWater='Water' ;or 'Vanad' or 'Plexi'
  txtCadm ='Cadm'  ;or 'B4C '  or  ?
  Mon_Tim =0       ;0 use monitors , 1 use time (default) for transm corrections
  disQdep =0       ;1 disable Qdep
  SAc     =1       ;1 Solide angle   correction (default)
  THc     =1       ;1 Thickness F(Q) correction (default)
  Raxe    =0.      ;rotation axe of detector (cm)
  NegMask =0       ;>=1 means nb of panels and negative counts must be masked (inter panels)
  xyUnit  =0       ;1 XYaxis=2theta at read

  bfit       =1 ;default gaussfit for center , 0 centroid
  ofa        =1 ;0:disable offset , 1:enable offset detector , 2:enable offset distance
  beamf      =1 ;1 rectangular , 2 elipsoid
  tau        =0.;dead time factor
  ftau       =1 ;fortau function index [None , I/(1-tau*tot/time) , I/exp(-tau*tot/time) , by tubes]
  FX         =0 ;0 first index is (1) , -1 first index is (0)
  attenuF    =0 ;attenuation factor versus collimation (1. for collima=8.m)
  FCT=[1.,1.]   ;128./[nX,nY] image detector

         ;0         1         2       3           4          5           6      7           8
         ;wavel   , distance, offset, angle     , collim   , duration  , numor, attenuator, in/out
  paridx=[15      , 9       , 19    , 18        , 17       , 2         , 26   , 27        , 28,$
          24      , 22      , 10    , 16        , 6        , 5         , -1   , -1        , -1  , -1  , -1]
         ;S height, deg S   , temp  , wavlSpread, slitWidth, slitHeight, Xdet , Ydet      , Mode, att2  Tau
         ;9         10        11      12          13         14          15     16        , 17    18  , 19

  StrInit = sans_inst_init('Inits', idxi, detres=sans_supl[8:9]) ;Get instrument dependent variables

  if n_tags(StrInit) gt 1 then begin
     collima  = StrInit.collima  & attenua = StrInit.attenua & Qdep    = StrInit.Qdep
     fct      = StrInit.fct      & centex  = StrInit.centex  & centey  = StrInit.centey
     FX       = StrInit.FX       & beamf   = StrInit.beamf   & Xdet    = StrInit.Xdet
     Ydet     = StrInit.Ydet     & tau     = StrInit.tau     & ftau    = StrInit.ftau
     ofa      = StrInit.ofa      & bfit    = StrInit.bfit    & paridx  = StrInit.paridx
     txtWater = StrInit.txtWater & Raxe    = StrInit.Raxe    & xyUnit  = StrInit.xyUnit
     Sac      = StrInit.Sac      & THc     = StrInit.THc     & Mon_Tim = StrInit.Mon_Tim
     XWdep    = StrInit.XWdep    & disQdep = StrInit.disQdep & attenuF = StrInit.attenuF
     NegMask  = StrInit.NegMask
  endif

 ;Keep instrument tables in global SANS_INST_T for PROX or Web Client
  sans_inst_T = {collim:collima, attenu:attenua, attenuF:attenuF, Qdep:Qdep, XWdep:XWdep}

 ;Options structure should come from a PROX or Web Client
  if n_tags(Options) ge 1 then begin
     Otags = tag_names(Options)
     for i = 0,n_elements(Otags)-1 do $
       case Otags[i] of
       'USEMONITOR':       begin & if strupcase(Options.UseMonitor) eq 'YES' then Mon_Tim  = 0 else Mon_Tim  = 1
                           end
       'WATERXSECTIONVAL': begin & if Options.WaterXsectionVal le 0 then fscal = 0  else fscal = 1
                                   if Options.WaterXsectionVal le 0 then crosW = 1. else crosW = Options.WaterXsectionVal
                           end
       'ABSSCALE':         begin   if strupcase(Options.absScale)   eq 'YES' then unScal   = 0 else unScal   = 1
                           end
       'REREAD':           begin   if strupcase(Options.reRead)     eq 'YES' then forceRead= 1 else forceRead= 0
                           end
       'QDEP':             begin   if strupcase(Options.Qdep)       eq 'YES' then disQdep  = 0 else disQdep  = 1
                           end
       else:
       endcase
  endif

;RD:read  **:fixed parameters  $$:changeable values  ==:calculated values
; -----------------------------------------------------------------------
;RD sans_supl(0)  Wavelength
;RD sans_supl(1)  Distance sample->detector sample
;RD sans_supl(2)  Offset detector (mm) sample
;RD sans_supl(3)  Angle detector (deg) sample
;RD sans_supl(4)  Collimation sample
;** sans_supl(5)  Instrument index
;$$ sans_supl(6)  TAU factor
;$$ sans_supl(7)  TAU function to be used
;** sans_supl(8)  X pixel size in Cm (may be RD)
;** sans_supl(9)  Y pixel size in Cm (may be RD)

;$$ sans_supl(10) WATER  thickness if >0.  and trans(0,1)
;$$ sans_supl(11) SAMPLE thickness if >0.  and trans(2,1) and trans(4:*,1)
;== sans_supl(12) SAMPLE Tr error          and trans(2,2) and trans(4:*,2)

;$$ sans_supl(13)   Sectors info->
;$$ sans_supl(19) <-Sectors info
;$$ sans_supl(20) Radial Increment
;$$ sans_supl(21) Integration mode

;RD sans_supl(22) Numor
;RD sans_supl(23) Duration
;RD sans_supl(24) Beam attenuation index
;RD sans_supl(25) Beam att on/off
;== sans_supl(26) Attenuator factor
;$$ sans_supl(27) enable or disable dead_time correction
;$$ sans_supl(28) enable or disable solid angle correction
;$$ sans_supl(29) 1=provided water cross section in (42)
;$$ sans_supl(30) FX
;$$ sans_supl(42) WATER scattering cross section input user
;$$ sans_supl(43) enable or disable thickness as f(Q)
;$$ sans_supl(44) 1=un_scale in absolute scaling (but correct from efficiencies, not from thickness)
;** sans_supl(45) rotation axe (cm)

;RD sans_supl(32) Water distance
;RD sans_supl(33) Water duration ****(unused as sample duration(23)-> taken in datp)****
;RD sans_supl(34) Water collimation

;== sans_supl(31) Phi_0 sample (N/Sec) corrected, masked
;RD sans_supl(36) Beam sample distance
;RD sans_supl(37) Beam sample duration
;RD sans_supl(38) Beam sample collimation

;== sans_supl(35) Phi_0 water  (N/Sec) corrected, masked
;RD sans_supl(39) Beam water  distance
;RD sans_supl(40) Beam water  duration
;RD sans_supl(41) Beam water  collimation

;RD sans_supl(46) Sample rotation
;RD sans_supl(47) Sample Temperature
;$$ sans_supl(48) UseTime=1 , UseMonitor=0
;== sans_supl(49) Delta_theta
;RD sans_supl(50) Delta_lambda/Lambda
;RD sans_supl(51) Offset detector (mm) water
;** sans_supl(52) Unit of X,Y axis
;$$ sans_supl(53) Re-read the data each time
;** sans_supl(54) X center of detector
;** sans_supl(55) Y center of detector
;== sans_supl(56) Water X center (calculated at beam water read)
;== sans_supl(57) Water Y center (calculated at beam water read)
;== sans_supl(58) =1 if water can't be used for correction (set in sans_cor)
;RD sans_supl(59) Water Angle detector (deg)
;== sans_supl(60) (alternative Phi_0 sample supl[31])
;== sans_supl(61) =1 disable Qdep correction
;RD sans_supl(62) Beam water  monitor related to sans_supl(40) in sans_settings_runs
;RD sans_supl(63) Beam sample monitor related to sans_supl(37) in sans_settings_runs
;RD sans_supl(64) Mode 1:Tof  3:Kin  4:TOFmerged
;** sans_supl(65) 0:disable offset  1:enable offset detector  2:enable offset distance
;** sans_supl(66) >=1 means at read negative counts must be masked (inter panels)
;$$ sans_supl(67) 2:low resolution panels  1:high resolution
;$$ sans_supl(68) 1:enable all panels conf 1
;$$ sans_supl(69) 1:enable all panels conf 2
;$$ sans_supl(70) 1:enable all panels conf 3
;$$ sans_supl(71) Attenuator factor comming from data parameters

;$$ sans_supl(78) 0 1 2 3 curLMH config in batch mod (defined in sans_center)
;$$ sans_supl(79) 0:GUI mode  1:Batch mode (/nowind)

 ;         0      1      2      3      4    5     6      7      8    9    10    11    12  13              14  15      16    17    18   19       20     21      22   23
 ;Center options  + toggles           ;fit ,grav ,Cmask, Tmask, log, d_t, s_a,  scal, Err Q_Qlog_Q^2_Q^4  efq unScale Pstyl Fincr LRP  M/T      reRead Squared Rock FREE
 ;bases6 =[basXC1,basXC2,basYC1,basYC2,bfit,1-bfit,   1,     1,   0,   1, SAc, fscal,  0,  0              ,1 ,unScal  ,1    ,0    ,1  ,Mon_Tim ,0     ,0      ,0   ,0,0,0,0,0,0,0,0]

  if not fresh then begin & sans_supl[5 : 9]=[idxi,Tau,Ftau,Xdet,Ydet]
;**                                         26 27 28  29    30 31 32 33 34 35 36 37 38 39 40 41 42    43  44      45   46 47 48
                          sans_supl[26:71]=[1 ,1 ,SAc,fscal,FX,0 ,1 ,1 ,1 ,0 ,.1,.1,.1,.1,.1,.1,crosW,THc,unScal ,Raxe,0 ,0 ,Mon_Tim,$

;**                                         49     50  51 52     53        54     55      56 57 58 59 60 61       62 63 64 65
                                            7.5e-4,0.1,0.,xyUnit,forceRead,centeX,centeY, 0, 0 ,0 ,0 ,0 ,disQdep ,0 ,0 ,0, ofa, $

;**                                         66       67 68 69 70 71
                                            NegMask ,0 ,0 ,0 ,0  ,1]

                          cur_fct[0:1]=round([cvh,cvh]/fct)*[Xdet,Ydet] ;FOR BEN when constant dimensions, otherwise see sans_settings_mask
                          if cur_fct[0] gt cur_fct[1] then cur_fct[0:1]=[cur_fct[0]/cur_fct[1],1.] else $
                          if cur_fct[0] lt cur_fct[1] then cur_fct[0:1]=[1.,cur_fct[1]/cur_fct[0]] else cur_fct[0:1]=[1.,1.]

                          if NegMask gt 1 then RDSET, panels=[1,lonarr(NegMask-1)] ;Use only first Panel (central) as  default
                          RDSET, step  = sans_supl[67]        ;Low resolution as default when many panels

  endif        else begin & FX=sans_supl[30] ;Force values even from saved files   ;never happen for batch mode
                          if paridx[15] eq -1 then sans_supl[8 : 9]=[Xdet,Ydet]
                          sans_supl[45]   =Raxe
                          sans_supl[54]   =centeX
                          sans_supl[55]   =centeY
                          sans_supl[65]   =ofa
                          sans_supl[66]   =NegMask
                          baset.Qdep      =Qdep
                          RDSET, step = sans_supl[67]
  endelse
  sans_supl[80:99]=paridx

  if sans_supl[10] le 0  then sans_supl[10]=thinW
  if sans_supl[11] le 0  then sans_supl[11]=thinS
  if sans_supl[12] le 0  then sans_supl[12]=crosS
  if sans_supl[50] le 0  then sans_supl[50]=0.1
  if keyword_set(nowind) then sans_supl[79]=1 else sans_supl[79]=0 ;1 is batch mode

  txtW=' '+strmid(txtWater,0,1)
  txt1water='  '+txtwater & txt2water=txt1water+'  cell'
  if FX eq 0 then begin & fxrg=' Xrange[1,n]' & fxln='Xlines (1,2,n)  '  & fxtog='Start index :1'
  endif      else begin & fxrg=' Xrange[0,n]' & fxln='Xlines (0,1,n)  '  & fxtog='Start index :0' & endelse
  
  collim=strsplit(/EXTRACT,collima,',')

 ;Get currently defined center
  if n_elements(sans_ctr  ) ge 2 then begin & xy0=sans_ctr+FX
                                            if xy0[0] lt 0 then xy0[0]=xy0[0]-2*FX
                                            if xy0[1] lt 0 then xy0[1]=xy0[1]-2*FX
                                            centex=strtrim(string(xy0[0] ,format='(f9.3)'),2)
                                            centey=strtrim(string(xy0[1] ,format='(f9.3)'),2)
                                            if fresh then widget_control,baset.bases2[2],set_value=centex
                                            if fresh then widget_control,baset.bases2[3],set_value=centey
                                 endif else sans_ctr=[float(centex),float(centey)]-FX
 ;Get currently defined beam size
  if n_elements(sans_beam ) ge 7 then begin & beamx =strtrim(string(sans_beam[0],format='(f5.1)'),2)
                                            beamy =strtrim(string(sans_beam[1],format='(f5.1)'),2)
                                            beamf =long(sans_beam[2])
                                            if n_elements(sans_beam ) lt 9 then sans_beam=[sans_beam,0,0]
                                            
                                            if fresh then begin ;never happen for batch mode
                                               widget_control,baset.bases2[8],set_value=beamx
                                               widget_control,baset.bases2[9],set_value=beamy
                                               if baset.bases8[1] ne 0 then begin
                                                  widget_control,baset.bases8[0],set_value=beamx
                                                  widget_control,baset.bases8[1],set_value=beamy
                                               endif else begin
                                                  if (sans_beam[7] eq 0) and (sans_beam[8] eq 0) then $
                                                      sans_beam[7:8]=round([sans_ctr[0:1]-sans_beam[0:1]/2.])
                                                  if (sans_beam[0] gt 0) and (sans_beam[1] gt 0) then begin
                                                      beambx=strtrim(long([sans_beam[7], $
                                                             sans_beam[7]+sans_beam[0]-1,sans_beam[8], $
                                                             sans_beam[8]+sans_beam[1]-1]+FX),2)
                                                      beambx=beambx[0]+','+beambx[1]+' , '+beambx[2]+','+beambx[3]
                                                  endif
                                                  widget_control,baset.bases8[0],set_value=beambx
                                               endelse
                                            endif ;fresh
                                 endif else sans_beam=[float(beamx),float(beamy),beamf,0,0,0,0 ,0,0]

 ;Get currently defined transmission
  trs5=5
  if n_elements(sans_trans) ge 4 then begin & transm=string(sans_trans[0:3,0],format='(f5.3)')
  
                                            if fresh then begin ;never happen for batch mode
                                             widget_control,baset.bases2[11],set_value =transm[0]+txt1water
                                             widget_control,baset.bases2[12],set_value =transm[1]+txt2water
                                             widget_control,baset.bases2[13],set_value =transm[2]+'  Sample 1..'
                                             widget_control,baset.bases2[14],set_value =transm[3]+'  Sample cell'
                                             widget_control,baset.bases2[15],set_value =transm[0]+txt1water
                                             widget_control,baset.bases2[16],set_value =transm[1]+txt2water
                                             widget_control,baset.bases2[17],set_value =transm[2]+'  Sample 1..'
                                             widget_control,baset.bases2[18],set_value =transm[3]+'  Sample cell'
                                            endif ;fresh
                                            
                                            trsize=size(sans_trans) ;for compatibility with old saved files
                                            if trsize[2] lt trs5 then sans_trans=[[sans_trans],[fltarr(trsize[1],trs5-trsize[2])]]
                                 endif else begin & sans_trans=fltarr(5,trs5) & sans_trans[*,0]=1. & sans_trans[0,1]=sans_supl[10]
                                                                              sans_trans[[2,4],1]=sans_supl[11] & endelse

 ;Get currenty defined wavelength,distance,offset,angle,collimation
  if n_elements(sans_supl ) gt 1 then begin & wavl  =strtrim(string(sans_supl[0] ,format='(f6.3)'),2)
                                            dista =strtrim(string(sans_supl[1] ,format='(f7.4)'),2)
                                            offset=strtrim(string(sans_supl[2] ,format='(f7.3)'),2)
                                            angle =strtrim(string(sans_supl[3] ,format='(f6.2)'),2)
                                            colidx=where(round(float(collim)*10.)/10. eq round(sans_supl[4]*10.)/10.)>0
                                            colidx=colidx[0]
                                            vcoll =strtrim(string(sans_supl[4] ,format='(f7.4)'),2)
                                            Wcoll =strtrim(string(sans_supl[34],format='(f7.4)'),2)
                                            distW =strtrim(string(sans_supl[32],format='(f7.4)'),2)
                                            thinW =strtrim(string(sans_supl[10],format='(f5.3)'),2)
                                            thinS =strtrim(string(sans_supl[11],format='(f5.3)'),2) & if sans_supl[11] ge 10. then thinS =strtrim(string(sans_supl[11],format='(f5.2)'),2)
                                            crosS =strtrim(string(sans_supl[12],format='(f6.3)'),2)
                                            crosW =strtrim(string(sans_supl[42],format='(f6.3)'),2)
                                            if strpos(thinW,'0') eq 0 then thinW=strmid(thinW,1,10)
                                            if strpos(thinS,'0') eq 0 then thinS=strmid(thinS,1,10)
                                            if strpos(crosS,'0') eq 0 then crosS=strmid(crosS,1,10)
                                            if strpos(crosW,'0') eq 0 then crosW=strmid(crosW,1,10)

                                            secTR =strtrim(string(sans_supl[13],format='(f4.1)'),2)
                                            secSR =strtrim(string(sans_supl[14],format='(f4.1)'),2)
                                            secOP =strtrim(string(sans_supl[15],format='(f4.1)'),2)
                                            secNS =strtrim(string(fix(sans_supl[16])>1<360),2)
                                            secIR =strtrim(string(round(sans_supl[17])),2) ;*cur_fct[0]!
                                            secOR =strtrim(string(round(sans_supl[18])),2) ;*cur_fct[0]!
                                            secSP =strtrim(string(fix(sans_supl[19])>0<1  ),2)

                                            if sans_supl[20] le 0 then sans_supl[20]=1.
                                               sans_supl[20]=sans_supl[20]>.5
                                            radin =strtrim(string(sans_supl[20],format='(f4.1)'),2)

                                            if sans_supl[26] le 0 then sans_supl[26]=1.
                                            if sans_supl[26] lt 1 then sans_supl[26]=round(1./sans_supl[26])
                                            Sphi0=strtrim(string(sans_supl[31],format='(I10)'),2)
                                            Wphi0=strtrim(string(sans_supl[35],format='(I10)'),2)

                                            strput,fortau,'    ',0 & ftau=sans_supl[7]
                                            tmp=fortau[ftau] & strput,tmp,'==> ',0 & fortau[ftau]=tmp
                                            strTau=string(sans_supl[6] ,format='(E9.2)')

                                            if fresh then begin ;never happen for batch mode
                                               widget_control,baset.bases7[4] ,set_value =thinW
                                               widget_control,baset.bases2[20],set_value =thinW
                                               widget_control,baset.bases7[5] ,set_value =thinS
                                               widget_control,baset.bases2[21],set_value =thinS
                                               widget_control,baset.bases7[6] ,set_value =crosW

                                               widget_control,baset.bases7[17],get_uvalue=btaufor
                                               for i=0,n_elements(fortau)-1 do widget_control,btaufor[i],set_value=fortau[i]

                                               widget_control,baset.bases9[0] ,get_value =fxcur
                                               if ((strpos(fxcur,'0') ge 0) and (FX eq 0 )) or $
                                                  ((strpos(fxcur,'0') lt 0) and (FX eq -1)) then begin
                                                    widget_control,baset.bases9[0] ,set_value =fxrg
                                                    widget_control,baset.bases9[1] ,set_value =fxrg
                                                    widget_control,baset.bases9[2] ,set_value =fxln
                                                    if baset.bases9[3] gt 0L then $
                                                    widget_control,baset.bases9[3] ,set_value =fxtog
                                               endif
                                            endif ; fresh

                                            if keyword_set(param) then begin ;never happen for batch mode
                                               widget_control,baset.bases2[0] ,set_value =wavl
                                               widget_control,baset.bases2[1] ,set_value =dista
                                               widget_control,baset.bases2[6] ,set_value =offset
                                               widget_control,baset.bases2[7] ,set_value =angle
                                               widget_control,baset.bases2[19],set_value =vcoll
                                               widget_control,baset.bases2[22],set_value =Wcoll
                                               widget_control,baset.bases2[23],set_value =distW
                                               widget_control,baset.bases1[5] ,set_list_select=colidx
                                               widget_control,baset.bases7[14],set_value =radin
                                               widget_control,baset.bases7[15],set_value =Wphi0
                                               widget_control,baset.bases7[16],set_value =strTau

                                               if baset.curLMH gt 0 then begin & bconf=([baset.bases2[24:26]])[baset.curLMH-1]
                                                                               widget_control,bconf,set_button=1 & endif

                                               baset.bases6[9] =sans_supl[27] ;Dead Time    ** BE SURE WE HAVE CORRECT BUTTON STATES        **
                                               baset.bases6[10]=sans_supl[28] ;Solid Ang    ** missing supl for 4,5,6,7,8,12,13,16,18,21,22 **
                                               baset.bases6[11]=sans_supl[29] ;Water Xsec   ** 17,20 are only sans_sheet , Qdep not handled **
                                               baset.bases6[14]=sans_supl[43] ;Tr f(2theta)
                                               baset.bases6[15]=sans_supl[44] ;Un_scale
                                               baset.bases6[19]=sans_supl[48] ;Mon<->Tim

                                               widget_control,baset.bases5[17],map =useCfit
                                               if (ofa eq 0) and (Raxe eq 0.) then widget_control,baset.bases5[18],sens=0 $
                                                                              else widget_control,baset.bases5[18],sens=1
                                               widget_control,baset.bases7[18],set_button=baset.bases6[4]  ;Fit center
                                               widget_control,baset.bases7[19],set_button=baset.bases6[5]  ;Gravity center
                                               widget_control,baset.bases7[20],set_button=baset.bases6[6]  ;Excl Mask center
                                               widget_control,baset.bases7[21],set_button=baset.bases6[7]  ;Excl Mask transm
                                               widget_control,baset.bases7[22],set_button=baset.bases6[8]  ;Log
                                               widget_control,baset.bases7[23],set_button=baset.bases6[9]  ;Dead Time
                                               widget_control,baset.bases7[24],set_button=baset.bases6[10] ;Solid Ang
                                               widget_control,baset.bases7[25],set_button=baset.bases6[11] ;Water Xsec
                                               widget_control,baset.bases7[26],set_button=baset.bases6[14] ;Tr f(2theta)
                                               widget_control,baset.bases7[28],set_button=baset.bases6[15] ;Un_scale
                                               if baset.bases6[19] eq 0 then $
                                               widget_control,baset.bases7[30],set_button=1 else $
                                               widget_control,baset.bases7[31],set_button=1                ;Mon<->Tim
                                               
                                               if baset.bases7[32] gt 0 then $
                                               widget_control,baset.bases7[32],set_button=1-sans_supl[61]  ;Qdep
                                               if baset.bases7[33] gt 0 then $
                                               widget_control,baset.bases7[33],set_button=baset.bases6[21] ;Squared sectors
                                              ;widget_control,baset.bases7(34),set_button=baset.bases6(22) ;Rocking sectors
                                               if n_elements(cur_eff) le 1 then begin & cur_menu[8]=0 & endif
                                                 ;widget_control,baset.bases5(20),set_button=0,sens=0 &  & endif

                                               for i=0,3 do widget_control,baset.bases9[i+5],sens=1

                                              ;Sectors: gives to Ben
                                               if secOR eq 0 then begin
                                                  siz0=round([cvh,cvh]/fct)
                                                  if siz0[0] eq siz0[1] then data=dist(siz0[0]) else data=findgen(siz0[0],siz0[1])
                                                  widget_control,bad_id=ii,get_uvalue=varAccess,sansBen
                                                  bid=int_loadRequest(varAccess,0,data,findgen(siz0[0])+1,findgen(siz0[1])+1,0,0,['0','0',' '])
                                               endif else begin
                                                widget_control,baset.bases7[7] ,set_value =secTR,get_uvalu=u_asl
                                                widget_control,baset.bases7[8] ,set_value =secSR,get_uvalu=u_gsl
                                                widget_control,baset.bases7[10],set_value =secNS,get_uvalu=u_zonID
                                                widget_control,baset.bases7[9] ,set_value =secOP,get_uvalu=u_bsl
                                                widget_control,baset.bases7[11],set_value =secIR,get_uvalu=u_r1ID
                                                widget_control,baset.bases7[12],set_value =secOR,get_uvalu=u_r2ID
                                               ;widget_control,baset.bases7(13),set_value =secSP
                                                INT_event,{TOP:sansBen,ID:baset.bases7[7] ,VALUE:long(secTR)},u_asl
                                                INT_event,{TOP:sansBen,ID:baset.bases7[8] ,VALUE:long(secSR)},u_gsl
                                                INT_event,{TOP:sansBen,ID:baset.bases7[10],VALUE:long(secNS)},u_zonID
                                                INT_event,{TOP:sansBen,ID:baset.bases7[9] ,VALUE:long(secOP)},u_bsl
                                                INT_event,{TOP:sansBen,ID:baset.bases7[11],VALUE:long(secIR)},u_r1ID
                                                INT_event,{TOP:sansBen,ID:baset.bases7[12],VALUE:long(secOR)},u_r2ID
                                               endelse
                                            endif ;keyword_set(param)
                                            endif ;sans-supl gt 1 elements
 ;Construct the GUI
  if keyword_set(nowind) then return
 
 ;*********************** 
  if not fresh then begin
 ;***********************
  bens     ; Ensure bens.pro is compiled
  regSheet = sans_settings_RegSheet() ; Is sans_sheet running

  win =((!version.os_family ne 'Windows') and (!version.os ne 'darwin'))
  wins= (!version.os_family ne 'Windows') & if not wins then anal='Treat' else anal='Treat'
  if sys_dep('VERSION') ge 5.6 then oktips=1 else oktips=0
  if mappi ne 0 then att=0 else att=8
  if sys_dep('VERSION') lt 6.1 then $
  base0 = widget_base(  title='Sans settings'+version,/column,resource_name='lamp',MBAR=mbar,rname_mbar='lampben',/tlb_size_event,tlb_frame_att=att)  else $
  base0 = widget_base(  title='Sans settings'+version,/column,resource_name='lamp',MBAR=mbar,rname_mbar='lampben',/tlb_size_event,tlb_frame_att=att,tab_mod=1)

  base01= widget_base(  base0)
  basej1= widget_base(  base01,/column,map=0,uvalue=0)            ;Config layout
  basej2= widget_base(  base01,/column,map=1) ;Others layout

 ;MENU BAR
 ; -------
  binst = widget_button(mbar  ,font=ft_propor,/menu,value='INSTRUM') ; instrument
  IF NOT SHEET THEN BEGIN
   bid  = widget_button(binst ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bidi = widget_button(binst ,font=ft_propor,value='Change Instrument NAME',/menu)
   for i=0,n_elements(InstList)-1 do $
       if InstIsOn[i] then bid=widget_button(bidi,font=ft_propor,value=InstList[i],uvalue=[3,i+1],sens=InstIsOn[i])
   bid  = widget_button(bidi  ,font=ft_propor,value='')     & widget_control,bid,sens=0
   bfxtg= widget_button(bidi  ,font=ft_propor,value= fxtog                         ,uvalue=[17])
   bid  = widget_button(binst ,font=ft_propor,value='---')  & widget_control,bid,sens=0

   mpth = widget_button(binst ,font=ft_propor,value= 'Set  Data path',/menu)
    bpth= widget_button(mpth  ,font=ft_propor,value= 'Set Data path using Explorer',uvalue=[1,2])
    bptl= widget_button(mpth  ,font=ft_propor,value= 'Set Data path using Lamp'    ,uvalue=[1,3])
    bid = widget_button(mpth  ,font=ft_propor,value= '---')  & widget_control,bid,sens=0
    bptw= widget_button(mpth  ,font=ft_propor,value= 'Set Working path'            ,uvalue=[1,5])
    bid = widget_button(mpth  ,font=ft_propor,value= '')     & widget_control,bid,sens=0
   calif= widget_button(mpth  ,font=ft_propor,value= 'Show current CALIBRATION File'   ,uvalue=[99,4])
   calif= widget_button(mpth  ,font=ft_propor,value= 'Set Optional CALIBRATION File'   ,uvalue=[99,3])
    bid = widget_button(mpth  ,font=ft_propor,value= '')     & widget_control,bid,sens=0
   bidwf= widget_button(mpth  ,font=ft_propor,value='Workflow schedule advice',/menu)
    bid = widget_button(bidwf ,font=ft_propor,value='First: select your PATH for data input (above this entry)')
    bid = widget_button(bidwf ,font=ft_propor,value='Then:')
    bid = widget_button(bidwf ,font=ft_propor,value=' ')
    bid1= widget_button(bidwf ,font=ft_propor,value='  Restore a configuration file if you have one.')
    bid = widget_button(bidwf ,font=ft_propor,value=' ')
    bid1= widget_button(bidwf ,font=ft_propor,value='  Follow menu_bar: Center> Transmission> Mask> Samples> Treat')
    bid = widget_button(bidwf ,font=ft_propor,value='   ')
    bid3= widget_button(bidwf ,font=ft_propor,value='  Go forward and backward from menu (most input fields are optional)')
    bid = widget_button(bidwf ,font=ft_propor,value=' ')
    bid1= widget_button(bidwf ,font=ft_propor,value='  Save your new configuration from the Samples menu.')
    bid = widget_button(bidwf ,font=ft_propor,value='   ')
    bid2= widget_button(bidwf ,font=ft_propor,value='  (you may calculate detectors efficiencies to adjust your mask)',/menu)
     bid= widget_button(bid2  ,font=ft_propor,value='- Have transmissions for '+txtWater+' and '+txtWater+' cell')
     bid= widget_button(bid2  ,font=ft_propor,value='- Select optional ' +txtWater+' box option in "Mask" and mask the beam of a '+txtwater+' file')
     bid= widget_button(bid2  ,font=ft_propor,value='- Enter numors for '+txtWater+', '+txtWater+' cell, ['+txtCadm+'] from "Samples"')
     bid= widget_button(bid2  ,font=ft_propor,value='- Select "Show detectors efficiencies" in (+) from menu_bar')
     bid= widget_button(bid2  ,font=ft_propor,value='- Return to "Mask" and select "Show masked files corrected by efficiencies"')

   bpts = widget_button(binst ,font=ft_propor,value= 'Show Data path'              ,uvalue=[1,4])
   bid  = widget_button(binst ,font=ft_propor,value='---')  & widget_control,bid,sens=0

   bid  = widget_button(binst ,font=ft_propor,value='Restore a configuration file'             ,uvalue=[60,1])
   bid  = widget_button(binst ,font=ft_propor,value='---')  & widget_control,bid,sens=0

   bids = widget_button(binst ,font=ft_propor,value='Export ...',/menu)
   bid  = widget_button(bids  ,font=ft_propor,value='Save current settings'                    ,uvalue=[4,0])
   bid  = widget_button(bids  ,font=ft_propor,value=' ')
   bid  = widget_button(bids  ,font=ft_propor,value='Export integration Result W19 -> gnum.100'      ,uvalue=[13,0,19])
   bid  = widget_button(bids  ,font=ft_propor,value='Export corrected Data     W18 -> tnum.100'      ,uvalue=[13,2,18])
  ;bid  = widget_button(bids  ,font=ft_propor,value='Export symbol image of   Mask -> num*.msk'      ,uvalue=[13,4,0])
   bid  = widget_button(bids  ,font=ft_propor,value=' ')
   bid  = widget_button(bids  ,font=ft_propor,value='Export integration Result W19 -> 1D_numor.txt'     ,uvalue=[13,1,19])
   bid  = widget_button(bids  ,font=ft_propor,value='Export corrected Data     W18 -> 2D_numor.txt'     ,uvalue=[13,3,18])
   bid  = widget_button(bids  ,font=ft_propor,value='Export corrected Data     W18 -> PNG images'       ,uvalue=[13,6,18])
   bid  = widget_button(bids  ,font=ft_propor,value='Export corrected Data     W18 -> NIST_2D_numor.dat',uvalue=[13,7,18])
   bid  = widget_button(binst ,font=ft_propor,value='---')  & widget_control,bid,sens=0

   bid  = widget_button(binst ,font=ft_propor,value='Exit sans_settings (Save Result LIST)',uvalue=[77,0,-1])
   bid  = widget_button(binst ,font=ft_propor,value='Exit sans_settings'                   ,uvalue=[3,0])
   bid  = widget_button(binst ,font=ft_propor,value='---')

  b_bar1= widget_button(mbar  ,font=ft_propor,/menu,value='Center>')
   bid  = widget_button(b_bar1,font=ft_propor,value='---')
   bid  = widget_button(b_bar1,font=ft_propor,value='FIND CENTER from attenuated Beam'    ,uvalue=[30,1])
   bid  = widget_button(b_bar1,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar1,font=ft_propor,value='Treatment options'   ,uvalue=[90])

  b_bar2= widget_button(mbar  ,font=ft_propor,/menu,value='Transmission>')
   bid  = widget_button(b_bar2,font=ft_propor,value='---')
   bid  = widget_button(b_bar2,font=ft_propor,value='CALCULATE TRANSMISSIONS from attenuated Data',uvalue=[40])
   bid  = widget_button(b_bar2,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar2,font=ft_propor,value='Traitment options'   ,uvalue=[90])

  b_bar3= widget_button(mbar  ,font=ft_propor,/menu,value='Mask>')
   bid  = widget_button(b_bar3,font=ft_propor,value='---')
   bid  = widget_button(b_bar3,font=ft_propor,value='SET MASK for Sample , detector , '+txtWater  ,uvalue=[50,0])
   bid  = widget_button(b_bar3,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar3,font=ft_propor,value='Defining SECTORS'                    ,uvalue=[80,0,0])
   bid  = widget_button(b_bar3,font=ft_propor,value='---')  & widget_control,bid,sens=0
   butf = widget_button(b_bar3,font=ft_propor,value='Hide/Show Beam sample'   ,uvalue=[54,8])
   butf = widget_button(b_bar3,font=ft_propor,value='Hide/Show Beam '+txtwater,uvalue=[54,9])
   bid  = widget_button(b_bar3,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar3,font=ft_propor,value='Save Detector Mask to   "file*.msk"' ,uvalue=[13,4,0])
   bid  = widget_button(b_bar3,font=ft_propor,value='Load Detector Mask from "file*.msk"' ,uvalue=[16])

  b_bar4= widget_button(mbar  ,font=ft_propor,/menu,value='Sample>')
   bid  = widget_button(b_bar4,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar4,font=ft_propor,value='INPUT SCATTERING DATA'               ,uvalue=[61])
   bid  = widget_button(b_bar4,font=ft_propor,value='Show/Save/Restore Configurations'    ,uvalue=[60,0])
   bid  = widget_button(b_bar4,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar4,font=ft_propor,value='Adjust thickness, transm. for Series',uvalue=[61])
   bid  = widget_button(b_bar4,font=ft_propor,value='Treatment options'                   ,uvalue=[90])
   bid  = widget_button(b_bar4,font=ft_propor,value='Using Sectors (see Mask)'            ,uvalue=[80,1,0])

  b_bar5= widget_button(mbar  ,font=ft_propor,/menu,value=anal)
   bid  = widget_button(b_bar5,font=ft_propor,value='Show Qrange'                         ,uvalue=[70,1])
   bid  = widget_button(b_bar5,font=ft_propor,value='Show Detectors efficiencies  ->W17'  ,uvalue=[72,3,0])
   bid  = widget_button(b_bar5,font=ft_propor,value='Show Corrected Data  (det.)  ->W18'  ,uvalue=[72,1,0])
   bid  = widget_button(b_bar5,font=ft_propor,value='---')  & widget_control,bid,sens=0

   bcurv= widget_button(b_bar5,font=ft_propor,value='RADIAL    AVERAGING CURVE(s) ->W19'  ,uvalue=[72,2,0]) ;!!Uvalue changed when Log is pressed
   bcurA= widget_button(b_bar5,font=ft_propor,value='AZIMUTHAL AVERAGING CURVE(s) ->W19'  ,uvalue=[72,4,0])
   bids = widget_button(b_bar5,font=ft_propor,value='SECTORS   INTEGRATIONS       ->W19'  ,/menu)
     bid= widget_button(bids  ,font=ft_propor,value='Show Sector total  integration symmetrical',uvalue=[72,5,0])
     bid= widget_button(bids  ,font=ft_propor,value='Show Sector total  integration non-sym'   ,uvalue=[72,6,0])
     bid= widget_button(bids  ,font=ft_propor,value='---')  & widget_control,bid,sens=0
     bid= widget_button(bids  ,font=ft_propor,value='Show Sector radial integration symmetrical',uvalue=[72,7,0])
     bid= widget_button(bids  ,font=ft_propor,value='Show Sector radial integration non-sym'   ,uvalue=[72,8,0])
   bid  = widget_button(b_bar5,font=ft_propor,value='---')  & widget_control,bid,sens=0

   bcurR= widget_button(b_bar5,font=ft_propor,value='PLOT OPTIONS and LIST OF RESULTS' ,/menu)
     bid= widget_button(bcurR ,font=ft_propor,value=' ')
     bid= widget_button(bcurR ,font=ft_propor,value='LIST of RADIAL    AVERAGING CURVE(s)'    ,uvalue=[71,2,1])
     bid= widget_button(bcurR ,font=ft_propor,value=' ')
     bid= widget_button(bcurR ,font=ft_propor,value='LIST of AZIMUTHAL AVERAGING CURVE(s)'    ,uvalue=[71,4,1])
     bid= widget_button(bcurR ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(b_bar5,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bids = widget_button(b_bar5,font=ft_propor,value='Tools (Overplot ,Lamp ,Wksp operations ,..',/menu)
     bid= widget_button(bids  ,font=ft_propor,value='OVERPLOTTING (superplot)'                 ,uvalue=[75])
     bid= widget_button(bids  ,font=ft_propor,value='DO input commands (Wksp operations)'      ,uvalue=[73])
     bid= widget_button(bids  ,font=ft_propor,value='List of BATCH macros (input commands)'    ,uvalue=[74,0])
     bid= widget_button(bids  ,font=ft_propor,value='IMPORT integration Result W20 <- g*.00*'  ,uvalue=[15,0])
     bid= widget_button(bids  ,font=ft_propor,value='Verify numors data parameters'            ,uvalue=[99,2])
     bid= widget_button(bids  ,font=ft_propor,value='Journal'                                  ,uvalue=[99,1])
     bid= widget_button(bids  ,font=ft_propor,value='Lamp'                                     ,uvalue=[1,3])
   bid  = widget_button(b_bar5,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bids = widget_button(b_bar5,font=ft_propor,value='About Sans_settings',/menu)
   bid  = widget_button(bids  ,font=ft_propor,value='Written at I.L.L. (2007)')
   bid  = widget_button(bids  ,font=ft_propor,value='Kernel support:')
   bid  = widget_button(bids  ,font=ft_propor,value='       Didier Richard')
   bid  = widget_button(bids  ,font=ft_propor,value='       Miguel Gonzalez')
   bid  = widget_button(bids  ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(bids  ,font=ft_propor,value='Scientific consultants:')
   bid  = widget_button(bids  ,font=ft_propor,value='       Peter Linder')
   bid  = widget_button(bids  ,font=ft_propor,value='       Isabelle Grillo')
   bid  = widget_button(bids  ,font=ft_propor,value='       Bruno Deme')
   bid  = widget_button(bids  ,font=ft_propor,value='       Ralf Schweins')

  ENDIF else begin & bfxtg=0L & bcurv=0L & ENDELSE ;FOR SHEET


 ;CONFIG fields
 ; ------------
  instrus=instru
  ids =strpos(instrus,'<') & if ids[0] ge 0 then strput,instrus,'_',ids[0]
  ids =strpos(instrus,'>') & if ids[0] ge 0 then strput,instrus,'_',ids[0]

  IF (NOT SHEET) THEN BEGIN
  b1list= widget_base(  basej1,/row,/frame,resource_name='don',/align_center)
  bselct= widget_button(b1list,value='LOADconfig'                       ,font=ft_propor,/menu)
    bid = widget_button(bselct,value='Load from a file'                 ,font=ft_propor            ,uvalue=[2,0])
    bid = widget_button(bselct,value=' '                                ,font=ft_propor) & widget_control,bid,sens=0
    bid = widget_button(bselct,value='Load from memory <- LOW    angle' ,font=ft_propor            ,uvalue=[2,1])
    bid = widget_button(bselct,value='Load from memory <- MIDDLE angle' ,font=ft_propor            ,uvalue=[2,2])
    bid = widget_button(bselct,value='Load from memory <- HIGH   angle' ,font=ft_propor            ,uvalue=[2,3])
  bcurf = widget_text(  b1list,value=strlowcase(instrus)+'set_conf.sav' ,font=ft_propor,/editable  ,uvalue=[2,0],xsize=18)
  label = widget_label( b1list,value=''                                 ,font=ft_b_normal)
  label = widget_label( b1list,value=' Filter:'                         ,font=ft_propor)

  b1expo= widget_base(  basej1,/row,/frame,resource_name='don',/align_center)
  bwrite= widget_button(b1expo,value='SAVEconfig'                       ,font=ft_propor,/menu)
    bid = widget_button(bwrite,value='Save in memory as LOW    angle'   ,font=ft_propor            ,uvalue=[4,1])
    bid = widget_button(bwrite,value='Save in memory as MIDDLE angle'   ,font=ft_propor            ,uvalue=[4,2])
    bid = widget_button(bwrite,value='Save in memory as HIGH   angle'   ,font=ft_propor            ,uvalue=[4,3])
    bid = widget_button(bwrite,value=' '                                ,font=ft_propor) & widget_control,bid,sens=0
    bid = widget_button(bwrite,value='Save in a file (LOW,MIDDLE,HIGH should be in memory)',font=ft_propor,uvalue=[4,0])
  bwrtf = widget_text(  b1expo,value=strlowcase(instrus)+'set_conf.sav' ,font=ft_propor,/editable  ,uvalue=[4,0],xsize=18)
  label = widget_label( b1expo,value=''                                 ,font=ft_b_normal)
  bfilt = widget_text(  b1expo,value='*.sav'                            ,font=ft_propor,/editable,xsize=11)
  ENDIF ;not sheet

  bmess = widget_label( basej1,value='                  Messages                  ',font=ft_propor,/dynamic)

  base20= widget_base(  basej1,/column,/frame,resource_name='did',/align_center)
  if (not sheet) then  $
  label = widget_label( base20,value='CONFIG'                       ,font=ft_b_normal)

  base2r= widget_base(  base20,/row)
  base2c= widget_base(  base2r,/column)

  base21= widget_base(  base2c,/row)
  label = widget_label( base21,value='Wavelength  (A)'              ,font=ft_propor)
  bwave = widget_text(  base21,value=wavl   ,xsize=6                ,font=ft_propor,uvalue=[10,0])
  label = widget_label( base21,value=' Distance(m)'                 ,font=ft_propor)
  bdist = widget_text(  base21,value=dista  ,xsize=6                ,font=ft_propor,uvalue=[10,1])
  base21= widget_base(  base2c,/row)
  label = widget_label( base21,value='Center (Px)  CX'              ,font=ft_propor)
  bcentx= widget_text(  base21,value=centex ,xsize=6                ,font=ft_propor,/editable,uvalue=[5 ,1,1])
  label = widget_label( base21,value='          CY'                 ,font=ft_propor)
  bcenty= widget_text(  base21,value=centey ,xsize=6                ,font=ft_propor,/editable,uvalue=[5 ,2,1])
  base21= widget_base(  base2c,/row)
  label = widget_label( base21,value='Beam size     X'              ,font=ft_propor)
  bbeamx= widget_text(  base21,value=beamx  ,xsize=6                ,font=ft_propor,/editable,uvalue=[7 ,1])
  label = widget_label( base21,value='      (Px) Y'                 ,font=ft_propor)
  bbeamy= widget_text(  base21,value=beamy  ,xsize=6                ,font=ft_propor,/editable,uvalue=[7 ,2])
  basofa= widget_base(  base2c,/row)
  labOffset='Offset Det (mm)' & if ofa eq 2 then labOffset='Offset Dist(mm)'
  label = widget_label( basofa,value=labOffset                      ,font=ft_propor)
  boffs = widget_text(  basofa,value=offset ,xsize=6                ,font=ft_propor,uvalue=[10,2])
  if ofa  eq 0  then begin & widget_control,label,sens=0 & widget_control,boffs,sens=0 & endif
  label = widget_label( basofa,value=' Angle (deg)'                 ,font=ft_propor)
  bangl = widget_text(  basofa,value=angle  ,xsize=6                ,font=ft_propor,uvalue=[10,3])
  if Raxe eq 0. then begin & widget_control,label,sens=0 & widget_control,bangl,sens=0 & endif

  base21= widget_base(  base2c,/row)
  base22= widget_base(  base21,/column)
  bupdt = 0L
 ;bupdt = widget_button(base22,value='  Large View  '               ,font=ft_propor,uvalue=[8])
  bidC  = widget_base(  base22,/exclusive,/row)
  bconf1 =widget_button(bidC  ,value='Conf 1'       ,/no_release    ,font=ft_b_normal,uvalue=[2,1])
  bconf2 =widget_button(bidC  ,value='2'            ,/no_release    ,font=ft_b_normal,uvalue=[2,2])
  bconf3 =widget_button(bidC  ,value='3'            ,/no_release    ,font=ft_b_normal,uvalue=[2,3])
  widget_control,bconf1, set_button=0
  wview = widget_draw(  base22,xsize=cvh,ysize=cvh,/button_events,uvalue=[20])

  base22= widget_base(  base22)
  base22= widget_base(  base21,/column)
  base23= widget_base(  base22,/row)
  label = widget_label( base23,value=' '                            ,font=ft_propor)
  base23= widget_base(  base22,/column)
  base23= widget_base(  base23,/row)
  label = widget_label( base23,value='Collimation  (m)  '           ,font=ft_propor)
  bvcoll= widget_text(  base23,value= vcoll ,xsize=6                ,font=ft_propor,uvalue=[6,0])

  junk22= widget_base(  base22)
  base2a= widget_base(  junk22,/column,map=1-sheet) ;For sans_settings
  base2b= widget_base(  junk22,/column,map=  sheet) ;For sans_sheet

  base22= base2a
   base23= widget_base(  base22,/row)
   labil = widget_label( base23,value='Thickness '+txtwater+' cm'    ,font=ft_propor)
   bvthiW= widget_text(  base23,value= thinW ,xsize=6                ,font=ft_propor,uvalue=[11,10,labil])
   base23= widget_base(  base22,/row)
   labil = widget_label( base23,value='Thickness sample1 '           ,font=ft_propor)
   bvthiS= widget_text(  base23,value= thinS ,xsize=6                ,font=ft_propor,uvalue=[11,11,labil])
   base23= widget_base(  base22,/row)
   label = widget_button(base23,value='Fixed '+txtWater+' X_section' ,font=ft_propor,uvalue=[90])
   widget_control,bvthiS,editable=1 & widget_control,bvthiW,editable=1
   widget_control,bvcoll,editable=1 & widget_control,bangl ,editable=1
   widget_control,boffs ,editable=1 & widget_control,bdist ,editable=1
   widget_control,bwave ,editable=1
   bQdep=0L

  base22=base2b
   base23= widget_base(  base22,/row)
   labil = widget_label( base23,value=txtwater+' Collimation '       ,font=ft_propor)
   bvcolW= widget_text(  base23,value= Wcoll ,xsize=6                ,font=ft_propor,uvalue=[11,10,labil])
   base23= widget_base(  base22,/row)
   labil = widget_label( base23,value=txtwater+' Distance    '       ,font=ft_propor)
   bvdisW= widget_text(  base23,value= distW ,xsize=6                ,font=ft_propor,uvalue=[11,11,labil])
   base23=widget_base(  base22,/row)
   trefq= widget_button(widget_base(base23,/nonexclusive),value='Tr (2th)',font=ft_b_normal,uvalue=[95])
          widget_control,trefq,set_button=sans_supl[43]
   bQdep= widget_button(widget_base(base23,/nonexclusive),value='Q dep'   ,font=ft_b_normal,uvalue=[98])
          widget_control,bQdep,set_button=1-sans_supl[61]
   bidTQ= widget_button(base23,value='?'                                  ,font=ft_b_normal,/menu)
     bid= widget_button(bidTQ,value='Tr(2TH) means correct Transmission by a function of (2theta,thickness)',font=ft_b_normal)
     bid= widget_button(bidTQ,value='')
     if Qdep le ' ' then TQdep='none' else TQdep=Qdep
     bid= widget_button(bidTQ,value='Q dep means correct Detector by the actual function:',font=ft_b_normal)
     bid= widget_button(bidTQ,value='     '+TQdep                               ,font=ft_b_normal)
  if sheet then begin
   base24=widget_base(  base23,map=0)
   bcurf= widget_text(  base24,value=strlowcase(instrus)+'set_conf.sav' ,xsize=1) ;Bluffing
   bwrtf= widget_text(  base24,value=strlowcase(instrus)+'set_conf.sav' ,xsize=1)
   bfilt= widget_text(  base24,value='*.sav'                            ,xsize=1)

   widget_control,bcentx,set_uvalue=[81,1,bcentx,bcenty]
   widget_control,bcenty,set_uvalue=[81,2,bcentx,bcenty]
  endif ;sheet

 ;INPUT NUMORS fields
 ; ------------------
  base30= widget_base(  base0 ,/column,/frame,resource_name='mic')
  junk  = widget_base(  base30)
  junkA = widget_base(  junk  ,/column,map=1)          ;transmission files
  widget_control,junkA ,sens=1-sheet
  junkD = widget_base(  junk  ,/column,map=0)          ;data         files
  widget_control,junkD ,sens=1-sheet
  junkWM= widget_base(  junk  ,/column,map=0)          ;water more   files
  widget_control,junkWM,sens=1-sheet
  junkFX= widget_base(  junk  ,/column,map=0)          ;flux transm. file
  widget_control,junkFX,sens=1-sheet
  junkM = widget_base(  junk  ,/column,map=0)          ;mask         files
  junkST= widget_base(  junk  ,/column,map=0)          ;show   transmission      (replace input files)
  junkVT= widget_base(  junk  ,/column,map=0)          ;verify transmission      (replace input files)
  junkTX= widget_base(  junk  ,/column,map=0,uvalue=0) ;edit trans and thickness (replace input files)
  junkRS= widget_base(  junk  ,/column,map=0)          ;list of results          (replace input files)

 ;            ------------------------SHOW TRANSMISSION SAMPLE SERIES
  label = widget_label( junkST,value='TRANSMISSION SAMPLE SERIES'       ,font=ft_b_normal,/dynamic)
  baseT1= widget_base(  junkST,/row)
  bSTsho= widget_text(  baseT1,value='',scr_xsize=320+win*50,ysize=7    ,font=ft_b_normal)
    bid = widget_button(baseT1,value='ok'                               ,font=ft_b_normal,uvalue=[46,1])

 ;            ------------------------VERIFY TRANSMISSION SAMPLE SERIES
  label = widget_label( junkVT,value='VERIFY TRANSMISSION SAMPLE SERIES',font=ft_b_normal,/dynamic)
  baseT1= widget_base(  junkVT,/row)
  bVTsho= widget_text(  baseT1,value='',scr_xsize=320+win*50,ysize=7    ,font=ft_b_normal)
    bid = widget_button(baseT1,value='ok'                               ,font=ft_b_normal,uvalue=[46,3])

 ;            ------------------------THICKNESS,X-SECTION SAMPLE SERIES
  label = widget_label( junkTX,value='TRANSMISSION,THICKNESS for SAMPLE SERIES',font=ft_b_normal,/dynamic)
  baseT1= widget_base(  junkTX,/row)
  bTXsho= widget_text(  baseT1,value='',scr_xsize=320+win*50,ysize=7    ,font=ft_b_normal,/editable)
    bid = widget_button(baseT1,value='ok'                               ,font=ft_b_normal,uvalue=[46,5])

 ;            ------------------------RESUMING
 ;label = widget_label (junkRS,value='LIST OF GROUPING RESULTS'         ,font=ft_b_normal,/dynamic)
  baseT1= widget_base(  junkRS,/column)
  bRSsho= widget_list(  baseT1,value='',scr_Xsize=dcs+win*30,ysize=8           ,font=ft_propor,uvalue=[76],/multiple,/align_center)
  bsort = widget_button(baseT1 ,value='Sorting and Selecting the List', /menu,  font=ft_propor                      ,/align_center)
    bid = widget_button(bsort ,value='Sort the List by Samples (default)'      ,font=ft_b_normal,uvalue=[77,15,0])
    bid = widget_button(bsort ,value='Sort the List by Conf 1, Conf 2, Conf 3' ,font=ft_b_normal,uvalue=[77,15,1])
    bid = widget_button(bsort ,value='Sort the List by Sectors'                ,font=ft_b_normal,uvalue=[77,15,2])
    bid = widget_button(bsort ,value='')
    bid = widget_button(bsort ,value='Select all'                              ,font=ft_b_normal,uvalue=[77,16,0])
    bid = widget_button(bsort ,value='Select all Conf 1 Curves'                ,font=ft_b_normal,uvalue=[77,16,1])
    bid = widget_button(bsort ,value='Select all Conf 2 Curves'                ,font=ft_b_normal,uvalue=[77,16,2])
    bid = widget_button(bsort ,value='Select all Conf 3 Curves'                ,font=ft_b_normal,uvalue=[77,16,3])
    bid = widget_button(bsort ,value='')
    bid = widget_button(bsort ,value='Remove Selected Curves'                  ,font=ft_b_normal,uvalue=[71,2 ,2])

 ;            ------------------------DATA FILE TO CREATE MASK------------------------
  junkM2= widget_base(  junkM)
  labkM2= widget_base(  junkM2,map=1)
   labro= widget_base(  labkM2,/row) ;SAMPLE BEAM   -- DETECTOR MASK --   WATER BEAM
  seckM2= widget_base(  junkM2,map=0,/row)
   seclb= widget_label( seckM2,value='CENTER:  Cx '                 ,font=ft_b_normal)
   secCx= widget_text(  seckM2,value='        ' ,xsize=8            ,font=ft_propor,/editable)
   seclb= widget_label( seckM2,value='  Cy '                        ,font=ft_b_normal)
   secCy= widget_text(  seckM2,value='        ' ,xsize=8            ,font=ft_propor,/editable)
   label= widget_label( seckM2,value=''                             ,font=ft_b_normal)
   secBt= widget_button(seckM2,value='Theta0'                       ,font=ft_b_normal)
   
  junkM3= widget_base(  junkM)
  
  seckS = widget_base(  junkM3,/row,map=0)
   secSq= widget_button(widget_base(seckS,/nonexclusive),value='SQUARED Sectors'   ,font=ft_b_normal)
   secRk= widget_button(seckS ,value='ROCKING: None ',font=ft_b_normal,/menu)
     bid= widget_button(secRk ,value='"Rocking curves for total integrations ONLY"',font=ft_propor)
     bid= widget_button(secRk ,value=''              ,font=ft_propor) & widget_control,bid,sens=0
     bid= widget_button(secRk ,value='None '         ,font=ft_propor  ,uvalue=[82,2,0])
     bid= widget_button(secRk ,value='     '         ,font=ft_propor) & widget_control,bid,sens=0
     bid= widget_button(secRk ,value='Index ->Counts versus index'             ,font=ft_propor,uvalue=[82,2,1]) ;& widget_control,bid,sens=0
     bid= widget_button(secRk ,value='Times ->Counts versus cumulated time'    ,font=ft_propor,uvalue=[82,2,2]) ;& widget_control,bid,sens=0
     bid= widget_button(secRk ,value='Tempe ->Counts versus sample temperature',font=ft_propor,uvalue=[82,2,3]) ;& widget_control,bid,sens=0
     bid= widget_button(secRk ,value='S.rot ->Counts versus sample rotation'   ,font=ft_propor,uvalue=[82,2,4]) ;& widget_control,bid,sens=0
     bid= widget_button(secRk ,value='.....'                                   ,font=ft_propor)                  & widget_control,bid,sens=0
   secMd= widget_button(seckS ,value='Mode'          ,font=ft_b_normal,uvalue=[82,2,-1])
    

  baseK0= widget_base(  junkM3,/row,map=1)
   excl = widget_base(  baseK0,/exclusive,/row)
    bids= widget_button(excl  ,value='Sample Beam -'                ,font=ft_b_normal,uvalue=[55,1,0],/no_release)
    bid2= widget_button(excl  ,value='Cur conf'                     ,font=ft_b_normal,uvalue=[55,4,0],/no_release)
    bid1= widget_button(excl  ,value='All conf  -'                  ,font=ft_b_normal,uvalue=[55,3,0],/no_release)
    bidw= widget_button(excl  ,value= txtWater+' Beam'              ,font=ft_b_normal,uvalue=[55,2,0],/no_release)
    geos= widget_info(  bids,/geometry) & geos=geos.Xsize
    geo2= widget_info(  bid2,/geometry)
    geo1= widget_info(  bid1,/geometry) & geo2=geo2.Xsize+geo1.Xsize
    geow= widget_info(  bidw,/geometry) & geow=geow.Xsize
   label= widget_label( labro ,value='SAMPLE BEAM'                  ,font=ft_b_normal,Xsize=geos,/align_center)
   label= widget_label( labro ,value='DETECTOR MASK'                ,font=ft_b_normal,Xsize=geo2,/align_center)
   label= widget_label( labro ,value=strupcase(txtWater)+' BEAM'    ,font=ft_b_normal,Xsize=geow,/align_center)
  baseK1= widget_base(  junkM ,/row,uvalue=[0L,bids,bidw,bid1,bid2])
  mlabs = widget_label( baseK1,value='Numor  '                      ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:7,par:'',opt:0})
                                                                    ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
  msampl= widget_text(  baseK1,value='w7=' ,xsize=14                ,font=ft_propor,/editable,uvalue=[9,26,7,0L])
  mlabw = widget_label( baseK1,value=' '+txtWater                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:8,par:'',opt:0})
  mwater= widget_text(  baseK1,value='w8=' ,xsize=13                ,font=ft_propor,/editable,uvalue=[9,27,8,0L])
  baseK3= widget_base(  junkM ,/row)
  Mhelp = widget_button(baseK3,value='?'   ,/menu                   ,font=ft_b_normal)
    bid = widget_button(mhelp ,value='100>200 + 205 + 220>240'      ,font=ft_propor)
    bid = widget_button(mhelp ,value='Means:'                       ,font=ft_propor)
    bid = widget_button(mhelp ,value='Add numors 100 till 200 and'  ,font=ft_propor)
    bid = widget_button(mhelp ,value='add numor  205 and'           ,font=ft_propor)
    bid = widget_button(mhelp ,value='add numors 220 till 240'      ,font=ft_propor)

  mlab  = widget_label( baseK3 ,value=' '                           ,font=ft_b_normal)
  MchkE = widget_button(widget_base(baseK3,/nonexclusive),value='Show corrected Numor (when possible)',font=ft_b_normal,uvalue=[56])
  widget_control,bids  ,set_button=1
  widget_control,mwater,sens=0
 ;widget_control,MchkE ,sens=0
  widget_control,secBt ,set_uvalue=[81,0,secCx,secCy] & if sans_supl[52] eq 0 then widget_control,secBt ,sens=0
  widget_control,secCX ,set_uvalue=[81,1,secCx,secCy]
  widget_control,secCY ,set_uvalue=[81,2,secCx,secCy]
  widget_control,secSq ,set_uvalue=[82,1]
  widget_control,seckM2,set_uvalue=[     secCx,secCy]

;               ---------------------TRANSMISSION DATA FILES---------------------
  base30= widget_base(  junkA ,/row) ;& widget_control,base30,sens=1-sheet
  Ahelp = widget_button(base30,value='?'   ,/menu                   ,font=ft_b_normal)
    bid = widget_button(ahelp ,value='Adding numors example:'        ,font=ft_propor)
    bid = widget_button(ahelp ,value='w1=100>200 + 205 + 220>240'    ,font=ft_propor)
    bid = widget_button(ahelp ,value=' '                             ,font=ft_propor)
    bid = widget_button(ahelp ,value='Input Sample series example'   ,font=ft_propor)
    bid = widget_button(ahelp ,value='w3 =10:20 , 22:22'             ,font=ft_propor)
    bid = widget_button(ahelp ,value='w3 =10:::19'                   ,font=ft_propor)
   ;bid = widget_button(ahelp ,value='w3 =10:::19+'                  ,font=ft_propor)
   ;bid = widget_button(ahelp ,value=' '                             ,font=ft_propor)
   ;bid = widget_button(ahelp ,value='WARNING! "+" means same titles are added'   ,font=ft_propor)
   ;bid = widget_button(ahelp ,value='IN LINE with transmission-sample series',font=ft_propor)
  label = widget_label( base30,value=' '                             ,font=ft_propor)
  junkTR= widget_base(  base30)
  jun1TR= widget_base(  junkTR,map=1)
  jun2TR= widget_base(  junkTR,map=0,uvalue=0)
  txttr = widget_text(  jun2TR,value=' ',xsize=6 ,/editable            ,font=ft_propor,uvalue=[45])
  buttr = widget_button(jun1TR,value='Show Transm.',/menu              ,font=ft_b_normal)
  atrans= lonarr(4) & txt1='  '+txtwater
  atrans[0]=widget_button(buttr,value=transm[0]+txt1water              ,font=ft_propor)
  atrans[1]=widget_button(buttr,value=transm[1]+txt2water              ,font=ft_propor)
  atrans[2]=widget_button(buttr,value=transm[2]+'  Sample 1..'         ,font=ft_propor)
  atrans[3]=widget_button(buttr,value=transm[3]+'  Sample cell'        ,font=ft_propor)
     bid   =widget_button(buttr,value='---'                            ,font=ft_propor)
     bidat1=widget_button(buttr,value='Enter Tr for '+txtwater         ,font=ft_propor,uvalue=[44,0,txttr])
     bidat2=widget_button(buttr,value='Enter Tr for '+txtWater+'  cell',font=ft_propor,uvalue=[44,1,txttr])
     bidat3=widget_button(buttr,value='Enter Tr for Sample 1'          ,font=ft_propor,uvalue=[44,2,txttr])
     bidat4=widget_button(buttr,value='Enter Tr for Sample cell'       ,font=ft_propor,uvalue=[44,3,txttr])
     bid   =widget_button(buttr,value='---'                            ,font=ft_propor)
     bid   =widget_button(buttr,value='-SERIES-'                       ,font=ft_propor)
     bid   =widget_button(buttr,value='SHOW Tr for Tsample series'     ,font=ft_propor,uvalue=[46,0])
     bid   =widget_button(buttr,value='EDIT Tr, Es ,Title transm'      ,font=ft_propor,uvalue=[46,4,0])
  txttr    =widget_text(  jun2TR,value=' ',xsize=6 ,/editable          ,font=ft_propor,uvalue=[45,0,txttr])
  label = widget_label( base30,value='   '                             ,font=ft_propor)
  label = widget_label( base30,value='TRANSMISSION DATA FILES'         ,font=ft_b_normal,/dynamic)
  base31= widget_base(  junkA ,/row) ;TEXTuv=[9, bases3 index LABEL, Wrkspace, OL]
  p=1-wins
                                                                    ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
  alabs = widget_label( base31,value='Sample'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:3,par:'',opt:0})
  asampl= widget_text(  base31,value='w3=' ,xsize=15                ,font=ft_propor,/editable,uvalue=[9,20,3,0L])
  alabw = widget_label( base31,value=' '+txtWater+' '               ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:1,par:'',opt:0})
  awatr = widget_text(  base31,value='w1=' ,xsize=13                ,font=ft_propor,/editable,uvalue=[9,18,1,0L])
  base32= widget_base(  junkA ,/row)
  alabse= widget_label( base32,value='S.cell'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:4,par:'',opt:0})
  asampe= widget_text(  base32,value='w4=' ,xsize=15                ,font=ft_propor,/editable,uvalue=[9,21,4,0L])
  alabwe= widget_label( base32,value=txtW+'.cell'                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:2,par:'',opt:0})
  awatre= widget_text(  base32,value='w2=' ,xsize=13                ,font=ft_propor,/editable,uvalue=[9,19,2,0L])
  base33= widget_base(  junkA ,/row)
  alabbS= widget_label( base33,value='S.beam'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:9,par:'',opt:0})
  abeamS= widget_text(  base33,value='w9=' ,xsize=15                ,font=ft_propor,/editable,uvalue=[9,22,9,0L])
  alabbW= widget_label( base33,value=txtW+'.Beam'                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:6,par:'',opt:0})
  abeamW= widget_text(  base33,value='w6=' ,xsize=13                ,font=ft_propor,/editable,uvalue=[9,23,6,0L])
  confus=base33
  widget_control,confus,set_uvalue={V:'w5=',S:[9,22,5,0L],T:'Trs',L:{data:0,datp:0,opr:' ',wi:5,par:'',opt:0}}

  widget_control,base31,sens=0 & widget_control,base32,sens=0
  widget_control,alabbW,sens=0 & widget_control,abeamW,sens=0
  widget_control,jun1TR,sens=0

;                                     ----------DATA FILES----------
  base34= widget_base(  junkD ,/row) ;& widget_control,base34,sens=1-sheet
  bhelp = widget_button(base34,value='?'   ,/menu                   ,font=ft_b_normal)
    bid = widget_button(bhelp ,value='Adding numors example:'        ,font=ft_propor)
    bid = widget_button(bhelp ,value='w10=100>200 + 205 + 220>240'   ,font=ft_propor)
    bid = widget_button(bhelp ,value=' '                             ,font=ft_propor)
    bid = widget_button(bhelp ,value='Input Sample series example'   ,font=ft_propor)
    bid = widget_button(bhelp ,value='w12=10:20 , 22:22'             ,font=ft_propor)
    bid = widget_button(bhelp ,value='w12=10:::19'                   ,font=ft_propor)
   ;bid = widget_button(bhelp ,value='w12=10:::19+'                  ,font=ft_propor)
   ;bid = widget_button(bhelp ,value=' '                             ,font=ft_propor)
   ;bid = widget_button(bhelp ,value='WARNING! "+" means same titles are added'   ,font=ft_propor)
   ;bid = widget_button(bhelp ,value='  Insert ,:, to be ALINED with transmissions',font=ft_propor)
  label = widget_label( base34,value=' '                             ,font=ft_propor)
  but   = widget_button(base34,value='Verify Transm.',/menu           ,font=ft_b_normal)
  btrans= lonarr(4)
  btrans[0]=widget_button(but  ,value=transm[0]+txt1water            ,font=ft_propor)
  btrans[1]=widget_button(but  ,value=transm[1]+txt2water            ,font=ft_propor)
  btrans[2]=widget_button(but  ,value=transm[2]+'  Sample 1..'       ,font=ft_propor)
  btrans[3]=widget_button(but  ,value=transm[3]+'  Sample cell'      ,font=ft_propor)
       bid =widget_button(but  ,value='----'                         ,font=ft_propor)
       bid =widget_button(but  ,value='Modify the 4 transmissions'   ,font=ft_propor,uvalue=[40])
       bid =widget_button(but  ,value='----'                         ,font=ft_propor)
       bud =widget_button(but  ,value='FIND transm. from previous results',/menu,font=ft_propor)
        bid=widget_button(bud  ,value='(First be sure to have good thickness)',font=ft_propor)
        bid=widget_button(bud  ,value='----'                                  ,font=ft_propor) & widget_control,bid,sens=0
        VET=widget_button(bud  ,value='->VERIFY/EDIT Thickness'               ,font=ft_propor,uvalue=[46,4,1])
        bid=widget_button(bud  ,value=' '                                     ,font=ft_propor)
        bid=widget_button(bud  ,value='->GET TRANSMISSIONS AUTOMATICALY'      ,font=ft_propor,uvalue=[72,0,0])
        bid=widget_button(bud  ,value='    (file is "sans_transmissions.txt")',font=ft_propor,uvalue=[72,0,0])

       bid =widget_button(but  ,value='----'                         ,font=ft_propor)
       bid =widget_button(but  ,value='-SERIES-'                     ,font=ft_propor)
       bid =widget_button(but  ,value='SHOW sample series transmissions'  ,font=ft_propor,uvalue=[46,2])
       bid =widget_button(but  ,value='EDIT Tr,Es,Title for series'       ,font=ft_propor,uvalue=[46,4,1])
  label = widget_label( base34,value='   '                          ,font=ft_propor)
  label = widget_label( base34,value='SCATTERING DATA FILES'        ,font=ft_b_normal)
  base35= widget_base(  junkD ,/row) ;TEXTuv=[9, bases3 index LABEL, Wrkspace, OL]
  p=1-wins
  base35= widget_base(  junkD ,/row)
                                                                    ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
  blabs = widget_label( base35,value='Sample'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:12,par:'',opt:0})
  bsampl= widget_text(  base35,value='w12=' ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,8 ,12,0L])
  blabw = widget_label( base35,value=' '+txtWater+' '               ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:10,par:'',opt:0})
  bwatr = widget_text(  base35,value='w10=' ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,6 ,10,0L])
  base36= widget_base(  junkD ,/row)
  blabse= widget_label( base36,value='S.cell'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:13,par:'',opt:0})
  bsampe= widget_text(  base36,value='w13=' ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,9 ,13,0L])
  blabwe= widget_label( base36,value=txtW+'.cell'                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:11,par:'',opt:0})
  bwatre= widget_text(  base36,value='w11=' ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,7 ,11,0L])
  base37= widget_base(  junkD ,/row)
  blabc = widget_label( base37,value='S.'+txtCadm                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:14,par:'',opt:0})
  bcadmi= widget_text(  base37,value='w14=' ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,10,14,0L])
  blabcw= widget_label( base37,value=txtW+'.'+txtCadm               ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:15,par:'',opt:0})
  bcadmw= widget_text(  base37,value='w15=' ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,11,15,0L])

;                                     ----------WATER MORE FILES----------Mainly used for sans_sheet
  base34= widget_base(  junkWM,/row)
  Whelp = widget_button(base34,value='?'   ,/menu                   ,font=ft_b_normal)
  label = widget_label( base34,value='   '                          ,font=ft_propor)
  label = widget_label( base34,value='ALTERNATIVE WATER FILES'      ,font=ft_b_normal)
  base35= widget_base(  junkWM,/row) ;TEXTuv=[9, bases3 index LABEL, Wrkspace, OL]
  p=1-wins
  base35= widget_base(  junkWM,/row)
                                                                    ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
  Wlabw2= widget_label( base35,value=txtWater+'2'                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wwatr2= widget_text(  base35,value=''     ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,35,24,0L])
  Wlabw3= widget_label( base35,value=' '+txtWater+'3'               ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wwatr3= widget_text(  base35,value=''     ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,36,24,0L])
  base36= widget_base(  junkWM,/row)
  Wlabe2= widget_label( base36,value='W.cell'                       ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wwate2= widget_text(  base36,value=''     ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,37,24,0L])
  Wlabe3= widget_label( base36,value=txtW+'.cell'                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wwate3= widget_text(  base36,value=''     ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,38,24,0L])
  base37= widget_base(  junkWM,/row)
  Wlabc2= widget_label( base37,value='W.'+txtCadm                   ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wcadc2= widget_text(  base37,value=''     ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,39,24,0L])
  Wlabc3= widget_label( base37,value=txtW+'.'+txtCadm               ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:24,par:'',opt:0})
  Wcadc3= widget_text(  base37,value=''     ,xsize=13               ,font=ft_propor,/editable,uvalue=[9,40,24,0L])

  ;                                 ----------FLUX ALTERNATIVE TRANSISSION----This junk base is unused
  base34= widget_base(  junkFX,/row)
  Xhelp = widget_button(base34,value='?'   ,/menu                   ,font=ft_b_normal)
  label = widget_label( base34,value='   '                          ,font=ft_propor)
  label = widget_label( base34,value='ALTERNATIVE FLUX TRANSMISSION',font=ft_b_normal)
  base35= widget_base(  junkFX,/row) ;TEXTuv=[9, bases3 index LABEL, Wrkspace, OL]
  p=1-wins
  base35= widget_base(  junkFX,/row)
                                                                    ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
  SlabFX= widget_label( base35,value='FLUX Trs.'                    ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:26,par:'',opt:0})
  SampFX= widget_text(  base35,value=''     ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,42,26,0L])


 ;CENTER ATTENUATION MASK OTHERs SECTORS OPTIONS fields
 ; ----------------------------------------------------
  labelB= widget_label( basej2,value='CENTER'                        ,font=ft_b_normal,/dynamic)
  junkWN= widget_base(  basej2)
  junkW1= widget_base(  junkWN,/row,map=1) ;detector view
  junkW2= widget_base(  junkWN,/row,map=0) ;grouping view
  junkW3= widget_base(  junkWN,/row,map=0) ;sectors  view

  bzoom = widget_base(  junkW1,/column)

  labzo = widget_label( bzoom ,value='Zoom'                          ,font=ft_b_normal) ;ZOOM In Out
   bzin = widget_button(bzoom ,value='In'                            ,font=ft_b_normal,uvalue=[25,1])
   bzout= widget_button(bzoom ,value='Out'                           ,font=ft_b_normal,uvalue=[25,2])
  bid   = widget_label( bzoom ,value=' '                             ,font=ft_b_normal)
  cmenu = widget_button(bzoom ,value='Color'                         ,font=ft_b_normal,/menu)
    bid = widget_button(cmenu ,value='White-blue-green-yellow-red'   ,font=ft_b_normal,uvalue=[26,27])
    bid = widget_button(cmenu ,value='---'                           ,font=ft_b_normal,uvalue=[26,27])
    bid = widget_button(cmenu ,value='Red--Blue-yellow-white'        ,font=ft_b_normal,uvalue=[26,15])
    bid = widget_button(cmenu ,value='Black and white'               ,font=ft_b_normal,uvalue=[26, 0])
    bid = widget_button(cmenu ,value='Blue'                          ,font=ft_b_normal,uvalue=[26, 1])
    bid = widget_button(cmenu ,value='Red'                           ,font=ft_b_normal,uvalue=[26, 3])
    bid = widget_button(cmenu ,value='Blue-green-red-yellow'         ,font=ft_b_normal,uvalue=[26, 4])
    bid = widget_button(cmenu ,value='Blue-red'                      ,font=ft_b_normal,uvalue=[26,11])
  bid   = widget_label( bzoom ,value=' '                             ,font=ft_b_normal)
  bsurf = widget_button(bzoom ,value='Surf'                          ,font=ft_b_normal,uvalue=[25,3])
  besid = widget_button(bzoom ,value='Image'                         ,font=ft_b_normal,uvalue=[25,4,1]) ;detector
  if not Wins then $
  bid   = widget_label( bzoom ,value=' '                             ,font=ft_smallest)
  bsliD = widget_slider(bzoom ,value=0,title='',min=0,max=1,/drag,Ysize=70-20*Wins,/vertical  ,uvalue=[27,0],/Suppress_Value,sens=0)
  bsliDN= widget_label( bzoom ,value='      '                        ,font=ft_b_normal,/dynamic)

  wview2= widget_draw(  junkW1,xsize=dcs,ysize=dcs,/button_events,uvalue=[19,1,1],/motion_events,retain=2) ;for detectors
  label = widget_label( junkW1,value=' '                             ,font=ft_propor)
  wview3= widget_draw(  junkW1,xsize=25 ,ysize=dcs)

  wview4= widget_draw(  junkW2,xsize=dcs+94+win*30,ysize=dcs,/button_events,uvalue=[19,2,1],/motion_events,retain=2) ;for groups
 
 ;SECTORS
  bzoom = widget_base(  junkW3,/column)
  cmenu = widget_button(bzoom ,value='Color'                         ,font=ft_b_normal,/menu)
    bid = widget_button(cmenu ,value='White-blue-green-yellow-red'   ,font=ft_b_normal,uvalue=[26,27])
    bid = widget_button(cmenu ,value='---'                           ,font=ft_b_normal,uvalue=[26,27])
    bid = widget_button(cmenu ,value='Red--Blue-yellow-white'        ,font=ft_b_normal,uvalue=[26,15])
    bid = widget_button(cmenu ,value='Black and white'               ,font=ft_b_normal,uvalue=[26, 0])
    bid = widget_button(cmenu ,value='Blue'                          ,font=ft_b_normal,uvalue=[26, 1])
    bid = widget_button(cmenu ,value='Red'                           ,font=ft_b_normal,uvalue=[26, 3])
    bid = widget_button(cmenu ,value='Blue-green-red-yellow'         ,font=ft_b_normal,uvalue=[26, 4])
    bid = widget_button(cmenu ,value='Blue-red'                      ,font=ft_b_normal,uvalue=[26,11])
  bid   = widget_label( bzoom ,value=' '                             ,font=ft_b_normal)
  bsurf = widget_button(bzoom ,value='Surf'                          ,font=ft_b_normal,uvalue=[25,3])
 ;besid = widget_button(bzoom ,value='Image'                         ,font=ft_b_normal,uvalue=[25,4,2]) ;sector
  bid   = widget_label( bzoom ,value=' '                             ,font=ft_smallest)
  bsliB = widget_slider(bzoom ,value=0,title='',min=0,max=1,/drag,Ysize=100,/vertical ,uvalue=[27,0],/Suppress_Value,sens=0)
  bsliBN= widget_label( bzoom ,value='      '                        ,font=ft_b_normal,/dynamic)

  wview5= widget_draw(  junkW3,Xsize=dcs,Ysize=dcs,/button_events,uvalue=[19,1,2],/motion_events,retain=2,$ ;for Ben
                               X_scroll_size=dcs,Y_scroll_size=dcs,/scroll)
  label = widget_label( junkW3,value=' '                             ,font=ft_propor)
  wview6= widget_draw(  junkW3,xsize=25 ,ysize=dcs)
  if not sheet then workflow='See WORKFLOW SCHELUDE ADVICE in Menu_bar' else workflow=' '
  btitl = widget_label( basej2,value=workflow ,font=ft_smaller,/dynamic)
  base41= widget_base(  basej2,/row)
  baslog= widget_button(widget_base(base41,/nonexclusive),Value='Log I',font=ft_b_normal,uvalue=[24])
  bcoord= widget_label( base41,value=' CX:'+centex+' CY:'+centey       ,font=ft_b_normal,/dynamic)

  junk  = widget_base(  basej2,/align_center)
  junkCE= widget_base(  junk  ,/column,map=1,resource_name='did') ;center options
  junkAT= widget_base(  junk  ,/column,map=0,resource_name='did') ;transm options
  junkMS= widget_base(  junk  ,/column,map=0,resource_name='did') ;mask   options
  junkCF= widget_base(  junk  ,/column,map=0,resource_name='did') ;thickn options
  junkCO= widget_base(  junk  ,/column,map=0,resource_name='did') ;correc options
  junkRO= widget_base(  junk  ,/column,map=0) ;result options
  junkCM= widget_base(  junk  ,/column,map=0)                   ;un_mapped (collimation and mask tables)
  junkBT= widget_base(  junk  ,/column,map=0,y_scroll_size=100,x_scroll_size=400+(win-1)*65) ;batch list
  junkSC= widget_base(  junk  ,/column,map=0,Event_pro="lamp_event_parser")             ;sectors options

 ;COMMUNICATION (futur)
 ; ------------
  base21= widget_base(  junkCM,/row)
  base22= widget_base(  base21,/column)
  label = widget_label( base22,value='Collimation'                  ,font=ft_propor)
  bcolim= widget_list(  base22,value=collim, xsize=6  ,ysize=6      ,font=ft_propor ,uvalue=[6,1])
  base22= widget_base(  base21,/column)
  label = widget_label( base22,value='Mask[x,y,nx,ny]'              ,font=ft_propor)
  bmask = widget_text(  base22,value=masq,   xsize=13 ,ysize=6      ,font=ft_propor)

 ;CENTER
 ; -----
  base41= widget_base(  junkCE,/row)
  base42= widget_base(  base41,/nonexclusive,/row)
  bCbidm= widget_button(base42,value='Exclude Masked pts'         ,font=ft_propor,uvalue=[33])
  basCfg= widget_base(  base41,/nonexclusive,/row,map=useCfit)
  bCbidf= widget_button(basCfg,value='GaussFit'                   ,font=ft_propor,uvalue=[31])
  bCbidg= widget_button(basCfg,value='Count Centroid'             ,font=ft_propor,uvalue=[32])
  base41= widget_base(  junkCE,/row)
  bfxrgc= widget_label( base41,value= fxrg                        ,font=ft_propor)
  basXC1= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  basXC2= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  label = widget_label( base41,value=' Yrange'                    ,font=ft_propor)
  basYC1= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  basYC2= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  base41= widget_base(  junkCE,/row)
  bas4CC= widget_button(base41,value='CALCULATE/set CENTER'       ,font=ft_b_normal,uvalue=[39,0])
  lab4CC= widget_label( base41,value=' [XYrg]:drag with left or middle button',font=ft_b_normal,/dynamic)
  widget_control,bCbidf,set_button=bfit
  widget_control,bCbidg,set_button=1-bfit
  widget_control,bCbidm,set_button=1

 ;ATTENUATION
 ; ----------
  base41= widget_base(  junkAT,/row)
  base42= widget_base(  base41,/nonexclusive,/row)
  bTbidm= widget_button(base42,value='Exclude Masked pts'    ,font=ft_b_normal,uvalue=[43])
  label = widget_label( base41,value='   '                   ,font=ft_propor)

  bid41 = widget_button(base41,value='?'                          ,font=ft_smaller,/menu)
     bid= widget_button(bid41 ,value='Flux0 (neutron/sec) = Attenuator_factor * Total(I)/time'        ,font=ft_b_normal)
     bid= widget_button(bid41 ,value='By default (I) is corrected (dead_time)'                        ,font=ft_b_normal)
     bid= widget_button(bid41 ,value='Phi0 is fixed when entering the Sample beam to calculate center',font=ft_b_normal)
     bid= widget_button(bid41 ,value='     and the Water beam to calculate transmissions'             ,font=ft_b_normal)
   Econf= widget_button(bid41 ,value='See also Treatment Options->'              ,font=ft_b_normal,uvalue=[90])
  label = widget_label( base41,value=' '+txtWater+' Phi0 (n/s)='                 ,font=ft_b_normal)
 ;bjAtSF= widget_text  (base41,value= Wphi0,xsize=8               ,font=ft_propor,/editable,uvalue=[12,31])
  bjAtSF= widget_label( base41,value='00000000'                   ,font=ft_propor,/dynamic) & widget_control,bjAtSF,set_value=Wphi0

  base41= widget_base(  junkAT,/row)
  bfxrga= widget_label( base41,value= fxrg                        ,font=ft_propor)
  basXA1= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  basXA2= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  label = widget_label( base41,value=' Yrange'                    ,font=ft_propor)
  basYA1= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  basYA2= widget_text(  base41,value=' ',xsize=4                  ,font=ft_propor,/editable)
  base41= widget_base(  junkAT,/row)
  bas4CA= widget_button(base41,value='CALCULATE TRANSMISSION'     ,font=ft_b_normal,uvalue=[49])
  labeCA= widget_label( base41,value=' [XYrg]:drag with left or middle button',font=ft_b_normal,/dynamic)
  widget_control,bas4CA,set_uvalue=[49,labeCA]
  widget_control,bTbidm,set_button=1

 ;MASK
 ; ---
  baseM1= widget_base(  junkMS,/row) & if beamf eq 1 then txtb=' rectang ' else txtb=' ellipse '
  if beammod then begin
  labXB1= widget_label( baseM1,value='Beam x1,x2,y1,y2'             ,font=ft_propor)
  basXB1= widget_text(  baseM1,value=beambx,xsize=15                ,font=ft_propor,/editable,uvalue=[51,6])
  labYB1= 0L
  basYB1= 0L
  endif else begin
  labXB1= widget_label( baseM1,value='Beamstop  Xsiz '              ,font=ft_propor)
  basXB1= widget_text(  baseM1,value=  beamx,xsize=4                ,font=ft_propor,/editable,uvalue=[51,1])
  labYB1= widget_label( baseM1,value=' Ysiz'                        ,font=ft_propor)
  basYB1= widget_text(  baseM1,value=  beamy,xsize=4                ,font=ft_propor,/editable,uvalue=[51,2])
  endelse
  label = widget_label( baseM1,value=' show'                        ,font=ft_b_normal)
  butf  = widget_button(baseM1,value=  txtb                         ,font=ft_b_normal,/menu)
    but = widget_button(butf  ,value='rectang'                      ,font=ft_b_normal        ,uvalue=[54,1,butf])
    but = widget_button(butf  ,value='ellipse'                      ,font=ft_b_normal        ,uvalue=[54,2,butf])
    but = widget_button(butf  ,value='---'                          ,font=ft_b_normal)
    but = widget_button(butf  ,value='hide/show beam (Sample)'      ,font=ft_b_normal        ,uvalue=[54,3,butf])
    but = widget_button(butf  ,value='hide/show mask (Instrument)'  ,font=ft_b_normal        ,uvalue=[54,4,butf])
    but = widget_button(butf  ,value='hide/show beam ('+txtWater+')',font=ft_b_normal        ,uvalue=[54,5,butf])
    but = widget_button(butf  ,value='hide all'                     ,font=ft_b_normal        ,uvalue=[54,6,butf])
    but = widget_button(butf  ,value='show all'                     ,font=ft_b_normal        ,uvalue=[54,7,butf])
  baseM2= widget_base(  junkMS,/row)
  bfxlin= widget_label( baseM2,value= fxln                          ,font=ft_propor)
  basXL1= widget_text(  baseM2,value=' ',xsize=9                    ,font=ft_propor,/editable,uvalue=[51,3])
  label = widget_label( baseM2,value=' Ylines'                      ,font=ft_propor)
  basYL1= widget_text(  baseM2,value=' ',xsize=9                    ,font=ft_propor,/editable,uvalue=[51,4])
  baseM3= widget_base(  junkMS,/row)
  labXYB= widget_label( baseM3,value='Box  x1,x2,y1,y2'             ,font=ft_propor)
  basXYB= widget_text(  baseM3,value=' ',xsize=15                   ,font=ft_propor,/editable,uvalue=[51,5])
  label = widget_label( baseM3,value=''                             ,font=ft_propor)
    but = widget_button(baseM3,value='Undo'                         ,font=ft_b_normal,uvalue=[52])
  label = widget_label( baseM3,value=''                             ,font=ft_propor)
  baseM4= widget_button(baseM3,value='Clear'                        ,font=ft_b_normal,/menu)
     but= widget_button(baseM4,value='Clear all mask'               ,font=ft_b_normal,uvalue=[53,1])
     but= widget_button(baseM4,value='Reset to default mask'        ,font=ft_b_normal,uvalue=[53,2])
  widget_control,baseM2,sens=0
  widget_control,labXYB,sens=0
  widget_control,basXYB,sens=0

 ;OPTIONS FOR DETECTORS CORRECTIONS
 ; --------------------------------
  base41= widget_base(  junkCO,/row)
  trddt = widget_button(widget_base(base41,/nonexclusive),value='Dead Times'  ,font=ft_b_normal,uvalue=[91])
  label = widget_label( base41,value='Tau:'                       ,font=ft_b_normal)
  bTauv = widget_text(  base41,value=strTau,xsize=9               ,font=ft_propor              ,uvalue=[12,6],/edit)
    bid = widget_label( base41,value=' '                          ,font=ft_propor)
    tmpf= lonarr(n_elements(fortau))
  bTauf = widget_button(base41,value='Tau function',/menu         ,font=ft_b_normal)
    for i=0,n_elements( fortau)-1 do begin & bid=widget_button(btauf,value=fortau[i],font=ft_propor    ,uvalue=[94,i,btauf])
                                           tmpf[i]=bid & endfor
                                           widget_control,btauf,set_uvalue=tmpf
     bid= widget_base(  base41,map=0)
  blistR= widget_button(bid   ,value='',uvalue=[71,0,1]) ;Used to link Resume,Options window !!!!

  base42= widget_base(  junkCO,/row)
  trsag = widget_button(widget_base(base42,/nonexclusive,/frame),value='Solid angles'    ,font=ft_b_normal,uvalue=[92])

  label = widget_label( base42,value=' '                              ,font=ft_b_normal)
    mot = widget_base(  base42,/exclusive,/row,/frame)
   Nmon = widget_button(mot   ,value='use Monitor' ,/no_release       ,font=ft_b_normal,uvalue=[97,0])
   Ntim = widget_button(mot   ,value='use Time'    ,/no_release       ,font=ft_b_normal,uvalue=[97,1])

  label = widget_label( base42,value='Group:'                     ,font=ft_b_normal)         & widget_control,label ,sens=1-sheet
  bjRadi= widget_text(  base42,value=radin,xsize=3                ,font=ft_propor,/editable) & widget_control,bjradi,sens=1-sheet
  base43= widget_base(  junkCO,/row)
  base44= widget_base(  base43,/row,/frame)
  trboc = widget_button(widget_base(base44,/nonexclusive),value='Use '+txtWater+' X section->'      ,font=ft_b_normal,uvalue=[93])
  bjCros= widget_text(  base44,value=crosW,xsize=5                                                  ,font=ft_propor,/editable,uvalue=[12,42])
  label = widget_label( base44,value=''                                                             ,font=ft_propor)

  if XWdep gt ' ' then begin
    bidf= widget_button(base44 ,value='Ask',/menu                                                   ,font=ft_propor)
    bidX= widget_button(bidf   ,value='GET and USE the Water Cross Section of the instrument'      ,font=ft_propor)
    bid = widget_button(bidf   ,value=''                                                            ,font=ft_propor)
    bidf= widget_button(bidf   ,value='SHOW correction Formula used by the application',/menu       ,font=ft_propor)
    bid = widget_button(bidf   ,value='Xsct_'+txtwater+'=Nw/(ew*Phi0w*TRw*SolAw) or is supplied'    ,font=ft_propor)
    bid = widget_button(bidf   ,value='Xsct_'+txtwater+'='+XWdep                                    ,font=ft_propor)
    widget_control,bidX,set_uvalue=[86,bjCros,trboc]
  endif else begin
    bidf= widget_button(base44 ,value='?',/menu                                                     ,font=ft_propor)
    bid = widget_button(bidf   ,value='Xsct_'+txtwater+'=Nw/(ew*Phi0w*TRw*SolAw) or is supplied'    ,font=ft_propor)
  endelse

    bid = widget_button(bidf  ,value='F1   =ew/es * tw/ts * Xsct_'+txtwater+' * (Phi0w*SolAw)/(Phi0s*SolAs)',font=ft_propor)
    bid = widget_button(bidf  ,value='F1   =ew/es * tw/ts * Xsct_'+txtwater+' * (Colls*Dists/Collw/Distw)^2',font=ft_propor)
    bid = widget_button(bidf  ,value='F2   =1/(es * ts * Phi0s * SolAs * EFFd)'                     ,font=ft_propor)
    bid = widget_button(bidf  ,value='F3   =Q dependent response of detector (Instrument specific)' ,font=ft_propor)
    bid = widget_button(bidf  ,value=' '                                                            ,font=ft_propor)
    bid = widget_button(bidf  ,value='Depending of the known flux at sample and/or '+txtwater       ,font=ft_propor)
    bid = widget_button(bidf  ,value='position we can use the folowing formula.'                    ,font=ft_propor)
    bid = widget_button(bidf  ,value='If no flux can be calculated, you should provide'             ,font=ft_propor)
    bid = widget_button(bidf  ,value='a '+txtwater+' cross section value and first formula is used.',font=ft_propor)
    bid = widget_button(bidf  ,value=' '                                                            ,font=ft_propor)
    bid = widget_button(bidf  ,value='** '+txtwater+' SCALING FORMULA **'                           ,font=ft_propor)
    bid = widget_button(bidf  ,value='I[cm-1]=[(Is - Icads)/(TRs*TRes) - (Ies - Icads)/TRes]  * F1 *(F3)'  ,font=ft_propor)
    bid = widget_button(bidf  ,value='             ----------------'                                ,font=ft_propor)
    bid = widget_button(bidf  ,value='        [(Iw - Icadw)/(TRw*TRew) - (Iew - Icadw)/TRew]'              ,font=ft_propor)
    bid = widget_button(bidf  ,value=' '                                                            ,font=ft_propor)
    bid = widget_button(bidf  ,value='** EFFd SCALING FORMULA  **'                                  ,font=ft_propor)
    bid = widget_button(bidf  ,value='I[cm-1]=[(Is - Icads)/(TRs*TRes) - (Ies - Icads)/TRes]  * F2 *(F3)'  ,font=ft_propor)
    bid = widget_button(bidf  ,value=' '                                                            ,font=ft_propor)
    bid = widget_button(bidf  ,value='I    =neutrons/sec (Is=sample Iw='+txtwater+')'               ,font=ft_propor)
    bid = widget_button(bidf  ,value='TR   =transmission Q dependent'     ,font=ft_propor)
    bid = widget_button(bidf  ,value='e    =thickness'                    ,font=ft_propor)
    bid = widget_button(bidf  ,value='t    =time'                         ,font=ft_propor)
    bid = widget_button(bidf  ,value='Phi0 =flux/sec'                     ,font=ft_propor)
    bid = widget_button(bidf  ,value='Xsct =cross_section'                ,font=ft_propor)
    bid = widget_button(bidf  ,value='Coll =Collimation'                  ,font=ft_propor)
    bid = widget_button(bidf  ,value='Dist =Distance'                     ,font=ft_propor)
    bid = widget_button(bidf  ,value='SolA =solid angle'                  ,font=ft_propor)
    bid = widget_button(bidf  ,value='EFFd =detector efficiencies'        ,font=ft_propor)
    bid = widget_button(bidf  ,value='Nw   =('+txtwater+'-Cell) average neutrons/sec'        ,font=ft_propor)
    bid = widget_button(bidf  ,value=' '                                                     ,font=ft_propor)
    bid = widget_button(bidf  ,value='*MISSING CORRECTIONS: Multiple Scat. ,Incoherent Bgrd' ,font=ft_propor)
  label = widget_label( base43,value=''                                                      ,font=ft_propor)
  truns = widget_button(widget_base(base43,/nonexclusive),value='No scaling' ,font=ft_b_normal ,uvalue=[96])
  widget_control,trddt,set_button=sans_supl[27]
  widget_control,trsag,set_button=sans_supl[28]
  widget_control,trboc,set_button=sans_supl[29] ;fscal  ->use water  X section
  widget_control,truns,set_button=sans_supl[44] ;unScal ->use absolute scaling
  if Mon_Tim then widget_control,Ntim,set_button=1 else widget_control,Nmon,set_button=1

 ;RESULT OPTIONS (resuming)
 ; -------------
  base41= widget_base(  junkRO,/row)

  bas41f= widget_base(  base41,/row,/frame)
  IofQ  = widget_button(bas41f,/menu,value='f(I)',font=ft_b_normal)
   bid  = widget_button(IofQ                             ,value='I'          ,font=ft_b_normal,uvalue=[77,10,0])
   bid  = widget_button(IofQ                             ,value='I*Q'        ,font=ft_b_normal,uvalue=[77,10,1])
   bid  = widget_button(IofQ                             ,value='I*Q^2'      ,font=ft_b_normal,uvalue=[77,10,2])
   bid  = widget_button(IofQ                             ,value='I*Q^4'      ,font=ft_b_normal,uvalue=[77,10,3])

  ROnexc= widget_base(  bas41f,/nonexclusive,/row)
  ROirg = widget_button(ROnexc                           ,value='I rg'       ,font=ft_b_normal        ,uvalue=[77,13,1])
  sclIr1= widget_text(  bas41f,value='    '              ,xsize=4            ,font=ft_propor,/editable,uvalue=[77,13,0])
  sclIr2= widget_text(  bas41f,value='    '              ,xsize=4            ,font=ft_propor,/editable,uvalue=[77,13,0])

  bas41f= widget_base(  base41,/row,/frame)
  ROexcl= widget_button(bas41f,/menu,value='f(Q)',font=ft_b_normal)
  ROQ   = widget_button(ROexcl                           ,value='Q'          ,font=ft_b_normal,uvalue=[77,3,3])
  ROlog = widget_button(ROexcl                           ,value='Log Q'      ,font=ft_b_normal,uvalue=[77,4,4])
  ROsqr = widget_button(ROexcl                           ,value='Q^2'        ,font=ft_b_normal,uvalue=[77,5,5])
  ROp4  = widget_button(ROexcl                           ,value='Q^4'        ,font=ft_b_normal,uvalue=[77,5,6])

  ROnexc= widget_base(  bas41f,/nonexclusive,/row)
  ROqrg = widget_button(ROnexc                           ,value='Q rg'       ,font=ft_b_normal        ,uvalue=[77,11,1])
  sclQr1= widget_text(  bas41f,value='    '              ,xsize=4            ,font=ft_propor,/editable,uvalue=[77,11,0])
  sclQr2= widget_text(  bas41f,value='    '              ,xsize=4            ,font=ft_propor,/editable,uvalue=[77,11,0])

  widget_control,ROirg ,set_button=0
  widget_control,ROqrg ,set_button=0

  if sys_dep('VERSION') ge 5.6 then begin
  widget_control,default_font=ft_propor
  junkROO=widget_base(junkRO,resource_name='did')
  tabase= widget_tab( junkROO) & if sys_dep('VERSION') ge 6.1 then widget_control,tabase,tab_mod=2,scr_Xsize=dcs+94+win*30
 ;*********

  TAB1  = widget_base(  tabase,title='PLOTS/PRINT   ')   ;SUPRPLOT & SURFACES & ITOOLS & PRINT
  tab3c = widget_base(  TAB1  ,/column)
  bid   = widget_label( tab3c ,value=' ',font=ft_smallest)
  base11= widget_base(  tab3c ,/row)
 ;See the code at the end of the TABs definition

 ;*********


  TAB2  = widget_base(  tabase,title='MERGING   ')   ;ADJUSTEMENT AND MERGE
  tab1c = widget_base(  TAB2  ,/column)
  bid   = widget_label( tab1c ,value=' ',font=ft_smallest)
  base42= widget_base(  tab1c ,/row)

  butact= widget_button(base42,value='Action',font=ft_b_normal,/menu)
  casbas= widget_base(  base42)
   QMbas= widget_base(  casbas,/row,map=0)
   IFbas= widget_base(  casbas,/row,map=0)
   WSbas= widget_base(  casbas,/row,map=0)
   labas= widget_base(  casbas,/row,map=1)
     bid= widget_label( labas ,value='Take an action to set Qranges, Scalings & Merges',font=ft_b_normal)
   
   QMbut= widget_button(butact,value='- Show QRANGE input fields to adjust in each configuration' ,font=ft_propor)
     bid= widget_label( QMbas ,value='l'                           ,font=ft_b_normal)
   FLfla= widget_text(  QMbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
   FLflb= widget_text(  QMbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
     bid= widget_label( QMbas ,value='m'                           ,font=ft_b_normal)
   FMfla= widget_text(  QMbas ,value='    '     ,xsize=4           ,font=ft_propor,/editable)
   FMflb= widget_text(  QMbas ,value='    '     ,xsize=4           ,font=ft_propor,/editable)
     bid= widget_label( QMbas ,value='h'                           ,font=ft_b_normal)
   FHfla= widget_text(  QMbas ,value='    '     ,xsize=4           ,font=ft_propor,/editable)
   FHflb= widget_text(  QMbas ,value='   '      ,xsize=4           ,font=ft_propor,/editable)
   
   IQbut= widget_button(butact,value='- Show SCALEs input fields to adjust in each configuration  (and Background)' ,font=ft_propor)
     bid= widget_label( IFbas ,value='low'                         ,font=ft_b_normal)
   FLfld= widget_text(  IFbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
     bid= widget_label( IFbas ,value='mid'                         ,font=ft_b_normal)
   FMfld= widget_text(  IFbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
     bid= widget_label( IFbas ,value='high'                        ,font=ft_b_normal)
   FHfld= widget_text(  IFbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
     bid= widget_label( IFbas ,value='(bg)'                        ,font=ft_b_normal)
   Fbcbg= widget_text(  IFbas ,value='    '     ,xsize=4           ,font=ft_propor,/editable)

   WSbut= widget_button(butact,value='- Show 4th configuration input fields to adjust' ,font=ft_propor)
     bid= widget_label( WSbas ,value='Scale'                       ,font=ft_b_normal)
   WSfld= widget_text(  WSbas ,value='     '    ,xsize=5           ,font=ft_propor,/editable)
     bid= widget_label( WSbas ,value='Qrange'                      ,font=ft_b_normal)
   WSfla= widget_text(  WSbas ,value='     '    ,xsize=6           ,font=ft_propor,/editable)
   WSflb= widget_text(  WSbas ,value='     '    ,xsize=6           ,font=ft_propor,/editable)

   ZEbut= widget_button(butact,value='- RESET Qranges,  Scales  (and Background)',font=ft_propor)
  
   bid  = widget_button(butact,value='') & widget_control,bid,sens=0
   bid  = widget_button(butact,font=ft_b_normal          ,value='FOR ALL SAMPLES (in radial integration List):')
   IQsgA= widget_button(butact,font=ft_propor            ,value='  - GET a SUGGESTION for Scales and Qranges, calculated from ALL samples (sum)')
   IQsgS= widget_button(butact,font=ft_propor            ,value='  - GET a SUGGESTION for Scales and Qranges, calculated from SELECTED samples')
   bid  = widget_button(butact,font=ft_propor            ,value='')  & widget_control,bid,sens=0
   MgALL= widget_button(butact,font=ft_propor            ,value='  - EXPORT ALL Samples      in 4columns files (MERGE according to Scales,Qranges)')
   MgSEL= widget_button(butact,font=ft_propor            ,value='  - EXPORT SELECTED Samples in 4columns files (MERGE according to Scales,Qranges)')
   bid  = widget_button(butact,font=ft_propor            ,value='')  & widget_control,bid,sens=0
   bid  = widget_button(butact,font=ft_b_normal          ,value='FOR ONE SELECTED SAMPLE (2,3 configuration curves):')
  ;bid  = widget_button(butact,font=ft_propor            ,value='  - Merge in workspace W22'                    ,uvalue=[77,23])
   bid  = widget_button(butact,font=ft_propor            ,value='  - Merge and show in superplot (W22)'         ,uvalue=[77,24])
   dltQ = widget_button(butact,font=ft_propor            ,value='  - Show   deltaQ for selected curves (W23)'   ,uvalue=[77,25,-1,-1])
  ;bid  = widget_button(butact,font=ft_propor            ,value='  - Export deltaQ in file numor_2col.txt'      ,uvalue=[13,1,23])
                                                                                                                                   ; 4TH CONFIGURATION .......
  widget_control,ZEbut ,set_uvalue=[77,7,0,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

  widget_control,QMbut ,set_uvalue=[77,7,1,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,IQbut ,set_uvalue=[77,7,2,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

  widget_control,IQsgA ,set_uvalue=[77,7,3,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,IQsgS ,set_uvalue=[77,7,4,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,MgALL ,set_uvalue=[77,7,5,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,MgSEL ,set_uvalue=[77,7,6,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
                                                                                                                                   ; 4TH CONFIGURATION .......
  widget_control,FLfld ,set_uvalue=[77,8,1,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FMfld ,set_uvalue=[77,8,2,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FHfld ,set_uvalue=[77,8,3,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

  widget_control,Fbcbg ,set_uvalue=[77,8,4,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
                                                                                                                                   ; 4TH CONFIGURATION .......
  widget_control,FLfla ,set_uvalue=[77,9,1,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FLflb ,set_uvalue=[77,9,2,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FMfla ,set_uvalue=[77,9,3,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FMflb ,set_uvalue=[77,9,4,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FHfla ,set_uvalue=[77,9,5,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,FHflb ,set_uvalue=[77,9,6,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

 ;4TH CONFIGURATION CONTROL                     3     4     5       6                                     12                 15      16
  widget_control,WSbut ,set_uvalue=[77,7,16,    QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,WSbut ,sensitive = 0
  widget_control,WSfld ,set_uvalue=[77,8,1,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,WSfla ,set_uvalue=[77,9,1,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]
  widget_control,WSflb ,set_uvalue=[77,9,2,     QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

  widget_control,bRSsho,set_uvalue=[76,0,butAct,QMbas,IFbas,labas , FLfla,FLflb,FMfla,FMflb,FHfla,FHflb , FLfld,FMfld,FHfld, Fbcbg , WSbas, WSfla,WSflb, WSfld ,WSbut]

 ;*********

  TAB3  = widget_base(  tabase,title='SAVE/RESTORE   ');SAVINGS
  tab2c = widget_base(  TAB3  ,/column)
  bid   = widget_label( tab2c ,value=' ',font=ft_smallest)
  base43= widget_base(  tab2c ,/row)
  but   = widget_button(base43,value='Save/Restore ->'      , font=ft_b_normal,/menu)
   bid  = widget_button(but   ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(but   ,font=ft_propor,value='SAVE    List of RESULTS in   a *_result_list.sav File' ,uvalue=[77,0,0])
   bid  = widget_button(but   ,font=ft_propor,value='MERGE   List of RESULTS in   a *_result_list.sav File' ,uvalue=[77,0,1])
   bid  = widget_button(but   ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(but   ,font=ft_propor,value='SAVE    Selected CURVES in files gnum.200'       ,uvalue=[78,5,1]) ;& widget_control,bid,sens=0
   bid  = widget_button(but   ,font=ft_propor,value='SAVE    Selected CURVES in files numor_4col.txt' ,uvalue=[78,5,2]) ;& widget_control,bid,sens=0
   bid  = widget_button(but   ,font=ft_propor,value='---')  & widget_control,bid,sens=0
   bid  = widget_button(but   ,font=ft_propor,value='RESTORE List of RESULTS from a *_result_list.sav File' ,uvalue=[77,1,0])
   
  bid   = widget_label( base43,value=' ')

  butc  = widget_button(base43,value='Clear LIST ->',/menu , font=ft_b_normal)
    bid= widget_button(butc  ,font=ft_propor,value='')  & widget_control,bid,sens=0
    bId= widget_button(butc  ,font=ft_propor,value='Clear ALL'            , uvalue=[71,2,0])   
    bid= widget_button(butc  ,font=ft_propor,value='')  & widget_control,bid,sens=0
    bid= widget_button(butc  ,font=ft_propor,value='Clear Selected Curves', uvalue=[71,2,2])

  bid   = widget_label( base43,value=' ')
  butc  = widget_button(base43,value='Get gnum.* files'     , font=ft_b_normal          ,uvalue=[78,4])

 ;*********

  TAB4  = widget_base(  tabase,title='OPERATIONS   ')   ;OPERATIONS
  tab4c = widget_base(  TAB4  ,/column)
  bid   = widget_label( tab4c ,value=' ',font=ft_smallest)
  base44= widget_base(  tab4c ,/row)
  bopr1 = widget_button(base44,value='Select 1'     , font=ft_b_normal)
  blabo1= widget_label( base44,value='*'            , uvalue=[''])
  bopr1T= widget_text(  base44,value=' 1. ',xsize=4 , font=ft_propor,/editable)
  bid   = widget_label( base44,value=' ')
  opropr= widget_button(base44,value='-',/menu      , font=ft_b_normal)
   boprm= widget_button(opropr,value='- Select 2'   , font=ft_b_normal)
   boprp= widget_button(opropr,value='+ Select 2'   , font=ft_b_normal)
   boprd= widget_button(opropr,value='/ Select 2'   , font=ft_b_normal)
   boprs= widget_button(opropr,value='* Select 2'   , font=ft_b_normal)
   bid  = widget_button(opropr,value=' '            , font=ft_b_normal)
   boprc= widget_button(opropr,value='- Constant'   , font=ft_b_normal)
  bid   = widget_label( base44,value=' ')
  bopr2 = widget_button(base44,value='Select 2'     , font=ft_b_normal)
  blabo2= widget_label( base44,value='*'            , uvalue=[''])
  bopr2T= widget_text(  base44,value=' 1. ',xsize=4 , font=ft_propor,/editable)
  bid   = widget_label( base44,value='  ')
  boprdo= widget_button(base44,value='Do'           , font=ft_b_normal)

  widget_control,bopr1 ,set_uvalue=[78, 0,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2] ;Select 1
  widget_control,bopr1T,set_uvalue=[78,-1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2] ;Factor 1
  widget_control,boprm ,set_uvalue=[78, 1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2] ;Operators
  widget_control,boprp ,set_uvalue=[78, 1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2]
  widget_control,boprd ,set_uvalue=[78, 1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2]
  widget_control,boprs ,set_uvalue=[78, 1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2]
  widget_control,boprc ,set_uvalue=[78, 1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2]
  widget_control,bopr2 ,set_uvalue=[78, 2,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2] ;Select 2
  widget_control,bopr2T,set_uvalue=[78,-1,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2] ;Factor 2 or minus constante
  widget_control,boprdo,set_uvalue=[78, 3,bopr1,bopr1T,opropr,bopr2,bopr2T,blabo1,blabo2]
 ;*********

  TAB5  = widget_base(  tabase,title='MODELS')   ;MODELS
  tab5c = widget_base(  TAB5  ,/column)
  bid   = widget_label( tab5c ,value=' ',font=ft_smallest)
  base45= widget_base(  tab5c ,/row)
  pdsp  = widget_button(base45,font=ft_b_normal          ,value='P Dispersity (None)     ',/menu)
     bid= widget_button(pdsp  ,font=ft_propor            ,value='None'       ,uvalue=[79,0,pdsp,1])
     bid= widget_button(pdsp  ,font=ft_propor            ,value='Gaussian'   ,uvalue=[79,1,pdsp,1])
     bid= widget_button(pdsp  ,font=ft_propor            ,value='Log_Norm'   ,uvalue=[79,2,pdsp,1])
     bid= widget_button(pdsp  ,font=ft_propor            ,value='Schultz '   ,uvalue=[79,3,pdsp,1])
  bid   = widget_label( base45,value=' ')

  Fstruc= widget_button(base45,font=ft_b_normal          ,value='F Structure (None)      ',/menu, uvalue=[0L])
    but9= widget_button(Fstruc,font=ft_b_normal,value='None'                              ,uvalue=[79,0,Fstruc,2])
    but9= widget_button(Fstruc,font=ft_b_normal,value='Hard_SPHERE'                       ,uvalue=[79,1,Fstruc,2])
   ;widget_control,but9,sens=0
    but9= widget_button(Fstruc,font=ft_b_normal,value='Multi_LAMELLAR (thermal disorder)' ,uvalue=[79,2,Fstruc,2])
    widget_control,but9,sens=0
    but9= widget_button(Fstruc,font=ft_b_normal,value='Multi_LAMELLAR (para crystalline)' ,uvalue=[79,3,Fstruc,2])
    widget_control,but9,sens=0
    but9= widget_button(Fstruc,font=ft_b_normal,value='Multi_LAMELLAR (CAILLE theorie)'   ,uvalue=[79,4,Fstruc,2])
    widget_control,but9,sens=0
    but9= widget_button(Fstruc,font=ft_b_normal,value='MSA (rescaled)'                    ,uvalue=[79,5,Fstruc,2])
    widget_control,but9,sens=0
    
  bid   = widget_label( base45,value=' ')
  
 ;if uv(1) eq 25 then if uv(2) ge 0 then baset.Fmodel=[uv(2),uv(3)] else baset.Fmodel(1)=uv(3)     ;Fitting or showing DeltaQ
 ;**************
  maud  = widget_button(base45,font=ft_b_normal          ,value='Model',/menu)
    str1= widget_button(maud  ,font=ft_propor            ,value='-Use STR_FIT to choose a model (one Sample merged)'  ,uvalue=[77,25,100, 0])
    str2= widget_button(maud  ,font=ft_propor            ,value='-Use STR_FIT to choose a model (independent curves)' ,uvalue=[77,25,100, 1])
     bid= widget_button(maud  ,font=ft_propor            ,value='')
    mau0= widget_button(maud  ,font=ft_propor            ,value='-Gauss , Lorentz , Voigt' ,/menu) ;& widget_control,mau0,sens=0
    ma01= widget_button(mau0  ,font=ft_propor            ,value='- Gaussian'                                ,uvalue=[77,25, 2, 1])
    ma02= widget_button(mau0  ,font=ft_propor            ,value='- Lorentzian'                              ,uvalue=[77,25, 3, 1])
    ma03= widget_button(mau0  ,font=ft_propor            ,value='- Pseudo-Voigt'                            ,uvalue=[77,25, 4, 1])

     bid= widget_button(maud  ,font=ft_propor            ,value='')

    mau1= widget_button(maud  ,font=ft_propor            ,value='-Shape-Independent (ONE SAMPLE)' ,/menu)
    bid = widget_button(mau1  ,font=ft_b_normal          ,value='Q RANGE IS USED EVEN IF NOT SELECTED')
    bid = widget_button(mau1  ,font=ft_propor            ,value=' ')
    mau11=widget_button(mau1  ,font=ft_propor            ,value='- Guinier approximation'                   ,uvalue=[77,25,11, 0])
    mau12=widget_button(mau1  ,font=ft_propor            ,value='- Porod limit'                             ,uvalue=[77,25,12, 0])
    mau13=widget_button(mau1  ,font=ft_propor            ,value='- Power law'                               ,uvalue=[77,25,13, 0])
    mau14=widget_button(mau1  ,font=ft_propor            ,value='- Debye (for linear polymer chain)'        ,uvalue=[77,25,14, 0])
    mau15=widget_button(mau1  ,font=ft_propor            ,value='- Teubner-Strey'                           ,uvalue=[77,25,15, 0])
    mau16=widget_button(mau1  ,font=ft_propor            ,value='- Invariant'                               ,uvalue=[77,25,16, 0])
    mau1= widget_button(maud  ,font=ft_propor            ,value='-Shape-Independent (BY CURVES)' ,/menu)
    bid = widget_button(mau1  ,font=ft_b_normal          ,value='Q RANGE IS USED EVEN IF NOT SELECTED')
    bid = widget_button(mau1  ,font=ft_propor            ,value=' ')
    mau11=widget_button(mau1  ,font=ft_propor            ,value='- Guinier approximation'                   ,uvalue=[77,25,11, 1])
    mau12=widget_button(mau1  ,font=ft_propor            ,value='- Porod limit'                             ,uvalue=[77,25,12, 1])
    mau13=widget_button(mau1  ,font=ft_propor            ,value='- Power law'                               ,uvalue=[77,25,13, 1])
    mau14=widget_button(mau1  ,font=ft_propor            ,value='- Debye (for linear polymer chain)'        ,uvalue=[77,25,14, 1])
    mau15=widget_button(mau1  ,font=ft_propor            ,value='- Teubner-Strey'                           ,uvalue=[77,25,15, 1])
    mau16=widget_button(mau1  ,font=ft_propor            ,value='- Invariant'                               ,uvalue=[77,25,16, 1])

     bid= widget_button(maud  ,font=ft_propor            ,value='')

    mau2 =widget_button(maud  ,font=ft_propor            ,value='-Form Factors (no initial values)' ,/menu)
          widget_control,Fstruc, set_uvalue=[mau2]       ; To deasable this entry if user got a Structure Factor
    mau21=widget_button(mau2  ,font=ft_propor            ,value='- SPHERE (merge selected  and show model)' ,uvalue=[77,25, 0, 0])
    mau22=widget_button(mau2  ,font=ft_propor            ,value='- SPHERE (Show Radius of selected curves)' ,uvalue=[77,25, 0, 1])
   ;mau23=widget_button(mau2  ,font=ft_propor            ,value='- SPHERE (fit2D 2,3 or 4 selected conf.)'  ,uvalue=[77,25, 0, 2])
    mau24=widget_button(mau2  ,font=ft_propor            ,value='- Hollow SPHERE (merging)'                 ,uvalue=[77,25, 1, 0])
    mau25=widget_button(mau2  ,font=ft_propor            ,value='- Hollow SPHERE (one by one)'              ,uvalue=[77,25, 1, 1])

    but9 =widget_button(mau2  ,value='- Bilayered VESICLE'      ,font=ft_propor)
    widget_control,but9 ,sens=0
    but9 =widget_button(mau2  ,value='- Spherical SHELL (linear varying contrast)'      ,font=ft_propor)
    widget_control,but9 ,sens=0
    but9 =widget_button(mau2  ,value='- Multi LAMELLAR VESICLE',font=ft_propor)
    widget_control,but9 ,sens=0
    bid  =widget_button(mau2  ,value='-'                        ,font=ft_propor)
    but9 =widget_button(mau2  ,value='- ELLIPSOID'              ,font=ft_propor)
    widget_control,but9 ,sens=0
    but9 =widget_button(mau2  ,value='- Ellipsoidal core SHELL' ,font=ft_propor)
    widget_control,but9 ,sens=0
    bid  =widget_button(mau2  ,value='-'                        ,font=ft_propor)
    but9 =widget_button(mau2  ,value='- STAR POLYMER (gaussian statistic)'              ,font=ft_propor     ,uvalue=[77,25,46, 0])
    but9 =widget_button(mau2  ,value='- STAR polydisperse POLYMER (gaussian statistic)' ,font=ft_propor     ,uvalue=[77,25,47, 0])
    bid  =widget_button(mau2  ,value='-'                        ,font=ft_propor)
    but9 =widget_button(mau2  ,value='- DISK'                   ,font=ft_propor                             ,uvalue=[77,25,48, 0])
    but9 =widget_button(mau2  ,value='- ROD'                    ,font=ft_propor                             ,uvalue=[77,25,49, 0])
    but9 =widget_button(mau2  ,value='- CYLINDER'               ,font=ft_propor)
    widget_control,but9 ,sens=0
    but9 =widget_button(mau2  ,value='- Hollow CYLINDER'        ,font=ft_propor)
    widget_control,but9 ,sens=0


 ;widget_control,tabase, SET_TAB_CURRENT=0

  endif else begin ;NoTabs
    base11=widget_base(junkRO,/row)
    bopr1 =0L & bopr2=0L
  endelse
  
   but  = widget_button(base11,value='Plotting Tools'               ,font=ft_b_normal,/menu)
   bid  = widget_button(but   ,value='THIS CONCERN SELECTED CURVES' ,font=ft_b_normal) & widget_control,bid,sens=0
   bid  = widget_button(but   ,value='')
   bit  = widget_button(but   ,value='Plot the curves on 3D axis (Off)'    ,font=ft_b_normal,uvalue=[77,14,2])
   bit  = widget_button(but   ,value='Overplotting with Suprplot (W21)'    ,font=ft_b_normal,uvalue=[77,22])
   bid  = widget_button(but   ,value=' ')
   bit  = widget_button(but   ,value='Annotation Tool'                     ,font=ft_b_normal,uvalue=[77,14,1])
   bid  = widget_button(but   ,value=' ')
   bit  = widget_button(but   ,value='Surface Selected Kinetic runs'       ,font=ft_b_normal  ,uvalue=[77,26])
   bit  = widget_button(but   ,value='Surface Selected Kinetic time scale' ,font=ft_b_normal  ,uvalue=[77,27])

   bid  = widget_label( base11,value=' ')
   but  = widget_button(base11,value='Print'                  ,font=ft_b_normal,uvalue=[77,12])
  label = widget_label( base11,value=' '                      ,font=ft_propor)

  ROnexc= widget_base(  base11,/nonexclusive,/row)
  ROerr = widget_button(ROnexc,value='Error'                  ,font=ft_b_normal,uvalue=[77,2,0])
  ROstyl= widget_button(ROnexc,value='Symbol'                 ,font=ft_b_normal,uvalue=[77,6,0])
  ROlrp = widget_button(ROnexc,value='Lrp'                    ,font=ft_b_normal,uvalue=[77,6,1])
  widget_control,ROerr ,set_button=0
  widget_control,ROstyl,set_button=1
  widget_control,ROlrp ,set_button=1

 ;OTHER SETTINGS
 ; -------------
  base41= widget_base(  junkCF,/row)

  if (not sheet) then begin
                                                                     ;TEXTuv=[9, bases3 index of its LABEL, Wrkspace, OL]
   SlabFX= widget_label( base41,value='(FLUX Trs. run)'              ,font=ft_propor,uvalue={data:0,datp:0,opr:' ',wi:26,par:'',opt:0})
   SampFX= widget_text(  base41,value=''     ,xsize=15               ,font=ft_propor,/editable,uvalue=[9,42,26,0L])
  ;trefq= widget_button(widget_base(base41,/nonexclusive),value='Correct sample thickness: F(2theta)',font=ft_b_normal,uvalue=[95])
  ;       widget_control,trefq,set_button=sans_supl[43]
  endif else $
   bid  = widget_button(base41, value='Treatment Options', font=ft_b_normal, uvalue=[90])

  label = widget_label( base41,value=' '                          ,font=ft_propor)

  base41= widget_base(  junkCF,/row) & widget_control,base41,sens=1-sheet
  label = widget_label( base41,value='Sample'                     ,font=ft_b_normal)
  labil = widget_label( base41,value='thickness(cm)'              ,font=ft_b_normal) ;& widget_control,labil,sens=0
  bjThiS= widget_text(  base41,value=thinS,xsize=3                ,font=ft_propor,/editable,uvalue=[11,11,labil])
  labil = widget_label( base41,value=' '+txtwater+' thickness'    ,font=ft_b_normal) ;& widget_control,labil,sens=0
  bjThiW= widget_text(  base41,value=thinW,xsize=3                ,font=ft_propor,/editable,uvalue=[11,10,labil])

  base41= widget_base(  junkCF,/row) & widget_control,base41,sens=1-sheet
   label= widget_label( base41,value='EDITOR for Transmission and Thickness',font=ft_b_normal)
     but= widget_button(base41,value='Sample series'              ,font=ft_b_normal,uvalue=[46,4,1])
  base41= widget_base(  junkCF,/row)
;     bid= widget_button(base41,value='?'                              ,font=ft_b_normal,/menu)
;     bod= widget_button(bid,   value='Defining Sectors'               ,font=ft_b_normal)
;     bod= widget_button(bid,   value=''                               ,font=ft_propor)
;     bod= widget_button(bid,   value='>TR | SR | OP | NS | IR | OR | SP',font=ft_propor)
;     bod= widget_button(bid,   value='TR =Total  rotation [0-90]'     ,font=ft_propor)
;     bod= widget_button(bid,   value='SR =Single rotation [0-90] 2 sectors',font=ft_propor)
;     bod= widget_button(bid,   value='OP =Opening [0-90]'             ,font=ft_propor)
;     bod= widget_button(bid,   value='NS =#  of  sectors'             ,font=ft_propor)
;     bod= widget_button(bid,   value='IR =Inner  radius'              ,font=ft_propor)
;     bod= widget_button(bid,   value='OR =Outer  radius'              ,font=ft_propor)
;     bod= widget_button(bid,   value='SP =Separate symmetrical sectors [0,1]',font=ft_propor)
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsTr = widget_text  (base41,value=secTR,xsize=3                    ,font=ft_propor,/editable,uvalue=[11,13,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsSr = widget_text  (base41,value=secSR,xsize=3                    ,font=ft_propor,/editable,uvalue=[11,14,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsOp = widget_text  (base41,value=secOP,xsize=3                    ,font=ft_propor,/editable,uvalue=[11,15,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsNs = widget_text  (base41,value=secNS,xsize=2                    ,font=ft_propor,/editable,uvalue=[11,16,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsIr = widget_text  (base41,value=secIR,xsize=3                    ,font=ft_propor,/editable,uvalue=[11,17,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsOr = widget_text  (base41,value=secOR,xsize=3                    ,font=ft_propor,/editable,uvalue=[11,18,label])
;   label= widget_label (base41,value='|'                              ,font=ft_smaller) & widget_control,label,sens=0
;   bsSp = widget_text  (base41,value=secSP,xsize=1                    ,font=ft_propor,/editable,uvalue=[11,19,label])


 ;SECTORS
 ; ------
  base41= widget_base(  junkSC,/row) & dim=64 & mx=1+round(sqrt(2)*dim) & ys=40
   asl  =widget_slider( base41,value=0.   ,title="Rotation 0.1 deg"      , min=-900,max=900,/drag,font=ft_b_normal,xsize=130)
   gsl  =widget_slider( base41,value=90.  ,title="Single rotation"       , min=0,max=90 ,/drag,font=ft_b_normal,xsize=130)
   bsl  =widget_slider( base41,value=30.  ,title="Opening"               , min=0,max=45 ,/drag,font=ft_b_normal,xsize=85 )
  ;widget_control,default_font=ft_b_normal
  ;asl  =CW_Fslider    (base41,value=0.   ,title="Total rotation",  min=0,max=90,/drag,xsize=130,format='(F5.2)',/edit,scroll=.25)
  ;gsl  =CW_Fslider    (base41,value=90.  ,title="Single rotation", min=0,max=90,/drag,xsize=130,format='(F5.2)',/edit,scroll=.25)
  ;bsl  =widget_slider (base41,value=30.  ,title="Opening",         min=0,max=45,/drag,font=ft_b_normal,xsize=85 )
  base41= widget_base(  junkSC,/row)
   r1ID =widget_slider( base41,value=dim/6,title="Inner radius",    min=0,max=mx,/drag,font=ft_b_normal,xsize=130);,scr_ysize=ys)
   r2ID =widget_slider( base41,value=dim/2,title="Outer radius",    min=0,max=mx,/drag,font=ft_b_normal,xsize=130);,scr_ysize=ys)
   zonID=widget_slider( base41,value=2    ,title="# of sectors",    min=1,max=8 ,/drag,font=ft_b_normal,xsize=85 );,scr_ysize=ys)

  widget_control,asl, pro_set_value='ben_int_slider_set', func_get_value='ben_int_slider_get'

  sansBen=fltarr(100)
  sansBen[0]=base0 & sansBen[1:10]=[asl,gsl,bsl,zonID,r1ID,r2ID, 0L, bcentx,bcenty, dcs]

 ;BATCH MACROS
 ; -----------
  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='   '                        ,font=ft_propor) & widget_control,label,sens=0
  bjcent= widget_text(  base41,value='sans_settings,/nowind,inst="'+instru+'"'  ,xsize=33,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='sans_settings, /nowind, instrument="'+instru+'"',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='First call before using other sans_macros',font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Ctr'                        ,font=ft_propor ,uvalue=[74,1])
  bjcent= widget_text(  base41,value='c=sans_center (w5,/fit,xr=[1,64])'  ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='c=sans_center(W1) keywords:',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/Fit'                      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/Gravity , /Mask'          ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Xrange=[1,n]'              ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Yrange=[1,m]'              ,font=ft_b_normal)
  ;bid  = widget_button(bjdo  ,value='More ...'                   ,font=ft_b_normal,uvalue=[18,1])

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Msk'                        ,font=ft_propor ,uvalue=[74,2])
  bjmask= widget_text(  base41,value='sans_mask, xline=10, xybeam=[5,6]'  ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='sans_mask,     keywords:'   ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Instrument="'+instru+'"'   ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Xline=[1,64]'              ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Yline=[1,64]'              ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Box=[x,y , nx,ny]'         ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',WaterBox=[x,y , nx,ny]'    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',XYbeam=[5,6]'              ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',XYbeam=[x,y , nx,ny]'      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/Back'                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/Clear'                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/ellipse'                  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/rectangle'                ,font=ft_b_normal)
  ;bid  = widget_button(bjdo  ,value='More ...'                   ,font=ft_b_normal,uvalue=[18,2])

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Att'                        ,font=ft_propor ,uvalue=[74,1])
  bjtran= widget_text(  base41,value='t=sans_transm (Sbeam=5,sample=3)'       ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='t=sans_transm(Sbeam=5,) keywords:',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',water  =1'                 ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Ewater =2'                 ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',sample =3'                 ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Esample=4'                 ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Wbeam  =6 , /Mask'         ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Xrange =[1,n]'             ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Yrange =[1,m]'             ,font=ft_b_normal)
  ;bid  = widget_button(bjdo  ,value='More ...'                   ,font=ft_b_normal,uvalue=[18,3])

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                        ,font=ft_propor ,uvalue=[74,3])
  bjcorr= widget_text(  base41,value='w18=sans_cor (water=10,Ewater=11,sample=12,Esample=13,scadmium=14)',xsize=33,uvalue=[14,0] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='w18=sans_cor(wat=10,Ewat=11,samp=12,Esamp=13,scad=14,wcad=15)',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='dead_time corrections'      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='empty cell subtractions'    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=txtCadm+' subtractions'      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='detector efficiency correction'  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='solid angle corrections'    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='thickness corrections'      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='*MISSING CORRECTIONS  --->' ,font=ft_normal)
   bid  = widget_button(bjdo  ,value='Multiple scattering'        ,font=ft_normal)
   bid  = widget_button(bjdo  ,value='Incoherent background'      ,font=ft_normal)
   bid  = widget_button(bjdo  ,value='...'                        ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='(Plot display result in Lamp)'   ,font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                         ,font=ft_propor ,uvalue=[74,3])
  bjgrpr= widget_text(  base41,value='W19=sans_radial (w18,increment=1.)',xsize=33,uvalue=[14,0] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='w19=sans_radial(w18,incr=1.)',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                            ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Options:'                    ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Mask     =M  ;your own mask',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Increment=1  ;(pixel)'      ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',/NoMask  ;ignore current mask'  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Center   =[32.,32.] ;(pix)' ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Distance =17.       ;(m)'   ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',PixSize  =[1.,1.]   ;(cm)'  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=',Sector   =[r1,r2,angle1,angle2]',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                             ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='(Plot display result in Lamp)',font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                         ,font=ft_propor ,uvalue=[74,3])
  bjsave= widget_text(  base41,value='sans_write_settings, "'+strlowcase(instrus)+'set_conf.sav"' ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Save current settings in a binary file' ,font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                         ,font=ft_propor ,uvalue=[74,3])
  bjrest= widget_text(  base41,value='sans_read_settings, "'+strlowcase(instrus)+'set_conf.sav"'  ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Restore settings from file'  ,font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                         ,font=ft_propor ,uvalue=[74,3])
  bjexpo= widget_text(  base41,value='sans_save_gt, w19'  ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='sans_save_gt, w19'           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='sans_save_gt, w18'           ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                            ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Export integration result in g0numor.seq format',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Export corrected   data   in t0numor.seq format',font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Dat'                         ,font=ft_propor ,uvalue=[74,3])
  bjimpo= widget_text(  base41,value='w20=sans_read_gt("filename")',xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='w20=sans_read_gt("g006923.001")',font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                            ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Import results from a g0numor.seq file'  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='You can then compare workspaces using superplot',font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='Msk'                         ,font=ft_propor ,uvalue=[74,2])
  bjRmsk= widget_text(  base41,value='sans_read_mask,"'+strlowcase(instrus)+'_numor.msk"'  ,xsize=33,uvalue=[14,1] ,font=ft_propor,/editable)
  bjdo  = widget_button(base41,value='?',/menu                     ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='sans_read_mask,"'+strlowcase(instrus)+'_numor.msk"'  ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value=''                            ,font=ft_b_normal)
   bid  = widget_button(bjdo  ,value='Set the mask file as the current mask'  ,font=ft_b_normal)

  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='DO '                         ,font=ft_propor) & widget_control,label,sens=0
  bjDo1 = widget_text(  base41,value=' '  ,xsize=36,uvalue=[14,1]  ,font=ft_propor,/editable)
  base41= widget_base(  junkBT,/row)
  label = widget_button(base41,value='DO '                         ,font=ft_propor) & widget_control,label,sens=0
  bjDo2 = widget_text(  base41,value=' '  ,xsize=36,uvalue=[14,1]  ,font=ft_propor,/editable)

  bid0  = widget_base(map=0) & wview0=widget_draw(bid0,xsize=dcs,ysize=dcs,retain=2) ;for zooming

  if SHEET then widget_control,base0,tlb_set_xoffset=770
  widget_control,lamp_b1,iconify=1
  widget_control,base0,group_leader=lamp_b1,/realize
  widget_control,bid0 ,group_leader=base0  ,/realize
  if not SHEET then widget_control,base0,iconify=0

  if oktips then begin
      ;widget_control,b_bar2,Tooltip=' CALCULATE TRANSMISSIONS from attenuated Data'
       widget_control,ahelp ,Tooltip=' BeamS is empty beam at sample distance if you define center'
       widget_control,mhelp ,Tooltip=' Beam: use Beamstop for sample , Box for '+txtwater+' '
       widget_control,bzin  ,Tooltip=' (Then drag image with the left mouse button) '
      ;widget_control,wview4,Tooltip=' CLICK LEFT:Results List and Plot Options , RIGHT:Access to e_logbook'
       widget_control,bas4CC,Tooltip=' To set your own Center, enter one value in X&Y range then set CENTER'
       if bopr1 gt 0L then $
       widget_control,bopr1 ,Tooltip=' Get current selected curves as operand 1'
       if bopr2 gt 0L then $
       widget_control,bopr2 ,Tooltip=' Get current selected curve(s) as operand 2'
       widget_control,ROlrp ,Tooltip=' Discard Low Resolution Points for plotting'
       widget_control,secRk ,Tooltip=' Rocking implies a total integration of sectors'
       
      if sys_dep('VERSION') ge 6.0 then begin
       widget_control,msampl ,/context_event & widget_control,mwater ,/context_event
       widget_control,asampl ,/context_event & widget_control,awatr  ,/context_event
       widget_control,asampe ,/context_event & widget_control,awatre ,/context_event
       widget_control,abeams ,/context_event & widget_control,abeamw ,/context_event
       widget_control,bsampl ,/context_event & widget_control,bwatr  ,/context_event
       widget_control,bsampe ,/context_event & widget_control,bwatre ,/context_event
       widget_control,bcadmi ,/context_event & widget_control,bcadmw ,/context_event
      endif
  endif

  widget_control,wview0,get_value= view0 & cur_zoom=[1L,wview0,view0,0,0,0,0,0] ;for zooming (not mapped)
  widget_control,wview ,get_value=bview  ;Mini view (config)
  widget_control,wview2,get_value=bview2 ;Detector
  widget_control,wview3,get_value=bview3 ;Cbar detector
  widget_control,wview4,get_value=bview4 ;Plot view (result list)
  widget_control,wview5,get_value=bview5 & sansBen[7]=bview5 ;Sectors
  widget_control,wview6,get_value=bview6 ;Cbar sectors

  if beammod then widget_control,bbeamx,editable=0
  if beammod then widget_control,bbeamy,editable=0

  cd,current=mee
 ;Views + miscellaneous
  bases1 =[bmess,bcurf,bwrtf,binst,bfilt,bcolim,base0,basej1,basej2,bview2,bview3,btitl,bcoord,butf,bview4,bcurv,bview5,$ ;0->16
           bview6, wview0,wview,wview2,wview3,wview4,wview5,wview6,0L,0L,0L,0L,0L,0L,0L,0L,0L,0L] ;17->24
 ;Config
  bases2 =[bwave,bdist,bcentx,bcenty,bmask,bview,boffs,bangl,bbeamx,bbeamy,bupdt,btrans[0:3],atrans[0:3],bvcoll,bvthiW,bvthiS,$ ;0-21
           bvcolW,bvdisW,bconf1,bconf2,bconf3,0L,0L,0L,0L,0L] ;22->26
 ;Runs
  bases3 =[bwatr ,bwatre,bsampl,bsampe,bcadmi,bcadmw,  blabw,blabwe,blabs,blabse,blabc ,blabcw,$   ;0 ->11 sample
           awatr ,awatre,asampl,asampe,abeamS,abeamW,  alabw,alabwe,alabs,alabse,alabbS,alabbW,$   ;12->23 transmission
                         msampl,mwater,                             mlabs,mlabw ,confus,$          ;24->28 mask ,center
           Wwatr2,Wwatr3,Wwate2,Wwate3,Wcadc2,Wcadc3,  Wlabw2,Wlabw3,Wlabe2,Wlabe3,Wlabc2,Wlabc3,$ ;29->40 water alternatives
                                              SampFX,                                     SlabFX,$ ;41->42 FLUX  alternatives
           0L,0L,0L,0L,0L,0L,0L,0L]

 ;Maps and Sens for Data
  bases4 =[junkA,junkD,jun1TR,jun2TR, labelB,base31,base32,junkM,junkST,bSTsho,junkVT,bVTsho,junkTX,bTXsho,$ ;0->13
           abeamW,alabbW,junkRS,bRSsho, labkM2,seckM2,baseK0,seckS,0L,0L,0L,0L,0L,0L] ;14->21

 ;Center Attenuation Mask Config large view resume MAPS
  bases5 =[junkCE,junkAT,junkMS,junkCF,junkW1,junkW2,junkBT,junkSC,junkW3,junkCO,baseM1,baseM2,baseM3,baseM4, $ ;0-13
           msampl,mwater,lab4CC,basCfg,basofa,junkRO, MchkE, labXB1,basXB1,labYB1,basYB1,labXYB,basXYB,Econf,0L,0L,0L,0L,0L,0L,0L,0L,0L] ;14->27

 ;         0      1      2      3      4    5     6      7      8    9    10    11    12  13              14  15      16    17    18   19       20     21      22   23
 ;Center options  + toggles           ;fit ,grav ,Cmask, Tmask, log, d_t, s_a,  scal, Err Q_Qlog_Q^2_Q^4  efq unScale Pstyl Fincr LRP  M/T      reRead Squared Rock FREE
  bases6 =[basXC1,basXC2,basYC1,basYC2,bfit,1-bfit,   1,     1,   0,   1, SAc, fscal,  0,  0              ,1 ,unScal  ,1    ,0  ,1   ,Mon_Tim ,0     ,0      ,0   ,0,0,0,0,0,0,0,0]
                                      ;[carefull mod[4:*]]

 ;Attenuation options + Sectors + other configs             ; bsTR,bsSR,bsOP,bsNS ,bsIR,bsOR,bsSP]
  bases7 =[basXA1,basXA2,basYA1,basYA2, bjThiW,bjThiS,bjCros, asl ,gsl ,bsl ,zonID,r1Id,r2Id, 0, $ ;0->13
           bjRadi,bjAtSF,bTauv,bTauf, bCbidf,bCbidg,bCbidm, bTbidm, baslog, trddt,trsag,trboc,trefq,blistR,truns,labeCA,$ ;14->29
           Nmon,Ntim,bQdep,secSq,secRk,0L,0L,0L,0L,0L,0L,0L] ;30->41
 ;Mask + Resume options
  bases8 =[basXB1,basYB1,basXL1,basYL1,basXYB, ROerr,ROQ,ROlog,ROsqr,ROstyl,ROp4,0L,0L,0L,0L,0L,0L,0L,0L,0L]

 ;Other Center + transm +slider series
  bases9 =[bfxrgc,bfxrga,bfxlin,bfxtg,lamp_b1,bidat1,bidat2,bidat3,bidat4, bsliD,bsliDN,bsliB,bsliBN, VET, 0L,0L,0L,0L,0L,0L,0L,0L,0L,0L]

  baset  ={bases1:bases1,bases2:bases2,bases3:bases3,bases4:bases4,bases5:bases5,bases6:bases6,$
           bases7:bases7,bases8:bases8,bases9:bases9,path:mee,factor:fct,idxi:idxi, regSheet:regSheet,$
           collim:collima,colidx:colidx,colval:float(collim[colidx]),attenu:attenua,txtWater:txtWater,txtCadm:txtCadm,$
           curLMH:0, savedLMH:[0,0,0], Qdep:Qdep, curWATER:1, curSAMtrs:0.,    Fdisper:0, Fmodel:[0,0], Fstruct:0    ,$
           IofQ:0, FIQ:0, Qrange:[0,sclQr1,sclQr2],Irange:[0,sclIr1,sclIr2], PrintON:0, ITOOLS:0, RepStyle:0, XWdep:XWdep, attenuF:attenuF}
           
  widget_control,base0 ,set_uvalue=baset
  widget_control,bcolim,set_list_select=colidx
  stri=widget_info(base0 ,/geometry)
 ;strj=widget_info(basej1,/geometry)
  wset,kpwin

  cur_run =0 & cur_log =0 & cur_eff=1.
  
 ;cur_menu(0)  Ranges/box input-> 0:Center Rg 1:Transm Rg 2:Mask box/beam 3:Scatt data
 ;             6:Batch macros 7:Sectors 9:Config 3buts 19:we are in LIST of RESULTS
 ;cur_menu(1)  Views 4:2d view 5:Averaging 8:Sector view
 ;cur_menu(2)  Averaging option 2:2PI Rad 4:2PI Azim 5:Sec tot sym 6:Sec tot asym 7:Sec rad sym 8:Sec rad asym
 ;cur_menu(3:5)Hide mask if 1. 3:Sample 4:Detector 5:Water
 ;cur_menu(6)  While in Mask 1:Sample beam 2:Water beam 3:Detect all conf 4:Detect cur conf
 ;cur_menu(7)  Workspace number i.e. 3,12,7,8,17,18
 ;cur_menu(8)  Enable efficiencies correct =1
 ;cur_menu(9)  cur_menu(2) curlist  71:List of Results 77:Restoring Results !!!
 ;cur_menu(10) data input-> 0:Center menu 1:Transmition menu 2:Mask menu 3:Data menu -1:Averaging
  cur_menu=[0,4,0  ,1,0,1  ,1,0 ,0,0 ,0]
  
  appTLB  =1      & secVIEW =bView5           ;FOR BEN...
  dataSiz1=0      & dataSiz2=0 & free=0       ;LIVC appTLB, Vsector, siz1     , siz2     ,draw_size(sector)
  cur_cxy =float([centex-FX, centey-FX, beamf, appTLB     , secVIEW, dataSiz1 , dataSiz2 ,cur_fct[3]       ,free])
  cur_wn =0
 ;wins=0 ;!!!!!
 ;
 ;cur_fct(0:1)  Factors between Ben sectors and real data size
 ;cur_fct(2)    128
 ;cur_fct(3,4)  Current view size 256<->384
 ;cur_fct(5:8)  Used for resizing 256<->384
 ;cur_fct(9:10) Current fct 128/[dataSZ1,dataSZ2]

  if not wins then cur_fct[5:6]=[stri.scr_xsize+2*stri.margin+stri.xpad, stri.scr_ysize+2*stri.margin] $
              else cur_fct[5:6]=[stri.xsize                            , stri.ysize]      ;default size of sans_settings
  if cur_fct[3] gt cur_fct[4] then cur_fct[5:6]=cur_fct[5:6]-(cur_fct[3] - cur_fct[4])+10 ;small   size
  cur_fct[7:8] =[stri.scr_xsize+2*stri.margin+stri.xpad , stri.scr_ysize+2*stri.margin]   ;current size
  cur_fct[9:10]=fct                                                                       ;current data fct
  widget_control,binst,set_value=instru
  sans_reform_series,clear=-1
 ;sans_settings_swap, baset,/reset
  trap_set,smoo=0 ; , fill=0
  xmanager, 'sans_settings_win', base0,event_handler='sans_settings_event',CLEANUP='SANS_DYING',/just_reg,/no_block
             sans_settings_mask, baset, run=0
             sectors=1
             if sectors then begin & sans_sector & p_ben_event,0,[-88,401,0]
                siz0=round([cvh,cvh]/baset.factor)
                if siz0[0] eq siz0[1] then data=dist(siz0[0]) else data=findgen(siz0[0],siz0[1])
                widget_control,bad_id=ii,get_uvalue=varAccess,sansBen
                bid=int_loadRequest(varAccess,0,data,findgen(siz0[0])+1,findgen(siz0[1])+1,0,0,['0','0',' '])
             endif else sansBen=0
 ;**********           
  endif else sans_settings_mask, baset ;endif not fresh
 ;**********
endif ;registered
baseSANS=base0
if mappi ge 0 then widget_control,baseSANS,iconify=1-mappi
end
