;  liv_objet.pro  v 1.0
;  Source:  d_obj.pro,v 1.10 1999/01/16 01:12:20 scottm Exp
;  Modified 2001/02/../11  D.Richard
;  Modified 2012  E.Pellegrini D.Richard

;  ORB Written by: RF, September 1996.
;   To initially create:   oOrb = OBJ_NEW('orb')
;   To retrieve a property value: oOrb->GetProperty
;   To set a property value:  oOrb->SetProperty
;   To print to the standard output stream the current properties of
;   the orb:       oOrb->Print
;   To destroy:    OBJ_DESTROY, oOrb
;
; KEYWORD PARAMETERS:
;   ORB::INIT:
;   <Note that keywords accepted by IDLgrModel::Init and/or
;    IDLgrPolygon::Init are also accepted here.>
;   POS:  A three-element vector, [x,y,z], specifying the position
;               of the center of the orb, measured in data units .
;               Defaults to [0,0,0].
;   RADIUS: A floating point number representing the radius of the
;               orb (measured in data units).  The default is 1.0.
;   DENSITY:A floating point number representing the density at which
;               the vertices should be generated along the surface of the
;               orb.  The default is 1.0.
;   TEX_COORDS: Set this keyword to a nonzero value if texture map
;               coordinates are to be generated for the orb.
;
;   ORB::GETPROPERTY: (POS= RADIUS= DENSITY=)
;   ORB::SETPROPERTY: (POS= RADIUS= DENSITY=)
;   <Note that keywords accepted by IDLgrModel::SetProperty and/or
;    IDLgrPolygon::SetProperty are also accepted here.>
; EXAMPLE:
;   Create an orb centered at the origin with a radius of 0.5:
;     oOrb = OBJ_NEW('Orb', POS=[0,0,0], RADIUS=0.5)
; ----------------------------------------------------------------------------
PRO d_livConst
;+
;** **********
;** Define constants
;**
;-
    common d_liv_con, as_rad, af_rad, af_lng, s_den, a_den
    common d_liv_keepath, kpath
    as_rad=1.
    af_rad=1./1.6
    af_lng=11. ;9.5
    s_den =1.
    a_den =1.5
    kpath={kppth:'',kpcol:([135,135,205]-120+1)>0<255}
end

; ---------------------------------------------------------------------------
;This function returns a 1 if initialization is successful, or 0 otherwise.
;******* *********
FUNCTION ellapsoid::Init, POS=pos, RADIUS=radius, ABC=abc, ANGLES=angles, DENSITY=density, TEXTURE=texture, _EXTRA=e
;+
;******* *********
;** define an ellipsoid
;-
    if (self->IDLgrModel::Init(_EXTRA=e) ne 1) then return, 0

    if (n_elements(pos) eq 3)     then self.pos = float(pos)

    if (n_elements(radius) eq 1)  then self.radius = radius

    self.radini  = self.radius

    if (n_elements(abc) eq 3)     then self.abc = float(abc)

    if (n_elements(angles) eq 3)  then self.angles = float(angles)

    if (n_elements(density) eq 1) then self.density = density

    self.texture = keyword_set(texture)

    ; Initialize the polygon object that will be used
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, /REJECT, _EXTRA=e)

    self->Add,self.oPoly

    ; Build the polygon vertices and connectivity based on property settings.
    self->BuildPoly

    RETURN, 1
END

; ---------------------------------------------------------------------------
;Cleans up all memory associated with the ellipsoid.
;** ************
PRO ellapsoid::Cleanup
;+
;** *********
;** ellipsoid stuff
;-

    ; Cleanup the polygon object used to represent the ellipsoid.
    OBJ_DESTROY, self.oPoly

    ; Cleanup the superclass.
    self->IDLgrModel::Cleanup
END

; ---------------------------------------------------------------------------
;Sets the value of properties associated with the ellipsoid object.
;** ****************
PRO ellapsoid::SetProperty, POS=pos, RADIUS=radius, ABC=abc, ANGLES=angles, DENSITY=density, TEXTURE=texture, _EXTRA=e
;+
;** *********
;** ellipsoid stuff
;-

    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e

    if (n_elements(pos) eq 3)     then self.pos = float(pos)

    if (n_elements(radius) eq 1)  then self.radius = radius

    if (n_elements(abc) eq 3)     then self.abc = float(abc)

    if (n_elements(angles) eq 3)  then self.angles = float(angles)

    if (n_elements(density) eq 1) then self.density = density

    self.texture = keyword_set(texture)

    ; Rebuild the polygon according to keyword settings.
    self->BuildPoly
END

; ---------------------------------------------------------------------------
;Retrieves the value of properties associated with the ellipsoid object.
;** ****************
PRO ellapsoid::GetProperty, POS=pos, RADIUS=radius, ABC=abc, ANGLES=angles, DENSITY=density, $
                            RADINI=radini, TEXTURE=texture, POBJ=pobj, _REF_EXTRA=re
;+
;** *********
;** ellipsoid stuff
;-

    self->IDLgrModel::GetProperty, _EXTRA=re
    self.oPoly->GetProperty, _EXTRA=re

    pos     = self.pos
    radius  = self.radius
    radini  = self.radini
    abc     = self.abc
    angles  = self.angles
    density = self.density
    texture = self.texture
    pobj    = self.oPoly
END

; ---------------------------------------------------------------------------
PRO ellapsoid::Print
;+
;** *********
;** ellipsoid stuff
;-
    print, self.pos
    print, self.abc
    print, self.density
    print, self.texture
END

; ---------------------------------------------------------------------------
;Sets the vertex and connectivity arrays for the polygon used to
;represent the ellipsoid.
;** **************
PRO ellapsoid::BuildPoly
;+
;** *********
;** ellipsoid builder
;-

    ; Number of rows and columns of vertices is based upon the density
    ; property.
    nrows = LONG(20.0*self.density) > 2
    ncols = LONG(20.0*self.density) > 2
    abc   = self.abc

    ; Create the vertex list and the connectivity array.
    nverts = nrows*ncols + 2
    nconn  = (ncols*(nrows-1)*5) + (2*ncols*4)
    conn   = LONARR(ncols*(nrows-1)*5 + 2*ncols*4)
    verts  = FLTARR(3, nverts)
    IF (self.texture NE 0) THEN tex = FLTARR(2,nverts)

    ; Fill in the vertices.
    i = 0L & j = 0L & k = 0L
   
    tzinc = (!PI)/FLOAT(nrows+1)
    tz    = (!PI/2.0) - tzinc

    IF (self.texture NE 0) THEN tinc = (2.*!PI)/FLOAT(ncols-1) $
                           ELSE tinc = (2.*!PI)/FLOAT(ncols)

    FOR k=0,(nrows-1) DO BEGIN
        z = SIN(tz)*abc[2]
        t = 0
        FOR j=0,(ncols-1) DO BEGIN
            verts[0,i] = abc[0]*COS(t)*cos(tz) + self.pos[0]
            verts[1,i] = abc[1]*SIN(t)*cos(tz) + self.pos[1]
            verts[2,i] = z + self.pos[2]

            IF (self.texture NE 0) THEN BEGIN
                tex[0,i] = t/(2.*!PI)
                tex[1,i] = (tz+(!PI/2.0))/(!PI)
            ENDIF
            t = t + tinc
            i = i + 1L
        ENDFOR
        tz = tz - tzinc
    ENDFOR
    top = i
    verts[0,i] = self.pos[0]
    verts[1,i] = self.pos[1]
    verts[2,i] = abc[2] + self.pos[2]
    i   = i + 1L
    bot = i
    verts[0,i] = self.pos[0]
    verts[1,i] = self.pos[1]
    verts[2,i] =-abc[2] + self.pos[2]

    IF (self.texture NE 0) THEN BEGIN
        tex[0,i]   = 0.5
        tex[1,i]   = 0.0
        tex[0,i-1] = 0.5
        tex[1,i-1] = 1.0
    ENDIF

    ; Fill in the connectivity array.
    i = 0
    FOR k=0,(nrows-2) DO BEGIN
        FOR j=0,(ncols-1) DO BEGIN
            conn[i]   = 4
            conn[i+4] = k*ncols + j

            w = k*ncols + j + 1L
            IF (j EQ (ncols-1)) THEN w = k*ncols
            conn[i+3] = w

            w = k*ncols + j + 1L + ncols
            IF (j EQ (ncols-1)) THEN w = k*ncols + ncols
            conn[i+2] = w

            conn[i+1] = k*ncols + j + ncols

            i = i + 5L
            IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
                i = i - 5L
        ENDFOR
    ENDFOR
    FOR j=0,(ncols-1) DO BEGIN
        conn[i]   = 3
        conn[i+3] = top
        conn[i+2] = j+1L
        IF (j EQ (ncols-1)) THEN conn[i+2] = 0
        conn[i+1] = j
        i = i + 4L
        IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
            i = i - 4L
    ENDFOR
    FOR j=0,(ncols-1) DO BEGIN
        conn[i]   = 3
        conn[i+3] = bot
        conn[i+2] = j+(nrows-1L)*ncols
        conn[i+1] = j+(nrows-1L)*ncols+1L
        IF (j EQ (ncols-1)) THEN conn[i+1] = (nrows-1L)*ncols
        i = i + 4L
        IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
            i = i - 4L
    ENDFOR

    self.oPoly->SetProperty, DATA=verts, POLYGONS=conn

    IF (self.texture NE 0) THEN $
        self.oPoly->SetProperty, TEXTURE_COORD=tex
END

; ---------------------------------------------------------------------------
;Defines the object structure for an ellipsoid object.
;** ***********
PRO ellapsoid__define
;+
;** *********
;** ellipsoid stuff
;-
    struct = { ellapsoid, $
               INHERITS IDLgrModel, $
               pos:    [0.0,0.0,0.0], $
               radius:  1.0, $
               radini:  1.0, $
               abc:    [1.0,1.0,1.0], $
               angles: [!pi/4,!pi/4,!pi/4], $
               density: 1.0          , $
               texture: 0            , $
               oPoly:   OBJ_NEW() }
END


; ---------------------------------------------------------------------------
;This function returns a 1 if initialization is successful, or 0 otherwise.
;******* *********
FUNCTION orbi::Init, POS=pos, RADIUS=radius, DENSITY=density, $
                     TEX_COORDS=tex_coords, _EXTRA=e
;+
;******* ***
;** sphere a sphere
;-

    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

    self.pos = [0.0,0.0,0.0]
    self.radius = 1.0
    self.density = 1.0

    IF (N_ELEMENTS(pos) EQ 3) THEN $
        self.pos = pos

    IF (N_ELEMENTS(radius) EQ 1) THEN $
        self.radius = radius

    self.radini  = self.radius

    IF (N_ELEMENTS(density) EQ 1) THEN $
        self.density = density

    IF (N_ELEMENTS(tex_coords) EQ 1) THEN $
        self.texture = tex_coords

    ; Initialize the polygon object that will be used to represent
    ; the orb.
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, /REJECT, _EXTRA=e)

    self->Add,self.oPoly

    ; Build the polygon vertices and connectivity based on property settings.
    self->BuildPoly

    RETURN, 1
END

; ---------------------------------------------------------------------------
;Cleans up all memory associated with the orb.
;** ************
PRO orbi::Cleanup
;+
;** ***
;** sphere stuff
;-

    ; Cleanup the polygon object used to represent the orb.
    OBJ_DESTROY, self.oPoly

    ; Cleanup the superclass.
    self->IDLgrModel::Cleanup
END

; ---------------------------------------------------------------------------
;Sets the value of properties associated with the orb object.
;** ****************
PRO orbi::SetProperty, POS=pos, RADIUS=radius, DENSITY=density, _EXTRA=e
;+
;** ***
;** sphere stuff
;-

    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e

    IF (N_ELEMENTS(pos) EQ 3) THEN $
        self.pos = pos

    IF (N_ELEMENTS(radius) EQ 1) THEN $
        self.radius = radius

    IF (N_ELEMENTS(density) EQ 1) THEN $
        self.density = density

    ; Rebuild the polygon according to keyword settings.
    self->BuildPoly
END

; ---------------------------------------------------------------------------
;Retrieves the value of properties associated with the orb object.
;** ****************
PRO orbi::GetProperty, POS=pos,  RADIUS=radius, DENSITY=density,$
                       POBJ=pobj,RADINI=radini, _REF_EXTRA=re
;+
;** ***
;** sphere stuff
;-

    self->IDLgrModel::GetProperty, _EXTRA=re
    self.oPoly->GetProperty, _EXTRA=re

    pos    = self.pos
    radius = self.radius
    radini = self.radini
    density= self.density
    pobj   = self.oPoly
END
; ---------------------------------------------------------------------------
PRO orbi::Print
;+
;** ***
;** sphere stuff
;-
    PRINT, self.pos
    PRINT, self.radius
    PRINT, self.density
END

; ---------------------------------------------------------------------------
;Sets the vertex and connectivity arrays for the polygon used to
;represent the orb.
;** **************
PRO orbi::BuildPoly
;+
;** ***
;** sphere builder
;-

    ; Number of rows and columns of vertices is based upon the density
    ; property.
    nrows = LONG(20.0*self.density)
    ncols = LONG(20.0*self.density)
    IF (nrows LT 2) THEN nrows = 2
    IF (ncols LT 2) THEN ncols = 2

    ; Create the vertex list and the connectivity array.
    nverts = nrows*ncols + 2
    nconn = (ncols*(nrows-1)*5) + (2*ncols*4)
    conn = LONARR(ncols*(nrows-1)*5 + 2*ncols*4)
    verts = FLTARR(3, nverts)
    IF (self.texture NE 0) THEN $
        tex = FLTARR(2,nverts)

    ; Fill in the vertices.
    i = 0L
    j = 0L
    k = 0L
    tzinc = (!PI)/FLOAT(nrows+1)
    tz = (!PI/2.0) - tzinc
    FOR k=0,(nrows-1) DO BEGIN
        z = SIN(tz)*self.radius
        r = COS(tz)*self.radius
        t = 0
        IF (self.texture NE 0) THEN BEGIN
            tinc = (2.*!PI)/FLOAT(ncols-1)
        ENDIF ELSE BEGIN
            tinc = (2.*!PI)/FLOAT(ncols)
        ENDELSE
        FOR j=0,(ncols-1) DO BEGIN
            verts[0,i] = r*COS(t) + self.pos[0]
            verts[1,i] = r*SIN(t) + self.pos[1]
            verts[2,i] = z + self.pos[2]

            IF (self.texture NE 0) THEN BEGIN
                tex[0,i] = t/(2.*!PI)
                tex[1,i] = (tz+(!PI/2.0))/(!PI)
            ENDIF

            t = t + tinc
            i = i + 1L
        ENDFOR
        tz = tz - tzinc
    ENDFOR
    top = i
    verts[0,i] = self.pos[0]
    verts[1,i] = self.pos[1]
    verts[2,i] = self.radius*1.0 + self.pos[2]
    i = i + 1L
    bot = i
    verts[0,i] = self.pos[0]
    verts[1,i] = self.pos[1]
    verts[2,i] = self.radius*(-1.0) + self.pos[2]

    IF (self.texture NE 0) THEN BEGIN
        tex[0,i] = 0.5
        tex[1,i] = 0.0
        tex[0,i-1] = 0.5
        tex[1,i-1] = 1.0
    ENDIF

    ; Fill in the connectivity array.
    i = 0
    FOR k=0,(nrows-2) DO BEGIN
        FOR j=0,(ncols-1) DO BEGIN
            conn[i] = 4

            conn[i+4] = k*ncols + j

            w = k*ncols + j + 1L
            IF (j EQ (ncols-1)) THEN w = k*ncols
            conn[i+3] = w

            w = k*ncols + j + 1L + ncols
            IF (j EQ (ncols-1)) THEN w = k*ncols + ncols
            conn[i+2] = w

            conn[i+1] = k*ncols + j + ncols

            i = i + 5L
            IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
                i = i - 5L
        ENDFOR
    ENDFOR
    FOR j=0,(ncols-1) DO BEGIN
        conn[i] = 3
        conn[i+3] = top
        conn[i+2] = j+1L
        IF (j EQ (ncols-1)) THEN conn[i+2] = 0
        conn[i+1] = j
        i = i + 4L
        IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
            i = i - 4L
    ENDFOR
    FOR j=0,(ncols-1) DO BEGIN
        conn[i] = 3
        conn[i+3] = bot
        conn[i+2] = j+(nrows-1L)*ncols
        conn[i+1] = j+(nrows-1L)*ncols+1L
        IF (j EQ (ncols-1)) THEN conn[i+1] = (nrows-1L)*ncols
        i = i + 4L
        IF ((self.texture NE 0) AND (j EQ (ncols-1))) THEN $
            i = i - 4L
    ENDFOR

    self.oPoly->SetProperty, DATA=verts, POLYGONS=conn

    IF (self.texture NE 0) THEN $
        self.oPoly->SetProperty, TEXTURE_COORD=tex
END
; ---------------------------------------------------------------------------
;Defines the object structure for an orb object.
;** ***********
PRO orbi__define
;+
;** ***
;** sphere stuff
;-
    struct = { orbi, $
               INHERITS IDLgrModel, $
               pos: [0.0,0.0,0.0], $
               radius: 1.0, $
               radini: 1.0, $
               density: 1.0, $
               texture: 0, $
               oPoly: OBJ_NEW() }
END

; ---------------------------------------------------------------------------
;Irrow
;*****
;This function returns a 1 if initialization is successful, or 0 otherwise.
;******* *********
FUNCTION Irrow::Init, POS=pos, RADIUS=radius, DENSITY=density, $
                      LENGTH=length, ANGLES=angles, _EXTRA=e
;+
;******* *****
;** define an arrow
;-

    common d_liv_con

    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0

    self.pos     =[0.0,0.0,0.0]
    self.radius  = 0.5
    self.density = 1.0
    self.length  = 1.5
    self.angles  =[!pi/4,!pi/4,!pi/4]

    IF (N_ELEMENTS(pos)     EQ 3) THEN $
        self.pos     = pos

    IF (N_ELEMENTS(radius)  EQ 1) THEN $
        self.radius  = radius

    self.radini  = self.radius
    self.length  = self.radius*af_lng

    IF (N_ELEMENTS(density) EQ 1) THEN $
        self.density = density

    IF (N_ELEMENTS(length)  EQ 1) THEN $
        self.length  = length

    IF (N_ELEMENTS(angles)  EQ 3) THEN $
        self.angles  = angles

    ; Initialize the polygon object that will be used to represent
    ; the Irrow.
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, /REJECT, _EXTRA=e)

    self->Add,self.oPoly

    ; Build the polygon vertices and connectivity based on property settings.
    self->BuildIrrow

    RETURN, 1
END
;Cleans up all memory associated with the Irrow.
;** ************
PRO Irrow::Cleanup
;+
;** *****
;** arrow stuff
;-

    ; Cleanup the polygon object used to represent the Irrow.
    OBJ_DESTROY, self.oPoly

    ; Cleanup the superclass.
    self->IDLgrModel::Cleanup
END
;Sets the value of properties associated with the Irrow object.
;** ****************
PRO Irrow::SetProperty, POS=pos, RADIUS=radius, DENSITY=density,$
                        LENGTH=length, ANGLES=angles, _EXTRA=e
;+
;** *****
;** arrow stuff
;-

    common d_liv_con

    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e

    IF (N_ELEMENTS(pos)     EQ 3) THEN $
        self.pos     = pos

    IF (N_ELEMENTS(radius)  EQ 1) THEN   BEGIN
        self.radius  = radius
       ;self.length  = self.radius*af_lng
    ENDIF

    IF (N_ELEMENTS(density) EQ 1) THEN $
        self.density = density

    IF (N_ELEMENTS(length)  EQ 1) THEN $
        self.length  = length

    IF (N_ELEMENTS(angles)  EQ 3) THEN $
        self.angles  = angles
self.density=1.5
    ; Rebuild the polygon according to keyword settings.
    self->BuildIrrow
END
;Retrieves the value of properties associated with the Irrow object.
;** ****************
PRO Irrow::GetProperty, POS=pos, RADIUS=radius, DENSITY=density, RADINI=radini,$
                        POBJ=pobj, LENGTH=length, ANGLES=angles, _REF_EXTRA=re
;+
;** *****
;** arrow stuff
;-

    self->IDLgrModel::GetProperty, _EXTRA=re
    self.oPoly->GetProperty, _EXTRA=re

    pos    = self.pos
    radius = self.radius
    radini = self.radini
    density= self.density
    length = self.length
    angles = self.angles
    pobj   = self.oPoly
END
;represent the Irrow.
;** ****************
PRO Irrow::BuildIrrow
;+
;** *****
;** arrow builder
;-

    p3=self.pos[2] - self.length/2.2
    p4=self.pos[2] + self.length/1.8
    n =round(self.density* 20)
    m =n*3/2
    array1=fltarr(n,m)+self.radius
    array1[*,0]=self.radius/2

    q =((self.radius*1.3)>.15)<(self.length/1.5) & l =(m/3)>3  &  d =q/(l)
    array1[*,m-l]=q
    for i=m-l+1,m-1 do array1[*,i]=q-d*(i-m+l)

    MESH_OBJ,3,V,P,array1,P3=p3,p4=p4

    self.oPoly->SetProperty, DATA=V, POLYGONS=P
END

;Defines the object structure for an Irrow object.
;** ***********
PRO Irrow__define
;+
;** *****
;** arrow stuff
;-
    struct = { Irrow, $
               INHERITS IDLgrModel, $
               pos: [0.0,0.0,0.0], $
               radius:  0.5, $
               radini:  0.5, $
               density: 1.5, $
               length : 1.5, $
               angles :[!pi/4,!pi/4,!pi/4], $
               oPoly: OBJ_NEW() }
END
; -------------------------------------------------------------------------------

;toggle off and on state.
;**
;******* ********************
Function d_livMenuToggleState, wid   ;  IN: widget identifier
;+
;******* ********************
;** toggle off and on state.
;-

WIDGET_CONTROL, wid, GET_VALUE=name

s = STRPOS(name, '(Off)')
ret = s ne -1                   ;TRUE if new state is on
if ret then strput, name, '(On )', s $
else strput, name, '(Off)', strpos(name, '(On )')
WIDGET_CONTROL, wid, SET_VALUE=name
RETURN, ret
end

;**
;** *****************
pro d_livMenuSetState, Wid, NAMES=names, Indices = indices, State
;+
;** *****************
;** Set the value of a menu toggle item to State.  Either supply the
;** Widget ID of the button (wid), or the button's name (wid) along with
;** the arrays of button names and indices.
;-

if n_elements(names) then begin ;Name supplied?
    index = where(strpos(names, Wid) ge 0, count)
    if count le 0 then print,Wid+' not found in menu'
    w = Indices[index[0]]
endif else w = wid

WIDGET_CONTROL, w, GET_VALUE=value
On = STRPOS(value, '(On )')
if (on ge 0) eq State then return ;Already at proper value?

if On ge 0 then strput, value, '(Off)', on $ ;Change value of string
else strput, value, '(On )', strpos(value, '(Off)')

WIDGET_CONTROL, w, SET_VALUE=value ;update the widget
end

;**
;******* ******************
function d_livMapMenuChoice, $
            Eventval, $         ; IN: uservalue from seleted menu button
            MenuItems, $        ; IN: menu item array, as returned by d_livMenuCreate
            MenuButtons         ; IN: button array as returned by d_livMenuCreate
;+
;******* ******************
;**    PURPOSE   Given a uservalue from a menu button created
;**              by d_livMenuCreate, return the index of the choice
;**              within the category.  Set the selected menu button
;**              to insensitive to signify selection, and set all
;**              other choices for the category to sensitive.
;**
;-

i = STRPOS(eventval, '|', 0)    ;Get the name less the last qualifier
while (i GE 0) do begin
    j = i
    i = STRPOS(eventval, '|', i+1)
endwhile

base = STRMID(eventval, 0, j+1) ;  Get the common buttons, includes last | .
buttons = WHERE(STRPOS(MenuItems, base) EQ 0) ;  buttons that share base name.
this = (WHERE(eventval EQ MenuItems))[0] ;  Get the Index of the selected item.
for i=0, N_ELEMENTS(buttons)-1 do begin ;Each button in this category
    index = buttons[i]
    WIDGET_CONTROL, MenuButtons[buttons[i]], SENSITIVE=index NE this
endfor

RETURN, this - buttons[0]       ;  Return the selected button's index.
end

;**
;** ***************
pro d_livMenuCreate, $
                MenuItems, $    ; IN/OUT: See below
                MenuButtons, $  ; OUT: Button widget id's of the created menu
                Bar_base, $     ; IN: menu base ID
                Prefix=prefix   ; IN: (opt) Prefix for this menu's button names.
                                ;     If omitted, no prefix
;** ***************
;**  PURPOSE  Create a menu from a string descriptor (MenuItems).
;**           Return the parsed menu items in MenuItems (overwritten),
;**           and the array of corresponding menu buttons in MenuButtons.
;**  MenuItems = (input/output), on input the menu structure
;**              in the form of a string array.  Each button
;**              is an element, encoded as follows:
;**  Character 1 = integer bit flag.  Bit 0 = 1 to denote a
;**                button with children.  Bit 1 = 2 to denote
;**                this is the last child of its parent.
;**                Bit 2 = 4 to show that this button should
;**                initially be insensitive, to denote selection.
;**                Any combination of bits may be set.
;**                On RETURN, MenuItems contains the fully
;**                qualified button names.
;**  Characters 2-end = Menu button text.  Text should NOT
;**                     contain the character |, which is used
;**                     to delimit menu names.
;**  Example:
;**      MenuItems = ['1File', '0Save', '2Quit', $
;**     '1Edit', '3Cut', $
;**     '3Help']
;**       Creates a menu with three top level buttons
;**       (file, edit and help). File has 2 choices
;**       (save and exit), Edit has one choice, and help has none.
;**       On RETURN, MenuItems contains the fully qualified
;**       menu button names in a string array of the
;**       form: ['<Prefix>|File', '<Prefix>|File|Save',
;**         '<Prefix>|File|Quit', '<Prefix>|Edit',..., etc. ]
level = 0
parent = [ bar_base, 0, 0, 0, 0, 0]
names = STRARR(5)
lflogs = INTARR(5)

MenuButtons = LONARR(N_ELEMENTS(MenuItems))

if (N_ELEMENTS(prefix)) then begin
    names[0] = prefix + '|'
endif else begin
    names[0] = '|'
endelse

widget_control,bad_id=i,Bar_base,UPDATE=0

if (!version.release ge '5.6') then $
    bid=WIDGET_BUTTON(bar_base,value='XobjView window', UVALUE='XobjV')

for i=0, N_ELEMENTS(MenuItems)-1 do begin
    flog = FIX(STRMID(MenuItems[i], 0, 1))
    txt = STRMID(MenuItems[i], 1, 100)
    uv = ''

    for j = 0, level do uv = uv + names[j]
    MenuItems[i] = uv + txt     ;  Create the button for fully qualified names.
    isHelp = txt eq 'Help' or txt eq 'About'
    MenuButtons[i] = WIDGET_BUTTON(parent[level], $
                                   VALUE= txt, UVALUE=uv+txt, $
                                   MENU=flog and 1, HELP=isHelp)

    if ((flog AND 4) NE 0) then begin
        WIDGET_CONTROL, MenuButtons[i], SENSITIVE = 0
    endif

    if (flog AND 1) then begin
        level = level + 1
        parent[level] = MenuButtons[i]
        names[level] = txt + '|'
        lflogs[level] = (flog and 2) NE 0
    endif else if ((flog AND 2) NE 0) then begin
        while lflogs[level] do level = level-1 ;  Pops the previous levels.
        level = level - 1
    endif
endfor
widget_control,bad_id=i,Bar_base,UPDATE=1
end

;**
;******* *************
function d_livReadNoff, $
             file, $            ; IN: filename
             xr, $              ; OUT: x radius
             yr, $              ; OUT: y radius
             zr, $              ; OUT: z radius
             mesh               ; IN: if filename=''
;+
;******* *************
;** restore a save file 
;-
s = ' '
npsize = 1

if file gt ' ' then RESTORE, file $
else begin
    x=xr & y=yr & z=zr & endelse

xr = [min(x, max = xx), xx]     ;Get ranges
yr = [min(y, max = xx), xx]
zr = [min(z, max = xx), xx]

sc = [xr[1]-xr[0], yr[1]-yr[0], zr[1]-zr[0]] ;Ranges
xr[0] = (xr[1] + xr[0])/2.0     ;Midpoint
yr[0] = (yr[1] + yr[0])/2.0
zr[0] = (zr[1] + zr[0])/2.0
s = max(sc)                     ;Largest range...

x = (x - xr[0]) / s
y = (y - yr[0]) / s
z = (z - zr[0]) / s

xr = [-0.7, 0.7]                ;Fudge the ranges
yr = xr
zr = xr
s = OBJ_NEW("IDLgrPolygon", TRANSPOSE([[x],[y],[z]]), $
            SHADING=1, $
            POLY=mesh, COLOR=[200,200,200])
RETURN, s
end


;Read the molecule data input file.
;**
;******* ************
function d_livMolRead, $
    filename, $          ; IN: filename
    atom_xyz, $
    atoms   , $
    amesh   , $
    img, Np, dup, cell, angl
;+
;******* *************
;**Read the molecule data input file or make caffeine.
;-

n_atoms = 0L
img     = 0
dup     =[1,1,1]
cell    = 0
angl    =[90.,90.,90.]

CASE filename of

'caffeine.mol':begin
   atom_xyz=[[-0.802,-0.906, 0.120 ],[-0.802, 0.494,-0.114 ],[ 0.411, 1.194, 0.120 ],$
         [ 1.622, 0.494,-0.114 ],[ 1.622,-0.906, 0.120 ],[ 0.411,-1.606,-0.114 ],$
         [-2.134, 0.928, 0.120 ],[-2.134,-1.337,-0.114 ],[-2.958,-0.203, 0.120 ],$
         [-2.134, 2.328,-0.114 ],[ 0.411,-3.006, 0.120 ],[ 2.836, 1.194,-0.114 ],$
         [ 2.836,-1.606, 0.120 ],[ 0.411, 2.594,-0.114 ],[-1.055,-1.102, 1.172 ],$
         [-1.055, 0.691,-1.166 ],[-4.021,-0.098, 0.281 ],[-2.838, 2.565,-0.925 ],$
         [-2.442, 2.853, 0.803 ],[-1.123, 2.652,-0.400 ],[ 0.664,-3.533,-0.812 ],$
         [ 1.156,-3.249, 0.892 ],[-0.586,-3.321, 0.460 ],[ 3.493, 0.789, 0.670 ],$
         [ 3.323, 1.084,-1.094 ],[ 2.644, 2.260, 0.082 ]]
   atoms   = [6,6,6,7,6,7,7,7,6,6,6,6,8,8,1,1,1,1,1,1,1,1,1,1,1,1]
   n_atoms = n_elements(atoms)
   angl    =[90.,90.,90.]
   amesh   = [8,  1,2,3,4,5,6,1,$
          6,  1,8,9,7,2,$
          3,  7,10,      3,6,11,   3,5,13,   3,3,14,   3,4,12,$
          3,  10,18,  3,10,19,  3,10,20,  3,9,17,   3,2,16,$
          3,  1,15,   3,11,21,  3,11,22,  3,11,23,  3,12,24,  3,12,25,  3,12,26] -1
   a=64 & img=fltarr(a,a)
   xmi=min(atom_xyz[0,*],max=xma)
   ymi=min(atom_xyz[1,*],max=yma)
   stx=(a)/(xma-xmi)
   sty=(a)/(yma-ymi)
   for  i=0,n_atoms-1 do begin
    px=round((atom_xyz[0,i]-xmi)*stx)<63
    py=round((atom_xyz[1,i]-ymi)*sty)<63
    if atoms[i] eq 1 then img[px,py]=-2 else img[px,py]=atoms[i]
   endfor
   img=smooth(smooth(img,3),3)
   Np =[xmi,xma,ymi,yma,min(atom_xyz[2,*],max=zma),zma]
   end
ELSE:begin
   atom_xyz = FLTARR(3,1000)
   x = (y = (z = 0.))
   atoms = INTARR(1000)

   OPENR, lun, filename, /GET_LUN, error = i
   if i lt 0 then print, filename + ' not found'
   s = " "

   while eof(lun) eq 0 do begin
    READF, lun, s
    i = strpos(s, ":")
    if i NE -1 then begin
        READS, strmid(s,i+1,strlen(s)-i+1),x,y,z
        case strmid(s,0,2) of
            "C:" : a_type = 6  ;z=6   m=12.0111
            "H:" : a_type = 1  ;z=1   m=1.00797
            "N:" : a_type = 7  ;z=7   m=14.0067
            "S:" : a_type = 16 ;z=16  m=32.064
            "X:" : a_type = 54 ;z=54  m=131.30
            "Ce" : a_type = 58 ;z=58  m=140.12
            "BR" : a_type = 35 ;z=35  m=79.909
            "O:" : a_type = 8  ;z=8   m=15.9994
            else : a_type = 0
        endcase

        if (a_type NE -1) then begin
            atom_xyz[0,n_atoms] = x
            atom_xyz[1,n_atoms] = y
            atom_xyz[2,n_atoms] = z
            atoms[n_atoms] = a_type
            n_atoms = n_atoms + 1
            if n_atoms ge n_elements(atoms) then $
              print, 'Too many atoms in molecule'
        endif
    endif
   endwhile
   endoffile:
   FREE_LUN,lun
   if (n_atoms GE 0) then begin
    atoms = atoms[0:n_atoms-1]
    atom_xyz = atom_xyz[*,0:n_atoms-1]
   endif
   end
ENDCASE
RETURN, n_atoms
end

;**
;** ***************
pro d_livMakeAtomes
;+
;** ***************
;** init atoms global variables
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj1,a_ElpStruct
common c_liv_obj2,a_coval , a_rvdw , a_mass

A_symb=['a' , $
        'H' ,'He','Li','Be','B' ,'C' ,'N' ,'O' ,'F' ,'Ne','Na','Mg','Al','Si','P' ,$
        'S' ,'Cl','Ar','K' ,'Ca','Sc','Ti','V' ,'Cr','Mn','Fe','Co','Ni','Cu','Zn',$
        'Ga','Ge','As','Se','Br','Kr','Rb','Sr','Y' ,'Zr','Nb','Mo','Tc','Ru','Rh',$
        'Pd','Ag','Cd','In','Sn','Sb','Te','I' ,'Xe','Cs','Ba','La','Ce','Pr','Nd']
A_symb=[A_symb,$
        'Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W' ,'Re',$
        'Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr','Ra','Ac','Th',$
        'Pa','U' ,'Np','Pu','Am','Cm','Bk','Cf','Es','Fm','Md','No','Lr'          ,$
        'Unq','Unp','Unh','Uns','Uno','Une','Unk','Elp','Unk']; Unk+Ellipsoide+Unk

A_name=['Atom', $
    'Hydrogen',  'Helium',    'Lithium',    'Beryllium', 'Bore',      $
    'Carbon',    'Azote',     'Oxygen',     'Fluor',     'Neon',      $
    'Sodium',    'Magnesium', 'Aluminium',  'Silicium',  'Phosphore', $
    'Sulfur',    'Chlore',    'Argon',      'Potassium', 'Calcium',   $
    'Scandium',  'Titane',    'Vanadium',   'Chrome',    'Manganese', $
    'Fer',       'Cobalt',    'Nickel',     'Cuivre',    'Zinc',      $
    'Gallium',   'Germanium', 'Arsenic',    'Selenium',  'Brome',     $
    'Krypton',   'Rubidium',  'Strontium',  'Yttrium',   'Zirconium', $
    'Nobium',    'Molybdene', 'Technetium', 'Ruthenium', 'Rhodium',   $
    'Palladium', 'Argent',    'Cadmium',    'Indium',    'Etain',     $
    'Antimoine', 'Tellurium', 'Iode',       'Xenon',     'Cesium']
     A_name=[A_name, $
    'Baryum',    'Lanthane',  'Cerium',     'Praseodyme','Neodyme',   $
    'Promethium','Samarium',  'Europium',   'Gadolinium','Terbium',   $
    'Dysprosium','Holmium',   'Erbium',     'Thulium',   'Ytterbium', $
    'Lutenium',  'Hafnium',   'Tantale',    'Tungstene', 'Rhenium',   $
    'Osmium',    'Iridium',   'Platine',    'Or',        'Mercure',   $
    'Thallium',  'Plomb',     'Bismuth',    'Polonium',  'Astate',    $
    'Radon',     'Francium',  'Radium',     'Actinium',  'Thorium',   $
    'Protactinium','Uranium', 'Neptunium',  'Plutonium', 'Americium', $
    'Curium',    'Berkelium', 'Californium','Einsteinium','Fermium',  $
    'Mendelevium', 'Nobelium', 'Lawrencium',                          $
    'Unnilquadium','Unnilpentium','Unnilhexium', 'Unnilseptium',      $
    'Unniloctium', 'Unnilennium' ,'Unknown'    , 'Ellipsoide', 'Unknown'] ;Unk+Ellipsoide+Unk

A_dens=[0.0700, $
    0.0899, 0.1787, 0.5300, 1.8500, 2.3400, 2.6200, 1.2510, 1.4290, 1.6960, 0.9010, $
    0.9700, 1.7400, 2.7000, 2.3300, 1.8200, 2.0700, 3.1700, 1.7840, 0.8600, 1.5500, $
    3.0000, 4.5000, 5.8000, 7.1900, 7.4300, 7.8600, 8.9000, 8.9000, 8.9600, 7.1400, $
    5.9100, 5.3200, 5.7200, 4.8000, 3.1200, 3.7400, 1.5300, 2.6000, 4.5000, 6.4900, $
    8.5500, 10.200, 11.500, 12.200, 12.400, 12.000, 10.500, 8.6500, 7.3100, 7.3000, $
    6.6800, 6.2400, 4.9200, 5.8900, 1.8700, 3.5000, 6.7000, 6.7800, 6.7700, 7.0000]
    A_dens=[A_dens, $
    6.4750, 7.5400, 5.2600, 7.8900, 8.2700, 8.5400, 8.8000, 9.0500, 9.3300, 6.9800, $
    9.8400, 13.100, 16.600, 19.300, 21.000, 22.400, 22.500, 21.400, 19.300, 13.530, $
    11.850, 11.400, 9.8000, 9.4000, 9.5000, 9.9100, 9.5000, 5.0000, 10.070, 11.700, $ ;verify 85 & 87 (9.5)
    15.400, 18.900, 20.400, 19.800, 13.600, 13.511, 14.000, 14.000, 14.000, 14.000, $ ;verify (14.0)
    14.000, 14.000, 14.000, 14.000, 14.000, 14.000, 14.000, 14.000, 14.000, 14.000, $
     0.070, 14.000] ; Unk+Ellipsoide+Unk

A_coval=[0.0 , 0.370000 , 0.700000 , 1.230000 , 0.890000 , 0.800000 , 0.850000 , 0.740000 , 0.740000 , $
    0.720000 , 0.700000 , 1.000000 , 1.360000 , 1.480000 , 1.170000 , 1.100000 , 1.100000 , 0.990000 , $
    0.700000 , 2.030000 , 1.740000 , 1.440000 , 1.320000 , 1.220000 , 0.000000 , 1.160000 , 0.000000 , $
    1.150000 , 1.170000 , 1.250000 , 1.250000 , 1.200000 , 1.210000 , 1.160000 , 0.700000 , 1.240000 , $
    1.910000 , 1.620000 , 1.450000 , 1.340000 , 1.290000 , 1.290000 , 1.240000 , 1.250000 , 0.000000 , $
    1.340000 , 1.410000 , 1.500000 , 1.400000 , 1.410000 , 1.370000 , 1.330000 , 0.700000 , 1.330000]
    A_coval=[A_coval, $
    1.980000 , 1.690000 , 1.690000 , 0.000000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , $
    1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , 1.690000 , $
    1.440000 , 1.340000 , 1.300000 , 1.280000 , 1.260000 , 1.290000 , 1.340000 , 1.440000 , 1.550000 , $
    1.540000 , 1.520000 , 1.520000 , 1.400000 , 0.700000 , 2.400000 , 2.000000 , 1.900000 , 1.900000 , $
    1.900000 , 1.900000 , 1.900000 , 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, $
    2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, $
    0.0      , 2.0] ; Unk+Ellipsoide+Unk

A_rvdw=[0.60 , 1.200000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.550000 , 1.520000 , $
    1.470000 , 1.700000 , 1.700000 , 1.700000 , 1.940000 , 2.100000 , 1.800000 , 1.800000 , 1.750000 , $
    1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.980000 , 1.940000 , 1.930000 , 1.930000 , $
    1.920000 , 1.700000 , 1.700000 , 1.700000 , 2.020000 , 1.700000 , 1.960000 , 1.700000 , 2.100000 , $
    1.700000 , 1.700000 , 1.700000 , 1.700000 , 2.210000 , 1.700000 , 2.060000 , 1.700000 , 2.010000 , $
    2.010000 , 2.040000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 2.150000]
    A_rvdw=[A_rvdw, $
    1.700000 , 1.700000 , 1.700000 , 0.800000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , $
    1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , $
    1.700000 , 1.700000 , 1.700000 , 1.700000 , 2.020000 , 2.030000 , 1.700000 , 1.700000 , 1.700000 , $
    1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , 1.700000 , $
    1.700000 , 1.700000 , 1.700000 , 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, $
    2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, 2.0 , 2.0, $
    0.60     , 2.0] ; Unk+Ellipsoide+Unk

A_mass=[0.50,  1.0079  ,  4.00260 ,  6.941   ,  9.01218 , 10.81    ,12.011    , 14.0067  , 15.9994   , $
   18.998403,  20.179  , 22.98977 , 24.305   , 26.98154 , 28.0855  , 30.97376 , 32.06    , 35.453    , $
   39.948   ,  39.0983 , 40.08    , 44.9559  , 47.90    , 50.9415  , 51.996   , 54.9380  , 55.9332   , $
   58.9332  , 58.70    , 63.546   , 65.38    , 69.72    , 72.59    , 74.9216  , 78.96    , 79.904    , $
   83.80    , 85.4678  , 87.62    , 88.9059  , 91.22    , 92.9064  , 95.94    , 98       , 101.07    , $
   102.9055 ,106.4     ,107.868   ,112.41    ,114.82    ,118.69    ,121.75    ,127.60    , 126.9045]
   A_mass=[A_mass, $
   131.30   ,132.9054  ,137.33    ,138.9055  ,140.12    ,140.9077  ,144.24    ,145       ,150.4      , $
   151.96   ,157.25    ,158.9254  ,162.50    ,164.9304  ,167.26    ,168.9342  ,173.04    ,174.967    , $
   178.49   ,180.9479  ,183.85    ,186.207   ,190.2     ,192.22    ,195.09    ,196.9665  ,200.59     , $
   204.37   ,207.2     ,208.9804  ,209       ,210       ,222       ,223       ,226.0254  ,227.0278   , $
   232.0381 ,231.0359  ,238.029   ,237.0482  ,244 , 243 , 247 , 247, 251 , 254, 257 , 258, 259 , 260 , $
   261, 262, 263, 264, 265, 267, 268, $
     0.50   ,268] ; Unk+Ellipsoide+Unk

d=n_elements(A_symb) ;110 +Unk+Ellipsoide+Unk
a_col =bytarr(3,d)
idx_1 =[3,4,11,12,13,19,20,30,31,37,38,48,49,50,51,55,56,80,81,82,83,84,87,88] ;Metaux simples     (blue)
idx_2 =[indgen(9)+21 ,indgen(9)+39 ,indgen(8)+72 ]                             ;Elts de transition (orange)
idx_3 =[5,6,7,8,9,14,15,16,17,32,33,34,35,52,53,85]                            ;Non-metaux         (violet)
idx_4 =[indgen(15)+57,indgen(15)+89]                                           ;Terres rares       (green)
idx_5 =[2,10,18,36,54,86]                                                      ;Gaz nobles         (pink)
idx_6 =[0,1,104,105,106,107,108,109,110,111,112]                               ;Others+Ellipsoide  (pale blue)

a_col[0,idx_1]= 20 & a_col[1,idx_1]= 20 & a_col[2,idx_1]=230
a_col[0,idx_2]=230 & a_col[1,idx_2]=100 & a_col[2,idx_2]= 20
a_col[0,idx_3]=100 & a_col[1,idx_3]= 20 & a_col[2,idx_3]=230
a_col[0,idx_4]= 20 & a_col[1,idx_4]=230 & a_col[2,idx_4]= 20
a_col[0,idx_5]=230 & a_col[1,idx_5]= 20 & a_col[2,idx_5]=190
a_col[0,idx_6]=150 & a_col[1,idx_6]=150 & a_col[2,idx_6]=230

a_col[0:2,0]  =[255,255,255] ;atom
a_col[0:2,1]  =[200,200,255] ;white blue
a_col[0:2,2]  =[255,200,200] ;light pink
a_col[0:2,3]  =[165,42 ,42 ] ;brown
a_col[0:2,4]  =[255,20 ,147] ;deep pink
a_col[0:2,5]  =[0  ,255,0  ] ;green
a_col[0:2,6]  =[128,128,128] ;carbon
a_col[0:2,7]  =[143,143,255] ;light blue
a_col[0:2,8]  =[255,0  ,0  ] ;oxygen
a_col[0:2,9]  =[200,165,24 ] ;gold
a_col[0:2,10] =[255,20 ,147] ;deep pink
a_col[0:2,11] =[0  ,0  ,255] ;blue
a_col[0:2,12] =[42 ,128,42 ] ;dark green
a_col[0:2,13] =[128,128,144] ;dark gray
a_col[0:2,14] =[200,165,24 ] ;gold
a_col[0:2,15] =[255,165,0  ] ;orange
a_col[0:2,16] =[230,230,0  ] ;sulfur
a_col[0:2,17] =[0  ,255,0  ] ;green
;                       ATOM     18 AR    Ar   deep pink     [255,20,147]   FF1493
;                       ATOM     19  K    K    deep pink     [255,20,147]   FF1493
a_col[0:2,20] =[128,128,144] ;dark gray
;                       ATOM     21 SC    Sc   deep pink     [255,20,147]   FF1493
;                       ATOM     22 TI    Ti   dark gray     [128,128,144]  808090
;                       ATOM     23  V    V    deep pink     [255,20,147]   FF1493
;                       ATOM     24 CR    Cr   dark gray     [128,128,144]  808090
;                       ATOM     25 MN    Mn   dark gray     [128,128,144]  808090
a_col[0:2,26] =[255,165,0  ] ;orange
;                       ATOM     27 CO    Co   deep pink     [255,20,147]   FF1493
;                       ATOM     28 NI    Ni   brown         [165,42,42]    A52A2A
;                       ATOM     29 CU    Cu   brown         [165,42,42]    A52A2A
a_col[0:2,35] =[165,42 ,42 ] ;zinc brown
;                       ATOM     31 GA    Ga   deep pink     [255,20,147]   FF1493
;                       ATOM     32 GE    Ge   deep pink     [255,20,147]   FF1493
;                       ATOM     33 AS    As   deep pink     [255,20,147]   FF1493
;                       ATOM     34 SE    Se   deep pink     [255,20,147]   FF1493
a_col[0:2,35] =[165,42 ,42 ] ;brome
;                       ATOM     36 KR    Kr   deep pink     [255,20,147]   FF1493
;                       ATOM     37 RB    Rb   deep pink     [255,20,147]   FF1493
;                       ATOM     38 SR    Sr   deep pink     [255,20,147]   FF1493
;                       ATOM     39  Y    Y    deep pink     [255,20,147]   FF1493
;                       ATOM     40 ZR    Zr   deep pink     [255,20,147]   FF1493
;                       ATOM     41 NB    Nb   deep pink     [255,20,147]   FF1493
;                       ATOM     42 MO    Mo   deep pink     [255,20,147]   FF1493
;                       ATOM     43 TC    Tc   deep pink     [255,20,147]   FF1493
;                       ATOM     44 RU    Ru   deep pink     [255,20,147]   FF1493
;                       ATOM     45 RH    Rh   deep pink     [255,20,147]   FF1493
;                       ATOM     46 PD    Pd   deep pink     [255,20,147]   FF1493
a_col[0:2,47] =[128,128,144] ;dark gray
;                       ATOM     48 CD    Cd   deep pink     [255,20,147]   FF1493
;                       ATOM     49 IN    In   deep pink     [255,20,147]   FF1493
;                       ATOM     50 SN    Sn   deep pink     [255,20,147]   FF1493
;                       ATOM     51 SB    Sb   deep pink     [255,20,147]   FF1493
;                       ATOM     52 TE    Te   deep pink     [255,20,147]   FF1493
a_col[0:2,53] =[143,143,255] ;light blue
;                       ATOM     54 XE    Xe   deep pink     [255,20,147]   FF1493
a_col[0:2,54] =[25 ,25 ,0  ] ;xenon
;                       ATOM     55 CS    Cs   deep pink     [255,20,147]   FF1493
a_col[0:2,56] =[255,165,0  ] ;orange
a_col[0:2,58] =[204,0  ,153] ;cerium
;                       ATOM     71 LU    Lu   deep pink     [255,20,147]   FF1493
;                       ATOM     72 HF    Hf   deep pink     [255,20,147]   FF1493
;                       ATOM     73 TA    Ta   deep pink     [255,20,147]   FF1493
;                       ATOM     74  W    W    deep pink     [255,20,147]   FF1493
;                       ATOM     75 RE    Re   deep pink     [255,20,147]   FF1493
;                       ATOM     76 OS    Os   deep pink     [255,20,147]   FF1493
;                       ATOM     77 IR    Ir   deep pink     [255,20,147]   FF1493
;                       ATOM     78 PT    Pt   deep pink     [255,20,147]   FF1493
a_col[0:2,58] =[200,165,24 ] ;gold
;                       ATOM     80 HG    Hg   deep pink     [255,20,147]   FF1493
;                       ATOM     81 TL    Tl   deep pink     [255,20,147]   FF1493
;                       ATOM     82 PB    Pb   deep pink     [255,20,147]   FF1493
;                       ATOM     83 BI    Bi   deep pink     [255,20,147]   FF1493
;                       ATOM     84 PO    Po   deep pink     [255,20,147]   FF1493
;                       ATOM     85 AT    At   deep pink     [255,20,147]   FF1493
;                       ATOM     86 RN    Rn   deep pink     [255,20,147]   FF1493

a_sphere    =objarr(d)
a_Irrow     =objarr(d)
a_ElpStruct ={elp:where(A_name eq 'Ellipsoide'),abc:[1.,1.,1.],angles:[0.,0.,0.]}
d_livConst
end

;**
;*********************
pro d_liv_change_table, type, col ,reset=reset, get=get, set=set
;+
;** ******************
;** play withthe atoms colors 
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_kep, k_col

if n_elements( k_col) eq 0 then k_col=a_col
if keyword_set(reset)      then a_col=k_col else $
if keyword_set(get)        then for i=0,n_elements(type)-1 do col[0:2,i]=a_col[0:2,type[i]] else $
if keyword_set(set)        then for i=0,n_elements(type)-1 do a_col[0:2,type[i]]=col[0:2,i]
end

;**
;******* ***************
function d_livSymbtoType, symb,nb
;+
;******* ***************
;** return atom number from symbol
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

if n_elements(nb) eq 0 then nb =n_elements(symb)
w=lonarr(nb)
table=strlowcase(a_symb) & lsym=strlowcase(symb)
for i=0,nb-1 do  w[i]= (where(table eq lsym[i]))[0]
if nb eq 1 then  w=w[0]
return ,w>0
end

;**
;******* ***************
function d_livTypetoName, typ
;+
;******* ***************
;** return atom name from number
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

return ,a_name[typ]
end

;******* ***************
function d_livTypetoMass, typ
  ;+
  ;******* ***************
  ;** return atom mass from number
  ;-

common c_liv_obj2,a_coval , a_rvdw , a_mass

  return ,a_mass[typ]
end


;**
;******* ***************
function d_livMasstoType, mass
;+
;******* ***************
;** return atom number from mass
;-

common c_liv_obj2,a_coval , a_rvdw , a_mass

tmp=min(abs(a_mass-mass),typ)
return ,typ
end

;**
;******* ****************
function d_livgetVDWtable
;+
;******* ****************
;** return rvdw table
;-

common c_liv_obj2,a_coval , a_rvdw , a_mass

return ,a_rvdw
end

;**
;******* ***************
function d_livTypetoSymb, typ
;+
;******* ***************
;** return atom symbol from number
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

return ,a_symb[typ]
end

;**
;******* ***************
function d_livMakeSphere, ty
;+
;******* ***************
;** return sphere object with correct radius and color
;-

common d_liv_con
common c_liv_obj , allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj2, a_coval , a_rvdw , a_mass

if a_sphere[ty] eq OBJ_NEW() then begin
   rad  = as_rad*a_rvdw[ty] *(.6 + .4*(a_mass[ty]/250.)) ;Using rvdw+mass or [.6*as_rad*(sqrt(a_dens(ty))>.6)]
   a_sphere[ty] =  OBJ_NEW('orbi', COLOR=a_col[*,ty], DENSITY=s_den, RADIUS=rad)
end

RETURN, a_sphere[ty]
end
;**
;******* ***************
function d_livMakeIrrow, ty
;+
;******* **************
;** return arrow object with correct radius and color
;-

common d_liv_con
common c_liv_obj , allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj2, a_coval , a_rvdw , a_mass

if a_Irrow[ty] eq OBJ_NEW() then begin
   rad  = as_rad*a_rvdw[ty] *(.6 + .4*(a_mass[ty]/250.)) ;Using rvdw+mass or [.6*af_rad*(sqrt(a_dens(ty))>.6)]
   a_Irrow[ty] =  OBJ_NEW('Irrow', COLOR=a_col[*,ty], DENSITY=a_den, RADIUS=rad, LENGTH=rad*af_lng, ANGLES=[0.,0.,0.])
endif

RETURN, a_Irrow[ty]
end
;**
;******* ***************
function d_livMakeEllips, ty
;+
;******* ***************
;** return ellipsoid object with correct radius ans color
;-

common d_liv_con
common c_liv_obj , allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj1, a_ElpStruct
common c_liv_obj2, a_coval , a_rvdw , a_mass

;Each ellipsoid should have different axis...
;if a_sphere(ty) eq OBJ_NEW() then begin
   abc = a_ElpStruct.abc
   a_sphere[ty] =  OBJ_NEW('ellapsoid', COLOR=a_col[*,ty], DENSITY=a_den, ABC=abc, ANGLES=[0.,0.,0.] )
;endif

RETURN, a_sphere[ty]
end


;**
;** *************
Pro d_livAddLight, Model
;+
;** *************
;** Set up the lighting
;-

Model->Add, OBJ_NEW('IDLgrLight', $ ;Directional
                    LOCATION=[2,2,5], TYPE=2, COLOR=[255,255,255], $
                    INTENSITY=0.6 )

Model->Add, OBJ_NEW('IDLgrLight', $ ;Directional
                    LOCATION=[2,-2,-5], TYPE=2, COLOR=[255,255,0], $
                    INTENSITY=0.25 )

Model->add, OBJ_NEW('IDLgrLight', TYPE=0, $ ;Ambient light
                    INTENSITY=0.575, COLOR=[255,255,255])
end

Pro d_livChangAxis, a,b,c, angles=angles, xyz=Mlj, getmodule=modu ,hkl=hkl
;+
;** **************
;** change atom coordinates and arrow direction due to new axis angles
;-
common d_livChAx, cosa,sina, cosb,sinb, cosc,sinc
common d_livChhkl,coza,sima, cozb,simb, cozc,simc

if n_elements(angles) gt 0 then begin
   cosa =cos(angles[2]*!dpi/180.) & sina=sin(angles[2]*!dpi/180.) ;gamma (axa c ->ab)   c|
   cosb =cos(angles[0]*!dpi/180.) & sinb=sin(angles[0]*!dpi/180.) ;alpha (axe a ->bc)  a/\b
   cosc =cos(angles[1]*!dpi/180.) & sinc=sin(angles[1]*!dpi/180.) ;beta  (axe b ->ac) /    \
   if angles[2] eq 90. then begin & cosa=0. & sina=1. & endif
   if angles[0] eq 90. then begin & cosb=0. & sinb=1. & endif
   if angles[1] eq 90. then begin & cosc=0. & sinc=1. & endif
   ingle=angles-90.
   coza =cos(ingle[2]*!dpi/180.) & sima=sin(ingle[2]*!dpi/180.) ;gamma (axa c ->ab)-90   c|
   cozb =cos(ingle[0]*!dpi/180.) & simb=sin(ingle[0]*!dpi/180.) ;alpha (axe a ->bc)-90  a/\b
   cozc =cos(ingle[1]*!dpi/180.) & simc=sin(ingle[1]*!dpi/180.) ;beta  (axe b ->ac)-90 /    \
   if ingle[2] eq 90. then begin & coza=0. & sima=1. & endif
   if ingle[0] eq 90. then begin & cozb=0. & simb=1. & endif
   if ingle[1] eq 90. then begin & cozc=0. & simc=1. & endif
endif
if n_elements(Mlj) eq 3 then modu=float(sqrt(total(Mlj^2)  +2.*      (cosb*Mlj[1]  *Mlj[2]  +cosc*Mlj[0]  *Mlj[2]  +cosa*Mlj[0]  *Mlj[1])))>0.
if n_elements(Mlj) gt 3 then modu=float(sqrt(total(Mlj^2,1)+2.*reform(cosb*Mlj[1,*]*Mlj[2,*]+cosc*Mlj[0,*]*Mlj[2,*]+cosa*Mlj[0,*]*Mlj[1,*])))>0.
if n_elements(a)   eq 0 then return
if n_elements(b)   ge 1 then begin
   if keyword_set(hkl) then begin
    bh=a*sima & ah=a*coza & bi=bh/sina & ai=ah-cosa*bi & ci=a*simc
    ah=b*sima & bh=b*coza & aj=ah/sina & bj=bh-cosa*aj & cj=b*simb
    a=ai+aj & b=bj+bi & c=c+ci+cj
   endif
    a=a+c*cosc & c=c*sinc & b=b+c*cosb & c=float(c*sinb) & a=float(a+b*cosa) & b=float(b*sina)
endif else begin
    a[0,*]=a[0,*]    + a[2,*]*cosc
    a[2,*]=a[2,*]*sinc
    a[1,*]=a[1,*]    + a[2,*]*cosb
    a[2,*]=a[2,*]*sinb
    a[0,*]=a[0,*]    + a[1,*]*cosa
    a[1,*]=a[1,*]*sina
endelse;www.rcsb.org/pdb/docs/format/pdbguide2.2/guide2.2_frame.html
       ;a   b(cos(gamma))   c(cos(beta))
       ;0   b(sin(gamma))   c(cos(alpha) - cos(beta) cos(gamma)) / sin(gamma)
       ;0   0               V/(ab sin(gamma))
       ;V = abc(1 - cos**2(alpha) - cos**2(beta) - cos**2(gamma) + 2(cos(alpha) cos(beta) cos(gamma)))**1/2
       ; ->
       ;V = SQRT(1 - cosb^2 - cosc^2 - cosa^2 + 2.*(cosb*cosc*cosa)) !!!!!!!
       ;a1=a+b*cosa+c*cosc & b1=b*sina+c*(cosb-cosc*cosa)/sina & c1=c*V/sina
end

;**
;** **************
pro d_livClearAnim ;
;+
;** **************
;** free memories
;-

common kp_for_anim, maxp,fa,fb,fc,ffb,coef,prov,p25,propag,propagi,atom_type,atom_coo,atom_xyz,N_dens,N_lims,idR,idI,IdP,otit,ppaobj,ltit,reste

atom_type=0 & atom_coo=0 & atom_xyz=0
propag   =0 & propagi =0 & N_dens=0
ppaobj   =0 & reste   =0
end

;**
;******* *****************
Function d_livMakeMolecule, filename, state ,atom_doo, atom_tip, amesh, img ,Np, dup, cell, angl, prof, propav, coev, tit
;+
;******* *****************
;** create the molecule or what is is the cell
;** oModelTop <- oModelSurface <- oModelAtom*n <- Polygon sharing data from MakeAtomes
;** oModelTop <- oModelEdges   <- oModelAtom*n <- Polygon sharing data from MakeAtomes
;** oSurface used for styles   <- oModelAtom*n
;-

common d_liv_con
common d_liv_keepath, kpath
common c_liv_obj , allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj1, a_ElpStruct
common kp_for_anim, maxp,fa,fb,fc,ffb,coef,prov,p25,propag,propagi,atom_type,atom_coo,atom_xyz,N_dens,N_lims,idR,idI,IdP,otit,ppaobj,ltit,reste

if  n_elements(a_symb) le 1 then d_livMakeAtomes
a_newcol =a_col
a_fact   =a_dens*0.+1.
ff       =0.

n =n_elements(atom_tip)
if n eq 0 then begin & n=d_livMolRead(filename, atom_doo, atom_tip, amesh, img, Np, dup, cell, angl) ;Get atoms
                     prov=0    & propag=fltarr(4+1*3*2+1+12,n_elements(atom_tip)) & coef=0 & tit=filename
endif     else begin & prov=prof & propag=propav & coef=coev & endelse

tdup     =dup[0]*dup[1]*dup[2]
minatom  =n
build    =0
rfact    =.5
if n_elements(angl) gt 3 then begin & minatom=angl[3] & build=angl[4] & rfact=angl[5] & angl=angl[0:2] & endif
if (build gt 0) or (n_elements(amesh) gt 1) then dfact=.33 else dfact=1.
if (rfact le 0) or (rfact gt 1)             then rfact=.5

nbfq     =(size(propag))[2] & ndisp=nbfq/n & if ndisp*n ne nbfq then ndisp=1
atom_coo =atom_doo
atom_type=atom_tip
atom_typs=abs(atom_type) <n_elements(a_name)
atom_xyz =atom_coo
N_dens   =0
N_lims   =0
otit     =objarr(6,50)      ;To keep current objects for future animations (planes)
                            ;   otit(0,*)=title,symbol,clouds,Bar,Palette,vol_dens 5,6 10,11
                            ;   otit(1:3,*)=planes  otit(4,*)=poly
                            ;   otit(5,0)=boxes  otit(5,2)=axes  otit(5,3)=pivot axes in surface
                            ;   otit(5,4)=atom symbol for pivot axes in surface
                            ;   otit(5,5)=pivot axes in top model (not used -> oModelPivot)
ltit     =fltarr(5,50)      ;To keep current properties for animations,planes,polygons
                            ;   tit(0,0:13)=miscel  ltit(1:3,*)=planes_attr ltit(4,*)=poly_attr
ppaobj   =objarr(8 ,n*tdup) ; " "
reste    =fltarr(14,n*tdup) ; " " 10:12 reserved for displacement
reste[13,*]=-2              ;        13 index of visible atoms -2=no object -1=hidden else >=0
atom_typn=lonarr(   n*tdup)
atom_coon=fltarr(3 ,n*tdup)
atom_xyzn=fltarr(3 ,n*tdup)
propagn  =fltarr((size(propag))[1],n*tdup+n*(ndisp-1)) ;prevent from growing displacement buffer!!
propagi  =lonarr(   n*tdup*ndisp)                      ;so use this buffer index

;propagation vector?
;******************
p25=n_elements(prov)/3  &  idm=4+p25*3*2+p25

d_livChangAxis, angles=angl
angles=angl*!dpi/180.

fa=Np[1]-Np[0] & fb=Np[3]-Np[2] & fc=Np[5]-Np[4] ;cell size
if cell eq 1 then begin
    atom_xyz[0,*]=atom_xyz[0,*]*fa & atom_xyz[1,*]=atom_xyz[1,*]*fb & atom_xyz[2,*]=atom_xyz[2,*]*fc
endif

;Change axes for angles ~= 90
;****************************
d_livChangAxis, atom_xyz

;Center to gravity point
;***********************
if (cell eq 0) and  ((SIZE(img))[0] lt 3) then begin
 QX=total(atom_coo[0,*]) /n
 QY=total(atom_coo[1,*]) /n
 QZ=total(atom_coo[2,*]) /n
;QX = .5*fa & QY = .5*fb & QZ = .5*fc ;Should be correct
 xoi=Np[0]-QX & xoa=Np[1]-QX
 yoi=Np[2]-QY & yoa=Np[3]-QY
 zoi= min(atom_coo[2,*]-QZ,max=zoa)
endif else begin
 QX = .5*fa & QY = .5*fb & QZ = .5*fc
 xoi=-.5*fa & yoi=-.5*fb & zoi=-.5*fc & xoa=-xoi & yoa=-yoi & zoa=-zoi
endelse
ltit[0,3]=xoa-xoi & ltit[0,4]=yoa-yoi & ltit[0,5]=zoa-zoi
ltit[0,6:8]=dup   & ltit[0,9]=rfact   & ltit[0,10]=n  & ltit[0,12]=dfact & ltit[0,13]=cell
ffb=[0.,0.,0.]

d_livChangAxis, QX,QY,QZ
atom_xyz[0,*]=atom_xyz[0,*]- QX
atom_xyz[1,*]=atom_xyz[1,*]- QY
atom_xyz[2,*]=atom_xyz[2,*]- QZ
;***********************

scx= 0.35 * 2.0 /dup[0] & if (xoa-xoi) ne 0 then scx=scx/(xoa-xoi) ;X extent
scy= 0.35 * 2.0 /dup[1] & if (yoa-yoi) ne 0 then scy=scy/(yoa-yoi) ;Y extent
scz= 0.35 * 2.0 /dup[2] & if (zoa-zoi) ne 0 then scz=scz/(zoa-zoi) ;Z extent
sc = min([scx,scy,scz])

fkx=(dup[0]-1)/2. & fky=(dup[1]-1)/2. & fkz=(dup[2]-1)/2.

state.oModelTop     = OBJ_NEW('IDLgrModel')   ;Top model
state.oModelSurface = OBJ_NEW('IDLgrModel')
state.oModelTop->add, state.oModelSurface

obox =OBJ_NEW('IDLgrModel') & otit[5,0]=obox  ;Boxes
oaxes=OBJ_NEW('IDLgrModel') & otit[5,2]=oaxes ;Main axes
oax0s=OBJ_NEW('IDLgrModel') & otit[5,3]=oax0s ;Centered pivot axes (View point in oModelSurface)
state.oModelSurface->Add,  [obox,oaxes,oax0s]

state.oModelPivot  = OBJ_NEW('IDLgrModel')    ;Centered pivot axes (View point in Top model)
state.oModelOffset = OBJ_NEW('IDLgrModel')
state.oModelOffset->translate, 0, 0, 0.005    ;Offset Z to make visible
state.oModelEdges  = OBJ_NEW('IDLgrModel')
state.oModelOffset->add, state.oModelEdges


; destroy the old objects prior to saving the new ones
obj_destroy, state.oSurface
state.oSurface = OBJ_NEW('IDL_CONTAINER')
state.oSticks  = OBJ_NEW()

;Create image of density if exists
;*********************************
iz= SIZE(img)
if (iz[0] ge 2) then begin
    if (sc eq scx) and (tdup eq 1) then begin ;(Place for color bar)
       ff=.125/sc & atom_xyz[0,*]=atom_xyz[0,*] - ff & xoi=xoi-ff & xoa=xoa-ff & ffb[0]=ff
    endif
    aa=(xoa-xoi)*(-fkx)     & bb=(yoa-yoi)*(-fky)     & cc=(zoa-zoi)*(-fkz)
    bx=[xoi,xoa,xoa,xoi]+aa & by=[yoi,yoi,yoa,yoa]+bb & bz=[zoi,zoi,zoi,zoi]+cc
    d_livChangAxis, bx,by,bz

    TVLCT ,r, g, b ,/get & sz=n_elements(r)

    dmi=min(img,max=dma) & dmii=dmi & dmaa=dma
    if iz[0] ge 3 then begin & deal=90 & bin1=(deal/iz[1])>1 & bin2=(deal/iz[2])>1 & bin3=(deal/iz[3])>1
                             admi=0. & if iz[0] eq 4 then n4=iz[4] else n4=1
                            ;Increase img if to small
                             if bin1*bin2*bin3 gt 1 then img=reform(rebin(img,bin1*iz[1],bin2*iz[2],bin3*iz[3],n4))
           iz= SIZE(img)
           N_dens=img*0. & N_lims=fltarr(2,n4+2)       ;N_lims first entry   is for current volume#
           N_lims[*,n4+1]=[dmi,dma]                    ;N_lims last  entries is for min,max ranges
           for i=n4-1,0,-1 do begin & if n4 gt 1 then dmii=min(img[*,*,*,i], max=dmaa)
                              N_dens[*,*,*,i]=smooth(img[*,*,*,i],3)>dmii<dmaa
                              N_lims[*,i+1]=[dmii,dmaa] & N_lims[0,0]=i & endfor
           imb=N_dens[*,*,0,0]
           if dmii ge 0. then imb=bytscl(sqrt(imb),min=sqrt(dmii),max=sqrt(dmaa)) $
                         else imb=bytscl(imb,min=dmii,max=dmaa)
    endif else  imb=bytscl(img,min=dmii,max=dmaa)

    rg=float(dma-dmi)/(sz-1) & zh=round(-dmi/rg)    & admi=abs(dmi)
    if (zh gt 1) and (zh lt sz) then begin & zh=zh+1
             gray=bytscl((findgen(zh))/zh*sz)
             if zh lt sz-2 then begin & q=sz-zh & r[zh:sz-1]=congrid(r,q) & g[zh:sz-1]=congrid(g,q) & b[zh:sz-1]=congrid(b,q) & endif
             r[0:zh-1]=gray & g[0:zh-1]=gray  & b[0:zh-1]=gray
    endif else admi=0.
    oP = OBJ_NEW('IDLgrPalette',r,g,b)
    oI = OBJ_NEW('IDLgrImage',imb,palette=oP) & otit[0,4]=oP

    state.oModelImage  = OBJ_NEW('IDLgrModel')
    state.oImage = OBJ_NEW('IDLgrPolygon', bx, by, bz,$
                                 poly=[4,0,1,2,3], color=[255,255,255], /texture_interp,shading=0, $
                                 texture_map=oI, texture_coord=[[0,0],[1,0],[1,1],[0,1]])

    IF (!VERSION.release ge '5.2') and (tdup eq 1) then begin
      xrange=[0,1] & yrange=[0,1] &  zrange=[0,1]
      Bar = OBJ_NEW('IDLgrColorBar', R,G,B, COLOR=[255, 255, 255], SHOW_AXIS=2, /THREED)
      Bar-> GetProperty, XRANGE=xrange, YRANGE=yrange, ZRANGE=zrange, TICKLEN=tl
      xLen = xrange[1]-xrange[0]
      yLen = yrange[1]-yrange[0]
      zLen = zrange[1]-zrange[0]
      maxLen =  yLen /(yoa-yoi)
      xs = [xoa+.4+aa]
      ys = [yoi   +bb]
      zs = [zoi   +cc]
      d_livChangAxis, xs,ys,zs
      xs = [xs  , 1.0 / maxLen]
      ys = [ys  , 1.0 / maxLen]
      zs = [zs  , 1.0 / maxLen]
      Bar-> SetProperty, XCOORD_CONV=xs, YCOORD_CONV=ys, ZCOORD_CONV=zs, TICKLEN=tl/2.5
      Bar-> GetProperty, TICKVALUES=tkv
      Tex = OBJ_NEW('IDLgrText',STRING(float(tkv*(dma+admi)/yLen-admi),FORMAT='(G)'))
      Bar-> SetProperty, TICKTEXT=Tex
      state.oModelImage   ->Add, Bar
      otit[0,3]=Bar
    endif

   ;state.oModelImage   ->Add, state.oImage
    state.oModelImage   ->add, OBJ_NEW('IDLgrLight', TYPE=0, INTENSITY=.9, COLOR=[255,255,255])
    state.oModelSurface ->Add, state.oModelImage
endif

;Do not use default size if magnetic or (width ne 1.)
;***********************
sized=0
if p25 ge 1 then begin ;We have some irrows
   idR=4+lindgen(p25*3) & idI=idR+p25*3 & idP=4+lindgen(p25)+p25*3*2
   idx=where(atom_type lt 0)
   if idx[0] ge 0 then begin
           sized=1
           allR=sqrt(total(propag[idR,*]^2,1))
           allI=sqrt(total(propag[idI,*]^2,1))
           maxp=max([max(allR[idx]),max(allI[idx])])/5.5 ;/min([fa,fb,fc]) ; Will normalize with maxp
           taba=fltarr(3,1)+1.
           for i=0,n_elements(idx)-1 do begin     ;Find average length for atoms in cell, put in TABA
               j=idx[i] & lenat=allR[j] & if allI[j] gt lenat then lenat=allI[j]
               ida=where(taba[0,*] eq atom_type[j]) & ida=ida[0]
               if ida eq -1 then taba=[[taba],[atom_type[j],lenat,1.]] $
               else begin & taba[2,ida]=taba[2,ida]+1. & taba[1,ida]=taba[1,ida]+lenat & endelse
           endfor
           taba[1,*]=taba[1,*]/taba[2,*] & tabaT=taba[0,1:*] & tabaL=taba[1,1:*]
   endif else begin
           idx=where(propag[1,*] ne 1.)
           if idx[0] ge 0 then sized=1
   endelse
   propag[0,*]=propag[0,*]>1<50 & nsites=max(propag[0,*]) & sfct=size(a_fact)
   if nsites gt 1 then begin & scol=size(a_newcol) & a_newcol=rebin(a_newcol,scol[1],scol[2],nsites)
                                                   a_fact  =rebin(a_fact  ,sfct[1],nsites,2)
                  endif else                       a_fact  =rebin(a_fact  ,sfct[1], 1    ,2)
   if min(atom_type) lt 0 then modules=fltarr(11,n,nsites,dup[0]*dup[1]*dup[2])
endif

nnn=-1
;*************
;For each cell
;*************
for kz=0,dup[2]-1 do begin
for ky=0,dup[1]-1 do begin
for kx=0,dup[0]-1 do begin

aa=(xoa-xoi)*(kx-fkx) & bb=(yoa-yoi)*(ky-fky) & cc=(zoa-zoi)*(kz-fkz)
d_livChangAxis, aa,bb,cc

;******************************************************
;For each atom calculate cell translation & make object
;******************************************************
 for i =0,n-1 do begin

    if atom_type[i] eq a_ElpStruct.elp then Elp=1 else Elp=0
    show=1
;   Do not show duplicated atoms (edges & corners)
;   **********************************************
    kkz=kz & kky=ky & kkx=kx
    if cell then begin
       if atom_coo[0,i] ge 1 then kkx=kkx+1 else if atom_coo[0,i] le 0 then begin & if kx gt 0 then show=0 else if atom_coo[0,i] lt 0 then kkx=kkx-1 & endif
       if atom_coo[1,i] ge 1 then kky=kky+1 else if atom_coo[1,i] le 0 then begin & if ky gt 0 then show=0 else if atom_coo[1,i] lt 0 then kky=kky-1 & endif
       if atom_coo[2,i] ge 1 then kkz=kkz+1 else if atom_coo[2,i] le 0 then begin & if kz gt 0 then show=0 else if atom_coo[2,i] lt 0 then kkz=kkz-1 & endif
    endif
    nnn=nnn+1
    if show then begin
       newradIrrow=0. & newlng=0. & newhig=0. & newrot=0. & afact=1. & lfact=1. & pa=obj_new()

       if Elp then begin                             ; Make an ellipsoid
          a_ElpStruct.abc   =propav[4  :4+2  ,i]
          off=(p25>1)*3
          a_ElpStruct.angles=propav[4+off:4+off+2,i]
          s = d_livMakeEllips(atom_typs[i])
          s ->GetProperty, POBJ=sh
          newradspher=1. & inispher=1. & curspher=1.
       endif else begin                              ; Make a  sphere for the atom
          s = d_livMakeSphere(atom_typs[i])
          s ->GetProperty, POBJ=sh  , RADINI=inispher, RADIUS=curspher
          newradspher=inispher
          if (atom_type[i] lt 0) then begin          ; Make an Irrow for the atom
            sa= d_livMakeIrrow(atom_typs[i])
            sa->GetProperty, POBJ=ah  , RADINI=iniIrrow, RADIUS=curIrrow, LENGTH=curlng
            newradIrrow=iniIrrow & newlng=curlng
          endif
       endelse
       if i lt minatom then isr =strtrim(string(i+1),2) else isr=''
       uval=string(a_name[atom_typs[i]]+isr, atom_coo[0,i], atom_coo[1,i],atom_coo[2,i], format='(A, " [", 3F6.2, "]")')
       if p25 ge 1 then begin
       ;Change default color and size? (we have some irrows)
        if propag[1,i] eq 0 then afact=0 else if propag[1,i] eq 1 then afact=a_fact[atom_typs[i],propag[0,i]-1,0] $ ;size
                            else begin & afact=propag[1,i]   & a_fact[atom_typs[i],propag[0,i]-1,0]=afact & endelse

        if propag[3,i] eq 0 then lfact=0 else if propag[3,i] eq 1 then lfact=a_fact[atom_typs[i],propag[0,i]-1,1] $ ;length
                            else begin & lfact=propag[3,i]   & a_fact[atom_typs[i],propag[0,i]-1,1]=lfact & endelse

        if propag[2,i] ge 0 then begin & v=long(propag[2,i]) & n1=v/10000 & n2=v/100-n1*100                           ;color
                            n3=v-n1*10000-n2*100 & a_newcol[*,atom_typs[i],propag[0,i]-1]=[n1,n2,n3]*255/10     & endif

        if (atom_type[i] lt 0) then begin  ; Irrow: use propagation vector PROV

           idx=where(tabaT eq atom_type[i])
           newradspher=tabaL[idx[0]]/maxp/af_lng/af_rad*afact
           MOMENT_CALC, i, kkx,kky,kkz,afact,lfact,rfact, newradspher,newradIrrow,newlng,newhig,newrot, Mlj,modu
          ;Fill modules table
           sit=propag[0,i]-1 & cece =(kx+1)*(ky+1)*(kz+1)-1
           modules[0   ,i, sit,cece]= atom_typs[i]
           modules[1:3 ,i, sit,cece]=[atom_coo[0,i],atom_coo[1,i],atom_coo[2,i]]
           modules[4:10,i, sit,cece]=[Mlj , modu , kx,ky,kz]
           uval=string(a_symb[atom_typs[i]]+isr, modu,Mlj[0], Mlj[1],Mlj[2], format='(A, " m=", F6.3," Mc=", 3F6.2)')

       endif  else newradspher=newradspher*afact
    endif

    oModelAtom = OBJ_NEW('IDLgrModel', /SELECT_TARGET) ;A label for the atom
    oModelAtom->SetProperty, UVALUE=uval
   
    ;Draw a sphere or an ellipsoid
    ;*****************************
    sh->GetProperty, POLY  =pmesh
    if p25 ge 1 then sit=propag[0,i]-1 else sit=0
    if atom_type[i] lt 0 then col=a_col[*,atom_typs[i]] else col=a_newcol[*,atom_typs[i],sit]
    if (sized) or (inispher ne curspher) then begin
               if (newradspher gt 0.) and (round(newradspher*100) ne round(curspher*100)) then s ->SetProperty, RADIUS=newradspher
               sh->GetProperty, DATA=data
               p = OBJ_NEW('IDLgrPolygon' , DATA=data,     POLY=pmesh, COLOR=col)
               if (newradspher le 0.) then p->SetProperty, HIDE=1
    endif else p = OBJ_NEW('IDLgrPolygon' , SHARE_DATA=sh, POLY=pmesh, COLOR=col)

    p ->SetProperty, SHADING=1
    oModelAtom    ->Add, p     ;OBJ 0 (position 0)

    tsym = a_symb[atom_typs[i]]

    if Elp then begin ;Rotate the ellipsoid if there
        tmx=a_ElpStruct.angles[0]
        tmy=a_ElpStruct.angles[1]
        tmz=a_ElpStruct.angles[2]
        d_livChangAxis, tmx,tmy,tmz

        if tmx ne 0. then oModelAtom->Rotate,[1,0,0],tmx          ;Rotate
        if tmy ne 0. then oModelAtom->Rotate,[0,1,0],tmy
        if tmz ne 0. then oModelAtom->Rotate,[0,0,1],tmz
        tsym = ''
        bIm = bytarr(4,2,2) & bIm[0:2,*,*]=200 & bIm[3,*,*]=220   ;Make it transparent
        oIm = OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
        p ->SetProperty, TEXTURE_map = oIm
    endif
   
    otsym=OBJ_NEW('IDLgrText',tsym,location=[0,0,0],color=[0,0,0],alignment=.5,vertical=.5)
    otsym->SetProperty,ONGLASS=1,HIDE=1
    oModelAtom    ->Add, otsym ;OBJ 1 (position 1)

    ;Draw an Irrow
    ;*************
    if atom_type[i] lt 0 then begin
        if (newradIrrow gt 0.) and ((round(newradIrrow*100) ne round(curIrrow*100)) $
                               or   (round(curlng*100) ne round(newlng*100))) then  $
                               sa->SetProperty, RADIUS=newradIrrow, LENGTH=newlng
        colo=(a_newcol[*,atom_typs[i],sit]-60)>0
        ah->GetProperty, POLY=pmosh, DATA=data
        pa= OBJ_NEW('IDLgrPolygon' , DATA=data,  POLY=pmosh, COLOR=colo)
        if (newradIrrow le 0.) then pa->SetProperty, HIDE=1
        if (p25 lt 1)          then p ->SetProperty, HIDE=1
        pa->SetProperty, SHADING=1
        if p25 ge 1 then begin ;Rotate the Irrow
           oModelAtom->Rotate,[0,1,0],newhig
           oModelAtom->Rotate,[0,0,1],newrot
        endif
        oModelAtom->Scale, .7,.7,.7
        oModelAtom->Add, pa
        state.oSurface->add, pa

    endif else if dfact ne 1. then oModelAtom->Scale, dfact,dfact,dfact

    oModelAtom->Translate,  atom_xyz[0,i]+aa, atom_xyz[1,i]+bb, atom_xyz[2,i]+cc

    state.oSurface->add, p

    state.oModelSurface->Add, oModelAtom

;   Add a mesh object
;   *****************
    if (kx+ky+kz eq 0) and (newradspher gt 0.) then begin
       oModelAtomEdge = OBJ_NEW('IDLgrModel')
       p1 = OBJ_NEW('IDLgrPolyline', SHARE_DATA=sh, POLYLINE=pmesh,LINESTYLE=1, COLOR=(fix(col)+10)<255)
       state.oModelEdges->add, oModelAtomEdge
       oModelAtomEdge->add, p1
       ;Don't add offset here.  Do it below to parent model
       oModelAtomEdge->translate, atom_xyz[0,i]+aa,atom_xyz[1,i]+bb, atom_xyz[2,i]+cc
       if Elp then begin ;Rotate the ellipsoid if there
        if tmx ne 0. then oModelAtomEdge->Rotate,[1,0,0],tmx ;Rotate Ellipse
        if tmy ne 0. then oModelAtomEdge->Rotate,[0,1,0],tmy
        if tmz ne 0. then oModelAtomEdge->Rotate,[0,0,1],tmz
       endif
    endif
  endif ;show
 
; Keep usefull informations for animations
; ****************************************
  if show then begin
    reste[0,nnn] =newradspher &  reste[1,nnn] =newradIrrow  &  reste[2,nnn] =newlng & reste[3,nnn] =newhig
    reste[4,nnn] =newrot      &  reste[5,nnn] =afact        &  reste[6:8,nnn]=[kkx,kky,kkz] & reste[9,nnn] =lfact
    reste[13,nnn]=nnn
    ppaobj[0,nnn]=oModelAtom  &  ppaobj[1,nnn]=p            &  ppaobj[2,nnn]=pa     & ppaobj[5,nnn]=otsym
  endif
  atom_xyzn[*,nnn]=atom_xyz[*,i]+[aa,bb,cc]                 &  atom_typn[  nnn]=atom_type[ i]
  kxkykz=[kx,ky,kz] & if not cell then kxkykz=kxkykz*[fa,fb,fc]
  atom_coon[*,nnn]=atom_coo[*,i]+kxkykz
  if p25 ge 1 then begin
       propagn[*,nnn]=propag[*,i]
       propagi[  nnn]=nnn
       ndup=fix(nnn/n)*n
       if nnn ge n then begin
          idc=where(propag[idm:idm+11,i] gt 0)  ;bonds
          idt=where(propag[idm:idm+11,i] lt 0)  ;polygons
          if idc[0] ge 0 then propagn[idm+idc,nnn]=propagn[idm+idc,nnn]+ndup
          if idt[0] ge 0 then propagn[idm+idt,nnn]=propagn[idm+idt,nnn]-ndup
       endif
       if ndisp gt 1 then begin & tmp=n*tdup*(lindgen(ndisp-1)+1) & idnn=tmp+nnn & idii=tmp/tdup+i
          if ndup eq 0 then propagn[*,idii+n*(tdup-1)]=propag[*,idii]
          propagi[  idnn]=idii+n*(tdup-1)
       endif
  endif
 endfor ;atoms in cell

;   Draw boxes
;   **********
if kx+ky+kz eq 0 then colo=[180,180,180] else colo=[180,180,180]
boxx=[xoi,xoi,xoa,xoa,xoi,xoi,xoi,xoi,xoi,xoa,xoa,xoa,xoa,xoi,xoa,xoa]
boxy=[yoi,yoi,yoi,yoi,yoi,yoa,yoa,yoi,yoa,yoa,yoi,yoi,yoa,yoa,yoa,yoa]
boxz=[zoi,zoa,zoa,zoi,zoi,zoi,zoa,zoa,zoa,zoa,zoa,zoi,zoi,zoi,zoi,zoa]

d_livChangAxis,boxx,boxy,boxz

oboxp= OBJ_NEW('IDLgrPolyline', boxx+aa,boxy+bb,boxz+cc, shading=0, thick=1,COLOR=colo,linestyle=0)
if !version.release ge 8.0 then oboxp->SetProperty, ANTIALIAS=1
obox-> Add, oboxp

;   Highlight axes
;   **************
if kx+ky+kz eq 0 then begin
   if coef  eq 4 then begin & tta='a*' & ttb='b*' & ttc='c*'    & endif else begin & tta='a' & ttb='b' & ttc='c' & endelse
   ot00=OBJ_NEW('IDLgrText','',location=[-.01,-.01,0.],color=[180,180,180],font=fntn)
   ot00->SetProperty,ONGLASS=1 & otit[5,4]=ot00 ;atom symbol for pivot point in surface
   av=2.
   ax_x=[xoi,xoi+(xoa-xoi)*(dup[0]+1/4.)] & ax_y=[yoi,yoi]    & ax_z=[zoi,zoi]
   ax0x=[0.-(xoa-xoi)*dup[0]/2.,0.+(xoa-xoi)*dup[0]/2.] & ax0y=[0.,0.] & ax0z=[0.,0.]
   ax2x=[-.1,.1] & ax2y=[0.,0.] & ax2z=[0.,0.]
   d_livChangAxis,ax_x,ax_y,ax_z
   d_livChangAxis,ax0x,ax0y,ax0z
   d_livChangAxis,ax2x,ax2y,ax2z
   axea=OBJ_NEW('IDLgrPolyline',ax_x+aa ,ax_y+bb ,ax_z+cc , shading=0, thick=4 ,COLOR=[180,180,180])
   ax0a=OBJ_NEW('IDLgrPolyline',ax0x    ,ax0y    ,ax0z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot X in surface
   ax2a=OBJ_NEW('IDLgrPolyline',ax2x    ,ax2y    ,ax2z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot X in top
   if !version.release ge 8.0 then begin
     axea->SetProperty, ANTIALIAS=1
     ax0a->SetProperty, ANTIALIAS=1
     ax2a->SetProperty, ANTIALIAS=1
   endif
   otxa=OBJ_NEW('IDLgrText',tta ,location=[ax_x[1]+aa,ax_y[1]+bb,ax_z[1]+cc],color=[180,180,180],font=fntn)
   ot1a=OBJ_NEW('IDLgrText','x' ,location=[ax0x[1]/av,ax0y[1]/av,ax0z[1]/av],color=[180,180,180],font=fntn)      ;pivot 'x' in surface
   ot2a=OBJ_NEW('IDLgrText','x' ,location=[ax2x[1]   ,ax2y[1]   ,ax2z[1]   ],color=[180,180,180],font=fntn)      ;pivot 'x' in top
   otxa->SetProperty,ONGLASS=1
   ot1a->SetProperty,ONGLASS=1
   ot2a->SetProperty,ONGLASS=1

   ax_x=[xoi,xoi] & ax_y=[yoi,yoi+(yoa-yoi)*(dup[1]+1/4.)]    & ax_z=[zoi,zoi]
   ax0x=[0.,0.] & ax0y=[0.-(yoa-yoi)*dup[1]/2.,0.+(yoa-yoi)*dup[1]/2.] & ax0z=[0.,0.]
   ax2x=[0.,0.] & ax2y=[-.1,.1] & ax2z=[0.,0.]
   d_livChangAxis,ax_x,ax_y,ax_z
   d_livChangAxis,ax0x,ax0y,ax0z
   d_livChangAxis,ax2x,ax2y,ax2z
   axeb=OBJ_NEW('IDLgrPolyline',ax_x+aa ,ax_y+bb ,ax_z+cc , shading=0, thick=4 ,COLOR=[180,180,180])
   ax0b=OBJ_NEW('IDLgrPolyline',ax0x    ,ax0y    ,ax0z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot Y in surface
   ax2b=OBJ_NEW('IDLgrPolyline',ax2x    ,ax2y    ,ax2z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot Y in top
   if !version.release ge 8.0 then begin
     axeb->SetProperty, ANTIALIAS=1
     ax0b->SetProperty, ANTIALIAS=1
     ax2b->SetProperty, ANTIALIAS=1
   endif
   otxb=OBJ_NEW('IDLgrText',ttb ,location=[ax_x[1]+aa,ax_y[1]+bb,ax_z[1]+cc],color=[180,180,180],font=fntn)
   ot1b=OBJ_NEW('IDLgrText','y' ,location=[ax0x[1]/av,ax0y[1]/av,ax0z[1]/av],color=[180,180,180],font=fntn)      ;pivot 'y' in surface
   ot2b=OBJ_NEW('IDLgrText','y' ,location=[ax2x[1]   ,ax2y[1]   ,ax2z[1]   ],color=[180,180,180],font=fntn)      ;pivot 'y' in top
   otxb->SetProperty,ONGLASS=1
   ot1b->SetProperty,ONGLASS=1
   ot2b->SetProperty,ONGLASS=1

   ax_x=[xoi,xoi] & ax_y=[yoi,yoi] & ax_z=[zoi,zoi+(zoa-zoi)*(dup[2]+1/4.)]
   ax0x=[0.,0.] & ax0y=[0.,0.] & ax0z=[0.-(zoa-zoi)*dup[2]/2.,0.+(zoa-zoi)*dup[2]/2.]
   ax2x=[0.,0.] & ax2y=[0.,0.] & ax2z=[-.1,.1]
   d_livChangAxis,ax_x,ax_y,ax_z
   d_livChangAxis,ax0x,ax0y,ax0z
   d_livChangAxis,ax2x,ax2y,ax2z
   axec=OBJ_NEW('IDLgrPolyline',ax_x+aa ,ax_y+bb ,ax_z+cc , shading=0, thick=4 ,COLOR=[180,180,180])
   ax0c=OBJ_NEW('IDLgrPolyline',ax0x    ,ax0y    ,ax0z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot Z in surface
   ax2c=OBJ_NEW('IDLgrPolyline',ax2x    ,ax2y    ,ax2z    , shading=0, thick=3 ,COLOR=[180,180,180],linestyle=1) ;pivot Z in top
   if !version.release ge 8.0 then begin
     axec->SetProperty, ANTIALIAS=1
     ax0c->SetProperty, ANTIALIAS=1
     ax2c->SetProperty, ANTIALIAS=1
   endif
   otxc=OBJ_NEW('IDLgrText',ttc ,location=[ax_x[1]+aa,ax_y[1]+bb,ax_z[1]+cc],color=[180,180,180],font=fntn)
   ot1c=OBJ_NEW('IDLgrText','z' ,location=[ax0x[1]/av,ax0y[1]/av,ax0z[1]/av],color=[180,180,180],font=fntn)      ;pivot 'z' in surface
   ot2c=OBJ_NEW('IDLgrText','z' ,location=[ax2x[1]   ,ax2y[1]   ,ax2z[1]   ],color=[180,180,180],font=fntn)      ;pivot 'z' in top
   otxc->SetProperty,ONGLASS=1
   ot1c->SetProperty,ONGLASS=1
   ot2c->SetProperty,ONGLASS=1

   otit[5,2]->Add,  [axea,axeb,axec,   otxa,otxb,otxc]      ; main axes
   otit[5,3]->Add,  [ax0a,ax0b,ax0c,   ot1a,ot1b,ot1c,ot00] & otit[5,3]->SetProperty,HIDE=1 ;(pivot point in oModelSurface)

   state.oModelPivot->Add,[ax2a,ax2b,ax2c,  ot2a,ot2b,ot2c] & state.oModelPivot->SetProperty,HIDE=1 ;(pivot point in Top model)

;   Title
;   *****
   fntn=OBJ_NEW('IDLgrFont','Helvetica*Bold*Italic',size=14.0)
   state.oView  ->GetProperty, viewplane_rect=vpr
   otit[0,0]=OBJ_NEW('IDLgrText',tit,location=[-vpr[2]/2.+.05,vpr[3]/2.-.05,-.5],color=[180,180,180],font=fntn,recompute=2)
  ;otit(0,0)->SetProperty,ONGLASS=1
   state.oModelTop-> Add, otit[0,0]         ;The title
   state.oModelTop-> Add, state.oModelPivot ;The pivot axes used for going "INCELL" the view (d_livDrawPrint)

;  planes
;  ******
   bIm = bytarr(4,2,2) & bIm[0:2,*,*]=180 & bIm[3,*,*]=60
   oIm1= OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
   oIm2= OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
   oIm3= OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
   ax_x=[xoi,xoi,xoi,xoi] & ax_y=[yoi,yoa,yoa,yoi] & ax_z=[zoi,zoi,zoa,zoa] & d_livChangAxis,ax_x,ax_y,ax_z
   opo1= OBJ_NEW('IDLgrPolygon',ax_x+aa,ax_y+bb,ax_z+cc,shading=0,texture_map=oIm1,poly=[4,0,1,2,3],/hide, $
                                color=[255,255,255])
   ax_x=[xoi,xoa,xoa,xoi] & ax_y=[yoi,yoi,yoi,yoi] & ax_z=[zoi,zoi,zoa,zoa] & d_livChangAxis,ax_x,ax_y,ax_z
   opo2= OBJ_NEW('IDLgrPolygon',ax_x+aa,ax_y+bb,ax_z+cc,shading=0,texture_map=oIm2,poly=[4,0,1,2,3],/hide, $
                                color=[255,255,255])
   ax_x=[xoi,xoa,xoa,xoi] & ax_y=[yoi,yoi,yoa,yoa] & ax_z=[zoi,zoi,zoi,zoi] & d_livChangAxis,ax_x,ax_y,ax_z
   if state.oImage ne OBJ_NEW() then opo3=state.oImage else $
   opo3= OBJ_NEW('IDLgrPolygon',ax_x+aa,ax_y+bb,ax_z+cc,shading=0,texture_map=oIm3,poly=[4,0,1,2,3], $
                                color=[255,255,255])
   state.oModelSurface->Add, [opo1,opo2,opo3]
   otit[1:3,0]=[opo1,opo2,opo3]
   ltit[1:3,0]=3 ;Transparent blue

endif
endfor ;a cells
endfor ;b cells
endfor ;c cells

;   We have all Atoms now
;   *********************
atom_coo=atom_coon ; [kx,ky,kz] is added (*[fa,fb,fc] if not cell)
atom_xyz=atom_xyzn ; [aa,bb,cc] is added + QX,QY,QZ + axis changed
if not cell  then begin & fa=1. & fb=1. & fc=1. & endif
if tdup gt 1 then begin & atom_type =atom_typn  & propag=propagn & propagn=0 & endif
nn=n_elements(atom_type)

;   Build connections if asked
;   **************************
if (build gt 0) or (n_elements(amesh) gt 1) then begin
 if build eq 1 then             d_livConnex, atom_type, atom_coo, propag, idm ,ABC=[fa,fb,fc],visidx=reste[13,*]
 if build ge 1 then             d_livBlinks, nn,propag,idm,amesh ;1=to build 2=already there
 if n_elements(amesh) gt 1 then d_livOlinks, state, amesh

 idt=where(propag[idm:idm+11,*] lt 0)
;   Build polyhedres if there
;   *************************
 if idt[0] ge 0 then begin
    links=0L & acenter=0
    for i=0,nn-1 do begin & idt=where(propag[idm:idm+11,i] lt 0) & nidt=n_elements(idt)
      if idt[0] ge 0 then links=[links,nidt+1,i,-propag[idm+idt,i]-1]
    endfor
    nel  =n_elements(links)
    if nel gt 1 then polyh=[links[1:nel-1],-1] else polyh=0
 endif
endif

;   Show polyhedres if there
;   ************************
if (n_elements(polyh) gt 1) then begin
   ;if p25 ge 1 then sit=propag(0,i)-1 else sit=0
   ;col=a_newcol(*,abs(atom_type(i)),sit)
   ;IDLgrTessellator::Tessellate
   ;****************************
   bid=obj_new()
   idx=0 & loo=polyh[idx]
   while loo gt -1 do begin
     vertex_list = [.0,.0,.0] & idx=idx+1 & iato=0
     for i=1,loo do begin & vertex_list = [[vertex_list],[atom_xyz[*,polyh[idx]]]] & iato=[iato,polyh[idx]] & idx=idx+1 & endfor
     vertex_list= vertex_list[*,1:*]
     iato       = iato[1:*]
     ppaobj[1,polyh[idx-1]]-> GetProperty,COLOR=colo
     d_livBuildPoly, vertex_list, opolygon=opoh, opolyline=opol,color=colo,uvalue=iato
     if opoh ne bid then state.oModelSurface->Add, opoh
     if opol ne bid then state.oModelSurface->Add, opol
     loo=polyh[idx]
   endwhile
endif


;   Scale to fit window
;   *******************
state.oModelOffset ->translate, 0, 0, 0.005
state.oModelSurface->Scale, sc, sc, sc
state.oModelEdges  ->Scale, sc, sc, sc

d_livAddLight, state.oModelSurface

if state.vert_coloring then begin
    state.vert_coloring = 0
    d_livMenuSetState, '|Vertex Coloring', 0, Names=state.menuitems, Indices=state.menubuttons
endif

state.oModelTop->add, state.oModelOffset

;   If magnetic structure, write the modules
;   ****************************************
if n_elements(modules) gt 1 then begin & smz=size(modules)
   on_ioerror,mismod & u=-1 & openw,u,'modules.xml',/get_lun
   if coef eq 2 then coeff='tk' else coeff='sk'
   celp=strtrim(string(fa),2)+','+strtrim(string(fb),2)+','+strtrim(string(fc),2)+' , '+strtrim(string(angl[0],format='(f6.2)'),2)+','+strtrim(string(angl[1],format='(f6.2)'),2)+','+strtrim(string(angl[2],format='(f6.2)'),2)
   printf,u,'<magdraw subject="modules" title="'+tit+'">'
   printf,u,'  <param cell="['+celp+']" coeff="'+coeff+'"/>' ;nprop="'+strtrim(string(p25),2)+'"'
   for k=0,p25-1 do begin
     if prov[k*3]   eq 0 then vp=   '0 , ' else vp=   strtrim(string(prov[k*3  ]),2)+' , '
     if prov[k*3+1] eq 0 then vp=vp+'0 , ' else vp=vp+strtrim(string(prov[k*3+1]),2)+' , '
     if prov[k*3+2] eq 0 then vp=vp+'0'    else vp=vp+strtrim(string(prov[k*3+2]),2)
     printf,u,'  <vprop k="'+strtrim(string(k+1),2)+'" value="['+vp+']"/>' &  endfor
   printf,u,''
   if smz[0] ge 4 then mf=smz[4]-1 else mf=0
   if smz[0] ge 3 then sf=smz[3]-1 else sf=0
   for m=0,mf do begin
     for s=0,sf do begin & idx=where(modules[0,*,s,m] gt 0)
       if idx[0] ge 0 then begin & ecel=''
         nat=n_elements(idx) & modu=modules[*,idx,s,m] & abc=strtrim(string(fix(modu[8,0])),2)+','+strtrim(string(fix(modu[9,0])),2)+','+strtrim(string(fix(modu[10,0])),2)
         printf,u,'  <cell abc="['+abc+']" site="'+strtrim(string(s+1),2)+'">'
         for a=0,nat-1 do begin
           xyz =strtrim(string(modu[1,a],format='(f4.2)'),2)+',' +strtrim(string(modu[2,a],format='(f4.2)'),2)+','+strtrim(string(modu[3,a],format='(f4.2)'),2)
           if modu[4,a] eq 0 then mxyz=     '0 ,'  else mxyz=     strtrim(string(modu[4,a]),2)+' , '
           if modu[5,a] eq 0 then mxyz=mxyz+' 0 ,' else mxyz=mxyz+strtrim(string(modu[5,a]),2)+' , '
           if modu[6,a] eq 0 then mxyz=mxyz+' 0'   else mxyz=mxyz+strtrim(string(modu[6,a]),2)
           if a eq nat-1 then ecel='</cell>'
           printf,u,'    <atom symb="'+a_symb[modu[0,a]]+'" xyz="['+xyz+']" m="'+strtrim(string(modu[7,a],format='(f7.4)'),2)+'" mxyz="['+mxyz+']"/>'+ecel
         endfor
       endif
     endfor
   endfor
   printf,u,'</magdraw>'
   if state.l_mess gt 0 then WIDGET_CONTROL, state.l_mess,bad_id=ii,set_value='file "module.xml" is updated ...      '
   mismod: if u gt 0 then free_lun,u
endif
return, state.oModelTop
end

pro d_livBuildPoly, vertex_list, opolygon=opoh, opolyline=opol, color=colo ,uvalue=iato
;+
;** **************
;** build polygon, polyline
;-
loo = n_elements(vertex_list)
if loo ge 6 then begin
   bIm = bytarr(4,2,2) & bIm[0:2,*,*]=200 & bIm[3,*,*]=220
   oIm = OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
   bid =obj_new()
   opoh=bid & opol=bid
   if loo eq 6 then begin            ;a line
        opol= OBJ_NEW('IDLgrPolyline',DATA=vertex_list,poly=[2,0,1] ,shading=0,thick=2,COLOR=colo,uvalue=iato)
        if !version.release ge 8.0 then opol->SetProperty, ANTIALIAS=1

   endif else if loo eq 9 then begin ;a plane
        opoh= OBJ_NEW('IDLgrPolygon' ,DATA=vertex_list,poly=[3,0,1,2],shading=1,texture_map=oIm,color=colo,uvalue=iato)
        opol= OBJ_NEW('IDLgrPolyline',DATA=vertex_list,poly=[2,0,1,2,1,2,2,2,0] ,shading=0,thick=3,COLOR=colo)
        if !version.release ge 8.0 then opol->SetProperty, ANTIALIAS=1

   endif else begin & loo=loo/3        ;a polyhedre
       tri=[.0,.0,.0]
       for i=0,loo-3 do $
        for j=i+1,loo-1 do $
         for k=j+1,loo-1 do tri=[[tri],[i,j,k]]
       if n_elements(tri)/3  ge 2 then begin
        tri =tri[*,1:*]
        ntri=n_elements(tri)/3
        polygon_list = [Replicate(3L, 1L, ntri), Temporary(tri)] & polygon_list = polygon_list[*]
        if ntri gt 6 then $
        opoh= OBJ_NEW('IDLgrPolygon',DATA=vertex_list,poly=polygon_list,shading=0,color=colo,uvalue=iato) else $
        opoh= OBJ_NEW('IDLgrPolygon',DATA=vertex_list,poly=polygon_list,shading=1,color=colo,uvalue=iato,texture_map=oIm)
       endif
   endelse
endif
end

pro d_livConnex, atom_type, atom_coo, propag, idm, angles=angl, abc=lat ,factor=fct ,visidx=adx
;+
;** ***********
;** identify atom connexions
;-
 common c_liv_obj2, a_coval , a_rvdw , a_mass

 nn=n_elements(atom_type) & X=reform(atom_coo[0,*])*lat[0] & Y=reform(atom_coo[1,*])*lat[1] & Z=reform(atom_coo[2,*])*lat[2]
 if n_elements(adx) ne nn then adx=lonarr(nn)
 atyp=abs(atom_type)

 if (n_elements(angl) eq 3) and (n_elements(lat) eq 3) then d_livChangAxis, angles=angl

 if n_elements(fct) ne 1 then fct=1.

 for i=0,nn-2 do if adx[i] ge 0 then begin
   XX=round((X-X[i])*100)/100. & YY=round((Y-Y[i])*100)/100. & ZZ=round((Z-Z[i])*100)/100. & kk=0
   d_livChangAxis, XYZ=transpose([[xx],[yy],[zz]]), GETMODULE=modu
   irv=a_rvdw[atyp[i]]*fct
   for j=i+1,nn-1 do if adx[j] ge 0 then begin
      if (modu[j] le irv) or (modu[j] le a_rvdw[atyp[j]]*fct) then begin
          if kk lt 12 then propag[idm+kk,i]=j+1
          kk=kk+1 & endif
   endif
 endif
end

pro d_livBlinks, nn,propag,idm,amesh
;+
;** ***********
;** make mesh for atom connections
;-
 links=0L
 for i=0,nn-1 do begin & k=0
   while ((k lt 11) and (propag[idm+k,i] ne 0)) do begin
     if propag[idm+k,i] gt 0 then links=[links,2,i,propag[idm+k,i]-1] & k=k+1
  endwhile & endfor
 nel  =n_elements(links)
 if nel gt 1 then amesh=links[1:nel-1] else amesh=0
end

pro d_livNlinks, nn,propag,idm,table
;+
;** ***********
;** used to show only connected atoms
;-
 table=lonarr(nn)
 for i=0,nn-1 do begin & k=0
   while ((k lt 11) and (propag[idm+k,i] ne 0)) do begin
     if propag[idm+k,i] gt 0 then begin & j=propag[idm+k,i]-1 & table[i]=table[i]+1 & table[j]=table[j]+1 & endif
     k=k+1
   endwhile & endfor
end

pro d_livOlinks, state, amesh, new_xyz=new_xyz
;+
;** ***********
;** plot atom connections
;-
common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common kp_for_anim, maxp,fa,fb,fc,ffb,coef,prov,p25,propag,propagi,atom_type,atom_coo,atom_xyz,N_dens,N_lims,idR,idI,IdP,otit,ppaobj,ltit,reste

if (n_elements(amesh) gt 1) then begin
    nn=n_elements(atom_type)
    vcol=bytarr(3,nn)
    bid=OBJ_NEW()
    for i=0,nn-1 do if ppaobj[1,i] ne bid then begin & ppaobj[1,i]->GetProperty,COLOR=col & vcol[*,i]=col & endif ;=a_col(*,abs(atom_type(i)))
    state.oModelSticks = OBJ_NEW('IDLgrModel')
    if n_elements(new_xyz) gt 1 then begin
      state.oSticks = OBJ_NEW('IDLgrPolyline', DATA=new_xyz,  POLYLINE=amesh, shading=1, thick=3, vert_colors=vcol) ;COLOR=[50,50,50])
    endif else $
      state.oSticks = OBJ_NEW('IDLgrPolyline', DATA=atom_xyz, POLYLINE=amesh, shading=1, thick=3, vert_colors=vcol) ;COLOR=[50,50,50])

    if !version.release ge 8.0 then state.oSticks->SetProperty, ANTIALIAS=1


    state.oModelSticks ->Add, state.oSticks
    state.oModelSurface->Add, state.oModelSticks
endif
end

pro moment_calc, i, kkx,kky,kkz,afact,lfact,rfact, newradspher,newradIrrow,newlng,newhig,newrot, Mlj,modu ,tmx,tmy,tmz
;+
;** ***********
;** calculate moment for arrow and return arrow dimension, direction
;** Input i,kkx,kky,kkz,afact,lfact,rfact
;** Output new... variables + Mlj,modu,tmx,tmy,tmz
;** Share this code for animations
;-
common kp_for_anim, maxp,fa,fb,fc,ffb,coef,prov,p25,propag,propagi,atom_type,atom_coo,atom_xyz,N_dens,N_lims,idR,idI,IdP,otit,ppaobj,ltit,reste
common d_liv_con

           RL=[kkx, kky, kkz]  & idm =4+p25*3*2+p25
           if coef eq 1 then RL= RL+ propag[idm+12:idm+14,i]                     mod 1. ;Sk care with translation
           if coef eq 2 then RL= RL+[atom_coo[0,i],atom_coo[1,i],atom_coo[2,i]]  mod 1. ;Tk
           RL=reform(RL,1,3)
           propaR=propag[idR,i]
           propaI=propag[idI,i]
           phase =propag[idP,i]
           Mlj=[0.,0.,0.]
           for k=0,p25-1 do begin & vv=RL#prov[k*3:k*3+2] &   vv=2*!pi*(vv[0]+ phase[k])
               Mlj=Mlj+ propaR[k*3:k*3+2]*cos(vv)   +   propaI[k*3:k*3+2]*sin(vv)
           endfor

       ;new length for Irrow
           tmx=Mlj[0]  & tmy=Mlj[1]  & tmz=Mlj[2]
           d_livChangAxis, xyz=Mlj, getmodule=modu
           newlng=modu/maxp
           newradIrrow=(newlng/af_lng)
           newlng=newlng * lfact

           if afact gt 0. then newradIrrow=newradIrrow*afact
           newradIrrow=newradIrrow<(newradspher*rfact)

       ;Calculate direction for Irrow
           d_livChangAxis, tmx,tmy,tmz
           newhig=90-atan(tmz,sqrt(tmx^2+tmy^2))*180./!pi
           newrot=atan(tmy,tmx)*180./!pi
end

;**
;******* **************
Function d_livMakeShape, filename, state ,xr,yr,zr,mesh
;+
;******* **************
;** make only a shape from 3D input (state)
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

state.oModelTop = OBJ_NEW('IDLgrModel')       ;Top model

state.oSurface = d_livReadNoff(filename, xr, yr, zr ,mesh)
state.oSurface->SetProperty, VERT_COLORS= vcc

state.oModelSurface = OBJ_NEW('IDLgrModel')
state.oModelTop->add, state.oModelSurface
state.oModelSurface->add, state.oSurface

state.oModelPivot   = OBJ_NEW('IDLgrModel')  ;Centered pivot axes (View point in Top model)
                                             ;To Do as in MakeMolecule
state.oModelOffset  = OBJ_NEW('IDLgrModel')
state.oModelOffset->translate, 0, 0, 0.005   ;Offset Z to make visible
state.oModelEdges   = OBJ_NEW('IDLgrModel')
state.oModelOffset->add, state.oModelEdges
state.oSurface->GetProperty, POLY=pmesh
p1 = OBJ_NEW('IDLgrPolyline', SHARE_DATA=state.oSurface, POLY=pmesh, COLOR=[0,0,0])

state.oModelEdges->add, p1                   ;Add the edging data

if state.vert_coloring eq 1 then begin       ;Default = vertex coloring OFF
    state.vert_coloring = 0
    d_livMenuSetState, '|Vertex Coloring', 0, NAMES=state.menuitems, INDICES=state.menubuttons
endif

state.oModelTop->add, state.oModelOffset

d_livAddLight, state.oModelTop

return, state.oModelTop
end

;**
;** ***************
pro d_livGetLattice, get=lat ,set=pp ,wdraw=wdraw
;+
;** ***************
;** analyse lamp workspace input parameter to show a cell or a shape
;-
common c_magpla,Pcur,Wdd

if arg_present(pp)  then begin & Pcur=pp & Wdd=wdraw & endif
lat=[0.,1.,0.,1.,0.,1.] & wdraw=Wdd
if n_elements(Pcur) gt 14 then if Pcur[14] eq 4 then lat=Pcur[1:6]
widget_control , wdraw  , get_value=bidon, bad_id=ii
if ii ne 0 then  wdraw=0
end

;**
;******* ****************
Function d_livMakeLampWks, filename, state
;+
;******* ****************
;** the lamp entry
;** call d_livMakeMolecule after analysing the molecule (symetries etc ...)
;** or   d_livMakeShape if input not in cell
;-

common c_liv_obj1, a_ElpStruct

wi=fix(filename)
if wi gt 0 then CALL_PROCEDURE,'take_datp',datp,W=wi
if wi gt 0 then state.did_wi=wi
if (n_elements (state.did_xx)  eq 1) and (wi gt 0) then xr  =datp.x     else xr  =state.did_xx
if (n_elements (state.did_yy)  eq 1) and (wi gt 0) then yr  =datp.y     else yr  =state.did_yy
if (n_elements (state.did_zz)  eq 1) and (wi gt 0) then zr  =datp.z     else zr  =state.did_zz
if (n_elements (state.did_ee)  eq 1) and (wi gt 0) then ee  =datp.e     else ee  =state.did_ee
if (n_elements (state.did_nn)  eq 1) and (wi gt 0) then N   =datp.n     else N   =state.did_nn
if (n_elements (state.did_tit) eq 1) and (wi gt 0) then tit =datp.w_tit else tit =state.did_tit
if (n_elements (state.did_vv)  eq 1) and (wi gt 0) then mesh=datp.PV    else mesh=state.did_vv
if (n_elements (state.did_pp)  eq 1) and (wi gt 0) then P   =datp.P     else P   =state.did_pp
if (n_elements (state.did_ww)  eq 1) and (wi gt 0) then CALL_PROCEDURE,'take_w',W,W=wi  else W=state.did_ww
P= float(P)
d_livGetLattice, set=P ,wdraw=state.wDraw
thresh=state.did_thresh
repr  =state.did_repr
case P[0] of
 1:   begin ;Molecule (p1->p6=dimension, p7->p9 =angles, p10=#atoms)
            ;         (p11->p13=dupl, p14=vector c or s or dyn or hkl, p15=modul p or a, p16=translat 0:8)
            ;         (p17=Fourier coef cell or atom, p18=Bonds to build)
            ;         (p19=Maximum Irrow/atom radius ratio)
            ;         (p20=1 extend molecule outside cell)
            ;         (p25=#propag vectors, p26->p#*3=pvectors x1,y1,z1,x2,y2,z1,...)

            ;         (mesh=links, N=density pv0=site, pv1=width, pv2=color(rrggbb)or-1 pv3=moment_length)
            ;         (pv4->pv4+p25*3-1=Real vectors, pv4+p25*3->pv4+p25*3*2-1=Imag vectors, pv4+p25*3*2->pv4+p25*3*2+p25-1=phase)
            ;         (pvx->pvx+11=connections)
      sz=SIZE(N)
      pe=n_elements(P)
      if pe ge  7 then Np =p[ 1: 6] else Np =[-1,1,-1,1,-1,1]
      if pe ge 11 then nat=p[10]    else nat= 0
      if pe ge 14 then dup=p[11:13] else dup=[1,1,1] & dup=dup>1<50
      if pe ge 15 then p14=p[14]    else p14= 0
      if pe ge 17 then p16=p[16]    else p16= 0
      if pe ge 18 then p17=p[17]    else p17= 0
      if pe ge 19 then p18=p[18]    else p18= 0
      if pe ge 20 then p19=p[19]    else p19= 0
      if pe ge 21 then p20=p[20]    else p20= 0 ; & p20=0
      if pe ge 26 then p25=p[25]    else p25= 0
      if p14 ge 3 then p17=p14

      m3d =0
      idm =4+p25*3*2+p25
      mina=min([xr,yr,zr],max=maxa)
      ;Cell or not cell ?
      if (mina  lt -1.) or (maxa  gt 2.) then   cell=0  else cell =1
      if (Np[0] ne 0) or (Np[2] ne 0)  or  (Np[4] ne 0) then cell =0
      if (Np[1]*Np[3]*Np[5] eq 1) then  begin &   cell=0 & Np[0]=min(xr,max=tib) & Np[1]=tib
                                                         Np[2]=min(yr,max=tib) & Np[3]=tib
                                                         Np[4]=min(zr,max=tib) & Np[5]=tib & endif
       pvsz=size(mesh) & pv_plan=0
       ;Move polygons in pv_plan
       if (pvsz[0] eq 2) and (pvsz[2] gt nat) then begin & tmp=mesh[0,*] & tmx=where(tmp lt 0)
           if tmx[0] ge 0 then begin & pv_plan=mesh[*,tmx]  &  mesh=mesh[*,where(tmp ge 0)]  & endif
           pvsz=size(mesh)
       endif

      ;Add space in mesh for translation info (mode Sk)
      if (p16 gt 1) or ((p25 ge 1) and (p17 eq 1)) then if pvsz[0] eq 2 then mesh=[mesh,fltarr(3,pvsz[2])] else mesh=[mesh,fltarr(3)]

      if cell then if (mina  lt 0.) or (maxa  gt 1.) then begin & xr=xr mod 1.  & yr=yr mod 1.  & zr=zr mod 1.  & endif
      if cell and (p20 eq 0) then begin

       if p16 gt 1 then begin
              ;Primitive     C-face centred    A-face centred      B-face centred    Body centred
              ;All-face centred                                    Rhombohedrally centred (obverse)
              ;Rhombohedrally centred (reverse)                    Hexagonally centred
       transl=[[0., 0., 0.], [1./2, 1./2, 0.], [0., 1./2, 1./2],   [1./2, 0., 1./2], [1./2, 1./2, 1./2]  $
              ,[1./2, 1./2, 0.],[0., 1./2, 1./2],[1./2, 0., 1./2], [2./3, 1./3, 1./3],[1./3, 2./3, 2./3] $
              ,[1./3, 2./3, 1./3],[2./3, 1./3, 2./3],              [2./3, 1./3, 0],[1./3, 2./3, 0.]]
       transi=[[0,1],[1,1],[2,1],[3,1],[4,1],[5,3],[8,2],[10,2],[12,2]]
       i1 =transi[0,p16-1] & i2=transi[1,p16-1] & tw=w & tx=xr & ty=yr & tz= zr & pv=mesh ;& pv(idm:idm+11,*)=0.
       psub=pv[idm:idm+11,*]

       idc=where(psub gt 0) ;bonds
       idp=where(psub lt 0) ;polygons
       for k=i1,i1+i2-1 do begin & W=[W,tw] & XR=[xr,tx+transl[0,k]] & YR=[yr,ty+transl[1,k]] & ZR=[zr,tz+transl[2,k]]
                           if idc[0] ge 0 then psub[idc]=psub[idc]+nat
                           if idp[0] ge 0 then psub[idp]=psub[idp]-nat
         pv[idm   :idm+11,*]=psub                             ;bonds,poly  info
         for i=0,nat-1 do pv[idm+12:idm+14,i]=transl[*,k]     ;translation info
         mesh=[[mesh],[pv]]
       endfor
       idx=where(xr gt 1.) & if idx[0] ge 0 then xr[idx]=xr[idx] mod 1.
       idx=where(yr gt 1.) & if idx[0] ge 0 then yr[idx]=yr[idx] mod 1.
       idx=where(zr gt 1.) & if idx[0] ge 0 then zr[idx]=zr[idx] mod 1.
       endif ;translations

       xr0 =xr & yr0=yr & zr0=zr
       nat =n_elements(w)
       pvsz=size(mesh) & m3d=1
       if (pvsz[0] eq 2) and (pvsz[2] gt nat) and ((pvsz[2] mod nat) eq 0) then mesh=reform(mesh,pvsz[1],nat,pvsz[2]/nat) else m3d=0

      ;check for missing atoms on corners.
       idc=where(xr0+yr0+zr0 eq 0)
       if (idc[0] ge 0) and (n_elements(idc) lt 8) then begin              ; corn=[[0,0,0],[0,1,0],[0,0,1],[0,1,1],[1,0,0],[1,1,0],[1,0,1],[1,1,1]]
        wtmp=w[idc[0]] & pvtmp=mesh[*,idc[0],*] & pvtmp[idm:idm+11,*,*]=0. & corc=[ 0     , 10    , 100   , 110   , 1     , 11    , 101   , 111   ]
        for i=0,n_elements(idc)-1 do begin & k=idc[i] & tmp=xr[k]+yr[k]*10+zr[k]*100
                                           idco=where(corc eq tmp) & if idco[0] ge 0 then corc[idco] =-1 & endfor
        for i=0,7 do begin
          case corc[i] of
               0:   begin & XR=[xr,0] & YR=[yr,0] & ZR=[zr,0]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               1:   begin & XR=[xr,1] & YR=[yr,0] & ZR=[zr,0]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               10:  begin & XR=[xr,0] & YR=[yr,1] & ZR=[zr,0]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               11:  begin & XR=[xr,1] & YR=[yr,1] & ZR=[zr,0]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               100: begin & XR=[xr,0] & YR=[yr,0] & ZR=[zr,1]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               101: begin & XR=[xr,1] & YR=[yr,0] & ZR=[zr,1]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               110: begin & XR=[xr,0] & YR=[yr,1] & ZR=[zr,1]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               111: begin & XR=[xr,1] & YR=[yr,1] & ZR=[zr,1]  & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
               else:
          endcase
        endfor
       endif
       ;check for missing atoms on edges.
       idx=where((yr0+zr0 eq 0) and (xr0 ne 0)) & if (idx[0] ge 0)  then begin & xt=xr[idx] & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*4 then begin ;edge=[[x,0,0],[x,1,0],[x,0,1],[x,1,1]]
        for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     , 10    , 11    ]
         for t=0,n_elements(idx)-1 do if xr[idx[t]] eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          tmp=yr[k]+zr[k]*10  & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
         for i=0,3 do begin
           case edgc[i] of
                0:   begin & XR=[xr,xs[r]] & YR=[yr,0] & ZR=[zr,0] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                1:   begin & XR=[xr,xs[r]] & YR=[yr,1] & ZR=[zr,0] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                10:  begin & XR=[xr,xs[r]] & YR=[yr,0] & ZR=[zr,1] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                11:  begin & XR=[xr,xs[r]] & YR=[yr,1] & ZR=[zr,1] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                else:
           endcase
         endfor
        endfor
       endif
       idx=where((xr0+zr0 eq 0) and (yr0 ne 0)) & if (idx[0] ge 0) then begin & xt=yr[idx] & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*4 then begin ;edge=[[0,y,0],[1,y,0],[0,y,1],[0,y,1]]
        for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     , 10    , 11    ]
         for t=0,n_elements(idx)-1 do if yr[idx[t]] eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          tmp=xr[k]+zr[k]*10  & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
         for i=0,3 do begin
           case edgc[i] of
             0:  begin & XR=[xr,0] & YR=[yr,xs[r]] & ZR=[zr,0] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             1:   begin & XR=[xr,1] & YR=[yr,xs[r]] & ZR=[zr,0] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             10:  begin & XR=[xr,0] & YR=[yr,xs[r]] & ZR=[zr,1] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             11:  begin & XR=[xr,1] & YR=[yr,xs[r]] & ZR=[zr,1] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             else:
           endcase
         endfor
        endfor
       endif
       idx=where((yr0+xr0 eq 0) and (zr0 ne 0)) & if (idx[0] ge 0) then begin & xt=zr[idx] & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*4 then begin ;edge=[[0,0,z],[1,0,z],[0,1,z],[1,1,z]]
        for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     , 10    , 11    ]
         for t=0,n_elements(idx)-1 do if zr[idx[t]] eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          tmp=xr[k]+yr[k]*10  & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
         for i=0,3 do begin
           case edgc[i] of
                0:   begin & XR=[xr,0] & YR=[yr,0] & ZR=[zr,xs[r]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                1:   begin & XR=[xr,1] & YR=[yr,0] & ZR=[zr,xs[r]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                10:  begin & XR=[xr,0] & YR=[yr,1] & ZR=[zr,xs[r]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                11:  begin & XR=[xr,1] & YR=[yr,1] & ZR=[zr,xs[r]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
                else:
           endcase
         endfor
        endfor
       endif
      ;check for missing atoms on faces.
       idx=where((xr0 eq 0) and (yr0 ne 0) and (zr0 ne 0)) & if (idx[0] ge 0)  then begin & xt=yr[idx]+zr[idx]*10 & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*2 then begin ;face=[[0,y,z],[1,y,z]]
         for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     ]
          for t=0,n_elements(idx)-1 do if yr[idx[t]]+zr[idx[t]]*10 eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          coo=[yr[k],zr[k]] & tmp=xr[k] & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
          for i=0,1 do begin
            case edgc[i] of
             0:   begin & XR=[xr,0] & YR=[yr,coo[0]] & ZR=[zr,coo[1]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             1:   begin & XR=[xr,1] & YR=[yr,coo[0]] & ZR=[zr,coo[1]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
             else:
            endcase
          endfor
         endfor
       endif
       idx=where((yr0 eq 0) and (xr0 ne 0) and (zr0 ne 0)) & if (idx[0] ge 0)  then begin & xt=xr[idx]+zr[idx]*10 & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*2 then begin ;face=[[x,0,z],[x,1,z]]
         for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     ]
          for t=0,n_elements(idx)-1 do if xr[idx[t]]+zr[idx[t]]*10 eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          coo=[xr[k],zr[k]] & tmp=yr[k] & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
          for i=0,1 do begin
            case edgc[i] of
              0:   begin & XR=[xr,coo[0]] & YR=[yr,0] & ZR=[zr,coo[1]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
              1:   begin & XR=[xr,coo[0]] & YR=[yr,1] & ZR=[zr,coo[1]] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
              else:
              endcase
          endfor
         endfor
       endif
       idx=where((zr0 eq 0) and (xr0 ne 0) and (yr0 ne 0)) & if (idx[0] ge 0)  then begin & xt=xr[idx]+yr[idx]*10 & xt=xt[sort(xt)] & xs=xt[uniq(xt)] & endif
       if (idx[0] ge 0) then if n_elements(idx) lt n_elements(xs)*2 then begin ;face=[[x,y,0],[x,y,1]]
         for r=0,n_elements(xs)-1 do begin &                                      edgc=[ 0     , 1     ]
          for t=0,n_elements(idx)-1 do if xr[idx[t]]+yr[idx[t]]*10 eq xs[r] then begin & k=idx[t] & wtmp=w[k] & pvtmp=mesh[*,k,*] & pvtmp[idm:idm+11,*,*]=0.
                                          coo=[xr[k],yr[k]] & tmp=zr[k] & ided=where(edgc eq tmp) & if ided[0] ge 0 then edgc[ided] =-1 & endif
          for i=0,1 do begin
            case edgc[i] of
              0:   begin & XR=[xr,coo[0]] & YR=[yr,coo[1]] & ZR=[zr,0] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
              1:   begin & XR=[xr,coo[0]] & YR=[yr,coo[1]] & ZR=[zr,1] & W=[W,wtmp] & mesh=[[mesh],[pvtmp]] & end
              else:
              endcase
          endfor
         endfor
       endif
      
   ;check for other bonds around uniq cell (see all molecule).
   if (total(dup) eq 3) and (p20 eq 1) then begin & nat =n_elements(w) & mosh=mesh[idm:idm+11,*,0] & mesh[idm:idm+11,*,0]=0.
         idxu=where((xr lt .5) and (xr gt 0.)) & idxd=where((xr gt .5) and (xr lt 1.))
         idyu=where((yr lt .5) and (yr gt 0.)) & idyd=where((yr gt .5) and (yr lt 1.))
         idzu=where((zr lt .5) and (zr gt 0.)) & idzd=where((zr gt .5) and (zr lt 1.))
   xrr=Xr & yrr=Yr & zrr=ZR & Wr=W & meshr=mesh
   if idxu[0] ge 0 then begin & XR=[xr,xrr[idxu]+1.] & YR=[yr,yrr[idxu]]    & ZR=[zr,zrr[idxu]]    & W=[w,wr[idxu]] & mesh=[[mesh],[meshr[*,idxu,*]]] & endif
   if idyu[0] ge 0 then begin & XR=[xr,xrr[idyu]]    & YR=[yr,yrr[idyu]+1.] & ZR=[zr,zrr[idyu]]    & W=[w,wr[idyu]] & mesh=[[mesh],[meshr[*,idyu,*]]] & endif
   if idzu[0] ge 0 then begin & XR=[xr,xrr[idzu]]    & YR=[yr,yrr[idzu]]    & ZR=[zr,zrr[idzu]+1.] & W=[w,wr[idzu]] & mesh=[[mesh],[meshr[*,idzu,*]]] & endif
   if idxd[0] ge 0 then begin & XR=[xr,xrr[idxd]-1.] & YR=[yr,yrr[idxd]]    & ZR=[zr,zrr[idxd]]    & W=[w,wr[idxd]] & mesh=[[mesh],[meshr[*,idxd,*]]] & endif
   if idyd[0] ge 0 then begin & XR=[xr,xrr[idyd]]    & YR=[yr,yrr[idyd]-1.] & ZR=[zr,zrr[idyd]]    & W=[w,wr[idyd]] & mesh=[[mesh],[meshr[*,idyd,*]]] & endif
   if idzd[0] ge 0 then begin & XR=[xr,xrr[idzd]]    & YR=[yr,yrr[idzd]]    & ZR=[zr,zrr[idzd]-1.] & W=[w,wr[idzd]] & mesh=[[mesh],[meshr[*,idzd,*]]] & endif
   nut =n_elements(w) & mush=lonarr(12,nut)
   d_livConnex, W, transpose([[xr],[yr],[zr]]), mush, 0, ANGLES=p[7:9], ABC=[Np[1],Np[3],Np[5]]

   idxf=lindgen(nat) & idxt=[-1] ;Starting index for the loop
   idx =where((total(mosh,1) ne total(mush[*,0:nat-1],1)) and (mesh[0,*,0] lt 10)) ;idx is index of Atoms which bound outside the cell.
   while idx[0] ge 0 do begin
      for i=0,n_elements(idx)-1 do begin  ; Atoms bond by idx -> idxt
          id=where(mush[*,idx[i]] gt nat) & if id[0] ge 0 then begin & idxt=[idxt,mush[id,idx[i]]-1] & mush[id,idx[i]]=0 & endif
      endfor
      if n_elements(idxt) gt 1 then begin & idxt=idxt[1:*] & ndxt=n_elements(idxt)
         for j=0,ndxt-1 do begin          ;+Atoms which bound to idxt
           for i=nat,nut-1 do begin
             id=where(mush[*,i] eq idxt[j]+1) & if id[0] ge 0 then begin & idxt=[idxt,i] & mush[id,i]=0 & endif
           endfor
         endfor
         idxt=idxt[uniq(idxt,sort(idxt))] & idxf=[idxf,idxt]
      endif
      idx =idxt & idxt=[-1]
   endwhile
   idxf=idxf[uniq(idxf,sort(idxf))]
   XR=XR[idxf] & YR=YR[idxf] & ZR=ZR[idxf] & W=W[idxf] & mesh=mesh[*,idxf,*]
  
   endif ;(see all molecule)
  
   pvsz=size(mesh)
   if m3d then mesh=reform(mesh,pvsz[1],(size(mesh))[2] * (size(mesh))[3])
  
   endif ;Cell

      atom_xyz=transpose([[xr],[yr],[zr]])

      if sz[0] ge 2 then begin ;We have also projections to show
        if cell     then Np =Np else $
        if pe lt 11 then Np =[min(atom_xyz[0,*],max=xma),xma,min(atom_xyz[1,*],max=yma),$
                              yma,min(atom_xyz[2,*],max=zma),zma] $
        else begin
         x1=min( xr,max=x2) & spx=abs((p[2]-p[1])/(sz[1]-1))
                              sx1=abs(fix( (x1-p[1])/spx)) & sx2=abs(fix((p[2]-x2)/spx))
         y1=min( yr,max=y2) & spy=abs((p[4]-p[3])/(sz[2]-1))
                              sy1=abs(fix( (y1-p[3])/spy)) & sy2=abs(fix((p[4]-y2)/spy))
         z1=min( zr,max=z2) & spz=abs((p[6]-p[5])/(sz[3]-1))
                              sz1=abs(fix( (z1-p[5])/spz)) & sz2=abs(fix((p[6]-z2)/spz))
         if sz[0] eq 3 then begin
            N=N[sx1:sz[1]-1-sx2,sy1:sz[2]-1-sy2,sz1:sz[3]-1-sz2]
         endif else $
            N=N[sx1:sz[1]-1-sx2,sy1:sz[2]-1-sy2]

         Np =[p[1]+sx1*spx, p[2]-sx2*spx, p[3]+sy1*spy, p[4]-sy2*spy, p[5]+sz1*spz, p[6]-sz2*spz]
        endelse
       ;if sz(0) eq 3 then N=TOTAL(N,3)
      endif ;projection

      prop=0
      prov=0
      nat =n_elements(w)
      sm  =size(mesh)
      if (sm[1] gt idm) and (((sm[2] mod nat) eq 0) or (m3d) or (nat eq 1)) then begin
        prop=mesh
        if total(abs(mesh[idm,*])) gt 0 then begin    ;Connections there or to be build
           p18=2
           if (p16 gt 1) or (total(dup) gt 3) then p18=1
        endif
        mesh=0

        if p14 lt 3 then $
        for i=0,nat-1 do $
        if (W[i] ne a_ElpStruct.elp) then $       ; If not an ellipsoide but moments
        if (total(abs(prop[4:4+p25*3*2-1,i]))) ne 0 then W[i]=-W[i] ;  then an Irrow

        if p25 ge 1 then begin & prov=P[26:26+p25*3-1]    ;Propagation vectors
          if p14 eq 2 then begin & p0=4+indgen(p25*2)*3   ;transform Cspheric to Ccartesien
           modu=prop[p0,*] & phi=prop[p0+1,*] & theta=prop[p0+2,*]
          ;Orthonorme->
           Mx=modu*cos(phi*!pi/180.)*cos(theta*!pi/180.)
           My=modu*cos(phi*!pi/180.)*sin(theta*!pi/180.)
           Mz=modu*sin(phi*!pi/180.)

           ctheta=(p[7]-theta+(p[8]-p[7])*(p[9]-phi)/p[9])
           print,[ctheta,theta,p[9]]
           ;FALSE!!!
           Mab=cos( ctheta    *90./(ctheta+theta) *!pi/180.)*modu
           Mx =cos( phi       *90./ p[9]          *!pi/180.)*Mab  & prop[p0+0,*]=Mx
           My =cos((p[9]-phi) *90./ p[9]          *!pi/180.)*Mab  & prop[p0+1,*]=My
           Mz =sin( theta     *90./(ctheta+theta) *!pi/180.)*modu & prop[p0+2,*]=Mz
           print,[mab,mx,my,mz,sqrt(mx^2+my^2+mz^2)]
          endif
        endif
       
      endif else mesh=0

     ;atom_xyz=coordinates
     ;W       =atom types [0-n] negative if magnetic
     ;mesh    =connections if size>1
     ;N       =ab plane projection if bi-dim
     ;Np      =[minx,maxx,miny,maxy,minz,maxz] dim abc for cell , else !!!
     ;dup     =abc duplicates
     ;cell    =0 or 1
     ;p(7:9)  =angles
     ;p(10,18)=Initial number of atoms , Bonds to build
     ;prov    =propagation vectors if cell
     ;prop    =PV
     ;p17     =coef Fourier (magnetic cell) (=3 dynamic =4 HKL)
     ;tit     =title

      o_ret = d_livMakeMolecule('', state ,atom_xyz, W, mesh, N, Np, dup, cell, [p[7:9],p[10],p18,p19], prov, prop, p17, tit)
      WIDGET_CONTROL, state.wDraw , GET_UVALUE=basets & baset=basets.baset
      WIDGET_CONTROL, baset       , SET_UVALUE=state

      if n_elements(pv_plan) gt 1 then d_livDrawPrint, state.wDraw , 'SET_PLANES', snap, PCENT=pv_plan

     ;Enable plane panel , magDraw editor , magDraw plot+animation panel
      for i=0,n_elements(state.did_planeBut)-1 do if state.did_planeBut[i] gt 0 then widget_control,state.did_planeBut[i] ,bad_id=ii,sensitive=1
      return, o_ret
      end
 2:   begin ;Shape
      return, d_livMakeShape(   '', state ,xr,yr,zr,mesh)
      end
 else:
endcase

return,OBJ_NEW()
end


;**
;** ************************
pro d_livSetObjectAttributes, state
;+
;** ************************
;** Set object's attrib to current settings
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

if state.oModelEdges ne obj_new() then $
   d_livSetProp, state.oModelEdges, HIDE=1-state.edging, EXCLUD=state.oSticks

if state.vert_coloring then vc = vcc else vc = 0
if state.bottomColor then bot = 0 else bot = [64, 192, 128]

d_livSetProp, state.oSurface, STYLE=([0,1,2,0,1])[state.style], $
  HIDDEN_LINE=([0,0,0,1,1])[state.style], BOTTOM=bot, REJECT = state.reject, $
  THICK=state.thick, VERT_COLORS=vc, SHADING=state.shading, EXCLUD=state.oSticks
end


;**
;** *************
pro d_livLoadItem, index, State
;+
;** *************
;** Load the new item...
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow

filename = Items[2,index]

state.oModelTop = call_function(Items[1,index], filename, state) ;Load it
tmg0=dblarr(4,4) ; !!! added instruction for IDL8.0
state.oModelSurface->GetProperty, TRANSFORM=tmg0
state.oView->add, state.oModelTop

state.tmg0 = tmg0
state.cur_sel = obj_new()          ; Nothing's selected
d_livSetObjectAttributes, state    ; Now set the attributes to current values

if filename eq 'caffeine.mol' then begin
  s=2.14
  state.scaling=s
  t=[[0.0693024,   -0.0396087,   0.00211983,     0.000000],$
    [ 0.0164601,    0.0326003,    0.0710102,     0.000000],$
    [-0.0360888,   -0.0611926,    0.0364585,     0.000000],$
    [ 0.000000,     0.000000,     0.000000,      1.00000]]
  state.oModelSurface->SetProperty, TRANSFORM = t
  state.oModelEdges  ->SetProperty, TRANSFORM = t
 ;state.oModelPivot  ->SetProperty, TRANSFORM = t ;???
  state.oModelTop->SetProperty, TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]]
  state.oModelTop->GetProperty, TRANSFORM = t
  state.tmg0 = t
endif
end


;**
;** ************
PRO d_livSetProp, o, EXCLUD=Sticks, _extra = e
;+
;** ************
;** Set a property on either a container that
;** contains a number of objects, or a single object.
;-

if obj_class(o) eq 'IDL_CONTAINER' then arr = o->get(/all) $
else arr = o
for i=0, n_elements(arr)-1 do if arr[i] ne Sticks then arr[i]->setproperty, _EXTRA=e
end


;**
pro d_livNewSelected, state, target
;+
;** ****************
;** identify the selected object (mouse click right)
;-

WIDGET_CONTROL, state.wDraw , GET_UVALUE=basets & baset=basets.baset
                targets=basets.targets & styles=basets.styles

if (N_ELEMENTS(target) EQ 1) then begin
    target->GetProperty, HIDE =hid
    ostyle = ([0,1,2,0,1])[state.style]   ;Original style
    if hid then hstyle=-(ostyle+1) else hstyle = ostyle ;Hid when Irrow is in

    idx=n_elements(targets)
    if idx gt 0 then idx=where(targets eq target) $
    else begin &  targets=[target] & styles=[hstyle] & endelse
    if  idx[0]  eq -1 then begin &   idx=n_elements(targets)
                targets=[targets,target] & styles=[styles,hstyle] & endif
    idx=idx[0]
    if styles[idx] ge 0 then nstyle=styles[idx] else nstyle= abs(styles[idx]+1)
   ;if nstyle eq 2 then nstyle = 1 else if nstyle eq 1 then nstyle = 0 else nstyle = 2
    if nstyle eq 2 then nstyle = 1 else nstyle = 2
    if styles[idx] ge 0 then styles[idx]=nstyle $
    else begin & if hid then hid=0 else if nstyle eq 1 then begin & nstyle=2 & hid=1 & endif
               styles[idx]=-(nstyle+1) & endelse
    target->SetProperty, STYLE=nstyle, HIDE=hid
    WIDGET_CONTROL, state.wDraw , SET_UVALUE={baset:baset,targets:targets,styles:styles}
endif
end

;**
;** **************
pro d_livDrawPrint, wDraw , flog, snap, PCENT=pcent
;+
;** **************
;** Handle print event , animations , adding objects
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common c_liv_obj2,a_coval , a_rvdw , a_mass
common d_liv_keepath, kpath
common kp_for_anim, maxp,fa,fb,fc,ffb,coef,prov,p25,propag,propagi,atom_type,atom_coo,atom_xyz,N_dens,N_lims,idR,idI,IdP,otit,ppaobj,ltit,reste

forward_function identity
;otit     =objarr(6,50)     ;To keep current objects for future animations (planes)
                            ;   otit(0,*)=title,symbol,clouds,Bar,Palette,vol_dens 5,6 10,11
                            ;   otit(1:3,*)=planes  otit(4,*)=poly
                            ;   otit(5,0)=boxes  otit(5,2)=axes  otit(5,3)=pivot axes in surface
                            ;   otit(5,4)=atom symbol for pivot axes in surface
                            ;   otit(5,5)=pivot axes in top model (not used -> oModelPivot)
;ltit     =fltarr(5,50)     ;To keep current properties for animations,planes,polygons
                            ;   tit(0,0:13)=miscel  ltit(1:3,*)=planes_attr ltit(4,*)=poly_attr

CATCH,stat & if stat ne 0 then return

if n_elements(pcent) eq 0 then pcent=100.
WIDGET_CONTROL, wDraw , GET_UVALUE=basets , bad_id=ii
if ii ne 0 then return else baset=basets.baset
WIDGET_CONTROL, baset , GET_UVALUE=state
kp_except=!except & !except=0

;*****************************************
;DIRECTION VIEW entered in animation panel (static)
;*****************************************
if (flog eq 'DIRECTION') then begin & ok=0
        t=state.tmg0
        state.oModelSurface->SetProperty,TRANSFORM= t
        state.oModelEdges->SetProperty,  TRANSFORM= t
        state.oModelPivot->SetProperty,  TRANSFORM= identity(4)
        if ltit[0,13] then ff=[fa,fb,fc] else ff=ltit[0,3:5] ;Cell or not!
        tmx=pcent[0]*ff[0] & tmy=pcent[1]*ff[1] & tmz=pcent[2]*ff[2] & d_livChangAxis, tmx,tmy,tmz,/hkl
        newhig=-90.+atan(tmz,sqrt(tmx^2+tmy^2))*180./!pi & newrot=atan(tmy,tmx)*180./!pi

  if (pcent[0] ne 0) or (pcent[1] ne 0) then begin
     if newrot ne 0 then state.oModelSurface->Rotate,[0,0,1],-newrot
     if newrot ne 0 then state.oModelEdges->Rotate  ,[0,0,1],-newrot
     if newrot ne 0 then state.oModelPivot->Rotate  ,[0,0,1],-newrot
     state.oModelSurface->Rotate,[0,1,0],newhig
     state.oModelEdges->Rotate  ,[0,1,0],newhig
     state.oModelPivot->Rotate  ,[0,1,0],newhig

     state.oModelSurface->Rotate,[0,0,1],-90.
     state.oModelEdges->Rotate  ,[0,0,1],-90.
     state.oModelPivot->Rotate  ,[0,0,1],-90.
  endif
  state.oWindow->Draw, state.oView
  did_mpeg,'put'

;******************
;ROTATION INCREMENT (dynamic)
;******************
endif else if (flog eq 'INCREMENT') then begin & ok=0
        if pcent[0] ne 0 then state.oModelSurface->Rotate,[1,0,0],360.*pcent[0]
        if pcent[1] ne 0 then state.oModelSurface->Rotate,[0,1,0],360.*pcent[1]
        if pcent[2] ne 0 then state.oModelSurface->Rotate,[0,0,1],360.*pcent[2]
        if pcent[0] ne 0 then state.oModelEdges->Rotate  ,[1,0,0],360.*pcent[0]
        if pcent[1] ne 0 then state.oModelEdges->Rotate  ,[0,1,0],360.*pcent[1]
        if pcent[2] ne 0 then state.oModelEdges->Rotate  ,[0,0,1],360.*pcent[2]
        if pcent[0] ne 0 then state.oModelPivot->Rotate  ,[1,0,0],360.*pcent[0]
        if pcent[1] ne 0 then state.oModelPivot->Rotate  ,[0,1,0],360.*pcent[1]
        if pcent[2] ne 0 then state.oModelPivot->Rotate  ,[0,0,1],360.*pcent[2]
        state.oWindow->Draw, state.oView
        did_mpeg,'put'

;**********************************
;PERSPECTIVE orthogonal or cavalier (static)
;**********************************
endif else if (flog eq 'ORTHOGONAL') then begin & ok=0
  state.oView  ->IDLgrView::getproperty,PROJECTION=Ptmp & if Ptmp eq 2 then Ptmp=1 else Ptmp=2
  state.oView  ->IDLgrView::setproperty,PROJECTION=Ptmp ;1=Orthogonal 2=Perspective
  state.oWindow->Draw, state.oView

;***************
;BAKGROUND COLOR (dynamic)
;***************
endif else if (flog eq 'BACKGROUND') then begin & ok=0
  kpath.kpcol=(pcent[1:3]-120+pcent[0])>0<255
  state.oView  ->IDLgrView::setproperty,color=kpath.kpcol
  state.oWindow->Draw, state.oView
       
endif else if (flog eq 'CUT') then begin & ok=0
  if pcent[0] eq 100 then clip=[1.5,-1.5] $
  else clip=[((pcent[0]-.5)*3.+.1)>(-1.5)<(1.5) , ((pcent[0]-.5)*3.-.1)>(-1.5)<(1.5)]
  state.oView  ->IDLgrView::setproperty,ZCLIP=clip
  state.oWindow->Draw, state.oView

;********************
;RESIZING draw window (static)
;********************      
endif else if (flog eq 'RESIZE') then begin & ok=0
  dim=[pcent[0],pcent[1]]
  aspect = float(pcent[0])/float(pcent[1])
  myview = [-0.5,-0.5,1,1]
  if (aspect > 1) then begin
      myview[0] = myview[0] - ((aspect-1.0)*myview[2])/2.0
      myview[2] = myview[2] * aspect
  endif else begin
      myview[1] = myview[1] - (((1.0/aspect)-1.0)*myview[3])/2.0
      myview[3] = myview[3] * aspect
  endelse
  state.oView    ->IDLgrView::getproperty,DIM=dimo
  state.oView    ->IDLgrView::setproperty,DIM=dim,VIEWPLANE_RECT=myview
  s=state.did_scalval/100.  * total(dim/dimo)/2.
  state.did_scalval=s*100.
  state.scaling    =s
  state.oModelTop->SetProperty,TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]]
  state.oTrack   ->Trackball::Reset, dim/2.,  dim[0]/2.
  state.oWindow  ->Draw, state.oView

;*******
;ZOOMING (dynamic)
;*******   
endif else if (flog eq 'ZOOM') then begin & ok=0
  kpscl= state.did_scalval
  state.did_scalval=pcent[0]<300>40
  if state.did_scalval ne kpscl then begin
         s=state.did_scalval/100.
         state.scaling =s
         state.oModelTop->SetProperty,TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]]
         state.oWindow->Draw, state.oView
         did_mpeg,'put'
         WIDGET_CONTROL, baset , SET_UVALUE=state
  endif

;*********************************
;CLOUDS , BEANS from displacements (static)
;*********************************
endif else if (flog eq 'CLOUDS') then begin & ok=0
  if coef eq 3 then begin ;3D density
   WIDGET_CONTROL,/hourglass
   nat  =n_elements(atom_type)
  ;nbfq =((size(propag))[2])
   nbfq =n_elements(propagi)
   rbld =0 & hid=0
   if  pcent[4] ge 2 then rbld=pcent[4]-1 else hid=1-pcent[4]

   if (not hid) and (nbfq gt nat) then begin
   if (otit[0,2] eq OBJ_NEW()) or (rbld gt 0) then begin
    if ltit[0,13] then ff=[fa,fb,fc] else ff=ltit[0,3:5] ;Cell or not!
    dup=reform(ltit[0,6:8]) & q=reform(ltit[0,3:5])*dup/2. +ffb
    famp =pcent[5]/10.; Amplitude factor
    mode =pcent[6]    ; Mode 1 2 3
    mass =1.
    bin  =7. & qq=round(ff*bin)*dup
    cloud=fltarr(qq[0]+1,qq[1]+1,qq[2]+1)
    propaR= propag[idR[0]+mode*3:idR[0]+mode*3+2,propagi] * famp
    frame = propag[idP[0]+mode,0]+total(propaR[*,0])      ; frame=0 -> frame mode
   ;Add propagations
    for i =0L,nbfq-1 do begin & j=i mod nat
                        if reste[13,j] ge 0 then begin    ; created and visible atoms
                          if frame ne 0. then mass=a_mass[atom_type[j]]
                          c=round((propaR[*,i]/mass+atom_coo[*,j])*ff*bin)>0.<qq
                          cloud[c[0],c[1],c[2]]=cloud[c[0],c[1],c[2]]+1.
                         endif
    endfor
   ;smooth clouds (beans) to have a clear image
    clud=smooth((cloud*6.)<6.,3)
    shade_volume,clud,.5,v,p
   
    if n_elements(p) ge 3 then begin ;we have polygons
     v=reform(v,3,n_elements(v)/3)
     for i=0,2 do v[i,*]=v[i,*]/(qq[i]/dup[i]) * ff[i] - q[i]
     d_livChangAxis,v
     if otit[0,2] eq OBJ_NEW() then begin
        bIm = bytarr(4,2,2) & bIm[0:2,*,*]=200 & bIm[3,*,*]=220
        oIm = OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
        otit[0,2]=OBJ_NEW("IDLgrPolygon",v,POLY=p,SHADING=1,style=2,COLOR=[250,250,250],UVALUE=[pcent[5:6]],texture_map=oIm)
        state.oModelSurface->Add,otit[0,2]
     endif else otit[0,2]->SetProperty, DATA=v, HIDE=0, POLY=p ;,shading=1,texture_map=OBJ_NEW(),style=2,linestyle=1,thick=2

    endif else  otit[0,2]->SetProperty, HIDE=1 ;no polygons
   
   endif else if otit[0,2] ne OBJ_NEW() then otit[0,2]->SetProperty,HIDE=0
   endif else  if otit[0,2] ne OBJ_NEW() then otit[0,2]->SetProperty,HIDE=1

   state.oWindow->Draw, state.oView
   did_mpeg,'put'

  endif ;coef eq 3

;**************
;VOLUME DENSITY (dynamic with the plane threshold slider)
;**************
endif else if (flog eq 'VOL_DENS') then begin & ok=0
        szn=size(N_dens)
        if szn[0] ge 3 then if (Pcent[0] le 0) or (szn[0] gt 3) then begin
         vn  =Pcent[0] ;volume #
         thrh=Pcent[1] ;threshold %
         shid=Pcent[2] ;toggle show/hide if =1
         PorS=0        ;point/wire/solid
         if vn lt 0 then vn=N_lims[0,0]
         if shid eq 2 then begin & shid=0 & PorS=-1 & endif   ;toggle point/wire/solid if 2
         if n_elements(Pcent) gt 3 then   PorS=Pcent[3]       ;point/wire/solid (0 1 2)+1

         if thrh gt 0 then begin
            if ltit[0,13] then ff=[fa,fb,fc] else ff=ltit[0,3:5] ;Cell or not!
            dup=reform(ltit[0,6:8]) & q=reform(ltit[0,3:5])*dup/2. +ffb
            tdup=dup[0]*dup[1]*dup[2]

            if tdup gt 1 then begin
               clud=bytarr(szn[1]*dup[0],szn[2]*dup[1],szn[3]*dup[2])
               for z=0,dup[2]-1 do for y=0,dup[1]-1 do for x=0,dup[0]-1 do $
                   clud[szn[1]*x,szn[2]*y,szn[3]*z] = N_dens[*,*,*,vn]
            endif else clud=N_dens[*,*,*,vn]

            minv=min(N_dens[*,*,*,vn],max=maxv)
            cut=float(maxv-minv)*thrh/100.+minv
            if minv ge 0. then begin & clud=sqrt(clud) & cut=float(sqrt(maxv)-sqrt(minv))*thrh/100.+sqrt(minv) & endif
            shade_volume,clud,cut,v,p
            clud = 0
            if n_elements(p) ge 3 then begin
              v=reform(v,3,n_elements(v)/3)
              for i=0,2 do v[i,*]=v[i,*]/(szn[i+1]-1) * ff[i] - q[i]
              d_livChangAxis,v
            endif
         endif
         if n_elements(p) ge 3 then begin ;we have polygons
          if otit[0,5+vn] eq OBJ_NEW() then begin
           bIm = bytarr(4,2,2) & bIm[0:2,*,*]=250 & bIm[3,*,*]=100
           oIm = OBJ_NEW('IDLgrImage',bIm,blend_function=[3,4])
           otit[0,5+vn]=OBJ_NEW("IDLgrPolygon",v,POLY=p,SHADING=0,style=0,thick=1,COLOR=[250,250,250]/(vn+1),UVALUE=[vn,pcent[1:*]]);,texture_map=oIm)
           state.oModelSurface->Add,otit[0,5+vn]
           state.oSurface->Add,otit[0,5+vn]
           oModelVolEdge = OBJ_NEW('IDLgrModel')
           otit[0,10+vn]=OBJ_NEW('IDLgrPolyline', SHARE_DATA=otit[0,5+vn], POLY=p,LINESTYLE=1, COLOR=[100,100,100])
           state.oModelEdges->add, oModelVolEdge
           oModelVolEdge->add, otit[0,10+vn]
          endif else begin  &  otit[0,5 +vn]->SetProperty, DATA=v, POLY=p ,HIDE=0 ,UVALUE=[vn,pcent[1:*]]
                               otit[0,10+vn]->SetProperty,         POLY=p ,HIDE=0 & endelse
         endif
         if  otit[0,5+vn] ne OBJ_NEW() then begin
          if shid then begin & otit[0,5 +vn]->GetProperty, HIDE=hid
                               otit[0,5 +vn]->SetProperty, HIDE=abs(1-hid)
                               otit[0,10+vn]->SetProperty, HIDE=abs(1-hid) & endif
          if PorS eq -1 then begin & otit[0,5+vn]->GetProperty, STYLE=sty
                               otit[0,5+vn]->SetProperty, STYLE=(sty+1) mod 3 & endif
          if PorS gt  0 then   otit[0,5+vn]->SetProperty, STYLE=PorS-1
         endif
        state.oWindow->Draw, state.oView
        did_mpeg,'put'
        endif

;*****************************************
;DISPLACEMENTS, VIBRATIONS or PROPAGATIONS(dynamic magnetic)
;*****************************************
endif else if (flog eq 'CELL_STEP') then begin & ok=0
  bid =OBJ_NEW()
  step=pcent[3]>1
 
  ;DISPLACEMENTS or VIBRATIONS
  ;***************************
  if coef eq 3 then begin & ns=pcent[4]
   sig=[1.,1.,1.]
   famp=pcent[5]/10. ; Amplitude factor
   mode=pcent[6]     ; Mode 1 2 3 (0 1 2)
   B_S =pcent[7]     ; 0=Ball&Stick   1=Ball   2=Stick
   vect=pcent[8]     ; 0=drawn        1=hidden 2=Hide/Show lines 3=red 4=Yellow 5=White 6=Black 7=Grey 8=Atom
   modn=pcent[9]     ; Displacement #
   rbld=pcent[10]    ; calculate bonds
   fctB=pcent[11]/10.; rvdW factor for calculated bonds
   nat =n_elements(atom_type)
   dup =total(ltit[0,6:8])
  ;nbfq=(size(propag))[2]
   nbfq=n_elements(propagi)
   ff  =[fa,fb,fc]
   if (B_S eq 1) then nostick=1 else nostick=0
   if (not nostick) and (not rbld) and (ltit[0,11]) then nostick=1
   if (B_S eq 2) and (state.oSticks ne bid) then noball=1 else noball=0
   if  otit[0,1] eq bid then otit[0,1]=OBJ_NEW('IDLgrSymbol',3,thick=2,size=.1,color=[255,255,255])
   if (not nostick) then mesh_xyz=atom_xyz
   if (not nostick) then mesh_coo=atom_coo

   pivot=[0.,0.,0.] & moss  =1.  ; frame=0 -> frame mode
   frame =propag[idP[0]+mode,0]+total(abs(propag[idR[0]:idR[0]+5,0]))
   
   adx   =where(reste[13,*] ge 0, complement=adxc) ;created and visible atoms
  
  ;Displacement
   if frame eq 0. then begin & mass=1. & f=1. & modn=((modn+(step-1)*ns) mod (nbfq/nat+1))>1
                             Dn  =((modn-1)*nat)<(nbfq-nat)>0 & modu=0
           if state.did_transval ge 0 then  $ ;We have a selected atom?
           pivot=propag[idR[0]+mode*3:idR[0]+mode*3+2,propagi[state.did_transval+Dn]]

  ;Vibration mode
   endif else begin
           mass=0                           & stup=(step-1) mod ns
           tsin=sin(findgen(ns+1)/ns*2*!pi) & f   =tsin[stup]
           Dn  =((modn-1)*nat)<(nbfq-nat)>0
           
           ;Be sure Irrows of unvisible atoms are hidden.
           if adxc[0] ge 0 then for inn = 0L,n_elements(adxc)-1 do begin & i = adxc[inn]
                                     if ppaobj[3,i] ne bid then begin
                                        ppaobj[3,i]->SetProperty, HIDE=1
                                        ppaobj[4,i]->SetProperty, HIDE=1 & endif
                                endfor
   endelse

   changed=0
   if vect le 1 then ltit[0,1]=vect
   ppr1 =idR[0]+mode*3 & ppr2 =ppr1+2
   ppi1 =idI[0]+mode*3 & ppi2 =ppi1+2
   rads =.06 ;Irrow width
   nselt=[0,-1,-1,-1]   ;prepare variable for distance or angle

  ;for i=0L,nat-1 do if ppaobj(0,i) ne bid then begin ;Loop for atoms
   for inn=0L,n_elements(adx)-1 do begin & i=adx[inn] ;Loop for created and visible atoms
           ;propaR=propag(idR(0)+mode*3:idR(0)+mode*3+2,i+Dn)-pivot
           ;propaI=propag(idI(0)+mode*3:idI(0)+mode*3+2,i+Dn)
            propaR=propag[ppr1:ppr2,propagi[i+Dn]]-pivot
            propaI=propag[ppi1:ppi2,propagi[i+Dn]]
            sigx  =where(propaR+propaI lt 0)  & sign=sig & if sigx[0] ge 0 then sign[sigx]=-1. ;keep the sign
      if mass eq 0 then moss=a_mass[atom_type[i]]
            Mlk   =sqrt( (propaR^2+propaI^2)/moss)  * sign * ff   & Mlj=Mlk*f*famp
            actu  =reste[10:12,i]

     ;Vibration mode
      if frame ne 0. then begin
       d_livChangAxis, XYZ=Mlk*famp, getmodule=modu
       ;Draw the red displacement line if not there
       if (ltit[0,0] ne (mode+1+Dn*3)*famp) and (ltit[0,1] ne 1) then begin ;mode change and not hidden
           changed=1
           if inn eq 0 then widget_control,/hourglass
           dx=Mlk[0]*famp & dy=Mlk[1]*famp & dz=Mlk[2]*famp
           d_livChangAxis, dx,dy,dz
           newhig=90.-atan(dz,sqrt(dx^2+dy^2))*180./!pi & newrot=atan(dy,dx)*180./!pi

           ;Set Irrow properties
           sa= d_livMakeIrrow( atom_type[i])
           sa->SetProperty, LENGTH=modu ,RADIUS=rads
           sa->GetProperty, POBJ=ah
           ah->GetProperty, POLY=pmosh, DATA=data
           ;Atom coord
           ex=atom_xyz[0,i] & ey=atom_xyz[1,i]  & ez=atom_xyz[2,i]
           ;Add Atom radius*dfact
           if modu gt 0. then rx=[dx,dy,dz]/modu*reste[0,i]*ltit[0,12] else rx=[0.,0.,0.]

           line1=[[ex,ey,ez],[ex+dx,ey+dy,ez+dz]]
           line2=[[ex,ey,ez],[ex-dx,ey-dy,ez-dz]]
           if ppaobj[3,i] eq bid then begin & col=[255,255,0]
              ppaobj[3,i]=OBJ_NEW('IDLgrModel')
              ppaobj[7,i]=OBJ_NEW('IDLgrPolygon' , DATA=data,  POLY=pmosh, COLOR=col ,shading=0)
              ppaobj[3,i]->Rotate,[0,1,0],newhig
              ppaobj[3,i]->Rotate,[0,0,1],newrot
              ppaobj[3,i]->Translate,  ex+dx/2.2+rx[0], ey+dy/2.2+rx[1], ez+dz/2.2+rx[2]
              ppaobj[3,i]->Add, ppaobj[7,i]
             ;ppaobj(3,i)=OBJ_NEW('IDLgrPolyline', DATA=line1, thick=3, COLOR=col ,linest=0)
              ppaobj[4,i]=OBJ_NEW('IDLgrPolyline', DATA=line2, thick=1, COLOR=col ,linest=[1,'9999'X], HIDE=1)
              state.oModelSurface->Add, ppaobj[3,i]
              state.oModelSurface->Add, ppaobj[4,i]
           endif else begin ;ppaobj(3,i)->SetProperty, DATA=line1
              ppaobj[3,i]->Reset
              ppaobj[3,i]->Rotate,[0,1,0], newhig
              ppaobj[3,i]->Rotate,[0,0,1], newrot
              ppaobj[3,i]->Translate,  ex+dx/2.2+rx[0], ey+dy/2.2+rx[1], ez+dz/2.2+rx[2]
              ppaobj[7,i]->SetProperty, DATA=data,  POLY=pmosh

              ppaobj[4,i]->SetProperty, DATA=line2
           endelse
       endif
      endif ;Vibrations (frame ne 0)

      if ppaobj[3,i] ne bid then begin
       if (vect le 1) or (abs(modu) lt rads/3.) then begin
           if (vect eq 1) or (abs(modu) lt rads/3.) then hid=1 else hid=0
           if hid eq 1 then ppaobj[4,i]->SetProperty, HIDE=hid
           ppaobj[3,i]->SetProperty, HIDE=hid & endif

       case vect of
       2: begin & ppaobj[4,i]->GetProperty, HIDE=hid & hid=abs(hid-1)
                ppaobj[4,i]->SetProperty, HIDE=hid & end

       3: begin & ppaobj[7,i]->SetProperty, COLOR=[255,0,0]
                ppaobj[4,i]->SetProperty, COLOR=[255,0,0]     & end
       4: begin & ppaobj[7,i]->SetProperty, COLOR=[255,255,0]
                ppaobj[4,i]->SetProperty, COLOR=[255,255,0]   & end
       5: begin & ppaobj[7,i]->SetProperty, COLOR=[255,255,255]
                ppaobj[4,i]->SetProperty, COLOR=[255,255,255] & end
       6: begin & ppaobj[7,i]->SetProperty, COLOR=[0,0,0]
                ppaobj[4,i]->SetProperty, COLOR=[0,0,0]       & end
       7: begin & ppaobj[7,i]->SetProperty, COLOR=[128,128,128]
                ppaobj[4,i]->SetProperty, COLOR=[128,128,128] & end
       8: begin & ppaobj[7,i]->SetProperty, COLOR=a_col[*,atom_type[i]]
                ppaobj[4,i]->SetProperty, COLOR=a_col[*,atom_type[i]] & end

       else:
       endcase
      endif

     ;Have a distance or angle to show?
      ppaobj[5,i]->GetProperty,HIDE=hid
      if not hid then begin & nselt[0]=nselt[0]+1 & if nselt[0] le 3 then nselt[nselt[0]]=i & endif

     ;Cell change ?
      if (frame eq 0) and (nostick or rbld) and (dup eq 3) then begin
         coo=atom_coo[*,i]+Mlk/ff
         idx=where(coo gt 1.) & if idx[0] ge 0 then Mlj[idx]=Mlj[idx]-ff[idx]
         idx=where(coo lt 0.) & if idx[0] ge 0 then Mlj[idx]=Mlj[idx]+ff[idx]
      endif

     ;Move atoms
      if (not noball) and (total(abs(Mlj-actu))) ne 0. then begin
         oModelAtom=ppaobj[0,i]
         dx=(Mlj[0]-actu[0]) & dy=(Mlj[1]-actu[1])
         dz=(Mlj[2]-actu[2]) & d_livChangAxis, dx,dy,dz ;????????????? verify
         oModelAtom->Translate,   dx,  dy,  dz
         reste[10:12,i]=Mlj
      endif
      if ltit[0,2] ne noball then ppaobj[0,i]->SetProperty, HIDE=noball
     ;Move bonds
      if (not nostick) then if total(abs(Mlj)) ne 0 then begin
         dx=Mlj[0] & dy=Mlj[1] &  dz=Mlj[2] & d_livChangAxis, dx,dy,dz ;????????????? verify
         mesh_coo[*,i]=mesh_coo[*,i]+Mlj/ff
         mesh_xyz[0,i]=mesh_xyz[0,i]+dx
         mesh_xyz[1,i]=mesh_xyz[1,i]+dy
         mesh_xyz[2,i]=mesh_xyz[2,i]+dz
      endif

   endfor ;Loop for created and visible atoms
  
   if changed then ltit[0,0]=(mode+1+Dn*3)*famp
   ltit[0,2]= noball
   otit[0,0]->SetProperty, string='Displacement '+strtrim(string(fix(modn)),2)+' , mode '+strtrim(string(fix(mode+1)),2)

   ;Care with the sticks
   if (not nostick) then begin
     if (state.oSticks eq bid) or rbld then begin
       nn=n_elements(atom_type) & pripag=fltarr(12,nn) & idm=4+p25*3*2+p25
       if rbld then d_livConnex, atom_type, mesh_coo, pripag,0 ,abc=ff, factor=fctB ,visidx=reste[13,*] $
               else d_livConnex, atom_type, atom_coo, pripag,0 ,abc=ff, visidx=reste[13,*]
       if rbld then d_livBlinks, nn,pripag,0,amesh $
               else d_livBlinks, nn,pripag,0,amesh
       if (n_elements(amesh) le 1) and (not rbld) then ltit[0,11]=1
     endif
     if state.oSticks eq bid then begin & d_livOlinks, state, amesh, new_xyz=mesh_xyz
                                        WIDGET_CONTROL, baset , SET_UVALUE=state
     endif else $
        if rbld then if n_elements(amesh) le 1 then state.oSticks->SetProperty,HIDE=1 $
                     else state.oSticks->SetProperty, DATA=mesh_xyz, HIDE=nostick, POLYLINE=amesh $
        else              state.oSticks->SetProperty, DATA=mesh_xyz, HIDE=nostick
   endif else $
   if state.oSticks ne bid then state.oSticks->SetProperty, HIDE=nostick

   ;Have a distance or angle to show?
   if nselt[0] gt 1 then $
      if nselt[0] eq 2 then begin & difr=reste[10:12,nselt[1]]/ff/famp+atom_coo[*,nselt[1]]-reste[10:12,nselt[2]]/ff/famp-atom_coo[*,nselt[2]]
                                  d_livChangAxis, XYZ=difr*ff, getmodule=dista & dista=round(dista*1000)/1000.
                                  widget_control,state.l_mess,bad_id=ii,set_value='Distance='+string(dista,format='(f7.3)')+'                      '
      ;angle
      endif else if nselt[0] eq 3 then begin & disp=reste[10:12,nselt[1:3]]/famp & d_livChangAxis,disp
                                             v1=disp[*,0]+atom_xyz[*,nselt[1]]
                                             v2=disp[*,1]+atom_xyz[*,nselt[2]]
                                             v3=disp[*,2]+atom_xyz[*,nselt[3]]
                                  angl3=abs(acos(total((v2-v3)*(v1-v3))/(sqrt(total((v2-v3)^2))*sqrt(total((v1-v3)^2))))*180./!pi)
                                  angl2=abs(acos(total((v1-v2)*(v3-v2))/(sqrt(total((v1-v2)^2))*sqrt(total((v3-v2)^2))))*180./!pi)
                                  angl1=abs(acos(total((v2-v1)*(v3-v1))/(sqrt(total((v2-v1)^2))*sqrt(total((v3-v1)^2))))*180./!pi)
          angl='\'+a_symb[atom_type[nselt[3]]]+'/'+strtrim(string(angl3,format='(f6.2)'),2)+ $
              ' \'+a_symb[atom_type[nselt[2]]]+'/'+strtrim(string(angl2,format='(f6.2)'),2)+ $
              ' \'+a_symb[atom_type[nselt[1]]]+'/'+strtrim(string(angl1,format='(f6.2)'),2)+' degrees'
                                        widget_control,state.l_mess,bad_id=ii,set_value=angl
      endif

  ;************
  ;PROPAGATIONS magnetic
  ;************
  endif else if (coef eq 1) or (coef eq 2) then begin & mp=0
   dup=reform(ltit[0,6:8])
   if total(dup) gt 3 then begin & mp=1 & if pcent[0] lt pcent[1] then mp=2 & if pcent[mp-1] lt pcent[2] then mp=3
                                 if dup[mp-1] gt 1 then begin & dd =[0.,0.,0.] & mpp=dd & mpp[mp-1]=.1 & nn=dup[mp-1]*10+1
                                                              tpp=fltarr(3,nn*3) & tdd=tpp & f=[fa,fb,fc]
                                 endif else mp=0 & endif
   kx=(pcent[0]*(step-1))>0 ;& kx=round(kx)
   ky=(pcent[1]*(step-1))>0 ;& ky=round(ky)
   kz=(pcent[2]*(step-1))>0 ;& kz=round(kz)
   for i=0L,ltit[0,10]-1 do if (atom_type[i] lt 0) and (ppaobj[2,i] ne bid) then begin
      oModelAtom=ppaobj[0,i] & p=ppaobj[1,i]       & pa=ppaobj[2,i]
      curspher  =reste[ 0,i] & curIrrow=reste[1,i] & curlng=reste[2,i] & curhig=reste[3,i]
      currot    =reste[ 4,i] & afact=reste[5,i]    & lfact =reste[9,i]
      kkx=reste[6,i]+kx      & kky=reste[7,i]+ky   & kkz=reste[8,i]+kz
      newradspher=curspher
      rfact=ltit[0,9]

      if mp eq 0 then begin
         MOMENT_CALC, i, kkx,kky,kkz,afact,lfact,rfact, newradspher,newradIrrow,newlng,newhig,newrot, Mlj,modu
         ak=0
         if ((round(curhig*100) ne round(newhig*100))) or ((round(currot*100) ne round(newrot*100))) then begin
           oModelAtom->Translate,  -atom_xyz[0,i], -atom_xyz[1,i], -atom_xyz[2,i]
           oModelAtom->Rotate,[0,0,1],-currot & reste[4,i]=newrot & ak=1
           oModelAtom->Rotate,[0,1,0],-curhig & reste[3,i]=newhig
           oModelAtom->Rotate,[0,1,0], newhig
           oModelAtom->Rotate,[0,0,1], newrot
           oModelAtom->Translate,   atom_xyz[0,i],  atom_xyz[1,i],  atom_xyz[2,i]
         endif
         if round(newradspher*100) ne round(curspher*100) then begin ;Need to rebuild a sphere/Irrow
               s=d_livMakeSphere(abs(atom_type[i]))
               s ->SetProperty, RADIUS=newradspher
               s ->GetProperty, POBJ=sh
               sh->GetProperty, DATA=data
               p ->SetProperty, DATA=data & reste[0,i]=newradspher & ak=1 & endif
         if (round(newradIrrow*100) ne round(curIrrow*100)) or (round(curlng*100) ne round(newlng*100)) then begin
               sa=d_livMakeIrrow(abs(atom_type[i]))
               sa->SetProperty, RADIUS=newradIrrow, LENGTH=newlng
               sa->GetProperty, POBJ=ah
               ah->GetProperty, DATA=data
               pa->SetProperty, DATA=data & reste[1,i]=newradIrrow & reste[2,i]=newlng & ak=1 & endif

         if ak then oModelAtom->SetProperty,UVALUE=string(d_livTypetoSymb(abs(atom_type[i]))+strtrim(string(i+1),2), modu,Mlj[0], Mlj[1],Mlj[2], format='(A, " m=", F6.3," Mc=", 3F6.2)')

      endif else if (reste[6:8,i])[mp-1] eq 0 then begin ;Show magnet bands
       ppaobj[5,i]->GetProperty,HIDE=hid
       if not hid then begin
        dd[*]=0. & xyz=atom_xyz[*,i] & n2=nn*2-1
        for j=0L,nn-1 do begin
         MOMENT_CALC, i, kkx+dd[0],kky+dd[1],kkz+dd[2],afact,lfact,rfact, newradspher,newradIrrow,newlng,newhig,newrot, Mlj,modu ,tmx,tmy,tmz
         j3=j*3
         tpp[*,j3] =[tmx,tmy,tmz]/maxp*lfact/2.2 + xyz
         tpp[*,j3+1] =xyz  & tpp[*,j3+2]=tpp[*,j3]
         tdd[*,j3]=dd*f & tdd[*,j3+1]=dd*f & tdd[*,j3+2]=dd*f
         dd =dd+mpp
        endfor
        d_livChangAxis,tdd
        if ppaobj[6,i] eq bid then begin & pa->GetProperty, COLOR=coco
           ppaobj[6,i]=OBJ_NEW('IDLgrPolyline',DATA=tpp+tdd, shading=0,COLOR=coco, thick=1)
           state.oModelSurface->Add,ppaobj[6,i]
        endif else ppaobj[6,i]->SetProperty,DATA=tpp+tdd, HIDE=0
       endif else if ppaobj[6,i] ne bid then ppaobj[6,i]->SetProperty, HIDE=1
      endif
   endif ;for
  
   if kx+ky+kz eq 0 then ttl='' else ttl='Cell:['+strtrim(string(kx+1),2)+' , '+strtrim(string(ky+1),2)+' , '+strtrim(string(kz+1),2)+']'
   otit[0,0]->SetProperty, string=ttl
  endif
  state.oWindow->Draw, state.oView
  did_mpeg,'put'

;************
;PLANES panel (static and dynamic)
;************
endif else if (flog eq 'PLANE') and (coef gt 0) then begin & ok=0
  bid =OBJ_NEW()
  prop=pcent[0]  & if prop eq -4 then N_lims[0,0]=pcent[17]
  npla=pcent[1]  & hid=pcent[2]  & app1=pcent[3] & app2=pcent[4] & cxyz=pcent[5:13]
  mov =pcent[14] & alo=pcent[15] & fixi=(pcent[16] eq 1)
  case abs(prop) of
  1: begin ;static
           bIm = bytarr(4,2,2)
           case app1 of
           1: begin & bIm[2  ,*,*]=200 & bIm[3,*,*]=200 & end          ;***** OPAQUE BLUE=1
           2: begin & bIm[0:2,*,*]=200 & bIm[3,*,*]=220 & end          ;***** OPAQUE WHITE=2
           3: begin & bIm[0:2,*,*]=200 & bIm[3,*,*]=60  & end          ;***** TRANSP. BLUE=3
           4: begin & bIm[0:2,*,*]=250 & bIm[3,*,*]=100 & end          ;***** TRANSP. WHITE=4
           else:
           endcase
           otit[npla,0]->GetProperty,texture_map=oIm
           oIm         ->SetProperty,data=bIm,blend_function=[3,4]
           otit[npla,0]->SetProperty,texture_map=oIm
           ltit[npla,0]=app1
     end
  2: begin ;static
       case app2 of
       1: begin & for i=0,n_elements(atom_type)-1 do $                  ;***** ATOMS IN CELL =1
                    if ppaobj[0,i] ne bid then ppaobj[0,i]->SetProperty, HIDE=0
                idx=where(reste[13,*] ne -2) & reste[13,idx]=idx

                if state.oSticks ne bid then begin
                   nn=n_elements(atom_type) & pripag=fltarr(12,nn) & idm=4+p25*3*2+p25
                   d_livConnex, atom_type, atom_coo , propag,idm, abc=[fa,fb,fc], visidx=reste[13,*]
                   d_livBlinks, nn,propag,idm, amesh
                   if n_elements(amesh) gt 1 then state.oSticks->SetProperty, DATA=atom_xyz, POLYLINE=amesh & endif
          end
       2: begin & dup=reform(ltit[0,6:8])                               ;***** ATOMS IN PLANE=2
                if ltit[0,13] then f=[fa,fb,fc] else f=ltit[0,3:5] ;Cell or not!
                q=reform(ltit[0,3:5])*dup/2. +ffb
                for i=0,n_elements(atom_type)-1 do if ppaobj[0,i] ne bid then ppaobj[0,i]->SetProperty, HIDE=1
                for npla=1,3 do begin & i=0
                  while otit[npla,i] ne bid do begin ;loop for planes
                    otit[npla,i]-> GetProperty,HIDE=hid,uvalue=uval,color=cocol
                    if (not hid) and (n_elements(uval) gt 9) then begin
                      ;idx=[0,3,6]+alo & cxyz(idx)=cxyz(idx)+mov ;for current
                       cxyz=uval[0:8]
                       m1=cxyz[0:2]/1000.*f-q & m2=cxyz[3:5]/1000.*f-q & m3=cxyz[6:8]/1000.*f-q
                       d_livChangAxis,m1 & d_livChangAxis,m2 & d_livChangAxis,m3
                       v1=[m2[0]-m1[0],m2[1]-m1[1],m2[2]-m1[2]] & v2=[m3[0]-m1[0],m3[1]-m1[1],m3[2]-m1[2]] ;2 directions
                       ap=v1[1]*v2[2]-v2[1]*v1[2] & bp=v1[2]*v2[0]-v2[2]*v1[0] & cp=v1[0]*v2[1]-v2[0]*v1[1]
                       dp=-(ap*m1[0]+bp*m1[1]+cp*m1[2])  ;AP.x + BP.y + CP.z + D = 0 (in cell coord.)
                       deno=sqrt(ap^2+bp^2+cp^2)
                       for j=0,n_elements(atom_type)-1 do if ppaobj[0,j] ne bid then begin
                         d=(ap*atom_xyz[0,j]+ bp*atom_xyz[1,j]+ cp*atom_xyz[2,j] +dp)/deno
                         if round(d*2) eq 0 then ppaobj[0,j]->SetProperty, HIDE=0
                       endif ;for
                    endif
                    i=i+1 & endwhile
                endfor
        end
     3: begin & end                                             ;***** Free

     4: for i=0,n_elements(atom_type)-1 do $                    ;***** ATOMS HIDDEN=4
            if ppaobj[0,i] ne bid then ppaobj[0,i]->SetProperty, HIDE=1

     5: if state.oSticks ne bid then begin                      ;***** ATOMS IN BOND
            nn=n_elements(atom_type) & pripag=fltarr(12,nn) & idm=4+p25*3*2+p25
            d_livConnex, atom_type, atom_coo , pripag,0, abc=[fa,fb,fc], visidx=reste[13,*]
            d_livNlinks, nn,pripag,0,table
            idx=where(table eq 0)
            if idx[0] ge 0 then for i=0,n_elements(idx)-1 do $
            if ppaobj[0,idx[i]] ne bid then begin & ppaobj[0,idx[i]]->SetProperty, HIDE=1
                                                  reste[13,idx[i]]=-1 & endif
        endif
     6: begin & N_lims[0,0]=0 & end                               ;***** Ch.electronic
     7: begin & N_lims[0,0]=1 & end                               ;***** Ch.magnetic
     else:
     endcase
     end
  3: begin & otit[npla,0]->SetProperty,HIDE=hid & end                     ;***** HIDE/SHOW A PLANE

  4: begin & idx=[0,3,6]+alo & kxyz=cxyz & cxyz[idx]=cxyz[idx]+mov        ;DYNAMIC
           m1=cxyz[0:2]/1000. & m2=cxyz[3:5]/1000. & m3=cxyz[6:8]/1000. ;***** BUILD A PLANE, MOVE, FIX-IT
           v1=[m2[0]-m1[0],m2[1]-m1[1],m2[2]-m1[2]] & v2=[m3[0]-m1[0],m3[1]-m1[1],m3[2]-m1[2]] ;2 directions
           ap=v1[1]*v2[2]-v2[1]*v1[2] & bp=v1[2]*v2[0]-v2[2]*v1[0] & cp=v1[0]*v2[1]-v2[0]*v1[1]
           if cp lt 0 then begin & ap=-ap & bp=-bp & cp=-cp & endif ;turned towards the son
           dp=-(ap*m1[0]+bp*m1[1]+cp*m1[2])  ;AP.x + BP.y + CP.z + D = 0 (in ortho-normal coord.)
           popo=fltarr(3,1) & dup=reform(ltit[0,6:8])
          ;Find corners
           if bp ne 0 then for a=0,dup[0],dup[0] do for c=0,dup[2],dup[2] do begin & b=-(ap*a+cp*c+dp)/bp
                                               if (b ge 0) and (b le dup[1]) then popo=[[popo],[a,b,c]] & endfor
           if ap ne 0 then for b=0,dup[1],dup[1] do for c=0,dup[2],dup[2] do begin & a=-(bp*b+cp*c+dp)/ap
                                               if (a ge 0) and (a le dup[0]) then popo=[[popo],[a,b,c]] & endfor
           if cp ne 0 then for a=0,dup[0],dup[0] do for b=0,dup[1],dup[1] do begin & c=-(ap*a+bp*b+dp)/cp
                                               if (c ge 0) and (c le dup[2]) then popo=[[popo],[a,b,c]] & endfor
           if n_elements(popo) ge 12 then begin
            ;Take uniq corners
             popo= popo[*,1:*]
             nc =n_elements(popo)/3 & add=[1.246,100.137,1000.589] & popi=fltarr(nc)  & for i=0,nc-1 do popi[i]=total(popo[*,i]*add)
             idx=sort(popi)    & idu= uniq(popi[idx])  & popo=popo[*,idx] & popo=popo[*,idu]
           endif
          
          ;Draw the plane (we have 3 points)
          ;**************
           if n_elements(popo) ge 9 then begin
            ;Sort corners
             nc  =(size(popo))[2] & popi= popo & costh=fltarr(nc) & costh[0]=1.
             poit=(popo[*,1]+popo[*,0])/2. ;Calculate directions from a point in the plane
             popi[0,*]=popo[0,*]-poit[0] & popi[1,*]=popo[1,*]-poit[1] & popi[2,*]=popo[2,*]-poit[2]
             qpoit=sqrt(total(popi[*,0]^2))
             for i=1,nc-1 do costh[i]=(popi[0,0]*popi[0,i]+popi[1,0]*popi[1,i]+popi[2,0]*popi[2,i])/(qpoit*sqrt(total(popi[*,i]^2)))
             popo=popo[*,sort(costh)]
             idm =sort(sqrt(total(popo^2,1)))-(nc-1)
             if idm[0] ne 0 then  popo=shift(popo,0,-idm[0])

            ;Draw the polygon
             pol =[nc,indgen(nc)] &  q=reform(ltit[0,3:5])*dup/2. +ffb
             t_a =fltarr(2,nc)    &  t_a[*,0]=[1.,0.] & t_a[*,1]=[1.,1.] & t_a[*,2]=[0.,1.]
             popi=popo
             if ltit[0,13] then ff=[fa,fb,fc] else ff=ltit[0,3:5] ;Cell or not!
             popo[0,*]=popo[0,*]*ff[0]-q[0] & popo[1,*]=popo[1,*]*ff[1]-q[1] & popo[2,*]=popo[2,*]*ff[2]-q[2]
             d_livChangAxis,popo
             szn =size(N_dens)
             kd  =0
             dens=0
            ;WE HAVE DENSITIES
            ;*****************
             if szn[0] ge 3 then if (N_lims[0,0] eq 0) or (szn[0] gt 3) then begin
                dens=1 & kd=N_lims[0,0]                                ;We have densities to project
                normal=-1 & for i=0,2 do if (m1[i] eq m2[i]) and (m1[i] eq m3[i]) then normal=i
                mia=N_lims[*,kd+1]
               ;NORMAL SLICE
               ;************
                if normal ge 0 then begin ;as normal slice
                   if normal ne alo then begin & alo=normal & mov=0 & endif ;Correct bad direction
                   if (mov eq 0) or (mov eq 1000) then begin
                             imb1=total(N_dens[*,*,*,kd],1) & max1=max(imb1,min=min1)
                             imb2=total(N_dens[*,*,*,kd],2) & max2=max(imb2,min=min2)
                             imb3=total(N_dens[*,*,*,kd],3) & max3=max(imb3,min=min3)
                             maxt=max([max1,max2,max3]) & mint=min([min1,min2,min3])
                             if alo eq 0 then imb=imb1 else if alo eq 1 then imb=imb2 else imb=imb3
                             imb1=0 & imb2=0 & imb3=0
                             if mint ge 0. then imb=bytscl(sqrt(imb),min=sqrt(mint),max=sqrt(maxt)) $
                                           else imb=bytscl(imb,min=mint,max=maxt)
                   endif
                   if alo eq 0 then begin & sli=round((szn[1]-1)*mov/1000.)>0
                      if (mov ne 0) and (mov ne 1000) then imb=reform(N_dens[sli,*,*,kd]) & t_c=[[dup[1],0],[dup[1],dup[2]],[0,dup[2]],[0,0]] & endif
                   if alo eq 1 then begin & sli=round((szn[2]-1)*mov/1000.)>0
                      if (mov ne 0) and (mov ne 1000) then imb=reform(N_dens[*,sli,*,kd]) & t_c=[[dup[0],0],[dup[0],dup[2]],[0,dup[2]],[0,0]] & endif
                   if alo eq 2 then begin & sli=round((szn[3]-1)*mov/1000.)>0
                      if (mov ne 0) and (mov ne 1000) then imb=reform(N_dens[*,*,sli,kd]) & t_c=[[dup[0],0],[dup[0],dup[1]],[0,dup[1]],[0,0]] & endif

                   if (mov ne 0) and (mov ne 1000) then begin
                           if mia[0] ge 0. then imb=bytscl(sqrt(imb),min=sqrt(mia[0]),max=sqrt(mia[1])) $
                                           else imb=bytscl(imb,min=mia[0],max=mia[1])
                   endif
;tv,congrid(imb,200,200),200,200
               ;**********
               ;NOT NORMAL ???
               ;**********
                endif else begin     ;otherwise
                   N_tmp=N_dens[*,*,*,kd]
                   if total(dup[0:2]) gt 3 then begin & N_ttm=bytarr(szn[1]*dup[0],szn[2]*dup[1],szn[3]*dup[2])
                     for k=0,dup[2]-1 do for j=0,dup[1]-1 do for i=0,dup[0]-1 do $
                         N_ttm[i*szn[1]:(i+1)*szn[1]-1,j*szn[2]:(j+1)*szn[2]-1,k*szn[3]:(k+1)*szn[3]-1]=N_tmp
                     N_tmp=N_ttm & N_ttm=0 & szn =size(N_tmp) & endif
                   cX=(total(popi[0,*])/nc*(szn[1]-1.))<szn[1]>0.
                   cY=(total(popi[1,*])/nc*(szn[2]-1.))<szn[2]>0.
                   cZ=(total(popi[2,*])/nc*(szn[3]-1.))<szn[3]>0.
                   abc=szn[sort(szn[1:3])+1] & Xs=long(sqrt(abc[1]^2+abc[2]^2)) & Ys=Xs
                   xnorm=[ap,bp,cp]
                   direct=popi[*,0]-popi[*,nc-1]
                   mxn=max(N_tmp,min=mxi)+1
                   imb=extract_slice(N_tmp, Xs,Ys, cX,cY,cZ, xnorm, direct, OUT_VAL=mxn, /sample)
                   idx=where(imb ne mxn)
                   if idx[0] ge 0 then begin
                   ;Calculate texture_coord
                    i1=-1 & i=0    & while i1 eq -1 do begin & idx=where(imb[i,*] ne mxn) & if idx[0] eq -1 then i=i+1 else i1=i & endwhile
                    i2=-1 & i=Xs-1 & while i2 eq -1 do begin & idx=where(imb[i,*] ne mxn) & if idx[0] eq -1 then i=i-1 else i2=i & endwhile
                    j1=-1 & i=0    & while j1 eq -1 do begin & idx=where(imb[*,i] ne mxn) & if idx[0] eq -1 then i=i+1 else j1=i & endwhile
                    j2=-1 & i=Ys-1 & while j2 eq -1 do begin & idx=where(imb[*,i] ne mxn) & if idx[0] eq -1 then i=i-1 else j2=i & endwhile
                    imb=imb[i1:i2,j1:j2] & Xs=(size(imb))[1] & Ys=(size(imb))[2]
                    if (Xs gt 1) and (Ys gt 1) then begin
                     if nc ne 4 then begin
                      i1=-1 & i=0    & while i1 eq -1 do if imb[i,0]    eq mxn then i=i+1 else i1=round(round(i*10./(Xs-1))*(Xs-1)/10.)
                      i2=-1 & i=Xs-1 & while i2 eq -1 do if imb[i,0]    eq mxn then i=i-1 else i2=round(round(i*10./(Xs-1))*(Xs-1)/10.)
                      i3=-1 & i=0    & while i3 eq -1 do if imb[i,Ys-1] eq mxn then i=i+1 else i3=round(round(i*10./(Xs-1))*(Xs-1)/10.)
                      i4=-1 & i=Xs-1 & while i4 eq -1 do if imb[i,Ys-1] eq mxn then i=i-1 else i4=round(round(i*10./(Xs-1))*(Xs-1)/10.)
                      j1=-1 & i=0    & while j1 eq -1 do if imb[0,i]    eq mxn then i=i+1 else j1=round(round(i*10./(Ys-1))*(Ys-1)/10.)
                      j2=-1 & i=Ys-1 & while j2 eq -1 do if imb[0,i]    eq mxn then i=i-1 else j2=round(round(i*10./(Ys-1))*(Ys-1)/10.)
                      j3=-1 & i=0    & while j3 eq -1 do if imb[Xs-1,i] eq mxn then i=i+1 else j3=round(round(i*10./(Ys-1))*(Ys-1)/10.)
                      j4=-1 & i=Ys-1 & while j4 eq -1 do if imb[Xs-1,i] eq mxn then i=i-1 else j4=round(round(i*10./(Ys-1))*(Ys-1)/10.)
                      t_c=[[i2,0.]]
                      if j3 ne 0    then t_c=[[t_c],[Xs-1,j3]] & if j4 ne j3 then t_c=[[t_c],[Xs-1,j4]]
                      if i4 ne Xs-1 then t_c=[[t_c],[i4,Ys-1]] & if i3 ne i4 then t_c=[[t_c],[i3,Ys-1]]
                      if j2 ne Ys-1 then t_c=[[t_c],[0,j2]]    & if j2 ne j1 then t_c=[[t_c],[0,j1]]
                      if j1 ne 0    then                         if i1 ne i2 then t_c=[[t_c],[i1,0]]
                     ;t_c=shift(t_c,0,1)
                      t_c[0,*]=t_c[0,*]/(Xs-1) & t_c[1,*]=t_c[1,*]/(Ys-1)
                     endif else t_c=t_a ;t_c=shift(t_a,0,1)
                     idx=where(imb eq mxn) & if idx[0] ne -1 then imb[idx]=mxi

                     if mia[0] ge 0. then imb=bytscl(sqrt(imb),min=sqrt(mia[0]),max=sqrt(mia[1])) $
                                     else imb=bytscl(imb,min=mia[0],max=mia[1])
;if (size(t_c))[2] ne nc then begin dens=-1 & print,'Bad slice !!!' & endif
;wset,0 & tv,congrid(imb,200,200) & print,t_c & print,'---' & print,direct & print,'---' & print,popi
;if mov eq 500 then stop
                     dens=0
                    endif else dens=0
                   endif else dens=0
                endelse ;not normal plane
               
                if dens eq 1 then begin
                         otit[npla,0]->GetProperty,texture_map=oI
                         oI->SetProperty,data=imb,palette=otit[0,4]
                endif
             endif ;we have densities
            
             if dens ge 0 then $
                 if fixi then begin ;FIXE plane or not
                       otit[npla,0]->GetProperty,texture_map=oIm
                       idx=where(otit[npla,*] eq bid) & idx=idx[0]>1
                       ik =1
                       if idx gt 1 then begin  ;Prevent from multiple "fix-it" clicks.
                          otit[npla,idx-1]->GetProperty,DATA=pupi
                          if n_elements(popo)  eq n_elements(pupi) then if total(abs(popo-pupi)) eq 0 then ik=0
                       endif
                       if ik then begin
                         if dens then begin & oIt = OBJ_NEW('IDLgrImage',reform(imb),palette=otit[0,4])
                                      otit[npla,idx]= OBJ_NEW('IDLgrPolygon',DATA=popo,shading=0,texture_map=oIt ,poly=pol,uvalue=[kxyz,mov,alo,kd],/texture_interp,texture_coord=t_c,color=[255,255,255])
                         endif   else otit[npla,idx]= OBJ_NEW('IDLgrPolygon',DATA=popo,shading=0,texture_map=oIm ,poly=pol,uvalue=[kxyz,mov,alo,kd],color=[255,255,255])
                         state.oModelSurface ->Add, otit[npla,idx]
                         ltit[npla,idx]=ltit[npla,0]
                       endif
                 endif else $
                         if dens then otit[npla,0]->SetProperty,DATA=popo, poly=pol,HIDE=hid, uvalue=[kxyz,mov,alo,kd],texture_map=oI,texture_interp=1,texture_coord=t_c $
                                 else otit[npla,0]->SetProperty,DATA=popo, poly=pol,HIDE=hid, uvalue=[kxyz,mov,alo,kd],texture_coord=t_a
             endif
     end
  5: begin & OBJ_DESTROY, otit[npla,1:*] & otit[npla,1:*]=bid & end  ;***** REMOVE THE PLANES (reset)

  6: begin & idx=where(otit[npla,*] eq bid) & idx=(idx[0]-1)>0 & fixi=pcent[16]>2
     case fixi of ;static
     2: begin &  cocol=[0L,0L,0L] & iatom=0  ;***** BUILD A POLYGON (REMOVE)
               if ltit[0,13] then ff=[fa,fb,fc] else ff=ltit[0,3:5] ;Cell or not!
               if total(cxyz) eq 0 then begin ;Build from selected atoms
                  nat=n_elements(atom_type) & nco=0 & sel=0 & sol=0
                  if prop lt 0 then begin & iatom=pcent[20:*] & ida=where(iatom lt nat) & if n_elements(ida) gt 1 then $
                    if n_elements(ida) ne n_elements(iatom) then iatom=iatom[ida]
                       sel=atom_xyz[*,iatom] & cocol=pcent[17:19] & sol=atom_coo[*,iatom]
                  endif else begin
                    for i=0,nat-1 do if ppaobj[5,i] ne bid then begin & ppaobj[5,i]->GetProperty,HIDE=hid
                        if not hid then begin & sel=[sel,i] & ppaobj[5,i]->SetProperty,HIDE=1
                                  ppaobj[1,i]-> SetProperty,STYLE=2
                                  ppaobj[1,i]-> GetProperty,COLOR=colo
                                  sol=[sol,i] & nco=nco+1 & cocol=cocol+colo & endif
                    endif
                    basets.styles[*]=2 & WIDGET_CONTROL, wDraw ,SET_UVALUE=basets
                    if n_elements(sel) gt 2 then begin & iatom=sel[1:*] & sel=atom_xyz[*,iatom] & cocol=cocol/nco
                                                       sol=atom_coo[*,iatom] & endif
                  endelse
               endif else begin                   ;Build from c1 c2 c3
                  sel=[[cxyz[0:2]/1000.],[cxyz[3:5]/1000.],[cxyz[6:8]/1000.]] & sol=sel
                  q  =reform(ltit[0,3:5]*ltit[0,6:8]/2.) +ffb
                  sel[0,*]=sel[0,*]*ff[0]-q[0] & sel[1,*]=sel[1,*]*ff[1]-q[1] & sel[2,*]=sel[2,*]*ff[2]-q[2]
                  d_livChangAxis,sel
               endelse
               if n_elements(sel) ge 6 then begin & ik=1
                  if otit[npla,idx] ne bid  then begin
                    if pcent[16] eq -1 then begin & ik=0 & k=idx+1  ;remove selected polygon
                      for i=0,idx do begin & otit[npla,i]->GetProperty,DATA=pupi
                        if n_elements(sel)  eq n_elements(pupi) then if total(abs(sel-pupi)) eq 0 then k =i
                      endfor
                      if k lt idx+1  then begin
                        OBJ_DESTROY, otit[npla,k] & otit[npla,k]=bid
                        if k lt idx then for i=k,idx do begin & otit[npla,i]=otit[npla,i+1] & ltit[npla,i]=ltit[npla,i+1] & endfor
                      endif
                    endif else begin                              ;or Prevent from multiple clicks.
                        otit[npla,idx]->GetProperty,DATA=pupi
                        if n_elements(sel)  eq n_elements(pupi) then if total(abs(sel-pupi)) eq 0 then ik=0
                    endelse
                  endif
                 ;Gives distance
                  if n_elements(sel) eq 6 then begin & diff=(sol[*,1]-sol[*,0])*ff & d_livChangAxis,XYZ=diff,GETMODUL=modu
                     dista=round(modu*1000)/1000.
                     widget_control,state.l_mess,bad_id=ii,set_value='Distance='+string(dista,format='(f7.3)')+'                      '
                  endif
                  if total(cxyz) eq 0 then $
                     if n_elements(sel) eq 9 then begin & v1=sel[*,0] & v2=sel[*,1] & v3=sel[*,2]
                        angl3=abs(acos(total((v2-v3)*(v1-v3))/(sqrt(total((v2-v3)^2))*sqrt(total((v1-v3)^2))))*180./!pi)
                        angl2=abs(acos(total((v1-v2)*(v3-v2))/(sqrt(total((v1-v2)^2))*sqrt(total((v3-v2)^2))))*180./!pi)
                        angl1=abs(acos(total((v2-v1)*(v3-v1))/(sqrt(total((v2-v1)^2))*sqrt(total((v3-v1)^2))))*180./!pi)
                        angl='\'+a_symb[atom_type[iatom[2]]]+'/'+strtrim(string(angl3,format='(f6.2)'),2)+ $
                             ' \'+a_symb[atom_type[iatom[1]]]+'/'+strtrim(string(angl2,format='(f6.2)'),2)+ $
                             ' \'+a_symb[atom_type[iatom[0]]]+'/'+strtrim(string(angl1,format='(f6.2)'),2)+' degrees'
                        widget_control,state.l_mess,bad_id=ii,set_value=angl
                  endif
                  if ik then begin & if otit[npla,idx] ne bid then idx=idx+1 ;Build
                           d_livBuildPoly, sel, opolygon=opoh, opolyline=opol,color=cocol,uvalue=iatom
                           if opol ne bid then begin & state.oModelSurface->Add, opol & otit[npla,idx]=opol & idx=idx+1   & endif
                           if opoh ne bid then begin & state.oModelSurface->Add, opoh & otit[npla,idx]=opoh & endif
                  endif
              endif
        end
     3: if otit[npla,idx] ne bid then begin     ;***** HIDE/SHOW LAST POLYGON
             otit[npla,idx]->GetProperty,HIDE=hid & hid=abs(hid-1)
             otit[npla,idx]->SetProperty,HIDE=hid
        endif
     4: if otit[npla,idx] ne bid then begin     ;***** HIDE/SHOW ALL POLYGONS
             otit[npla,idx]->GetProperty,HIDE=hid & hid=abs(hid-1)
             for i=0,idx do otit[npla,i]->SetProperty,HIDE=hid
        endif
     5: if otit[npla,idx] ne bid then begin     ;***** REMOVE LAST POLYGON
             OBJ_DESTROY, otit[npla,idx] & otit[npla,idx]=bid & endif
     6: begin & OBJ_DESTROY, otit[npla,*]   & otit[npla,*]  =bid & end  ;***** REMOVE ALL POLYGONS

     7: begin & for i=0,n_elements(atom_type)-1 do $                    ;***** HIDE (not)SELECTED ATOMS
                  if ppaobj[5,i] ne bid then begin & ppaobj[5,i]->GetProperty,HIDE=txthid
                  ;txthid=0 means atom selected (symbol visible)
                  ;hid=0    means hide selected atoms
                  ;hid=1    means hide not selected
                  ;hid=-1   means hide selected atoms type
                   if ((not txthid) and (hid le 0)) or ((txthid) and (hid eq 1)) then begin
                     if hid eq -1 then begin & a1=0 & a2=n_elements(atom_type)-1 & endif else begin & a1=i & a2=i & endelse
                     for j=a1,a2 do if atom_type[i] eq atom_type[j] then begin
                       reste[13,j]=-1
                       ppaobj[5,i]-> SetProperty,HIDE=1
                       ppaobj[1,i]-> SetProperty,STYLE=2
                       ppaobj[0,j]-> SetProperty,HIDE=1
                       endif
                   endif else if (not txthid) then begin
                     ppaobj[5,i]-> SetProperty,HIDE=1
                     ppaobj[1,i]-> SetProperty,STYLE=2
                   endif
                  endif
                  basets.styles[*]=2 & WIDGET_CONTROL, wDraw ,SET_UVALUE=basets

                  if  state.oSticks ne bid then begin
                    nn=n_elements(atom_type) & pripag=fltarr(12,nn) & idm=4+p25*3*2+p25
                    d_livConnex, atom_type, atom_coo , pripag,0, abc=[fa,fb,fc], visidx=reste[13,*]
                    d_livBlinks, nn,pripag,0, amesh
                    state.oSticks->GetProperty,HIDE=hid
                    if n_elements(amesh) gt 1 then state.oSticks->SetProperty, DATA=atom_xyz, POLYLINE=amesh, HIDE=hid $
                                              else state.oSticks->SetProperty, HIDE=1
                  endif
          end

     8: begin & ccc=0                                             ;***** RETURN C1 C2 C3 FOR PLANE
              for i=0,n_elements(atom_type)-1 do $
              if ppaobj[5,i] ne bid then begin & ppaobj[5,i]->GetProperty,HIDE=txthid
                if (not txthid)     then begin
                   ppaobj[5,i]-> SetProperty,HIDE=1
                   ppaobj[1,i]-> SetProperty,STYLE=2
                   if ccc lt 3 then begin & cxyz[ccc*3:ccc*3+2]=round(atom_coo[*,i]*1000.) & ccc=ccc+1
              endif & endif & endif
              basets.styles[*]=2 & WIDGET_CONTROL, wDraw ,SET_UVALUE=basets
              if ccc lt 3 then cxyz=cxyz*0
              PCENT[5:13]=cxyz
          end

     10:begin & idx=where(otit[npla,*] eq bid) & idx=(idx[0]-1)>0  ;***** HIDE/SHOW ALL BOXES
               if otit[npla,idx] ne bid then begin
                  otit[npla,idx]->GetProperty,HIDE=hid & hid=abs(hid-1)
                  for i=0,idx do otit[npla,i]->SetProperty,HIDE=hid   & endif & end
                 
     11: if ltit[0,11] eq 0 then begin                           ;***** SHOW/HIDE BONDS
             if  state.oSticks eq bid then begin
               nn=n_elements(atom_type) & pripag=fltarr(12,nn) & idm=4+p25*3*2+p25
               d_livConnex, atom_type, atom_coo, pripag,0 ,abc=[fa,fb,fc] ,visidx=reste[13,*]
               d_livBlinks, nn,pripag,0,amesh
               if (n_elements(amesh) le 1) then ltit[0,11]=0 $
               else begin & d_livOlinks, state, amesh
                          WIDGET_CONTROL, baset , SET_UVALUE=state & endelse
             endif else begin
                  state.oSticks->GetProperty,HIDE=Mhid
                  state.oSticks->SetProperty,HIDE=abs(1-Mhid) & endelse
          endif
     12: if otit[0,3] ne bid then begin                         ;***** HIDE/SHOW COLOR-BAR
             otit[0,3]->GetProperty,HIDE=hid & hid=abs(hid-1)
             otit[0,3]->SetProperty,HIDE=hid & endif

     13: if hid eq 1 then begin & oldi=state.did_transval ;***** VIEW POINT: RESET TO DEFAULT
            if oldi ge 0 then otit[5,3]->translate,-atom_xyz[0,oldi],-atom_xyz[1,oldi],-atom_xyz[2,oldi]
            if oldi ge 0 then ppaobj[1,oldi]-> SetProperty,HIDE=0  ;show again pivot Atom
            state.oModelSurface->SetProperty,TRANSFORM= state.tmg0 ;reset to origin rotation
            state.oModelEdges  ->SetProperty,TRANSFORM= state.tmg0 ; ""
            state.oModelPivot  ->SetProperty,TRANSFORM=identity(4) ,HIDE=1
            otit[5,3]  ->SetProperty,HIDE=1, transform=identity(4) ;Pivot axes, be sure being at origine
            otit[5,2]  ->SetProperty,HIDE=0 ;Main axes             ;replace pivot axes by main axes
            otit[5,0]  ->SetProperty,HIDE=0 ;Boxes
           ;reset to origin eye and clip
            state.oView->SetProperty,EYE=3., ZCLIP=[1.5,-1.5],PROJECTION=2,DEPTH_CUE=[-.5,1.5]

            state.did_transval=-1
            WIDGET_CONTROL, baset , SET_UVALUE=state

          endif else begin                             ;***** VIEW POINT: SET ON SELECTED ATOM or CENTER IT
            idx=-1
            for i=0,n_elements(atom_type)-1 do $       ;check for a selected atom
                if ppaobj[5,i] ne bid then begin & ppaobj[5,i]->GetProperty,HIDE=txthid
                   if not txthid then begin
                      ppaobj[5,i]-> SetProperty,HIDE=1  ;set atom not delected
                      ppaobj[1,i]-> SetProperty,STYLE=2 ; ""
                      idx=[idx,i]                       ; add one more selected atom
                endif & endif

            basets.styles[*]=2 & WIDGET_CONTROL, wDraw ,SET_UVALUE=basets
           ;Ok we have one atom selected only
            if n_elements(idx) eq 2 then begin & oldi=state.did_transval
             ;reset old view point
              if oldi ge 0 then ppaobj[1,oldi]-> SetProperty,STYLE=2,HIDE=0
              if oldi ge 0 then otit[5,3]->translate,-atom_xyz[0,oldi],-atom_xyz[1,oldi],-atom_xyz[2,oldi]
              ppaobj[1,idx[1]]   ->SetProperty,HIDE=1  & state.did_transval=idx[1] ;hide Atom
              ppaobj[1,idx[1]]   ->GetProperty,COLOR  =symc
              ppaobj[5,idx[1]]   ->GetProperty,STRINGS=symb
              state.oModelSurface->SetProperty,TRANSFORM= state.tmg0               ;reset to origin rotation
              state.oModelEdges  ->SetProperty,TRANSFORM= state.tmg0               ; ""
              state.oModelPivot  ->SetProperty,TRANSFORM= identity(4)              ; ""

             ;CENTER SELECTED ATOM hid=2: Keep eye outside the scene             
             ;POSITION ON THE ATOM hid=0: Set new eye to the center of view and change the clip
              if hid eq 0 then state.oView->SetProperty,EYE=.051,ZCLIP=[.05,-3.],PROJECTION=1,DEPTH_CUE=[-.5,3.]

              state.oModelSurface->getProperty,TRANSFORM =ssc & ssc=[ssc[0,0],ssc[1,1],ssc[2,2]]
              newtrans=-atom_xyz[*,idx[1]]*ssc                                     ;translate Model on front screen
              state.oModelSurface->translate, newtrans[0],newtrans[1],newtrans[2]  ; ""
              state.oModelOffset ->translate, newtrans[0],newtrans[1],newtrans[2]
              otit[5,4]->SetProperty,STRINGS=symb,COLOR=symc                       ;put atom symbol
              otit[5,3]->translate,atom_xyz[0,idx[1]],atom_xyz[1,idx[1]],atom_xyz[2,idx[1]]
              otit[5,3]->SetProperty,HIDE=0 ;Pivot axes                            ;replace main axes by pivot axes
              otit[5,2]->SetProperty,HIDE=1 ;Main axes
              otit[5,0]->SetProperty,HIDE=1 ;Boxes
              WIDGET_CONTROL, baset , SET_UVALUE=state
            endif
          endelse
                            ;To keep current objects for future animations (planes)
                            ;   otit(0,*)=title,symbol,clouds,Bar,Palette,vol_dens 5,6 10,11
                            ;   otit(1:3,*)=planes  otit(4,*)=poly
                            ;   otit(5,0)=boxes  otit(5,2)=axes  otit(5,3)=pivot axes in surface
                            ;   otit(5,4)=atom symbol for pivot axes in surface
                            ;   otit(5,5)=pivot axes in top model (not used -> oModelPivot)

     16: if otit[5,2] ne bid then begin                              ;***** HIDE/SHOW AXIS
             otit[5,2]->GetProperty,HIDE=hid & hid=abs(hid-1)
             otit[5,2]->SetProperty,HIDE=hid & endif
            
     17: if otit[0,0] ne bid then begin                              ;***** HIDE/SHOW TITLE
             otit[0,0]->GetProperty,HIDE=hid & hid=abs(hid-1)
             otit[0,0]->SetProperty,HIDE=hid & endif
     else:
     endcase
    
    end
  else:
  endcase
 
  state.oWindow->Draw, state.oView
  did_mpeg,'put'
 
;************
;ENTERING IN CELL (dynamic)
;************
endif else if (flog eq 'INCELL') and (coef gt 0) then begin & ok=0
  GoDo = Pcent[0]
  slidV= Pcent[1]
  ssc  =[state.tmg0[0,0],state.tmg0[1,1],state.tmg0[2,2]]
  state.oView->GetProperty,EYE=eye

  if (GoDo eq 1) and (slidV ne 0.) then begin ;MOVE inside view

            if eye ge 1. then begin           ;ENTERING firt time
             if slidV gt 0 then begin
              fabc=[fa,fb,fc]*ssc & fabc=fabc[sort(fabc)]/2. & fabc=sqrt(fabc[1]^2 + fabc[2]^2)
              state.oView->SetProperty,EYE=.051,ZCLIP=[.01,-3.],PROJECTION=1,DEPTH_CUE=[-.5,3.]
              state.oModelSurface->translate, 0. , 0. , -fabc ; Move model to Z border
              state.oModelOffset ->translate, 0. , 0. , -fabc ; Move model to Z border
              state.oModelPivot  ->SetProperty, HIDE=0
             endif
             slidV=0.
            ;print,'baset=',baset ;For debug:  widget_control,baset ,get_uvalue=state
            endif

           ;Directions [-1,1]: +Xleft -Xright -Yupper +Ylower -Zinside
            state.oModelSurface->translate, 0. , 0. , slidV* fc/2. * ssc[2] ; Move inside model
            state.oModelOffset->translate,  0. , 0. , slidV* fc/2. * ssc[2] ; Move inside model
            state.oModelSurface->GetProperty,TRANSFORM=t
            tt=abs(t[3,0:2])
            if (tt[0] ge .6) or (tt[1] ge .6) or (tt[2] ge .6) then  GoDo = 0 ;Outside the cell so -> reset
  endif
 
  if  GoDo eq 0 then begin ;RESET to origin
      if eye lt 1. then begin
            state.oModelSurface->SetProperty,TRANSFORM= state.tmg0 ;reset to origin rotation
            state.oModelEdges  ->SetProperty,TRANSFORM= state.tmg0 ; ""
            state.oModelPivot  ->SetProperty,TRANSFORM= identity(4); ""
            state.oModelPivot  ->SetProperty,HIDE=1
           ;reset to origin eye and clip
            state.oView->SetProperty,EYE=3., ZCLIP=[1.5,-1.5],PROJECTION=2,DEPTH_CUE=[-.5,1.5]
      endif
  endif
 
  state.oWindow->Draw, state.oView
  did_mpeg,'put'

endif else if (flog eq 'GET_INFO') and (coef gt 0) then begin & ok=0            ;***** RETURN PLANE & POLYGON INFO
      PCENT=fltarr((size(propag))[1]) & tmp=PCENT & bid =OBJ_NEW()
      for npla=1,3 do begin & i=1
          while otit[npla,i] ne bid do begin                                      ;loop for planes
            otit[npla,i]-> GetProperty,HIDE=hid,uvalue=uval,color=cocol
            if (not hid) and (n_elements(uval) gt 9) then begin
              tmp[*]=0 & tmp[0]=-4 & tmp[1]=ltit[npla,i] & tmp[2]=cocol & tmp[5]=npla & tmp[6]=uval   & PCENT=[[PCENT],[tmp]] & endif
          i=i+1 & endwhile
      endfor
      i=0 & npla=4
      while otit[npla,i] ne bid do begin                                          ;loop for polygons
        otit[npla,i]-> GetProperty,HIDE=hid,uvalue=uval,color=cocol
        if (not hid) and (n_elements(uval) ge 2) then begin
          tmp[*]=0 & tmp[0]=-6 & tmp[1]=ltit[npla,i] & tmp[2]=cocol & tmp[5]=npla & tmp[6]=uval+1 & PCENT=[[PCENT],[tmp]] & endif
      i=i+1 & endwhile

      if otit[0,2] ne bid then begin & otit[0,2]-> GetProperty,HIDE=hid,uvalue=uval
         if (not hid) and (n_elements(uval) gt 1) then begin                      ;Clouds
           tmp[*]=0 & tmp[0]=-8 & tmp[4]=1 & tmp[5]=uval[0] & tmp[6]=uval[1] & PCENT=[[PCENT],[tmp]]
      endif & endif

      for nv=0,4 do if otit[0,5+nv] ne bid then begin                             ;Volume Densities
            otit[0,5+nv]-> GetProperty,HIDE=hid,uvalue=uval
            if (not hid) and (n_elements(uval) ge 3) then begin
              otit[0,5+nv]-> GetProperty,STYLE=sty
              tmp[*]=0 & tmp[0]=-9 & tmp[1]=uval[0] & tmp[2]=uval[1] & tmp[3]=0 & tmp[4]=sty+1 & PCENT=[[PCENT],[tmp]]
      endif & endif

      if (size(PCENT))[0] eq 2 then PCENT=PCENT[*,1:*] else PCENT=0

;*******************************************************
;REDRAW PLANES , POLY ... from a saved file or Edit<->plot
;*******************************************************
endif else if (flog eq 'SET_PLANES') and (coef gt 0) then begin & ok=0         ;***** CONSTRUCT PLANES & POLYGONS
      if n_elements(PCENT) gt 1 then begin
        sizpc= size(PCENT)
        if sizpc[0] eq 1 then npc=1 else npc=sizpc[2]
        for i=0,npc-1 do begin
          if pcent[0,i] eq -4 then begin
             N_lims[0,0]=pcent[17,i]
             d_livDrawPrint, wDraw , 'PLANE',    snap, PCENT=[ 1,pcent[5,i],0,pcent[1,i],1,pcent[6:14,i],0,0,0] ;texture
             d_livDrawPrint, wDraw , 'PLANE',    snap, PCENT=[-4,pcent[5,i],0,pcent[1,i],1,pcent[6:14,i],pcent[15:16,i],1,pcent[17,i]]
          endif
          if pcent[0,i] eq -6 then begin
             sel=pcent[6:*,i] & sel=sel[where(sel gt 0)]-1
             d_livDrawPrint, wDraw , 'PLANE',    snap, PCENT=[-6,pcent[5,i],0,0,1,fltarr(9),0,0,1,pcent[2:4,i],sel]
          endif
          if pcent[0,i] eq -8 then begin
             d_livDrawPrint, wDraw , 'CLOUDS',   snap, PCENT=[0,0,0,0,pcent[4,i],pcent[5,i],pcent[6,i]]
          endif
          if pcent[0,i] eq -9 then begin
             d_livDrawPrint, wDraw , 'VOL_DENS', snap, PCENT=[pcent[1:4,i]]
          endif
        endfor
      endif
endif else begin

;************
;OUTPUT FILES
;************
 WIDGET_CONTROL,/hourglass
 file='lamp' & if state.did_wi gt 0 then file=file+'_W'+strtrim(string(state.did_wi),2)
 flig=flog   & ok=1
 if  (flog eq 'GIF') or (flog eq 'PNG') or (flog eq 'JPEG') or (flog eq 'SNAP') then flig='IMG'
 case flig of
'PS':   begin & file='file'
        oPrinter= obj_new('IDLgrPrinter')
        state.oWindow->GetProperty,resolution=res,dimensions=Sxy
        if Sxy[1] lt Sxy[0] then oPrinter->SetProperty,landscape=1
        if (!version.os eq 'Win32') or (!version.os eq 'MacOS') then begin
             ok=dialog_printersetup(oPrinter)
             if ok ne 0 then ok=dialog_printjob(oPrinter)
        endif else begin & file='xprinter.eps'
          filt='*.eps' & if !version.release ge '5.4' then filt=['*.eps;*.ps']
          file=dialog_pickfile(filter=filt,file=file,path=kpath.kppth,get_path=kkpth,/write) & kpath.kppth=kkpth
          if file eq '' then ok=0
        endelse
        if ok ne 0 then begin
                state.oWindow->draw, state.oView ; Expose.
                DPI = 2.54/float(res)
                ;Hack, swap from pixels to inches for views...
                ;If a scene is defined: oViews=scene->get(/all)
                state.oView  ->IDLgrView::getproperty,loc=loc ,dim=vdim ,color=col
                    locp  = loc /DPI
                    vdimp = Sxy /DPI
                state.oView  ->IDLgrView::setproperty,loc=locp,dim=vdimp,units=1,color=[255,255,255]

                ;PRINT...
                oPrinter->Draw,state.oView
                oPrinter->newdocument

                ;...and back to pixels
                state.oView  ->IDLgrView::setproperty,loc=loc ,dim=vdim ,units=0,color=col
        endif
        obj_destroy,oPrinter
    end
'IMG':  begin
           if flog eq 'GIF'  then begin & file=file+'.gif'  & filt='*.gif'        & endif
           if flog eq 'PNG'  then begin & file=file+'.png'  & filt='*.png'        & endif
           if flog eq 'JPEG' then begin & file=file+'.jpeg' & filt='*.jpeg'       & endif
           if flog ne 'SNAP' then $
             if n_elements(pcent) eq 2 then begin
               seq =strtrim(string(fix(pcent[1])),2) & if strlen(seq) eq 1 then seq='00'+seq else if strlen(seq) eq 2 then seq='0'+seq
               file='magdraw_'+seq+'.jpeg'
             endif else begin
               file=dialog_pickfile(filter=filt,file=file,path=kpath.kppth,get_path=kkpth,/write) & kpath.kppth=kkpth
               if file eq '' then ok=0
               if strpos(strlowcase(file),'.gif')  ge 0 then flog='GIF'  else $
               if strpos(strlowcase(file),'.png')  ge 0 then flog='PNG'  else $
               if strpos(strlowcase(file),'.jpeg') ge 0 then flog='JPEG' else $
               if strpos(strlowcase(file),'.jpg')  ge 0 then flog='JPEG'
             endelse
           if ok ne 0 then begin
               state.oWindow->GetProperty, dimensions=Sxy & Kxy=Sxy & Sxy=Sxy*((pcent[0]/100.)<15.)
               Obuf= obj_new('IDLgrBuffer')
               Obuf->GetDeviceInfo, MAX_VIEWPORT_DIMENSIONS=Mxy
               if Sxy[0] gt Mxy[0] then Sxy=[Mxy[0],Sxy[1]*Mxy[0]/Sxy[0]]
               if Sxy[1] gt Mxy[1] then Sxy=[Sxy[0]*Mxy[1]/Sxy[1],Mxy[1]]
               if flog eq 'SNAP' then Sxy=[192,192]
               Obuf->SetProperty,dimensions=Sxy
               state.oView  ->IDLgrView::setproperty,dim=Sxy
               Obuf->draw, state.oView
               Oimg=Obuf->read() & Oimg->GetProperty,data=snap
               state.oView  ->IDLgrView::setproperty,dim=Kxy
               obj_destroy,Obuf  & obj_destroy,Oimg
               tz  =size(snap)
               if (flog eq 'JPEG') and (tz[0] eq 3) then WRITE_JPEG,file,snap,true=1,quality=95 $
               else begin
                 r=0 & if tz[0] eq 3 then begin & snap=Color_Quan(snap,1,r,g,b,/dither)
                                  tz=size(snap) & tvlct,kr,kg,kb,/get & endif
         ;tbas=widget_base(map=0)
         ;tdrw=widget_draw(tbas,retain=2,xsize=tz(1),ysize=tz(2))
         ;widget_control,tbas,/realize & widget_control,tdrw,get_value=WinId
         ;wset,WinId & if n_elements(r) gt 1 then tvlct,r,g,b   & tv,snap & snap=tvrd()
         ;widget_control,tbas,/destroy
         ;if n_elements(r) gt 1 then tvlct,kr,kg,kb

                 if flog eq 'GIF'  then if n_elements(r) gt 1 then CALL_PROCEDURE,'write_kif' ,file,snap,r,g,b else CALL_PROCEDURE,'write_kif' ,file,snap
                 if flog eq 'PNG'  then if n_elements(r) gt 1 then WRITE_PNG ,file,snap,r,g,b,TRANSP=[0] else WRITE_PNG ,file,snap,TRANSP=[0]
                 if flog eq 'JPEG' then                            WRITE_JPEG,file,snap
                 if flog eq 'SNAP' then  ok=0
               endelse
           endif
         end
'SCREEN':begin &  file=file+'.ps'
         state.oWindow->GetProperty, dimensions=Sxy
         Obuf= obj_new('IDLgrBuffer',dimensions=Sxy)
         state.oView  ->IDLgrView::getproperty,color=col
         state.oView  ->IDLgrView::setproperty,color=[180,180,180]
         Obuf->draw, state.oView
         state.oView  ->IDLgrView::setproperty,color=col
         Oimg=Obuf->read() & Oimg->GetProperty,data=snap
         obj_destroy,Obuf  & obj_destroy,Oimg

         tz  =size(snap)   & tbas=widget_base(map=0) & r=0
         if tz[0] eq 3 then begin & snap=Color_Quan(snap,1,r,g,b,/dither)
                                  tz=size(snap) & tvlct,kr,kg,kb,/get & endif
         tdrw=widget_draw(tbas,retain=2,xsize=tz[1],ysize=tz[2])
         widget_control,tbas,/realize & widget_control,tdrw,get_value=WinId
         wset,WinId & if n_elements(r) gt 1 then tvlct,r,g,b   & tv,snap

         CALL_PROCEDURE,'p_did_event', 0, [-88,350,WinId,0,state.did_wi,0,1,0]
         widget_control,tbas,/destroy
         if n_elements(r) gt 1 then tvlct,kr,kg,kb
         end
'VRML': begin & filu=file+'.wrl'
          filu=dialog_pickfile(filter=['*.wrl','*x3d*'],file=filu,path=kpath.kppth,get_path=kkpth,/write) & kpath.kppth=kkpth
          if filu eq '' then ok=0
          if ok ne 0 then begin
                state.oWindow->GetProperty   ,dimensions=wdims, $
                          resolution=res,color_model=cm,n_colors=icolors
                oVRML=obj_new('IDLgrVRML',dimensions=wdims, $
                          resolution=res,color_model=cm,n_colors=icolors)
                oVRML->setproperty, filename=filu
                oVRML->Draw,state.oView
                obj_destroy,oVRML
;TEST X3D
;********
;x3d = obj_new('VISgrX3DOM', filename=file+'.x3d.html', dimensions=wdims)
;x3d->draw, state.oView, /full_html
;obj_destroy, x3d
;********
          endif
         end
'XOBJV': begin & ok=0 & Xobjview,state.oModelTop,/just_reg,group=state.wBase,title=state.did_tit
         end

 ELSE:  ok=0
 endcase
endelse
if ok ne 0 then WIDGET_CONTROL,state.l_mess,bad_id=ii,set_value=strmid(file+' is updated                          ',0,38)
cm = CHECK_MATH() & !except=kp_except
end

;**
;** **************
pro d_livDrawEvent, ev
;+
;** **************
;** Handle mouse events from the draw window
;-

CATCH,stat & if stat ne 0 then return

WIDGET_CONTROL, ev.id , GET_UVALUE=basets & baset=basets.baset
WIDGET_CONTROL, baset , GET_UVALUE=state, /NO_COPY
kp_except=!except & !except=0
if (ev.type EQ 4) then begin          ; Expose.
    WIDGET_CONTROL, baset, /HOURGLASS ; Redraw entire window
    state.oWindow->draw, state.oView

endif else begin                      ; Not expose
    bHaveTransform=0
    bHaveScaling  =0
    drag_it       =0
    draw_it       =0
    draw_itV      =1
    if state.did_scaling eq 0 then begin
      bHaveTransform = state.oTrack->Update(ev, TRANSFORM=qmat) ; trackball update: rotating
      if (bHaveTransform NE 0) then begin
        state.oModelSurface->GetProperty, TRANSFORM= t
        state.oModelSurface->SetProperty, TRANSFORM= t # qmat
        state.oModelEdges  ->SetProperty, TRANSFORM= t # qmat
       
        state.oModelPivot  ->GetProperty, TRANSFORM= t
        state.oModelPivot  ->SetProperty, TRANSFORM= t # qmat
      endif
    endif else if (ev.Y ge 0) then begin       ; ZOOMING
      delta2=(ev.Y - state.did_oldny)
      delta =delta2/2                          ; delta/2 is more sensitive to a rapid motion
      kpscl = state.did_scalval
      keyPress=( (ev.modifiers eq 1) or (ev.modifiers eq 2) or (ev.modifiers eq 8) ); SHIFT,CONTROL or ALT pressed
     
      state.did_scalval = (state.did_scalval+delta) > 40
      if not keyPress then state.did_scalval = state.did_scalval <800 ;Zoom limitation !
      state.did_oldny  = ev.Y
     
      if state.did_scalval ne kpscl then begin ; zooming
         if keyPress then state.did_scalval = (state.did_scalval-delta+delta/abs(delta)) > 40 ;Little scaling if keyPress
         s=state.did_scalval/100.
         bHaveScaling  =1
         state.did_cntxtfl=0
         state.scaling =s
         state.oModelTop->SetProperty   , TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]]
        ;state.oModelPivot->SetProperty , TRANSFORM = [[1./s,0,0,0],[0,1./s,0,0], [0,0,1./s,0], [0,0,0,1]] ;Keep pivot unscaled-> PB rotation

         if keyPress then begin ; -> IN CELL
          ;state.did_scalval = kpscl
           WIDGET_CONTROL, baset , SET_UVALUE=state
           d_livDrawPrint,ev.id,'INCELL',PCENT=[1,delta/20.]
           draw_itV=0          ; Prevent from drawing again (done in d_livDrawPrint)
         endif
      endif
    endif

    if (ev.type EQ 0) then begin    ; Button press.
        if ev.press gt 1 then begin ; Press with right or middle button
            picked = state.oWindow->select(state.oView, [ev.x, ev.y])
            if (size(picked))[0] eq 0 then hit=0 else $ ; Hit anything?
            if obj_class(picked[0]) ne "IDLGRMODEL" then hit=0 else hit=1
            if not hit then begin
                     state.did_scaling=1
                     state.did_cntxtfl=1
                     state.did_oldny=ev.Y
                     drag_it=1
            endif else begin
              picked[0]->GetProperty, UVALUE=uval, TRANSFORM=tm
              if state.l_mess gt 0 then WIDGET_CONTROL, state.l_mess,bad_id=ii,set_value=strmid(uval+'                   ',0,38) $
                                   else print,uval
              symbo=picked[0]->get(position=1)
              symbo->GetProperty,HIDE=hid & hid=abs(hid-1) & symbo->SetProperty,HIDE=hid  ;Toggle symbol
              d_livNewSelected, state, picked[0]->get()                                   ;Toggle style
              draw_it=1
            endelse
        endif else drag_it=1        ;Left button
    endif else if ((ev.type eq 2) and (state.btndown eq 1)) then begin ; MOTION.
        if (bHaveTransform) then state.oWindow->Draw, state.oView
        if (bHaveTransform) then did_mpeg,'put'
        if (bHaveScaling)   then state.oWindow->Draw, state.oView
        if (bHaveScaling)   then did_mpeg,'put'
    endif else if (ev.type eq 1) and state.btndown then begin          ; Release
        state.btndown    = 0
        state.did_scaling= 0
        state.oWindow->SetProperty, QUALITY=2
        if (ev.release eq 4) and (state.did_cntxtid gt 0) and (state.did_cntxtfl) then begin
           WIDGET_CONTROL, state.wDraw, DRAW_MOTION=0
           call_procedure,'WIDGET_DISPLAYCONTEXTMENU', ev.ID, ev.X, ev.Y, state.did_cntxtid
        endif else draw_it=draw_itV
    endif
    if drag_it then begin
            state.btndown = 1
            state.oWindow->SetProperty, QUALITY=state.dragq
            WIDGET_CONTROL, state.wDraw, /DRAW_MOTION
            endif
    if draw_it then begin
           WIDGET_CONTROL, ev.top, /HOURGLASS
           state.oWindow->Draw, state.oView
           did_mpeg,'put'
           WIDGET_CONTROL, state.wDraw, DRAW_MOTION=0
           endif
endelse

done: WIDGET_CONTROL, baset, SET_UVALUE=state, /NO_COPY
cm = CHECK_MATH() & !except=kp_except
end

;**
;** **********
pro d_livEvent, ev
;+
;** **********
;** other event handler
;-

catch,stat & if stat ne 0 then return

WIDGET_CONTROL, ev.id, GET_UVALUE=uval
if strpos(uval,'PLANE') eq 0 then begin & widget_control,/hourglass & magnet_win,/none
                                        magnet_plane ,ev.top ,mbas         & return  & endif

if strpos(uval,'MAGDR') eq 0 then begin & widget_control,/hourglass & magnet_win,/none & ww=0
    if strpos(uval,'_') gt 0 then begin & widget_control,ev.top,/destroy & ww=fix(strmid(uval,6,3))
    endif                    else begin & d_we=long(strmid(uval,6,10))
                                        widget_control,d_we,bad_id=ii,get_uvalue=basets
                                        if ii eq 0 then widget_control,basets.baset,bad_id=ii,get_uvalue=state
                                        if ii eq 0 then ww=state.did_wi
                                        if ii eq 0 then widget_control,widget_info(d_we,/parent),bad_id=ii,map=0
                                       ;if ii eq 0 then d_livCleanup, state.wbase,state
    endelse    &  if ww gt 0 then magnet_win,W=ww & return  & endif

if strpos(uval,'ANIDR') eq 0 then begin & widget_control,/hourglass & magnet_win,/none & ww=0
    if strpos(uval,'_') gt 0 then begin & widget_control,ev.top,/destroy & ww=fix(strmid(uval,6,3))
    endif                    else begin & d_we=long(strmid(uval,6,10))
                                        widget_control,d_we,bad_id=ii,get_uvalue=basets
                                        if ii eq 0 then widget_control,basets.baset,bad_id=ii,get_uvalue=state
                                        if ii eq 0 then ww=state.did_wi
                                        if ii eq 0 then widget_control,widget_info(d_we,/parent),bad_id=ii,map=0
                                       ;if ii eq 0 then d_livCleanup, state.wbase,state
    endelse    &  if ww gt 0 then magnet_win,W=ww,/PLOT,/ANIDR & return  & endif

WIDGET_CONTROL, ev.handler, GET_UVALUE=state, /NO_COPY

reset_transform=0
redraw=0
if uval eq 'RESET' then begin ;
    if ev.value eq 'Defaults' then begin ;Reset the defaults?
        state.shading = 1
        state.thick = 1
        state.edging = 0
        state.reject = 0
        state.bottomColor = 1
        state.style = 2
        d_livMenuSetState, '|Edging', state.edging,Names=state.menuitems, Indices=state.menubuttons
        d_livMenuSetState, '|Bottom Color', state.bottomColor,Names=state.menuitems, Indices=state.menubuttons
        d_livMenuSetState, '|Backface', state.reject,Names=state.menuitems, Indices=state.menubuttons
    endif
    reset_transform=1
    redraw=1

endif else if strpos(uval, 'Vrml') ge 0 then begin & drole=state.wDraw
    WIDGET_CONTROL, ev.handler, SET_UVALUE=state
    d_livDrawPrint, drole ,'VRML'

endif else if strpos(uval, 'XobjV') ge 0 then begin & drole=state.wDraw
    WIDGET_CONTROL, ev.handler, SET_UVALUE=state
    d_livDrawPrint, drole ,'XOBJV'

endif else if strpos(uval, '|Shading') ge 0 then begin
    state.shading = d_livMapMenuChoice(uval, state.MenuItems,state.MenuButtons)
    redraw=1

endif else if strpos(uval, '|Drag ') ge 0 then begin
    state.dragq = d_livMapMenuChoice(uval, state.MenuItems, state.MenuButtons)

endif else if strpos(uval, '|Vertex') ge 0 then begin
    state.vert_coloring = d_livMenuToggleState(ev.id)
    redraw=1

endif else if strpos(uval, '|Line ') ge 0 then begin
    state.thick = ([1,3,5,7])[d_livMapMenuChoice(uval, state.MenuItems,state.MenuButtons)]
    redraw=1

endif else if strpos(uval, '|Edging') ge 0 then begin
    state.edging = d_livMenuToggleState(ev.id)
    redraw=1

endif else if strpos(uval, '|Backface') ge 0 then begin
    state.reject = d_livMenuToggleState(ev.id) ;New backface culling
    redraw=1

endif else if strpos(uval, '|Bottom Color') ge 0 then begin
    state.bottomColor = d_livMenuToggleState(ev.id)    ;New backface culling
    redraw=1

endif else if strpos(uval, '|Style') ge 0 then begin
    state.style = d_livMapMenuChoice(uval, state.MenuItems, state.MenuButtons)
    redraw=1

endif else if uval eq 'OBJ' then begin ;Load a new object
    prev = state.oModelTop
    state.oView->remove, prev
    obj_destroy, state.oModelTop
    d_livLoadItem, ev.value, state
    obj_destroy, prev
    reset_transform=1
    redraw=1
endif

if reset_transform then begin
    state.scaling = 1.0
    state.oModelSurface->SetProperty, TRANSFORM=state.tmg0  ;And fall thru
    state.oModelEdges->SetProperty,   TRANSFORM=state.tmg0  ;And fall thru
    state.oModelPivot->SetProperty,   TRANSFORM=identity(4) ;And fall thru

    s = state.scaling
    ;state.oModelSurface->SetProperty, $
    ;  TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]] ;New transform
    ;state.oModelEdges->SetProperty, $
    ;  TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]] ;New transform
     state.oModelTop->SetProperty, $
       TRANSFORM = [[s,0,0,0],[0,s,0,0], [0,0,s,0], [0,0,0,1]] ;New transform
    endif
if redraw then begin
    d_livSetObjectAttributes, state ;Update attributes
    widget_control, ev.top, /HOURGLASS
    state.oWindow->Draw, state.oView
    did_mpeg,'put'
    endif

done : WIDGET_CONTROL, ev.handler, SET_UVALUE=state, /NO_COPY
end


;**
;** ************
pro d_livCleanup, wBase ,state
;+
;** ************
;** clean up befor dying
;-

if n_elements(state) eq 0 then more=1 else more=0
if n_elements(state) eq 0 then WIDGET_CONTROL, wBase, GET_UVALUE=state, /NO_COPY

OBJ_DESTROY , state.oView        ;  Destroy the top objects
OBJ_DESTROY , state.oTrack
OBJ_DESTROY , state.oModelTop

OBJ_DESTROY , state.oSurface
OBJ_DESTROY , state.oModelEdges
if more then begin
 OBJ_DESTROY, state.oModelOffset
 OBJ_DESTROY, state.oModelSticks
 OBJ_DESTROY, state.oModelImage
 OBJ_DESTROY, state.oModelSurface
 OBJ_DESTROY, state.oModelPivot
 OBJ_DESTROY, state.oSticks
 OBJ_DESTROY, state.oImage
endif
end


;**
;** *********
PRO liv_objet, WDRAW=wDraw, DIM=dim, MENUBAS=menuBas ,ALTER=alter, L_MESS=l_mess, EE=ee,NN=nn,VV=vv, TITLE=tit,$
               MYTEM=mytem, WI =wi , PS=ps, PCENT=pcent, XX=xx,YY=yy,ZZ=zz,WW=ww, PP=pp, THRESH=thresh, REPR=repr, SNAP=snap
;+
;** *********
;** liv_objet entry
;-

common c_liv_obj, allstates,allbases, vcc, Items, a_sphere, a_dens, a_name, a_symb ,a_col, a_Irrow
common d_liv_keepath, kpath

IF n_elements(ps) eq 1 then IF ps gt ' ' then begin & d_livDrawPrint, wDraw ,ps ,snap, PCENT=pcent  & RETURN & ENDIF ;event handler

IF n_elements(a_symb) le 1 then d_livMakeAtomes
IF n_elements(wDraw)  eq 0 then  RETURN

if n_elements(mytem)  eq 0 then mytem =6
if n_elements(l_mess) eq 0 then l_mess=0
if n_elements(dim)    eq 0 then dim   =[512,350]
if n_elements(wi)     ne 1 then wi    =0
if n_elements(xx)     eq 0 then xx    =0
if n_elements(yy)     eq 0 then yy    =0
if n_elements(zz)     eq 0 then zz    =0
if n_elements(ee)     eq 0 then ee    =0
if n_elements(nn)     eq 0 then nn    =0
if n_elements(vv)     eq 0 then vv    =0
if n_elements(ww)     eq 0 then ww    =0
if n_elements(pp)     eq 0 then pp    =0
if n_elements(tit)    eq 0 then tit   =''
if n_elements(thresh) eq 0 then thresh=-99
if n_elements(repr)   eq 0 then repr  =0
if wi gt 0 then mytem=6
xdim =dim[0]
ydim =dim[1]

Items = [ ['Seashell', 'd_livMakeShape', 'seashell.dat'], $
          ['Knot'    , 'd_livMakeShape', 'knot.dat'],     $
          ['Teapot'  , 'd_livMakeShape', 'teapot.dat'],   $
          ['Valium Molecule'   , 'd_livMakeMolecule', 'valium.mol'],   $
          ['Aspartame Molecule', 'd_livMakeMolecule', 'aspartam.mol'], $
          ['Caffeine Molecule' , 'd_livMakeMolecule', 'caffeine.mol'], $
          ['Lamp Object'       , 'd_livMakeLampWks' ,  string(wi) ]]

wBase=menuBas

MenuItems = ['1Shading', '0Flat', '6Gouraud', $
             '1Style', '0Points', '0Wire', '4Solid', '0Hidden Points', '2Hidden Wire', $
             '1Coloring', $
             '0Backface Culling (Off)', $
             '0Vertex Coloring (On )', $
             '0Edging (Off)', $
             '2Bottom Color (On )', $
             '1Line Thickness', '41', '03', '05','27', $
             '3Drag Quality', '0Low','4Medium','2High']

WIDGET_CONTROL, wDraw  , GET_VALUE =oWindow ; Grab the window id of the drawable.
WIDGET_CONTROL, wDraw  , SET_UVALUE={baset:menuBas,targets:obj_new(),styles:[0]}
WIDGET_CONTROL, wDraw  , /HOURGLASS
WIDGET_CONTROL, menuBas, SENSITIVE =1
WIDGET_CONTROL, menuBas, GET_UVALUE=state
newobj=0
IF   n_tags(state) gt 20 THEN BEGIN
     d_livCleanup, menuBas
     d_livClearAnim
     newobj=1
     MenuButtons=state.MenuButtons     & MenuItems=state.MenuItems
     dragq=state.dragq                 & shading=state.shading
     vert_coloring=state.vert_coloring & thick=state.thick
     edging=state.edging               & reject=state.reject
     bottomColor=state.bottomColor     & style=state.style
     cntxtid=state.did_cntxtid         & planeBut=state.did_planeBut
ENDIF else begin & d_livMenuCreate , MenuItems, MenuButtons, menuBas
     dragq=1  & shading=1 & vert_coloring=1 & thick=1
     edging=0 & reject=0  & bottomColor=1   & style=2 & cntxtid=0L
     if n_elements(state) ge 1 then planeBut =state else planeBut=[0L]
ENDELSE

aspect = float(xdim)/float(ydim) ; viewplane rect based on aspect ratio.
myview = [-0.5,-0.5,1,1]
if (aspect > 1) then begin
    myview[0] = myview[0] - ((aspect-1.0)*myview[2])/2.0
    myview[2] = myview[2] * aspect
endif else begin
    myview[1] = myview[1] - (((1.0/aspect)-1.0)*myview[3])/2.0
    myview[3] = myview[3] * aspect
endelse

oView = OBJ_NEW('idlgrview', PROJECTION=2, EYE=3, ZCLIP=[1.5,-1.5],$
                 VIEWPLANE_RECT=myview, COLOR=kpath.kpcol, DEPTH_CUE=[-.5,1.5])

oTrack= OBJ_NEW('Trackball', [xdim/2.0, ydim/2.0], xdim/2.0)

tvlct,rr,gg,bb,/get & vcc = transpose([[rr], [gg],[bb]])

if sys_dep('VERSION') ge 5.5 then begin       ;Context menu
   cntxtid=0L & call_procedure,'did_ContextCreat',wDraw,oView,cntxtid
endif

state = $
  { wBase: wBase, $             ; Main base
    wDraw: wDraw, $             ; Widget draw ID
    l_mess: l_mess, $           ; base label for message
    currentItem : mytem, $      ; Index of current item
    oView : oView, $            ; the view
    oModelTop: obj_new(), $
    oModelSurface: obj_new(), $
    oModelEdges: obj_new(), $
    oModelOffset: obj_new(), $
    oModelSticks: obj_new(), $
    oSticks: obj_new(), $
    oModelImage: obj_new(), $
    oModelPivot: obj_new(), $   ;  Visible pivot axes in Top model
    oImage: obj_new(), $
    oTrack: oTrack, $
    oWindow : oWindow, $
    oSurface : obj_new(), $     ;  Visible surface
    tmg0: fltarr(4,4), $        ;  Initial transform
    dragq: dragq, $             ;  Drag quality
    btndown: 0, $               ;  Button down flag
    MenuItems: MenuItems, $
    MenuButtons: MenuButtons, $
    cur_sel: obj_new(), $
    scaling: 1.0, $
    shading: shading, $
    vert_coloring: vert_coloring, $
    thick: thick, $
    edging: edging, $
    reject: reject, $
    bottomColor: bottomColor, $
    style: style, $
    did_wi:wi     , $
    did_scaling: 0, $
    did_oldny:   0, $
    did_xx: xx    , $
    did_yy: yy    , $
    did_zz: zz    , $
    did_ee: ee    , $
    did_nn: nn    , $
    did_vv: vv    , $
    did_ww: ww    , $
    did_pp: pp    , $
    did_tit: tit  , $
    did_thresh: thresh , $
    did_repr:   repr   , $
    did_cntxtid: cntxtid,$
    did_cntxtfl: 0  , $
    did_scalval: 110, $
    did_transval: -1, $
    did_planeBut:planeBut}

for i=0,n_elements(planeBut)-1 do if planeBut[i] gt 0 then widget_control,planeBut[i] ,bad_id=ii,sensitive=0
WIDGET_CONTROL, wBase, SET_UVALUE=  state
WIDGET_CONTROL, wDraw, EVENT_PRO = 'd_livDrawEvent'
WIDGET_CONTROL, wDraw, map=1
if n_elements(alter) eq 1 then if alter gt 0 then WIDGET_CONTROL,alter, bad_id=ii, map=0

d_livLoadItem , mytem, state
state.oWindow->Draw, state.oView

idx=-1
IF n_elements(allbases) gt 0 then begin        ;Clear old objects
   FOR i=0,n_elements(allbases)-1 do begin
    IF   allbases[i] eq 0 then idx=i $
    ELSE BEGIN
         WIDGET_CONTROL, allbases[i],bad_id=ii, GET_UVALUE=curstate
         IF (ii gt 0) or (n_tags(curstate) lt 20) or (allbases[i] eq wBase) then begin
           bb={oView:allstates[0,i],oTrack:allstates[1,i],oModelTop:allstates[2,i],$
           oSurface:allstates[3,i],oModelEdges:allstates[4,i]}
           idx=i & allbases[i]=0L & d_livCleanup, 0,bb & ENDIF
    ENDELSE
   ENDFOR
ENDIF

BB=[state.oView,state.oTrack,state.oModelTop,state.oSurface,state.oModelEdges] ;Keep object in mind
IF n_elements(allstates) eq 0 then begin & allstates=BB & allbases=[wBase]
ENDIF ELSE IF idx ge 0 then begin & allstates[*,idx]=BB & allbases[idx]=wBase
           ENDIF       else begin & allstates=[[allstates],[BB]] & allbases=[allbases,wBase]
ENDELSE

WIDGET_CONTROL, wBase, SET_UVALUE=state, /NO_COPY
WIDGET_CONTROL, wBase, EVENT_PRO = 'd_livEvent'

end
